/****************************************************************************
 *
 *  Copyright (c) 2001 by LSI Logic Corporation, all rights reserved.
 *
 *                         LSI Logic confidential
 *  The information contained herein is proprietary of LSI Logic Corporation.
 *
 *  This software is provided "as is" without any express or implied warranty.
 *  LSI Logic disclaims all representations and warranties with regard to
 *  this software, including all implied warranties of merchantability and
 *  fitness for a particular purpose. In no event shall LSI Logic be liable
 *  for any special, indirect or consequential damages or any damages
 *  whatsoever resulting from loss of use, data or profits, whether in an
 *  action of contract, negligence or other tortious act or omission,
 *  arising out of or in connection with the use or performance of this
 *  software.
 *
 *  File Description:
 *
 *  Header file for the Getafix bootloader.
 *
 ****************************************************************************/

#ifndef _BOOTLOADER_H
#define _BOOTLOADER_H

#include "bootrom.h"
/* Linux kernel header includes needed to pick up the tag declarations */
typedef __u32 u32; /* Needed to match Linux defines */
typedef __u16 u16; /* Needed to match Linux defines */
typedef __u8 u8; /* Needed to match Linux defines */
#include "linux/flashfloppy.h"
#include "asm-arm/setup.h"
#include "asm-arm/arch-obelix/setup.h"

/* Getafix specific addresses for the EBB */
#define EBB_BASE		0x93000000
#define EBB_CERW0		(__u32 *)(EBB_BASE + 0x0000)
#define EBB_CERW1		(__u32 *)(EBB_BASE + 0x0004)
#define EBB_CERW2		(__u32 *)(EBB_BASE + 0x0008)
#define EBB_CERW3		(__u32 *)(EBB_BASE + 0x000c)
#define EBB_CECS0		(__u32 *)(EBB_BASE + 0x0100)
#define EBB_CECS1		(__u32 *)(EBB_BASE + 0x0104)
#define EBB_CECS2		(__u32 *)(EBB_BASE + 0x0108)
#define EBB_CECS3		(__u32 *)(EBB_BASE + 0x010c)
#define EBB_CECFG0		(__u32 *)(EBB_BASE + 0x0200)
#define EBB_CECFG1		(__u32 *)(EBB_BASE + 0x0204)
#define EBB_CECFG2		(__u32 *)(EBB_BASE + 0x0208)
#define EBB_CECFG3		(__u32 *)(EBB_BASE + 0x020c)
#define EBB_CEWAIT0		(__u32 *)(EBB_BASE + 0x0300)
#define EBB_CEWAIT1		(__u32 *)(EBB_BASE + 0x0304)
#define EBB_CEWAIT2		(__u32 *)(EBB_BASE + 0x0308)
#define EBB_CEWAIT3		(__u32 *)(EBB_BASE + 0x030c)
#define EBB_CEBC0		(__u32 *)(EBB_BASE + 0x0400)
#define EBB_CEBC1		(__u32 *)(EBB_BASE + 0x0404)
#define EBB_CEBC2		(__u32 *)(EBB_BASE + 0x0408)
#define EBB_CEBC3		(__u32 *)(EBB_BASE + 0x040c)
#define EBB_CEMSTB0		(__u32 *)(EBB_BASE + 0x0500)
#define EBB_CEMSTB1		(__u32 *)(EBB_BASE + 0x0504)
#define EBB_CEALE		(__u32 *)(EBB_BASE + 0x0600)
#define EBB_CEBUSMODE	(__u32 *)(EBB_BASE + 0x0700)
#define EBB_CEPINMODE	(__u32 *)(EBB_BASE + 0x0800)
#define EBB_CEMASKADDR0	(__u32 *)(EBB_BASE + 0x0900)
#define EBB_CEMASKADDR1	(__u32 *)(EBB_BASE + 0x0904)
#define EBB_CEMASKADDR2	(__u32 *)(EBB_BASE + 0x0908)
#define EBB_CEMASKADDR3	(__u32 *)(EBB_BASE + 0x090c)
#define EBB_CEBOOT		(__u32 *)(EBB_BASE + 0x0a00)
#define EBB_CETADDR		(__u32 *)(EBB_BASE + 0x0a04)
#define EBB_CETRW		(__u32 *)(EBB_BASE + 0x0a08)

#define BOOTROM_RE_ENTRY_VECTOR	(0xffff0014)
typedef	void (*ErrorVector)(int param);
typedef	void (*ImageVector)();

/* 
	Used when the bootloader has a fatal error and needs to jump back to the
	bootrom.  This exits the bootloader with no possibility of return and
	destroys all variables & data.
	The parameter passed is an error number which ends up in r0 when the
	bootrom code is run.  The bootrom can't do anything specific with this
	code as the meaning varies depending on which bootloader is used, however
	it may choose to log it or print it out as a diagnostic aid.
*/
#define EXIT_BOOTLOADER(p)	(*(ErrorVector)BOOTROM_RE_ENTRY_VECTOR)(p)

/*
	Error codes returned from this bootloader. *NOTE* The return code must
	not be zero - this is reserved by the bootrom for flagging a hard reset
	and it will cause the FLASH boot sequence to be re-run.
*/
#define BOOTLOADER_NOIMAGES (-1)
#define BOOTLOADER_BADDATES (-2)
#define BOOTLOADER_CANNOTCOPY (-3)

/*
	Used to jump out of the bootloader and into the downloaded image
	in RAM (which will start the Linux kernel boot process).  At this point
	the bootloader is finished and isn't called again.
*/
#define LINUX_KERNEL_EXECUTION_ADDRESS (0x8000)
#define JUMP_TO_IMAGE	(*(ImageVector)LINUX_KERNEL_EXECUTION_ADDRESS)()

/* Tell the bootloader where to build the configuration tags that will
   be parsed by the Linux kernel on startup */
#define LINUX_CONFIG_PARAMS_BASE (0x100)

/* The total memory size on the target system (in MBytes) */
#define SYSTEM_MEMORY_SIZE (16)
/* The number of MBytes to reserve for the ABM buffers */
#define ABM_BUFFER_SIZE_IN_MB (1)

/* Size of ramdisk to use whilst booting the kernel (in kBytes) */
#define BOOT_RAMDISK_SIZE (3000)
/* Where the ramdisk lives (0 = kernel chooses?) */
#define BOOT_RAMDISK_START (0)

struct LinuxKernelConfigTags
{
  struct tag_header CoreHeader;
  struct tag_core Core;
  struct tag_header MemHeader;
  struct tag_mem32 Mem;
  struct tag_header RamdiskHeader;
  struct tag_ramdisk Ramdisk;
  struct tag_header Obelix2Header; /* Copes with V2 tags and above */
  struct tag_obelix4 Obelix4;
  struct tag_header ObelixHeader;
  struct tag_obelix Obelix;
  struct tag_header FlashHeader;
  /* This is a variable length structure - and this structure
     only marks the start of it.  For this reason, do NOT
     put any elements after this one.
  */
  struct tag_obelix_flashdata FlashData;
};

/* Check every 1kB for a Linux kernel structure */
#define KERNEL_STRUCT_SEARCH_STEP (0x400)

/* Check every 1kB for a R/W segment kernel structure */
#define RW_SEG_SEARCH_STEP (0x400)

PUBLIC void BootloaderMain(void);
PUBLIC void SetupCachesAndMMU(void);
PUBLIC void NoCachesNoMMU(void);
PUBLIC inline const void *PtrFixup(const void *);

#endif
