/*********************************************************************************
 *
 *  Copyright (c) 2001 by LSI Logic Corporation, all rights reserved.
 *
 *                         LSI Logic confidential
 *  The information contained herein is proprietary of LSI Logic Corporation.
 *
 *  This software is provided "as is" without any express or implied warranty.
 *  LSI Logic disclaims all representations and warranties with regard to
 *  this software, including all implied warranties of merchantability and
 *  fitness for a particular purpose. In no event shall LSI Logic be liable
 *  for any special, indirect or consequential damages or any damages
 *  whatsoever resulting from loss of use, data or profits, whether in an
 *  action of contract, negligence or other tortious act or omission,
 *  arising out of or in connection with the use or performance of this
 *  software.
 *
 *  File Description:
 *
 *  Definition of FLASH chip layouts used by the bootloader.
 *
 *********************************************************************************/

#ifndef _FLASHDEFINITION_H
#define _FLASHDEFINITION_H

/*********************************************************************************
 *                                                                               *
 *                          FLASH device definition area                         *
 *                                                                               *
 *********************************************************************************/

/*
	Change the definitions below to build a bootloader that targets devices
	with arbitary FLASH device configurations.  You should not need to change
	any other parts of this file.
	
	Valid options for settings:
	FLASH_n_DEVICE        : If not defined then the settings for chip select n
	                        will not be modified.
	                        If defined then chip select n is initialised based
	                        on the speed, width and size settings.  The value
	                        of the definition is for readability only, it is
	                        not used by the code.
	FLASH_n_SPEED_IN_NS   : Read access time in nanoseconds.
	FLASH_n_WIDTH         : FLASH databus width.  Valid options are 8 and 16.
	FLASH_n_SIZE_IN_MBITS : Device size in megabits.
*/

/* The primary clock frequency of the EBB - fixed by the hardware */
#define EBB_BUS_CLK_IN_MHZ		100

/* FLASH 0 has the lowest FLASH linear address */
#define FLASH_0_DEVICE			Present
#define FLASH_0_SPEED_IN_NS		120
#define FLASH_0_WIDTH			16
#define FLASH_0_SIZE_IN_MBITS	64

#undef  FLASH_1_DEVICE
#define FLASH_1_SPEED_IN_NS		120
#define FLASH_1_WIDTH			8
#define FLASH_1_SIZE_IN_MBITS	16

#undef  FLASH_2_DEVICE
#define FLASH_2_SPEED_IN_NS		120
#define FLASH_2_WIDTH			8
#define FLASH_2_SIZE_IN_MBITS	16

/* FLASH 3 has the highest FLASH linear address */
#undef  FLASH_3_DEVICE
#define FLASH_3_SPEED_IN_NS		120
#define FLASH_3_WIDTH			8
#define FLASH_3_SIZE_IN_MBITS	16

/**************************** END FLASH definitions *********************************/

#endif
