/*********************************************************************************
 *
 *  Copyright (c) 2001 by LSI Logic Corporation, all rights reserved.
 *
 *                         LSI Logic confidential
 *  The information contained herein is proprietary of LSI Logic Corporation.
 *
 *  This software is provided "as is" without any express or implied warranty.
 *  LSI Logic disclaims all representations and warranties with regard to
 *  this software, including all implied warranties of merchantability and
 *  fitness for a particular purpose. In no event shall LSI Logic be liable
 *  for any special, indirect or consequential damages or any damages
 *  whatsoever resulting from loss of use, data or profits, whether in an
 *  action of contract, negligence or other tortious act or omission,
 *  arising out of or in connection with the use or performance of this
 *  software.
 *
 *  File Description:
 *
 *  Header file for flash-specific routines.
 *
 *********************************************************************************/

#ifndef _FLASHSPECIFIC_H
#define _FLASHSPECIFIC_H

#include "bootrom.h"
#include "flash-definition.h"

/****************************************************************************
 *	FLASH chip configurations are defined in flash-definition.h.  Change    *
 *	the definitions there - not here!                                       * 
 ****************************************************************************/

/****************************************************************************
 *	What follows now sets up variables and structure definitions to match   *
 *	the requirements in flash-definition.h.                                 *
 ****************************************************************************/

#define FLASH_WIDTHCODE_8_BIT (2)
#define FLASH_WIDTHCODE_16_BIT (1)
 
#ifdef FLASH_0_DEVICE
  #define FLASH_0_PRESENT (1)
	#if (FLASH_0_WIDTH != 8) && (FLASH_0_WIDTH != 16)
		#error Unsupported FLASH width: FLASH_0_WIDTH should be 8 or 16
	#endif
	#define READ_CLOCKS_0 ((int)(((FLASH_0_SPEED_IN_NS + 1) * (EBB_BUS_CLK_IN_MHZ)) / 1000))
	#define FLASH_0_SIZE (0x100000 * (int)((FLASH_0_SIZE_IN_MBITS) / 8))
	#if (FLASH_0_WIDTH == 8)
		#define FLASH_0_WIDTHCODE (FLASH_WIDTHCODE_8_BIT)
	#else
		#define FLASH_0_WIDTHCODE (FLASH_WIDTHCODE_16_BIT)
	#endif
#else
  #define FLASH_0_SIZE (0)
  #define FLASH_0_PRESENT (0)
	#define FLASH_0_WIDTHCODE (FLASH_WIDTHCODE_8_BIT)
#endif

#ifdef FLASH_1_DEVICE
  #define FLASH_1_PRESENT (1)
	#if (FLASH_1_WIDTH != 8) && (FLASH_1_WIDTH != 16)
		#error Unsupported FLASH width: FLASH_1_WIDTH should be 8 or 16
	#endif
	#define READ_CLOCKS_1 ((int)(((FLASH_1_SPEED_IN_NS + 1) * (EBB_BUS_CLK_IN_MHZ)) / 1000))
	#define FLASH_1_SIZE (0x100000 * (int)((FLASH_1_SIZE_IN_MBITS) / 8))
	#if (FLASH_1_WIDTH == 8)
		#define FLASH_1_WIDTHCODE (FLASH_WIDTHCODE_8_BIT)
	#else
		#define FLASH_1_WIDTHCODE (FLASH_WIDTHCODE_16_BIT)
	#endif
#else
  #define FLASH_1_SIZE (0)
  #define FLASH_1_PRESENT (0)
	#define FLASH_1_WIDTHCODE (FLASH_WIDTHCODE_8_BIT)
#endif

#ifdef FLASH_2_DEVICE
  #define FLASH_2_PRESENT (1)
	#if (FLASH_2_WIDTH != 8) && (FLASH_2_WIDTH != 16)
		#error Unsupported FLASH width: FLASH_2_WIDTH should be 8 or 16
	#endif
	#define READ_CLOCKS_2 ((int)(((FLASH_2_SPEED_IN_NS + 1) * (EBB_BUS_CLK_IN_MHZ)) / 1000))
	#define FLASH_2_SIZE (0x100000 * (int)((FLASH_2_SIZE_IN_MBITS) / 8))
	#if (FLASH_2_WIDTH == 8)
		#define FLASH_2_WIDTHCODE (FLASH_WIDTHCODE_8_BIT)
	#else
		#define FLASH_2_WIDTHCODE (FLASH_WIDTHCODE_16_BIT)
	#endif
#else
  #define FLASH_2_PRESENT (0)
  #define FLASH_2_SIZE (0)
	#define FLASH_2_WIDTHCODE (FLASH_WIDTHCODE_8_BIT)
#endif

#ifdef FLASH_3_DEVICE
  #define FLASH_3_PRESENT (1)
	#if (FLASH_3_WIDTH != 8) && (FLASH_3_WIDTH != 16)
		#error Unsupported FLASH width: FLASH_3_WIDTH should be 8 or 16
	#endif
	#define READ_CLOCKS_3 ((int)(((FLASH_3_SPEED_IN_NS + 1) * (EBB_BUS_CLK_IN_MHZ)) / 1000))
	#define FLASH_3_SIZE (0x100000 * (int)((FLASH_3_SIZE_IN_MBITS) / 8))
	#if (FLASH_3_WIDTH == 8)
		#define FLASH_3_WIDTHCODE (FLASH_WIDTHCODE_8_BIT)
	#else
		#define FLASH_3_WIDTHCODE (FLASH_WIDTHCODE_16_BIT)
	#endif
#else
  #define FLASH_3_PRESENT (0)
  #define FLASH_3_SIZE (0)
	#define FLASH_3_WIDTHCODE (FLASH_WIDTHCODE_8_BIT)
#endif

#define FLASH_NUMBER_OF_CHIPS (FLASH_0_PRESENT + FLASH_1_PRESENT + FLASH_2_PRESENT + FLASH_3_PRESENT)
#define FLASH_TOTAL_SIZE (FLASH_0_SIZE + FLASH_1_SIZE + FLASH_2_SIZE + FLASH_3_SIZE)

/* Keep the FLASH definitions in strict ascending virtual address order */
#define FLASH0_LINEAR_OFFSET (0)
#define FLASH1_LINEAR_OFFSET (FLASH0_LINEAR_OFFSET + FLASH_0_SIZE)
#define FLASH2_LINEAR_OFFSET (FLASH1_LINEAR_OFFSET + FLASH_1_SIZE)
#define FLASH3_LINEAR_OFFSET (FLASH2_LINEAR_OFFSET + FLASH_2_SIZE)

/* Define the physical address where each EBB chip select address range starts */
#define CS0_VIRTUAL_ADDR (0xEC000000)
#define CS1_VIRTUAL_ADDR (0xE8000000)
#define CS2_VIRTUAL_ADDR (0xE4000000)
#define CS3_VIRTUAL_ADDR (0xE0000000)

PUBLIC __u32 ReadFlash32(__u32 FlashLinearAddress);
PUBLIC __u32 FlashLin2Virt(__u32 FlashLinearAddress, int *pAddressOK);
PUBLIC __u32 CopyFlashLin2Virt(__u32 LinStartAddr, __u32 Length, __u32 VirtAddress);
PUBLIC void  ConfigureEBB(void);
PUBLIC __u32 MaxFlashLinearAddress(void);
PUBLIC __u32 checkFlashCRC32edBlock(__u32 LinStartAddr, __u32 Length);
PUBLIC __u32 FlashVirt2Lin(__u32 FlashVirtualAddress, int *pAddressOK);

#endif
