/****************************************************************************
 *
 *  Copyright (c) 2001 by LSI Logic Corporation, all rights reserved.
 *
 *                         LSI Logic confidential
 *  The information contained herein is proprietary of LSI Logic Corporation.
 *
 *  This software is provided "as is" without any express or implied warranty.
 *  LSI Logic disclaims all representations and warranties with regard to
 *  this software, including all implied warranties of merchantability and
 *  fitness for a particular purpose. In no event shall LSI Logic be liable
 *  for any special, indirect or consequential damages or any damages
 *  whatsoever resulting from loss of use, data or profits, whether in an
 *  action of contract, negligence or other tortious act or omission,
 *  arising out of or in connection with the use or performance of this
 *  software.
 *
 *  File Description:
 *
 *  Non-ARM library, non-GPL version of memcpy for use in the bootrom and
 *	in bootloaders.
 *
 ****************************************************************************/

/****************************************************************************
 * Function Name  : FlashMemcpy
 * Arguments      : ARM virtual target address.
 *					        ARM virtual source address.
 *					        Number of bytes to copy.
 * Returns        : None
 * Modifies       : nothing
 * Description    : Copies a block of memory from one ARM virtual address
 *                  to another.  Handles arbitary lengths and byte
 *                  alignments.
 *                  The copy is done using as many dword (32 bit) reads and
 *                  writes as possible.
 *                  Len=0 copies no bytes and returns immediately.
 *                  No check on the source or destination pointers is made as
 *                  all values could be valid.
 *                  This version is semi-optimised, it does not use ldmia
 *                  or stmia to provoke bursts on the AMBA bus.
 ****************************************************************************/

.global FlashMemcpy

/*  PUBLIC void FlashMemcpy(__u32 dest, __u32 src, __u32 len) */
/*                          dest => r0, src => r1, len => r2  */
FlashMemcpy:
    cmp   r2, #0
    moveq pc, r14
	mov	ip, sp
	stmfd	sp!, {r4-r8, fp, ip, lr, pc}
	sub	fp, ip, #4
    ands  r4, r0, #3
    beq   dest_aligned
align_dest:
    ldrb  r4, [r1], #1
    strb  r4, [r0], #1
    subs  r2, r2, #1
    beq   copy_done
    ands  r5, r0, #3
    bne   align_dest

    /* dest now points to a word aligned boundary */
dest_aligned:
    ands  r7, r1, #0x3
    beq   aligned
    mov   r7, r7, lsl #3
    rsb   r8, r7, #32

    /*
      The src and dest do not point to the same byte within a dword
      so we need to barrel shift the data as it is copied.  r7 has
      the shift coefficient in bits, r8 contains 32 - r7, i.e. the
      amount of shift "the other way".
    */
    /* Step the source address back to the last word boundary */
    and   r1, r1, #0xfffffffc
    ldr   r4, [r1], #4
    cmp   r2, #4
    blo   non_aligned_words_finished
non_aligned:
    ldr   r5, [r1], #4
    mov   r6, r4, lsr r7
    orr   r6, r6, r5, lsl r8
    str   r6, [r0], #4
    mov   r4, r5
    subs  r2, r2, #4
    cmp   r2, #4
    bhs   non_aligned
non_aligned_words_finished:
    cmp   r2, #0
    beq   copy_done
    /* Copy the last partial word */
    sub   r1, r1, #4
    add   r1, r1, r7, lsr #3
non_aligned_trailing_bytes:
    ldrb  r4, [r1], #1
    strb  r4, [r0], #1
    subs  r2, r2, #1
    bne   non_aligned_trailing_bytes
    b   copy_done

    /* 
      Src and dest started on the same byte offset within a dword and
      we have copied up to a dword boundary, so we can now copy
      dwords until there are less than 4 bytes remaining when we
      finish up by copying bytes.
    */
aligned:
    cmp   r2, #4
    blo   words_finished
copy_words:
    ldr   r4, [r1], #4
    str   r4, [r0], #4
    subs  r2, r2, #4
    cmp   r2, #4
    bhs   copy_words
words_finished:
    cmp   r2, #0
    beq   copy_done
finish_bytes:
    ldrb  r4, [r1], #1
    strb  r4, [r0], #1
    subs  r2, r2, #1
    bne   finish_bytes
copy_done:
	ldmea	fp, {r4-r8, fp, sp, pc}

