// Albert Aug. 2005
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "cli.h"
#include "arp_cmd.h"

extern int do_command(char *cmd, ...);

static int isHex(char c)
{
	if(c>='0' && c<='9')
		return 1;
	if(c>='A' && c<='F')
		return 1;
	if(c>='a' && c<='f')
		return 1;
	return 0;
}

static int chk_input_mac(char *m)
{
	int i=0;

	if(strlen(m)!=17)
		return -1;
	
	if(m[2]!=':'||m[5]!=':'||m[8]!=':'||m[11]!=':'||m[14]!=':')
		return -1;
	
	for(i=0;i<5;i++){
		if( !isHex(m[i*3]) || !isHex(m[i*3+1]))
			return -1;
	}
	return 0;
}

static int chk_is_number(const char *m)
{
	int i=0;
	for(i=0; i<strlen(m); i++){
		if(isdigit(m[i])==0)
			return -1;
	}
	return 0;
}

static arp_show_config()
{
	char buf[80]={0}, now_ip[20]={0}, now_mac[20]={0};
	FILE *op=NULL;
	int i=0;

	printf("-------------  Static ARP Table  --------------\n");
	printf("%-8s%-20s%-20s\n","index","IP address","MAC address");
	printf("-----------------------------------------------\n");
	op = fopen(ARP_CONF_FILE_PATH, "rb");
	if(op) {
		while(fgets(buf, sizeof(buf), op)){
			sscanf(buf, "|%20[^|]|%20[^|]|", now_ip, now_mac);
			printf("%-8d%-20s%-20s\n", i, now_ip, now_mac);
			i++;
		}
		fclose(op);
	}
	if(i == 0) {
		printf(" empty !!!\n");
	} else {
		printf("-----------------------------------------------\n");
	}
	return 0;
}

static arp_chk_have_the_same(const char *ip, const char *mac)
{
	char buf[80];
	char now_ip[20]={0}, now_mac[20]={0};
	struct ITEMS items[10];
	FILE *op=NULL;
	int i=0;

	op = fopen(ARP_CONF_FILE_PATH, "rb");
	if(op) {
		while(fgets(buf, sizeof(buf), op)){
			sscanf(buf, "|%20[^|]|%20[^|]|", now_ip, now_mac);
			strncpy(items[i].ip, now_ip, 20);
			strncpy(items[i].mac, now_mac, 20);
			i++;
			if(i>9)
				break;
		}
		fclose(op);
	}

	for(i=0;i<10;i++){
		if(!strncmp(ip, items[i].ip, 20))
			return -1;
		if(!strncmp(mac, items[i].mac, 20))
			return -1;
	}

	return 0;
}

static arp_add_to_config(const char *ip, const char *mac)
{
	FILE *op=NULL;
	char buf[80]={0};
	int i=0;

	op = fopen(ARP_CONF_FILE_PATH, "rb");
	if(op) {
		while(fgets(buf, sizeof(buf), op)){
			i++;
		}
		fclose(op);
	}

	if(i>9) {
		return -1;
	}

	sprintf(buf, "|%s|%s|\n", ip, mac);
	op = fopen(ARP_CONF_FILE_PATH, "ab");
	fputs(buf, op);
	fclose(op);
	
	return 0;
}

static arp_del_from_config(const char *item)
{
	char buf[80];
	char now_ip[20]={0}, now_mac[20]={0};
	struct ITEMS items[10];
	FILE *op=NULL;
	int i=0, delete_item=0, total=0;


	if(chk_is_number(item)==-1){
		printf("incorrect index number \"%s\"!\n", item);
		return 0;
	}

	op = fopen(ARP_CONF_FILE_PATH, "rb");
	if(op) {
		while(fgets(buf, sizeof(buf), op)){
			sscanf(buf, "|%20[^|]|%20[^|]|", now_ip, now_mac);
			strncpy(items[total].ip, now_ip, 20);
			strncpy(items[total].mac, now_mac, 20);
			total++;
			if(total>9)
				break;
		}
		fclose(op);
	}

	delete_item = atoi(item);
	if( delete_item > (total-1) || delete_item < 0) {
		printf("incorrect index number \"%d\"!\n", delete_item);
		return 0;
	}

	do_command("arp", "arp", "-i", "br0", "-d", items[i].ip, 0);

	op = fopen(ARP_CONF_FILE_PATH, "wb");
	if(op) {
		for(i=0; i<total; i++){
			if(i!=delete_item) {
				sprintf(buf, "|%s|%s|\n", items[i].ip, items[i].mac);
				//printf(buf);
				fputs(buf, op);
			}
		}
		fclose(op);
	}

	return 0;
}

int doArpList(int argc, char *argv[], void *p)
{
	do_command("arp", "arp", "-n", "-v", 0);
	return 0;
}

int doArpShow(int argc,char *argv[],void *p)
{
	arp_show_config();
	return 0;
} /* doArpShow */

int doArpAdd(int argc,char *argv[],void *p)
{
	int i;
	struct in_addr arpip;
	
	if(argc != 3) {
		printf("add <ip> <macaddr>\n");
		return 0;
	}

	if(inet_aton(argv[1], &arpip)==0) {
		// ip is incorrect
		printf("ip invalid\n");
		return 0;
	}

	if(chk_input_mac(argv[2])==-1) {
		// mac is incorrect
		printf("mac invalid\n");
		return 0;
	}

	if(arp_chk_have_the_same(argv[1], argv[2])==-1) {
		// have the same ip or mac
		printf("the ip or the mac exist\n");
		return 0;
	}

	if(arp_add_to_config(argv[1], argv[2])==-1) {
		// file can't op or write error
		printf("config full\n");
		return 0;
	}

	do_command("arp", "arp", "-i", "br0", "-s", argv[1], argv[2], 0);
	return 0;
} /* doArpAdd */

int doArpDel(int argc,char *argv[],void *p)
{
	if(argc != 2) {
		printf("del <index number>\n");
		return 0;
	}

	if(arp_del_from_config(argv[1])==-1) {
		// del item from config
		printf("item value error ! or remove from config fail !\n");
		return 0;
	}

	return 0;
} /* doArpDel */

const cmds_table arpCmds[] = {

        {"show",      doArpShow,    Normal_Level, 0, "sys arp show"},
	{"add",       doArpAdd,     Normal_Level, 0, "sys arp add <ip> <hwaddr>"},
	{"del",       doArpDel,     Normal_Level, 0, "sys arp del <ip>"},
      	//{"list",      doArpList,    Debug_Level,  0, "sys arp list"},
	{NULL,        NULL,		0,	  0, NULL},

};
