#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <linux/if.h>

FILE* TTYS0;
#define DPRINTF(args...) \
	TTYS0=fopen("/dev/ttyS0","w"); \
	fprintf(TTYS0,args); \
	fclose(TTYS0) 


#define _PATH_PROCNET_DEV               "/proc/net/dev"

struct user_net_device_stats {
#if 1
	unsigned long long rx_packets;	/* total packets received       */
	unsigned long long tx_packets;	/* total packets transmitted    */
	unsigned long long rx_bytes;	/* total bytes received         */
	unsigned long long tx_bytes;	/* total bytes transmitted      */
#else
    unsigned  long rx_packets;	/* total packets received       */
    unsigned  long tx_packets;	/* total packets transmitted    */
    unsigned  long rx_bytes;	/* total bytes received         */
    unsigned  long tx_bytes;	/* total bytes transmitted      */
#endif
    unsigned long rx_errors;	/* bad packets received         */
    unsigned long tx_errors;	/* packet transmit problems     */
    unsigned long rx_dropped;	/* no space in linux buffers    */
    unsigned long tx_dropped;	/* no space available in linux  */
    unsigned long rx_multicast;	/* multicast packets received   */
    unsigned long rx_compressed;
    unsigned long tx_compressed;
    unsigned long collisions;

    /* detailed rx_errors: */
    unsigned long rx_length_errors;
    unsigned long rx_over_errors;	/* receiver ring buff overflow  */
    unsigned long rx_crc_errors;	/* recved pkt with crc error    */
    unsigned long rx_frame_errors;	/* recv'd frame alignment error */
    unsigned long rx_fifo_errors;	/* recv'r fifo overrun          */
    unsigned long rx_missed_errors;	/* receiver missed packet     */
    /* detailed tx_errors */
    unsigned long tx_aborted_errors;
    unsigned long tx_carrier_errors;
    unsigned long tx_fifo_errors;
    unsigned long tx_heartbeat_errors;
    unsigned long tx_window_errors;
};

struct interface {
    struct interface *next, *prev; 
    char name[IFNAMSIZ];	/* interface name        */
    short type;			/* if type               */
    short flags;		/* various flags         */
    int metric;			/* routing metric        */
    int mtu;			/* MTU value             */
    int tx_queue_len;		/* transmit queue length */
    struct ifmap map;		/* hardware setup        */
    struct sockaddr addr;	/* IP address            */
    struct sockaddr dstaddr;	/* P-P IP address        */
    struct sockaddr broadaddr;	/* IP broadcast address  */
    struct sockaddr netmask;	/* IP network mask       */
    struct sockaddr ipxaddr_bb;	/* IPX network address   */
    struct sockaddr ipxaddr_sn;	/* IPX network address   */
    struct sockaddr ipxaddr_e3;	/* IPX network address   */
    struct sockaddr ipxaddr_e2;	/* IPX network address   */
    struct sockaddr ddpaddr;	/* Appletalk DDP address */
    struct sockaddr ecaddr;	/* Econet address        */
    int has_ip;
    int has_ipx_bb;
    int has_ipx_sn;
    int has_ipx_e3;
    int has_ipx_e2;
    int has_ax25;
    int has_ddp;
    int has_econet;
    char hwaddr[32];		/* HW address            */
    int statistics_valid;
    struct user_net_device_stats stats;		/* statistics            */
    int keepalive;		/* keepalive value for SLIP */
    int outfill;		/* outfill value for SLIP */
};

static char *get_name(char *name, char *p)
{
    while (isspace(*p))
	p++;
    while (*p) {
	if (isspace(*p))
	    break;
	if (*p == ':') {	/* could be an alias */
	    char *dot = p, *dotname = name;
	    *name++ = *p++;
	    while (isdigit(*p))
		*name++ = *p++;
	    if (*p != ':') {	/* it wasn't, backup */
		p = dot;
		name = dotname;
	    }
	    if (*p == '\0')
		return NULL;
	    p++;
	    break;
	}
	*name++ = *p++;
    }
    *name++ = '\0';
    return p;
}

static int get_dev_fields(char *bp, struct interface *ife)
{
    sscanf(bp,
	   "%Lu %Lu %lu %lu %lu %lu %lu %lu %Lu %Lu %lu %lu %lu %lu %lu %lu",
	   &ife->stats.rx_bytes,
	   &ife->stats.rx_packets,
	   &ife->stats.rx_errors,
	   &ife->stats.rx_dropped,
	   &ife->stats.rx_fifo_errors,
	   &ife->stats.rx_frame_errors,
	   &ife->stats.rx_compressed,
	   &ife->stats.rx_multicast,

	   &ife->stats.tx_bytes,
	   &ife->stats.tx_packets,
	   &ife->stats.tx_errors,
	   &ife->stats.tx_dropped,
	   &ife->stats.tx_fifo_errors,
	   &ife->stats.collisions,
	   &ife->stats.tx_carrier_errors,
	   &ife->stats.tx_compressed);
#if 0
    DPRINTF("1: -%s-\n", bp);
    printf("2: %Lu %Lu %lu %lu %lu %lu %lu %lu %Lu %Lu %lu %lu %lu %lu %lu %lu\n",
	   ife->stats.rx_bytes,
	   ife->stats.rx_packets,
	   ife->stats.rx_errors,
	   ife->stats.rx_dropped,
	   ife->stats.rx_fifo_errors,
	   ife->stats.rx_frame_errors,
	   ife->stats.rx_compressed,
	   ife->stats.rx_multicast,

	   ife->stats.tx_bytes,
	   ife->stats.tx_packets,
	   ife->stats.tx_errors,
	   ife->stats.tx_dropped,
	   ife->stats.tx_fifo_errors,
	   ife->stats.collisions,
	   ife->stats.tx_carrier_errors,
	   ife->stats.tx_compressed);
#endif
    return 0;
}

static int check_if_name(char *name,char *name1)
{
    char *namelist[] = { "vlan2", "vlan3", "vlan4", "vlan5",
			 "nas0", "nas10", "nas1", "nas2", "nas3", "br0", "wlan0"};
    char *toname[] = { "Port1", "Port2", "Port3", "Port4",
		       "ATM0(Wan)", "ATM1(Vod1)", "ATM2(Vod2)", "ATM3(Vod3)", "ATM4(Vod4)", "LAN", "Wireless"};
    int i;

    //DPRINTF("size  %d \n", sizeof(namelist)/sizeof(char*));

    for(i=0;i<sizeof(namelist)/sizeof(char*);i++){
	if( !strcmp(name, namelist[i]) ){
	    strcpy(name1, toname[i]);
	    return 1; // OK
	}
    }
    return 0; // ERROR
}

int interface_status()
{
    char buf[512];
    char print_buf[512];
    struct interface ife;
    FILE *fh=NULL;
    //char *fmt="<tr><td>%s</td><td>%s</td></tr>\n";
    // bytes, packets, errors, dropped
    //char *fmt="<tr><td>%s</td><td>%lu</td><td>%lu</td><td>%lu</td><td>%lu</td><td>%lu</td><td>%lu</td><td>%lu</td><td>%lu</td></tr>";
    //char *strfmt="Name, Rx Bytes, Rx Packets, Rx Errors, Rx Droppd, Tx Bytes, Tx Packets, Tx Errors, Tx Dropped";
    char *header="%-14s%-14s%-14s%-14s%-14s%-14s%-14s%-14s%-14s\n";
    char *content="%-14s%-14Lu%-14Lu%-14lu%-14lu%-14Lu%-14Lu%-14lu%-14lu\n";

    fh = fopen(_PATH_PROCNET_DEV, "r");
    if (!fh) {
	    fprintf(stderr, "can't get net information\n");
	    return -1;
    }

    fgets(buf, sizeof buf, fh);	/* eat line */
    fgets(buf, sizeof buf, fh);

    printf(header,
	   "Name", "Rx Bytes", "Rx Packets", "Rx Errors", "Rx Droppd", "Tx Bytes", "Tx Packets", "Tx Errors", "Tx Dropped");

    // using version 3
    while (fgets(buf, sizeof buf, fh)) {
	char *s, name[IFNAMSIZ], name1[32];
	s = get_name(name, buf);
	//ife = add_interface(name);
	//printf("%s name\n", name);

	get_dev_fields(s, &ife);
	ife.statistics_valid = 1;

	if(check_if_name(name, name1)) {
		printf(content,
		    name1,
		    ife.stats.rx_bytes,
		    ife.stats.rx_packets,
		    ife.stats.rx_errors,
		    ife.stats.rx_dropped,
		    
		    ife.stats.tx_bytes,
		    ife.stats.tx_packets,
		    ife.stats.tx_errors,
		    ife.stats.tx_dropped
		);
	}
    }
    fclose(fh);
    return 0;
}
