
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <netdb.h>
#include <paths.h>
#include <signal.h>
#include <time.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/un.h>
#include <sys/param.h>

#include <linux/atm.h>
#include <linux/atmdev.h>
#include <asm/tc3162/tc3162_atm_ioc.h>


#include "cli.h"
#include "wan_cmd.h"
#include "wanproto.h"


int g_nAdslFdCli = -1;
int g_nAtmFdCli=-1;
const cmds_table wanCmds[] = {

	{"adsl", doadsl,  Normal_Level, 0, "adsl subcmd"},
	{"atm", doatm,  Normal_Level, 0, "atm subcmd"},
	{"proto", doProto, Normal_Level, 0, "proto subcmd"},
	{NULL,	  NULL,	 0,	  0, NULL},
};


static const cmds_table doadslSubcmds[] = {
	
	{"annex",  doadslannex,  Normal_Level,    1,  "wan adsl annex <a|b>"},
        {"chandata",  doadslchandata,  Normal_Level,    0,  "wan adsl chandata"},
	{"close",  doadslclose,  Normal_Level,   0,  "wan adsl close"},
        {"linestatus",  doadsllinestatus,  Normal_Level,    0,  "wan adsl linestatus"},
	{"linedatan",  doadsllinedatan,  Normal_Level,    0,  "wan adsl linedatan"},
	{"linedataf1",  doadsllinedataf,  Normal_Level,    0,  "wan adsl linedataf"},
        {"linedataf2p",  doadsllinedataf2p,  Normal_Level,    0,  "wan adsl2plus linedataf"},
        {"fwversion",  doadslfwversion,  Normal_Level,    0,  "wan adsl fwversion"},
     	{"opmode",  doadslopmode,  Normal_Level,    0,  "wan adsl opmode"},
        {"opencmd",  doadslopencmd,  Normal_Level,    1,  "wan adsl opencmd <gdmt|t1413|glite|multi|adsl2|adsl2p>"},
	{"perfdata",  doadslcount,  Normal_Level,    0,  "wan adsl perfdata"},
        {"reset",  doadslreset,  Normal_Level,    0,  "wan adsl reset"},
        {"rtsdbg",  doadslrtsdbg,  Debug_Level,    0,  "wan adsl rtsdbg <on|off>"},
        {"uptime",  doadsluptime,  Normal_Level,    0,  "wan adsl uptime"},
	{NULL,	NULL,	0,	0,	NULL}
};

static const cmds_table doatmSubcmds[] = {
	
	{"setoam",  doatmsetoam,  Normal_Level,    5,  "wan atm setoam <vpi> <vci> <f4/f5> <end to end> <function type>"},
	{NULL,	NULL,	0,	0,	NULL}
};


static const cmds_table doProtoSubcmds[] = {
	
	{"vod",  doVod,  Normal_Level,    3,  "wan proto vod <num> <vpi> <vci> <llc|vcmux>"},
	{"dhcp",  doDhcp,  Normal_Level,    3,  "wan proto dhcp <vpi> <vci> <llc|vcmux>"},
	{NULL,	NULL,	0,	0,	NULL}
};




/*	wan cli command*/
/***************************************************************************
 * Function Name: OpenTCDMT
 * Description  : Opens the dmt device.
 * Returns      : device handle if successsful or -1 if error
 ***************************************************************************/
static int OpenTCDmt_Cli( void )
{
    int nFd = open( "/dev/dmtdev", O_RDWR );

    if( nFd == -1 )
        printf( "DMT: open error %d\n", errno );
	
    return( nFd );
} /* OpenTCDMT */

void show_usage_atmcli(void)
{
      

      fprintf(stdout,"Usage : setoam <vpi> <vci> <f4/f5> <end to end> <function type>\n");
      fprintf(stdout,"vpi   : vpi number(0-255)\n");
      fprintf(stdout,"vci   : vci number(0-65535)\n");
      fprintf(stdout,"f4/f5 : 0 for OAM F4, 1 for OAM F5\n");
      fprintf(stdout,"end to end    : 0 for segment to segment, 1 for end to end\n");
      fprintf(stdout,"Function type : 0 for OAM_AIS, 1 for OAM_RDI, 2 for OAM_LOOPBACK, 3 for OAM_CC\n");
}


int doadsl(int argc,char *argv[],void	*pp)
{

       //printf("Enter adsl subcmd\n");

	return subcmd(doadslSubcmds, argc, argv, pp);


} /* doadsl */


int doatm(int argc,char *argv[],void	*pp)
{

     

	return subcmd(doatmSubcmds, argc, argv, pp);


} /* doadsl */



int doProto(int argc,char *argv[],void	*pp)
{
	return subcmd(doProtoSubcmds, argc, argv, pp);
} /* doadsl */






int doadslcount (int argc,char *argv[],void *p)
{
    DMTCTRL_LINE_INFO Arg;
    int ret=0;
    ret = DmtCtrl_STATUS_ERROR;

    if( g_nAdslFdCli == -1  )
        g_nAdslFdCli = OpenTCDmt_Cli();

   memset(&Arg,0,sizeof(DMTCTRL_LINE_INFO));
    if( g_nAdslFdCli != -1  )
    {
		
        ioctl( g_nAdslFdCli, DMTIOCTL_GET_LINE_INFO, &Arg );
        ret=DmtCtrl_STATUS_SUCCESS;
        
        printf("near-end FEC error fast: %d \n",Arg.near_end.FecNotInterleaved);
        printf("near-end FEC error interleaved: %d \n",Arg.near_end.FecInterleaved);
        printf("near-end CRC error fast: %d \n",Arg.near_end.CrcNotInterleaved);
        printf("near-end CRC error interleaved: %d \n",Arg.near_end.CrcInterleaved);
        printf("near-end HEC error fast: %d \n",Arg.near_end.HecNotInterleaved);
        printf("near-end HEC error interleaved: %d \n",Arg.near_end.HecInterleaved);
 
        printf("far-end FEC error fast: %d \n",Arg.far_end.FecNotInterleaved);
        printf("far-end FEC error interleaved: %d \n",Arg.far_end.FecInterleaved);
        printf("far-end CRC error fast: %d \n",Arg.far_end.CrcNotInterleaved);
        printf("far-end CRC error interleaved: %d \n",Arg.far_end.CrcInterleaved);
        printf("far-end HEC error fast: %d \n",Arg.far_end.HecNotInterleaved);
        printf("far-end HEC error interleaved: %d \n",Arg.far_end.HecInterleaved);
        
    }
    else
        ret = DmtCtrl_STATUS_ERROR;

    return( ret );
}

int doadslclose (int argc,char *argv[],void *p)
{
    unsigned char Arg;
    int ret=0;
    ret = DmtCtrl_STATUS_ERROR;

    if( g_nAdslFdCli == -1  )
        g_nAdslFdCli = OpenTCDmt_Cli();

    if( g_nAdslFdCli != -1  )
    {
		
        ioctl( g_nAdslFdCli, DMTIOCTL_SET_DMTCLOSE, &Arg );
        ret=DmtCtrl_STATUS_SUCCESS;
        
         
    }
    else
        ret = DmtCtrl_STATUS_ERROR;

    return( ret );
}
int doadslreset (int argc,char *argv[],void *p)
{
    unsigned char Arg;
    int ret=0;
    ret = DmtCtrl_STATUS_ERROR;

    if( g_nAdslFdCli == -1  )
        g_nAdslFdCli = OpenTCDmt_Cli();

    if( g_nAdslFdCli != -1  )
    {
        ioctl( g_nAdslFdCli, DMTIOCTL_SET_DMTRESET, &Arg );
        ret=DmtCtrl_STATUS_SUCCESS;
    }
    else
        ret = DmtCtrl_STATUS_ERROR;

    return( ret );
}

int doadsllinestatus (int argc,char *argv[],void *p)
{
   DMTCTRL_DMT_INFO Arg;
    int ret=0;
    ret = DmtCtrl_STATUS_ERROR;

    if( g_nAdslFdCli == -1  )
        g_nAdslFdCli = OpenTCDmt_Cli();

    if( g_nAdslFdCli != -1  )
    {
		
        ioctl( g_nAdslFdCli, DMTIOCTL_GET_DMT_INFO, &Arg );
        ret=DmtCtrl_STATUS_SUCCESS;
        
        
        if(Arg.dmt_status==ADSL_DOWN)
            printf("ADSL Line Down!\n");
        else if(Arg.dmt_status==ADSL_TRAINING_1 || Arg.dmt_status==ADSL_TRAINING_2)
            printf("ADSL Line Training!\n");
        else
            printf("ADSL Line Up!\n");
         
    }
    else
        ret = DmtCtrl_STATUS_ERROR;

    return( ret );
}
int doadslchandata (int argc,char *argv[],void *p)
{
    DMTCTRL_CONNECTION_INFO Arg;
    int ret=0;
    ret = DmtCtrl_STATUS_ERROR;

    if( g_nAdslFdCli == -1  )
        g_nAdslFdCli = OpenTCDmt_Cli();

    memset(&Arg,0,sizeof(DMTCTRL_CONNECTION_INFO));
    if( g_nAdslFdCli != -1  )
    {
		
        ioctl( g_nAdslFdCli, DMTIOCTL_GET_CONNECTION_INFO, &Arg );
        ret=DmtCtrl_STATUS_SUCCESS;
        
        printf("near-end interleaved channel bit rate: %d kbps\n",Arg.actualBitrateNearInt);
        printf("near-end fast channel bit rate: %d kbps\n",Arg.actualBitrateNearFast);
        printf("far-end interleaved channel bit rate: %d kbps\n",Arg.actualBitrateFarInt);
        printf("far-end fast channel bit rate: %d kbps\n",Arg.actualBitrateFarFast);
        
    }
    else
        ret = DmtCtrl_STATUS_ERROR;

    return( ret );
      
}

int doadsllinedatan (int argc,char *argv[],void *p)
{
    DMTCTRL_LINE_DATA_N_INFO Arg;
    int ret=0;
    unsigned int a2tmp=0, a3tmp=0, a4tmp=0;
    unsigned int a2tmpmod=0, a3tmpmod=0, a4tmpmod=0;
    DMTCTRL_DMT_INFO dmtstatus0;
    ret = DmtCtrl_STATUS_ERROR;

    if( g_nAdslFdCli == -1  )
        g_nAdslFdCli = OpenTCDmt_Cli();

    memset(&Arg,0,sizeof(DMTCTRL_LINE_DATA_N_INFO));
    if( g_nAdslFdCli != -1  )
    {
	ioctl( g_nAdslFdCli, DMTIOCTL_GET_DMT_INFO, &dmtstatus0 );
        if(dmtstatus0.dmt_status == ADSL_UP)
        {	
        ioctl( g_nAdslFdCli, DMTIOCTL_GET_LINE_DATA_N_INFO, &Arg );
        ret=DmtCtrl_STATUS_SUCCESS;
        
         
         a2tmp=Arg.noiseMarginDnstr/10;a3tmp=Arg.outputPowerUpstr/10;a4tmp=Arg.attenuationDnstr/10;
         a2tmpmod=Arg.noiseMarginDnstr % 10;a3tmpmod=Arg.outputPowerUpstr % 10; a4tmpmod= Arg.attenuationDnstr % 10;
    
         printf("relative capacity occupation: %d%%\n",Arg.relCapacityOccupationDnstr);
         printf("noise margin downstream: %d.%d db\n",a2tmp,a2tmpmod);
         printf("output power upstream: %d.%d dbm\n",a3tmp,a3tmpmod);
         printf("attenuation downstream: %d.%d db\n",a4tmp,a4tmpmod);
       }else
         printf("ADSL doesn't sync up!\n");
        
    }
    else
        ret = DmtCtrl_STATUS_ERROR;

    return( ret );
}

int doadsllinedataf (int argc,char *argv[],void *p)
{
    DMTCTRL_LINE_DATA_F_INFO_ADSL Arg;
    int ret=0,i=0,j=0;
    unsigned int a2tmp=0, a3tmp=0, a4tmp=0;
    unsigned int a2tmpmod=0, a3tmpmod=0, a4tmpmod=0;
    unsigned char adslmode;
    DMTCTRL_DMT_INFO dmtstatus1;
    ret = DmtCtrl_STATUS_ERROR;

    if( g_nAdslFdCli == -1  )
        g_nAdslFdCli = OpenTCDmt_Cli();

    memset(&Arg,0,sizeof(DMTCTRL_LINE_DATA_F_INFO_ADSL));
    if( g_nAdslFdCli != -1  )
    {
        ioctl( g_nAdslFdCli, DMTIOCTL_GET_DMT_INFO, &dmtstatus1 );
        if(dmtstatus1.dmt_status == ADSL_UP)
        {
	  ioctl( g_nAdslFdCli, DMTIOCTL_GET_LINE_OPINFO, &adslmode );
          if(adslmode != ADSL2Plus_Mode)
          {
           ioctl( g_nAdslFdCli, DMTIOCTL_GET_LINE_DATA_F_INFO_ADSL, &Arg );
           ret=DmtCtrl_STATUS_SUCCESS;
           a2tmp=Arg.noiseMarginUpstr/10;a3tmp=Arg.outputPowerDnstr/10;a4tmp=Arg.attenuationUpstr/10;
           a2tmpmod=Arg.noiseMarginUpstr % 10;a3tmpmod=Arg.outputPowerDnstr % 10; a4tmpmod= Arg.attenuationUpstr % 10;
    
           printf("relative capacity occupation: %d%%\n",Arg.relCapacityOccupationUpstr);
           printf("noise margin upstream: %d.%d db\n",a2tmp,a2tmpmod);
           printf("output power downstream: %d.%d dbm\n",a3tmp,a3tmpmod);
           printf("attenuation upstream: %d.%d db\n",a4tmp,a4tmpmod);
           printf("carrier load : number of bits per symbol(tone)\n");

           for( i=0;i<8;i++)
           {
            printf("tone  %3d-%3d:",i*32,i*32+31);
            for(j=i*16;j<((i*16)+16);j++)
            {
             printf("%1x", Arg.carrierLoad[j] & 0xF );
	     printf("%1x ",(Arg.carrierLoad[j] & 0xF0) >> 4);
	    }
            printf("\n"); 
           }
            printf("\n");
           }else
              {
               printf("Operation Mode is not ADSL/ADSL2\n");
               ret=DmtCtrl_STATUS_SUCCESS;
              }
        }else
          printf("ADSL doesn't sync up!\n");
    }
    else
        ret = DmtCtrl_STATUS_ERROR;

    return( ret );
}

int doadsllinedataf2p (int argc,char *argv[],void *p)
{
    DMTCTRL_LINE_DATA_F_INFO_ADSL2P Arg;
    int ret=0,i=0,j=0;
    unsigned int a2tmp=0, a3tmp=0, a4tmp=0;
    unsigned int a2tmpmod=0, a3tmpmod=0, a4tmpmod=0;
    unsigned char adslmode;
   
    DMTCTRL_DMT_INFO dmtstatus2;
    ret = DmtCtrl_STATUS_ERROR;

   memset(&Arg,0,sizeof(DMTCTRL_LINE_DATA_F_INFO_ADSL2P));
    if( g_nAdslFdCli == -1  )
        g_nAdslFdCli = OpenTCDmt_Cli();

    if( g_nAdslFdCli != -1  )
    {
        ioctl( g_nAdslFdCli, DMTIOCTL_GET_DMT_INFO, &dmtstatus2 );
        if(dmtstatus2.dmt_status == ADSL_UP)
        {
	  ioctl( g_nAdslFdCli, DMTIOCTL_GET_LINE_OPINFO, &adslmode );
          if(adslmode == ADSL2Plus_Mode)
          {
           ioctl( g_nAdslFdCli, DMTIOCTL_GET_LINE_DATA_F_INFO_ADSL2P, &Arg );
           ret=DmtCtrl_STATUS_SUCCESS;
           a2tmp=Arg.noiseMarginUpstr/10;a3tmp=Arg.outputPowerDnstr/10;a4tmp=Arg.attenuationUpstr/10;
           a2tmpmod=Arg.noiseMarginUpstr % 10;a3tmpmod=Arg.outputPowerDnstr % 10; a4tmpmod= Arg.attenuationUpstr % 10;
    
           printf("relative capacity occupation: %d%%\n",Arg.relCapacityOccupationUpstr);
           printf("noise margin upstream: %d.%d db\n",a2tmp,a2tmpmod);
           printf("output power downstream: %d.%d dbm\n",a3tmp,a3tmpmod);
           printf("attenuation upstream: %d.%d db\n",a4tmp,a4tmpmod);
         
           printf("carrier load : number of bits per symbol(tone)\n");

           for( i=0;i<16;i++)
           {
            printf("tone  %3d-%3d:",i*32,i*32+31);
            for(j=i*16;j<((i*16)+16);j++)
            {
             printf("%1x", Arg.carrierLoad[j] & 0xF );
	     printf("%1x ",(Arg.carrierLoad[j] & 0xF0) >> 4);
	    }
           printf("\n"); 
           }
              printf("\n");
          
        } else
          {
           printf("Operation Mode is not ADSL2+\n");
            ret=DmtCtrl_STATUS_SUCCESS;
          }
       }else
          printf("ADSL doesn't sync up!\n");
    }
    else
        ret = DmtCtrl_STATUS_ERROR;

    return( ret );
}

int doadslopmode (int argc,char *argv[],void *p)
{
    unsigned char Arg;
    int ret=0;
    ret = DmtCtrl_STATUS_ERROR;

    if( g_nAdslFdCli == -1  )
        g_nAdslFdCli = OpenTCDmt_Cli();

    if( g_nAdslFdCli != -1  )
    {
	ioctl( g_nAdslFdCli, DMTIOCTL_GET_LINE_OPINFO, &Arg );
        ret=DmtCtrl_STATUS_SUCCESS;
         switch(Arg)
          {
           case T1413_Mode:
                printf("operational mode:ITU G.992.1(T1.413)\n");
                break;
           case GDMT_Mode:
                printf("operational mode:ITU G.992.1(G.DMT)\n");
                break;
           case GLite_Mode:
                printf("operational mode:ITU G.992.1(G.lite)\n");
                break;
           case ADSL2_Mode:
                printf("operational mode:ITU G.992.3(ADSL2)\n");
                break;
           case ADSL2Plus_Mode:
                printf("operational mode:ITU G.992.5(ADSL2Plus)\n");
                break;
           default:
                break;
          }
         
    }
    else
        ret = DmtCtrl_STATUS_ERROR;

    return( ret );
}

int doadslopencmd (int argc,char *argv[],void *p)
{
     int ret=0;
     char *cmd=NULL; 
     unsigned int Arg=0; 
     ret = DmtCtrl_STATUS_ERROR;

    if( g_nAdslFdCli == -1  )
        g_nAdslFdCli = OpenTCDmt_Cli();

    if( g_nAdslFdCli != -1  )
    {
       if(argc <=1 )
       {
          printf(" not enough arg\n");
          printf("Usage: wan adsl opencmd <gdmt|t1413|glite|adsl1_multi|multi|adsl2|adsl2p>\n");
          return 0;
       }
       else
       {
          cmd=argv[1];
          if( strcasecmp(cmd, "gdmt") == 0)
            {
              Arg=GDMT;
              ioctl( g_nAdslFdCli, DMTIOCTL_SET_LINE_OPMODE , &Arg );
            } 
          else if( strcasecmp(cmd, "glite") == 0)
            {
              Arg=GLite;
              ioctl( g_nAdslFdCli, DMTIOCTL_SET_LINE_OPMODE , &Arg );
            } 
          else if( strcasecmp(cmd, "t1413") == 0)
            {
              Arg=ANSI;
              ioctl( g_nAdslFdCli, DMTIOCTL_SET_LINE_OPMODE , &Arg );
            } 
          else if (strcasecmp(cmd, "multi") == 0)
            {
              Arg=Multi;
              ioctl( g_nAdslFdCli, DMTIOCTL_SET_LINE_OPMODE , &Arg );
            } 
          else if (strcasecmp(cmd, "adsl2") == 0)
            {
              Arg=ADSL2;
              ioctl( g_nAdslFdCli, DMTIOCTL_SET_LINE_OPMODE , &Arg );
            }  
          else if (strcasecmp(cmd, "adsl2p") == 0)
           {
              Arg=ADSL2Plus;
              ioctl( g_nAdslFdCli, DMTIOCTL_SET_LINE_OPMODE , &Arg );
           } 
          else if (strcasecmp(cmd, "adsl1_multi") == 0)
           {
              Arg=ADSL1_MultiMode;
              ioctl( g_nAdslFdCli, DMTIOCTL_SET_LINE_OPMODE , &Arg );
           } 
          else
             {
              printf("no such arg available\n");
              printf("Usage: wan adsl opencmd <gdmt|t1413|glite|adsl1_multi|multi|adsl2|adsl2p>\n"); 
             }
      } 

     ret=DmtCtrl_STATUS_SUCCESS;
    }
    else
        ret = DmtCtrl_STATUS_ERROR;

    return( ret ); 
}


int doadslannex (int argc,char *argv[],void *p)
{
     int ret=0;
     char *cmd=NULL; 
     unsigned char Arg=0; 
     ret = DmtCtrl_STATUS_ERROR;

    if( g_nAdslFdCli == -1  )
        g_nAdslFdCli = OpenTCDmt_Cli();

    if( g_nAdslFdCli != -1  )
    {
       if(argc <=1 )
       {
          printf(" not enough arg\n");
          printf("Usage: wan adsl annex <a|b|i|j|al|bjm|aijlm|>\n");
          return 0;
       }
       else
       {
          cmd=argv[1];
          if( strcasecmp(cmd, "a") == 0)
           {
             Arg=ANNEX_A;
             ioctl( g_nAdslFdCli, DMTIOCTL_SET_ANNEX , &Arg );
           }
          else if( strcasecmp(cmd, "b") == 0)
           {
             Arg=ANNEX_B;
             ioctl( g_nAdslFdCli, DMTIOCTL_SET_ANNEX , &Arg );
           }
          else if( strcasecmp(cmd, "i") == 0)
           {
             Arg=ANNEX_I;
             ioctl( g_nAdslFdCli, DMTIOCTL_SET_ANNEX , &Arg );
           }
          else if( strcasecmp(cmd, "j") == 0)
           {
             Arg=ANNEX_J;
             ioctl( g_nAdslFdCli, DMTIOCTL_SET_ANNEX , &Arg );
           } 
          else if( strcasecmp(cmd, "al") == 0)
           {
             Arg=ANNEX_L;
             ioctl( g_nAdslFdCli, DMTIOCTL_SET_ANNEX , &Arg );
           } 
          else if( strcasecmp(cmd, "m") == 0)
           {
             Arg=ANNEX_M;
             ioctl( g_nAdslFdCli, DMTIOCTL_SET_ANNEX , &Arg );
           } 
          else if( strcasecmp(cmd, "bjm") == 0)
           {
             Arg=ANNEX_BJM;
             ioctl( g_nAdslFdCli, DMTIOCTL_SET_ANNEX , &Arg );
           } 
          else if( strcasecmp(cmd, "aijlm") == 0)
           {
             Arg=ANNEX_AIJLM;
             ioctl( g_nAdslFdCli, DMTIOCTL_SET_ANNEX , &Arg );
           } 
          else
             {
              printf("no such arg available\n");
              printf("Usage: wan adsl annex <a|b>\n"); 
             }
      } 

     ret=DmtCtrl_STATUS_SUCCESS;
    }
    else
        ret = DmtCtrl_STATUS_ERROR;

    return( ret ); 
}


int doadslrtsdbg (int argc,char *argv[],void *p)
{
     int ret=0;
     char *cmd=NULL; 
     unsigned char Arg=0; 
     ret = DmtCtrl_STATUS_ERROR;

    if( g_nAdslFdCli == -1  )
        g_nAdslFdCli = OpenTCDmt_Cli();

    if( g_nAdslFdCli != -1  )
    {
       if(argc <=1 )
       {
          printf(" not enough arg\n");
          printf("Usage: wan adsl rtsdbg <on|off>\n");
          return 0;
       }
       else
       {
          cmd=argv[1];
          if( strcasecmp(cmd, "on") == 0)
           {
             Arg=DMTRTSON;
             ioctl( g_nAdslFdCli, DMTIOCTL_SET_RTS_DBG , &Arg );
           }
          else if( strcasecmp(cmd, "off") == 0)
           {
             Arg=DMTRTSOFF;
             ioctl( g_nAdslFdCli, DMTIOCTL_SET_RTS_DBG , &Arg );
           }
          else
             {
              printf("no such arg available\n");
              printf("Usage: wan adsl rtsdbg <on|off>\n"); 
             }
      } 

     ret=DmtCtrl_STATUS_SUCCESS;
    }
    else
        ret = DmtCtrl_STATUS_ERROR;

    return( ret ); 
}




int doadslfwversion (int argc,char *argv[],void *p)
{
    DMTCTRL_DMT_VERSION Arg;
    int ret=0;

    ret = DmtCtrl_STATUS_ERROR;

    if( g_nAdslFdCli == -1  )
        g_nAdslFdCli = OpenTCDmt_Cli();

    if( g_nAdslFdCli != -1  )
    {
		
        ioctl( g_nAdslFdCli, DMTIOCTL_GET_DMT_FW_VERSION, &Arg );
        ret=DmtCtrl_STATUS_SUCCESS;
        printf("DMT FwVer: %d.%d.%d.%d_%c_%s, HwVer:T%dF%d_%d.%d\n",Arg.fw_ver_r,Arg.fw_ver_p,Arg.fw_rev,Arg.b_ver,(Arg.annex_afe & 0x02)?'A':'B',(Arg.annex_afe & 0x01)?"TC":"SI",Arg.tdm_ver,Arg.fdm_ver,Arg.major,Arg.minor);
        
    }
    else
        ret = DmtCtrl_STATUS_ERROR;

    return( ret );
}

int doadsluptime (int argc,char *argv[],void *p)
{
    DMTCTRL_ADSL_UPTIME DmtUptime;
    unsigned long int Arg=4294967295UL;
    int ret=0;
    
    unsigned int hr=0;
    unsigned int mm=0;
    unsigned int sstemp=0;
    unsigned int dd=0;
    unsigned int ss=0;
    ret = DmtCtrl_STATUS_ERROR;

    DmtUptime.showtime_stamp=0;
    DmtUptime.currenttime_stamp=0;
    //printf("Arg=%08x, bytes %d\n",Arg, sizeof(Arg));
    if( g_nAdslFdCli == -1  )
        g_nAdslFdCli = OpenTCDmt_Cli();

    if( g_nAdslFdCli != -1  )
    {
	ioctl( g_nAdslFdCli, DMTIOCTL_GET_LINE_UPTIME, &DmtUptime );
        ret=DmtCtrl_STATUS_SUCCESS;
        
        if(DmtUptime.dmtstatus ==ADSL_UP)
        {
        if(DmtUptime.currenttime_stamp > DmtUptime.showtime_stamp)
         {
            sstemp= DmtUptime.currenttime_stamp - DmtUptime.showtime_stamp;
            hr=sstemp/3600;
            ss= sstemp % 3600;
            mm= ss/60;
            ss=ss % 60;
            dd=hr/24;
            hr=hr % 24;
            
         }
         if( DmtUptime.currenttime_stamp < DmtUptime.showtime_stamp)
          {
            sstemp=Arg-DmtUptime.showtime_stamp;
            sstemp=sstemp + DmtUptime.currenttime_stamp;
            hr=sstemp/3600;
            ss= sstemp % 3600;
            mm= ss/60;
            ss=ss % 60;
            dd=hr/24;
            hr=hr % 24;
         }
        }
        printf("ADSL uptime: %d day %d hr %d min %d sec\n",dd,hr,mm,ss);
            
    }
    else
        ret = DmtCtrl_STATUS_ERROR;

    return( ret );
}




int doatmsetoam (int argc,char *argv[],void *p)
{
     int ret=0;
     char *cmd=NULL; 
     unsigned int Arg=0; 
     int ioctl_ret,oam_cmd_type_cli;
     int sock_id;
	struct atmif_sioc atmioc;
     if(argc < 6)
       {
          printf("Not enough args\n");
          show_usage_atmcli();
          return DmtCtrl_STATUS_ERROR;
       } 
     oamdatareq_t oamdatareqcli;
     oam_cmd_type_cli = 0;
     oamdatareqcli.vpi = atoi(argv[1]);
     oamdatareqcli.vci = atoi(argv[2]);
     oamdatareqcli.f5  = atoi(argv[3]);
     oamdatareqcli.endtoEnd =atoi(argv[4]);
     oamdatareqcli.funcType =atoi(argv[5]);
     printf("args: %d %d %d %d %d\n",oamdatareqcli.vpi,oamdatareqcli.vci,oamdatareqcli.f5,oamdatareqcli.endtoEnd,oamdatareqcli.funcType);
    if ( oamdatareqcli.vpi < 0 || oamdatareqcli.vpi > 255 || oamdatareqcli.vci < 0 || oamdatareqcli.vci > 65535 )
	{
	  show_usage_atmcli();
	  return -EINVAL;
	}
    if ((oamdatareqcli.f5 != 0 && oamdatareqcli.f5 !=1 )||(oamdatareqcli.endtoEnd != 0 && oamdatareqcli.endtoEnd != 1))
      {
	show_usage_atmcli();
        return -EINVAL;
      }
    if (oamdatareqcli.funcType < 0 || oamdatareqcli.funcType > 3)
     {
	show_usage_atmcli();
	return -EINVAL;
     }
       atmioc.number = 0;
       atmioc.arg = &oamdatareqcli;
       atmioc.length = sizeof(oamdatareqcli);
                                                                                                                                               
       sock_id = socket(AF_ATMPVC,SOCK_DGRAM,ATM_AAL5);
       if (sock_id < 0)
        {
	  fprintf(stdout,"Can not create ATM ioctl socket\n");
	  return -ENODEV;
	}
	ioctl_ret = ioctl( sock_id, ATM_SETOAM, &atmioc);
	//fprintf(stdout,"ioctl return value is %d \n",ioctl_ret);
	close(sock_id);
  
    return( ret ); 
}



