/* Albert Sep.-Oct. 2005 */
#include <stdio.h>
#include "wanproto.h"

/* begin dovod */
static void Load_Wan_Vod_Config(void);
static void Save_Wan_Vod_Config(const char *pathname, WAN_CONFIG_DATA *d);

#define BLANK_SYMBOL "&"

static int load_blank_string(void *d)
{
	if (strlen(d)==0)
              strcpy(d,BLANK_SYMBOL);
	return 1;
}

static int print_blank_string(void *d)
{
	if (strcmp(d,BLANK_SYMBOL)==0)
              strcpy(d,"");
	return 1;
}

// albert, 2005.02.21
// albert, 2005.02.18
// | 0         | 1 | 2 |  3     |      4      |   5        |      6           | 7 | 8 | 9 |
// |Description|vpi|vci|atmclass|encapsulation|ether filter|spanning interface|PCR|SCR|Max|
// |1483_Bridge_mode|0|0|0|0|0|0|0|0|0|
static void Load_WanX_Config (const char *pathname, WAN_CONFIG_DATA *d)
{
	FILE *fp;
	char line[1024];
	
	if ((fp=fopen(pathname, "r"))!=NULL) {
	     //1483 Bridged
	     if (fgets(line, 1024, fp))
	     {
		  sscanf(line,
			 "|%50[^|]|%hu|%hu|%d|%d|%d|%d|%hu|%hu|%hu|\n",
			 d->Bridged_Description,
			 &d->Bridged_VPI,
			 &d->Bridged_VCI,
			 (int *)&d->Bridged_ATM_Class,
			 (int *)&d->Bridged_Encapsulation_Method,
			 (int *)&d->Bridged_Ether_Filter_Type,
			 (int *)&d->Bridged_Spanning_Bridge_Interface,
			 &d->Bridged_PCR,
			 &d->Bridged_SCR,
			 &d->Bridged_MBS
		       );
	     }
	     print_blank_string(&d->Bridged_Description);
//	     DPRINTF("desc %s\n", d->Bridged_Description);
//	     DPRINTF("vpi %d\n", d->Bridged_VPI);
//	     DPRINTF("vci %d\n", d->Bridged_VCI);
//	     DPRINTF("class %d\n", d->Bridged_ATM_Class);
//	     DPRINTF("enc %d\n", d->Bridged_Encapsulation_Method);
//	     DPRINTF("eth %d\n", d->Bridged_Ether_Filter_Type);
//	     DPRINTF("spa %d\n", d->Bridged_Spanning_Bridge_Interface);
	     fclose(fp);
	}
}

static void Save_Wan_Vod_Config (const char *pathname, WAN_CONFIG_DATA *d)
{
	FILE *fp;

	if((fp=fopen(pathname, "w"))!=NULL) {
	     //1483 Bridged
	     load_blank_string(&d->Bridged_Description);
	     fprintf(fp,
		     "|%s|%hu|%hu|%d|%d|%d|%d|%hu|%hu|%hu|\n",
		     d->Bridged_Description,
		     d->Bridged_VPI,
		     d->Bridged_VCI,
		     d->Bridged_ATM_Class,
		     d->Bridged_Encapsulation_Method,
		     d->Bridged_Ether_Filter_Type,
		     d->Bridged_Spanning_Bridge_Interface,
		     d->Bridged_PCR,
		     d->Bridged_SCR,
		     d->Bridged_MBS
		  );

	     fclose(fp);
	}
}

static WAN_CONFIG_DATA* WanConfig[]= {
  &wan0_config, &wan1_config, &wan2_config, &wan3_config
};

static char *vod_start[] = {
	VOD_0_START_SCRIPT,
	VOD_1_START_SCRIPT,
  	VOD_2_START_SCRIPT, 
	VOD_3_START_SCRIPT
};

static char *vod_stop[] = {
	VOD_0_STOP_SCRIPT,
	VOD_1_STOP_SCRIPT,
  	VOD_2_STOP_SCRIPT, 
	VOD_3_STOP_SCRIPT
};

static char *vod_config[] = {
	WAN_0_CONFIG_FILE,
	WAN_1_CONFIG_FILE,
	WAN_2_CONFIG_FILE,
	WAN_3_CONFIG_FILE
};

static char *enc[]={"0","1"};

int doVod (int argc,char *argv[],void *p)
{
	int encap=0, vod_num=0;
	if(argc != 5){
		printf(VOD_USAGE_MESSAGES);
		return 0;
	}

	vod_num = VOD_NUM;
	if(CHK_RANGE_VOD_NUM(vod_num)){
		printf(VOD_NUM_MESSAGES);
		return 0;
	}

	if(CHK_RANGE_VPI(VOD_VPI)){
		printf(VOD_VPI_MESSAGES);
		return 0;
	}

	if(CHK_RANGE_VCI(VOD_VCI)){
		printf(VOD_VCI_MESSAGES);
		return 0;
	}

	if(IS_ENC_LLC(VOD_ENC_S)){
		encap=0;
	}else if(IS_ENC_VC(VOD_ENC_S)){
		encap=1;
	}else{
		printf(VOD_ENC_MESSAGES);
		return 0;
	}

	do_command(vod_stop[vod_num], vod_stop[vod_num], 0);
	do_command(vod_start[vod_num], vod_start[vod_num], VOD_VPI_S, VOD_VCI_S, enc[encap], 0);

	Load_WanX_Config (vod_config[vod_num], WanConfig[vod_num]);
	WanConfig[vod_num]->Bridged_VPI = VOD_VPI;
	WanConfig[vod_num]->Bridged_VCI = VOD_VCI;
	WanConfig[vod_num]->Bridged_Encapsulation_Method = encap;
	Save_Wan_Vod_Config(vod_config[vod_num], WanConfig[vod_num]);
	return 0;
}
/* end dovod */

/* begin wan */
static void Load_Wan_Config (void)
{
	FILE *fp;
	char line[1024];
	//resetCurrentWanCfg();
	
	if ((fp=fopen(WAN_CONFIG_FILE,"r"))!=NULL)
	{
	if (fgets(line, 1024, fp))
		sscanf(line,"#:%d:%d\n",(int *)&wan_config.Wan_Type,(int *)&wan_config.Quick_Wan_Type);
		
	//1483 Routed
	if (fgets(line, 1024, fp))
	{
		sscanf(line,
		"|%50[^|]|%hu|%hu|%d|%d|%d|%d|%20[^|]|%20[^|]|%20[^|]|%d|%hu|\n",
		wan_config.Routed_Description,
		&wan_config.Routed_VPI,
		&wan_config.Routed_VCI,
		(int *)&wan_config.Routed_ATM_Class,
		(int *)&wan_config.Routed_NAT,
        	(int *)&wan_config.Routed_Encapsulation_Method,
	        (int *)&wan_config.Routed_IP_Assignment,
		wan_config.Routed_IP,
		wan_config.Routed_Netmask,
		wan_config.Routed_Gateway,
		(int *)&wan_config.Routed_RIP,
		&wan_config.Routed_MTU
		);
	}
	print_blank_string(&wan_config.Routed_Description);
	print_blank_string(&wan_config.Routed_IP);
	print_blank_string(&wan_config.Routed_Netmask);
	print_blank_string(&wan_config.Routed_Gateway);
	
	//1483 Bridged
	if (fgets(line, 1024, fp))
	{
		sscanf(line,
		"|%50[^|]|%hu|%hu|%d|%d|%d|%d|\n",
		wan_config.Bridged_Description,
		&wan_config.Bridged_VPI,
		&wan_config.Bridged_VCI,
		(int *)&wan_config.Bridged_ATM_Class,
		(int *)&wan_config.Bridged_Encapsulation_Method,
        	(int *)&wan_config.Bridged_Ether_Filter_Type,
	        (int *)&wan_config.Bridged_Spanning_Bridge_Interface
		);
	}
	
	print_blank_string(&wan_config.Bridged_Description);	
	//PPPOA
	if (fgets(line, 1024, fp))
	{
		sscanf(line,
		"|%50[^|]|%hu|%hu|%d|%d|%256[^|]|%256[^|]|%20[^|]|%d|%d|%hu|%d|%hu|\n",
		wan_config.PPPOA_Description,
		&wan_config.PPPOA_VPI,
		&wan_config.PPPOA_VCI,
		(int *)&wan_config.PPPOA_ATM_Class,
		(int *)&wan_config.PPPOA_NAT,
		wan_config.PPPOA_Username,
		wan_config.PPPOA_Password,
		wan_config.PPPOA_IP,
	        (int *)&wan_config.PPPOA_Authentication_Protocol,
	        (int *)&wan_config.PPPOA_Connection,
	        &wan_config.PPPOA_Idle_Timeout,
		(int *)&wan_config.PPPOA_RIP,
		&wan_config.PPPOA_MTU
		);
	}
	
	print_blank_string(&wan_config.PPPOA_Description);
	print_blank_string(&wan_config.PPPOA_Username);
	print_blank_string(&wan_config.PPPOA_Password);
	print_blank_string(&wan_config.PPPOA_IP);	
	//IPOA
	if (fgets(line, 1024, fp))
	{
		sscanf(line,"|%50[^|]|%hu|%hu|%d|%d|%d|%20[^|]|%20[^|]|%20[^|]|%d|%hu|\n",wan_config.IPOA_Description,
		&wan_config.IPOA_VPI,
		&wan_config.IPOA_VCI,
		(int *)&wan_config.IPOA_ATM_Class,
		(int *)&wan_config.IPOA_NAT,
	        (int *)&wan_config.IPOA_IP_Assignment,
		wan_config.IPOA_IP,
		wan_config.IPOA_Netmask,
		wan_config.IPOA_Gateway,
		(int *)&wan_config.IPOA_RIP,
		&wan_config.IPOA_MTU
		);
	}
	
	print_blank_string(&wan_config.IPOA_Description);
	print_blank_string(&wan_config.IPOA_IP);
	print_blank_string(&wan_config.IPOA_Netmask);
	print_blank_string(&wan_config.IPOA_Gateway);	
	//PPPOE
	if (fgets(line, 1024, fp))
	{
		sscanf(line,
		"|%50[^|]|%hu|%hu|%d|%d|%256[^|]|%256[^|]|%32[^|]|%20[^|]|%d|%d|%hu|%d|%hu|%d|\n",
		wan_config.PPPOE_Description,
		&wan_config.PPPOE_VPI,
		&wan_config.PPPOE_VCI,
		(int *)&wan_config.PPPOE_ATM_Class,
		(int *)&wan_config.PPPOE_NAT,
		wan_config.PPPOE_Username,
		wan_config.PPPOE_Password,
		wan_config.PPPOE_Service_Name,
		wan_config.PPPOE_IP,
	        (int *)&wan_config.PPPOE_Authentication_Protocol,
	        (int *)&wan_config.PPPOE_Connection,
	        &wan_config.PPPOE_Idle_Timeout,
		(int *)&wan_config.PPPOE_RIP,
	        &wan_config.PPPOE_MTU,
		&wan_config.PPPOE_RELAY
		);
	}
	print_blank_string(&wan_config.PPPOE_Description);
	print_blank_string(&wan_config.PPPOE_Username);
	print_blank_string(&wan_config.PPPOE_Password);
	print_blank_string(&wan_config.PPPOE_Service_Name);
	print_blank_string(&wan_config.PPPOE_IP);
	fclose(fp);
	}
}

void Save_Wan_Config (int i,WAN_CONFIG_DATA *d)
{
	
	FILE *fp;
	d->Wan_Type=i;
	if((fp=fopen(WAN_CONFIG_FILE,"w"))!=NULL)
	{
	fprintf(fp, "#:%d:%d\n",d->Wan_Type,d->Quick_Wan_Type);
	//1483 Routed
	load_blank_string(&d->Routed_Description);
	load_blank_string(&d->Routed_IP);
	load_blank_string(&d->Routed_Netmask);
	load_blank_string(&d->Routed_Gateway);
	fprintf(fp,
		"|%s|%hu|%hu|%d|%d|%d|%d|%s|%s|%s|%d|%hu|\n",
		d->Routed_Description,
		d->Routed_VPI,
		d->Routed_VCI,
		d->Routed_ATM_Class,
		d->Routed_NAT,
        	d->Routed_Encapsulation_Method,
	        d->Routed_IP_Assignment,
		d->Routed_IP,
		d->Routed_Netmask,
		d->Routed_Gateway,
		d->Routed_RIP,
		d->Routed_MTU
		);
	
	load_blank_string(&d->Bridged_Description);	
	//1483 Bridged
	fprintf(fp,
		"|%s|%hu|%hu|%d|%d|%d|%d|\n",
		d->Bridged_Description,
		d->Bridged_VPI,
		d->Bridged_VCI,
		d->Bridged_ATM_Class,
		d->Bridged_Encapsulation_Method,
        	d->Bridged_Ether_Filter_Type,
	        d->Bridged_Spanning_Bridge_Interface
		);

	load_blank_string(&d->PPPOA_Description);
	load_blank_string(&d->PPPOA_Username);
	load_blank_string(&d->PPPOA_Password);
	load_blank_string(&d->PPPOA_IP);
	//PPPOA
	fprintf(fp,
		"|%s|%hu|%hu|%d|%d|%s|%s|%s|%d|%d|%hu|%d|%hu|\n",
		d->PPPOA_Description,
		d->PPPOA_VPI,
		d->PPPOA_VCI,
		d->PPPOA_ATM_Class,
		d->PPPOA_NAT,
		d->PPPOA_Username,
		d->PPPOA_Password,
		d->PPPOA_IP,
	        d->PPPOA_Authentication_Protocol,
	        d->PPPOA_Connection,
	        d->PPPOA_Idle_Timeout,
		d->PPPOA_RIP,
		d->PPPOA_MTU
		);
	
	load_blank_string(&d->IPOA_Description);
	load_blank_string(&d->IPOA_IP);
	load_blank_string(&d->IPOA_Netmask);
	load_blank_string(&d->IPOA_Gateway);
	//IPOA
	fprintf(fp,
		"|%s|%hu|%hu|%d|%d|%d|%s|%s|%s|%d|%hu|\n",
		d->IPOA_Description,
		d->IPOA_VPI,
		d->IPOA_VCI,
		d->IPOA_ATM_Class,
		d->IPOA_NAT,
	        d->IPOA_IP_Assignment,
		d->IPOA_IP,
		d->IPOA_Netmask,
		d->IPOA_Gateway,
		d->IPOA_RIP,
		d->IPOA_MTU
		);
		
	//PPPOE
	load_blank_string(&d->PPPOE_Description);
	load_blank_string(&d->PPPOE_Username);
	load_blank_string(&d->PPPOE_Password);
	load_blank_string(&d->PPPOE_Service_Name);
	load_blank_string(&d->PPPOE_IP);
	fprintf(fp,
		"|%s|%hu|%hu|%d|%d|%s|%s|%s|%s|%d|%d|%hu|%d|%hu|%d|\n",
		d->PPPOE_Description,
		d->PPPOE_VPI,
		d->PPPOE_VCI,
		d->PPPOE_ATM_Class,
		d->PPPOE_NAT,
		d->PPPOE_Username,
		d->PPPOE_Password,
		d->PPPOE_Service_Name,
		d->PPPOE_IP,
	        d->PPPOE_Authentication_Protocol,
	        d->PPPOE_Connection,
	        d->PPPOE_Idle_Timeout,
		d->PPPOE_RIP,
		d->PPPOE_MTU,
		d->PPPOE_RELAY
		);
	fclose(fp);
	}
//	d->Quick_Wan_Type=d->Wan_Type;
}
/* end wan */

/* begin dodhcp */
int doDhcp(int argc,char *argv[],void *p)
{
	int encap=0;

	if(argc != 4){
		printf(DHCP_USAGE_MESSAGES);
		return 0;
	}

	if(CHK_RANGE_VPI(DHCP_VPI)) {
		printf(DHCP_VPI_MESSAGES);
		return 0;
	}

	if(CHK_RANGE_VCI(DHCP_VCI)){
		printf(DHCP_VCI_MESSAGES);
		return 0;
	}

	if(IS_ENC_LLC(DHCP_ENC_S)){
		encap=0;
	}else if(IS_ENC_VC(DHCP_ENC_S)){
		encap=1;
	}else{
		printf(DHCP_ENC_MESSAGES);
		return 0;
	}
	Load_Wan_Config ();
	wan_config.Routed_VPI = DHCP_VPI;
	wan_config.Routed_VCI = DHCP_VCI;
	wan_config.Routed_Encapsulation_Method = encap;
	Save_Wan_Config (1,&wan_config);
	return 0;
}
/* end dodhcp */
