
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <netdb.h>
#include <paths.h>
#include <signal.h>
#include <stdarg.h>
#include <time.h>
#include <string.h>

#include <sys/socket.h>
#include <sys/types.h>
#include <sys/un.h>
#include <sys/param.h>
#include <linux/tc3162dmtctrl.h>
#include <fcntl.h>      /* open */ 
#include <unistd.h>     /* exit */
#include <sys/ioctl.h>  /* ioctl */
#define MAXLINE         1024            /* maximum line length */


/* Globals. */
int g_nAdslFd = -1;
void childprocess (void);


static void dmtsignal(int sig)
{

	printf("receive signal\n");	
	
}

/***************************************************************************
 * Function Name: OpenTCDMT
 * Description  : Opens the dmt device.
 * Returns      : device handle if successsful or -1 if error
 ***************************************************************************/
static int OpenTCDmt( void )
{
    int nFd = open( "/dev/dmtdev", O_RDWR );

    if( nFd == -1 )
        printf( "DMT: open error %d\n", errno );
	
    return( nFd );
} /* OpenTCDMT */


static int TCDMT_doadsluptime (void)
{
    DMTCTRL_ADSL_UPTIME DmtUptime;
    unsigned long int Arg=4294967295UL;
    int ret=0;
    
    unsigned int hr=0;
    unsigned int mm=0;
    unsigned int sstemp=0;
    unsigned int dd=0;
    unsigned int ss=0;
    ret = DmtCtrl_STATUS_ERROR;

    DmtUptime.showtime_stamp=0;
    DmtUptime.currenttime_stamp=0;
    //printf("Arg=%08x, bytes %d\n",Arg, sizeof(Arg));
    if( g_nAdslFd == -1  )
        g_nAdslFd = OpenTCDmt();

    if( g_nAdslFd != -1  )
    {
	ioctl( g_nAdslFd, DMTIOCTL_GET_LINE_UPTIME, &DmtUptime );
        ret=DmtCtrl_STATUS_SUCCESS;
        
        if(DmtUptime.dmtstatus ==ADSL_UP)
        {
        if(DmtUptime.currenttime_stamp > DmtUptime.showtime_stamp)
         {
            sstemp= DmtUptime.currenttime_stamp - DmtUptime.showtime_stamp;
            hr=sstemp/3600;
            ss= sstemp % 3600;
            mm= ss/60;
            ss=ss % 60;
            dd=hr/24;
            hr=hr % 24;
            
         }
         if( DmtUptime.currenttime_stamp < DmtUptime.showtime_stamp)
          {
            sstemp=Arg-DmtUptime.showtime_stamp;
            sstemp=sstemp + DmtUptime.currenttime_stamp;
            hr=sstemp/3600;
            ss= sstemp % 3600;
            mm= ss/60;
            ss=ss % 60;
            dd=hr/24;
            hr=hr % 24;
         }
        }
        printf("ADSL uptime: %d day %d hr %d min %d sec\n",dd,hr,mm,ss);
            
    }
    else
        ret = DmtCtrl_STATUS_ERROR;

    return( ret );
}





static int TCDMT_SetDmtReset(void )
{
    unsigned char Arg;
    int ret=0;
    ret = DmtCtrl_STATUS_ERROR;

    if( g_nAdslFd == -1  )
        g_nAdslFd = OpenTCDmt();

    if( g_nAdslFd != -1  )
    {
		
        ioctl( g_nAdslFd, DMTIOCTL_SET_DMTRESET, &Arg );
        ret=DmtCtrl_STATUS_SUCCESS;
        
         
    }
    else
        ret = DmtCtrl_STATUS_ERROR;

    return( ret );
} /* TCDMT_SetDmtReset */


static int TCDMT_SetDmtClose(void )
{
    unsigned char Arg;
    int ret=0;
    ret = DmtCtrl_STATUS_ERROR;

    if( g_nAdslFd == -1  )
        g_nAdslFd = OpenTCDmt();

    if( g_nAdslFd != -1  )
    {
		
        ioctl( g_nAdslFd, DMTIOCTL_SET_DMTCLOSE, &Arg );
        ret=DmtCtrl_STATUS_SUCCESS;
        
         
    }
    else
        ret = DmtCtrl_STATUS_ERROR;

    return( ret );
} /* TCDMT_SetDmtClose */

static int TCDMT_SetDmtOpmode(unsigned int Arg )
{
    
    int ret=0;
    ret = DmtCtrl_STATUS_ERROR;

    if( g_nAdslFd == -1  )
        g_nAdslFd = OpenTCDmt();

    if( g_nAdslFd != -1  )
    {
	ioctl( g_nAdslFd, DMTIOCTL_SET_LINE_OPMODE , &Arg );
        ret=DmtCtrl_STATUS_SUCCESS;
    }
    else
        ret = DmtCtrl_STATUS_ERROR;

    return( ret );
} /* TCDMT_SetDmtOpmode */

static int TCDMT_SetDmtAnnex(unsigned char Arg )
{
    
    int ret=0;
    ret = DmtCtrl_STATUS_ERROR;

    if( g_nAdslFd == -1  )
        g_nAdslFd = OpenTCDmt();

    if( g_nAdslFd != -1  )
    {
	ioctl( g_nAdslFd, DMTIOCTL_SET_ANNEX , &Arg );
        ret=DmtCtrl_STATUS_SUCCESS;
    }
    else
        ret = DmtCtrl_STATUS_ERROR;

    return( ret );
} /* TCDMT_SetDmtAnnex */

static int TCDMT_SetDmtRtsDbg(unsigned char Arg )
{
    
    int ret=0;
    ret = DmtCtrl_STATUS_ERROR;

    if( g_nAdslFd == -1  )
        g_nAdslFd = OpenTCDmt();

    if( g_nAdslFd != -1  )
    {
	ioctl( g_nAdslFd, DMTIOCTL_SET_RTS_DBG , &Arg );
        ret=DmtCtrl_STATUS_SUCCESS;
    }
    else
        ret = DmtCtrl_STATUS_ERROR;

    return( ret );
} /* TCDMT_SetDmtRtsDbg */



static int TCDMT_SetDmtPID(unsigned int Arg)
{
    int ret=0;
    ret = DmtCtrl_STATUS_ERROR;

    if( g_nAdslFd == -1  )
        g_nAdslFd = OpenTCDmt();

    if( g_nAdslFd != -1  )
    {
	ioctl( g_nAdslFd, DMTIOCTL_SET_PID , &Arg );
        ret=DmtCtrl_STATUS_SUCCESS;
    }
    else
        ret = DmtCtrl_STATUS_ERROR;

    return( ret ); 
} /* TCDMT_SetDmtPID */

static int TCDMT_SetDmtPPPOE(unsigned int Arg)
{
    int ret=0;
    ret = DmtCtrl_STATUS_ERROR;

    if( g_nAdslFd == -1  )
        g_nAdslFd = OpenTCDmt();

    if( g_nAdslFd != -1  )
    {
	ioctl( g_nAdslFd, DMTIOCTL_SET_PPPOE , &Arg );
        ret=DmtCtrl_STATUS_SUCCESS;
    }
    else
        ret = DmtCtrl_STATUS_ERROR;

    return( ret ); 
} /* TCDMT_SetDmtPPPOE */


int main(int argc, char **argv)
{
	
        char* cmd=NULL;
        char dmtcmd[40];
        FILE *fp;
        char data[40];
         
if ((argc < 2) ) {
      fprintf(stdout,"Expecting argc to be 2-3, inclusive.  Was: %d\n",argc);
      exit(1);
   }
   else {
      cmd = argv[1];
     }
        
    if (strcasecmp(cmd, "annex") == 0) 
    {
     cmd=argv[2];
     if( strcasecmp(cmd, "a") == 0)
         TCDMT_SetDmtAnnex(ANNEX_A); 
     if( strcasecmp(cmd, "b") == 0)
         TCDMT_SetDmtAnnex(ANNEX_B); 
     if( strcasecmp(cmd, "i") == 0)
         TCDMT_SetDmtAnnex(ANNEX_I); 
     if( strcasecmp(cmd, "j") == 0)
         TCDMT_SetDmtAnnex(ANNEX_J);  
     if( strcasecmp(cmd, "al") == 0)
         TCDMT_SetDmtAnnex(ANNEX_L);
     if( strcasecmp(cmd, "m") == 0)
         TCDMT_SetDmtAnnex(ANNEX_M); 
     if( strcasecmp(cmd, "bjm") == 0)
         TCDMT_SetDmtAnnex(ANNEX_BJM); 
     if( strcasecmp(cmd, "aijlm") == 0)
         TCDMT_SetDmtAnnex(ANNEX_AIJLM);    
          
    }   
    if (strcasecmp(cmd, "reset") == 0) 
    {
     TCDMT_SetDmtReset();
    }
    if (strcasecmp(cmd, "close") == 0) 
    {
     TCDMT_SetDmtClose();
    }
    if (strcasecmp(cmd, "opmode") == 0) 
    {
       cmd=argv[2];
      if( strcasecmp(cmd, "gdmt") == 0)
         TCDMT_SetDmtOpmode(GDMT);
      else if( strcasecmp(cmd, "glite") == 0)
         TCDMT_SetDmtOpmode(GLite);
      else if( strcasecmp(cmd, "t1413") == 0)
         TCDMT_SetDmtOpmode(ANSI);
      else if (strcasecmp(cmd, "multi") == 0)
         TCDMT_SetDmtOpmode(Multi);
      else if (strcasecmp(cmd, "adsl2") == 0)
         TCDMT_SetDmtOpmode(ADSL2);
      else if (strcasecmp(cmd, "adsl2p") == 0)
         TCDMT_SetDmtOpmode(ADSL2Plus);
      else if(strcasecmp(cmd, "adsl1_multi") == 0)
         TCDMT_SetDmtOpmode(ADSL1_MultiMode);
      else
        printf("no such arg available\n"); 
    }
    if ( strcasecmp(cmd, "setpid") == 0)
    {
      cmd=argv[2];
      if(cmd)
	TCDMT_SetDmtPID(atoi(argv[2]));
      else
        printf("no such arg available\n");	
    }
    if (strcasecmp(cmd, "uptime") == 0) 
    {
     TCDMT_doadsluptime();
    }
    
    if (strcasecmp(cmd, "rtsdbg") == 0) 
    {
     cmd=argv[2];
     if( strcasecmp(cmd, "on") == 0)
         TCDMT_SetDmtRtsDbg(DMTRTSON); 
     if( strcasecmp(cmd, "off") == 0)
         TCDMT_SetDmtRtsDbg(DMTRTSOFF); 
     
    }

    if ( strcasecmp(cmd, "setpppoe") == 0)
    {
      cmd=argv[2];
        printf("dmt setpppoe %d\n", atoi(cmd));
      if(cmd)
	TCDMT_SetDmtPPPOE(atoi(argv[2]));
      else
        printf("no such arg available\n");
    }   
  return 0;
}

