/******************************************************************************
 * Copyright(c) 2002 LSI Logic.
 *
 * $ID
 *
 * Description:
 *
 *****************************************************************************/

/* Include files */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <signal.h>
#include <linux/if.h>
#include <errno.h>
#include <syslog.h>

//mark by pikachu
//#include "linux/ppp_public.h"
//#include "dhcp_config.h"
//#include "connections_config.h"
//#include "program_config.h"
//#include "route_config.h"
//#include "ipsec_config.h"
//#include "filem.h"

#define PROC_FILE "/proc/net/ppp"
#define SIGUNUSED 31  //add by pikachu

#ifdef LIBVER
#define MAINFUNC dnsupdate_main
#else
#define MAINFUNC main
#endif

char *load_dns(char *dest, char *src)
{
	char *p;

	p = strchr(src, '=');
	if (!p)
	{
		return 0;
	}
	p++;
	p[strlen(p) - 1] = '\0';
	strcpy(dest, p);
	return dest;
}

/******************************************************************************
 ** <Summary>
 *  <Description>
 *
 * @param <Parameter> <Explanation>
 *
 * @return <Explanation>
 *
 *****************************************************************************/
int getDnsFromProc(int dev_offset, char *pdns, char *sdns)
{
  FILE *fp;
  char buf[255];
  char procfile[64];
  char *p;

  sprintf(procfile, "%s%d", PROC_FILE, dev_offset);

  fp = fopen(procfile, "r");
  if (!fp)
  {
    syslog(LOG_DAEMON | LOG_ERR, "couldn't open %s", procfile);
    return 0;
  }

  while (fgets(buf, 255, fp))
  {
    if (strncmp("pdns", buf, 4) == 0)
    {
			if (load_dns(pdns, buf) == 0)
			{
				fclose(fp);
				return 0;
			}
			continue;
    }
    if (strncmp("sdns", buf, 4) == 0)
    {
			if (load_dns(sdns, buf) == 0)
			{
				fclose(fp);
				return 0;
			}
			continue;
    }
  }
  fclose(fp);
  syslog(LOG_DAEMON | LOG_ERR, "couldn't find primary dns field in %s", procfile);
  return 1;
}

/******************************************************************************
 ** <Summary>
 *  <Description>
 *
 * @param <Parameter> <Explanation>
 *
 * @return <Explanation>
 *
 *****************************************************************************/
int updateConfigFiles(int dev_offset)
{
	char pdns[20];
	char sdns[20];

  if (getDnsFromProc(dev_offset, pdns, sdns) == 0)
  {
    return -1;
  }
  if (dhcp_load_config(0) != 0)
  {
    syslog(LOG_DAEMON | LOG_ERR, "couldn't load dhcp/dns config file");
    return -1;
  }
  dhcp_set_domain_name_server(pdns, 1);
  dhcp_set_domain_name_server(sdns, 2);
  if (dhcp_save_config(0) != 0)
  {
    syslog(LOG_DAEMON | LOG_ERR, "couldn't save dhcp/dns config file");
    return -1;
  }
  return 0;
}

/****************************************************************************
 * Function Name  : kill_by_pidfile
 * Arguments      : filename
 * Returns        : int
 * Variables      : 
 * Modifies       : 
 * Description    : Fork and exec.
 ****************************************************************************/
int kill_by_pidfile(char *filename)
{
	FILE *fp;
	char buf[24];
	int ret;

	/* Kill the dhcpd server */
	fp = fopen(filename, "r");
	if (!fp)
	{
		return -1;
	}
	ret = fread(buf, 1, 24, fp);
	if (ret <= 0)
	{
		return -1;
	}
	buf[ret] = '\0';
	ret = kill(atoi(buf), SIGHUP);
	fclose(fp);
	return ret;
}


/******************************************************************************
 ** <Summary>
 *  <Description>
 *
 * @param <Parameter> <Explanation>
 *
 * @return <Explanation>
 *
 *****************************************************************************/
#define DHCPD_PID_FILE "/var/run/dhcpd.pid"
#define NAMED_PID_FILE "/var/run/named.pid"
#define DHCPD_SCRIPT "/etc/init.d/rcRunDhcpd"
#define NAMED_SCRIPT "/etc/init.d/rcRunNamed"
static int restart_daemons(void)
{
	/* load the program configuration into memory */
	if (program_load_config(0) < 0)
	{
		return -1;
	}

	/* Not interested whether kill succeeds or not - might not be running */
	kill_by_pidfile(DHCPD_PID_FILE);
	kill_by_pidfile(NAMED_PID_FILE);

	/* Only run programs if the configuration requires it */
	//mark by pikachu
	/*if (pr_is_program_activated(r_program_dhcp))
	{
		run_script(DHCPD_SCRIPT, 0, 0, 0);
	}
	if (pr_is_program_activated(r_program_named))
	{
		run_script(NAMED_SCRIPT, 0, 0, 0);
	}
*/

	return 0;
}

void sighandler(int signal)
{
  int dev_offset = signal - SIGUNUSED;
  char fwscript[40];
  char devname[20];

  if (dev_offset >= 0 && dev_offset <= 7)
  {
    connections_load_config(0);
    /* only update the dns if this is the default connection */
    if (conn_get_defaultroute_v(dev_offset))
    {
      /* Signalled by the PPP driver when there is a new dns address available. */
      if (updateConfigFiles(dev_offset) == 0)
      {
        restart_daemons();
        syslog(LOG_DAEMON | LOG_NOTICE, "successful update of dhcp/dns (%d) config files with dns", dev_offset);
      }
      else
      {
        syslog(LOG_DAEMON | LOG_NOTICE, "failed to update dhcp/dns config files");
      }
    }
    /* run the firewall script */
    sprintf(fwscript, "/flash/fwscript%d", conn_get_profilenumber_v(dev_offset));
    sprintf(devname, "ppp%d", dev_offset);
		if (run_script(fwscript, "-A", devname, 0) == 0)
    {
      syslog(LOG_DAEMON | LOG_NOTICE, "successfully ran firewall script for %d", dev_offset);
    }
    else
    {
      syslog(LOG_DAEMON | LOG_NOTICE, "failed to run firewall script for %d", dev_offset);
    }

    /* always run the static route configuration script for this connection */
    if (run_rcfg(dev_offset, 1/* add the routes */) == 0)
    {
      syslog(LOG_DAEMON | LOG_NOTICE, "successfully ran static routes script for %d", dev_offset);
    }
    else
    {
      syslog(LOG_DAEMON | LOG_NOTICE, "failed to run static routes script for %d", dev_offset);
    }

    /* start any pendent IPsec connection */
#ifdef include_ipsec    
    // For Dynamic IP, we need to change the IP address of running IPSec connections.
    // Check if any connection is running IPSec. If so, disconnect it and reconnect it again.
    //ipsec_load_config1();
    ////ipsec_start_connections(dev_offset); 
#endif /* include_ipsec */

		if (run_script("/etc/mcreload", 0, 0, 0) == 0)
    {
      syslog(LOG_DAEMON | LOG_NOTICE, "successfully ran mcreload script");
    }
    else
    {
      syslog(LOG_DAEMON | LOG_NOTICE, "failed to run mcreload script");
    }
  }
}

/******************************************************************************
 ** <Summary>
 *  <Description>
 *
 * @param <Parameter> <Explanation>
 *
 * @return <Explanation>
 *
 *****************************************************************************/
int MAINFUNC(int argc, char *argv[])
{
  int fd;
  struct ifreq ifr;
//	struct ppp_ioctl pio;

  // KenWang 2004/12/16
  //openlog("DDNS", LOG_PID | LOG_CONS, LOG_DAEMON);
  openlog("DDNS", LOG_CONS, LOG_DAEMON);

  syslog(LOG_DAEMON | LOG_NOTICE, "started");

  signal(SIGUNUSED, sighandler);
  signal(SIGUNUSED + 1, sighandler);
  signal(SIGUNUSED + 2, sighandler);
  signal(SIGUNUSED + 3, sighandler);
  signal(SIGUNUSED + 4, sighandler);
  signal(SIGUNUSED + 5, sighandler);
  signal(SIGUNUSED + 6, sighandler);
  signal(SIGUNUSED + 7, sighandler);

  if ((argc == 2) && ((strcmp(argv[1], "-d") == 0)))
  {
    if (daemon(0, 0) == -1)
    {
      syslog(LOG_DAEMON | LOG_ERR, "failed to daemonize - exiting");
      return -1;
    }
  }

	/* open an inet socket */
  fd = socket(PF_INET, SOCK_STREAM, 0);
  if (fd < 0)
	{
    syslog(LOG_DAEMON | LOG_ERR, "failed to open socket - exiting");
		return -1;
	}

  /* Send the pid of this process to the PPP driver */
//	pio.cmd = PPPIODNSPID; //mark by pikachu

//  pio.dns_pid = getpid();  //mark by pikachu
  memset((void *)&ifr, '\0', sizeof(struct ifreq));
  memcpy(ifr.ifr_name, "ppp0", IFNAMSIZ);
//  ((unsigned long *)(&ifr.ifr_data))[0] = (unsigned long)&pio; //mark by pikachu

  if (ioctl(fd, SIOCDEVPRIVATE, &ifr) < 0)
  {
    syslog(LOG_DAEMON | LOG_ERR, "ioctl to ppp failed - exiting");
    close(fd);
    return -1;
  }
  close(fd);
    
  while (1)
  {
    /* Process will block on this call */
    sleep(10);
  }
  return 0;
}
