/*
 * Author: Paul.Russell@rustcorp.com.au and mneuling@radlogic.com.au
 *
 * Based on the ipchains code by Paul Russell and Michael Neuling
 *
 * (C) 2000-2002 by the netfilter coreteam <coreteam@netfilter.org>:
 * 		    Paul 'Rusty' Russell <rusty@rustcorp.com.au>
 * 		    Marc Boucher <marc+nf@mbsi.ca>
 * 		    James Morris <jmorris@intercode.com.au>
 * 		    Harald Welte <laforge@gnumonks.org>
 * 		    Jozsef Kadlecsik <kadlec@blackhole.kfki.hu>
 *
 *	iptables -- IP firewall administration for kernels with
 *	firewall table (aimed for the 2.3 kernels)
 *
 *	See the accompanying manual page iptables(8) for information
 *	about proper usage of this program.
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <iptables.h>

#ifdef LIBVER 
static char *get_app_name(char *path)
{
	char *p;

	p = strrchr(path, '/');
	if (!p)
	{
		return path;
	}
	p++;
	return p;
}
#endif


static void preprocess_iptable(int* argc, char* argv[])
{
	int i,j;
	int z;
	for(i=0;i<(*argc);i++)
	{
		if(strcmp(argv[i],"-z") == 0)
		{
			z = atoi(argv[i+1]);
			for(j=i;j<(*argc-2);j++)
				argv[j] = argv[j+2];
			*argc = *argc - 2;
			return;
		}
	}
}


int
main(int argc, char *argv[])
{
	int ret;
	char *table = "filter";
	iptc_handle_t handle = NULL;
#ifdef LIBVER
	char *app_name = get_app_name(argv[0]);

	if (strcmp(app_name, "runFunc") == 0)
	{
		return runFunc_main(argc, argv);
	}

#endif
	
	program_name = "iptables";
	program_version = IPTABLES_VERSION;

//#define OLD_IPTABLE_ARCH
//#undef OLD_IPTABLE_ARCH
	/* Robert added for new firewall arch. */
	if (strcmp(app_name, program_name) == 0)
//#ifdef OLD_IPTABLE_ARCH
		preprocess_iptable(&argc, argv);
//#else
//		preprocess_iptable1(&argc, argv);
//#endif

	
#ifdef NO_SHARED_LIBS
	init_extensions();
#endif

	ret = do_command(argc, argv, &table, &handle);
	if (ret)
		ret = iptc_commit(&handle);

	if (!ret)
		fprintf(stderr, "iptables: %s\n",
			iptc_strerror(errno));

	exit(!ret);
}
