#include <stdio.h>
#include <string.h>
#include "rtable.h"


struct R_IPTABLE_NODE;
struct R_IPTABLE_TABLE;
typedef struct R_IPTABLE_NODE
{
  int value;
  int checked;
  struct R_IPTABLE_TABLE *ptr;
} R_iptable_node;

typedef struct R_IPTABLE_TABLE
{
  struct R_IPTABLE_NODE *nr;
  int limit;
  int count;
  struct R_IPTABLE_TABLE *parent;
  int type;
} R_iptable_table;

#define R_MKPTABLE_START(x) R_iptable_node t##x[] = {
#define R_MKPTABLE_END(x) }; \
R_iptable_table x = {t##x,sizeof(t##x)/sizeof(R_iptable_node),0,0,0}
#define R_MKETABLE_START(x) R_iptable_node t##x[] = {
#define R_MKETABLE_END(x) }; \
R_iptable_table x = {t##x,sizeof(t##x)/sizeof(R_iptable_node),0,0,1}

R_iptable_table t_INPUT_TABLE;
R_iptable_table t_FORWARD_TABLE;
R_iptable_table t_OUTPUT_TABLE;
R_iptable_table t_NAT_PREROUTING_TABLE;
R_iptable_table t_NAT_POSTROUTING_TABLE;
R_iptable_table t_MANGLE_PREROUTING_TABLE;
R_iptable_table t_MANGLE_FORWARD_TABLE;
R_iptable_table t_MANGLE_POSTROUTING_TABLE;
R_iptable_table t_MANGLE_MYSHAPER_IN_TABLE;
R_iptable_table t_MANGLE_MYSHAPER_OUT_TABLE;
R_iptable_table t_MF_INT_TABLE;
R_iptable_table t_MF_FWD_TABLE;
R_iptable_table t_UPNP_PRT_TABLE;
R_iptable_table t_UPNP_FWD_TABLE;
R_iptable_table t_HCK_PRT_TABLE;
R_iptable_table t_PKT_FLT_TABLE;
R_iptable_table t_VS_PRT_TABLE;
R_iptable_table t_DMZ_PRT_TABLE;
R_iptable_table t_UF_REJ_TABLE;
R_iptable_table t_URL_FLT_TABLE;
R_iptable_table t_BAD_PING_TABLE;
R_iptable_table t_TMSS_TABLE;
R_iptable_table t_HCK_FWD_TABLE;
R_iptable_table t_HCK_INP_TABLE;

/* INPUT TABLE start */
R_MKPTABLE_START (t_INPUT_TABLE)
  {RUNFUNC_C_INPUT_TCP_DEST_ACCEPT,0,0}, /* Remote Config */
  {RUNFUNC_C_INPUT_TCP_SRC_DPORT_ACCEPT,0,0}, /* UPNP */
  {RUNFUNC_C_INPUT_UDP_SRC_DPORT1900_ACCEPT,0,0}, /* UPNP */
  {RUNFUNC_C_INPUT_TCP_SRC_DPORT2600_ACCEPT,0,0},/* RIP */
  {RUNFUNC_C_INPUT_UDP_SRC_DPORT520_ACCEPT,0,0}, /* RIP */
  {RUNFUNC_C_INPUT_UDP_SRC_DPORT161_ACCEPT,0,0}, /* SNMP */
  {RUN_IPSEC_C_INPUT_UDP_WAN_IF_RIP_DPORT500_ACCEPT,0,0}, /* IPSEC */
  {RUN_IPSEC_C_INPUT_ESP_WAN_IF_RIP_ACCEPT,0,0}, /* IPSEC */
  {RUN_IPSEC_C_INPUT_AH_WAN_IF_RIP_ACCEPT,0,0}, /* IPSEC */
  {RUN_IPSEC_C_INPUT_IPSEC_ACCEPT,0,0}, /* IPSEC */
  {EMBEDDED_SH_INPUT_LAN_IF_TCP_DPORT_ACCEPT,0,0}, /* Lan access webs */
  {MACF_C_INPUT_MF_INT,0,0},
  {0,0,&t_MF_INT_TABLE},
//  {FW_INIT_SH_INPUT_BLK_WAN_INT,0,0},
  {RUNFUNC_C_INPUT_TCP_HTTPSR_PORT_DEST_DROP,0,0},
  {BLK_WAN_START_IPT_SH_INPUT_ICMP_WAN_IF_DROP,0,0},
  {BLK_WAN_START_IPT_SH_INPUT_ICMP_LOG_WAN_IF_DROP,0,0},
  {CLOSE_WAN_PORT_IPT_ADD_SH_INPUT_TCP_DEST_DROP,0,0},
  {CLOSE_WAN_PORT_IPT_ADD_SH_INPUT_UDP_DEST_DROP,0,0},
  {FW_INIT_SH_INPUT_ICMP_ECHO_REQUEST_BAD_PING,0,0},
  {0,0,&t_BAD_PING_TABLE},
  {FW_INIT_SH_INPUT_HCK_INP,0,0},
  {0,0,&t_HCK_INP_TABLE},
//  {FW_INIT_SH_INPUT_REMOTE_INT,0,0},
//  {FW_INIT_SH_INPUT_LAN_ACCESS,0,0},
//  {FW_INIT_SH_INPUT_SIP_INT,0,0},
//  {FW_INIT_SH_INPUT_MSS_INT,0,0},
//  {FW_INIT_SH_INPUT_UPNP_INT,0,0},
//  {FW_INIT_SH_INPUT_SNMP_INT,0,0},
//  {FW_INIT_SH_INPUT_RIP_INT,0,0},
  {SIP_IPT_ADD_SH_VOIP_INPUT_DEST_DROP,0,0}, /* INVALID state */
  {SIP_IPT_ADD_SH_VOIP_INPUT_DEST_ACCEPT,0,0}, /* Related, Established state */
  {SIP_IPT_ADD_SH_VOIP_INPUT_DEST_NEW_ACCEPT,0,0}, /* New state */
  {SIP_IPT_ADD_SH_VOIP_INPUT_DEST_LAN_NEW_ACCEPT,0,0}, /* New state for lan interface */
  {SIP_IPT_ADD_SH_INPUT_WAN_IP_UDP_DPORT_1024_65535_DROP,0,0}
R_MKPTABLE_END (t_INPUT_TABLE);
/* INPUT TABLE end */
/* FORWARD TABLE start */
R_MKPTABLE_START (t_FORWARD_TABLE)
  {RUN_IPSEC_C_FORWARD_IN_IPSEC_ACCEPT,0,0},
  {RUN_IPSEC_C_FORWARD_OUT_IPSEC_ACCEPT,0,0},
  {RUN_IPSEC_C_FORWARD_OUT_IPSEC_TCPMSS,0,0},
  {PF_C_FORWARD_PKT_FLT,0,0},
  {MACF_C_FORWARD_MF_FWD,0,0},
  {URLFLT_C_FORWARD_TCP_LANIP_DPORT80_URL_FLT,0,0},
  {FW_INIT_SH_FORWARD_BLK_WAN_FWD,0,0},
  {FW_INIT_SH_FORWARD_HCK_FWD,0,0},
  {FW_INIT_SH_FORWARD_DMZ_FWD,0,0},
  {FW_INIT_SH_FORWARD_VS_FWD,0,0},
  {FW_INIT_SH_FORWARD_UPNP_FWD,0,0},
  {FW_INIT_SH_FORWARD_MSS_FWD,0,0},
  {RUN_IPSEC_C_FORWARD_IN_IPSEC_TCPMSS,0,0},
  {TCPMSS_IPT_SPRAY_ADD_FORWARD_OUT_LAN_IF_TCP_SET_MSS1450,0,0},
  {TCPMSS_IPT_SPRAY_ADD_FORWARD_IN_LAN_IF_TCP_CLAMP_MSS_TO_PMTU,0,0},
  {TMSS_SH_FORWARD_TCP_SRC_DPORT80_TMSS,0,0},
  {BLK_WAN_START_IPT_SH_FORWARD_ICMP_LAN_IF_DROP,0,0},
  {BLK_WAN_START_IPT_SH_FORWARD_ICMP_LOG_LAN_IF_DROP,0,0}
R_MKPTABLE_END (t_FORWARD_TABLE);
/* FORWARD TABLE end */
/* OUTPUT TABLE start */
R_MKPTABLE_START (t_OUTPUT_TABLE)
  {FW_INIT_SH_OUTPUT_MSS_OUT,0,0},
  {TCPMSS_IPT_SPRAY_ADD_OUTPUT_TCP_CLAMP_MSS_TO_PMTU,0,0}
R_MKPTABLE_END (t_OUTPUT_TABLE);
/* OUTPUT TABLE end */
/* NAT PREROUTING TABLE start */
R_MKPTABLE_START (t_NAT_PREROUTING_TABLE)
  {IPT_SH_NAT_PREROUTING_TCP80_REDIRECT_TO10080,0,0},
  {RUNFUNC_C_NAT_PREROUTING_TCP_HTTPSR_PORT_DEST_DROP,0,0},
  {RUNFUNC_C_NAT_PREROUTING_TCP_DEST_ACCEPT,0,0},
  {RUNFUNC_C_NAT_PREROUTING_TCP_SRC_DPORT_ACCEPT,0,0},
  {RUNFUNC_C_NAT_PREROUTING_UDP_SRC_DPORT1900_ACCEPT,0,0},
  {RUNFUNC_C_NAT_PREROUTING_TCP_SRC_DPORT2600_ACCEPT,0,0},
  {RUNFUNC_C_NAT_PREROUTING_UDP_SRC_DPORT520_ACCEPT,0,0},
  {RUNFUNC_C_NAT_PREROUTING_UDP_SRC_DPORT161_ACCEPT,0,0},
  {RUN_IPSEC_C_NAT_PREROUTING_ESP_WAN_IF_RIP_ACCEPT,0,0},
  {RUN_IPSEC_C_NAT_PREROUTING_AH_WAN_IF_RIP_ACCEPT,0,0},
  {RUN_IPSEC_C_NAT_PREROUTING_IN_IPSEC_ACCEPT,0,0},
  {FAKED_DNS_PROXY_STOP_SH_NAT_PREROUTING_UDP_53REDIRECTTO10053,0,0},
  {FW_INIT_SH_NAT_PREROUTING_HCK_PRT,0,0},
  {FW_INIT_SH_NAT_PREROUTING_DMZ_PRT,0,0},
  {FW_INIT_SH_NAT_PREROUTING_VS_PRT,0,0},
  {FW_INIT_SH_NAT_PREROUTING_UPNP_PRT,0,0},
  {SIP_IPT_DEL_SH_VOIP_NAT_PREROUTING_INVALID_DROP,0,0},
  {SIP_IPT_DEL_SH_VOIP_NAT_PREROUTING_ACCEPT,0,0},
  {SIP_IPT_DEL_SH_VOIP_NAT_PREROUTING_LO_NEW_ACCEPT,0,0},
  {SIP_IPT_DEL_SH_VOIP_NAT_PREROUTING_LAN_IF_NEW_ACCEPT,0,0},
  {BLK_WAN_START_IPT_SH1_NAT_PREROUTING_ICMP_WAN_IF_DROP,0,0},
  {BLK_WAN_START_IPT_SH1_NAT_PREROUTING_ICMP_LOG_WAN_IF_DROP,0,0},
  {CLOSE_WAN_PORT_IPT_DEL_SH_NAT_PREROUTING_TCP_WAN_IP_DROP,0,0},
  {CLOSE_WAN_PORT_IPT_DEL_SH_NAT_PREROUTING_UDP_WAN_IP_DROP,0,0},
  {SIP_IPT_DEL_SH_NAT_PREROUTING_UDP_DPORT_1024_65535_DROP,0,0}
R_MKPTABLE_END (t_NAT_PREROUTING_TABLE);
/* NAT PREROUTING TABLE end */
/* NAT POSTROUTING TABLE start */
R_MKPTABLE_START (t_NAT_POSTROUTING_TABLE)
  {IPT_SH_NAT_POSTROUTING_MASQUERADE,0,0},
  {RUN_IPSEC_C_NAT_POSTROUTING_OUT_IPSEC_MASQUERADE,0,0},
  {FW_INIT_SH_NAT_POSTROUTING_NAT_POSTRT,0,0},
  {FW_INIT_SH_NAT_POSTROUTING_SIP_POSTRT,0,0},
  {SIP_IPT_ADD_SH1_VOIP_NAT_POSTROUTING_WAN_IF_SRC_DEST_DROP,0,0},
  {CLOSE_WAN_PORT_IPT_DEL_SH_NAT_POSTROUTING_TCP_SPORT_0_1_DROP,0,0},
  {NAT_IPT_SH_NAT_POSTROUTING_MASQUERADE_WAN_IF,0,0},
  {NAT_IPT_SH_NAT_POSTROUTING_SRC_MASQUERADE,0,0},
  {TCPMSS_IPT_ADD1_NAT_POSTROUTING_TCP_CLAMP_MSS_TO_PMTU,0,0}
R_MKPTABLE_END (t_NAT_POSTROUTING_TABLE);
/* NAT POSTROUTING TABLE end */
/* MANGLE PREROUTING TABLE start */
R_MKPTABLE_START (t_MANGLE_PREROUTING_TABLE)
  {FW_INIT_SH_MANGLE_PREROUTING_MSS_MG_PRT,0,0}
R_MKPTABLE_END (t_MANGLE_PREROUTING_TABLE);
/* MANGLE PREROUTING TABLE end */
/* MANGLE FORWARD TABLE start */
R_MKPTABLE_START (t_MANGLE_FORWARD_TABLE)
  {RUN_QOS_C_MANGLE_FORWARD_WAN_IF_MYSHAPER_IN,0,0},
  {RUN_QOS_C_MANGLE_FORWARD_DSCP,0,0},
  {RUN_QOS_C_MANGLE_FORWARD_UDP_DSCP,0,0}
R_MKPTABLE_END (t_MANGLE_FORWARD_TABLE);
/* MANGLE FORWARD TABLE end */
/* MANGLE POSTROUTING TABLE start */
R_MKPTABLE_START (t_MANGLE_POSTROUTING_TABLE)
  {RUN_QOS_C_MANGLE_POSTROUTING_WAN_IF_MYSHAPER_OUT,0,0},
  {FW_INIT_SH_MANGLE_POSTROUTING_MSS_MG_POSTRT,0,0},
  {TCPMSS_IPT_SPRAY_ADD_MANGLE_POSTROUTING_TCP_CLAMP_MSS_TO_PMTU,0,0}
R_MKPTABLE_END (t_MANGLE_POSTROUTING_TABLE);
/* MANGLE POSTROUTING TABLE end */
/* MANGLE MYSHAPER_IN TABLE start */
R_MKPTABLE_START (t_MANGLE_MYSHAPER_IN_TABLE)
  {RUN_QOS_C_MANGLE_MYSHAPER_IN_IMQ_TODEV0,0,0},
  {RUN_QOS_C_MANGLE_MYSHAPER_IN_MARK,0,0},
  {RUN_QOS_C_MANGLE_MYSHAPER_IN_UDP_MARK,0,0}
R_MKPTABLE_END (t_MANGLE_MYSHAPER_IN_TABLE);
/* MANGLE MYSHAPER_IN TABLE end */
/* MANGLE MYSHAPER_OUT TABLE start */
R_MKPTABLE_START (t_MANGLE_MYSHAPER_OUT_TABLE)
  {RUN_QOS_C_MANGLE_MYSHAPER_OUT_MARK,0,0},
  {RUN_QOS_C_MANGLE_MYSHAPER_OUT_UDP_MARK,0,0}
R_MKPTABLE_END (t_MANGLE_MYSHAPER_OUT_TABLE);
/* MANGLE MYSHAPER_OUT TABLE end */
/* MF_INT TABLE start */
R_MKPTABLE_START (t_MF_INT_TABLE)
  {FACTORYRESET_C_MF_INT_MAC,0,0},
  {MACF_C_MF_INT_TCP_LANIP_ACCEPT,0,0},
  {MACF_C_MF_INT_MAC_LOG,0,0},
  {MACF_C_MF_INT_DROP,0,0}
R_MKPTABLE_END (t_MF_INT_TABLE);
/* MF_INT TABLE end */
/* MF_FWD TABLE start */
R_MKPTABLE_START (t_MF_FWD_TABLE)
  {FACTORYRESET_C_MF_FWD_MAC,0,0},
  {MACF_C_MF_FWD_TCP_LANIP_ACCEPT,0,0},
  {MACF_C_MF_FWD_MAC_LOG,0,0},
  {MACF_C_MF_FWD_DROP,0,0}
R_MKPTABLE_END (t_MF_FWD_TABLE);
/* MF_FWD TABLE end */
/* UPNP_PRT TABLE start */
R_MKPTABLE_START (t_UPNP_PRT_TABLE)
  {UPNP_LINUX_IGD_C_NAT_UPNP_PRT_WAN_IF_DNAT_TO_LAN,0,0}
R_MKPTABLE_END (t_UPNP_PRT_TABLE);
/* UPNP_PRT TABLE end */
/* UPNP_FWD TABLE start */
R_MKPTABLE_START (t_UPNP_FWD_TABLE)
  {UPNP_LINUX_IGD_C_UPNP_FWD_LAN_ACCEPT,0,0}
R_MKPTABLE_END (t_UPNP_FWD_TABLE);
/* UPNP_FWD TABLE end */
/* HCK_PRT TABLE start */
R_MKPTABLE_START (t_HCK_PRT_TABLE)
  {RUNFUNC_C_NAT_HCK_PRT_TCP_DEST_ACCEPT,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_TCP_WAN_IP_DPORT_0_1055_DROP,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_UDP_WAN_IP_DPORT_0_1055_DROP,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_TCP_WAN_IP_DROP,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_HACK16_LOG,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_INVALID_HACK16_DROP,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_TCP_DPORTS_HACK15_LOG,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_TCP_DPORTS_HACK15_DROP,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_UDP_DPORT31337_HACK14_LOG,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_UDP_DPORT31337_HACK14_DROP,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_TCP_DPORTS_HACK13_LOG,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_TCP_DPORTS_HACK13_DROP,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_TCP_HACK12_LOG,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_TCP_HACK12_DROP,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_TCP_HACK10_LOG,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_TCP_HACK10_DROP,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_TCP_DEST_DROP,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_TCP_HACK9_LOG,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_TCP_HACK9_DROP,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_UDP_HACK6_LOG,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_UDP_HACK6_DROP,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_UDP_HACK3_LOG,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_UDP_HACK3_DROP,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_UDP_HACK2_7_LOG,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_UDP_HACK2_7_DROP,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_HACK1_LOG,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_HACK1_DROP,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_WAN_IP_WAN_IP_DROP,0,0},
  {HACK_IPT_START_SH_NAT_HCK_PRT_LAN_IP_LAN_IP_DROP,0,0}
R_MKPTABLE_END (t_HCK_PRT_TABLE);
/* HCK_PRT TABLE end */
/* PKT_FLT TABLE start */
R_MKPTABLE_START (t_PKT_FLT_TABLE)
  {PF_C_PKT_FLT_LOG_PFDROP,0,0},
  {PF_C_PKT_FLT_PFDROP,0,0},
  {PF_C_PKT_FLT_SRC_DST_LOG_PFDROP,0,0},
  {PF_C_PKT_FLT_SRC_DST_PFDROP,0,0}
R_MKPTABLE_END (t_PKT_FLT_TABLE);
/* PKT_FLT TABLE end */
/* VS_PRT TABLE start */
R_MKPTABLE_START (t_VS_PRT_TABLE)
  {SETVS_C_VS_PRT_NAT_DEST_DNAT,0,0}
R_MKPTABLE_END (t_VS_PRT_TABLE);
/* VS_PRT TABLE end */
/* DMZ_PRT TABLE start */
R_MKPTABLE_START (t_DMZ_PRT_TABLE)
  {SETVS_C_DMZ_PRT_NAT_DEST_DNAT,0,0}
R_MKPTABLE_END (t_DMZ_PRT_TABLE);
/* DMZ_PRT TABLE end */
/* UF_REJ TABLE start */
R_MKPTABLE_START (t_UF_REJ_TABLE)
  {URLFLT_C_UF_REJ_TCP_REJECT,0,0}
R_MKPTABLE_END (t_UF_REJ_TABLE);
/* UF_REJ TABLE end */
/* URL_FLT TABLE start */
R_MKPTABLE_START (t_URL_FLT_TABLE)
  {URLFLT_C_URL_FLT_TCP_WEBSTR_CONTENT_UF_REJ,0,0},
  {URLFLT_C_URL_FLT_TCP_WEBSTR_URL_UF_REJ,0,0},
  {URLFLT_C_URL_FLT_TCP_WEBSTR_HOST_UF_REJ,0,0}
R_MKPTABLE_END (t_URL_FLT_TABLE);
/* URL_FLT TABLE end */
/* BAD_PING TABLE start */
R_MKPTABLE_START (t_BAD_PING_TABLE)
  {HACK_IPT_STOP_SH_BAD_PING_ICMP_RETURN,0,0},
  {HACK_IPT_STOP_SH_BAD_PING_ICMP_LOG,0,0},
  {HACK_IPT_STOP_SH_BAD_PING_ICMP_REJECT,0,0}
R_MKPTABLE_END (t_BAD_PING_TABLE);
/* BAD_PING TABLE end */
/* TMSS TABLE start */
R_MKPTABLE_START (t_TMSS_TABLE)
  {TMSS_SH_TMSS_TCP_1RETURN,0,0},
  {TMSS_SH_TMSS_TCP_TMSS_AVE,0,0},
  {TMSS_SH_TMSS_TCP_4RETURN,0,0},
  {TMSS_SH_TMSS_TCP_TMSS_BLK,0,0},
  {TMSS_SH_TMSS_TCP_DROP,0,0},
  {TMSS_SH_TMSS_AVE_TCP_REJECT_WITH_ICMP_ADMIN_PROHIBITED,0,0},
  {TMSS_SH_TMSS_BLK_TCP_REJECT_WITH_ICMP_HOST_PROHIBITED,0,0}
R_MKPTABLE_END (t_TMSS_TABLE);
/* TMSS TABLE end */
/* HCK_FWD TABLE start */
R_MKPTABLE_START (t_HCK_FWD_TABLE)
  {HACK_IPT_START_SH_HCK_FWD_HACK16_LOG,0,0},
  {HACK_IPT_START_SH_HCK_FWD_INVALID_HACK16_DROP,0,0},
  {HACK_IPT_START_SH_HCK_FWD_ICMP_LAN_IF_DROP,0,0}
R_MKPTABLE_END (t_HCK_FWD_TABLE);
/* HCK_FWD TABLE end */
/* HCK_INP TABLE start */
R_MKPTABLE_START (t_HCK_INP_TABLE)
  {HACK_IPT_START_SH_HCK_INP_TCP_WAN_IP_DPORT_0_65535_DROP,0,0},
  {HACK_IPT_START_SH_HCK_INP_UDP_WAN_IP_DPORT_0_1055_DROP,0,0},
  {HACK_IPT_START_SH_HCK_INP_TCP_WAN_IP_DROP,0,0},
  {HACK_IPT_START_SH_HCK_INP_HACK16_LOG,0,0},
  {HACK_IPT_START_SH_HCK_INP_INVALID_HACK16_DROP,0,0},
  {HACK_IPT_START_SH_HCK_INP_DEST_LO_8_REJECT,0,0},
  {HACK_IPT_START_SH_HCK_INP_TCP_DPORTS_HACK15_LOG,0,0},
  {HACK_IPT_START_SH_HCK_INP_TCP_DPORTS_HACK15_DROP,0,0},
  {HACK_IPT_START_SH_HCK_INP_UDP_DPORT31337_HACK14_LOG,0,0},
  {HACK_IPT_START_SH_HCK_INP_UDP_DPORT31337_HACK14_DROP,0,0},
  {HACK_IPT_START_SH_HCK_INP_TCP_DPORTS_HACK13_LOG,0,0},
  {HACK_IPT_START_SH_HCK_INP_TCP_DPORTS_HACK13_DROP,0,0},
  {HACK_IPT_START_SH_HCK_INP_TCP_HACK12_LOG,0,0},
  {HACK_IPT_START_SH_HCK_INP_TCP_HACK12_DROP,0,0},
  {HACK_IPT_START_SH_HCK_INP_TCP_HACK10_LOG,0,0},
  {HACK_IPT_START_SH_HCK_INP_TCP_HACK10_DROP,0,0},
  {HACK_IPT_START_SH_HCK_INP_TCP_DEST_DROP,0,0},
  {HACK_IPT_START_SH_HCK_INP_TCP_HACK9_LOG,0,0},
  {HACK_IPT_START_SH_HCK_INP_TCP_HACK9_DROP,0,0},
  {HACK_IPT_START_SH_HCK_INP_UDP_HACK6_LOG,0,0},
  {HACK_IPT_START_SH_HCK_INP_UDP_HACK6_DROP,0,0},
  {HACK_IPT_START_SH_HCK_INP_UDP_HACK3_LOG,0,0},
  {HACK_IPT_START_SH_HCK_INP_UDP_HACK3_DROP,0,0},
  {HACK_IPT_START_SH_HCK_INP_UDP_HACK2_7_LOG,0,0},
  {HACK_IPT_START_SH_HCK_INP_UDP_HACK2_7_DROP,0,0},
  {HACK_IPT_START_SH_HCK_INP_HACK1_LOG,0,0},
  {HACK_IPT_START_SH_HCK_INP_HACK1_DROP,0,0},
  {HACK_IPT_START_SH_HCK_INP_WAN_IP_WAN_IP_DROP,0,0},
  {HACK_IPT_START_SH_HCK_INP_LAN_IP_LAN_IP_DROP,0,0}
R_MKPTABLE_END (t_HCK_INP_TABLE);
/* HCK_INP TABLE end */

enum R_IPTABLE_OPMODE
{
  OPADD,
  OPDEL,
  OPNONE
};

#define R_ERR_START  0x1000
#define R_ERR (R_ERR_START + 0)
#define R_ERR_ADD (R_ERR_START + 1)
#define R_ERR_DEL (R_ERR_START + 2)

/**************************************************************/
/* Start of Mapping of table name and table */
/**************************************************************/
struct builtintbl {
	const char *name;
	R_iptable_table* table;
/*dbg:	int debug_flag; */
};
/* FILTER Table */
static const struct builtintbl r_Filter[] = {
    {"FORWARD",&t_FORWARD_TABLE}, /* dbg: ,1},*/
    {"HCK_FWD",&t_HCK_FWD_TABLE}, /* dbg: ,2},*/
    {"HCK_INP",&t_HCK_INP_TABLE}, /* dbg: ,3},*/
      {"INPUT",&t_INPUT_TABLE}, /* dbg: ,4},*/
     {"MF_FWD",&t_MF_FWD_TABLE}, /* dbg: ,5},*/
     {"MF_INT",&t_MF_INT_TABLE}, /* dbg: ,6},*/
     {"OUTPUT",&t_OUTPUT_TABLE}, /* dbg: ,7},*/
    {"PKT_FLT",&t_PKT_FLT_TABLE}, /* dbg: ,8},*/
       {"TMSS",&t_TMSS_TABLE}, /* dbg: ,9},*/
  {"TMSS_AVE",0}, /* dbg: ,10},*/
  {"TMSS_BLK",0}, /* dbg: ,11},*/
    {"UF_REJ",&t_UF_REJ_TABLE}, /* dbg: ,12},*/
    {"UPNP_FWD",&t_UPNP_FWD_TABLE}, /* dbg: ,13},*/
   {"URL_FLT",&t_URL_FLT_TABLE}, /* dbg: ,14},*/
  {"bad-ping",&t_BAD_PING_TABLE}, /* dbg: ,15},*/
};
/* NAT Table */
#define NUMR_FILTER ( sizeof (r_Filter) / sizeof (struct builtintbl) )
static const struct builtintbl r_Nat[] = {
	{"DMZ_PRT",&t_DMZ_PRT_TABLE}, /* dbg: ,1},*/
	{"HCK_PRT",&t_HCK_PRT_TABLE}, /* dbg: ,2},*/
     {"NAT_POSTRT",0}, /* dbg: ,3},*/
	 {"OUTPUT",0}, /* dbg: ,4},*/
    {"POSTROUTING",&t_NAT_POSTROUTING_TABLE}, /* dbg: ,6},*/
     {"PREROUTING",&t_NAT_PREROUTING_TABLE}, /* dbg: ,5},*/
     {"SIP_POSTRT",0}, /* dbg: ,7},*/
       {"UPNP_PRT",&t_UPNP_PRT_TABLE}, /* dbg: ,8},*/
	 {"VS_PRT",&t_VS_PRT_TABLE}, /* dbg: ,9},*/
};
#define NUMR_NAT ( sizeof (r_Nat) / sizeof (struct builtintbl) )
/* MANGLE Table */
static const struct builtintbl r_Mangle[] = {
	{"FORWARD",&t_MANGLE_FORWARD_TABLE}, /* dbg: ,1},*/
	  {"INPUT",0}, /* dbg: ,2},*/
  {"MSS_MG_POSTRT",0}, /* dbg: ,3},*/
     {"MSS_MG_PRT",0}, /* dbg: ,4},*/
    {"MYSHAPER-IN",&t_MANGLE_MYSHAPER_IN_TABLE}, /* dbg: ,5},*/
   {"MYSHAPER-OUT",&t_MANGLE_MYSHAPER_OUT_TABLE}, /* dbg: ,6},*/
	 {"OUTPUT",0}, /* dbg: ,7},*/
    {"POSTROUTING",&t_MANGLE_POSTROUTING_TABLE}, /* dbg: ,8},*/
     {"PREROUTING",&t_MANGLE_PREROUTING_TABLE}, /* dbg: ,9},*/
};
#define NUMR_MANGLE ( sizeof (r_Mangle) / sizeof (struct builtintbl) )
typedef enum R_TABLE_NAME
{
	T_filter=0,
	T_nat,
	T_mangle
}R_Table_Name;
static const struct builtintbl* R_TBLs[] = { r_Filter, r_Nat, r_Mangle };
static const int R_TBLsize[] = {NUMR_FILTER,NUMR_NAT,NUMR_MANGLE};
typedef enum R_TABLE_HANDLER
{
	T_None,T_D,T_F,T_I,T_A
}R_Table_Handler;
static int pstrcmp(const void *name, const void *b)
{
	return strcmp((const char *)name, (*(const char *const *) b));
}
void R_Table_Handler_D(int* argc, char* argv[], int z, int Z_index, int index, int T_index)
{
	struct builtintbl *t;
	int i;
	if(!Z_index)
		return;
	if( argv[index+1][0] != '-' ) //have argument 
		t= (struct builtintbl*)bsearch(argv[index+1],R_TBLs[T_index],R_TBLsize[T_index],sizeof(struct builtintbl),pstrcmp);
	if(t == 0) // can not find table
		return;
	/* remove -z xx */
	for(i=Z_index;i<((*argc)-2);i++)
		argv[i] = argv[i+2];
	(*argc)-=2;
	for(i=0;i<(*argc);i++)
		printf("%s ",argv[i]);
	printf("\n");
	/* delete table counts */
}
void R_Table_Handler_F(int* argc, char* argv[], int z, int Z_index, int index, int T_index)
{
	/* iptables -F */
	/* iptables -F -t nat */
	/* iptables -F -t mangle */
	struct builtintbl *t;
	/* need more tests, steal time here */
	if((*argc) == 2 || (*argc) == 4)
	{
		printf("clear tables %d\n",T_index);
		return;
	}
	if( argv[index+1][0] != '-' ) //have argument 
		t= (struct builtintbl*)bsearch(argv[index+1],R_TBLs[T_index],R_TBLsize[T_index],sizeof(struct builtintbl),pstrcmp);
	if(t == 0) // can not find table
		return;
	
	/* clear table counts */
}
void R_Table_Handler_I(int* argc, char* argv[], int z, int Z_index, int index, int T_index)
{
	struct builtintbl *t;
	int i,j;
	char *argv1;
	if(!Z_index)
		return;
	if( argv[index+1][0] != '-' ) //have argument 
		t= (struct builtintbl*)bsearch(argv[index+1],R_TBLs[T_index],R_TBLsize[T_index],sizeof(struct builtintbl),pstrcmp);
	if(t == 0) // can not find table
		return;
	/* remove -z xx, and add -I xx number */
	if(Z_index < index)
	{
		strcpy(argv[Z_index],"101\0");
		argv1=argv[Z_index];
		for(i=Z_index;i<index;i++)
			argv[i] = argv[i+2];
		argv[index]=argv1;
		for(j=index+1;j<((*argc)-1);j++)
			argv[j] = argv[j+1];
	}
	else
	{
		strcpy(argv[Z_index],"121\0");
		argv1=argv[Z_index];
		for(i=Z_index;i>(index+2);i--)
			argv[i] = argv[i-1];
		argv[index+2]=argv1;
		for(j=Z_index+1;j<((*argc)-1);j++)
			argv[j] = argv[j+1];
	}
	(*argc)-=1;
	for(i=0;i<(*argc);i++)
		printf("%s ",argv[i]);
	printf("\n");
	/* add table counts */
}
void R_Table_Handler_A(int* argc, char* argv[], int z, int Z_index, int index, int T_index)
{
	struct builtintbl *t;
	int i,j;
	char *argv1;
	if(!Z_index)
		return;
	if( argv[index+1][0] != '-' ) //have argument 
		t= (struct builtintbl*)bsearch(argv[index+1],R_TBLs[T_index],R_TBLsize[T_index],sizeof(struct builtintbl),pstrcmp);
	if(t == 0) // can not find table
		return;
	/* remove -z xx, and change to -I xx number */
	argv[index][1]='I';
	if(Z_index < index)
	{
		strcpy(argv[Z_index],"101\0");
		argv1=argv[Z_index];
		for(i=Z_index;i<index;i++)
			argv[i] = argv[i+2];
		argv[index]=argv1;
		for(j=index+1;j<((*argc)-1);j++)
			argv[j] = argv[j+1];
	}
	else
	{
		strcpy(argv[Z_index],"121\0");
		argv1=argv[Z_index];
		for(i=Z_index;i>(index+2);i--)
			argv[i] = argv[i-1];
		argv[index+2]=argv1;
		for(j=Z_index+1;j<((*argc)-1);j++)
			argv[j] = argv[j+1];
	}
	(*argc)-=1;
	for(i=0;i<(*argc);i++)
		printf("%s ",argv[i]);
	printf("\n");
	/* add table counts */
}
typedef void (*pt2Func)(int* argc, char* argv[], int z, int Z_index, int index, int T_index);
pt2Func R_Table_Handler_Func[]={0,&R_Table_Handler_D,&R_Table_Handler_F,&R_Table_Handler_I,&R_Table_Handler_A};

/**************************************************************/
/* End of Mapping of table name and table */
/**************************************************************/

static void preprocess_iptable1(int* argc, char* argv[])
{
	int i,j,k;
	int index=0;
	int Z_index=0;
	int z=0;
	R_Table_Name T_index=T_filter;
	R_Table_Handler H_index=T_None;
	for(i=0;i<(*argc);i++)
	{
		if(strcmp(argv[i],"-z") == 0)
		{
			z = atoi(argv[i+1]);
			Z_index = i;
		}
		if(strcmp(argv[i],"-D") == 0)
		{
			//delete rules
			index=i;
			H_index=T_D;
		}
		else if(strcmp(argv[i],"-F") == 0)
		{
			//clear table counts
			index=i;
			H_index=T_F;
		}
		else if(strcmp(argv[i],"-I") == 0)
		{
			index=i; //add iptables rule order number
			H_index=T_I;
		}
		else if(strcmp(argv[i],"-A") == 0)
		{
			index=i; //add iptables rule order number and change to -I rules
			H_index=T_A;
		}
		else if(strcmp(argv[i],"-t") == 0)
		{
			//clear table counts
			if(strcmp(argv[i+1],"nat") == 0)
				T_index=T_nat;
			else if (strcmp(argv[i+1],"mangle") == 0)
				T_index=T_mangle;
		}
	}
	if( R_Table_Handler_Func[H_index] )
		(*R_Table_Handler_Func[H_index])(argc,argv,z,Z_index,index,T_index);

	
}


void
R_iptable_init ()
{
	/*
  t3.parent = &t2;
  t2.parent = &t1;
  */
}


R_iptable_table *
R_next (R_iptable_table * t, int debug)
{
  if (t->count == (t->limit - 1))
    {
      if (t->nr[t->count].ptr == 0)
	{
	  if (debug)
	    printf ("%d:%d\n", t->nr[t->count].value,
		    t->nr[t->count].checked);
	  return t->parent;
	}
      else
	{
	  if (debug)
	    printf ("%d:%d\n", t->nr[t->count].value,
		    t->nr[t->count].checked);
	  return t->nr[t->count++].ptr;
	}
    }
  else if (t->count == (t->limit))
    {
      return t->parent;
    }
  else if (t->nr[t->count].ptr != 0)
    {
      if (debug)
	printf ("%d:%d\n", t->nr[t->count].value, t->nr[t->count].checked);
      return t->nr[t->count++].ptr;
    }
  else
    {
      if (debug)
	printf ("%d:%d\n", t->nr[t->count].value, t->nr[t->count].checked);
      t->count++;
      return t;
    }
}

inline void
clear_all_R_table ()
{
	/*
  t1.count = 0;
  t2.count = 0;
  t3.count = 0;
  */
}

int
op (R_iptable_table * table, int value, int mode)
{
  R_iptable_table *t;
  int order = 0;
  int debug = 0;
  int ret_value = R_ERR;
  if (mode == OPNONE)
    debug = 1;


  clear_all_R_table ();
  t = table;
  while (t)
    {
      if (t->nr[t->count].value == value)
	{
	  switch (mode)
	    {
	    case OPADD:
	      if (t->nr[t->count].checked != 1)
		{
		  t->nr[t->count].checked = 1;
		  return (order + 1);
		}
	      else
		ret_value = R_ERR_ADD;
	      break;
	    case OPDEL:
	      if (t->nr[t->count].checked != 0)
		{
		  t->nr[t->count].checked = 0;
		  if (ret_value != R_ERR_DEL && t->type != 1)
		    return (order + 1);
		  else
		    return R_ERR_DEL;
		}
	      else
		ret_value = R_ERR_DEL;
	      break;
	    case OPNONE:
	    default:
	      break;
	    }
	}
      if (t->nr[t->count].checked)
	order++;
      t = R_next (t, debug);
    }
  if (mode == OPNONE)
    {
      return order;
    }
  else
    {
      return ret_value;
    }

}

int
main (int argc, char *argv[])
{
  int id, op1;
  int i;
  R_iptable_init ();
preprocess_iptable1(&argc,argv);
#if 0
  for (i = 0; i < argc - 1; i++)
    {
      printf ("%s ", argv[i + 1]);
    }
  printf ("\n");
  for (i = 0; i < argc - 1; i++)
    {
      sscanf (argv[i + 1], "%d:%d", &id, &op1);
      printf ("ret: %d\n", op (&t1, id, op1));
      printf ("total: %d\n", op (&t1, 0, OPNONE));

    }
#endif
}

