/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file thpd.c
 *  @brief TMOG HTTP Proxy Module - main
 */

/*
 *  Change Log
 *  xxxx-xx-xx name    description
 */
//#undef NDEBUG
#define NDEBUG
#include <stdio.h>
#include <unistd.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/poll.h>
#include <arpa/inet.h>
#include <signal.h>
#include <errno.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <locale.h>
#include <libintl.h>
//#define	TCP_MAXSEG	 2	/* Set maximum segment size  */
#define IP_REDIRECT
#ifdef  IP_REDIRECT
#include <limits.h>
#include <linux/netfilter_ipv4.h>
#endif
#include <stdio.h>
#include <linux/tc3162dmtctrl.h>
#include <../websconfig/lan_conf.h>
#include <../websconfig/home.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "../tmss/include/log.h"
#include "../tmss/include/tmPidUtil.h"
#include "rr_types.h"
#include "dns.h"
extern void *mempcpy (void *__restrict __dest,
		      __const void *__restrict __src, size_t __n) __THROW;
/* special rr type codes for queries */
#define QT_MIN    251
#define QT_IXFR   251
#define QT_AXFR   252
#define QT_MAILB  253
#define QT_MAILA  254
#define QT_ALL    255
#define QT_MAX    255
#define QT_NUM      5

/* rr classes */
#define C_MIN       1
#define C_IN        1
#define C_CS        2
#define C_CH        3
#define C_HS        4
#define C_MAX       4
#define C_NUM       4

/* special classes for queries */
#define QC_ALL    255

/* status codes */
#define RC_OK       0
#define RC_FORMAT   1
#define RC_SERVFAIL 2
#define RC_NAMEERR  3
#define RC_NOTSUPP  4
#define RC_REFUSED  5

/*
 * special internal retvals
 */
#define RC_TCPREFUSED 254
#define RC_TRUNC      255

/* query/response */
#define QR_QUERY    0
#define QR_RESP     1

/*opcodes */
#define OP_QUERY    0
#define OP_IQUERY   1
#define OP_STATUS   2

/* Macro for standard records. No need to use it. Use with care (can produce
 * strange-looking error messages) */
#define RR_ENT(name,tp) {name, RRCL_ ## tp, RRX_ ## tp}

/* There could be a separate table detailing the relationship of types, but this
 * is slightly more flexible, as it allows a finer granularity of exclusion. Also,
 * Membership in multiple classes could be added. */
struct rr_infos rr_info[]= {
	RR_ENT("A", 		RECORD),
	RR_ENT("NS", 		IDEM),
	RR_ENT("MD", 		IDEM),
	RR_ENT("MF", 		IDEM),
	RR_ENT("CNAME", 	ALIAS),
	RR_ENT("SOA", 		IDEM),
	RR_ENT("MB", 		IDEM),
	RR_ENT("MG", 		IDEM),
	RR_ENT("MR", 		IDEM),
	RR_ENT("NULL",		IDEM),
	RR_ENT("WKS", 		RECORD),
	RR_ENT("PTR", 		PTR),
	RR_ENT("HINFO", 	RECORD),
	RR_ENT("MINFO", 	IDEM),
	RR_ENT("MX", 		IDEM),
	RR_ENT("TXT", 		IDEM),
#ifdef DNS_NEW_RRS
	RR_ENT("RP",		RECORD),
	RR_ENT("AFSDB",		RECORD),
	RR_ENT("X25",		RECORD),
	RR_ENT("ISDN",		RECORD),
	RR_ENT("RT",		RECORD),
	RR_ENT("NSAP",		RECORD),
	RR_ENT("NSAP_PTR",	PTR),		/* broken */
	RR_ENT("SIG",		IDEM),
	RR_ENT("KEY",		IDEM),
	RR_ENT("PX",		IDEM),
	RR_ENT("GPOS",		RECORD),
	RR_ENT("AAAA",		RECORD),
	RR_ENT("LOC",		RECORD),
	RR_ENT("NXT",		IDEM),
	RR_ENT("EID",		RECORD),
	RR_ENT("NIMLOC",	RECORD),
	RR_ENT("SRV",		RECORD),
	RR_ENT("ATMA",		RECORD),
	RR_ENT("NAPTR",		RECORD),
	RR_ENT("KX",		RECORD)
#endif
};

/*
 * the flag values for whole domains in the cache
 */
#define DF_NEGATIVE    1       /* this one is for whole-domain negative caching (created on NXDOMAIN)*/
#define DF_LOCAL       2       /* local record (in conj. with DF_NEGATIVE) */
#define DF_AUTH        4       /* authoritative record */
#define DF_WILD        8       /* subdomains of this domain have wildcard records */
/*
 * the flag values for RR sets in the cache
 */
#define CF_NEGATIVE    1       /* this one is for per-RRset negative caching*/
#define CF_LOCAL       2       /* Local zone entry */
#define CF_AUTH        4       /* authoritative record */
#define CF_NOCACHE     8       /* Only hold for the cache latency time period, then purge. Not really written to cache records, but used by add_cent_rrset */
#define CF_ADDITIONAL 16       /* This was fetched as an additional or "off-topic" record. */
#define CF_NOPURGE    32       /* Do not purge this record */
#define CF_ROOTSERV   64       /* This record was directly obtained from a root server */

/*******************************************************************************
* Porting Required Section Start
*******************************************************************************/

#define PIDFILE             "/var/run/FakedDnsProxy.pid"

#define pdnsd_calloc	calloc
#define pdnsd_malloc	malloc
#define pdnsd_realloc	realloc
#define pdnsd_free	free

/*******************************************************************************
* Porting Required Section End
*******************************************************************************/

static const char ModuleName[]="FakedDnsProxy";

//#define THPD_MAX_PROC	          (50)
#define THPD_MAX_PROC	          (1)
//#define THPD_HTTP_PROXY_PORT      (10080)

#define DNS_PROXY_PORT (10053)
static volatile int GO_HELL=0;

/************************************************************************
 *
 *
 *
 ***********************************************************************/


/*******************************************************************************
* Porting Required Section Start
*******************************************************************************/

static void sig_hup(int signo)
{
    LOGERROR("Faked DNS shall BYE BYTE rec sig_hup signal=%d",signo);
	GO_HELL=1;
}

/*******************************************************************************
* Porting Required Section End
*******************************************************************************/

static void go_hell_handler(int signo)
{
    LOGERROR("Faked DNS shall BYE BYTE:signal=%d",signo);
    GO_HELL=1;
}

int proxyServiceOn=0;//Yes, we only support one proxy connection at a time.
//int poll_ret;
//int sd;
//int optval;
//struct sockaddr_in service_addr;
//int client_sd;
//struct sockaddr_in client_addr;
//socklen_t client_addr_len;

#if 1
/*
 * These values are converted to host byte order. the data is _not_.
 */
typedef struct rr_b_s  {
	struct rr_b_s    *next;                   /* this is the next pointer in the dns_cent_t list. */
	unsigned short   rdlen;
} rr_bucket_t;

typedef struct {
	struct rr_lent_s *lent;                   /* this points to the list entry */
	time_t           ttl;
	time_t           ts;
	unsigned short   flags;
	rr_bucket_t      *rrs;
} rr_set_t;

typedef struct {
	unsigned char    *qname;                  /* Name of the query in length byte - string notation. */
	size_t           cs;                      /* size of the rrs*/
	short            num_rrs;                 /* The number of rrs. When this decreases to 0, the cent is deleted. */
	unsigned short   flags;                   /* Flags for the whole cent */
	time_t           ts;                      /* Timestamp (only for negative cached records) */
	time_t           ttl;                     /* TTL       (  "   "     "       "       "   ) */ 
	struct rr_lent_s *lent;                   /* lent for the whole cent, only for neg. cached recs */
	rr_set_t         *(rr[T_NUM]);            /* The records. Use the type id-T_MIN as index, */
	unsigned char    c_ns,c_soa;              /* Number of trailing name elements in qname to use to find NS or SOA
						     records to add to the authority section of a response. */
} dns_cent_t;

/* This value is used to represent an undefined c_ns or c_soa field. */
#define cundef 0xff

typedef struct rre_s {
	unsigned short tp;
	unsigned short tsz;
	unsigned char  tnm[0];		/* Name for the domain a record refers to */
	/* unsigned char  nm[0]; */	/* Name of the domain the record is for (if tp==T_NS or T_SOA) */
	/* time_t         ttl; */	/* ttl of the record in the answer (if tp==T_NS or T_SOA) */
} rr_ext_t;

typedef struct {
	unsigned short qtype;
	unsigned short qclass;
	unsigned char  query[0];
} dns_queryel_t;


#define S_ANSWER     1
#define S_AUTHORITY  2
#define S_ADDITIONAL 3


typedef struct {
	unsigned short tp,dlen;
	unsigned char nm[0];
	/* unsigned char data[0]; */
} sva_t; 


/*
 * Assert macro, used in some places. For now, it should be always defined, not
 * only in the DEBUG case, to be on the safe side security-wise.
 */
#define dlist_free free

#define pdnsd_calloc	calloc
#define pdnsd_malloc	malloc
#define pdnsd_realloc	realloc 
#define pdnsd_free	free
#define AR_NUM 5
static const int ar_recs[AR_NUM]={T_NS, T_MD, T_MF, T_MB, T_MX}; 
static const int ar_offs[AR_NUM]={0,0,0,0,2}; /* offsets from record data start to server name */

static time_t ans_ttl(time_t ttl, time_t ts, time_t queryts, unsigned flags)
{
	if (!(flags&CF_LOCAL)) {
		time_t tpassed=queryts-ts;
		if(tpassed<0) tpassed=0;
		ttl -= tpassed;
		if(ttl<0) ttl=0;
	}
	return ttl;
}

#if 1
//DEBUG>0
/*
 * Const decoders for debugging display
 */
char *c_names[C_NUM] = {"IN","CS","CH","HS"};
char *qt_names[QT_NUM]={"IXFR","AXFR","MAILB","MAILA","*"};

char *get_cname(int id)
{
	if (id>=C_MIN && id<=C_MAX)
		return c_names[id-C_MIN];
	if (id==QC_ALL)
		return "*";
	return "[unknown]";
}

char *get_tname(int id)
{
	if (id>=T_MIN && id<=T_MAX)
		return rr_info[id-T_MIN].name;
        else if (id>=QT_MIN && id<=QT_MAX)
		return qt_names[id-QT_MIN];
	return "[unknown]";
}

char *e_names[RC_REFUSED+1]={"no error","query format error","server failed","unknown domain","not supported","query refused"};

char *get_ename(int id)
{
	if (id<0 || id>RC_REFUSED)
		return "[unknown]";
	return e_names[id];
}


#endif
/* Note added by Paul Rombouts:
   Compared to the definition used by Thomas Moestl (strlen(rhn)+1), the following definition of rhnlen
   may yield a different result in certain error situations (when a domain name segment contains null byte).
*/
static unsigned int rhnlen(const unsigned char *rhn)
{
	unsigned int i=0,lb;

	while((lb=rhn[i]))
		i+=lb+1;
	return i+1;
}

/* Skip a name in length-byte string notation and return a pointer to the
   position right after the terminating null byte.
*/
static unsigned char *skiprhn(unsigned char *rhn)
{
	unsigned lb;

	while((lb= *rhn))
		rhn += lb+1;
	return rhn+1;
}

/*
 * Non-validating rhn copy (use with checked or generated data only).
 * Returns number of characters copied. The buffer dst points to is assumed to be 256 (or
 * at any rate large enough) bytes in size.
 * The answer assembly code uses this; it is guaranteed to not clobber anything
 * after the name.
 */
unsigned int rhncpy(unsigned char *dst, const unsigned char *src)
{
	unsigned int len = rhnlen(src);

	//PDNSD_ASSERT(len<=256,"rhncpy: src too long!");
	memcpy(dst,src,len>256?256:len);
	return len;
}
/* compare two names in length byte - string format */
static int rhnicmp(const unsigned char *a, const unsigned char *b)
{
	int i=0;
	unsigned char lb;
	for(;;) {
		lb=a[i];
		if(lb!=b[i]) return 0;
		if(!lb) break;
		++i;
		do {
			if(tolower(a[i])!=tolower(b[i])) return 0;
			++i;
		} while(--lb);
	}
	return 1;
}

#define DLISTALIGN(len) (((len) + (sizeof(size_t)-1)) & ~(sizeof(size_t)-1))
#define DLISTCHUNKSIZEMASK 0x3ff

/* Add space for a new item of size len to the list a. */
dlist dlist_grow(dlist a, size_t len)
{
	size_t szincr=DLISTALIGN(len+sizeof(size_t)), sz=0, allocsz=0, newsz;
	if(a) {
		sz=a->last+a->lastsz;
		allocsz = (sz+DLISTCHUNKSIZEMASK)&(~DLISTCHUNKSIZEMASK);
		*((size_t *)&a->data[a->last])=a->lastsz;
	}
	newsz=sz+szincr;
	if(!a || newsz>allocsz) {
		dlist tmp;
		allocsz = (newsz+DLISTCHUNKSIZEMASK)&(~DLISTCHUNKSIZEMASK);
		tmp=realloc(a, sizeof(struct _dynamic_list_head)+allocsz);
		if (!tmp)
			free(a);
		a=tmp;
	}
	if(a) {
		a->last=sz;
		a->lastsz=szincr;
		*((size_t *)&a->data[sz])=0;
	}
	return a;
}
/* Decompress a name record, taking the whole message as msg, returning its results in tgt (max. 255 chars),
 * taking sz as the remaining msg size (it is returned decremented by the name length, ready for further use) and
 * a source pointer (it is returned pointing to the location after the name). msgsize is the size of the whole message,
 * len is the total name length.
 * msg and msgsz are needed for decompression (see rfc1035). The returned data is decompressed, but still in the
 * rr name form (length byte - string of that length, terminated by a 0 length byte).
 *
 * Returned is a dns return code, with one exception: RC_TRUNC, as defined in dns.h, indicates that the message is
 * truncated at the name (which needs a special return code, as it might or might not be fatal).
 */
int decompress_name(unsigned char *msg, long msgsz, unsigned char **src, long *sz, unsigned char *tgt, int *len)
{
	unsigned char lb;
	unsigned offs;
	unsigned char *lptr;
	int hops=0;
	int tpos=0;
	long osz=*sz;

	lptr=*src;
	for(;;) {
		if (*sz<=0)
			return RC_TRUNC;
		if (lptr-msg>=msgsz)
			return RC_FORMAT;
		(*sz)--;
		lb=*lptr++;

		if(lb>0x3f) {
 			if (lb<0xc0)     /* The two highest bits must be either 00 or 11 */
				return RC_FORMAT;
			if (*sz<=0)
				return RC_TRUNC;
			if (lptr-msg>=msgsz)
				return RC_FORMAT;
			(*sz)--;
			offs=(((unsigned int)lb&0x3f)<<8)|(*lptr);
			if (offs>=msgsz) 
				return RC_FORMAT;
			lptr=msg+offs;
			goto jumped;
		}
 		tgt[tpos++]=lb;
		if (lb==0)
			break;

		if (tpos+lb>255) /* terminating null byte has to follow */
			return RC_FORMAT;
		do {
			if (*sz<=0)
				return RC_TRUNC;
			if (lptr-msg>=msgsz)
				return RC_FORMAT;
			/* if (!*lptr || *lptr=='.')
				return RC_FORMAT; */
			(*sz)--;
			tgt[tpos++]=*lptr++;
		} while(--lb);
	}
	goto return_OK;

 jumped:
	++hops;
	for(;;) {
		lb=*lptr++;

		while(lb>0x3f) {
 			if (lb<0xc0)     /* The two highest bits must be either 00 or 11 */
				return RC_FORMAT;
			if (lptr-msg>=msgsz)
				return RC_FORMAT;
			if (++hops>255)
				return RC_FORMAT;
			offs=(((unsigned int)lb&0x3f)<<8)|(*lptr);
			if (offs>=msgsz) 
				return RC_FORMAT;
			lptr=msg+offs;
			lb=*lptr++;
		}
 		tgt[tpos++]=lb;
		if (lb==0)
			break;

		if (lptr+lb-msg>=msgsz || tpos+lb>255) /* terminating null byte has to follow */
			return RC_FORMAT;
		do {
			/* if (!*lptr || *lptr=='.')
				return RC_FORMAT; */
			tgt[tpos++]=*lptr++;
		} while(--lb);
	}
 return_OK:
	*src+=osz-*sz;
	if(len) *len=tpos;
	return RC_OK;
}
/* Compare the names (in length byte-string notation) back-to-forth and return the longest match.
   The comparison is done at name granularity.
   The return value is the length of the match in name elements.
   *os (*od) is set to the offset in the domain name ms (md) of the match.
 */
int domain_match(const unsigned char *ms, const unsigned char *md, int *os, int *od)
{
	int i,j,k,n,ns=0,nd=0,offs,offd;
	unsigned char lb,ls[128],ld[128];

	/* first collect all length bytes */
	i=0;
	while((lb=ms[i])) {
		PDNSD_ASSERT(ns<128, "domain_match: too many name segments");
		ls[ns++]=lb;
		i += ((unsigned)lb)+1;
	}

	j=0;
	while((lb=md[j])) {
		PDNSD_ASSERT(nd<128, "domain_match: too many name segments");
		ld[nd++]=lb;
		j += ((unsigned)lb)+1;
	}

	n=ns;  if(n>nd) n=nd;

	for(k=1; offs=i,offd=j,k<=n; ++k) {
		lb=ls[ns-k];
		if(lb!=ld[nd-k]) goto mismatch;
		for(;lb;--lb)
			if(tolower(ms[--i]) != tolower(md[--j])) goto mismatch;
		--i; --j;
	}
 mismatch:

	if(os) *os=offs;
	if(od) *od=offd;
	return k-1;
}
/*
 * Add data from a rr_bucket_t (as in cache) into a dns message in ans. Ans is grown
 * to fit, sz is the old size of the packet (it is modified so at the end of the procedure
 * it is the new size), type is the rr type and ltime is the time in seconds the record is
 * old.
 * cb is the buffer used for message compression. *cb should be NULL when you call compress_name
 * or add_to_response the first time.
 * It gets filled with a pointer to compression information that can be reused in subsequent calls
 * to add_to_response.
 * sect is the section (S_ANSWER, S_AUTHORITY or S_ADDITIONAL) in which the record 
 * belongs logically. Note that you still have to add the rrs in the right order (answer rrs first,
 * then authority and last additional).
 */
static int add_rr(dns_hdr_t **ans, long *sz, unsigned short type, int dlen, void *data, char section, dlist *cb,
		  char udp, unsigned char *rrn, time_t ttl)
{
	int ilen,blen,osz,rdlen;
	unsigned char *rrht;

	osz=*sz;
	{
		int nlen;
		unsigned char nbuf[256];

		if (!(nlen=compress_name(rrn,nbuf,*sz,cb)))
		{
			LOGERROR("can not compress_name ");
			goto failed;
		}
		/* This buffer is over-allocated usually due to compression. Never mind, just a few bytes,
		 * and the buffer is freed soon*/
		{
			dns_hdr_t *nans=(dns_hdr_t *)pdnsd_realloc(*ans,*sz+nlen+sizeof(rr_hdr_t)+dlen+2);
			if (!nans)
			{
				LOGERROR("can not alloca nans");
				goto failed;
			}
			*ans=nans;
		}
		memcpy((unsigned char *)(*ans)+*sz,nbuf,nlen);
		*sz+=nlen;
	}

	/* the rr header will be filled in later. Just reserve some space for it. */
	rrht=((unsigned char *)(*ans))+(*sz);
	*sz+=sizeof(rr_hdr_t);
	
	switch (type) {
	case T_CNAME:
	case T_MB:
	case T_MD:
	case T_MF:
	case T_MG:
	case T_MR:
	case T_NS:
	case T_PTR:
		if (!(rdlen=compress_name(((unsigned char *)data), ((unsigned char *)(*ans))+(*sz),*sz,cb)))
		{
			LOGERROR("can not compress_name for type=%d ",type);
			goto failed;
		}
		*sz+=rdlen;
		break;
	case T_MINFO:
#ifdef DNS_NEW_RRS
	case T_RP:
#endif
		if (!(rdlen=compress_name(((unsigned char *)data), ((unsigned char *)(*ans))+(*sz),*sz,cb)))
		{
			LOGERROR("can not compress_name for type=%d ",type);
			goto failed;
		}
		*sz+=rdlen;
		ilen=rhnlen((unsigned char *)data);
		PDNSD_ASSERT(rdlen <= ilen, "T_MINFO/T_RP: got longer");
		if (!(blen=compress_name(((unsigned char *)data)+ilen, ((unsigned char *)(*ans))+(*sz),*sz,cb)))
		{
			LOGERROR("can not compress_name for type=%d 2222 ",type);
			goto failed;
		}
		rdlen+=blen;
		*sz+=blen;
		break;
	case T_MX:
#ifdef DNS_NEW_RRS
	case T_AFSDB:
	case T_RT:
	case T_KX:
#endif
		PDNSD_ASSERT(dlen > 2, "T_MX/T_AFSDB/...: rr botch");
		memcpy(((unsigned char *)(*ans))+(*sz),(unsigned char *)data,2);
		*sz+=2;
		if (!(blen=compress_name(((unsigned char *)data)+2, ((unsigned char *)(*ans))+(*sz),*sz,cb)))
		{
			LOGERROR("can not compress_name for type=%d",type);
			goto failed;
		}
		rdlen=2+blen;
		*sz+=blen;
		break;
	case T_SOA:
		if (!(rdlen=compress_name(((unsigned char *)data), ((unsigned char *)(*ans))+(*sz),*sz,cb)))
		{
			LOGERROR("can not compress_name for type=%d",type);
			goto failed;
		}
		*sz+=rdlen;
		ilen=rhnlen((unsigned char *)data);
		PDNSD_ASSERT(rdlen <= ilen, "T_SOA: got longer");
		if (!(blen=compress_name(((unsigned char *)data)+ilen, ((unsigned char *)(*ans))+(*sz),*sz,cb)))
		{
			LOGERROR("can not compress_name for type=%d 22222\n",type);
			goto failed;
		}
		rdlen+=blen;
		*sz+=blen;
		ilen+=rhnlen(((unsigned char *)data)+ilen);
		PDNSD_ASSERT(rdlen <= ilen, "T_SOA: got longer");
		memcpy(((unsigned char *)(*ans))+(*sz),((unsigned char *)data)+ilen,20);
		*sz+=20;
		rdlen+=20;
		break;
#ifdef DNS_NEW_RRS
	case T_PX:
		PDNSD_ASSERT(dlen > 2, "T_PX: rr botch");
		memcpy(((unsigned char *)(*ans))+(*sz),(unsigned char *)data,2);
		*sz+=2;
		ilen=2;
		if (!(blen=compress_name(((unsigned char *)data)+ilen, ((unsigned char *)(*ans))+(*sz),*sz,cb)))
		{
			LOGERROR("can not compress_name for type=%d",type);
			goto failed;
		}
		rdlen=2+blen;
		*sz+=blen;
		ilen+=rhnlen(((unsigned char *)data)+ilen);
		PDNSD_ASSERT(rdlen <= ilen, "T_PX: got longer");
		if (!(blen=compress_name(((unsigned char *)data)+ilen, ((unsigned char *)(*ans))+(*sz),*sz,cb)))
		{
			LOGERROR("can not compress_name for type=%d  22222",type);
			goto failed;
		}
		rdlen+=blen;
		*sz+=blen;
		break;
	case T_SRV:
		PDNSD_ASSERT(dlen > 6, "T_SRV: rr botch");
		memcpy(((unsigned char *)(*ans))+(*sz),(unsigned char *)data,6);
		*sz+=6;
		if (!(blen=compress_name(((unsigned char *)data)+6, ((unsigned char *)(*ans))+(*sz),*sz,cb)))
		{
			LOGERROR("can not compress_name for type=%d",type);
			goto failed;
		}
		rdlen=6+blen;
		*sz+=blen;
		break;
	case T_NXT:
		if (!(blen=compress_name(((unsigned char *)data), ((unsigned char *)(*ans))+(*sz),*sz,cb)))
		{
			LOGERROR("can not compress_name for type=%d",type);
			goto failed;
		}
		rdlen=blen;
		*sz+=blen;
		ilen=rhnlen((unsigned char *)data);
		PDNSD_ASSERT(rdlen <= ilen, "T_NXT: got longer");
		PDNSD_ASSERT(dlen >= ilen, "T_NXT: rr botch");
		{
			int wlen=dlen < ilen ? 0 : (dlen - ilen);
			memcpy(((unsigned char *)(*ans))+(*sz),((unsigned char *)data)+ilen,wlen);
			*sz+=wlen;
			rdlen+=wlen;
		}
		break;
	case T_NAPTR:
		PDNSD_ASSERT(dlen > 4, "T_NAPTR: rr botch");
		ilen=4;
		{
			int j;
			for (j=0;j<3;j++) {
				ilen += ((int)*(((unsigned char *)data)+ilen)) + 1;
				PDNSD_ASSERT(dlen > ilen, "T_NAPTR: rr botch 2");
			}
		}
		memcpy(((unsigned char *)(*ans))+(*sz),((unsigned char *)data),ilen);
		(*sz)+=ilen;

		if (!(blen=compress_name(((unsigned char *)data)+ilen, ((unsigned char *)(*ans))+(*sz),*sz,cb)))
		{
			LOGERROR("can not compress_name for type=%d",type);
			goto failed;
		}
		rdlen=ilen+blen;
		*sz+=blen;
		break;
#endif
	default:
		{
			LOGDEBUG("default type=%d",type);
		}
		rdlen=dlen;
		memcpy(((unsigned char *)(*ans))+(*sz),((unsigned char *)data),dlen);
		*sz+=dlen;
	}

	if (udp && (*sz)>512 && section==S_ADDITIONAL) /* only add the record if we do not increase the length over 512 */
		*sz=osz;                               /* in additionals for udp answer*/
	else {
		PUTINT16(type,rrht);
		PUTINT16(C_IN,rrht);
		PUTINT32(ttl,rrht);
		PUTINT16(rdlen,rrht);

		switch (section) {
		case S_ANSWER:
			(*ans)->ancount=htons(ntohs((*ans)->ancount)+1);
			break;
		case S_AUTHORITY:
			(*ans)->nscount=htons(ntohs((*ans)->nscount)+1);
			break;
		case S_ADDITIONAL:
			(*ans)->arcount=htons(ntohs((*ans)->arcount)+1);
			break;
		}
	}

	return 1;

 failed:
	pdnsd_free(*ans); *ans=NULL;
	return 0;
}
/*
 * Mark an additional record as added to avoid double records.
 */
static int sva_add(dlist *sva, const unsigned char *rhn, unsigned short tp, unsigned short dlen, void* data)
{
	if (sva) {
		size_t rlen=rhnlen(rhn);
		sva_t *st;
		if (!(*sva=dlist_grow(*sva,sizeof(sva_t)+rlen+dlen))) {
			LOGERROR("can not dlist grow\n");
			return 0;
		}
		st=dlist_last(*sva);
		st->tp=tp;
		st->dlen=dlen;
		memcpy(mempcpy(st->nm,rhn,rlen),data,dlen);
	}
	return 1;
}

/*
 * Add an additional
 */
static int add_additional_rr(unsigned char *rhn, dlist *sva, dns_hdr_t **ans,
			     long *rlen, char udp, dlist *cb, unsigned tp,
			     unsigned dlen,void *data, time_t ttl, int sect)
{
	sva_t *st;

	/* Check if already added; no double additionals */
	/* We do NOT look at the data field for addresses, because I feel one address is enough. */
	for (st=dlist_first(*sva); st; st=dlist_next(st)) {
		if (st->tp==tp && rhnicmp(st->nm,rhn) && st->dlen==dlen &&
		    (memcmp(skiprhn(st->nm),data, dlen)==0))
		{
			return 1;
		}
	}
	/* add_rr will do nothing when sz>512 bytes. */
	if(!add_rr(ans, rlen, tp, dlen,data, sect, cb, udp, rhn, ttl))
		return 0;
	/* mark it as added */
	if (!sva_add(sva,rhn,tp,dlen,data)) {
		pdnsd_free(*ans); *ans=NULL;
		return 0;
	}
	return 1;
}

/*
 * The code below actually handles A and AAAA additionals.
 */
static int add_additional_a(unsigned char *rhn, dlist *sva, dns_hdr_t **ans, long *rlen, char udp, time_t queryts, dlist *cb) 
{
	dns_cent_t theae,*ae=&theae;
	int retval = 1;

	//if ((ae=lookup_cache(rhn,NULL))) 
	{
		rr_set_t *rrset; rr_bucket_t *rr;
		rrset=ae->rr[T_A-T_MIN];
		if (rrset && (rr=rrset->rrs))
		  
			if (!add_additional_rr(rhn, sva, ans, rlen, udp, cb, T_A, rr->rdlen,rr+1,
					       ans_ttl(rrset->ttl,rrset->ts,queryts,rrset->flags),S_ADDITIONAL))
				retval = 0;

#ifdef DNS_NEW_RRS
		if(retval) {
			rrset=ae->rr[T_AAAA-T_MIN];
			if (rrset && (rr=rrset->rrs))
				if (!add_additional_rr(rhn, sva, ans, rlen, udp, cb, T_AAAA, rr->rdlen,rr+1,
						       ans_ttl(rrset->ttl,rrset->ts,queryts,rrset->flags),S_ADDITIONAL))
					retval = 0;
		}
#endif
		//free_cent(ae  DBG1);
		//pdnsd_free(ae);
	}
	return retval;
}
/* compress the domain name in in and put the result (of maximum length of strlen(in)) and
 * fill cb with compression information for further strings.*cb may be NULL initially. 
 * offs is the offset the generated string will be placed in the packet.
 * retval: 0 - error, otherwise length
 * When done, just free() cb (if it is NULL, free will behave correctly).
 * It is guaranteed (and insured by assertions) that the output is smaller or equal in
 * size to the input.
 */
int compress_name(unsigned char *in, unsigned char *out, int offs, dlist *cb)
{
	compel_t *ci;
	int add=1;
	int longest=0,lrem=0,coffs=0;
	int rl=0;
	unsigned ilen = rhnlen(in);

	PDNSD_ASSERT(ilen<=256, "compress_name: name too long");

	/* part 1: compression */
	for (ci=dlist_first(*cb); ci; ci=dlist_next(ci)) {
		int rv,rem,to;
		if ((rv=domain_match(in, ci->s, &rem,&to))>longest) {
			/*
			 * This has some not obvious implications that should be noted: If a 
			 * domain name as saved in the list has been compressed, we only can
			 * index the non-compressed part. We rely here that the first occurence
			 * can't be compressed. So we take the first occurence of a given length.
			 * This works perfectly, but watch it if you change something.
			 */
			longest=rv;
			lrem=rem;
			coffs= ci->index + to;
		}
	}
	if (longest>0) {
		PDNSD_ASSERT(lrem+2 <= ilen, "compress_name: length increased");
		memcpy(out, in,lrem);
		out[lrem]=0xc0|((coffs&0x3f00)>>8);
		out[lrem+1]=coffs&0xff;
		rl=lrem+2;
		add= lrem!=0;
	}
	else {
		memcpy(out,in,ilen);
		rl=ilen;
	}

	/* part 2: addition to the cache structure */
	if (add) {
		if (!(*cb=dlist_grow(*cb,sizeof(compel_t)+ilen)))
			return 0;
		ci=dlist_last(*cb);
		ci->index=offs;
		memcpy(ci->s,in,ilen);
	}
	return rl;
}



/* types for the tp field */
/* #define RRETP_NS	T_NS */		/* For name server: add to authority, add address to additional. */
/* #define RRETP_SOA	T_SOA */	/* For SOA record: add to authority. */
#define RRETP_ADD	0		/* For other records: add the address of buf to additional */
static int add_ar(dlist *ar,unsigned short tp, unsigned short tsz,void *tnm,unsigned char *nm, time_t ttl)
{
	rr_ext_t *re;
	unsigned char *p;
	size_t nmsz=0,size=sizeof(rr_ext_t)+tsz;
	if(tp==T_NS || tp==T_SOA) {
		nmsz=rhnlen(nm);
		size += nmsz + sizeof(time_t);
	}
	if (!(*ar=dlist_grow(*ar,size)))
	{
		LOGERROR("add_ar:can not dlist_grow");
		return 0;
	}
	re=dlist_last(*ar);
	re->tp=tp;
	re->tsz=tsz;
	p=mempcpy(re->tnm,tnm,tsz);
	if(tp==T_NS || tp==T_SOA) {
		p=mempcpy(p,nm,nmsz);
		memcpy(p,&ttl,sizeof(time_t));
	}
	return 1;
}
/* This adds an rrset, optionally randomizing the first element it adds.
 * if that is done, all rrs after the randomized one appear in order, starting from
 * that one and wrapping over if needed. */
static int add_rrset(unsigned tp, dns_hdr_t **ans, long *sz, dns_cent_t *cached, dlist *cb,
		     char udp, unsigned char *rrn, time_t queryts, dlist *sva, dlist *ar)
{
	rr_bucket_t *b;
	rr_bucket_t *first=NULL; /* Initialized to inhibit compiler warning */
	int cnt,i;
	rr_set_t *crrset=cached->rr[tp-T_MIN];
	
	
//	LOGDEBUG("add_rrset is called for tp=%d, rlen=%d",tp,*sz);
	if (crrset && crrset->rrs) {
		int rnd_recs=1;//global.rnd_recs;
		b=crrset->rrs;
		if (rnd_recs) {
			/* in order to have equal chances for each records to be the first, we have to count first. */
			first=b;
			cnt=0;
			while (b) {
				b=b->next;
				cnt++;
			}
			/* We do not use the pdnsd random functions (these might use /dev/urandom if the user is paranoid,
			 * and we do not need any good PRNG here). */
			cnt=random()%cnt;
			while (cnt) {
				cnt--;
				first=first->next;
			}
			b=first;
		}
		while (b) {
			if (!add_rr(ans, sz, tp,b->rdlen,b+1, S_ANSWER,cb,udp,rrn,
				    ans_ttl(crrset->ttl,crrset->ts,queryts,crrset->flags))) 
			{
				LOGDEBUG("no more add_rr\n");
				return 0;
			}
			LOGDEBUG("b present add rr OK\n");
			if (tp==T_NS || tp==T_A || tp==T_AAAA) {
				/* mark it as added */
				if (!sva_add(sva,rrn,tp,b->rdlen,b+1))
				{
					LOGERROR("can not sva_add\n");
					goto failed;
				}					
			}
			/* Mark for additional address records. XXX: this should be a more effective algorithm; at least the list is small */
			for (i=0;i<AR_NUM;i++) {
				if (ar_recs[i]==tp) {
					if (!add_ar(ar, RRETP_ADD,b->rdlen-ar_offs[i],((unsigned char *)(b+1))+ar_offs[i],
						    "", 0))
					{
						LOGERROR("can not add_ar,it is failed\n");
						goto failed;
					}					
					break;
				}
			}
			b=b->next;
			if (rnd_recs) {
				if(!b) b=crrset->rrs; /* wraparound */
				if(b==first)	break;
			}
		}
	}
	return 1;

 failed:
	pdnsd_free(*ans); *ans=NULL;
	return 0;
}
/*
 * Add the fitting elements of the cached record to the message in ans, where ans
 * is grown to fit, sz is the size of the packet and is modified to be the new size.
 * The query is in qe. 
 * cb is the buffer used for message compression. *cb should be NULL if you call add_to_response
 * the first time. It gets filled with a pointer to compression information that can be
 * reused in subsequent calls to add_to_response.
 */
static int add_to_response(unsigned qtype, dns_hdr_t **ans, long *sz, dns_cent_t *cached, dlist *cb,
			   char udp, unsigned char *rrn, time_t queryts, dlist *sva, dlist *ar)
{
	/* first of all, add cnames. Well, actually, there should be at max one in the record. */
	if (qtype!=T_CNAME && qtype!=QT_ALL)
		if (!add_rrset(T_CNAME, ans, sz, cached, cb, udp, rrn, queryts, sva, ar))
			return 0;

	/* We need no switch for qclass, since we already have filtered packets we cannot understand */
	if (qtype==QT_AXFR || qtype==QT_IXFR) {
		/* I do not know what to do in this case. Since we do not maintain zones (and since we are
		   no master server, so it is not our task), I just return an error message. If anyone
		   knows how to do this better, please notify me. 
		   Anyway, this feature is rarely used in client communication, and there is no need for
		   other name servers to ask pdnsd. Btw: many bind servers reject an ?XFR query for security
		   reasons. */
		pdnsd_free(*ans); *ans=NULL;
		return 0; 
	} else if (qtype==QT_MAILB) {
		if (!add_rrset(T_MB, ans, sz, cached, cb, udp, rrn, queryts, sva, ar))
			return 0;
		if (!add_rrset(T_MG, ans, sz, cached, cb, udp, rrn, queryts, sva, ar))
			return 0;
		if (!add_rrset(T_MR, ans, sz, cached, cb, udp, rrn, queryts, sva, ar))
			return 0;
	} else if (qtype==QT_MAILA) {
		if (!add_rrset(T_MD, ans, sz, cached, cb, udp, rrn, queryts, sva, ar))
			return 0;
		if (!add_rrset(T_MF, ans, sz, cached, cb, udp, rrn, queryts, sva, ar))
			return 0;
	} else if (qtype==QT_ALL) {
		unsigned i;
		for (i=T_MIN;i<=T_MAX;i++) {
			if (!add_rrset(i, ans, sz, cached, cb, udp, rrn, queryts, sva, ar))
				return 0;
		}
	} else {
		/* Unsupported elements have been filtered.*/
		if (!add_rrset(qtype, ans, sz, cached, cb, udp, rrn, queryts, sva, ar))
			return 0;
	}
#if 0
	if (!ntohs((*ans)->ancount)) {
		/* Add a SOA if we have one and no other records are present in the answer.
		 * This is to aid caches so that they have a ttl. */
		if (!add_rrset(T_SOA , ans, sz, cached, cb, udp, rrn, queryts, sva, ar))
			return 0;
	}
#endif
	return 1;
}

/*					    if (tp==255) {
						    if (!init_cent(&cent, name, ttl, 0, DF_LOCAL|DF_NEGATIVE  DBG1))
							    goto out_of_memory;
					    } else {
						    if (!init_cent(&cent, name, 0, 0, 0  DBG1))*/
/*
 * Adds an empty rrset_t with the requested data to a cent. This is exactly what you need to
 * do to create a negatively cached cent.
 */
dns_cent_t theCached;
int add_cent_rrset(dns_cent_t *cent, int tp, time_t ttl, time_t ts, unsigned flags)//  DBGPARAM)
{
	/*rr_set_t *rrset=cache_malloc(sizeof(rr_set_t));
	cent->rr[tp-T_MIN]=rrset;
	if (!rrset)
		return 0; */
	rr_set_t *rrset=cent->rr[tp-T_MIN];
	rrset->lent=NULL;
	rrset->ttl=ttl;
	rrset->ts=ts;
	rrset->flags=flags;
	rrset->rrs=NULL;
	cent->cs += sizeof(rr_set_t);
	cent->num_rrs++;
	/* If we add a rrset, even a negative one, the domain is not negative any more. */
	if (cent->flags&DF_NEGATIVE) {
		cent->flags &= ~DF_NEGATIVE;
		cent->ttl=0;
		/* need to remove the cent from the lent list. */
		if (cent->lent) {
			//remove_rrl(cent->lent);
			cent->lent=NULL;
		}
	}
	return 1;
}
/*
 * Create a rr record holder using the given values.
 */
rr_bucket_t theRr;// Only one rr ??
 static rr_bucket_t *create_rr(unsigned short dlen, void *data)
{
	rr_bucket_t *rrb;
	rrb=&theRr;//(rr_bucket_t *)cache_malloc(sizeof(rr_bucket_t)+dlen);
	if (rrb == NULL)
		return NULL;
	rrb->next=NULL;

	rrb->rdlen=dlen;
	memcpy(rrb+1,data,dlen);
	return rrb;
}
/*
 * Adds a rr record to a cent. For cache.c internal use.
 */

static int add_cent_rr_int(dns_cent_t *cent, int tp, time_t ttl, time_t ts, unsigned flags,
			   unsigned dlen, void *data, rr_bucket_t **rtail)
{
	rr_bucket_t *rr;
	rr_set_t *rrset;
	if ((cent->flags&DF_LOCAL) && !(flags&CF_LOCAL)) {
		/* ignore. Local has precedence. */
		return 1;
	}
	if (!(rr=create_rr(dlen,data)))
		return 0;
	if(!(rtail && *rtail)) {
		if (!cent->rr[tp-T_MIN]) {
			if (!add_cent_rrset(cent, tp, ttl, ts, flags))
				goto cleanup_return;
		}
		/* do the linking work */
		rrset=cent->rr[tp-T_MIN];
		rr->next=rrset->rrs;
		rrset->rrs=rr;
	}
	else {
		/* append at the end */
		rr->next=(*rtail)->next;
		(*rtail)->next=rr;
	}
	if(rtail) *rtail=rr;
	cent->cs += sizeof(rr_bucket_t)+rr->rdlen;
	rrset=cent->rr[tp-T_MIN];
	if (rrset->flags&CF_NEGATIVE)
	{
		LOGDEBUG("Tried to add rr to a rrset with CF_NEGATIVE set! flags=%i\n",rrset->flags);
	}
	return 1;
 cleanup_return:
//	free_rr(*rr);
//	free(rr);
	return 0;
}
/* Initialize a dns cache record (dns_cent_t) with the query name (in
 * transport format), a flag value, a timestamp indicating
 * the time the query was done, and a TTL. The timestamp and TTL
 * are only used if DF_NEGATIVE is set in the flags. Otherwise,
 * the timestamps of the individual records are used. DF_NEGATIVE
 * is used for whole-domain negative caching.
 * By convention, the ttl should be set to 0, and the ttl should
 * be set correctly when DF_NEGATIVE is not set. */
int init_cent(dns_cent_t *cent, const unsigned char *qname, time_t ttl, time_t ts, unsigned flags)
{
	rr_bucket_t *rtail;
	size_t namesz=rhnlen(qname);
	
	struct in_addr dbuf;

	/*cent->qname=cache_malloc(namesz);
	if (cent->qname == NULL)
		return 0; */
	memcpy(cent->qname,qname,namesz);
	cent->num_rrs=0;
	cent->cs=sizeof(dns_cent_t)+namesz;
	cent->flags=flags;
	cent->ts=ts;
	cent->ttl=ttl;
	cent->lent=NULL;
	//for(i=0;i<T_NUM;i++)
	//	cent->rr[i]=NULL;
	rtail=NULL;
									/*if ((rrset=servent->rr[T_A-T_MIN]) && rrset->rrs) {
										struct in_addr ina;

										memcpy(&ina,rrset->rrs+1,sizeof(ina));
										IPV6_MAPIPV4(&ina,&serva.ipv6);
									}
					    if (read(rs,dbuf,sizeof(struct in_addr))!=sizeof(struct in_addr))
						    goto bad_arg;
					    sz=; */
	//inet_aton("139.175.212.36",&dbuf); www.billion.com.tw
	//inet_aton("66.102.7.104",&dbuf);//www.google.com
	inet_aton("10.77.139.104",&dbuf);//a faked private one
	//inet_aton("82.209.166.23",&dbuf);//www.brikks.com
	add_cent_rr_int(cent,1,ttl, ts, flags,sizeof(struct in_addr), (void *)&dbuf, &rtail);
	cent->c_ns=cundef;
	cent->c_soa=cundef;
	return 1;
}
/*
 * Compose an answer message for the decoded query in q, hdr is the header of the dns request
 * rlen is set to be the answer length.
 */ 
unsigned char theCacheQname[256];

rr_set_t rr[T_NUM];
static unsigned char *compose_answer(dlist q, dns_hdr_t *hdr, long *rlen, char udp) 
{
	short aa=1;
	dlist sva=NULL;
	dlist ar=NULL;
	dlist cb=NULL;
	time_t queryts=time(NULL);
	dns_queryel_t *qe;
	dns_hdr_t *ans;
	dns_cent_t *cached=&theCached;
	int i;
	
	cached->flags=DF_LOCAL;
	cached->qname=theCacheQname;
	
	for(i=0;i<T_NUM;i++)
	{
		cached->rr[i]=&(rr[i]);
		add_cent_rrset(cached, i+T_MIN, 10, 0, DF_LOCAL|DF_NEGATIVE);
	}


	ans=(dns_hdr_t *)pdnsd_malloc(sizeof(dns_hdr_t));
	if (!ans)
		return NULL;
	ans->id=hdr->id;
	ans->qr=QR_RESP;
	ans->opcode=OP_QUERY;
	ans->aa=0;
	ans->tc=0; /* If tc is needed, it is set when the response is sent in udp_answer_thread. */
	ans->rd=hdr->rd;
	ans->ra=1;
	ans->z1=0;
	ans->au=0;
	ans->z2=0;
	ans->rcode=RC_OK;
	ans->qdcount=0; /* this is first filled in and will be modified */
	ans->ancount=0;
	ans->nscount=0;
	ans->arcount=0;

	*rlen=sizeof(dns_hdr_t);
	/* first, add the query to the response */
	for (qe=dlist_first(q); qe; qe=dlist_next(qe)) {
		int qclen;
		dns_hdr_t *nans=(dns_hdr_t *)pdnsd_realloc(ans,*rlen+rhnlen(qe->query)+4);
		if (!nans)
			goto error_ans;
		ans=nans;
		{
			unsigned char *p = ((unsigned char *)ans) + *rlen;
			/* the first name occurrence will not be compressed,
			   but the offset needs to be stored for future compressions */
			if (!(qclen=compress_name(qe->query,p,*rlen,&cb)))
				goto error_ans;
			p += qclen;
			PUTINT16(qe->qtype,p);
			PUTINT16(qe->qclass,p);
		}
		*rlen += qclen+4;
		ans->qdcount=htons(ntohs(ans->qdcount)+1);
	}

	/* Barf if we get a query we cannot answer */
	for (qe=dlist_first(q); qe; qe=dlist_next(qe)) {
		if (((qe->qtype<T_MIN || qe->qtype>T_MAX) &&
		     (qe->qtype!=QT_MAILB && qe->qtype!=QT_MAILA && qe->qtype!=QT_ALL)) ||
		    (qe->qclass!=C_IN && qe->qclass!=QC_ALL))
		{
			ans->rcode=RC_NOTSUPP;
			return (unsigned char *)ans;
		}
	}
	
	LOGDEBUG("second, the answer section add the answer section");
	/* second, the answer section */
	for (qe=dlist_first(q); qe; qe=dlist_next(qe)) {
		unsigned char qname[256];

	LOGDEBUG("second, the answer section add the answer section  222222");
		rhncpy(qname,qe->query);
		//make up the cache
		//rhncpy(cached->qname,qe->query);
		init_cent(cached, qe->query, 10, time(NULL), DF_LOCAL|DF_NEGATIVE);
		
		if (!add_to_response(qe->qtype,&ans,rlen,cached,&cb,udp,qname,queryts,&sva,&ar))
		{
		LOGERROR("add_to_response failed\n");
			goto error_cached;
		}
	}

	{
		rr_ext_t *rre;
		/* Add the authority section */
		for (rre=dlist_first(ar); rre; rre=dlist_next(rre)) {
			if (rre->tp == T_NS || rre->tp == T_SOA) {
				unsigned char *nm = rre->tnm + rre->tsz;
				time_t ttl;
				memcpy(&ttl,skiprhn(nm),sizeof(time_t));
				if (!add_additional_rr(nm, &sva, &ans, rlen, udp, &cb, rre->tp, rre->tsz,rre->tnm,
						       ttl, S_AUTHORITY))
				{
					/* ans has already been freed and set to NULL */
					goto cleanup_return;
				}
			}
		}

		/* now add the name server addresses */
		for (rre=dlist_first(ar); rre; rre=dlist_next(rre)) {
			if (rre->tp == T_NS || rre->tp == RRETP_ADD) {
				if (!add_additional_a(rre->tnm, &sva, &ans, rlen, udp, queryts, &cb))
					goto cleanup_return;
			}
		}
	}
	if (aa)
		ans->aa=1;
	goto cleanup_return;

	/* You may not like goto's, but here we avoid lots of code duplication. */
error_cached:
	//free_cent(cached  DBG1);
	//pdnsd_free(cached);
error_ans:
	pdnsd_free(ans);
	ans=NULL;
cleanup_return:
	dlist_free(sva);
	dlist_free(ar);
	dlist_free(cb);
	return (unsigned char *)ans;
}

/*
 * Decode the query (the query messgage is in data and rlen bytes long) into q
 * XXX: data needs to be aligned
 */
static int decode_query(unsigned char *data, long rlen, dlist *qp)
{
	int i,res;
	dns_hdr_t *hdr=(dns_hdr_t *)data; /* aligned, so no prob. */
	unsigned char *ptr=(unsigned char *)(hdr+1);
	long sz=rlen-sizeof(dns_hdr_t);
	dlist q;
	uint16_t qdcount=ntohs(hdr->qdcount);
	
	if (qdcount==0) 
		return RC_FORMAT;
	
	q=NULL;
	for (i=0;i<qdcount;i++) {
		dns_queryel_t *qe;
		int qlen;
		unsigned char qbuf[256];
		res=decompress_name(data,rlen,&ptr,&sz,qbuf,&qlen);
		if (res==RC_TRUNC) {
			if (hdr->tc) {
				if (i==0) /*not even one complete query*/
					goto return_rc_format;
				break;
			}
			else
				goto return_rc_format;
		}
		if (res!=RC_OK)
			goto cleanup_return;
		if (sz<4) {
			/* truncated in qname or qclass*/
			if (i==0) /*not even one complete query*/
				goto return_rc_format;
			break;
		}
		if(!(q=dlist_grow(q,sizeof(dns_queryel_t)+qlen)))
			return RC_SERVFAIL;
		qe=dlist_last(q);
		GETINT16(qe->qtype,ptr);
		GETINT16(qe->qclass,ptr);
		sz-=4;
		memcpy(qe->query,qbuf,qlen);
	}
	*qp=q;
	return RC_OK;

 return_rc_format:
	res=RC_FORMAT;
 cleanup_return:
	dlist_free(q);
	return res;
}
/* Make a dns error reply message
 * Id is the query id and still in network order.
 * op is the opcode to fill in, rescode - name says it all.
 */
static void mk_error_reply(unsigned short id, unsigned short opcode,unsigned short rescode,dns_hdr_t *rep)
{
	rep->id=id;
	rep->qr=QR_RESP;
	rep->opcode=opcode;
	rep->aa=0;
	rep->tc=0;
	rep->rd=0;
	rep->ra=1;
	rep->z1=0;
	rep->au=0;
	rep->z2=0;
	rep->rcode=rescode;
	rep->qdcount=0;
	rep->ancount=0;
	rep->nscount=0;
	rep->arcount=0;
}
dns_hdr_t dns_resp;
/*
 * Analyze and answer the query in data. The answer is returned. rlen is at call the query length and at
 * return the length of the answer. You have to free the answer after sending it.
 */
static unsigned char *process_query(unsigned char *data, long *rlenp, char udp)
{
	long rlen= *rlenp;
	int res;
	dns_hdr_t *hdr;
	dlist q;
	dns_hdr_t *ans;


	LOGDEBUG("Received query.");
	/*
	 * We will ignore all records that come with a query, except for the actual query records.
	 * We will send back the query in the response. We will reject all non-queries, and
	 * some not supported thingies. 
	 * If anyone notices behaviour that is not in standard conformance, please notify me!
	 */
	hdr=(dns_hdr_t *)data;
	if (rlen<2) { 
		LOGDEBUG("Message too short.\n");
		return NULL; /* message too short: no id provided. */
	}
	if (rlen<sizeof(dns_hdr_t)) {
		LOGDEBUG("Message too short.\n");
		res=RC_FORMAT;
		goto error_reply;
	}
	if (hdr->qr==QR_RESP) {
		LOGDEBUG("Response, not query.\n");
		return NULL; /* RFC says: discard */
	}
	if (hdr->opcode!=OP_QUERY) {
		LOGDEBUG("No query.\n");
		res=RC_NOTSUPP;
		goto error_reply;
	}
	if (hdr->z1!=0 || hdr->z2!=0) {
		LOGDEBUG("Malformed query.\n");
		res=RC_FORMAT;
		goto error_reply;
	}
	if (hdr->rcode!=RC_OK) {
		LOGDEBUG("Bad rcode.\n");
		return NULL; /* discard (may cause error storms) */
	}

	res=decode_query(data,rlen,&q);
	if (res!=RC_OK) {
		goto error_reply;
	}

#if 0
	{
		dns_queryel_t *qe;
		LOGDEBUG("Questions are:\n");
		for (qe=dlist_first(q); qe; qe=dlist_next(qe)) {
			LOGDEBUG("\tqc=%s (%i), qt=%s (%i), query=\"%s\"\n",get_cname(qe->qclass),qe->qclass,get_tname(qe->qtype),qe->qtype,qe->query);
		}
	}
#endif
#if 1
	if (!(ans=(dns_hdr_t *)compose_answer(q, hdr, rlenp, udp))) {
		/* An out of memory condition or similar could cause NULL output. Send failure notification */
		dlist_free(q);
		res=RC_SERVFAIL;
		goto error_reply;
	}
	dlist_free(q);
#endif
	return (unsigned char *)ans;


 error_reply:
	*rlenp=sizeof(dns_hdr_t);
	{
		dns_hdr_t *resp=&dns_resp;
		mk_error_reply(hdr->id,rlen>=3?hdr->opcode:OP_QUERY,res,resp);
		return (unsigned char *)resp;
	}
}
#else
static unsigned char *process_query(unsigned char *data, long *rlenp, char udp)
{
	return NULL;
}
#endif
#define udp_buf_len 512
char buf[udp_buf_len];
struct msghdr msg;
struct iovec v;
struct cmsghdr *cmsg;
char ctrl[512];
unsigned char *resp;
# define SET_SOCKA_LEN4(socka)
int init_udp_socket(int dns_proxy_port)
{
	int sock;
	int so=1;
	union {
		struct sockaddr_in sin4;
	} sin;
	socklen_t sinl;

	if ((sock=socket(PF_INET,SOCK_DGRAM,IPPROTO_UDP))==-1) {
			LOGERROR("Could not open udp socket: %s\n",strerror(errno));
			return -1;
	}
	memset(&sin.sin4,0,sizeof(struct sockaddr_in));
	sin.sin4.sin_family=AF_INET;
	sin.sin4.sin_port=htons(dns_proxy_port);//global.port);
	sin.sin4.sin_addr.s_addr=INADDR_ANY;//global.a.ipv4;
	SET_SOCKA_LEN4(sin.sin4);
	sinl=sizeof(struct sockaddr_in);

	if (setsockopt(sock,SOL_IP,IP_PKTINFO,&so,sizeof(so))!=0) {
			LOGERROR("Could not set options on udp socket: %s\n",strerror(errno));
			close(sock);
			return -1;
	}
	if (bind(sock,(struct sockaddr *)&sin,sinl)!=0) {
		LOGERROR("Could not bind to udp socket: %s\n",strerror(errno));
		close(sock);
		return -1;
	}
	return sock;
}

#define _PROXY_DNS_53_
#ifdef _PROXY_DNS_53_
#define MAX_LISTEN_APPL 2
int udp_socket_53;
#else
#define MAX_LISTEN_APPL 1
#endif

static struct pollfd pollfdList[MAX_LISTEN_APPL];
static int poolfd_nfds=MAX_LISTEN_APPL;
	int qlen,rlen;
int udp_socket_10053,udp_socket;
	int poll_ret;
#ifdef LIBVER
#define MAINFUNC FakedDnsProxy_main
#else
#define MAINFUNC main
#endif
int MAINFUNC(int argc, char *argv[])
{
	union {
		struct sockaddr_in  sin4;
	}                  addr;
	//char *buf_in;
    LOGOPEN("FDNSLOGIN");
    LOGINFO("init");
    LOGDEBUG_ON();

 /*	if (argc < 2){
		fprintf(stderr,"FakedDnsProxy [proxyPort]\n");
		return -1;
	} */
	//dns_proxy_port=atoi(argv[1]);
    //service_table.sd = -1;
    //service_table.port = DNS_PROXY_PORT;
 


    LOGDEBUG("hook SIGHUP");
    { /* monitor for config reload signal */
        if(SIG_ERR==signal(SIGHUP, sig_hup))
        {
            LOGERROR("signal: %m");
            return 1;
        }
    }

/*******************************************************************************
* Porting Required Section End
*******************************************************************************/

    LOGDEBUG("hook SIGTERM");
    { /* monitor for shutdown signal */
        if(SIG_ERR==signal(SIGTERM, go_hell_handler))
        {
            LOGERROR("signal: %m");
            return 1;
        }
    }

    LOGDEBUG("hook SIGINT");
    { /* monitor for shutdown signal */
        if(SIG_ERR==signal(SIGINT, go_hell_handler))
        {
            LOGERROR("signal: %m");
            return 1;
        }
    }

    LOGINFO("create pid file: %s", PIDFILE);
    write_pid(PIDFILE);

    LOGDEBUG("bind service port");
	/* bind service port */
 

/* 	if ((sd=socket(PF_INET,SOCK_DGRAM,IPPROTO_UDP))==-1) {
			LOGERROR("Could not open udp socket: %m");
			return -1;
	}

	optval=1;
	if (setsockopt(sd, SOL_IP, IP_PKTINFO, &optval, sizeof(optval)) == -1)
	{
		LOGERROR("failed to set options on FAKED DNS PROXY socket: %m");
 		close(sd);
		return -1;
	}
  
	
	memset(&service_addr, 0, sizeof(service_addr));
	service_addr.sin_family=AF_INET;
	service_addr.sin_port=htons(service_table.port);
	service_addr.sin_addr.s_addr=INADDR_ANY;
	if (bind(sd, (struct sockaddr *)&service_addr, sizeof(service_addr)!=0))
	{
		LOGERROR("Could not bind to udp socket: %m");
		close(sd);
		return -1;
	}

	
	
	
	*/
// udp_socket=init_udp_socket(dns_proxy_port);
 udp_socket_10053=init_udp_socket(10053);
	if (udp_socket_10053==-1) {
		LOGERROR("udp initialization failed for port 10053. Exiting.");
		return -1;
	}
#ifdef _PROXY_DNS_53_
 udp_socket_53=init_udp_socket(53);
	if (udp_socket_53==-1) {
		LOGERROR("udp initialization failed for port 53. Exiting.");
		close(udp_socket_10053);
		return -1;
	}
	pollfdList[1].fd=udp_socket_53;
	pollfdList[1].events=POLLIN;
#endif		
	pollfdList[0].fd=udp_socket_10053;
	pollfdList[0].events=POLLIN;
	//listen(sd, 5);
	//listen(sd, 1);


 
   /* wait for connections */
   
	 
	LOGERROR("begin service loop for FakedDnsProxy");
	/* service loop */
	for(;GO_HELL==0;)
	{
		poll_ret=poll(pollfdList, poolfd_nfds, 2*1000);
		if(-1==poll_ret)
		{
			if(errno==EINTR)
			{
				/* go on */
				continue;
			}
			else
			{
				LOGERROR("poll: %m");
				break;
			}
		}
		else if(0==poll_ret)
		{
			continue;
		}
		if(pollfdList[0].revents & POLLIN)
		{
				udp_socket=udp_socket_10053;
			////	LOGERROR("FakedDNSProxy: 10053 DNS ");
		}
#ifdef _PROXY_DNS_53_
		else
		if(pollfdList[1].revents & POLLIN)
		{
				udp_socket=udp_socket_53;
			////	LOGERROR("FakedDNSProxy: 53 DNS ");
		}
#endif
		else
		{
				continue;
		}
			
  //buf_in = pdnsd_malloc(udp_buf_len);
		v.iov_base=buf;
		//v.iov_base=buf_in; 
		v.iov_len=udp_buf_len;
		msg.msg_iov=&v;
		msg.msg_iovlen=1;
		msg.msg_control=NULL;//ctrl;
		msg.msg_controllen=0;//sizeof(ctrl);
		msg.msg_name=&addr.sin4;
		msg.msg_namelen=sizeof(struct sockaddr_in);
		qlen=recvmsg(udp_socket,&msg,0);
		if (qlen<0 && errno!=EINTR) 
		{
			////	LOGERROR("error in UDP recv: %m");
			////	LOGERROR("error in UDP recv:qlen=%i",qlen);
				//???? Why???? then we ignore it?????
			    continue;
		}
		rlen=qlen;
		resp=process_query(buf,(long *)&rlen,1);
		//pdnsd_free(buf_in);
		//if (!(resp=process_query(buf,(long *)&rlen,1))) {
		if (!resp) {
		/*
		 * A return value of NULL is a fatal error that prohibits even the sending of an error message.
		 * logging is already done. Just exit the thread now.
		 */
				LOGERROR("Fatal error: can not send DNS response ");
				continue;
		}
		if (rlen>512) {
			rlen=512;
			((dns_hdr_t *)resp)->tc=1; /*set truncated bit*/
		}
		LOGDEBUG("Outbound  msg len %li, tc=%i, rc=\"%s\"\n",rlen,((dns_hdr_t *)resp)->tc,get_ename(((dns_hdr_t *)resp)->rcode));

		v.iov_base=(char *)resp;
		v.iov_len=rlen;
		msg.msg_iov=&v;
		msg.msg_iovlen=1;
		msg.msg_control=NULL;
		msg.msg_controllen=0;
		msg.msg_flags=0;  /* to avoid warning message by Valgrind */

		msg.msg_name=&addr.sin4;
		msg.msg_namelen=sizeof(struct sockaddr_in);

		msg.msg_name=&addr.sin4;
		msg.msg_namelen=sizeof(struct sockaddr_in);
		if (sendmsg(udp_socket,&msg,0)<0) {
			LOGERROR("Error in udp send: %s\n",strerror(errno));
		} 
		else 
		{
			int tmp;
			socklen_t sl=sizeof(tmp);
			getsockopt(udp_socket, SOL_SOCKET, SO_ERROR, &tmp, &sl);
		}
	}
	LOGERROR("FakedDnsProxy is closed");
#ifdef _PROXY_DNS_53_
    close(udp_socket_53);
#endif
    close(udp_socket_10053);

/*******************************************************************************
* Porting Required Section Start
*******************************************************************************/

    LOGINFO("remove pid file: %s", PIDFILE);
    remove_pid(PIDFILE);


 
/*******************************************************************************
* Porting Required Section End
*******************************************************************************/

    LOGINFO("shutdown");
    return 0;
}


