/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file daemon.c
 *  @brief TMOG HTTP Proxy Module
 */

/*
 *  Change Log
 *  xxxx-xx-xx name    description
 */

#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <errno.h>
#include <unistd.h>

#include "daemon.h"
#include "log.h"
static const char ModuleName[]="daemon";

static volatile int daemon_max_proc=50;
static volatile int daemon_cur_proc=0;

static void daemon_reap_child(int signo)
{
    pid_t pid;
    int status;
    int olderrno=errno;

    while((pid=waitpid(-1, &status, WNOHANG))>0)
    {
        if(daemon_cur_proc>0)
            daemon_cur_proc--;
    }

    errno=olderrno;
}

int daemon_init(int foreground, int max_proc)
{
    pid_t pid;

    if(foreground==0)
    {
        close(STDIN_FILENO);
        close(STDOUT_FILENO);
        close(STDERR_FILENO);
        pid = vfork();
        if(pid==0)
        {
            setsid();
        }
        else if(pid<0)
        {
            LOGERROR("Failed to fork background process.");
            return -1;
        }
        else
        {
            _exit(0);
        }
    }

    umask(07);  // set ug+rw

    /* set max process number */
    daemon_max_proc=max_proc;

    { /* set signal handler of SIGCHLD */
        struct sigaction act, old_act;
        act.sa_handler=daemon_reap_child;
        sigemptyset(&act.sa_mask);
        act.sa_flags=0;
        if(sigaction(SIGCHLD, &act, &old_act)<0)
        {
            LOGERROR("daemon_init(): sigaction(): %m");
            return -1;
        }
    }

    return 0;
}

int daemon_fork(void)
{
    pid_t pid;

    if(daemon_max_proc>0)
    {
        while(daemon_cur_proc>=daemon_max_proc)
        {
            pause();
        }
    }

    { /* block SIGCHLD before enter critial section */
        sigset_t sset, oset;

        sigemptyset(&sset);
        sigaddset(&sset, SIGCHLD);
        if(sigprocmask(SIG_BLOCK, &sset, &oset)<0)
        {
            return -1;
        }
    }

    if((pid=vfork())<0)
    {}
    else if(pid>0)
    {
        daemon_cur_proc++;
    }
    else if(pid==0)
    {
        daemon_max_proc=0;
        daemon_cur_proc=0;
        { /* reset signal handler of SIGCHLD for child process */
            struct sigaction act, old_act;
            act.sa_handler=SIG_IGN;
            sigemptyset(&act.sa_mask);
            act.sa_flags=0;
            if(sigaction(SIGCHLD, &act, &old_act)<0)
            {
                LOGERROR("daemon_init(): sigaction(): %m");
                return -1;
            }
        }
    }

    { /* unblock SIGCHLD before leave critial section */
        sigset_t sset, oset;

        sigemptyset(&sset);
        sigaddset(&sset, SIGCHLD);
        if(sigprocmask(SIG_UNBLOCK, &sset, &oset)<0)
        {
            return -1;
        }
    }

    return pid;
}

int daemon_fini(void)
{
    while(daemon_cur_proc>0)
    {
        pause();
    }
    return 0;
}
