/*
 	DetectCRC
	Behavior:
		1. Every 5 seconds look up CRC errors
		2. If CRC errors > 5000 then
		3. flash all leds
		4. reboot
*/
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <fcntl.h>		/* open */
#include <sys/ioctl.h>		/* ioctl */
#include <linux/tc3162ioctl.h>
#include <linux/tc3162dmtctrl.h>
#include <ctype.h>
#include <dirent.h>
#include <stdarg.h>
/* For reset switch ioctl */
#include <sys/socket.h>
#include <linux/tc3162ioctl.h>
#include <linux/if.h>
/* reset switch ioctl end*/

#define DPRINTF printf

void
TCDMT_SetDmtClose (void)
{
  unsigned char Arg;
  int nFd = open ("/dev/dmtdev", O_RDWR);
  if (nFd == -1)
    {
      DPRINTF
	("getDownUpStreamRate:severe error for open /dev/dmtdev for ADSL device\n");
      return;
    }
  ioctl (nFd, DMTIOCTL_SET_DMTCLOSE, &Arg);
  close (nFd);
}				/* TCDMT_SetDmtClose */
static inline void
GUop (FILE * filep, char *si, char *fn)
{
  filep = fopen (fn, "w");
  fprintf (filep, "%s", si);
  fclose (filep);
  fflush (filep);
}

#define READ_BUF_SIZE	50


long *
find_pid_by_name (char *pidName)
{

  DIR *dir;
  struct dirent *next;
  long *pidList = NULL;
  int i = 0;

  dir = opendir ("/proc");
  if (!dir)
    {
      DPRINTF ("Cannot open /proc");
    }
  while ((next = readdir (dir)) != NULL)
    {
      FILE *status;
      char filename[READ_BUF_SIZE];
      char buffer[READ_BUF_SIZE];
      char name[READ_BUF_SIZE];

      /* Must skip ".." since that is outside /proc */
      if (strcmp (next->d_name, "..") == 0)
	continue;

      /* If it isn't a number, we don't want it */
      if (!isdigit (*next->d_name))
	continue;

      sprintf (filename, "/proc/%s/status", next->d_name);
      if (!(status = fopen (filename, "r")))
	{
	  continue;
	}
      if (fgets (buffer, READ_BUF_SIZE - 1, status) == NULL)
	{
	  fclose (status);
	  continue;
	}
      fclose (status);

      /* Buffer should contain a string like "Name:   binary_name" */
      sscanf (buffer, "%*s %s", name);
      if (strcmp (name, pidName) == 0)
	{
	  pidList = realloc (pidList, sizeof (long) * (i + 2));
	  pidList[i++] = strtol (next->d_name, NULL, 0);
	}
    }

  if (pidList)
    {
      pidList[i] = 0;
    }
  else
    {
      pidList = realloc (pidList, sizeof (long));
      pidList[0] = -1;
    }
  return pidList;
}


static int
KillAllProcess ()
{
  FILE *fp;
  long *killpid;
  char line[255];
  char str_process_name[255];

  fp = fopen ("/etc/preUpload", "r");
  if (fp == NULL)
    return -1;

  while (fgets (line, 255, fp))
    {
      sscanf (line, "%s", str_process_name);
      if (strncmp (line, "detectcrc", 9))
	{
	  killpid = find_pid_by_name (str_process_name);

	  if (!killpid || *killpid <= 0)
	    {
	      //DPRINTF(" %s : No Such Process\n",str_process_name);
	    }
	  else
	    {
	      kill (*killpid, SIGTERM);
	      DPRINTF (" %s : Process killed\n", str_process_name);
	    }
	}
    }
  fclose (fp);
  DPRINTF ("\n");
  return 1;
}

#define CRC_THESHOLD 5000
int
main (int argc, char *argv[])
{
  /* Add for software switch reset */
  struct IP175C_vlan_ioctl_args request_args;
  int fd;
  struct ifreq ifr;
  /* software switch reset end */
  FILE *proc_sys, *proc_ppp, *proc_adsl, *proc_reset, *dev_switch, *fp;
  int resetvalue = 1;
  int i = 0, j = 0;
  int pid = 0;
  int near_end_fast=0;
  int near_end_int=0;
  int old_near_end_fast=0;
  int old_near_end_int=0;
  
   int nFd = open( "/dev/dmtdev", O_RDWR );
  //  DPRINTF("AdslDiagnostic_Form is called\n");
    if( nFd == -1 )
    {
	    DPRINTF("DMT: open error \n");
	    return;
    }

  DPRINTF ("detectcrc daemon start!\n");

  while (1)
    {
	/* get CRC error values */
    DMTCTRL_LINE_INFO Arg;

    memset(&Arg,0,sizeof(DMTCTRL_LINE_INFO));
    ioctl( nFd, DMTIOCTL_GET_LINE_INFO, &Arg );
        
    old_near_end_fast = near_end_fast;
    old_near_end_int = near_end_int;
    near_end_fast = Arg.near_end.CrcNotInterleaved;
    near_end_int = Arg.near_end.CrcInterleaved;
    if (near_end_fast - old_near_end_fast > CRC_THESHOLD )
	   resetvalue  = 0;
    if (near_end_int - old_near_end_int > CRC_THESHOLD )
	   resetvalue  = 0;

    //syslog(LOG_INFO,"near-end CRC error fast: %d \n",Arg.near_end.CrcNotInterleaved);
    //syslog(LOG_INFO,"near-end CRC error interleaved: %d \n",Arg.near_end.CrcInterleaved);

    
	    
      if (resetvalue == 0)
	{
#if 1
	  DPRINTF ("detect abnormal CRC \n");
	  DPRINTF ("reset switch and flash all leds\n");
	  DPRINTF ("near end fast crc %d -> %d\n",old_near_end_fast,near_end_fast);
	  DPRINTF ("near end int crc %d -> %d\n",old_near_end_int,near_end_int);
	  //turn off lights
	  GUop (proc_sys, "0", "/proc/GPIOUSER/sysled");
	  GUop (proc_ppp, "0", "/proc/GPIOUSER/pppled");
	  /* 7300G adsl led can not be closed,robert,01192006 */
	  //GUop(proc_adsl,"0","/proc/GPIOUSER/adslled");
	  sleep (1);
	  //reset switch
	  //turn on lights
	  //GUop(dev_switch,"reset","/dev/switch");

	  /* reset switch */
	  if ((fd = socket (AF_INET, SOCK_STREAM, 0)) < 0)
	    {
	      DPRINTF ("open socket failed\n");
	      return 0;
	    }

	  memset (&ifr, 0, sizeof (struct ifreq));
	  strcpy (ifr.ifr_name, "eth0");

	  request_args.request_cmd = Switch_Reset;
	  ifr.ifr_data = (char *) &request_args;
	  if (ioctl (fd, SIOCSVLANPORT, &ifr) < 0)
	    {
	      DPRINTF ("try to reset switch failed\n");
	    }
	  /* reset switch end */

	  GUop (proc_sys, "1", "/proc/GPIOUSER/sysled");
	  GUop (proc_ppp, "1", "/proc/GPIOUSER/pppled");
	  /* 7300G adsl led can not be closed,robert,01192006 */
	  //GUop(proc_adsl,"1","/proc/GPIOUSER/adslled");

	  sleep (1);
	  //turn off lights
	  GUop (proc_sys, "0", "/proc/GPIOUSER/sysled");
	  GUop (proc_ppp, "0", "/proc/GPIOUSER/pppled");
	  /* 7300G adsl led can not be closed,robert,01192006 */
	  //GUop(proc_adsl,"0","/proc/GPIOUSER/adslled");


#endif
	/* Sleep for 10 seconds */
	sleep(10);
	  
	  DPRINTF ("Sleep 10 Seconds\n");
	  KillAllProcess ();
	  DPRINTF ("kill all\n");
	  sleep (3);
	  TCDMT_SetDmtClose ();
	  /*
	     fp = fopen("/proc/dmtctrl/dmtwrite", "w");
	     if(fp!=NULL){
	     //shutdown dmt
	     fprintf(fp, "5");
	     fclose(fp);
	     }
	     DPRINTF("shutdown dmt in /proc/dmtctrl/dmtwrite\n");
	   */
	  sleep (2);
	  //reboot
	  break;

	}
      sleep (5);
    }



  //reboot process
  kill (1, SIGTERM);
  exit (1);

}
