/* ###########################################################################
# (c) Billion Electric Co., Limited 2001
#
# Do with Daylight Saving Time
#
# ##########################################################################*/


/************************************************************************
 *                                                                      *
 *            Included Headers                                          *
 *            ================                                          *
 *                                                                      *
 ************************************************************************/

/* Standard includes */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

/* Timer */
//#include <timelib.h>

#include "dst.h"



/************************************************************************
 *                                                                      *
 *            Debugging and Tracing                                     *
 *            =====================                                     *
 *                                                                      *
 ************************************************************************/
//#define DEBUG_DST
#ifdef	DEBUG_DST
#define PRINTF_D(args...)       kprintf("DST: "args);
#else
#define PRINTF_D(args...)
#endif



/************************************************************************
 *                                                                      *
 *            Definition and Macro                                      *
 *            ====================                                      *
 *                                                                      *
 ************************************************************************/
#define ISLEAP( year )  (((year) % 4) == 0)

//----- Enumeration for struct tm -----//
enum TM_MONTH
{
    TM_M_JAN=0, TM_M_FEB,  TM_M_MAR,  TM_M_APR,  TM_M_MAY,  TM_M_JUN,
    TM_M_JUL,   TM_M_AUG,  TM_M_SEP,  TM_M_OCT,  TM_M_NOV,  TM_M_DEM
};
enum TM_WEEK
{
    TM_W_SUN=0,	TM_W_MON,  TM_W_TUE,  TM_W_WED,  TM_W_THU,  TM_W_FRI,  TM_W_SAT
};

#define DAYS_WEEK	7


typedef unsigned short U16;
typedef unsigned char BYTE;
typedef int BOOL
#define FALSE  0
#define TRUE 1
;

/************************************************************************
 *                                                                      *
 *            Data Structure                                            *
 *            ==============                                            *
 *                                                                      *
 ************************************************************************/
typedef struct
{
    U16  flag;		// See FG_DST_xxx
    BYTE tm_mon;	// [TM_M_JAN TM_M_DEM]: Month
    BYTE tm_mday;	// [1 31]: Day of Month
    BYTE tm_wday;	// [TM_W_SUN TM_W_SAT]: Day of Week
    BYTE order;		// [1 4]: Order	W-day	5: Last W-day	(W: SUN - SAT)
    U16	 tm_hour;	// [0 23]: Hour (in Standard Time)
} DST_MONTH;

#define FG_DST_M_WEEK	0
#define FG_DST_M_DAY	1

#define LASK_WEEK	5

typedef struct
{
    U16 		idxDST;			// DST Index, See DST_INDEX_xxx
    U16 		hemisphere;		// North/South Hemisphere
    DST_MONTH	monthBegin;		// Infomation of Begin Month
    DST_MONTH	monthEnd;		// Infomation of Enf Month
} DST_RANGE;

#define HSPHERE_NORTH	0		// North Hemisphere
#define HSPHERE_SOUTH	1		// South Hemisphere



/************************************************************************
 *                                                                      *
 *            Local Functions                                           *
 *            ===============                                           *
 *                                                                      *
 ************************************************************************/
DST_RANGE *get_dst_range(U16 indexDST);
BOOL dst_check_by_index(struct tm *ptmST, U16 indexDST);
BOOL dst_check_north_hemisphere(struct tm *ptmST, DST_RANGE *pRange); 
BOOL dst_check_south_hemisphere(struct tm *ptmST, DST_RANGE *pRange); 

int hour_CMP(struct tm *ptm1, struct tm *ptm2);

void fill_dst_month(struct tm *ptm, DST_MONTH *pDST_Month);
void fill_mday_by_wday(struct tm *ptm, int order);


/************************************************************************
 *                                                                      *
 *            Global Variables                                          *
 *            ================                                          *
 *                                                                      *
 ************************************************************************/
int gcDayMonth[12] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

#define	DEF_HOUR_BEGIN	2						// Default Begin Hour
#define	DEF_HOUR_END	(DEF_HOUR_BEGIN-1)		// Default End Hour

// Index of DST Region
#define	DST_INDEX_NA		1					// Northern America
#define	DST_INDEX_EU		2					// Europe
#define DST_INDEX_ISRAEL	3					// Israel
#define DST_INDEX_BRAZIL	4					// Brazil
#define	DST_INDEX_AU		5					// Australia: Most regions
#define	DST_INDEX_AU_T		6					// Australia: Tasmania
#define	DST_INDEX_NZ		7					// New Zealand

DST_RANGE gaDST_Range[] =
{   
	// Northern America
	{	DST_INDEX_NA,	HSPHERE_NORTH,
     	{ FG_DST_M_WEEK, TM_M_APR, 0, TM_W_SUN, 1,         2},
     	{ FG_DST_M_WEEK, TM_M_OCT, 0, TM_W_SUN, LASK_WEEK, 1}  },
	// Europe
    {	DST_INDEX_EU,	HSPHERE_NORTH,
     	{ FG_DST_M_WEEK, TM_M_MAR, 0, TM_W_SUN, LASK_WEEK, 1},
     	{ FG_DST_M_WEEK, TM_M_OCT, 0, TM_W_SUN, LASK_WEEK, 1}  },
	// Israel
    {	DST_INDEX_ISRAEL,	HSPHERE_NORTH,
     	{ FG_DST_M_WEEK, TM_M_APR, 0, TM_W_FRI, 1,         2},
     	{ FG_DST_M_WEEK, TM_M_SEP, 0, TM_W_FRI, 1,         1}  },
     	
	// Brazil
    {	DST_INDEX_BRAZIL,	HSPHERE_SOUTH,
     	{ FG_DST_M_WEEK, TM_M_OCT, 0, TM_W_SUN, 1,         DEF_HOUR_BEGIN},
     	{ FG_DST_M_WEEK, TM_M_FEB, 0, TM_W_SUN, LASK_WEEK, DEF_HOUR_END  }  },
	// Australia: Most regions
    {	DST_INDEX_AU, 	HSPHERE_SOUTH,
     	{ FG_DST_M_WEEK, TM_M_OCT, 0, TM_W_SUN, LASK_WEEK, DEF_HOUR_BEGIN},
     	{ FG_DST_M_WEEK, TM_M_MAR, 0, TM_W_SUN, LASK_WEEK, DEF_HOUR_END  }  },
	// Australia: Tasmania
    {	DST_INDEX_AU_T, HSPHERE_SOUTH,
     	{ FG_DST_M_WEEK, TM_M_OCT, 0, TM_W_SUN, 1,         DEF_HOUR_BEGIN},
     	{ FG_DST_M_WEEK, TM_M_MAR, 0, TM_W_SUN, LASK_WEEK, DEF_HOUR_END  }  },
	// New Zealand
    {	DST_INDEX_NZ, 	HSPHERE_SOUTH,
     	{ FG_DST_M_WEEK, TM_M_OCT, 0, TM_W_SUN, 1,         2},
     	{ FG_DST_M_WEEK, TM_M_MAR, 0, TM_W_SUN, 3, 		   2}  },
    
    // NULL
    {	0,              0,
     	{ 0,             0,        0, 0,        0,         0},
     	{ 0,             0,        0, 0,        0,         0}  },
};




/***********************************************************************
 *                                                                     *
 *            DST Check for each Time Zone                             *
 *            ============================                             *
 *                                                                     *
 ***********************************************************************/

//============================================================================
//  Name:   fnDST_XXX	(XXX: Time Zone)
//
//  Description:
//      Check whether it's in D.S.T. range
//
//  Parameter:
//   	(IN) ptmST:	Standard Time in (struct tm) format
//
//  Return:
//  	TRUE:	D.S.T.
//  	FALSE:	Standard Time
//============================================================================

//============================================================================
// fnDST_NA
//
//	Location: North America
//
//	Start: First Sunday in April	(at 2:00am Standard)
//	End:   Last  Sunday in October	(at 2:00am DST)
//
//	2001	April 1 	October 28 
//	2002 	April 7 	October 27 
//	2003 	April 6 	October 26 
//	2004 	April 4 	October 31
//============================================================================
int fnDST_NA(struct tm *ptmST)
{
    PRINTF_D("fnDST_NA(): DST Check for North America\n");
    
    return dst_check_by_index(ptmST, DST_INDEX_NA);
}


//============================================================================
// fnDST_EU
//
//	Location: Europe
//
//	Start: Last Sunday in March		(at 1:00am GMT)
//	End:   Last Sunday in October	(at 1:00am GMT)
//============================================================================
int fnDST_EU(struct tm *ptmST)
{
    PRINTF_D("fnDST_EU(): DST Check for Europe\n");
    
    return dst_check_by_index(ptmST, DST_INDEX_EU);
}


//============================================================================
// fnDST_Israel
//
//	Location: Israel
//
//	Start: First Friday in April 		(from 2:00am IST to 3:00am IDT)
//	End:   First Friday in September	(from 2:00am IDT to 1:00am IST)
//============================================================================
int fnDST_Israel(struct tm *ptmST)
{
    PRINTF_D("fnDST_EU(): DST Check for Europe\n");
    
    return dst_check_by_index(ptmST, DST_INDEX_ISRAEL);
}

/****************************************
 *                      				*
 *  	The Southern Hemisphere			*
 *                      				*
 ***************************************/
 
//============================================================================
// fnDST_Brazil
//
//	Location: Brazil
//
//	Start: First Sunday in October
//	End:   Last  Sunday in February
//============================================================================
int fnDST_Brazil(struct tm *ptmST)
{
    PRINTF_D("fnDST_Brazil(): DST Check for Brazil\n");

    return dst_check_by_index(ptmST, DST_INDEX_BRAZIL);
}

//============================================================================
// fnDST_AU
//
//	Location: South Australia, Victoria, Australian Capital Territory, 
//  		  New South Wales, Lord Howe Island
//
//	Start: Last Sunday in October
//	End:   Last Sunday in March
//
// fnDST_AU_T
//
//	Location: Tasmania
//
//	Start: First Sunday in October
//	End:   Last  Sunday in March
//============================================================================
int fnDST_AU(struct tm *ptmST)
{
    PRINTF_D("fnDST_AU(): DST Check for Australia\n");
    
    return dst_check_by_index(ptmST, DST_INDEX_AU);
}

int fnDST_AU_T(struct tm *ptmST)
{
    PRINTF_D("fnDST_AU_T(): DST Check for Tasmania Australia\n");
    
    return dst_check_by_index(ptmST, DST_INDEX_AU_T);
}

//============================================================================
// fnDST_NZ
//
//	Location: New Zealand 
//
//	Start: First Sunday in October	(at 2.00am Standard Time)
//	End:   Third Sunday in March	(at 2.00am Standard Time)
//============================================================================
int fnDST_NZ(struct tm *ptmST)
{
    PRINTF_D("fnDST_NZ(): DST Check for New Zealand\n");
    
    return dst_check_by_index(ptmST, DST_INDEX_NZ);
}



//============================================================================
//  Name:   dst_check_by_index
//
//  Description:
//      Check whether it's in D.S.T. range by Index
//
//  Parameter:
//  	(IN) ptmST:	Standard Time in (struct tm) format
//  	(IN) indexDST:	Index of D.S.T. Range
//
//  Return:
//  	TRUE:	D.S.T.
//  	FALSE:	Standard Time
//============================================================================
BOOL dst_check_by_index(struct tm *ptmST, U16 indexDST)
{
    DST_RANGE *pRangeDST;
    
    if( (pRangeDST = get_dst_range(indexDST)) == NULL )
    	return FALSE;
    
    if( pRangeDST->hemisphere == HSPHERE_NORTH )
    	return dst_check_north_hemisphere(ptmST, pRangeDST);
    else
    	return dst_check_south_hemisphere(ptmST, pRangeDST);
}



//============================================================================
//  Name:   get_dst_range
//
//  Description:
//      Check whether it's in D.S.T. range by Index
//
//  Parameter:
//  	(IN)  indexDST:	 Index of D.S.T. Range (Nonzero)
//
//  Return:
//  	DST Range
//============================================================================
DST_RANGE *get_dst_range(U16 indexDST)
{
	DST_RANGE *pRangeDST = gaDST_Range;
	U16	idx;
	
	while( (idx = pRangeDST->idxDST) != 0 )
	{
		if( idx == indexDST )
		{
    		return pRangeDST;
    	}
    	
    	pRangeDST++;
	}
    
    return NULL;
}



//============================================================================
//  Name:   dst_check_north_hemisphere
//  	    dst_check_south_hemisphere
//
//  Description:
//      Check whether it's in D.S.T. range for Northern/Southern Hemisphere
//
//  Parameter:
//  	(IN) ptmST:	Standard Time in (struct tm) format
//  	(IN) pRange:	Range of D.S.T
//
//  Return:
//  	TRUE:	D.S.T.
//  	FALSE:	Standard Time
//============================================================================

BOOL dst_check_north_hemisphere(struct tm *ptmST, DST_RANGE *pRange)
{
    struct tm tmBeg, tmEnd;
    
    tmBeg.tm_year = tmEnd.tm_year = ptmST->tm_year;
    tmBeg.tm_hour = pRange->monthBegin.tm_hour;
    tmEnd.tm_hour = pRange->monthEnd.tm_hour;
    
    PRINTF_D("dst_check_north_hemisphere(): Compare with Begin\n");
    
    fill_dst_month(&tmBeg, &pRange->monthBegin);
    if( hour_CMP(ptmST, &tmBeg) < 0 )
    {
    	return FALSE;
    }
    
    PRINTF_D("dst_check_north_hemisphere(): Compare with End\n");
    
    fill_dst_month(&tmEnd, &pRange->monthEnd);
    if( hour_CMP(ptmST, &tmEnd) < 0 )
    {
    	return TRUE;
    }
    
    return FALSE;
}

BOOL dst_check_south_hemisphere(struct tm *ptmST, DST_RANGE *pRange)
{
    struct tm tmBeg, tmEnd;
    
    tmBeg.tm_year = tmEnd.tm_year = ptmST->tm_year;
    tmBeg.tm_hour = pRange->monthBegin.tm_hour;
    tmEnd.tm_hour = pRange->monthEnd.tm_hour;
    
    PRINTF_D("dst_check_south_hemisphere(): Compare with End\n");
    
    fill_dst_month(&tmEnd, &pRange->monthEnd);
    if( hour_CMP(ptmST, &tmEnd) < 0 )
    {
    	return TRUE;
    }
    
    PRINTF_D("dst_check_south_hemisphere(): Compare with Begin\n");
    
    fill_dst_month(&tmBeg, &pRange->monthBegin);
    if( hour_CMP(ptmST, &tmBeg) < 0 )
    {
    	return FALSE;
    }
    
    return TRUE;
}



/***********************************************************************
 *                                                                     *
 *            Time Comparison                                          *
 *            ===============                                          *
 *                                                                     *
 ***********************************************************************/

//============================================================================
//  Name:   hour_CMP
//
//  Description:
//      Compare 2 times (Month, Day, Hour)
//
//  Parameter:
//  	(IN) ptm1:	1'st operand (in struct format)
//  	(IN) ptm2:	2'nd operand (in struct format)
//
//  Return:
//  	Zero:		=
//  	Positive:	>
//  	Negative:	<
//============================================================================
int hour_CMP(struct tm *ptm1, struct tm *ptm2)
{
    int Data1, Data2;
    
    Data1 = (((ptm1->tm_mon << 5) + ptm1->tm_mday) << 5) + ptm1->tm_hour;
    Data2 = (((ptm2->tm_mon << 5) + ptm2->tm_mday) << 5) + ptm2->tm_hour;
    
    return (Data1 - Data2);
}



/***********************************************************************
 *                                                                     *
 *            Time Data                                                *
 *            =========                                                *
 *                                                                     *
 ***********************************************************************/
 
//============================================================================
//  Name:   fill_dst_month
//
//  Description:
//      Fill D.S.T. Month
//
//  Parameter:
//  	(IN/OUT) ptm:	Time in (struct tm) format
//  	(IN)  pRange:	Range of D.S.T
//
//  Return:
//  	None
//============================================================================
void fill_dst_month(struct tm *ptm, DST_MONTH *pDST_Month)
{
    switch( pDST_Month->flag )
    {
    case FG_DST_M_WEEK:
	    ptm->tm_mon  = pDST_Month->tm_mon;
	    ptm->tm_wday = pDST_Month->tm_wday;
	    fill_mday_by_wday(ptm, pDST_Month->order);
	    break;
	    
	case FG_DST_M_DAY:
	    ptm->tm_mday = pDST_Month->tm_mday;
	    break;
    }
}



//============================================================================
//  Name:   fill_mday_by_wday
//
//  Description:
//      Fill ptm->tm_mday
//
//  Parameter:
//  	(IN/OUT) ptm:	Time in (struct tm) format
//  	(IN)     order:	[1 4]: Order
//  			 5: Last
//
//  Return:
//  	None
//============================================================================

#define WDAY_1999_DEC_31	TM_W_FRI	// Friday   -- DEC 31 1999
#define WDAY_2000_JAN_01	TM_W_SAT	// Saturday -- JAN 01 2000

void fill_mday_by_wday(struct tm *ptm, int order)
{
    int yc = ptm->tm_year + TIME_BEGIN_YEAR_TM - 2000;
    int cDay;		// Number of days since JAN 1 2000
    int mday_last;
    int diff;
    int wday_first;	// WDay of the 1'st day this month
    int mday;		// Result MDay
    int i;
    
    PRINTF_D("fill_mday_by_wday(): %d/%02d  %d'th WDay=%d'th\n", 
    	     yc+2000, ptm->tm_mon+1, order, ptm->tm_wday);
    	       
    //----- Day Accumulate by 4 Years -----//
    cDay = (yc >> 2) * (366+365+365+365);
    
    PRINTF_D("fill_mday_by_wday(): cDay(4Y) = %d\n", cDay);
    
    yc = yc & 0x3;
    
    //----- Day Accumulate by Year -----//
    cDay += 365 * yc;
    if( yc > 0 ) 	// Adjust for past Leap Year
        cDay++;

    PRINTF_D("fill_mday_by_wday(): cDay(Y) = %d\n", cDay);

    //----- Day Accumulate by Month -----//
    for(i=TM_M_JAN; i<ptm->tm_mon; i++)
        cDay += gcDayMonth[i];
    if( yc==0  &&  ptm->tm_mon>TM_M_FEB ) 	// Adjust for 
    	cDay++;					//	past FEB of Leap Year
    	
    PRINTF_D("fill_mday_by_wday(): cDay(M) = %d\n", cDay);
    
    //----- Compute WDay of the 1'st day this month -----//
    wday_first = (cDay + WDAY_2000_JAN_01) % DAYS_WEEK;

    PRINTF_D("fill_mday_by_wday(): WDay of 1'st day = %d'th\n", wday_first);
    
    //----- Compute MDay of the 1'st WDay this month -----//
    diff = ptm->tm_wday - wday_first;
    mday = 1 + diff;
    if( mday <= 0 )
    	mday += DAYS_WEEK;
    
    PRINTF_D("fill_mday_by_wday(): MDay of 1'st WDay = %d\n", mday);
    
    //----- Get MDay -----//
    mday += DAYS_WEEK * (order-1);

    PRINTF_D("fill_mday_by_wday(): MDay = %d\n", mday);
    
    //----- Adjust MDay -----//
    if( order == LASK_WEEK )
    {
//    	assert(LASK_WEEK == 5);
    	
    	//----- Get the last day of the month -----//	
    	mday_last = gcDayMonth[ptm->tm_mon];
    	if( ptm->tm_mon==TM_M_FEB  &&  yc==0 ) 	// Adjust if the month is
            mday_last++;			//	 FEB of Leap Year
        	
    	if( mday > mday_last )
    	    mday -= DAYS_WEEK;
    	    
    	PRINTF_D("fill_mday_by_wday(): MDay (Adjust) = %d\n", mday);
    }
    	
    ptm->tm_mday = mday;
}



