// small file system Albert Chuang
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <dirent.h>
#include "profile.h"

#define A_GET 0
#define A_PUT 1
#define A_LIST 2
#define A_GET_MAC 3
#define A_PUT_MAC 4
#define A_FORMAT_MAC 5

static struct flashMap flash;
static struct flashMapMac flashMac;
static int action=0;
static char default_mac[6] = { DEFAULT_MAC0, DEFAULT_MAC1,
	DEFAULT_MAC2, DEFAULT_MAC3,
	DEFAULT_MAC4, DEFAULT_MAC5 };


unsigned int calcOffset(unsigned int i)
{
	unsigned int offset,j;
	offset=j=0;
	for(j=0; j<i; j++)
	{
		offset += flash.blocks[j].size;
	}
	return offset;
}

unsigned int calcCsum(unsigned char *pbuf, unsigned int size)
{
	unsigned int sum, i;
	sum=0;
	for(i=0; i<size; i++)
		sum+=pbuf[i];
	return sum;
}

int setBlock(unsigned int i, struct Table_T *block, unsigned char *pbuf)
{
	memcpy(&flash.blocks[i], block, sizeof(struct Table_T));
	memcpy(&flash.buffer[flash.blocks[i].offset], pbuf, flash.blocks[i].size);
	if(pbuf)
		free(pbuf);
	return 0;
}

int getBlock()
{
	int i;
	for(i=0; i<BLOCK_TOTAL; i++)
	{
		if(flash.blocks[i].status==0xff)
			break;
	}
	return i;
}

int writeToFlash(FILE *infp, char *filename)
{
	unsigned int i, size, offset, csum;
	struct Table_T block;
	unsigned char *pbuf;

	if(strlen(filename) > (NAME_SIZE-1) ){
		fprintf(stderr, "%s filename too long\n", filename);
		return -1;
	}
		
	i = getBlock();
	if(i==-1) {
		fprintf(stderr, "not block\n");
		return -1;
	}
	
	offset = calcOffset(i);
	fseek(infp, 0L, SEEK_END);
	size = ftell(infp);
	pbuf = (unsigned char *) malloc(size);
	if(!pbuf && size) 
	{
		fprintf(stderr, "malloc error filename %s\n",filename);
		return -1;
	}
	rewind(infp);
	fread(pbuf, size, 1, infp);
	csum = calcCsum(pbuf, size);
	strncpy(block.name, filename, NAME_SIZE);
	block.status = 0x00;
	block.offset = offset;
	block.size = size;
	block.csum = csum;
	return setBlock(i, &block, pbuf);
}

int getFileInDir(char *dir, struct dirent ***namelist, int *n)
{
	int s=0;
	struct dirent **nlist;
	s=scandir(dir, &nlist, 0, alphasort);
	if(s<0)
	{
		fprintf(stderr, "scandir %s error\n", dir);
		return -1;
	}
	*n=s; *namelist=nlist;
	return 0;
}

int putFileToFlash(char *filename)
{
	FILE *infp;
	int i;

	infp=fopen(filename, "rb");
	if(!infp) {
	       	fprintf(stderr, "file not exist or can't access\n");
		return -1;
	}
	i=writeToFlash(infp, filename);
	fclose(infp);
	return i;
}

int findBlock(char *filename)
{
	int i=0;
	for(i=0; i<BLOCK_TOTAL; i++)
	{
		if(!strcmp(flash.blocks[i].name, filename))
			break;
	}
	return i;
}

int readFromFlash(FILE *outfp, char *filename)
{
	int i;
	i=findBlock(filename);
	if(i==-1){
		fprintf(stderr, "can't find in flash\n");
		return -1;
	}
	fwrite(&flash.buffer[flash.blocks[i].offset], flash.blocks[i].size, 1, outfp);
	return 0;
}

int getFileFromFlash(char *filename)
{
	FILE *outfp;
	int i;

	outfp=fopen(filename, "w+b");
	if(!outfp){
		fprintf(stderr, "file can't open\n");
		return -1;
	}
	i=readFromFlash(outfp, filename);
	fflush(outfp);
	fclose(outfp);
	return i;
}

int formatFlash()
{
	FILE* f1=0;
	char iibuf[]="\ndefault_mac0=\"00:04:ed:aa:bb:cc\"";
	if(action == A_PUT_MAC || action == A_FORMAT_MAC)
	{
		memset(&flashMac, 0xff, FLASH_MAC_SIZE);
		strncpy(flashMac.signature, "0123456abcde", SIGNATURE_SIZE);
		//restore 3 default macs
		memcpy(flashMac.macs.lan_mac,default_mac,MAC_LENGTH);
		memcpy(flashMac.macs.wan_mac,default_mac,MAC_LENGTH);
		memcpy(flashMac.macs.wireless_mac,default_mac,MAC_LENGTH);
		flashMac.macs.wan_mac[5]++;
		flashMac.macs.wireless_mac[5]+=2;
		f1 = fopen("/etc/scripts/sysdef","a+w");
		if(f1 == 0)
		{
			fprintf(stderr, "sysdef can't open!\n");
			return -1;
		}
		
		fseek(f1, 0L, SEEK_END);
		
		fwrite(iibuf,1,sizeof(iibuf),f1);
		iibuf[12]='1';
		fwrite(iibuf,1,sizeof(iibuf),f1);
		fprintf(stderr, "open! %s\n",iibuf);
		fclose(f1);
		
	}
	else
	{
		memset(&flash, 0xff, FLASH_CONF_SIZE);
		strncpy(flash.signature, "abcdefg01234", SIGNATURE_SIZE);
	}
	return -1;
}

int readFlash()
{
	FILE *fp;
	if(action == A_GET_MAC)
	{
		fp=fopen(FLASH_MAC_DEV,"r+b");
		if(!fp) {
			fprintf(stderr, "readflash open error\n");
			return -1;
		}
		fread(&flashMac,FLASH_MAC_SIZE, 1,fp); 
		if( strncmp(flashMac.signature, "0123456abcde", SIGNATURE_SIZE) )
		{
			formatFlash();
		}
		fclose(fp);
	}
	else if (action==A_GET)
	{
		fp=fopen(FLASH_CONF_DEV, "r+b");
		if(!fp) {
			fprintf(stderr, "readflash open error\n");
			return -1;
		}
		fread(&flash,FLASH_CONF_SIZE, 1,fp);
		if( strncmp(flash.signature, "abcdefg01234", SIGNATURE_SIZE) )
		{
			formatFlash();
		}
		fclose(fp);
	}

	return 0;
}

int writeFlash()
{
	FILE *fp;
	if (action == A_PUT_MAC || action == A_FORMAT_MAC)
	{
		fp=fopen(FLASH_MAC_DEV, "wb");
		if(!fp) {
			fprintf(stderr, "writeflash open error\n");
			return -1;
		}
		fwrite(&flashMac, FLASH_MAC_SIZE, 1,fp);
		fflush(fp);
		fclose(fp);
	}
	else if( A_PUT==action)
	{
		fp=fopen(FLASH_CONF_DEV, "wb");
		if(!fp) {
			fprintf(stderr, "writeflash open error\n");
			return -1;
		}
		fwrite(&flash, FLASH_CONF_SIZE, 1,fp);
		fflush(fp);
		fclose(fp);
	}
	return 0;
}

int main(int argc, char *argv[])
{
	int i=0;
	int n;
	struct dirent **namelist;
	FILE* fp;
	char iibuf[]="\ndefault_mac0=\"00:04:ed:aa:bb:cc\"";
#if 0
	int __S_Table[] = {
	48,49,50,51,52,53,54,55,56,57,	/* 48, .., 57 */
	97,98,99,100,101,102
	};
#endif
	if(argc < 3)
	{
		fprintf(stderr,"%s <get|put> directory\n", argv[0]);
		return -1;
	}

	if(strcmp(argv[1], "get")==0) {
		fprintf(stderr,"get %s directory\n", argv[2]);
		if(strcmp(argv[2],"/mac")==0)
			action=A_GET_MAC;
		else
			action=A_GET;
	}else if(strcmp(argv[1], "put")==0){
		fprintf(stderr,"put %s directory\n", argv[2]);
		if(strcmp(argv[2],"/mac")==0)
			action=A_PUT_MAC;
		else
			action=A_PUT;
	}else if(strcmp(argv[1], "format") == 0){
		if(strcmp(argv[2],"/mac")==0)
			action=A_FORMAT_MAC;
			
	}else if(strcmp(argv[1], "list")==0){
		fprintf(stderr,"list %s directory\n", argv[2]);
		action=A_LIST;
	}else{
		fprintf(stderr,"%s <get|put> directory\n", argv[0]);
		return -1;
	}

	i=readFlash();
	if(i==-1) {
		fprintf(stderr, "can't read flash\n");
		return -1;
	}
	
	switch(action){
		case A_FORMAT_MAC:
			formatFlash();
			break;
		case A_GET:
			//i=getFileFromFlash(argv[2]);
			i=chdir(argv[2]);
			if(!i){
			  for(n=0; n<BLOCK_TOTAL; n++){
				if(flash.blocks[n].status==0x00){
					getFileFromFlash(flash.blocks[n].name);
				}
			  }
			}else{
				fprintf(stderr," %s isn't a directory\n", argv[2]);
			}
			break;
		case A_GET_MAC:
			//write macs to files
			fp = fopen("/etc/scripts/sysdef","a+w");
			if ( fp == 0)
			{
				fprintf(stderr,"%s write failed!\n","/etc/scripts/sysdef");
				break;
			}
		fseek(fp, 0L, SEEK_END);

		sprintf(iibuf,"\ndefault_mac0=\"%02x:%02x:%02x:%02x:%02x:%02x\"\n",
			(flashMac.macs.lan_mac[0] & 0xff),(flashMac.macs.lan_mac[1] & 0xff),
			(flashMac.macs.lan_mac[2] & 0xff),(flashMac.macs.lan_mac[3] & 0xff),
			(flashMac.macs.lan_mac[4] & 0xff),(flashMac.macs.lan_mac[5] & 0xff));
#if 0
		iibuf[15] = __S_Table[(flashMac.macs.lan_mac[0] >> 4)];
		iibuf[16] = __S_Table[(flashMac.macs.lan_mac[0] & 0x0f)];
		iibuf[18] = __S_Table[(flashMac.macs.lan_mac[1] >> 4)];
		iibuf[19] = __S_Table[(flashMac.macs.lan_mac[1] & 0x0f)];
		iibuf[21] = __S_Table[(flashMac.macs.lan_mac[2] >> 4)];
		iibuf[22] = __S_Table[(flashMac.macs.lan_mac[2] & 0x0f)];
		iibuf[24] = __S_Table[(flashMac.macs.lan_mac[3] >> 4)];
		iibuf[25] = __S_Table[(flashMac.macs.lan_mac[3] & 0x0f)];
		iibuf[27] = __S_Table[(flashMac.macs.lan_mac[4] >> 4)];
		iibuf[28] = __S_Table[(flashMac.macs.lan_mac[4] & 0x0f)];
		iibuf[30] = __S_Table[(flashMac.macs.lan_mac[5] >> 4)];
		iibuf[31] = __S_Table[(flashMac.macs.lan_mac[5] & 0x0f)];
#endif
		fwrite(iibuf,1,sizeof(iibuf),fp);
		iibuf[12]='1';
		fwrite(iibuf,1,sizeof(iibuf),fp);
		fclose(fp);

			//write to lan mac
			fp = fopen(DEFAULT_MAC1_FN,"wb");
			if ( fp == 0)
			{
				fprintf(stderr,"%s write failed!\n",DEFAULT_MAC1_FN);
				break;
			}
			fwrite(flashMac.macs.lan_mac,1,MAC_LENGTH,fp);
			fclose(fp);
			//write to wan mac
			fp = fopen(DEFAULT_MAC2_FN,"wb");
			if ( fp == 0)
			{
				fprintf(stderr,"%s write failed!\n",DEFAULT_MAC2_FN);
				break;
			}
			fwrite(flashMac.macs.wan_mac,1,MAC_LENGTH,fp);
			fclose(fp);
			//write to wireless mac
			fp = fopen(DEFAULT_MAC3_FN,"wb");
			if ( fp == 0)
			{
				fprintf(stderr,"%s write failed!\n",DEFAULT_MAC3_FN);
				break;
			}
			fwrite(flashMac.macs.wireless_mac,1,MAC_LENGTH,fp);
			fclose(fp);
			break;
		case A_PUT:
			//i=putFileToFlash(argv[2]);
			formatFlash();
			//i=chdir(argv[2]);
			i=getFileInDir(argv[2], &namelist, &n);
			if(!i) {
			  while(n--) {
				if(strcmp(namelist[n]->d_name, ".")!=0 &&
			 	   strcmp(namelist[n]->d_name, "..")!=0) {
				  chdir(argv[2]);
					putFileToFlash(namelist[n]->d_name);
					chdir("../");
				}
				free(namelist[n]);
			  }
			  free(namelist);
			} else {
				fprintf(stderr," %s isn't a directory\n", argv[2]);
			}
			break;
		case A_PUT_MAC:
			//get macs from files
			// get lan_mac
			fp = fopen(DEFAULT_MAC1_FN,"rb");
			if ( fp == 0)
			{
				fprintf(stderr,"%s read failed!\n",DEFAULT_MAC1_FN);
				break;
			}
			fread(flashMac.macs.lan_mac,1,MAC_LENGTH,fp);
			fclose(fp);
			// get wan_mac
			fp = fopen(DEFAULT_MAC2_FN,"rb");
			if ( fp == 0)
			{
				fprintf(stderr,"%s read failed!\n",DEFAULT_MAC2_FN);
				break;
			}
			fread(flashMac.macs.wan_mac,1,MAC_LENGTH,fp);
			fclose(fp);
			// get wireless_mac
			fp = fopen(DEFAULT_MAC3_FN,"rb");
			if ( fp == 0)
			{
				fprintf(stderr,"%s read failed!\n",DEFAULT_MAC3_FN);
				break;
			}
			fread(flashMac.macs.wireless_mac,1,MAC_LENGTH,fp);
			fclose(fp);
			
			break;
		case A_LIST:
			getFileInDir("/flash", &namelist, &n);
			while(n--) {
				printf("%s %d\n", namelist[n]->d_name, strlen(namelist[n]->d_name));
				free(namelist[n]);
			}
			free(namelist);
			break;
		default:
			fprintf(stderr, "no this command\n");
			break;

	}
	writeFlash();
	return i;
}
