/******************************************************************************
Copyright LSI Logic Ltd 2001
  
sar_test.c - User program to exercise the SAR driver
******************************************************************************/


#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/socket.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <net/if.h>
#include <ctype.h>
#include <stdlib.h>
#if __GLIBC__ >= 2 && __GLIBC_MINOR >= 1
#include <netpacket/packet.h>
#include <net/ethernet.h>     /* the L2 protocols */
#else
#include <asm/types.h>
#include <linux/if_packet.h>
#include <linux/if_ether.h>   /* The L2 protocols */
#endif
#include <linux/ip.h>
#include <fcntl.h>

#include <linux/sar.h>

#undef PDEBUG             /* undef it, just in case */
#ifdef _DEBUG
#  ifdef __KERNEL__
     /* This one if debugging is on, and kernel space */
#define PDEBUG(fmt, args...) printk( KERN_EMERG fmt, ## args)
#  else
     /* This one for user space */
#    define PDEBUG(fmt, args...) fprintf(stderr, fmt, ## args)
#  endif
#else
#  define PDEBUG(fmt, args...) /* not debugging: nothing */
#endif

char buffer[1000];

void fill_req(struct ifreq *req, char *name, void *data)

{	strcpy(req->ifr_name, name);
	req->ifr_data = data;
}

void fill_buffer(char *p, int l)

{	int i=0;
	while(i<l)
		*p++=(i++)&0xff;
}
/**********************************************************************
   Dump a buffer to display 

*/
void hexdump(unsigned char *data, int len)
{
  char *p;
  int i, j, k;
  char line[255];

  printf("\nSartest Hexdump 0x%08x %d bytes\n", (unsigned)data, len);

  j = 0;
  while (j < len)
    {
      k = len - j > 8 ? 8 : len - j;
      p = line;
      p += sprintf(p, "    ");
      for (i = 0; i < k; i++)
	{
	  p += sprintf(p, "0x%02x ", *(data + j + i));
	}
      if (k < 8)
	{
	  for (i = 0; i < 8 - k; i++)
	    {
	      p += sprintf(p, "      ");
	    }
	}
      p += sprintf(p, "    ");
      for (i = 0; i < k; i++)
	{
	  p += sprintf(p, "%c", (isalnum((int)*(data + j + i)) == 0) ? '.' : *(data + j + i));
	}
      printf("%s\n", line);
      j += k;
    }
  if (len % 8)
    printf("\n");
}

/**********************************************************************
   Initialise raw socket and bind to interface 

   Returns socket handle 
   fflag : 1 for non block
*/

int raw_init(char *dev_name, int fflag)
{
	struct ifreq ifr;
	int fd = -1;
	struct sockaddr_ll txaddr;

	fd = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL));
	if (fd < 0)
	{
		perror("main: socket");
		return -1;
	}

	/* Get the address */
	memset((void *)&ifr, '\0', sizeof(struct ifreq));
	strncpy(ifr.ifr_name, dev_name, strlen(dev_name));
	if (ioctl(fd, SIOCGIFINDEX, &ifr) < 0)
	{
		perror("init: ioctl");
		return -1;
	}

	memset(&txaddr, 0, sizeof(struct sockaddr_ll));
	txaddr.sll_family = AF_PACKET;
	txaddr.sll_protocol = htons(ETH_P_ALL);
	txaddr.sll_ifindex = ifr.ifr_ifindex;
	printf("raw_init:ifr_index=%x\n",ifr.ifr_ifindex);
	txaddr.sll_halen = 6;
	memcpy(&txaddr.sll_addr, &ifr.ifr_addr, 6);
	if (bind(fd, (struct sockaddr *)&txaddr, sizeof(struct sockaddr_ll)) < 0)
	{	perror("init: bind");
		return -1;
	}

	if (fflag)
	{
		if (fcntl(fd, F_SETFL, fflag) < 0)
		{	perror("init: fnctl");
			return -1;
		}
	}
	return fd;
}

/***********************************************************************/
#define BLOCKING			0


int main(int argc, char **argv)

{	int skt;
	struct ifreq	req;
	struct sar_connection con_data={0x0,35,0,UBR_QUEUE,SAR_AAL5,0};
	struct sar_traffic_params tx_q_config={0, 0, 0, 0, 0x5A, 0, UBR };
	
	if ((argc>1) && (strcmp(argv[1],"recv")!=0) && (strcmp(argv[1],"recv")!=0))
	{	PDEBUG("Opening socket..");
		if ((skt = socket(PF_SAR,SOCK_RAW,0))!=-1)
			PDEBUG("OK\n");
		else
		{	printf("Open Socket failed\n");
			_exit(-1);
		}
	}
	if ((argc>1) && (strcmp(argv[1],"open")==0))
	{	int arg=2;
		char *name = "sar1";
		int vpi = 0;
		int vci = 35;
		while (arg<argc)
		{	if (argv[arg][0]!='-')
				name = argv[arg];
			else
			{	switch (argv[arg][1])
				{
				case 'p':	arg++;
							if (arg<argc)
								vpi=atoi(argv[arg]);
							if (vpi>255)
								vpi=255;
							break;
				case 'c':	arg++;
							if (arg<argc)
								vci=atoi(argv[arg]);
							if (vci>0xffff)
								vci=0xffff;
							break;
				default:	break;
				}
			}
			arg++;
		}
		con_data.vci=vci;
		con_data.vpi=vpi;
		fill_req(&req, name, &con_data);
		printf("Perfoming ioctl to create connection %s vci=%d vpi=%d",name,vci,vpi);
		if (ioctl(skt,SIOC_SAR_CREATE,&req)!=-1)
			PDEBUG("Ok\n");
		else
			printf("IOCTL Failed, error = %d\n",errno);
	}
	if ((argc>1) && (strcmp(argv[1],"close")==0))
	{	char *name;
		if (argc>2)
			name = argv[2];
		else
			name = "sar1";
		fill_req(&req, name,NULL);
		PDEBUG("Perfoming ioctl to destroy connection ..");
		if (ioctl(skt,SIOC_SAR_DESTROY,&req)!=-1)
			PDEBUG("Ok\n");
		else
			printf("IOCTL Failed, error = %d\n",errno);
	}
	if ((argc>1) && (strcmp(argv[1],"send")==0))
	{	
		int fd;
		int arg=2;
		char *name = "sar1";
		int tx_size = 100;
		int do_read = 0;
		while (arg<argc)
		{	if (argv[arg][0]!='-')
				name = argv[2];
			else
			{	switch (argv[arg][1])
				{
				case 's':	arg++;
							if (arg<argc)
								tx_size=atoi(argv[arg]);
							if (tx_size==0)
								tx_size = 100;
							else
								if (tx_size>sizeof(buffer))
									tx_size = sizeof(buffer);
							
							break;
				case 'r':	do_read = 1;
							break;
				default:	break;
				}
			}
			arg++;
		}
		printf("Send to %s..",name);
		if ((fd = raw_init(name,BLOCKING)) > 0)
		{
			fill_buffer(buffer, tx_size);
			if (send(fd,buffer,tx_size,0)!=-1)
			{	int len;
				printf("Ok\n");
				if (do_read)
				{	printf("Receive from %s, fd=%8.8x..",name,fd);
					if ((len = recv(fd,buffer,sizeof(buffer),0))!=-1)
					{	
						printf("Ok\n");
						hexdump(buffer,len);
					}
					else
						printf("Failed, error = %d :(%s)\n",errno, strerror(errno));
				}
			}
			else
				printf("Failed, error = %d\n",errno);
			close(fd);
		}
	}

	if ((argc>1) && (strcmp(argv[1],"recv")==0))
	{	int fd;
		char *name;
		if (argc>2)
			name = argv[2];
		else
			name = "sar1";

		printf("Receive from %s, fd=%8.8x..",name,fd);
		if ((fd = raw_init(name,BLOCKING)) > 0)
		{	int len;
			if ((len = recv(fd,buffer,sizeof(buffer),0))!=-1)
			{	
				printf("Ok\n");
				hexdump(buffer,len);
			}
			else
				printf("Failed, error = %d\n",errno);
			close(fd);
		}
	}
	if ((argc>2) && (strcmp(argv[1],"loopback")==0))
	{	struct ifreq *req;
		unsigned long res;
		if (argc>3)
		{	int data=*argv[3]=='0';
			fill_req(req, argv[2], (void *)(&data));
			res=ioctl(skt,SIOC_SAR_LOOPBACK,req);
		}
		else
			printf("%s: bad args",argv[0]); 
	}
	if ((argc>2) && (strcmp(argv[1],"dump")==0))
	{	int offset;
		unsigned long res;
		sscanf(argv[2],"%x", &offset);
		res=ioctl(skt,SIOC_SAR_DUMP_REG,offset);
		printf("[%x]=0x%8.8lx\n",offset,res);
	}
	if ((argc>2) && (strcmp(argv[1],"traffic")==0))
	{	int state;
		unsigned long res;
		sscanf(argv[2],"%x", &state);
		res=ioctl(skt,SIOC_SAR_ENABLE_TRAFFIC,state);
	}
	if ((argc>=2) && (strcmp(argv[1],"power")==0))
	{	unsigned long res;
		res=ioctl(skt,SIOC_SAR_POWER_DOWN,0);
	}
	if ((argc>3) && (strcmp(argv[1],"write")==0))
	{	struct sar_wrt_reg_params params;
		sscanf(argv[2],"%x", &params.offset);
		sscanf(argv[3],"%lx", &params.data);
		ioctl(skt,SIOC_SAR_WRITE_REG,&params);
	}
	if ((argc>1) && (strcmp(argv[1],"configtx")==0))
	{	PDEBUG("Perfoming ioctl to config Tx Q ..");
		if (ioctl(skt,SIOC_SAR_TX_CONFIG,&tx_q_config)!=-1)
			PDEBUG("Ok\n");
		else
			printf("IOCTL Failed, error = %d\n",errno);
	}
	if ((argc>1) && (strcmp(argv[1],"adsl")==0))
	{	struct sar_adsl_state_params params;
		if (argc>2)
		{	if (strcmp(argv[2],"up")==0)
			{	params.state = SAR_ADSL_STATE_UP;
				params.tx_rate_phy0 = 7600000;
				params.tx_rate_phy1 = 0;
			}
			else
				params.state = SAR_ADSL_STATE_DOWN;
		}
		else
				params.state = SAR_ADSL_STATE_DOWN;

		ioctl(skt,SIOC_SAR_ADSL_STATE,&params);
	}
	if ((argc>1) && (strcmp(argv[1],"recv")!=0) && (strcmp(argv[1],"recv")!=0))
	{	PDEBUG("Closing socket..");
		if (close(skt)!=-1)
			PDEBUG("Ok\n");
		else 
			printf("Close Socket failed\n");
	}
	_exit(0);
}
