/******************************************************************************
 * Copyright(c) 2002 Us.
 *
 * $ID
 *
 * Description:
 * A wrapper program to attach the standard in/out of an application to a
 * named pipe. The other end of the named pipe is serviced by the scam program.
 *****************************************************************************/

/* Include files */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "scamcommon.h"

/******************************************************************************
 * print_usage
 *
 * @param thisProgram Pointer to the program name
 *
 * @return void
 *
 *****************************************************************************/
void print_usage(char *thisProgram)
{
  printf("usage: %s -h -p userProgram\n", thisProgram);
  printf("\t-h - help and exit\n");
  printf("\t-p userProgram - full path name of program to run\n\n");
}

/******************************************************************************
 * mybasename
 *
 * @param filename Pointer to full path name of a file
 *
 * @return Pointer to file name
 *
 *****************************************************************************/
static char *mybasename(char *filename)
{
  char *p;

  p = strrchr(filename, '/');
  if (p)
  {
    p++;
    return p;
  }
  return filename;
}

/******************************************************************************
 * main
 *
 * @param argc Parameter count
 * @param argv Array of char pointers to parameters
 *
 * @return 
 *
 *****************************************************************************/
int main(int argc, char *argv[])
{
  int rx_fd;
  int tx_fd;
  int c, i, j;
  char *userProgram = 0;
  char *thisProgram;
  char rx_pipe[80];
  char tx_pipe[80];
  char **pargv = 0;

  thisProgram = strdup(argv[0]);

  while ((c = getopt(argc, argv, "hp:")) != -1)
  {
    switch (c)
    {
      case 'h':
        return 0;
      case 'p':
        userProgram = strdup(optarg);
        break;
    }
  }

  if (!userProgram)
  {
    print_usage(thisProgram);
    return -1;
  }

  if (optind > 0)
  {
    /* Create the argument list for the exec. */
    pargv = (char **)malloc(sizeof(char *) * (argc - optind + 2));
    if (!pargv)
    {
      perror("malloc");
      return -1;
    }
    pargv[0] = userProgram;
    for (i = optind, j = 1; i < argc; i++, j++)
    {
      pargv[j] = argv[i];
    }
    pargv[j] = 0;
  }

  sprintf(rx_pipe, "%s/%s_rx", FIFO_DIR, mybasename(userProgram));

  sprintf(tx_pipe, "%s/%s_tx", FIFO_DIR, mybasename(userProgram));

  rx_fd = open(rx_pipe, O_RDWR);
  if (rx_fd == -1)
  {
    perror(rx_pipe);
    /* return -1;*/
    printf("Opening in read-only mode\n");
  }
  
  tx_fd = open(tx_pipe, O_RDWR);
  if (tx_fd == -1)
  {
    perror(tx_pipe);
    return -1;
  }
  
  if (rx_fd != -1)
  {
    close(0);
    dup(rx_fd);
  }
  close(1);
  dup(tx_fd);
  close(2);
  dup(tx_fd);
  close(tx_fd);
  close(rx_fd);

  /* Fix this to pass the arguments through */
  execv(userProgram, pargv);

  return 0;
}
