/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file   testaushm.c
 *  @brief   
 */

/*
 *  Change Log
 *  2004-03-04  Jeremy  initial version
 */
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <errno.h>
#include <unistd.h>
#include "tmauIniParser.h"
#include "tmGetAUInfo.h"

#define  TM_FILE_MAX_LEN    256


void showUsage(void)
{
    printf("./testaushm -reset \n");
    printf("./testaushm -dump  \n");

}


int main(int argc, char **argv)
{
    TmuAUVersion_t  TmuVerShm;              /* structure to hold the AU information, from shared memory */
    char            szEngine[TM_AU_MAX_VER];/* buffer to hold the readable engine information */

    
    if (argc != 2)
    {
        showUsage();
        return 0;
    }

    memset(&TmuVerShm, 0x00, sizeof(TmuAUVersion_t));

    if (strcmp(argv[1], "-reset") == 0)
    {
        tmau_ResetAUInfo();
    }
    else if (strcmp(argv[1], "-dump") == 0)
    {
      /**
        *  The fields declared in shared memory for TmuAUVersion_t:
        * -product= supported product ID, according to AU module's assignment
        * -engine = a TmuVerion_t data type to record VSAPI engine version number
        *           for the supported product.  
        *           For example, 6.910 will store as 0x00069100
        * -pattern= a TmuVersion_t data type to record VSAPI pattern version 
        *           number for the supported product.  
        *           E.g., 1.782.00 will store as 0x0002B818(hex), i.e., 178200(dec)
        */

      /**
        * Start to read the product AU information from the shared memory.
        * The usage sample here can be leveraged by CGI program or discovery routine.
        */
       printf("Start to read the AU information from shared memory\n");

       memset(szEngine, 0x00, TM_AU_MAX_VER);

       /* call the query AU information routine in libtmGetAUInfo.a */
       tmau_GetNewestAUInfo(&TmuVerShm);

       tmau_MakeVersionString(TmuVerShm.engine.version, szEngine, TM_AU_MAX_VER);

        printf("The result from shared memory are:\n");
        printf("          product ID is: [%d]\n",   TmuVerShm.product);
        printf(" VASPI Engine number is: [%08x] or [%s]\n", TmuVerShm.engine.version, szEngine);
        printf("VASPI Pattern number is: [%d]\n", TmuVerShm.pattern.version);
    }

    return 0;
}

