/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file tmAVEnforcer.c
 *  @brief TMSS AVEnforcer Module
 */

/*
 *  Change Log
 *  2004-02-18 Eric Li    
 *  Initialization version
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <assert.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include "tmClientInfo.h"
#include "tmdpCommonDef.h"
#include "tmnfRequestInterface.h"
#include "tmAVEnforcer.h"
#include "tmException.h"
#include "tmcfg.h"
#include "tmShm.h"
#include "tmSemaphore.h"
#include "log.h"
#include <sys/socket.h>
#include <arpa/inet.h>
#include <linux/netfilter_ipv4/ipt_webstr.h>

#include <unistd.h>
#include <stdlib.h>


static const char ModuleName[] = "tmAVEnforcer";
unsigned int AVEConfigVersion=0;

static unsigned int gEnable = 1;           /* enable avenforcer? */
static unsigned int gTimeReport = 86400;     /* interval to show report */
static unsigned int gTimeOutdate = 1800;   /* Timeout for any record */
static unsigned int gTimeUpdate = 300;     /* Timeout for triggering update */
static char gAvSvrURL[MAX_URL_LENGTH] = "http://www.hackercheck.com/";
//static char gAvSvrURL[MAX_URL_LENGTH] = "http://211.76.136.195/dashboard/dashboard.aspx";

/**
  * @brief return the Health-Check URL
  * @return URL of Health-Check
  */
const char *tmave_GetAVServerURL(void)
{
    return gAvSvrURL;
}


/**
  * @brief set client's AV status to PENDING
  * @param ip IP address of the client
  * @param mac MAC address of the client
  */
void SetPendingAV(uint32_t ip, unsigned char *mac)
{
    struct client_id id;
    struct client_av av;
    struct client_time my_time;
  
    //LOGINFO("SetPendingAV for IP=%lx\n",ip);
    memset(&id, 0, sizeof(id));
    memset(&av, 0, sizeof(av));
    memset(&my_time, 0, sizeof(my_time));

    id.ip = ip;
    memcpy(id.mac, mac, sizeof(id.mac));
    av.status = STATUS_PENDING;

    tmci_SetAV(&id, &av);
    
    my_time.create_time = time(NULL);
    tmci_SetTime(&id, &my_time);
}

/**
  * @brief decide whether to trigger Discovery to detect the client
  * @param ip IP address of the client
  * @return 1 for triggered, 0 means not triggered
  */
int tmave_HookTrigger(uint32_t ip)
{
    ClientInfo         info;
    unsigned char      mac[MAC_SIZE];
    unsigned long      now;

    if (!gEnable)
        return 0; 

    memset(mac, 0, sizeof(mac));
    if (IP_MAC(&ip, mac) != 0)
    {
        LOGERROR("tmave_HookTrigger(), ARP table lookup fail for client %u.%u.%u.%u", NIPQUAD(ip));
        goto END;
    }

    if ((tmci_GetClient(mac, &info) < 0)||(CSTATUS(&info)== STATUS_READ_FROM_CONFIG) )
    {
        LOGINFO("tmave_HookTrigger(), New machine, %u.%u.%u.%u, try to have HTTP request", NIPQUAD(ip));
        /* Case 1: not client record */
        SetPendingAV(ip, mac);
        goto TODO;
    }
	
	// NOTE SINCE WE have do the tmave_HookTriggerKernel in Kernel, so the only one can call this function,
	// shall be called from the kenel sendMessageUp(nClientIP,NULL,KERNEL_TO_THPD_CMD_UPDATE_CLIENT_INFO,NULL) 
	// The thpd task will receive the request and call this functions.
	// WE still have to check if it is reasonable.
	// For the 30 Min. CLient inform update DQM, shall be process here.
	// If reach here, the client record SHALL BE out of date, shall we have issue the DQM by tmnf_UpdateClientInfo

/* 
    now = time(NULL);

    // Case 2: client record is out of date 
    if (now - CTAV(&info) > gTimeOutdate)
    {
        LOGINFO("tmave_HookTrigger(), Time to refresh %u.%u.%u.%u again", NIPQUAD(ip));
        goto TODO;
    }

    // Case 3: client pattern or engine is old 
    if (CSTATUS(&info) == STATUS_AVOLD && now - CTUPDATE(&info) > gTimeUpdate)
    {
        LOGINFO("tmave_HookTrigger(), Notify %u.%u.%u.%u to update pattern\n", NIPQUAD(ip));
        tmnf_UpdateClientPattern(ip);
        CTUPDATE(&info) = now;
        tmci_SetTime(&(info.idinfo), &(info.timeinfo));
        goto END;
    } 
       
    goto END;*/

TODO:
    LOGINFO("tmave_HookTrigger(), Trigger tmnf_UpdateClientInfo(%u.%u.%u.%u)\n", NIPQUAD(ip));

    if (tmnf_UpdateClientInfo(ip) < 0)
        LOGERROR("tmave_HookTrigger(), Fail to tmnf_UpdateClientInfo()");

    return 1;

END:
    ; /* do nothing */
    return 0;
}

#define VAR_SYS_STATUS_FILE "/var/etc/status"
char* read_file_para_value(char *filename, char *arg1 ,char *buf)
{
	FILE *fp;
	char tmp_line[255];
	char str_return[50];
	//char_t *str_return=NULL;
	char str_temp[50];
	memset(tmp_line,0,sizeof(tmp_line));
	*buf='\0';
	if ((fp=fopen(filename,"r"))!=NULL)
        {
        	while(fgets(tmp_line,sizeof(tmp_line),fp)){
		        if (strncmp(tmp_line,arg1,strlen(arg1)) == 0){
       				sscanf(tmp_line,"%50[^=]=\"%50[^\"]\"\n",str_temp,str_return);
	        		fclose(fp);
				strcpy(buf,str_return);
			//	DPRINTF("1.%s:%s\n",str_temp,buf);
				return buf;	
			}
		}
	       	fclose(fp);
	}
	return buf;
}
int sendAVEConfToKernel(void)
{
    int nRetVal = 0;
    AVEConf aAVEConf;
    size_t len;
    int sockfd = -1; 
    char buf[32];
	char str_lanif[10];
    aAVEConf.gEnable=gEnable;
    aAVEConf.gTimeReport=gTimeReport;
    aAVEConf.gTimeOutdate=gTimeOutdate;
    aAVEConf.gTimeUpdate=gTimeUpdate;
    strcpy(aAVEConf.gAvSvrURL,gAvSvrURL);

	strcpy(str_lanif,(char *)read_file_para_value(VAR_SYS_STATUS_FILE,"LAN_IF=",buf));
	aAVEConf.lan_if=LAN_IF_ETH0;// default t
	if(strcmp(str_lanif,"br0")==0)
	    aAVEConf.lan_if=LAN_IF_BR0;
	
	// We need to send the AVE configuration to tmss_kernel(ipt_webstr)
	sockfd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW);
	if (sockfd < 0)
	{
	    fprintf(stderr,"Can not create sock:%s\a\n",strerror(errno)); 
		return -1;
	}

	len=sizeof(AVEConf);
	if (setsockopt(sockfd, IPPROTO_IP, TMSS_SO_SET_AVE_CONFIG, &aAVEConf,len ) < 0)
	{
	    fprintf(stderr,"Can not setsockopt sock:%s\a\n",strerror(errno)); 
	close(sockfd);
	sockfd = -1;
		return -1;
	}
	close(sockfd);
	sockfd = -1;
    return nRetVal;
}
/**
  * @brief load settings from a config file
  * @param file configuration file path
  */
void tmave_LoadSettings(const char *file)
{
    tmcfg_t cfg;
    const char *ptr;

    LOGINFO("tmave_LoadSettings() [%s]", file);
    if ((cfg = tmcfg_open(file, TMCFG_MODE_READ)) == NULL)
    {
        LOGERROR("failed to open config file: [%s]", file);
        return;
    }

    /* LOAD SETTING OF AV ENFORCER */
    ptr = tmcfg_get_param_value(cfg, CONF_MODULE, "enable", NULL);
    if (ptr != NULL)
        gEnable = atoi(ptr);

    ptr = tmcfg_get_param_value(cfg, CONF_MODULE, "time_report", NULL);
    if (ptr != NULL)
        gTimeReport = atoi(ptr);

    ptr = tmcfg_get_param_value(cfg, CONF_MODULE, "time_outdate", NULL);
    if (ptr != NULL)
        gTimeOutdate = atoi(ptr);

    ptr = tmcfg_get_param_value(cfg, CONF_MODULE, "time_update", NULL);
    if (ptr != NULL)
        gTimeUpdate = atoi(ptr);

    ptr = tmcfg_get_param_value(cfg, CONF_MODULE, "url_avserver", NULL);
    if (ptr != NULL)
    {
        strncpy(gAvSvrURL, ptr, sizeof(gAvSvrURL));
        gAvSvrURL[sizeof(gAvSvrURL)-1] = 0;
    }

    tmcfg_close(cfg);
    //loadAVEConfInCache();//
    sendAVEConfToKernel();

    LOGINFO("AVE: Enable=%u URL=%s", gEnable, gAvSvrURL);
    LOGINFO("AVE: TR=%u TO=%u TU=%u", gTimeReport, gTimeOutdate, gTimeUpdate);   
}


