/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file tmIPMAC.c
 *  @brief TMOG ClientInfo Module
 */

/*
 *  Change Log
 *  2004-02-18 Eric Li Initialization version
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <string.h>
#include "log.h"
#include "tmClientInfo.h"


#define ARP_DB         "/proc/net/arp"

#define MAX_LINE       128

static const char ModuleName[] = "tmClientInfo";


/*
 * Convert a string like "02-2D-98-46-DE-F9", "12:34:DE:CA:FE:93" to MAC
 */
void StrToMac(const char *str, unsigned char *mac)
{
   int i = 0;
   unsigned int val;

   for (i = 0; i < MAC_SIZE && i < strlen(str); i++)
   {
        sscanf(str+3*i, "%02X", &val);
        mac[i] = (unsigned char) val;
   }
}

/* 
 * Convert MAC to String format
 * str must have enough space (at least 18)
 */
void MacToStr(const unsigned char *mac, char *str)
{
    sprintf(str, "%02X-%02X-%02X-%02X-%02X-%02X", mac[0], mac[1], mac[2], 
                                                  mac[3], mac[4], mac[5]);
}


/*
 * @brief Convert IP to MAC, or MAC to IP
 *  If from IP to MAC, mac should be memset to 0 before calling this func
 *  If from MAC to IP, ip should be memset to 0 before calling this func
 */
int IP_MAC(uint32_t *ip, unsigned char *mac)
{
    FILE *fp;
    char buf[MAX_LINE];
    char strTYPE[MAX_LINE], strFLAG[MAX_LINE];
    char strIP[MAX_LINE], strMAC[MAX_LINE];
    uint32_t  nIP;
    unsigned char nMAC[MAC_SIZE];
    int nRetVal = -1;

    if ((fp = fopen(ARP_DB, "r")) == NULL)
    {
        LOGERROR("Cannot read ARP db %s\n", ARP_DB);
        return nRetVal;
    }

    /* skip first line */
    if (fgets(buf, sizeof(buf), fp) == NULL)
    {
        LOGERROR("Cannot read first line of ARP db %s\n", ARP_DB);
        goto END;
    }

    while (fgets(buf, sizeof(buf), fp) != NULL)
    {
        if (sscanf(buf, "%s %s %s %s", strIP, strTYPE, strFLAG, strMAC) != 4)
        {
            LOGERROR("Cannot read one line of ARP db %s\n", ARP_DB);
            goto END;
        }
         if (inet_pton(AF_INET, strIP, &nIP) <= 0)
        {
            LOGERROR("Cannot create a network address structure by inet_pton\n");
            goto END;
        }
        StrToMac(strMAC, nMAC);
        if (nIP == *ip)
        {
            memcpy(mac, nMAC, sizeof(nMAC));
            nRetVal = 0;
            break;
        }
        else if (!memcmp(mac, nMAC, sizeof(nMAC)))
        {
            *ip = nIP;
            nRetVal = 0;
            break;
        }
    }

END:
    fclose(fp);
    return nRetVal;
}

