/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/


/**
 *  @testclientinfo.c
 *  @TMSS ClientInfo Module
 */

/*
 *  Change Log
 *  2004-02-18 Eric Li
 *  Initialization version
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <time.h>
#include <arpa/inet.h>
#include "tmClientInfo.h"



static void DumpMac(unsigned char *mac)
{
    int i;
    printf("mac=");
    for (i = 0; i < MAC_SIZE; i++)
         printf("%02X%c", mac[i], i==MAC_SIZE-1?' ':'-');
    printf("\n");
}

static void DumpClient(ClientInfo *pClient)
{
    int i;

    printf("==========================\n");

    /* ID info */
    DumpMac(CMAC(pClient));
    printf("ip=%u.%u.%u.%u\n", NIPQUAD(CIP(pClient)));
    printf("hostname=%s\n", CHOSTNAME(pClient));

    /* AV info */
    printf("status=%s\n", MsgStatus[CSTATUS(pClient)]);
    //printf("product=%d\n", CPRODUCT(pClient));
    printf("pattern=0x%08X (%d)\n", CPATTERN(pClient), CPATTERN(pClient));
    printf("engine=0x%08X (%d)\n", CENGINE(pClient), CENGINE(pClient));
    printf("license=%s\n", MsgLicense[CLICENSE(pClient)]);

    /* Exception info */
    printf("exception=");
    printf("%s ", CISEXPREPORT(pClient)?"report":" ");
    printf("%s\n", CISEXPFILTER(pClient)?"urlfilter":" ");

    /* TIME info, no newline is needed */
    printf("av_time=%s", ctime(&CTAV(pClient)));
    printf("va_time=%s", ctime(&CTVA(pClient)));
    printf("report_time=%s", ctime(&CTREDIRECT(pClient)));
    printf("update_time=%s", ctime(&CTUPDATE(pClient)));
    printf("create_time=%s", ctime(&CTCREATE(pClient)));
    printf("redir_flag=%d\n", CTREDIRECTFLAG(pClient));
    printf("redir_retry_count=%d\n", CTREDIRECTRETRY(pClient));

    /* VA info */
    printf("MSPatchCount=%d\n", CVAMSPATCHCOUNT(pClient));
    printf("TrojanCount=%d\n", CVATROJANCOUNT(pClient));
    printf("NetshareCount=%d\n", CVANETSHARECOUNT(pClient));
    printf("AVSoftCount=%d\n", CVAAVSOFTCOUNT(pClient));
    printf("SpywareCount=%d\n", CVASPYWARECOUNT(pClient));
    printf("ScanDate=%lu\n", CVASCANDATE(pClient));
    for (i = 0; i < VATYPE_SIZE; i++)
    {
         printf("va%d:%s=%s\n", i, MsgVA[i], CVA(pClient)[i]);
    }
}

static void doDump(void) 
{ 
    tmci_DumpClientTable(); 
}

static void doReset(void) 
{
    tmci_ResetClientTable(); 
}

static void doSetID(int argc, char **argv, struct client_id *id)
{
    int i;

    for (i = 0; i < argc; i+=2)
    {
        if (!strcmp(argv[i], "ip"))
            inet_pton(AF_INET, argv[i+1], &id->ip);
        else if (!strcmp(argv[i], "hostname"))
            strncpy(id->hostname, argv[i+1], HOSTNAME_SIZE-1);
        else if (!strcmp(argv[i], "mac"))
            StrToMac(argv[i+1], id->mac);
    }
}

static void doSetAV(int argc, char **argv)
{
    int    i, ret;
    struct client_av av;
    struct client_id id;

    if(argc < 3 || argc%2 != 1)
    {
       printf("%s [name value]...\n", argv[0]);
       printf("\tID-name (mac, ip, hostname)\n");
       printf("\tAV-name (status, product, engine, pattern, license)\n");
       return;
    }

    argc--, argv++;

    memset(&av, 0, sizeof(struct client_av));
    memset(&id, 0, sizeof(struct client_id));

    doSetID(argc, argv, &id);

    for (i = 0; i < argc; i+=2)
    {
        if (!strcmp(argv[i], "status"))
            av.status = atoi(argv[i+1]);
        else if (!strcmp(argv[i], "product"))
            av.auversion.product =  atoi(argv[i+1]);
        else if (!strcmp(argv[i], "engine"))
            av.auversion.engine.version =  atoi(argv[i+1]);
        else if (!strcmp(argv[i], "pattern"))
            av.auversion.pattern.version =  atoi(argv[i+1]);
        else if (!strcmp(argv[i], "license"))
            av.license =  atoi(argv[i+1]);
    }
    printf("Call tmci_SetAV\n");
    ret = tmci_SetAV(&id, &av);
    printf("\n==Return=%d\n", ret);
}


static void doSetVA(int argc, char **argv)
{
    int    i, ret;
    struct client_va va;
    struct client_id id;

    if(argc < 3 || argc%2 != 1)
    {
       printf("%s [name value]...\n", argv[0]);
       printf("\tID-name (mac, ip, hostname)\n");
       printf("\tVA-name (va0, va1, ...., va%d)\n", VATYPE_SIZE);
       return;
    }

    argc--, argv++;

    memset(&va, 0, sizeof(struct client_va));
    memset(&id, 0, sizeof(struct client_id));

    doSetID(argc, argv, &id);

    for (i = 0; i < argc; i+=2)
    {
         if (!strncmp(argv[i], "va", 2))
         {
             unsigned int type = 0;
             if (sscanf(argv[i], "va%u", &type) != 1 || type >= VATYPE_SIZE)
             {
                printf("must be between va0 - va%u\n", VATYPE_SIZE);
                return;
             }
             strncpy(va.va[type], argv[i+1], VADETAIL_SIZE-1);
         }
    }
    printf("Call tmci_SetVA\n");
    ret = tmci_SetVA(&id, &va);
    printf("\n==Return=%d\n", ret);
}

static void doSetExp(int argc, char **argv)
{
    int    i, ret;
    struct client_id id;
    struct client_exp exp;

    if(argc < 3 || argc%2 != 1)
    {
       printf("%s [name value]...\n", argv[0]);
       printf("\tID-name (mac, ip, hostname)\n");
       printf("\tEXP-name (urlfilter report)\n");
       return;
    }

    argc--, argv++;

    memset(&id, 0, sizeof(struct client_id));
    memset(&exp, 0, sizeof(struct client_exp));

    doSetID(argc, argv, &id);

    for (i = 0; i < argc; i+=2)
    {
        if (!strcmp(argv[i], "urlfilter"))
            exp.urlfilter = atoi(argv[i+1]);
        else if (!strcmp(argv[i], "report"))
            exp.report = atoi(argv[i+1]);
    }

    printf("Call tmci_SetExp\n");
    ret = tmci_SetExp(&id, &exp);
    printf("\n==Return=%d\n", ret);
}



static void doDel(int argc, char **argv)
{
    int ret;
    unsigned char mac[MAC_SIZE];

    if(argc != 2)
    {
       printf("%s [mac]\n", argv[0]);
       return;
    }

    StrToMac(argv[1], mac);
    DumpMac(mac);
    ret = tmci_DelClient(mac);
    printf("\n==Return=%d\n", ret);
}

static void doUpdate(int argc, char **argv)
{
    int ret;
    TmuAUVersion_t au;

    if(argc != 4)
    {
       printf("%s [product] [engine] [pattern]\n", argv[0]);
       return;
    }

    au.product =  atoi(argv[1]);
    au.engine.version =  atoi(argv[2]);
    au.pattern.version =  atoi(argv[3]);

    ret = tmci_UpdateClientStatus(&au);

    printf("\n==Return=%d\n", ret);
}

static void doSizeClient(int argc, char **argv)
{
    printf("Size=%d\n", tmci_SizeClient());
}

static void doSizeLicense(int argc, char **argv)
{
    uint8_t idx;
    int size;

    if(argc != 2)
    {
       printf("%s [idx]\n", argv[0]);
       return;
    }

    idx = atoi(argv[1]);
    if (idx >= LICENSE_SIZE)
        printf("out of range\n");
    else
    {
        printf("idx=%s\n", MsgLicense[idx]);
        size = tmci_SizeLicense(idx);
        printf("Size=%d\n", size);
    }
}

static void doGet(int argc, char **argv)
{
    int ret;
    unsigned char mac[MAC_SIZE];
    ClientInfo info;

    if(argc != 2)
    {
       printf("%s [mac]\n", argv[0]);
       return;
    }

    StrToMac(argv[1], mac);
    DumpMac(mac);
    if ((ret = tmci_GetClient(mac, &info)) >= 0)
    {
         DumpClient(&info);
    }
    printf("\n==Return=%d\n", ret);
}

static void doGetFrom(int argc, char **argv)
{
    int index, size;
    int i, ret;
    ClientInfo *pBuf;

    if(argc != 3)
    {
       printf("%s [index] [size]\n", argv[0]);
       return;
    }

    index = atoi(argv[1]);
    size = atoi(argv[2]);

    printf("index=%d size=%d\n", index, size);

    pBuf = (ClientInfo *)malloc(size * sizeof(ClientInfo));
    if (pBuf == NULL)
    {
        printf("memory allocation error\n");
        return;
    }

    if ((ret = tmci_GetClientsFrom(pBuf, &size, index)) >= 0)
    {
         printf("output size=%d\n", size);
         for (i = 0; i < size; i++)
              DumpClient(pBuf+i);
    }
    free(pBuf);
    printf("==\nReturn=%d\n", ret);
}

static void doDestroy(void)
{
  tmci_DestroyClientTable();
}

static void doUsage(char *name)
{
    int i;
    char *mesg[] =
    {
        "-setav           Update client's AV info",
        "-setva           Update client's VA info",
        "-setexp          Update client's Exception info",
        "-get             Query a client record",
        "-from            Get clients from an index",
        "-del             Delete a client record",
        "-update          Update client status",
        "-size            Size of type clients",
        "-license         Size of license clients",
        "-reset           Delete all the records",
        "-dump            Dump the database",
        "-destroy         Destroy the shared memory",
        NULL,
    };

    fprintf(stderr, "Usage: %s [OPTIONS]...\n\n", name);
    i = 0;
    while (mesg[i] != NULL)
        fprintf (stderr, "%10c%s\n", ' ', mesg[i++]);

    fprintf(stderr, "\n[status] ");
    for (i = 0; i < STATUS_SIZE; i++)
         fprintf(stderr, "%d=%s, ", i, MsgStatus[i]);

    fprintf(stderr, "\n[license] ");
    for (i = 0; i < LICENSE_SIZE; i++)
         fprintf(stderr, "%d=%s, ", i, MsgLicense[i]);

    fprintf(stderr, "\n[va] ");
    for (i = 0; i < VATYPE_SIZE; i++)
         fprintf(stderr, "va%d:%s,", i, MsgVA[i]);

    printf("\n\n");
    exit(1);
}

int main(int argc, char **argv)
{
    char *prog, *cmd;

    if (argc < 2) 
        doUsage(argv[0]);

    tmci_InitClientTable();

    prog = argv[0], cmd = argv[1];
    argc--, argv++;

    if (!strcmp(cmd, "-setav"))
        doSetAV(argc, argv);
    else if (!strcmp(cmd, "-setva"))
        doSetVA(argc, argv);
    else if (!strcmp(cmd, "-setexp"))
        doSetExp(argc, argv);
    else if (!strcmp(cmd, "-get"))
        doGet(argc, argv);
    else if (!strcmp(cmd, "-from"))
        doGetFrom(argc, argv);
    else if (!strcmp(cmd, "-del"))
        doDel(argc, argv);
    else if (!strcmp(cmd, "-update"))
        doUpdate(argc, argv);
    else if (!strcmp(cmd, "-size"))
        doSizeClient(argc, argv);
    else if (!strcmp(cmd, "-license"))
        doSizeLicense(argc, argv);
    else if (!strcmp(cmd, "-dump"))
        doDump();
    else if (!strcmp(cmd, "-reset"))
        doReset();
    else if (!strcmp(cmd, "-destroy"))
        doDestroy();
    else
        doUsage(prog);

    return 0;
}

