/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file   TmDateType.h
 *  @brief  This file defines some primitive data type for this project.  It 
 *          originate from IMSS project.  We extract the necessary information
 *          from the files in oringinal ~/src/common/include/unicode/ folder.
 */

/*
 *  Change Log
 *  2004-02-14  Jeremy  initial version
 */
#ifndef _TMDATATYPE_H_
#define _TMDATATYPE_H_


/* Define whether inttypes.h is available */
#ifndef U_HAVE_INTTYPES_H
#define U_HAVE_INTTYPES_H 1
#endif

/* Determines whether specific types are available */
#ifndef HAVE_INT8_T
#define HAVE_INT8_T 1
#endif

#ifndef HAVE_UINT8_T
#define HAVE_UINT8_T 0
#endif

#ifndef HAVE_INT16_T
#define HAVE_INT16_T 1
#endif

#ifndef HAVE_UINT16_T
#define HAVE_UINT16_T 0
#endif

#ifndef HAVE_INT32_T
#define HAVE_INT32_T 1
#endif

#ifndef HAVE_UINT32_T
#define HAVE_UINT32_T 0
#endif

#ifndef HAVE_BOOL_T
#define HAVE_BOOL_T 0 
#endif

/* Determines the endianness of the platform */
#define U_IS_BIG_ENDIAN 0



/**
 * Generic data types
 *
 * If your platform does not have the <inttypes.h> header, you may
 * need to edit the typedefs below. 
 * In Linux, the file locate in "/usr/include/inttypes.h"
 */
#if U_HAVE_INTTYPES_H
#   include <inttypes.h>
#else
#   include <sys/types.h>
#   if STDC_HEADERS
#       include <stdlib.h>
#       include <stddef.h>
#   endif

#if ! HAVE_INT8_T  
typedef signed char int8_t;
#endif

#if ! HAVE_UINT8_T
typedef unsigned char uint8_t;
#endif

#if ! HAVE_INT16_T
typedef signed short int16_t;
#endif

#if ! HAVE_UINT16_T
typedef unsigned short uint16_t;
#endif

#if ! HAVE_INT32_T
#   if defined(_LP64)
      typedef signed int  int32_t;
#   else
      typedef signed long int32_t;
#   endif
#endif

#if ! HAVE_UINT32_T
#   if defined(_LP64)
      typedef unsigned int  uint32_t;
#   else
      typedef unsigned long uint32_t;
#   endif
#endif

#endif  /* U_HAVE_INTTYPES_H */

/**
 * Boolean data type                                                         
 */
#if ! HAVE_BOOL_T
    typedef int8_t bool_t;
#endif

#ifndef TRUE
#   define TRUE  1
#endif

#ifndef FALSE
#   define FALSE 0
#endif


#endif /* _TMDATATYPE_H_ */
