/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file   TmErrors.h
 *  @brief  This file define the error code for this whole project, it originate  
 *          from IMSS project.
 */

/*
 *  Change Log
 *  2004-02-14  Jeremy  initial version
 */
#ifndef _TMERRORS_H_
#define _TMERRORS_H_

/*******************/
/*  Include files  */
/*******************/

/**
 * We do not use the whole unicode definition from ICU package, 
 * just define some primitive data type for our program use 
 */
//#include <unicode/utypes.h>
#include "TmDataType.h"

/***************************************************************************
 *
 *  TmErrorCode encoding scheme
 *
 *
 *  3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
 *  1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
 *  +-+-+-----------+---------------+-------------------------------+
 *  |O|R|  Facility |   Source ID   |        Condition Code         |
 *  +-+-+-----------+---------------+-------------------------------+
 *
 *  O = outcome
 *  R = reserved
 *
 ***************************************************************************/

/**********************/
/*  Type definitions  */
/**********************/

typedef int32_t TmErrorCode;

#define TM_ERR_ZERO_ERROR ((TmErrorCode)0)

/*******************/
/*  Outcome codes  */
/*******************/

enum TmErrorOutcome {
  TM_ERR_O_SUCCESS = 0,
  TM_ERR_O_FAILURE = 1,
  TM_ERR_O_LIMIT
};

/********************/
/*  Facility codes  */
/********************/

enum TmErrorFacility {
  TM_ERR_F_UNKNOWN = 0,
  TM_ERR_F_INTEG   = 1,
  TM_ERR_F_WIN32   = 2,
  TM_ERR_F_POSIX   = 3,
  TM_ERR_F_LDAP    = 4,   /* above are old definition from IMSS project */
  TM_ERR_F_TMSS    = 5,
  TM_ERR_F_LIMIT
};

/****************/
/*  Source IDs  */
/****************/

typedef enum TmModuleID TmErrorSource;

/*********************/
/*  Condition codes  */
/*********************/

/* Integration module interface condition codes */
enum TmErrorCodeIntegration {
  TM_ERR_C_INT_UNKNOWN                               = 0x0000, /* (0) */

  /* To add generic interface condition codes here.  By convention,
     all generic condition codes are assigned within the 0x0000 (0)
     and 0x0FFF (4095) range, inclusively. */

  TM_ERR_C_INT_NOT_SUPPORTED                         = 0x0001, /* (1) */
  TM_ERR_C_INT_INTERNAL_ERROR                        = 0x0002, /* (2) */
  TM_ERR_C_INT_MISC                                  = 0x0003, /* (3) */
  TM_ERR_C_INT_OPERATION_FAILED                      = 0x0004, /* (4) */

  TM_ERR_C_INT_MEMORY_ALLOC                          = 0x0005, /* (5) */

  TM_ERR_C_INT_BUFFER_OVERFLOW                       = 0x0006, /* (6) */
  TM_ERR_C_INT_ILLEGAL_ACCESS                        = 0x0007, /* (7) */

  TM_ERR_C_INT_INVALID_ARGUMENT                      = 0x0008, /* (8) */
  TM_ERR_C_INT_INVALID_ENCODING                      = 0x0009, /* (9) */
  TM_ERR_C_INT_INVALID_FORMAT                        = 0x000A, /* (10) */
  TM_ERR_C_INT_INVALID_HANDLE                        = 0x000B, /* (11) */
  TM_ERR_C_INT_INDEX_OUT_OF_BOUND                    = 0x000C, /* (12) */

  TM_ERR_C_INT_ACCESS_DENIED                         = 0x000D, /* (13) */
  TM_ERR_C_INT_TIMEOUT                               = 0x000E, /* (14) */

  TM_ERR_C_INT_DISK_FULL                             = 0x000F, /* (15) */
  TM_ERR_C_INT_INVALID_PATHNAME                      = 0x0010, /* (16) */
  TM_ERR_C_INT_FILE_NOT_FOUND                        = 0x0011, /* (17) */
  TM_ERR_C_INT_FILE_EXISTS                           = 0x0012, /* (18) */
  TM_ERR_C_INT_DIRECTORY_NOT_FOUND                   = 0x0013, /* (19) */
  TM_ERR_C_INT_FILE_ACCESS_DENIED                    = 0x0014, /* (20) */
  TM_ERR_C_INT_FILE_SHARING_CONFLICT                 = 0x0015, /* (21) */
  TM_ERR_C_INT_FILE_LOCK_CONFLICT                    = 0x0016, /* (22) */
  TM_ERR_C_INT_FILE_CREATION_FAILED                  = 0x0017, /* (23) */
  TM_ERR_C_INT_FILE_OPEN_FAILED                      = 0x0018, /* (24) */
  TM_ERR_C_INT_EOF                                   = 0x0019, /* (25) */
  TM_ERR_C_INT_READ_FAILED                           = 0x001A, /* (26) */
  TM_ERR_C_INT_WRITE_FAILED                          = 0x001B, /* (27) */

  TM_ERR_C_INT_PROTOCOL_VIOLATION                    = 0x001C, /* (28) */
  TM_ERR_C_INT_PROTOCOL_NOT_SUPPORTED                = 0x001D, /* (29) */

  Tm_ERR_C_INT_UNKNOWN_HOST                          = 0x001E, /* (30) */
  TM_ERR_C_INT_UNKNOWN_SERVICE                       = 0x001F, /* (31) */
  TM_ERR_C_INT_MALFORMED_URL                         = 0x0020, /* (32) */

  TM_ERR_C_INT_LIB_LOAD_FAILED                       = 0x0021, /* (33) */
  TM_ERR_C_INT_LIB_OPEN_FAILED                       = 0x0022, /* (34) */
  TM_ERR_C_INT_LIB_FUNC_FAILED                       = 0x0023, /* (35) */

  TM_ERR_C_INT_LICENSE_EXPIRED                       = 0x0024, /* (36) */

  __TM_ERR_C_INT_NEXT_UNUSED_CODE__                  = 0x0025,

  /* Module-specific/component-specific condition codes begin with
     0x1000 (4096) and terminate with 0xFFFF (65535).  Right now, each
     module is assigned sufficient space for 256 different condition
     code.  If we run out space for additional modules, we should be
     able to interleave the additional modules between existing
     partitions. */

  /* Remark for the TMSS project.
     The following ranges are reserved and used by IMSS integration project. 
     We honor their usages and define ours after their condition codes.  
     - Message/MIME related condition codes,              0x1000 - 0x10FF
     - Filter and filter manager related condition codes, 0x1100 - 0x11FF
     - Virus filter group related condition codes,        0x1200 - 0x12FF
     - eManager filter group related condition codes,     0x1300 - 0x13FF
     - Classification related condition codes,            0x1400 - 0x14FF
     - Notification related condition codes,              0x1500 - 0x15FF
     - Scan action related condition codes,               0x1600 - 0x16FF
     - Scan module related condition codes,               0x1700 - 0x17FF
     - Policy related condition codes,                    0x1800 - 0x18FF
     - ACL (access control list) condition codes,         0x1900 - 0x19FF     
   */

  /* Acitve Update Information Module condition codes */

  TM_ERR_C_TMSS_AU_SECTION_NOT_FOUND                 = 0x1A00, /* 6656 */
  TM_ERR_C_TMSS_AU_OTHERS                            = 0x1A01, /* 6657 */

  __TM_ERR_C_TMSS_AU_NEXT_UNUSED_CODE__              = 0x1A02,

  TM_ERR_C_INT_LIMIT                                 = 0xFFFF  /* 65535 */
};

/***********************/
/*  Macro definitions  */
/***********************/

/* Success/failure condition macros */

#ifdef __cplusplus
inline bool_t TM_SUCCESS(TmErrorCode cond) { return (cond >= 0); }
inline bool_t TM_FAILURE(TmErrorCode cond) { return (cond <  0); }
#else /* not __cplusplus */
#define TM_SUCCESS(cond)    ((TmErrorCode)cond >= 0)
#define TM_FAILURE(cond)    ((TmErrorCode)cond <  0)
#endif /* __cplusplus */

/* Error code construction macros */

#define TM_CONDITION(outcome,facility,source,code) \
  ((TmErrorCode)((((outcome)  & 0x01) << 31) |     \
                 (((facility) & 0x3F) << 24) |     \
                 (((source)   & 0xFF) << 16) |     \
                  ((code)     & 0xFFFF)))

#define TM_CONDITION_SUCCESS(source) \
  TM_CONDITION(TM_ERR_O_SUCCESS, TM_ERR_F_UNKNOWN, source, TM_ERR_C_INT_UNKNOWN)

#define TM_CONDITION_INTEG(outcome,source,code) \
  TM_CONDITION(outcome, TM_ERR_F_INTEG, source, code)
#define TM_CONDITION_WIN32(outcome,source,code) \
  TM_CONDITION(outcome, TM_ERR_F_WIN32, source, (int16_t)code)
#define TM_CONDITION_POSIX(outcome,source,code) \
  TM_CONDITION(outcome, TM_ERR_F_POSIX, source, (int16_t)code)
#define TM_CONDITION_TMSS(outcome,source,code) \
  TM_CONDITION(outcome, TM_ERR_F_TMSS, source, (int16_t)code)

/* Field extraction macros */

#define TM_COND_OUTCOME(cond)       ((enum TmErrorOutcome)        (((cond) >> 31) & 0x01))
#define TM_COND_FACILITY(cond)      ((enum TmErrorFacility)       (((cond) >> 24) & 0x3F))
#define TM_COND_SOURCE(cond)        ((TmErrorSource)         (((cond) >> 16) & 0xFF))
#define TM_COND_CODE(cond)                                         ((cond)        & 0xFFFF)
#define TM_COND_CODE_INTEG(cond)    ((enum TmErrorCodeIntegration) ((cond)        & 0xFFFF))
#define TM_COND_CODE_WIN32(cond)    ((int16_t)                     ((cond)        & 0xFFFF))
#define TM_COND_CODE_POSIX(cond)    ((int16_t)                     ((cond)        & 0xFFFF))

#endif /* _TMERRORS_H_ */




