/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/** 
 *  @file tmdpd.c
 *  @brief TMOG Discovery Module
 */

/*
 *  Change Log
 *  2004-02-17 Jim Wu    Initial Version
 */
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <arpa/inet.h> 
#include <string.h>
#include "log.h"          /* for debug logs */
#include "tmdpCommonDef.h"  /* common type define */
#include "tmrqRequestQueue.h"
#include "tmdpClientInfoWrapper.h"  /* for updating client info table */
#include "tmSignalHandler.h"

static const char ModuleName[]="tmpd";

/* function prototype */

/* for shared memory */
extern int SetupshareMem(int nReset);
int DetachshareMem(int nShmid);

int Child_Process(void)
{
	/* need to retrieve item from the queue */
	int bIsEmpty = tmrq_IsQueueEmpty();
	MESSAGE_ITEM messageItem;
	__u32 nIP    = 0;
	int nResult  = 0;
	
	if (bIsEmpty)
	{
		return 0;
	}
	
   /* retrieve the last item from queue, does not remove item */
   	tmrq_GetLastItem(&messageItem);
	
	inet_pton(AF_INET, messageItem.szClientIP, &nIP);    
  
    /* check if PCC is installed */		    		
    LOGDEBUG("before calling tmdp_CheckClientInfo, command type = %d", messageItem.nCommandType);
    
    nResult = tmdp_CheckClientInfo(nIP, messageItem.nCommandType);
    
    LOGINFO("Client IP = <%s>\n", messageItem.szClientIP);	
  
    switch (nResult)
    {
  	  case STATUS_NOAVWIN:  		
  		LOGDEBUG("*** NO PCC installed ***\n");  		
  		break;
      case STATUS_AVOK:
  		LOGINFO("PCC had already installed ***\n\n");  		  		
        break;
      case STATUS_AVOLD:
  	   /* just notify PCC */
  	   LOGINFO("Just Notify PCC to update pattern/engine ***\n\n");  		  		  	     	   
  	   break;
      default:
        LOGINFO("Default case from tmdp_CheckClientInfo");  		  		  	   
  		break;		  		
   }			  	
  
   LOGDEBUG("Child_Process End\n");
   
   tmrq_DeleteLastItem(&messageItem);
	
   LOGINFO("Remove Last Item from Queue\n");
	
   return 0;
}

int main(int argc, char *argv[])
{		
   int nShmid;
   LOGOPEN("TMDP");
   LOGINFO("init");
   LOGDEBUG_ON();	

   if (SetupPIdTemp() == FAIL)
   {
  	  /* fail to setup temp pid file */
      exit(1);
   }  	  	
  	
   nShmid = SetupshareMem(1);
  	
   if (nShmid == -1)
   {
  	  return 0;
   }
 tmci_InitClientTable();
 
  DiscoveryDaemon() ;

/*  if (DiscoveryDaemon() == CHILD_PROCESS)
   {
  	  // from child process, need to check PCC client info    		   		
  	  Child_Process();  		
  		
  	  //LOGDEBUG(" return from child process 
  	  return 0;
   }
	*/  
   /* LOGDEBUG(" return from parent process */
	  
   DetachshareMem(nShmid);
	
   LOGCLOSE();	    
   return 0;  
}
