// OptionSheet.cpp : implementation file
//

#include "stdafx.h"
#include <afxpriv.h>
#include "OptionSheet.h"

#include "resource.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define DEFAULT_DLG_STYLE	(WS_POPUPWINDOW		|\
							 WS_VISIBLE			|\
							 WS_CLIPSIBLINGS	|\
							 WS_DLGFRAME		|\
							 WS_OVERLAPPED		|\
							 WS_MINIMIZEBOX		|\
							 DS_MODALFRAME)

#define DEFAULT_DLG_EXSTYLE	(WS_EX_LEFT				|\
							 WS_EX_LTRREADING		|\
							 WS_EX_RIGHTSCROLLBAR	|\
							 WS_EX_DLGMODALFRAME	|\
							 WS_EX_WINDOWEDGE		|\
							 WS_EX_CONTROLPARENT)


// DLGTEMPLATEEX taken from afximpl.h
#pragma pack(push, 1)
typedef struct
{
	WORD dlgVer;
	WORD signature;
	DWORD helpID;
	DWORD exStyle;
	DWORD style;
	WORD cDlgItems;
	short x;
	short y;
	short cx;
	short cy;
} DLGTEMPLATEEX;
#pragma pack(pop)

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

COptionSheet::COptionList::COptionList()
{
}

COptionSheet::COptionList::~COptionList()
{
}

BOOL COptionSheet::COptionList::NotifyParentOfSelChanging()
{
	CWnd *wnd;
	NMHDR hdr;
	wnd = GetControlWnd();
	ASSERT_KINDOF(CWnd, wnd);

	hdr.code = OSN_SELCHANGING;
	hdr.idFrom = ::GetWindowLong(wnd->GetSafeHwnd(), GWL_ID);
	hdr.hwndFrom = wnd->GetSafeHwnd();
	wnd = wnd->GetParent();
	ASSERT_KINDOF(CWnd, wnd);

	return (BOOL)wnd->SendMessage(WM_NOTIFY, hdr.idFrom, (DWORD)&hdr);
}

IMPLEMENT_DYNAMIC(COptionSheet, CTrayDialog);

BEGIN_MESSAGE_MAP(COptionSheet, CTrayDialog)
	//{{AFX_MSG_MAP(COptionSheet)
	ON_WM_PAINT()
	ON_WM_SIZE()
	ON_WM_SYSCOMMAND()
	ON_WM_LBUTTONDOWN()
	ON_WM_CTLCOLOR()
	//}}AFX_MSG_MAP
	ON_NOTIFY(OSN_SELCHANGING, IDC_PAGELIST, ListSelectionChanging)
	ON_COMMAND(IDOK, OnOK)
	ON_COMMAND(IDCANCEL, OnCancel)
	ON_COMMAND(ID_APPLY_NOW, OnApply)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COptionSheet

void COptionSheet::CommonConstruct(CWnd *pParentWnd, UINT iSelectPage)
{
	ASSERT(iSelectPage >= 0);
	memset(&m_psh, 0, sizeof(m_psh));

	m_ListWindow = NULL;
	m_StartPageIndex = iSelectPage;
	m_psh.dwFlags = OSH_PROPSHEETPAGE;
	m_ButtonYLocOffset = 0;
	m_ButtonWidth = 0;
	m_AllButtonsWidth = 0;
	m_YBorder = 0;
	m_XBorder = 0;
	m_ButtonSpacing = 0;
	m_ButtonHeight = 0;
	m_CurrentPage = NULL;
	m_DlgTemplate = NULL;
	m_ListWidth = 150;
}

COptionSheet::COptionSheet() : CTrayDialog()
{
	m_strCaption = _T("");
	CommonConstruct(NULL, 0);
	m_hIcon = AfxGetApp()->LoadIcon(IDR_TRAYICON);
}

COptionSheet::COptionSheet(UINT nIDCaption, CWnd* pParentWnd, UINT iSelectPage)
	: CTrayDialog()
{
	ASSERT(nIDCaption >= 0);

	VERIFY(m_strCaption.LoadString(nIDCaption));
	CommonConstruct(pParentWnd, iSelectPage);
	m_hIcon = AfxGetApp()->LoadIcon(IDR_TRAYICON);
}

COptionSheet::~COptionSheet()
{
	if (NULL != m_DlgTemplate) {
		free(m_DlgTemplate);
		m_DlgTemplate = NULL;
	}

	if (NULL != m_Font.GetSafeHandle())
		m_Font.DeleteObject();

	while (FALSE == m_Buttons.IsEmpty())
		delete m_Buttons.RemoveHead();

	int i;
	for (i = 0; i < m_Pages.GetSize(); i++) {
		delete m_Pages[i];
		m_Pages[i] = NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//	Internal helper methods
//////////////////////////////////////////////////////////////////////////////
DLGTEMPLATE* COptionSheet::CreateDlgTemplate(DWORD dwStyle, DWORD dwExStyle)
{
	int size = sizeof(DLGTEMPLATE) + 6;
	DLGTEMPLATE *tmpl = (DLGTEMPLATE *)malloc(size);
	memset(tmpl, 0, size);

	if (dwStyle == (DWORD)-1)
		tmpl->style = DEFAULT_DLG_STYLE;
	else
		tmpl->style = dwStyle;

	if (dwExStyle == 0)
		tmpl->dwExtendedStyle = DEFAULT_DLG_EXSTYLE;
	else
		tmpl->dwExtendedStyle = dwExStyle;

	tmpl->cdit = 0;
	tmpl->x = 0;
	tmpl->y = 0;
	tmpl->cx = 100;
	tmpl->cy = 100;

	return tmpl;
}

void COptionSheet::ClearFlags(DWORD flags)
{
	m_psh.dwFlags &= flags;
}

void COptionSheet::SetFlags(DWORD flags)
{
	m_psh.dwFlags |= flags;
}

void COptionSheet::SetModified(BOOL bModified)
{
	ASSERT(TRUE == bModified || FALSE == bModified);
	m_Modified = bModified;

	UpdateAllButtonsState();
}

BOOL COptionSheet::IsModified()
{
	return m_Modified;
}

//////////////////////////////////////////////////////////////////////////////
//	Options and settings methods
//////////////////////////////////////////////////////////////////////////////
void COptionSheet::SetListControl(COptionList *listCtrl)
{
	m_ListWindow = listCtrl;
}

CWnd *COptionSheet::GetListControlWnd()
{
	// Get the child list control window
	ASSERT(NULL != m_ListWindow);
	CWnd *wnd = m_ListWindow->GetControlWnd();
	ASSERT_KINDOF(CWnd, wnd);
	return wnd;
}

void COptionSheet::SetListWidth(int nWidth)
{
	ASSERT(nWidth >= 0);
	m_ListWidth = nWidth;
}

int COptionSheet::GetActiveIndex() const
{
	if (NULL == m_CurrentPage)
		return -1;

	return GetPageIndex(m_CurrentPage);
}

int COptionSheet::GetPageIndex(COptionPage *page) const
{
	ASSERT(NULL != page);

	for (int i = 0; i < m_Pages.GetSize(); i++) {
		if (GetPage(i) == page)
			return i;
	}
	return - 1;
}

int COptionSheet::GetPageCount()
{
	return m_Pages.GetSize();
}

COptionPage *COptionSheet::GetPage(int nPage) const
{
	ASSERT(nPage >= 0 && nPage < m_Pages.GetSize());
	return m_Pages[nPage]->m_Page;
}

COptionPage *COptionSheet::GetPageGroup(int nPage) const
{
	ASSERT(nPage >= 0 && nPage < m_Pages.GetSize());
	return m_Pages[nPage]->m_GroupPage;
}

COptionPage* COptionSheet::GetActivePage() const
{
	return m_CurrentPage;
}

BOOL COptionSheet::SetActivePage(COptionPage *newPage)
{
	ASSERT(NULL != newPage);
	ASSERT_KINDOF(COptionPage, newPage);

	BOOL result;
	COptionPage *oldPage;
	oldPage = m_CurrentPage;

	if (NULL != oldPage) {
		result = oldPage->OnKillActive();
		if (FALSE == result)
			return FALSE;

		oldPage->OnOK();
		oldPage->DestroyWindow();
	}

	CreatePage(newPage);
	newPage->OnSetActive();
	newPage->ShowWindow(SW_SHOW);

	for (int i = 0; i < m_Pages.GetSize(); i++) {
		if (newPage == GetPage(i)) {
			m_PageIndex = i;
			break;
		}
	}

	m_CurrentPage = newPage;
	if (NULL != m_ListWindow)
		m_ListWindow->SetSelection(m_CurrentPage);

	UpdateAllButtonsState();

	return TRUE;
}

BOOL COptionSheet::SetActivePage(int nPage)
{
	ASSERT(nPage < GetPageCount());
	COptionPage	*newPage;

	if (nPage >= 0)
		newPage = GetPage(nPage);
	else
		newPage = NULL;

	return SetActivePage(newPage);
}

//////////////////////////////////////////////////////////////////////////////
//	Operations
//////////////////////////////////////////////////////////////////////////////
BOOL COptionSheet::DoModal()
{
	BOOL result;
	int oldWidth;

	oldWidth = m_ListWidth;
	m_DlgTemplate = CreateDlgTemplate(-1, 0);

	CTrayDialog::InitModalIndirect(m_DlgTemplate, m_pParentWnd);
	result = CTrayDialog::DoModal();
	m_hDialogTemplate = NULL;
	m_pParentWnd = NULL;
	m_ListWidth = oldWidth;

	return result;
}


BOOL COptionSheet::Create(CWnd* pParentWnd, DWORD dwStyle, DWORD dwExStyle)
{
	BOOL result;

	dwStyle |= WS_POPUP;
	m_DlgTemplate = CreateDlgTemplate(dwStyle, dwExStyle);
	result = CTrayDialog::CreateIndirect(m_DlgTemplate, pParentWnd);
	m_hDialogTemplate = NULL;
	m_pParentWnd = NULL;

	return result;
}

void COptionSheet::AddPage(COptionPage *page, COptionPage *pGroupPage)
{
	ASSERT(NULL != page);
	ASSERT_KINDOF(COptionPage, page);
	if (NULL != pGroupPage) {
		ASSERT_KINDOF(COptionPage, pGroupPage);
	}

	PageInfo *info;

	info = new PageInfo;
	info->m_Page = page;
	info->m_GroupPage = pGroupPage;
	info->m_IsGroup = FALSE;

	m_Pages.Add(info);

}

void COptionSheet::AddGroup(COptionPage *page)
{
	ASSERT(page != NULL);
	ASSERT_KINDOF(COptionPage, page);

	PageInfo *info;

	info = new PageInfo;
	info->m_Page = page;
	info->m_GroupPage = NULL;
	info->m_IsGroup = TRUE;

	m_Pages.Add(info);
}

BOOL COptionSheet::PressButton(int nButton)
{
	BOOL result = FALSE;

	switch(nButton) {
	case OSBTN_OK:
		result = SendMessage(WM_COMMAND, IDOK, 0);
		break;

	case OSBTN_APPLYNOW:
		result = SendMessage(WM_COMMAND, ID_APPLY_NOW, 0);
		break;

	case OSBTN_CANCEL:
		result = SendMessage(WM_COMMAND, IDCANCEL, 0);
		break;

	default:
		ASSERT(0);
	}
	return result;
}

//////////////////////////////////////////////////////////////////////////////
//	Initialization methods
//////////////////////////////////////////////////////////////////////////////

void COptionSheet::CalcTreeRect(CRect &rect)
{
	GetClientRect(rect);
	rect.left = PAGE_SPACING;
	rect.right = rect.left + m_ListWidth;
	rect.top = PAGE_SPACING + m_csBmp.cy;
	rect.bottom -= (PAGE_SPACING + m_ButtonSpacing);
}

void COptionSheet::AddButton(UINT nIDRes, int x, int w, UINT id, BOOL showButton)
{
	ASSERT(nIDRes >= 0);
	CButton *btn;
	CRect rect;
	CRect rcClient;

	GetClientRect(rcClient);

	rect.left = x;
	rect.right = x + w;
	rect.top = rcClient.Height() - (PAGE_SPACING2 + m_ButtonHeight);
	rect.bottom = rect.top + m_ButtonHeight;

	btn = new CButton();
	DWORD style = WS_TABSTOP | BS_DEFPUSHBUTTON | BS_FLAT;
	if (TRUE == showButton)
		style |= WS_VISIBLE;

	CString csBtn;
	VERIFY(csBtn.LoadString(nIDRes));
	btn->Create(csBtn, style, rect, this, id);
	btn->SetFont(&m_Font);
	m_Buttons.AddTail(btn);
}

int COptionSheet::GetButtonWidth(UINT id)
{
	int retval = 0;

	switch(id) {
	case IDOK:
	case IDCANCEL:
		retval = m_ButtonWidth;
		break;
	default:
		break;
	}
	return 0;
}

int COptionSheet::AddButtons(BOOL addBut)
{
	int		x;
	CRect	rcClient;

	GetClientRect(rcClient);

	if (TRUE == addBut)
		x = rcClient.Width() - (PAGE_SPACING2 + m_AllButtonsWidth);
	else
		x = 0;

	if (TRUE == addBut)
		AddButton(IDS_BTN_HIDE, x, m_ButtonWidth, IDOK);

	x += m_ButtonWidth;

	if (!(m_psh.dwFlags & OSH_NOAPPLYNOW)) {
		if (TRUE == addBut)
			AddButton(IDS_BTN_APPLY, x, m_ButtonWidth, ID_APPLY_NOW);
		x += m_ButtonWidth;
	}

	x += BUTTON_SPACING;
	if (TRUE == addBut)
		AddButton(IDS_BTN_EXIT, x, m_ButtonWidth, IDCANCEL);

	x += m_ButtonWidth;

	return x;
}

BOOL COptionSheet::CalcSpacing(CFont &buttonFont)
{
	TEXTMETRIC	txtMetric;
	CRect		rcClient;
	CRect		rcWindow;
	COptionPage *page;
	CRect		rect;
	
	GetClientRect(rcClient);
	GetWindowRect(rcWindow);

 	m_XBorder = rcWindow.Width() - rcClient.Width();
	m_YBorder = rcWindow.Height() - rcClient.Height();

	CDC *dc = GetDC();
	dc->SelectObject(buttonFont);
	dc->GetTextMetrics(&txtMetric);
	ReleaseDC(dc);

	m_ButtonHeight = (txtMetric.tmHeight + txtMetric.tmExternalLeading) + 10;
	m_ButtonYLocOffset = PAGE_SPACING2 + m_ButtonHeight;
	m_ButtonSpacing = m_ButtonHeight + PAGE_SPACING;
	m_ButtonWidth = BUTTON_WIDTH;
	m_AllButtonsWidth = 0;
	m_AllButtonsWidth = AddButtons(FALSE);
	m_MaxChildSize.cx = 0;
	m_MaxChildSize.cy = 0;

	for (int i = 0; i < GetPageCount(); i++) {
		BOOL result;
		CDialogTemplate tmpl;
		SIZE size;
		page = GetPage(i);

		if (NULL != page->GetTemplateName()) {
			// Create the child dialog and get the rect
			result = tmpl.Load(page->GetTemplateName());
			ASSERT(TRUE == result);
			tmpl.GetSizeInPixels(&size);

			if (size.cx > m_MaxChildSize.cx)
				m_MaxChildSize.cx = size.cx;

			if (size.cy > m_MaxChildSize.cy)
				m_MaxChildSize.cy = size.cy;
		}
	}

	return TRUE;
}

BOOL COptionSheet::IsPageAGroup(int nPage)
{
	ASSERT(nPage >= 0 && nPage < m_Pages.GetSize());
	return m_Pages[nPage]->m_IsGroup;
}

void COptionSheet::BuildList()
{
	ASSERT(NULL != m_ListWindow);

	CWnd *wnd;
	int i;

	wnd = GetListControlWnd();
	m_ListWindow->Create(this, IDC_PAGELIST);
	wnd->SetFont(&m_Font);

	for (i = 0; i < GetPageCount(); i++) {
		if (TRUE == IsPageAGroup(i))
			m_ListWindow->AddGroup(GetPage(i));
	}

	for (i = 0; i < GetPageCount(); i++) {
		if (FALSE == IsPageAGroup(i))
			m_ListWindow->AddPage(GetPage(i), GetPageGroup(i));
	}

	wnd->ShowWindow(SW_SHOW);
}

BOOL COptionSheet::OnInitDialog() 
{
	CTrayDialog::OnInitDialog();

	BOOL		result;
	CRect		rcClient;
	int			width;
	int			height;

	CBitmap bitmap;
	CPalette palette;
	GetBitmapAndPalette(IDB_TMGORED, bitmap, palette, &m_csBmp);
	m_brush.CreateSolidBrush(TRDLG_BKCLR_RGB);

	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL) {
		pSysMenu->DeleteMenu(0, MF_BYPOSITION);
		pSysMenu->DeleteMenu(1, MF_BYPOSITION);
		pSysMenu->DeleteMenu(2, MF_BYPOSITION);
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty()) {
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}
	
	m_CurrentPage = NULL;
	if (NULL != m_Font.GetSafeHandle())
		m_Font.DeleteObject();
	m_Font.Attach(GetStockObject(DEFAULT_GUI_FONT));

	// Calculate spacing needed by all items
	result = CalcSpacing(m_Font);
	if (FALSE == result) {
		ASSERT(0);
		DestroyWindow();
		return TRUE;
	}

	// Adjust our X position to compensate for the biggest child dialog
	width = m_MaxChildSize.cx;
	width += (PAGE_SPACING * 2);
	if (0 != m_ListWidth)
		width += m_ListWidth + PAGE_SPACING;
	if (width < m_AllButtonsWidth)
		width = m_AllButtonsWidth;
	width += m_XBorder;

	// Adjust our Y position to compensate for the biggest child dialog
	height = m_YBorder + m_MaxChildSize.cy + m_ButtonSpacing + 
			(PAGE_SPACING * 2) + m_csBmp.cy;

	SetWindowPos(&wndTop, 0, 0, width, height, SWP_NOMOVE|SWP_NOZORDER);

	if (0 != m_strCaption.GetLength())
		SetWindowText(m_strCaption);

	AddButtons();

	CRect rect;
	BuildList();
	CalcTreeRect(rect);
	GetListControlWnd()->MoveWindow(rect);
	SetActivePage(m_StartPageIndex);
	SetModified(FALSE);

	CString stips;
	stips.LoadString(IDS_SEG_TOOL);
	TraySetIcon(IDR_TRAYICON);	
	TraySetToolTip((LPCTSTR)stips);
	TraySetMenu(IDR_POPUP_MENU, 1);
	TraySetMinimizeToTray(TRUE);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

BOOL COptionSheet::GetBitmapAndPalette(UINT nIDResource, CBitmap &bitmap, CPalette &pal, CSize* pcs)
{
	HBITMAP hBmp = (HBITMAP)::LoadImage(AfxGetInstanceHandle(), 
										(LPCTSTR)nIDResource, 
										IMAGE_BITMAP, 0, 0, 
										LR_CREATEDIBSECTION);
	if (hBmp == NULL)
		return FALSE;

	bitmap.Attach(hBmp);

	DIBSECTION ds;
	BITMAPINFOHEADER &bmInfo = ds.dsBmih;
	bitmap.GetObject(sizeof(ds), &ds);
	if (pcs) {
		pcs->cx = bmInfo.biWidth;
		pcs->cy = bmInfo.biHeight + 3;
		return FALSE;
	}
	
	CClientDC dc(NULL);
	int nColors = bmInfo.biClrUsed ? bmInfo.biClrUsed : 1 << bmInfo.biBitCount;
	if (nColors > 256)
		pal.CreateHalftonePalette(&dc);
	else {
		RGBQUAD *pRGB = new RGBQUAD[nColors];
		CDC memDC;
		memDC.CreateCompatibleDC(&dc);
		memDC.SelectObject(&bitmap);
		::GetDIBColorTable(memDC, 0, nColors, pRGB);
		UINT nSize = sizeof(LOGPALETTE) + (sizeof(PALETTEENTRY) * nColors);
		LOGPALETTE *pLP = (LOGPALETTE *) new BYTE[nSize];
		pLP->palVersion = 0x300;
		pLP->palNumEntries = nColors;

		for (int i=0; i < nColors; i++) {
			pLP->palPalEntry[i].peRed = pRGB[i].rgbRed;
			pLP->palPalEntry[i].peGreen = pRGB[i].rgbGreen;
			pLP->palPalEntry[i].peBlue = pRGB[i].rgbBlue;
			pLP->palPalEntry[i].peFlags = 0;
		}
		pal.CreatePalette(pLP);
		delete[] pLP;
		delete[] pRGB;
	}
	return TRUE;
}

void COptionSheet::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if ((point.x > 387 && point.x < 397) &&
		(point.y > 25 && point.y < 35))
		AfxMessageBox("Tracker Report Project Team:\n\n    Kelvin Liu\n    Joe Lin\n    Tina Chen\n", MB_ICONINFORMATION);

	CTrayDialog::OnLButtonDown(nFlags, point);
}

void COptionSheet::OnPaint() 
{
	CPaintDC	dc(this);
	COLORREF	clrDark;
	COLORREF	clrLite;
	CRect		rcClient;
	CPen		penDark;
	CPen		penLite;
	int			centerX;

	clrDark = GetSysColor(COLOR_3DSHADOW);
	clrLite = GetSysColor(COLOR_3DHILIGHT);
	penDark.CreatePen(PS_SOLID, 1, clrDark);
	penLite.CreatePen(PS_SOLID, 1, clrLite);
	GetClientRect(rcClient);
	centerX = rcClient.left + PAGE_SPACING + PAGE_SPACING2 + m_ListWidth;
	rcClient.InflateRect(-PAGE_SPACING2,
						 -PAGE_SPACING2,
						 -PAGE_SPACING2,
						 -(PAGE_SPACING2 + m_ButtonSpacing));

	if (0 != m_ListWidth) {
		// Draw the center divider
		dc.SelectObject(penDark);
		dc.MoveTo(centerX, rcClient.top);
		dc.LineTo(centerX, rcClient.bottom);
		dc.SelectObject(penLite);
		dc.MoveTo(centerX + 1, rcClient.top);
		dc.LineTo(centerX + 1, rcClient.bottom);
	}
	dc.Draw3dRect(rcClient, clrDark, clrLite);
	rcClient.InflateRect(-1, -1, -1, -1);
	dc.Draw3dRect(rcClient, clrLite, clrDark);

	CDC memDC;
	CBitmap bitmap;
	CPalette palette;

	memDC.CreateCompatibleDC(&dc);
	GetBitmapAndPalette(IDB_TMGORED, bitmap, palette);
	memDC.SelectObject(&bitmap);
	dc.BitBlt(rcClient.left, rcClient.top, 
		rcClient.Width()-1, m_csBmp.cy,
		&memDC, 0, 0, SRCCOPY);
}

void COptionSheet::OnSize(UINT nType, int cx, int cy) 
{
	CTrayDialog::OnSize(nType, cx, cy);
	
	if (NULL != m_ListWindow) {
		CRect rect;
		CWnd *wnd;
		CalcTreeRect(rect);
		wnd = GetListControlWnd();
		if (FALSE != ::IsWindow(wnd->m_hWnd)) {
			ASSERT_KINDOF(CWnd, wnd);
			GetListControlWnd()->MoveWindow(rect);
		}
	}
}

void COptionSheet::CreatePage(COptionPage *page)
{
	ASSERT(NULL != page);
	BOOL	result;
	CRect	rect;

	// Determine the X and Y position of the dialog in dialog units
	rect.top = PAGE_SPACING + m_csBmp.cy;
	rect.left = PAGE_SPACING;
	if (0 != m_ListWidth)
		rect.left += (m_ListWidth + PAGE_SPACING);

	rect.right = rect.left + m_MaxChildSize.cx;
	rect.bottom = rect.top + m_MaxChildSize.cy;
	result = page->Create(rect, this);
	ASSERT(FALSE != result);
}

void COptionSheet::UpdateAllButtonsState()
{
	CWnd *wnd = GetDlgItem(ID_APPLY_NOW);
	if (NULL != wnd)
		wnd->EnableWindow(m_Modified);
}

void COptionSheet::ListSelectionChanging(NMHDR *pNMHDR, LRESULT *pResult)
{
	COptionPage *page;

	page = m_ListWindow->GetSelection();
	if (NULL != page)
		ASSERT_KINDOF(COptionPage, page);
	*pResult = SetActivePage(page);
}

void COptionSheet::OnOK()
{
	ShowWindow(SW_HIDE);
	CTrayDialog::TrayShow();
	return;
/*
	if (!ConfirmClose())
		return;

	for (int i = 0; i < GetPageCount(); i++)
		GetPage(i)->OnOK();

	EndDialog(IDOK);
*/
}

void COptionSheet::OnApply()
{
	if (TRUE == IsModified()) {
		for (int i = 0; i < GetPageCount(); i++) {
			BOOL result = GetPage(i)->OnApply();
			if (FALSE == result)
				return;
		}
		SetModified(FALSE);
	}
}

void COptionSheet::OnCancel()
{
	if (!ConfirmClose())
		return;
	
	int i;
	for (i = 0; i < GetPageCount(); i++) {
		BOOL result = GetPage(i)->OnQueryCancel();
		if (FALSE == result)
			return;
	}
	for (i = 0; i < GetPageCount(); i++) {
		GetPage(i)->OnReset();
		GetPage(i)->OnCancel();
	}
	EndDialog(IDCANCEL);
}

void COptionSheet::EndDialog(int nEndID)
{
	CTrayDialog::EndDialog(nEndID);
}

void COptionSheet::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	} else if ((nID & 0xFFF0) == SC_CLOSE) {
		ShowWindow(SW_HIDE);
		CTrayDialog::TrayShow();
	} else {
		CTrayDialog::OnSysCommand(nID, lParam);
	}
}

BOOL COptionSheet::ConfirmClose()
{
	return TRUE;

	/*if (IDYES == AfxMessageBox(IDS_CONFIRM_CLOSE, MB_YESNO))
		return TRUE;
	else
		return FALSE;
		*/
}

HBRUSH COptionSheet::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) 
{
	//HBRUSH hbr = CTrayDialog::OnCtlColor(pDC, pWnd, nCtlColor);
	
	// TODO: Change any attributes of the DC here
	
	// TODO: Return a different brush if the default is not desired
	return m_brush;
	//return hbr;
}
