#if !defined(AFX_OPTIONSHEET_H__05EA0917_4E3E_11D6_9C1E_00B0D0452EEC__INCLUDED_)
#define AFX_OPTIONSHEET_H__05EA0917_4E3E_11D6_9C1E_00B0D0452EEC__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// OptionSheet.h : header file
//

#include "traydialog.h"
#include "OptionPage.h"

/////////////////////////////////////////////////////////////////////////////
// COptionSheet window

#define OSBTN_OK		3	//!<	Use when calling PressButton to simulate pressing the OK button.
#define OSBTN_APPLYNOW	4	//!<	Use when calling PressButton to simulate pressing the Apply Now button.
#define OSBTN_CANCEL	5	//!<	Use when calling PressButton to simulate pressing the Cancel button.

#define OSH_DEFAULT			0x00000000	//!<	Default mode.
#define OSH_USEHICON		0x00000002	//!<	Use the hIcon member of the m_psh attribute.
#define OSH_PROPSHEETPAGE	0x00000008	//!<	Use option sheet mode.
#define OSH_NOAPPLYNOW		0x00000080	//!<	Option sheet has apply button.

#define BUTTON_WIDTH		75		//!<	Default button width.
#define BUTTON_SPACING		10		//!<	Default button spacing.
#define PAGE_SPACING		12		//!<	Default layout spacing.
#define FULLPAGE_SPACING	0		//!<	Default layout spacing.
#define PAGE_SPACING2		(PAGE_SPACING / 2)	//!<	Default layout spacing for half steps.
#define IDC_PAGELIST		5000	//!<	ID of the control used to list all available option pages.
#define OSN_SELCHANGE		1		//!<	Notification code to alert the option sheet a selection in the child list has changed.
#define OSN_SELCHANGING		2		//!<	Notification code to alert the option sheet a selection in the child list is changing.

#define WM_ICON_NOTIFY		WM_USER+10

class COptionSheet : public CTrayDialog
{
	DECLARE_DYNAMIC(COptionSheet);

public:
	class COptionList {
	public:
		COptionList();
		virtual ~COptionList();
		virtual BOOL Create(COptionSheet *pParentWnd, UINT nID) = 0;
		virtual BOOL AddGroup(COptionPage *pPage) = 0;
		virtual BOOL AddPage(COptionPage *pPage, COptionPage *pGroupPage) = 0;
		virtual BOOL SetSelection(COptionPage *page) = 0;
		virtual COptionPage *GetSelection() = 0;
		virtual CWnd *GetControlWnd() = 0;

	protected:
		BOOL NotifyParentOfSelChanging();
	};

	struct OPTIONSHEETCFG {
		DWORD	dwFlags;	//!<	Option sheet initialization flags
		HICON	hIcon;		//!<	Handle to the window icon (optional);
	};

private:
	void		*m_DlgTemplate;		//!<	Pointer to the dialog template
	int			m_StartPageIndex;	//!<	Index of the starting page
	int			m_PageIndex;		//!<	Index of the currently active option page
	BOOL		m_Modified;			//!<	TRUE if a page is modified or FALSE if no pages have been modified
	int			m_ButtonYLocOffset;	//!<	Y location offset from the bottom of the page for button placement
	int			m_ButtonWidth;		//!<	Width of a button
	int			m_ButtonHeight;		//!<	Height of a button
	int			m_AllButtonsWidth;	//!<	Width of all buttons and spacing
	int			m_YBorder;			//!<	Horizontal Padding used between items
	int			m_XBorder;			//!<	Vertical Padding used between items
	int			m_ButtonSpacing;	//!<	Spacing between buttons
	CSize		m_MaxChildSize;		//!<	Maximum size of an option page
	CSize		m_csBmp;
	CBrush		m_brush;

	CFont		m_Font;				//!<	Font used in buttons
	COptionPage	*m_CurrentPage;		//!<	Pointer to the currently active option page

	int			m_ListWidth;		//!<	Width of the control containing a list of the option pages
	COptionList	*m_ListWindow;		//!<	Control containing the list of option pages

	CString		m_strCaption;		//!<	Caption used on the option sheet dialog

	struct PageInfo {
		COptionPage	*m_Page;		//!<	The page (NULL if this is a group).
		COptionPage	*m_GroupPage;	//!<	The group this page belongs to.
		BOOL		m_IsGroup;		//!<	TRUE if this is a group or FALSE if a page.
	};

	CArray<PageInfo*, PageInfo*>	m_Pages;	//!<	Array of option pages
	CList<CButton*, CButton*>		m_Buttons;	//!<	List of buttons used by the page

private:
	virtual void ListSelectionChanging(NMHDR *pNMHDR, LRESULT *pResult);

// Construction
private:
	void CommonConstruct(CWnd *pParentWnd, UINT iSelectPage);

public:
	OPTIONSHEETCFG	m_psh;
	COptionSheet();
	COptionSheet(UINT nIDTemplate, CWnd* pParentWnd = NULL, UINT iSelectPage = 0);
	virtual ~COptionSheet();

	void AddPage(COptionPage *pPage, COptionPage *pGroupPage = NULL);
	void AddGroup(COptionPage *pGroup);
	int GetPageIndex(COptionPage *page) const;
	int GetPageCount();
	int GetActiveIndex() const;
	BOOL SetActivePage(COptionPage *page);
	BOOL SetActivePage(int nPage);
	COptionPage *GetPage(int nPage) const;
	COptionPage *GetPageGroup(int nPage) const;
	COptionPage* GetActivePage() const;
	
	BOOL Create(CWnd* pParentWnd = NULL, DWORD dwStyle = (DWORD)-1, DWORD dwExStyle = 0);
	BOOL IsModified();
	void SetModified(BOOL bModified = TRUE);
	BOOL PressButton(int nButton);
	void EndDialog(int nEndID);
	BOOL ConfirmClose();
	virtual int DoModal();

protected:
	HICON m_hIcon;
	virtual void OnOK(void);
	virtual void OnApply(void);
	virtual void OnCancel(void);

private:
	DLGTEMPLATE* CreateDlgTemplate(DWORD dwStyle = DWORD(-1), DWORD dwExStyle = (DWORD)0);
	BOOL GetBitmapAndPalette(UINT nIDResource, CBitmap &bitmap, CPalette &pal, CSize* pcs=NULL);
	void SetFlags(DWORD flags);
	void ClearFlags(DWORD flags);
	void CalcTreeRect(CRect &rect);
	BOOL CalcSpacing(CFont &buttonFont);
	void AddButton(UINT nIDRes, int x, int w, UINT id, BOOL showBut = TRUE);
	int AddButtons(BOOL addBut = TRUE);
	int GetButtonWidth(UINT id);
	void UpdateAllButtonsState();
	void BuildList();
	void CreatePage(COptionPage *page);
	BOOL IsPageAGroup(int nPage);

// Attributes
public:

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(COptionSheet)
	public:
	//}}AFX_VIRTUAL

// Implementation
public:
	void SetListWidth(int nWidth);
	void SetListControl(COptionList *listCtrl);
	CWnd *GetListControlWnd();
	// Generated message map functions
protected:
	//{{AFX_MSG(COptionSheet)
	virtual BOOL OnInitDialog();
	afx_msg void OnPaint();
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg HBRUSH OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_OPTIONSHEET_H__05EA0917_4E3E_11D6_9C1E_00B0D0452EEC__INCLUDED_)
