/*
 *****************************************************************************
 *
 *	(C) Copyright 1989-2000 Trend Micro, Inc.
 *	All Rights Reserved.
 *
 *	This program is an unpublished copyrighted work which is proprietary
 *	to Trend Micro, Inc. and contains confidential information that is not
 *	to be reproduced or disclosed to any other person or entity without
 *	prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *	WARNING:  Unauthorized reproduction of this program as well as
 *	unauthorized preparation of derivative works based upon the
 *	program or distribution of copies by sale, rental, lease or
 *	lending are violations of federal copyright laws and state trade
 *	secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************
 */

#include "stdafx.h"
#include <WinUser.h>
#include <stdio.h>
#include <stdlib.h>

#include "SmexDbg.h"

///////////////////////////////////////////////////////////////////////////////
// Extract from Microsoft C Runtime Library

// Temp Debug Log Module. Need to merge with other modules.

BOOL				g_bInitSuccess=FALSE;
BOOL				g_bDebugEnable;
UINT				g_nLogLevel;
UINT				g_nCategory;
TCHAR				g_szLogFileName[_MAX_PATH];
TCHAR				g_szOldLogFileName[_MAX_PATH];
TCHAR				g_szExceptLogFileName[_MAX_PATH];
TCHAR				g_szOldExceptLogFileName[_MAX_PATH];
TCHAR				g_szThisModule[_MAX_PATH];
HANDLE				g_hFile;
HANDLE				g_hExFile;
Lock				g_lock;
DWORD				g_fileSize;
DWORD				g_exFileSize;

extern int			g_ESELogLevel;

BOOL SmxDebugEx30(UINT nLevel, UINT nCategory, TCHAR* szFormat, ...);

void ReplacePercentChar(TCHAR* szInput)
{
	TCHAR* p=NULL;
	p=_tcschr(szInput,_T('%'));
	while(p)
	{
		*p=_T('_');
		p=_tcschr(szInput,_T('%'));
	}
}

void _MSG_ESE(UINT nLevel, UINT nCategory, TCHAR* szFormat, ...)
{
	if(!g_bInitSuccess) 
		return;

	nLevel--;
	if (!SMXESE_DBG_SEVERE_ERROR(nCategory))
	{
		if(!g_bDebugEnable)
			return;

		if (nLevel > g_nLogLevel)
			return;

		if (!(g_nCategory & nCategory))
			return;
	}

	TCHAR	szMsg[1024];

	va_list args;

	va_start(args, szFormat);
	_vsntprintf(szMsg, sizeof(szMsg)/sizeof(TCHAR), szFormat, args );
	va_end(args);    

	ReplacePercentChar(szMsg);
	SmxDebugEx30(nLevel, nCategory, szMsg);
}

BOOL SmxDebugInit(HANDLE hModule)
{
	if (g_bInitSuccess) return TRUE;
	g_bInitSuccess	= FALSE;
	g_bDebugEnable	= FALSE;
	g_nLogLevel		= 0;
	g_nCategory		= SMXESE_DBG_GENERIC|SMXESE_DBG_ESEAPI|SMXESE_DBG_SOCKET|SMXESE_DBG_SESSION|SMXESE_DBG_LIST;
	g_ESELogLevel	= 0;

	DWORD dwResult;
	TCHAR szModulePath[_MAX_PATH];
	TCHAR drive[_MAX_DRIVE], dir[_MAX_DIR], fname[_MAX_FNAME], ext[_MAX_EXT];

	dwResult = GetModuleFileName((HINSTANCE)hModule, szModulePath, sizeof(szModulePath));
	if (dwResult == 0)
		return FALSE;

	_tsplitpath( szModulePath, drive, dir, fname, ext );
	if (!GetDebugLogPath(TREND_ESE_LIBRARY_NAME, g_szLogFileName, g_szOldLogFileName, g_szExceptLogFileName, g_szOldExceptLogFileName))
	{
		_tmakepath( g_szLogFileName, drive, dir, TREND_ESE_LIBRARY_NAME, _T(".dbg") );
		wsprintf(g_szOldLogFileName, _T("%s%s"), g_szLogFileName, _T(".old"));
		_tmakepath( g_szExceptLogFileName, drive, dir, TREND_ESE_LIBRARY_NAME, _T("ex.dbg") );
		wsprintf(g_szOldExceptLogFileName, _T("%s%s"), g_szExceptLogFileName, _T(".old"));
	}

	wsprintf(g_szThisModule, _T("%s;"), TREND_ESE_LIBRARY_NAME);
	_tcsupr( g_szThisModule );

	SmxDebugGetRegistrySetting();
	g_bInitSuccess = TRUE;
	g_hFile=INVALID_HANDLE_VALUE;
	g_hExFile=INVALID_HANDLE_VALUE;
	g_fileSize=0;
	g_exFileSize=0;
	return TRUE;
}

BOOL SmxDebugGetRegistrySetting()
{
	BOOL bResult = FALSE;

	HKEY hKey;
	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, SMX30_DEBUG_KEY, 0, KEY_QUERY_VALUE, &hKey) == ERROR_SUCCESS)
	{
		DWORD dwSize;
		DWORD dwType;
		DWORD dwValue;

		TCHAR szValue[1024];
		dwSize = sizeof(szValue);
		dwType = REG_SZ;
		if (RegQueryValueEx(hKey, _T("ModuleName"), 0, &dwType, (BYTE*)szValue, &dwSize) == ERROR_SUCCESS)
		{
			_tcsupr( szValue );
			// looking for module name from registry
			if (_tcsstr(szValue, g_szThisModule) != NULL)
				InterlockedExchange((LPLONG)&g_bDebugEnable, TRUE);
			else
				InterlockedExchange((LPLONG)&g_bDebugEnable, FALSE);
		}

		dwSize = sizeof(DWORD);
		dwType = REG_DWORD;
		if (RegQueryValueEx(hKey, _T("DebugLevel"), 0, &dwType, (LPBYTE)&dwValue, &dwSize) == ERROR_SUCCESS)
			InterlockedExchange((LPLONG)&g_nLogLevel, dwValue);

		if (RegQueryValueEx(hKey, _T("Category"), 0, &dwType, (LPBYTE)&dwValue, &dwSize) == ERROR_SUCCESS)
			InterlockedExchange((LPLONG)&g_nCategory, dwValue);

		if (RegQueryValueEx(hKey, _T("ESELogLevel"), 0, &dwType, (LPBYTE)&dwValue, &dwSize) == ERROR_SUCCESS)
			InterlockedExchange((LPLONG)&g_ESELogLevel, dwValue);

		RegCloseKey(hKey);
		bResult = TRUE;
	}

	return bResult;
}

void SmxDebugTerm()
{
	g_bInitSuccess = FALSE;
	if (g_hFile!=INVALID_HANDLE_VALUE) CloseHandle(g_hFile);
	if (g_hExFile!=INVALID_HANDLE_VALUE) CloseHandle(g_hExFile);
}

const UINT CATEGORY[]=
{
	SMXESE_DBG_GENERIC,
	SMXESE_DBG_ESEAPI,
	SMXESE_DBG_SOCKET,
	SMXESE_DBG_SESSION,
	SMXESE_DBG_LIST,
	SMXESE_ASSERT,
	SMXESE_EXCEPTION,
};

LPCTSTR TAG[30]=
{
	_T("GENERIC"),
	_T("ESEAPI"),
	_T("Socket"),
	_T("Session"),
	_T("DataList"),
	_T("*** ASSERTION FAILURE ***"),
	_T("*** EXCEPTION FAILURE ***")
};

LPCTSTR NONE=_T("NONE");
const int sizeOfBuffer=2048;

BOOL SmxDebugEx30(UINT nLevel, UINT nCategory, TCHAR* szFormat, ...)
{
	Guard<Lock> guard(g_lock);
	HANDLE* pFile=(SMXESE_DBG_SEVERE_ERROR(nCategory)?&g_hExFile:&g_hFile);
	DWORD* pSize=(SMXESE_DBG_SEVERE_ERROR(nCategory)?&g_exFileSize:&g_fileSize);
	if (*pFile==INVALID_HANDLE_VALUE){
		LPCTSTR log=(SMXESE_DBG_SEVERE_ERROR(nCategory)?g_szExceptLogFileName:g_szLogFileName);	
		*pFile = CreateFile(log, GENERIC_WRITE, FILE_SHARE_READ, NULL, OPEN_ALWAYS,
							FILE_ATTRIBUTE_NORMAL, NULL);
		if (*pFile==INVALID_HANDLE_VALUE){ 
			*pFile = CreateFile(log, GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS,
								FILE_ATTRIBUTE_NORMAL, NULL);
			if (*pFile==INVALID_HANDLE_VALUE) return  FALSE;
			*pSize=0;
		}
		else if ((*pSize=SetFilePointer(*pFile, 0, 0, FILE_END))==0xffffffff)
			return FALSE;
	}
	if (*pSize > 500 * 1024){
		CloseHandle(*pFile);
		LPCTSTR log=(SMXESE_DBG_SEVERE_ERROR(nCategory)?g_szExceptLogFileName:g_szLogFileName);;
		LPCTSTR old=(SMXESE_DBG_SEVERE_ERROR(nCategory)?g_szOldExceptLogFileName:g_szOldLogFileName);
		MoveFileEx(log, old, MOVEFILE_REPLACE_EXISTING);
		*pFile = CreateFile(log, GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS,
							FILE_ATTRIBUTE_NORMAL, NULL);                
		*pSize=0;
	}	
	if (*pFile==INVALID_HANDLE_VALUE) return  FALSE;
	TCHAR szBuf[sizeOfBuffer];
	va_list args;
	va_start(args, szFormat);
	_vsntprintf(szBuf, sizeOfBuffer, szFormat, args);
	va_end(args);
	LPCTSTR tag=NONE;
	for (int i=0; i < sizeof(CATEGORY)/sizeof(CATEGORY[0]); i++)
		if (nCategory & CATEGORY[i]){
			tag=TAG[i];
			break;
		}
	SYSTEMTIME st;
	GetLocalTime(&st);
	TCHAR szLog[sizeOfBuffer+100];
	wsprintf(szLog, _T("%d,[%s]-[%2d-%02d %02d:%02d:%02d]-[P%d:T%d] %s\r\n"), 
			nLevel, tag, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond, 
			GetCurrentProcessId(), GetCurrentThreadId(), szBuf);
    char szBuffer[sizeOfBuffer+101];
	szBuffer[sizeOfBuffer+100]=0;
    WideCharToMultiByte(CP_ACP, 0, szLog, -1, szBuffer, sizeOfBuffer+100, NULL, NULL);
    DWORD dwWritten;
    BOOL ret=WriteFile(*pFile, szBuffer, strlen(szBuffer), &dwWritten, NULL);
	*pSize+=dwWritten;
	return ret;
}
