// SMLDoc.h: interface for the CSMLDoc class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SMLDOC_H__198B6DF4_8CCC_445E_AEAD_82674B31A7F6__INCLUDED_)
#define AFX_SMLDOC_H__198B6DF4_8CCC_445E_AEAD_82674B31A7F6__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define COLOR_HOTLIGHT					26
#define COLOR_GRADIENTACTIVECAPTION     27
#define COLOR_GRADIENTINACTIVECAPTION   28

#define JUSTIFY_CENTER	0x0001	//!<	Specifies center justification
#define JUSTIFY_RIGHT	0x0002	//!<	Specifies right justification
#define JUSTIFY_LEFT	0x0004	//!<	Specifies left justification
#define STYLE_BOLD		0x0010	//!<	Specifies text block is bold
#define STYLE_ITALICS	0x0020	//!<	Specifies text block is italics
#define STYLE_UNDERLINE	0x0040	//!<	Specifies text block is underlined

#define JUSTIFY_MASK	(JUSTIFY_CENTER | JUSTIFY_LEFT | JUSTIFY_RIGHT)
#define STYLE_MASK		(STYLE_BOLD | STYLE_ITALICS | STYLE_UNDERLINE)

class CSMLDoc {
private:
	struct CSMLTagOption {
		CSMLTagOption(CString &param, CString &value);

		CString		m_Param;
		CString		m_Value;
	};

	class CSMLTagOptionList : public CList<CSMLTagOption*, CSMLTagOption*> {
	public:
		virtual ~CSMLTagOptionList();
		void Reset();
		CString *GetParam(LPCSTR param);
		CString GetParamTypeface(CString &defaultFace);
		DWORD GetParamSize(DWORD defaultSize);
		DWORD GetParamColor(DWORD defaultColor);
		DWORD GetBkgndColor(DWORD defaultColor);
	};

	class CSMLFontInfo {
	public:
		CSMLFontInfo();
		CSMLFontInfo(CSMLFontInfo *info);
		CSMLFontInfo(DWORD size, DWORD style, COLORREF color, CString &face);
		CSMLFontInfo(DWORD size, DWORD style, COLORREF color, LPCSTR face);

		DWORD		m_Size;
		DWORD		m_Style;
		COLORREF	m_Color;
		CString		m_Face;
	};

public:
	typedef CList<CSMLFontInfo*, CSMLFontInfo*> CSMLFontInfoList;

	class CSMLTextGroup {

	public:
		class CSMLTextBlock {
		public:
			CSMLTextBlock(LPCSTR text, CSMLFontInfo *info);
			CString &GetTypeface();
			DWORD GetPointSize();
			DWORD GetStyle();
			COLORREF GetColor();
			CString &GetText();

		private:
			CString		m_Text;			//!<	CString object containing the text for the block.
			DWORD		m_Style;		//!<	Formatting styles for the text block.
			DWORD		m_PointSize;	//!<	Point size for the font that will be applied to the text block.
			COLORREF	m_Color;		//!<	Text Color.
			CString		m_Typeface;		//!<	Typeface for the text block.
		};

	private:
		CArray<CSMLTextBlock*, CSMLTextBlock*>	m_Blocks;	//!<	Array of text blocks belonging to the group.

	public:
		CSMLTextGroup();
		~CSMLTextGroup();
		void Add(CSMLTextBlock *block);
		void Empty(BOOL bDelete);
		int GetBlockCount();
		CSMLTextBlock *GetBlock(int index);
		void operator=(CSMLTextGroup &line);
	};

private:
	CArray<CSMLTextGroup*, CSMLTextGroup*>	m_Blocks;	//!< Array of text blocks.
	COLORREF	m_BkgndColor;	//!<	Background color of the document.
	CString		m_PageTypeface;	//!<	Global typeface of the document.

	LPCSTR ParseTag(LPCSTR src, CString &tagText, DWORD &tagID, CSMLTagOptionList &paramList);
	void AddGroup(CSMLTextGroup &line);
	void PurgeBlocks();

public:
	~CSMLDoc();
	int GetGroupCount();
	CSMLTextGroup *GetTextGroup(int index);
	BOOL ParseSMLText(LPCSTR stream);
	BOOL ParseSMLText(UINT nID);
	COLORREF GetBkgndColor();
	CString &GetTypeface();
};

#endif // !defined(AFX_SMLDOC_H__198B6DF4_8CCC_445E_AEAD_82674B31A7F6__INCLUDED_)

