// TrackerReport.cpp : Defines the class behaviors for the application.
//
// This is a part of the Microsoft Foundation Classes C++ library.
// Copyright (C) 1992-1998 Microsoft Corporation
// All rights reserved.
//
// This source code is only intended as a supplement to the
// Microsoft Foundation Classes Reference and related
// electronic documentation provided with the library.
// See these sources for detailed information regarding the
// Microsoft Foundation Classes product.

#include "stdafx.h"
#include "TrackerReport.h"
#include "OptionListBox.h"
#include "OptionPageStaticText.h"
#include "PageOnDemandReport.h"
#include "Resource.h"
#include <process.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTrackerReportApp

BEGIN_MESSAGE_MAP(CTrackerReportApp, CWinApp)
	//{{AFX_MSG_MAP(CTrackerReportApp)
	//}}AFX_MSG_MAP
	ON_COMMAND(ID_HELP, CWinApp::OnHelp)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTrackerReportApp construction

CTrackerReportApp::CTrackerReportApp()
{
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CTrackerReportApp object

CTrackerReportApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CTrackerReportApp initialization
#include <afxsock.h>		// MFC socket extensions
#include  <io.h>
#include "pccipc.h"

BOOL g_bExit = FALSE;

void CTrackerReportApp::BUFFER_PRINT(char* lpszBuffer, int cbBuffer)
{	
	if (lpszBuffer == NULL)
	{
		return;
	}

	char	szMsg[2048];
	char	szTemp[512];

	ULONG	ulLine, ulLast, i, j;
	LPBYTE  p;

	ulLine = cbBuffer >> 4;
	ulLast = cbBuffer & 0xF;

	for (i = 0; i < ulLine; i++)
	{
		szMsg[0] = _T('\0');
		wsprintf(szTemp, _T("%03X0  "), i);
		lstrcat(szMsg, szTemp);

		p = (LPBYTE)lpszBuffer + (i << 4);
		wsprintf(szTemp,
				_T("%02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X     "),
				*p,*(p+1),*(p+2),*(p+3),*(p+4),*(p+5),*(p+6),
				*(p+7),*(p+8),*(p+9),*(p+10),*(p+11),*(p+12),
				*(p+13),*(p+14),*(p+15));
		lstrcat(szMsg, szTemp);

		for(j = 0; j < 16; j++) 
		{
			if (*(p+j) < 0x20)
				wsprintf(szTemp, _T("."));
			else
				wsprintf(szTemp, _T("%c"), *(p+j));
			lstrcat(szMsg, szTemp);
		}

		DB(szMsg);
	}

	if(ulLast > 0) 
	{
		szMsg[0] = _T('\0');
		wsprintf(szTemp, _T("%03X0  "), i);
		lstrcat(szMsg, szTemp);

		p = (LPBYTE)lpszBuffer + (i << 4);
		for (j = 0; j < ulLast; j++)
		{
			wsprintf(szTemp, _T("%02X "),*(p+j));
			lstrcat(szMsg, szTemp);
		}
		lstrcat(szMsg, _T("    "));

		for(j = ulLast; j < 16; j++)
		{
			wsprintf(szTemp, _T("   "),*(p+j));
			lstrcat(szMsg, szTemp);
		}

		for (j = 0; j < ulLast; j++)
		{
			if (*(p+j) < 0x20)
				wsprintf(szTemp, _T("."));
			else
				wsprintf(szTemp, _T("%c"), *(p+j));
			lstrcat(szMsg, szTemp);
		}
	
		DB(szMsg);
	}
}
int CTrackerReportApp::DB(char *lpszFormat, ...)
{	
	FILE *fp;
	char sBuf[600];
	va_list args;
	va_start(args, lpszFormat);
	vsprintf(sBuf, lpszFormat, args);
	strcat(sBuf,"\r\n");

	char szPath[MAX_PATH];

	GetAppPath(szPath, MAX_PATH, NULL);

	strcat(szPath, "TMSS-PCC.dbg");

	SYSTEMTIME st;
	GetLocalTime(&st);
	char szLog[700];
	wsprintf(szLog, _T("[%2d-%02d %02d:%02d:%02d]- %s\r\n"), 
			st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond, 
			sBuf);

	if (_access(szPath, 0) == 0) {
		// file exist, append message
		fp = fopen(szPath, "a+");
		if (fp) {

			fprintf(fp, "%s", szLog);
			fclose(fp);
		}
	}	
	else {
		// not exist write message
		fp = fopen(szPath, "w");
		if (fp) {
			fprintf(fp, "%s", szLog);
			fclose(fp);
		}
	}
	fflush(fp);
	return TRUE;
}

void CTrackerReportApp::SetStatusMsg(CString szMsg)
{
	COptionSheet *pWnd = (COptionSheet *)AfxGetMainWnd();
	
	while (pWnd == NULL)
	{
		pWnd = (COptionSheet *)AfxGetMainWnd();
	}

	DB((char *)(LPCTSTR)szMsg);

	CWnd *pPage = pWnd->GetPage(1);

	if (pPage)
	{
		if (IsWindow(pPage->GetSafeHwnd()))
		{
			CWnd *pDlg = pPage->GetDlgItem(IDC_STATUS_MSG);
			
			if (pDlg)
			{
				pDlg->SetWindowText(szMsg);					
			}
		}
	}
	
	//->SetWindowText(szMsg);
}

BOOL CTrackerReportApp::SplitFileName(LPCTSTR lpszPath, LPTSTR lpszDir, LPTSTR lpszFile)
{
	TCHAR 	szName[MAX_PATH];
	TCHAR *pPos;
	
	_tcscpy(szName, lpszPath);
	pPos = _tcsrchr(szName, _T('\\'));
	if (!pPos)	{
		if (lpszFile)
			_tcscpy(lpszFile, szName);
		return FALSE;
	}
	if (lpszFile)
		_tcscpy(lpszFile, (pPos + 1));
	*(pPos + 1) = _T('\0');
	if (lpszDir)
		_tcscpy(lpszDir, szName);

	return TRUE;
}

BOOL CTrackerReportApp::GetAppPath(LPTSTR szPath, DWORD nSize, HINSTANCE hInst)
{
	if (GetModuleFileName(hInst, szPath, nSize) == 0)
		return FALSE;

	return SplitFileName(szPath, szPath, NULL);
}


void CTrackerReportApp::PrepareDRM(char *szBuff, int nLen)
{
	SetStatusMsg("preparing DRM");

	char szPath[MAX_PATH];

	GetAppPath(szPath, MAX_PATH, NULL);

	char szFileName [] = "DRM_DATA.bin";

	strcat(szPath, szFileName);

	OFSTRUCT    ofStruct;	

	SetStatusMsg("Openning DRM buffer file");

	HFILE hFile = OpenFile(szPath, &ofStruct, OF_READ);

	if (hFile == HFILE_ERROR)
	{
		CString szMsg;
		
		szMsg.Format("Fail to open DRM_DATA.bin, please place it under %s", szPath);
		AfxMessageBox(szMsg);
		return;
	}

	DWORD dwBytesRead = 0;
	if (!ReadFile((HANDLE)hFile, szBuff, nLen,  &dwBytesRead, NULL))
	{
		return;
	}

/*	DP_MSG_HEADER *pHeader = (DP_MSG_HEADER *)szBuff;
	
	DP_RESPONSE_MSG_COMP *pRespMsgComp = (DP_RESPONSE_MSG_COMP *)(szBuff + sizeof(DP_MSG_HEADER));
	DP_COMP_INFO *pCompInfo = (DP_COMP_INFO *)(szBuff + sizeof(DP_MSG_HEADER) + sizeof(DP_RESPONSE_MSG_COMP));
*/

	CloseHandle((HANDLE)hFile);
}

int CTrackerReportApp::ParseDRM(char *szBuff)
{
	DP_MSG_HEADER *pHeader = (DP_MSG_HEADER *)szBuff;

 	/*printf("+++++++++++++++++++++++++\n");
	printf("+ PCC Major Version is %s\n", pHeader->szMajorVersion);	
	printf("+ PCC Minor Version is %s\n", pHeader->szMinorVersion);	
	printf("+ PCC Command Type is %s\n", pHeader->szCommandType);		
	
	DP_RESPONSE_MSG_COMP *pRespMsgComp = (DP_RESPONSE_MSG_COMP *)(szBuff + sizeof(DP_MSG_HEADER));
	
	printf("+ ProductVersion = %s\n", pRespMsgComp->szProductVer);
	
	printf("+++++++++++++++++++++++++\n");

  */

	CString szCommand = "Unknown";

	if ((pHeader->szCommandType[0] == '1') && (pHeader->szCommandType[1] == '2'))
	{
		/* DRM */
		szCommand = "Discovery Response Message";
	}
	else if ((pHeader->szCommandType[0] == '1') && (pHeader->szCommandType[1] == '1'))
	{
		/* DRM */
		szCommand = "Discovery Query Message";
	}


	CString szMsg;
	szMsg.Format("Command Type is %s", szCommand);

	SetStatusMsg(szMsg);
	
	/* no use for COMP_INFO */
	/*DP_COMP_INFO *pCompInfo = (DP_COMP_INFO *)(szBuff + sizeof(DP_MSG_HEADER) + sizeof(DP_RESPONSE_MSG_COMP));*/
		
	return 0;	
}
unsigned __stdcall ThreadWaitFunc( void* pArguments ) 
{		
	char Buffer[256];
    char *interface2 = NULL;
    unsigned short port=40116;
    int retval;
    int fromlen;
    struct sockaddr_in local, from;
    WSADATA wsaData;
    SOCKET listen_socket, msgsock;

    /* Parse arguments */
    int socket_type = SOCK_DGRAM;           
	
	CTrackerReportApp *pApp = (CTrackerReportApp *)pArguments;

    if ((retval = WSAStartup(0x202,&wsaData)) != 0) 
	{
        fprintf(stderr,"WSAStartup failed with error %d\n",retval);
        WSACleanup();
        return -1;
    }    

    local.sin_family = AF_INET;
    local.sin_addr.s_addr = INADDR_ANY; 
    
    local.sin_port = htons(port);

    listen_socket = socket(AF_INET, socket_type,0); 
    
    if (listen_socket == INVALID_SOCKET){
        fprintf(stderr,"socket() failed with error %d\n",WSAGetLastError());
        WSACleanup();
        return -1;
    }

    if (bind(listen_socket,(struct sockaddr*)&local,sizeof(local) ) 
        == SOCKET_ERROR) {
        fprintf(stderr,"bind() failed with error %d\n",WSAGetLastError());
        WSACleanup();
        return -1;
    }

    // We cannot listen() on a UDP socket.
    while(!g_bExit) 
	{
        fromlen =sizeof(from);
         msgsock = listen_socket;

		int retval = recvfrom(msgsock,Buffer,sizeof (Buffer),0,
            (struct sockaddr *)&from,&fromlen);

		CString szMsg;
		szMsg.Format("Received data from TMSS, data size is %d", retval);

		pApp->SetStatusMsg(szMsg);

		//pApp->BUFFER_PRINT(Buffer, retval);

		pApp->ParseDRM(Buffer);

		/* need to compose DRM and send it back here */
		int nBuffSize = sizeof(DP_MSG_HEADER) + sizeof(DP_RESPONSE_MSG_COMP) + sizeof(DP_COMP_INFO);
	
	    char *szBuff = new char[nBuffSize];
		memset(szBuff, 0, nBuffSize);

		pApp->PrepareDRM(szBuff, nBuffSize);

		pApp->SetStatusMsg("Send DRM back to TMSS");

	    retval = sendto(msgsock, szBuff,sizeof (Buffer),0,
                (struct sockaddr *)&from,fromlen);

        if (retval == SOCKET_ERROR) 
		{
            fprintf(stderr,"send() failed: error %d\n",WSAGetLastError());
        }

		delete [] szBuff;

		printf("UDP server looping back for more requests\n");        
    }

	return 0;
}

BOOL CTrackerReportApp::InitInstance()
{
	AfxEnableControlContainer();

	HANDLE hMutex = NULL;
	hMutex = CreateMutex(NULL, TRUE, _T("PCCClientSimulationIsRunning"));

	if (GetLastError() == ERROR_ALREADY_EXISTS) 
	{
		return FALSE;
	}

	// Standard initialization
	COptionSheet			optionSheet(IDS_TRACKERTOOL);
	COptionListBox			listBox;
	COptionPageStaticText	pageTrackerToolGroup(IDS_INTRO, IDR_INTRODOC);

	CPageOnDemandReport		pageOnDemandReport;

	m_pMainWnd = &optionSheet;

	optionSheet.SetListControl(&listBox);
	optionSheet.AddGroup(&pageTrackerToolGroup);
	
	optionSheet.AddPage(&pageOnDemandReport, &pageTrackerToolGroup);

    unsigned threadID = 0;

    HANDLE hThread = (HANDLE)_beginthreadex( NULL, 0, &ThreadWaitFunc, this, 0,
        &threadID );

	optionSheet.DoModal();

	g_bExit = TRUE;

	ReleaseMutex(hMutex);

	return FALSE;
}