/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/** 
 *  @file tmdpClientInfoWrapper.c
 *  @brief TMOG DPComposer Module
 */

/*
 *  Change Log
 *  2004-02-23 Jim Wu    Initial Version
 */

#include <string.h>
#include <stdlib.h>
#include "tmdpClientInfoWrapper.h"  /* for updating client info table */
#include "log.h"
#include "tmGetAUInfo.h"

static const char ModuleName[]="updtclntinfo";

int tmdp_UpdateClientInfo(unsigned short nStatus, unsigned int nIP,
                          char szAUProductId, char *szProductVer, char szEngineId,
                          char *szEngineVer, char *szPatternNum, int nAuLangID, 
                          unsigned char *szMac, char *szHostName, unsigned int nLicense)
{
	
    /* update client information table */	            
    struct client_id clientId;
    struct client_av clientAV;
    TmuAUVersion_t  g_TmuVer;  /* structure to hold the AU information, from shared memory */
    
    LOGDEBUG("Entering tmdp_UpdateClientInfo");
    
    /* always translate MAC from IP. */    
    LOGDEBUG("convert IP to MAC");
    
    if (IP_MAC(&nIP, szMac) != 0)
    {
        LOGERROR("tmdp_UpdateClientInfo(), ARP table lookup fail for client %u.%u.%u.%u", NIPQUAD(nIP));
        return -1;
    }

    memset(&clientId, 0, sizeof(struct client_id));
    memset(&clientAV, 0, sizeof(struct client_av));
    
    clientId.ip = nIP;
       
    memcpy(clientId.mac, szMac, MAC_SIZE);             
    strcpy(clientId.hostname, szHostName);   

    clientAV.status = nStatus;    
    clientAV.license = nLicense;
    
    LOGDEBUG("nStatus = %d", nStatus);

    if (nStatus == STATUS_AVOK)
    {
       /* need to fill in the product information only when PCC installed */      
       LOGDEBUG("<Update Client Info> AUProductId is %x", szAUProductId);
       
       LOGDEBUG("<Update Client Info> szEngineId is %x", szEngineId);
   
       clientAV.auversion.product = szAUProductId;        
       clientAV.auversion.pattern.version = atoi(szPatternNum);        
       
       clientAV.auversion.engine.version = tmau_MakeDWordVersion(szEngineVer);
       
       /* call the query AU information routine in libtmGetAUInfo.a */
       tmau_GetNewestAUInfo(&g_TmuVer);

       
       if ((g_TmuVer.engine.version > clientAV.auversion.engine.version) || 
           (g_TmuVer.pattern.version > clientAV.auversion.pattern.version))
       {
           /* out of date */
           clientAV.status = STATUS_AVOLD;           
       }       
       
       clientAV.szAUProductId[0] = szAUProductId;
       strncpy(clientAV.szProductVer, szProductVer, 16);
       clientAV.szEngineId[0] =  szEngineId;  
       clientAV.auLangID = nAuLangID;            
    }
    
    LOGDEBUG("before calling tmAddClient()");

    if (tmci_SetAV(&clientId, &clientAV) != 0)
    {
        LOGDEBUG("fail to call tmci_SetAV");
        return -1;
    }
   
    LOGDEBUG("after calling  tmci_SetAV");

    return 0;
}

int tmdp_GetClientInfo(unsigned int nIP, char *szAUProductId, 
                  char *szProductVer, char *szEngineId, char *szAULangID)
{	
    ClientInfo info;    
    int ret = 0;
    unsigned char macAddr[MAC_SIZE];
    
    LOGDEBUG("Entering tmdp_GetClientInfo");
    
    memset(macAddr, 0, MAC_SIZE);
  
    if (IP_MAC(&nIP, macAddr) != 0)
    {
        LOGERROR("tmdp_GetClientInfo(), ARP table lookup fail for client %u.%u.%u.%u", NIPQUAD(nIP));
        return -1;
    }
  
    if ((ret = tmci_GetClient(macAddr, &info)) < 0)
    {	
        LOGERROR("calling tmGetClient fail, ret = %d", ret);	
        return -1;
    }
    
    LOGINFO("calling tmGetClient OK, ret = %d", ret);	

    /*szAUProductId[3] = 83;  for PCC 11*/
    szAUProductId[3] = info.avinfo.szAUProductId[0];        	
	
	LOGINFO("szAUProductId = %d, %d, %d, %d", szAUProductId[0], szAUProductId[1], szAUProductId[2],
	        szAUProductId[3]);	
	
	szAULangID[3] = info.avinfo.auLangID; 	/* 1-en,  2-tw, 4-jp */		
	
	LOGINFO("szAULangID = %d, %d, %d, %d", szAULangID[0], szAULangID[1], szAULangID[2],
	        szAULangID[3]);		        	
    
    strncpy(szProductVer,  info.avinfo.szProductVer, 16);
    
    LOGINFO("szProductVer = %s", szProductVer);
    
    /*szEngineId[3] = 0x10 for PCC 11;*/    
    szEngineId[3] = info.avinfo.szEngineId[0];    
    
    LOGINFO("szEngineId = %d, %d, %d, %d", szEngineId[0], szEngineId[1], szEngineId[2],
	        szEngineId[3]);		        
    
    LOGDEBUG("after calling tmgetClient()");
    
    return 0;
}
