/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/** 
 *  @file tmdpMsgComposer.c
 *  @brief TMOG DPComposer Module
 */

/*
 *  Change Log
 *  2004-02-23 Jim Wu    Initial Version
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "tmdpPCCProtocol.h"         /* for PCC communicatio protocol header */
#include "tmdpClientInfoWrapper.h"   /* for updating client info table */
#include "log.h"
#include "tmClientInfo.h"
#include "tmdpMsgComposer.h"        /* header file for this Msg Composer*/
#include "tmGetHardwareInfo.h"      /* hardware dependent information */

static const char ModuleName[]="dpcomposer";

static void GetBoxId(char *szBoxId)
{
   char szTempBoxId[17];
   
   LOGINFO("Entering GetBoxId");
   
   if (tmGetBoxID(szTempBoxId) != -1)
   {   
      LOGINFO("tmGetBoxID succeed, szTempBoxId is %s", szTempBoxId);
      strncpy(szBoxId, szTempBoxId, 16);	
   }
   else
   {
      LOGERROR("tmGetBoxID failed");
      strcpy(szBoxId, "");
   }
   
   LOGINFO("Leaving GetBoxId, szBoxId is %s", szBoxId);
}

static void GetVendorId(char *szVendorId)
{
   /* get vendor Id  */
   char szTempVendorId[17];
   
   if (tmGetVendorID(szTempVendorId) != -1)
   {   
      strncpy(szVendorId, szTempVendorId, 16);	
   }
   else
   {
      strcpy(szVendorId, "");
   }
}

static int GetBitSize(int nSize, char *szLength)
{	    
    szLength[0] = nSize / 256;
    szLength[1] = nSize % 256;    
    
    return 0;
}

int tmdpx_GetPCCDRMSize(void)
{
    /* discovery response from PCC */
    int nSize = sizeof(DP_MSG_HEADER) + sizeof(DP_PCC_RESPONSE_MSG_COMP);   
    
    return nSize;
}

int tmdpx_GetDRMSize(void)
{
    /* 32 for component info */
    /* send Notification to PCC */
    int nSize = sizeof(DP_MSG_HEADER) + sizeof(DP_RESPONSE_MSG_COMP) - 32;   
    
    return nSize;
}

int tmdpx_GetDQMSize(void)
{
    int nSize = sizeof(DP_MSG_HEADER) + sizeof(DP_QUERY_MSG_COMP);
    return nSize;
}

int tmdpx_ParseDQM(char *szBuff)
{
    DP_MSG_HEADER *pHeader;
    DP_QUERY_MSG_COMP *pQueryMsgComp;
    
    /* used when PCC try to get box id from TMSS, parse DQM from PCC */
    LOGDEBUG("PCC is querying Box Id from TMSS\n");
	
    if (!szBuff)
    {
       /* null buffer */
       LOGWARNING("NULL buffer");
       return -1;
    }
	
    pHeader = (DP_MSG_HEADER *)szBuff;    
    
    if (strncmp(pHeader->szMagicNumber, TMDP_MagicNumber, strlen(TMDP_MagicNumber)) != 0)
    {
       /* magic number is wrong */
       LOGWARNING("Magic number not correct, %s", pHeader->szMagicNumber);
       return -1;   
    }
    
    if (pHeader->szCommandType[1] != 0x11)
    {
	   /* not DQM */
	   LOGWARNING("Not Correct DQM command type\n");
	   return -1;
    }
	
    pQueryMsgComp = (DP_QUERY_MSG_COMP *)(szBuff + sizeof(DP_MSG_HEADER));
	
    if (!pQueryMsgComp)
    {
       /* null buffer */
       LOGWARNING("NULL component buffer");
       return -1;
    }
    
    if ((pQueryMsgComp->szID[0] != 0x00) || 
        (pQueryMsgComp->szID[1] != 0x02))      /* TMDP_DQM = 0x0002 */ 
        {   
          LOGWARNING("TMDP_component id not correct");
          return -1;             
	    }
	
    /*	
    if ((pQueryMsgComp->szPlatformId[0] != 0xff) ||
        (pQueryMsgComp->szPlatformId[1] != 0xff) ||
        (pQueryMsgComp->szPlatformId[2] != 0xff) ||
        (pQueryMsgComp->szPlatformId[3] != 0xff))
    {
       platform id = 0x00000001 for windows 0x00000002 for TMSS, 0xffffff for unknown 
       LOGWARNING("Not Correct DQM platform\n");
       return -1;
    } 	
	*/
    LOGINFO("Correct DQM\n");	
		
    return 0;	
}

int tmdpx_ParsePCCDRM(char *szBuff, unsigned int nIP)
{
    /* used when TMSS got the information from PCC, parse DRM from PCC */
    DP_MSG_HEADER *pHeader;
    DP_PCC_RESPONSE_MSG_COMP *pRespMsgComp;
    int nAULangID = 1;
    int nLicense = LICENSE_OK;
    int nMajor = 0;
    int nMinor = 0;    
    unsigned char szMacAddr[MAC_SIZE];
    char szHostName[32] = "";
    int nIdx = 0;
	
	LOGDEBUG("<Parse PCC DRM> Parsing DRM...");
	
    if (!szBuff)
    {
	   /* null buffer */
	   LOGERROR("<Parse PCC DRM> Parsing DRM, null buffer");
	   return -1;
    }
    
    LOGINFO("<Parse PCC DRM> Parsing DRM, NOT null buffer");
	
    pHeader = (DP_MSG_HEADER *)szBuff;

    /* 0x11 for DQM, 0x12 for DRM */
    if (pHeader->szCommandType[1] != 0x12)
    {
       /* not DRM */
       LOGWARNING("<Parse PCC DRM> Not Correct DRM command type\n");
       return -1;
    }	
    
    LOGINFO("<Parse PCC DRM> Parsing DRM, Command is OK");
		
    pRespMsgComp = (DP_PCC_RESPONSE_MSG_COMP *)(szBuff + sizeof(DP_MSG_HEADER));
	
    if (!pRespMsgComp)
    {
        /* null buffer */
        LOGDEBUG("<Parse PCC DRM> Parsing DRM..., NO Component");
        
        return -1;
    }
    
    LOGINFO("<Parse PCC DRM> Parsing DRM, Component is OK, platform ID is %d", pRespMsgComp->szPlatformId[3]);
	
    if (pRespMsgComp->szPlatformId[3] != 0x1)
    {
         /* platform id = 0x00000001 for windows 0x00000002 for TMSS */
        LOGWARNING("<Parse PCC DRM> Not Correct windows platform\n");
        return -1;
    } 		
    
    if (pRespMsgComp->szPlatformInfo[1] == 0x1)
	{
		LOGINFO("+ Windows 9x platform"); 
	}
	else if (pRespMsgComp->szPlatformInfo[1] == 0x2)
	{
		LOGINFO("+ Windows NT platform"); 
	}

    nMajor = pRespMsgComp->szPlatformInfo[3]; /* windows major version */
    
	nMinor = pRespMsgComp->szPlatformInfo[5]; /* windows minor version */    
	
	LOGINFO("+ Windows Major Version is %d, Minor Version is %d", nMajor, nMinor);
	
	/* LOGINFO("Parsing DRM, get au lang id"); */
	
    /* DRM is OK, fill in to the client information table */	                     
    
    nAULangID = 1; /*pRespMsgComp->szAULangID[3]; */

    LOGDEBUG("+ license is %x", *(pRespMsgComp->szProductState));
    
    if (*(pRespMsgComp->szProductState) == 0x00)
    {
       nLicense = LICENSE_OK;
    }
    else if (*(pRespMsgComp->szProductState) == 0x11)
    {
       nLicense = LICENSE_EXPIRED;
    }
    else
    {
       nLicense = LICENSE_OK;
    }        
    
    for (nIdx = 0; nIdx < 6; nIdx++)
    {
        szMacAddr[nIdx] = pRespMsgComp->szNodeId[nIdx + 4];  /* first 4 bytes are MAC= */        
    }    
    
    
    strncpy(szHostName, pRespMsgComp->szNodeName, 32);
    
    LOGINFO("+ HostName = %s", szHostName);
    
    LOGDEBUG("+ AUProductId is %d, %d, %d, %d", pRespMsgComp->szAUProductId[0],
              pRespMsgComp->szAUProductId[1], pRespMsgComp->szAUProductId[2], pRespMsgComp->szAUProductId[3]);
    
    LOGDEBUG("+ EngineId is %d, %d, %d, %d", pRespMsgComp->szEngineId[0],
              pRespMsgComp->szEngineId[1], pRespMsgComp->szEngineId[2], pRespMsgComp->szEngineId[3]);    
    
    LOGDEBUG("+ Engine Verson is %s", pRespMsgComp->szEngineVer);    
    
    LOGDEBUG("+ Pattern Number is %s", pRespMsgComp->szPatternNum);    
                   
    tmdp_UpdateClientInfo(STATUS_AVOK, nIP, 
                          pRespMsgComp->szAUProductId[3], 
                          pRespMsgComp->szProductVer, 
                          pRespMsgComp->szEngineId[3],
                          pRespMsgComp->szEngineVer,
                          pRespMsgComp->szPatternNum,                     
                          nAULangID, szMacAddr, szHostName,
                          nLicense);

    LOGDEBUG("Parsing DRM OK., updating client info finished.");

    LOGINFO("+ PCC Major Version is %d\n", pHeader->szMajorVersion[0] + 0);	
    LOGINFO("+ PCC Minor Version is %d\n", pHeader->szMinorVersion[0] + 0);	
    LOGINFO("+ PCC Command Type is %d\n", pHeader->szCommandType[1]);		
    LOGINFO("+ ProductVersion = %s\n", pRespMsgComp->szProductVer);	
    LOGINFO("+ Node Name = %s\n", pRespMsgComp->szNodeName);
    LOGINFO("+ Node Id = %s\n", pRespMsgComp->szNodeId);		    

    return 0;	
}

void tmdpx_PrepareDQM(char *szBuff)
{
    /* preparethe DQM message based on Discovery protocol */	
    /* used when TMSS want the information from PCC, send DQM to PCC */
    DP_QUERY_MSG_COMP *pQueryMsgComp;
    DP_MSG_HEADER *pHeader;    
	
    /* LOGDEBUG("preparing DQM\n");*/
		
    /* prepare for the message header */		
	
    pHeader = (DP_MSG_HEADER *)szBuff;
    memset(pHeader, 0, sizeof(DP_MSG_HEADER));

    strncpy(pHeader->szMagicNumber, TMDP_MagicNumber, 4);  /* magic number, this field can not be modified */
 	
    pHeader->szMajorVersion[0] = 0x1;	/* this field can not be modified */
    pHeader->szMinorVersion[0] = 0x0;	/* this field can not be modified */	
	
   	pHeader->szCommandType[0] =  0x00;  /* 0x0011 for DQM, 0x0012 for DRM */	
	pHeader->szCommandType[1] =  0x11;  /* 0x0011 for DQM, 0x0012 for DRM */		
	
    /* get query component size and fill in the field */    
    GetBitSize(sizeof(DP_QUERY_MSG_COMP), pHeader->szLength);
	
    /* prepare for the message component */	
	
    pQueryMsgComp = (DP_QUERY_MSG_COMP *)(szBuff + sizeof(DP_MSG_HEADER));
	
    memset(pQueryMsgComp, 0, sizeof(DP_QUERY_MSG_COMP));
	
    pQueryMsgComp->szID[0] = 0x00;   /* TMDP_DQM = 0x0002 */ 
	pQueryMsgComp->szID[1] = 0x02;   /* TMDP_DQM = 0x0002 */ 
	
	GetBitSize(sizeof(DP_QUERY_MSG_COMP) -4, pQueryMsgComp->szLength); // not include "id" and "length" field
    	
    /* pQueryMsgComp->szIPAddr[16];   removed */
	
    strncpy(pQueryMsgComp->szNodeName, TMDP_Node_Name, 32);
	
	/* use BoxId to fillin NodeId */
	/* get box id (MAC address + checksum)*/
	GetBoxId(pQueryMsgComp->szNodeId);    
    
    /* for reserved part, does not need to fill anything */
		
    pQueryMsgComp->szPlatformId[0] = (char)0xFF;
    pQueryMsgComp->szPlatformId[1] = (char)0xFF;
    pQueryMsgComp->szPlatformId[2] = (char)0xFF;
    pQueryMsgComp->szPlatformId[3] = (char)0xFF;

	pQueryMsgComp->szProductId[7] = 0x1;         /* 0x1 for PCC, 0x2 for TMSS */		
	
    pQueryMsgComp->szCompID[0] = (char)0xFF;     /* All components */
    pQueryMsgComp->szCompID[1] = (char)0xFF;     
    pQueryMsgComp->szCompID[2] = (char)0xFF;     
    pQueryMsgComp->szCompID[3] = (char)0xFF;     
    pQueryMsgComp->szCompID[4] = (char)0xFF;     
    pQueryMsgComp->szCompID[5] = (char)0xFF;     
    pQueryMsgComp->szCompID[6] = (char)0xFF;     
    pQueryMsgComp->szCompID[7] = (char)0xFF;     	
}

void tmdpx_PrepareDRM(char *szBuff, unsigned int nIP, 
                char *szEngineVer, char *szPatternNum)
{
    /* ....  preparethe DRM message based on Discovery protocol */    
    DP_RESPONSE_MSG_COMP *pRespMsgComp;
		
    /* 1. used when PCC want the Box ID, send DRM back to PCC 
       2. when TMSS notify PCC to update engine and pattern information*/
	
    /* LOGDEBUG("preparing DRM\n"); */
	
    DP_MSG_HEADER *pHeader = (DP_MSG_HEADER *)szBuff;
    memset(pHeader, 0, sizeof(DP_MSG_HEADER));

    strncpy(pHeader->szMagicNumber, TMDP_MagicNumber, 4); /* magic number, this field can not be modified */
 	
    pHeader->szMajorVersion[0] = 0x1;	    /* this field can not be modified */
    pHeader->szMinorVersion[0] = 0x0;		/* this field can not be modified */
	
    pHeader->szCommandType[0] =  0x0;       /* 0x11 for DQM, 0x12 for DRM */	
    pHeader->szCommandType[1] =  0x12;      /* 0x11 for DQM, 0x12 for DRM */		
	
    /* get response component size and fill in the field */
    GetBitSize(sizeof(DP_RESPONSE_MSG_COMP) - 32, pHeader->szLength);	
	
    pRespMsgComp = (DP_RESPONSE_MSG_COMP *)(szBuff + sizeof(DP_MSG_HEADER));
	
    memset(pRespMsgComp, 0, sizeof(DP_RESPONSE_MSG_COMP) - 32);
	
    pRespMsgComp->szID[0] = 0x0;   /* TMDP_DRM = 0x3 */
    pRespMsgComp->szID[1] = 0x3;   /* TMDP_DRM = 0x3 */

    GetBitSize(sizeof(DP_RESPONSE_MSG_COMP) - 4 - 32, pRespMsgComp->szLength);		    
	
    /* pRespMsgComp->szIPAddr[16]; removed */

    strncpy(pRespMsgComp->szNodeName, TMDP_Node_Name, 32);	
		
	/* get box id (MAC address + checksum)*/
	/* use BoxId to fillin NodeId */
	GetBoxId(pRespMsgComp->szNodeId);    
    
    /* for reserved part, does not need to fill anything */
		
    /* TMSS 1.0 does not use P2P, just fill 0x0 */		
    /* pRespMsgComp->szP2PProtocol[0] = 0x0; */
	
    /* TMSS 1.0 does not use P2P, just fill 0x0 */
    /* pRespMsgComp->szP2PPort[0] = 0x0;   */

    /*	Unknown is 0,	Windows is 0x1	Linux is 0x2 */
    /* fill in unknown here, does not need to fill in 
	pRespMsgComp->szPlatformId[0] = (char)0x0;  
	pRespMsgComp->szPlatformId[1] = (char)0x0;
	pRespMsgComp->szPlatformId[2] = (char)0x0;
	pRespMsgComp->szPlatformId[3] = (char)0x0;  */
	
    /* platform info to be defined by IHV   platform[2], majorver[2], minorver[2], reserved[10]
    pRespMsgComp->szPlatformInfo */	
	
    pRespMsgComp->szProductId[7] = 0x2;         /* 0x1 for PCC, 0x2 for TMSS */				
		
    /* The rest 128 bytes are Product Information,*/
    /* get box id (MAC address + checksum)*/    
    GetBoxId(pRespMsgComp->szBoxId);
    
    GetVendorId(pRespMsgComp->szVendorId);  /* need to fill in the vendor ID */	
	
    /* AU ProductID eg: 83 for PCC11 */
    /* product version for PCC, eg: 11.30.1234  */	
    /* engine ID 0x2 VXD for win9x, 0x10 for NT */	
	
	LOGINFO("before GetClientInfo");
	
    tmdp_GetClientInfo(nIP, pRespMsgComp->szAUProductId, 
                  pRespMsgComp->szProductVer,
                  pRespMsgComp->szEngineId,
                  pRespMsgComp->szAULangID);		
                  
    LOGINFO("After GetClientInfo");
    
    LOGINFO("szAUProductId = %d, %d, %d, %d", pRespMsgComp->szAUProductId[0], pRespMsgComp->szAUProductId[1], 
            pRespMsgComp->szAUProductId[2],
	        pRespMsgComp->szAUProductId[3]);		        
	        
    LOGINFO("szEngineId = %d, %d, %d, %d", pRespMsgComp->szEngineId[0], pRespMsgComp->szEngineId[1], 
            pRespMsgComp->szEngineId[2],
	        pRespMsgComp->szEngineId[3]);		        
	        
	LOGINFO("szProductVer = %s", pRespMsgComp->szProductVer);
	
	LOGINFO("szAULangID = %d, %d, %d, %d", pRespMsgComp->szAULangID[0], pRespMsgComp->szAULangID[1], 
	         pRespMsgComp->szAULangID[2], pRespMsgComp->szAULangID[3]);		        	    
	
    strncpy(pRespMsgComp->szEngineVer, szEngineVer, 16);   /* engine version eg: 6.680.1234  */
    strncpy(pRespMsgComp->szPatternNum, szPatternNum, 12); /* pattern number string  */	
	
    /* no components, fill with 0x0, no need to fill here */
	/*pRespMsgComp->szCompID[0] = (char)0x0; 
	pRespMsgComp->szCompID[1] = (char)0x0;     
	pRespMsgComp->szCompID[2] = (char)0x0;     
	pRespMsgComp->szCompID[3] = (char)0x0;     
	pRespMsgComp->szCompID[4] = (char)0x0;     
	pRespMsgComp->szCompID[5] = (char)0x0;     
	pRespMsgComp->szCompID[6] = (char)0x0;     
	pRespMsgComp->szCompID[7] = (char)0x0;     	
  */	
}
