/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @ht_parser.h -- HTTP header parser
 *  @HTTP header parsing
 */

/*
 *  Change Log
 *  2001-08-07 Amos Hsu     Initialization version
 *  2001-08-13 Amos Hsu     reformat comment into robodoc style.
 *  2001-08-24 Amos Hsu     use ht_tok_get() to do "header: value" paring
 *                          for more preciseness.
 *  2001-08-24 Amos Hsu     add ht_parser_reset() function.
 */

#ifndef	_HTTP_PARSER_H_
#define	_HTTP_PARSER_H_

#ifdef	__cplusplus
extern "C"
{
#endif

    /****s* ht_parser/hooktab_t *************************************************
    *                                                                            
    * NAME                                                                       
    *   hooktab_t                                                                
    *                                                                            
    * SYNOPSIS                                                                   
    *   struct hooktab_st;                                                       
    *   typedef struct hooktab_st *  hooktab_t;                                  
    *                                                                            
    * DESCRIPTION                                                                
    *   pointer handle to hook table                                             
    *                                                                            
    ****************************************************************************/
    struct hooktab_st;
    typedef struct hooktab_st *  hooktab_t;

    /****d* ht_parser/FP_HTTP_PRE_PROCESS_HANDLER *******************************
    *                                                                            
    * NAME                                                                       
    *   FP_HTTP_PRE_PROCESS_HANDLER                                              
    *                                                                            
    * SYNOPSIS                                                                   
    *   typedef int (*FP_HTTP_PRE_PROCESS_HANDLER)(                              
    *                           void *pPreArg,                                   
    *                           void *fpMajorCallBack,                           
    *                           void *pMajorArg,                                 
    *                           const char *szHookName,                          
    *                           const char *szValue)                             
    *                                                                            
    * DESCRIPTION                                                                
    *   function prototype of pre-process parser                                 
    *                                                                            
    * PARAMETERS                                                                 
    *   void *pPreArg:          IN : argument for pre-parser                     
    *   void *fpMajorCallBack:  IN : callback function pointer of major parser   
    *   void *pMajorArg:        IN : argument for major parser                   
    *   const char *szHookName: IN : header name                                 
    *   const char *szValue:    IN : value of header                             
    *                                                                            
    * WARNINGS                                                                   
    *   if you want to use your own pre-parser, you MUST follow this prototype   
    *   for your pre-parser!                                                     
    *                                                                            
    * SEE ALSO                                                                   
    *                                                                            
    ****************************************************************************/
    typedef int (*FP_HTTP_PRE_PROCESS_HANDLER)(void *pPreArg, void *fpMajorCallBack, void *pMajorArg, const char *szHookName, const char *szValue);

    /****f* ht_parser/hooktab_open() ********************************************
    *                                                                            
    * NAME                                                                       
    *   hooktab_open()                                                           
    *                                                                            
    * SYNOPSIS                                                                   
    *   hooktab_t  hooktab_open(void)                                            
    *                                                                            
    * DESCRIPTION                                                                
    *   create the handle of hook table.                                         
    *                                                                            
    * PARAMETERS                                                                 
    *   none                                                                     
    *                                                                            
    * WARNINGS                                                                   
    *   the multi-threads safe mechanism for reference count of hook table does  
    *   NOT implement yet!!                                                      
    *                                                                            
    * NOTES                                                                      
    *   none                                                                     
    *                                                                            
    * RETURN VALUE                                                               
    *   hooktab_t   -> handle of hook table                                      
    *                                                                            
    * ERRORS                                                                     
    *   NULL        -> indicate error                                            
    *                                                                            
    * SEE ALSO                                                                   
    *   hooktab_t, hooktab_close(), hooktab_reg()                                
    *                                                                            
    ****************************************************************************/
    extern hooktab_t  hooktab_open(void);

    /****f* ht_parser/hooktab_close() *******************************************
    *                                                                            
    * NAME                                                                       
    *   hooktab_close()                                                          
    *                                                                            
    * SYNOPSIS                                                                   
    *   int hooktab_close(hooktab_t  *pHookTable)                                
    *                                                                            
    * DESCRIPTION                                                                
    *   close the handle of hook table.                                          
    *                                                                            
    * PARAMETERS                                                                 
    *   hooktab_t  *pHookTable: IN : pointer to handle of hook table             
    *                                                                            
    * WARNINGS                                                                   
    *   the multi-threads safe mechanism for reference count of hook table does  
    *   NOT implement yet!!                                                      
    *                                                                            
    * NOTES                                                                      
    *   none                                                                     
    *                                                                            
    * RETURN VALUE                                                               
    *   0   -> close successfully, hooktab_t will set to NULL.                   
    *                                                                            
    * ERRORS                                                                     
    *   < 0 -> indicate error                                                    
    *                                                                            
    * SEE ALSO                                                                   
    *   hooktab_t, hooktab_open(), hooktab_reg()                                 
    *                                                                            
    ****************************************************************************/
    extern int hooktab_close(hooktab_t  *pHookTable);

    /****f* ht_parser/hooktab_reg() *********************************************
    *                                                                            
    * NAME                                                                       
    *   hooktab_reg()                                                            
    *                                                                            
    * SYNOPSIS                                                                   
    *   int hooktab_reg(hooktab_t  hHookTable,                                   
    *                   const char *szHookName,                                  
    *                   FP_HTTP_PRE_PROCESS_HANDLER  fpPreCallBack,              
    *                   void *fpMajorCallBack)                                   
    *                                                                            
    * DESCRIPTION                                                                
    *   register pre-parser and major parser to specific header name you wanna   
    *   hook.                                                                    
    *                                                                            
    * PARAMETERS                                                                 
    *   hooktab_t  *pHookTable: IN : pointer to handle of hook table             
    *                                                                            
    * WARNINGS                                                                   
    *   none                                                                     
    *                                                                            
    * NOTES                                                                      
    *   none                                                                     
    *                                                                            
    * RETURN VALUE                                                               
    *   1   -> the header you wanna register has already been hooked, so just    
    *          replace with new fpPreCallBack and fpMajorCallBack.               
    *   0   -> register successfully.                                            
    *                                                                            
    * ERRORS                                                                     
    *   < 0 -> indicate error                                                    
    *                                                                            
    * SEE ALSO                                                                   
    *   hooktab_t, hooktab_open(), hooktab_close()                               
    *                                                                            
    ****************************************************************************/
    extern int hooktab_reg(hooktab_t  hHookTable, const char *szHookName, FP_HTTP_PRE_PROCESS_HANDLER  fpPreCallBack, void *fpMajorCallBack);

    /****s* ht_parser/parser_t **************************************************
    *                                                                            
    * NAME                                                                       
    *   parser_t                                                                 
    *                                                                            
    * SYNOPSIS                                                                   
    *   struct parser_st;                                                        
    *   typedef struct parser_st *  parser_t;                                    
    *                                                                            
    * DESCRIPTION                                                                
    *   pointer handle to parser                                                 
    *                                                                            
    ****************************************************************************/
    struct parser_st;
    typedef struct parser_st *  parser_t;

    /****f* ht_parser/ht_parser_open() ******************************************
    *                                                                            
    * NAME                                                                       
    *   ht_parser_open()                                                         
    *                                                                            
    * SYNOPSIS                                                                   
    *   parser_t  ht_parser_open(hooktab_t  hHookTable,                          
    *                            void *pPreCallBackPara,                         
    *                            void *pCallBackPara)                            
    *                                                                            
    * DESCRIPTION                                                                
    *   create the handle of parser.                                             
    *                                                                            
    * PARAMETERS                                                                 
    *   hooktab_t  hHookTable:  IN : handler of hook table                       
    *   void *pPreCallBackPara: IN : pre-parser callback parameter               
    *   void *pCallBackPara:    IN : major callback parameter                    
    *                                                                            
    * WARNINGS                                                                   
    *   the multi-threads safe mechanism for reference count of hook table does  
    *   NOT implement yet!!                                                      
    *                                                                            
    * NOTES                                                                      
    *   when you attach hook table to parser, the reference count of hook table  
    *   will increase by 1.                                                      
    *                                                                            
    * RETURN VALUE                                                               
    *   parser_t -> handle of parser                                             
    *                                                                            
    * ERRORS                                                                     
    *   NULL     -> indicate error                                               
    *                                                                            
    * SEE ALSO                                                                   
    *   ht_parser_close(), hooktab_open(), hooktab_t                             
    *                                                                            
    ****************************************************************************/
    extern parser_t  ht_parser_open(hooktab_t  hHookTable, void *pPreCallBackPara, void *pCallBackPara);

    /****f* ht_parser/ht_parser_close() *****************************************
    *                                                                            
    * NAME                                                                       
    *   ht_parser_close()                                                        
    *                                                                            
    * SYNOPSIS                                                                   
    *   int	ht_parser_close(parser_t  *pParser)                                  
    *                                                                            
    * DESCRIPTION                                                                
    *   close the handle of parser.                                              
    *                                                                            
    * PARAMETERS                                                                 
    *   parser_t  *pParser:  IN : pointer to handle of parser                    
    *                                                                            
    * WARNINGS                                                                   
    *   the multi-threads safe mechanism for reference count of hook table does  
    *   NOT implement yet!!                                                      
    *                                                                            
    * NOTES                                                                      
    *   when you attach hook table to parser, the reference count of hook table  
    *   will decrease by 1.                                                      
    *                                                                            
    * RETURN VALUE                                                               
    *   1   -> reference count isn't equal to zero, abort close.                 
    *   0   -> close successfully, parser_t will set to NULL.                    
    *                                                                            
    * ERRORS                                                                     
    *   < 0 -> indicate error                                                    
    *                                                                            
    * SEE ALSO                                                                   
    *   ht_parser_open()                                                         
    *                                                                            
    ****************************************************************************/
    extern int	ht_parser_close(parser_t  *pParser);

    /****f* ht_parser/ht_parser_reset() *****************************************
    *                                                                            
    * NAME                                                                       
    *   ht_parser_reset()                                                        
    *                                                                            
    * SYNOPSIS                                                                   
    *   int ht_parser_reset(parser_t  p)                                         
    *                                                                            
    * DESCRIPTION                                                                
    *   reset parsing state to initial state.                                    
    *                                                                            
    * PARAMETERS                                                                 
    *   parser_t  p: IN : handle of parser                                       
    *                                                                            
    * WARNINGS                                                                   
    *   you MUST call this function before each new HTTP connection start!       
    *                                                                            
    * NOTES                                                                      
    *   none                                                                     
    *                                                                            
    * RETURN VALUE                                                               
    *   0   -> reset ok.                                                         
    *                                                                            
    * ERRORS                                                                     
    *   < 0 -> indicate error                                                    
    *                                                                            
    * SEE ALSO                                                                   
    *   ht_parse_line(), ht_parse_block()                                        
    *                                                                            
    ****************************************************************************/
    extern int	ht_parser_reset(parser_t  p);

    /****f* ht_parser/ht_parse_block() ******************************************
    *                                                                            
    * NAME                                                                       
    *   ht_parse_block()                                                         
    *                                                                            
    * SYNOPSIS                                                                   
    *   long ht_parse_block(parser_t *  hParser,                                 
    *                       const char *szBlock,                                 
    *                       long lBlockSize)                                     
    *                                                                            
    * DESCRIPTION                                                                
    *   parsing HTTP header by block.                                            
    *                                                                            
    * PARAMETERS                                                                 
    *   parser_t hParser:    IN : handle of parser                               
    *   const char *szBlock: IN : input block                                    
    *   long lBlockSize:     IN : length of szBlock                              
    *                                                                            
    * WARNINGS                                                                   
    *   none                                                                     
    *                                                                            
    * NOTES                                                                      
    *   NOT implement yet.                                                       
    *                                                                            
    * RETURN VALUE                                                               
    *   ulBlockSize         -> keep on feed block.                               
    *   0 ~ (ulBlockSize-1) -> all headers have been parsed, return offset to    
    *                          indicate where CRLF is.                           
    * ERRORS                                                                     
    *   < 0                 -> indicate error.                                   
    *                                                                            
    * SEE ALSO                                                                   
    *   ht_parse_line(), ht_parser_reset()                                       
    *                                                                            
    ****************************************************************************/
    //  extern long ht_parse_block(parser_t *  hParser, const char *szBlock, long lBlockSize);

    /****f* ht_parser/ht_parse_line() *******************************************
    *                                                                            
    * NAME                                                                       
    *   ht_parse_line()                                                          
    *                                                                            
    * SYNOPSIS                                                                   
    *   int ht_parse_line(parser_t hParser, const char *szLine, long lLineLen)   
    *                                                                            
    * DESCRIPTION                                                                
    *   parsing HTTP header by line.                                             
    *                                                                            
    * PARAMETERS                                                                 
    *   parser_t hParser:   IN : handle of parser                                
    *   const char *szLine: IN : input line                                      
    *   long lLineLen:      IN : length of szLine                                
    *                                                                            
    * WARNINGS                                                                   
    *   1. you MUST feed a line without CRLF.                                    
    *   2. you MUST feed a empty line at the end to indicate CRLF reached, or    
    *      the last callback won't be triggered!                                 
    *                                                                            
    * NOTES                                                                      
    *   none                                                                     
    *                                                                            
    * RETURN VALUE                                                               
    *   1   -> reach CRLF, all headers have been parsed.                         
    *   0   -> keep on feeding lines.                                            
    *                                                                            
    * ERRORS                                                                     
    *   < 0 -> indicate error.                                                   
    *                                                                            
    * SEE ALSO                                                                   
    *   ht_parse_block(), ht_parser_reset()                                      
    *                                                                            
    ****************************************************************************/
    extern int ht_parse_line(parser_t  hParser, const char *szLine, long lLineLen);

#ifdef	__cplusplus
}
#endif

#endif
