/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @proxy_http.c
 *  @HTTP header parsing utility.
 */

/*
 *  Change Log
 *  2001-08-07 Amos Hsu     Initialization version
 *  2001-08-10 Amos Hsu     1.move default pre-parser to individual file.
 *                          2.av parser allows attr [ "=" value ]
 *                          3.Add token/parameter parser.
 *  2001-08-10 Amos Hsu     1.ht_av_parse(): tolerant for paring error.
 *                            when error occur, skip to next ";"
 *                          2.ht_av_parse() and ht_tokpara_list_parse():
 *                            free previous value before parsing.
 */

#ifndef _HTTP_PARSER_UTIL_H_
#define _HTTP_PARSER_UTIL_H_

#ifdef  __cplusplus
extern "C"
{
#endif

#define IS_FOLDING(str,len) ( len > 0 && ( str[0]==' ' || str[0]=='\t' ) ? 1 : 0 )

    /***************************************************************************
     *                                                                         * 
     * parsing HTTP attribute/value pairs.                                     * 
     *                                                                         * 
     ***************************************************************************/

    /*                                                                         *
     *                                                                         * 
     * list of attribute/value pairs.  *( ";" attr [ "=" value ] )             * 
     *                                                                         * 
     * please see RFC1945 Page.19 (parameter) for more details.                * 
     *                                                                         * 
     * NOTE: if value is quoted-string, two quotes will be stripped and escape * 
     *       characters will be recovery also.                                 * 
     *                                                                         * 
     *                                                                         */
    struct avpair_st;
    typedef struct avpair_st *  avpair_t;

    struct avpair_st
    {
        const char	*szAttr;
        const char	*szValue;
        avpair_t    next;
    };

    /*                                                                         *
     * parsing HTTP attribute/value pairs.   *( ";" attr [ "=" value ] )       * 
     *                                                                         * 
     * CAUTION: Be SURE to set avpair=NULL before the first time calling this  * 
     *          function.                                                      * 
     *                                                                         * 
     * RETURN:   1    -> error occur, terminate parsing and return parsed data.* 
     *           0    -> parsing ok.                                           * 
     *           < 0  -> parsing fail, indicate error.                         * 
     *                                                                         */
    extern int ht_av_parse( const char *pBegin, const char *pEnd, avpair_t *p_avpair );

    /*                                                                         *
     * free attribute/value pairs list.                                        * 
     *                                                                         * 
     * RETURN:   0    -> free ok.                                              * 
     *           < 0  -> free fail, indicate error.                            * 
     *                                                                         */
    extern int ht_av_free( avpair_t  *p_avpair );

    /*                                                                         *
     * get value by attribute.                                                 * 
     *                                                                         * 
     * RETURN:   NON-NULL  -> return value.                                    * 
     *           NULL      -> no such attribute field.                         * 
     *                                                                         */
    extern const char * ht_av_get( avpair_t  avpair, const char *szName);

    /***************************************************************************
     *                                                                         * 
     * parsing HTTP token/parameters.                                          * 
     *                                                                         * 
     ***************************************************************************/

    /*                                                                         *
     *                                                                         * 
     * list of generic token/parameters.  #( token *( ";" avpair ) )           * 
     *                                                                         * 
     * please see RFC1945 Page.10 (#rule) for more details.                    * 
     *                                                                         * 
     *                                                                         */
    struct tokpara_st;
    typedef struct tokpara_st *  tokpara_t;

    struct tokpara_st
    {
        const char	*szToken;
        avpair_t    para;
        tokpara_t   next;
    };

    /*                                                                         *
     * parsing HTTP token/parameters list.   #( token *( ";" avpair ) )        * 
     *                                                                         * 
     * CAUTION: Be SURE to set tokpara=NULL before the first time calling this * 
     *          function.                                                      * 
     *                                                                         * 
     * RETURN:   1    -> error occur, terminate parsing and return parsed data.* 
     *           0    -> parsing ok.                                           * 
     *           < 0  -> parsing fail, indicate error.                         * 
     *                                                                         */
    extern int ht_tokpara_list_parse( const char *pBegin, const char *pEnd, tokpara_t *p_tokpara );

    /*                                                                         *
     * free token/parameters list.                                             * 
     *                                                                         * 
     * RETURN:   0    -> free ok.                                              * 
     *           < 0  -> free fail, indicate error.                            * 
     *                                                                         */
    extern int ht_tokpara_free( tokpara_t *p_tokpara );

#ifdef  __cplusplus
}
#endif

#endif
