/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @ht_pre_parser.h
 *  @default HTTP header pre-parsing callback.
 */

/*
 *  Change Log
 *  2001-08-10 Amos Hsu     Initialization version
 */

#ifndef _HT_PRE_PARSER_H_
#define _HT_PRE_PARSER_H_

#ifdef  __cplusplus
extern "C"
{
#endif

    struct tokpara_st;
    struct avpair_st;

    /*                                                                         *
     * prototype of default major parser.                                      * 
     *                                                                         * 
     * REMARK: 1.you MUST follow the prototype in our major callback, or core  * 
     *           dump will happen.                                             * 
     *         2.we CAN'T offer prototype checking, be careful with the proto- * 
     *           type of your major callback function.                         * 
     *                                                                         */
    typedef int (*FP_HT_PARSER_P_CHAR1)     (void *pMajorArg, const char *szHookName, const char *szValue);
    typedef int (*FP_HT_PARSER_ULONG1)      (void *pMajorArg, const char *szHookName, const unsigned long ulValue);
    typedef int (*FP_HT_PARSER_LONG1)       (void *pMajorArg, const char *szHookName, const long lValue);
    typedef int (*FP_HT_PARSER_TOKPARA)     (void *pMajorArg, const char *szHookName, const char *szToken, const struct avpair_st  *para);
    typedef int (*FP_HT_PARSER_TOKPARA_LIST)(void *pMajorArg, const char *szHookName, const struct tokpara_st *tokpara);

    /*                                                                         *
     * default pre-parser of p_char1 type.                                     * 
     *                                                                         * 
     * REMARK: 1.pass original value behind "header:" to major callback without* 
     *           any preprocess.                                               * 
     *         2.prototype of major callback is FP_HTTP_PARSER_P_CHAR1.        * 
     *                                                                         */
    extern int ht_pre_parser_p_char1  (void *pPreArg, void *fpMajorCallBack, void *pMajorArg, const char *szHookName, const char *szValue);

    /*                                                                         *
     * default pre-parser of ulong1 type.                                      * 
     *                                                                         * 
     * REMARK: 1.transfer value to unsigned long then pass to major callback.  * 
     *         2.prototype of major callback is FP_HTTP_PARSER_ULONG1.         * 
     *                                                                         */
    extern int ht_pre_parser_ulong1   (void *pPreArg, void *fpMajorCallBack, void *pMajorArg, const char *szHookName, const char *szValue);

    /*                                                                         *
     * default pre-parser of long1 type.                                       * 
     *                                                                         * 
     * REMARK: 1.transfer value to long then pass to major callback.           * 
     *         2.prototype of major callback is FP_HTTP_PARSER_ULONG1.         * 
     *                                                                         */
    extern int ht_pre_parser_long1   (void *pPreArg, void *fpMajorCallBack, void *pMajorArg, const char *szHookName, const char *szValue);

    /*                                                                         *
     * default pre-parser of token-parameters.                                 * 
     *                                                                         * 
     * REMARK: 1.parsing value to token-parameters.                            * 
     *         2.prototype of major callback is FP_HTTP_PARSER_TOKPARA.        * 
     *                                                                         */
    extern int ht_pre_parser_tokpara (void *pPreArg, void *fpMajorCallBack, void *pMajorArg, const char *szHookName, const char *szValue);

    /*                                                                         *
     * default pre-parser of token-parameters list.                            * 
     *                                                                         * 
     * REMARK: 1.parsing value to token-parameters list.                       * 
     *         2.prototype of major callback is FP_HTTP_PARSER_TOKPARA_LIST.   * 
     *                                                                         */
    extern int ht_pre_parser_tokpara_list (void *pPreArg, void *fpMajorCallBack, void *pMajorArg, const char *szHookName, const char *szValue);

#ifdef  __cplusplus
}
#endif

#endif
