/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file log.h
 *  @brief TMOG Log Module
 */

/*
 *  Change Log
 *  2004-04-12 Henry Lin    Initialization Version
 */

#ifndef LOG_H
#define LOG_H

#include <syslog.h>

#define NIPQUAD(addr) \
        ((unsigned char *)&addr)[0], \
        ((unsigned char *)&addr)[1], \
        ((unsigned char *)&addr)[2], \
        ((unsigned char *)&addr)[3]

#define HIPQUAD(addr) \
        ((unsigned char *)&addr)[3], \
        ((unsigned char *)&addr)[2], \
        ((unsigned char *)&addr)[1], \
        ((unsigned char *)&addr)[0]

/* ModuleName: defined in each module */
/* fmt: quoted-string only */

#define LOGOPEN(ProgName)         openlog(ProgName, LOG_CONS | LOG_NDELAY, LOG_DAEMON)
#define LOGCLOSE()                closelog()

#ifdef NDEBUG
#define LOGDEBUG(fmt, args...) 
#define LOGINFO(fmt, args...)     
#else
//#define LOGDEBUG(fmt, args...)    syslog(LOG_DEBUG,   "%s [DEBUG]" fmt, ModuleName, ## args)
#define LOGDEBUG(fmt, args...)    syslog(LOG_INFO,   "%s [INFO]" fmt, ModuleName, ## args)
#define LOGINFO(fmt, args...)     syslog(LOG_INFO,    "%s [INFO]" fmt, ModuleName, ## args)
#endif

/*******************************************************************************
* Porting Required Section Start
*******************************************************************************/
#define LOGNOTICE(fmt, args...)   syslog(LOG_NOTICE,  "%s [NOTICE]" fmt, ModuleName, ## args)
#define LOGWARNING(fmt, args...)  syslog(LOG_WARNING, "%s [WARNING]" fmt, ModuleName, ## args)
#define LOGERR(fmt, args...)      syslog(LOG_ERR,     "%s [ERR]" fmt, ModuleName, ## args)
#define LOGCRIT(fmt, args...)     syslog(LOG_CRIT,    "%s [CRIT]" fmt, ModuleName, ## args)
#define LOGALERT(fmt, args...)    syslog(LOG_ALERT,   "%s [ALERT]" fmt, ModuleName, ## args)
#define LOGEMERG(fmt, args...)    syslog(LOG_EMERG,   "%s [EMERG]" fmt, ModuleName, ## args)

/*******************************************************************************
* Porting Required Section End
*******************************************************************************/

#define LOGERROR                  LOGERR
#define LOGDEBUG_ON()             setlogmask(~0x0000)
#define LOGDEBUG_OFF()            setlogmask(~(LOG_MASK(LOG_DEBUG)))

#ifdef __cplusplus
extern "C"
{
#endif

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* LOG_H */
