/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file tmClientInfo.h
 *  @brief TMOG ClientInfo Module
 */

/*
 *  Change Log
 *  2004-02-18 Eric Li Initialization version
 */


#ifndef CLIENTINFO_H
#define CLIENTINFO_H

#ifndef __KERNEL__
#include "list.h"
#include "tmauIniParser.h"
#include "tmdpCommonDef.h"
#endif

#define TMSS_AV_SHMKEY               ((key_t)0x711)
#define TMSS_AV_SEMKEY               ((key_t)0x712)
#define TMSS_AV_SHMFLG               (IPC_CREAT | IPC_EXCL | 0666)
#define TMSS_AV_SHMADDR              ((const void *)0x45000000)

#define STATUS_SIZE            5
#define STATUS_READ_FROM_CONFIG         99
#define STATUS_PENDING         0
#define STATUS_NOAVWIN         1
#define STATUS_NOAVUNIX        2
#define STATUS_AVOLD           3
#define STATUS_AVOK            4

#define MAC_SIZE                      6 
#define HOSTNAME_SIZE                32

#define LICENSE_SIZE                  2
#define LICENSE_EXPIRED               0
#define LICENSE_OK                    1

#define VADETAIL_SIZE               512

#define VATYPE_SIZE                   6
#define VATYPE_MSPATCH                0
#define VATYPE_TROJAN                 1
#define VATYPE_SPYWARE                2
#define VATYPE_NETSHARE               3
#define VATYPE_AVSOFT                 4
#define VATYPE_AVPATTERN              5

/* shortcut of ClientInfo structure */
#define CMAC(ptr)           ((ptr)->idinfo.mac)
#define CIP(ptr)            ((ptr)->idinfo.ip)
#define CHOSTNAME(ptr)      ((ptr)->idinfo.hostname)
#define CLICENSE(ptr)       ((ptr)->avinfo.license)
#define CPRODUCT(ptr)       ((ptr)->avinfo.auversion.product)
#define CPATTERN(ptr)       ((ptr)->avinfo.auversion.pattern.version)
#define CENGINE(ptr)        ((ptr)->avinfo.auversion.engine.version)
#define CSTATUS(ptr)        ((ptr)->avinfo.status)
#define CVA(ptr)            ((ptr)->vainfo.va)
#define CVAMSPATCHCOUNT(ptr)    ((ptr)->vainfo.nMSPatchCount)
#define CVATROJANCOUNT(ptr)     ((ptr)->vainfo.nTrojanCount)
#define CVANETSHARECOUNT(ptr)   ((ptr)->vainfo.nNetshareCount)
#define CVAAVSOFTCOUNT(ptr)     ((ptr)->vainfo.nAVSoftCount)
#define CVASPYWARECOUNT(ptr)    ((ptr)->vainfo.nSpywareCount)
#define CVASCANDATE(ptr)        ((ptr)->vainfo.nScanDate)
#define CTUPDATE(ptr)       ((ptr)->timeinfo.update_time)
#define CTAV(ptr)           ((ptr)->timeinfo.av_time)
#define CTVA(ptr)           ((ptr)->timeinfo.va_time)
#define CTREDIRECT(ptr)     ((ptr)->timeinfo.report_time)
#define CTCREATE(ptr)     ((ptr)->timeinfo.create_time)
#define CTREDIRECTFLAG(ptr) ((ptr)->timeinfo.redir_flag)
#define CTREDIRECTRETRY(ptr) ((ptr)->timeinfo.redir_retry_count)
#define CEXP(ptr)           ((ptr)->expinfo)
#define CISEXPREPORT(ptr)   ((ptr)->expinfo.report)
#define CISEXPFILTER(ptr)   ((ptr)->expinfo.urlfilter)

/* shortcut of TmuAUVersion_t structure */
#define AUPRODUCT(ptr)    ((ptr)->product)
#define AUPATTERN(ptr)    ((ptr)->pattern.version)
#define AUENGINE(ptr)     ((ptr)->engine.version)

extern const char *MsgStatus[];
extern const char *MsgType[];
extern const char *MsgVA[];
extern const char *MsgLicense[];

struct client_id
{
    uint32_t       ip;
    unsigned char  mac[MAC_SIZE];
    char           hostname[HOSTNAME_SIZE];
};

struct client_av
{
    uint16_t          status;              /* av status */

    /* Trend Micro Only Section */
    TmuAUVersion_t    auversion;
    char              szAUProductId[4];    /*TMSS should use 0x0 */
    char              szProductVer[16];    /* for PCC */
    char              szEngineId[4];       /* TMSS use 0x0 */
    unsigned int      license;            /* if license expired */
    unsigned int      auLangID;

};

struct client_va
{
    unsigned int      nMSPatchCount;
    unsigned int      nTrojanCount;
    unsigned int      nNetshareCount;
    unsigned int      nAVSoftCount;
    unsigned int      nSpywareCount;
    unsigned long     nScanDate;
    char              va[VATYPE_SIZE][VADETAIL_SIZE];
};

struct client_exp
{
    uint8_t           urlfilter;     /* Is exception for URL Filter */
    uint8_t           report;        /* Is exception for Health Report */
};

struct client_time
{
    unsigned long     av_time;       /* time AV info updated */
    unsigned long     va_time;       /* time VA info updated */
    unsigned long     update_time;   /* time triggering client to update */
    unsigned long     report_time;   /* time redirecting client to report */
    unsigned long     create_time;   /* time client info created */
    int redir_flag;                  /* the redirect flag to prevent icq or msn prevent dashboard pop-up, value 0 will do redirect every 1 minute, value 1 will do redirect every 1 day */
    int redir_retry_count;           /* the retry count for redirect when redirect fail */
};

typedef struct 
{
    struct client_id    idinfo;           /* ID information */
    struct client_av    avinfo;           /* AV information */
    struct client_va    vainfo;           /* VA information */
    struct client_time  timeinfo;         /* Time information */
    struct client_exp   expinfo;          /* Exception information */

    struct list_head  fifo_head;          /* fifo list pointer */
    struct list_head  hash_head;          /* hash pointer */

} ClientInfo;


#define CLIENT_HASH_SIZE              16 
#define CLIENT_MAX_SIZE               16    /* must be greater than 0 */

typedef struct
{
    struct list_head    hash[CLIENT_HASH_SIZE];
    unsigned int        size;           /* number of clients */
    struct list_head    fifo;
    unsigned int        license_size[LICENSE_SIZE];
} ClientTable;

#ifdef __cplusplus
extern "C" {
#endif

int IP_MAC(uint32_t *ip, unsigned char *mac);
void StrToMac(const char *str, unsigned char *mac);
void MacToStr(const unsigned char *mac, char *str);

#ifndef __KERNEL__
void tmci_InitClientTable(void);
#else
int tmci_InitClientTable(void);
#endif
int tmci_DelClient(unsigned char *mac);
int tmci_GetClient(unsigned char *mac, ClientInfo *pClient);

int tmci_SetVA(struct client_id *id, struct client_va *va);
int tmci_SetAV(struct client_id *id, struct client_av *av);
int tmci_SetTime(struct client_id *id, struct client_time *timeinfo);
int tmci_SetExp(struct client_id *id, struct client_exp *exp);

int tmci_IsExpFilter(unsigned char *mac);
int tmci_IsExpReport(unsigned char *mac);

int tmci_UpdateClientStatus(TmuAUVersion_t *pAU);
void tmci_ResetClientTable(void);
int tmci_DestroyClientTable(void);
unsigned int tmci_SizeLicense(uint8_t idx);
unsigned int tmci_SizeClient(void);
int tmci_GetClientsFrom(ClientInfo *pBuf,int *pSize, int index);
int tmci_DumpClientTable(void);

void tmci_ReadClient(const char *file);
void tmci_WriteClient(const char *file);

#ifdef __cplusplus
} /* extern "C" */
#endif


#endif  /* CLIENTINFO_H */
