/*****************************************************************************
 *
 * Program Name:	MD5 Message Digest Creator
 *
 * Filename:		md5.h
 *
 * Date Created:	January 4, 1995
 *
 * Version:			1.0
 *
 * Programmers:
 *
 * Files used:		globals.h, md5.h, md5c.c
 *
 * Date Modified: 
 *
 * Modifications: 
 *
 * Comments:		This implements the RSA MD5 "checksum" or digest
 *						algorithm. It is derived from the RSA Data Security, Inc.
 *						MD5 Message-Digest Algorithm.
 *
 * COPYRIGHT (c) 1995 by Intel Corp.  All Rights Reserved.
 *
 ****************************************************************************/


/* Copyright (C) 1991-2, RSA Data Security, Inc. Created 1991. All
rights reserved.

License to copy and use this software is granted provided that it
is identified as the "RSA Data Security, Inc. MD5 Message-Digest
Algorithm" in all material mentioning or referencing this software
or this function.

License is also granted to make and use derivative works provided
that such works are identified as "derived from the RSA Data
Security, Inc. MD5 Message-Digest Algorithm" in all material
mentioning or referencing the derived work.

RSA Data Security, Inc. makes no representations concerning either
the merchantability of this software or the suitability of this
software for any particular purpose. It is provided "as is"
without express or implied warranty of any kind.

These notices must be retained in any copies of any part of this
documentation and/or software.
*/
#ifndef _SMEX_MD5_H
#define _SMEX_MD5_H
/* POINTER defines a generic pointer type */
typedef unsigned char *POINTER;

/* UINT2 defines a two byte word */
typedef unsigned short int UINT2;

/* UINT4 defines a four byte word */
typedef unsigned long int UINT4;

#define PROTO_LIST(list) list


/* MD5 context. */
typedef struct {
  UINT4 state[4];                                   /* state (ABCD) */
  UINT4 count[2];        /* number of bits, modulo 2^64 (lsb first) */
  unsigned char buffer[64];                         /* input buffer */
} MD5_CTX;

void 
	MD5Init PROTO_LIST ((MD5_CTX *)),
	MD5Update PROTO_LIST
  		((MD5_CTX *, unsigned char *, unsigned int)),
	MD5Final PROTO_LIST ((unsigned char [16], MD5_CTX *));



// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the LIBMD5_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// LIBMD5_API functions as being imported from a DLL, wheras this DLL sees symbols
// defined with this macro as being exported.
#ifdef WIN32
    # ifdef LIBMD5_EXPORTS
    # define LIBMD5_API __declspec(dllexport)
    # else
    # define LIBMD5_API __declspec(dllimport)
    # endif
#else
    #define LIBMD5_API extern
#endif

#ifdef __cplusplus
extern "C" {
#endif 

//extern int MD5(char *input, unsigned char result[16]);
LIBMD5_API int MD5(char *input, unsigned char result[16]);
LIBMD5_API int MakeCheckSum(char szInput[13], char szResult[17]);

#ifdef WIN32
   LIBMD5_API int IsCorrectCheckSum(char *szInput);
#endif

#ifdef __cplusplus
}
#endif 

#endif // _SMEX_MD5_H

