/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file tmURLCache.h
 *  @brief TMOG URL Filtering Cache Module
 */

/*
 *  Change Log
 *  2004-02-13 Henry Lin    Initialization version
 */

#ifndef __TMURLCACHE_H__
#define __TMURLCACHE_H__

#ifndef __KERNEL__
#include <sys/types.h>      /* key_t */
#include <time.h>           /* time_t */
#endif

#define TMSS_URL_FILTER_CACHE_SHMKEY            ((key_t)1926)
#define TMSS_URL_FILTER_CACHE_SEMLOCKKEY        ((key_t)1927)

#define MAX_URL_BUF           80

#define CACHE_HASH_TABLE_SIZE 27

#define URL_OK             0 /* concat url success for cache */
#define URL_TOO_LONG       1 /* concat url fail because too long */

/* Cache query return value */
#define URL_IN_CACHE               0
#define URL_NOT_INCACHE            -1
#define URL_IN_CACHE_BUT_EXPIRED   -2

/*
* ------------------------------------------------------------------------------
* Type Definitions
* ------------------------------------------------------------------------------
*/

typedef struct requested_url_st
{
    char req_url[MAX_URL_BUF];
    time_t expire_time;
    int action;//0:PASS, 1:DROP, 2:PASS_ON_ERROR, 3:DROP_ON_ERROR.  4: CAN NOT DERTERMINED
    unsigned long rsp_blkc;
    struct requested_url_st *next;
    struct requested_url_st *lru;
    struct requested_url_st *mru;
}
requested_url_t;

typedef struct url_cache_st
{
    requested_url_t *hash_table[CACHE_HASH_TABLE_SIZE]; // [0] to [25] for a to z and [26] for others
    requested_url_t *cache_lru;
    requested_url_t *cache_mru;
    requested_url_t *free_node;
}
url_cache_t;
typedef struct add_url_cache_s
{
    char szReqUrlWithoutArg[MAX_URL_BUF];
    int nRateRetVal;
    int rsp_blkc;
}
add_url_cache;

/*
* ------------------------------------------------------------------------------
* Function Prototypes Section.
* ------------------------------------------------------------------------------
*/

#ifdef __cplusplus
extern "C" {
#endif

int RemoveOptionalArgInURL(const char *pszHost, int nHostLen, const char *pszUrl, int nUrlLen, char *szReqUrlWithoutArg, int nUrlBufLen);
int tmInitUrlFilterCache(void);//int nUrlFilterCacheSize);
int tmUrlCacheAddNode(add_url_cache *a);
int tmQueryUrlCache(/*int nUrlFilterCacheSize,*/ const char* szURL,unsigned long *rsp_blkc);//, int nUrlFilterCacheExpireTime);
//void tmDumpUrlCache(int nUrlFilterCacheSize, int nBeforeAfter);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif
