/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file tmURLFilterConf.h
 *  @brief TMOG URL Filtering Configuration Loading Module
 */

/*
 *  Change Log
 *  2004-03-03 Henry Lin    Initialization version
 */


#ifndef __TMURLFILTERCONF_H__
#define __TMURLFILTERCONF_H__

#ifndef __KERNEL__
#include <sys/types.h>      /* key_t */
#endif

#define TMSS_URL_FILTER_CONFSHMKEY              ((key_t)1924)
#define TMSS_URL_FILTER_CONFSEMLOCKKEY          ((key_t)1925)

/* HV defined Categories */
#define HV_DEFINED_CAT_PORN                0
#define HV_DEFINED_CAT_ALCOHOL_TOBACCO     1
#define HV_DEFINED_CAT_ILLEGAL             2
#define HV_DEFINED_CAT_GAMBLING            3
#define HV_DEFINED_CAT_VIOLENCE            4
#define HV_DEFINED_CAT_ABORTION            5
#define HV_DEFINED_CAT_ILLEGAL_DRUGS       6
#define HV_DEFINED_CAT_OTHERS              7
#define HV_DEFINED_CAT_NUM                 8

#define MAX_WEEK_DAYS                      7

/* Configuration Module name and Parameter name */
#define CFG_MODULE_URL_FILTER                   "url_filtering"
#define CFG_PARAM_URL_FILTER_ENABLE             "url_filter_enable"
#define CFG_PARAM_VENDOR_ID                     "vendor_id"
#define CFG_PARAM_LICENSE_ID                    "license_id"
#define CFG_PARAM_USER_ID                       "user_id"
#define CFG_PARAM_ORIGIN_PROTOCOL               "origin_protocol"
#define CFG_PARAM_TM_URL_FILTER_HOST            "url_filter_host"
#define CFG_PARAM_CATEGORY                      "category"
#define CFG_PARAM_TM_URL_FILTER_HOST_PORT       "url_filter_host_port"
#define CFG_PARAM_DRTR_MODE                     "drtr_mode"
#define CFG_PARAM_DENY_ON_ERROR                 "deny_on_error"
#define CFG_PARAM_URL_CACHE_SIZE                "url_cache_size"
#define CFG_PARAM_URL_LICENSE_ENABLE            "license_enable"
#define CFG_PARAM_CACHE_EXPIRE_TIME             "url_cache_expire_time"
#define CFG_PARAM_RATING_TIMEOUT                "url_rating_timeout"
#define CFG_PARAM_URL_FILTER_EXCEPTION_ENABLE   "url_filter_exception_enable"
#define CFG_PARAM_LIC_CHK_INTERVIAL             "url_filter_lic_chk_interval"
#define CFG_PARAM_ALWAYS_BLOCK             	"always_block"
#define CFG_PARAM_SCHEDULE_DAY                  "schedule_day_"
#define CFG_PARAM_SCHEDULE_FRM_HOUR              "schedule_from_hour"
#define CFG_PARAM_SCHEDULE_FRM_MIN              "schedule_from_min"
#define CFG_PARAM_SCHEDULE_TO_HOUR              "schedule_to_hour"
#define CFG_PARAM_SCHEDULE_TO_MIN              "schedule_to_min"

#ifdef _PROXY_
#define CFG_MODULE_PROXY                        "proxy"
#define CFG_PARAM_PROXY_ENABLE                  "proxy_enable"
#define CFG_PARAM_PROXY_HOST                    "proxy_host"
#define CFG_PARAM_PROXY_PORT                    "proxy_port"
#define CFG_PARAM_PROXY_USERNAME                "proxy_username"
#define CFG_PARAM_PROXY_PASSWORD                "proxy_password"
#endif

#define CFG_MODULE_ENTITLEMENT                  "entitlement"
#define CFG_PARAM_ENTITLE_HOST                  "entitlement_host"
#define CFG_PARAM_ENTITLE_PORT                  "entitlement_port"
#define CFG_PARAM_ENTITLE_PATH                  "entitlement_path"

/* Configuration Parameter Value */
#define CFG_PARAM_VALUE_URL_FILTER_DISABLE      "0"
#define CFG_PARAM_VALUE_URL_FILTER_ENABLE       "1"

/* Configuration Default Value */
#define URL_FILTER_CFG_DEFAULT_VENDOR_ID                    "TRE"
#define URL_FILTER_CFG_DEFAULT_LICENSE_ID                   "TRE00002"
#define URL_FILTER_CFG_DEFAULT_USER_ID                      "-"
#define URL_FILTER_CFG_DEFAULT_ORIGIN_PROTOCOL              "0"
#define URL_FILTER_CFG_DEFAULT_TM_URL_FILTER_HOST           "sp.cwfservice.net"
#define URL_FILTER_CFG_DEFAULT_CATEGORY                     "00010203"
#define URL_FILTER_CFG_DEFAULT_URL_FILTER_ENABLE            1
#define URL_FILTER_CFG_DEFAULT_TM_URL_FILTER_HOST_PORT      80
#define URL_FILTER_CFG_DEFAULT_DRTR_MODE                    0
#define URL_FILTER_CFG_DEFAULT_DENY_ON_ERROR                0
#define URL_FILTER_CFG_DEFAULT_URL_CACHE_SIZE               500
#define URL_FILTER_CFG_DEFAULT_URL_LICENSE_ENABLE           0
#define URL_FILTER_CFG_DEFAULT_URL_FILTER_EXCEPTION_ENABLE  0
#define URL_FILTER_CFG_DEFAULT_CACHE_EXPIRE_TIME            3600
#define URL_FILTER_CFG_DEFAULT_RATING_TIMEOUT               3
#define URL_FILTER_CFG_DEFAULT_PROXY_ENABLE                 0
#define URL_FILTER_CFG_DEFAULT_TM_URL_FILTER_PROXY_PORT     80
#define URL_FILTER_CFG_DEFAULT_TM_URL_FILTER_ENTITLE_PORT   80
#define URL_FILTER_CFG_DEFAULT_TM_ENTITLE_HOST              "TBD"
#define URL_FILTER_CFG_DEFAULT_TM_ENTITLE_PATH              "TBD"
#define URL_FILTER_CFG_DEFAULT_TM_URL_FILTER_LIC_CHK_INTERVAL 86400
#define URL_FILTER_CFG_DEFAULT_ALWAYS_BLOCK                 1
#define URL_FILTER_CFG_DEFAULT_SCHEDULE_DAY                 1
#define URL_FILTER_CFG_DEFAULT_SCHEDULE_FRM_HOUR             8
#define URL_FILTER_CFG_DEFAULT_SCHEDULE_FRM_MIN              0
#define URL_FILTER_CFG_DEFAULT_SCHEDULE_TO_HOUR             18
#define URL_FILTER_CFG_DEFAULT_SCHEDULE_TO_MIN              0

/* Configuration Max Buffer size */
#define URL_FILTER_MAX_VENDORID             32
#define URL_FILTER_MAX_LICENSEID            32
#define URL_FILTER_MAX_USERID               32
#define URL_FILTER_MAX_ORIGINPROTOCOL       2
#define URL_FILTER_MAX_HOST                 128
#define URL_FILTER_MAX_PATH                 256
#define URL_FILTER_MAX_CATEGORY             (HV_DEFINED_CAT_NUM * 2 + 1)
#define URL_FILTER_ONE_CATEGORY_LEN         2
#define URL_FILTER_MAX_PROXY_USERNAME       128
#define URL_FILTER_MAX_PROXY_PASSWORD       128

/* Configuration Max and Min value */
#define URL_FILTER_ORIGIN_PROTOCOL_MIN      0
#define URL_FILTER_ORIGIN_PROTOCOL_MAX      0
#define URL_FILTER_PORT_MIN                 0
#define URL_FILTER_PORT_MAX                 65535
#define URL_FILTER_CACHE_MIN                1
#define URL_FILTER_CACHE_MAX                65535
#define URL_FILTER_CACHE_EXPIRE_TIME_MIN    10 /* 10 seconds */
#define URL_FILTER_CACHE_EXPIRE_TIME_MAX    86400 /* one day */
#define URL_FILTER_RATING_TIMEOUT_MIN       1 /* 1 seconds */
#define URL_FILTER_RATING_TIMEOUT_MAX       120 /* 2 minutes */
#define URL_FILTER_LIC_CHK_INTERVAL_MIN     30 /* 30 seconds */
#define URL_FILTER_LIC_CHK_INTERVAL_MAX     86400 /* 1 day */
#define URL_FILTER_SCHEDULE_FRM_HOUR_MIN     0
#define URL_FILTER_SCHEDULE_FRM_MIN_MIN     0
#define URL_FILTER_SCHEDULE_TO_HOUR_MIN     0
#define URL_FILTER_SCHEDULE_TO_MIN_MIN     0
#define URL_FILTER_SCHEDULE_FRM_HOUR_MAX     23
#define URL_FILTER_SCHEDULE_FRM_MIN_MAX     59
#define URL_FILTER_SCHEDULE_TO_HOUR_MAX     23
#define URL_FILTER_SCHEDULE_TO_MIN_MAX     59


/*
* ------------------------------------------------------------------------------
* Type Definitions
* ------------------------------------------------------------------------------
*/

typedef struct url_filter_config_st
{
	unsigned int version;// use to notify the kernel that configuration has been changed.
    unsigned char szVendorId[URL_FILTER_MAX_VENDORID];
    unsigned char szLicenseId[URL_FILTER_MAX_LICENSEID];
    unsigned char szUserId[URL_FILTER_MAX_USERID];
    unsigned char szOriginProtocol[URL_FILTER_MAX_ORIGINPROTOCOL];
    unsigned char szTmUrlFilterHost[URL_FILTER_MAX_HOST];
    unsigned char szCategory[URL_FILTER_MAX_CATEGORY];
    unsigned short nCategory[HV_DEFINED_CAT_NUM];
    unsigned short nUrlFilterEnable;
    unsigned short nTmUrlFilterHostPort;
    unsigned short nDrtrMode;
    unsigned short nDenyOnError;
    unsigned int nUrlCacheSize;
    unsigned short nUrlLicenseEnable;
    unsigned short nUrlFilterExceptionEnable;
    unsigned int nUrlFilterCacheExpireTime;
    unsigned short nUrlFilterRatingTimeout;
#ifdef _PROXY_
    unsigned short nProxyEnable;
    unsigned char szProxyHost[URL_FILTER_MAX_HOST];
    unsigned short nProxyPort;
    unsigned char szProxyUsername[URL_FILTER_MAX_PROXY_USERNAME];
    unsigned char szProxyPassword[URL_FILTER_MAX_PROXY_PASSWORD]; 
#endif
    unsigned char szEntitleHost[URL_FILTER_MAX_HOST];
    unsigned short nEntitlePort;
    unsigned char szEntitlePath[URL_FILTER_MAX_PATH];
    unsigned int nLicChkInterval;
    unsigned short alwaysBlock;
    unsigned short nScheduleDay[MAX_WEEK_DAYS];
    unsigned short nScheduleFromHour;
    unsigned short nScheduleFromMin;
    unsigned short nScheduleToHour;
    unsigned short nScheduleToMin;
}
url_filter_config_t;
/*
* ------------------------------------------------------------------------------
* Function Prototypes Section.
* ------------------------------------------------------------------------------
*/

#ifdef __cplusplus
extern "C" {
#endif

void CopyUrlConfToLocal(url_filter_config_t *pUrlFilterConfig);
int tmLoadUrlFilterConf(const char* szUrlConfFile);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif
