/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file tmURLRater.h
 *  @brief TMOG URL Rating Module
 */

/*
 *  Change Log
 *  2004-01-08 Henry Lin    Initialization version
 */

#ifndef __TM_URL_RATER_H__
#define __TM_URL_RATER_H__

#include <arpa/inet.h>
#include <netdb.h>
#include "tmURLFilterConf.h" /* url_filter_config_t */

/* ----------------------------------------------------------------------------
* Macros Section.
* ----------------------------------------------------------------------------
*/
/*
* Request Flags.
*/
#define TM_URL_FILTER_ACCESS_DETERMINATION_REQUEST 0x0
// API Y/N Type
// request
#define TM_URL_FILTER_CATEGORY_MEMBERSHIP_REQUEST 0x1
// API Cat Member
// request
#define TM_URL_FILTER_CATEGORY_MEMBERSHIP_NO_DRTR_REQUEST 0x2
// API Cat Member
// request, no DRTR
#define TM_URL_FILTER_SERVICE_MAX_REQUEST_SIZE 2048 // Max Service Request
// size
#define TM_URL_FILTER_SERVICE_MAX_RESPONSE_SIZE 2048 // Max Service Response
// size
#define TM_URL_FILTER_MAX_NUM_CATEGORIES 128 // Number of supported
// categories
#define TM_URL_FILTER_MAX_CATEGORY_LIST_SIZE (TM_URL_FILTER_MAX_NUM_CATEGORIES*2)
// Category Membership
// list size in bytes
#define TM_URL_FILTER_MAX_COMPONENT_LEN 64 // Max size of TrendMicro URL Filter
// Service individual
// message components
#define TM_URL_FILTER_MAX_USER_ID_LEN 32 // Max size of TrendMicro URL Filter
// user id
#define TM_URL_FILTER_MAX_PORT_LEN 6 // Max size of TrendMicro URL Filter Server Port
/* Result flags.
 */
#define TM_URL_FILTER_RESULT_DENY 0x00000 // Deny access
//
#define TM_URL_FILTER_RESULT_ALLOW 0x00001 // Allow access
//
#define TM_URL_FILTER_RESULT_CAT_NOT_PRESENT TM_URL_FILTER_RESULT_DENY
// Category not in list
#define TM_URL_FILTER_RESULT_CAT_PRESENT TM_URL_FILTER_RESULT_ALLOW
// Category in list
#define TM_URL_FILTER_RESULT_NO_LICENSE 0x00002 // No valid License
//
#define TM_URL_FILTER_RESULT_NO_USER_NAME 0x00004 // User name bad
//
#define TM_URL_FILTER_RESULT_NO_POLICY 0x00008 // No rating possible
//
#define TM_URL_FILTER_RESULT_NO_TIME 0x00010 // Bad time range value
//
#define TM_URL_FILTER_RESULT_LICENSE_FAILURE 0x00020 // DB error from
// License DB
#define TM_URL_FILTER_RESULT_IDENTITY_FAILURE 0x00040 // DB error from
// Identity DB
#define TM_URL_FILTER_RESULT_POLICY_FAILURE 0x00080 // DB error from
// Policy DB
#define TM_URL_FILTER_RESULT_OVERRIDE_FAILURE 0x00100 // DB error from
// Override DB
#define TM_URL_FILTER_RESULT_RATING_FAILURE 0x00200 // DB error from
// Rating DB
#define TM_URL_FILTER_RESULT_NO_SPREAD_SERVER 0x00400 // Connection Server
// communications to
// blocking server
// broken
#define TM_URL_FILTER_RESULT_NO_BLOCKING_SERVER 0x00800 // No blocking servers
// available
#define TM_URL_FILTER_RESULT_RATING_UNKNOWN 0x01000 // No rating possible
//
#define TM_URL_FILTER_RESULT_URL_NOT_FOUND 0x02000 // Unknown URL
//
#define TM_URL_FILTER_RESULT_BAD_REQUEST 0x04000 // Invalid request from
// the client
#define TM_URL_FILTER_RESULT_BAD_URL 0x08000 // URL too long
//
#define TM_URL_FILTER_RESULT_METHOD_NOT_ALLOWED 0x10000 // Invalid command type
//
#define TM_URL_FILTER_RESULT_REQUEST_TOO_LONG 0x20000 // Request too long
//
#define TM_URL_FILTER_RESULT_SERVICE_UNAVAILABLE 0x40000 // No service
//
/*
* Information bits flags for the XML <Code> result tags in the Entity Body.
*/
#define TM_URL_FILTER_INFO_SYSTEM_OVERRIDE 0x0001 // Used System Override
//
#define TM_URL_FILTER_INFO_USER_OVERRIDE 0x0002 // Used User Override
//
#define TM_URL_FILTER_INFO_NO_BLOCKING_SERVER 0x0004 // No blocking servers
// are available
#define TM_URL_FILTER_INFO_NO_SPREAD_DAEMON 0x0008 // Cannot access Spread
// daemon
#define TM_URL_FILTER_INFO_LICENSE_FAILURE 0x0010 // Error to License DB
//
#define TM_URL_FILTER_INFO_IDENTITY_FAILURE 0x0020 // Error to Identity DB
//
#define TM_URL_FILTER_INFO_POLICY_FAILURE 0x0040 // Error to Policy DB
//
#define TM_URL_FILTER_INFO_OVERRIDE_FAILURE 0x0080 // Error to Override DB
//
#define TM_URL_FILTER_INFO_RATING_FAILURE 0x0100 // Error to Rating DB
//
#define TM_URL_FILTER_INFO_INVALID_TIME 0x0200 // Time not found in
// Identity DB
#define TM_URL_FILTER_INFO_INVALID_USERID 0x0400 // Entry not found in
// Identity DB
#define TM_URL_FILTER_INFO_INVALID_POLICY 0x0800 // Entry not found in
// Policy DB
#define TM_URL_FILTER_INFO_INVALID_LICENSE 0x1000 // Entry not found in
// License DB
#define TM_URL_FILTER_INFO_RATING_UNKNOWN 0x2000 // Rating returned
// Unknown (90)
#define TM_URL_FILTER_INFO_URL_UNRATEABLE 0x4000 // No rating returned
//
#define TM_URL_FILTER_INFO_RATE_STATIC 0x8000 // Static rating
//
#define TM_URL_FILTER_INFO_RATE_DYNAMIC 0x10000 // Dynamic rating
//
#define TM_URL_FILTER_INFO_RATE_ADVERTISEMENT 0x20000 // Rating was an
// Advertisement
#define TM_URL_FILTER_INFO_NOT_LOGGED 0x40000 // Access not logged
//
#define TM_URL_FILTER_INFO_DRT_NOT_LICENSED 0x80000 // DRTR not licensed
//
#define TM_URL_FILTER_INFO_INVALID_PROTOCOL 0x100000 // Invalid protocol num
//
#define TM_URL_FILTER_INFO_FEATURE_NOT_LICENSED 0x200000 // Feature not licensed
//
#define TM_URL_FILTER_INFO_DOMR_RETURNED 0x400000 // DomR Tag Set
//
#define TM_URL_FILTER_INFO_DIRR_RETURNED 0x800000 // DirR Tag Set
//
#define TM_URL_FILTER_INFO_BLKC_RETURNED 0x1000000 // Block Cat Num
// Tag Set
#define TM_URL_FILTER_INFO_DOMT_RETURNED 0x2000000 // Domain Cat
// Tag Set
#define TM_URL_FILTER_TMUFSACI_ERROR 0x0 // Error getting rating
#define TM_URL_FILTER_TMUFSACI_SUCCESS 0x1 // Success got rating
#define TM_URL_FILTER_TMUFSACI_WOULD_BLOCK 0x2 // We use the non-blocking url-filtering. So, we need to tell
					// if the receiving of the url filtering response shall be blocking.  Would go off box
// and block thread
#define TM_URL_FILTER_DRTR_MODE_OFF 0x0
#define TM_URL_FILTER_DRTR_MODE_ON 0x1
#define TM_URL_FILTER_TMUFSACI_RETURN_DENIED 0 // Rating denied
#define TM_URL_FILTER_TMUFSACI_RETURN_APPROVED 1 // Rating approved
#define TM_URL_FILTER_TMUFSACI_RETURN_BLOCK 2 // We use the non-blocking url-filtering. So, we need to tell them blocking
#define TM_URL_FILTER_TMUFSACI_RETURN_DISCARD 3 // discard the message
#define TM_URL_FILTER_TMUFSACI_RETURN_ERROR -1 // Error
/*
* String definitions for TrendMicro URL Filter Service Access Transport Protocol (TMUFSATP)
*/
#define TM_URL_FILTER_TMUFSATP_CIMP_ADR "A"
#define TM_URL_FILTER_TMUFSATP_CIMP_ADR_LEN (sizeof(TM_URL_FILTER_TMUFSATP_CIMP_ADR)-1)
#define TM_URL_FILTER_TMUFSATP_CIMP_CMR "C"
#define TM_URL_FILTER_TMUFSATP_CIMP_CMR_LEN (sizeof(TM_URL_FILTER_TMUFSATP_CIMP_CMR)-1)
#define TM_URL_FILTER_TMUFSATP_CIMP_CMR_NO_DRTR "N"
#define TM_URL_FILTER_TMUFSATP_CIMP_CMR_NO_DRTR_LEN (sizeof(TM_URL_FILTER_TMUFSATP_CIMP_CMR_NO_DRTR)-1)
#define TM_URL_FILTER_TMUFSATP_HTTP_METHOD "GET"
#define TM_URL_FILTER_TMUFSATP_HTTP_METHOD_LEN (sizeof(TM_URL_FILTER_TMUFSATP_HTTP_METHOD)-1)
#define TM_URL_FILTER_TMUFSATP_HTTP_OSP "0"
#define TM_URL_FILTER_TMUFSATP_HTTP_OSP_LEN (sizeof(TM_URL_FILTER_TMUFSATP_HTTP_OSP)-1)
#define TM_URL_FILTER_TMUFSATP_HTTP_PROTOCOL "HTTP/1.1"
#define TM_URL_FILTER_TMUFSATP_HTTP_PROTOCOL_LEN (sizeof(TM_URL_FILTER_TMUFSATP_HTTP_PROTOCOL)-1)
#define TM_URL_FILTER_TMUFSATP_HTTP_ACCEPT "Accept: */*"
#define TM_URL_FILTER_TMUFSATP_HTTP_ACCEPT_LEN (sizeof(TM_URL_FILTER_TMUFSATP_HTTP_ACCEPT)-1)
#define TM_URL_FILTER_TMUFSATP_HTTP_LANG "Accept-Language: en-us"
#define TM_URL_FILTER_TMUFSATP_HTTP_LANG_LEN (sizeof(TM_URL_FILTER_TMUFSATP_HTTP_LANG)-1)
#define TM_URL_FILTER_TMUFSATP_HTTP_AGENT "User-Agent: TMUFSATP/1.0"
#define TM_URL_FILTER_TMUFSATP_HTTP_AGENT_LEN (sizeof(TM_URL_FILTER_TMUFSATP_HTTP_AGENT)-1)
#define TM_URL_FILTER_TMUFSATP_HTTP_HOST "Host: "
#define TM_URL_FILTER_TMUFSATP_HTTP_HOST_LEN (sizeof(TM_URL_FILTER_TMUFSATP_HTTP_HOST)-1)
#define TM_URL_FILTER_TMUFSATP_HTTP_CONNECTION "Connection: close"
#define TM_URL_FILTER_TMUFSATP_HTTP_CONNECTION_LEN (sizeof(TM_URL_FILTER_TMUFSATP_HTTP_CONNECTION)-1)
#ifdef _PROXY_
#define TM_URL_FILTER_TMUFSATP_HTTP_PROXY_CONNECTION "Proxy-Connection: close"
#define TM_URL_FILTER_TMUFSATP_HTTP_PROXY_CONNECTION_LEN (sizeof(TM_URL_FILTER_TMUFSATP_HTTP_PROXY_CONNECTION)-1)
#endif
#define TM_URL_FILTER_TMUFSATP_HTTP_PORT "80"
#define TM_URL_FILTER_TMUFSATP_HTTP_PORT_LEN (sizeof(TM_URL_FILTER_TMUFSATP_HTTP_PORT)-1)
#define TM_URL_FILTER_TMUFSATP_HTTP_REQUEST "http://"
#define TM_URL_FILTER_TMUFSATP_HTTP_REQUEST_LEN (sizeof(TM_URL_FILTER_TMUFSATP_HTTP_REQUEST)-1)

/*
* URL Encoding Request Flags.
*/
#define TM_URL_FILTER_NO_URL_ENCODE 0x0
#define TM_URL_FILTER_URL_ENCODE 0x1
/*
* Service Host Information.
*/
#define TM_URL_FILTER_MAX_SERVICE_NODES 32
#define TM_URL_FILTER_SERVICE_HOST_NAME "admin.cwfservice.net"

#define TM_URL_FILTER_MAX_CAT_NUMBER 100

/*
* ----------------------------------------------------------------------------
* Type Definitions Section.
* ----------------------------------------------------------------------------
*/
typedef struct tm_url_filter_str
{
    int cs_len; // String length
    unsigned char *p_cs_str; // String text pointer
}
tm_url_filter_str;
typedef struct tmufsaci_request_meta
{
    unsigned char *p_req_vendor_id; // Vendor ID string
    unsigned char *p_req_license_id; // License id string
    unsigned char *p_req_user_id; // Optional user id string
    unsigned char *p_req_cat_list; // Optional denied sites
    // category list
    tm_url_filter_str req_origin_name; // Host name of the origin
    // server string
    tm_url_filter_str req_origin_port; // Port number on origin server
    unsigned char *p_req_origin_protocol; // Service protocol type string
    // only HTTP/1.1 is valid
    int req_drtr_mode; // DRTR mode for CMR
    unsigned char *p_req_tm_url_filter_host; // Host name of TrendMicor URL Filter
    // Service string
    struct in_addr *p_req_tm_url_filter_host_ip_addr; // Host IP address
    // of the TrendMicro URL Filter
    // Service
    unsigned short req_tm_url_filter_host_port; // Port number on TrendMicro URL Filter
    // Service
    tm_url_filter_str req_origin_url; // URL string, without protocol
    // hostname or port number
    int req_timeout; // Request timeout
    int req_deny_on_error; // Deny on error flag
   int nIsNotTooLong; // If Url is not too long
    char * szReqUrlWithoutArg; // If Url is not too long

}
tmufsaci_request_meta;
typedef struct tmufsaci_response_meta
{
    unsigned long rsp_result; // Result of request
    unsigned long rsp_code; // Additional code information
    unsigned long rsp_domr; // Domain rating
    unsigned long rsp_dirr; // Directory rating
    unsigned long rsp_domt; // Domain rating category
    unsigned long rsp_blkc; // Block category
    tm_url_filter_str rsp_info_url; // URL for information page,
    // usually block page
}
tmufsaci_response_meta;
typedef struct tmufsaci_host_ip_table
{
    int hit_num_services; /* num service points */
    int hit_num_active; /* num active service points */
    struct in_addr hit_ip_list[TM_URL_FILTER_MAX_SERVICE_NODES];
    /* list of IP addresses */
    int hit_status[TM_URL_FILTER_MAX_SERVICE_NODES];
    /* node status, up or down */
    unsigned int hit_latency[TM_URL_FILTER_MAX_SERVICE_NODES];
    /* node latency in millisecs */
}
tmufsaci_host_ip_table;
/*
* ----------------------------------------------------------------------------
* Function Prototypes Section.
* ----------------------------------------------------------------------------
*/
extern int tm_url_filter_tmufsaci_request(
        tmufsaci_request_meta *ap_req_data,
        tmufsaci_response_meta *ap_rsp_data,
        unsigned int a_req_command,
void (*pf_call_back)(tmufsaci_response_meta *ap_rsp_data,
                                        int a_result));
/*	,
        void (*pf_call_back)
        (tmufsaci_response_meta *ap_rsp_data,
         int a_result)); */
extern int tm_url_filter_tmufsaci_cmr(
        tmufsaci_request_meta *ap_req_data,
        tmufsaci_response_meta *ap_rsp_data,
        url_filter_config_t *pUrlFilterConfig);
extern int tm_url_filter_tmufsaci_check_mimetype(unsigned char *ap_url, int a_url_len);
#endif /* NOT TMUFSACIAPI */
