/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file   tmauIniParser.h
 *  @brief  TMOG ActiveUpdate (Server.ini) Parsing Module
 */

/*
 *  Change Log
 *  2004-02-14  Jeremy  initial version
 */
#ifndef INI_PARSER_H
#define INI_PARSER_H

#ifndef __KERNEL__
#include "TmCommon.h"
#include "log.h"
#endif

#define AU_INFO_SUCCESS        (TM_CONDITION(TM_ERR_O_SUCCESS,TM_ERR_F_TMSS,TM_MODULE_ID_AU_INFO,0))
#define AU_INFO_FAILURE(code)  (TM_CONDITION(TM_ERR_O_FAILURE,TM_ERR_F_TMSS,TM_MODULE_ID_AU_INFO,code))

/*
const int32_t	TM_AU_MAX_STR       = 256;
const int32_t	TM_AU_MAX_VER	    = 32;
const int32_t	TM_AU_PRODUCT_NUM   = 1;      // number of supported client products 
*/
#define TM_AU_MAX_STR       256
#define TM_AU_MAX_VER       32
#define TM_AU_PRODUCT_NUM   1

#define TMSS_AUINFO_SHMKEY     ((key_t)0x777)

typedef struct tag_Version {
    uint32_t    version;    /* major and minor*/
    uint32_t    build;      /* only for engine, or product if needed not for pattern */
    uint32_t    reserved;   /* must be zero */
} TmuVersion_t;

typedef struct tag_AUInfo {
    uint32_t        product;
    TmuVersion_t    engine;
    TmuVersion_t    pattern;
} TmuAUVersion_t;



#ifdef __cplusplus
extern "C" {
#endif
#ifndef __KERNEL__
uint32_t  tmau_MakeDWordVersion(const char* pVer);

void      tmau_MakeVersionString(uint32_t dwVersion, char* strVersion, int32_t str_len);

void      tmau_MakeVersionBuildString(TmuVersion_t ver, char *str, uint32_t str_len);

bool_t    tmau_GetPatternVer(char pszINIFile[], char *pszPatType, TmuVersion_t *pTmuVer, TmErrorCode *pErrCode);

bool_t    tmau_GetEngineVer(char pszINIFile[], char *pszEngType, TmuVersion_t *pTmuVer, TmErrorCode *pErrCode);
#endif

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif

