/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/** 
 *  @file tmdpClientInfoWrapper.h
 *  @brief TMOG Client Info Wrapper header
 */

/*
 *  Change Log
 *  2004-02-23 Jim Wu    Initial Version
 */

#ifndef UPDT_CLIENT_INFO_H
#define UPDT_CLIENT_INFO_H

#include "tmClientInfo.h"


#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief   Update the client information to client information table.
 *
 * @param   nStatus:       status of the client, refer to tmClientInfo.h. eg: STATUS_AVOK
 * @param   nIP:           client ip
 * @param   szAUProductId: client's AU product id
 * @param   szProductVer:  client's product version
 * @param   szEngineId:    client's scan engine id
 * @param   szEngineVer:   client's scan engine version
 * @param   szPatternNum:  client's pattern number
 * @param   nAuLangID:     client's AU language id
 * @param   szMac:         client's MAC address
 * @param   szHostName:    client's host name
 * @param   nLicense:      client's license is expired or not
 * 
 * @see     
 * @remarks 
 * 
 * @return  0: ok, -1: fail
 */
int tmdp_UpdateClientInfo(unsigned short nStatus, unsigned int nIP,
                          char szAUProductId, char *szProductVer, char szEngineId,
                          char *szEngineVer, char *szPatternNum, int nAuLangID, 
                          unsigned char *szMac, char *szHostName, unsigned int nLicense);

/**
 * @brief   Get the client information from client information table.
 *
 * @param   nIP:           client ip
 * @param   szAUProductId: client's AU product id
 * @param   szProductVer:  client's product version
 * @param   szEngineId:    client's scan engine id
 * @param   nAuLangID:     client's AU language id 
 * 
 * @see     
 * @remarks 
 * 
 * @return  0: ok, -1: fail
 */
int tmdp_GetClientInfo(unsigned int nIP, char *szAUProductId, 
                       char *szProductVer, char *szEngineId, char *szAULangID);

/* jeremy, last time I put it here, that might not so suitable
 * please find some other suitable header file for this
 */
int tmdp_CheckClientInfo(__u32 nClientIP, int nCommandType);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* LOG_H */
