/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/** 
 *  @file tmdpPCCProtocol.h
 *  @brief TMOG PCC Protocol header
 */

/*
 *  Change Log
 *  2004-02-23 Jim Wu    Initial Version
 */

#ifndef PCC_IPC_H
#define PCC_IPC_H

typedef struct 
{
	char szMagicNumber[4];
	char szMajorVersion[1];
	char szMinorVersion[1];
	char szCommandType[2];
	char szLength[2];
	char szReserved[2];
}DP_MSG_HEADER; /* size is 12 */

typedef struct 
{
	char szID[2];
	char szLength[2];
	/* char szIPAddr[16]; removed */
	char szNodeName[32];
	char szNodeId[32];
	char szReserved[32];
	
	char szPlatformId[4];  /*	Unknown is 0,	Windows is 0x1	Linux is 0x2 */
	char szProductId[8];  /* 0x2 here for TMSS */
	char szCompID[8];		
}DP_QUERY_MSG_COMP;  /* size is 120 */

typedef struct 
{
	char szID[2];
	char szLength[2];
	/* char szIPAddr[16]; removed */
	char szNodeName[32];
	char szNodeId[32];
	char szReserved[32];
	
	char szP2PProtocol[2]; /* no use */
	char szP2PPort[2];     /* no use */
	char szPlatformId[4];  /*	Unknown is 0,	Windows is 0x1	Linux is 0x2 */
	
	char szPlatformInfo[16]; /* variable for IHV */	
	
	char szProductId[8];  /* 0x2 here for TMSS */
		
	/* The rest 128 bytes are Product Information,*/
	char szBoxId[16];		
	char szVendorId[16];	
	char szAUProductId[4];	 /*TMSS should use 0x0  yas2 83 - PCC11	 */
	char szAULangID[4];		 /* 1 - EN,  2 - TW,  4 - JP */
	char szProductVer[16];	/* for PCC 11 83*/
	char szEngineId[4];		/* TMSS use 0x10 */
	char szEngineVer[16];   /* TMSS fill engine version  */
	char szPatternNum[12];  /* TMSS fill pattern version  */
	char szReserved2[40];	
	
	/* component information */
	char szCompID[8];			
	char szCompinfo[32];	 /* need this field, even there is no component */
}DP_RESPONSE_MSG_COMP;  /* size is 300 */


typedef struct 
{
	char szID[2];
	char szLength[2];
	/* char szIPAddr[16]; removed */
	char szNodeName[32];     /* host name */
	char szNodeId[32];       /* MAC address, eg: MAC=00 0C 29 20 B5 53 */
	char szReserved[32];
	
	char szP2PProtocol[2];   /* no use */
	char szP2PPort[2];       /* no use */
	char szPlatformId[4];    /*	Unknown is 0,	Windows is 0x1	Linux is 0x2 */
	
	char szPlatformInfo[16]; /* variable for IHV */	
	
	char szProductId[8];     /* eg: szProductId[7] = 0x1 */
		
	/* The rest 128 bytes are Product Information,*/	
	char szProductVer[16];	 /* for PCC, eg: 11.00.9999 */
	char szGUID[15];         /* GUID of PCC */
	char szReserved2[1];     
	char szProductFalg[1];   /* 0-normal, 1-TMSS */
	char szProductState[1];  /* 00 not expired, 01: 30 days left, 10: 15 days left, 11 expired */
	char szReserved3[2]; 
	char szAUProductId[4];	 /*TMSS should use 0x0  yas2 83 - PCC11	 */
	char szReserved4[88];		
	
	/* component information */
	char szCompID[8];		 /* component id: szCompID[7] = 1 */
	
	char szEngineId[4];		/* TMSS use 0x0 eg: szEngineId[3] = 16 */
	char szEngineVer[16];   /* TMSS fill 0x0  eg: 6.810.1005*/
	char szPatternNum[12];  /* TMSS fill 0x0  eg: 174300*/
}DP_PCC_RESPONSE_MSG_COMP;  /* size is 300 */


#ifdef __cplusplus
extern "C" {
#endif

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* LOG_H */
