/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/** 
 *  @file tmShareMem.c
 *  @brief TMOG Request Module
 */

/*
 *  Change Log
 *  2004-02-18 Jim Wu    Initialization Version
 */

/* for share memory */
#include <stdio.h>
#include <signal.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <errno.h>
#include "tmdpCommonDef.h"  /* common type define */
#include "tmrqRequestQueue.h"
#include "log.h"

void * g_shared_memory;

static const char ModuleName[]="sharemem";

int SetupshareMem(int nReset)
{	
    int nShmid = 0;
    /* set up the size to be total of queue size + queue head + queue tail index */
    int nSharedSize = sizeof(MESSAGE_ITEM) * (REQUEST_QUEUE_SIZE) + 
                      sizeof(int) *2;
                      
     LOGINFO("[setup sharemen] nSharedSize = %d", nSharedSize);
    	
    /*nShmid = shmget((key_t)SHARE_KEY, nSharedSize, 0666 | IPC_CREAT);*/
    
    if ((nShmid = shmget((key_t)SHARE_KEY, nSharedSize, 0)) == -1)
    {
        LOGINFO("[setup sharemen] shmget = -1");
        nShmid= shmget((key_t)SHARE_KEY, nSharedSize, IPC_CREAT | IPC_EXCL | 0666);        
        
        LOGINFO("[setup sharemen] shmget = %d", nShmid);
    }
    
    if (nShmid == -1)
    {
       if (errno == EEXIST)  
       {
          LOGERROR("[setup sharemen error] shmat error. errno=EEXIST");
          if (shmdt(g_shared_memory) == -1)
          {
              LOGERROR("shmdt fail\n");
              return -1;
          }
    	
          if (shmctl(nShmid, IPC_RMID, 0) == -1)
          {
              LOGERROR("delete share memory. shmctl(IPC_RMID) fail\n");
              return -1;
          }
       }
       else if (errno ==  EINVAL)
       {
          LOGERROR("[setup sharemen error] shmat error. errno=EINVAL");
       }
       else if (errno ==  EIDRM )
       {
          LOGERROR("[setup sharemen error] shmat error. errno=EIDRM");
       }
       else if (errno ==  ENOSPC )
       {
          LOGERROR("[setup sharemen error] shmat error. errno=ENOSPC");
       }
       else if (errno ==  ENOENT )
       {
          LOGERROR("[setup sharemen error] shmat error. errno=ENOENT");
       }
       else if (errno ==  EACCES  )
       {
          LOGERROR("[setup sharemen error] shmat error. errno=EACCES ");
       }
       else if (errno ==  ENOMEM   )
       {
          LOGERROR("[setup sharemen error] shmat error. errno=ENOMEM");
       }
       else
       {       
          LOGERROR("[setup sharemen error] shmat error. errno=%d", errno);
       }
       return -1;       
    }
    
    g_shared_memory = shmat(nShmid, (void *)0, 0);
    /*g_shared_memory = shmat(nShmid, (const void *)0x47000000, 0);    */
    
    LOGDEBUG("[discovery] g_shared_memory = %p", g_shared_memory);
	
    if (g_shared_memory == (void *)-1)
    {
       LOGERROR("[discovery] shmat fail\n");
       return -1;
    }
	
    /*LOGDEBUG("Memory attached at %X\n", (int)g_shared_memory);*/    
    tmrq_SetQueueHead((int *)(g_shared_memory));
    tmrq_SetQueueTail((int *)(g_shared_memory + sizeof(int)));
    tmrq_SetMsgItemHead((MESSAGE_ITEM *)(g_shared_memory + sizeof(int) *2));	

    /* LOGDEBUG("reset shared memory\n");	*/
    if (nReset == 1)
    {
       LOGINFO("[discovery] reset queue");
       tmrq_ResetQueue();
    }
	
	LOGINFO("leaving SetupshareMem");
	
    return nShmid;
}

int DetachshareMem(int nShmid)
{
    LOGINFO("Entering DetachshareMem");
    
    /* cause error in daemon */
    if (nShmid == -1)
    {
       return 0;
    }	
		
    if (g_shared_memory != (void *)-1)
    {
       if (shmdt(g_shared_memory) == -1)
       {
          LOGERROR("shmdt fail\n");
          return -1;
       }
	
       if (shmctl(nShmid, IPC_RMID, 0) == -1)
       {
          LOGERROR("delete share memory. shmctl(IPC_RMID) fail\n");
          return -1;
       }
       
       LOGINFO("Leaving DetachshareMem");
		
       return 0;
    }
    
    LOGERROR("g_shared_memory == -1, Leaving DetachshareMem");
	
    return -1;
}

int SignalTMDPDaemon(int nPId, int nSignal)
{
    /* send a signal to other process with pid = nPId, and signal id = nSignal */
    return kill(nPId, nSignal);	
}

int ReadPIdTemp(void)
{
    // read pid from file
    FILE *fp;
    char line[256] = {0};
    int pid = 0;
	
    fp = fopen(TempPIdPath, "r");
	
    if ((FILE *)0 == fp) 
    {
       LOGERROR("fail to open file\n");
       return -1; // no file found or fopen error
    }
	
    /* read the tmp line */
    if ((char *)0 == fgets(line, sizeof(line), fp))
    {
       fclose(fp);
       LOGERROR("fail to read file\n");
       return -1; // no information inside the file
    }
	
    fclose(fp);	

    pid =0;
	
    pid = atoi(line);
	
    return pid;	
}
