/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/** 
 *  @file tmnfRequestInterface.c
 *  @brief TMOG Request Module
 */

/*
 *  Change Log
 *  2004-02-18 Jim Wu    Initialization Version
 */

#include <stdio.h>
#include <signal.h>
#include <stdlib.h>
#include <string.h>
#include <arpa/inet.h> 
#include <sys/types.h>
#include <unistd.h>

#include "tmdpCommonDef.h"  /* common type define */
#include "tmrqRequestQueue.h"
#include "log.h"

extern int SetupshareMem(int nReset);
extern int DetachshareMem(int nShmid);
extern int SignalTMDPDaemon(int nPId, int nSignal);
extern int ReadPIdTemp(void);

/* global variables */
int g_bInit = 0; 
int g_nPId = -1;
int g_nShmid = -1;		
 
/* internal function */ 

static const char ModuleName[]= "reqinterface";

int Init(void)
{
    /* 1. get the pid of tmpd for send signal 
       2. check if share memory is ready */
	   
    g_nPId = ReadPIdTemp();
 	
    if (g_nPId < 0)
    {
       /* fail to get pid */
       return 0;
    }	
	
    return 1;
} 
 
static int UpdateInfo(__u32 nClientIP, int nCommandType)   
{
    /*
      Maintain a first in, first out request queue. 
      The queue size should be smaller than the size of Client Information Table.  
      When there is a new request, it will check if the request exist in the request queue,
      If yes, just drop the request
      If not, then it will first update the client information, add a new record and set
      the status to "In detecting", then send a message to discovery protocol handler to
      compose and send DQM to PCC client.
      After that, the client information table will be updated by discovery protocol handler.
      If the queue is full, it will return a error number. (define in error.h).
    */		
    /* construct a message item */
    MESSAGE_ITEM messageItem;    
    int bIsExist;
    int nRet;
    int nRetKill;
		
    if (!g_bInit)
    {
       /* initialize */
       g_bInit = Init();
			
       g_nShmid = SetupshareMem(0);
    }	
		
    if (g_nPId == -1)
    {
       LOGERROR("(%d) discovery moduel daemon is not running !", getpid());
       return 0;
    }
		
    inet_ntop(AF_INET, &nClientIP, messageItem.szClientIP, 20); 	  
		
    messageItem.nCommandType = nCommandType;		
    
    LOGDEBUG("(%d) UpdateInfo, IP is %s, type is %d\n", getpid(), messageItem.szClientIP, nCommandType);
		
    /* check if there is duplicate item */
    /* LOGDEBUG("before check message item ip is %d, commandType is %d\n", messageItem.nClientIP, messageItem.nCommandType);*/

    bIsExist = tmrq_IsItemExist(&messageItem); 
		
    if (bIsExist)
    {
       /* item already exist, just return */
       LOGINFO("(%d) Client IP %s already exist, drop the request", getpid(), messageItem.szClientIP);
       return 0;
    }
		
    /* insert item in the request queue */
    nRet = tmrq_InsertQueue(&messageItem);
		
    if (nRet != 0)
    {
        /* fail to insert item to queue, return the error number */
        return nRet;
    }
		
    /* send a signal to discovery protocol handler */
    nRetKill = 0;
    nRetKill = SignalTMDPDaemon(g_nPId, SIGUSR2);

     LOGDEBUG("send signal to pid = %d, nRetKill = %d\n", g_nPId, nRetKill);		
	
   return 0;
}

/* interface to notify PCC Communication module to check PCC client info
   return 0 for OK, -1 for error  */   
int tmnf_UpdateClientInfo(__u32 nClientIP)
{
    LOGDEBUG("(%d) Sending Discover Information Request to PCC client", getpid()); 
		  
    return UpdateInfo(nClientIP, UPDATE_CLIENT_INFO);		
}

/* interface to notify PCC Communication module to notify PCC client to update
   pattern and engine. return 0 for OK, -1 for error  */
int tmnf_UpdateClientPattern(__u32 nClientIP)
{		
    LOGDEBUG("(%d) Sending Update Pattern/Engine Notification to PCC client", getpid());
		
    return UpdateInfo(nClientIP, UPDATE_PATTERN_ENGINE);		  		
}

int tmnf_TerminateDaemon(void)
{
    int nRetKill;
    if (!g_bInit)
    {
       /* initialize */
       g_bInit = Init();
    }
		
    if (g_nPId == -1)
    {
        LOGERROR("(%d) discovery moduel daemon is not running !", getpid());
        return 0;
    }
		
    /* LOGDEBUG("before send signal to pid = %d\n", g_nPId);		*/
		
    LOGINFO("(%d) *** Terminating the PCC communication daemon ! ***", getpid());
    nRetKill = 0;
    nRetKill = SignalTMDPDaemon(g_nPId, SIGUSR1);		
		
    /* LOGDEBUG("send signal to pid = %d, nRetKill = %d\n", g_nPId, nRetKill);	*/
	
    DetachshareMem(g_nShmid);
		
    return nRetKill;
}
