#!/bin/sh


start() {

    add_exception

    echo "Start up HTTP Proxy ..."
    ${BINDIR}/thpd ${BINDIR}/tm.conf &

    echo "Start up Discovery ..."
    ${BINDIR}/tmdpd &

    echo "Start up AU Checker ..."
    ${BINDIR}/tmAUChecker ${BINDIR}/tm.conf &

    echo "Start up mini HTTPD ..."
    ${BINDIR}/mini_httpd ${MINI_ARG} &
}

reload()
{
    echo -n $"Reloading: "
    add_exception
    if [ -f $THPD_PIDFILE ]; then
       kill -HUP `cat $THPD_PIDFILE` 
       RETVAL=$?
       if [ "${RETVAL}" != 0 ]; then
          fail
       else
          ok
       fi
    else
       fail
    fi

    if [ -f $AU_PIDFILE ]; then
       kill -HUP `cat $AU_PIDFILE`
       RETVAL=$?
       if [ "${RETVAL}" != 0 ]; then
          fail
       else
          ok
       fi
    else
       fail
    fi

}


ok() {
        echo -e "[60G[  [0;32mOK[0;39m  ]"
}

fail() {
        echo -e "[60G[  [0;31mFAIL[0;39m  ]"
}

stop() {

    killall thpd
    killall tmdpd
    # replace "killall" to "kill -9 PID_NUM" to solve the unkillable case
    #killall tmAUChecker
    if [ -f $AU_PIDFILE ]; then
        kill -9 `cat $AU_PIDFILE`
        # The shared memory key, 0x777, is defined in tmauIniParser.h
        ipcrm -M 0x777
        RETVAL=$?
        if [ "${RETVAL}" != 0 ]; then
            fail
        else
            rm -f $AU_PIDFILE
            ok
        fi
    else
        fail
    fi

    killall mini_httpd
}

THIS=`basename $0`
BINDIR=`echo $0 | sed "s/\/$THIS\$//"`
if [ "$BINDIR" = "" ]; then
    BINDIR="."
fi
THPD_PIDFILE="/var/run/thpd.pid"
AU_PIDFILE="/var/run/tmAUChecker.pid"

###########################################################
# mini_httpd settings
###########################################################
MINI_ARG="-d ${BINDIR}/../www -c trend/* -p 80"
MINI_ARG="${MINI_ARG} -i /var/run/mini_httpd.pid"
MINI_ARG="${MINI_ARG} -l /var/log/mini_httpd.log -u root -D"


host_filter () {

  URL=$1

  #Remove http://
  P1=`echo $URL | sed 's/^.*:\/\/\(.*\)/\1/'`

  #Remove URI
  P2=`echo $P1  | sed 's/\/\(.*\)$//'`

  #Remove port
  P3=`echo $P2  | sed 's/^\(.*\):[0-9]*/\1/'`

  echo $P3
}

add_exception() {

    FILE=`${BINDIR}/tmcfg ${BINDIR}/tm.conf val exception path_avedst`
    AV=`${BINDIR}/tmcfg ${BINDIR}/tm.conf val avenforcer url_avserver`
    AU=`${BINDIR}/tmcfg ${BINDIR}/tm.conf val au_checker au_server_url`
    H1=`host_filter $AV`
    H2=`host_filter $AU`
    ${BINDIR}/tmHostent  $H1 > $FILE
    ${BINDIR}/tmHostent  $H2 >> $FILE
}


case "$1" in
    start)
           start
           ;;
    stop)
           stop
           ;;
    reload)
           reload
           ;;
    restart)
           stop
           start
           ;;
    *)
           echo $"Usage: $0 {start|stop|reload|restart}"
           exit 1
esac

exit $?
