/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file tmGetHardwareInfo.c
 *  @brief TMSS Get Hardware Information
 */

/*
 *  Change Log
 *  2004-05-19 Henry Lin    Initialization version
 */

#include <string.h>
#include <net/if.h> 
#include <netinet/in.h>
#include <sys/ioctl.h> /* for ioctl() */
#include <stdio.h>
#include "tmMd5.h"
#include "tmGetHardwareInfo.h"
#include "log.h"

static const char *ModuleName = "tmGetHardwareInfo";

/**
  * @brief Gets MAC address from system.
  *
  * @see 
  * @return 
  * @remarks 
  */
  
/* assume eth1 is the LAN port, need hardware vendor to customerize this, and get the MAC addrss of WAN port */
//#define DEFAULT_IF  "eth1"
// For ADSL, we take lAN port.
#define DEFAULT_IF  "eth0"  

int GetMACAddress(char *mac, size_t size) 
{
    /*
     * In embedded systems, there might be some other way to get MAC address
     * from system. Here, we abstract the implemtation for cross-platform
     * compatibility.
     */
    int retval;
    int sock;
    struct ifreq ifr;
    
    LOGINFO("Entering GetMACAddress");

    sock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
    memset(&ifr, 0, sizeof(struct ifreq));
    strcpy(ifr.ifr_name, DEFAULT_IF);
    retval = ioctl(sock, SIOCGIFHWADDR, &ifr);
    if(retval < 0) 
    {
        /* cannot invoke ioctl succefully */
        LOGERROR("cannot invoke ioctl succefully ");
        return -1;
    }
    
    /* mac contains terminating character, so size should be 13 */
    snprintf(mac, size, "%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X",
            (unsigned char)ifr.ifr_hwaddr.sa_data[0],
            (unsigned char)ifr.ifr_hwaddr.sa_data[1],
            (unsigned char)ifr.ifr_hwaddr.sa_data[2],
            (unsigned char)ifr.ifr_hwaddr.sa_data[3],
            (unsigned char)ifr.ifr_hwaddr.sa_data[4],
            (unsigned char)ifr.ifr_hwaddr.sa_data[5]);
         
/////sprintf(mac,"000EA61380AF");
   LOGINFO("mac is %s", mac);
             
    return 0;
}

int tmGetBoxID(char *szBoxId)
{   
    char szMac[13] = "";
    char szTempBoxId[17] = "";
    
    LOGINFO("Entering tmGetBoxID");
    
    /* get mac address, HW vendor needs to port this function */
    GetMACAddress(szMac, 13);
    
    /* make checksum based on mac address */
    MakeCheckSum(szMac, szTempBoxId);
    
    strncpy(szBoxId, szTempBoxId, TMSS_MAX_BUF_BOX_ID - 1);    
    
    szBoxId[TMSS_MAX_BUF_BOX_ID - 1] = '\0';
        
    return 0;
}

int tmGetVendorID(char *szVendorId)
{
    strncpy(szVendorId, TMDP_Vendor_ID, TMSS_MAX_BUF_VENDER_ID - 1);
    
    return 0;
}

int tmGetBoxIp(char *szBoxIp)
{
    strncpy(szBoxIp, TM_BOX_CONFIG_URL, TMSS_MAX_BUF_BOX_IP - 1);

    return 0;
}

int tmGetBoxVersion(char *szBoxVersion)
{
    strncpy(szBoxVersion, TM_BOX_FIRMWARE_VERSION, TMSS_MAX_BUF_BOX_VERSION - 1);

    return 0;
}

