/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file tmSemaphore.c
 *  @brief TMOG Semaphore Module
 */

/*
 *  Change Log
 *  2004-02-19 Henry Lin    Initialization version
 */


#include <stdio.h>   
#include "tmSemaphore.h"
#include "log.h"
static const char ModuleName[]="tm_semaphore";

/* #define LOGERROR(fmt, args...) fprintf(stderr, fmt, ## args)   >>> for independent module test only */
static struct sembuf op_lock = { 0, -1, SEM_UNDO };	/* then increment sem#0 by 1 */
static struct sembuf op_unlock = {0, 1, SEM_UNDO};	/* decrement sem#0 by 1 (sets it to 0) */

int tm_lock_create(key_t key, struct tm_sem_st *sem)/*tm_lock_t sem)*/
{
    int status = 0;
    union semun arg;

    if(sem==NULL) return -1;

    sem->semkey=key;
    arg.val=SEM_CAPACITY; /* set here not to increase period of dangerous zone */
    if ((sem->semid = semget(sem->semkey, 1, SEMPERM|IPC_CREAT|IPC_EXCL)) == -1) 
    {/* add IPC_EXCL has reviewed by Arthur, Oct 17, 2001; PS: final decision is made by Edward, to remove 
    	IPC_EXCL temporarily because performance issue. Without IP_EXCL, it seems not harmful. So it would
    	be like that. */
    	if (errno == EEXIST)  
    	{
    		sem->semid = semget(sem->semkey, 1, 0);
    	}
    }
    else /* to be improve for race condition */
    {
	    status = semctl(sem->semid, 0, SETVAL, arg);
    }

    if (sem->semid == -1 || status == -1) 
    {
        LOGERROR("Semaphore initialization failed");
        return -1;
    } 
    else
    {
	    return sem->semid;
    }
}

int tm_lock(struct tm_sem_st *sem)/*tm_lock_t sem)*/
{ /* wait on a semaphore */
    if (semop(sem->semid, &op_lock, 1) == -1) 
    {
       LOGERROR("Semaphore lock error occurs. errno: %d",errno);
       return -1;
    } 
    else
       return 0;
}

int tm_unlock(struct tm_sem_st *sem)/*tm_lock_t sem)*/
{ /* post to a semaphore */ 
    if (semop(sem->semid, &op_unlock, 1) == -1) 
    {
       LOGERROR("Semaphore unlock error occurs. errno: %d", errno);
       return -1;
    } 
    else
       return (0);
}
	
void tm_lock_close(struct tm_sem_st *sem)/*tm_lock_t sem)*/        
{
    union semun arg;
    int nSemncnt;
    
    arg.val=0;
    nSemncnt = semctl(sem->semid, 0, GETNCNT, arg);
    if (nSemncnt == -1)
    {
        LOGERROR("Get semncnt fail, errno: %d", errno);
    }
    
    /* if there is any process waiting for this lock, do not close it */
    if (nSemncnt == 0 || nSemncnt == -1)
    {
        semctl(sem->semid, 0, IPC_RMID, arg);
    }
}


