/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file tmShm.c
 *  @brief TMOG Shared Memory Module
 */

/*
 *  Change Log
 *  2004-02-17 Henry Lin    Initialization version
 */

#include <sys/shm.h>
#include <sys/ipc.h>
#include <string.h>
#include <stdlib.h>
#include "log.h"
#include "tmShm.h"

#define TMSS_SHMFLG          (IPC_CREAT | IPC_EXCL | 0666)

static const char ModuleName[] = "tmShm";

int shm_init(key_t key, int size)
{
    int shmid = -1;

    if ((shmid = shmget(key, size, 0)) == -1)
    {
        shmid = shmget(key, size, TMSS_SHMFLG);
    }

    return shmid;
}

void *shm_attach(int shmid)
{
    void *addr = (void *)shmat(shmid, 0, 0);

    if ((void *)addr == (void *)-1)
        return NULL;

    return addr;
}

int shm_detach(void *addr)
{
    if (addr == NULL)
        return 0;

    if (shmdt(addr) == -1)
        return -1;

    return 0;
}

int shm_fini(int shmid)
{
    /* Mark as DELETE, so that after all process detachment, deleted */
    return shmctl(shmid, IPC_RMID, NULL);
}
