/******************************************************************************
*
*  (C) Copyright 2004 Trend Micro, Inc.
*
*  All Rights Reserved.
*  This program is an unpublished copyrighted work which is proprietary
*  to Trend Micro, Inc. and contains confidential information that is not
*  to be reproduced or disclosed to any other person or entity without
*  prior written consent from Trend Micro, Inc. in each and every instance.
*
*  WARNING:  Unauthorized reproduction of this program as well as
*  unauthorized preparation of derivative works based upon the
*  program or distribution of copies by sale, rental, lease or
*  lending are violations of federal copyright laws and state trade
*  secret laws, punishable by civil and criminal penalties.
*
*****************************************************************************/

/**
*  @tmSemaphoreTest.c
*  @TMSS semaphore module testing
*/

/*
*  Change Log
*  2004-02-20 Henry Lin    Initialization version
*/

#include <stdio.h>
#include "tmURLFilter.h"
#include "time.h"
#include "tmURLFilterStatistic.h"

#define HTML_PAGE_START \
"<html>"\
"<head>"\
"<title>URL Filtering Statistic Report </title>"\
"</head>"\
"<body>"\
"<table border=\"1\" width=\"100%%\">"\
"  <tr>"\
"    <td width=\"20%%\"></td>"\
"    <td width=\"20%%\">Access Times</td>"\
"    <td width=\"20%%\">Last Access Time</td>"\
"    <td width=\"20%%\">Block Times</td>"\
"    <td width=\"20%%\">Last Block Time</td>"\
"  </tr>"\
"  <tr>"\
"    <td width=\"20%%\">Pornography</td>"\
"    <td width=\"20%%\"> %d </td>"\
"    <td width=\"20%%\"> %04d/%02d/%02d %2d:%02d:%02d </td>"\
"    <td width=\"20%%\"> %d </td>"\
"    <td width=\"20%%\"> %04d/%02d/%02d %2d:%02d:%02d </td>"\
"  </tr>"

#define HTML_PAGES \
"  <tr>"\
"    <td width=\"20%%\">%s</td>"\
"    <td width=\"20%%\"> %d </td>"\
"    <td width=\"20%%\"> %04d/%02d/%02d %2d:%02d:%02d </td>"\
"    <td width=\"20%%\"> %d </td>"\
"    <td width=\"20%%\"> %04d/%02d/%02d %2d:%02d:%02d </td>"\
"  </tr>"



#define HTML_PAGE_END \
"<br>The records have been recorded since %04d/%02d/%02d %2d:%02d:%02d <br>"\
"</body>"\
"</html>"

extern const char *UrlFilterCategoryList[HV_DEFINED_CAT_NUM];

int main(void)
{
    url_filter_statistic_t Report[HV_DEFINED_CAT_NUM];
    int i;
    struct tm *access;
    struct tm *block;
    time_t StartTime;
    struct tm *start;
    
    tmGetURLStatisticCount(Report, sizeof(url_filter_statistic_t)*HV_DEFINED_CAT_NUM, &StartTime);

    printf("Content-type: text/html\r\n\r\n");
    
    access = localtime(&Report[0].LastAccessTime);
    block = localtime(&Report[0].LastBlockedTime);
    printf(HTML_PAGE_START,
      Report[0].nAccessCount,
      access->tm_year + 1900, access->tm_mon+1, access->tm_mday,access->tm_hour, access->tm_min, access->tm_sec,
      Report[0].nBlockedCount,
      block->tm_year + 1900, block->tm_mon+1, block->tm_mday,block->tm_hour, block->tm_min, block->tm_sec);
      
    for ( i = 1; i < HV_DEFINED_CAT_NUM - 1; i++)
    {
        access = localtime(&Report[i].LastAccessTime);
        block = localtime(&Report[i].LastBlockedTime);
        printf(HTML_PAGES,UrlFilterCategoryList[i],
            Report[i].nAccessCount,
            access->tm_year + 1900, access->tm_mon+1, access->tm_mday,access->tm_hour, access->tm_min, access->tm_sec,
            Report[i].nBlockedCount,
            block->tm_year + 1900, block->tm_mon+1, block->tm_mday,block->tm_hour, block->tm_min, block->tm_sec);
    }

    start = localtime(&StartTime);
    printf(HTML_PAGE_END,
      start->tm_year + 1900, start->tm_mon+1, start->tm_mday,start->tm_hour, start->tm_min, start->tm_sec);
      
    printf("\n");
    printf("<!-- ");
    for (i = 0; i < HV_DEFINED_CAT_NUM - 1; i++)
    {
        printf("Category %d Access %d times\n",i,Report[i].nAccessCount);
        access = localtime(&Report[i].LastAccessTime);
        printf("Category %d Last Access Time: %04d/%02d/%02d %2d:%02d:%02d\n",i,access->tm_year + 1900, access->tm_mon+1, access->tm_mday,access->tm_hour, access->tm_min, access->tm_sec);
        printf("Category %d Block %d times\n",i,Report[i].nBlockedCount);
        block = localtime(&Report[i].LastBlockedTime);
        printf("Category %d Last Block Time: %04d/%02d/%02d %2d:%02d:%02d\n",i,block->tm_year + 1900, block->tm_mon+1, block->tm_mday,block->tm_hour, block->tm_min, block->tm_sec);
    }
    start = localtime(&StartTime);
    printf("The records have been recorded since %04d/%02d/%02d %2d:%02d:%02d\n", start->tm_year + 1900, start->tm_mon+1, start->tm_mday,start->tm_hour, start->tm_min, start->tm_sec);
    printf("\n -->");

    return 0;
}

