/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file tmSocketClient.c
 *  @brief TMOG URL Filtering Socket Module
 */

/*
 *  Change Log
 *  2004-02-17 Henry Lin    Initial Version
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <netinet/tcp.h>
#include "tmSocketClient.h"
#include "log.h"

static const char ModuleName[] = "tmSocketClient";

int tm_url_filter_tcp_connect(const struct in_addr * pAddr, unsigned short port)
{
    struct sockaddr_in saddr;
    int rc = 0;
    int sd = socket( AF_INET, SOCK_STREAM, 0 );
    if (0 > sd)
    {
        LOGERROR("tm_url_filter_tcp_connect(), error! socket() return < 0");
        return TM_URL_FILTER_SOCK_FAILURE;
    }

    // Asynchornize socket setting from Cheton
    // int optval = 1;
    // ioctl(sd, FIONBIO, &optval);

    memset(&saddr, 0, sizeof(saddr));
    saddr.sin_family = AF_INET;
    saddr.sin_port = htons(port);
    saddr.sin_addr = *pAddr;

    rc = connect(sd, (const struct sockaddr *)&saddr, sizeof(saddr));

    if (0 > rc)
    {
        close(sd);
        LOGERROR("tm_url_filter_tcp_connect(), error! connect() return < 0");
        return TM_URL_FILTER_SOCK_FAILURE;
    }
    else
    {
	    return sd;
    }
}

int tm_url_filter_sock_srv_write(int sd, const unsigned char * pData, int dataSize, long timeout)
{
    struct timeval tv;
    int rc;
    fd_set writefd;
    tv.tv_sec = timeout;
    tv.tv_usec = 0;

    FD_ZERO(&writefd);
    FD_SET(sd, &writefd);

    // Wait for the timeout period until the socket is ready to send

    rc = select(sd+1, NULL, &writefd, NULL, &tv);

    if (0 >= rc)
    {
        LOGERROR("tm_url_filter_sock_srv_write(), error! select() return <= 0");
        return TM_URL_FILTER_SOCK_FAILURE;
    } 

    rc = send(sd, (const char *)pData, dataSize, 0);
    if (0 > rc)
    {
        LOGERROR("tm_url_filter_sock_srv_write(), error! send() return < 0");
        return TM_URL_FILTER_SOCK_FAILURE;
    }
    else
    {
        if(rc!=dataSize)
	{
		LOGERROR("tm_url_filter_sock_srv_write(), can only write rc=%d but datasize=%d",rc,dataSize);
	}
         return rc;
    }
}

// Only for url filter liecense
int tm_url_filter_sock_srv_read(int sd, unsigned char * pOutputBuffer, int bufferSize, long timeout)
{
    struct timeval tv;
    int rc;
    fd_set readfd;
    int offset;

    tv.tv_sec = timeout;
    tv.tv_usec = 0;

    offset = 0;

    do
    {
        FD_ZERO(&readfd);
        FD_SET(sd, &readfd);

        // Wait for the timeout period until there is data available
        rc = select(sd+1, &readfd, NULL, NULL, &tv);

        if (0 == rc)
        {
            LOGERROR("tm_url_filter_sock_srv_read(), error! select() return = 0, wait packet timeout");
            return TM_URL_FILTER_SOCK_FAILURE;
        }
        else if (0 > rc)
        {
            LOGERROR("tm_url_filter_sock_srv_read(), error! select() return < 0");
            return TM_URL_FILTER_SOCK_FAILURE;
        }
        
        if (FD_ISSET(sd, &readfd))
        {
            rc = recv(sd, (char *)pOutputBuffer + offset, bufferSize - offset, MSG_WAITALL);
            
            // If it's 0, then the socket was closed.  This indicates completion of the response
            // or an error if no data has been received yet
            if (0 == rc)
            {
                FD_CLR(sd, &readfd);
                if (0 == offset)
                {
                    LOGERROR("tm_url_filter_sock_srv_read(), error! get packet offset = 0");
                    return TM_URL_FILTER_SOCK_FAILURE;
                }
                else
                {
                    return offset;
                }
            }
            else if (0 > rc)
            {
                FD_CLR(sd, &readfd);
                LOGERROR("tm_url_filter_sock_srv_read(), error! recv() return < 0");
                return TM_URL_FILTER_SOCK_FAILURE;
            }
            
            offset += rc;
            
            if (offset >= bufferSize)
            {
                FD_CLR(sd, &readfd);
                LOGERROR("tm_url_filter_sock_srv_read(), error! buffer size not enough for packet");
                return TM_URL_FILTER_SOCK_FAILURE;
            }
        }
    }
    while (1);
}

