/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file tmURLCache.c
 *  @brief TMOG URL Filtering Cache Module
 */

/*
 *  Change Log
 *  2004-02-13 Henry Lin    Initialization version
 */

//#define NDEBUG
#ifndef TM_NO_URL_FILTERING_CACHE

#include <string.h>
#include "log.h"
#include "tmURLCache.h"
#include "tmShm.h"
#include "tmSemaphore.h"
#include "tmURLFilter.h"

static const char ModuleName[] = "tmURLCache";

/**
  * @brief concatenate the host name with url and remove the optional arguments
  * @param pszHost input - host name
  * @param nHostLen input - host name length
  * @param pszUrl input - url
  * @param nUrlLen input - url length
  * @param szReqUrlWithoutArg output - url
  * @param nUrlBufLen input - output url buffer length
  * @return URL_OK concat successful
  * @return URL_TOO_LONG url too long
  */
int RemoveOptionalArgInURL(
    const char *pszHost,
    int nHostLen,
    const char *pszUrl,
    int nUrlLen,
    char *szReqUrlWithoutArg,
    int nUrlBufLen)
{
    int nBufTempIndex = 0;
    int i;

    LOGDEBUG("Entering RemoveOptionalArgInURL()");

    if (nHostLen > (nUrlBufLen - 1))
    {
        LOGDEBUG("Host name too long, host name = %s, host length = %d", pszHost, nHostLen);
        szReqUrlWithoutArg[0] = '\0';
        return URL_TOO_LONG;
    }

    for (i = 0; i < nHostLen; i++)
    {
        szReqUrlWithoutArg[nBufTempIndex++] = pszHost[i];
    }

    for (i = 0; i < nUrlLen; i++)
    {
        if (pszUrl[i] == '?')
        {
            break;
        }
        szReqUrlWithoutArg[nBufTempIndex++] = pszUrl[i];
        if (nBufTempIndex >= nUrlBufLen)
        {
            LOGDEBUG("URL too long, url = %s , url length = %d", pszUrl, nUrlLen);
            szReqUrlWithoutArg[0] = '\0';
            return URL_TOO_LONG;
        }
    }
    
    szReqUrlWithoutArg[nBufTempIndex] = '\0';
    
    return URL_OK;
}



#endif //#ifndef TM_NO_URL_FILTERING_CACHE
