/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file tmURLFilter.c
 *  @brief TMOG URL Filtering Module
 */

/*
 *  Change Log
 *  2004-01-08 Henry Lin    Initialization version
 */


//#undef NDEBUG
#include <string.h>
#include <signal.h>
#include <setjmp.h>
#include <unistd.h>
#include <stdio.h> /* sprintf() */
#include <stdlib.h> /* atoi() */
#include <netdb.h> /* gethostbyaddr() */
#include <time.h>
#include "log.h"
#include "tmURLRater.h"
#include "TmCommon.h"
#include "tmClientInfo.h"
#include "tmFunction.h"
#include "tmURLFilter.h"
#include "tmURLFilterConf.h"
#include "tmURLFilterStatistic.h"
#include "tmCGIErrMsg.h"

#ifndef TM_NO_URL_FILTERING_CACHE
#include "tmURLCache.h"
#endif
#include <errno.h>
#include <linux/netfilter_ipv4/ipt_webstr.h>
#include <sys/socket.h>
#include <arpa/inet.h>

#include <unistd.h>
#include <stdlib.h>

static const char ModuleName[] = "tmURLFilter";

/*
   To change HV Defined Categories, change the followings:
   1. tmURLFilter.h: All HV defined Categories
   2. tmURLFilter.c: *gszAllHVCat
   3. tmURLFilter.c: gTmUrlFilterDefinedCatMap[]
   4. tmURLFilter.c: UrlFilterCategoryList[]
*/

/* 
   Put all HV defined Categories in gszAllHVCat
   HV_DEFINED_CAT_PORN               01,03,06
   HV_DEFINED_CAT_ALCOHOL_TOBACCO    08
   HV_DEFINED_CAT_ILLEGAL            09
   HV_DEFINED_CAT_GAMBLING           0B
   HV_DEFINED_CAT_VIOLENCE           0E
   HV_DEFINED_CAT_ABORTION           10
   HV_DEFINED_CAT_ILLEGAL_DRUGS      19
   HV_DEFINED_CAT_OTHERS
*/
unsigned char* gszAllHVCat = "01030608090B0E1019"; /* In HEX */

const char *UrlFilterCategoryList[HV_DEFINED_CAT_NUM] =
{
    URLFILTER_CATEGORY_PORNOGRAPHY,
    URLFILTER_CATEGORY_ALCOHOL_TABACCO,
    URLFILTER_CATEGORY_ILLEGAL_QUEST,
    URLFILTER_CATEGORY_GAMBLING,
    URLFILTER_CATEGORY_VIOLENCE,
    URLFILTER_CATEGORY_ABORTION,
    URLFILTER_CATEGORY_ILLEGAL_DRUGS,
    "Unused"
};

/* Map the TrendMicro defined Categories with HV defined Categories */
const int gTmUrlFilterDefinedCatMap[TM_URL_FILTER_MAX_CAT_NUMBER] = 
{
    HV_DEFINED_CAT_OTHERS,         //"Overides"                     00, HEX=00
    HV_DEFINED_CAT_PORN,           //"Adult/Mature Content"         01, HEX=01
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       02, HEX=02
    HV_DEFINED_CAT_PORN,           //"Pornography"                  03, HEX=03
    HV_DEFINED_CAT_OTHERS,         //"Sex Education"                04, HEX=04
    HV_DEFINED_CAT_OTHERS,         //"Intimate Apparel/Swimsuit"    05, HEX=05
    HV_DEFINED_CAT_PORN,           //"Nudity"                       06, HEX=06
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       07, HEX=07
    HV_DEFINED_CAT_ALCOHOL_TOBACCO,//"Alcohol/Tabacco"              08, HEX=08
    HV_DEFINED_CAT_ILLEGAL,        //"Illegal/Questionable"         09, HEX=09
    HV_DEFINED_CAT_OTHERS,         //"Drugs"                        10, HEX=0A
    HV_DEFINED_CAT_GAMBLING,       //"Gambling"                     11, HEX=0B
    HV_DEFINED_CAT_OTHERS,         //"Hate"                         12, HEX=0C
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       13, HEX=0D
    HV_DEFINED_CAT_VIOLENCE,       //"Violence/Hate/Racism"         14, HEX=0E
    HV_DEFINED_CAT_OTHERS,         //"Weapons"                      15, HEX=0F
    HV_DEFINED_CAT_ABORTION,       //"Abortion"                     16, HEX=10
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       17, HEX=11
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       18, HEX=12
    HV_DEFINED_CAT_OTHERS,         //"Activist/Advocacy Group"      19, HEX=13
    HV_DEFINED_CAT_OTHERS,         //"Arts/Entertainment"           20, HEX=14
    HV_DEFINED_CAT_OTHERS,         //"Business/Economy"             21, HEX=15
    HV_DEFINED_CAT_OTHERS,         //"Cult/Occult"                  22, HEX=16
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       23, HEX=17
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       24, HEX=18
    HV_DEFINED_CAT_ILLEGAL_DRUGS,  //"Illegal Drugs"                25, HEX=19
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       26, HEX=1A
    HV_DEFINED_CAT_OTHERS,         //"Education"                    27, HEX=1B
    HV_DEFINED_CAT_OTHERS,         //"Educational Institutions"     28, HEX=1C
    HV_DEFINED_CAT_OTHERS,         //"Cultural Institutions"        29, HEX=1D
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       30, HEX=1E
    HV_DEFINED_CAT_OTHERS,         //"Financial Services"           31, HEX=1F
    HV_DEFINED_CAT_OTHERS,         //"Brokerage/Trading"            32, HEX=20
    HV_DEFINED_CAT_OTHERS,         //"Games"                        33, HEX=21
    HV_DEFINED_CAT_OTHERS,         //"Government/Legal"             34, HEX=22
    HV_DEFINED_CAT_OTHERS,         //"Millitary"                    35, HEX=23
    HV_DEFINED_CAT_OTHERS,         //"Political/Activist Group"     36, HEX=24
    HV_DEFINED_CAT_OTHERS,         //"Health"                       37, HEX=25
    HV_DEFINED_CAT_OTHERS,         //"Computer/Internet"            38, HEX=26
    HV_DEFINED_CAT_OTHERS,         //"Hacking/Proxy Avoidance"      39, HEX=27
    HV_DEFINED_CAT_OTHERS,         //"Search Engines/ Portals"      40, HEX=28
    HV_DEFINED_CAT_OTHERS,         //"Web Hosting"                  41, HEX=29
    HV_DEFINED_CAT_OTHERS,         //"Web Communications"           42, HEX=2A
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       43, HEX=2B
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       44, HEX=2C
    HV_DEFINED_CAT_OTHERS,         //"Job Search/Careers"           45, HEX=2D
    HV_DEFINED_CAT_OTHERS,         //"News/Media"                   46, HEX=2E
    HV_DEFINED_CAT_OTHERS,         //"Personals/Dating"             47, HEX=2F
    HV_DEFINED_CAT_OTHERS,         //"Recreation and Hobbies"       48, HEX=30  
    HV_DEFINED_CAT_OTHERS,         //"Reference"                    49, HEX=31
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       50, HEX=32
    HV_DEFINED_CAT_OTHERS,         //"Chat/Instant Message"         51, HEX=33
    HV_DEFINED_CAT_OTHERS,         //"Email"                        52, HEX=34
    HV_DEFINED_CAT_OTHERS,         //"Newsgroups"                   53, HEX=35
    HV_DEFINED_CAT_OTHERS,         //"Religion"                     54, HEX=36
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       55, HEX=37
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       56, HEX=38
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       57, HEX=39
    HV_DEFINED_CAT_OTHERS,         //"Shopping"                     58, HEX=3A
    HV_DEFINED_CAT_OTHERS,         //"Auctions"                     59, HEX=3B
    HV_DEFINED_CAT_OTHERS,         //"Real Estate"                  60, HEX=3C
    HV_DEFINED_CAT_OTHERS,         //"Society/Lifestyle"            61, HEX=3D
    HV_DEFINED_CAT_OTHERS,         //"Gay/Lesbian"                  62, HEX=3E
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       63, HEX=3F
    HV_DEFINED_CAT_OTHERS,         //"Restaurants/Dining/Food"      64, HEX=40
    HV_DEFINED_CAT_OTHERS,         //"Sports/Recreation/Hobbies"    65, HEX=41
    HV_DEFINED_CAT_OTHERS,         //"Travel"                       66, HEX=42
    HV_DEFINED_CAT_OTHERS,         //"Vehicles"                     67, HEX=43
    HV_DEFINED_CAT_OTHERS,         //"Humor/Jokes"                  68, HEX=44
    HV_DEFINED_CAT_OTHERS,         //"Streaming Media/MP3"          69, HEX=45
    HV_DEFINED_CAT_OTHERS,         //"Educational Cheating"         70, HEX=46
    HV_DEFINED_CAT_OTHERS,         //"Software Downloads"           71, HEX=47
    HV_DEFINED_CAT_OTHERS,         //"Pay to Surf"                  72, HEX=48
    HV_DEFINED_CAT_OTHERS,         //"Glamour"                      73, HEX=49
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       74, HEX=4A
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       75, HEX=4B
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       76, HEX=4C
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       77, HEX=4D
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       78, HEX=4E
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       79, HEX=4F
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       80, HEX=50
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       81, HEX=51
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       82, HEX=52
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       83, HEX=53
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       84, HEX=54
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       85, HEX=55
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       86, HEX=56
    HV_DEFINED_CAT_OTHERS,         //"For Kids"                     87, HEX=57
    HV_DEFINED_CAT_OTHERS,         //"Web Advertisement"            88, HEX=58
    HV_DEFINED_CAT_OTHERS,         //"Web Hosting"                  89, HEX=59
    HV_DEFINED_CAT_OTHERS,         //"Unrated"                      90, HEX=5A
    HV_DEFINED_CAT_OTHERS,         //"Miscellaneous"                91, HEX=5B
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       92, HEX=5C
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       93, HEX=5D
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       94, HEX=5E
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       95, HEX=5F
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       96, HEX=60
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       97, HEX=61
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       98, HEX=62
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       99, HEX=63
};

static sigjmp_buf jmp_env_buf;

static void sig_alrm(int signo)
{
    siglongjmp(jmp_env_buf, 1);
}



/*
static struct in_addr  theTm_url_filter_ip;

void getHostByNameForFilterHost(void)
{
	struct hostent          *tm_url_filter_hostent;
    url_filter_config_t     UrlFilterConfig;
    char                    str[INET6_ADDRSTRLEN];
   CopyUrlConfToLocal(&UrlFilterConfig);
	LOGDEBUG("getHostByNameForFilterHost for filter host: %s  ",UrlFilterConfig.szTmUrlFilterHost);
	tm_url_filter_hostent = tmGethostbyname(UrlFilterConfig.szTmUrlFilterHost);
	if (tm_url_filter_hostent == NULL)
	{
		LOGERROR( "getHostByNameForFilterHost fail: %s: %s", UrlFilterConfig.szTmUrlFilterHost, hstrerror(h_errno));
	}
	else
	{
			theTm_url_filter_ip.s_addr = inet_addr(inet_ntop(tm_url_filter_hostent->h_addrtype,tm_url_filter_hostent->h_addr_list[0], str, sizeof(str)));
	}

}

*/
#ifndef TM_NO_URL_FILTERING_CACHE
 int sendUrlRatingResultToKernel(char *szReqUrlWithoutArg,int nRateRetVal,int rsp_blkc);
#endif

/**                                                                          
  * @brief Send a URL Rating request and get response
  * @param srcip Source ip to be used in exception list
  * @param req_url Input for requested website's URL without http:// and host name. ex. /foldername/filename.html
  * @param hdr_host Input for requested website's host name. ex. www.yahoo.com
  * @param hdr_port Input for requested website's port number. ex. 80
  * @param p_rating_response Output for the rating result.
  * @return HTTP_MSG_ACTION_RELAY Allow the request for the website.
  * @return HTTP_MSG_ACTION_BLOCK Deny the request for the website.
  */
int tmURLFilter(
    uint32_t srcip,
    unsigned char *req_url,
    unsigned char *hdr_host,
    unsigned char *hdr_port,
    tmufsaci_response_meta *p_rating_response)
{
    tmufsaci_request_meta   rating_request;
    struct in_addr          tm_url_filter_ip;
    struct hostent          *tm_url_filter_hostent;
   char                    str[INET6_ADDRSTRLEN];
    int                     nRateRetVal;
    int                     nHostLen;
    int                     nUrlLen;
    url_filter_config_t     UrlFilterConfig;
    //unsigned char           szMacAddress[MAC_SIZE];
    unsigned char           *p_req_url;
    //struct hostent          *p_hostent;
   // uint32_t                destip;
    //unsigned char           ip_temp[4];
    unsigned char           *p_hdr_host;
	//int ret;
#ifndef TM_NO_URL_FILTERING_CACHE
    char                    szReqUrlWithoutArg[MAX_URL_BUF]; // concat the host name: www.yahoo.com/index.htm
    int                     nRetValRemoveOptionalArgInURL;
#endif

 
	CopyUrlConfToLocal(&UrlFilterConfig);
     LOGDEBUG("src ip = %u.%u.%u.%u , rating host=%s, url=%s, port=%s", NIPQUAD(srcip), hdr_host, req_url, hdr_port);
	
    p_req_url = req_url;
    if (strncmp(req_url,"http://",7) == 0)
    {
        p_req_url += 7;
        while(*p_req_url != '\0')
        {
            if (*p_req_url == '/')
            {
                break;
            }
            p_req_url++;
        }
    }
    if (*p_req_url == '\0')
    {
        LOGERROR("Input URL Error: %s",req_url);
    }

    p_hdr_host = hdr_host;
    nHostLen = strlen(p_hdr_host);
    nUrlLen = strlen(p_req_url);
	
 #ifndef TM_NO_URL_FILTERING_CACHE
    /* Remove optional arguments, the characters after '?' and concat with hostname */
    nRetValRemoveOptionalArgInURL = RemoveOptionalArgInURL(p_hdr_host, nHostLen, p_req_url, nUrlLen, szReqUrlWithoutArg, MAX_URL_BUF);
    
    if (nRetValRemoveOptionalArgInURL == URL_TOO_LONG)
    {
         LOGERROR("concat URL too long: %s",szReqUrlWithoutArg);
    }
 #endif
	// Vendor ID
    rating_request.p_req_vendor_id = UrlFilterConfig.szVendorId;
    // License ID
    rating_request.p_req_license_id = UrlFilterConfig.szLicenseId;
    // User ID
    rating_request.p_req_user_id = UrlFilterConfig.szUserId;

    // Original requested server port
    rating_request.req_origin_port.p_cs_str = hdr_port;
    rating_request.req_origin_port.cs_len = strlen(hdr_port);

    // "0" for HTTP and "1" for FTP.  Only the HTTP protocol is supported now
    rating_request.p_req_origin_protocol = UrlFilterConfig.szOriginProtocol;
    tm_url_filter_hostent = tmGethostbyname(UrlFilterConfig.szTmUrlFilterHost);
     
    if (tm_url_filter_hostent == NULL)
    {
	    nRateRetVal=HTTP_MSG_ACTION_RELAY;
	    goto leaveFunc;
    }
   
    // TrendMicro URL Filter Server Host name
    rating_request.p_req_tm_url_filter_host = UrlFilterConfig.szTmUrlFilterHost;
    tm_url_filter_ip.s_addr = inet_addr(inet_ntop(tm_url_filter_hostent->h_addrtype,tm_url_filter_hostent->h_addr_list[0], str, sizeof(str)));
    rating_request.p_req_tm_url_filter_host_ip_addr = &tm_url_filter_ip;
    
    // TrendMicro URL Filter Server Port
    rating_request.req_tm_url_filter_host_port = UrlFilterConfig.nTmUrlFilterHostPort;
    
    // 0 for no Direct Real Time Rating
    rating_request.req_drtr_mode = UrlFilterConfig.nDrtrMode;
    // Rating Categories
    rating_request.p_req_cat_list = gszAllHVCat;
    // Deny on Error
    rating_request.req_deny_on_error = UrlFilterConfig.nDenyOnError;
    // Original requested server host name
    rating_request.req_origin_name.p_cs_str = p_hdr_host;
    rating_request.req_origin_name.cs_len = nHostLen;
    // Original requested url, without http:// and host name
    rating_request.req_origin_url.p_cs_str = p_req_url;
    rating_request.req_origin_url.cs_len = nUrlLen;

    /* Timeout UrlFilterConfig.nUrlFilterRatingTimeout seconds here , if timeout, treate as not rated */
   signal(SIGALRM, sig_alrm);
    if (!sigsetjmp(jmp_env_buf, 1))
    {
        LOGDEBUG("Set URL Rating Timeout %d seconds", UrlFilterConfig.nUrlFilterRatingTimeout);
        alarm(UrlFilterConfig.nUrlFilterRatingTimeout);
        LOGDEBUG("Call Rating API tm_url_filter_tmufsaci_cmr() for URL: %s%s",p_hdr_host,p_req_url);
        nRateRetVal = tm_url_filter_tmufsaci_cmr(&rating_request, p_rating_response, &UrlFilterConfig);
        alarm(0);
    }
    else
    {
        LOGERROR("Rating Result: Error!, Rate URL: %s%s Timeout",p_hdr_host,p_req_url);
        nRateRetVal = TM_URL_FILTER_TMUFSACI_RETURN_ERROR;
    }
    
    if (p_rating_response->rsp_blkc >= TM_URL_FILTER_MAX_CAT_NUMBER)
    {
        LOGERROR("tmURLFilter(), Block Category too big: %lu (decimal)", p_rating_response->rsp_blkc);
        nRateRetVal = TM_URL_FILTER_TMUFSACI_RETURN_ERROR;
    }

 switch (nRateRetVal)
    {
        case TM_URL_FILTER_TMUFSACI_RETURN_APPROVED:
            LOGINFO("Rating Result: URL: %s%s Not in TrendMicro Defined Category: %s",  p_hdr_host,p_req_url,gszAllHVCat);
            nRateRetVal=HTTP_MSG_ACTION_RELAY;
            break;
        case TM_URL_FILTER_TMUFSACI_RETURN_DENIED:
	//  This One is not really successfully, since we only keep one. So, we will not add this one
	// to statistics
           // tmAddUrlFilterStatisticCount(p_rating_response->rsp_blkc,
           //                              gTmUrlFilterDefinedCatMap, &UrlFilterConfig);
         /* NOTE NOTE!!! AKI 12162004. Very NOTICE:
	    Since we will not clear the URL FILTER CACHE each time when URL Filter configuration is changed,
	    So, we need to keep the denied in the cache and not check if it is user configured blocking category here.
	    Then, we to check the category??? If the rating return is DENIED, then we check if it is configured to be blocking
	    in the KERNEL. The value HTTP_MSG_ACTION_BLOCK may be a little bit confusing. But if someday the Clear URL FILTER CACHE
	    NEED TO be implmented, then we can simple resume following codes.   Remeber, check the category in kenerl and only if
	    it is DENIED (HTTP_MSG_ACTION_BLOCK_.
	     if (UrlFilterConfig.nCategory[gTmUrlFilterDefinedCatMap[p_rating_response->rsp_blkc]] == 1)
            {
                LOGINFO("Rating Result: URL: %s%s In Category, TrendMicro Definedand: %lu, HV Defined: %d, match config setting, return HTTP_MSG_ACTION_BLOCK",p_hdr_host,p_req_url, p_rating_response->rsp_blkc, gTmUrlFilterDefinedCatMap[p_rating_response->rsp_blkc]);
                nRateRetVal=HTTP_MSG_ACTION_BLOCK;
            }
            else
            {
                LOGINFO("Rating Result: URL: %s%s In Category, TrendMicro Definedand: %lu, HV Defined: %d do not match config setting, return HTTP_MSG_ACTION_RELAY", p_hdr_host,p_req_url,p_rating_response->rsp_blkc, gTmUrlFilterDefinedCatMap[p_rating_response->rsp_blkc]);
                nRateRetVal=HTTP_MSG_ACTION_RELAY;
            } */
            LOGINFO("Rating Result: URL: %s%s:TM_URL_FILTER_TMUFSACI_RETURN_DENIED",  p_hdr_host,p_req_url,nRateRetVal);
           nRateRetVal=HTTP_MSG_ACTION_BLOCK;
           break;
        case TM_URL_FILTER_TMUFSACI_RETURN_ERROR:
            if(UrlFilterConfig.nDenyOnError)
            {
                LOGERROR("Rating Result: Error!, URL: %s%s not Rated, Deny On Error enabled, return HTTP_MSG_ACTION_BLOCK", p_hdr_host,p_req_url);
                nRateRetVal=HTTP_MSG_ACTION_BLOCK;
            }
            else
            {
                LOGERROR("Rating Result: Error!, URL: %s%s not Rated, Deny On Error disabled, return HTTP_MSG_ACTION_RELAY", p_hdr_host,p_req_url);
                nRateRetVal=HTTP_MSG_ACTION_RELAY;
            }
            break;
        default:
            LOGINFO("Rating Result: ERROR in default :URL: %s%s:nRateRetVal=%d",  p_hdr_host,p_req_url);
                  nRateRetVal=HTTP_MSG_ACTION_RELAY;
          break;
    }
    
leaveFunc:
#ifndef TM_NO_URL_FILTERING_CACHE
   //tmDumpUrlCache(UrlFilterConfig.nUrlCacheSize, 0);
   //LOGDEBUG("Call tmUrlCacheAddNode(%d,%s,%d)", UrlFilterConfig.nUrlCacheSize, szReqUrlWithoutArg, UrlFilterConfig.nUrlFilterCacheExpireTime);
   //tmUrlCacheAddNode(UrlFilterConfig.nUrlCacheSize, szReqUrlWithoutArg, nRateRetVal,p_rating_response->rsp_blkc,UrlFilterConfig.nUrlFilterCacheExpireTime);
   //send the rating Result to Kernel
   sendUrlRatingResultToKernel(szReqUrlWithoutArg, nRateRetVal,p_rating_response->rsp_blkc);
   //tmDumpUrlCache(UrlFilterConfig.nUrlCacheSize, 1);
 #endif
    return nRateRetVal;
}

#ifndef TM_NO_URL_FILTERING_CACHE
int sendUrlRatingResultToKernel(char *szReqUrlWithoutArg,int nRateRetVal,int rsp_blkc)
{
    int nRetVal = 0;
    size_t len;
    int sockfd = -1; 
    add_url_cache the_add_url_cache;
    
	LOGINFO("sendUrlRatingResultToKernel is call, the %s  nRateRetVal=%d,rsp_blkc=%d \n",szReqUrlWithoutArg,nRateRetVal, rsp_blkc);
	// We need to send the AVE configuration to tmss_kernel(ipt_webstr)
	sockfd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW);
	if (sockfd < 0)
	{
	    fprintf(stderr,"sendUrlRatingResultToKernel:Can not create sock:%s\a\n",strerror(errno)); 
		return -1;
	}

	len=sizeof(add_url_cache);
	strcpy(the_add_url_cache.szReqUrlWithoutArg,szReqUrlWithoutArg);
	the_add_url_cache.nRateRetVal=nRateRetVal;
	the_add_url_cache.rsp_blkc=rsp_blkc;
	if (setsockopt(sockfd, IPPROTO_IP, TMSS_SO_ADD_URL_FILTER_CACHE_NODE, &the_add_url_cache,len ) < 0)
	{
	    fprintf(stderr,"sendUrlRatingResultToKernel:Can not setsockopt sock:%s\a\n",strerror(errno)); 
	close(sockfd);
	sockfd = -1;
		return -1;
	}
	close(sockfd);
	sockfd = -1;
    return nRetVal;
}
#endif
