/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file tmURLRater.c
 *  @brief TMOG URL Rating Module
 */

/*
 *  Change Log
 *  2004-01-08 Henry Lin    Initialization version
 */


//#define NDEBUG
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "tmSocketClient.h"
#include "tmURLRater.h"
#include "log.h"
#include "tmURLFilterConf.h" /* url_filter_config_t */

static const char ModuleName[] = "tmURLRater";
static const url_filter_config_t *pUrlFilterConfigLocal;

/*
* ----------------------------------------------------------------------------
* Global Declaration Section.
* ----------------------------------------------------------------------------
*/
static unsigned char *g_encode_table[] = {
            "%00", "%01", "%02", "%03", "%04", "%05", "%06", "%07",
            "%08", "%09", "%0A", "%0B", "%0C", "%0D", "%0E", "%0F",
            "%10", "%11", "%12", "%13", "%14", "%15", "%16", "%17",
            "%18", "%19", "%1A", "%1B", "%1C", "%1D", "%1E", "%1F",
            "%20", "%21", "%22", "%23", "%24", "%25", "%26", "%27",
            "%28", "%29", NULL, "%2B", "%2C", NULL, NULL, "%2F",
            NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
            NULL, NULL, "%3A", "%3B", "%3C", "%3D", "%3E", "%3F",
            "%40", NULL, NULL, NULL, NULL, NULL, NULL, NULL,
            NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
            NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
            NULL, NULL, NULL, "%5B", "%5C", "%5D", "%5E", NULL,
            "%60", NULL, NULL, NULL, NULL, NULL, NULL, NULL,
            NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
            NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
            NULL, NULL, NULL, "%7B", "%7C", "%7D", "%7E", "%7F",
            "%80", "%81", "%82", "%83", "%84", "%85", "%86", "%87",
            "%88", "%89", "%8A", "%8B", "%8C", "%8D", "%8E", "%8F",
            "%90", "%91", "%92", "%93", "%94", "%95", "%96", "%97",
            "%98", "%99", "%9A", "%9B", "%9C", "%9D", "%9E", "%9F",
            "%A0", "%A1", "%A2", "%A3", "%A4", "%A5", "%A6", "%A7",
            "%A8", "%A9", "%AA", "%AB", "%AC", "%AD", "%AE", "%AF",
            "%B0", "%B1", "%B2", "%B3", "%B4", "%B5", "%B6", "%B7",
            "%B8", "%B9", "%BA", "%BB", "%BC", "%BD", "%BE", "%BF",
            "%C0", "%C1", "%C2", "%C3", "%C4", "%C5", "%C6", "%C7",
            "%C8", "%C9", "%CA", "%CB", "%CC", "%CD", "%CE", "%CF",
            "%D0", "%D1", "%D2", "%D3", "%D4", "%D5", "%D6", "%D7",
            "%D8", "%D9", "%DA", "%DB", "%DC", "%DD", "%DE", "%DF",
            "%E0", "%E1", "%E2", "%E3", "%E4", "%E5", "%E6", "%E7",
            "%E8", "%E9", "%EA", "%EB", "%EC", "%ED", "%EE", "%EF",
            "%F0", "%F1", "%F2", "%F3", "%F4", "%F5", "%F6", "%F7",
            "%F8", "%F9", "%FA", "%FB", "%FC", "%FD", "%FE", "%FF"
        };
/*
* ----------------------------------------------------------------------------
* Forward Function Declaration Section.
* ----------------------------------------------------------------------------
*/
static void tm_url_filter_get_xml_tags(unsigned char *ap_http_receive,
                              tmufsaci_response_meta *ap_rsp_data);
static unsigned char *tm_url_filter_memcpy_encode(
    unsigned char *ap_encoded_buf,
    unsigned char *ap_src, int a_src_len,
    int a_mode);

/* Remove static, because this function is needed for tmURLFilter.c */
int tm_url_filter_tmufsaci_check_mimetype(
    unsigned char *ap_url,
    int a_url_len);

static void tm_url_filter_parse_http(
    unsigned char *ap_http_receive,
    tmufsaci_response_meta *ap_rsp_data);
static int tm_url_filter_build_tmufsatp_request(
    tmufsaci_request_meta *ap_req_data,
    unsigned char *ap_http_string,
    unsigned int a_command_type);
static int tm_url_filter_send_tmufsatp_request(
    tmufsaci_request_meta *ap_req_data,
    unsigned char *ap_http_transmit,
    int a_count,
    unsigned char *ap_http_receive);

/*
* ----------------------------------------------------------------------------
* Function Declaration Section.
* ----------------------------------------------------------------------------
*/

int
tm_url_filter_tmufsaci_request(tmufsaci_request_meta *ap_req_data,
                   tmufsaci_response_meta *ap_rsp_data,
                   unsigned int a_command_type,
                   void (*pf_call_back)(tmufsaci_response_meta *ap_rsp_data,
                                        int a_result))
{
    /*
     * These buffers should be dynamically allocated and should ensure
     * enough space to allow for a 3 to 1 expansion for items that are
     * URL encoded.
     */
    unsigned char tp_http_transmit[TM_URL_FILTER_SERVICE_MAX_REQUEST_SIZE];
    unsigned char tp_http_receive[TM_URL_FILTER_SERVICE_MAX_REQUEST_SIZE];
    int t_count;


    (void)memset((void *)ap_rsp_data, 0x0, sizeof(tmufsaci_response_meta));

    /* the data is ok let's build and send the http string*/
    if((t_count = tm_url_filter_build_tmufsatp_request(ap_req_data, tp_http_transmit,
                                           a_command_type)) <= 0)
    {
        LOGDEBUG("Call tm_url_filter_build_tmufsatp_request() fail");
        return(TM_URL_FILTER_TMUFSACI_ERROR);
    }
    LOGDEBUG("transmit rating request = %s",tp_http_transmit);

    if(tm_url_filter_send_tmufsatp_request(ap_req_data, tp_http_transmit, t_count,
                               tp_http_receive) == TM_URL_FILTER_TMUFSACI_SUCCESS)
    {
        LOGDEBUG("receive from server for rating request = %s",tp_http_receive);
        (void)tm_url_filter_parse_http(tp_http_receive, ap_rsp_data);
        if(pf_call_back)
            (*pf_call_back)(ap_rsp_data, TM_URL_FILTER_TMUFSACI_SUCCESS);
        return(TM_URL_FILTER_TMUFSACI_SUCCESS);
    }
    if(pf_call_back)
        (*pf_call_back)(ap_rsp_data, TM_URL_FILTER_TMUFSACI_ERROR);

    LOGDEBUG("Fail at tm_url_filter_tmufsaci_request() end");
    return(TM_URL_FILTER_TMUFSACI_ERROR);
}


static int
tm_url_filter_build_tmufsatp_request(tmufsaci_request_meta *ap_req_data,
                         unsigned char *ap_transmit, unsigned int a_command_type)
{
    unsigned char *tp_buf;
    unsigned char tm_url_filter_host_port[TM_URL_FILTER_MAX_PORT_LEN];


    tp_buf = ap_transmit;

    /* copy method, no URL encoding */
    tp_buf = tm_url_filter_memcpy_encode(tp_buf, TM_URL_FILTER_TMUFSATP_HTTP_METHOD,
                                TM_URL_FILTER_TMUFSATP_HTTP_METHOD_LEN, TM_URL_FILTER_NO_URL_ENCODE);
    /* add a space separator and beginning "/" */
    *tp_buf++ = ' ';
    
#ifdef _PROXY_
    if (pUrlFilterConfigLocal->nProxyEnable == 1) /* With proxy */
    {
        tp_buf = tm_url_filter_memcpy_encode(tp_buf, TM_URL_FILTER_TMUFSATP_HTTP_REQUEST,
                                TM_URL_FILTER_TMUFSATP_HTTP_REQUEST_LEN, TM_URL_FILTER_NO_URL_ENCODE);
        tp_buf = tm_url_filter_memcpy_encode(tp_buf, ap_req_data->p_req_tm_url_filter_host,
                                TM_URL_FILTER_MAX_COMPONENT_LEN, TM_URL_FILTER_NO_URL_ENCODE);
        /* add colon between host and port */
        *tp_buf++ = ':';
        /*
         * Copy HTTP TrendMicor URL Filter Service host port number to Method header, no URL
         * encoding.
         */
        tm_url_filter_host_port[TM_URL_FILTER_MAX_PORT_LEN - 1] = '\0';
        snprintf(tm_url_filter_host_port,TM_URL_FILTER_MAX_PORT_LEN - 1,"%d",ap_req_data->req_tm_url_filter_host_port);
    
        tp_buf = tm_url_filter_memcpy_encode(tp_buf, tm_url_filter_host_port,
                                strlen(tm_url_filter_host_port), TM_URL_FILTER_NO_URL_ENCODE);
    }
#endif
    
    *tp_buf++ = '/';

    switch(a_command_type)
    {
        case TM_URL_FILTER_ACCESS_DETERMINATION_REQUEST:
            /* copy transport message type, no URL encoding */
            tp_buf = tm_url_filter_memcpy_encode(tp_buf, TM_URL_FILTER_TMUFSATP_CIMP_ADR,
                                        TM_URL_FILTER_TMUFSATP_CIMP_ADR_LEN, TM_URL_FILTER_NO_URL_ENCODE);
            break;
        case TM_URL_FILTER_CATEGORY_MEMBERSHIP_REQUEST:
            /* copy transport message type, no URL encoding */
            tp_buf = tm_url_filter_memcpy_encode(tp_buf, TM_URL_FILTER_TMUFSATP_CIMP_CMR,
                                        TM_URL_FILTER_TMUFSATP_CIMP_CMR_LEN, TM_URL_FILTER_NO_URL_ENCODE);
            break;
        case TM_URL_FILTER_CATEGORY_MEMBERSHIP_NO_DRTR_REQUEST:
            /* copy transport message type, no URL encoding */
            tp_buf = tm_url_filter_memcpy_encode(tp_buf,
                                        TM_URL_FILTER_TMUFSATP_CIMP_CMR_NO_DRTR,
                                        TM_URL_FILTER_TMUFSATP_CIMP_CMR_NO_DRTR_LEN,
                                        TM_URL_FILTER_NO_URL_ENCODE);
            break;
        default:
            LOGDEBUG("tm_url_filter_build_tmufsatp_request() fail return 1");
            return(0);
    }

    /* add a slash separator */
    *tp_buf++ = '/';
    /* copy vendor id, no URL encoding */
    tp_buf = tm_url_filter_memcpy_encode(tp_buf, ap_req_data->p_req_vendor_id,
                                TM_URL_FILTER_MAX_COMPONENT_LEN, TM_URL_FILTER_NO_URL_ENCODE);

    /* add a slash separator */
    *tp_buf++ = '/';
    /* copy license id, no URL encoding */
    tp_buf = tm_url_filter_memcpy_encode(tp_buf, ap_req_data->p_req_license_id,
                                TM_URL_FILTER_MAX_COMPONENT_LEN, TM_URL_FILTER_NO_URL_ENCODE);

    /* add a slash separator */
    *tp_buf++ = '/';
    switch(a_command_type)
    {
        case TM_URL_FILTER_ACCESS_DETERMINATION_REQUEST:
            /* copy user id, with URL encoding */
            tp_buf = tm_url_filter_memcpy_encode(tp_buf, ap_req_data->p_req_user_id,
                                        TM_URL_FILTER_MAX_USER_ID_LEN, TM_URL_FILTER_URL_ENCODE);
            /* add a slash separator */
            *tp_buf++ = '/';
            /* add a dummy category list */
            *tp_buf++ = '-';
            /* add a slash separator */
            *tp_buf++ = '/';
            break;
        case TM_URL_FILTER_CATEGORY_MEMBERSHIP_REQUEST:
        case TM_URL_FILTER_CATEGORY_MEMBERSHIP_NO_DRTR_REQUEST:
            /* copy user id, with URL encoding, if specified */
            if(ap_req_data->p_req_user_id != (unsigned char *)NULL)
                tp_buf = tm_url_filter_memcpy_encode(tp_buf,
                                            ap_req_data->p_req_user_id,
                                            TM_URL_FILTER_MAX_USER_ID_LEN, TM_URL_FILTER_URL_ENCODE);
            else
                /* add a dummy user id */
                *tp_buf++ = '-';
            /* add a slash separator */
            *tp_buf++ = '/';
            /* copy category list, no URL encoding */
            tp_buf = tm_url_filter_memcpy_encode(tp_buf, ap_req_data->p_req_cat_list,
                                        TM_URL_FILTER_MAX_CATEGORY_LIST_SIZE,
                                        TM_URL_FILTER_NO_URL_ENCODE);
            /* add a slash separator */
            *tp_buf++ = '/';
            break;
        default:
            LOGDEBUG("tm_url_filter_build_tmufsatp_request() fail return 2");
            return(0);
    }

    /* copy origin protocol id, no URL encoding */
    tp_buf = tm_url_filter_memcpy_encode(tp_buf, ap_req_data->p_req_origin_protocol,
                                TM_URL_FILTER_MAX_COMPONENT_LEN, TM_URL_FILTER_NO_URL_ENCODE);

    /* add a slash separator */
    *tp_buf++ = '/';
    /* copy origin server name, no URL encoding */
    tp_buf = tm_url_filter_memcpy_encode(tp_buf,
                                ap_req_data->req_origin_name.p_cs_str,
                                ap_req_data->req_origin_name.cs_len,
                                TM_URL_FILTER_NO_URL_ENCODE);

    /* add a slash separator */
    *tp_buf++ = '/';
    /* set origin port if not set */
    if((ap_req_data->req_origin_port.p_cs_str != (unsigned char *)NULL))
        /* copy origin server port number, no URL encoding */
        tp_buf = tm_url_filter_memcpy_encode(tp_buf,
                                    ap_req_data->req_origin_port.p_cs_str,
                                    ap_req_data->req_origin_port.cs_len,
                                    TM_URL_FILTER_NO_URL_ENCODE);
    else
        /* copy default origin server port number, no URL encoding */
        tp_buf = tm_url_filter_memcpy_encode(tp_buf, TM_URL_FILTER_TMUFSATP_HTTP_PORT,
                                    TM_URL_FILTER_TMUFSATP_HTTP_PORT_LEN, TM_URL_FILTER_NO_URL_ENCODE);

    /* add a slash separator */
    *tp_buf++ = '/';
    /* origin URL - skip over leading '/' if "//" is present */
    if((char)*ap_req_data->req_origin_url.p_cs_str == '/')
        /* copy URL, with URL encoding - already should be encoded! */
        tp_buf = tm_url_filter_memcpy_encode(tp_buf,
                                    ap_req_data->req_origin_url.p_cs_str+1,
                                    ap_req_data->req_origin_url.cs_len-1,
                                    TM_URL_FILTER_URL_ENCODE);
    else
        /* copy URL, with URL encoding - already should be encoded! */
        tp_buf = tm_url_filter_memcpy_encode(tp_buf,
                                    ap_req_data->req_origin_url.p_cs_str,
                                    ap_req_data->req_origin_url.cs_len,
                                    TM_URL_FILTER_URL_ENCODE);

    /* add a space separator */
    *tp_buf++ = ' ';
    /* copy origin server port number, no URL encoding */
    tp_buf = tm_url_filter_memcpy_encode(tp_buf, TM_URL_FILTER_TMUFSATP_HTTP_PROTOCOL,
                                TM_URL_FILTER_TMUFSATP_HTTP_PROTOCOL_LEN, TM_URL_FILTER_NO_URL_ENCODE);

    /* add a CRLF pair */
    *tp_buf++ = '\r';
    *tp_buf++ = '\n';


    /* copy HTTP Accept header, no URL encoding */
    tp_buf = tm_url_filter_memcpy_encode(tp_buf, TM_URL_FILTER_TMUFSATP_HTTP_ACCEPT,
                                TM_URL_FILTER_TMUFSATP_HTTP_ACCEPT_LEN, TM_URL_FILTER_NO_URL_ENCODE);

    /* add a CRLF pair */
    *tp_buf++ = '\r';
    *tp_buf++ = '\n';


    /* copy HTTP Language header, no URL encoding */
    tp_buf = tm_url_filter_memcpy_encode(tp_buf, TM_URL_FILTER_TMUFSATP_HTTP_LANG,
                                TM_URL_FILTER_TMUFSATP_HTTP_LANG_LEN, TM_URL_FILTER_NO_URL_ENCODE);

    /* add a CRLF pair */
    *tp_buf++ = '\r';
    *tp_buf++ = '\n';


    /* copy HTTP Agent header, no URL encoding */
    tp_buf = tm_url_filter_memcpy_encode(tp_buf, TM_URL_FILTER_TMUFSATP_HTTP_AGENT,
                                TM_URL_FILTER_TMUFSATP_HTTP_AGENT_LEN, TM_URL_FILTER_NO_URL_ENCODE);

    /* add a CRLF pair */
    *tp_buf++ = '\r';
    *tp_buf++ = '\n';


    /* copy HTTP Host header, no URL encoding */
    tp_buf = tm_url_filter_memcpy_encode(tp_buf, TM_URL_FILTER_TMUFSATP_HTTP_HOST,
                                TM_URL_FILTER_TMUFSATP_HTTP_HOST_LEN, TM_URL_FILTER_NO_URL_ENCODE);
    /* add a space separator */
    *tp_buf++ = ' ';
    /*
     * Copy HTTP TrendMicor ULR Filter Service host name to Host header, no URL
     * encoding.
     */
    tp_buf = tm_url_filter_memcpy_encode(tp_buf, ap_req_data->p_req_tm_url_filter_host,
                                TM_URL_FILTER_MAX_COMPONENT_LEN, TM_URL_FILTER_NO_URL_ENCODE);
    /* add colon between host and port */
    *tp_buf++ = ':';
    /*
     * Copy HTTP TrendMicor URL Filter Service host port number to Host header, no URL
     * encoding.
     */
    tm_url_filter_host_port[TM_URL_FILTER_MAX_PORT_LEN - 1] = '\0';
    snprintf(tm_url_filter_host_port,TM_URL_FILTER_MAX_PORT_LEN - 1,"%d",ap_req_data->req_tm_url_filter_host_port);
    
    tp_buf = tm_url_filter_memcpy_encode(tp_buf, tm_url_filter_host_port,
                                strlen(tm_url_filter_host_port), TM_URL_FILTER_NO_URL_ENCODE);

    /* add a CRLF pair */
    *tp_buf++ = '\r';
    *tp_buf++ = '\n';


#ifdef _PROXY_
    /* copy HTTP Connection header, no URL encoding */
    if (pUrlFilterConfigLocal->nProxyEnable == 0) /* no proxy */
    {
#endif
        tp_buf = tm_url_filter_memcpy_encode(tp_buf, TM_URL_FILTER_TMUFSATP_HTTP_CONNECTION,
                                TM_URL_FILTER_TMUFSATP_HTTP_CONNECTION_LEN, TM_URL_FILTER_NO_URL_ENCODE);
#ifdef _PROXY_
    }
    else /* with proxy */
    {
        tp_buf = tm_url_filter_memcpy_encode(tp_buf, TM_URL_FILTER_TMUFSATP_HTTP_PROXY_CONNECTION,
                                TM_URL_FILTER_TMUFSATP_HTTP_PROXY_CONNECTION_LEN, TM_URL_FILTER_NO_URL_ENCODE);
    }
#endif

    /* add a CRLF pair */
    *tp_buf++ = '\r';
    *tp_buf++ = '\n';


    /* add a CRLF pair for EOF */
    *tp_buf++ = '\r';
    *tp_buf++ = '\n';


    /*
     * We now have a HTTP string we can send to the TrendMicor URL Filter
     * Service.
     */
    /* return the length */
    return((int)(tp_buf-ap_transmit));
}
static int
tm_url_filter_send_tmufsatp_request(tmufsaci_request_meta *ap_req_data,
                        unsigned char *ap_http_transmit, int a_count,
                        unsigned char *ap_http_receive)
{
    int t_sd;
    
#ifdef _PROXY_
    if (pUrlFilterConfigLocal->nProxyEnable == 1) /* With proxy */
    {
        t_sd = tm_url_filter_tcp_connect(ap_req_data->p_req_tm_url_filter_host_ip_addr,
                                pUrlFilterConfigLocal->nProxyPort);
    }
    else /* Without proxy */
    {
#endif
        t_sd = tm_url_filter_tcp_connect(ap_req_data->p_req_tm_url_filter_host_ip_addr,
                                ap_req_data->req_tm_url_filter_host_port);
#ifdef _PROXY_
    }
#endif
    
    if(t_sd == TM_URL_FILTER_SOCK_FAILURE)
    {
	    LOGERROR("tm_url_filter_tcp_connect return TM_URL_FILTER_SOCK_FAILURE");
         return(TM_URL_FILTER_TMUFSACI_ERROR);
    }

    if(tm_url_filter_sock_srv_write(t_sd, ap_http_transmit, a_count, 60) ==
            TM_URL_FILTER_SOCK_FAILURE)
    {
        (void)close(t_sd);
        return(TM_URL_FILTER_TMUFSACI_ERROR);
    }

     if(tm_url_filter_sock_srv_read(t_sd, ap_http_receive,
                          TM_URL_FILTER_SERVICE_MAX_RESPONSE_SIZE, 60) == TM_URL_FILTER_SOCK_FAILURE)
    {
        (void)close(t_sd);
        return(TM_URL_FILTER_TMUFSACI_ERROR);
    }
    (void)close(t_sd);
    return(TM_URL_FILTER_TMUFSACI_SUCCESS); 
}
static void
tm_url_filter_parse_http(unsigned char *ap_http_receive,
                tmufsaci_response_meta *ap_rsp_data)
{
    unsigned int t_http_resp_code;
    
    if((sscanf(ap_http_receive, "HTTP/1.0 %d", &t_http_resp_code) == 1) ||
            (sscanf(ap_http_receive, "HTTP/1.1 %d", &t_http_resp_code) == 1) )
    {
        LOGDEBUG("t_http_resp_code=%d",t_http_resp_code);
        switch(t_http_resp_code)
        {
            case 200:
                ap_rsp_data->rsp_result = TM_URL_FILTER_RESULT_ALLOW;
                break;
            case 400:
                ap_rsp_data->rsp_result = TM_URL_FILTER_RESULT_BAD_REQUEST;
                break;
            case 401:
                ap_rsp_data->rsp_result = TM_URL_FILTER_RESULT_NO_USER_NAME |
                                          TM_URL_FILTER_RESULT_NO_POLICY |
                                          TM_URL_FILTER_RESULT_NO_TIME;
                break;
            case 402:
                ap_rsp_data->rsp_result = TM_URL_FILTER_RESULT_NO_LICENSE;
                break;
            case 404:
                ap_rsp_data->rsp_result = TM_URL_FILTER_RESULT_DENY;
                break;
            case 405:
                ap_rsp_data->rsp_result =
                    TM_URL_FILTER_RESULT_METHOD_NOT_ALLOWED;
                break;
            case 410:
                ap_rsp_data->rsp_result = TM_URL_FILTER_RESULT_RATING_UNKNOWN |
                                          TM_URL_FILTER_RESULT_URL_NOT_FOUND;
                break;
            case 413:
                ap_rsp_data->rsp_result = TM_URL_FILTER_RESULT_REQUEST_TOO_LONG;
                break;
            case 414:
                ap_rsp_data->rsp_result = TM_URL_FILTER_RESULT_BAD_URL;
                break;
            case 500:
                ap_rsp_data->rsp_result = TM_URL_FILTER_RESULT_LICENSE_FAILURE |
                                          TM_URL_FILTER_RESULT_IDENTITY_FAILURE |
                                          TM_URL_FILTER_RESULT_POLICY_FAILURE |
                                          TM_URL_FILTER_RESULT_OVERRIDE_FAILURE |
                                          TM_URL_FILTER_RESULT_RATING_FAILURE |
                                          TM_URL_FILTER_RESULT_NO_SPREAD_SERVER |
                                          TM_URL_FILTER_RESULT_NO_BLOCKING_SERVER;
                break;
            case 503:
            default:
                ap_rsp_data->rsp_result =
                    TM_URL_FILTER_RESULT_SERVICE_UNAVAILABLE;
                break;
        }
        /* Decode XML from entity body to find <code> NNN </code> */
        (void)tm_url_filter_get_xml_tags(ap_http_receive, ap_rsp_data);
    }
    return;
}
static void
tm_url_filter_get_xml_tags(unsigned char *ap_http_receive,
                  tmufsaci_response_meta *ap_rsp_data)
{
    /* zero out results */
    ap_rsp_data->rsp_code = 0x0;
    ap_rsp_data->rsp_domr = 0x0;
    ap_rsp_data->rsp_dirr = 0x0;
    ap_rsp_data->rsp_domt = 0x0;
    ap_rsp_data->rsp_blkc = 0x0;
    /* look for the opening tag and retrieve the code */
    while(1)
    {
        /* check for an XML tag */
        if(*ap_http_receive == (unsigned char)NULL)
            break;
        if(*ap_http_receive != '<')
        {
            ap_http_receive++;
            continue;
        }
        if(sscanf(ap_http_receive, "<Code>%lx</Code>\r\n",
                  &(ap_rsp_data->rsp_code))
                || sscanf(ap_http_receive, "<BlkC>%lx</BlkC>\r\n",
                          &(ap_rsp_data->rsp_blkc))
                || sscanf(ap_http_receive, "<DomT>%lx</DomT>\r\n",
                          &(ap_rsp_data->rsp_domt)))
        {
            /* step over this entry to the start of the next */
            while(*ap_http_receive && (*ap_http_receive !=
                                       (unsigned char)('\n')))
                ap_http_receive++;
            /* eat the LF character */
            if(*ap_http_receive == '\n')
                ap_http_receive++;
        }
        else
            ap_http_receive++;
    }
    LOGDEBUG("ap_rsp_data->rsp_blkc = %lx",ap_rsp_data->rsp_blkc);
    return;
}
static unsigned char *
tm_url_filter_memcpy_encode(unsigned char *ap_encoded_buf,
                   unsigned char *ap_src, int a_src_len, int a_mode)
{
    int t_ii;
    /* copy the text across - encode it as we go if required */
    for(t_ii = 0; (*ap_src) && (t_ii < a_src_len); t_ii++)
    {
        if(a_mode)
        {
            /*
            * Get the ASCII code of the current character
            * and check to see if we have a translation value.
                */
            if((*ap_src == (unsigned char)'%') &&
                    (t_ii <= (a_src_len - 3)))
            {
                /* already URL encoded - so copy over %xy string */
                *ap_encoded_buf++ = *ap_src++;
                *ap_encoded_buf++ = *ap_src++;
                *ap_encoded_buf++ = *ap_src++;
            }
            else if(*(g_encode_table+((unsigned int)(*ap_src))) !=
                    (unsigned char *)NULL)
            {
                /*
                * If there is a pointer in this location in the
                * able then we need to URL encode this
                * character.
                */
                (void)strncpy(ap_encoded_buf, *(g_encode_table
                                                +((unsigned int)(*ap_src))), 3);
                ap_encoded_buf += 3;
                ap_src++;
            }
            else
            {
                /* regular character, copy the original */
                *ap_encoded_buf++ = *ap_src++;
            }
        }
        else
        {
            /*
            * No encoding necessary - copy over original buffer
                */
            *ap_encoded_buf++ = *ap_src++;
        }
    }
    /* NULL terminate things */
    *ap_encoded_buf = (unsigned char)NULL;
    /* return pointer to end of buffer */
    return(ap_encoded_buf);
}


int
tm_url_filter_tmufsaci_cmr(tmufsaci_request_meta *ap_req_data,
               tmufsaci_response_meta *ap_rsp_data, url_filter_config_t *pUrlFilterConfig)
{
    LOGDEBUG("Entering tm_url_filter_tmufsaci_cmr()");
    if (pUrlFilterConfig == NULL)
    {
        return(TM_URL_FILTER_TMUFSACI_RETURN_ERROR);
    }
    pUrlFilterConfigLocal = pUrlFilterConfig;
    //int t_ii; /* loop control */
    //unsigned char *tp_buf; /* temp pointer */
    /* check to see if the mime type matches one we must rate */
    // already check,
 //   if(!tm_url_filter_tmufsaci_check_mimetype(ap_req_data->req_origin_url.p_cs_str,
  //                                ap_req_data->req_origin_url.cs_len))
        // access approved - just filler URL - no nead to rate 
     //   return(TM_URL_FILTER_TMUFSACI_RETURN_APPROVED);
    /* rate the requested URL and compare to category list for membership */
    if(tm_url_filter_tmufsaci_request(ap_req_data, ap_rsp_data,
                          ((ap_req_data->req_drtr_mode) ?
                           TM_URL_FILTER_CATEGORY_MEMBERSHIP_REQUEST :
                           TM_URL_FILTER_CATEGORY_MEMBERSHIP_NO_DRTR_REQUEST),
                          (void *)NULL) == TM_URL_FILTER_TMUFSACI_SUCCESS)
    {
	LOGDEBUG("ap_rsp_data->rsp_result=%lu (decimal)",ap_rsp_data->rsp_result);
	LOGDEBUG("ap_rsp_data->rsp_blkc=%lu (decimal)",ap_rsp_data->rsp_blkc);
        
	if (ap_rsp_data->rsp_blkc >= TM_URL_FILTER_MAX_CAT_NUMBER)
	{
		LOGERROR("tm_url_filter_tmufsaci_cmr(), Block Category too big: %lu (decimal)", ap_rsp_data->rsp_blkc);
		return(TM_URL_FILTER_TMUFSACI_RETURN_ERROR);
	}
	/* check result information */
	switch(ap_rsp_data->rsp_result)
	{
            case TM_URL_FILTER_RESULT_ALLOW:
                /* access approved */
                return(TM_URL_FILTER_TMUFSACI_RETURN_APPROVED);
            case TM_URL_FILTER_RESULT_DENY:
                /* access denied */
                return(TM_URL_FILTER_TMUFSACI_RETURN_DENIED);
            case (TM_URL_FILTER_RESULT_RATING_UNKNOWN |
                            TM_URL_FILTER_RESULT_URL_NOT_FOUND):
                /* access not determinable - allow on error setting */
                return(TM_URL_FILTER_TMUFSACI_RETURN_ERROR);
                //if(ap_req_data->req_deny_on_error)
                    //return(TM_URL_FILTER_TMUFSACI_RETURN_DENIED);
                //else
                    //return(TM_URL_FILTER_TMUFSACI_RETURN_APPROVED);
            case TM_URL_FILTER_RESULT_BAD_REQUEST:
            case (TM_URL_FILTER_RESULT_NO_USER_NAME |
                            TM_URL_FILTER_RESULT_NO_POLICY |
                            TM_URL_FILTER_RESULT_NO_TIME):
            case TM_URL_FILTER_RESULT_NO_LICENSE:
            case TM_URL_FILTER_RESULT_METHOD_NOT_ALLOWED:
            case TM_URL_FILTER_RESULT_REQUEST_TOO_LONG:
            case TM_URL_FILTER_RESULT_BAD_URL:
            case (TM_URL_FILTER_RESULT_LICENSE_FAILURE |
                            TM_URL_FILTER_RESULT_IDENTITY_FAILURE |
                            TM_URL_FILTER_RESULT_POLICY_FAILURE |
                            TM_URL_FILTER_RESULT_OVERRIDE_FAILURE |
                            TM_URL_FILTER_RESULT_RATING_FAILURE |
                            TM_URL_FILTER_RESULT_NO_SPREAD_SERVER |
                            TM_URL_FILTER_RESULT_NO_BLOCKING_SERVER):
            default:
                /* access not determinable - allow on error setting */
                if(ap_req_data->req_deny_on_error)
                    return(TM_URL_FILTER_TMUFSACI_RETURN_DENIED);
                else
                    return(TM_URL_FILTER_TMUFSACI_RETURN_APPROVED);
	}
    }
    LOGDEBUG("Call tm_url_filter_tmufsaci_request() fail");
	/* should never get here */
    return(TM_URL_FILTER_TMUFSACI_RETURN_DENIED);
}


