/******************************************************************************
*
*  (C) Copyright 2004 Trend Micro, Inc.
*
*  All Rights Reserved.
*  This program is an unpublished copyrighted work which is proprietary
*  to Trend Micro, Inc. and contains confidential information that is not
*  to be reproduced or disclosed to any other person or entity without
*  prior written consent from Trend Micro, Inc. in each and every instance.
*
*  WARNING:  Unauthorized reproduction of this program as well as
*  unauthorized preparation of derivative works based upon the
*  program or distribution of copies by sale, rental, lease or
*  lending are violations of federal copyright laws and state trade
*  secret laws, punishable by civil and criminal penalties.
*
*****************************************************************************/

/**
*  @file tmSemaphoreTest.c
*  @brief TMOG Semaphore Module Testing
*/

/*
*  Change Log
*  2004-02-20 Henry Lin    Initialization version
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "tmShm.h"
#include "tmSemaphore.h"
#include "tmURLFilter.h"
#include "time.h"
#include "tmURLCache.h"

void print_help(void)
{
    printf("./testUrlCache -reset [cache_size]\n");
    printf("./testUrlCache -dump [cache_size]\n");
    printf("./testUrlCache -add [cache_size] [expire_time] [url]\n");
    printf("./testUrlCache -query [cache_size] [expire_time] [url]\n");
}

static char *charmap[27] = {
    "A",
    "B",
    "C",
    "D",
    "E",
    "F",
    "G",
    "H",
    "I",
    "J",
    "K",
    "L",
    "M",
    "N",
    "O",
    "P",
    "Q",
    "R",
    "S",
    "T",
    "U",
    "V",
    "W",
    "X",
    "Y",
    "Z",
    "Others"
};



static requested_url_t *tmOffset2Addr(url_cache_t *pBaseAddr, requested_url_t *pOffset)
{
    return ((requested_url_t *)((unsigned int)pBaseAddr + (unsigned int)pOffset));
}

void testDumpUrlCache(int nUrlFilterCacheSize)
{
    int i,j;
    requested_url_t *pTemp = NULL;
    int nUrlFilterCacheShmid;
    url_cache_t *p_url_cache = NULL;
    time_t now;
    struct tm_sem_st UrlFilterCacheLock;

    nUrlFilterCacheShmid = shm_init(TMSS_URL_FILTER_CACHE_SHMKEY, sizeof(url_cache_t)+ sizeof(requested_url_t)*(nUrlFilterCacheSize));
    tm_lock_create(TMSS_URL_FILTER_CACHE_SEMLOCKKEY, &UrlFilterCacheLock);
    tm_lock(&UrlFilterCacheLock);
    p_url_cache = (url_cache_t*)shm_attach(nUrlFilterCacheShmid);

    for (i = 0; i < CACHE_HASH_TABLE_SIZE; i++)
    {
        j = 1;
        pTemp = (p_url_cache->hash_table)[i];
        printf("Hash table %d for %s:\n",i, charmap[i]);
        while(pTemp != NULL)
        {
            pTemp = tmOffset2Addr(p_url_cache, pTemp); /* use offset */
            printf("    Node %6d: ULR = %s , ExpireTime = %lu\n", j++, pTemp->req_url ,pTemp->expire_time);
            pTemp = pTemp->next;
        }
    }
    
    printf("LRU List:\n");
    pTemp = p_url_cache->cache_lru;
    j = 1;
    while(pTemp != NULL)
    {
        pTemp = tmOffset2Addr(p_url_cache, pTemp); /* use offset */
        printf("    Node %6d: ULR = %s , ExpireTime = %lu\n", j++, pTemp->req_url, pTemp->expire_time);
        pTemp = pTemp->lru;
    }
    
    printf("MRU List:\n");
    pTemp = p_url_cache->cache_mru;
    j = 1;
    while(pTemp != NULL)
    {
        pTemp = tmOffset2Addr(p_url_cache, pTemp); /* use offset */
        printf("    Node %6d: ULR = %s , ExpireTime = %lu\n", j++, pTemp->req_url, pTemp->expire_time);
        pTemp = pTemp->mru;
    }
    pTemp = p_url_cache->free_node;
    j = 0;
    while(pTemp != NULL)
    {
        pTemp = tmOffset2Addr(p_url_cache, pTemp); /* use offset */
        j++;
        pTemp = pTemp->next;
    }
    printf("Free node number: %d\n",j);
    time(&now);
    printf("Current time = %lu\n",now);
    tm_unlock(&UrlFilterCacheLock);
    tm_lock_close(&UrlFilterCacheLock);
    shm_detach(p_url_cache);
}


int main(int argc, char* argv[])
{
    int nRetVal;
    
    if (argc == 1)
    {
        print_help();
        return 0;
    }
    
    if (strcmp(argv[1],"-dump") == 0)
    {
        if (argc != 3)
        {
            print_help();
            return 0;
        }
        testDumpUrlCache(atoi(argv[2]));
        tmDumpUrlCache(atoi(argv[2]), 4);
    }
    else if (strcmp(argv[1],"-add") == 0)
    {
        if (argc != 5)
        {
            print_help();
            return 0;
        }
        tmDumpUrlCache(atoi(argv[2]), 0);
        tmUrlCacheAddNode(atoi(argv[2]), argv[4], atoi(argv[3]));
        tmDumpUrlCache(atoi(argv[2]), 1);
    }
    else if (strcmp(argv[1],"-reset") == 0)
    {
        if (argc != 3)
        {
            print_help();
            return 0;
        }
        tmInitUrlFilterCache(atoi(argv[2]));
    }
    else if (strcmp(argv[1],"-query") == 0)
    {
        if (argc != 5)
        {
            print_help();
            return 0;
        }
        tmDumpUrlCache(atoi(argv[2]), 3);
        nRetVal = tmQueryUrlCache(atoi(argv[2]), argv[4], atoi(argv[3]));
        if (nRetVal == URL_IN_CACHE)
        {
            printf("In cache\n");
        }
        else if (nRetVal == URL_NOT_INCACHE)
        {
            printf("Not in cache\n");
        }
        else
        {
            printf("In cache but expired\n");
        }
    }
    else
    {
        print_help();
        return 0;
    }
    
    return 0;
}

