/************************************************************************
 * platform.h -
 ************************************************************************
 * Log No.  Date         Modified By    Description
 * =======  ==========   ============   =================================
 *
 ************************************************************************/

#ifndef _PLATFORM_H_
#define _PLATFORM_H_

#undef PR_BIG_ENDIAN

/* Win base related platforms */
#ifdef WIN32
	#include <windows.h>
	#ifdef _DEBUG
		#include <crtdbg.h>
	#endif
#endif
/* Sun related platforms */
#ifdef PR_SOL
	#define PR_BIG_ENDIAN
	/* #include <m_sol.h> */
#endif
/* IBM related platforms */
#ifdef PR_OS390
	#define PR_BIG_ENDIAN
	/* #include <m_os390.h> */
	#define _TM_OS390
#endif
#ifdef PR_AIX
	#define PR_BIG_ENDIAN
	/* #include <m_aix.h> */
	#define _TM_OS390
	#define _TM_AIX
#endif
#ifdef PR_AS400
	#define PR_BIG_ENDIAN
	#include "m_as400.h"
	#define _TM_AS400
#endif

/* Standard types for AES & DES cryptography source code */
typedef unsigned char   u1byte; /* an 8 bit unsigned character type */
typedef unsigned short  u2byte; /* a 16 bit unsigned integer type   */
typedef unsigned long   u4byte; /* a 32 bit unsigned integer type   */
typedef signed char     s1byte; /* an 8 bit signed character type   */
typedef signed short    s2byte; /* a 16 bit signed integer type     */
typedef signed long     s4byte; /* a 32 bit signed integer type     */

/* define default ASCII <--> EBCDIC code convertion function */
#define __VSA2E(a,b) ;
#define __VSE2A(a,b) ;
#define __VSA2EC(asc) asc
#define __VSE2AC(ebc) ebc

/* AES option */
//#define PR_LARGE_TABLES
/* DES option */
//#define PR_BIG_TABLES

/* Basic macros for speeding up generic operations */
/* Circular rotate of 32 bit values                */
#ifdef _MSC_VER /* WIN32 */
    #include <stdlib.h>
    #pragma intrinsic(_lrotr,_lrotl)
    #define rotr(x,n) _lrotr(x,n)
    #define rotl(x,n) _lrotl(x,n)
#else           /* UNIX */
    #define rotr(x,n)   (((x) >> ((int)((n) & 0x1f))) | \
						((x) << ((int)((32 - ((n) & 0x1f))))))
    #define rotl(x,n)   (((x) << ((int)((n) & 0x1f))) | \
						((x) >> ((int)((32 - ((n) & 0x1f))))))
#endif

/* Invert byte order in a 32 bit variable */
#define bswap(x)    (rotl(x, 8) & 0x00ff00ff | rotr(x, 8) & 0xff00ff00)

/* Put or get a 32 bit word (v) in machine order */
/* from a byte address in (x)					 */
#ifdef  PR_BIG_ENDIAN  /* e.g. PowerPC */
    #define u4byte_in(x)        bswap( *(u4byte*)(x) )
    #define u4byte_in2(po,pi)       {int i;for(i=0;i<sizeof(u4byte);i++) \
									 {((u1byte*)(po))[i]= \
									 (pi)[sizeof(u4byte)-1-i];}}
    #define u4byte_out(x, v)    (*(u4byte*)(x) = bswap(v))
    #define u4byte_out2(po,pi)       {int i;for(i=0;i<sizeof(u4byte);i++) \
									 {(po)[i]= \
									 ((u1byte*)(pi))[sizeof(u4byte)-1-i];}}
#else               /* e.g Intel CPU */
    #define u4byte_in(x)        (*(u4byte*)(x))
    #define u4byte_in2(po,pi)       {int i;for(i=0;i<sizeof(u4byte);i++) \
									 {((u1byte*)(po))[i]=(pi)[i];}}
    #define u4byte_out(x, v)    (*(u4byte*)(x) = (u4byte)(v))
    #define u4byte_out2(po,pi)       {int i;for(i=0;i<sizeof(u4byte);i++) \
									 {(po)[i]=((u1byte*)(pi))[i];}}
#endif	/* PR_BIG_ENDIAN */

#endif /* _PLATFORM_H_ */
