#include <memory.h>
#include "rijndael.h"

//namespace PR_COMMON {

#ifdef  __cplusplus
    namespace
    {
#endif

static u1byte f_mult(aes_sec_cnt* pCnt, u1byte a, u1byte b)
{
    u1byte aa = pCnt->log_tab[a], cc = aa + pCnt->log_tab[b];

    return pCnt->pow_tab[cc + (cc < aa ? 1 : 0)];

}

/* Extract byte from a 32 bit quantity (little endian notation) */

#define byte(x,n)   ((u1byte)((x) >> (8 * (n))))

#define ff_mult(c,a,b)    (a && b ? f_mult(c, a, b) : 0)

#define f_rn(bo, bi, n, k)                          \
    bo[n] =  pCnt->ft_tab[0][byte(bi[n],0)] ^             \
             pCnt->ft_tab[1][byte(bi[(n + 1) & 3],1)] ^   \
             pCnt->ft_tab[2][byte(bi[(n + 2) & 3],2)] ^   \
             pCnt->ft_tab[3][byte(bi[(n + 3) & 3],3)] ^ *(k + n)

#define i_rn(bo, bi, n, k)                          \
    bo[n] =  pCnt->it_tab[0][byte(bi[n],0)] ^             \
             pCnt->it_tab[1][byte(bi[(n + 3) & 3],1)] ^   \
             pCnt->it_tab[2][byte(bi[(n + 2) & 3],2)] ^   \
             pCnt->it_tab[3][byte(bi[(n + 1) & 3],3)] ^ *(k + n)

#ifdef PR_LARGE_TABLES
    #define ls_box(x)                \
    ( pCnt->fl_tab[0][byte(x, 0)] ^    \
      pCnt->fl_tab[1][byte(x, 1)] ^    \
      pCnt->fl_tab[2][byte(x, 2)] ^    \
      pCnt->fl_tab[3][byte(x, 3)] )
    #define f_rl(bo, bi, n, k)                          \
    bo[n] =  pCnt->fl_tab[0][byte(bi[n],0)] ^             \
             pCnt->fl_tab[1][byte(bi[(n + 1) & 3],1)] ^   \
             pCnt->fl_tab[2][byte(bi[(n + 2) & 3],2)] ^   \
             pCnt->fl_tab[3][byte(bi[(n + 3) & 3],3)] ^ *(k + n)
    #define i_rl(bo, bi, n, k)                          \
    bo[n] =  pCnt->il_tab[0][byte(bi[n],0)] ^             \
             pCnt->il_tab[1][byte(bi[(n + 3) & 3],1)] ^   \
             pCnt->il_tab[2][byte(bi[(n + 2) & 3],2)] ^   \
             pCnt->il_tab[3][byte(bi[(n + 1) & 3],3)] ^ *(k + n)
#else
    #define ls_box(x)                            \
    ((u4byte)pCnt->sbx_tab[byte(x, 0)] <<  0) ^    \
    ((u4byte)pCnt->sbx_tab[byte(x, 1)] <<  8) ^    \
    ((u4byte)pCnt->sbx_tab[byte(x, 2)] << 16) ^    \
    ((u4byte)pCnt->sbx_tab[byte(x, 3)] << 24)
    #define f_rl(bo, bi, n, k)                                      \
    bo[n] = (u4byte)pCnt->sbx_tab[byte(bi[n],0)] ^                    \
        rotl(((u4byte)pCnt->sbx_tab[byte(bi[(n + 1) & 3],1)]),  8) ^  \
        rotl(((u4byte)pCnt->sbx_tab[byte(bi[(n + 2) & 3],2)]), 16) ^  \
        rotl(((u4byte)pCnt->sbx_tab[byte(bi[(n + 3) & 3],3)]), 24) ^ *(k + n)
    #define i_rl(bo, bi, n, k)                                      \
    bo[n] = (u4byte)pCnt->isb_tab[byte(bi[n],0)] ^                    \
        rotl(((u4byte)pCnt->isb_tab[byte(bi[(n + 3) & 3],1)]),  8) ^  \
        rotl(((u4byte)pCnt->isb_tab[byte(bi[(n + 2) & 3],2)]), 16) ^  \
        rotl(((u4byte)pCnt->isb_tab[byte(bi[(n + 1) & 3],3)]), 24) ^ *(k + n)
#endif

static void gen_tabs(aes_sec_cnt* pCnt)
{

    u4byte  i, t;
    u1byte  p, q;

    /********************************************************/
    /* log and power tables for GF(2**8) finite field with  */
    /* 0x011b as modular polynomial - the simplest prmitive */
    /* root is 0x03, used here to generate the tables       */
    /********************************************************/

    for(i = 0,p = 1; i < 256; ++i)
    {
        pCnt->pow_tab[i] = (u1byte)p;
		pCnt->log_tab[p] = (u1byte)i;

        p ^=  (p << 1) ^ (p & 0x80 ? 0x01b : 0);
    }

    pCnt->log_tab[1] = 0;

    for(i = 0,p = 1; i < 10; ++i)
    {
        pCnt->rco_tab[i] = p;

        p = (p << 1) ^ (p & 0x80 ? 0x01b : 0);
    }

    for(i = 0; i < 256; ++i)
    {
        p = (i ? pCnt->pow_tab[255 - pCnt->log_tab[i]] : 0);
        q  = ((p >> 7) | (p << 1)) ^ ((p >> 6) | (p << 2));
        p ^= 0x63 ^ q ^ ((q >> 6) | (q << 2));
        pCnt->sbx_tab[i] = p; pCnt->isb_tab[p] = (u1byte)i;
    }

    for(i = 0; i < 256; ++i)
    {
        p = pCnt->sbx_tab[i];

#ifdef  PR_LARGE_TABLES
        t = p; pCnt->fl_tab[0][i] = t;
        pCnt->fl_tab[1][i] = rotl(t,  8);
        pCnt->fl_tab[2][i] = rotl(t, 16);
        pCnt->fl_tab[3][i] = rotl(t, 24);
#endif
        t = ((u4byte)ff_mult(pCnt, 2, p)) |
            ((u4byte)p <<  8) |
            ((u4byte)p << 16) |
            ((u4byte)ff_mult(pCnt, 3, p) << 24);

        pCnt->ft_tab[0][i] = t;
        pCnt->ft_tab[1][i] = rotl(t,  8);
        pCnt->ft_tab[2][i] = rotl(t, 16);
        pCnt->ft_tab[3][i] = rotl(t, 24);

        p = pCnt->isb_tab[i];

#ifdef  PR_LARGE_TABLES

        t = p; pCnt->il_tab[0][i] = t;
        pCnt->il_tab[1][i] = rotl(t,  8);
        pCnt->il_tab[2][i] = rotl(t, 16);
        pCnt->il_tab[3][i] = rotl(t, 24);
#endif
        t = ((u4byte)ff_mult(pCnt, 14, p)) |
            ((u4byte)ff_mult(pCnt,  9, p) <<  8) |
            ((u4byte)ff_mult(pCnt, 13, p) << 16) |
            ((u4byte)ff_mult(pCnt, 11, p) << 24);

        pCnt->it_tab[0][i] = t;
        pCnt->it_tab[1][i] = rotl(t,  8);
        pCnt->it_tab[2][i] = rotl(t, 16);
        pCnt->it_tab[3][i] = rotl(t, 24);
    }

    pCnt->tab_gen = 1;
}

#define star_x(x) (((x) & 0x7f7f7f7f) << 1) ^ \
        ((((x) & 0x80808080) >> 7) * 0x1b)

#define imix_col(y,x)       \
    u   = star_x(x);        \
    v   = star_x(u);        \
    w   = star_x(v);        \
    t   = w ^ (x);          \
   (y)  = u ^ v ^ w;        \
   (y) ^= rotr(u ^ t,  8) ^ \
          rotr(v ^ t, 16) ^ \
          rotr(t,24)

#ifdef  __cplusplus
    }   /* end of anonymous namespace */
#endif

/* initialise the key schedule from the user supplied key */

#define loop4(i)                                    \
{   t = rotr(t,  8); t = ls_box(t) ^ pCnt->rco_tab[i];    \
    t ^= pCnt->e_key[4 * i];     pCnt->e_key[4 * i + 4] = t;    \
    t ^= pCnt->e_key[4 * i + 1]; pCnt->e_key[4 * i + 5] = t;    \
    t ^= pCnt->e_key[4 * i + 2]; pCnt->e_key[4 * i + 6] = t;    \
    t ^= pCnt->e_key[4 * i + 3]; pCnt->e_key[4 * i + 7] = t;    \
}

#define loop6(i)                                    \
{   t = rotr(t,  8); t = ls_box(t) ^ pCnt->rco_tab[i];    \
    t ^= pCnt->e_key[6 * i];     pCnt->e_key[6 * i + 6] = t;    \
    t ^= pCnt->e_key[6 * i + 1]; pCnt->e_key[6 * i + 7] = t;    \
    t ^= pCnt->e_key[6 * i + 2]; pCnt->e_key[6 * i + 8] = t;    \
    t ^= pCnt->e_key[6 * i + 3]; pCnt->e_key[6 * i + 9] = t;    \
    t ^= pCnt->e_key[6 * i + 4]; pCnt->e_key[6 * i + 10] = t;   \
    t ^= pCnt->e_key[6 * i + 5]; pCnt->e_key[6 * i + 11] = t;   \
}

#define loop8(i)                                    \
{   t = rotr(t,  8); ; t = ls_box(t) ^ pCnt->rco_tab[i];  \
    t ^= pCnt->e_key[8 * i];     pCnt->e_key[8 * i + 8] = t;    \
    t ^= pCnt->e_key[8 * i + 1]; pCnt->e_key[8 * i + 9] = t;    \
    t ^= pCnt->e_key[8 * i + 2]; pCnt->e_key[8 * i + 10] = t;   \
    t ^= pCnt->e_key[8 * i + 3]; pCnt->e_key[8 * i + 11] = t;   \
    t  = pCnt->e_key[8 * i + 4] ^ ls_box(t);    \
    pCnt->e_key[8 * i + 12] = t;                \
    t ^= pCnt->e_key[8 * i + 5]; pCnt->e_key[8 * i + 13] = t;   \
    t ^= pCnt->e_key[8 * i + 6]; pCnt->e_key[8 * i + 14] = t;   \
    t ^= pCnt->e_key[8 * i + 7]; pCnt->e_key[8 * i + 15] = t;   \
}

void set_key(aes_sec_cnt* pCnt, const u1byte in_key[], const u4byte key_len, u1byte f)
{

    u4byte  i, t, u, v, w;

    memset(pCnt, 0, sizeof(aes_sec_cnt));

    if(!pCnt->tab_gen)
        gen_tabs(pCnt);

    pCnt->k_len = (key_len + 31) / 32;

	u4byte_in2(pCnt->e_key  , in_key     );
    u4byte_in2(pCnt->e_key+1, in_key +  4);
    u4byte_in2(pCnt->e_key+2, in_key +  8);
    u4byte_in2(pCnt->e_key+3, in_key + 12);

    switch(pCnt->k_len)
    {
        case 4: t = pCnt->e_key[3];
                for(i = 0; i < 10; ++i)
                    loop4(i);
                break;
    }

    if(f != 1) /*mode != 1)*/
    {
        pCnt->d_key[0] = pCnt->e_key[0]; pCnt->d_key[1] = pCnt->e_key[1];
        pCnt->d_key[2] = pCnt->e_key[2]; pCnt->d_key[3] = pCnt->e_key[3];

        for(i = 4; i < 4 * pCnt->k_len + 24; ++i)
        {
            imix_col(pCnt->d_key[i], pCnt->e_key[i]);
        }
    }

    return;

}

#ifdef _TMPRSRV_
/* encrypt a block of text */

#define f_nround(bo, bi, k) \
    f_rn(bo, bi, 0, k);     \
    f_rn(bo, bi, 1, k);     \
    f_rn(bo, bi, 2, k);     \
    f_rn(bo, bi, 3, k);     \
    k += 4

#define f_lround(bo, bi, k) \
    f_rl(bo, bi, 0, k);     \
    f_rl(bo, bi, 1, k);     \
    f_rl(bo, bi, 2, k);     \
    f_rl(bo, bi, 3, k)


void encrypt(aes_sec_cnt* pCnt, const u1byte in_blk[16], u1byte out_blk[16])
{

    u4byte  b0[4], b1[4], *kp;

	u4byte_in2(b0  , in_blk     );
    u4byte_in2(b0+1, in_blk +  4);
    u4byte_in2(b0+2, in_blk +  8);
    u4byte_in2(b0+3, in_blk + 12);
    b0[0] ^= pCnt->e_key[0];
    b0[1] ^= pCnt->e_key[1];
    b0[2] ^= pCnt->e_key[2];
    b0[3] ^= pCnt->e_key[3];

    kp = pCnt->e_key + 4;

    f_nround(b1, b0, kp); f_nround(b0, b1, kp);
    f_nround(b1, b0, kp); f_nround(b0, b1, kp);
    f_nround(b1, b0, kp); f_nround(b0, b1, kp);
    f_nround(b1, b0, kp); f_nround(b0, b1, kp);
    f_nround(b1, b0, kp); f_lround(b0, b1, kp);

    u4byte_out2(out_blk,      b0  ); u4byte_out2(out_blk +  4, b0+1);
    u4byte_out2(out_blk +  8, b0+2); u4byte_out2(out_blk + 12, b0+3);

}
#endif /* _TMPRSRV_ */

/* decrypt a block of text */

#define i_nround(bo, bi, k) \
    i_rn(bo, bi, 0, k);     \
    i_rn(bo, bi, 1, k);     \
    i_rn(bo, bi, 2, k);     \
    i_rn(bo, bi, 3, k);     \
    k -= 4

#define i_lround(bo, bi, k) \
    i_rl(bo, bi, 0, k);     \
    i_rl(bo, bi, 1, k);     \
    i_rl(bo, bi, 2, k);     \
    i_rl(bo, bi, 3, k)

void decrypt(aes_sec_cnt* pCnt, const u1byte in_blk[16], u1byte out_blk[16])
{

    u4byte  b0[4], b1[4], *kp;

	u4byte_in2(b0  , in_blk     );
    u4byte_in2(b0+1, in_blk +  4);
    u4byte_in2(b0+2, in_blk +  8);
    u4byte_in2(b0+3, in_blk + 12);
    b0[0] ^= pCnt->e_key[4 * pCnt->k_len + 24];
    b0[1] ^= pCnt->e_key[4 * pCnt->k_len + 25];
    b0[2] ^= pCnt->e_key[4 * pCnt->k_len + 26];
    b0[3] ^= pCnt->e_key[4 * pCnt->k_len + 27];

    kp = pCnt->d_key + 4 * (pCnt->k_len + 5);

    i_nround(b1, b0, kp); i_nround(b0, b1, kp);
    i_nround(b1, b0, kp); i_nround(b0, b1, kp);
    i_nround(b1, b0, kp); i_nround(b0, b1, kp);
    i_nround(b1, b0, kp); i_nround(b0, b1, kp);
    i_nround(b1, b0, kp); i_lround(b0, b1, kp);

    u4byte_out2(out_blk,      b0  ); u4byte_out2(out_blk +  4, b0+1);
    u4byte_out2(out_blk +  8, b0+2); u4byte_out2(out_blk + 12, b0+3);

}

//} /* namespace PR_COMMON */
