/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file   testINIPaser.c
 *  @brief   
 */

/*
 *  Change Log
 *  2004-02-14  Jeremy  initial version
 */
#include <stdio.h>
#include <unistd.h>
#include "tmauIniParser.h"

char *strEngineVxDKey  = "E.2=";   /* Leading key for the VxD engine */
char *strEngineNTKDKey = "E.10=";  /* Leading key for the NTKD engine */
char *strPatternKey    = "P.4";    /* Leading key for the VSAPI pattern */


int main(int argc, char **argv)
{
    TmErrorCode     tmErrCode = TM_ERR_ZERO_ERROR;
    TmuVersion_t    TmuPatVer, TmuEngVer;

    char            szPat[TM_AU_MAX_VER];
    char            szEngineVxD[TM_AU_MAX_VER];
    char            szEngineNTKD[TM_AU_MAX_VER];


    if (argc != 2) 
    {
        fprintf(stderr, "Usage: %s <server.ini> \n", argv[0]);
        exit(1);
    }
   
    if (access(argv[1], R_OK) != TM_ERR_ZERO_ERROR)
    {
        fprintf(stderr, "Cannot find the target file, [%s]\n", argv[1]);
        exit(1);
    }
    
    memset(&TmuPatVer, 0, sizeof(TmuPatVer));
    memset(&TmuEngVer, 0, sizeof(TmuEngVer));
        
    if (tmau_GetPatternVer(argv[1], strPatternKey, &TmuPatVer, &tmErrCode))
    {
        snprintf (szPat, TM_AU_MAX_STR, "%d", TmuPatVer.version);        
        printf("The lastest version of pattern[%s] is: %s\n", strPatternKey, szPat);        
    }
    else if (TM_FAILURE(tmErrCode))
    {
        printf("The error return of parsing pattern version is:\n");
        printf("\t\"Facility\" is       [%02x]\n", TM_COND_FACILITY(tmErrCode));
        printf("\t\"Source ID\" is      [%02x]\n", TM_COND_SOURCE(tmErrCode));
        printf("\t\"Condition Code\" is [%04x]\n", TM_COND_CODE(tmErrCode));
    }    
    
    if (tmau_GetEngineVer(argv[1], strEngineVxDKey, &TmuEngVer, &tmErrCode))
    {
        tmau_MakeVersionBuildString(TmuEngVer, szEngineVxD, TM_AU_MAX_VER);
        printf("The latest version of engine VxD is: %s\n", szEngineVxD);
    }
    else if (TM_FAILURE(tmErrCode))
    {
        printf("The error return of parsing VxD engine is:\n");
        printf("\t\"Facility\" is       [%02x]\n", TM_COND_FACILITY(tmErrCode));
        printf("\t\"Source ID\" is      [%02x]\n", TM_COND_SOURCE(tmErrCode));
        printf("\t\"Condition Code\" is [%04x]\n", TM_COND_CODE(tmErrCode));
    }

    memset(&TmuEngVer, 0, sizeof(TmuEngVer));   
    if (tmau_GetEngineVer(argv[1], strEngineNTKDKey, &TmuEngVer, &tmErrCode))
    {
        tmau_MakeVersionBuildString(TmuEngVer, szEngineNTKD, TM_AU_MAX_VER);
        printf("The latest version of engine NTKD is: %s\n", szEngineNTKD);
    }
    else if (TM_FAILURE(tmErrCode))
    {
        printf("The error return of parsing NTKD engine are:\n");
        printf("\t\"Facility\" is       [%02x]\n", TM_COND_FACILITY(tmErrCode));
        printf("\t\"Source ID\" is      [%02d]\n", TM_COND_SOURCE(tmErrCode));
        printf("\t\"Condition Code\" is [%04x]\n", TM_COND_CODE(tmErrCode));
    }

    return 0;
}


