/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @tmcfg.c
 *  @TMSS configuration Module
 */

/*
 *  Change Log
 *  xxxx-xx-xx Name    Description
 */


#include <unistd.h>
//#include <mcheck.h>
#include <stdio.h>

#include "tmcfg.h"
#include "log.h"
//static char ModuleName[]="tmcfg";

void usage(const char *ProgName) {
  fprintf(stderr, "Usage:\n  tmcfg\t<config file> val <module name> <param name> \n");
  fprintf(stderr, "\t<config file> opt <module name> <param name> \n");
  fprintf(stderr, "\t<config file> set <module name> <param name> <value> <option>\n");
  fprintf(stderr, "\t<config file> del <module name>\n");
  fprintf(stderr, "\t<config file> del <module name> <param name>\n");
}

int main(int argc, char *argv[]) {

  const char *filename=argv[1];
  const char *action=argv[2];
  const char *module=argv[3];
  const char *param=argv[4];
  const char *value=argv[5];
  const char *option=argv[6];

  LOGOPEN("tmcfg");

  //  mtrace();

  if(argc<3 || !((action[0]=='v' && argc==5) ||
       (action[0]=='o' && argc==5) ||
       (action[0]=='s' && (argc==6 || argc==7)) ||
       (action[0]=='d' && (argc==4 || argc==5)) )) {
    usage(argv[0]);
    return 1;
  }


  if(action[0]=='v' || action[0]=='o') {
    tmcfg_t config;
    const char *pv, *po;

    if(NULL==(config=tmcfg_open(filename, TMCFG_MODE_READ))) {
      fprintf(stderr, "Failed to open configuration file: %m\n");
      return 2;
    }
    pv=tmcfg_get_param_value(config, module, param, &po);
    if(action[0]=='v' && pv!=NULL) {
      fputs(pv, stdout);
    } else if(action[0]=='o' && po!=NULL) {
      fputs(po, stdout);
    } else {
      fprintf(stderr, "<NULL>\n");
    }
    tmcfg_close(config);
    return 0;
  } else if(action[0]=='s' || action[0]=='d') {
    tmcfg_t config;

    if(NULL==(config=tmcfg_open(filename, TMCFG_MODE_UPDATE))) {
      fprintf(stderr, "Failed to open configuration file: %m\n");
      return 3;
    }

    if(action[0] == 's') {
    	if(0!=tmcfg_set_param_value(config, module, param, value, (argc==7?option:NULL))) {
      		fprintf(stderr, "Failed to set param value: %m\n");
      		return 4;
    	}
    } else if(action[0] == 'd') {
	int ret=0;
	if(argc==4) {// delete module
		if((ret=tmcfg_del_module(config, module)) < 0) {
			fprintf(stderr, "Failed to del module: %s\n", module);
			return 5;
		}
		if(ret == 0)
			fprintf(stderr, "Module %s not found\n", module);
	} else if(argc==5) {	// delete parameter
		if((ret=tmcfg_del_param(config, module, param)) < 0) {
			fprintf(stderr, "Failed to del parameter: %s(%s)\n", module, param);
			return 6;
		}
		if(ret == 0)
			fprintf(stderr, "Module %s parameter %s not found\n", module, param);
	}
    }

    if(0!=tmcfg_flush(config)) {
      fprintf(stderr, "Failed to write configuration file: %m\n");
      return 7;
    }
    tmcfg_close(config);
    return 0;
  }

  //  muntrace();

  LOGCLOSE();

  return 0;
}
