/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file  tmException.c
 *  @brief TMOG AVEnforcer Module - Exception List
 */

/*
 *  Change Log
 *  2004-02-18 Eric Li Initialization version
 */

#include <arpa/inet.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include "tmException.h"
#include "tmcfg.h"
#include "tmShm.h"
#include "tmSemaphore.h"
#include "log.h"
#include <linux/netfilter_ipv4/ipt_webstr.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <unistd.h>

#define MAX_IP_LIST             100
#define MAX_PATH                256
#define MAX_LINE                 80

#define EXCEPTION_SIZE           1
#define EXCEPTION_DST            0


static const char ModuleName[] = "exception";
//unsigned int exceptionConfigVersion=0;

typedef struct 
{
 //   uint32_t version;
    uint32_t enabled;
    uint32_t size;
    uint32_t addr[MAX_IP_LIST];
    char     path[MAX_PATH];
} ExceptionList;

static ExceptionList gList[EXCEPTION_SIZE] = 
{
    {1, 0, {0}, "avedst.dat"},
};

static void bsort(uint32_t data[], int size)
{
    int i, j;
    uint32_t tmp;

    for (i = size-1; i >= 0; i--)
    {
         for (j = 1; j <= i; j++)
         {
              if (data[j-1] > data[j])
              {
                  tmp = data[j-1];
                  data[j-1] = data[j];
                  data[j] = tmp;
              }
         }
    }
}

#if 0
static int bisearch(uint32_t data[], int length, uint32_t target)
{
    int high = length, low = -1, probe;

    while (high - low > 1)
    {
        probe = (high + low) / 2;
        if (data[probe] < target)
            low = probe;
        else
            high = probe;
    }
    if (high == length || data[high] != target)
        return -1;
    else
        return high;
}
#endif

static void tmWriteException(ExceptionList *exp)
{
    FILE *fp;
    int  i;

    LOGINFO("Write exception file: %s", exp->path);

    fp = fopen(exp->path, "w");
    if (fp == NULL)
    {
        LOGERROR("Cannot write exception file: %s\n", exp->path);
        return;
    }

    for (i = 0; i < exp->size; i++)
    {
         fprintf(fp, "%u.%u.%u.%u\n", NIPQUAD(exp->addr[i]));
    }

    fclose(fp);
}

static void tmReadException(ExceptionList *exp)
{
    int   i;
    FILE *fp;
    char buf[MAX_LINE];
    uint32_t ip;

    LOGINFO("Load exception file: %s", exp->path);

    exp->size = 0;
    if ((fp = fopen(exp->path, "r")) == NULL)
    {
        LOGERROR("Cannot read exception file: %s\n", exp->path);
        return;
    }

    while (exp->size < MAX_IP_LIST && fgets(buf, sizeof(buf), fp) != NULL) 
    {
           /* strip the newline */
           for (i = 0; i < MAX_LINE && buf[i] != '\0'; i++)
           {
                if (buf[i] == '\r' || buf[i] == '\n')
                {
                    buf[i] = '\0';
                    break;
                }
           }
           if (inet_pton(AF_INET, buf, &ip) <= 0)
           {
                LOGERROR("WRONG IP: [%s]\n", buf);
           }
           else
           {
                int idx, duplicate = 0;

                /* check if there is already a duplicate */
                for (idx = 0; idx < exp->size; idx++)
                {
                     if (exp->addr[idx] == ip)
                     {   
                         duplicate = 1;
                         break;
                     }
                }

                if (!duplicate)
                {
                    exp->addr[exp->size++] = ip;
                    LOGDEBUG("IP=%u.%u.%u.%u\n", NIPQUAD(ip));
                }
           }
    }

    fclose(fp);
    bsort(exp->addr, exp->size);
}



int sendTMExceptionToKernel(ExceptionList *exlist)
{
    int nRetVal = 0;
    size_t len;
    int sockfd = -1; 
    
	// We need to send the Exception to tmss_kernel(ipt_webstr)
	sockfd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW);
	if (sockfd < 0)
	{
	    fprintf(stderr,"sendTMExceptionToKernel:Can not create sock:%s\a\n",strerror(errno)); 
		return -1;
	}

	len=sizeof(ExceptionList);
	if (setsockopt(sockfd, IPPROTO_IP, TMSS_SO_TM_EXCEPTION, exlist,len ) < 0)
	{
	    fprintf(stderr,"sendTMExceptionToKernel:Can not setsockopt sock:%s\a\n",strerror(errno)); 
	close(sockfd);
	sockfd = -1;
		return -1;
	}
	close(sockfd);
	sockfd = -1;
    return nRetVal;
}

/**
  * @brief load settings from a config file
  * @param file configuration file path
  */
void tmexp_LoadSettings(const char *file)
{
    int   i;
    tmcfg_t cfg;
    const char *ptr;
    char buffer[128];

    const char *mode[EXCEPTION_SIZE] = {"avedst"};

    LOGINFO("tmexp_LoadSettings:Load config file: [%s]", file);
    if ((cfg = tmcfg_open(file, TMCFG_MODE_READ)) == NULL)
    {
        LOGERROR("failed to open config file: [%s]", file);
        return;
    }

    for (i = 0; i < EXCEPTION_SIZE; i++)
    {   
         /* no buffer overflow */
         sprintf(buffer, "enable_");
         strcat(buffer, mode[i]);

         ptr = tmcfg_get_param_value(cfg, ModuleName, buffer, NULL);
         if (ptr != NULL)
             gList[i].enabled = atoi(ptr);

         /* no buffer overflow */
         sprintf(buffer, "path_");
         strcat(buffer, mode[i]);

         ptr = tmcfg_get_param_value(cfg, ModuleName, buffer, NULL);
         if (ptr != NULL)
         {
             strncpy(gList[i].path, ptr, sizeof(gList[i].path));
             gList[i].path[sizeof(gList[i].path)-1] = 0;
         }

         tmReadException(gList+i);
         tmWriteException(gList+i);
         LOGINFO("Settings(%s): [%u][%s][%d]", mode[i], gList[i].enabled,
                       gList[i].path, gList[i].size);

//	gList[i].version=exceptionConfigVersion;
//	exceptionConfigVersion++;
     }
    

    //loadExceptionConfInCache(&gList[0]);// Only EXCEPTION SIZE =1 is support
    sendTMExceptionToKernel(&gList[0]);// Only EXCEPTION SIZE =1 is support
    tmcfg_close(cfg);
}

