/******************************************************************************
*
*  (C) Copyright 2004 Trend Micro, Inc.
*
*  All Rights Reserved.
*  This program is an unpublished copyrighted work which is proprietary
*  to Trend Micro, Inc. and contains confidential information that is not
*  to be reproduced or disclosed to any other person or entity without
*  prior written consent from Trend Micro, Inc. in each and every instance.
*
*  WARNING:  Unauthorized reproduction of this program as well as
*  unauthorized preparation of derivative works based upon the
*  program or distribution of copies by sale, rental, lease or
*  lending are violations of federal copyright laws and state trade
*  secret laws, punishable by civil and criminal penalties.
*
*****************************************************************************/

/**
*  @file tmCGICommon.c
*  @brief TMOG CGI Common
*/

/*
*  Change Log
*  2004-03-04 Henry Lin    Initialization version
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <signal.h>
#include "tmCGILib.h"
#include "log.h"
//#include "tmURLFilterSetting.h"
#include "tmcfg.h"
#include "tmURLFilterConf.h"
#include "tmCGICommon.h"
#include "tmClientInfo.h" /* MacToStr() */

static const char ModuleName[] = "tmCGICommon";

void ShowErrorPage(const char *pszErrTitle, const char *pszErrStr)
{
    TM_Entry *replace = NULL;
    replace = tmSedArgAdd(replace, ERROR_PAGE_REPLACE_TOKEN_ERROR_TITLE, "%s", pszErrTitle);
    replace = tmSedArgAdd(replace, ERROR_PAGE_REPLACE_TOKEN_ERROR_STR, "%s", pszErrStr);
    tmSedFile(replace, ERROR_PAGE_PATH, stdout);
    tmSedArgFree(replace);
    return;
}


int ReloadThpdConf(void)
{
    FILE *fp;
    char szBuf[MAX_LINE];
    long longPid;
    int nRetVal = 0;
    
    fp = fopen(THPD_PID_PATH, "r");
    
    if (fp == NULL)
    {
        return -1;
    }
    
    szBuf[MAX_LINE - 1] = '\0';
    fgets(szBuf, MAX_LINE - 1, fp);
    
    longPid = strtol(szBuf, NULL, 10);
   
    LOGDEBUG("Sending Signal to thpd pid: %lu", longPid);
    kill((pid_t)longPid, SIGHUP);
    
    fclose(fp);
    return nRetVal;
}


int ReloadAUCheckerConf(void)
{
    FILE *fp;
    char szBuf[MAX_LINE];
    long longPid;
    int nRetVal = 0;
    
    fp = fopen(AUCHECKER_PID_PATH, "r");
    
    if (fp == NULL)
    {
        return -1;
    }
    
    szBuf[MAX_LINE - 1] = '\0';
    fgets(szBuf, MAX_LINE - 1, fp);
    
    longPid = strtol(szBuf, NULL, 10);
   
    LOGINFO("Sending Signal to tmAUChecker pid: %lu", longPid);
    kill((pid_t)longPid, SIGHUP);
    
    fclose(fp);
    return nRetVal;
}

/* to concat a string: <option value="00-04-76-E2-BE-5E">10.1.2.4 (Computer Name) to the buffer */
int AddListItem(const char* ip, const char* name, const unsigned char* value, char* buf, int bufsize)
{
    int nBufLeft;
    char szMAC[18];
    
    if (buf == NULL)
    {
        return -1;
    }
    
    if (strcmp(ip,"0.0.0.0") == 0)
    {
        return 0;
    }

    nBufLeft = bufsize - strlen(buf) - 1;
    
    if (nBufLeft <= 0)
    {
        return -1;
    }
    
    strncat(buf, "<option value=\"", nBufLeft);
    
    nBufLeft -= strlen("<option value=\"");

    if (nBufLeft <= 0)
    {
        return -1;
    }
    
    szMAC[0] = '\0';
    MacToStr(value, szMAC);
    
    strncat(buf, szMAC, nBufLeft);
    nBufLeft -= strlen(szMAC);

    if (nBufLeft <= 0)
    {
        return -1;
    }

    strncat(buf, "\">", nBufLeft);
    nBufLeft -= strlen("\">");

    if (nBufLeft <= 0)
    {
        return -1;
    }

    strncat(buf, ip, nBufLeft);
    nBufLeft -= strlen(ip);

    if (nBufLeft <= 0)
    {
        return -1;
    }

    strncat(buf, " (", nBufLeft);
    nBufLeft -= strlen(" (");

    if (nBufLeft <= 0)
    {
        return -1;
    }

    if (name != NULL && strcmp(name,"") != 0)
    {
        strncat(buf, name, nBufLeft);
        nBufLeft -= strlen(name);
    }
    else
    {
        strncat(buf, HTML_PAGE_REPLACE_VALUE_NA, nBufLeft);
        nBufLeft -= strlen(HTML_PAGE_REPLACE_VALUE_NA);
    }

    if (nBufLeft <= 0)
    {
        return -1;
    }
    
    strncat(buf, ")", nBufLeft);

    return 0;
}

void SetExceptionStatus(const char *pszMAC, uint8_t status, int nOption)
{
    struct client_exp Buf;
    ClientInfo Client;
    unsigned char  mac[MAC_SIZE];
    
    if (pszMAC == NULL)
    {
        LOGERROR("SetExceptionStatus(), NULL input");
        return;
    }
    
    StrToMac(pszMAC, mac);

    if (tmci_GetClient(mac, &Client) == 0)
    {
        if (nOption == CGI_EXCEPT_FILTER)
        {
            Buf.urlfilter = status;
            Buf.report = CISEXPREPORT(&Client);
        }
        else
        {
            Buf.urlfilter = CISEXPFILTER(&Client);
            Buf.report = status;        
        }
        if (tmci_SetExp(&(Client.idinfo), &Buf) != 0)
        {
            LOGERROR("SetExceptionStatus(), can't set client %s Exception to %d", pszMAC,status);
        }
        else
        {
            LOGDEBUG("SetExceptionStatus(), set client %s Exception to %d",pszMAC,status);
        }
    }
    else
    {
        LOGERROR("SetExceptionStatus(), can't find client %s", pszMAC);
    }
}

void SetExceptionComputerList(uint8_t status, int nOption)
{
    const char *pszMAC = NULL;
    
    if (status == 0)
    {
        pszMAC = tmValueFirst("%s", HTML_PAGE_VAR_NAME_EXCEPT_1);
    }
    else
    {
        pszMAC = tmValueFirst("%s", HTML_PAGE_VAR_NAME_EXCEPT_2);
    }
    
    if (pszMAC == NULL)
    {
        return;
    }
    else
    {
        LOGDEBUG("SetExceptionComputerList(), Call SetExceptionStatus(mac=%s,status=%d,option=%d)",pszMAC,status, nOption);
        SetExceptionStatus(pszMAC, status, nOption);
    }

    while(1)
    {    
        pszMAC = tmValueNext();
        if (pszMAC == NULL)
        {
            break;
        }
        else
        {
            LOGDEBUG("SetExceptionComputerList(), Call SetExceptionStatus(mac=%s,status=%d,option=%d)",pszMAC,status, nOption);
            SetExceptionStatus(pszMAC, status, nOption);
        }
    }
    
    return;
}

