/*
 * Copyright 2001-2003, Broadcom Corporation
 * All Rights Reserved.
 * 
 * THIS SOFTWARE IS OFFERED "AS IS", AND BROADCOM GRANTS NO WARRANTIES OF ANY
 * KIND, EXPRESS OR IMPLIED, BY STATUTE, COMMUNICATION OR OTHERWISE. BROADCOM
 * SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A SPECIFIC PURPOSE OR NONINFRINGEMENT CONCERNING THIS SOFTWARE.
 *
 * $Id: linux_main.c,v 1.2.36.1 2005/11/07 15:56:31 aki Exp $
 */

#include <errno.h>	    // for errno, of course.
#include <error.h>	    // for perror
#include <signal.h>	    // for signal, etc.
#include <assert.h>	    // for assert, of course.
#include <stdlib.h>	    // for malloc, free, etc.
#include <string.h>	    // for memset, strncasecmp, etc.
#include <stdarg.h>	    // for va_list, etc.
#include <stdio.h>	    // for printf, perror, fopen, fclose, etc.
#include <net/if.h>	    // for struct ifreq, etc.
#include <sys/ioctl.h>	    // for SIOCGIFCONF, etc.
#include <fcntl.h>	    // for fcntl, F_GETFL, etc.
#include <unistd.h>	    // for read, write, etc.
#include <arpa/inet.h>	    // for inet_aton, inet_addr, etc.
#include <time.h>	    // for time
#include <netinet/in.h>	    // for sockaddr_in
#include <wait.h>	    // for sockaddr_in


#include "ctype.h"
#include "upnp_dbg.h"
#include "upnp.h"

extern void define_variable(char *name, char *value);
extern void uuidstr_create(char *);
extern char *strip_chars(char *, char *);
extern void init_devices();
extern char* read_file_para_value(char *filename, char *arg1,char *buf);
extern int nvram_set(const char *name, const char *value);
void init_event_queue(int);

extern struct net_connection *net_connections;
extern int global_exit_now;
extern struct iface *global_lans;

static void
reap(int sig)
{
	pid_t pid;

	while ((pid = waitpid(-1, NULL, WNOHANG)) > 0)
		fprintf(stderr,"Reaped %d\n", pid);
}

void write_client_pid_file ();

int log_perror_client = 1;

void go_daemon ()
{
	static int state = 0;
	int pid;


	/* Only do it once. */
	if (state)
		return;
	state = 1;

	/* Stop logging to stderr... */
	log_perror_client = 0;

	/* Become a daemon... */
	if ((pid = vfork ()) < 0)
		fprintf(stderr,"Can't fork daemon: %m\n");
	else if (pid)
		exit (0);
	/* Become session leader and get pid... */
	pid = setsid ();

	/* Close standard I/O descriptors. */
        close(0);
        close(1);
        close(2);

	write_client_pid_file ();
}
#define _PATH_UPNP_PID "/var/run/upnp.pid"
char *path_dhclient_pid = _PATH_UPNP_PID;



void write_client_pid_file ()
{
	FILE *pf;
	int pfdesc;

	pfdesc = open (path_dhclient_pid, O_CREAT | O_TRUNC | O_WRONLY, 0644);

	if (pfdesc < 0) {
		fprintf(stderr,"Can't create %s: %m\n", path_dhclient_pid);
		return;
	}

	pf = fdopen (pfdesc, "w");
	if (!pf)
		fprintf(stderr, "Can't fdopen %s: %m\n", path_dhclient_pid);
	else {
		fprintf (pf, "%ld\n", (long)getpid ());
		fclose (pf);
	}
}


typedef struct {
	
	int Wan_Type; 
	int Quick_Wan_Type;		
	// 1483 Routed
	char Routed_Description[50];
	unsigned short Routed_VPI;
	unsigned short Routed_VCI;
	int Routed_ATM_Class;
	int Routed_NAT;
	int Routed_Encapsulation_Method;
	int Routed_IP_Assignment;
	char Routed_IP[20];
	char Routed_Netmask[20];
	char Routed_Gateway[20];
	int Routed_RIP;
	unsigned short Routed_MTU;	

	//1483 Bridge
     char Bridged_Description[50];
	unsigned short Bridged_VPI;
     unsigned short Bridged_VCI;
	int Bridged_ATM_Class;
	// Camiu added 2006/06/20 begin ---
	unsigned short Bridged_PCR;
     unsigned short Bridged_SCR;
	unsigned short Bridged_MBS;
	// Camiu added 2006/06/20 end ---
	int Bridged_Encapsulation_Method;
	int Bridged_Ether_Filter_Type;
	int Bridged_Spanning_Bridge_Interface;

	//PPPOA
     char PPPOA_Description[50];
	unsigned short PPPOA_VPI;
     unsigned short PPPOA_VCI;
     int PPPOA_ATM_Class;
	int PPPOA_Encapsulation_Method;
	int PPPOA_NAT;
	char PPPOA_Username[256];
     char PPPOA_Password[256];
	char PPPOA_IP[20];
	int PPPOA_Authentication_Protocol;
	int PPPOA_Connection;
	unsigned short PPPOA_Idle_Timeout;
     int PPPOA_RIP;
	unsigned short PPPOA_MTU;
		
	//IPOA
	char IPOA_Description[50];
	unsigned short IPOA_VPI;
	unsigned short IPOA_VCI;
	int IPOA_ATM_Class;
	int IPOA_NAT;
	int IPOA_IP_Assignment;
	char IPOA_IP[20];
	char IPOA_Netmask[20];
	char IPOA_Gateway[20];
	int IPOA_RIP;
	unsigned short IPOA_MTU;
		
	//PPPOE
        char PPPOE_Description[50];
	unsigned short PPPOE_VPI;
        unsigned short PPPOE_VCI;
        int PPPOE_ATM_Class;
	int PPPOE_Encapsulation_Method;
	int PPPOE_NAT;
	char PPPOE_Username[257];
        char PPPOE_Password[257];
	char PPPOE_Service_Name[32];
	char PPPOE_IP[20];
	int PPPOE_Authentication_Protocol;
	int PPPOE_Connection;
	unsigned short PPPOE_Idle_Timeout;
        int PPPOE_RIP;
	unsigned short PPPOE_MTU;
	int PPPOE_RELAY;
} WAN_CONFIG_DATA;
WAN_CONFIG_DATA wan_config;
static void resetCurrentWanCfg(void){
	//reboot_flag=0;
	//WAN_CONFIG_DATA *d = &old_wan_config;
	WAN_CONFIG_DATA *d1 = &wan_config;
	//memset(d,0,sizeof(WAN_CONFIG_DATA));
	memset(d1,0,sizeof(WAN_CONFIG_DATA));
	/*
		d->Wan_Type=0;
		d->Quick_Wan_Type=0;
				
		//1483 Routed
		strcpy(d->Routed_Description,"");
		d->Routed_VPI=0;
		d->Routed_VCI=0;
		d->Routed_ATM_Class=0;
		d->Routed_NAT=0;
        	d->Routed_Encapsulation_Method=0;
	        d->Routed_IP_Assignment=0;
		strcpy(d->Routed_IP,"");
		strcpy(d->Routed_Netmask,"");
		strcpy(d->Routed_Gateway,"");
		d->Routed_RIP=0;
		d->Routed_MTU=0;

		//1483 Bridged
	        strcpy(d->Bridged_Description,"");
        	d->Bridged_VPI=0;
	        d->Bridged_VCI=0;
	        d->Bridged_ATM_Class=0;
	        d->Bridged_Encapsulation_Method=0;
	        d->Bridged_Ether_Filter_Type=0;
	        d->Bridged_Spanning_Bridge_Interface=0;
	
		//PPPOA
	        strcpy(d->PPPOA_Description,"");
        	d->PPPOA_VPI=0;
        	d->PPPOA_VCI=0;
        	d->PPPOA_ATM_Class=0;
        	d->PPPOA_NAT=0;
        	strcpy(d->PPPOA_Username,"");
	        strcpy(d->PPPOA_Password,"");
	        strcpy(d->PPPOA_IP,"");
	        d->PPPOA_Authentication_Protocol=0;
	        d->PPPOA_Connection=0;
	        d->PPPOA_Idle_Timeout=0;
		d->PPPOA_RIP=0;
	        d->PPPOA_MTU=0;
	
		//IPOA
	        strcpy(d->IPOA_Description,"");
        	d->IPOA_VPI=0;
        	d->IPOA_VCI=0;
        	d->IPOA_ATM_Class=0;
        	d->IPOA_NAT=0;
        	d->IPOA_IP_Assignment=0;
        	strcpy(d->IPOA_IP,"");
        	strcpy(d->IPOA_Netmask,"");
        	strcpy(d->IPOA_Gateway,"");
        	d->IPOA_RIP=0;
        	d->IPOA_MTU=0;
	
		//PPPOE
	        strcpy(d->PPPOE_Description,"PPPOE");
        	d->PPPOE_VPI=0;
        	d->PPPOE_VCI=0;
        	d->PPPOE_ATM_Class=0;
        	d->PPPOE_NAT=0;
        	strcpy(d->PPPOE_Username,"");
	        strcpy(d->PPPOE_Password,"");
	        strcpy(d->PPPOE_Service_Name,"");
		strcpy(d->PPPOE_IP,"0.0.0.0");
	        d->PPPOE_Authentication_Protocol=0;
	        d->PPPOE_Connection=0;
	        d->PPPOE_Idle_Timeout=0;
		d->PPPOE_RIP=0;
	        d->PPPOE_MTU=0;
	*/	
}
void Load_Wan_Config (void){
	FILE *fp;
	char line[1024];
	resetCurrentWanCfg();
	
	if ((fp=fopen("/flash/wan.conf","r"))!=NULL)
	{
	if (fgets(line, 1024, fp))
		sscanf(line,"#:%d:%d\n",(int *)&wan_config.Wan_Type,(int *)&wan_config.Quick_Wan_Type);
		
	//1483 Routed
	if (fgets(line, 1024, fp))
	{
		sscanf(line,
		"|%50[^|]|%hu|%hu|%d|%d|%d|%d|%20[^|]|%20[^|]|%20[^|]|%d|%hu|\n",
		wan_config.Routed_Description,
		&wan_config.Routed_VPI,
		&wan_config.Routed_VCI,
		(int *)&wan_config.Routed_ATM_Class,
		(int *)&wan_config.Routed_NAT,
        	(int *)&wan_config.Routed_Encapsulation_Method,
	        (int *)&wan_config.Routed_IP_Assignment,
		wan_config.Routed_IP,
		wan_config.Routed_Netmask,
		wan_config.Routed_Gateway,
		(int *)&wan_config.Routed_RIP,
		&wan_config.Routed_MTU
		);
	}
	
	print_blank_string(&wan_config.Routed_Description);
	print_blank_string(&wan_config.Routed_IP);
	print_blank_string(&wan_config.Routed_Netmask);
	print_blank_string(&wan_config.Routed_Gateway);
	
	//1483 Bridged
	if (fgets(line, 1024, fp))
	{
		sscanf(line,
		"|%50[^|]|%hu|%hu|%d|%d|%d|%d|\n",
		wan_config.Bridged_Description,
		&wan_config.Bridged_VPI,
		&wan_config.Bridged_VCI,
		(int *)&wan_config.Bridged_ATM_Class,
		(int *)&wan_config.Bridged_Encapsulation_Method,
        	(int *)&wan_config.Bridged_Ether_Filter_Type,
	        (int *)&wan_config.Bridged_Spanning_Bridge_Interface
		);
	}
	
	print_blank_string(&wan_config.Bridged_Description);	
	//PPPOA
	if (fgets(line, 1024, fp))
	{
		sscanf(line,
		"|%50[^|]|%hu|%hu|%d|%d|%d|%256[^|]|%256[^|]|%20[^|]|%d|%d|%hu|%d|%hu|\n",
		wan_config.PPPOA_Description,
		&wan_config.PPPOA_VPI,
		&wan_config.PPPOA_VCI,
		(int *)&wan_config.PPPOA_ATM_Class,
        	(int *)&wan_config.PPPOA_Encapsulation_Method,
		(int *)&wan_config.PPPOA_NAT,
		wan_config.PPPOA_Username,
		wan_config.PPPOA_Password,
		wan_config.PPPOA_IP,
	        (int *)&wan_config.PPPOA_Authentication_Protocol,
	        (int *)&wan_config.PPPOA_Connection,
	        &wan_config.PPPOA_Idle_Timeout,
		(int *)&wan_config.PPPOA_RIP,
		&wan_config.PPPOA_MTU
		);
	}
	
	print_blank_string(&wan_config.PPPOA_Description);
	print_blank_string(&wan_config.PPPOA_Username);
	print_blank_string(&wan_config.PPPOA_Password);
	print_blank_string(&wan_config.PPPOA_IP);	
	//IPOA
	if (fgets(line, 1024, fp))
	{
		sscanf(line,"|%50[^|]|%hu|%hu|%d|%d|%d|%20[^|]|%20[^|]|%20[^|]|%d|%hu|\n",wan_config.IPOA_Description,
		&wan_config.IPOA_VPI,
		&wan_config.IPOA_VCI,
		(int *)&wan_config.IPOA_ATM_Class,
		(int *)&wan_config.IPOA_NAT,
	        (int *)&wan_config.IPOA_IP_Assignment,
		wan_config.IPOA_IP,
		wan_config.IPOA_Netmask,
		wan_config.IPOA_Gateway,
		(int *)&wan_config.IPOA_RIP,
		&wan_config.IPOA_MTU
		);
	}	
	
	print_blank_string(&wan_config.IPOA_Description);
	print_blank_string(&wan_config.IPOA_IP);
	print_blank_string(&wan_config.IPOA_Netmask);
	print_blank_string(&wan_config.IPOA_Gateway);	
	//PPPOE
	if (fgets(line, 1024, fp))
	{
		sscanf(line,
		"|%50[^|]|%hu|%hu|%d|%d|%d|%256[^|]|%256[^|]|%32[^|]|%20[^|]|%d|%d|%hu|%d|%hu|%d|\n",
		wan_config.PPPOE_Description,
		&wan_config.PPPOE_VPI,
		&wan_config.PPPOE_VCI,
		(int *)&wan_config.PPPOE_ATM_Class,
        	(int *)&wan_config.PPPOE_Encapsulation_Method,
		(int *)&wan_config.PPPOE_NAT,
		wan_config.PPPOE_Username,
		wan_config.PPPOE_Password,
		wan_config.PPPOE_Service_Name,
		wan_config.PPPOE_IP,
	        (int *)&wan_config.PPPOE_Authentication_Protocol,
	        (int *)&wan_config.PPPOE_Connection,
	        &wan_config.PPPOE_Idle_Timeout,
		(int *)&wan_config.PPPOE_RIP,
	       &wan_config.PPPOE_MTU,
		&wan_config.PPPOE_RELAY
		);
	}
	print_blank_string(&wan_config.PPPOE_Description);
	print_blank_string(&wan_config.PPPOE_Username);
	print_blank_string(&wan_config.PPPOE_Password);
	print_blank_string(&wan_config.PPPOE_Service_Name);
	print_blank_string(&wan_config.PPPOE_IP);
	fclose(fp);
	}
//	if (wan_config.Quick_Wan_Type==0)
//		wan_config.Quick_Wan_Type=wan_config.Wan_Type;	
}
typedef struct {
        int Enable;
        int hostIsDnsServer;
	char DomainName[20];
        char PrimaryDNS[20];
        char SecondaryDNS[20];
        int DefaultLeaseTime;
        int MaxLeaseTime;
	char Subnet[20];
        char Netmask[20];
        char Router[20];
        char Broadcast_address[20];
        char Range_start[20];
        char Range_end[20];
	
	char name[10][20];
	char macaddr[10][20];
	char ipaddr[10][20];
        char Relay[20];
} DHCPD_CONFIG_DATA;
DHCPD_CONFIG_DATA dhcpd_config;
static void resetCurrentDhcpdCfg(void)
{
	//int i=0;
	DHCPD_CONFIG_DATA *d = &dhcpd_config;
	memset(d,0,sizeof(DHCPD_CONFIG_DATA));
/*	d->Enable=0;
	strcpy(d->DomainName,"");
	strcpy(d->PrimaryDNS,"");
	strcpy(d->SecondaryDNS,"");
	d->DefaultLeaseTime=0;
	d->MaxLeaseTime=0;
	strcpy(d->Subnet,"");
	strcpy(d->Netmask,"");
	strcpy(d->Router,"");
	strcpy(d->Broadcast_address,"");
	strcpy(d->Range_start,"");
	strcpy(d->Range_end,"");
	strcpy(d->Relay,"");
	for(i=0;i<10;i++){
		strcpy(d->name[i],"");
		strcpy(d->macaddr[i],"");
		strcpy(d->ipaddr[i],"");
	} */
	d->hostIsDnsServer=1;
}
void Load_Dhcpd_Config (void)
{
        FILE *fp;
	int i=0;
	char line[255];
        resetCurrentDhcpdCfg();
	//Load_Lan_Config();	
	if((fp = fopen("/flash/dhcpd.conf", "r"))!=NULL){
	if (fgets(line, 255, fp))
	{
	        if (sscanf(line, "#:%d:\n",
	        &dhcpd_config.Enable)  != 1)
	        {
			//error format
		}
	}
	if (fgets(line, 255, fp))
	{
	        if (sscanf(line, "#:%d:\n",
	        &dhcpd_config.hostIsDnsServer)  != 1)
	        {
			//error format
		}
	}

    	if (fgets(line, 255, fp))
    	{
    		if (sscanf(line, "option domain-name \"%[^\";]\";\n", 
    		dhcpd_config.DomainName)  != 1)
    		{
     			//error format
    		}
    	}
    	if (fgets(line, 255, fp))
    	{
    		if (sscanf(line, "option domain-name-servers %[^,], %[^;];s",
               	dhcpd_config.PrimaryDNS,
               	dhcpd_config.SecondaryDNS) != 2)
    		{
     			//error format 
    		}
    	}
	/*
	if (dhcpd_config.hostIsDnsServer==1){
		strcpy(dhcpd_config.PrimaryDNS,lan_config.HostAddress);
		strcpy(dhcpd_config.SecondaryDNS,"");
	}
	else{
	if (!isValidIP(dhcpd_config.SecondaryDNS))
		sscanf(dhcpd_config.PrimaryDNS,"%[^;];s",dhcpd_config.PrimaryDNS);
	}
    */
    	if (fgets(line, 255, fp))
    	{
    		if (sscanf(line, "default-lease-time %d\n",
               	&dhcpd_config.DefaultLeaseTime) != 1)
    		{
     			//error format 
    		}
    	}
    	

    	if (fgets(line, 255, fp))
    	{
     		if (sscanf(line, "max-lease-time %d\n",
               	&dhcpd_config.MaxLeaseTime) != 1)
    		{
     			//error format 
    		}
    	} 
    	
    	if (fgets(line, 255, fp))
    	{
    		if (sscanf(line, "subnet %s netmask %s {\n",
               	dhcpd_config.Subnet, dhcpd_config.Netmask) != 2)
    		{
     			//error format 
    		}
    	}
    	
    	if (fgets(line, 255, fp))
    	{
    		if (sscanf(line, "\toption routers %[^;]s\n",
               	dhcpd_config.Router) != 1)
    		{
     			//error format 
    		}
    	}
  	
  	    	if (fgets(line, 255, fp))
    	{
    		if (sscanf(line, "\toption broadcast-address %[^;]s\n",
               	dhcpd_config.Broadcast_address) != 1)
    		{
     			//error format 
    		}
    	}
    	    	if (fgets(line, 255, fp))
    	{
    		if (sscanf(line, "\trange %s %[^;]s\n",
               	dhcpd_config.Range_start, dhcpd_config.Range_end) != 2)
    		{
     			//error format 
    		}
    	}	

    	    	fgets(line, 255, fp);
    	    	fgets(line, 255, fp);
	
	for (i=0;i<10;i++){	
		if (fgets(line, 255, fp) !="\t}\n}\n")
    		{
			sscanf(line, "\t\thost %s {\n",dhcpd_config.name[i]);
		}
		if (fgets(line, 255, fp) !="\t}\n}\n")
    		{
			sscanf(line, "\t\t\thardware ethernet %[^;]s\n",dhcpd_config.macaddr[i]);
		}
		if (fgets(line, 255, fp) !="\t}\n}\n")
    		{
			sscanf(line, "\t\t\tfixed-address %[^;]s\n",dhcpd_config.ipaddr[i]);
		}
		if (fgets(line, 255, fp) !="\t}\n}\n")
    		{
			sscanf(line, "\t\t}\n");
		}
	}
	fclose(fp);
	}
	
	if((fp = fopen("/flash/dhcprelay.conf", "r"))!=NULL){
  	if (fgets(line, 255, fp))
    	{
    		if (sscanf(line, "%s\n",dhcpd_config.Relay)!=1)	{
     			//error format 
    		}
    	}
	fclose(fp);
	}
}
int Load_Router_Status(){
/*	
char LAN_IP[20];
char WAN_IP[20];
char WAN_IF[10];
char LAN_IF[10];
char LAN_DOMAIN[50];
char LAN_NETMASK[20];
char DHCP_START[20];
char DHCP_END[20];
char MAC_ADDRESS[20];
char LOG_LEVEL[10];
char PPPOE_USERNAME[256];
char PPPOE_PASSWORD[256];
char PPPOE_IDLETIME[10];
char WAN[10];
    char buf[256];
    char buf1[17];
    char mac1[2];
    char mac2[2];
    char mac3[2];
    char mac4[2];
    char mac5[2];
    char mac6[2];
    */
    char buf[256];
    char is_wireless[10],TmpStr1[128];
    char *pdest1,*pdest2;
    /*
	memset(buf,0,sizeof(buf));
	memset(DEV_MFR,0,sizeof(DEV_MFR));
	memset(DEV_MFR_URL,0,sizeof(DEV_MFR_URL));
	memset(DEV_MODEL_URL,0,sizeof(DEV_MODEL_URL));
	memset(DEV_MODEL_NO,0,sizeof(DEV_MODEL_NO));
	memset(FRIENDLYNAME,0,sizeof(FRIENDLYNAME));
	memset(DEV_MODEL_DESCRIPTION,0,sizeof(DEV_MODEL_DESCRIPTION));
	memset(DEV_MODEL_NAME,0,sizeof(DEV_MODEL_NAME));
	memset(LAN_IF,0,sizeof(LAN_IF));
	memset(WAN_IF,0,sizeof(WAN_IF));
	memset(LAN_IP,0,sizeof(LAN_IP));
	memset(WAN_IP,0,sizeof(WAN_IP));
	memset(LAN_DOMAIN,0,sizeof(LAN_DOMAIN));
	memset(WAN,0,sizeof(WAN));
	memset(MAC_ADDRESS,0,sizeof(MAC_ADDRESS));
	memset(PPPOE_USERNAME,0,sizeof(PPPOE_USERNAME));
	memset(PPPOE_PASSWORD,0,sizeof(PPPOE_PASSWORD));
	memset(PPPOE_IDLETIME,0,sizeof(PPPOE_IDLETIME));
	memset(LOG_LEVEL,0,sizeof(LOG_LEVEL));
	memset(DHCP_START,0,sizeof(DHCP_START));
	memset(DHCP_END,0,sizeof(DHCP_END));
	*/
	strcpy(DEV_MFR,(char *)read_file_para_value("/etc/default/config","HOME_URL_NAME=",buf));
	strcpy(DEV_MFR_URL,(char *)read_file_para_value("/etc/default/config","URL=",buf));
	strcpy(DEV_MODEL_URL,(char *)read_file_para_value("/etc/default/config","URL=",buf));
	strcpy(DEV_MODEL_NO,(char *)read_file_para_value("/etc/default/config","SOFTWARE_VER=",buf));
	strcpy(FRIENDLYNAME,(char *)read_file_para_value("/etc/default/config","GATEWAY_NAME=",buf));
	strcpy(DEV_MODEL_DESCRIPTION,(char *)read_file_para_value("/etc/default/config","HOME_URL_NAME=",buf));
//	strcpy(DEV_MODEL_DESCRIPTION,(char *)read_file_para_value("/default/config","HOME_URL_NAME=",buf));
	strcat(DEV_MODEL_DESCRIPTION,"UPNP IGD");
	strcpy(is_wireless,(char *)read_file_para_value("/etc/default/config","WIRELESS=",buf));
	if (!strcmp(is_wireless,"no")){
		strcpy(DEV_MODEL_NAME,(char *)read_file_para_value("/etc/default/config","MODEL_NAME=",buf));
	}else{
		strcpy(DEV_MODEL_NAME,(char *)read_file_para_value("/etc/default/config","MODEL_NAME_G=",buf));
	}
	strcpy(LAN_IF,(char *)read_file_para_value("/var/etc/status","LAN_IF=",buf));
	strcpy(LAN_IP,(char *)read_file_para_value("/var/etc/status","LAN_IP=",buf));
    
    //JJJ 20-07-06
    #ifdef _LEVEL_ONE_
    //sprintf(FRIENDLYNAME,"%s(%s)",DEV_MODEL_NAME,LAN_IP);
//     strcpy(FRIENDLYNAME,"");
//     strcat(FRIENDLYNAME,(char *)read_file_para_value("/etc/default/config","MODEL_NAME=",buf));
//     strcat(FRIENDLYNAME,"(");
//     strcat(FRIENDLYNAME,LAN_IP);
//     strcat(FRIENDLYNAME,")");

    strcpy(FRIENDLYNAME,"");
    read_file_para_value("/etc/default/config","MODEL_NAME=",buf);
    pdest1=buf;
    memset(TmpStr1,0,sizeof(TmpStr1));
    if (pdest2=strstr(buf," "))
        memcpy(TmpStr1,pdest1,pdest2-pdest1);  
    strcpy(FRIENDLYNAME,TmpStr1);
 /* strcat(FRIENDLYNAME,"(");
    strcat(FRIENDLYNAME,LAN_IP);
    strcat(FRIENDLYNAME,")"); */
    
    strcpy(DEV_MFR,"LevelOne");
    strcpy(DEV_MODEL_NO,TmpStr1);
    strcpy(DEV_MODEL_DESCRIPTION,"Wireless ADSL2+ VoIP Router");
    if (!strcmp(is_wireless,"no")){
        strcpy(DEV_MODEL_NAME,"Wireless ADSL2+ VoIP Router");
    }else{
        strcpy(DEV_MODEL_NAME,"Wireless ADSL2+ VoIP Router");
    }    
    #else
    strcpy(DEV_MFR,(char *)read_file_para_value("/etc/default/config","HOME_URL_NAME=",buf));
    strcpy(FRIENDLYNAME,(char *)read_file_para_value("/etc/default/config","GATEWAY_NAME=",buf));
    strcpy(DEV_MODEL_NO,(char *)read_file_para_value("/etc/default/config","SOFTWARE_VER=",buf));
    strcpy(DEV_MODEL_DESCRIPTION,(char *)read_file_para_value("/etc/default/config","HOME_URL_NAME=",buf));
//  strcpy(DEV_MODEL_DESCRIPTION,(char *)read_file_para_value("/default/config","HOME_URL_NAME=",buf));
    strcat(DEV_MODEL_DESCRIPTION,"UPNP IGD");    
    if (!strcmp(is_wireless,"no")){
        strcpy(DEV_MODEL_NAME,(char *)read_file_para_value("/etc/default/config","MODEL_NAME=",buf));
    }else{
        strcpy(DEV_MODEL_NAME,(char *)read_file_para_value("/etc/default/config","MODEL_NAME_G=",buf));
    }    
    #endif 
       
 
       
    	
	strcpy(WAN_IF,(char *)read_file_para_value("/var/etc/status","WAN_IF=",buf));
	//DPRINTF("LAN_IP:%s\n",LAN_IP);
	//sprintf(WAN_IP,"%s",(char *)read_file_para_value("/var/etc/status","WAN_IP=",buf));
	
	strcpy(WAN_IP,(char *)read_file_para_value("/var/etc/status","WAN_IP=",buf));
	//DPRINTF("WAN_IP:%s\n",WAN_IP);
	if(!isValidIP(WAN_IP)){
		strcpy(WAN_IP,"unknown");
	}
	//DPRINTF("WAN_IP:%s\n",WAN_IP);
	strcpy(LAN_DOMAIN,(char *)read_file_para_value("/var/etc/status","GATEWAY_NAME=",buf));
	strcpy(WAN,(char *)read_file_para_value("/var/etc/status","WAN=",buf));
	strncpy(MAC_ADDRESS,(char *)read_file_para_value("/var/etc/status","ETH0_MAC=",buf),17);
	//sscanf(buf1,"%s:%s:%s:%s:%s:%s",mac1,mac2,mac3,mac4,mac5,mac6);
	//snprintf(MAC_ADDRESS,"%s%s%s%s%s%s",mac1,mac2,mac3,mac4,mac5,mac6,12);
	
	Load_Wan_Config();
	strcpy(PPPOE_USERNAME,wan_config.PPPOE_Username);
	strcpy(PPPOE_PASSWORD,wan_config.PPPOE_Password);
	sprintf(PPPOE_IDLETIME,"%d",wan_config.PPPOE_Idle_Timeout);
	sprintf(LOG_LEVEL,"%d",3);
	Load_Dhcpd_Config();
	strcpy(DHCP_START,dhcpd_config.Range_start);
	strcpy(DHCP_END,dhcpd_config.Range_end);
	//DPRINTF("######(UPnP)Load Router Config######\n");
	/*
	DPRINTF("DEV_MFR:%s\n",DEV_MFR);
	DPRINTF("DEV_MFR_URL:%s\n",DEV_MFR_URL);
	DPRINTF("DEV_MODEL_URL:%s\n",DEV_MODEL_URL);
	DPRINTF("DEV_MODEL_NO:%s\n",DEV_MODEL_NO);
	DPRINTF("FRIENDLYNAME:%s\n",FRIENDLYNAME);
	DPRINTF("DEV_MODEL_DESCRIPTION:%s\n",DEV_MODEL_DESCRIPTION);
	DPRINTF("DEV_MODEL_NAME:%s\n",DEV_MODEL_NAME);
	DPRINTF("LAN_IF:%s\n",LAN_IF);
	DPRINTF("WAN_IF:%s\n",WAN_IF);
	DPRINTF("LAN_DOMAIN:%s\n",LAN_DOMAIN);
	DPRINTF("WAN:%s\n",WAN);
	DPRINTF("MAC_ADDRESS:%s\n",MAC_ADDRESS);
	DPRINTF("PPPOE_USERNAME:%s\n",PPPOE_USERNAME);
	DPRINTF("PPPOE_PASSWORD:%s\n",PPPOE_PASSWORD);
	DPRINTF("PPPOE_IDLETIME:%s\n",PPPOE_IDLETIME);
	DPRINTF("LOG_LEVEL:%s\n",LOG_LEVEL);
	DPRINTF("DHCP_START:%s\n",DHCP_START);
	DPRINTF("DHCP_END:%s\n",DHCP_END);
	DPRINTF("###################################\n");
	*/
}

int main(int argc, char *argv[])
{
    extern char g_wandevs[];
   // extern char g_landevs[];
    extern DeviceTemplate IGDeviceTemplate;
//    char **argp = &argv[1];
    //char *wanif = NULL;
    //char *lanif = NULL;
    char str_temp[2];
    char buf[32];
    char line[255];
    char str_http_port[10];
    char lanif[10];
    char wanif[10];
    FILE *fp;
    int daemonize = 0;
	HTTP_PORT=2800;
	Load_Router_Status();
//    while (argp < &argv[argc]) {
//	if (strcasecmp(*argp, "-L") == 0) {
//		if (read_file_para_value("/var/etc/status","LAN_IF=",buf))
//    			strcpy(lanif,(char *)read_file_para_value("/var/etc/status","LAN_IF=",buf));
		fprintf(stderr, "UPnP lan_ifname:%s\t", LAN_IF);
		//strcpy(g_landevs, LAN_IF);
		//nvram_set("lan_ifname",LAN_IF);
		//lanif = *++argp;
//	} 
//	else if (strcasecmp(*argp, "-W") == 0) {
//		if (read_file_para_value("/var/etc/status","WAN_IF=",buf))
//			strcpy(wanif,(char *)read_file_para_value("/var/etc/status","WAN_IF=",buf));
		
		fprintf(stderr, "wan_ifname:%s\t", WAN_IF);
		//wanif = *++argp;
		strcpy(g_wandevs, WAN_IF);
		nvram_set("wan_ifname",WAN_IF);
//	}
//	else if (strcasecmp(*argp, "-P") == 0) {
		if ((fp=fopen("/flash/upnpcfg.conf","r"))!=NULL) {
        		if(fgets(line,255,fp))
   				sscanf(line,"%s %s\n",str_temp,str_http_port);
	       		fclose(fp);
		}
		else
			strcpy(str_http_port,"2800");
		
		HTTP_PORT=atoi(str_http_port);
		fprintf(stderr, "port:%d\n", HTTP_PORT);
		//HTTP_PORT = atoi(*++argp);
//	}  
//	else if (strcasecmp(*argp, "-D") == 0) {
//	    daemonize = 1;
//	}
//	argp++;
//  }

    init_event_queue(40);

    if (lanif == NULL || wanif == NULL) {
	fprintf(stderr, "usage: %s -L lan_ifname -W wan_ifname -P tcp_port\n", argv[0]);
    } else {
#if !defined(__UCLIBC__) || defined(__UCLIBC_HAS_MMU__)
	if (daemonize && daemon(1, 1) == -1) {
	    perror("daemon");
	    exit(errno);
	}
#else
	if (daemonize)
	{    
		go_daemon();
//	perror("daemon not supported");
//
		}
#endif

	/* We need to have a reaper for child processes we may create.
	   That happens when we send signals to the dhcp process to
	   release an renew a lease on the external interface. */
	signal(SIGCHLD, reap);

	/* For some reason that I do not understand, this process gets
	   a SIGTERM after sending SIGUSR1 to the dhcp process (to
	   renew a lease).  Ignore SIGTERM to avoid being killed when
	   this happens.  */
	//	signal(SIGTERM, SIG_IGN);
	signal(SIGUSR1, SIG_IGN);

	fprintf(stderr, "calling upnp_main\n");
	upnp_main(&IGDeviceTemplate, LAN_IF);
    }
    
    return 0;
}
