/****************************************************************************
 *
 *  Copyright (c) 2001 by LSI Logic Corporation, all rights reserved.
 *
 *                         LSI Logic confidential
 *  The information contained herein is proprietary of LSI Logic Corporation.
 *
 *  This software is provided "as is" without any express or implied warranty.
 *  LSI Logic disclaims all representations and warranties with regard to
 *  this software, including all implied warranties of merchantability and
 *  fitness for a particular purpose. In no event shall LSI Logic be liable
 *  for any special, indirect or consequential damages or any damages
 *  whatsoever resulting from loss of use, data or profits, whether in an
 *  action of contract, negligence or other tortious act or omission,
 *  arising out of or in connection with the use or performance of this
 *  software.
 *
 *  $Id: vpctrl.c,v 1.2 2005/09/29 11:53:49 robert Exp $
 *
 *  Revision Details
 *  $Log: vpctrl.c,v $
 *  Revision 1.2  2005/09/29 11:53:49  robert
 *  release_1_03_rc4_08252005
 *
 *  Revision 1.1.1.1  2005/09/15 13:37:09  root
 *  BINOS
 *
 *
 *
 *  File Description:
 *  This program is a utility used by the firewall script.
 *  With no arguments, the program prints out all the interface names in the system.
 *  With a single argument of an interface name (eg. eth0), the program will print out
 *  the IP address of the interface.
 *
 ****************************************************************************/

/****************************************************************************
 * standard include files
 ****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <linux/sockios.h>
#include <linux/tc3162ioctl.h>
#include <linux/if.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#ifdef LIBVER
#define MAINFUNC vpctrl_main
#else
#define MAINFUNC main
#endif

#define MAX_IFINDEX 255

/****************************************************************************
 * private function prototypes
 ****************************************************************************/
static void get_interfaces(int fd);
static void get_address(int fd, char *ifname);

/****************************************************************************
 * public functions defined here
 ****************************************************************************/


/****************************************************************************
 * Function Name  : main
 * Arguments      : argc
 *                  argv
 * Returns        : -1 fail, 0 success
 * Variables      : 
 * Modifies       : 
 * Description    : 
 ****************************************************************************/
int MAINFUNC(int argc, char *argv[])
{
	int fd;
        char* cmd=NULL;
	struct ifreq ifr;
	struct IP175C_vlan_ioctl_args request_args;
	unsigned int readback=0;
        if ((fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
           fprintf(stderr, "FATAL:  Couldn't open a socket..go figure!\n");
           exit(2);
          }   
	memset(&ifr, 0, sizeof(struct ifreq));
	memset(&request_args, 0, sizeof(struct IP175C_vlan_ioctl_args));
   
   if ((argc < 3) ) {
      fprintf(stdout,"Expecting argc to be 3-5, inclusive.  Was: %d\n",argc);
      exit(1);
   }
   else {
      cmd = argv[1];
     }
 
      strcpy(ifr.ifr_name, "eth0"); 

    if (strcasecmp(cmd, "rst") == 0) {
     
      
      request_args.request_cmd=Switch_Reset;
      ifr.ifr_data=(char*)&request_args;

      if (ioctl(fd, SIOCSVLANPORT, &ifr) < 0)  	
         fprintf(stderr,"ERROR: trying to add VLAN \n");                 
     }
   if (strncasecmp(cmd, "vlan",4) == 0) {
     
      request_args.portmask=atoi(argv[2]);
      printf("portmask==%d\n",request_args.portmask);
      request_args.request_cmd=Set_VLAN_Port;
      ifr.ifr_data=(char*)&request_args;

      if (ioctl(fd, SIOCSVLANPORT, &ifr) < 0)  	
         fprintf(stderr,"ERROR: trying to add VLAN \n");                 
     }
   
   if (strcasecmp(cmd, "miir") == 0) {
     
      request_args.phyaddr=atoi(argv[2]);
      request_args.regid=atoi(argv[3]);
      
      request_args.request_cmd=Get_Lan_Mii;
      ifr.ifr_data=(char*)&request_args;
      // KenWang modified 2006/01/02
      //if(request_args.regvalue==0 || ioctl(fd, SIOCGVLANPORT, &ifr)<0)
      //if(ioctl(fd, SIOCSVLANPORT, &ifr)<0 || request_args.regvalue==0)
      if(ioctl(fd, SIOCSVLANPORT, &ifr) < 0)
         printf("try to read lan mii reg fail\n");
      else
      printf("reg %d.%d ==%x\n",request_args.phyaddr,request_args.regid,request_args.regvalue);                
     }

   if (strcasecmp(cmd, "miiw") == 0) {
     
      request_args.phyaddr=atoi(argv[2]);
      request_args.regid=atoi(argv[3]);
      request_args.regvalue=strtol(argv[4],NULL,16);
      printf("value==%lx\n",request_args.regvalue);
      request_args.request_cmd=Set_Lan_Mii;
      ifr.ifr_data=(char*)&request_args;
      if(ioctl(fd, SIOCSVLANPORT, &ifr)<0)
         printf("try to write lan mii reg fail\n");
                      
     }

   if (strcasecmp(cmd, "active") == 0)
      {
        if ( strcasecmp(argv[2], "igmp") ==0)
        { 
           request_args.request_cmd=Enable_IGMP_Snoop;
           ifr.ifr_data=(char*)&request_args;
           if (ioctl(fd, SIOCSVLANPORT, &ifr) < 0) 	
               fprintf(stderr,"ERROR: trying to enable VLAN Tag \n");                 
        }
        if ( strcasecmp(argv[2], "no") ==0)
        { 
           request_args.request_cmd=Disable_VLAN;
           ifr.ifr_data=(char*)&request_args;
           if (ioctl(fd, SIOCSVLANPORT, &ifr) < 0) 	
               fprintf(stderr,"ERROR: trying to disable VLAN Tag \n");                 
        }
      }
   if (strncasecmp(cmd, "vodx", 4) == 0) {
      /* KeenWang added 2005/12/28 for add user specific VID -- begin -- */ 
      /* useage: vpctrl vodx <wireless> <vlanmask> [tagmask] [vid0] [vid1] [vid2] [vid3]
	         [wireless] : Required. Not used.
		 [vlanmask] : Required. xxxxxxxx, note the difference of port ordering between 7202 and 7202G.
		 [tagmask]  : Optional. x, note the difference of port ordering between 7202 and 7202G.
		 [vid0-vid3]: Optional. vid0 will be used for vlan group 1 and so on. Default is 2 3 4 5.
      */
      if(argc > 4) {
	      request_args.tagmask = atoi(argv[4]);
	      request_args.VID[0] = atoi(argv[5]);
	      request_args.VID[1] = atoi(argv[6]);
	      request_args.VID[2] = atoi(argv[7]);
	      request_args.VID[3] = atoi(argv[8]);
	      request_args.VID[4] = atoi(argv[9]);
	      request_args.VID[5] = atoi(argv[10]);
	      request_args.VID[6] = atoi(argv[11]);
	      request_args.VID[7] = atoi(argv[12]);
      }
      else {
	      request_args.tagmask = 0;
	      request_args.VID[0] = 2;
	      request_args.VID[1] = 3;
	      request_args.VID[2] = 4;
	      request_args.VID[3] = 5;
      }
      /* KeenWang added 2005/12/28 for add user specific VID -- end -- */ 
      request_args.portmask=atoi(argv[3]);
      //request_args.wireless=atoi(argv[2]);
      printf("portmask==%d\n",request_args.portmask);
      //printf("wireless==%d\n",request_args.wireless);
      request_args.request_cmd=VODX;
      ifr.ifr_data=(char*)&request_args;

      if (ioctl(fd, SIOCSVLANPORT, &ifr) < 0)  	
         fprintf(stderr,"ERROR: trying to add VODX \n");
   }

	return 0;
}

/****************************************************************************
 * Function Name  : get_interfaces
 * Arguments      : fd
 * Returns        : void
 * Variables      : 
 * Modifies       : 
 * Description    : 
 ****************************************************************************/
static void get_interfaces(int fd)
{
	int i;
	struct ifreq ifr;

	for (i = 1; i < MAX_IFINDEX; i++)
	{
		memset((void *)&ifr, '\0', sizeof(struct ifreq));
		ifr.ifr_ifindex = i;
	
		if (ioctl(fd, SIOCGIFNAME, &ifr) < 0)
		{
			return;
		}
		printf("%s ", ifr.ifr_name);
	}
}

/****************************************************************************
 * Function Name  : get_address
 * Arguments      : fd
 *                  ifname
 * Returns        : void
 * Variables      : 
 * Modifies       : 
 * Description    : 
 ****************************************************************************/
static void get_address(int fd, char *ifname)
{
	struct ifreq ifr;
	struct in_addr in;

	memset((void *)&ifr, '\0', sizeof(struct ifreq));
	strncpy(ifr.ifr_name, ifname, IFNAMSIZ);
	if (ioctl(fd, SIOCGIFADDR, &ifr) < 0)
	{
		return;
	}
	in.s_addr = *((unsigned long *)&ifr.ifr_addr.sa_data[2]);
	printf("%s", inet_ntoa(in));
}
