/*
 *  adsl.asp: ADSL CONFIGURATION
 */
#include "webs.h"
#include "lan_conf.h"
#include "wan.h"
#include "adsl.h"
#include "../iptables-1.2.11/runFunc.h"
#include <sys/ioctl.h>  /* ioctl */
/* dmt status */
#include <linux/tc3162dmtctrl.h>
#include <sys/syslog.h>
#include "locale.h"
extern char *ADSL_NOT_UP[];

void resetCurrentAdslCfg(void);
static int get_adsl_status (int ejid, webs_t wp, int argc, char_t **argv);
static int is_dmt_connected (int ejid, webs_t wp, int argc, char_t **argv);
static int Load_Adsl_Config (void);
static int Save_Adsl_Config (ADSL_CONFIG_DATA *d);
static void Adsl_Form(webs_t wp,char_t * path,char_t * query);
static int get_adsl_mod (int eid, webs_t wp, int argc, char_t ** argv);
extern void dmt_write_cmd(int cmd);
extern char *dmt_read_status(char *buf);
//static char_t buf_512[512];
static char_t  buf2_255[255], buf3_255[255];
static int get_adsl_mod (int eid, webs_t wp, int argc, char_t ** argv)
{
	int i,j=0;
	Load_Adsl_Config();
	i=atoi(argv[1]);
	if (!strcmp(argv[0],"ADSLMODE"))
		j=adsl_config.adslrate;
	if (i==j)
   		return websWrite(wp, T("selected"));
	else
   		return websWrite(wp, T(""));
}

static void AdslDiagnostic_Form(webs_t wp,char_t * path,char_t * query)
{	
 	//char_t *buf=NULL;
   int nFd = open( "/dev/dmtdev", O_RDWR );
  //  DPRINTF("AdslDiagnostic_Form is called\n");
    if( nFd == -1 )
    {
	    syslog(LOG_INFO,"DMT: open error %d\n", errno );
	    websRedirect(wp, T("adsl_diag.asp"));
	    return;
    }
	
//w ad f
{
    DMTCTRL_DMT_VERSION Arg;

    ioctl( nFd, DMTIOCTL_GET_DMT_FW_VERSION, &Arg );
    syslog(LOG_INFO,"DMT FwVer: %d.%d.%d.%d_%c_%s, HwVer:T%dF%d_%d.%d\n",Arg.fw_ver_r,Arg.fw_ver_p,Arg.fw_rev,Arg.b_ver,(Arg.annex_afe & 0x02)?'A':'B',(Arg.annex_afe & 0x01)?"TC":"SI",Arg.tdm_ver,Arg.fdm_ver,Arg.major,Arg.minor);
}

//w ad uptime
{
    DMTCTRL_ADSL_UPTIME DmtUptime;
    unsigned long int Arg=4294967295UL;
    
    unsigned int hr=0;
    unsigned int mm=0;
    unsigned int sstemp=0;
    unsigned int dd=0;
    unsigned int ss=0;
 
    DmtUptime.showtime_stamp=0;
    DmtUptime.currenttime_stamp=0;
    //DPRINTF("Arg=%08x, bytes %d\n",Arg, sizeof(Arg));

    ioctl( nFd, DMTIOCTL_GET_LINE_UPTIME, &DmtUptime );
    if(DmtUptime.dmtstatus !=ADSL_UP)
    {
	    syslog(LOG_INFO,"ADSL doesn't sync up!\n");
	    close(nFd);
	    websRedirect(wp, T("syslog.asp"));
	    return;
    }
    if(DmtUptime.currenttime_stamp > DmtUptime.showtime_stamp)
    {
            sstemp= DmtUptime.currenttime_stamp - DmtUptime.showtime_stamp;
            hr=sstemp/3600;
            ss= sstemp % 3600;
            mm= ss/60;
            ss=ss % 60;
            dd=hr/24;
            hr=hr % 24;
            
    }
    if( DmtUptime.currenttime_stamp < DmtUptime.showtime_stamp)
    {
            sstemp=Arg-DmtUptime.showtime_stamp;
            sstemp=sstemp + DmtUptime.currenttime_stamp;
            hr=sstemp/3600;
            ss= sstemp % 3600;
            mm= ss/60;
            ss=ss % 60;
            dd=hr/24;
            hr=hr % 24;
    }
    syslog(LOG_INFO,"ADSL uptime: %d day %d hr %d min %d sec\n",dd,hr,mm,ss);
}       
 

//w ad perf
{
    DMTCTRL_LINE_INFO Arg;

    memset(&Arg,0,sizeof(DMTCTRL_LINE_INFO));
    ioctl( nFd, DMTIOCTL_GET_LINE_INFO, &Arg );
         
    syslog(LOG_INFO,"near-end FEC error fast: %d \n",Arg.near_end.FecNotInterleaved);
    syslog(LOG_INFO,"near-end FEC error interleaved: %d \n",Arg.near_end.FecInterleaved);
    syslog(LOG_INFO,"near-end CRC error fast: %d \n",Arg.near_end.CrcNotInterleaved);
    syslog(LOG_INFO,"near-end CRC error interleaved: %d \n",Arg.near_end.CrcInterleaved);
    syslog(LOG_INFO,"near-end HEC error fast: %d \n",Arg.near_end.HecNotInterleaved);
    syslog(LOG_INFO,"near-end HEC error interleaved: %d \n",Arg.near_end.HecInterleaved);
 
    syslog(LOG_INFO,"far-end FEC error fast: %d \n",Arg.far_end.FecNotInterleaved);
    syslog(LOG_INFO,"far-end FEC error interleaved: %d \n",Arg.far_end.FecInterleaved);
    syslog(LOG_INFO,"far-end CRC error fast: %d \n",Arg.far_end.CrcNotInterleaved);
    syslog(LOG_INFO,"far-end CRC error interleaved: %d \n",Arg.far_end.CrcInterleaved);
    syslog(LOG_INFO,"far-end HEC error fast: %d \n",Arg.far_end.HecNotInterleaved);
    syslog(LOG_INFO,"far-end HEC error interleaved: %d \n",Arg.far_end.HecInterleaved);
}

//w ad linedatan
{
    DMTCTRL_LINE_DATA_N_INFO Arg;
    unsigned int a2tmp=0, a3tmp=0, a4tmp=0;
    unsigned int a2tmpmod=0, a3tmpmod=0, a4tmpmod=0;
 
    memset(&Arg,0,sizeof(DMTCTRL_LINE_DATA_N_INFO));
    ioctl( nFd, DMTIOCTL_GET_LINE_DATA_N_INFO, &Arg );
    a2tmp=Arg.noiseMarginDnstr/10;a3tmp=Arg.outputPowerUpstr/10;a4tmp=Arg.attenuationDnstr/10;
    a2tmpmod=Arg.noiseMarginDnstr % 10;a3tmpmod=Arg.outputPowerUpstr % 10; a4tmpmod= Arg.attenuationDnstr % 10;
    
    syslog(LOG_INFO,"relative capacity occupation: %d%%\n",Arg.relCapacityOccupationDnstr);
    syslog(LOG_INFO,"noise margin downstream: %d.%d db\n",a2tmp,a2tmpmod);
    syslog(LOG_INFO,"output power upstream: %d.%d dbm\n",a3tmp,a3tmpmod);
    syslog(LOG_INFO,"attenuation downstream: %d.%d db\n",a4tmp,a4tmpmod);
}
//w ad ch
{
    DMTCTRL_CONNECTION_INFO Arg;
    memset(&Arg,0,sizeof(DMTCTRL_CONNECTION_INFO));
    ioctl( nFd, DMTIOCTL_GET_CONNECTION_INFO, &Arg );
    syslog(LOG_INFO,"near-end interleaved channel bit rate: %d kbps\n",Arg.actualBitrateNearInt);
    syslog(LOG_INFO,"near-end fast channel bit rate: %d kbps\n",Arg.actualBitrateNearFast);
    syslog(LOG_INFO,"far-end interleaved channel bit rate: %d kbps\n",Arg.actualBitrateFarInt);
    syslog(LOG_INFO,"far-end fast channel bit rate: %d kbps\n",Arg.actualBitrateFarFast);
}
//w ad linedataf2p for ADSL2+ and w ad linedataf for ADSL/ADSL2
{
    int i=0,j=0;
    unsigned int a2tmp=0, a3tmp=0, a4tmp=0;
    unsigned int a2tmpmod=0, a3tmpmod=0, a4tmpmod=0;
    unsigned char adslmode;  
    char ps[128],szCat[32];
 
    ioctl( nFd, DMTIOCTL_GET_LINE_OPINFO, &adslmode );
    if(adslmode == ADSL2Plus_Mode)
    {
        DMTCTRL_LINE_DATA_F_INFO_ADSL2P Arg;
	memset(&Arg,0,sizeof(DMTCTRL_LINE_DATA_F_INFO_ADSL2P));
        ioctl( nFd, DMTIOCTL_GET_LINE_DATA_F_INFO_ADSL2P, &Arg );
        a2tmp=Arg.noiseMarginUpstr/10;a3tmp=Arg.outputPowerDnstr/10;a4tmp=Arg.attenuationUpstr/10;
        a2tmpmod=Arg.noiseMarginUpstr % 10;a3tmpmod=Arg.outputPowerDnstr % 10; a4tmpmod= Arg.attenuationUpstr % 10;
    
        syslog(LOG_INFO,"relative capacity occupation: %d%%\n",Arg.relCapacityOccupationUpstr);
        syslog(LOG_INFO,"noise margin upstream: %d.%d db\n",a2tmp,a2tmpmod);
        syslog(LOG_INFO,"output power downstream: %d.%d dbm\n",a3tmp,a3tmpmod);
        syslog(LOG_INFO,"attenuation upstream: %d.%d db\n",a4tmp,a4tmpmod);
         
        syslog(LOG_INFO,"carrier load : number of bits per symbol(tone)\n");
	   
        for( i=0;i<16;i++)
        {
            ps[0]='\0';//string
            sprintf(szCat,"tone  %3d-%3d:",i*32,i*32+31);
            strcat(ps,szCat);
            for(j=i*16;j<((i*16)+16);j++)
            {
		    sprintf(szCat,"%1x%1x ", Arg.carrierLoad[j] & 0xF,(Arg.carrierLoad[j] & 0xF0) >> 4);
		    strcat(ps,szCat);
            }
            strcat(ps,"\n");
            syslog(LOG_INFO,ps);
        }
    } 
    else
    {
        DMTCTRL_LINE_DATA_F_INFO_ADSL Arg;
 	memset(&Arg,0,sizeof(DMTCTRL_LINE_DATA_F_INFO_ADSL));
       ioctl( nFd, DMTIOCTL_GET_LINE_DATA_F_INFO_ADSL, &Arg );
        a2tmp=Arg.noiseMarginUpstr/10;a3tmp=Arg.outputPowerDnstr/10;a4tmp=Arg.attenuationUpstr/10;
        a2tmpmod=Arg.noiseMarginUpstr % 10;a3tmpmod=Arg.outputPowerDnstr % 10; a4tmpmod= Arg.attenuationUpstr % 10;
    
        syslog(LOG_INFO,"relative capacity occupation: %d%%\n",Arg.relCapacityOccupationUpstr);
        syslog(LOG_INFO,"noise margin upstream: %d.%d db\n",a2tmp,a2tmpmod);
        syslog(LOG_INFO,"output power downstream: %d.%d dbm\n",a3tmp,a3tmpmod);
        syslog(LOG_INFO,"attenuation upstream: %d.%d db\n",a4tmp,a4tmpmod);
        syslog(LOG_INFO,"carrier load : number of bits per symbol(tone)\n");

        for( i=0;i<8;i++)
        {
            ps[0]='\0';//string
            sprintf(szCat,"tone  %3d-%3d:",i*32,i*32+31);
            strcat(ps,szCat);
            for(j=i*16;j<((i*16)+16);j++)
            {
		    sprintf(szCat,"%1x%1x ", Arg.carrierLoad[j] & 0xF,(Arg.carrierLoad[j] & 0xF0) >> 4);
		    strcat(ps,szCat);
            }
            strcat(ps,"\n");
            syslog(LOG_INFO,ps);
         }
    }
}
	close(nFd);
	//buf = websGetVar(wp, T("ADSLRATE"),"");
	websRedirect(wp, T("syslog.asp"));
}

static void Adsl_Form(webs_t wp,char_t * path,char_t * query)
{
	char_t *buf=NULL;
	char funcNum[4];
	Load_Adsl_Config();
	
//	DPRINTF("Adsl_Form is called\n");
 	buf = websGetVar(wp, T("ADSLRATE"),"");
 	adsl_config.adslrate=atoi(buf); 
	buf = websGetVar(wp, T("ADSLMODE"),"");
 	adsl_config.adslmode=atoi(buf); 
	Save_Adsl_Config(&adsl_config);
	sprintf(funcNum,"%d",FUNCTION_NUM_ADSL);		
	run_task("/usr/sbin/runFunc","/usr/sbin/runFunc", "start",funcNum,NULL);
	websRedirect(wp, T("adsl.asp"));
}

void resetCurrentAdslCfg(void)
{
	ADSL_CONFIG_DATA *d=&adsl_config;
	memset(d,0,sizeof(ADSL_CONFIG_DATA));
}

static int Load_Adsl_Config (void)
{
	FILE *fp;
	resetCurrentAdslCfg();
	fp=fopen(ADSL_CONFIG_FILE,"r");
	if(fp) {
		fscanf(fp,"%d %d\n",&adsl_config.adslmode,&adsl_config.adslrate);
		fclose(fp);
	}
	return 0;
}

static int Save_Adsl_Config (ADSL_CONFIG_DATA *d)
{
	FILE *fp;
	fp=fopen(ADSL_CONFIG_FILE,"w");
	if(fp) {
		fprintf(fp,"%d %d\n",d->adslmode,d->adslrate);
		fclose(fp);
	}
	return 0;
}

#if 0
void getDownUpStreamRate(int *downlinkspeed,int *uplinkspeed)
{
	
	dmt_write_cmd(DMT_LINERATE);
	strcpy(buf1_255,(char *)dmt_read_status(buf3_255));
	int dumb1,dumb2,dumb3,dumb4;

	if( strlen(buf1_255) > 0)
		sscanf(buf1_255,"%d %d %d %d", 
		&dumb1,&dumb2,&dumb3,&dumb4);
		if(dumb1)
			*downlinkspeed=dumb1;
		if(dumb2)
			*downlinkspeed=dumb2;
		if(dumb3)
			*uplinkspeed=dumb3;
		if(dumb4)
			*uplinkspeed=dumb4;

		//DPRINTF("getDownUpStreamRate:downlinkspeed=%d,uplinkspeed=%d\n",*downlinkspeed,*uplinkspeed);

}
#else
void getDownUpStreamRate(int *downlinkspeed,int *uplinkspeed)
{
    DMTCTRL_CONNECTION_INFO Arg;
    int nFd = open( "/dev/dmtdev", O_RDWR );
    if( nFd == -1  )
	{
		DPRINTF("getDownUpStreamRate:severe error for open /dev/dmtdev for ADSL device\n");
		return;
	}
    ioctl( nFd, DMTIOCTL_GET_CONNECTION_INFO, &Arg );
        
	DPRINTF("near-end interleaved channel bit rate: %d kbps\n",Arg.actualBitrateNearInt);
	DPRINTF("near-end fast channel bit rate: %d kbps\n",Arg.actualBitrateNearFast);
	DPRINTF("far-end interleaved channel bit rate: %d kbps\n",Arg.actualBitrateFarInt);
	DPRINTF("far-end fast channel bit rate: %d kbps\n",Arg.actualBitrateFarFast);
 	if(Arg.actualBitrateNearInt)
		*downlinkspeed=Arg.actualBitrateNearInt;
	if(Arg.actualBitrateNearFast)
		*downlinkspeed=Arg.actualBitrateNearFast;
	if(Arg.actualBitrateFarInt)
		*uplinkspeed=Arg.actualBitrateFarInt;
	if(Arg.actualBitrateFarFast)
		*uplinkspeed=Arg.actualBitrateFarFast;
	close(nFd);      
}
#endif

static int get_adsl_status (int ejid, webs_t wp, int argc, char_t **argv)
{
	//int rate1=0, rate2=0, rate3=0, rate4=0;
	//int status=0;
	int nFd = open( "/dev/dmtdev", O_RDWR );
	if( nFd == -1  )
	{
		DPRINTF("get_atm_status:severe error for open /dev/dmtdev for ADSL device\n");
		return websWrite(wp, T("DMT Failed"));;
	}
	if (strcmp(argv[0],"DMTSTATUS")==0)
	{
		DMTCTRL_DMT_INFO Arg;
        ioctl( nFd, DMTIOCTL_GET_DMT_INFO, &Arg );
 		close(nFd);
       if(Arg.dmt_status==ADSL_UP)
			return websWrite(wp, T("Up"));
        else if(Arg.dmt_status==ADSL_TRAINING_2)
			return websWrite(wp, T("Training"));
		else
			return websWrite(wp, T("Down"));
	}
	else if (strcmp(argv[0],"DMT_Version")==0)
	{
		DMTCTRL_DMT_VERSION Arg;
        ioctl( nFd, DMTIOCTL_GET_DMT_FW_VERSION, &Arg );
		close(nFd);
        //DPRINTF("DMT FwVer: %d.%d.%d.%d_%c_%s, HwVer:T%dF%d_%d.%d\n",Arg.fw_ver_r,Arg.fw_ver_p,Arg.fw_rev,Arg.b_ver,(Arg.annex_afe & 0x02)?'A':'B',(Arg.annex_afe & 0x01)?"TC":"SI",Arg.tdm_ver,Arg.fdm_ver,Arg.major,Arg.minor);
		//strcpy(buf2_255,read_file_para_value(DEFAULT_CONFIG_FILE,"DMT_VERSION=",buf3_255));
        sprintf(buf2_255,"DMT FwVer: %d.%d.%d.%d_%c_%s, HwVer:T%dF%d_%d.%d\n",Arg.fw_ver_r,Arg.fw_ver_p,Arg.fw_rev,Arg.b_ver,(Arg.annex_afe & 0x02)?'A':'B',(Arg.annex_afe & 0x01)?"TC":"SI",Arg.tdm_ver,Arg.fdm_ver,Arg.major,Arg.minor);
		return websWrite(wp, T("%s"),buf2_255);
	}
	else if (strcmp(argv[0],"Sync_Mode")==0)
	{
    	unsigned char Arg;
		ioctl( nFd, DMTIOCTL_GET_LINE_OPINFO, &Arg );
		close(nFd);
        switch(Arg)
        {
		case T1413_Mode:
			return websWrite(wp, T("ADSL t1.413"));
		case GDMT_Mode:
			return websWrite(wp, T("ADSL G.Dmt"));
		case GLite_Mode:
			return websWrite(wp, T("ADSL g.lite"));
		case ADSL2_Mode:
			return websWrite(wp, T("ADSL2"));
		case ADSL2Plus_Mode:
			return websWrite(wp, T("ADSL2+"));
		default:
			return websWrite(wp, T("------------"));
		}
	}
	else if (strcmp(argv[0],"ADSL_RATE")==0)
	{
 		close(nFd);
		Load_Adsl_Config();
		return websWrite(wp, T("%d"),adsl_config.adslrate);
	}
	else if (strcmp(argv[0],"ADSL_MODE")==0)
	{
 		close(nFd);
		Load_Adsl_Config();
		return websWrite(wp, T("%d"),adsl_config.adslmode);
	}
	else if (strcmp(argv[0],"Annex")==0)
	{
 		close(nFd);
		strcpy(buf2_255,read_file_para_value(DEFAULT_CONFIG_FILE,"ANNEX=",buf3_255));
		return websWrite(wp, T("%s"),buf2_255);
	}
	else if ( (strcmp(argv[0],"DMTRATE_1")==0) ||(strcmp(argv[0],"DMTRATE_2")==0) )
	{
		DMTCTRL_CONNECTION_INFO Arg;
		DMTCTRL_DMT_INFO ArgStatus;
        ioctl( nFd, DMTIOCTL_GET_DMT_INFO, &ArgStatus );
		if(ArgStatus.dmt_status!=ADSL_UP)
		{
			close(nFd);
			return websWrite(wp, T("0"));
		}

 		ioctl( nFd, DMTIOCTL_GET_CONNECTION_INFO, &Arg );
 		close(nFd);
		if (strcmp(argv[0],"DMTRATE_1")==0){
			if (Arg.actualBitrateNearInt)	
		 	        return websWrite(wp, T("%d"), Arg.actualBitrateNearInt);
			if (Arg.actualBitrateNearFast)	
		 	        return websWrite(wp, T("%d"), Arg.actualBitrateNearFast);
		}
		if (strcmp(argv[0],"DMTRATE_2")==0){
			if (Arg.actualBitrateFarInt)	
		 	        return websWrite(wp, T("%d"), Arg.actualBitrateFarInt);
			if (Arg.actualBitrateFarFast)	
		 	        return websWrite(wp, T("%d"), Arg.actualBitrateFarFast);
		}
	}
	else if ( (strcmp(argv[0],"NM_UP")==0) ||(strcmp(argv[0],"ATT_UP")==0) )
	{
		unsigned int a2tmp=0,  a4tmp=0;
		unsigned int a2tmpmod=0,  a4tmpmod=0;
		unsigned char adslmode;  
		DMTCTRL_DMT_INFO ArgStatus;
		ioctl( nFd, DMTIOCTL_GET_DMT_INFO, &ArgStatus );
		if(ArgStatus.dmt_status!=ADSL_UP)
		{
			close(nFd);
			return websWrite(wp, T(ADSL_NOT_UP[locale_config.locale]));
		}

		ioctl( nFd, DMTIOCTL_GET_LINE_OPINFO, &adslmode );
		if(adslmode == ADSL2Plus_Mode)
		{
			DMTCTRL_LINE_DATA_F_INFO_ADSL2P Arg;
			memset(&Arg,0,sizeof(DMTCTRL_LINE_DATA_F_INFO_ADSL2P));
			ioctl( nFd, DMTIOCTL_GET_LINE_DATA_F_INFO_ADSL2P, &Arg );
			a2tmp=Arg.noiseMarginUpstr/10;
			a2tmpmod=Arg.noiseMarginUpstr % 10; 
 			a4tmp=Arg.attenuationUpstr/10;
			a4tmpmod= Arg.attenuationUpstr % 10;
  		}
		else
		{
			DMTCTRL_LINE_DATA_F_INFO_ADSL Arg;
			memset(&Arg,0,sizeof(DMTCTRL_LINE_DATA_F_INFO_ADSL));
			ioctl( nFd, DMTIOCTL_GET_LINE_DATA_F_INFO_ADSL, &Arg );
 			a2tmp=Arg.noiseMarginUpstr/10;
			a2tmpmod=Arg.noiseMarginUpstr % 10; 
 			a4tmp=Arg.attenuationUpstr/10;
			a4tmpmod= Arg.attenuationUpstr % 10;
		}
		close(nFd);
		if (strcmp(argv[0],"NM_UP")==0)
			return websWrite(wp, T("%d.%d db"),a2tmp,a2tmpmod);
		if (strcmp(argv[0],"ATT_UP")==0)
			return websWrite(wp, T("%d.%d db"),a4tmp,a4tmpmod);
	}
	else if ( (strcmp(argv[0],"NM_DOWN")==0) ||(strcmp(argv[0],"ATT_DOWN")==0) )
	{
		DMTCTRL_LINE_DATA_N_INFO Arg;
 		unsigned int a2tmp=0,  a4tmp=0;
		unsigned int a2tmpmod=0,  a4tmpmod=0;
		DMTCTRL_DMT_INFO ArgStatus;
		ioctl( nFd, DMTIOCTL_GET_DMT_INFO, &ArgStatus );
		if(ArgStatus.dmt_status!=ADSL_UP)
		{
			close(nFd);
			return websWrite(wp, T(ADSL_NOT_UP[locale_config.locale]));
		}

		memset(&Arg,0,sizeof(DMTCTRL_LINE_DATA_N_INFO));
		ioctl( nFd, DMTIOCTL_GET_LINE_DATA_N_INFO, &Arg );
		close(nFd);
		if (strcmp(argv[0],"NM_DOWN")==0)
		{
			a2tmp=Arg.noiseMarginDnstr/10;
			a2tmpmod=Arg.noiseMarginDnstr % 10; 
			return websWrite(wp, T("%d.%d db"),a2tmp,a2tmpmod);
		}
		if (strcmp(argv[0],"ATT_DOWN")==0)
		{
			a4tmp=Arg.attenuationDnstr/10;
			a4tmpmod= Arg.attenuationDnstr % 10;
   			return websWrite(wp, T("%d.%d db"),a4tmp,a4tmpmod);
		}
	}
	return websWrite(wp, T(""));
}
static int is_dmt_connected (int ejid, webs_t wp, int argc, char_t **argv)
{
	DMTCTRL_DMT_INFO ArgStatus;
 	int nFd = open( "/dev/dmtdev", O_RDWR );
	if( nFd == -1  )
	{
		DPRINTF("get_atm_status:severe error for open /dev/dmtdev for ADSL device\n");
		return websWrite(wp, T("DMT Failed"));;
	}
    ioctl( nFd, DMTIOCTL_GET_DMT_INFO, &ArgStatus );
	close(nFd);
	if(ArgStatus.dmt_status==ADSL_UP)
	{
		return websWrite(wp, T("<img src=tick.gif border=0>"));
	}
	else
	{
		strcpy(buf2_255,read_file_para_value(VAR_SYS_STATUS_FILE,"WAN=",buf3_255));
		if ((!strcmp(buf2_255,"fixed")) || (!strcmp(buf2_255,"routed")))
			write_value_to_file("/tmp/ppp/wan_connect_status",CONNECTED);
		return websWrite(wp, T("<img src=cross.gif border=0>"));
	}

}
void DefineWebFunctions_adsl() {
	websAspDefine(T("get_adsl_mod"), get_adsl_mod);
	websAspDefine(T("is_dmt_connected"), is_dmt_connected);
	websAspDefine(T("get_adsl_status"), get_adsl_status);	
	websFormDefine(T("adsl"), Adsl_Form);
	websFormDefine(T("adsldiag"), AdslDiagnostic_Form);
}
