/****************************************************************************
 *
 *  Copyright (c) 2001 by LSI Logic Corporation, all rights reserved.
 *
 *                         LSI Logic confidential
 *  The information contained herein is proprietary of LSI Logic Corporation.
 *
 *  This software is provided "as is" without any express or implied warranty.
 *  LSI Logic disclaims all representations and warranties with regard to
 *  this software, including all implied warranties of merchantability and
 *  fitness for a particular purpose. In no event shall LSI Logic be liable
 *  for any special, indirect or consequential damages or any damages
 *  whatsoever resulting from loss of use, data or profits, whether in an
 *  action of contract, negligence or other tortious act or omission,
 *  arising out of or in connection with the use or performance of this
 *  software.
 *
 *  $Id: $
 *
 *  Revision Details
 *  $Log:  $
 *
 *
 *  File Description:
 *  This file contains definitions and prototypes for the connection config CGI script.
 *
 ****************************************************************************/

#ifndef CONNECTIONS_CONFIG_H
#define CONNECTIONS_CONFIG_H

/****************************************************************************
 * includes
 ****************************************************************************/
#include "gen_config.h"
#include "linux/config.h"

/****************************************************************************
 * definitions (macros)
 ****************************************************************************/
#ifdef CONFIG_OBELIX_MAX_CONNECTIONS_4
#define MAX_CONNECTIONS 4
//#warning Web i/f configured for 4 connections
#elif CONFIG_OBELIX_MAX_CONNECTIONS_8
#define MAX_CONNECTIONS 8
//#warning Web i/f configured for 8 connections
#else
#define MAX_CONNECTIONS 8
/*#error Max connections not configured in autoconf.h*/
#endif

#define CONNECTIONS_MAX_VPI 255
#define CONNECTIONS_MAX_VCI 65535

#define CONN_MAX_FIELD_NAME 50
#define CONNECTIONS_CONFIG_FILE CONFIG_DIR"/connections.conf"
#define CONNECTIONS_SCRIPT_FILE CONFIG_DIR"/createcon"
#define CONNECTIONS_CLEANUP_SCRIPT_FILE CONFIG_DIR"/destroycon"

#define IGMP_RELOAD_FILE "/etc/mcreload"

/****************************************************************************
 * definitions (types)
 ****************************************************************************/
/*
	:enable:vpi:vci:stack:encap:param1:param2:param3:defaultroute:profileNumber:
*/
typedef enum {
	stack_bridge,
	stack_pppoe,
	stack_pppoa,
	stack_ipoa,
  stack_wbridge,
	stack_wbridge_dynamic,
	stack_atm_only,
	stack_bridge_null,
	stack_bridge_ether,
	stack_null
}StackConfig;

typedef enum {
  encap_llc,
  encap_vc
}EncapType;

typedef enum {
  auth_papchap = 1,
  auth_chap = 2,
  auth_pap = 3,
  auth_noauth = 4
}AuthProt;

typedef struct {
		int enable;

    int defaultRoute;

		int vpi;
		int vci;

		StackConfig stack;
		EncapType encap;
    AuthProt authProtocol;

#define MAX_FIELD_WIDTH 50
		char param1[MAX_FIELD_WIDTH];
		char param2[MAX_FIELD_WIDTH];
		char param3[MAX_FIELD_WIDTH];

    int inactivityTimer;
    int echoTimer;

    int profileNumber;/* Firewall profile index */

    int tdProfileNumber;/* Traffic profile index */
}ConnectionData;


/****************************************************************************
 * function prototypes
 ****************************************************************************/
extern int connections_load_config(void *);
extern int connections_save_config(void *);
extern int connections_save_script(void *);
extern int connections_validate(char **, void *);
extern int connections_change(time_t *lastConnectionsChange); /* implemented in lsilAdslMib.c */

extern int conn_get_enable_v(int instance);
extern void conn_set_enable_v(int instance, int value);

extern int conn_get_vpi_v(int instance);
extern void conn_set_vpi_v(int instance, int value);

extern int conn_get_vci_v(int instance);
extern void conn_set_vci_v(int instance, int value);

extern int conn_get_stack_v(int instance);
extern void conn_set_stack_v(int instance, StackConfig value);

extern int conn_get_encap_v(int instance);
extern void conn_set_encap_v(int instance, EncapType value);

extern char *conn_get_param1_v(int instance);
extern void conn_set_param1_v(int instance, char *value);
extern void conn_reset_param1_v(int instance);

extern char *conn_get_param2_v(int instance);
extern void conn_set_param2_v(int instance, char *value);
extern void conn_reset_param2_v(int instance);

extern char * conn_get_param3_v(int instance);
extern void conn_set_param3_v(int instance, char *value);
extern void conn_reset_param3_v(int instance);

extern int conn_get_profilenumber_v(int instance);
extern void conn_set_profilenumber_v(int instance, int value);

extern int conn_get_defaultroute_v(int instance);
extern void conn_set_defaultroute_v(int instance, int value);

extern int conn_get_auth_v(int instance);
extern void conn_set_auth_v(int instance, AuthProt value);

extern int conn_get_inactivity_timer_v(int instance);
extern void conn_set_inactivity_timer_v(int instance, int value);

extern int conn_get_echo_timer_v(int instance);
extern void conn_set_echo_timer_v(int instance, int value);

extern int conn_get_td_profilenumber_v(int instance);
extern void conn_set_td_profilenumber_v(int instance, int value);

extern void conn_create(int index);
extern void conn_destroy(int index);
extern void conn_restart(int index);
extern void create_detroy_con_ts(int , int );
extern void create_detroy_con_fw(int , int );


extern int conn_get_ifName(int index, char * ifName);
extern char * get_ifaddr(char *name);

extern int loopback_apply(int index, int enable);
extern int loopback_applied(int index);

#endif

