/*
 *  ddns.htm: Dynamic DNS Configuration
 */

#include "webs.h"
#include "ddns.h"
#include "lan_conf.h"
#include "wan.h"
#include "../iptables-1.2.11/runFunc.h"

extern void run_task(char *cmd, ...);
extern char* read_file_para_value(char *filename, char *arg1,char *buf);
static int get_ddns_status(int eid, webs_t wp, int argc, char_t **argv);
static int get_ddns_mod (int ejid, webs_t wp, int argc, char_t **argv);
static void form_DDNS_conf(webs_t wp, char_t *path, char_t *query);

static void resetCurrentDdnsCfg(void);
static void Load_Ddns_Config (void);
static void Save_Ddns_Config (DDNS_CONFIG_DATA *d);

static char* DDNS_SERVER_LIST [] = {
/*0*/"dyndns-custom",
/*1*/"dyndns",
/*2*/"dyndns-static",
/*3*/"zoneedit",
/*4*/"orgdns",//orgdns
/*5*/"dhs",
/*6*/"dyns",
/*7*/"minidns",//minidns
/*8*/"no-ip",
/*9*/"c3322",
/*10*/"dyndns-dk",
/*11*/"tzo",
/*12*/"enom",
/*13*/"3domain",//3domain
/*14*/"dyfi",// www.dy.fi
/*15*/"mweb",// www.mweb.net
};
DDNS_CONFIG_DATA ddns_config;
// Mulder added 2005-6-28
static int Select = 0, temp_ddns_server;

static void resetCurrentDdnsCfg(void)
{
	int i;
 	DDNS_CONFIG_DATA *d = &ddns_config;

	d->enable=0;
	d->ddns_server=1;//DS_WWW_DYNDNS_ORG_DYNAMIC
        for(i=0;i<MAXNUM_DDNS_SERVER;i++)
        {
	        strcpy(d->hostname[i],"");
	        strcpy(d->username[i],"");
	        strcpy(d->password[i],"");
	        strcpy(d->period[i],"28");
	        strcpy(d->timeunit[i],"1");
	        strcpy(d->wildcard[i],"0");
	}
}

static void Load_Ddns_Config(void)
{
    FILE *fp;
	int i;
    char_t line[1024];
    
    resetCurrentDdnsCfg();
    fp=fopen(DDNS_CONFIG_FILE,"r");
	if (!fp) {
		return;
	}
    if (!fgets(line, 1024, fp))
    {
		return;
	}
	sscanf(line,"#:%d:\n",&ddns_config.enable);
    if (!fgets(line, 1024, fp))
    {
		return;
	}
    sscanf(line,"#:%d:\n",&ddns_config.ddns_server);

    for(i=0;i<MAXNUM_DDNS_SERVER;i++){
		if (fgets(line, 1024, fp)){
	        	sscanf(line,"#|%50[^|]|%256[^|]|%256[^|]|%4[^|]|%2[^|]|%2[^|]|\n",
	               		ddns_config.hostname[i],
		               	ddns_config.username[i],
		               	ddns_config.password[i],
	        	       	ddns_config.period[i],
	               		ddns_config.timeunit[i],
	               		ddns_config.wildcard[i]
	              );
		print_blank_string(ddns_config.hostname[i]);
		print_blank_string(ddns_config.username[i]);
		print_blank_string(ddns_config.password[i]);
		print_blank_string(ddns_config.period[i]);
		print_blank_string(ddns_config.timeunit[i]);
		print_blank_string(ddns_config.wildcard[i]);
		}
	}
        fclose(fp);	
}


static void Save_Ddns_Config (DDNS_CONFIG_DATA *d)
{

    FILE *fp;
	int i;

    fp=fopen(DDNS_CONFIG_FILE,"w");
	fprintf(fp,"#:%d:\n",d->enable);
	fprintf(fp,"#:%d:\n",d->ddns_server);
    for(i=0;i<MAXNUM_DDNS_SERVER;i++) {
		load_blank_string(d->hostname[i]);
		load_blank_string(d->username[i]);
		load_blank_string(d->password[i]);
		load_blank_string(d->period[i]);
		load_blank_string(d->timeunit[i]);
		load_blank_string(d->wildcard[i]);
		fprintf(fp,"#|%s|%s|%s|%s|%s|%s|\n",
			d->hostname[i],
    	        	d->username[i],
			d->password[i],
		    d->period[i],
		    d->timeunit[i],
			d->wildcard[i]);
    }

	fprintf(fp,"host=%s\n",d->hostname[d->ddns_server]);	
	fprintf(fp,"user=%s:%s\n",d->username[d->ddns_server],d->password[d->ddns_server]);  
	fprintf(fp,"period=%d\n",(atoi(d->timeunit[d->ddns_server])?86400:3600)*atoi(d->period[d->ddns_server]));  
	fprintf(fp,"service-type=%s\n",DDNS_SERVER_LIST[d->ddns_server]);  
//	fprintf(fp,"interface=\n");// use environmental var -i WAN_IF
//	prevent ddns process terminated after ddns update 
	fprintf(fp,"daemon\n");  // Mulder noted: use this to check DDNS enabled
	fprintf( fp,"foreground\n");  // Mulder added 2004-09-01
	if (!(strcmp(d->wildcard[d->ddns_server],"1")))
		fprintf(fp,"wildcard\n");  // Mulder added 2004-09-01		
     fclose(fp);

	// Mulder added 2005-6-16 for keeping blank in current settings 
	 for(i=0;i<MAXNUM_DDNS_SERVER;i++) {
		print_blank_string(d->hostname[i]);
		print_blank_string(d->username[i]);
		print_blank_string(d->password[i]);
		print_blank_string(d->period[i]);
		print_blank_string(d->timeunit[i]);
		print_blank_string(d->wildcard[i]);
    }
			
}
static int get_ddns_status(int eid, webs_t wp, int argc, char_t **argv)
{
// Mulder modified 2005-6-28 for fixing DNS server change issue -- Begin --
	int server_idx;
	
	if (Select)
		server_idx = temp_ddns_server;
	else
		server_idx = ddns_config.ddns_server;
	
	if (strcmp(argv[0],"Enable")==0)
	{
// Mulder added 2005-6-28 for fixing DNS server change issue -- Begin --
		if (Select)
			return websWrite(wp,T("1"));
		else
// Mulder added 2005-6-28 for fixing DNS server change issue -- end --
			return websWrite(wp,T("%d"),ddns_config.enable);
	}
	if (strcmp(argv[0],"Hostname")==0)
		return websWrite(wp,T("%s"),ddns_config.hostname[server_idx]);
	if (strcmp(argv[0],"Username")==0)
		return websWrite(wp,T("%s"),ddns_config.username[server_idx]);
	if (strcmp(argv[0],"Password")==0)
		return websWrite(wp,T("%s"),ddns_config.password[server_idx]);
	if (strcmp(argv[0],"Period")==0)
		return websWrite(wp,T("%s"),ddns_config.period[server_idx]);

// Mulder added 2005-6-28 for same as 7402 -- Begin --
	if (strcmp(argv[0],"wildcard")==0)
	{
		// Note: Onlly "dyndns" and "dyndns-static" have Wildcard option
		if (!((server_idx == 1) || (server_idx == 2)) )
			return websWrite(wp,T("disabled"));  
		if (!(strcmp(ddns_config.wildcard[server_idx],"1")))
			return websWrite(wp, T("%s"),"checked");
		else
	        return websWrite(wp, T(""));
	}
// Mulder added 2005-6-28 for same as 7402 -- end --
// Mulder modified 2005-6-28 for fixing DNS server change issue -- end --
	return 1;
}



static int get_ddns_mod (int ejid, webs_t wp, int argc, char_t **argv)
{
    int i=0,j;

// Mulder modified 2005-6-28 for fixing DNS server change issue -- Begin --
	int server_idx;
	
	if (Select)
		server_idx = temp_ddns_server;
	else
		server_idx = ddns_config.ddns_server;

    j=atoi(argv[0]);
    if (strcmp(argv[1],"ddns_server")==0)
		i=server_idx;
    if (strcmp(argv[1],"unit")==0)
	{
		i=atoi(ddns_config.timeunit[server_idx]);
		
		// Mulder
		if (j == 0)   
			Select = 0;  // Note: Use last item to clear flag
	}
// Mulder modified 2005-6-28 for fixing DNS server change issue -- end --
// Mulder marked 2005-6-28 for same as 7402 -- Begin --
/*		
        if (strcmp(argv[1],"wildcard")==0)
		i=atoi(ddns_config.wildcard[ddns_config.ddns_server]);
*/
// Mulder marked 2005-6-28 for same as 7402 -- end --
	if (i==j)
	        return websWrite(wp, T("%s"),argv[2]);
	else
	        return websWrite(wp, T(""));
}		
static void form_DDNS_conf(webs_t wp, char_t * path,char_t * query)
{
    char *buf=NULL;
//   	int Select;
	char funcNum[4];
    char_t buf1[64];
	// Mulder added 2005-6-27 to compare with old and new configuretion. 
	DDNS_CONFIG_DATA old_ddns_config;

	buf=websGetVar(wp, T("select"), "0");
    Select=atoi(buf);

	if (Select == 1) {  // not click "Apply"
		buf=websGetVar(wp, T("ddnsServ"),"1");
// Mulder modified 2005-6-28 for fixing DNS server change issue
//			ddns_config.ddns_server=atoi(buf); 
		temp_ddns_server=atoi(buf); 
		websRedirect(wp, T("ddnsEnable.asp"));
		return;	
	}	

	Load_Ddns_Config();
	
	// Mulder added 2005-6-27 to compare with old and new configuretion. 
	memcpy(&old_ddns_config, &ddns_config, sizeof(DDNS_CONFIG_DATA));
	
	buf=websGetVar(wp, T("EnableDDNS"), "0");
    ddns_config.enable=atoi(buf);

	if (ddns_config.enable==0)   // disable
	{
		if (old_ddns_config.enable != ddns_config.enable)  // enable --> disable
		{
			sprintf(funcNum,"%d",FUNCTION_NUM_DDNS);		
			run_task("/usr/sbin/runFunc","/usr/sbin/runFunc", "stop",funcNum,NULL);		

			Save_Ddns_Config(&ddns_config);
		}
		websRedirect(wp, T("ddnsDisable.asp"));
	}
	else 	// enable 
	{	
		// click "Apply"
		char_t str_wanip[20];
	
		buf=websGetVar(wp, T("ddnsServ"),"1");
		ddns_config.ddns_server=atoi(buf); 
                   	       
		buf=websGetVar(wp, T("ddnsTimeunit"),"1");
		strcpy(ddns_config.timeunit[ddns_config.ddns_server],buf);
		buf=websGetVar(wp, T("ddnsPeriod"),"28");
		strcpy(ddns_config.period[ddns_config.ddns_server],buf); 
		buf=websGetVar(wp, T("ddnsWildcard"),"0");
		strcpy(ddns_config.wildcard[ddns_config.ddns_server],buf); 
		buf=websGetVar(wp, T("ddnsHost"),"");
		strcpy(ddns_config.hostname[ddns_config.ddns_server],buf);
		
		buf=websGetVar(wp, T("ddnsUser"),"");
		strcpy(ddns_config.username[ddns_config.ddns_server],buf);
			
		buf=websGetVar(wp, T("ddnsPass"),"");
		strcpy(ddns_config.password[ddns_config.ddns_server],buf);


		// Mulder moidified 2005-6-27 to compare with old and new configuretion. 
		if (memcmp(&old_ddns_config, &ddns_config, sizeof(DDNS_CONFIG_DATA)) != 0)
		{
			sprintf(funcNum,"%d",FUNCTION_NUM_DDNS);		
			run_task("/usr/sbin/runFunc","/usr/sbin/runFunc", "stop",funcNum,NULL);		
			Save_Ddns_Config(&ddns_config);
	  	
			strcpy(str_wanip,(char *)read_file_para_value(VAR_SYS_STATUS_FILE,"WAN_IP=",buf1));
			if(isValidIP(str_wanip))
			{   //NOTE ONLY WHEN WAN INTERFACE is ready, then we start the function
				run_task("/usr/sbin/runFunc","/usr/sbin/runFunc", "start",funcNum,NULL);	
			}
		}
		websRedirect(wp, T("ddnsEnable.asp"));	
	}
}

void DefineWebFunctions_ddns() {
	Load_Ddns_Config();
	websAspDefine(T("get_ddns_status"), get_ddns_status);
	websAspDefine(T("get_ddns_mod"), get_ddns_mod);
	websFormDefine(T("ddns"), form_DDNS_conf);
}
