/****************************************************************************
 *
 *  Copyright (c) 2001 by LSI Logic Corporation, all rights reserved.
 *
 *                         LSI Logic confidential
 *  The information contained herein is proprietary of LSI Logic Corporation.
 *
 *  This software is provided "as is" without any express or implied warranty.
 *  LSI Logic disclaims all representations and warranties with regard to
 *  this software, including all implied warranties of merchantability and
 *  fitness for a particular purpose. In no event shall LSI Logic be liable
 *  for any special, indirect or consequential damages or any damages
 *  whatsoever resulting from loss of use, data or profits, whether in an
 *  action of contract, negligence or other tortious act or omission,
 *  arising out of or in connection with the use or performance of this
 *  software.
 *
 *  $Id: $
 *
 *  Revision Details
 *  $Log:  $
 *
 *
 *  File Description:
 *  This file contains definitions for the Address Sharing & Firewall CGI script handlers.
 *
 ****************************************************************************/

#ifndef _FW_CONFIG_H
#define _FW_CONFIG_H

/****************************************************************************
 * includes
 ****************************************************************************/
#include "gen_config.h"

/****************************************************************************
 * definitions (types)
 ****************************************************************************/
#define FW_PORT_FIELD_MAX 40
#define FW_IP_ADDRESS_FIELD_MAX 20

#define FW_MAX_SERVERS 10

typedef struct {
    int masqueradingOn;
    struct {
		int enabled;
        char forwardingIpAddress[FW_IP_ADDRESS_FIELD_MAX + 1];
		char sourceIpAddress[FW_IP_ADDRESS_FIELD_MAX + 1];
        int protocol;
        char ports[FW_PORT_FIELD_MAX + 1];
    }server_record[FW_MAX_SERVERS];
}FwRecord;

/****************************************************************************
 * definitions (literals)
 ****************************************************************************/
#define FW_SCRIPT_CONFIG_FILE CONFIG_DIR"/fw.conf"
#define FW_IPTABLES_FILE CONFIG_DIR"/fwscript"

#define FW_MAX_RECORD_LENGTH 255


#define MAX_NUM_IFINDEX_TO_SEARCH 30

#define IPTABLES "iptables "
#define ECHOCMD "echo "

enum pf_protocol_types {
	pf_protocol_tcp = 0,
	pf_protocol_udp,
	pf_protocol_http,
	pf_protocol_ftp,
	pf_protocol_netmeeting,
	pf_protocol_smtp,
	pf_protocol_pop3,
	pf_protocol_l2tp,
	pf_protocol_ipsec,
	pf_protocol_telnet,
	pf_protocol_all
};

struct pf_app {
		char *udp_ports;
		char *tcp_ports;
};

/****************************************************************************
 * function prototypes
 ****************************************************************************/

extern int fw_save_config(void *);
extern int fw_load_config(void *);
extern int fw_get_ip_masquerading_v(int);
extern void fw_set_ip_masquerading_v(int, int);
extern int fw_get_port_forwarding_enabled_v(int, int);
extern void fw_set_port_forwarding_enabled_v(int, int, int);
extern char *fw_get_port_forwarding_ipaddress_v(int, int);
extern void fw_set_port_forwarding_ipaddress_v(int, int, char *);
extern char *fw_get_port_forwarding_srcipaddress_v(int, int);
extern void fw_set_port_forwarding_srcipaddress_v(int, int, char *);
extern char *fw_get_port_forwarding_ports_v(int, int);
extern void fw_set_port_forwarding_ports_v(int, int, char *);
extern int fw_get_port_forwarding_protocol_v(int, int);
extern void fw_set_port_forwarding_protocol_v(int, int, int);

#endif
