/*
 * IPSEC
 */
#ifndef __IPSEC_CONF_H__
#define __IPSEC_CONF_H__

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define IPSEC_Rule_No 0
#define IPSEC_Rule_Yes 1

#define IPSEC_PROPOSAL_ESP 0
#define IPSEC_PROPOSAL_AH 1

#define IPSEC_ESP_3DES 0
#define IPSEC_ESP_AES 1
#define IPSEC_ESP_DES 2

#define IPSEC_PFS_No 0
#define IPSEC_PFS_Yes 1

#define IPSEC_ResponderOnly_No 0
#define IPSEC_ResponderOnly_Yes 1

#define IPSEC_AggrMode_No 0
#define IPSEC_AggrMode_Yes 1

#define IPSEC_PSK_MAXSTRING 129
#define IPSEC_REMOTEIP_STRING_MAX 100

#define IPSEC_APPLICATION_TYPE_LAN_LAN 0
#define IPSEC_APPLICATION_TYPE_HOST_LAN_RESP 1
#define IPSEC_APPLICATION_TYPE_HOST_LAN_INIT 2
#define IPSEC_APPLICATION_TYPE_HOST_HOST 3

typedef struct IPSEC_RULE {
	int Item;	
	int Active;	
	char name[IPSEC_REMOTEIP_STRING_MAX];
	int ApplicationType;
	//struct in_addr LocalSubnet;
	//struct in_addr LocalSubnetMask;
	char LocalSubnet[20];	
	char LocalSubnetMask[20];	

 	char LocalIKE_ID[IPSEC_REMOTEIP_STRING_MAX];
        char RemoteIP[IPSEC_REMOTEIP_STRING_MAX];	
	//struct in_addr RemoteSubnet;	
	//struct in_addr RemoteSubnetMask;
	char RemoteSubnet[20];	
	char RemoteSubnetMask[20];	
	char RemoteIKE_ID[IPSEC_REMOTEIP_STRING_MAX];
	int Proposal;
	int Esp;
	char PreSharedKey[IPSEC_PSK_MAXSTRING];
	int Phase1_SA_Lifetime;
	int Phase2_SA_Lifetime;
	int perfect_forward_secrecy;
        int AggressiveMode;
	int   responder_only;
} IPSEC_RULE_T;

#define MAX_RULE 8
typedef struct IPSEC_TABLE {
	int TotalNum;
	int NextItem;
	IPSEC_RULE_T Rule[MAX_RULE];
#define IPSEC_NAT_T_No 0
#define IPSEC_NAT_T_Yes 1
	int NAT_Traversal_Enabled;// default disabled
	int NAT_T_KL_Sec;  //0..20 seconds. default 20 seconds
} IPSEC_TABLE_T;


//#undef IPSEC_SCRIPT_DEBUG
#define IPSEC_SCRIPT_DEBUG
#ifdef IPSEC_SCRIPT_DEBUG
#define DEBUG_LOG syslog
#else
#define DEBUG_LOG(a, ...)
#endif


/****************************************************************************
 * definitions (literals)
 ****************************************************************************/
#define CONFIG_DIR "/flash"
#define IPSEC_SECRETS_FILE     CONFIG_DIR"/ipsec.secrets"
#define IPSEC_CONFIG_FILE     CONFIG_DIR"/ipsec.conf"
#define VAR_ETC_IPSEC_VAR_FILE "/var/etc/ipsec_var"

typedef struct _IPSEC_CFG_STATUS
{
//	UI8_T isRulePutIntoSPD; 
      /* Used to indicate if the rule has been put into the SPD (the connection object is created from the rules).
	     When the 0.0.0.0 is configured for localGW of an rule, we can not put it in SPD until we have valid localGW (get
		 from get_WAN_IF_ip().  
		 Since it is configured to 0.0.0.0 for the localGW, then  we may later
		 to examine to check if the rule can be put into the SPD if the localGW is given in:
		 1. The IPSec task startup (e.g, the static IP is given when system startup)
		 2. packet is outgoing(by ipsec_outgoing_filter) or incoming(by ipsec_incoming_filter) 
		 3. user dial out a rules.(by sendIPSecCmdStrIkeInit())
		 If then IP can get the valid localGW, then the flag is set to 1 (TRUE). Initially, it is 0( FALSE).
		 Note that the rule shall be ACTIVE. And, if the LocalGW is not 0.0.0.0, then of course, this flag
		 shall be TRUE;
		 */

#define IPSEC_CHANNEL_DISCONNECTED 0
#define IPSEC_CHANNEL_CONNECTING 1
#define IPSEC_CHANNEL_CONNECTED 2
#define IPSEC_CHANNEL_DISCONNECTING 3
  int connected;// Is Connected?

#define REASON_NORMAL 0	//Normal disconnection or not yet disconnected
#define REASON_NO_VALID_REMOTE_GW_IP 1	//Normal disconnection or not yet disconnected
	int disconnectedReason;

#define SA_MODE_BYTE 0
#define SA_TYPE_BYTE 1
#define SA_HMAC_ALG_BYTE 2
#define SA_ENC_ALG_BYTE 3
	int  connectType[4];	// 1 BYTE is for SA mode, 2nd BYTE is SA Type, 3rd Byte is for SA Hmac
	                        // 4 BYTE is for SA EncAlg
							
// SA Mode Values 
#define IPSEC_SA_MODE_TUNNEL	1
#define IPSEC_SA_MODE_TRANSPORT	2

// Sa Type Values
#define	IPSEC_SA_TYPE_AH	2
#define IPSEC_SA_TYPE_ESP	3	

// Sa HmacAlg Values
#define IPSEC_SA_HMAC_NULL	0
#define IPSEC_SA_HMAC_MD5	1
#define IPSEC_SA_HMAC_SHA1	2

// Sa EncAlg Values 
#define IPSEC_SA_ENC_DES	2
#define IPSEC_SA_ENC_3DES	3
#define IPSEC_SA_ENC_NULL	11
#define IPSEC_SA_ENC_AES  12


	time_t connectTimeStart;// the timestamp that connect is estalblished.
	int txPacket;//when connected, the tx packets.
	int rxPacket;//when connected, the Rx packets.

	char remoteGatewayStrForDomainName[20];
} IPSEC_CFG_STATUS;

#define IPSEC_STATUS_SHMKEY            ((key_t)1980)
#define IPSEC_STATUS_SEMLOCKKEY        ((key_t)1981)

#define COPY_IPSEC_STATUS_TO_LOCAL 0
#define SAVE_IPSEC_STATUS_FROM_LOCAL 1

#endif /* end of __IPSEC_CONF_H__ */

