/****************************************************************************
 *
 *  Copyright (c) 2001 by LSI Logic Corporation, all rights reserved.
 *
 *                         LSI Logic confidential
 *  The information contained herein is proprietary of LSI Logic Corporation.
 *
 *  This software is provided "as is" without any express or implied warranty.
 *  LSI Logic disclaims all representations and warranties with regard to
 *  this software, including all implied warranties of merchantability and
 *  fitness for a particular purpose. In no event shall LSI Logic be liable
 *  for any special, indirect or consequential damages or any damages
 *  whatsoever resulting from loss of use, data or profits, whether in an
 *  action of contract, negligence or other tortious act or omission,
 *  arising out of or in connection with the use or performance of this
 *  software.
 *
 *  $Id: $
 *
 *  Revision Details
 *  $Log:  $
 *
 *
 *  File Description:
 *  Header file for the DHCP Configuration CGI script.
 *
 ****************************************************************************/

#ifndef _PROGRAM_CONFIG_H
#define _PROGRAM_CONFIG_H

/****************************************************************************
 * definitions (macros)
 ****************************************************************************/
#define PR_DHCPD_RUN_SCRIPT "/etc/init.d/rcRunDhcpd"
#define PR_DHCPD_STOP_SCRIPT "/etc/init.d/rcStopDhcpd"

#define PR_DHCPRELAY_RUN_SCRIPT "/etc/init.d/rcRunDhcpRelay"
#define PR_DHCPRELAY_STOP_SCRIPT "/etc/init.d/rcStopDhcpRelay"

#define PR_DHCPCLIENT_RUN_SCRIPT "/etc/init.d/rcRunDhcpClient"
#define PR_DHCPCLIENT_STOP_SCRIPT "/etc/init.d/rcStopDhcpClient"

#define PR_NAMED_RUN_SCRIPT "/etc/init.d/rcRunNamed"
#define PR_NAMED_STOP_SCRIPT "/etc/init.d/rcStopNamed"

#define PR_GATED_RUN_SCRIPT "/etc/init.d/rcRunGated"
#define PR_GATED_STOP_SCRIPT "/etc/init.d/rcStopGated"

#define PR_FIREWALL_RUN_SCRIPT "/etc/init.d/rcActivateFw"
#define PR_FIREWALL_STOP_SCRIPT "/etc/init.d/rcDeactivateFw"

#define PR_IGMP_PROXY_RUN_SCRIPT "/etc/init.d/rcRunIgmpp"
#define PR_IGMP_PROXY_STOP_SCRIPT "/etc/init.d/rcStopIgmpp"

#define PR_SNMP_RUN_SCRIPT "/etc/init.d/rcRunSnmp"
#define PR_SNMP_STOP_SCRIPT "/etc/init.d/rcStopSnmp"

#define PR_WLAN_RUN_SCRIPT "/etc/init.d/rcRunWlan"
#define PR_WLAN_STOP_SCRIPT "/etc/init.d/rcStopWlan"

#define PR_FTPD_RUN_SCRIPT "/etc/init.d/rcRunFtpd"
#define PR_FTPD_STOP_SCRIPT "/etc/init.d/rcStopFtpd"

#define PR_TFTPD_RUN_SCRIPT "/etc/init.d/rcRunTftpd"
#define PR_TFTPD_STOP_SCRIPT "/etc/init.d/rcStopTftpd"

#define PR_SSHD_RUN_SCRIPT "/etc/init.d/rcRunSshd"
#define PR_SSHD_STOP_SCRIPT "/etc/init.d/rcStopSshd"

#define PR_TELNETD_RUN_SCRIPT "/etc/init.d/rcRunTelnetd"
#define PR_TELNETD_STOP_SCRIPT "/etc/init.d/rcStopTelnetd"

#define PR_BOA_RUN_SCRIPT "/etc/init.d/rcRunBoa"
#define PR_BOA_STOP_SCRIPT "/etc/init.d/rcStopBoa"

#define PROGRAM_MAX_RECORD_LENGTH 80
#define PROGRAM_CONFIG_FILE CONFIG_DIR"/progs.conf"
#define PROGRAM_SCRIPT_FILE CONFIG_DIR"/rcProgs"
#define INETD_CONFIG_FILE "/flash/inetd.conf"
#define INETD_TELNET_ENTRY "telnet  stream  tcp  nowait  root  /bin/telnetd  telnetd"
#define INETD_TFTP_ENTRY "tftp  dgram  udp  wait  root  /bin/tftpd  -c -t 5 -s /tmp"
#define INETD_FTP_ENTRY	"ftp  stream  tcp  nowait  root  /bin/ftpd  -c /etc/ftpd.conf"
#define INETD_SSH_ENTRY "ssh  stream  tcp  nowait  root  /bin/sshd  -d /etc/dss_key -r /etc/rsa_key"

/****************************************************************************
 * definitions (types)
 ****************************************************************************/
#define PR_FLAGS_INCLUDED 1
#define PR_FLAGS_ACTIVATED 2
typedef struct {
    int flags;
    char *run;
    char *stop;
}ProgramField;

typedef enum {
  r_program_dhcp,
  r_program_dhcprelay,
  r_program_dhcpclient,
  r_program_named,
  r_program_gated,
  r_program_firewall,
  r_program_igmpp,
  r_program_snmp,
  r_program_wlan,
  r_program_ftpd,
  r_program_tftpd,
  r_program_sshd,
  r_program_telnetd,
  r_program_boa,
  r_program_max
}Program;

/* The following macros are for the interface. Programs before
	 PROGRAM_CONFIG_START in the programs array are not configurable. */
#define PROGRAM_CONFIG_START r_program_dhcp
#define PROGRAM_CONFIG_END r_program_max

/* The following definitions have to match with the values created in the makedisk script
   for the /etc/fsconfig file. */
#define PROGRAM_INCLUDED_DHCPD 1
#define PROGRAM_INCLUDED_DHCRELAY 2
#define PROGRAM_INCLUDED_DHCPCLIENT 4

/****************************************************************************
 * function prototypes
 ****************************************************************************/
extern void *program_get_user_data(void);
extern int program_load_config(void *);
extern int program_save_config(void *);
extern int pr_is_program_activated(int index);
extern int pr_is_program_included(int index);
extern void pr_activate(int index);
extern void pr_deactivate(int index);
extern int pr_run_program(unsigned int index, int run);
extern char *pr_get_program_run(int index);
extern int update_inetd_config(void);

#endif
