#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "webs.h"

/* ----------------------------------- */
#define SKIP_WHITE_SPACE(fp)  \
            while ( (c = fgetc(fp)) )   {  \
                if ( c != ' ' )   break;  \
            }

#define SKIP_THIS_LINE(fp)  \
            while ( (c = fgetc(fp)) )   {  \
                if ( c == '\n' || c == EOF )   break;  \
            }

/* ----------------------------------- */
/*  Read the content of config file    */
/* ----------------------------------- */
#define PARAM_LEN 31
#define VALUE_LEN 1000

FILE *cfg_fp = NULL;

static int read_next_config(char *var_name, char *value)
{
	char *p;
    int  i, c, limit;

	if ( cfg_fp == NULL )   return 0;

  start_read_config_line:
  	if ( (c = fgetc(cfg_fp)) == EOF )    {
  		fclose(cfg_fp);
  		cfg_fp = NULL;
  		return 0;
  	}

   	*var_name = *value = 0;
  	if ( c == ' ' )
  		SKIP_WHITE_SPACE(cfg_fp);
   	if ( c == '#' )   {
   		SKIP_THIS_LINE(cfg_fp);
   		return 1;
   	}
   	i = 0;
   	p = var_name;
   	limit = PARAM_LEN;
   	while ( c != EOF )  {
   		if ( c == '=' && value[0] == 0 )   {
   		    p[i] = 0;
   			i = 0;
    		p = value;
   			limit = VALUE_LEN;
    		SKIP_WHITE_SPACE(cfg_fp);
    	}
    	if ( c == '\n' || c == EOF )  break;
    	if ( i < limit-1 )  p[i++] = c, p[i] = 0;
    	c = fgetc(cfg_fp);
    }
	if ( p == var_name )  // no assignment
		goto start_read_config_line;

	i = strlen(var_name);
	while ( i && var_name[i-1] == ' ' )  var_name[--i] = 0;

	i = strlen(value);
	while ( i && value[i-1] == ' ' )  value[--i] = 0;
	if ( i>=2 && value[i-1] == '\"' && value[0] == '\"' )   {
		memmove(value, value+1, i-2);
		value[i-2] = 0;
	}
	return 1;
}

static int read_first_config(char *filename, char *var_name, char *value)
{
	cfg_fp = fopen(filename, "r");
	if ( cfg_fp == NULL )
	   	return 0;
    return read_next_config(var_name, value);
}

/*---------------------------------------------------*/
#define VALUE_SEG_LEN 		30
#define MAX_CONFIG_ITEM  	200

typedef struct  {
	char param[PARAM_LEN];
	char value[VALUE_SEG_LEN+1];
}  CONFIG_PARAM;

CONFIG_PARAM *param;
int param_num;

static int read_param_file(char *fn)
{
	int  go, i, j, len;
	char var_name[PARAM_LEN], value[VALUE_LEN], *p;

  // Read in all config....
    go = read_first_config(fn, var_name, value);
    while ( go && param_num < MAX_CONFIG_ITEM )   {
    	i = param_num++;
    	strcpy(param[i].param, var_name);
    	// copy value to param[]....
    	param[i].value[0] = 0;
    	p = value;
    	len = strlen(p);
    	while ( len )  {
    		j = len > VALUE_SEG_LEN ? VALUE_SEG_LEN : len;
    		strncpy(param[i].value, p, j);   param[i].value[j] = 0;
    		p += j;
    		len -= j;
    		if ( len )   {
    			if ( param_num == MAX_CONFIG_ITEM )  {
    				printf( "Parameter number exceeds %d!<br>\n", MAX_CONFIG_ITEM);
    				break;
    			}
    			i = param_num++;
    			strcpy(param[i].param, var_name);
    		}
    	}
        go = read_next_config(var_name, value);
    }
    return 1;
}

/*---------------------------------------------------*/
void get_param_value(char *cmd, char *val, int max_len)
{
	int i, len, total;
	char *p, *buffer = val;
	total = 0;

	for ( i = 0 ; i < param_num ; i++ )   {
	    if ( strcmp(param[i].param, cmd) )  // not what i looking for
	    	continue;
	    p = param[i].value;
	    len = strlen(p);
	    if ( total+len >= max_len-1 )   len = max_len-1-total;
	    strncpy(buffer, p, len);
	    buffer += len;
	    total += len;
     	while ( i+1 < param_num && !strcmp(param[i+1].param, cmd) )  {
    		p = param[++i].value;
    		len = strlen(p);
    		if ( total+len >= max_len-1 )   len = max_len-1-total;
    		strncpy(buffer, p, len);
    		buffer += len;
	    	total += len;
	    }
	    break;
	}
    *buffer = 0;
}
/*-----------------------------------------------*/
#define IF_WAN_CONFIG_FN "/var/etc/status"
//#define IP_WAN_CONFIG_FN "/tmp/ppp/wan_status"
//#define IF_WAN_CONFIG_FN "config.txt"
//#define IP_WAN_CONFIG_FN "config.txt"
char WAN_IF[32]="";
char WAN_IP[32]="";
int read_status(void)
{
	param_num = 0;
	param = (CONFIG_PARAM *)malloc(sizeof(CONFIG_PARAM)*MAX_CONFIG_ITEM);
	if( param == NULL ) return 0;
	read_param_file(IF_WAN_CONFIG_FN);
	get_param_value("WAN_IP", WAN_IP, sizeof(WAN_IP));
	//DPRINTF("wan_ip = %s \n", WAN_IP);
	read_param_file(IF_WAN_CONFIG_FN);
	get_param_value("WAN_IF", WAN_IF, sizeof(WAN_IF));
	//DPRINTF("wan_if = %s \n", WAN_IF);
	free(param);
	return 1;
}

