/*
 *  setvs.htm: Virtual Server Configuration
 */
#include "webs.h"
#include "setvs.h"
#include "pf.h"
#include "lan_conf.h"
#include "wan.h"
#include "../iptables-1.2.11/runFunc.h"

extern void run_task(char *cmd, ...);
extern char* read_file_para_value(char *filename, char *arg1,char *buf);

/*
 * Unmark if load config from outside functions 
 */
// #define __LoadConfig
int             tempindex;
#define selectlist_option(op) \
	for(tempindex=0;tempindex<op;tempindex++) \
		websWrite(wp,"<OPTION VALUE=%d>%s</OPTION>\n",tempindex+1,argv[tempindex]); \
	websWrite(wp,"<OPTION VALUE=%d SELECTED>%s</OPTION>\n",tempindex+1,argv[op]); \
	for(tempindex=op+1;tempindex<argc;tempindex++) \
		websWrite(wp,"<OPTION VALUE=%d>%s</OPTION>\n",tempindex+1,argv[tempindex])

static int aspVS1_fwNow2(void);
static char     tmpbufferstr[40];
static inline char *
inet_ntoa_b(struct in_addr inet_addr, char *dest_string)
{
    strcpy(dest_string, inet_ntoa(inet_addr));
    return dest_string;
}

#define VS_CFG_FILE       "/flash/vscfg.conf"
//#define VS_SCRIPT_FILE    "/flash/vsscript"
#define VS_CONF_FN_ADD    "/var/etc/vs.conf.add"
#define VS_CONF_FN_DEL    "/var/etc/vs.conf.del"
//#define DMZ_SCRIPT_FILE   "/flash/dmzscript"
#define DMZ_CONF_FN_ADD   "/var/etc/dmz.conf.add"
#define DMZ_CONF_FN_DEL   "/var/etc/dmz.conf.del"
extern char WAN_IP[];

int      DMZ_Enable;
struct in_addr DMZ_HostIP;

NAT_VIRTUAL_SERVER_T web_vs1[MAX_NAT_VIRTUAL_SERVER];
enum ClickOption VS_Func = Click_Func_CENCEL;
int      VS_Edit_Index = 0,
	VS_NextItem = 0,
	VS_TotalCount = 0;

#undef __LOADCONFIGOUTSIDE__
#ifdef __LoadConfig
#define __LOADCONFIGOUTSIDE__ extern
#else
#define __LOADCONFIGOUTSIDE__
#endif
/*
 * These functions can be exported 
 */
__LOADCONFIGOUTSIDE__ int Load_Virtual_Server_Config();
__LOADCONFIGOUTSIDE__ uint32_t SYSTEM_GetDmzHostIp();
__LOADCONFIGOUTSIDE__ int SYSTEM_GetDmzHostState();
__LOADCONFIGOUTSIDE__ void SYSTEM_SetDmzHostState(int);
__LOADCONFIGOUTSIDE__ void SYSTEM_SetDmzHostIp(uint32_t);
__LOADCONFIGOUTSIDE__ void DMZ_Host_reinit();
__LOADCONFIGOUTSIDE__ void SetVirtualServertoFile();
__LOADCONFIGOUTSIDE__ int CreateVSTableToScript();
/*
 * For Test~~ 
 */
void
SYSTEM_SetDmzHostState(int hoststate)
{
    DMZ_Enable = hoststate;
}

void
SYSTEM_SetDmzHostIp(uint32_t dmzhostip)
{
    DMZ_HostIP.s_addr = dmzhostip;
}

void
DMZ_Host_reinit()
{
#if 0
    UINT32          dmz_host_ip;

    dmz_host_ip = SYSTEM_GetDmzHostIp();

    if ((SYSTEM_GetDmzHostState() == DMZ_HOST_ENABLED)
	&& (dmz_host_ip != 0)) {
	DMZ_Host_Init(dmz_host_ip);
	SetDenyIncomingOff();
    } else {
	DMZ_Host_Init(0);
	SetDenyIncomingOn();
	NAT_CleanupAliasDataBySrcAddr(htonl(dmz_host_ip));
    }
#endif
}

int
SYSTEM_GetDmzHostState()
{
    return DMZ_Enable;
}

uint32_t
SYSTEM_GetDmzHostIp()
{
//    DPRINTF("DMZ_HostIP.s_addr:%x\n", DMZ_HostIP.s_addr);
    return DMZ_HostIP.s_addr;
}

void
SetVirtualServertoFile()
{
    int             i;
    FILE           *fp;

    fp = fopen(VS_CFG_FILE, "w");

    /*
     * Write DMT Enable and DMT Host Ip address 
     */
    // 1 2
    // |DMT Enable|DMT HostIP|
    fprintf(fp, "|%d|%08X|\n", DMZ_Enable,	// 1
	    DMZ_HostIP.s_addr	// 2
	);

    /*
     * save Virtual Server Entry 
     */
    for (i = 0; i < VS_TotalCount; i++) {
	// 1 2 3 4
	// |type|Port start|Port End|IP Address
	fprintf(fp, "|%d|%hu|%hu|%08X|%hu|\n", web_vs1[i].proto_type,	// 1
		web_vs1[i].public_port,	// 2
		web_vs1[i].end_public_port,	// 3
		web_vs1[i].host_ip.s_addr,	// 4
		web_vs1[i].private_port	// 5
	    );
    }
    fclose(fp);

    CreateVSTableToScript();
}

int
CreateVSTableToScript()
{
    int             i=0;
    FILE           *vsafp,*vsdfp;
    FILE           *dmzafp, *dmzdfp;
    NAT_VIRTUAL_SERVER_T *rule;
    char pt_str[10];
   // FILE           *fp;

    /*
     * Remove Old Iptables 
     */

    /*
     * Write new vsscript file 
     */
    if(VS_TotalCount)
    {
	vsdfp = fopen(VS_CONF_FN_DEL, "w");
	if(vsdfp == NULL){
		DPRINTF("can't open %s\n", VS_CONF_FN_DEL);
		return 0;
	}
	fprintf(vsdfp, "#!/bin/sh\n");
	fprintf(vsdfp, "###VIRTUAL SERVER ZONE###\n");
	fprintf(vsdfp, ". /var/etc/status\n");
	fprintf(vsdfp, "iptables -t nat -D PREROUTING -j VS_PRT\n");
	//fprintf(vsdfp, "iptables -D FORWARD -j VS_FWD\n");
	fprintf(vsdfp, "iptables -t nat -F VS_PRT\n");
	fprintf(vsdfp, "iptables -t nat -X VS_PRT\n");
	//fprintf(vsdfp, "iptables -F VS_FWD\n");
	//fprintf(vsdfp, "iptables -X VS_FWD\n");
	fclose(vsdfp);
	
	vsafp = fopen(VS_CONF_FN_ADD, "w");
	if(vsafp == NULL){
		DPRINTF("can't open %s\n", VS_CONF_FN_ADD);
		return 0;
	}
    	fprintf(vsafp, "#!/bin/sh\n");
	//fprintf(vsafp, "###VIRTUAL SERVER ZONE###\n");
	fprintf(vsafp, ". /var/etc/status\n");
	fprintf(vsafp, "iptables -t nat -N VS_PRT\n");
	//fprintf(vsafp, "iptables -N VS_FWD\n");
	fprintf(vsafp, "iptables -t nat -I PREROUTING -j VS_PRT\n");
	//fprintf(vsafp, "iptables -I FORWARD -j VS_FWD\n");

	//Robert add for lan access on VS 20050516
//	fprintf(vsafp, "iptables -t nat -A POSTROUTING -j MASQUERADE -o $WAN_IF\n");

	for (i = 0; i < VS_TotalCount; i++) {
		rule = &web_vs1[i];
		
		if (rule->proto_type == IPPROTO_IP)   // both for UDP and TCP
		{// We support "both" here.
			if (rule->private_port)
			{	
			fprintf(vsafp, "iptables -A VS_PRT -t nat -p tcp -d $WAN_IP --dport %d:%d -j DNAT --to-destination %s:%d\n",
		    		 rule->public_port, rule->end_public_port, inet_ntoa_b(rule->host_ip, tmpbufferstr), rule->private_port);
			fprintf(vsafp, "iptables -A VS_PRT -t nat -p udp -d $WAN_IP --dport %d:%d -j DNAT --to-destination %s:%d\n",
		    		 rule->public_port, rule->end_public_port, inet_ntoa_b(rule->host_ip, tmpbufferstr), rule->private_port);
#if 0
			fprintf(vsafp, "iptables -A VS_FWD -p tcp --dport %d -j ACCEPT\n",
				, rule->private_port);
#endif
			}
			else
			{
			fprintf(vsafp, 
				"iptables -A VS_PRT -t nat -p tcp -d $WAN_IP --dport %d:%d -j DNAT --to-destination %s\n",
		    		 rule->public_port, rule->end_public_port, inet_ntoa_b(rule->host_ip, tmpbufferstr));
			fprintf(vsafp, 
				"iptables -A VS_PRT -t nat -p udp -d $WAN_IP --dport %d:%d -j DNAT --to-destination %s\n",
		    		 rule->public_port, rule->end_public_port, inet_ntoa_b(rule->host_ip, tmpbufferstr));
#if 0
			fprintf(vsafp, 
				"iptables -A VS_FWD -p tcp --dport %d:%d -j ACCEPT\n",
		    		 rule->public_port, rule->end_public_port);
#endif
			}	
		}
		else
		{
			if (rule->proto_type == IPPROTO_UDP)   // UDP ?
				strcpy(pt_str,"udp");
			else
				strcpy(pt_str,"tcp");
			if (rule->private_port)
			{	
			fprintf(vsafp, "iptables -A VS_PRT -t nat -p %s -d $WAN_IP --dport %d:%d -j DNAT --to-destination %s:%d\n",
		    		pt_str, rule->public_port, rule->end_public_port, inet_ntoa_b(rule->host_ip, tmpbufferstr), rule->private_port);
#if 0
			fprintf(vsafp, "iptables -A VS_FWD -p %s --dport %d -j ACCEPT\n",
				pt_str, rule->private_port);
#endif
			}
			else
			{
			fprintf(vsafp, 
				"iptables -A VS_PRT -t nat -p %s -d $WAN_IP --dport %d:%d -j DNAT --to-destination %s\n",
		    		pt_str, rule->public_port, rule->end_public_port, inet_ntoa_b(rule->host_ip, tmpbufferstr));
#if 0
			fprintf(vsafp, 
				"iptables -A VS_FWD -p %s --dport %d:%d -j ACCEPT\n",
		    		pt_str, rule->public_port, rule->end_public_port);
#endif
			}	
		}

	}				// end for loop
	fclose(vsafp);

	chmod(VS_CONF_FN_ADD, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IWGRP | S_IXGRP | S_IROTH | S_IWOTH | S_IXOTH) ;
	chmod(VS_CONF_FN_DEL, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IWGRP | S_IXGRP | S_IROTH | S_IWOTH | S_IXOTH) ;
    }
    else
    {
	unlink(VS_CONF_FN_ADD);
	unlink(VS_CONF_FN_DEL);
    }
    
    // Write new DMZ script

    // $IPTABLES -A INPUT -p ALL -i $DMZ_IFACE -d $DMZ_IP -j ACCEPT
    // iptables -t nat -A PREROUTING --dst $INET_IP -p tcp --dport 80 -j
    // DNAT
    // --to-destination $HTTP_IP

    if (DMZ_Enable) {
	dmzafp = fopen(DMZ_CONF_FN_ADD, "w");

	if(dmzafp==NULL){
	     DPRINTF("can't open %s\n", DMZ_CONF_FN_ADD);
	     return 0;
	}
	fprintf(dmzafp, "#!/bin/sh\n");
	//fprintf(dmzafp, "######DMZ ZONE######\n");
	fprintf(dmzafp, ". /var/etc/status\n");
#if 1
	fprintf(dmzafp,"iptables -t nat -N DMZ_PRT\n");
	//fprintf(dmzafp,"iptables -N DMZ_FWD\n");
	fprintf(dmzafp,"iptables -t nat -I PREROUTING -j DMZ_PRT\n");
	//fprintf(dmzafp,"iptables -I FORWARD -j DMZ_FWD\n");
	//Robert add for lan access on VS 20050516
//	fprintf(vsafp, "iptables -t nat -A POSTROUTING -j MASQUERADE -o $WAN_IF\n");
	//for lan access
	fprintf(dmzafp,"iptables -A DMZ_PRT -t nat -d $WAN_IP -j DNAT --to-destination %s\n",inet_ntoa_b(DMZ_HostIP, tmpbufferstr));
	//fprintf(dmzafp,"iptables -A DMZ_FWD -i $WAN_IF -o $LAN_IF -m state --state NEW -d %s -j ACCEPT\n",inet_ntoa_b(DMZ_HostIP, tmpbufferstr));
#else
	fprintf(dmzafp,"iptables -I PREROUTING -t nat -i $WAN_IF -d $WAN_IP -j DNAT --to-destination %s\n",inet_ntoa_b(DMZ_HostIP, tmpbufferstr));
	//fprintf(dmzafp,"iptables -I FORWARD -i $WAN_IF -o $LAN_IF -m state --state NEW -d %s -j ACCEPT\n",inet_ntoa_b(DMZ_HostIP, tmpbufferstr));
#endif
	
	fclose(dmzafp);

	dmzdfp = fopen(DMZ_CONF_FN_DEL, "w");
	if(dmzdfp==NULL){
	     DPRINTF("can't open %s\n", DMZ_CONF_FN_DEL);
	     return 0;
	}
	fprintf(dmzdfp, "#!/bin/sh\n");
	//fprintf(dmzdfp, "######DMZ ZONE######\n");
	fprintf(dmzdfp, ". /var/etc/status\n");
#if 1
	fprintf(dmzdfp, "iptables -t nat -F DMZ_PRT\n");
	//fprintf(dmzdfp, "iptables -F DMZ_FWD\n");
	fprintf(dmzdfp, "iptables -t nat -D PREROUTING -j DMZ_PRT");
	//fprintf(dmzdfp, "iptables -D FORWARD -j DMZ_FWD\n");
	fprintf(dmzdfp, "iptables -t nat -X DMZ_PRT\n");
	//fprintf(dmzdfp, "iptables -X DMZ_FWD\n");
#else
	fprintf(dmzdfp,"iptables -D PREROUTING -t nat -i $WAN_IF -d $WAN_IP -j DNAT --to-destination %s\n",inet_ntoa_b(DMZ_HostIP, tmpbufferstr));
	//fprintf(dmzdfp,"iptables -D FORWARD -i $WAN_IF -o $LAN_IF -m state --state NEW -d %s -j ACCEPT\n",inet_ntoa_b(DMZ_HostIP, tmpbufferstr));
#endif	
	fclose(dmzdfp);
	   chmod(DMZ_CONF_FN_ADD, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IWGRP | S_IXGRP | S_IROTH | S_IWOTH | S_IXOTH) ;
	   chmod(DMZ_CONF_FN_DEL, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IWGRP | S_IXGRP | S_IROTH | S_IWOTH | S_IXOTH) ;	
    }else{
	 unlink(DMZ_CONF_FN_DEL);
	 unlink(DMZ_CONF_FN_ADD);
    }
    
    return 0;
}

/*
 * LOCAL FUNCTIONS 
 */
static void     Init_Virtual_Server();
static void     DefineVirtualServer();

#define inet_ntoa_b(inet_addr,dest_string) strcpy(dest_string, inet_ntoa(inet_addr))

/*
 * The Entry point function 
 */
void
DefineWebFunctions_VirtualServer()
{
    /*
     * Initialize the structure 
     */
    Init_Virtual_Server();
    /*
     * Load configuration 
     */
    Load_Virtual_Server_Config();
    CreateVSTableToScript();
    /*
     * Goahead setting 
     */
    DefineVirtualServer();
}

int
Load_Virtual_Server_Config()
{
    int             i;
    FILE           *fp;
    VS_TotalCount = 0;
    VS_NextItem = 1;

    fp = fopen(VS_CFG_FILE, "r");
    if (!fp) {
	return -1;
    }

    /*
     * read DMT Enable and DMT Host Ip address 
     */
    // 1 2
    // |DMT Enable|DMT HostIP|
    if (fscanf(fp, "|%d|%08X|\n", (int *) &DMZ_Enable,	// 1
	       &DMZ_HostIP.s_addr	// 2
	) == -1){
	fclose(fp);
	return -1;
    }

    /*
     * read Virtual Server Entry 
     */
    for (i = 0; i < MAX_NAT_VIRTUAL_SERVER; i++) {
	// 1 2 3 4
	// |type|Port start|Port End|IP Address
	if (fscanf(fp, "|%d|%hu|%hu|%08X|%hu|\n", (int *) &web_vs1[i].proto_type,	// 1
		   &web_vs1[i].public_port,	// 2
		   &web_vs1[i].end_public_port,	// 3
		   &web_vs1[i].host_ip.s_addr,	// 4
		   &web_vs1[i].private_port	// 5
	    ) == -1)
	    break;
	VS_TotalCount++;
	VS_NextItem = VS_TotalCount + 1;
    }
    fclose(fp);
    return 1;
}

static void
Init_Virtual_Server()
{
    VS_TotalCount = 0;
    VS_NextItem = 0;
    DMZ_Enable = 0;
    DMZ_HostIP.s_addr = 0;
}

/*
 *
 * Virtual Server Functions
 *
 */

// setvs.asp
static int      appGetVirtualServerTable(int eid, webs_t wp, int argc,
					 char_t ** argv);
static int      DMZ_IP_addr(int eid, webs_t wp, int argc, char_t ** argv);
static int      DMZ_state(int eid, webs_t wp, int argc, char_t ** argv);
// static void form_addvs (webs_t wp, char_t * path, char_t * query);
static void     formVirtualServer(webs_t wp, char_t * path,
				  char_t * query);
void            SetVirtualServertoFile();
// setvs1.asp
static int      appVS1_ProtocolType(int eid, webs_t wp, int argc,
				    char_t ** argv);
static int      appVS1_StartPort(int eid, webs_t wp, int argc,
				 char_t ** argv);
static int      appVS1_EndPort(int eid, webs_t wp, int argc,
			       char_t ** argv);
static int      appVS1_IP(int eid, webs_t wp, int argc, char_t ** argv);
static int      appVS1_Port(int eid, webs_t wp, int argc, char_t ** argv);
static int      appVS1_Now(int eid, webs_t wp, int argc, char_t ** argv);
static void     formVS1_Edit(webs_t wp, char_t * path, char_t * query);

/*
 *
 * Define Virtual Server Functions
 *
 */

void
DefineVirtualServer()
{
    // setvs.asp
    websAspDefine(T("appGetVirtualServerTable"), appGetVirtualServerTable);


    websAspDefine(T("DMZ_IP_addr"), DMZ_IP_addr);
    websAspDefine(T("DMZ_state"), DMZ_state);
    websFormDefine(T("setvs"), formVirtualServer);

    // setvs1.asp
    websAspDefine(T("VS_ProtocolType"), appVS1_ProtocolType);
    websAspDefine(T("VS_StartPort"), appVS1_StartPort);
    websAspDefine(T("VS_EndPort"), appVS1_EndPort);
    websAspDefine(T("VS_IP"), appVS1_IP);
    websAspDefine(T("VS_Port"), appVS1_Port);
    websFormDefine(T("setvs1"), formVS1_Edit);
    websAspDefine(T("vs1Now"), appVS1_Now);
}

static int
appGetVirtualServerTable(int eid, webs_t wp, int argc, char_t ** argv)
{
    int             nBytes = 0,
	i;
	char buf1[32];
	char pp_str[6];
	char pt_str[10];
    char_t         *TableFmt =
	"<tr><td><input type=\"radio\" name=\"vsIdx\" value=\"%d\" %s></td>"
	"<td>&nbsp;%d&nbsp;</td>"
	"<td>&nbsp;%s&nbsp;</td>"
	"<td>&nbsp;%d&nbsp;</td>"
	"<td>&nbsp;%d&nbsp;</td>"
	"<td>&nbsp;%s&nbsp;</td>"
	"<td>&nbsp;%s&nbsp;</td>"
	"</tr>\n";

    // nBytes = websWrite(wp, T(TableFmt), 1, 1,
    // "65535", "65535", "1.1.1.1",);

    if (VS_NextItem == 0) {
	/*
	 * Find out how many entrys are there ! 
	 */
	// while (NAT_VS_GetEntry (entryCounter, &web_vs1[entryCounter])
	// == 1)
	// entryCounter++;
	/*
	 * Load configuration 
	 */
	Load_Virtual_Server_Config();
	// VS_TotalCount = entryCounter;
	// VS_NextItem = entryCounter + 1;
    }

    for (i = 0; i < VS_TotalCount; i++) {
	    if(web_vs1[i].private_port)
		    sprintf(pp_str,"%d",web_vs1[i].private_port);
	    else
		    sprintf(pp_str," ");

	    if(web_vs1[i].proto_type == IPPROTO_TCP)
		strcpy(pt_str,"TCP");
	    else if(web_vs1[i].proto_type == IPPROTO_UDP)
		strcpy(pt_str,"UDP");
	    else
		strcpy(pt_str,"BOTH");
	    
	nBytes += websWrite(wp, T(TableFmt),
			    // Radio value, int
			    i + 1,
			    // checked or not
			    (i == aspVS1_fwNow2() ? "checked" : ""),
			    // 0 Rule's number, int
			    i + 1,
			    // 1 type, string
			    pt_str,
			    // 2 Service Port start, int
			    web_vs1[i].public_port,
			    // 3 Service Port end, int
			    web_vs1[i].end_public_port,
			    // 4 IP address, string
			     inet_ntoa_b(web_vs1[i].host_ip,buf1),
			    // 5 Mapped private port int
			    pp_str);
    }

    return nBytes;
}



#if 1
static int
DMZ_IP_addr(int eid, webs_t wp, int argc, char_t ** argv)
{
    char            buf[24];
    struct in_addr  addr;
    addr.s_addr = htonl(SYSTEM_GetDmzHostIp());
    //DPRINTF("addr.s_addr:%d\n", addr.s_addr);
    if (addr.s_addr != 0) {
	inet_ntoa_b(addr, buf);
	//DPRINTF("addr:%s\n", buf);

	return websWrite(wp, T("%s"), buf);
    } else
	return websWrite(wp, T(""));
}
#endif

#if 1
static int
DMZ_state(int eid, webs_t wp, int argc, char_t ** argv)
{
    char           *szCheck[] = { " ", "checked" };
    return websWrite(wp, T("%s"), szCheck[SYSTEM_GetDmzHostState()]);
}
#endif

#if 1
static void
formVirtualServer(webs_t wp, char_t * path, char_t * query)
{
    char           *pStr;
    unsigned int          dmz_host_ip;
    char_t         *vsFUNC,
                   *vsIdx;
    int             index;
    //FILE           *fp;


    vsFUNC = websGetVar(wp, T("vsFUNC"), "");
    VS_Func = atoi(vsFUNC);
    vsIdx = websGetVar(wp, T("vsIdx"), "");
    index = atoi(vsIdx);

    if (index)
	VS_Edit_Index = index - 1;

    websHeader(wp);
    switch (VS_Func) {
    case Click_Func_ADD:
	// printf("formStaticRoute: Click Add button\n");
	if (VS_NextItem <= MAX_NAT_VIRTUAL_SERVER) {
	    websWrite(wp,
		      T
		      ("<script language=\"JavaScript\">location.replace(\"/setvs1.asp\");</script>"));
	    websFooter(wp);
	    websDone(wp, 200);
	} else {
	    websWrite(wp,
		      T
		      ("<script language=\"JavaScript\">location.replace(\"/setvs.asp\");</script>"));
	    websFooter(wp);
	    websDone(wp, 200);
	}
	break;
    case Click_Func_EDIT:
	if (index > 0) {
	    websWrite(wp,
		      T
		      ("<script language=\"JavaScript\">location.replace(\"/setvs1.asp\");</script>"));
	    websFooter(wp);
	    websDone(wp, 200);
	} else			// no select an item
	{
	    websWrite(wp,
		      T
		      ("<script language=\"JavaScript\">location.replace(\"/setvs.asp\");</script>"));
	    websFooter(wp);
	    websDone(wp, 200);
	}
	break;
    case Click_Func_DEL:
	if ((index > 0) && (index < VS_NextItem)) {
	    memmove(&web_vs1[VS_Edit_Index], &web_vs1[VS_Edit_Index + 1],
		    sizeof(web_vs1[0]) * (MAX_NAT_VIRTUAL_SERVER -
					  VS_Edit_Index - 1));
	    --VS_NextItem;
	    --VS_TotalCount;
	    websWrite(wp,
		      T
		      ("<script language=\"JavaScript\">location.replace(\"/setvs.asp\");</script>"));
	    websFooter(wp);
	    websDone(wp, 200);
	} else			// no select an item
	{
	    websWrite(wp,
		      T
		      ("<script language=\"JavaScript\">location.replace(\"/setvs.asp\");</script>"));
	    websFooter(wp);
	    websDone(wp, 200);
	}
	break;
    case Click_Func_APPLY:

	VS_NextItem = 0;
	// SetVirtualServertoFile ();

	// DMZ
	pStr = websGetVar(wp, T("DMZ_state"), "0");
	SYSTEM_SetDmzHostState(atoi(pStr));
	pStr = websGetVar(wp, T("DMZ_IP_addr"), "");
	if (strlen(pStr) == 0) {
	    DMZ_HostIP.s_addr = 0;
	} else {
	    dmz_host_ip = ntohl(inet_addr(pStr));
	    SYSTEM_SetDmzHostIp(dmz_host_ip);
	}


	{
		char 	funcNum[4];
		char str_wanip[20];
			char buf1[64];

		sprintf(funcNum,"%d",FUNCTION_NUM_VIRTUAL_SERVER_DMZ);
		run_task("/usr/sbin/runFunc","/usr/sbin/runFunc", "stop",funcNum,NULL);
		SetVirtualServertoFile();	
		if((DMZ_Enable && SYSTEM_GetDmzHostIp())||(VS_TotalCount))
		{// we then need to start the functions.
			strcpy(str_wanip,(char *)read_file_para_value(VAR_SYS_STATUS_FILE,"WAN_IP=",buf1));
			if(isValidIP(str_wanip))
				run_task("/usr/sbin/runFunc","/usr/sbin/runFunc", "start",funcNum,NULL);
			//else running when wan_ip ready.
		}
	}
	websWrite(wp,
		  T
		  ("<script language=\"JavaScript\">location.replace(\"/setvs.asp\");</script>"));
	websFooter(wp);
	websDone(wp, 200);
	break;
    case Click_Func_CENCEL:	// Get flash data
	VS_NextItem = 0;
	websWrite(wp,
		  T
		  ("<script language=\"JavaScript\">location.replace(\"/setvs.asp\");</script>"));
	websFooter(wp);
	websDone(wp, 200);
	break;
    default:
	websRedirect(wp, "/setvs.asp");
    }

}
#endif

#if 1
static int
appVS1_ProtocolType(int eid, webs_t wp, int argc, char_t ** argv)
{
    int             proto_type=1;//default is TCP

    //proto_type =
//	(web_vs1[VS_Edit_Index].proto_type == IPPROTO_TCP ? 1 : 2);
    if(VS_Func!=Click_Func_ADD)
    {	
	    if(web_vs1[VS_Edit_Index].proto_type == IPPROTO_IP)
		proto_type=3;
	    else if(web_vs1[VS_Edit_Index].proto_type == IPPROTO_UDP)
		proto_type=2;
	    else
		proto_type=1;
    }    
   // printf("get protocol type=%d\n", proto_type);
    proto_type -= 1;
    selectlist_option(proto_type);
    return 0;
}
#endif

#if 1
static int
appVS1_StartPort(int eid, webs_t wp, int argc, char_t ** argv)
{
    if (VS_Func == Click_Func_EDIT) {
	return websWrite(wp, T("%d"), web_vs1[VS_Edit_Index].public_port);
    } else {
	return websWrite(wp, T(""));
    }
}
#endif

static int aspVS1_fwNow2(void)
{
	switch(VS_Func)
	{
		case Click_Func_EDIT:
		return VS_Edit_Index;
		default:
		return VS_TotalCount-1;
	}
}

#if 1
static int
appVS1_EndPort(int eid, webs_t wp, int argc, char_t ** argv)
{
    if (VS_Func == Click_Func_EDIT) {
	return websWrite(wp, T("%d"),
			 web_vs1[VS_Edit_Index].end_public_port);
    } else {
	return websWrite(wp, T(""));
    }
}
#endif

#if 1
static int
appVS1_IP(int eid, webs_t wp, int argc, char_t ** argv)
{
	char buf1[32];
    if (VS_Func == Click_Func_EDIT) {
	return websWrite(wp, T("%s"),
			 inet_ntoa_b(web_vs1[VS_Edit_Index].host_ip,buf1));
    } else {
	return websWrite(wp, T(""));
    }
}
#endif

#if 1
static int
appVS1_Port(int eid, webs_t wp, int argc, char_t ** argv)
{
   if (VS_Func == Click_Func_EDIT) {
	return websWrite(wp, T("%d"),
			 web_vs1[VS_Edit_Index].private_port);
    } else {
	return websWrite(wp, T(""));
    }
}
#endif

#if 1
static int
appVS1_Now(int eid, webs_t wp, int argc, char_t ** argv)
{
    switch (VS_Func) {
    case Click_Func_ADD:
	return websWrite(wp, T("%d"), VS_NextItem);
    case Click_Func_EDIT:
	return websWrite(wp, T("%d"), VS_Edit_Index + 1);
    default:
	return websWrite(wp, T("%d"), VS_NextItem);
    }

}
#endif

#if 1
static void
formVS1_Edit(webs_t wp, char_t * path, char_t * query)
{
    char_t         *vs1ProtType;	// Protocol Type
    char_t         *vs1StartPort;	// Start Port
    char_t         *vs1EndPort;	// End Port
    char_t         *vs1IP;	// IP addrss
    char_t         *vs1Port;	// The internal private port for port mapping
    char_t         *vs1Now;
    char_t         *vsFUNC;
    int             item;
    int             proto_type;

    vsFUNC = websGetVar(wp, T("vsFUNC"), "0");
    // 0, Rule number
    vs1Now = websGetVar(wp, T("vs1Now"), "0");
    // 1, Protocol Type
    vs1ProtType = websGetVar(wp, T("VS_ProtocolType"), "1");
    // 2, Start Port                                    
    vs1StartPort = websGetVar(wp, T("VS_StartPort"), "0");
    // 3, End Port
    vs1EndPort = websGetVar(wp, T("VS_EndPort"), "0");
    // 4, IP address
    vs1IP = websGetVar(wp, T("VS_IP"), "0");
    // 5, Private port mapping
    vs1Port = websGetVar(wp, T("VS_Port"), "0");

    websHeader(wp);
    if (atoi(vsFUNC) == 6)
	websWrite(wp,
		  T
		  ("<script language=\"JavaScript\">location.replace(\"/setvs1.asp\");</script>"));
    else {
	item = atoi(vs1Now);
	// printf("formeVS1_Edit: item number=%d\n", item);

	switch (VS_Func) {
	case Click_Func_ADD:
	    if (item <= MAX_NAT_VIRTUAL_SERVER) {
		++VS_NextItem;
		++VS_TotalCount;
	    }
	    break;
	case Click_Func_EDIT:
	    break;
	case Click_Func_CENCEL:
	    break;
	case Click_Func_APPLY:
	    break;
	case Click_Func_DEL:
	    break;
	}


	--item;
	// web_vs1[item].host_ip = StringToIpAddress (vs1IP);
	web_vs1[item].host_ip.s_addr = inet_addr(vs1IP);
	web_vs1[item].private_port = atoi(vs1Port);
	web_vs1[item].public_port = atoi(vs1StartPort);
	web_vs1[item].end_public_port = atoi(vs1EndPort);
	proto_type = atoi(vs1ProtType);
	    if(proto_type == 1)
		web_vs1[item].proto_type = IPPROTO_TCP;
	    else if(proto_type == 2)
		web_vs1[item].proto_type = IPPROTO_UDP;
	    else
		web_vs1[item].proto_type = IPPROTO_IP;
	// printf("port=%d to %d, protocol=%d (%d)\n", 
	// web_vs1[item].public_port, web_vs1[item].end_public_port, 
	// web_vs1[item].proto_type, proto_type);
	websWrite(wp,
		  T
		  ("<script language=\"JavaScript\">location.replace(\"/setvs.asp\");</script>"));
    }
    websFooter(wp);
    websDone(wp, 200);

}
#endif
