/******************************************************************************
*
*  (C) Copyright 2004 Trend Micro, Inc.
*
*  All Rights Reserved.
*  This program is an unpublished copyrighted work which is proprietary
*  to Trend Micro, Inc. and contains confidential information that is not
*  to be reproduced or disclosed to any other person or entity without
*  prior written consent from Trend Micro, Inc. in each and every instance.
*
*  WARNING:  Unauthorized reproduction of this program as well as
*  unauthorized preparation of derivative works based upon the
*  program or distribution of copies by sale, rental, lease or
*  lending are violations of federal copyright laws and state trade
*  secret laws, punishable by civil and criminal penalties.
*
*****************************************************************************/

/**
*  @file tmGetReport.c
*  @brief TMOG Get Report CGI
*/

/*
*  Change Log
*  2004-03-01 Henry Lin    Initialization version
*/

#include <stdio.h>
#include <string.h>
#include "webs.h"
#include "tmURLFilter.h"
#include "time.h"
#include "tmGetReport.h"
#include "tmCGILib.h"
#include "tmClientInfo.h"
#include "tmException.h"
//#include "tmCGICommon.h"
#include "tmcfg.h"
#include "tmURLFilterConf.h"
#include "tmURLFilterStatistic.h"
#include "tmCGIErrMsg.h"
#include "tmssConf.h"

extern int run_script(char *filename, char *arg1, char *arg2, char *arg3);
//static const char ModuleName[] = "tmURLFilterStatisticInfoCGI";

const char *UrlFilterCategoryListCGI[HV_DEFINED_CAT_NUM] =
{
    URLFILTER_CATEGORY_PORNOGRAPHY,
    URLFILTER_CATEGORY_ALCOHOL_TABACCO,
    URLFILTER_CATEGORY_ILLEGAL_QUEST,
    URLFILTER_CATEGORY_GAMBLING,
    URLFILTER_CATEGORY_VIOLENCE,
    URLFILTER_CATEGORY_ABORTION,
    URLFILTER_CATEGORY_ILLEGAL_DRUGS,
    "Unused"
};
//#define DEBUG // Turn on this line to show from browser

extern const char *MsgVA[];

/*  Given a string like "ms-001:blaster;ms-002:doom;trojan;"
 *  Format it like:
 *    M_id_1=ms-001
 *    M_n_1=blaster  n means name
 *    M_id_2=ms-002
 *    M_n_2=doom
 *    M_n_3=trojan
 *
 *    @param str: the input string
 *    @param idx: the starting index
 *    @param title: the prefix string
 *    @return: number of records in str
 */
int DumpOneVACGI(webs_t wp,const char *str, int idx, const char *title)
{
   int i;
   int iEnd = -1;      /* point to end of this record */
   int iAssign = -1;   /* point to '=' of this record if any */
   char c;
   char *pBuf;

   if (str[0] == 0)
       return 0;

   for (i = 0; ; i++)
   {
        c = str[i];
        if (c == ';' || c == 0)
        {
            iEnd = i;
            break;
        }
        else if (c == ':')
        {
            iAssign = i;
        }
   }

   if (iEnd > 0)
   {
      pBuf = (char *)malloc(iEnd+1);
      strncpy(pBuf, str, iEnd);
      pBuf[iEnd] = 0;
      if (iAssign > 0)
      {
          pBuf[iAssign] = 0;
          websWrite (wp, T("%s_id_%u=%s\r\n"), title, idx, pBuf);
          websWrite (wp, T("%s_n_%u=%s\r\n"), title, idx, pBuf+iAssign+1);
      }
      else
      {
          websWrite (wp, T("%s_n_%u=%s\r\n"), title, idx, pBuf);
      }
      free(pBuf);

      /* recursive */
      if (str[iEnd] != 0)
          return 1 + DumpOneVACGI(wp,str+iEnd+1, idx+1, title);
      else
          return 1;
   }

   return 0;
}

/*
 * count => c
 * real_count => rc
 */
static void DumpClientAllVA(webs_t wp,ClientInfo *pClient)
{
    unsigned int i, total;

    for (i = 0; i < VATYPE_SIZE; i++)
    {
        total = DumpOneVACGI(wp,CVA(pClient)[i], 0, MsgVA[i]);
        /* this AP_c is not used in Windows side */
        /* it should be deleted */
        websWrite (wp, T("%s_c=%u\r\n"), MsgVA[i], total);
        switch (i)
        {
            case VATYPE_MSPATCH:
                websWrite (wp, T("%s_rc=%u\r\n"), MsgVA[i], CVAMSPATCHCOUNT(pClient));
                break;
            case VATYPE_TROJAN:
                websWrite (wp, T("%s_rc=%u\r\n"), MsgVA[i], CVATROJANCOUNT(pClient));
                break;
            case VATYPE_SPYWARE:
                websWrite (wp, T("%s_rc=%u\r\n"), MsgVA[i], CVASPYWARECOUNT(pClient));
                break;
            case VATYPE_NETSHARE:
                websWrite (wp, T("%s_rc=%u\r\n"), MsgVA[i], CVANETSHARECOUNT(pClient));
                break;
            case VATYPE_AVSOFT:
                websWrite (wp, T("%s_rc=%u\r\n"), MsgVA[i], CVAAVSOFTCOUNT(pClient));
                break;
            /* this AP_rc is not used in Windows side */
            /* it should be deleted */
            /* case VATYPE_AVPATTERN:  
                printf("%s_rc=\r\n", MsgVA[i]);
                break;*/
            default:
                break;
        }
    }
    websWrite (wp, T("ScanDate=%lu\r\n"), CVASCANDATE(pClient));
}


#define ALIGNMENT       (4)

void tmGetReportCGI( webs_t wp,int nContentLen)
{
    url_filter_statistic_t Report[HV_DEFINED_CAT_NUM+ALIGNMENT];
    url_filter_statistic_t *pAlignedReport = (Report+ALIGNMENT-((long)Report%ALIGNMENT));
    int i;
    time_t StartTime;
    char *action=NULL;
 
    //DPRINTF("tm_GetReport entering tm_GetReport.cgi\n");
    
    tmci_InitClientTable();

    //tmDecoder();

    //tmContentType("text/plain\r\n\r\n");
     websWrite(wp, T("HTTP/1.0 200 OK\n"));
   websWrite (wp, T("Content-type: text/html\r\n\r\n%c%c"),10,10);

    //check action type
  /*  action = tmValueDefault("","%s", CGI_PARAM_ACTION_NAME);
    if(nContentLen<10)
    {
	    // at least "action=get"
	    //DPRINTF("nContentLen(%d) to small(<10)\n",nContentLen);
    }
    else {
	    action=strstr(wp->query,"action=");
	    if(action)
		    action+=7;//skip "action="
    }  */
    action=websGetVar(wp, T("action"), "");

    if (strncmp (action, CGI_PARAM_ACTION_VALUE_GET,3) == 0)
    {
       // DPRINTF("tm_GetReport receive get command\n");

        tmGetURLStatisticCount(pAlignedReport, sizeof(url_filter_statistic_t)*HV_DEFINED_CAT_NUM, &StartTime);

        websWrite (wp, T("[%s]\r\n"),MODULE_URL_FILTER_STATISTIC);
        websWrite (wp, T("%s=%d\r\n"),PARAM_CATEGORY_COUNT, HV_DEFINED_CAT_NUM - 1);
        websWrite (wp, T("%s=%lu\r\n\r\n"),PARAM_START_COUNT_SYNCE,StartTime);

        for (i = 0; i < HV_DEFINED_CAT_NUM - 1; i++)
        {
            websWrite (wp, T("[%s%d]\r\n"),MODULE_URL_FILTER_STATISTIC_CATEGORY_, i);
            websWrite (wp, T("%s=%s\r\n"), PARAM_CATEGORY_NAME,        UrlFilterCategoryListCGI[i]);
            websWrite (wp, T("%s=%d\r\n"), PARAM_ACCESS_TIMES,         pAlignedReport[i].nAccessCount);
            websWrite (wp, T("%s=%lu\r\n"),PARAM_LAST_ACCESS_TIME,     pAlignedReport[i].LastAccessTime);
            websWrite (wp, T("%s=%d\r\n"), PARAM_BLOCK_TIMES,          pAlignedReport[i].nBlockedCount);
            websWrite (wp, T("%s=%lu\r\n\r\n"),PARAM_LAST_BLOCK_TIME,  pAlignedReport[i].LastBlockedTime);
        }

        websWrite (wp, T("\r\n"));

        /* START OF VA REPORT */
        {
            unsigned int size, idx;
            ClientInfo   *pBuf, *pClient;

            size = tmci_SizeClient();
	    if(size<=0)
	    {
		    //DPRINTF("client size is 0\n");
	    }
	    else
	    {		    
		    pBuf = malloc(size * sizeof(ClientInfo));
		    tmci_GetClientsFrom(pBuf, &size, 0);
		    for (idx = 0; idx < size; idx++)
		    {
			    pClient = pBuf + idx;
			    websWrite (wp, T("\r\n[Client_Report_%u]\r\n"), idx);
			    websWrite (wp, T("IP=%u.%u.%u.%u\r\n"), NIPQUAD(CIP(pClient)));
			    websWrite (wp, T("HostName=%s\r\n"), CHOSTNAME(pClient));
			    websWrite (wp, T("ISEXPREPORT=%d\r\n"), CISEXPREPORT(pClient));
			    DumpClientAllVA(wp,pClient);
		    }
		    free(pBuf);
	    }
            websWrite (wp, T("\r\n[Network Report]"));
            websWrite (wp, T("\r\nClientCount=%u\r\n\r\n"), size);
        }
        /* END OF VA REPORT */

        websWrite (wp, T("\r\n"));
    }
    else if (strcmp (action, CGI_PARAM_ACTION_VALUE_PUT) == 0)
    {
        int i;
        const char *str;
        struct client_id id;
        struct client_va va;
        ClientInfo info;
        
        //DPRINTF("tm_GetReport receive put command\n");

        memset(&id, 0, sizeof(id));
        memset(&va, 0, sizeof(va));

	str=websGetVar(wp, T("IP"), "");
       /*str  = tmValueDefault("","%s", "IP");
        szTemp = strdup(str); 
        tmURLdecode(szTemp); */
        inet_pton(AF_INET, str, &(id.ip));
       // free(szTemp); 
	

        //str  = tmValueDefault("","%s", "HostName");
 	str=websGetVar(wp, T("HostName"), "");
        strncpy(id.hostname, str, HOSTNAME_SIZE-1);
       // tmURLdecode(id.hostname);

        //str  = tmValueDefault("","%s", "MAC");
        //szTemp = strdup(str);
        //tmURLdecode(szTemp);
 	str=websGetVar(wp, T("MAC"), "");
        StrToMac(str, id.mac);
        //free(szTemp);
        
        memset(&info, 0, sizeof(info));
        
        if (tmci_GetClient(id.mac, &info) < 0)
        {
            //DPRINTF("there is no such client record in client information table, just return\n");
        }
        else
        {                  
           /* str  = tmValueDefault("","%s", "MSPATCH_COUNT");
            szTemp = strdup(str);
            tmURLdecode(szTemp); */
	    str=websGetVar(wp, T("MSPATCH_COUNT"), "");
            va.nMSPatchCount = atoi(str);
            //free(szTemp);
            
            //str  = tmValueDefault("","%s", "TROJAN_COUNT");
            //szTemp = strdup(str);
            //tmURLdecode(szTemp);
	    str=websGetVar(wp, T("TROJAN_COUNT"), "");
            va.nTrojanCount = atoi(str);
            //free(szTemp);
            
            //str  = tmValueDefault("","%s", "NETSHARE_COUNT");
            //szTemp = strdup(str);
            //tmURLdecode(szTemp);
 	    str=websGetVar(wp, T("NETSHARE_COUNT"), "");
           va.nNetshareCount = atoi(str);
            //free(szTemp);
            
            //str  = tmValueDefault("","%s", "AVSOFT_COUNT");
            //szTemp = strdup(str);
            //tmURLdecode(szTemp);
	    str=websGetVar(wp, T("AVSOFT_COUNT"), "");
            va.nAVSoftCount = atoi(str);
            //free(szTemp);
    
            //str  = tmValueDefault("","%s", "SPYWARE_COUNT");
            //szTemp = strdup(str);
            //tmURLdecode(szTemp);
	    str=websGetVar(wp, T("SPYWARE_COUNT"), "");
            va.nSpywareCount = atoi(str);
            //free(szTemp);

            //str  = tmValueDefault("","%s", "SCAN_DATE");
            //szTemp = strdup(str);
            //tmURLdecode(szTemp);
	    str=websGetVar(wp, T("SCAN_DATE"), "");
            va.nScanDate = atol(str);
            //free(szTemp);
    
            for (i = 0; i < VATYPE_SIZE; i++)
            {
                 int j;
		 char msgVa[3];
    
                 //str  = tmValueDefault("","%s", MsgVA[i]);
		 sprintf(msgVa,"%s",MsgVA[i]);
		  str=websGetVar(wp, T(msgVa), "");

                 strncpy(va.va[i], str, VADETAIL_SIZE-1);
                 //tmURLdecode(va.va[i]);
    
                 /* string must be end of ';' */
                 for (j = strlen(va.va[i]) - 1; j >= 0; j--)
                 {
                      if (va.va[i][j] == ';')
                          break;
                      else
                          va.va[i][j] = 0;
                 }
            }
    
            if (tmci_SetVA(&id, &va) < 0)
                websWrite (wp, T("SetVA FAIL\r\n\r\n"));
            else
            {
                websWrite (wp, T("SetVA SUCCESS\r\n\r\n"));
    
                /* Update current setting to config file */
               tmci_WriteClient(THPD_CONF_PATH); 
			   run_script("/etc/scripts/flash","write",0,0);
            } 
        }
    }
    else if (strcmp (action, CGI_PARAM_ACTION_VALUE_CHK) == 0)
    {
        const char        *str;
        struct client_id  id;
        struct client_exp exp;
        ClientInfo        info;
        
        //DPRINTF("tm_GetReport receive chk command\n");
        memset(&info, 0, sizeof(info));
        memset(&id, 0, sizeof(id));
        memset(&exp, 0, sizeof(exp));


	str=websGetVar(wp, T("IP"), "");
       /*str  = tmValueDefault("","%s", "IP");
        szTemp = strdup(str); 
        tmURLdecode(szTemp); */
        inet_pton(AF_INET, str, &(id.ip));
       // free(szTemp); 
	

         //str  = tmValueDefault("","%s", "HostName");
 	str=websGetVar(wp, T("HostName"), "");
        strncpy(id.hostname, str, HOSTNAME_SIZE-1);
       // tmURLdecode(id.hostname);

        //str  = tmValueDefault("","%s", "MAC");
        //szTemp = strdup(str);
        //tmURLdecode(szTemp);
 	str=websGetVar(wp, T("MAC"), "");
        StrToMac(str, id.mac);
        //free(szTemp);

        if (tmci_GetClient(id.mac, &info) >= 0)
            memcpy(&exp, &CEXP(&info), sizeof(exp));

        //str  = tmValueDefault("","%s", "CHK");
	str=websGetVar(wp, T("CHK"), "");
        if (!strcmp(str, "OKSEE"))
            exp.report = 0;
        else if (!strcmp(str, "NOSEE"))
            exp.report = 1;
        
        if (tmci_SetExp(&id, &exp) < 0)
            websWrite (wp, T("SetException FAIL\r\n\r\n"));
        else
        {
		   // We need to save the flash?????????
		  // extern int flash_write();
		  extern int run_script(char *filename, char *arg1, char *arg2, char *arg3);

            websWrite (wp, T("SetException SUCCESS\r\n\r\n"));
         /* Update current setting to config file */
           tmci_WriteClient(THPD_CONF_PATH);
 		   run_script("/etc/scripts/flash","write",0,0);
        }
    }
    else
    {
        //DPRINTF("tm_GetReport receive unknown command\n");
	//websWrite(wp,T("%s"),wp->query);
        websWrite (wp, T("UNKNOWN\r\n\r\n"));
    }

    websDone(wp, 200);

#if 0
    tmPrint();
#endif

   // tmFree();
}
