/******************************************************************************
*
*  (C) Copyright 2004 Trend Micro, Inc.
*
*  All Rights Reserved.
*  This program is an unpublished copyrighted work which is proprietary
*  to Trend Micro, Inc. and contains confidential information that is not
*  to be reproduced or disclosed to any other person or entity without
*  prior written consent from Trend Micro, Inc. in each and every instance.
*
*  WARNING:  Unauthorized reproduction of this program as well as
*  unauthorized preparation of derivative works based upon the
*  program or distribution of copies by sale, rental, lease or
*  lending are violations of federal copyright laws and state trade
*  secret laws, punishable by civil and criminal penalties.
*
*****************************************************************************/

/**
*  @file tmGetRouterInfo.c
*  @brief TMOG Get Router Info CGI
*/

/*
*  Change Log
*  2004-02-23 Jim Wu    Initialization version
*/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "webs.h"
#include "tmrqRequestQueue.h"
#include "tmdpMsgComposer.h"        /* header file for message composer.h */

//static const char ModuleName[]="tmGetRouterInfoCGI";
#define ALIGNMENT       (4)


void FlushConnectionCGI(webs_t wp,int nSize)
{
    /* Beneath output will close the HTTP connection for the sender */    
    websWrite (wp, T ("Server: TMSS router\r\nConnection: Close\r\n"));
    websWrite (wp, T ("Content-Length: %d\r\nContent-Type: application/x-tmdp-drm\r\n\r\n"),nSize); 
}

void tmGetRouterInfoCGI( webs_t wp,int nContentLen)
{
    int nDRMSize = 0;    
    unsigned char   pszDQM[1020+ALIGNMENT];
    unsigned char   *pszDRM;    
    unsigned char   *pAlignedDQM = (pszDQM+ALIGNMENT-((long)pszDQM%ALIGNMENT));
    unsigned char   *pAlignedDRM;

   //DPRINTF( "tmGetRouterInfoCGI\n");
   memset(pAlignedDQM, 0, 1020);
   memcpy(pAlignedDQM,wp->query, nContentLen);
   //DPRINTF( "tmSetPostDataPtr: pAlignedDQM=%s\n",pAlignedDQM);
   //DPRINTF( "tmSetPostDataPtr: nContentLen=%d\n",nContentLen);
    

    if (nContentLen != tmdpx_GetDQMSize())
    {
        /* size does not match */
        //DPRINTF( "CONTENT_LENGTH does not macth DQM %d\n", nContentLen);
        FlushConnectionCGI(wp,0);
        return;       
    }
    
    if (tmdpx_ParseDQM(pAlignedDQM) != 0)
    {
        //DPRINTF("Parse dqm fail, not correct DRM\n"); 
        /* free (pszDQM); */
        FlushConnectionCGI(wp,0);
        return;
    }    
    
     /* DQM parse OK */
    //DPRINTF("Parse dqm OK\n");
        

    nDRMSize = tmdpx_GetDRMSize();
   
    //DPRINTF("The DRM size returned: %d\n", nDRMSize);
 
    pszDRM = malloc(nDRMSize+ALIGNMENT);
    pAlignedDRM = (pszDRM+ALIGNMENT-((long)pszDRM%ALIGNMENT));
    
    if (pszDRM == 0)
    {
        //DPRINTF( "DRM size is zero, ready to return with 200\n");
        FlushConnectionCGI(wp,0);
        return;
    }
    
    memset(pAlignedDRM, 0, nDRMSize);
    
    //DPRINTF("Before prepare DRM\n");

    /* We don't need client infomation in this DRM, so the second parameter nIP is 0 here */
    tmdpx_PrepareDRM(pAlignedDRM, 0, "", "");
    
    //DPRINTF("After prepare DRM\n");
    
    websWrite(wp, T("HTTP/1.0 200 OK\n"));

   /* send out the DRM packet */
    FlushConnectionCGI(wp,nDRMSize);  /* for \r\n */
    
     
    websWriteBlock(wp,pAlignedDRM,nDRMSize);
    
    
    websDone(wp, 200);
       
    //DPRINTF( "Leaving tm_GetRouterInfo.cgi\n");

    return;
}


