/******************************************************************************
*
*  (C) Copyright 2004 Trend Micro, Inc.
*
*  All Rights Reserved.
*  This program is an unpublished copyrighted work which is proprietary
*  to Trend Micro, Inc. and contains confidential information that is not
*  to be reproduced or disclosed to any other person or entity without
*  prior written consent from Trend Micro, Inc. in each and every instance.
*
*  WARNING:  Unauthorized reproduction of this program as well as
*  unauthorized preparation of derivative works based upon the
*  program or distribution of copies by sale, rental, lease or
*  lending are violations of federal copyright laws and state trade
*  secret laws, punishable by civil and criminal penalties.
*
*****************************************************************************/

/**
*  @file tmSecurityStatus.c
*  @brief TMSS Virus Protection CGI
*/

/*
*  Change Log
*  2004-03-06 Eric Li    Initialization version
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <signal.h>
#include "tmCGILib.h"
#include "webs.h"
#include "tmcfg.h"
//#include "tmCGICommon.h"
#include "tmClientInfo.h"
#include "tmGetAUInfo.h"
#include "tmCGIErrMsg.h"
#include "tmURLFilterStatistic.h"
//#include "tmSecurityStatus.h"
//#include "tmURLFilterSetting.h"
#include "tmURLFilterConf.h"
#include "tmssConf.h"


//tmSecStat.asp
static int tmEngineVersion (int ejid, webs_t wp, int argc, char_t **argv);
static int tmPatternVersion (int ejid, webs_t wp, int argc, char_t **argv);
static int tmClientTable(int ejid, webs_t wp, int argc, char_t **argv);

//tmGetSet.asp
static int tmReadConf(int ejid, webs_t wp, int argc, char_t **argv);// Function use to trigger the reading of the configuration
static int tmUFLic(int ejid, webs_t wp, int argc, char_t **argv);
static int getUFEnable (int ejid, webs_t wp, int argc, char_t **argv);
static int getBlkMod (int ejid, webs_t wp, int argc, char_t **argv);
static int getBlkDate (int ejid, webs_t wp, int argc, char_t **argv);
static int getFltPlcy (int ejid, webs_t wp, int argc, char_t **argv);
static int getBlkTime (int ejid, webs_t wp, int argc, char_t **argv);
static void formTmGenSet(webs_t wp, char_t * path,char_t * query);


//tmSecConf.asp
static int get_auTimer (int ejid, webs_t wp, int argc, char_t **argv);
static int get_avGraceTime (int ejid, webs_t wp, int argc, char_t **argv);
static void form_tmSecConf(webs_t wp, char_t * path,char_t * query);

//tmUrlFltExLst.asp
static void form_tmUrlFltExLst(webs_t wp, char_t * path,char_t * query);
static int tmExceptionList(int ejid, webs_t wp, int argc, char_t **argv);
static int urlpage(int ejid, webs_t wp, int argc, char_t **argv);

//tmSStati.asp
static int tmStat (int ejid, webs_t wp, int argc, char_t **argv);
static void form_clear_tmStati (webs_t wp, char_t* path, char_t* query);


static void DefineTMSSConfig(void);

TmuAUVersion_t au;


//since we have keep a copy of configuraiton in the shared memory for usages between kernel/ask and task/task
// we need to increase the version to notice the kernel that the configuration have been changed.
extern unsigned int AVEConfigVersion;
extern unsigned int urlFilterConfigVersion;
extern unsigned int exceptionConfigVersion;


void DefineWebFunctions_TMSS() {
	DefineTMSSConfig();
}
//#define VAR_SYS_STATUS_FILE "/var/etc/status"
#define ETC_SYS_STATUS_FILE "/etc/default/config"
extern char* read_file_para_value(char *filename, char *arg1,char *buf);
int checked_function_enabled (char *parameter, webs_t wp, char_t **argv)
{
    char_t buf1[64];
	char_t is_yes[20];
	int i=0;
	strcpy(is_yes,(char *)read_file_para_value(ETC_SYS_STATUS_FILE,parameter,buf1));
 	if(strcmp(is_yes,"yes")==0)
		return websWrite(wp, T(""));
	
	//Then function is not present
	i=atoi(argv[0]);
 	if(i==0)//comment begin
		return websWrite(wp, T("<!--"));
	else
		return websWrite(wp, T("-->"));
}

static int is_tmss_enabled (int ejid, webs_t wp, int argc, char_t **argv)
{
 	return checked_function_enabled2("TMSS=",wp,argv);
}
static int is_tmss_enabled1 (int ejid, webs_t wp, int argc, char_t **argv)
{
 	return checked_function_enabled("TMSS=",wp,argv);
}

static void DefineTMSSConfig()
{	
	websAspDefine(T("is_tmss_enabled"), is_tmss_enabled);
	websAspDefine(T("is_tmss_enabled1"), is_tmss_enabled1);
	// tmSecStat.asp
	websAspDefine(T("tmEngineVersion"), tmEngineVersion);
	websAspDefine(T("tmPatternVersion"), tmPatternVersion);
	websAspDefine(T("ClientTable"), tmClientTable);

	// tmGenSet.asp
	websFormDefine(T("tmGenSet"), formTmGenSet);
	websAspDefine(T("tmReadConf"), tmReadConf);
	websAspDefine(T("tmUFLic"), tmUFLic);
	websAspDefine(T("getUFEnable"), getUFEnable);
	websAspDefine(T("getBlkMod"), getBlkMod);
	websAspDefine(T("getBlkDate"), getBlkDate);
	websAspDefine(T("getFltPlcy"), getFltPlcy);
	websAspDefine(T("getBlkTime"), getBlkTime);
	
	
	// tmSecConf.asp
	websAspDefine(T("get_auTimer"), get_auTimer);
	websAspDefine(T("get_avGraceTime"), get_avGraceTime);
	websAspDefine(T("urlpage"), urlpage);
	websFormDefine(T("tmSecConf"), form_tmSecConf);
	
	websAspDefine(T("tmExList"), tmExceptionList);
	

	websFormDefine(T("tmUrlFltExLst"), form_tmUrlFltExLst);

	websAspDefine(T("tmStat"), tmStat);
	websFormDefine(T("clear_tmStati"), form_clear_tmStati);
}

/**
 * @brief   Translate the string type version field to double word format and
 *          return this 32 bits double word.
 *
 * @param   pVer: the field of string type version field to be translated
 *
 * @return  32 bits double word
 *
 * @remarks For example, the input format is 6.801, output format is 0x00068010
 */
uint32_t 
tmau_MakeDWordVersion(const char* pVer)
{
    // major version first !
    uint32_t wMajor = atoi(pVer);
    uint32_t wMinor = 0;

    //  minor !
    char* pMinor =(char *)strchr(pVer,'.');
    char* pStop;
    if ( pMinor != NULL )
    {
        char szVer[5]={ ('0'),('0'),('0'),('0'),('\0') };
        char ch;
        int32_t len;
        for (len=0; (ch=*++pMinor) && len<4; len++ ) {
            if ( !isdigit(ch) ) break;
            szVer[len] = ch;
        }
        wMinor = strtoul(szVer, &pStop, 16);
    }

    return ((uint32_t)wMajor<<16 | ((uint32_t)wMinor));
}


static int tmClientTable(int ejid, webs_t wp, int argc, char_t **argv)
{
#define CLIENT_BUF_SIZE 4096
    ClientInfo *pBuf = NULL;
    ClientInfo *pClient;
    int i, size, nBufLeftSize;
    const char *style[] = {STR_CLASS_DATA1B, STR_CLASS_DATA2B};
    char szOneClientBuf[WEBS_BUFSIZE];
    //char szClientTable[WEBS_BUFSIZE];
   char szClientTable[CLIENT_BUF_SIZE];
    char strIP[BUF_SIZE];
    char strHost[BUF_SIZE];
    char strAV[BUF_SIZE];
    char strPattern[BUF_SIZE];
    char strEngine[BUF_SIZE];
    char strStatus[BUF_SIZE];
    uint32_t   ptnA, ptnB, ptnC, major, minor;
    
    tmci_InitClientTable();
    size = tmci_SizeClient();
    if(size>0)
    {
	    pBuf = (ClientInfo *)malloc(size * sizeof(ClientInfo));
	    if (pBuf == NULL)
	    {
		    DPRINTF("tmClientTable:memory allocation error.\n");
			websWrite(wp,T("Internal Error! memory allocation error for size=%d"),size * sizeof(ClientInfo));  
  		    goto END;
	    }
    }
    if ((size <= 0) || (tmci_GetClientsFrom(pBuf, &size, 0) < 0 ))
    {
        snprintf(szClientTable, WEBS_BUFSIZE - 1, NO_CLIENT, STR_CLIENT_TABLE_HIGHT,STR_CLASS_DATA1B);
       // replace = tmSedArgAdd(replace, TOKEN_CLIENT_TABLE, "%s", szClientTable);
 		websWrite(wp,T("%s"), szClientTable);  
       goto END;
    }
    
    szClientTable[0] = '\0';
    nBufLeftSize = CLIENT_BUF_SIZE - 1;
    for (i = 0; i < size; i++)
    {
        pClient = pBuf + i;
       // DPRINTF("tmClientTable: item %d,  size=%d\n",i,size);
        sprintf(strIP, "%u.%u.%u.%u", NIPQUAD(CIP(pClient)));
    
        if (CHOSTNAME(pClient)[0] != '\0')
        {
            sprintf(strHost, "%s", CHOSTNAME(pClient));
        }
        else
        {
            sprintf(strHost, "%s", STR_NA);
        }
    
        if (CSTATUS(pClient) == STATUS_AVOLD || CSTATUS(pClient) == STATUS_AVOK)
        {
            ptnA = CPATTERN(pClient) / 100000;
            ptnB = (CPATTERN(pClient) % 100000)/100;
            ptnC = CPATTERN(pClient) % 100;
            minor = (CENGINE(pClient) & 0xFFFF) >> 4;
            major = (CENGINE(pClient) & 0xFFFF0000) >> 16;
    
            sprintf(strAV, PCCILLIN_NAME);
            if (CPATTERN(pClient) < AUPATTERN(&au))
                sprintf(strPattern, "%u.%u.%02u %s", ptnA, ptnB, ptnC, STR_OLD);
            else
                sprintf(strPattern, "%u.%u.%02u", ptnA, ptnB, ptnC);
    
            if (CENGINE(pClient) < AUENGINE(&au))
                sprintf(strEngine, "%X.%03X %s", major, minor, STR_OLD);
            else
                sprintf(strEngine, "%X.%03X", major, minor);
        }
        else
        {
            sprintf(strAV, STR_NA);
            sprintf(strPattern, STR_NA);
            sprintf(strEngine, STR_NA);
        }
        
        switch(CSTATUS(pClient))
        {
            case STATUS_AVOK:
                sprintf(strStatus, STR_STATUS_AVOK);
                break;
            case STATUS_AVOLD:
                sprintf(strStatus, STR_STATUS_AVOLD);
                break;
            default:
                sprintf(strStatus, STR_STATUS_AVOTHERS);
                break;
        }
              
        snprintf(szOneClientBuf, CLIENT_BUF_SIZE - 1, CLIENT_DATA,
                 STR_CLIENT_TABLE_HIGHT, style[i%2], strIP,
                 STR_CLIENT_TABLE_HIGHT, style[i%2], strHost,
                 STR_CLIENT_TABLE_HIGHT, style[i%2], strAV,
                 STR_CLIENT_TABLE_HIGHT, style[i%2], strPattern,
                 STR_CLIENT_TABLE_HIGHT, style[i%2], strEngine,
                 STR_CLIENT_TABLE_HIGHT, style[i%2], strStatus);

        //strncat(szClientTable,szOneClientBuf, nBufLeftSize);
	websWrite(wp,T("%s"), szOneClientBuf);  
       
        nBufLeftSize -= strlen(szOneClientBuf);
        if (nBufLeftSize <= 0)
        {
            //DPRINTF("Error! Buffer size for client table not enough\n");
           // *nErrCode = -1;
           // ShowErrorPage(VIRUS_PROTECTION_PAGE_TITLE, ERR_STRING_CLIENT_BUFFER_SIZE);
 			websWrite(wp,T("Error! Buffer size for client table not enough"));  
           goto END;
        } 
    }
    //websWrite(wp,T("%s"), szClientTable);  
    

END:

    if (pBuf != NULL)
       free(pBuf);
      return 0;
 }

extern int checkIfAUInfoShareMemExist(void);

static int tmEngineVersion (int ejid, webs_t wp, int argc, char_t **argv)
{	
    uint32_t       ptnA, ptnB, ptnC;
	if(checkIfAUInfoShareMemExist())
	{
 	 tmau_GetNewestAUInfo(&au);
	 ptnA = au.pattern.version / 100000;
	 ptnB = (au.pattern.version % 100000)/100;
	 ptnC = au.pattern.version % 100;
	 websWrite(wp,T("%u.%u.%02u"),ptnA, ptnB, ptnC); 
	}
	else
	{
	 websWrite(wp,T("n/a")); 
	}
		
	return 0;
}
static int tmPatternVersion (int ejid, webs_t wp, int argc, char_t **argv)
{	
    uint32_t    major, minor;
   // tmau_GetNewestAUInfo(&au);   Done in tmEngineVersion since it is first and will done in a sequence.
	if(checkIfAUInfoShareMemExist())
	{
    minor = (au.engine.version & 0xFFFF) >> 4;
    major = (au.engine.version & 0xFFFF0000) >> 16;
	websWrite(wp,T("%X.%03X"), major, minor);  
	}
	else
	{
	 websWrite(wp,T("n/a")); 
	}
	return 0;
}

static    url_filter_config_t UrlFilterConf;

static int tmReadConf(int ejid, webs_t wp, int argc, char_t **argv)
{	
// use to trigger the reading of the configuration, write nothing.
    CopyUrlConfToLocal(&UrlFilterConf);
 	websWrite(wp,T(""));  
	return 0;
}
static int tmUFLic(int ejid, webs_t wp, int argc, char_t **argv)
{	
    if (UrlFilterConf.nUrlLicenseEnable)
    {
	websWrite(wp,T(""));  
    }
    else
    {
        /* No License */
	websWrite(wp,T("disabled"));  
    }
	return 0;
}

static int getUFEnable (int ejid, webs_t wp, int argc, char_t **argv)
{
#define URL_FILTER_ENABLE 1
        int i=0;
	i=atoi(argv[0]);
        if (i==URL_FILTER_ENABLE)
	{// it is Enable(=1) radio
		if(UrlFilterConf.nUrlFilterEnable)
			return websWrite(wp, T("%s"),"checked");
	}
	else
	{
		if(!UrlFilterConf.nUrlFilterEnable)
			return websWrite(wp, T("%s"),"checked");
	}		
	return websWrite(wp, T(""));
}
static int getBlkMod (int ejid, webs_t wp, int argc, char_t **argv)
{
#define URL_FILTER_ALWAYS_BLOCK 1
	int i=0;
	i=atoi(argv[0]);
        if (i==URL_FILTER_ALWAYS_BLOCK)
	{ //it is Always block radio
		if(UrlFilterConf.alwaysBlock)
			return websWrite(wp, T("%s"),"checked");
	}
	else
	{
		if(!UrlFilterConf.alwaysBlock)
			return websWrite(wp, T("%s"),"checked");
	}
				
	return websWrite(wp, T(""));
}

static int getBlkTime (int ejid, webs_t wp, int argc, char_t **argv)
{
#define URL_FILTER_BLOCK_TIME_FROM_HOUR 0
#define URL_FILTER_BLOCK_TIME_FROM_MIN 1
#define URL_FILTER_BLOCK_TIME_TO_HOUR 2
#define URL_FILTER_BLOCK_TIME_TO_MIN 3
	int i=0;
	i=atoi(argv[0]);
        switch(i)
	{
		case 0:
			return websWrite(wp, T("%2d"),UrlFilterConf.nScheduleFromHour);
		case 1:
			return websWrite(wp, T("%2d"),UrlFilterConf.nScheduleFromMin);
		case 2:
			return websWrite(wp, T("%2d"),UrlFilterConf.nScheduleToHour);
		case 3:
			return websWrite(wp, T("%2d"),UrlFilterConf.nScheduleToMin);
	}
				
	return websWrite(wp, T(""));
}
static int getBlkDate (int ejid, webs_t wp, int argc, char_t **argv)
{
#define URL_FILTER_BLOCK_DAY_SUN 0
#define URL_FILTER_BLOCK_DAY_MON 1
#define URL_FILTER_BLOCK_DAY_TUE 2
#define URL_FILTER_BLOCK_DAY_WED 3
#define URL_FILTER_BLOCK_DAY_THU 4
#define URL_FILTER_BLOCK_DAY_FRI 5
#define URL_FILTER_BLOCK_DAY_SAT 6
	if(UrlFilterConf.nScheduleDay[atoi(argv[0])])
		return websWrite(wp, T("checked"));
	return websWrite(wp, T(""));
}

static int getFltPlcy (int ejid, webs_t wp, int argc, char_t **argv)
{
	if(UrlFilterConf.nCategory[atoi(argv[0])])
	{
		if (strcmp(argv[1],"BLK")==0)
			return websWrite(wp, T("checked"));
	}
	else
	{
		if (strcmp(argv[1],"BLK")!=0)
			return websWrite(wp, T("checked"));
	}
	return websWrite(wp, T(""));
}

      
static int ReloadThpdConf(void);
 		
static void formTmGenSet(webs_t wp, char_t * path,char_t * query)
{
    tmcfg_t cfg = NULL;
	const int nCategoriesCount = HV_DEFINED_CAT_NUM - 1;
 	int i;
	char scheduleFromHour[5];
	char scheduleFromMin[5];
	char scheduleToHour[5];
	char scheduleToMin[5];
	char szCatgoryBuf[URL_FILTER_MAX_CATEGORY];
	char szCat[4];
	char *ptr;
	
	/* Open Config file for write */
	if( NULL == (cfg=tmcfg_open(THPD_CONF_PATH,TMCFG_MODE_UPDATE)) )
	{
		//DPRINTF("Open config file failed.\n");
		return;
	}
    
        UrlFilterConf.nUrlFilterEnable=atoi(websGetVar(wp, T("ufEnable"), "1"));
        if(UrlFilterConf.nUrlFilterEnable)
		tmcfg_set_param_value(cfg, CFG_MODULE_URL_FILTER, CFG_PARAM_URL_FILTER_ENABLE, CFG_PARAM_VALUE_URL_FILTER_ENABLE,NULL);
	else
		tmcfg_set_param_value(cfg, CFG_MODULE_URL_FILTER, CFG_PARAM_URL_FILTER_ENABLE, CFG_PARAM_VALUE_URL_FILTER_DISABLE,NULL);

        UrlFilterConf.alwaysBlock=atoi(websGetVar(wp, T("blockmode"), "1"));
        if(UrlFilterConf.alwaysBlock)
		tmcfg_set_param_value(cfg, CFG_MODULE_URL_FILTER, CFG_PARAM_ALWAYS_BLOCK, CFG_PARAM_VALUE_URL_FILTER_ENABLE,NULL);
	else
		tmcfg_set_param_value(cfg, CFG_MODULE_URL_FILTER, CFG_PARAM_ALWAYS_BLOCK, CFG_PARAM_VALUE_URL_FILTER_DISABLE,NULL);
        ptr=websGetVar(wp, T("BTSun"), "");
	if(ptr)//Only when presents
		UrlFilterConf.nScheduleDay[0]=atoi(ptr);
        ptr=websGetVar(wp, T("BTMon"), "");
	if(ptr)
		UrlFilterConf.nScheduleDay[1]=atoi(ptr);
        ptr=websGetVar(wp, T("BTTue"), "");
	if(ptr)
		UrlFilterConf.nScheduleDay[2]=atoi(ptr);
        ptr=websGetVar(wp, T("BTWed"), "");
	if(ptr)
		UrlFilterConf.nScheduleDay[3]=atoi(ptr);
        ptr=websGetVar(wp, T("BTThu"), "");
	if(ptr)
		UrlFilterConf.nScheduleDay[4]=atoi(ptr);
        ptr=websGetVar(wp, T("BTFri"), "");
	if(ptr)
		UrlFilterConf.nScheduleDay[5]=atoi(ptr);
        ptr=websGetVar(wp, T("BTSat"), "");
	if(ptr)
		UrlFilterConf.nScheduleDay[6]=atoi(ptr);
        for (i = 0; i < MAX_WEEK_DAYS; i++)
        {
            char szTemp[32];
            
            sprintf(szTemp,"%s%d",CFG_PARAM_SCHEDULE_DAY,i);
	    if (UrlFilterConf.nScheduleDay[i])
            {
                tmcfg_set_param_value(cfg, CFG_MODULE_URL_FILTER, szTemp, CFG_PARAM_VALUE_URL_FILTER_ENABLE,NULL) ;
            }
            else
            {
                tmcfg_set_param_value(cfg, CFG_MODULE_URL_FILTER, szTemp, CFG_PARAM_VALUE_URL_FILTER_DISABLE,NULL);
            }
            
        }
	
        UrlFilterConf.nScheduleFromHour=atoi(websGetVar(wp, T("urlfBHour"), "8"));
        UrlFilterConf.nScheduleFromMin=atoi(websGetVar(wp, T("urlfBMin"), "0"));
        UrlFilterConf.nScheduleToHour=atoi(websGetVar(wp, T("urlfEHour"), "18"));
        UrlFilterConf.nScheduleToMin=atoi(websGetVar(wp, T("urlfEMin"), "0"));
	sprintf(scheduleFromHour,"%d",UrlFilterConf.nScheduleFromHour);
	sprintf(scheduleFromMin,"%d",UrlFilterConf.nScheduleFromMin);
	sprintf(scheduleToHour,"%d",UrlFilterConf.nScheduleToHour);
	sprintf(scheduleToMin,"%d",UrlFilterConf.nScheduleToMin);
        tmcfg_set_param_value(cfg, CFG_MODULE_URL_FILTER, CFG_PARAM_SCHEDULE_FRM_HOUR, scheduleFromHour,NULL); 
        tmcfg_set_param_value(cfg, CFG_MODULE_URL_FILTER, CFG_PARAM_SCHEDULE_FRM_MIN, scheduleFromMin,NULL);
        tmcfg_set_param_value(cfg, CFG_MODULE_URL_FILTER, CFG_PARAM_SCHEDULE_TO_HOUR, scheduleToHour,NULL);
        tmcfg_set_param_value(cfg, CFG_MODULE_URL_FILTER, CFG_PARAM_SCHEDULE_TO_MIN, scheduleToMin,NULL);

        /* Get and Check categories */
        szCatgoryBuf[0] = '\0';
        for(i=0;i<nCategoriesCount;i++)
	{
		sprintf(szCat,"t%d",i+1);
		UrlFilterConf.nCategory[i]=atoi(websGetVar(wp, T(szCat), "1"));
		if(UrlFilterConf.nCategory[i])
		{
			sprintf(szCat,"%02d",i);
			strcat(szCatgoryBuf,szCat);
		}
	}
 	tmcfg_set_param_value(cfg, CFG_MODULE_URL_FILTER, CFG_PARAM_CATEGORY, szCatgoryBuf,NULL);
        tmcfg_flush(cfg);
        tmcfg_close(cfg);
 
        /* Send Signal to thpd to reload the configuration setting */
        ReloadThpdConf();
	
	//WE need also to reload here from the webs task itself.
    	tmLoadUrlFilterConf(THPD_CONF_PATH);
	websRedirect(wp, T("tmGenSet.asp"));	
}
static int get_auTimer (int ejid, webs_t wp, int argc, char_t **argv)
{
	int i;
	int check_time=0;
	int auEnable = 0;
    const char* pEnable = NULL;
    const char* pTime = NULL;
    tmcfg_t cfg = NULL;
	
   /* Open Config file for read */
    if( NULL == (cfg=tmcfg_open(THPD_CONF_PATH,TMCFG_MODE_READ)) )
    {
       //DPRINTF("readSecuritySetting:Open config file failed.\n");
       return -1;
    }
    
    pEnable = tmcfg_get_param_value(cfg, AU_CHECKER, "enable", NULL);
    pTime = tmcfg_get_param_value(cfg, AU_CHECKER, AU_TIME, NULL);
   
    if (pTime == NULL ||  pEnable==NULL )
    {
        //DPRINTF("get_auTimer:Read config file  failed.\n");
        tmcfg_close(cfg);
	return -1;
    }

    auEnable = atoi(pEnable);
    if (auEnable != 0 && auEnable != 1)
    {
         //DPRINTF("readSecuritySetting:Read au enabled failed=%s\n",pEnable);
	 auEnable=1;//default
    }

    check_time = atoi(pTime);
    tmcfg_close(cfg);
    
    i=atoi(argv[0]);
    if(i==0)
    {
	    if(!auEnable)
		    return websWrite(wp, T("selected"));
			
    }
    else
    {//au is enabled
	    if(auEnable)
	    {
		    if((check_time/600)==i)// by 10 minutes
			    return websWrite(wp, T("selected"));
	    }
    }
    return websWrite(wp, T(""));
}

 	
static int get_avGraceTime (int ejid, webs_t wp, int argc, char_t **argv)
{
	int i;
	int nDisplayInterval = 0;
	tmcfg_t cfg = NULL;
	const char *pszDisplayInterval = NULL;
	/* Open Config file for read */
	if( NULL == (cfg=tmcfg_open(THPD_CONF_PATH,TMCFG_MODE_READ)) )
	{
		//DPRINTF("readSecuritySetting:Open config file failed.\n");
		return -1;
	}
    
	pszDisplayInterval = tmcfg_get_param_value(cfg, SERVICE_PAGE_CONF_MODULE_AVENFORCER, SERVICE_PAGE_CONF_PARAM_TIMEREPORT, NULL);
   
	if ( pszDisplayInterval == NULL )
	{
        //DPRINTF("get_avGraceTime:Read config file  failed.\n");
        tmcfg_close(cfg);
	return -1;
	}

	nDisplayInterval = atoi(pszDisplayInterval);
	tmcfg_close(cfg);

	i=atoi(argv[0]);
	if((nDisplayInterval/86400)==i)// by days
		return websWrite(wp, T("selected"));
	return websWrite(wp, T(""));
}


static int tmExceptionList(int ejid, webs_t wp, int argc, char_t **argv)
{
char_t *tableFmt1 =
"<TR>\n"
"<input type=hidden value=\"%s\" name=macVal%d>\n"
"<TD class=title colspan=1>%s</TD>\n"
"<TD class=title colspan=1><INPUT type=radio %s value=1 name=%s%d> Enabled\n"
"<INPUT type=radio %s value=0 name=%s%d>Disabled\n"
"<TD class=title colspan=1><INPUT type=\"button\" value=\"Remove And Apply\" onclick=\"DeleteItem(%d)\"> </NOBR></TD>\n"
"</TD></TR>\n";
	/* show computer list */
	unsigned int size, idx;
	ClientInfo   *pBuf = NULL, *pClient = NULL;
	char szIPTemp[16],value[32];
	int ret;

	CopyUrlConfToLocal(&UrlFilterConf);
	if (!UrlFilterConf.nUrlLicenseEnable)
	{
		if (strcmp(argv[0],"tmurl")==0)
			return websWrite(wp, "");
	}
	tmci_InitClientTable();
	size = tmci_SizeClient();
	websWrite(wp, T("<input type=hidden value=%d name=clsize>\n\r"),size);

	if(size>0)
	{
		pBuf = malloc(size * sizeof(ClientInfo));
		tmci_GetClientsFrom(pBuf, &size, 0);
               
        for (idx = 0; idx < size; idx++)
        {
            pClient = pBuf + idx;

            snprintf(szIPTemp,16,"%u.%u.%u.%u",NIPQUAD(CIP(pClient)));

			if (strcmp(argv[0],"tmav")==0)
				ret=tmci_IsExpReport(CMAC(pClient));
			else
				ret=tmci_IsExpFilter(CMAC(pClient)); 
			MacToStr(CMAC(pClient), value);
			if (ret)
			{
				websWrite(wp, T(tableFmt1),
				value,idx,
				szIPTemp,
				"CHECKED",
				argv[0],idx,
				"",
				argv[0],idx,idx+1);
			}
			else
			{
				websWrite(wp, T(tableFmt1),
				value,idx,
				szIPTemp,
				"",
				argv[0],idx,
				"CHECKED",
				argv[0],idx,idx+1);
			}
		}
		if(strcmp(argv[0],"tmurl")==0)
			websWrite(wp,"<TR><TD colSpan=3><INPUT type=submit value=Apply></TD> </TR>");
	}
	return 1;
}

static int urlpage(int ejid, webs_t wp, int argc, char_t **argv)
{
char_t *tableFmt1 =
"\t<TR><TD class=\"test3\"><A target=\"main\" href=\"tmUrlFltExLst.asp\">Exception List</A></TD></TR> \n\r"
"\t<TR><TD class=\"test3\"><A target=\"main\" href=\"tmStati.asp\">Statistics</A></TD></TR> \n\r";

    CopyUrlConfToLocal(&UrlFilterConf);
    if (UrlFilterConf.nUrlLicenseEnable)
	return websWrite(wp, T(tableFmt1));
    else
 	return websWrite(wp, "");
}

static void SetExceptionStatus(const char *pszMAC, uint8_t status, int nOption);
extern int tmci_DelClient(unsigned char *mac);
static void tmExLstSetting(webs_t wp, int nOption)
{
 	int i,clientSize;
	unsigned char      tmExEnable[10],macVal[8];
	int enable,delKeyItem;
    const char *pszMAC = NULL;
 	
    clientSize=atoi(websGetVar(wp, T("clsize"), "0"));
 	delKeyItem = atoi(websGetVar (wp, T ("tmssDel"), "0"));
	for(i=0;i<clientSize;i++)
	{
		if((delKeyItem)&&(i==(delKeyItem-1)))
			continue;//skip the to be delete one
		sprintf(macVal,"macVal%d",i);
		pszMAC = websGetVar(wp, T(macVal), "");
		if(CGI_EXCEPT_FILTER==nOption)
			sprintf(tmExEnable,"tmurl%d",i);
		else
			sprintf(tmExEnable,"tmav%d",i);
			
		enable=atoi(websGetVar(wp, T(tmExEnable), "0"));

		SetExceptionStatus(pszMAC,enable,nOption);
	}
 	if (delKeyItem)
	{
		sprintf(macVal,"macVal%d",delKeyItem-1);
		pszMAC = websGetVar(wp, T(macVal), "");
		if(pszMAC)
		{			
			unsigned char  mac[MAC_SIZE];
        
	 		StrToMac(pszMAC, mac);
			tmci_DelClient(mac);
		}
	}
}

static void form_tmUrlFltExLst(webs_t wp, char_t * path,char_t * query)
{ 	
	tmExLstSetting(wp,CGI_EXCEPT_FILTER);
	tmci_WriteClient(THPD_CONF_PATH);
	websRedirect(wp, T("tmUrlFltExLst.asp"));	
	//run_script("/etc/scripts/flash","write",0,0);
 }
static int ReloadAUCheckerConf(void);

static void form_tmSecConf(webs_t wp, char_t * path,char_t * query)
{
    int nDisplayInterval,nDisplayIntervalOld;
    tmcfg_t cfg = NULL;
    const char *pszDisplayInterval = NULL;
	int auEnable = 0;
    const char* pEnable = NULL;
    const char* pTime = NULL;
    
 
    /* Open Config file for write */
    if( NULL == (cfg=tmcfg_open(THPD_CONF_PATH,TMCFG_MODE_UPDATE)) )
    {
	//DPRINTF("Open config file failed.\n");
	return;
    }
    
    //For auTimer
     pEnable = tmcfg_get_param_value(cfg, AU_CHECKER, "enable", NULL);
    pTime = tmcfg_get_param_value(cfg, AU_CHECKER, AU_TIME, NULL);
   
    if (pTime == NULL ||  pEnable==NULL )
    {
        //DPRINTF("get_auTimer:Read config file  failed.\n");
 	 nDisplayIntervalOld=3;
    }
    else
    {
	    auEnable = atoi(pEnable);
	    if (auEnable != 0 && auEnable != 1)
	    {
		    //DPRINTF("readSecuritySetting:Read au enabled failed=%s\n",pEnable);
		    nDisplayIntervalOld=3;
 	    }
	    else
	    {
		    if(auEnable)
			    nDisplayIntervalOld = atoi(pTime)/600;
		    else
			    nDisplayIntervalOld = 0;
	    }
     }

    
    
    nDisplayInterval = atoi(websGetVar(wp, T("auTimer"), "3"));
    if(nDisplayInterval!=nDisplayIntervalOld)
    {
	    if (!nDisplayInterval)
	    {
		    tmcfg_set_param_value(cfg, AU_CHECKER, "enable", "0", NULL);// disabled
	    }
	    else 
	    {
		    char tmpStr[10];
		    sprintf(tmpStr,"%d",nDisplayInterval*600);
		    tmcfg_set_param_value(cfg, AU_CHECKER, "enable", "1", NULL);//enabled
		    tmcfg_set_param_value(cfg, AU_CHECKER, AU_TIME,tmpStr,NULL);
	    } 
	    ReloadAUCheckerConf();
    }
    
    //For avenforecr time_report
    pszDisplayInterval = tmcfg_get_param_value(cfg, SERVICE_PAGE_CONF_MODULE_AVENFORCER, SERVICE_PAGE_CONF_PARAM_TIMEREPORT, NULL);
   
   if ( pszDisplayInterval == NULL )
   {
        //DPRINTF("get_avGraceTime:Read config file  failed.\n");
	nDisplayIntervalOld=1;
    }
    else
    {

	nDisplayIntervalOld = atoi(pszDisplayInterval)/86400;
    }

    nDisplayInterval = atoi(websGetVar(wp, T("avGraceTime"), "1"));
    if(nDisplayInterval!=nDisplayIntervalOld)
    {
	    char tmpStr[10];
	    sprintf(tmpStr,"%d",nDisplayInterval*86400);
	    tmcfg_set_param_value(cfg, SERVICE_PAGE_CONF_MODULE_AVENFORCER, SERVICE_PAGE_CONF_PARAM_TIMEREPORT, tmpStr,NULL);
	    /* Send Signal to thpd to reload the configuration setting */
	    ReloadThpdConf();
   }
   if (cfg != NULL)
   {
	   tmcfg_flush(cfg);
	   tmcfg_close(cfg);
   }
   /* Write Client Info to Config file for exception list change */
    tmExLstSetting(wp,CGI_EXCEPT_REPORT);
    tmci_WriteClient(THPD_CONF_PATH);
    websRedirect(wp, T("tmSecConf.asp"));	
	//run_script("/etc/scripts/flash","write",0,0);
}

static int tmStat (int ejid, webs_t wp, int argc, char_t **argv)
{
char_t *tableFmt1 =
"\t<TD>%d</TD>\n\r"
"\t<TD>%s</TD>\n\r"
"\t<TD>%d</TD>\n\r"
"\t<TD>%s</TD>\n\r";
    url_filter_statistic_t Report[HV_DEFINED_CAT_NUM];
    int i;
    time_t StartTime;
    struct tm *start;
    char szTempTime1[MAX_TIME]="N/A";
    char szTempTime2[MAX_TIME]="N/A";
    
     tmGetURLStatisticCount(Report, sizeof(url_filter_statistic_t)*HV_DEFINED_CAT_NUM, &StartTime);
     i=atoi(argv[0]);
     if(Report[i].nAccessCount>0)
     {
	     start = localtime(&Report[i].LastAccessTime);
	     sprintf(szTempTime1, "%02d/%02d/%04d %2d:%02d:%02d", start->tm_mon+1, start->tm_mday, start->tm_year + 1900, start->tm_hour, start->tm_min, start->tm_sec);
     }
     if(Report[i].nBlockedCount>0)
     {
	     start = localtime(&Report[i].LastBlockedTime);
	     sprintf(szTempTime2, "%02d/%02d/%04d %2d:%02d:%02d", start->tm_mon+1, start->tm_mday, start->tm_year + 1900, start->tm_hour, start->tm_min, start->tm_sec);
     }
     return websWrite(wp, T(tableFmt1),
     Report[i].nAccessCount,
     szTempTime1,
     Report[i].nBlockedCount,
     szTempTime2
      );
}

static void form_clear_tmStati (webs_t wp, char_t* path, char_t* query)
{
	tmResetURLStatisticCount();
	websRedirect(wp, T("tmStati.asp"));
}


static int ReloadThpdConf(void)
{
    FILE *fp;
    char szBuf[MAX_LINE];
    long longPid;
    int nRetVal = 0;
    
    fp = fopen(THPD_PID_PATH, "r");
    
    if (fp == NULL)
    {
        return -1;
    }
    
    szBuf[MAX_LINE - 1] = '\0';
    fgets(szBuf, MAX_LINE - 1, fp);
    
    longPid = strtol(szBuf, NULL, 10);
   
    ////DPRINTF("Sending Signal to thpd pid: %lu\n", longPid);
    kill((pid_t)longPid, SIGHUP);
    
    fclose(fp);
    return nRetVal;
}


static int ReloadAUCheckerConf(void)
{
    FILE *fp;
    char szBuf[MAX_LINE];
    long longPid;
    int nRetVal = 0;
    
    fp = fopen(AUCHECKER_PID_PATH, "r");
    
    if (fp == NULL)
    {
        return -1;
    }
    
    szBuf[MAX_LINE - 1] = '\0';
    fgets(szBuf, MAX_LINE - 1, fp);
    
    longPid = strtol(szBuf, NULL, 10);
   
    ////DPRINTF("Sending Signal to tmAUChecker pid: %lu\n", longPid);
    kill((pid_t)longPid, SIGHUP);
    
    fclose(fp);
    return nRetVal;
}


static void SetExceptionStatus(const char *pszMAC, uint8_t status, int nOption)
{
    struct client_exp Buf;
    ClientInfo Client;
    unsigned char  mac[MAC_SIZE];
    
    if (pszMAC == NULL)
    {
        //DPRINTF("SetExceptionStatus(), NULL input\n");
        return;
    }
    
    StrToMac(pszMAC, mac);

    if (tmci_GetClient(mac, &Client) == 0)
    {
        if (nOption == CGI_EXCEPT_FILTER)
        {
            Buf.urlfilter = status;
            Buf.report = CISEXPREPORT(&Client);
        }
        else
        {
            Buf.urlfilter = CISEXPFILTER(&Client);
            Buf.report = status;        
        }
        if (tmci_SetExp(&(Client.idinfo), &Buf) != 0)
        {
            //DPRINTF("SetExceptionStatus(), can't set client %s Exception to %d\n", pszMAC,status);
        }
        else
        {
            ////DPRINTF("SetExceptionStatus(), set client %s Exception to %d\n",pszMAC,status);
        }
    }
    else
    {
        //DPRINTF("SetExceptionStatus(), can't find client %s\n", pszMAC);
    }
}

static char  buf3_255[255];
#include "../../websconfig/lan_conf.h"
#include "../../websconfig/home.h"
extern char* read_file_para_value(char *filename, char *arg1,char *buf);

int tmGetBoxBuild(char *szBoxBuild)
{
	FILE *f;
	if ((f=fopen("/etc/default/config","r"))!=NULL){
		fclose(f);
		strcpy(szBoxBuild,read_file_para_value("/etc/default/config","SOFTWARE_VER=",buf3_255));

	}
	else{
		strcpy(szBoxBuild,"");		
	}
    //strncpy(szBoxBuild, TM_BOX_FIRMWARE_BUILD, TMSS_MAX_BUF_BOX_BUILD - 1);
	//strcpy(szBoxBuild, FIRMWARE_BUILD_VERSION);
   return 0;
}


