/*
 *  update.htm: Upgrade Firmware
 */
/*****************************************************************************
 * standard include files
 ****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <unistd.h>
#include <syslog.h>
#include <stdarg.h>
#include <signal.h>

#define _NON_POWER_2_
/****************************************************************************
 * project include files
 ****************************************************************************/
/* goahead web server */
#include "webs.h"
#include "update.h"
#include "um.h"
#include "uemf.h"
#include "wsIntrn.h"
#include "lan_conf.h"
#include "wan.h"
#include "misc.h"
#include <linux/tc3162dmtctrl.h>
#include <sys/mount.h>


//notice, when you change these values, you might need to change
//apps/appsbox/profile.c
//apps/appsbox/profile.h
//apps/websconfig/profile.c
#ifdef VOIP
#define AC_KERNEL_MTD "/dev/mtd5"
#define AC_APPS_MTD "/dev/mtd6"
#define AC_CONFIG_DSP_MTD "/dev/mtd4"
#define AC_KERNEL_MTD_NUM 5
#define AC_CONFIG_DSP_MTD_NUM 4
#else // HW_7202
#define AC_KERNEL_MTD "/dev/mtd4"
#define AC_APPS_MTD "/dev/mtd5"
#define AC_KERNEL_MTD_NUM 4
#endif
#define AC_BOOTROM1_MTD	"/dev/mtd0"
#define AC_MAC_MTD "/dev/mtd1"
#define AC_CONFIG_MTD "/dev/mtd2"
#define AC_BOOTROM2_MTD "/dev/mtd3"
#define AC_CONFIG_MTD_NUM 2


extern long* find_pid_by_name( char* pidName);

static int get_firmware_size (int ejid, webs_t wp, int argc, char_t **argv);

//static int read_mtd4_size(unsigned long *j);
static int do_upgrade_firmware (int ejid, webs_t wp, int argc, char_t **argv);
//static int upgrade_redirect (int ejid, webs_t wp, int argc, char_t **argv);
//static void form_upgradefirmware(webs_t wp, char_t* path, char_t* query); 

static void form_update_conf (webs_t wp, char_t* path, char_t* query);
static void form_update_factory_conf (webs_t wp, char_t* path, char_t* query);
static void form_update_complete (webs_t wp, char_t* path, char_t* query);
extern char* read_file_para_value(char *filename, char *arg1,char *buf);

static int read_mtd_size(int mtdnum, unsigned long *mtdsize);
static int clear_mtd(const char *mtd_path, unsigned long size);
void clear_config();
int isclean_config=1; //return to factory reset after upgrading

static int get_factory_value (int ejid, webs_t wp, int argc, char_t **argv);
/* the read_mtd_size function */
/* read the size of mtd device from /proc filesystem */
/* the mtdnum is from zero to mtd_max_value */
/* then return the size of mtd  */
static int read_mtd_size(int mtdnum, unsigned long *mtdsize)
{
	FILE *fp;
	char_t line[255];
	char_t dev[6], name[8]; 
	//unsigned long size, erasesize;
	unsigned int size, erasesize, i;
	
	fp = fopen("/proc/mtd", "r");
/* dev: size erasesize   name 
 * mtd0: xx        xx   "boot"
 * ....
 */
	if(fp==NULL) return -1;

	for(i=0;i<mtdnum+1; i++) {
		fgets(line, 255, fp);
		//DPRINTF("line1 %s\n", line);
	}

	fgets(line, 255, fp);
	//our kernel is located on mtd4
	DPRINTF("line%d %s\n", i, line);
	//DPRINTF("%s %08X %08X %s\n", dev,size,erasesize, name);
	sscanf(line, "%s %08X %08X %s\n", dev, &size, &erasesize, name);
	DPRINTF("|%s|%08X|%08X|%s|", dev,size,erasesize, name);
	*mtdsize=size;
	DPRINTF("\n return size %08X \n", *mtdsize);
	fclose(fp);
	return 0;
}

#if 0
static int read_mtd4_size(unsigned long *j)
{
	FILE *fp;
	char_t line[255];
	char_t dev[6], name[8]; 
	//unsigned long size, erasesize;
	unsigned int size, erasesize;
	
	fp = fopen("/proc/mtd", "r");
/* dev: size erasesize   name 
 * mtd0: xx        xx   "boot"
 * ....
 */
	if(fp==NULL) return -1;

	fgets(line, 255, fp);
	//DPRINTF("line1 %s\n", line);

	fgets(line, 255, fp);
	//DPRINTF("line2 %s\n", line);

	fgets(line, 255, fp);
	//DPRINTF("line3 %s\n", line);

	fgets(line, 255, fp);
	fgets(line, 255, fp);
	fgets(line, 255, fp);
	//our kernel is located on mtd4
	DPRINTF("line4 %s\n", line);
	//DPRINTF("%s %08X %08X %s\n", dev,size,erasesize, name);
	sscanf(line, "%s %08X %08X %s\n", dev, &size, &erasesize, name);
	DPRINTF("|%s|%08X|%08X|%s|", dev,size,erasesize, name);
	*j=size;
	DPRINTF("\n return size %08X \n", *j);
	//DPRINTF("\n test size %08X \n", *j);
//	*j=0;
	fclose(fp);
	return 0;
}
#endif

#if 0
static int upgrade_redirect (int ejid, webs_t wp, int argc, char_t **argv)
{
	FILE *fp;
	char_t line[255];
	char_t update_ok[2];

	
	/*  /flash/upgrade.conf
	 *  #:0:   is upgraded error
	 *  #:1:   is upgraded ok
	 */
	fp = fopen(UPGRADE_CONFIG_FILE, "r");
	
	if(fp==NULL) return -1;
	
	if (fgets(line, 255, fp))
    	{
    		if (sscanf(line, "#:%[^:]s\n", update_ok)  != 1)
    		{
     			//error format
    		}
    	}
	fclose(fp);
	
	if (strcmp(update_ok,"0")==0) 
		return websWrite(wp,"location.replace('update_error.asp')");
	else
		return websWrite(wp,"location.replace('update_complete.asp')");
}
#endif

extern int configurationIsRestored;

#ifdef _NON_POWER_2_
extern char *firmwareUpdateStorage;// pointer to the begin of the buffer
extern unsigned long  firmwareUpdateStorageLen;//pointer to current file position
#define FM_header_SIZE 128
#define FM_VERSION_SIZE 8
#define FM_FBID_SIZE 1
#define FM_SIGNATURE_SIZE 15
#define FM_CHECKSUM_SIZE 4
#define FM_BD_SIZE 4
#define FM_SUBVERSION_SIZE 16
#define FM_FWID_SIZE 1
#define FM_UGID_SIZE 1
#define FM_MGID_SIZE 1
#define FM_UNB0_SIZE 1
#define FH_CHECKSUM_SIZE 4
#define FH_KERNEL_START_SIZE 4
#define FH_KERNEL_INPUTLEN_SIZE 4
#define FH_TOTAL_SIZE 4
#define FH_PRODUCT_ID 4
#define FT_SIZE 1
#define BUFFER_SIZE (FM_header_SIZE - FM_VERSION_SIZE - FM_FBID_SIZE - FM_SIGNATURE_SIZE - FM_CHECKSUM_SIZE - FM_BD_SIZE - FM_SUBVERSION_SIZE - FM_FWID_SIZE - FM_UGID_SIZE - FM_MGID_SIZE - FM_UNB0_SIZE - FH_CHECKSUM_SIZE - FH_KERNEL_START_SIZE - FH_KERNEL_INPUTLEN_SIZE - FH_TOTAL_SIZE -FH_PRODUCT_ID - FT_SIZE)


struct FM_header {
	char version[FM_VERSION_SIZE];
	char FB_id[FM_FBID_SIZE];
	char signature[FM_SIGNATURE_SIZE];
	int checksum;
	int bdsize;
	char subversion[FM_SUBVERSION_SIZE];
	char flashW_id[FM_FWID_SIZE];
	char ug_id[FM_UGID_SIZE];
	char mg_id[FM_MGID_SIZE];
	char unb0[FM_UNB0_SIZE];
	int fh_checksum;
	int fh_kernel_start;
	int fh_kernel_inputlen;
	int fh_total_size;
	int fh_product_id;
	char flash_type[FT_SIZE];
	char buffer[BUFFER_SIZE];
};


volatile unsigned char* firmwareUpdateStorageStart=0; 
#define firmwareUpdateStorageLenStart (firmwareUpdateStorageLen - FM_header_SIZE)
#define FM_TMP_HALFBUFSIZE	64
#define FM_TMP_BUFSIZE	128 //FM_TMP_HALFBUFSIZE*2
static struct FM_header fm_header;
static char FM_tmp_buf[FM_TMP_HALFBUFSIZE] ={ 0 };
static char* fm =0;
extern int KillAllProcess(void);
extern void dmt_write_cmd(int cmd);
#endif

static int get_firmware_size (int ejid, webs_t wp, int argc, char_t **argv){
#ifdef _NON_POWER_2_
		return websWrite(wp,T("%d"),firmwareUpdateStorageLen);
#else	
	unsigned long image_size;
	FILE *updateFd;
  	updateFd = fopen("/tmp1/sysFlashImage.bin","rb");
	if (updateFd){
		fseek(updateFd, 0L, SEEK_END);
		image_size=ftell(updateFd);
//		DPRINTF("firmware image size=%lu\n", image_size);
		fclose(updateFd);
		
		return websWrite(wp,T("%lu"),image_size);
	}
	else
		return websWrite(wp,T("0"));
#endif	
}

// the ram1 disk is only for configuraiton restoration
int img_f = 0;

static void MinusOp(short* src1, char* src2, char* dest)
{
//char tmp=0;
char* c=0;
c = (char*) src1;
//printf("src1 = 0x%08x\n",(*src1)&0xffff);
//swap the short's bytes
//tmp = c[1];
//c[1] = c[0];
//c[0] = tmp;
*(dest) = (char)( *src1 - (short)(*src2 & 0xff) );
//printf("dest = %d, src1=%d, src2=%d\n",(int)(*dest & 0xff),(int)(*src1 & 0xffff),(int)(*src2 & 0xff));

}


static int flash_update(FILE *image)
{
	//unsigned long len=0, mtd4_size, image_size;
	unsigned long len=0, mtd_size, image_size;
	FILE *ofp;
	FILE *fud;
	FILE *fmg;
	char_t buf[0x4000]={0};
	char_t buff[12]={0};
	char str_wan[20];
	char buf1[64];
	int kk;
	int m_sum=0;
	int super_bdsize =0;
	int	super_mysum = 0;
	int i;
#ifdef _NON_POWER_2_
	unsigned long currentPos=0;
#endif
	firmwareUpdateStorageStart =(volatile unsigned char*)(firmwareUpdateStorage+FM_header_SIZE);
	if(!firmwareUpdateStorage)
	{
		DPRINTF("ERROR !!flash_update: No valid firmwareUpdateStorage\n");
		return -1;
	}
	if(!configurationIsRestored)
	{
		if(firmwareUpdateStorageLen<=30000)
		{
			DPRINTF("ERROR !!flash_update: firmwareUpdateStorageLen(%ld)<=30000 for update firmware\n",firmwareUpdateStorageLen);
			return -1;
		}
	}
	else
	{// only for configuraiton restore in power-2 versions.
		////mount_ram1();
		//run_script("/etc/scripts/cp.sh",0,0,0);
		
		isclean_config=0; // Julian set isclean_config flag = 0 means use current config when restore config 20050603
	
		if (!(img_f = open ("/tmp/sysFlashImage.bin",O_WRONLY|O_APPEND|O_CREAT ,0644))) {
			// memcpy(buf,"open temp file error\n",20);
			//write(fp,buf,20);
			DPRINTF("open upload file error for configuration restoration\n");
			return -1;
		}
		if((firmwareUpdateStorage)||(firmwareUpdateStorageLen<=30000))
		{
			write(img_f, firmwareUpdateStorage, firmwareUpdateStorageLen);
			close(img_f);
			DPRINTF("OK to write to ram1 for configuration\n");
		}
		else
		{
			DPRINTF("open upload file error for configuration restoration  222\n");
			close(img_f);
			return -1;
		}
	}
	/* check if it is config   */
	strcpy(str_wan,(char *)read_file_para_value(VAR_SYS_STATUS_FILE,"WAN=",buf1));

	if (strcmp(str_wan,"bridge")==0)
	{

		run_script("/etc/scripts/purebridged_stop.sh",0,0,0);
	
		run_script("/etc/scripts/purebridged_init.sh",0,0,0);

	}
	
	//ShutDown(0);
	//read_mtd4_size(&mtd4_size);
	read_mtd_size(AC_KERNEL_MTD_NUM,&mtd_size);
	DPRINTF("\n mtd size %08X \n", mtd_size);
	//image_size=firmwareUpdateStorageLen;
	image_size=firmwareUpdateStorageLenStart;
	memset(&fm_header,0,FM_header_SIZE);
	memcpy(&fm_header,firmwareUpdateStorage,FM_header_SIZE);
	DPRINTF("fm_header.version = %s\n",fm_header.version);
	DPRINTF("fm_header.FB_id = %c\n",fm_header.FB_id[0]);
	DPRINTF("fm_header.checksum = %d\n",fm_header.checksum);
	DPRINTF("fm_header.bdsize = %d\n",fm_header.bdsize);
	DPRINTF("fm_header.mg_id = %x\n",fm_header.mg_id[0]);
	fm = (char*)(firmwareUpdateStorageStart);
//	printf("firmwareUpdateStorage = 0x%08x\n",*(volatile unsigned int*)firmwareUpdateStorage);
	
//		DPRINTF("fm[0]=%x\n",fm[image_size-2]);
//		DPRINTF("fm[1]=%x\n",fm[image_size-1]);
//		DPRINTF("fm[2]=%x\n",fm[image_size]);
//		DPRINTF("fm[3]=%x\n",fm[image_size+1]);
//		DPRINTF("fm[0]=%x\n",fm[image_size-3]);
	fmg = fopen("/proc/GPIOUSER/flashtype", "r");
	if(fmg == 0)
	{
		DPRINTF("flash_type not set!\n");
	}
	else
	{
		fscanf(fmg, "%d\n", &kk);
		DPRINTF("flash type = %d\n", kk );
		fclose(fmg);
		DPRINTF("fm_header.flash_type = %d\n", fm_header.flash_type[0]);
		if(kk > 0)
		{
			if(((int)fm_header.flash_type[0]==0xf) || ((int)fm_header.flash_type[0]<kk))
			{
				DPRINTF("\n\nIncoming firmware do not support the FLASH onboard!!\n\n");
				return -4;
			}
		}
	}
	if(fm_header.FB_id[0] == 'F')
	{
			DPRINTF("fm_header.mg_id=%d\n",fm_header.mg_id[0]);
			//DPRINTF("fm_header.mg_id=%x\n",fm_header.mg_id[0]);
			fmg = fopen("/proc/GPIOUSER/myguard","r");
			if(fmg == 0)
			{
				DPRINTF("myguard bit not detect!\n");
			}
			else
			{
				fscanf(fmg,"%d\n",&kk);
				DPRINTF("myguard bit = %d\n",kk);
				fclose(fmg);

			DPRINTF("fm_header.mg_id=%d\n",fm_header.mg_id[0]);
				if(kk != fm_header.mg_id[0])
				{
					DPRINTF("myguard protection!\n");
					return -2;
				}
			}
//			printf("flash.mg_id=%d\n",*(char*)(KERNEL1_START_ADDRESS+0x32));
//			printf("flash.fh_product_id=%d\n",*(char*)(KERNEL1_START_ADDRESS+0x68));
			
			fmg = fopen("/proc/GPIOUSER/productid","r");
			if(fmg == 0)
			{
				DPRINTF("product id not set!\n");
			}
			else
			{
				fscanf(fmg,"%d\n",&kk);
				DPRINTF("product id = %d\n",kk);
				fclose(fmg);

			DPRINTF("fm_header.product_id=%d\n",fm_header.fh_product_id);
				if(kk != fm_header.fh_product_id)
				{
					DPRINTF("product id protection!\n");
					return -4;
				}
			}
CP_FIRMWARE:		
		i=0;
		if(memcmp(fm_header.signature,"abcde1234567890",FM_SIGNATURE_SIZE))
			i++;
		//check checksum
		for(kk=0;kk<image_size;kk++)
		{
			m_sum += (int)(fm[kk] & 0xff);
			//m_sum += (int)(firmwareUpdateStorageStart[kk] & 0xff);
		}
//		DPRINTF("fm[0]=%x\n",fm[image_size-2]);
//		DPRINTF("fm[1]=%x\n",fm[image_size-1]);
//		DPRINTF("fm[2]=%x\n",fm[image_size]);
//		DPRINTF("fm[3]=%x\n",fm[image_size+1]);
//		DPRINTF("fm[0]=%x\n",fm[image_size-3]);
		DPRINTF("fm_header.checksum = %d\n",fm_header.checksum);
		DPRINTF("m_sum = %d\n",m_sum);
		for(kk=0;kk<0x100;kk++);
		if(m_sum != fm_header.checksum)
			i++;
		
		if(i ==0)
		{
			ofp=NULL;
		ofp=fopen(AC_KERNEL_MTD,"wb");
		}
		else
		//wrong file firmware format
		{
			DPRINTF("wrong firmware format");
			return -1;
	 	}

		DPRINTF("image_size = 0x%08x(%d)\n",image_size,image_size);
		DPRINTF("fm = 0x%08x\n",(unsigned int)fm);
		DPRINTF("fm_header.bdsize=%d\n",fm_header.bdsize);
		DPRINTF("fm_header.fh_total_size=%d\n",fm_header.fh_total_size);
		
		
		for(kk=0;kk<FM_TMP_HALFBUFSIZE;kk++)
			MinusOp((short*)fm+kk,(char*)fm+FM_TMP_BUFSIZE+kk,FM_tmp_buf+kk);

		memcpy((void*)(fm),FM_tmp_buf,FM_TMP_HALFBUFSIZE);
		memcpy((void*)(fm+FM_TMP_HALFBUFSIZE),(char*)fm+FM_TMP_BUFSIZE,fm_header.bdsize-FM_TMP_HALFBUFSIZE);

		for(kk=0;kk<FM_TMP_HALFBUFSIZE;kk++)
			MinusOp((short*)((char*)fm+FM_TMP_HALFBUFSIZE+fm_header.bdsize)+kk,(char*)fm+FM_TMP_HALFBUFSIZE+fm_header.bdsize+FM_TMP_BUFSIZE+kk,FM_tmp_buf+kk);
		
		memcpy((void*)(fm+fm_header.bdsize),FM_tmp_buf,FM_TMP_HALFBUFSIZE);
		memcpy((void*)(fm+fm_header.bdsize+FM_TMP_HALFBUFSIZE),(char*)fm+FM_TMP_HALFBUFSIZE+fm_header.bdsize+FM_TMP_BUFSIZE,image_size - fm_header.bdsize - FM_TMP_BUFSIZE - FM_TMP_HALFBUFSIZE);
		image_size -= FM_TMP_BUFSIZE;
//	DPRINTF("firmwareUpdateStorage = 0x%08x\n",*(volatile unsigned int*)firmwareUpdateStorage);
//	DPRINTF("firmwareUpdateStorage addr= 0x%08x\n",(unsigned int)firmwareUpdateStorage);
		DPRINTF("image_size = 0x%08x(%d)\n",image_size,image_size);
		DPRINTF("fm = 0x%08x\n",(unsigned int)fm);
		//for new header, we write header to flash
		image_size += FM_header_SIZE;
		fm -= FM_header_SIZE;
		DPRINTF("image_size = 0x%08x(%d)\n",image_size,image_size);
		DPRINTF("fm = 0x%08x\n",(unsigned int)fm);
	}
	else if(fm_header.FB_id[0] == 'B')
//	if(image_size <= 0x30000)
	{
		DPRINTF("bootrom update not support!\n");
		return 0;
	}
	else if(fm_header.FB_id[0] == 'S')
	{
		DPRINTF("superload detect!\n");
			super_bdsize = fm_header.bdsize;
			super_mysum = image_size;
			printf("super_bdsize = %d, super_mysum = %d\n",super_bdsize,super_mysum);
		//write bootrom to mtd0 and mtd3
		//mtd0:
		fmg=fopen(AC_BOOTROM1_MTD,"wb");

			if(fmg == NULL)
			{
				DPRINTF("%s open failed!\n",AC_BOOTROM1_MTD);
				return -1;
			}
			DPRINTF("%s open ok!\n",AC_BOOTROM1_MTD);

		for(kk=0;kk<8;kk++)
		{
			DPRINTF("%x ",*(fm+kk));
		}
		fwrite(fm, 0x2000, 1, fmg);
		fclose(fmg);	

		fmg=fopen(AC_BOOTROM2_MTD,"wb");

			if(fmg == NULL)
			{
				DPRINTF("%s open failed!\n",AC_BOOTROM2_MTD);
				return -1;
			}
			DPRINTF("%s open ok!\n",AC_BOOTROM2_MTD);
			
		//mtd3:
		for(kk=0;kk<8;kk++)
		{
			DPRINTF("%x ",*(fm+0x2000+kk));
		}
		fwrite(fm+0x2000,0x4000,1,fmg);
		fwrite(fm+0x6000,0x4000,1,fmg);
		fwrite(fm+0xa000,0x4000,1,fmg);
		fwrite(fm+0xe000,0x4000,1,fmg);
		
		fclose(fmg);

			
//			for(k=0;k<0x100;k++);
//			my_sum = my_sum - 64 - 64 - super_bdsize;
			image_size = image_size - FM_header_SIZE - super_bdsize;
	memset(&fm_header,0,FM_header_SIZE);
	memcpy(&fm_header,firmwareUpdateStorage+FM_header_SIZE+super_bdsize,FM_header_SIZE);
	DPRINTF("fm_header.fh_total_size=%d\n",fm_header.fh_total_size);
	//		for(k=0;k<FM_header_SIZE;k++)
			//for(k=0;k<FM_TMP_HALFBUFSIZE;k++)
	//			tmpb[k] = *(volatile unsigned char*)(TMP_KERNELZ_ADDRESS+64+firm_start-9 +TOTAL_SIZE +super_bdsize +k);
				//tmpb[k] = *(volatile unsigned char*)(TMP_KERNELZ_ADDRESS+64+firm_start-9 +64 +super_bdsize +k);
//			for(k=0;k<0x100;k++);
			printf("bdsize = 0x%08x\n",fm_header.bdsize);
			fm = (char*)(firmwareUpdateStorage+FM_header_SIZE+super_bdsize+FM_header_SIZE);
//			fm = (char*)(TMP_KERNELZ_ADDRESS+64+firm_start-9+ TOTAL_SIZE +super_bdsize + TOTAL_SIZE);
//			for(k=0;k<0x100;k++);
//			printf("fm assign= 0x%08x\n",(unsigned int)fm);
//			for(k=0;k<0x100;k++);
//
//			copy bootrom region to "prepare to put bootrom" region
			//for(k=0;k<super_bdsize;k++)
			//	*(volatile unsigned char*)(TMP_KERNELZ_ADDRESS+64+firm_start-9 +super_mysum+16+k) = *(volatile unsigned char*)(TMP_KERNELZ_ADDRESS+64+firm_start-9 +TOTAL_SIZE+k);
				//*(volatile unsigned char*)(TMP_KERNELZ_ADDRESS+64+firm_start-9 +super_mysum+16+k) = *(volatile unsigned char*)(TMP_KERNELZ_ADDRESS+64+firm_start-9 +64+64+k);

			goto CP_FIRMWARE;

	}
	else
	{
		DPRINTF("start restore config to %s\n",AC_CONFIG_MTD);
		run_script("/etc/scripts/admag.sh",0,0,0);
		fud = fopen("/tmp/admag_rst","rb");
		if(fud==NULL)
		{
			DPRINTF("/tmp/admag_rst open failed\n");
			return -1;
		}
		fseek(fud,0L, SEEK_SET);
		fread(buff,12,1,fud);
		fseek(fud,0L, SEEK_SET);
		if(!strncmp(buff,"abcdefg01234",12))
		{
			ofp=fopen(AC_CONFIG_MTD, "wb");
			if(ofp == NULL)
			{
				DPRINTF("%s open failed!\n",AC_CONFIG_MTD);
				return -1;
			}
			DPRINTF("%s open ok!\n",AC_CONFIG_MTD);
			image = fud;
		}
		else
		{
			DPRINTF("not config.cfg format!\n");
			return -1;
		}
	}
//	else
//	{
//		ofp=fopen("/dev/mtd4", "wb");
//	}
	if(ofp==NULL) {
		DPRINTF("open %s error\n",AC_KERNEL_MTD);
		return -1;
	}
	
	if(!strncmp(buff,"abcdefg01234",12))
		//len = 0x20000; //config change from 128k to 48k
		len = 0xC000; //config change from 128k to 48k
	// first, write mtd2
	else
	//len = (image_size >= mtd4_size ? mtd4_size : image_size);
	len = (image_size >= mtd_size ? mtd_size : image_size);
	DPRINTF("\n write1 size %08X \n", (unsigned int)len);
	while(len)
	if(len > 0x4000){
		if(!configurationIsRestored)
		{
			fwrite(&(fm[currentPos]), 0x4000, 1, ofp);
			//fwrite(&(firmwareUpdateStorage[currentPos]), 0x4000, 1, ofp);
			//fwrite(&(firmwareUpdateStorageStart[currentPos]), 0x4000, 1, ofp);
			len =  len  - 0x4000;
			currentPos+=0x4000;
		}
		else
		{
			fread(buf, 0x4000, 1, image);
			fwrite(buf, 0x4000, 1, ofp);
			len =  len  - 0x4000;
		}
		//DPRINTF("0x4000 \n");
	}else{
		memset(buf, 0, 0x4000);
		if(!configurationIsRestored)
		{
			memcpy(buf,&(fm[currentPos]),len);
			//memcpy(buf,&(firmwareUpdateStorage[currentPos]),len);
			//memcpy(buf,&(firmwareUpdateStorageStart[currentPos]),len);
			fwrite(buf, 0x4000, 1, ofp);
			currentPos+=len;
			//DPRINTF("0x%04X \n", len);
			len =  0;
		}
		else
		{
			fread(buf, len, 1, image);
			fwrite(buf, 0x4000, 1, ofp);
			DPRINTF("0x%04X \n", (unsigned int)len);
			len = 0;
		}
	}
	//fflush(ofp);
	fclose(ofp);

	//if(!strncmp(buff,"abcdefg01234",12))
	if(configurationIsRestored)
		goto update_suc;


	// later, write mtd5
	ofp=NULL;
	ofp=fopen(AC_APPS_MTD, "wb");
        if(ofp==NULL) {
                DPRINTF("open %s error\n",AC_APPS_MTD);
                return -1;
        }
		
	//len = (image_size >= mtd4_size ? image_size - mtd4_size : 0 );
	len = (image_size >= mtd_size ? image_size - mtd_size : 0 );
	for(kk=0;kk<8;kk++)
		DPRINTF("%x ",fm[currentPos+kk]);
		//DPRINTF("%x ",firmwareUpdateStorage[currentPos+kk]);
		//DPRINTF("%x ",firmwareUpdateStorageStart[currentPos+kk]);
	DPRINTF("\n");
	DPRINTF("\n write2 size %08X \n", (unsigned int)len);
	while(len)
	if(len > 0x4000){
#ifdef _NON_POWER_2_

		fwrite(&(fm[currentPos]), 0x4000, 1, ofp);
		//fwrite(&(firmwareUpdateStorage[currentPos]), 0x4000, 1, ofp);
		//fwrite(&(firmwareUpdateStorageStart[currentPos]), 0x4000, 1, ofp);
		len =  len  - 0x4000;
		currentPos+=0x4000;
#else
		fread(buf, 0x4000, 1, image);
		fwrite(buf, 0x4000, 1, ofp);
		len =  len  - 0x4000;
#endif
		//DPRINTF("mtd3:0x4000 \n");
	}else{
		memset(buf, 0, 0x4000);
#ifdef _NON_POWER_2_
		memcpy(buf,&(fm[currentPos]),len);
		//memcpy(buf,&(firmwareUpdateStorage[currentPos]),len);
		//memcpy(buf,&(firmwareUpdateStorageStart[currentPos]),len);
		fwrite(buf, 0x4000, 1, ofp);
		//DPRINTF("mtd3:0x%04X \n", len);
		len =  0;
#else
		fread(buf, len, 1, image);
		fwrite(buf, 0x4000, 1, ofp);
		DPRINTF("mtd5:0x%04X \n", (unsigned int)len);
		len = 0;
#endif
	}

update_suc:
	DPRINTF("\n    upgrade completely\n");
	//fflush(ofp);
	fclose(ofp);
	DPRINTF("\n    upgrade successfully\n");
	DPRINTF("  ------------------------------\n");
	return 0;
}

#define UPDATE_DEFAULT
static int do_upgrade_firmware (int ejid, webs_t wp, int argc, char_t **argv)
{
	int rc;
	char buf1[65], str_factory[10];

	DPRINTF("\nPrint do_upgrade_firmware or restore config");
#ifndef _NON_POWER_2_
	FILE *updateFd,*fp_grade;	  
	pid_t pid,status;
#endif	


	/* Child */
	/*if (( pid = vfork() ) < 0 ) 
	{
		//DPRINTF("fork error1\n");
	}
	else if ( pid == 0 )  
	{ */
	/* make file rwxrwxrwx to avoid any problems with running as scripts */
#ifdef _NON_POWER_2_
	if(firmwareUpdateStorage==NULL)
	{
		DPRINTF("update: firmwareUpdateStorage== NULL\n");
		return websWrite(wp, T("%d"),-3);
	}
 	DPRINTF("calling flash_update\n");
	rc = flash_update(NULL);
	
#else
 	chmod("/tmp1/sysFlashImage.bin", 
          S_IRUSR | S_IWUSR | S_IXUSR |
          S_IRGRP | S_IWGRP | S_IXGRP |
          S_IROTH | S_IWOTH | S_IXOTH );
  	updateFd = fopen("/tmp1/sysFlashImage.bin","rb");
 	if (updateFd == NULL)
   	{
		DPRINTF("updateFd == NULL\n");
		fp_grade = fopen(UPGRADE_CONFIG_FILE, "w");
    		fprintf(fp_grade, "#:0:\n");
    		fclose(fp_grade);
	}
 	else
 	{
 		DPRINTF("calling flash_update\n");
		rc = flash_update(updateFd);
		fclose(updateFd);	
	}
#endif  	
	//DPRINTF("rc=%d\n",rc);
	if ((rc==-2)||(rc==-4)){
		free(firmwareUpdateStorage);
		return websWrite(wp, T("%d"),rc);
	}else if(rc==-1){
		free(firmwareUpdateStorage);
	           return websWrite(wp, T("%d"),rc);
	}

#ifdef UPDATE_DEFAULT
	//strcpy(str_factory,(char *)read_file_para_value(DEFAULT_CONFIG_FILE,"FACTORY=",buf1));
	//if(strcmp("yes", str_factory)==0)
	if(isclean_config==1)
	  clear_config();
#endif

	//unlink("/tmp1/sysFlashImage.bin");
	//umount("/tmp1");
	//DPRINTF("umount ok\n");
	//sleep(8);
		
	//wait for profile action
	sleep(3);	
	DPRINTF("Reboot System\n");
	//reboot system 
        kill(1, SIGTERM);	
/*	exit(1);
	}
	//if ( ( pid = waitpid ( pid , &status , 0) ) < 0 )   
	 	DPRINTF("waitpid error:%d\n",getpid()); */
	return 1;
}

static int clear_mtd(const char *mtd_path, unsigned long size)
{
	FILE *fp=NULL;
	unsigned long i=0;
	char buf[1]= {0xff};

	fp=fopen(mtd_path, "wb");
	DPRINTF("clear %s\n", mtd_path);
	if(fp){
	  DPRINTF("clear_mtd start\n");
		for(i=0; i< size; i++)
			fwrite(buf, 1, 1, fp);
		fclose(fp);
	  DPRINTF("clear_mtd final\n");
	}
	return 0;
}

void clear_config()
{
	unsigned long size=0;
	DPRINTF("Clean default config start!!\n");
	read_mtd_size(AC_CONFIG_MTD_NUM, &size);
	DPRINTF("%s=%04X\n",AC_CONFIG_MTD, size);
	clear_mtd(AC_CONFIG_MTD, size);
	DPRINTF("Clean default config OK!!\n");
}
static void form_update_factory_conf(webs_t wp, char_t* path, char_t* query) 
{
	//julian add for check if clean config or not when upgrading 05252005
	char_t *buf=NULL;
        buf=websGetVar(wp, T("factory_v"), "");
        isclean_config=atoi(buf);

        DPRINTF("Set factory reset flag : %d\n",isclean_config);
	
	websRedirect(wp, T("update.asp"));

}
static void form_update_conf(webs_t wp, char_t* path, char_t* query) 
{
	
//	do_upgrade_firmware ();
//	websRedirect(wp, T("update_progress.asp"));
	websRedirect(wp, T("update_frame.asp"));

}
static void form_update_complete(webs_t wp, char_t* path, char_t* query)
{
	char_t *buf=NULL;
	FILE *fp;
	int factory_buf;
	char_t buf1[64];
	char_t str_lanip[20];
	char_t str_lanmask[20];
	char_t str_gatewayname[20];
	strcpy(str_lanip,(char *)read_file_para_value(DEFAULT_CONFIG_FILE,"LAN_IP=",buf1));
	strcpy(str_lanmask,(char *)read_file_para_value(DEFAULT_CONFIG_FILE,"LAN_MASK=",buf1));
	strcpy(str_gatewayname,(char *)read_file_para_value(DEFAULT_CONFIG_FILE,"GATEWAY_NAME=",buf1));
	buf=websGetVar(wp, T("factory"), "");
        factory_buf=atoi(buf);
	if (factory_buf==1)
		flash_write();
	if (factory_buf==2){
		flash_reset();
		if((fp = fopen(HOSTS_CONFIG_FILE, "w"))!=NULL){
			fprintf(fp, "%s %s %s 0\n", 
				str_lanip, 
				str_lanmask, 
				str_gatewayname 
	          	);
	        fclose(fp);
		}
		if((fp = fopen(EMBEDDED_CONFIG_FILE, "w"))!=NULL){
			fprintf(fp, "80 0.0.0.0 180\n"); 
	        fclose(fp);
		}
	}
	websRedirect(wp, T("rbnosv_apply.asp"));
}

static int get_factory_value (int ejid, webs_t wp, int argc, char_t **argv){
	return websWrite(wp,T("%d"),isclean_config);
}
void DefineWebFunctions_upgrade() 
{
	websAspDefine ( T("do_upgrade_firmware"), do_upgrade_firmware);
//	websAspDefine ( T("upgrade_redirect"), upgrade_redirect);
	websAspDefine ( T("get_firmware_size"), get_firmware_size);
	websAspDefine ( T("get_factory_value"), get_factory_value);
	
  	websFormDefine ( T("update"), form_update_conf);
  	websFormDefine ( T("factory"), form_update_factory_conf);
  	websFormDefine ( T("updatecomplete"), form_update_complete);
 }          
