/*
 *    upnp.htm: upnp configuration
 */
#include "webs.h"
#include "upnp.h"
#include "wan.h"
#include "../iptables-1.2.11/runFunc.h"

extern void run_task(char *cmd, ...);
extern char* read_file_para_value(char *filename, char *arg1,char *buf);
                                                                                
static int get_upnp_status (int ejid, webs_t wp, int argc, char_t **argv);
static int get_upnp_mod (int ejid, webs_t wp, int argc, char_t **argv);
static int get_upnp_protocol (int ejid, webs_t wp, int argc, char_t **argv);
                                                                                
static void form_upnp_conf(webs_t wp, char_t * path,char_t * query);
                                                                                
static void resetCurrentUpnpCfg(void);
static void Load_Upnp_Config (void);
static void Save_Upnp_Config (UPNP_CONFIG_DATA *d);
extern void run_task(char *cmd, ...);

static void resetCurrentUpnpCfg(void)
{
	int i;
	UPNP_CONFIG_DATA *d = &upnp_config;   
	for(i=0;i<MaxUpnpNumber;i++)
	{
		strcpy(d->protocol[i],"");
		strcpy(d->description[i],"");
             	strcpy(d->ipaddress[i],"");
	     	strcpy(d->enable[i],"");
	     	strcpy(d->ext_port[i],"");
	     	strcpy(d->int_port[i],"");
	}
			     
}
static void Load_Upnp_Config (void)
{
        FILE *fp;
	int i;
        char_t line[255];
        resetCurrentUpnpCfg();
        if((fp = fopen(UPNP_CONFIG_FILE, "r"))!=NULL){
	for(i=0;i<MaxUpnpNumber;i++)
	{
		
		if (fgets(line, 255, fp))
		{
			if (strcmp(line,"|?upnp_ex|End of File|")!=0)
			{
        	       sscanf(line, "|forward_port%d|%20[^-]-%20[^>]>%20[^:]:%20[^-]-%20[^,],%20[^,],%20[^,],%20[^|]|\n",
				&i,
			       	upnp_config.ext_port[i],
			       	upnp_config.ext_port[i],
			       	upnp_config.ipaddress[i],
			       	upnp_config.int_port[i],
			       	upnp_config.int_port[i],
			       	upnp_config.protocol[i],
			       	upnp_config.enable[i],
			       	upnp_config.description[i]
			       );         
			}
			
		}
		//DPRINTF("i:%d\n",i);
		//DPRINTF("upnp_config.ext_port[%d]:%s\n",i,upnp_config.ext_port[i]);
		//DPRINTF("upnp_config.ipaddress[%d]:%s\n",i,upnp_config.ipaddress[i]);
		//DPRINTF("upnp_config.int_port[%d]:%s\n",i,upnp_config.int_port[i]);
		//DPRINTF("upnp_config.protocol[%d]:%s\n",i,upnp_config.protocol[i]);
		//DPRINTF("upnp_config.enable[%d]:%s\n",i,upnp_config.enable[i]);
		//DPRINTF("upnp_config.str_enable[%s][%d]:%s\n",i,strlen(str_enable),str_enable);
	//	DPRINTF("upnp_config.description[%d]:%s\n",i,upnp_config.description[i]);
	}
	fclose(fp);
	}
}
static void Save_Upnp_Config (UPNP_CONFIG_DATA *d)
{
        FILE *fp;
	int i;
	if((fp=fopen(UPNP_CONFIG_FILE,"w"))!=NULL){
	for(i=0;i<MaxUpnpNumber;i++)
	{
			
		if (strlen(d->description[i])!=0)	
		fprintf(fp,"|forward_port%d|%s-%s>%s:%s-%s,%s,%s,%s|\n",
			i,
			d->ext_port[i],
			d->ext_port[i],
			d->ipaddress[i],
			d->int_port[i],
			d->int_port[i],
			d->protocol[i],
			d->enable[i],
			d->description[i]
		       );
	}	
			fprintf(fp,"|?upnp_ex|End of File|");
	fclose(fp);
	}
}

static int get_upnp_mod (int ejid, webs_t wp, int argc, char_t **argv)
{
	int i;
	Load_Upnp_Config();
	i=atoi(argv[0]);
	if (strcmp(upnp_config.enable[i],"on")==0)
		return websWrite(wp,T("checked"));
	else
		return websWrite(wp,T(""));
}
static int get_upnp_status (int ejid, webs_t wp, int argc, char_t **argv)
{
	int i=atoi(argv[1]);
	Load_Upnp_Config();
	if (strcmp("description",argv[0])==0)
		return websWrite(wp,T("%s"),upnp_config.description[i]);
	else if (strcmp("ext_port",argv[0])==0)
			return websWrite(wp,T("%s"),upnp_config.ext_port[i]);
	else if (strcmp("int_port",argv[0])==0)
			return websWrite(wp,T("%s"),upnp_config.int_port[i]);
	else if (strcmp("ipaddress",argv[0])==0)
		return websWrite(wp,T("%s"),upnp_config.ipaddress[i]);
	return 1;
}

static int get_upnp_protocol (int ejid, webs_t wp, int argc, char_t **argv)
{
	int i,j,k;	
	Load_Upnp_Config();
	i=atoi(argv[0]);
	k=atoi(argv[1]);
	//DPRINTF("protocol[%d]:%s\n",i,upnp_config.protocol[i]);
	if (strcmp(upnp_config.protocol[i],"udp")==0) 
		j=1;
	else
		j=0;
	//DPRINTF("i:j:k:%d,%d,%d\n\n",i,j,k);
	if (k==j) 
		return websWrite(wp,T("selected"));
	else
		return websWrite(wp,T(""));
		
}
                                                                                
static void form_upnp_conf(webs_t wp, char_t * path,char_t * query)
{
	int i;
	char_t *buf=NULL;
	Load_Upnp_Config();
	buf=websGetVar(wp, T("enablevalue"), "");
        for (i=0;i<15;i++){
		strcpy(upnp_config.enable[i],"");
		strcpy(upnp_config.description[i],"");
		strcpy(upnp_config.protocol[i],"");
		strcpy(upnp_config.ext_port[i],"");
		strcpy(upnp_config.int_port[i],"");
		strcpy(upnp_config.ipaddress[i],"");
	}
	sscanf(buf,"%s %s %s %s %s %s %s %s %s %s %s %s %s %s %s\n",
		upnp_config.enable[0],
		upnp_config.enable[1],
		upnp_config.enable[2],
		upnp_config.enable[3],
		upnp_config.enable[4],
		upnp_config.enable[5],
		upnp_config.enable[6],
		upnp_config.enable[7],
		upnp_config.enable[8],
		upnp_config.enable[9],
		upnp_config.enable[10],
		upnp_config.enable[11],
		upnp_config.enable[12],
		upnp_config.enable[13],
		upnp_config.enable[14]
	        );
       
	buf=websGetVar(wp, T("descriptionvalue"), "");
        sscanf(buf,"%s %s %s %s %s %s %s %s %s %s %s %s %s %s %s\n",
		upnp_config.description[0],
		upnp_config.description[1],
		upnp_config.description[2],
		upnp_config.description[3],
		upnp_config.description[4],
		upnp_config.description[5],
		upnp_config.description[6],
		upnp_config.description[7],
		upnp_config.description[8],
		upnp_config.description[9],
		upnp_config.description[10],
		upnp_config.description[11],
		upnp_config.description[12],
		upnp_config.description[13],
		upnp_config.description[14]
		);
	
	buf=websGetVar(wp, T("protocol"), "");
        sscanf(buf,"%s %s %s %s %s %s %s %s %s %s %s %s %s %s %s\n",
		upnp_config.protocol[0],
		upnp_config.protocol[1],
		upnp_config.protocol[2],
		upnp_config.protocol[3],
		upnp_config.protocol[4],
		upnp_config.protocol[5],
		upnp_config.protocol[6],
		upnp_config.protocol[7],
		upnp_config.protocol[8],
		upnp_config.protocol[9],
		upnp_config.protocol[10],
		upnp_config.protocol[11],
		upnp_config.protocol[12],
		upnp_config.protocol[13],
		upnp_config.protocol[14]
	        );
	 
	buf=websGetVar(wp, T("ext_portvalue"), "");
        sscanf(buf,"%s %s %s %s %s %s %s %s %s %s %s %s %s %s %s\n",
		upnp_config.ext_port[0],
		upnp_config.ext_port[1],
		upnp_config.ext_port[2],
		upnp_config.ext_port[3],
		upnp_config.ext_port[4],
		upnp_config.ext_port[5],
		upnp_config.ext_port[6],
		upnp_config.ext_port[7],
		upnp_config.ext_port[8],
		upnp_config.ext_port[9],
		upnp_config.ext_port[10],
		upnp_config.ext_port[11],
		upnp_config.ext_port[12],
		upnp_config.ext_port[13],
		upnp_config.ext_port[14]
	        );
	
	buf=websGetVar(wp, T("int_portvalue"), "");
        sscanf(buf,"%s %s %s %s %s %s %s %s %s %s %s %s %s %s %s\n",
		upnp_config.int_port[0],
		upnp_config.int_port[1],
		upnp_config.int_port[2],
		upnp_config.int_port[3],
		upnp_config.int_port[4],
		upnp_config.int_port[5],
		upnp_config.int_port[6],
		upnp_config.int_port[7],
		upnp_config.int_port[8],
		upnp_config.int_port[9],
		upnp_config.int_port[10],
		upnp_config.int_port[11],
		upnp_config.int_port[12],
		upnp_config.int_port[13],
		upnp_config.int_port[14]
	        );
	
	buf=websGetVar(wp, T("ipaddressvalue"), "");
        sscanf(buf,"%s %s %s %s %s %s %s %s %s %s %s %s %s %s %s\n",
		upnp_config.ipaddress[0],
		upnp_config.ipaddress[1],
		upnp_config.ipaddress[2],
		upnp_config.ipaddress[3],
		upnp_config.ipaddress[4],
		upnp_config.ipaddress[5],
		upnp_config.ipaddress[6],
		upnp_config.ipaddress[7],
		upnp_config.ipaddress[8],
		upnp_config.ipaddress[9],
		upnp_config.ipaddress[10],
		upnp_config.ipaddress[11],
		upnp_config.ipaddress[12],
		upnp_config.ipaddress[13],
		upnp_config.ipaddress[14]							                	       );
	 for (i=0;i<15;i++){
		if (strcmp(upnp_config.enable[i],"1")==0)
			strcpy(upnp_config.enable[i],"on");
		else
			strcpy(upnp_config.enable[i],"off");
		if (strcmp(upnp_config.description[i],"&")==0)
			strcpy(upnp_config.description[i],"");
		if (strcmp(upnp_config.ext_port[i],"&")==0)
			strcpy(upnp_config.ext_port[i],"");
		if (strcmp(upnp_config.int_port[i],"&")==0)
			strcpy(upnp_config.int_port[i],"");
		if (strcmp(upnp_config.ipaddress[i],"&")==0)
			strcpy(upnp_config.ipaddress[i],"");
	}
	printf(funcNum,"%d",FUNCTION_NUM_UPNP);
	run_task("/usr/sbin/runFunc","/usr/sbin/runFunc", "stop",funcNum,NULL);
	Save_Upnp_Config(&upnp_config);
	if(pfilterIfAnyRulesActive(&PF_Table))
	{// UPnP No need a valid WAN_IP to run
		run_task("/usr/sbin/runFunc","/usr/sbin/runFunc", "start",funcNum,NULL);
	}
	websRedirect(wp, T("upnp.asp"));
}
void DefineWebFunctions_upnp()
{
	        websAspDefine(T("get_upnp_status"), get_upnp_status);
	        websAspDefine(T("get_upnp_protocol"), get_upnp_protocol);
	        websAspDefine(T("get_upnp_mod"), get_upnp_mod);
	        websFormDefine ( T("upnp"), form_upnp_conf);
}

