/*
 *  wan_adv.htm: WAN Configuration
 */
#include "webs.h"
#include "lan_conf.h"
#include "wan.h"
#include "wan_vod.h"
#include "locale.h"
#include "../iptables-1.2.11/runFunc.h" // KenWang added 2006/01/02
#include "vlan.h" // KenWang added 2006/03/01
#define R_ADMIN_LOGIN "admin"

OTHER_WAN_BRIDGE_DATA other_wan_bridge_data;
KNOWN_VOD_TYPE_LIST known_vod_type_list;
//#define _DPRINTF_
#ifdef _DPRINTF_
FILE* TTYS0;
#define DPRINTF(args...) \
	TTYS0=fopen("/dev/ttyS0","w"); \
	fprintf(TTYS0,args); \
	fclose(TTYS0) 
#else
	#define DPRINTF(args...)  
#endif /* _DPRINTF_ */

extern char *WAN_ADV_L1[];
extern char *Route_L2[];
extern void read_wan_pvc (int *Wan_Vpi, int *Wan_Vci);
extern int Load_Vlan_Config (VLAN_CONFIG_DATA *d);
extern void vlan_link_wanbr(VLAN_CONFIG_DATA *d, int vlan_index);
static int currentEditingWanConnectionIndex=0;
void resetCurrentWanVodCfg(OTHER_WAN_BRIDGE_DATA *d)
{
     memset(d,0,sizeof(OTHER_WAN_BRIDGE_DATA));
}

// albert, 2005.02.21
// albert, 2005.02.18
// | 0         | 1 | 2 |  3     |      4      |   5        |      6           |    
// |Description|vpi|vci|atmclass|encapsulation|ether filter|spanning interface|
// |1483_Bridge_mode|0|0|0|0|0|0|
void Load_Other_Wan_Bridge_Config (OTHER_WAN_BRIDGE_DATA *d)
{
	FILE *fp;
	char_t line[1024];
	int i;
	
	//DPRINTF("Load_Other_Wan_Bridge_Config is called\n");
	resetCurrentWanVodCfg(d);
	if ((fp=fopen(OTHER_WAN_BRGIDE_CONFIG_FILE, "r"))==NULL) 
		return;
	//1483 Bridged only for current stage
	fscanf(fp,"%d\n",&d->wan_bridge_no);
	if(!d->wan_bridge_no)
	{
		fclose(fp);
		return;
	}
	for(i=0;i<d->wan_bridge_no;i++)
	{
		if (fgets(line, 1024, fp))
		{
			sscanf(line,
			"|%50[^|]|%d|%hu|%hu|%d|%d|%d|%d|\n",
			 d->Bridged_Description[i],
			 &d->usingSamePVCFromExistingWanConnection[i],
			 &d->Bridged_VPI[i],
			 &d->Bridged_VCI[i],
			 (int *)&d->Bridged_ATM_Class[i],
			 (int *)&d->Bridged_Encapsulation_Method[i],
			 (int *)&d->Bridged_Ether_Filter_Type[i],
			 (int *)&d->Bridged_Spanning_Bridge_Interface[i]
		       );
		       print_blank_string(&d->Bridged_Description[i]);
		      // DPRINTF("desc %s\n", d->Bridged_Description[i]);
		       		     //DPRINTF("usingSamePVCFromExistingWanConnection %d\n", d->usingSamePVCFromExistingWanConnection[i]);
		       		     //DPRINTF("vpi %d\n", d->Bridged_VPI[i]);
				     //DPRINTF("vci %d\n", d->Bridged_VCI[i]);
				     //DPRINTF("class %d\n", d->Bridged_ATM_Class);
				     //DPRINTF("enc %d\n", d->Bridged_Encapsulation_Method);
				    // DPRINTF("eth %d\n", d->Bridged_Ether_Filter_Type);
				    // DPRINTF("spa %d\n", d->Bridged_Spanning_Bridge_Interface);
		}
		else
			break;
	}
	fclose(fp);
}
void Save_Other_Wan_Bridge_Config (OTHER_WAN_BRIDGE_DATA *d)
{
	FILE *fp;
	int i;
	
	if ((fp=fopen(OTHER_WAN_BRGIDE_CONFIG_FILE, "w"))==NULL) 
		return;
	//1483 Bridged only for current stage
	fprintf(fp,"%d\n",d->wan_bridge_no);
	for(i=0;i<d->wan_bridge_no;i++)
	{
	     load_blank_string(&d->Bridged_Description[i]);
	     fprintf(fp,"|%s|%d|%hu|%hu|%d|%d|%d|%d|\n",
		     d->Bridged_Description[i],
		     d->usingSamePVCFromExistingWanConnection[i],
		     d->Bridged_VPI[i],
		     d->Bridged_VCI[i],
		     d->Bridged_ATM_Class[i],
		     d->Bridged_Encapsulation_Method[i],
		     d->Bridged_Ether_Filter_Type[i],
		     d->Bridged_Spanning_Bridge_Interface[i]
		  );
	}
	fclose(fp);
}

static int read_other_wan_bridge (int ejid, webs_t wp, int argc, char_t **argv)
{
	char_t Wan_Name[30];
	int nBytes=0,i;
	OTHER_WAN_BRIDGE_DATA *d=&other_wan_bridge_data;
	
	char_t *TableFmt_Wan0="<tr>\n"
	                "\t<td><nobr>%s</nobr></td>\n"
		        "\t\t<td><nobr>%s</nobr></td>\n"
			"\t\t<td><nobr>%s</nobr></td>\n"
			"\t\t<td><nobr>%d</nobr></td>\n"
			"\t\t<td><nobr>%d</nobr></td>\n";
	char_t *TableFmt_Wanx_non_editable="\t\t<td><nobr>&nbsp;</nobr></td>\n"
			"<form name=wanbdel%d action=/goform/wan_br_del method=post>\n"
			"<INPUT type=hidden name=del_wanbr_id value=%d>\n"
			"<td><nobr><a href=\"javascript:wanbdel_submit(%d)\">%s<img align=middle border=0 src=/link1.gif></a></nobr></td>\n"	
			"</form></tr>\n";
	char_t *TableFmt_Wanx_editable="<form name=wanbedit%d action=/goform/wan_br_edit method=post>\n"
			"<INPUT type=hidden name=edit_wanbr_id value=%d>\n"	
			"<td><nobr><a href=\"javascript:wanbedit_submit(%d)\">%s<img align=middle border=0 src=/link1.gif></a></nobr></td>\n"
			"</form>\n"
			"<form name=wanbdel%d action=/goform/wan_br_del method=post>\n"
			"<INPUT type=hidden name=del_wanbr_id value=%d>\n"
			"<td><nobr><a href=\"javascript:wanbdel_submit(%d)\">%s<img align=middle border=0 src=/link1.gif></a></nobr></td>\n"	
			"</form></tr>\n";
	
	Load_Other_Wan_Bridge_Config(d);
	if(!d->wan_bridge_no)
	{
	  return websWrite(wp, T(""));
	}
	for(i=0;i<d->wan_bridge_no;i++)
	{
		int vpi,vci;
		int index;
		sprintf(Wan_Name,"WAN Bridge Connection #%d",i+1);
		index=d->usingSamePVCFromExistingWanConnection[i];
		if(index)
		{
			if(index==1)// main WAN
			{
				read_wan_pvc(&vpi,&vci);
			}
			else
			{
				vpi=d->Bridged_VPI[index-2],
				vci=d->Bridged_VCI[index-2];
			}
		}
		else
		{
		    vpi=d->Bridged_VPI[i];
		    vci=d->Bridged_VCI[i];
		}
		nBytes+=websWrite(wp, T(TableFmt_Wan0),
		    Wan_Name,
		    d->Bridged_Description[i],
		    R_ADMIN_LOGIN,
		    vpi,
		    vci);
		if(d->usingSamePVCFromExistingWanConnection[i])
			nBytes+=websWrite(wp, T(TableFmt_Wanx_non_editable),
			i+1,
			i+1,
			i+1,
			Route_L2[locale_config.locale]);
		else
			nBytes+=websWrite(wp, T(TableFmt_Wanx_editable),
			i+1,
			i+1,
			i+1,
		    	WAN_ADV_L1[locale_config.locale],
			i+1,
			i+1,
			i+1,
			Route_L2[locale_config.locale]);
	}
	return nBytes;
}

int other_wan_bridge_no(void)
{
	OTHER_WAN_BRIDGE_DATA *d=&other_wan_bridge_data;
	
	Load_Other_Wan_Bridge_Config(d);
	return d->wan_bridge_no;
}

static int read_other_wan_bridge_select (int ejid, webs_t wp, int argc, char_t **argv)
{
#if 0
	int nBytes=0,i;
	OTHER_WAN_BRIDGE_DATA *d=&other_wan_bridge_data;
	
	Load_Other_Wan_Bridge_Config(d);
	if(!d->wan_bridge_no)
	{
	  return websWrite(wp, T(""));
	}
	for(i=0;i<d->wan_bridge_no;i++)
	{
		nBytes+=websWrite(wp, T("<OPTION value=%d >WAN Bridge Connection #%d</option>"),3+i,i+1);
	}
	return nBytes;
#else
	return websWrite(wp, T(""));
#endif
}

int wanb_max_reach (int ejid, webs_t wp, int argc, char_t **argv)
{
  	int i=0;
	OTHER_WAN_BRIDGE_DATA *d=&other_wan_bridge_data;
	Load_Other_Wan_Bridge_Config(d);
	if(d->wan_bridge_no<MAX_OTHER_WAN_BRIDGE)
	{
	  return websWrite(wp, T(""));
	}
	
	i=atoi(argv[0]);
 	if(i==0)//comment begin
		return websWrite(wp, T("<!--"));
	else
		return websWrite(wp, T("-->"));
}

void get_other_wan_br_PVC_data(int wanbr,int *vpi,int *vci)
{
	OTHER_WAN_BRIDGE_DATA *d=&other_wan_bridge_data;
	int index=wanbr;
	
	Load_Other_Wan_Bridge_Config(d);
	if (d->usingSamePVCFromExistingWanConnection[wanbr])
	{
		index=d->usingSamePVCFromExistingWanConnection[wanbr]-1;
		if (index==0)
		{// the main one with currently settings. THe current main PVC values
			read_wan_pvc(vpi,vci);
		}
		return;
	}
	*vpi=d->Bridged_VPI[index];
	*vci=d->Bridged_VCI[index];
}

static int get_wan_conn_no(int ejid, webs_t wp, int argc, char_t **argv)
{
	OTHER_WAN_BRIDGE_DATA *d=&other_wan_bridge_data;
	int isEdit=atoi(argv[0]);
	Load_Other_Wan_Bridge_Config(d);
	if(isEdit)
		return websWrite(wp, T("%d"),d->wan_bridge_no);// at least the main WAN 
	else
		return websWrite(wp, T("%d"),d->wan_bridge_no+1);// at least the main WAN 
}

static int get_wan_conn_pvc(int ejid, webs_t wp, int argc, char_t **argv)
{
	int wan_vpi=0;
	int wan_vci=0;
	int nbyte=0;
	int i;
	OTHER_WAN_BRIDGE_DATA *d=&other_wan_bridge_data;
	
	read_wan_pvc(&wan_vpi,&wan_vci);
	Load_Other_Wan_Bridge_Config(d);
	int isEdit=atoi(argv[1]);
	if (strcmp(argv[0],"VPI")==0)
		nbyte+=websWrite(wp, T("%d"),wan_vpi);
	else
		nbyte+=websWrite(wp, T("%d"),wan_vci);
	if(d->wan_bridge_no)
	{
		for(i=0;i<d->wan_bridge_no;i++)
		{
			if((isEdit)&&((currentEditingWanConnectionIndex-1)==i))
			{
				// skip the current edit ones.
			}
			else
			{
				if (strcmp(argv[0],"VPI")==0)
					nbyte+=websWrite(wp, T(",%d"),d->Bridged_VPI[i]);
				else
					nbyte+=websWrite(wp, T(",%d"),d->Bridged_VCI[i]);
			}
		}
	}	
	return nbyte;
}	

static int get_other_wan_br_data(int ejid, webs_t wp, int argc, char_t **argv)
{
	OTHER_WAN_BRIDGE_DATA *d=&other_wan_bridge_data;
	int i;
	
	if (!currentEditingWanConnectionIndex)
		return websWrite(wp, T(""));
	if (strcmp(argv[0],"ID")==0)
		return websWrite(wp, T("%d"),currentEditingWanConnectionIndex);
	Load_Other_Wan_Bridge_Config(d);
	i=currentEditingWanConnectionIndex-1;
	if (strcmp(argv[0],"WBD")==0)
		return websWrite(wp, T("%s"),d->Bridged_Description[i]);
	if (strcmp(argv[0],"WB_VPI")==0)
		return websWrite(wp, T("%d"),d->Bridged_VPI[i]);
	if (strcmp(argv[0],"WB_VCI")==0)
		return websWrite(wp, T("%d"),d->Bridged_VCI[i]);
	if (strcmp(argv[0],"WBEM")==0)
	{
		int j=atoi(argv[1]);
		if(j>=2)
		{
			// print out the Encapsultion in string
			if (d->Bridged_Encapsulation_Method[i]==0)
				return websWrite(wp, T("LLC Bridged"));
			else
				return websWrite(wp, T("VcMux Bridged"));
		}
		else
		{// OTherwise choose the encapsulation type with printed "checked" or ""
			if (d->Bridged_Encapsulation_Method[i]==j)
				return websWrite(wp, T("selected"));
			else
				return websWrite(wp, T(""));
		}
	}
	return websWrite(wp, T(""));
}

void form_wanb_create_conf(webs_t wp, char_t * path, char_t * query)
{
	OTHER_WAN_BRIDGE_DATA *d=&other_wan_bridge_data;
	char_t *buf=NULL;
	char_t line[32]; // KenWang added 2006/01/02
	
	Load_Other_Wan_Bridge_Config(d);
	if(d->wan_bridge_no<MAX_OTHER_WAN_BRIDGE)
	{
		buf=websGetVar(wp, T("WBD"), "");
		strcpy(d->Bridged_Description[d->wan_bridge_no],buf);
        
    		buf=websGetVar(wp, T("wan_conn"), "1");
        	d->usingSamePVCFromExistingWanConnection[d->wan_bridge_no]=atoi(buf)-1;
 
	      	if(!d->usingSamePVCFromExistingWanConnection[d->wan_bridge_no])
		{
			buf=websGetVar(wp, T("WB_VPI"), "0");
			d->Bridged_VPI[d->wan_bridge_no]=atoi(buf);
		
        		buf=websGetVar(wp, T("WB_VCI"), "0");
			d->Bridged_VCI[d->wan_bridge_no]=atoi(buf);
	
        		buf=websGetVar(wp, T("WBEM"), "0");
			d->Bridged_Encapsulation_Method[d->wan_bridge_no]=atoi(buf);
		}
		d->wan_bridge_no++;

		Save_Other_Wan_Bridge_Config(d);
		/* KenWang added 2006/01/02 -- begin -- */
		if(d->usingSamePVCFromExistingWanConnection[d->wan_bridge_no-1] == 1) {
			read_file_para_value(VAR_SYS_STATUS_FILE, "WAN=", line);
			if(!strcmp(line, "bridge")) {
				// save_globe_config(SYS_STATUS_FILE, "WAN_IF=", "br0");
				/* nothing need to do because WAN_IF is br0 as we wish it to be */
			}
			else {
				save_globe_config(SYS_STATUS_FILE, "WAN_IF=", "brwan");
				flash_write();
				websRedirect(wp, T("rbnosv_apply.asp"));
			}
		}
		else {
			sprintf(line, "%d", FUNCTION_NUM_OTHER_WAN_BRIDGE);
			sprintf(line+10, "o%d", d->wan_bridge_no+1);
			run_task("runFunc", "runFunc", "start", line, line+10, NULL);
		}
		/* KenWang added 2006/01/02 -- end -- */
	}
	websRedirect(wp, "wan_adv.asp");
}
void form_wanb_edit_conf(webs_t wp, char_t * path,char_t * query)
{
	OTHER_WAN_BRIDGE_DATA *d=&other_wan_bridge_data;
	char_t *buf=NULL;
	int i=atoi(websGetVar(wp, T("conn"), "0"));
	char_t line[32]; // KenWang added 2006/01/02
	/* KenWang added 2006/03/01 link vlan to wanbr -- begin -- */
	VLAN_CONFIG_DATA d1;
	int j, k;
	/* KenWang added 2006/03/01 link vlan to wanbr -- end -- */
        if(i)
	{
		Load_Other_Wan_Bridge_Config(d);
		if((d->wan_bridge_no)&&(i<=d->wan_bridge_no))
		{
			buf=websGetVar(wp, T("WBD"), "");
			strcpy(d->Bridged_Description[i-1],buf);
        
 			buf=websGetVar(wp, T("wan_conn"), "1");
			d->usingSamePVCFromExistingWanConnection[i-1]=atoi(buf)-1;
 
	      		if(!d->usingSamePVCFromExistingWanConnection[i-1])
			{
				buf=websGetVar(wp, T("WB_VPI"), "0");
				d->Bridged_VPI[i-1]=atoi(buf);
	
        			buf=websGetVar(wp, T("WB_VCI"), "0");
				d->Bridged_VCI[i-1]=atoi(buf);
	
        			buf=websGetVar(wp, T("WBEM"), "0");
				d->Bridged_Encapsulation_Method[i-1]=atoi(buf);
			}
			
			Save_Other_Wan_Bridge_Config(d);
			/* KenWang added 2006/01/02 -- begin -- */
			sprintf(line, "%d", FUNCTION_NUM_OTHER_WAN_BRIDGE);
			sprintf(line+10, "o%d", i+1);
			run_task("runFunc", "runFunc", "stop", line, line+10, NULL);
			run_task("runFunc", "runFunc", "start", line, line+10, NULL);
			/* KenWang added 2006/01/02 -- end -- */
			Load_Vlan_Config(&d1);
			k = 0;
			for(j=0; j<MAX_RULE; j++) {
				if(d1.Rule[j].WAN_connection_index == i+1 ||
				   d->usingSamePVCFromExistingWanConnection[d1.Rule[j].WAN_connection_index-2] == i+1) {
					if(d1.Rule[i].WAN_TAGGING_enable) {
						if(k == 0) {
							sprintf(line, "br%d", d1.Rule[i].WAN_connection_index);
							sprintf(line+10, "nas%d", d1.Rule[i].WAN_connection_index);
							run_task("brctl", "brctl", "delif", line, line+10, NULL);
							k = 1;
						}
						sprintf(line, "nas%d", d1.Rule[i].WAN_connection_index);
						sprintf(line+10, "%d", d1.Rule[i].VLAN_ID);
						run_task("vconfig", "vconfig", "add", line, line+10, NULL);
					}
					vlan_link_wanbr(&d1, j);
				}
			}
		}
	}
	websRedirect(wp, "wan_adv.asp");
}

int mainWanHasOtherWanBridgeUseTheSamePVC(void)
{
	OTHER_WAN_BRIDGE_DATA *d=&other_wan_bridge_data;
	int i;
	Load_Other_Wan_Bridge_Config(d);
	if(d->wan_bridge_no)
	{
		for(i=0;i<d->wan_bridge_no;i++)
		{
			if(d->usingSamePVCFromExistingWanConnection[i]==1)
			{//other Bridged WAN conenction has use the same PVC as Main Wan Connection
				return 1;
			}
		}
	}
	return 0;
}

extern int wanConnectionRemoveNotif(int removedWanConnectionIndex);

void form_wanb_delete_conf(webs_t wp, char_t * path,char_t * query)
{
	OTHER_WAN_BRIDGE_DATA *d=&other_wan_bridge_data;
	int i=atoi(websGetVar(wp, T("conn"), "0"));
	int j;
	char line[32]; // KenWang added 2006/01/02
	
        if(i)
	{
		Load_Other_Wan_Bridge_Config(d);
		if((d->wan_bridge_no)&&(i<=d->wan_bridge_no))
		{
			//We then have to remove the link that vlan group associated to this WAN connection interface
			//wanConnectionRemoveNotif(i+1);// the first one is MAIN(=1). So, the first WANB to be deleted is 1+1=2
			if(d->wan_bridge_no==i)
			{// i is the last one we want to delete
			}
			else
			{
				//shift, the other behide us to left.
				for(j=i-1;j<d->wan_bridge_no-1;j++)
				{
					strcpy(d->Bridged_Description[j],d->Bridged_Description[j+1]);
					d->usingSamePVCFromExistingWanConnection[j]=d->usingSamePVCFromExistingWanConnection[j+1];
					d->Bridged_VPI[j]=d->Bridged_VPI[j+1];
					d->Bridged_VCI[j]=d->Bridged_VCI[j+1];
					d->Bridged_ATM_Class[j]=d->Bridged_ATM_Class[j+1];
					d->Bridged_Encapsulation_Method[j]=d->Bridged_Encapsulation_Method[j+1];
					d->Bridged_Ether_Filter_Type[j]=d->Bridged_Ether_Filter_Type[j+1];
					d->Bridged_Spanning_Bridge_Interface[j]=d->Bridged_Spanning_Bridge_Interface[j+1];
				}
			}
			d->wan_bridge_no-=1;

			/* KenWang added 2006/01/02 -- begin -- */
			sprintf(line, "%d", FUNCTION_NUM_OTHER_WAN_BRIDGE);
			sprintf(line+10, "u%d", i+1);
			run_task("runFunc", "runFunc", "stop", line, line+10, NULL);
			Save_Other_Wan_Bridge_Config(d);
			if(d->usingSamePVCFromExistingWanConnection[d->wan_bridge_no] == 1) {
				read_file_para_value(VAR_SYS_STATUS_FILE, "WAN=", line);
				if(!strcmp(line, "bridge")) {
					// save_globe_config(SYS_STATUS_FILE, "WAN_IF=", "br0");
					/* nothing need to do because WAN_IF is br0 as we wish it to be */
				}
				else {
					save_globe_config(SYS_STATUS_FILE, "WAN_IF=", "nas0");
					flash_write();
					websRedirect(wp, "rbnosv_apply.asp");
					return;
				}
			}
			else {
				run_task("runFunc", "runFunc", "start", line, line+10, NULL);
				wanConnectionRemoveNotif(i+1);
			}
		}
	}
	//Save_Other_Wan_Bridge_Config(d);
	websRedirect(wp, "wan_adv.asp");
}

void form_wan_br_edit_asp(webs_t wp, char_t * path,char_t * query)
{
	int i=atoi(websGetVar(wp, T("edit_wanbr_id"), "0"));
	//Set Current Edit one for later wan_br_edit.asp reference
	currentEditingWanConnectionIndex=i;
	websRedirect(wp, "wan_br_edit.asp");
}

void form_wan_br_del_asp(webs_t wp, char_t * path,char_t * query)
{
	int i=atoi(websGetVar(wp, T("del_wanbr_id"), "0"));
	//Set Current Edit one for later wan_br_edit.asp reference
	currentEditingWanConnectionIndex=i;
	websRedirect(wp, "wan_br_del.asp");
}


extern void write_ripd_config_Internal(void);
extern void dhcp_change_internal(int enabled);
void resetCurrentVodTypeCfg(KNOWN_VOD_TYPE_LIST *d)
{
     memset(d,0,sizeof(KNOWN_VOD_TYPE_LIST));
}
void Load_Known_VOD_Type (KNOWN_VOD_TYPE_LIST *d)
{
	FILE *fp;
	char_t line[1024];
	int i;
	
	//DPRINTF("Load_Other_Wan_Bridge_Config is called\n");
	resetCurrentVodTypeCfg(d);
	if ((fp=fopen(KNOWN_VOD_TYPE_LIST_CONFIG_FILE, "r"))==NULL) 
		return;
	fscanf(fp,"%d\n",&d->previous_selected_vod_type_no);
	fscanf(fp,"%d\n",&d->known_vod_type_no);
	if(!d->known_vod_type_no)
	{
		fclose(fp);
		return;
	}
	for(i=0;i<d->known_vod_type_no;i++)
	{
		if (fgets(line, 1024, fp))
		{
			sscanf(line,
			"|%d|%128[^|]|%d|%100[^|]|%100[^|]|%d|%d|\n",
			 &d->pvc_no[i],d->description[i],
			 &d->indexToWanConfigFile[i],
			 d->WAN[i],
			 d->WAN_IF[i],
			 &d->tcwd[i],&d->autoScanNeed[i]);
		       print_blank_string(&d->description[i]);
		       print_blank_string(&d->WAN[i]);
		       print_blank_string(&d->WAN_IF[i]);
		      // DPRINTF("desc %s\n", d->Bridged_Description[i]);
		       		     //DPRINTF("usingSamePVCFromExistingWanConnection %d\n", d->usingSamePVCFromExistingWanConnection[i]);
		       		     //DPRINTF("vpi %d\n", d->Bridged_VPI[i]);
				     //DPRINTF("vci %d\n", d->Bridged_VCI[i]);
				     //DPRINTF("class %d\n", d->Bridged_ATM_Class);
				     //DPRINTF("enc %d\n", d->Bridged_Encapsulation_Method);
				    // DPRINTF("eth %d\n", d->Bridged_Ether_Filter_Type);
				    // DPRINTF("spa %d\n", d->Bridged_Spanning_Bridge_Interface);
		}
		else
			break;
	}
	fclose(fp);
}
#if 1
void Save_Known_VOD_Type (KNOWN_VOD_TYPE_LIST *d)
{
	FILE *fp;
	int i;
	
	if ((fp=fopen(KNOWN_VOD_TYPE_LIST_CONFIG_FILE, "w"))==NULL) 
		return;
	fprintf(fp,"%d\n",d->previous_selected_vod_type_no);
	fprintf(fp,"%d\n",d->known_vod_type_no);
	for(i=0;i<d->known_vod_type_no;i++)
	{
	     load_blank_string(&d->description[i]);
	     load_blank_string(&d->WAN[i]);
	     load_blank_string(&d->WAN_IF[i]);
	     fprintf(fp,"|%d|%s|%d|%s|%s|%d|%d|\n",
		     d->pvc_no[i],
		     d->description[i],
		     d->indexToWanConfigFile[i],
		     d->WAN[i],d->WAN_IF[i],d->tcwd[i],d->autoScanNeed[i]);
	}
	fclose(fp);
}
#endif
extern void Save_TCWD_Config (int TC_WD);

static void form_vod_type(webs_t wp, char_t* path, char_t* query)
{
	char_t *buf=NULL;
	int VOD_TYPE;
	char filename[80];
	KNOWN_VOD_TYPE_LIST *d=&known_vod_type_list;
	
	Load_Known_VOD_Type(d);
	buf=websGetVar(wp, T("vod_type"),"1");// index is from 1, instance is from 0.
	VOD_TYPE=atoi(buf)-1; 
	d->previous_selected_vod_type_no=VOD_TYPE;
	
	// So, add a new type, what we need:
	//wan.conf, config, wanbr.conf, vlan.conf, config, dhcpd.conf and also, the tcwd.conf
	// first of all reset the /flash direction.....
        run_task("profile", "profile",  "put", "/etc/default", 0);
        run_task("profile", "profile",  "get", "/flash", 0);
	sleep(3);	

	Save_Known_VOD_Type(d);
	
	//We start to copy the wanx.conf wanbrx.conf and vlanx.conf to wan.conf, wanbr.conf and vlan.conf and adjust the HDCP server and reboot.
	sprintf(filename,"/flash/wan%d.conf",d->indexToWanConfigFile[VOD_TYPE]);
	CopyFile(filename,"/flash/wan.conf");		
	sprintf(filename,"/flash/wanbr%d.conf",d->indexToWanConfigFile[VOD_TYPE]);
	CopyFile(filename,"/flash/wanbr.conf");		
	sprintf(filename,"/flash/vlan%d.conf",d->indexToWanConfigFile[VOD_TYPE]);
	CopyFile(filename,"/flash/vlan.conf");
	write_ripd_config_Internal();
	//pure bridge, then disable the dhcp server
	if (strcmp(d->WAN[VOD_TYPE],"bridge")==0)
	{
		dhcp_change_internal(0);
		save_globe_config(SYS_STATUS_FILE,"ROUTER=","no");
	}
	else
	{
		dhcp_change_internal(1);
		// How about if user want to have the NAT is no for router?, .....
		save_globe_config(SYS_STATUS_FILE,"ROUTER=","nat");
	}
	Save_TCWD_Config(d->tcwd[VOD_TYPE]);
	save_globe_config(SYS_STATUS_FILE,"WAN=",d->WAN[VOD_TYPE]);
	save_globe_config(SYS_STATUS_FILE,"WAN_IF=",d->WAN_IF[VOD_TYPE]);
	sprintf(filename, "%d", VOD_TYPE);
	save_globe_config(SYS_STATUS_FILE,"VOD_TYPE=",filename);

	if(d->autoScanNeed[VOD_TYPE])// Type 1 also turn off the auto PVC because of IP is not released for Markku
	{ 
		// turn on the autoscan.
		save_globe_config("/flash/config","AUTOSCAN=","yes");// If user edit the wan and apply, then autoSCAN PVC shall stop in next reboot.
	}
	else
	{
		// turn off the autoscan 
		save_globe_config("/flash/config","AUTOSCAN=","no");
	}

	flash_write();
	websRedirect(wp, T("rbnosv_apply.asp"));
}

static int known_vod_type (int ejid, webs_t wp, int argc, char_t **argv)
{
	int nBytes=0,i;
	KNOWN_VOD_TYPE_LIST *d=&known_vod_type_list;
	
	Load_Known_VOD_Type(d);
	if(!d->known_vod_type_no)
	{
	  return websWrite(wp, T(""));
	}
	for(i=0;i<d->known_vod_type_no;i++)
	{
		nBytes+=websWrite(wp, T("<option value=%d >%s</option>"),i+1,d->description[i]);
	}
	return nBytes;
}

int known_vod (int ejid, webs_t wp, int argc, char_t **argv)
{
 	KNOWN_VOD_TYPE_LIST *d=&known_vod_type_list;
	int i;
	
	Load_Known_VOD_Type(d);
	if(d->known_vod_type_no)
	{
	  return websWrite(wp, T(""));
	}
 	
	i=atoi(argv[0]);
 	if(i==0)//comment begin
		return websWrite(wp, T("<!--"));
	else
		return websWrite(wp, T("-->"));
}



void DefineWebFunctions_WanVod(void) {
       websAspDefine (T("read_other_wan_bridge"), read_other_wan_bridge);
       websAspDefine (T("read_other_wan_bridge_select"), read_other_wan_bridge_select);
       websAspDefine (T("wanb_max_reach"), wanb_max_reach);
       websAspDefine (T("get_wan_br_data"), get_other_wan_br_data);
       websAspDefine (T("get_wan_conn_pvc"), get_wan_conn_pvc);
       websAspDefine (T("get_wan_conn_no"), get_wan_conn_no);
       websAspDefine (T("known_vod_type"), known_vod_type);
       websAspDefine (T("known_vod"), known_vod);
       websFormDefine(T("wanbcreate"), form_wanb_create_conf);
       websFormDefine(T("wanbdel"), form_wanb_delete_conf);
       websFormDefine(T("wanbedit"), form_wanb_edit_conf);
       websFormDefine(T("wan_br_edit"), form_wan_br_edit_asp);
       websFormDefine(T("wan_br_del"), form_wan_br_del_asp);
       websFormDefine(T("vod_type"), form_vod_type);
}
