/*
 *	Define Wireless LAN Web Pages
 *  wlan.asp
 *  Albert, Nov. 11 2003
 */

/****************************************************************************
 * project include files
 ****************************************************************************/
#include "webs.h"
#include "wlan.h"
#include "wlan_conf.h"

/****************************************************************************
 * private variables
 ****************************************************************************/
/* use for testing */
//char	WEP_Key[WLAN_WEP_NKEYS][WLAN_WEP_MAXKEYLEN];
//char	PassPhraseString[PASS_PHRASE_MAXLEN+1];
//char	ESSID[WLAN_SSID_MAXLEN+1];
//int		Channel_Id;
//int		Encryption;
//int		Key_HexorChar;
//int		Key_Id;
//int		KeyLen=5;
//int		ErrKeyLen=0;
/* end testing */
static char		ESSID[WLAN_SSID_MAXLEN+1]={0};
static int		PassPhraseFlag=0;
static char		WEP_Key[WLAN_WEP_NKEYS][WLAN_WEP_MAXKEYLEN]={{0}};
static int		Encryption=0;
static char		PassPhraseString[PASS_PHRASE_MAXLEN+1]={0};
static int		Channel_Id;
static int		Key_Id;
static int		HideESSID=0;
static int		WLAN_Enable=0;

/****************************************************************************
 * private function prototypes
 ****************************************************************************/
static int Wlan_Config_Load (int eid, webs_t wp, int argc, char_t **argv);
static int Wireless_Enable(int eid, webs_t wp, int argc, char_t **argv);
static int Wireless_ESSID (int eid, webs_t wp, int argc, char_t **argv);
static int Wireless_Channel (int eid, webs_t wp, int argc, char_t **argv);
static int Wireless_Encryption (int eid, webs_t wp, int argc, char_t **argv);
static int Wireless_Key_HexorChar (int eid, webs_t wp, int argc, char_t **argv);
static int Wireless_Key_ID(int eid, webs_t wp, int argc, char_t**argv);
static int Wireless_Hide_ESSID(int eid, webs_t wp, int argc, char_t **argv);
static int Wireless_Key1(int eid, webs_t wp, int argc, char_t **argv);
static int Wireless_Key2(int eid, webs_t wp, int argc, char_t **argv);
static int Wireless_Key3(int eid, webs_t wp, int argc, char_t **argv);
static int Wireless_Key4(int eid, webs_t wp, int argc, char_t **argv);
static int Wireless_PassPhrase(int eid, webs_t wp, int argc, char_t **argv);
static void form_wlan(webs_t wp, char_t *path, char_t *query);
static void nwepgen (char *genstr, int keylen,char wep_key[WLAN_WEP_NKEYS][WLAN_WEP_MAXKEYLEN]);

/*
 *	Wireless LANConfigFunction: wlan.asp
 */
void Define_WLAN_conf()
{
	websAspDefine(T("Wlan_Config_Load"), Wlan_Config_Load);
	websAspDefine(T("Wireless_Enable"), Wireless_Enable);
	websAspDefine(T("Wireless_ESSID"), Wireless_ESSID);
	websAspDefine(T("Wireless_Channel"), Wireless_Channel);
	websAspDefine(T("Wireless_Encryption"), Wireless_Encryption);
	websAspDefine(T("Wireless_Key_HexorChar"), Wireless_Key_HexorChar);
	websAspDefine(T("Wireless_Key_ID"), Wireless_Key_ID);
	websAspDefine(T("Wireless_Hide_ESSID"), Wireless_Hide_ESSID);
	websAspDefine(T("Wireless_Key1"), Wireless_Key1);
	websAspDefine(T("Wireless_Key2"), Wireless_Key2);
	websAspDefine(T("Wireless_Key3"), Wireless_Key3);
	websAspDefine(T("Wireless_Key4"), Wireless_Key4);
	websAspDefine(T("Wireless_PassPhrase"), Wireless_PassPhrase);
	websFormDefine(T("wlan"), form_wlan);
}

/*
 *	Wireless Configuration File Load
 *  function: load config file to memory
 */
static int Wlan_Config_Load (int eid, webs_t wp, int argc, char_t **argv)
{
//	printf("Before wlan config load\n");
//	wlan_config_printf();
	wlan_conf_load();
	printf("After wlan config load\n");
	wlan_config_printf();
	return websWrite(wp, T("<!-- Wlan Config Load  -->"));
}

/*
 *	Wireless Run or Stop
 *  function: show wireless status
 */
static int Wireless_Enable(int eid, webs_t wp, int argc, char_t **argv)
{
	int mode=0;
	int now_mode=0;

	if ( ejArgs(argc, argv, T("%d"), &mode) < 1 ) {
		websError(wp, 400, T("Insufficient args\n"));
		return -1;
	}

	if(PassPhraseFlag) {
		now_mode = WLAN_Enable;
	} else {
		wlan_conf_get_status(&now_mode);
	}

	if ( now_mode == mode )
		return websWrite(wp, "checked");
	else
		return websWrite(wp, "");
}


/*
 *	Wireless ESSID
 *  function: show wireless ESSID
 *		like ESSID = "wlan-linux"
 */
static int Wireless_ESSID(int eid, webs_t wp, int argc, char_t **argv)
{
	char buf[WLAN_SSID_MAXLEN+1]={0};

	if(PassPhraseFlag) {
		strncpy(buf, ESSID, WLAN_SSID_MAXLEN);
	} else {
		wlan_conf_get_essid(buf);
	}
	return websWrite(wp, buf);
}

/*
 *	Wireless Channel
 *  function: show wireless channel
 *  value : starting from 1
 */
static int Wireless_Channel(int eid, webs_t wp, int argc, char_t **argv)
{
	char buf[32]={0};
	int id;

	if(PassPhraseFlag) {
		id = Channel_Id;
	} else {
		wlan_conf_get_channel_id(&id);
	}

	sprintf(buf, "%d", id);
	return websWrite(wp, buf);
}

/*
 *	Wireless Encryption
 *  function: enable WEP or not
 *  	get value from page
 *  	0: disable, 1: enable 64(40)bits, 2: enable 128(104)bits
 */
static int Wireless_Encryption(int eid, webs_t wp, int argc, char_t **argv)
{
	int mode=0;
	int now_mode=0;

	if ( ejArgs(argc, argv, T("%d"), &mode) < 1 ) {
		websError(wp, 400, T("Insufficient args\n"));
		return -1;
	}

	if(PassPhraseFlag) {
		now_mode = Encryption;
	} else {
		wlan_conf_get_encryption(&now_mode);
	}

	if ( now_mode == mode )
		return websWrite(wp, "checked");
	else
		return websWrite(wp, "");
}

/*
 *	Wireless key use HEX or CHAR
 *  function: use HEX or CHAR format
 *		get value from page
 *		0: HEX, 1: CHAR
 */
static int Wireless_Key_HexorChar(int eid, webs_t wp, int argc, char_t **argv)
{
	int mode=0;
	int now_mode=0;

	if ( ejArgs(argc, argv, T("%d"), &mode) < 1 ) {
		websError(wp, 400, T("Insufficient args\n"));
		return -1;
	}

	if(PassPhraseFlag) {
		now_mode = 0;
	} else {
		wlan_conf_get_key_hexorchar(&now_mode);
	}
	if ( now_mode == mode )
		return websWrite(wp, "checked");
	else
		return websWrite(wp, "");
}

/*
 *	Wireless encryption key id
 *  function : use which key
 *		get value from page
 *		0: key1, 1: key2, 2:key3, 3:key4
 */
static int Wireless_Key_ID(int eid, webs_t wp, int argc, char_t **argv)
{
	int mode=0;
	int now_mode=0;

	if ( ejArgs(argc, argv, T("%d"), &mode) < 1 ) {
		websError(wp, 400, T("Insufficient args\n"));
		return -1;
	}

	if(PassPhraseFlag) {
		now_mode = Key_Id;
	} else {
		wlan_conf_get_key_id(&now_mode);
	}
	if ( now_mode == mode )
		return websWrite(wp, "checked");
	else
		return websWrite(wp, "");
}

/*
 *	Wireless ESSID hide or not
 *  function : hide essid or not
 *		get value from page
 *		0: disable, 1: enable
 */
static int Wireless_Hide_ESSID(int eid, webs_t wp, int argc, char_t **argv)
{
	int mode=0;
	int now_mode=0;

	if ( ejArgs(argc, argv, T("%d"), &mode) < 1 ) {
		websError(wp, 400, T("Insufficient args\n"));
		return -1;
	}

	if( PassPhraseFlag ) {
		now_mode = HideESSID;
	} else {
		wlan_conf_get_hidden_essid(&now_mode);
	}
	if ( now_mode == mode )
		return websWrite(wp, "checked");
	else
		return websWrite(wp, "");
}

/*
 *	Wireless encryption key1
 *  function: wireless key1
 *		format by Wireless_Key_HexorChar and Wireless_Encryption
 *		ex. 4142434445 (HEX, 64(40)bits)
 */
static int Wireless_Key1(int eid, webs_t wp, int argc, char_t **argv)
{
	char buf[20]={0};
	char wepkey[WLAN_WEP_MAXKEYLEN+1]={0};
	int hexorchar;
	int i, keylen;

	if(PassPhraseFlag) {
		keylen = (Encryption==1)?5:13;
		for(i=0; i<keylen; i++)
			sprintf(buf+i*2, "%02x", WEP_Key[0][i]);
	}else{
		wlan_conf_get_wep_key(wepkey, 0);
		wlan_conf_get_key_hexorchar(&hexorchar);
		wlan_conf_get_key_len(&keylen);
		if( hexorchar == 0) {
			for(i=0; i<keylen; i++)
				sprintf(buf+i*2, "%02x", wepkey[i]);
		} else {
			memcpy(buf, wepkey, keylen);
		}
	}

	return websWrite(wp, buf);
}

/*
 *	Wireless encryption key2
 *  function: wireless key2
 *		format by Wireless_Key_HexorChar and Wireless_Encryption
 *		ex. ABCDE (CHAR, 64(40)bits)
 */
static int Wireless_Key2(int eid, webs_t wp, int argc, char_t **argv)
{
	char buf[20]={0};
	char wepkey[WLAN_WEP_MAXKEYLEN+1]={0};
	int hexorchar;
	int i, keylen;

	if(PassPhraseFlag) {
		keylen = (Encryption==1)?5:13;
		for(i=0; i<keylen; i++)
			sprintf(buf+i*2, "%02x", WEP_Key[1][i]);
	}else{
		wlan_conf_get_wep_key(wepkey, 1);
		wlan_conf_get_key_hexorchar(&hexorchar);
		wlan_conf_get_key_len(&keylen);
		if( hexorchar == 0) {
			for(i=0; i<keylen; i++)
				sprintf(buf+i*2, "%02x", wepkey[i]);
		} else {
			memcpy(buf, wepkey, keylen);
		}
	}
	return websWrite(wp, buf);
}

/*
 *	Wireless encryption key3
 *  function: wireless key3
 *		format by Wireless_Key_HexorChar and Wireless_Encryption
 *		ex. 30313233343536373839414243 (HEX, 128(104)bits)
 */
static int Wireless_Key3(int eid, webs_t wp, int argc, char_t **argv)
{
	char buf[20]={0};
	char wepkey[WLAN_WEP_MAXKEYLEN+1]={0};
	int hexorchar;
	int i,keylen;

	if(PassPhraseFlag) {
		keylen = (Encryption==1)?5:13;
		for(i=0; i<keylen; i++)
			sprintf(buf+i*2, "%02x", WEP_Key[2][i]);
	}else{
		wlan_conf_get_wep_key(wepkey, 2);
		wlan_conf_get_key_hexorchar(&hexorchar);
		wlan_conf_get_key_len(&keylen);
		if( hexorchar == 0) {
			for(i=0; i<keylen; i++)
				sprintf(buf+i*2, "%02x", wepkey[i]);
		} else {
			memcpy(buf, wepkey, keylen);
		}
	}
	return websWrite(wp, buf);
}

/*
 *	Wireless encryption key4
 *  function: wireless key4
 *		format by Wireless_Key_HexorChar and Wireless_Encryption
 *		ex. 0123456789ABC (CHAR, 128(104)bits)    aaaa
 */
static int Wireless_Key4(int eid, webs_t wp, int argc, char_t **argv)
{
	char buf[20]={0};
	char wepkey[WLAN_WEP_MAXKEYLEN+1]={0};
	int hexorchar;
	int i, keylen;

	if(PassPhraseFlag) {
		keylen = (Encryption==1)?5:13;
		for(i=0; i<keylen; i++)
			sprintf(buf+i*2, "%02x", WEP_Key[3][i]);
	}else{
		wlan_conf_get_wep_key(wepkey, 3);
		wlan_conf_get_key_hexorchar(&hexorchar);
		wlan_conf_get_key_len(&keylen);
		if( hexorchar == 0) {
			for(i=0; i<keylen; i++)
				sprintf(buf+i*2, "%02x", wepkey[i]);
		} else {
			memcpy(buf, wepkey, keylen);
		}
	}
	return websWrite(wp, buf);
}

/*
 *	Wireless use pass phrase
 *	function: show passphrase string
 */
static int Wireless_PassPhrase(int eid, webs_t wp, int argc, char_t **argv)
{
	char str[PASS_PHRASE_MAXLEN+1]={0};

	if(PassPhraseFlag) {
		strcpy(str, PassPhraseString);
	} else {
		wlan_conf_get_passphrase_string(str);
	}
	return websWrite(wp, str);
}

/*
 *	Wireless FORM handle
 */
static void form_wlan(webs_t wp, char_t *path, char_t *query)
{
	char	*buf=NULL;
	int		Key_HexorChar;
	int		KeyLen=0;
	int		ErrKeyLen=0;
	int		i=0;

	buf = websGetVar(wp, T("Wireless_Enable"), T("0"));
	printf("form_wlan %s %s\n", "Wireless_Enable",buf);
	WLAN_Enable = atoi(buf);
	if( WLAN_Enable < 0 || WLAN_Enable > 1) {
		printf("Please choose enable or disable\n");
		WLAN_Enable = 1;
	}

	buf = websGetVar(wp, T("Wireless_ESSID"), T(OEM_Wireless_ESSID));
	printf("form_wlan %s %s\n", "Wireless_ESSID",buf);
	if( strlen(buf) > WLAN_SSID_MAXLEN ) {
		printf("SSID too long\n");
	} else {
		strcpy(ESSID, buf);
	}

	buf = websGetVar(wp, T("Wireless_Channel"), T(OEM_Wireless_Channel));
	printf("form_wlan %s %s\n", "Wireless_Channel",buf);
	Channel_Id = atoi(buf);
	if( Channel_Id < 0 || Channel_Id > 14 ) {
		printf("Channl number error\n");
		Channel_Id = 6;
	}

	buf = websGetVar(wp, T("Wireless_Key_ID"), T(OEM_Wireless_Key_ID));
	printf("form_wlan %s %s\n", "Wireless_Key_ID",buf);
	Key_Id = atoi(buf);
	if( Key_Id < 0 || Key_Id > 3 ) {
		printf("Please choose correct Key Id\n");
		Key_Id = 0;
	}

	buf = websGetVar(wp, T("PassPhrase_Flag"), T(OEM_PassPhrase_Flag));
	printf("form_wlan %s %s\n", "PassPhrase_Flag",buf);
	PassPhraseFlag = atoi(buf);

	buf = websGetVar(wp, T("Wireless_Encryption"), T(OEM_Wireless_Encryption));
	printf("form_wlan %s %s\n", "Wireless_Encryption",buf);
	Encryption = atoi(buf);
	if( Encryption < 0 || Encryption > 2 ) {
		printf("Please choose correct Disable, 64(40)bits or 128(104)bits\n");
		Encryption = 0;
	} else {
		if( Encryption > 0 )
			KeyLen = (Encryption == 1 ? 5:13);
	}
	printf("form_wlan KeyLen %d\n", KeyLen);

	buf = websGetVar(wp, T("Wireless_Key_HexorChar"), T(OEM_Wireless_Key_HexorChar));
	printf("form_wlan %s %s\n", "Wireless_Key_HexorChar",buf);
	Key_HexorChar = atoi(buf);
	if( Key_HexorChar < 0 || Key_HexorChar > 1) {
		printf("Please choose correct Hex or Char\n");
		Key_HexorChar = 0;
	} else {
		Key_HexorChar ? memset( WEP_Key, 0x30, sizeof(WEP_Key)):memset( WEP_Key, 0, sizeof(WEP_Key));
	}

	buf = websGetVar(wp, T("Wireless_PassPhrase"), T(OEM_Wireless_PassPhrase));
	printf("form_wlan %s %s\n", "Wireless_PassPhrase",buf);
	if( strlen(buf) > sizeof(PassPhraseString) ) {
		printf("Input PassPhraseString too long\n");
	} else {
		strcpy(PassPhraseString, buf);
	}

	/* not support */
	buf = websGetVar(wp, T("Wireless_Hide_ESSID"), T(OEM_Wireless_Hide_ESSID));
	printf("form_wlan %s %s\n", "Wireless_Hide_ESSID",buf);
	HideESSID = atoi(buf);
	if( HideESSID < 0 || HideESSID > 1) {
		printf("Input HideESSID incorrect %d\n", HideESSID);
		HideESSID = 0;
	}

	/* PassPhraseFlag:1 enable, 0 disable
	 * if PassPhrase is do , only generate keys and not save
	 */
	if( PassPhraseFlag == 1 && Encryption > 0) {
		nwepgen( PassPhraseString, KeyLen, WEP_Key);
	} else if (PassPhraseFlag == 0) {
		/* Encryption:
		 * 0: disable, 1:64(40)bits, 2:128(104)bits
		 */
		if(Encryption) {
			buf = websGetVar(wp, T("Wireless_Key1"), T(OEM_Wireless_Key1));
			printf("form_wlan %s %s\n", "Wireless_Key1",buf);
			if( Key_HexorChar ) { // 1:char
				if( strlen(buf) > KeyLen ) {
					printf("WEP key1 length %d is incorrect\n", strlen(buf));
					memcpy(WEP_Key[0], buf, KeyLen);
					ErrKeyLen = 1;
				} else {
					memcpy(WEP_Key[0], buf, strlen(buf));
				}
			} else { // 0:hex
				if( strlen(buf) > KeyLen*2 ) {
					printf("WEP key1 length %d is incorrect\n", strlen(buf));
					HexToCharString(buf, WEP_Key[0], KeyLen*2);
				} else {
					HexToCharString(buf, WEP_Key[0], strlen(buf));
				}
			}

			buf = websGetVar(wp, T("Wireless_Key2"), T(OEM_Wireless_Key2));
			printf("form_wlan %s %s\n", "Wireless_Key2",buf);
			if( Key_HexorChar ) { // 1:char
				if( strlen(buf) > KeyLen ) {
					printf("WEP key2 length is incorrect\n");
					memcpy(WEP_Key[1], buf, KeyLen);
					ErrKeyLen = 1;
				} else {
					memcpy(WEP_Key[1], buf, strlen(buf));
				}
			} else { // 0:hex
				if( strlen(buf) > KeyLen*2 ) {
					printf("WEP key2 length is incorrect\n");
					HexToCharString(buf, WEP_Key[1], KeyLen*2);
					ErrKeyLen = 1;
				} else {
					HexToCharString(buf, WEP_Key[1], strlen(buf));
				}
			}

			buf = websGetVar(wp, T("Wireless_Key3"), T(OEM_Wireless_Key3));
			printf("form_wlan %s %s\n", "Wireless_Key3",buf);
			if( Key_HexorChar ) { // 1:char
				if( strlen(buf) > KeyLen ) {
					printf("WEP key3 length is incorrect\n");
					memcpy(WEP_Key[2], buf, KeyLen);
					ErrKeyLen = 1;
				} else {
					memcpy(WEP_Key[2], buf, strlen(buf));
				}
			} else { // 0:hex
				if( strlen(buf) > KeyLen*2 ) {
					printf("WEP key3 length is incorrect\n");
					HexToCharString(buf, WEP_Key[2], KeyLen*2);
					ErrKeyLen = 1;
				} else {
					HexToCharString(buf, WEP_Key[2], strlen(buf));
				}
			}

			buf = websGetVar(wp, T("Wireless_Key4"), T(OEM_Wireless_Key4));
			printf("form_wlan %s %s\n", "Wireless_Key4",buf);
			if( Key_HexorChar ) { // 1:char
				if( strlen(buf) > KeyLen ) {
					printf("WEP key4 length is incorrect\n");
					memcpy(WEP_Key[3], buf, KeyLen);
					ErrKeyLen = 1;
				} else {
					memcpy(WEP_Key[3], buf, strlen(buf));
				}
			} else { // 0:hex
				if( strlen(buf) > KeyLen*2 ) {
					printf("WEP key4 length is incorrect\n");
					HexToCharString(buf, WEP_Key[3], KeyLen*2);
					ErrKeyLen = 1;
				} else {
					HexToCharString(buf, WEP_Key[3], strlen(buf));
					for(i=0; i<13; i++) {
						printf(  (i <12) ? "%02x-" : "%02X\n", WEP_Key[3][i]);
					}
				}
			}
		}// if(Encryption)

		// if the length of key is incorrect, web display original format
		if( ErrKeyLen ) {
			//Key_HexorChar = (Key_HexorChar ? 0 : 1);
			//Encryption = (Encryption==1)
			ErrKeyLen = 0;
		}

		// save configuration files
		wlan_conf_set_enabled(WLAN_Enable);
		wlan_conf_set_essid(ESSID);
		wlan_conf_set_channel_id(Channel_Id);
		wlan_conf_set_encryption(Encryption);
		wlan_conf_set_key_hexorchar(Key_HexorChar);
		wlan_conf_set_key_id(Key_Id);
		wlan_conf_set_passphrase_string(PassPhraseString);
		wlan_conf_set_hidden_essid(HideESSID);
		//wlan_conf_set_version(char ver[WLAN_MAX_VERSION_LENGTH]);
		//wlan_conf_set_beacon_rate(100);
		if(Encryption) {
			wlan_conf_set_key_len(KeyLen);
			for( i=0; i<4; i++)
				wlan_conf_set_wep_key(WEP_Key[i], i);
		}
		printf("---submit--\n");
		wlan_config_printf();
		printf("---submit end--\n");
		wlan_conf_save();
	}/* if( PassPhraseFlag == 1 && Encryption > 0) */

	websRedirect(wp, T("/wlan.asp"));
}

/*----------------------------------------------------------------
* nwepgen
*
* Generates a set of WEP keys from a generator string.  This is
* intended as a convenience.  Entering hex bytes can be a pain.
*
* Based on an algorithm supplied by Neesus Datacom,
* http://www.neesus.com
*
* This function was authored by Zoom Telephonics Engineer
* Juan Arango.
* http://www.zoomtel.com
*
* Juan's Note:
* Changing the code in this function could make this product
* incompatible with other ZoomAir wireless products because
* these other products rely on Microsoft's rand() and srand()
* function implementations!!!  This code uses the same algorithm.
*
* Distributed with permission from Zoom Telephonics.
*
* Arguments:
* 	genstr		a null terminated string
*	keylen		number of bytes in key
* 	wep_key		a 2d array that is filled with the wep keys
* Returns:
*	nothing
----------------------------------------------------------------*/

void nwepgen(char *genstr, int keylen, char wep_key[WLAN_WEP_NKEYS][WLAN_WEP_MAXKEYLEN])
{
	unsigned int i,j;
	unsigned char pseed[4]={0,0,0,0};
	unsigned int len;
	int randNumber=0;

	len = strlen(genstr);
	if (len) {
		/* generate seed for random number generator using */
		/* key string... */
		for (i=0; i<len; i++) {
			pseed[i%4]^= genstr[i];
		}

		/* init PRN generator... note that this is equivalent */
		/*  to the Microsoft srand() function. */
		randNumber =	(int)pseed[0] |
				((int)pseed[1])<<8 |
				((int)pseed[2])<<16 |
				((int)pseed[3])<<24;

		/* generate keys. */
		for (i=0; i<WLAN_WEP_NKEYS; i++) {
			for (j=0; j<keylen; j++) {
				/* Note that these three lines are */
				/* equivalent to the Microsoft rand() */
				/* function. */
				randNumber *= 0x343fd;
				randNumber += 0x269ec3;
				wep_key[i][j] = (unsigned char)((randNumber>>16) & 0x7fff);
			}
		}
	}
	return;
}
