/*
 *	Define Wireless LAN Configuration
 *  wlan.asp
 *  Albert, Nov. 11 2003
 */

/****************************************************************************
 * standard include files
 ****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <linux/if.h>
#include <netinet/in.h>
#include <arpa/inet.h>

/****************************************************************************
 * project include files
 ****************************************************************************/
#include "wlan_conf.h"
#include "program_config.h"

/****************************************************************************
 * private variables
 ****************************************************************************/
static WLAN_CONFIG_S wlan_config;

/****************************************************************************
 * private function prototypes
 ****************************************************************************/
static void wlan_config_init();
static int create_wlan_conf_script();
static int wlan_conf_card_state(void);
static void wlan_exec_pr_script(int mode);


/****************************************************************************
 * public functions defined here
 ****************************************************************************/
static void wlan_config_init()
{
	memset( &wlan_config, 0, sizeof(wlan_config) );

	wlan_config.Enabled = wlan_conf_card_state();
	strcpy(wlan_config.Essid, OEM_Wireless_ESSID);
	wlan_config.Channel_Id = atoi(OEM_Wireless_Channel);
	wlan_config.Key_Len = 0;
	//wlan_config.Encryption = 0;
	//wlan_config.Key_HexorChar = 0;
	//wlan_config.WEP_Key[WLAN_WEP_NKEYS][WLAN_WEP_MAXKEYLEN];
	//wlan_config.PassPhraseString[PASS_PHRASE_MAXLEN+1];
	//wlan_config.Hidden_ESSID = 0;
	strncpy( wlan_config.Version, "v2.0", WLAN_MAX_VERSION_LENGTH);
	wlan_config.Beacon_Rate = 100;
}

void wlan_config_printf()
{
	int i, j;

	printf("--------------wlan_config context-------------\n");
	printf("enabled: %d\n", wlan_config.Enabled);
	printf("essid: %s\n", wlan_config.Essid);
	printf("channel: %d\n", wlan_config.Channel_Id);
	printf("encryption: %d\n", wlan_config.Encryption);
	printf("KeyLen: %d\n", wlan_config.Key_Len);
	printf("HexorChar: %d\n", wlan_config.Key_HexorChar);
	printf("KeyId: %d\n", wlan_config.Key_Id);

	for( i=0; i<4; i++) {
		printf("WEP Key%d: ", i);
		for(j=0; j<13; j++) {
			printf(  (j <12) ? "%02x" : "%02X\n", wlan_config.WEP_Key[i][j]);
		}
	}

	printf("PassPhrase: %s\n", wlan_config.PassPhraseString);
	printf("Hidden Essid: %d\n", wlan_config.Hidden_ESSID);
	printf("Version: %s\n", wlan_config.Version);
	printf("Beacon Rate: %lu\n", wlan_config.Beacon_Rate);
	printf("--------------wlan_config context end ----------\n\n");
}

/****************************************************************************
 * Function Name  : wlan_save_config
 * Arguments      : user_data
 * Returns        : -1 fail, 0 success
 * Variables      :
 * Modifies       :
 * Description    : Saves the configuration stored in the global data structure
 ****************************************************************************/
int wlan_conf_save()
{
	FILE *fp;
	char buf[128]={0};
	int i=0,j=0,k=0;
	
	fp = fopen(WLAN1_CONFIG_FILE, "w");
	if (!fp)
	{
		return -1;
	}

	//	1: Enabled
	//	2: Essid
	//	3: Channel_Id
	//	4: Encryption
	//. 5: Key_Len
	//	6: Key_HexorChar
	//. 7: Key_Id
	//	8: WEP_Key1
	//	9: WEP_Key2
	//	10: WEP_Key3
	//	11: WEP_Key4
	//	12: PassPhraseString
	//	13: Version
	//	14: Beacon_Rate

	fprintf(fp, "%d\n", wlan_config.Enabled);

	fprintf(fp, "%s\n", wlan_config.Essid);

	fprintf(fp, "%d\n", wlan_config.Channel_Id);

	fprintf(fp, "%d\n", wlan_config.Encryption);

	fprintf(fp, "%d\n", wlan_config.Key_Len);

	fprintf(fp, "%d\n", wlan_config.Key_HexorChar);

	fprintf(fp, "%d\n", wlan_config.Key_Id);

	for(i=0; i<4; i++) {
		for(j=0; j<13; j++) {
			k += sprintf(buf+k, (j<12)?"%02x":"%02x\n", wlan_config.WEP_Key[i][j]);
		}
	}

	fprintf(fp, buf);

	fprintf(fp, "%s\n", wlan_config.PassPhraseString);
	printf("save pass phrase %s\n", wlan_config.PassPhraseString);

	fprintf(fp, "%d\n", wlan_config.Hidden_ESSID);
	printf("save hidden essid %d\n", wlan_config.Hidden_ESSID);

	fprintf(fp, "%s\n", wlan_config.Version);
	printf("save version %s\n", wlan_config.Version);

	fprintf(fp, "%lu\n", wlan_config.Beacon_Rate);
	printf("save Beacon %lu\n", wlan_config.Beacon_Rate);

	fclose(fp);
	
  	
	create_wlan_conf_script();
	wlan_exec_pr_script(wlan_config.Enabled);
	return 1;
}

/****************************************************************************
 * Function Name  : wlan_load_config
 * Arguments      : user_data
 * Returns        : -1 fail, 0 success
 * Variables      :
 * Modifies       :
 * Description    : Loads the configuration file into wlan_config. Format:
 ****************************************************************************/
int wlan_conf_load()
{
	FILE *fp;
	char buf[64]={0};
	int i;

	wlan_config_init();

	fp = fopen(WLAN1_CONFIG_FILE, "r");
	if (!fp)
	{
		return -1;
	}

	fscanf(fp, "%1d\n", &wlan_config.Enabled);

	fscanf(fp, "%32[0-9a-zA-Z.:-]\n", wlan_config.Essid);

	fscanf(fp, "%d\n", &wlan_config.Channel_Id);

	fscanf(fp, "%d\n", &wlan_config.Encryption);

	fscanf(fp, "%d\n", &wlan_config.Key_Len);

	fscanf(fp, "%d\n", &wlan_config.Key_HexorChar);

	fscanf(fp, "%d\n", &wlan_config.Key_Id);

	for(i=0;i<4;i++) {
		fgets(buf, 64, fp);
		printf("get wep key%d %s", i, buf);
		HexToCharString(buf, wlan_config.WEP_Key[i], 26);
	}

	fgets(wlan_config.PassPhraseString, 64, fp);

	fscanf(fp, "%d\n", &wlan_config.Hidden_ESSID);

	fgets(wlan_config.Version, 64, fp);

	fscanf(fp, "%lu\n", &wlan_config.Beacon_Rate);

	fclose(fp);
	return 0;
}

static int create_wlan_conf_script()
{
	FILE *fp;
	int i=0, j=0, k=0;
	char buf[128]={0};
	
	fp = fopen(WLAN_SCRIPT_FILE, "w");
	if (!fp) {
		return -1;
	}

	fprintf(fp, "#!/bin/sh\n");
	fprintf(fp, "WLAN_ENABLED=%s\n", wlan_config.Enabled ? "true" : "false");
	fprintf(fp, "SSID=%s\n", wlan_config.Essid);
	fprintf(fp, "CHANNEL=%d\n", wlan_config.Channel_Id);
	fprintf(fp, "BCNINT=%lu\n", wlan_config.Beacon_Rate);
	fprintf(fp, "p2CnfEnhSecurity=%d\n", wlan_config.Hidden_ESSID);

	if (wlan_config.Encryption) {
		fprintf(fp,
						"dot11PrivacyInvoked=true\n"
						"dot11ExcludeUnencrypted=true\n"
						"AuthType=\"sharedkey\"\n"
						"PRIV_GENSTR=\"\"\n"
						"PRIV_KEY128=\"%s\"\n",
						(wlan_config.Encryption==1)? "false": "true");
		fprintf(fp, "dot11WEPDefaultID=%d\n", wlan_config.Key_Id);
		for(i=0; i<4; i++) {
		    k=0;
			for(j=0; j < wlan_config.Key_Len; j++) {
				k += sprintf(buf+k, (j<(wlan_config.Key_Len-1))?"%02x:":"%02x", wlan_config.WEP_Key[i][j]);
			}
			fprintf(fp, "dot11WEPDefaultKey%d=%s\n", i, buf);
		}
	}
	else
	{
		fprintf(fp,
						"dot11PrivacyInvoked=false\n"
						"dot11ExcludeUnencrypted=false\n"
						"AuthType=\"opensystem\"\n"
						"PRIV_GENSTR=\"\"\n");
	}
	fclose(fp);
	
  	
	return 0;
}


static void wlan_exec_pr_script(int mode)
{
	int nouse;
	program_load_config(&nouse);
	if ( mode ) { // on
		if (!pr_is_program_activated(r_program_wlan)) {
			pr_activate(r_program_wlan);
			pr_run_program(r_program_wlan, 1);
		} else {
				pr_deactivate(r_program_wlan);
				pr_run_program(r_program_wlan, 0);
				pr_activate(r_program_wlan);
				pr_run_program(r_program_wlan, 1);
		}
	} else { // off
		if (pr_is_program_activated(r_program_wlan)) {
			pr_deactivate(r_program_wlan);
			pr_run_program(r_program_wlan, 0);
		}
    }
	program_save_config(&nouse);
}


/****************************************************************************
 * Function Name  : wlan_on_the_system
 * Arguments      :
 * Returns        : 1 - SUCCESS
 *                  0 - FAILED
 * Variables      :
 * Modifies       :
 * Description    : Checks if the Wirelan is on the system
 ****************************************************************************/
int wlan_conf_on_the_system(void)
{
  struct stat testFile;

  if(lstat("/var/lib/pcmcia/stab", &testFile)==0)
    return 1;

  return 0;
}

static int wlan_conf_card_state(void)
{
	int fd;
	struct ifreq ifr;
	struct stat lstat;

	if (stat("/var/lib/pcmcia/stab", &lstat) != 0)
	{
		return -1;
	}

	fd = socket(PF_INET, SOCK_STREAM, 0);
	if (fd < 0)
	{
		return 0;
	}

	memset((void *)&ifr, '\0', sizeof(struct ifreq));
	memcpy(ifr.ifr_name, "wlan0", IFNAMSIZ);
	if (ioctl(fd, SIOCGIFFLAGS, &ifr) < 0)
	{
		close(fd);
		return 0;
	}
	close(fd);
	if (!(ifr.ifr_flags & IFF_UP))
	{
		return 0;
	}
	return 1;
}

/****************************************************************************
 * Function Name  : wlan_conf_get_status
 * Arguments      :
 * Returns        : 0 - SUCCESS
 *                  -1 - FAILED
 * Variables      :
 * Modifies       :
 * Description    :
 ****************************************************************************/
int wlan_conf_get_status(int *status)
{
	*status = wlan_config.Enabled;
	return 0;
}

/****************************************************************************
 * Function Name  : wlan_conf_get_essid
 * Arguments      :
 * Returns        : 0 - SUCCESS
 *                  -1 - FAILED
 * Variables      :
 * Modifies       :
 * Description    :
 ****************************************************************************/
int wlan_conf_get_essid(char id[WLAN_SSID_MAXLEN+1])
{
	strcpy( id, wlan_config.Essid);
	return 0;
}

/****************************************************************************
 * Function Name  : wlan_conf_get_channel_id
 * Arguments      :
 * Returns        : 0 - SUCCESS
 *                  -1 - FAILED
 * Variables      :
 * Modifies       :
 * Description    :
 ****************************************************************************/
int wlan_conf_get_channel_id(int *id)
{
	*id = wlan_config.Channel_Id;
	return 0;
}

/****************************************************************************
 * Function Name  : wlan_conf_get_encryption
 * Arguments      :
 * Returns        : 0 - SUCCESS
 *                  -1 - FAILED
 * Variables      :
 * Modifies       :
 * Description    :
 ****************************************************************************/
int wlan_conf_get_encryption(int *encryption)
{
	*encryption = wlan_config.Encryption;
	return 0;
}

/****************************************************************************
 * Function Name  : wlan_conf_get_key_len
 * Arguments      :
 * Returns        : 0 - SUCCESS
 *                  -1 - FAILED
 * Variables      :
 * Modifies       :
 * Description    :
 ****************************************************************************/
int wlan_conf_get_key_len(int *len)
{
	*len = wlan_config.Key_Len;
	return 0;
}

/****************************************************************************
 * Function Name  : wlan_conf_get_key_hexorchar
 * Arguments      :
 * Returns        : 0 - SUCCESS
 *                  -1 - FAILED
 * Variables      :
 * Modifies       :
 * Description    :
 ****************************************************************************/
int wlan_conf_get_key_hexorchar(int *hexorchar)
{
	*hexorchar = wlan_config.Key_HexorChar;
	return 0;
}

/****************************************************************************
 * Function Name  : wlan_conf_get_key_id
 * Arguments      :
 * Returns        : 0 - SUCCESS
 *                  -1 - FAILED
 * Variables      :
 * Modifies       :
 * Description    :
 ****************************************************************************/
int wlan_conf_get_key_id(int *id)
{
	*id = wlan_config.Key_Id;
	return 0;
}

/****************************************************************************
 * Function Name  : wlan_conf_get_wep_key
 * Arguments      :
 * Returns        : 0 - SUCCESS
 *                  -1 - FAILED
 * Variables      :
 * Modifies       :
 * Description    :
 ****************************************************************************/
int wlan_conf_get_wep_key(char key[WLAN_WEP_MAXKEYLEN], int keyid)
{
	memcpy(key, wlan_config.WEP_Key[keyid], 13);
	return 0;
}

/****************************************************************************
 * Function Name  : wlan_conf_get_passphrase_string
 * Arguments      :
 * Returns        : 0 - SUCCESS
 *                  -1 - FAILED
 * Variables      :
 * Modifies       :
 * Description    :
 ****************************************************************************/
int wlan_conf_get_passphrase_string(char str[PASS_PHRASE_MAXLEN+1])
{
	strcpy( str, wlan_config.PassPhraseString);
	return 0;
}

/****************************************************************************
 * Function Name  : wlan_conf_get_hidden_essid
 * Arguments      :
 * Returns        : 0 - SUCCESS
 *                  -1 - FAILED
 * Variables      :
 * Modifies       :
 * Description    :
 ****************************************************************************/
int wlan_conf_get_hidden_essid(int *hidden)
{
	*hidden = wlan_config.Hidden_ESSID;
	return 0;
}

/****************************************************************************
 * Function Name  : wlan_conf_get_version
 * Arguments      :
 * Returns        : 0 - SUCCESS
 *                  -1 - FAILED
 * Variables      :
 * Modifies       :
 * Description    :
 ****************************************************************************/
int wlan_conf_get_version(char ver[WLAN_MAX_VERSION_LENGTH])
{
	return 0;
}

/****************************************************************************
 * Function Name  : wlan_conf_get_beacon_rate
 * Arguments      :
 * Returns        : 0 - SUCCESS
 *                  -1 - FAILED
 * Variables      :
 * Modifies       :
 * Description    :
 ****************************************************************************/
int wlan_conf_get_beacon_rate(unsigned long *rate)
{
	*rate = wlan_config.Beacon_Rate;
	return 0;
}

/****************************************************************************
 * Function Name  : wlan_conf_set_enabled
 * Arguments      :
 * Returns        : 0 - SUCCESS
 *                  -1 - FAILED
 * Variables      :
 * Modifies       :
 * Description    :
 ****************************************************************************/
int wlan_conf_set_enabled(int status)
{
	wlan_config.Enabled = status;
	return 0;
}

/****************************************************************************
 * Function Name  : wlan_conf_set_essid
 * Arguments      :
 * Returns        : 0 - SUCCESS
 *                  -1 - FAILED
 * Variables      :
 * Modifies       :
 * Description    :
 ****************************************************************************/
int wlan_conf_set_essid(char id[WLAN_SSID_MAXLEN+1])
{
	strcpy(wlan_config.Essid, id);
	return 0;
}

/****************************************************************************
 * Function Name  : wlan_conf_set_channel_id
 * Arguments      :
 * Returns        : 0 - SUCCESS
 *                  -1 - FAILED
 * Variables      :
 * Modifies       :
 * Description    :
 ****************************************************************************/
int wlan_conf_set_channel_id(int id)
{
	wlan_config.Channel_Id = id;
	return 0;
}

/****************************************************************************
 * Function Name  : wlan_conf_set_encryption
 * Arguments      :
 * Returns        : 0 - SUCCESS
 *                  -1 - FAILED
 * Variables      :
 * Modifies       :
 * Description    :
 ****************************************************************************/
int wlan_conf_set_encryption(int encryption)
{
	wlan_config.Encryption = encryption;
	return 0;
}

/****************************************************************************
 * Function Name  : wlan_conf_set_key_len
 * Arguments      :
 * Returns        : 0 - SUCCESS
 *                  -1 - FAILED
 * Variables      :
 * Modifies       :
 * Description    :
 ****************************************************************************/
int wlan_conf_set_key_len(int len)
{
	if( len != 5 && len != 13 && len != 0 )
		return -1;
	wlan_config.Key_Len = len;
	return 0;
}

/****************************************************************************
 * Function Name  : wlan_conf_set_key_hexorchar
 * Arguments      :
 * Returns        : 0 - SUCCESS
 *                  -1 - FAILED
 * Variables      :
 * Modifies       :
 * Description    :
 ****************************************************************************/
int wlan_conf_set_key_hexorchar(int hexorchar)
{
	wlan_config.Key_HexorChar = hexorchar;
	return 0;
}

/****************************************************************************
 * Function Name  : wlan_conf_set_key_id
 * Arguments      :
 * Returns        : 0 - SUCCESS
 *                  -1 - FAILED
 * Variables      :
 * Modifies       :
 * Description    :
 ****************************************************************************/
int wlan_conf_set_key_id(int id)
{
	wlan_config.Key_Id = id;
	return 0;
}

/****************************************************************************
 * Function Name  : wlan_conf_set_wep_key
 * Arguments      :
 * Returns        : 0 - SUCCESS
 *                  -1 - FAILED
 * Variables      :
 * Modifies       :
 * Description    :
 ****************************************************************************/
int wlan_conf_set_wep_key(char key[WLAN_WEP_MAXKEYLEN], int keyid)
{
	if( keyid < 0 || keyid > WLAN_WEP_NKEYS)
		return -1;

	memcpy(wlan_config.WEP_Key[keyid], key, 13);
	return 0;
}

/****************************************************************************
 * Function Name  : wlan_conf_set_passphrase_string
 * Arguments      :
 * Returns        : 0 - SUCCESS
 *                  -1 - FAILED
 * Variables      :
 * Modifies       :
 * Description    :
 ****************************************************************************/
int wlan_conf_set_passphrase_string(char str[PASS_PHRASE_MAXLEN+1])
{
	strcpy( wlan_config.PassPhraseString, str);
	return 0;
}

/****************************************************************************
 * Function Name  : wlan_conf_set_hidden_essid
 * Arguments      :
 * Returns        : 0 - SUCCESS
 *                  -1 - FAILED
 * Variables      :
 * Modifies       :
 * Description    :
 ****************************************************************************/
int wlan_conf_set_hidden_essid(int hidden)
{
	wlan_config.Hidden_ESSID = hidden;
	return 0;
}


/****************************************************************************
 * Function Name  : wlan_conf_set_version
 * Arguments      :
 * Returns        : 0 - SUCCESS
 *                  -1 - FAILED
 * Variables      :
 * Modifies       :
 * Description    :
 ****************************************************************************/
int wlan_conf_set_version(char ver[WLAN_MAX_VERSION_LENGTH])
{
	return 0;
}

/****************************************************************************
 * Function Name  : wlan_conf_set_beacon_rate
 * Arguments      :
 * Returns        : 0 - SUCCESS
 *                  -1 - FAILED
 * Variables      :
 * Modifies       :
 * Description    :
 ****************************************************************************/
int wlan_conf_set_beacon_rate(unsigned long rate)
{
	wlan_config.Beacon_Rate = rate;
	return 0;
}

void HexToCharString(char hex[], char charstring[], int len)
{
	char buf[3]={0};
	int i;

	for(i=0; i<len; i+=2) {
		buf[0] = hex[i];
		buf[1] = hex[i+1];
		charstring[i/2] = HexToChar(buf);
		//printf("Char string %02X\n", charstring[i/2]);
	}
}

char HexToChar(char hex[])
{
	unsigned int i;
	sscanf(hex, "%2X", &i);
	return i;
}
