
/*
 * do_ri.c - Reserved instruction handler
 *            for the IA-7546-4080 processor.
 */

/*
** Copyright 1998 sunplus, Inc.
*/

#include <linux/mm.h>
#include <linux/signal.h>
#include <linux/smp.h>
#include <linux/smp_lock.h>
 
#include <asm/asm.h>
#include <asm/branch.h>
#include <asm/byteorder.h>
#include <asm/inst.h>
#include <asm/uaccess.h>


#define STR(x)  __STR(x)
#define __STR(x)  #x

#ifndef __RI_HANDLER_H__
#define __RI_HANDLER_H__

#ifdef  __cplusplus
extern "C" {
#endif

//Sub opcode of mac instruction

#define SUBOP_MULT	0x18
#define SUBOP_MULTU	0x19
#define SUBOP_DIV	0x1A
#define SUBOP_DIVU	0x1B


#ifndef Branch_t
typedef enum {
  BRANCH_T_NONE,
  BRANCH_T_OFFSET,
  BRANCH_T_TARGET,
  BRANCH_T_REGISTER
} Branch_t;
#endif /* Branch_t */

typedef signed int LONG;
typedef signed int INT;
typedef unsigned int ULONG;
typedef unsigned int UINT;

#ifdef  __cplusplus
}
#endif


#endif

#undef TRUE
#define TRUE (0 == 0)

#undef FALSE
#define FALSE (0 != 0)

static int isJumpInstruction(struct pt_regs *regs);
static Branch_t emulateBranch(struct pt_regs *regs);
static unsigned int   *computeBranchAddress (struct pt_regs *regs, Branch_t type);
extern void checkaddress(unsigned long addr);
unsigned long c_lwl, c_lwr, c_swl, c_swr;
unsigned long c_mult, c_multu, c_div, c_divu;
/*================================================*/
unsigned long ll_bit;
#ifdef CONFIG_PROC_FS
extern unsigned long ll_ops;
extern unsigned long sc_ops;
#endif

int emulateIA7546RI(struct pt_regs *regs)
{
  register ULONG rs;
  register ULONG rt;
   union mips_instruction insn;
  ULONG va;
  ULONG newVA;
  ULONG mem;
  ULONG newPC = 0;
  ULONG pc;
  unsigned int byte;
  int status;
  int branchDelay;
  unsigned long value;
  unsigned long fixup;

  branchDelay = ((0 != (CAUSEF_BD & regs->cp0_cause)) |
		 isJumpInstruction(regs));

  if (branchDelay) 
  {
    newPC = (ULONG) computeBranchAddress (regs,
					  emulateBranch(regs));
  }

   pc = regs->cp0_epc + (branchDelay ? 4:0);
   __get_user(insn.word, (unsigned int *)pc);
//	printk("KERN_ALERT""handle RI_instr OP = 0x%x epc=0x%x\n",insn.i_format.opcode,regs->cp0_epc);

  /* In case the emulated zero register is somehow trashed. */
  regs->regs[0] = 0;

  rs = regs->regs[insn.i_format.rs];
  rt = regs->regs[insn.i_format.rt];

  va = rs + insn.i_format.simmediate;
  byte = va & 3;
  newVA = va - byte;

  status = 0;

  switch (insn.i_format.opcode)
  {
  case 0x30:   /* ll */
    cli();
    status = 1;
  break;

  case 0x38:   /* sc */
   sti();
   status = 1;
   regs->regs[insn.i_format.rt] = 1;
  break;

  case 0x22:			/* lwl */
#ifdef __MIPSEB__
    mem = *(ULONG *)(va - byte);
    mem = mem << byte*8;

    rt = (rt & ~(-1UL << byte*8)) | mem;
#else /*__MIPSEL__ */
    //mem = *(ULONG *)(va - byte);
	asm(
	".set\tnoat\n"
	"1:\tlw\t%0,0(%1)\n"
	"nop\n"
	".set\tat\n\t"
	".section\t__ex_table,\"a\"\n\t"
	STR(PTR)"\t1b,%2\n\t"
	".previous"
	:"=&r" (mem)
	:"r" (newVA), "i" (&&fault)
	:"$1");
	
    mem = mem << (3 - byte)*8;

    rt = (rt & ~(-1UL << (3 - byte)*8)) | mem;
#endif

	regs->regs[insn.i_format.rt] = rt;
	c_lwl++;
	printk(KERN_ALERT "lwl\n");
    status = 1;

    break;

  case 0x26:			/* lwr */

#ifdef __MIPSEB__
    mem = *(ULONG *)(va - byte);
    mem = mem >> (3-byte)*8;

    rt = (rt & ~(-1UL >> (3-byte)*8)) | mem;
#else /* __MIPSEL__ */
    //mem = *(ULONG *)(va - byte);
	asm(
	".set\tnoat\n"
	"1:\tlw\t%0,0(%1)\n"
	"nop\n"
	".set\tat\n\t"
	".section\t__ex_table,\"a\"\n\t"
	STR(PTR)"\t1b,%2\n\t"
	".previous"
	:"=&r" (mem)
	:"r" (newVA), "i" (&&fault)
	:"$1");
    mem = mem >> byte*8;

    rt = (rt & ~(-1UL >> byte*8)) | mem;
#endif
	regs->regs[insn.i_format.rt] = rt;
	c_lwr++;
	printk(KERN_ALERT "lwr\n");
    status = 1;

    break;

  case 0x2A:			/* swl */

#ifdef __MIPSEB__
    mem = *(ULONG *)(va - byte);
    mem = mem & ~(-1UL >> byte*8);

    rt = (rt >> byte*8) | mem;

#else /* __MIPSEL__ */
    //mem = *(ULONG *)(va - byte);
	asm(
	".set\tnoat\n"
	"1:\tlw\t%0,0(%1)\n"
	"nop\n"
	".set\tat\n\t"
	".section\t__ex_table,\"a\"\n\t"
	STR(PTR)"\t1b,%2\n\t"
	".previous"
	:"=&r" (mem)
	:"r" (newVA), "i" (&&fault)
	:"$1");
    mem = mem & ~(-1UL >> (3-byte)*8);

    rt = (rt >> (3-byte)*8) | mem;
#endif
    *(ULONG *)(va - byte) = rt;
	c_swl++;
	printk(KERN_ALERT "swl\n");
    status = 1;

    break;

  case 0x2E:			/* swr */

#ifdef __MIPSEB__
    mem = *(ULONG *)(va - byte);
    mem = mem & ~(-1UL << (3-byte)*8);

    rt = (rt << (3-byte)*8) | mem;
#else /* __MIPSEL__ */
    //mem = *(ULONG *)(va - byte);
	asm(
	".set\tnoat\n"
	"1:\tlw\t%0,0(%1)\n"
	"nop\n"
	".set\tat\n\t"
	".section\t__ex_table,\"a\"\n\t"
	STR(PTR)"\t1b,%2\n\t"
	".previous"
	:"=&r" (mem)
	:"r" (newVA), "i" (&&fault)
	:"$1");
    mem = mem & ~(-1UL << byte*8);

    rt = (rt << byte*8) | mem;
#endif

    *(ULONG *)(va - byte) = rt;
	c_swr++;
	printk(KERN_ALERT "swr\n");
    status = 1;

    break;
 	case 0x00:
    	switch(insn.word & 0x1f)
		{
			//printk("KERN_ALERT""OPCODE == 0\n");
			case SUBOP_MULT:
			{
				register unsigned int X = (unsigned int) rs;
				register unsigned int Y = (unsigned int) rt;
				register unsigned int i;
				register unsigned int Mask = 0x80000000;
				register unsigned int sign = (Mask & X) ^ (Mask & Y);
				register unsigned int HI = 0;
				register unsigned int LO = Mask&Y ? -Y : Y;
				register unsigned int CARRY;

				X = Mask&X ? -X : X;

				for (i=0; i<32; i++) 
				{
	  				CARRY = 0;
	  				if (LO&1) 
					{
	    				CARRY = HI;
	    				HI += X;
	    				CARRY = (HI < CARRY) | (HI < X);
	  				}
	  				LO >>= 1;
	  				if (HI&1) 
					{
	    				LO |= Mask;
	  				}
	  				HI >>= 1;
	  				if (CARRY) 
					{
	    				HI |= Mask;
	  				}
				}
				if (sign) 
				{
	  				LO = ~LO;
	  				HI = ~HI;
	  				CARRY = LO;
	  				LO += 1;
	  				if (LO < CARRY) 
					{
	    				HI += 1;
	  				}
				}

				regs->hi = HI;
				regs->lo = LO;
				c_mult++;
      		}
      		status = 1;
			break;
			case SUBOP_MULTU:
			{
				register unsigned int X = (unsigned int) rs;
				register unsigned int i;
				register unsigned int Mask = 0x80000000;
				register unsigned int HI = 0;
				register unsigned int LO = (unsigned int) rt;
				register unsigned int CARRY;

				for (i=0; i<32; i++) 
				{
	  				CARRY = 0;
	  				if (LO&1) 
					{
	    				CARRY = HI;
	    				HI += X;
	    				CARRY = (HI < CARRY) | (HI < X);
	  				}
	  				LO >>= 1;
	  				if (HI&1) 
					{
	    				LO |= Mask;
	  				}
	  				HI >>= 1;
	  				if (CARRY) 
					{
	    				HI |= Mask;
	  				}
				}

				//_STKOFFSET(EXC_DATA_HI) = HI;
				//_STKOFFSET(EXC_DATA_LO) = LO;
				regs->hi = HI;
				regs->lo = LO;
				c_multu++;
      		}

      		status = 1;
			break;
			case SUBOP_DIV:
			{
				register unsigned int X = (unsigned int) rs;
				register unsigned int Y = (unsigned int) rt;
				register unsigned int i;
				register unsigned int Mask = 0x80000000;
				register unsigned int signHI = (Mask & X);
				register unsigned int sign = signHI ^ (Mask & Y);
				register unsigned int HI = 0;
				register unsigned int LO = Mask&X ? -X : X;

				Y = Mask&Y ? -Y : Y;

				for (i=0; i<32; i++) 
				{
	  				HI <<= 1;
	  				if (LO&Mask)
	    				HI |= 1;
	  				LO <<= 1;

	  				if (Y > HI) 
					{
	    				LO &= ~1;
	  				}
	  				else 
					{
	    				HI -= Y;
	    				LO |= 1;
	  				}
				}
				LO = sign ? -LO : LO;
				HI = signHI ? -HI : HI;

				//_STKOFFSET(EXC_DATA_HI) = HI;
				//_STKOFFSET(EXC_DATA_LO) = LO;
				regs->hi = HI;
				regs->lo = LO;
				c_div++;
      		}
      		status = 1;
			break;
			case SUBOP_DIVU:
			{
				register unsigned int X = (unsigned int) rs;
				register unsigned int Y = (unsigned int) rt;
				register unsigned int i;
				register unsigned int Mask = 0x80000000;
				register unsigned int HI = 0;
				register unsigned int LO = X;

				for (i=0; i<32; i++) 
				{
	  				HI <<= 1;
	  				if (LO&Mask)
	    				HI |= 1;
	  				LO <<= 1;
			
	  				if (Y > HI) 
					{
	    				LO &= ~1;
	  				}
	  				else 
					{
	    				HI -= Y;
	    				LO |= 1;
	  				}
				}

				//_STKOFFSET(EXC_DATA_HI) = HI;
				//_STKOFFSET(EXC_DATA_LO) = LO;
				regs->hi = HI;
				regs->lo = LO;
				c_divu++;
      		}

      		status = 1;		
			break;
			default:

		} //end of switch
		break;

  default:
	printk("KERN_ALERT""Unable to handle RI_instr OP = 0x%x epc=0x%x\n",insn.i_format.opcode,regs->cp0_epc);
	while (1);
    
  }
  if (status) 
  {
    if (branchDelay) 
    {
	  regs->cp0_epc = newPC;
    }
    else 
    {
	  regs->cp0_epc += 4;
    }
  }
fault:
#if 0
      /* Did we have an exception handler installed? */
      fixup = search_exception_table(regs->cp0_epc);
	  if (fixup) {
			long new_epc;
         printk("fixup\n");
			new_epc = fixup_exception(dpf_reg, fixup, regs->cp0_epc);
			printk(KERN_DEBUG "%s: Forwarding exception at [<%lx>] (%lx)\n",
			current->comm, regs->cp0_epc, new_epc);
			regs->cp0_epc = new_epc;
			return;
	  }
#endif
  return (status);
}

static int isJumpInstruction(struct pt_regs *regs)
{
  ULONG pc;
  union mips_instruction insn;

  pc = regs->cp0_epc;
  __get_user(insn.word, (unsigned int *)pc);

  switch (insn.i_format.opcode) 
  {
  case 0x00:			/* Special */
	if ( (insn.r_format.func == 0x08)   //jr
	   | (insn.r_format.func == 0x09))  //jalr
	{
		return TRUE;
	}
	else
		return FALSE;
      
  case 0x02:			/* j */
  case 0x03:			/* jal */
    return TRUE;

  default:
    return FALSE;
  }
}


static Branch_t emulateBranch(struct pt_regs *regs)
{
  register ULONG rs;
  register ULONG rt;
  Branch_t branchStatus;
  ULONG pc;
  union mips_instruction insn;

  pc = regs->cp0_epc;
  __get_user(insn.word, (unsigned int *)pc);


  rs = regs->regs[insn.i_format.rs];
  rt = regs->regs[insn.i_format.rt];

  branchStatus = BRANCH_T_NONE;
  switch(insn.i_format.opcode)
  {
  case 0x00:			/* Special */
    if ((insn.r_format.func) == 0x08)  /* jr */
    {	
      branchStatus = BRANCH_T_REGISTER;
    }
    if ((insn.r_format.func) == 0x09)  /* jalr */
    {	
	  regs->regs[insn.r_format.rd] = pc + 8; // skip 2 instruction
      branchStatus = BRANCH_T_REGISTER;
    }
    break;

  case 0x03:			/* jal */
	regs->regs[31] = pc + 8; // skip 2 instruction
  case 0x02:			/* j */
    branchStatus = BRANCH_T_TARGET;
    break;

  case 0x04:			/* beq */
    if (rs == rt) branchStatus = BRANCH_T_OFFSET;
    break;

  case 0x05:			/* bne */
    if (rs != rt) 
		branchStatus = BRANCH_T_OFFSET;
    break;

  case 0x06:			/* blez */
    if ((signed long) rs <= (signed long) 0)
      branchStatus = BRANCH_T_OFFSET;
    break;

  case 0x07:			/* bgtz */
    if ((signed long) rs > (signed long) 0)
      branchStatus = BRANCH_T_OFFSET;
    break;

  case 0x01:			/* regimm */
    switch(insn.i_format.rt) 
    {
    case 0x10:			/* bltzal */
	  regs->regs[31] = pc + 8; //skip 2 insn
    case 0x00:			/* bltz */
      if ((signed long) rs < (signed long) 0)
	branchStatus = BRANCH_T_OFFSET;
      break;

    case 0x11:			/* bgezal */
	  regs->regs[31] = pc + 8; //skip 2 insn
    case 0x01:			/* bgez */
      if ((signed long) rs >= (signed long) 0)
	branchStatus = BRANCH_T_OFFSET;
      break;
    }
    break;

    /* Todo: bcxf and bcxt */

  case 0x10:			/* cop0 */
    branchStatus = BRANCH_T_OFFSET;
    break;

  case 0x11:			/* cop1 */
    branchStatus = BRANCH_T_OFFSET;
    break;

  case 0x12:			/* cop2 */
    branchStatus = BRANCH_T_OFFSET;
    break;

  case 0x13:			/* cop3 */
    branchStatus = BRANCH_T_OFFSET;
    break;
  }

  return branchStatus;
}


static UINT *computeBranchAddress (struct pt_regs *regs, Branch_t type)
{
  register ULONG rs;
  signed int offset;

  ULONG pc;
  union mips_instruction insn;

  pc = regs->cp0_epc;
  __get_user(insn.word, (unsigned int *)pc);

  switch (type) 
  {
  case BRANCH_T_OFFSET:
    offset = insn.i_format.simmediate;
    return (pc + 4 + (offset << 2));

  case BRANCH_T_TARGET:
    offset = insn.j_format.target;
	return (((pc+4) & 0xf0000000) | (offset << 2)); //?????

  case BRANCH_T_REGISTER:
	rs = regs->regs[insn.i_format.rs];
	return rs;

  case BRANCH_T_NONE:
	return (pc + 8);

  default:
    return (0);
  }
}

