/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 1992 Linus Torvalds
 * Copyright (C) 1994 - 2000 Ralf Baechle
 */
#include <linux/delay.h>
#include <linux/init.h>
#include <linux/irq.h>
#include <linux/interrupt.h>
#include <linux/kernel.h>
#include <linux/spinlock.h>

#include <asm/io.h>
#include <asm/sunplus/regmap.h>
#include <asm/sunplus/interrupt.h>

extern volatile RegisterFile *dvd_regs;
spinlock_t sunplus_lock = SPIN_LOCK_UNLOCKED;

//extern asmlinkage void sunplus_handle_int(void);
extern void do_IRQ(int irq, struct pt_regs *regs);

static void enable_sunplus_irq(unsigned int irq);

static unsigned int startup_sunplus_irq(unsigned int irq)
{
	enable_sunplus_irq(irq);
	return 0; /* never anything pending */
}

#define shutdown_sunplus_irq	disable_sunplus_irq

void disable_sunplus_irq(unsigned int irq)
{
	unsigned long flags;
#if 0
        unsigned int  mask=0x100; // bit 8 , irq 0
        unsigned int status_reg=0;
        mask = mask < irq;
        save_and_cli(flags);
        __asm__ __volatile__ (
        "mfc0\t%0,$12\t\n"
        "nop\t\n"
        "not\t%1\t\n"
        "and\t%0,%0,%1\t\n"
        "mtc0\t%0,$12\t\n"
        "nop\t\n"
        : "=r" (status_reg) , "=r" (mask)
        );
        restore_flags(flags);
#endif
	spin_lock_irqsave(&sunplus_lock, flags);
	dvd_regs->intr_mask &= ~(1<<irq);
	spin_unlock_irqrestore(&sunplus_lock, flags);
}

static void enable_sunplus_irq(unsigned int irq)
{
	unsigned long flags;
#if 0
	unsigned int  mask=0x100; // bit 8 , irq 0
	unsigned int status_reg=0;
	mask = mask < irq;
	save_and_cli(flags);
	__asm__ __volatile__ (
		"mfc0\t%0,$12\t\n"
	        "nop\t\n"
	        "or\t%0,%0,%1\t\n"
	        "or\t%0,%0,0x1\t\n"  // Enable IEc bit
	        "mtc0\t%0,$12\t\n"
	        "nop\t\n"
	        : "=r" (status_reg)
	        : "r" (mask)
	);
	                                                                     
	restore_flags(flags);
#endif	

	spin_lock_irqsave(&sunplus_lock, flags);
	dvd_regs->intr_mask |= (1<<irq);
	spin_unlock_irqrestore(&sunplus_lock, flags);
}

static void ack_sunplus_irq(unsigned int irq)
{
	unsigned long flags;
	spin_lock_irqsave(&sunplus_lock, flags);
	dvd_regs->intr_flag = (1<<irq);
	spin_unlock_irqrestore(&sunplus_lock, flags);
}

static void end_sunplus_irq(unsigned int irq)
{
	if (!(irq_desc[irq].status & (IRQ_DISABLED|IRQ_INPROGRESS)))
		enable_sunplus_irq(irq);
}

static struct hw_interrupt_type sunplus_irq_type = {
	"SUNPLUS",
	startup_sunplus_irq,
	shutdown_sunplus_irq,
	enable_sunplus_irq,
	disable_sunplus_irq,
	ack_sunplus_irq,
	end_sunplus_irq,
	NULL	//priority
};

void __init init_sunplus_irq(void)
{
	unsigned int flags;
	int i;
#if 1	
	unsigned int mask=0xffff0000;
	unsigned int status_reg=0;
	/* Clear and Disable All Interrupts */
	__asm__ __volatile__ (
		"mfc0\t%0,$12\n\t"
	        "nop\n\t"
	        "and\t%0,%0,%1\n\t"
	        "mtc0\t%0,$12\n\t"
	        "nop\n\t"
	        : "=r" (status_reg)
	        : "r" (mask)
	);
#endif	                                                
	spin_lock_irqsave(&sunplus_lock, flags);
	dvd_regs->intr_mask = 0x0;
	dvd_regs->intr_flag = 0xFFFFFFFF;
	spin_unlock_irqrestore(&sunplus_lock, flags);
	for (i = 0; i < 16; i++) {
		irq_desc[i].status = IRQ_DISABLED;
	        irq_desc[i].action = NULL;
	        irq_desc[i].depth = 1;
	        irq_desc[i].handler = &sunplus_irq_type;
	}     
}

void __init init_IRQ (void)
{
	extern void init_generic_irq(void);
	extern asmlinkage void dvd_handle_int(void);
	
	set_except_vector(0, dvd_handle_int);
	//init_generic_irq();
	init_sunplus_irq();	
}

void dvd_irq_disp(struct pt_regs *regs)
{
        unsigned int irq;
        unsigned int mask = 0x400;  // CPU hardware int 0
        volatile unsigned int irq_status;
//for now, all irqs are line in CPU hardware int 0
//        	if ( regs->cp0_cause & mask )
        	{
        		irq_status = dvd_regs->intr_flag;
        		for ( irq=0; irq<=NR_IRQS; irq++)
        		{
        		   mask = 1 << irq;
        		   if ((irq_status&mask)&&(irq_desc[irq].action != NULL))
        		   {
     				do_IRQ(irq, regs);
     			   }	        		
        		}
                }
}
                                
