/*
 * RI_handler.c.c - Reserved instruction handler
 *            for the IA-7546-4080 processor.
 */

/*
** Copyright 1998 sunplus, Inc.
*/

#include <linux/config.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/smp.h>
#include <linux/smp_lock.h>
#include <linux/spinlock.h>

#include <asm/uaccess.h>

#include "ri_exception.h"
#include "ri_handler.h"


#undef TRUE
#define TRUE (0 == 0)

#undef FALSE
#define FALSE (0 != 0)

static int isJumpInstruction(char *pEXCdata);
static Branch_t emulateBranch(char *pEXCdata);
static unsigned int   *computeBranchAddress (char *pEXCdata, Branch_t type);
#if 0
ULONG emulatedHI;
ULONG emulatedLO;
#endif

/*================================================*/
unsigned long ll_bit;
#ifdef CONFIG_PROC_FS
extern unsigned long ll_ops;
extern unsigned long sc_ops;
#endif
//static struct task_struct *ll_task = NULL;
/*================================================*/

int emulate_dvd_ri(char *pEXCdata)
{
  register ULONG rs;
  register ULONG rt;
  ULONG va;
  ULONG mem;
  ULONG newPC = 0;
  UINT inst;
  unsigned int byte;
  int status;
  int branchDelay;
  
//  int signal = 0;
  
  branchDelay = ((0 != (CAUSE_BD & _STKOFFSET(EXC_DATA_CAUSE))) |
		 isJumpInstruction(pEXCdata));

  if (branchDelay) 
  {
    newPC = (ULONG) computeBranchAddress (pEXCdata,
					  emulateBranch(pEXCdata));
  }

  inst = *((UINT *)(_STKOFFSET(EXC_DATA_EPC)) + (branchDelay?1:0));
  
  /* In case the emulated zero register is somehow trashed. */
  _GPR_STKOFFSET(0) = 0;

  rs = _GPR_STKOFFSET(_RS_(inst));
  rt = _GPR_STKOFFSET(_RT_(inst));


  va = rs + (ULONG)((short)_OFFSET_(inst));
  byte = va & 3;

  status = 0;

  switch (_OP_(inst)) 
  {
  case 0x22:			/* lwl */
printk("lwl in EPC= 0x%x, ...\n",_STKOFFSET(EXC_DATA_EPC));
#ifdef __MIPSEB__
    mem = *(ULONG *)(va - byte);
    mem = mem << byte*8;

    rt = (rt & ~(-1UL << byte*8)) | mem;
#else /*__MIPSEL__ */
    mem = *(ULONG *)(va - byte);
    mem = mem << (3 - byte)*8;

    rt = (rt & ~(-1UL << (3 - byte)*8)) | mem;
#endif

    _GPR_STKOFFSET(_RT_(inst)) = rt;
    status = 1;

    break;

  case 0x26:			/* lwr */
printk("lwr in EPC= 0x%x, ...\n",_STKOFFSET(EXC_DATA_EPC));
#ifdef __MIPSEB__
    mem = *(ULONG *)(va - byte);
    mem = mem >> (3-byte)*8;

    rt = (rt & ~(-1UL >> (3-byte)*8)) | mem;
#else /* __MIPSEL__ */
    mem = *(ULONG *)(va - byte);
    mem = mem >> byte*8;

    rt = (rt & ~(-1UL >> byte*8)) | mem;
#endif
    _GPR_STKOFFSET(_RT_(inst)) = rt;
    status = 1;

    break;

  case 0x2A:			/* swl */
printk("swl in EPC= 0x%x, ...\n",_STKOFFSET(EXC_DATA_EPC));

#ifdef __MIPSEB__
    mem = *(ULONG *)(va - byte);
    mem = mem & ~(-1UL >> byte*8);

    rt = (rt >> byte*8) | mem;

#else /* __MIPSEL__ */
    mem = *(ULONG *)(va - byte);
    mem = mem & ~(-1UL >> (3-byte)*8);

    rt = (rt >> (3-byte)*8) | mem;
#endif
    *(ULONG *)(va - byte) = rt;
    status = 1;

    break;

  case 0x2E:			/* swr */
printk("swr in EPC= 0x%x, ...\n",_STKOFFSET(EXC_DATA_EPC));

#ifdef __MIPSEB__
    mem = *(ULONG *)(va - byte);
    mem = mem & ~(-1UL << (3-byte)*8);

    rt = (rt << (3-byte)*8) | mem;
#else /* __MIPSEL__ */
    mem = *(ULONG *)(va - byte);
    mem = mem & ~(-1UL << byte*8);

    rt = (rt << byte*8) | mem;
#endif

    *(ULONG *)(va - byte) = rt;
    status = 1;

    break;
  case 0x30:			//ll
    cli();
    status = 1;
#if 0
    #ifdef CONFIG_PROC_FS
       ll_ops++;
    #endif
    if (byte)
       signal = SIGBUS;
    else if (get_user(mem, &va))
       signal = SIGSEGV;
    else {
 //      printk("ri_handler: ll  epc=%08x  inst=%08x\n",_STKOFFSET(EXC_DATA_EPC),inst);
       if (ll_task == current || ll_task == NULL) {
          ll_bit = 1;
       } else {
          ll_bit = 0;
       }
       _GPR_STKOFFSET(_RT_(inst)) = mem;
       status = 1;
       ll_task = current;
    }
    if (signal) {send_sig(signal, current, 1);}
#endif  
    break;
  case 0x38:			//sc
    sti();
    status = 1;
    _GPR_STKOFFSET(_RT_(inst)) = 1;
#if 0
    #ifdef CONFIG_PROC_FS
       sc_ops++;
    #endif
    if (byte)
       signal = SIGBUS;
    else if (ll_bit == 0 || ll_task != current){
       _GPR_STKOFFSET(_RT_(inst)) = 0;
       status = 1;
    }else if (put_user(_GPR_STKOFFSET(_RT_(inst)), &va))
       signal = SIGSEGV;
    else {
       _GPR_STKOFFSET(_RT_(inst)) = 1;
       status = 1;
    }
    if (signal) {send_sig(signal, current, 1);}
#endif
    break;        
  case 0x00:
		switch(_SUBOP_(inst))
		{
#if 0
			case SUBOP_MFHI:
				_GPR_STKOFFSET(_RD_(inst)) = _STKOFFSET(EXC_DATA_HI);
      			status = 1;	
			break;
			case SUBOP_MTHI:
				_STKOFFSET(EXC_DATA_HI) = _GPR_STKOFFSET(_RS_(inst));
      			status = 1;
			break;
			case SUBOP_MFLO:
				 _GPR_STKOFFSET(_RD_(inst)) = _STKOFFSET(EXC_DATA_LO);
      			status = 1;
			break;
			case SUBOP_MTLO:
				_STKOFFSET(EXC_DATA_LO) = _GPR_STKOFFSET(_RS_(inst));
      			status = 1;
			break;
#endif
			case SUBOP_MULT:
			{
				register unsigned int X = (unsigned int) rs;
				register unsigned int Y = (unsigned int) rt;
				register unsigned int i;
				register unsigned int Mask = 0x80000000;
				register unsigned int sign = (Mask & X) ^ (Mask & Y);
				register unsigned int HI = 0;
				register unsigned int LO = Mask&Y ? -Y : Y;
				register unsigned int CARRY;

				X = Mask&X ? -X : X;

				for (i=0; i<32; i++) 
				{
	  				CARRY = 0;
	  				if (LO&1) 
					{
	    				CARRY = HI;
	    				HI += X;
	    				CARRY = (HI < CARRY) | (HI < X);
	  				}
	  				LO >>= 1;
	  				if (HI&1) 
					{
	    				LO |= Mask;
	  				}
	  				HI >>= 1;
	  				if (CARRY) 
					{
	    				HI |= Mask;
	  				}
				}
				if (sign) 
				{
	  				LO = ~LO;
	  				HI = ~HI;
	  				CARRY = LO;
	  				LO += 1;
	  				if (LO < CARRY) 
					{
	    				HI += 1;
	  				}
				}

				_STKOFFSET(EXC_DATA_HI) = HI; 
				_STKOFFSET(EXC_DATA_LO) = LO;
      		}
      		status = 1;
			break;
			case SUBOP_MULTU:
			{
				register unsigned int X = (unsigned int) rs;
				register unsigned int i;
				register unsigned int Mask = 0x80000000;
				register unsigned int HI = 0;
				register unsigned int LO = (unsigned int) rt;
				register unsigned int CARRY;

				for (i=0; i<32; i++) 
				{
	  				CARRY = 0;
	  				if (LO&1) 
					{
	    				CARRY = HI;
	    				HI += X;
	    				CARRY = (HI < CARRY) | (HI < X);
	  				}
	  				LO >>= 1;
	  				if (HI&1) 
					{
	    				LO |= Mask;
	  				}
	  				HI >>= 1;
	  				if (CARRY) 
					{
	    				HI |= Mask;
	  				}
				}

				_STKOFFSET(EXC_DATA_HI) = HI;
				_STKOFFSET(EXC_DATA_LO) = LO;
      		}

      		status = 1;
			break;
			case SUBOP_DIV:
			{
				register unsigned int X = (unsigned int) rs;
				register unsigned int Y = (unsigned int) rt;
				register unsigned int i;
				register unsigned int Mask = 0x80000000;
				register unsigned int signHI = (Mask & X);
				register unsigned int sign = signHI ^ (Mask & Y);
				register unsigned int HI = 0;
				register unsigned int LO = Mask&X ? -X : X;

				Y = Mask&Y ? -Y : Y;

				for (i=0; i<32; i++) 
				{
	  				HI <<= 1;
	  				if (LO&Mask)
	    				HI |= 1;
	  				LO <<= 1;

	  				if (Y > HI) 
					{
	    				LO &= ~1;
	  				}
	  				else 
					{
	    				HI -= Y;
	    				LO |= 1;
	  				}
				}
				LO = sign ? -LO : LO;
				HI = signHI ? -HI : HI;

				_STKOFFSET(EXC_DATA_HI) = HI;
				_STKOFFSET(EXC_DATA_LO) = LO;
      		}
      		status = 1;
			break;
			case SUBOP_DIVU:
			{
				register unsigned int X = (unsigned int) rs;
				register unsigned int Y = (unsigned int) rt;
				register unsigned int i;
				register unsigned int Mask = 0x80000000;
				register unsigned int HI = 0;
				register unsigned int LO = X;

				for (i=0; i<32; i++) 
				{
	  				HI <<= 1;
	  				if (LO&Mask)
	    				HI |= 1;
	  				LO <<= 1;
			
	  				if (Y > HI) 
					{
	    				LO &= ~1;
	  				}
	  				else 
					{
	    				HI -= Y;
	    				LO |= 1;
	  				}
				}

				_STKOFFSET(EXC_DATA_HI) = HI;
				_STKOFFSET(EXC_DATA_LO) = LO;
      		}

      		status = 1;		
			break;
			default:
			// Sub OP error
		}
	break;
	case 0x3c:
		switch(_SUBOP_(inst))
		{
			case SUBOP_MADH:
			{
				register signed int X = (unsigned int) (rs & 0xffff);
				register signed int Y = (unsigned int) (rt & 0xffff);
				//register signed int result;
				register signed int HI = _STKOFFSET(EXC_DATA_HI);
				if( X & 0x8000) 
					X |= 0xffff0000;
				if( Y & 0x8000) 
					Y |= 0xffff0000;
				HI = HI + (X * Y);
#if 0
				asm(
					"move\t$4,%1\n"
					"move\t$5,%2\n"
					"jal\t__Xmulsi3"
					"nop\n"
					"move\t%0,$2\n"
					:"=r" (result)
					:"r"(X) , "r" (Y)
				);
				HI = HI + result;
#endif
				_STKOFFSET(EXC_DATA_HI) = HI;
			}
			status = 1;
			break;
			case SUBOP_MADL:
			{
				register signed int X = (signed int) (rs & 0xffff);
				register signed int Y = (signed int) (rt & 0xffff);
				//register signed int result;
				register signed int LO = _STKOFFSET(EXC_DATA_LO);
				if( X & 0x8000) 
					X |= 0xffff0000;
				if( Y & 0x8000) 
					Y |= 0xffff0000;
				LO = LO + (X * Y);
#if 0
				asm(
					"move\t$4,%1\n"
					"move\t$5,%2\n"
					"jal\t__Xmulsi3"
					"nop\n"
					"move\t%0,$2\n"
					:"=r" (result)
					:"r"(X) , "r" (Y)
				);
				LO = LO + result;
#endif
				_STKOFFSET(EXC_DATA_LO) = LO;
			}
			status = 1;
			break;
			case SUBOP_MAZH:
			{
				register signed int X = (signed int) (rs & 0xffff);
				register signed int Y = (signed int) (rt & 0xffff);
				//register signed int result;
				register signed int HI = 0;
				if( X & 0x8000) 
					X |= 0xffff0000;
				if( Y & 0x8000) 
					Y |= 0xffff0000;
				HI = HI + (X * Y);
#if 0
				asm(
					"move\t$4,%1\n"
					"move\t$5,%2\n"
					"jal\t__Xmulsi3"
					"nop\n"
					"move\t%0,$2\n"
					:"=r" (result)
					:"r"(X) , "r" (Y)
				);
				HI = HI + result;
#endif
				_STKOFFSET(EXC_DATA_HI) = HI;
			}
			status = 1;
			break;
			case SUBOP_MAZL:
			{
				register signed int X = (signed int) (rs & 0xffff);
				register signed int Y = (signed int) (rt & 0xffff);
				//register signed int result;
				register signed int LO = 0;
				if( X & 0x8000) 
					X |= 0xffff0000;
				if( Y & 0x8000) 
					Y |= 0xffff0000;
				LO = LO + (X * Y);
				_STKOFFSET(EXC_DATA_LO) = LO;
			}
			status = 1;
			break;
			case SUBOP_MSBH:
			{
				register signed int X = (signed int) (rs & 0xffff);
				register signed int Y = (signed int) (rt & 0xffff);
				register signed int HI = _STKOFFSET(EXC_DATA_HI);
				if( X & 0x8000) 
					X |= 0xffff0000;
				if( Y & 0x8000) 
					Y |= 0xffff0000;
				HI = HI - (X * Y);
				_STKOFFSET(EXC_DATA_HI) = HI;
			}
			status = 1;
			break;
			case SUBOP_MSBL:
			{
				register signed int X = (signed int) (rs & 0xffff);
				register signed int Y = (signed int) (rt & 0xffff);
				register signed int LO = _STKOFFSET(EXC_DATA_LO);
				if( X & 0x8000) 
					X |= 0xffff0000;
				if( Y & 0x8000) 
					Y |= 0xffff0000;
				LO = LO - (X * Y);
				_STKOFFSET(EXC_DATA_LO) = LO;
			}
			status = 1;
			break;
			case SUBOP_MSZH:
			{
				register signed int X = (signed int) (rs & 0xffff);
				register signed int Y = (signed int) (rt & 0xffff);
				register signed int HI = 0;
				if( X & 0x8000) 
					X |= 0xffff0000;
				if( Y & 0x8000) 
					Y |= 0xffff0000;
				HI = HI - (X * Y);
				_STKOFFSET(EXC_DATA_HI) = HI;
			}
			status = 1;
			break;
			case SUBOP_MSZL:
			{
				register signed int X = (signed int) (rs & 0xffff);
				register signed int Y = (signed int) (rt & 0xffff);
				register signed int LO = 0;
				if( X & 0x8000) 
					X |= 0xffff0000;
				if( Y & 0x8000) 
					Y |= 0xffff0000;
				LO = LO - (X * Y);
				_STKOFFSET(EXC_DATA_LO) = LO;
			}
			status = 1;
			break;
			default:
			//SUB OP error
		}

	break;
  default:
    
  }

  if (status) 
  {
    if (branchDelay) 
    {
      _STKOFFSET(EXC_DATA_EPC) = newPC;
    }
    else 
    {
      _STKOFFSET(EXC_DATA_EPC) += 4;
    }
  }

  return (status);
}

static int isJumpInstruction(char *pEXCdata)
{
  UINT *pc;
  UINT inst;
  pc = (UINT *)(_STKOFFSET(EXC_DATA_EPC));
  inst = *pc;
  switch (_OP_(inst)) 
  {
  case 0x00:			/* Special */
    if ((_OPS_(inst) == 0x08)	/* jr */
	| (_OPS_(inst) == 0x09))    /* jalr */
	{ 
      return TRUE;
    }
    return FALSE;
      
  case 0x02:			/* j */
  case 0x03:			/* jal */
    return TRUE;

  default:
    return FALSE;
  }
}


static Branch_t emulateBranch(char *pEXCdata)
{
  register ULONG rs;
  register ULONG rt;
  UINT *pc;
  UINT inst;
  Branch_t branchStatus;

  pc = (UINT *)(_STKOFFSET(EXC_DATA_EPC));

  inst = *pc;

  rs = _GPR_STKOFFSET(_RS_(inst));
  rt = _GPR_STKOFFSET(_RT_(inst));

  branchStatus = BRANCH_T_NONE;
  switch (_OP_(inst)) 
  {
  case 0x00:			/* Special */
    if (_OPS_(inst) == 0x08)  /* jr */
    {	
      branchStatus = BRANCH_T_REGISTER;
    }
    if (_OPS_(inst) == 0x09)  /* jalr */
    {	
      _GPR_STKOFFSET(_RD_(inst)) = (ULONG) (pc + 2);
      branchStatus = BRANCH_T_REGISTER;
    }
    break;

  case 0x03:			/* jal */
    _STKOFFSET(EXC_DATA_RA) = (ULONG) (pc + 2);
  case 0x02:			/* j */
    branchStatus = BRANCH_T_TARGET;
    break;

  case 0x04:			/* beq */
    if (rs == rt) branchStatus = BRANCH_T_OFFSET;
    break;

  case 0x05:			/* bne */
    if (rs != rt) branchStatus = BRANCH_T_OFFSET;
    break;

  case 0x06:			/* blez */
    if ((signed long) rs <= (signed long) 0)
      branchStatus = BRANCH_T_OFFSET;
    break;

  case 0x07:			/* bgtz */
    if ((signed long) rs > (signed long) 0)
      branchStatus = BRANCH_T_OFFSET;
    break;

  case 0x01:			/* regimm */
    switch(_RT_(inst)) 
    {
    case 0x10:			/* bltzal */
      _STKOFFSET(EXC_DATA_RA) = (ULONG) (pc + 2);
    case 0x00:			/* bltz */
      if ((signed long) rs < (signed long) 0)
	branchStatus = BRANCH_T_OFFSET;
      break;

    case 0x11:			/* bgezal */
      _STKOFFSET(EXC_DATA_RA) = (ULONG) (pc + 2);
    case 0x01:			/* bgez */
      if ((signed long) rs >= (signed long) 0)
	branchStatus = BRANCH_T_OFFSET;
      break;
    }
    break;

    /* Todo: bcxf and bcxt */

  case 0x10:			/* cop0 */
    branchStatus = BRANCH_T_OFFSET;
    break;

  case 0x11:			/* cop1 */
    branchStatus = BRANCH_T_OFFSET;
    break;

  case 0x12:			/* cop2 */
    branchStatus = BRANCH_T_OFFSET;
    break;

  case 0x13:			/* cop3 */
    branchStatus = BRANCH_T_OFFSET;
    break;
  }

  return branchStatus;
}


static UINT *computeBranchAddress (char *pEXCdata, Branch_t type)
{
  register ULONG rs;
  UINT *pc;
  UINT inst;
  signed int offset;

  pc = (UINT *)(_STKOFFSET(EXC_DATA_EPC));

  inst = *pc;

  switch (type) 
  {
  case BRANCH_T_OFFSET:
    offset = (signed short) (inst & 0xffff);
    return (pc + 1 + offset);

  case BRANCH_T_TARGET:
    offset = inst & 0x3ffffff;
    return (UINT *)(((ULONG)(pc + 1) & 0xf0000000) | (offset << 2));

  case BRANCH_T_REGISTER:
    rs = _GPR_STKOFFSET(_RS_(inst));
    return (UINT *)rs;

  case BRANCH_T_NONE:
    return (pc + 2);

  default:
    return (0);
  }
}

