#include <asm/tc3160/gpio.h>
#include <asm/tc3160/eth.h>
#include <asm/virtconvert.h>
#include <asm/regdef.h>
#include <asm/mipsregs.h>

extern void Bputs(char *,int);
extern void ltoa(char *dst, unsigned long l);
extern void puts(const char *);
extern void sio_setup(int flag);
#define PAGEOFFSET 0xa0000000

/*
 * Change virtual addresses to physical addresses and vv.
 * These are trivial on the 1:1 Linux/MIPS mapping
 */
inline unsigned long virt_to_phys(volatile void * address)
{
	return (unsigned long)(address-PAGEOFFSET) ;
}

inline void * phys_to_virt(unsigned long address)
{
	return (void *)(address +PAGEOFFSET);
}

/* tc3160 hardware defines */
/* Hack: this is the mac address by default */
#define DEF_MAC_HI	0x00000000
#define DEF_MAC_LO	0xaabbccdd
#define DEF_LINK_LOOP   1000 
#define ADM6996_SWITCH 1
#define CONFIG_TC3160_RAMSIZE 0x800000

/* Number of descriptors and buffer size */
#define NUM_RDESC	48
#define NUM_TDESC	16
#define R_BUF_SIZE	0x700	/* needs to be < 2KB */
#define T_BUF_SIZE	0x700	/* needs to be < 2KB */

/* Number of descriptors and buffer size */
//#define NUM_RDESC	5
//#define NUM_TDESC	5
//#define R_BUF_SIZE	0x10	/* needs to be < 2KB */
//#define T_BUF_SIZE	0x10	/* needs to be < 2KB */


/* Descriptors and buffers: notice the alignments */
struct tc3160_desc rxdsc[NUM_RDESC] __attribute__ ((aligned(16), unused, __section__(".data.ramblock")));
struct tc3160_desc txdsc[NUM_TDESC] __attribute__ ((aligned(16), unused, __section__(".data.ramblock")));
unsigned char rxbuf[NUM_RDESC * R_BUF_SIZE] __attribute__ ((aligned(4), unused, __section__(".data.ramblock")));
unsigned char txbuf[NUM_TDESC * T_BUF_SIZE] __attribute__ ((aligned(4), unused, __section__(".data.ramblock")));

/* Private data for this ethernet device */
typedef struct {
  /* struct net_device_stats stats;*/ /* stats */
  unsigned long last_rxidx;      /* last rx idx to descriptor */
  unsigned long next_txidx;      /* next tx idx to descriptor */
  unsigned long last_txbytes;    /* last tx transferred bytes */
} TC3160_ETH_PRIV;

unsigned char *nData;   

/* NI | AI | RS/ | RU | RI */
#define ETH_IRQ_FLAGS	((1 << 16) | (1 << 15) | (1 << 8) | (1 << 7) | (1 << 6))
//#define ETH_IRQ_FLAGS	((1 << 16) | (1 << 15))

/* ST | SR */
#define ETH_RXTX_FLAGS	((1 << 13) | (1 << 1))
//#define ETH_RXTX_FLAGS	((1 << 13))

/* TE | RE | PR | RA | ASTP | HP | HO | DRTY | DBF */
#define ETH_MAC_FLAGS	((1 << 3) | (1 << 2) | (1 << 18) | (1 << 31) | (1 << 8) | \
		        (1 << 13) | (1 << 15) | (1 << 10) | (1 << 11))

/*#define ETH_MAC_FLAGS	((1 << 3) | (1 << 18) | (1 << 31) | (1 << 8) | \
		        (1 << 13) | (1 << 15) | (1 << 10) | (1 << 11))
*/
/* global defines */
int def_mac_hi;
int def_mac_lo;
int max_link_loop;
int dev_addr[6];

/* static allocate memory for private data */
TC3160_ETH_PRIV private;

void  initializeNetworkDrivers();
static void tc3160_eth_setup_desc(void);

void printl(unsigned long x)
{
char buf[32];
ltoa(buf,x); 
puts(buf);
}

void initializeNetworkDrivers()
{
/* local defines */
volatile int stat;
int i;
unsigned long word=0;
unsigned int* pTemp;
unsigned int nTemp;
unsigned char* aTemp;

sio_setup(1);

/*
 * Disable all interrupts
 *
 */
//  *(volatile unsigned long *)0x12000300 = 0;
 puts("IMR = ");
 printl(*(volatile unsigned long *)0xb2000300);
 puts("\n\r");
 //enable ethernet module interrupt mask
 *(volatile unsigned long *)0xb2000300 = 0xffffffff;
 puts("IMR = ");
 printl(*(volatile unsigned long *)0xb2000300);
 puts("\n\r");
 puts("IHPR = ");
 printl(*(volatile unsigned long *)0xb2000304);
 puts("\n\r");
 puts("ILPR = ");
 printl(*(volatile unsigned long *)0xb2000308);
 puts("\n\r");

/*
 * Initialization:
 * Configure MAC and external PHYs
 */ 

/* Step1: Write mac address to MAC_HI and MAC_LO registers */
  /* Turn off IRQ and stop receive/transmit */
  *(volatile unsigned long *)TC3160_CR_REG = 0;
  *(volatile unsigned long *)TC3160_IER_REG = 0;

  *(volatile unsigned long *)TC3160_MACAHR_REG = DEF_MAC_HI;
  *(volatile unsigned long *)TC3160_MACALR_REG = DEF_MAC_LO;

/* Step2: Write Predefined Multicast Hash Table value to Multicast Address HI and LO registers */
  /* Set multicast hash table to accept all */
  *(volatile unsigned long *)TC3160_MCHTHR_REG = 0xffffffff; 
  *(volatile unsigned long *)TC3160_MCHTLR_REG = 0xffffffff; 

/* Step3: Configure IFR to determine PHY interface(MII/RMII) */
  /* configure PHY if, MII with 100Mbps or 10Mbps */
  *(volatile unsigned long *)TC3160_IFR_REG = MiiIfExtPhyMii | MiiIfSpeed100M; 

/* Step4: Configure ADM6996 Switch */
#if ADM6996_SWITCH
  *(volatile unsigned long *)TC3160_GPOEN_REG = 0x0000feff;

  /* Reset ADM6996 switch */
  /* delay for prepare lan reset */
  for(i=0;i<100000;i++);

  *(volatile unsigned long *)TC3160_GPVAL_REG = 0x0000feff;
  for(i=0;i<109;i++);

  *(volatile unsigned long *)TC3160_GPVAL_REG = 0x0000ffff;
  for(i=0;i<109;i++);


  *(volatile unsigned long *)TC3160_GPOEN_REG = 0x0000ffff;
  for(i=0;i<109;i++);
  /* Reset ADM6996 switch end */

#endif

/* Step5: Configure MAC Control Register (operation mode such as store and forward, threshold bit) */

/*
 * Transmit/Receive Operations:
 * 
 */

/* Step1: Set up the transmit and receive descriptor list */

  //private = (TC3160_ETH_PRIV *)dev->priv;

  //memset(dev->priv, 0, sizeof(TC3160_ETH_PRIV));

  /* Clear all tx/rx buffer memory */
  memset((void *)KSEG1ADDR(&rxbuf[0]), 0, NUM_RDESC * R_BUF_SIZE);
  memset((void *)KSEG1ADDR(&txbuf[0]), 0, NUM_TDESC * T_BUF_SIZE);

  /* Initialize the indices */
  //private->last_rxidx = NUM_RDESC - 1;
  //private->next_txidx = 0;

  /* Intialize the descriptors */
  tc3160_eth_setup_desc();

/* Step2: Set up Bus Mode Register(BMR) for host access parameters */
  *(volatile unsigned long *)TC3160_BMR_REG = 0x00000001; /* DMA reset on */
  *(volatile unsigned long *)TC3160_BMR_REG = 0x00000001; /* Hack: reset it more */
  *(volatile unsigned long *)TC3160_BMR_REG = 0x00000001; /* than once due to HW */
  *(volatile unsigned long *)TC3160_BMR_REG = 0x00000001; /* problem */
  *(volatile unsigned long *)TC3160_BMR_REG = 0x00000001;
  *(volatile unsigned long *)TC3160_BMR_REG = 0x00000000; /* DMA reset off */

  /* burst length = 32 | BLE | ~DBO */
  word = (32 << 8) | (1 << 7) /* | (1 << 20) */;
  *(volatile unsigned long *)TC3160_BMR_REG = word; /* Bus mode */

/* Step3: Write to Interrupt Enable Register(IER) to mask unnecessary interrupts */
  /* Turn off IRQ and stop receive/transmit */
  *(volatile unsigned long *)TC3160_CR_REG = 0;
  *(volatile unsigned long *)TC3160_IER_REG = 0;

/* Step4: Write to the Transmit List Base Address Register(TLBAR) to indicate the start address of the descriptor list */
  /* Write to Receive List Base Address Register(RLBAR) to indicate the start address of the receive descriptor list */
  *(volatile unsigned long *)TC3160_TLBAR_REG = virt_to_phys(&(txdsc[0]));
  *(volatile unsigned long *)TC3160_RLBAR_REG = virt_to_phys(&(rxdsc[0]));
  puts("TLBAR RLBAR reg start\n\r");
  puts("TLBAR reg addr = ");
  printl((unsigned long)TC3160_TLBAR_REG);
  puts(" ,value = ");
  printl(*(volatile unsigned long *)TC3160_TLBAR_REG);
  puts("\n\rRLBAR reg addr = ");
  printl((unsigned long)TC3160_RLBAR_REG);
  puts(" ,value = ");
  printl(*(volatile unsigned long *)TC3160_RLBAR_REG);
  puts("\n\rTLBAR RLBAR reg end\n\r");
/* Step5: Write to the Control Register to program a suitable Data Mode to the TLI layer */
  /* FCE */
  word = (1 << 1);
  *(volatile unsigned long *)TC3160_FCR_REG = word; /* enable MAC flow ctrl */
    
  word = ETH_MAC_FLAGS;
  word |= MacLoopbackOff; /* OM = 0 */
  word |= MacFullDuplex;   /* FDM */               

  *(volatile unsigned long *)TC3160_MACCR_REG = word; /* configure MAC ctrller */

/* Step6: Set OWN bit of DMA controller to grant the descriptor list */

/* Step7: Set ST bit of the Control Register(CR) to initiates the transmit */
  /* Turn on the IRQ and start receive/transmit */
  stat = read_32bit_cp0_register(CP0_STATUS);
  stat |= 0x0400; /* Timer */
  stat |= 0x1000; /* Ethernet/UART */
  stat |= 0x3c00; /* 3-6 on */
  stat |= 0x0001; //Enable Interruptke 
  write_32bit_cp0_register(CP0_STATUS, stat);

  *(volatile unsigned long *)TC3160_IER_REG = ETH_IRQ_FLAGS;
  puts("TC3160_IER_REG = ");
  printl(*(volatile unsigned long*)TC3160_IER_REG);
  puts(", addr = ");
  printl((unsigned long)TC3160_IER_REG);
  puts("\n\r");

  *(volatile unsigned long *)TC3160_CR_REG = ETH_RXTX_FLAGS; /* enable MAC engine */

//test send a packet out
#if 1
//wait for a while
for(i=0;i<0x800000;i++); 

for (i=0,pTemp=(unsigned int*)(txdsc[0].desc2);i<0x100;i++,pTemp++)
  *pTemp = i+1;
//*(volatile unsigned int*)(0xa0400000+i*4) = i+1;

  txdsc[0].desc1 = 0x61000050;
  txdsc[0].desc0 = 0x80000000;

  *(volatile unsigned int*)0xb2000184 = 0x1;  

  while(1)
  {
         stat = read_32bit_cp0_register(CP0_STATUS);
	puts("SR = ");
	printl(stat);
	puts("\n\r");
         stat = read_32bit_cp0_register(CP0_CAUSE);
	puts("CAUSE = ");
	printl(stat);
	puts("\n\r");
	puts("ETH SR = ");
	printl(*(volatile unsigned long*)0xb2000194);
	puts("\n\r");
	puts("IRR = ");
        printl(*(volatile unsigned long*)0xb200030c);
        puts("\n\r");
    for(i=0;i<0x100000;i++);
  }

//  puts("sent a packet\n\r");
//  puts("start receiving test\n\r");
//test receive operation:
/*
  while(1) 
  {
    puts("txdsc[0].desc0 = ");
    printl(txdsc[0].desc0);
    puts("\n\r");
    puts("rxdsc[0].desc0 = ");
    printl(rxdsc[0].desc0);
    puts("\n\r");
    puts("rxdsc[20].desc0 = ");
    printl(rxdsc[20].desc0);
    puts("\n\r");

    for(i=0;i<0x100000;i++); 
    for(i=0,aTemp=(unsigned char*)(rxdsc[0].desc2);i<0x40;i++,aTemp++)
    {
      printl(*aTemp);
      puts("\n\r");
    }
  }   
*/

/*
if(0)
while (1)
{
 printl (*(volatile unsigned int*)0xb2000190);
 puts("\n\r");
 printl (*(volatile unsigned int*)0xb2000194);
 puts("\n\r");
 printl (*(volatile unsigned int*)0xb2000198);
 puts("\n\r");
 printl (*(volatile unsigned int*)0xb2000100);
 puts("\n\r");
 printl (0x77777777);

for(i=0;i<0x100000;i++); 


}
*/
#endif


/* Step8: The transmit operation can be resume by writing to the Transmit Poll Demand Register(TPDR) if DMA controller enters its suspended mode because its underflow error or transmit buffer unavailable error. */
/* Step9: If the last descriptor has been detected, it indicated the buffers. */
/* Step10: If completed, update the TDES0 status and the control and status registers of the Ethernet module. The host is also interrupted to check the status of the transfer */


/*
 * Receive Operations
 *
 */
/* Step1: The host driver sets up the receive descriptor list */
/* Step2: The host driver writes Bus Mode Register(BMR) to set up host access parameterss need by receive operation, such as burst length(PBL) bits */
/* Step3: Write to IER to mask unnecessary interrupts */
/* Step4: Write to Receive List Base Address Register(RLBAR) to indicate the start address of the receive descriptor list */
/* Step5: Set the OWN bit to grant the descriptor of the receive descriptor list to the DAM controller for its acquiring */
/* Step6: Set the SR bit of the Control Register(CR) to initiate the receive process */
/* Step7: If DMA controller suspends, write to Receive Poll Demand Register(RPDR) to resume receive operations */
/* Step8: If completed, driver should update the REDS0 status and the control and status registers of the Ethernet module */



}

/* Setting rx/tx descriptors */
static void tc3160_eth_setup_desc(void)
{
  register int i;
//  struct tc3160_desc *desc_ptr =(struct tc3160_desc*) 0;
  struct tc3160_desc *desc_ptr;
  unsigned long base_addr = 0;

#if 0
  unsigned int nValue1 = TXDESC_ADDR;
  unsigned int nValue2 = RXDESC_ADDR;
  unsigned int* nTemp1;
  unsigned int* nTemp2;
  txdsc = (struct tc3160_desc*)nValue1;  
  rxdsc = (struct tc3160_desc*)nValue2;
  txbuf = (unsigned char*) 0x00500000;
  rxbuf = (unsigned char*) 0x00600000;

printl (nValue1);
printl (nValue2);
printl (0x11111111);
printl ((unsigned int)rxdsc);
printl ((unsigned int)txdsc);
printl (0x99999999);
printl ((unsigned int)txbuf);
printl ((unsigned int)rxbuf);
printl (0xaaaaaaaa);

  nTemp1 = (unsigned int*)TXDESC_ADDR;
  nTemp2 = (unsigned int*)RXDESC_ADDR;
printl((unsigned int)nTemp1);
printl((unsigned int)nTemp2);
printl(*nTemp1);
printl(*nTemp2);
  for (i =0; i< (NUM_RDESC-1);i++)
  {
	*(nTemp1+0+i*4) = 0;
	*(nTemp1+1+i*4) =(0x61000000 | T_BUF_SIZE);
 	*(nTemp1+2+i*4) = (unsigned int) txbuf;
	*(nTemp1+3+i*4) = ((unsigned int)(nTemp1+((i+1)*4)))&0x0fffffff;
	*(nTemp2+0+i*4) = 0x80000000;
	*(nTemp2+1+i*4) =(0x01000000 | R_BUF_SIZE);
 	*(nTemp2+2+i*4) = (unsigned int) txbuf;
	*(nTemp2+3+i*4) = ((unsigned int)(nTemp2+((i+1)*4)))&0x0fffffff;
	printl((unsigned int)(0+TXDESC_ADDR));
	printl(*(volatile unsigned int*)(i*0x10+0x0+TXDESC_ADDR));
	printl(*(volatile unsigned int*)(i*0x10+0x4+TXDESC_ADDR));
	printl(*(volatile unsigned int*)(i*0x10+0x8+TXDESC_ADDR));
	printl(*(volatile unsigned int*)(i*0x10+0xc+TXDESC_ADDR));
        printl (0xbbbbbbbb);
	printl((unsigned int)(0+RXDESC_ADDR));
	printl(*(volatile unsigned int*)(i*0x10+0x0+RXDESC_ADDR));
	printl(*(volatile unsigned int*)(i*0x10+0x4+RXDESC_ADDR));
	printl(*(volatile unsigned int*)(i*0x10+0x8+RXDESC_ADDR));
	printl(*(volatile unsigned int*)(i*0x10+0xc+RXDESC_ADDR));
        printl (0xcccccccc);
  }

  rxdsc-> desc3 = (unsigned long)virt_to_phys(rxdsc + 1);
  rxdsc-> desc2 = (unsigned int) rxbuf;    
  rxdsc-> desc1 = (0x01000000 | R_BUF_SIZE);    
  rxdsc-> desc0 = 0x80000000; /* Owned by DMA */

  txdsc-> desc3 = (unsigned long)virt_to_phys(txdsc + 1);
  txdsc-> desc2 = (unsigned int)txbuf;    
  txdsc-> desc1 = (0x61000000 | T_BUF_SIZE);    
  txdsc-> desc0 = 0x0; /* Owned by DMA */
  printl ((unsigned int)rxdsc);
  printl (*(volatile unsigned int*)(rxdsc+0));
  printl (*(volatile unsigned int*)(rxdsc+1));
  printl (*(volatile unsigned int*)(rxdsc+2));
  printl (*(volatile unsigned int*)(rxdsc+3));
  printl ((unsigned int)rxbuf);
  printl (0x12345678);
  printl ((unsigned int)txdsc);
  printl (*(volatile unsigned int*)(txdsc+0));
  printl (*(volatile unsigned int*)(txdsc+1));
  printl (*(volatile unsigned int*)(txdsc+2));
  printl (*(volatile unsigned int*)(txdsc+3));
  printl ((unsigned int)txbuf);
  printl (0x87654321);
  rxbuf+= R_BUF_SIZE;
  rxdsc++;
  txbuf += T_BUF_SIZE;
  txdsc++;\

#endif

  /* Setup rx descriptor */
  for (i = 0, desc_ptr = (struct tc3160_desc *)&rxdsc[0]; 
	i < (NUM_RDESC - 1); i++, desc_ptr++) {
    desc_ptr->desc0 = (unsigned long)(0x80000000); /* Owned by DMA */
    desc_ptr->desc1 = (unsigned long)(0x01000000 | R_BUF_SIZE);
    desc_ptr->desc3 = (unsigned long)virt_to_phys(desc_ptr + 1);

    puts("rx descriptor ");
    printl(i);
    puts(" :\n\raddr = ");
    printl((unsigned long)desc_ptr);
    puts("\n\rdesc0 = ");
    printl((unsigned long)(desc_ptr->desc0));
    puts(" ,addr = ");
    printl((unsigned long)(&(desc_ptr->desc0)));
    puts("\n\r");
    puts("desc1 = ");
    printl((unsigned long)(desc_ptr->desc1));
    puts(" ,addr = ");
    printl((unsigned long)(&(desc_ptr->desc1)));
    puts("\n\r");
    puts("desc3 = ");
    printl((unsigned long)(desc_ptr->desc3));
    puts(" ,addr = ");
    printl((unsigned long)(&(desc_ptr->desc3)));
    puts("\n\r");

  }
  desc_ptr->desc0 = (unsigned long)(0x80000000); /* Owned by DMA */
  desc_ptr->desc1 = (unsigned long)(0x03000000 | R_BUF_SIZE);
  desc_ptr->desc3 = (unsigned long)virt_to_phys(&rxdsc[0]);
  puts("rx descriptor ");
  printl(i);
  puts(" :\n\raddr = ");
  printl((unsigned long)desc_ptr);
  puts("\n\rdesc0 = ");
  printl((unsigned long)(desc_ptr->desc0));
  puts("\n\rdesc1 = ");
  printl((unsigned long)(desc_ptr->desc1));
  puts("\n\rdesc3 = ");
  printl((unsigned long)(desc_ptr->desc3));
  puts("\n\r");




  /* Setup tx descriptor */
  for (i = 0, desc_ptr = (struct tc3160_desc *)&txdsc[0]; 
	i < (NUM_TDESC - 1); i++, desc_ptr++) {
    desc_ptr->desc0 = (unsigned long)0x0; /* Owned by CPU */
    desc_ptr->desc1 = (unsigned long)(0x61000000 | T_BUF_SIZE);
    desc_ptr->desc3 = (unsigned long)virt_to_phys(desc_ptr + 1);
    puts("tx descriptor ");
    printl(i);
    puts(" :\n\raddr = ");
    printl((unsigned long)desc_ptr);
    puts("\n\rdesc0 = ");
    printl((unsigned long)(desc_ptr->desc0));
    puts("\n\rdesc1 = ");
    printl((unsigned long)(desc_ptr->desc1));
    puts("\n\rdesc3 = ");
    printl((unsigned long)(desc_ptr->desc3));
    puts("\n\r");
  }
  desc_ptr->desc3 = (unsigned long)virt_to_phys(&txdsc[0]);
  desc_ptr->desc1 = (0x63000000 | T_BUF_SIZE);
  desc_ptr->desc0 = 0x0; /* Owned by CPU */
    puts("tx descriptor ");
    printl(i);
    puts(" :\n\raddr = ");
    printl((unsigned long)desc_ptr);
    puts("\n\rdesc0 = ");
    printl((unsigned long)(desc_ptr->desc0));
    puts("\n\rdesc1 = ");
    printl((unsigned long)(desc_ptr->desc1));
    puts("\n\rdesc3 = ");
    printl((unsigned long)(desc_ptr->desc3));
    puts("\n\r");

  /* Point rx descriptor to buffer */
  for (i = 0, desc_ptr = (struct tc3160_desc *)&rxdsc[0], 
        base_addr = (unsigned long)virt_to_phys(&rxbuf[0]); 
        i < NUM_RDESC; i++, desc_ptr++, base_addr += R_BUF_SIZE) 
  {
    desc_ptr->desc2 = base_addr;
    puts("rx descriptor desc2 =");
    printl(desc_ptr->desc2);
    puts("\n\r");
  }

  /* Point tx descriptor to buffer */
  for (i = 0, desc_ptr = (struct tc3160_desc *)&txdsc[0],
        base_addr = (unsigned long)virt_to_phys(&txbuf[0]);
        i < NUM_TDESC; i++, desc_ptr++, base_addr += T_BUF_SIZE) 
  {
    desc_ptr->desc2 = base_addr;
    puts("tx descriptor desc2 = ");
    printl(desc_ptr->desc2);
    puts("\n\r");
  }
}

