/***********************************************************************/
/*  start.S							       */
/*								       */
/*  start the CPU and configure SDRAM				       */
/*								       */
/*  This file is modified from ini_mips.s from TrendChip Technology Co.*/
/*  Robert Wu, Billion Electric Co., Ltd.			       */
/*  2004/04/26							       */
/*								       */
/***********************************************************************/

//  ========================================================================
//  File Name	    = ini_mip.s
//  File Revision   = 1.0
//  File Serial No. =
//  ========================================================================
//
//  TRENDCHIP TECHNOLOGIES PROPRIETARY! DO NOT COPY!
//
//  configure gpio, arbiter, dram, flash here
//  v0.9 HuanShiou Tseng	2004/04/23

#include "testled.h"

#define DMT_MBIST_BASE		0xb4000040
#define CR_ARB_BASE		0xBFB00000
#define GPIOBASE		0xBFBF0200
#define CR_SMC_BASE		0xBFB10000
#define CR_DMC_BASE		0xBFB20000
#define ROUND_ROBIN_ARBITER		1
#define TEST_MEM_SIZE		32768

/* Test GPIO LED */
// configure GPIO for display
#define LIGHT_LED(gpioreg) \
	la v0, GPIOBASE; \
	li a0, ~(gpioreg); \
	sw a0,	0x4(v0)
#define DISPLAYLED(d0,d1) \
	la v0, GPIOBASE; \
1:	li t5, d0; \
	li t6, d1; \
	sh t5, 0x6(v0); \
2:	addi t6, -1; \
	bnez t6, 2b;\
	nop; \
	li t5, 0xff; \
	li t6, d1; \
	sh t5, 0x6(v0); \
3:	addi t6, -0x10; \
	bnez t6, 3b; \
	nop


#include <asm/asm.h>
#include <asm/cacheops.h>
#include <asm/current.h>
#include <asm/offset.h>
#include <asm/processor.h>
#include <asm/regdef.h>
#include <asm/mipsregs.h>
#include <asm/stackframe.h>
#include <asm/addrspace.h>
#include <asm/tc3160/gpio.h>
#include "address.h"

#define STATUS_BEV 22

/****************************************************/
/*              External Functions                  */
/****************************************************/


/****************************************************/
/*              Start Code Begin                    */
/****************************************************/

.text

LEAF(start)
EXPORT(__start)
	.set	noreorder
	.set	noat

	/* Reset vector */
	.org	0x0
	b	STARTBOOTCODE 
	nop

	/* TLB exception */
	.org	0x100
	b	__tlb_exception
	addiu	k0, zero, 0x0
	
	/* General exception */
	.org	0x180
	b	__general_exception
	nop

	.org	0x400

STARTBOOTCODE:
// configure GPIO for display
		la		v0,		GPIOBASE
		li		a0,		0x00000155
// configure gpio 0,1,2,3,4 as ouput
		li		a1,		0x001f
// also set it as open drain
		sw		a0,		0(v0)
		sw		a1,		0x14(v0)


//		li		t1,		0xbfbf010c
//		li		t2,		0xbfc00000
#if 0
	mfc0	k1, CP0_STATUS
	nop
	srl	k1, k1, STATUS_BEV		// Shift BEV bit into bit #0
	andi	k1, k1, 1				// Check BEV bit being 0 or 1
	bne	zero, k1, $$Cold_Reset	// BEV is 1 -> must be cold reset
	nop

$$Warm_Reset:
	ori	k0, zero, 1				// Set $k0 if warm reset
	beq	zero, zero, reset_Handler
	nop
#endif

$$Cold_Reset:
	//---------------------------------------------------------------------
	// Start SDRAM initialization process
	//---------------------------------------------------------------------
#define SDRAM16			1


#if 1
// configure arbiter
	    la		v1,		CR_ARB_BASE
#ifdef ROUND_ROBIN_ARBITER
	    li		a0,		0x8000ffff		// round robin scheme
	    li		a1,		0x0
#else
	    li		a0,		0x0000ffff
	    li		a1,		0x32107777		// atm > mac > apb > cpu > others

#endif
	    sw		a0,		0x00(v1)
	    sw		a1,		0x04(v1)




// configure DMC bank range
// SDRAM address:	0x80000000 to 0x807fffff  size: 8MB



		li	a0,		0x80000000
		li	a1,		0x80000101	// bank1 --> 0000_0000 ~ 007f_ffff
		li	a2,		0x80000202
		li	a3,		0x80000303

		sw	a0,		0x10(v1)
		sw	a1,		0x14(v1)
		sw	a2,		0x18(v1)
		sw	a3,		0x1c(v1)
#endif


// configure SMC
// SRAM address:
//		1. FLASH	 16bit begin at 0x1fc00000 to 0x1fdfffff  size:	  2MB
//		2. SRAM		8bit begin at 0x1fe00000 to 0x1fe3ffff	size: 256KB

		li	a0,		0x80071e1e
		li	a1,		0x00071f1f
		li	a2,		0x00040000

		sw	a0,		0x20(v1)
		sw	a1,		0x24(v1)
		sw	a2,		0x28(v1)
// SRAM rank 3,4,5
		li	a0,		0x00040808
		li	a1,		0x00041010	//without bridge
		li	a2,		0x00050000
		sw	a0,		0x2c(v1)
		sw	a1,		0x30(v1)
		sw	a2,		0x34(v1)


		li	v1,		CR_SMC_BASE

		li	a0,		0x102a9040	/* 16-bit flash configure  */
		li	a1,		0x002a8410	/*  8-bit sram	configure  */
		li	a2,		0x10aa8100	/* 16-bit hpi8	configure  */
		li	a3,		0x20001060	/* 32-bit dmt	configure  */


// no sram, hpi8 here



		sw	a0,		0x0(v1)	/* CR_SMC_BANK0	    */
//		sw	a1,		0x4(v1)	/* CR_SMC_BANK1	    */
//		sw	a2,		0x10(v1)	/* CR_SMC_BANK4	    */
//		sw	a3,		0x14(v1)	/* CR_SMC_BANK5	    */
	nop


DISPLAYLED(~(LED1),0x30000)

// init SDRAM
// wait 250 us
		li		a0,		25000
1:				/* wait 250 us */
		subu		a0,		0x01
		bnez		a0,		1b
		nop
		la		v1,		CR_DMC_BASE

//	li	a0,		0x00e44400	/* 16bit width, raw 8 col 12, 2T CAS Latency */
	li	a0,		0x00e84400	/* 16bit width, raw 8 col 12, 3T CAS Latency */

		sw		a0,		0x00(v1)

		li		a0,		0x02
		sb		a0,		0x07(v1)     /* enter precharge all mode */

		la		a1,		0xa0000000
		sw		a1,		0x00(a1)

		li		a0,		0x04
		sb		a0,		0x07(v1)     /* enter CBR mode */

		li		a0,		0x08
		la		a1,		0xa0000000

1:					  /* additional 8 CBR Auto Refresh Cycle */
		sub		a0,		0x01
		sw		a1,		0x00(a1)
		bnez		a0,		1b
		nop
		li		a0,		0x03   		    /* enter MRS mode */
		sb		a0,		0x07(v1)
		li		a1,		0xa0000066    /* CAS Latency */
.set at
//		sh		a0,		0x00(a1)
		sb		$0,		0x00(a1)

		li		a0,		0x02   		    /* enter precharge all mode */
		sb		a0,		0x07(v1)
.set noat

		la		a0,		0xa0000000
		nop

		sb		a1,		0x03(a0)
		li		a1,		0x41
		li		a0,		0x4e

		sb		a1,		0x06(v1)     /* page mode enable   */
		sb		zero,		0x07(v1)     /* normal mode	    */
		sb		a0,		0x00(v1)     /* dram refresh timer */

//#endif
		nop
		nop
		nop

DISPLAYLED(~(LED2),0x30000)
				

//=============================================================================
// Reset Handler Entry Point
//=============================================================================
		.globl	reset_Handler
reset_Handler:
		//---------------------------------------------------------------------
		// (1) CU0 <- 1, let CoPocessor 0 to be usable in user mode
		//	  /* however, we should stay in kernel mode, so CP0 is always
		//		 usable regardless of how CU0 is set */
		// (2) BEV <- 1, use alternate bootstrap exception vector, which point
		//	  into ROM, until we have copied exception handlers to RAM
		// (3) IntMask ( Int[5:0]Sw[1:0] ) <- 0, turn off all interrupt source
		// (4) KUold/KUprevious/KUcurrent <- 0, kernel Mode
		// (5) IEcurrent <- 0, disable interrupt
		//---------------------------------------------------------------------
		// init interrupt controler
		// all interrupts belong to IRQ
		// disable all interrupt

		li		t0, 0xbfb40000
		nop

		sw		zero, 0x0(t0)				/* all interrupt belong to IRQ */
		sw		zero, 0x04(t0)			/* set all interrupt mask to 0 .. disable all interrupt */

		li		t0, 0x10400000				// Initialize the status register
		mtc0		t0, CP0_STATUS
		nop

		//---------------------------------------------------------------------
		// Clear exception cause register
		//---------------------------------------------------------------------
		mtc0	zero, CP0_CAUSE
		nop

DISPLAYLED(~(LEDf),0x60000)
		//---------------------------------------------------------------------
		// The following program is generated from ccmipe driver. To copy code
		// from flash to SDRAM, you have to decide column size of SDRAM since
		// the column is the lower part in the address.
		// The original code resides in ramtest.c
		//---------------------------------------------------------------------



		//---------------------------------------------------------------------
		// Before copying code from flash to SDRAM, do DRAM test of 32K and
		// stop at a loop if an error occurs.
		//---------------------------------------------------------------------

//		li		t0, 0x80000000				// Start address of SDRAM
		li		t0, 0xa0000000				// Start address of SDRAM
		li		t1, 0xFFFFFFFF


//		li		t2, 32768
		li		t2, TEST_MEM_SIZE


$$TestFirst32KBLoop1:
1:		sw		t1, (t0)
		nop
		lw		t3, (t0)
		nop
//		bne		t3, t1, $$DRAMTestError
		bne		t3, t1, 1b
		nop
		addiu	t0, t0, 4
		addiu	t2, t2, -4
		bgtz	t2, $$TestFirst32KBLoop1
		nop
		bne		t1, zero, $$TestFirst32KBLoop2
		nop
		beq		zero, zero, $$TestFirst32KBComplete
		nop
$$TestFirst32KBLoop2:
		li		t1, 0x00000000
//		li		t0, 0x80000000				// Start address of SDRAM
		li		t0, 0xa0000000				// Start address of SDRAM
//		li		t2, 32768
		li		t2, TEST_MEM_SIZE 
		beq		zero, zero, $$TestFirst32KBLoop1
		nop
$$TestFirst32KBComplete:

DISPLAYLED(~(LED3),0x60000)

	/* Setup stack here */
	la	t0, stack_sp
	lw	t1, 0(t0)
#if 1
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
#endif
// KWU delay ends
	add	sp, t1, 0

	/* Initialize tftpd */
//	jal	tftpd
//	nop



#if 0
		//---------------------------------------------------------------------
		// Copy codes from flash memory to SDRAM
		//---------------------------------------------------------------------
		// __ghsbegin_ROMExcept.. you can define your own souce address in flash.
		la	t0, __ghsbegin_ROMExcept	  // source address in ROM
		li	t1, 0x80000000		      // Start address of SDRAM
		li	t2, 0x4000			// length = 16K bytes

$$CopyLoop:
		lw	t3, (t0)
		addiu	t0, t0, 4
		sw	t3, (t1)
		addiu	t1, t1, 4
		addiu	t2, t2, -4
		bgtz	t2, $$CopyLoop
		nop

		//---------------------------------------------------------------------
		// Setup PIC pointer($s7), PID/SDA pointer($gp) and stack pointer($sp)
		//---------------------------------------------------------------------

		li		s7, 0x00000000
		la		gp, _gp
		li		sp, 0x80007ffc

#endif
		//---------------------------------------------------------------------
		// (1) CU0 <- 1, let CoPocessor 0 to be usable in user mode
		//	  /* however, we should stay in kernel mode, so CP0 is always
		//		 usable regardless of how CU0 is set */
		// (2) BEV <- 0, use regular bootstrap exception vector, which point
		//	  into RAM
		// (3) IntMask ( Int[5:0]Sw[1:0] ) <- 1, turn on all interrupt source
		// (4) KUold/KUprevious/KUcurrent <- 0, kernel Mode
		// (5) IEold/IEprevious/IEcurrent <- 1, enable interrupt
		//---------------------------------------------------------------------
		//li		t0, 0xf200ff15	// Set status register
		li		t0, 0x0240ff15
		mtc0		t0, CP0_STATUS
		nop
		nop


DISPLAYLED(~(LED4),0x3000) \

#if 1
	/* Decompressing the kernel */
	jal	decompress_kernel
	nop
#endif 

	
DISPLAYLED(~(LED5),0x30000)

	/* Jump to kernel */
	la	t0, kernel_start_addr
	lw	t1, 0(t0)
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	jr	t1	
	nop

__tlb_exception:
	mfc0	a0, CP0_CAUSE
	mfc0	a1, CP0_STATUS
	jal	puts_tlb_exception
	nop
	b	reset_Handler
	nop

__general_exception:
	.set noreorder
	.set noat
	//mfc0	a0, CP0_CAUSE
	//mfc0	a1, CP0_STATUS
	//jal	puts_general_exception
	//nop
	mfc0	k1, CP0_EPC
	nop
	jr	k1
	rfe
	.set at
	.set reorder

	END(start)

	.rdata

kernel_start_addr:
	.long	KERNEL_START_ADDRESS

stack_sp:
	/* Keeping 64KB (0x10000) for ram data */
	/* .long	(KSEG1 + CONFIG_TC3160_RAMSIZE - 0x10000)*/
	.long  0xa07f0000

