#include <linux/config.h>
#include <linux/version.h>

#define RS_STROBE_TIME (10*HZ)

/*
 * End of serial driver configuration section.
 */

#include <linux/module.h>

#include <linux/types.h>
#include <linux/serial.h>
#include <linux/serialP.h>
#include <asm/tc3162/tc3162.h>
#include <asm/tc3162/interrupts.h>
#include <asm/tc3162/sio.h>
//#include <linux/serial_reg.h>
#include <asm/serial.h>
#define LOCAL_VERSTRING ""

#include <linux/errno.h>
#include <linux/signal.h>
#include <linux/sched.h>
#include <linux/timer.h>
#include <linux/interrupt.h>
#include <linux/tty.h>
#include <linux/tty_flip.h>
#include <linux/major.h>
#include <linux/string.h>
#include <linux/fcntl.h>
#include <linux/ptrace.h>
#include <linux/ioport.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <linux/delay.h>

/* the real base address will be defined on asm/serial.h */
#define SERIAL_BASE  0 

#define UART_DATA	(SERIAL_BASE+0*4)
#define UART_LSR	(SERIAL_BASE+1*4)
#define UART_MSR	(SERIAL_BASE+2*4)
#define UART_LCR	(SERIAL_BASE+3*4)
#define UART_MCR	(SERIAL_BASE+4*4)
#define UART_DIV_L	(SERIAL_BASE+5*4)
#define UART_DIV_H	(SERIAL_BASE+6*4)
#define UART_ISC	(SERIAL_BASE+7*4)

/*
 * These are the definitions for the Line Status Register
 */
#define UART_LSR_SOFTRST	0x80
#define UART_LSR_TXE		0x40
#define UART_LSR_BI		0x20
#define UART_LSR_FE		0x10
#define UART_LSR_OE		0x08
#define UART_LSR_PE		0x04
#define UART_LSR_RXRDY		0x02
#define UART_LSR_TXRDY		0x01

/*
 * These are the definitions for the Modem Status Register
 */
#define UART_MSR_TERI	0x80
#define UART_MSR_DDCD	0x40
#define UART_MSR_DCTS	0x20
#define UART_MSR_DDSR	0x10
#define UART_MSR_RI	0x08
#define UART_MSR_DCD	0x04
#define UART_MSR_CTS	0x02
#define UART_MSR_DSR	0x01
#define UART_MSR_ANY_DELTA 0xF0	/* Any of the delta bits! */

/*
 * These are the definitions for the Line Control Register
 * 
 * Note: if the word length is 5 bits (UART_LCR_WLEN5), then setting 
 * UART_LCR_STOP will select 1.5 stop bits, not 2 stop bits.
 */
#define UART_LCR_SBC	0x20	/* Set break control */
#define UART_LCR_EPAR	0x10	/* Even parity select */
#define UART_LCR_PARITY	0x08	/* Parity Enable */
#define UART_LCR_STOP	0x04	/* Stop bits: 0=1 stop bit, 1= 2 stop bits */
#define UART_LCR_WLEN5  0x00	/* Wordlength: 5 bits */
#define UART_LCR_WLEN6  0x01	/* Wordlength: 6 bits */
#define UART_LCR_WLEN7  0x02	/* Wordlength: 7 bits */
#define UART_LCR_WLEN8  0x03	/* Wordlength: 8 bits */

/*
 * These are the definitions for the Modem Control Register
 */
#define UART_MCR_AUTO_CALIBRATE_CLOCK	0x80
#define UART_MCR_AUTOCTS	0x40
#define UART_MCR_AUTORTS	0x20
#define UART_MCR_LOOP	0x10	/* Enable loopback test mode */
#define UART_MCR_OUT2	0x08	/* Out2 complement */
#define UART_MCR_OUT1	0x04	/* Out1 complement */
#define UART_MCR_RTS	0x02	/* RTS complement */
#define UART_MCR_DTR	0x01	/* DTR complement */
/*
 * These are the definitions for the Interrupt status/control Register
 */
#define UART_IIR_NO_INT	0x0F
#define UART_IIR_THRI	0x01
#define UART_IIR_RDI	0x02
#define UART_IIR_RLSI	0x04
#define UART_IIR_MSI	0x08

#define UART_IER_THRI	0x10
#define UART_IER_RDI	0x20
#define UART_IER_RLSI	0x40
#define UART_IER_MSI	0x80


static void tc3162_sio_putc(char data);
static char tc3162_sio_getc(void);
static int tc3162_sio_tstc(void);

static struct timer_list serial_timer;

static unsigned long  uclk_65000[13]={
			357500, 	// uclk 5.5     Baud Rate 115200
			175500, 	// uclk 2.7     Baud Rate 57600
			119808, 	// uclk 1.8432  Baud Rate 38400
			89856,	// uclk	1.3824	Baud Rate 28800
			59904,	// uclk 0.9216	Baud Rate 19200
			44928,	// uclk 0.6912	Baud Rate 14400
			29952,	// uclk 0.4608	Baud Rate 9600
			14976,	// uclk 0.2304	Baud Rate 4800
			7488,	// uclk 0.1152	Baud Rate 2400 
			3744,	// uclk 0.0576	Baud Rate 1200
			1872,	// uclk 0.0288	Baud Rate 600
			936,		// uclk 0.0144	Baud Rate 300
			343		// uclk 0.00528	Baud Rate 110
};

static int uart_setup()
{
	int i;
	unsigned long	div_x,div_y;
	unsigned long	word;

//	pause(100);
// Set FIFO controo enable, reset RFIFO, TFIFO, 16550 mode, watermark=0x00 (1 byte)
	VPchar(CR_UART_FCR) = UART_FCR|UART_WATERMARK;

// Set modem control to 0
	VPchar(CR_UART_MCR) = UART_MCR;

// Disable IRDA, Disable Power Saving Mode, RTS , CTS flow control
	VPchar(CR_UART_MISCC) = UART_MISCC;

// Set interrupt Enable to, enable Tx, Rx and Line status
	VPchar(CR_UART_IER) = UART_IER;	
//	

	/* access the bardrate divider */

	VPchar(CR_UART_LCR) = UART_BRD_ACCESS;

	div_y = UART_XYD_Y;
	div_x = (unsigned int)(uclk_65000[0]/SYS_HCLK)*2;
	word = (div_x<<16)|div_y;
	VPint(CR_UART_XYD) = word;

/* Set Baud Rate Divisor to 3*16		*/
	VPchar(CR_UART_BRDL) = UART_BRDL;

	VPchar(CR_UART_BRDH) = UART_BRDH;

/* Set DLAB = 0, clength = 8, stop =1, no parity check 	*/
	VPchar(CR_UART_LCR) = UART_LCR;

	return 0;
}
static void tc3162_sio_putc(char data)
{
	while (!(VPchar(CR_UART_LSR) & LSR_THRE))
	VPchar(CR_UART_THR) = data; 
}

static char tc3162_sio_getc(void)
{
	int i;
	char ch;

    while (1)
	{
		if 	(VPchar(CR_UART_LSR) & 0x1)
			break;		
	}	
	i=VPchar(CR_UART_RBR);
	tc3162_sio_putc( i & 0xff);
	return (i & 0xff);
}

static int tc3162_sio_tstc(void)
{
	return (VPchar(CR_UART_LSR) & 0x1);
}

static void dect_timer(unsigned long dummy)
{
	static unsigned long last_strobe;
	struct async_struct *info;
	unsigned int	i;
	unsigned long flags;

	last_strobe = jiffies;
	mod_timer(&serial_timer, jiffies + RS_STROBE_TIME);

		save_flags(flags); cli();
		dect_interrupt_single1(0, NULL, NULL);
		restore_flags(flags);

		mod_timer(&serial_timer, jiffies );
}

void dect_interrupt_single1(int irq, void *dev_id, struct pt_regs * regs)
{
	char ch=0;
		if (tc3162_sio_tstc()) {
		ch = (unsigned char) tc3162_sio_getc();
		}

}

void __init robert_dect_init()
{
	int i;
	printk("robert rs init\n");
	uart_setup();
	init_timer(&serial_timer);
	serial_timer.function = dect_timer;
	mod_timer(&serial_timer, jiffies + RS_STROBE_TIME);
	printk("return %d\n",i);
}

void __exit robert_dect_fini(void) 
{
	printk("robert rs fini\n");
}

module_init(robert_dect_init);
module_exit(robert_dect_fini);

MODULE_LICENSE("GPL");

