/*  
 * Implementations used to for Ethernet MAC Controller Operation register.
 *
 * Written by Robert Wu, Billion Co.
 * Support for TrendChip TC3162
 */


#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/spinlock.h>
#include <linux/interrupt.h>
#include <linux/signal.h>
#include <linux/sched.h>
#include <linux/module.h>
#include <linux/compatmac.h>
#include <linux/mm.h>
#include <linux/init.h>
#include <linux/timer.h>
#include <linux/delay.h>
// Sam add
#include <linux/reboot.h>
// End Sam add
#include <asm/virtconvert.h>
#include <asm/tc3162/interrupts.h>
#include <asm/tc3162/gpio.h>
//added by Robert
#include <asm/tc3162/testled.h>
#include <asm/tc3162/emac.h>
//Add Brad

#include <linux/sockios.h>
#include <linux/tc3162ioctl.h>
#define DBG_PRINT       printk
#define ERR_PRINT       printk
#define MSG_PRINT       printk
#define DEBUG_PRINT	printk

/* Add gpio major */
#define GPIO_MAJOR 154

/* Hack: this is the mac address by default */
#define DEF_MAC_HI	0x00000000
#define DEF_MAC_LO	0xaabbccff

/* Number of descriptors and buffer size */
//#define NUM_RDESC	48
#define NUM_RDESC	64
#define NUM_TDESC	64
//#define NUM_TDESC	16
#define R_BUF_SIZE	0x00000700 /* needs to be < 2KB */
#define T_BUF_SIZE	0x700	/* needs to be < 2KB */

/* Sam static phyDeviceList_t phyDeviceList[] = {
	{ , "LXT972" },
	{ 0x0022, "AC101L" },
	{ 0x0243, "IP101" },
	{ 0x8201, "RTL8201" } //tc3162 demo board phy
}; */
#if 0 //jj
unsigned int byte_swap(volatile unsigned int);
#else //refer to TC
#define ROL(x,n)                (((x)<<(n))|((x)>>(32-(n))))
#define byte_swap(x)    ((ROL(x,8) & 0x00ff00ff) | ROL(x & 0x00ff00ff,24))
#endif
void intMaskSet(int,int);
void reset_emac_reg_param(void);
static int tc3162_emac_notify_sys(struct notifier_block*, unsigned long , void *);
static struct notifier_block tc3162_emac_notifier = {tc3162_emac_notify_sys , NULL, 0};
#define ENABLE 1
#define DISABLE 0
unsigned long PHY_ADDR=0;

#define TC3162_GPDATA1_REG	(TC3162_GPIO_BASE + 0x4 + 0x2)

/* End Sam */


/* Device data */
static struct net_device tc3162_emac_dev;

/* Descriptors and buffers: notice the alignments */
static macRxDescr_t rxdsc[NUM_RDESC] __attribute__ ((__aligned__(16)));
static macTxDescr_t txdsc[NUM_TDESC] __attribute__ ((__aligned__(16)));

#define SKB_MEM_COPY 1
#if SKB_MEM_COPY //jj
static unsigned char rxbuf[NUM_RDESC * R_BUF_SIZE] __attribute__ ((__aligned__(4)));
#else
static struct sk_buff *rxskb[NUM_RDESC];
#endif
static struct sk_buff *txskb[NUM_TDESC];

#undef ISR_TASKLET
#ifdef ISR_TASKLET
struct tasklet_struct   tasklet_p;
#endif

// if we use the sk_buff to be transmit buffer, we dont need txbuf 
//static unsigned char txbuf[NUM_TDESC * T_BUF_SIZE] __attribute__ ((__aligned__(4)));

/* Reference count for device */
static int dev_count = 0;

/* For storing MAC address */
static unsigned long def_mac_hi = 0L;
static unsigned long def_mac_lo = 0L;

/* Timer */
static struct timer_list eth_timer;

/* Link status flag */
static volatile int link_up = 0;
static volatile int link_configured = 0;
static unsigned long max_link_loop = 0;
static unsigned long ld_count = 0;
static int reset_flag = 0;

// Sam  ARP test data
/*unsigned char test_pattern[68]={0xff,0xff,0xff,0xff,0xff,0xff,
				0x00,0x00,0xaa,0xbb,0xcc,0xff,
				0x08,0x06,0x00,0x01,0x08,0x00,
				0x06,0x04,0x00,0x01,0x00,0x00,
				0xaa,0xbb,0xcc,0xff,0xc0,0xa8,
				0x15,0xa1,0x00,0x00,0x00,0x00,
				0x00,0x00,0xc0,0xa8,0xc0,0xa0,
				0x00,0x00,0x00,0x00,0x00,0x00,
				0x00,0x00,0x00,0x00,0x00,0x00,
				0x00,0x00,0x00,0x00,0x00,0x00,
				0x00,0x00,0x00,0x00,0x00,0x00,
				0x00,0x00};
*/
//

static buffer_ioctl[64];

/*  MX29LV160BTC sector architecture for Word Mode(x16)
 *  total = SA0 - SA34 = 35 sectors
 *  SA0  - SA30 32Kwords = 32Kwords x 31 = (F8xxx)
 *  SA31	16Kwords = 16Kwords x  1 = (F8xxx)
 *  SA32 - SA33  4Kwords = 4Kwords  x  2 = (FFxxx)
 *  SA34	 8Kwords = 8Kwords  x  1 = (FExxx)
 */
 
#define FLASH_BASE	0xbc000000
#define FLASH_SIZE	0x200000
#define ADDR_1346	0x555
#define ADDR_25		0x2AA
#define FALSE		-1
#define TRUE		0



/*
 * be care for it will clear all data in chip
 */
unsigned int flash_Erase(void)
{
	int i;
	short int buf[17];
	volatile unsigned int *flash = (unsigned int *) 0;

	buf[16]='\n';
	for(i=0;i<8;i++)
		buf[i] = flash[FLASH_BASE+i];
	
	//MSG_PRINT("%s",buf);
	MSG_PRINT("flash Erase start\n");

#if 0

	flash[FLASH_BASE+ADDR_1346] = 0xAA; /* 1 */
	flash[FLASH_BASE+ADDR_25] = 0x55; /* 2 */
	flash[FLASH_BASE+ADDR_1346] = 0x80; /* 3 */
	flash[FLASH_BASE+ADDR_1346] = 0xAA; /* 4 */
	flash[FLASH_BASE+ADDR_25] = 0x55; /* 5 */
	flash[FLASH_BASE+ADDR_1346] = 0x10; /* 6 */

	MSG_PRINT("flash 1\n");

	buf[16]='\n';
	for(i=0;i<16;i++)
		buf[i] = flash[FLASH_BASE+i];
	MSG_PRINT("%s",buf);

	while(1) {
		
		if( flash[FLASH_BASE] & 0x80 == 
		    flash[FLASH_BASE+FLASH_SIZE-1] & 0x80 )
			break;
		
		if( flash[FLASH_BASE+FLASH_SIZE-1] & 0x20 )
			if( flash[FLASH_BASE] & 0x80 == 
		    	    flash[FLASH_BASE+FLASH_SIZE-1] & 0x80 )
				break;
	}

	buf[16]='\n';
	for(i=0;i<16;i++)
		buf[i] = flash[FLASH_BASE+i];
	MSG_PRINT("%s",buf);
*/
#endif
	return TRUE;
}
/* Function prototypes */
static void tc3162_emac_isr(int irq, void *dev_id, struct pt_regs *regs);
static int tc3162_emac_reset_desc(struct net_device *dev, int *reset);
static int tc3162_emac_reginit(void);


/* Get the device mac address */
static int tc3162_get_macaddr(unsigned char *addr)
{
  /* TODO: Set up mac address, later on we need to read it from ROM */
  *addr++ = (def_mac_hi & 0x0000ff00) >> 8;
  *addr++ = (def_mac_hi & 0x000000ff);
  *addr++ = (def_mac_lo & 0xff000000) >> 24;
  *addr++ = (def_mac_lo & 0x00ff0000) >> 16;
  *addr++ = (def_mac_lo & 0x0000ff00) >> 8;
  *addr = (def_mac_lo & 0x000000ff);

  return(0);
}

/* Setting up MAC address of ethernet PHY and device data */
static int tc3162_set_mac(struct net_device *dev)
{
  unsigned long hi_mac, low_mac;

  /* Set up device mac address */
  if (tc3162_get_macaddr(&(dev->dev_addr[0])))
    return(-EIO);

  if (!is_valid_ether_addr(&(dev->dev_addr[0])))
    MSG_PRINT("%s: bogus mac address detected.\n", dev->name);

  hi_mac = (dev->dev_addr[0] << 8) | dev->dev_addr[1];
  low_mac = (dev->dev_addr[2] << 24) | (dev->dev_addr[3] << 16) | (dev->dev_addr[4] << 8) | dev->dev_addr[5];

  /* Set up MAC address */
  *(volatile unsigned long *)TC3162_EMAC_MAC_MADR_REG = hi_mac;
  *(volatile unsigned long *)TC3162_EMAC_MAC_LADR_REG = low_mac;

  return(0);
}

/* Setting customized mac address */
static int tc3162_emac_set_macaddr(struct net_device *dev, void *addr)
{
  unsigned long hi_mac, low_mac;

  /* Check if given address is valid ethernet MAC address */
  if (!is_valid_ether_addr(addr))
    return(-EIO);

  /* Turn off IRQ and stop receive/transmit */
  *(volatile unsigned long *)TC3162_EMAC_MACCR_REG &= ~(MACCR_RCV_EN | MACCR_XMT_EN);
  *(volatile unsigned long *)TC3162_EMAC_IMR_REG = 0;

  netif_carrier_off(dev); /* Shutdown interface */

  /* Save the customize mac address */
  memcpy(&(dev->dev_addr[0]), addr+2, 6);

  hi_mac = (dev->dev_addr[0] << 8) | dev->dev_addr[1];
  low_mac = (dev->dev_addr[2] << 24) | (dev->dev_addr[3] << 16) | (dev->dev_addr[4] << 8) | dev->dev_addr[5];
  def_mac_hi = hi_mac;
  def_mac_lo = low_mac;

  /* Set up MAC address */
  *(volatile unsigned long *)TC3162_EMAC_MAC_MADR_REG = hi_mac;
  *(volatile unsigned long *)TC3162_EMAC_MAC_LADR_REG = low_mac;

  /* Restart everything */
  link_up = link_configured = 0;

  return(0);
}

/* According sample code from tc3162 */
/* Writing to transceiver register */
static void tc3162_emac_mii_write(unsigned long paddr, unsigned long phyreg, unsigned long data)
{
  volatile unsigned long word;
  int cnt=10000;

  *(volatile unsigned long*)(TC3162_EMAC_PHYWDATA_REG) = (data & 0x0000ffff);
  word = 0;
  word |= ((paddr & 0x1f) << PHY_ADDR_SHIFT) | ((phyreg & 0x1f) << PHY_REG_ADDR_SHIFT);
  word &= ~(MIIRD_BIT);
  word |= (MIIWR_BIT);
  *(volatile unsigned long*)(TC3162_EMAC_PHYCR_REG) = word;
    
  /* wait until phy write ok */
  do
  {
	word = *(volatile unsigned int*)(TC3162_EMAC_PHYCR_REG);
	cnt--;
  }
  while( (word & MIIWR_BIT) && cnt>0);

  if(cnt == 0)
  {
	printk("mii write time out\n\r");
  }

}

/* According to sample code from tc3162 */ 
/* Reading from transceiver register */
static unsigned long tc3162_emac_mii_read(unsigned long paddr, unsigned long phyreg)
{
  volatile unsigned long word;
  int cnt;

  cnt=10000;
  word =0;
  word |= ((paddr & 0x1f) << PHY_ADDR_SHIFT) | ((phyreg & 0x1f) << PHY_REG_ADDR_SHIFT);
  word |= MIIRD_BIT;
  word &= ~(MIIWR_BIT);
  *(volatile unsigned long*)(TC3162_EMAC_PHYCR_REG) = word;
 
  /* wait until read ok */
  do
  {
	word = *(volatile unsigned long*)(TC3162_EMAC_PHYCR_REG);
	cnt--;
  }
  while( (word & MIIRD_BIT) && (cnt>0));

  if(cnt == 0)
  {
 	printk("mii read time out\n\r");
  }
  return (*(volatile unsigned int*)(TC3162_EMAC_PHYCR_REG)&0x0000ffff);
}

/* Initialize the ethernet link status, mac, and flow control */
static int tc3162_PHY_Setup(struct net_device *dev, int timeout)
{
  const unsigned long link_loop = timeout; /* Max. wait for link to established */
  int value32;
  /* for phyaddr searching */
  int miiReg=0;
  int enetAddr;
  
//  *(volatile unsigned long *)TC3162_GPCTL_REG |= 0x00044041; 
//  *(volatile unsigned long *)TC3162_GPOPENDRAIN_REG |= 0x00000289;  	
  //printk("tc3162_PHY_Setup \n");
  /* If link is up and configured, don't do anything */
  if ((link_up != 0) && (link_configured != 0))
    return(0);
  else
    DBG_PRINT("%s: re-configure link (%ld).\n", dev->name, link_loop);

  //setting full duplex and 100M
  //search phy address by scanning MII station registers
//  for(enetAddr = 0; enetAddr < 32; enetAddr++)
////  {
//	miiReg = tc3162_emac_mii_read( enetAddr, PHY_ID_REG1 );
	//printk("Sam 330 %02d PHY_ID_REG1=%04X\n",enetAddr,miiReg);
//	if(miiReg == 0)
//	{
//	  miiReg = tc3162_emac_mii_read( enetAddr, PHY_ID_REG2 );
	  //printk("Sam 335 %02d PHY_ID_REG2=%04X\n",enetAddr,miiReg);
//	}
	/* realtek 8201 or ic-plus IP101 */
//	if (miiReg == 0x8201 || miiReg == 0x0243)
//	{
//		PHY_ADDR = enetAddr;
//		break;
//	}
 // }
/* using GPIO9(h/w 1.00a), GPIO0(h/w 1.00b) to reset LAN switch IP175C*/
  /*
#ifdef HW_100B
   
   
   *(volatile unsigned long *)TC3162_GPDATA_REG &= 0xfffffffe; 
   mdelay(50);
   *(volatile unsigned long *)TC3162_GPDATA_REG |= 0x00000001; 
#else
  
  *(volatile unsigned long *)TC3162_GPDATA_REG &= 0xfffffdff; 
   mdelay(50);
  *(volatile unsigned long *)TC3162_GPDATA_REG |= 0x00000200;
  
#endif	
*/
/*
  value32 = 0;
  value32 |= PHY_RESET;
  tc3162_emac_mii_write(PHY_ADDR, PHY_CONTROL_REG, value32);
*/
  //disable loopback mode
/*
  value32 = 0;
  value32 = tc3162_emac_mii_read( PHY_ADDR, PHY_CONFIG_REG );
  value32 |= PHY_TP_LOOPBACK;
  tc3162_emac_mii_write( PHY_ADDR, PHY_CONFIG_REG, value32 );
 */
  //configure phy advertisement register
/*
  value32 = 0;
  value32 |= PHY_SELECTOR |
		PHY_100_BASE_TX_FD |
		PHY_100_BASE_TX_HD |
		PHY_10_BASE_T_FD |
		PHY_10_BASE_T_HD |
		PHY_RX_FC |
		PHY_TX_FC;

  tc3162_emac_mii_write(PHY_ADDR, PHY_ANAR, value32);
*/
  //configure PHY control registers
/*
  value32 = 0;
  value32 |= PHY_RESTART_AN |
  	     PHY_AN_ENABLE  |
	     PHY_SPEED_SELECT_100;
  value32 &= ~( PHY_LOOPBACK );
  tc3162_emac_mii_write(PHY_ADDR, PHY_CONTROL_REG, value32);
*/
     

  /* Turn off IRQ and stop receive/transmit */
  *(volatile unsigned long *)TC3162_EMAC_MACCR_REG &= ~(MACCR_RCV_EN | MACCR_XMT_EN);
  *(volatile unsigned long *)TC3162_EMAC_IMR_REG = 0;
// Sam add 
  link_configured = 1;
  link_up=1;
// 
  return(0);
}

/* Ethernet hardware initialization */
static int tc3162_emac_hw_init(struct net_device *dev)
{
   int value32;

  /* Disable Mac Controller */
   value32 = *(volatile unsigned long*)TC3162_EMAC_MACCR_REG;
   value32 &= ~(MACCR_RDMA_EN); 
   value32 &= ~(MACCR_XDMA_EN);
   *(volatile unsigned long*)TC3162_EMAC_MACCR_REG = value32; 
  /* Software Reset MAC and PHY */
  /* Clear Regs all */
  /* Reset Software Mac */
  /* Reset Software PHY */

  /* Setup MAC address */
  if (tc3162_set_mac(dev))
    return(-EIO);

  /* Set up multicast hash table to MCHTHR MCHTLR */
  /* Set multicast hash table to accept all */

  /* Resetting descriptors */
  if (tc3162_emac_reset_desc(&tc3162_emac_dev, &reset_flag))
    return(-EIO);

  /* Configure IFR with PHY and speed, as well as PHY */
  tc3162_PHY_Setup(dev, max_link_loop);
   //SwitchInit_IP175C();
  /* Sam add Config IMR */
  tc3162_emac_reginit(); 

  return(0);
}

static int flag=0;
/* Monitor the status of link, re-do link initialization if necessary. */
static void tc3162_emac_link_monitor(unsigned long dummy)
{
#if 0
  unsigned long link_status = 0;
  struct net_device *dev = (struct net_device *)dummy;
  int res;
  /* Setup the link/configuration is not done so. */
  if ((link_up == 0) || (link_configured == 0))
    res = tc3162_emac_hw_init(dev);

  /* Get link status and adjust the delay loop accordingly */
  link_status = ((tc3162_emac_mii_read(PHY_ADDR, PHY_STATUS_REG) & PHY_LINK_STATUS) ? 1 : 0);
  link_status = 1;
  if (link_status) 
  {
    if ((link_configured != 0) && (link_up != 0)) 
	{
       	 	if (!netif_carrier_ok(dev))
		 {
             		 netif_carrier_on(dev); /* Up and running, turn netif on */
       			 max_link_loop = DEF_LINK_LOOP;
       			 ld_count >>= 2;
       			 if (ld_count == 0)
	 			 ld_count = 1;
      		 }
    	}
    else 
	{
      		ld_count <<= 1;
      		max_link_loop = ld_count * DEF_LINK_LOOP;
      		if (max_link_loop > DEF_MAX_LINK_LOOP)
			max_link_loop = DEF_MAX_LINK_LOOP;
      		netif_carrier_off(dev);
      		link_configured = 0;
    	}
  } 
  else 
  {
    MSG_PRINT("%s: detected link down.\n", dev->name);
    ld_count <<= 1;
    max_link_loop = ld_count * DEF_LINK_LOOP;
    if (max_link_loop > DEF_MAX_LINK_LOOP)
      max_link_loop = DEF_MAX_LINK_LOOP;
    netif_carrier_off(dev);
    link_up = link_configured = 0;
  }

  /* Schedule for the next time */
  mod_timer(&eth_timer, jiffies + (HZ / 2)); /* Check roughly every 0.5 seconds */
#endif
  unsigned long link_status = 0;
  unsigned int phystatus=0;
  struct net_device *dev = (struct net_device *)dummy;
  int res;
  /* Setup the link/configuration is not done so. */
  if ((link_up == 0) &&  (link_configured == 0))
    {
      printk("Do emac_hw_init\n");
      res = tc3162_emac_hw_init(dev);
 
    }
  /* Get link status and adjust the delay loop accordingly */
#ifdef SINGLE_PHY
  phystatus=tc3162_emac_mii_read(PHY_ADDR, PHY_STATUS_REG);
  link_up = link_status = ((phystatus & PHY_LINK_STATUS) ? 1 : 0);
  if(!flag && link_status){printk("AN ON link up\n");printk("%02X\n",phystatus);flag=1;};
#else
  link_status=1;
#endif
  if (link_status)
  {
    if ((link_configured != 0) && (link_up != 0))
        {
                if (!netif_carrier_ok(dev))
                 {
                         netif_carrier_on(dev); /* Up and running, turn netif on */
                         max_link_loop = DEF_LINK_LOOP;
                         ld_count >>= 2;
                         if (ld_count == 0)
                                 ld_count = 1;
                 }
        }
    else
        {
                printk("link not configured\n");
                ld_count <<= 1;
                max_link_loop = ld_count * DEF_LINK_LOOP;
                if (max_link_loop > DEF_MAX_LINK_LOOP)
                        max_link_loop = DEF_MAX_LINK_LOOP;
                netif_carrier_off(dev);
                link_configured = 0;
        }
  }
  else
  {
    if(flag){MSG_PRINT("%s: detected link down.\n", dev->name);printk("%02X\n", phystatus);}
    flag=0;
    ld_count <<= 1;
    max_link_loop = ld_count * DEF_LINK_LOOP;
    if (max_link_loop > DEF_MAX_LINK_LOOP)
      max_link_loop = DEF_MAX_LINK_LOOP;
    netif_carrier_off(dev);
  }
  /* Schedule for the next time */
  mod_timer(&eth_timer, jiffies + (HZ / 2)); /* Check roughly every 0.5 seconds */
}

/* Setting rx/tx descriptors */
static void tc3162_emac_descinit(char first)
{
  register int i;
  macTxDescr_t *pTxDesc = NULL;
  macRxDescr_t *pRxDesc = NULL;

  printk("Initialize Descriptors\n\r");

  pTxDesc = (macTxDescr_t*)(KSeg0_to_KSeg1(txdsc));

/* Setup TX descriptor initialization */

  for(i=0;i<NUM_TDESC;i++,pTxDesc++)
  {
	//Init descriptor
	//printk("Sam 547 pTxDesc=%08x\n",pTxDesc);
	pTxDesc->tdes0 = 0;
	pTxDesc->tdes1 = 0;
	pTxDesc->tdes2 = 0;
	// Sam: if we use the sk_buff to be transmit buffer, we dont need initial extra txbuf here 
	//assign owner bit
	pTxDesc->tdes0 &= ~(TXDMA_OWN_BIT);
	pTxDesc->tdes1 |= FTS_BIT;
	pTxDesc->tdes1 |= LTS_BIT;
	if (!first && txskb[i])
		dev_kfree_skb(txskb[i]);
 	txskb[i] = NULL;
 }
  // set the last tx descriptor
  pTxDesc--;
  pTxDesc->tdes1 |= EDOTR_BIT;


  //Descriptor Byte Swap (Big Endian)
  pTxDesc = (macTxDescr_t*)(KSeg0_to_KSeg1(txdsc));
  for(i=0;i<NUM_TDESC;i++,pTxDesc++)
  {
     pTxDesc->tdes0 = byte_swap(pTxDesc->tdes0);
     pTxDesc->tdes1 = byte_swap(pTxDesc->tdes1);
     pTxDesc->tdes2 = byte_swap(pTxDesc->tdes2);
  }


/* Setup xX descriptior initialization */

  pRxDesc = (macRxDescr_t*)(KSeg0_to_KSeg1(rxdsc));

  for(i=0;i <NUM_RDESC; i++,pRxDesc++)
  {
	//Init descriptor
	pRxDesc->rdes0 = 0;
	pRxDesc->rdes1 = 0;
	pRxDesc->rdes2 = 0;
	pRxDesc->rdes0 = RXDMA_OWN_BIT;
	pRxDesc->rdes1 = R_BUF_SIZE;
#if SKB_MEM_COPY
	pRxDesc->rdes2 = KSeg1_to_phy(&rxbuf[i*R_BUF_SIZE]); 
#else
	if (first)
        {
	   if ((rxskb[i] = dev_alloc_skb(R_BUF_SIZE+2+40)) == NULL)
	   {
		printk("alloc error ..\n");
		return;
	   }
 	}
	skb_reserve(rxskb[i], 40);
	pRxDesc->rdes2= KSeg1_to_phy(rxskb[i]->data);
#endif
  }
  pRxDesc--;
  pRxDesc->rdes1|=EDORR_BIT;

  pRxDesc = (macRxDescr_t*)(KSeg0_to_KSeg1(rxdsc));
 
  for(i=0;i<NUM_RDESC;i++,pRxDesc++)
  {
     pRxDesc->rdes0 = byte_swap(pRxDesc->rdes0);
     pRxDesc->rdes1 = byte_swap(pRxDesc->rdes1);
     pRxDesc->rdes2 = byte_swap(pRxDesc->rdes2);
  }

}  /* tc3162_emac_descinit */


/* Starting up the ethernet device */
static int tc3162_emac_open(struct net_device *dev)
{
  TC3162_EMAC_PRIV *private = (TC3162_EMAC_PRIV *)dev->priv;

  if ((dev == NULL) || (private == NULL))
    return(-EIO);

  MOD_INC_USE_COUNT;
  DBG_PRINT("%s: starting interface.\n", dev->name);

  /* Schedule timer for monitoring link status */
  init_timer(&eth_timer);
  eth_timer.function = tc3162_emac_link_monitor;
  eth_timer.data = (unsigned long)dev;
  mod_timer(&eth_timer, jiffies + (HZ * 2));

  netif_start_queue(dev);
  // Sam add 0806
  *(volatile unsigned int *)(TC3162_EMAC_MACCR_REG)= MACCR_SW_RST;
  intMaskSet( INTR_EMAC, ENABLE );
  // end 0806
  return(0);
}

/* Stopping the ethernet device */
static int tc3162_emac_close(struct net_device *dev)
{
  if (dev == NULL)
    return(-EIO);

  /* Turn off IRQ and stop receive/transmit */
  del_timer_sync(&eth_timer);

  /* Stop the transmit queue */
  netif_stop_queue(dev);
  link_up = link_configured = 0;

  MOD_DEC_USE_COUNT;
//Sam add 
*(volatile unsigned int*)(TC3162_EMAC_IMR_REG) = 0;
intMaskSet( INTR_EMAC, DISABLE );
reset_emac_reg_param();
// End 
  return(0);
}

/* Setup multicast list */
static void tc3162_emac_set_multicast_list(struct net_device *dev)
{
  return; /* Do nothing */
}

/* Transmit a packet */
static int tc3162_emac_tx(struct sk_buff *skb, struct net_device *dev)
{
  TC3162_EMAC_PRIV *private = NULL;
  struct net_device_stats *stats = NULL;
  macTxDescr_t *desc_ptr = NULL;
  unsigned long length = 0;

#if 0 //jj
  if ((skb == NULL) || (dev == NULL)) {
    return(-EIO);
  } else if ((link_up == 0) || (link_configured == 0)) {
    netif_carrier_off(dev);
    return(-EIO); /* Link is not up yet */
  } else {
#else
 {
#endif
    private = (TC3162_EMAC_PRIV *)dev->priv;
    stats = &(private->stats);
  }

  if ((length = ((ETH_ZLEN < skb->len) ? skb->len : ETH_ZLEN)) > T_BUF_SIZE)
    return(-ENOMEM); /* Too big */


  /* Send packet to device */
    desc_ptr = (macTxDescr_t *)KSEG1ADDR(&txdsc[private->next_txidx]);
    if (desc_ptr->tdes0 & (1<<7)){  //do byte-order change and check TXDMA_OWN_BIT 
    stats->tx_dropped++;
    ////ERR_PRINT("%s: tx error (descriptor not owned by CPU).\n", dev->name);
    dev_kfree_skb(skb);
    return(-EIO);
  } 

  /* Check if previous transmission has error */
#if 1 //jj
  /* Free up socket buffer */
  if (txskb[private->next_txidx])
    dev_kfree_skb(txskb[private->next_txidx]);
  txskb[private->next_txidx] = skb;
#endif


  /* Copy packet data to tx buffer */
	//desc_ptr->tdes2=byte_swap(KSeg1_to_phy(&test_pattern[0]));
	// Sam debug desc_ptr->tdes2=byte_swap(KSeg1_to_phy(skb->data));
	desc_ptr->tdes2=byte_swap(KSeg1_to_phy(KSeg0_to_KSeg1(skb->data)));
	//memcpy(&txbuf[(private->next_txidx)*T_BUF_SIZE ],skb->data,length);
//	printk("after  skb->data address=%08x\n",desc_ptr->tdes2);

        // tx buffer size for test_pattern
        /*desc_ptr->tdes1&=~0xFF070000;
        desc_ptr->tdes1|=(68&0x000000ff)<<24;
        desc_ptr->tdes1|=(68&0x00000700)<<8;*/

	// Sam: do byte swap for length and write into TX descriptor	
	desc_ptr->tdes1&=~0xFF070000;
        desc_ptr->tdes1|=(length&0x000000ff)<<24;
        desc_ptr->tdes1|=(length&0x00000700)<<8;
	// Sam: do byte swap and set DMA
	desc_ptr->tdes0|=(1<<7);

  dev->trans_start = jiffies;
  private->next_txidx = ((private->next_txidx + 1) % NUM_TDESC);

  wmb(); //jj; from tc3162_0516
  /* Start transmission (should be in suspend mode already) */
  *(volatile unsigned long *)TC3162_EMAC_TXPD_REG = 0x0;

  /* Update stats */
  //stats->tx_packets++;
  stats->tx_bytes += length;
  private->last_txbytes = length;

#if 0 //jj
  /* Free up socket buffer */
  dev_kfree_skb(skb);
#endif

  return(0);
}

/* Receiving packet(s) */
static int tc3162_emac_rx(struct net_device *dev)
{
  TC3162_EMAC_PRIV *private = NULL;
  struct sk_buff *skb = NULL;
  struct net_device_stats *stats = NULL;
  volatile macRxDescr_t *desc_ptr = NULL;
  unsigned long cnt = 0;
  unsigned long nidx = 0, tempidx = 0;
  unsigned long frameSize=0;
  int bradtest=0;
  if (dev == NULL)
    return(-EIO);
  else {
    private = (TC3162_EMAC_PRIV *)dev->priv;
    stats = &(private->stats);
  }

  /* The one we're starting to read from */
  nidx = private->last_rxidx;
  desc_ptr = (macRxDescr_t *)KSEG1ADDR(&rxdsc[nidx]);  

//printk("Sam 755 nidx = %d tempidx=%d\n",nidx,tempidx);

  /* byte swap first */
  while( (desc_ptr->rdes0 & (1<<7)) == 0)  //owned by DMA
  {
	tempidx = nidx;
#if 0 //jj
	if( ((desc_ptr->rdes0 & (1<<11)) == 0) ||
	    ((desc_ptr->rdes0 & (1<<10)) == 0) ||
	    ((desc_ptr->rdes0 & (1<<13)) == 0) ||
	    ((desc_ptr->rdes0 & (1<<12)) == 0) ||
	    ((desc_ptr->rdes0 & (1<<14)) == 0) )
#else
	if ((desc_ptr->rdes0 & (1<<11 | 1<<10 | 1<<14)) == 0 )	
#endif
	{
	   /* received good frames */
	   frameSize = ((desc_ptr->rdes0)&0xff000000)>>24;
	   frameSize |= ((desc_ptr->rdes0)&0x00070000)>>8;

	   if((frameSize < 60) || (frameSize > 1514))
	   {
		/* frame size is out of range */
		/* discard this packet and repost this mbuf */	
		// Sam marked		goto DISCARD;
	   }

	/* Go thru descriptors list until we ran out or encounterred the one
		being processed by DMA */
      	/* DMA is processing this one, break out the loop */
	/* We dropped any error packet */
	/* We don't handle multi-packet frame, for now */


#if SKB_MEM_COPY //jj	   
	    if ((skb = dev_alloc_skb(frameSize + 2)) == NULL) {
                DBG_PRINT("%s: rx dropped (memory unavailable)\n", dev->name);
		desc_ptr->rdes0 |= (1 << 7); // set RXDMA_OWN_BIT to Ethernet MAC
		goto DISCARD; //jjbreak; //return 0;  // Sam add return 0 to avoid access violation
      	    }
      	    //data = (unsigned char *)KSEG1ADDR(byte_swap(desc_ptr->rdes2));
      	    skb->dev = dev;
            skb_reserve(skb, 2);
             //printk("\nether packet\n");
             //for(bradtest=0; bradtest<30;bradtest++)
             //{
             // printk("%x ", rxbuf[(nidx*R_BUF_SIZE)+bradtest]);
             //} 
             //printk("\n");
	    // Sam test      memcpy(skb_put(skb, frameSize), &rxbuf[nidx*R_BUF_SIZE], frameSize); 
	    skb_put(skb, frameSize);
	    memcpy(skb->data, &rxbuf[nidx*R_BUF_SIZE], frameSize);
	    //memcpy(skb_put(skb, frameSize), KSEG1ADDR(&rxbuf[nidx*R_BUF_SIZE]), frameSize); 
	   
            desc_ptr->rdes0 |= (1 << 7); // set RXDMA_OWN_BIT to Ethernet MAC 
#else
	    skb = rxskb[nidx];
	    skb->dev = dev;
	{
	//jj    unsigned long save_flags;
	   //jj local_irq_save(save_flags);
	    rxskb[nidx] = dev_alloc_skb(R_BUF_SIZE+2+40);
            if (rxskb[nidx] == NULL) {
                DBG_PRINT("%s: rx dropped (memory unavailable)\n", dev->name);
		rxskb[nidx] = skb;
		desc_ptr->rdes0 |= (1 << 7); // set RXDMA_OWN_BIT to Ethernet MAC
		//jjlocal_irq_restore(save_flags);
#ifdef ISR_TASKLET
	    /* check interrupt status */
            intMaskSet(INTR_EMAC, ENABLE);
            //Force emac to poll rx
            *(volatile unsigned int*)(TC3162_EMAC_RXPD_REG) = 0xffffffff;
#endif
                goto DISCARD; //jj; return 0;  // Sam add return 0 to avoid access violation
            }
	    skb_reserve(rxskb[nidx], 40);
	    //skb_reserve(skb, 2);
	    skb_put(skb, frameSize);

	    desc_ptr->rdes2= byte_swap(KSeg1_to_phy(rxskb[nidx]->data)); 
	    desc_ptr->rdes0 |= (1 << 7);  //set RXDMA_OWN_BIT to Ethernet MAC
//jj	    local_irq_restore(save_flags);
	}
#endif

	    skb->protocol = eth_type_trans(skb, dev);
            skb->ip_summed = CHECKSUM_NONE;

  	    /* Update stats */
	    //stats->rx_packets++;
      	    stats->rx_bytes += frameSize;
            dev->last_rx = jiffies;

            /* Send the packet to kernel */
            netif_rx(skb);
	  
    	} //end of if
        else
	     desc_ptr->rdes0 |= (1 << 7); // set RXDMA_OWN_BIT to Ethernet MAC 
    
DISCARD:	
	nidx=(nidx+1)%NUM_RDESC;
	desc_ptr = (macRxDescr_t *)KSEG1ADDR(&rxdsc[nidx]);
  } // end of  while
private->last_rxidx = nidx;

#if 0 //jj
  if (cnt >= NUM_RDESC) 
    ERR_PRINT("%s: rx suspended (no receive buffer available).\n", dev->name);
#endif 

#ifdef ISR_TASKLET //jj; use the tasklet
  /* check interrupt status */
   intMaskSet(INTR_EMAC, ENABLE);
  //Force emac to poll rx
  *(volatile unsigned int*)(TC3162_EMAC_RXPD_REG) = 0xffffffff;
#endif

  return(0);
}

#if 0
/* Resetting rx descriptor: error recovery for "rx buf unavailable" */
static void tc3162_emac_reset_rx_buf(struct net_device *dev)
{
  register int i;
/* Sam  struct tc3160_desc *desc_ptr = NULL;*/
  macRxDescr_t *desc_ptr = NULL;

  ERR_PRINT("%s: resetting rx buf ..\n", dev->name);

  /*  Sam Reset rx desc 
  for (i = 0, desc_ptr = (struct tc3160_desc *)KSEG1ADDR(&rxdsc[0]); 
	i < NUM_RDESC; i++, desc_ptr++) 
    desc_ptr->desc0 = 0x80000000;  Owned by DMA */

  for (i = 0, desc_ptr = (macRxDescr_t *)KSEG1ADDR(&rxdsc[0]); 
	i < NUM_RDESC; i++, desc_ptr++) 
    desc_ptr->rdes0 = 0x80000000; /* Owned by DMA */

/* Sam   *(volatile unsigned long *)TC3160_RPDR_REG = 0x0;
  *(volatile unsigned long *)TC3160_CR_REG |= 0x00000002; */ /* restart process */

  *(volatile unsigned long *)TC3162_EMAC_RXPD_REG = 0x0;
  *(volatile unsigned long *)TC3162_EMAC_MACCR_REG |= MACCR_RCV_EN; /* set RCV_EN bit */
}
#endif

/* Get the stats information */
static struct net_device_stats *tc3162_emac_stats(struct net_device *dev)
{
  TC3162_EMAC_PRIV *private=NULL;
  struct net_device_stats *stats=NULL;
  if (dev != NULL) {
	// Sam add
    private = (TC3162_EMAC_PRIV *)dev->priv;
    stats = &private->stats;
    stats -> rx_packets = *(volatile unsigned long *)TC3162_EMAC_RP_CNTR_REG; 
    stats -> tx_packets = *(volatile unsigned long *)TC3162_EMAC_XP_CNTR_REG;
    stats -> tx_errors = (*(volatile unsigned long *)TC3162_EMAC_XM_PC_CNTR_REG) >> 16;
    stats -> multicast= *(volatile unsigned long *)TC3162_EMAC_MULCA_CNTR_REG;
    stats -> rx_length_errors = ((*(volatile unsigned long *)TC3162_EMAC_CRCER_FTL_CNTR_REG)&0x0000ffff);
    stats -> rx_crc_errors = (*(volatile unsigned long *)TC3162_EMAC_CRCER_FTL_CNTR_REG) >> 16;
    stats -> rx_fifo_errors =  (*(volatile unsigned long *)TC3162_EMAC_RLC_RCC_CNTR_REG) >> 16; 
    stats -> rx_missed_errors = (*(volatile unsigned long *)TC3162_EMAC_RLC_RCC_CNTR_REG) >> 16;
    
    return (stats);
  } else
    return(NULL);
}

/* Ethernet interrupt handler */
static void tc3162_emac_isr(int irq, void *dev_id, struct pt_regs *regs)
{
  volatile unsigned int status=0;
  struct net_device *dev = (struct net_device *)dev_id;

  if (dev_id != (void *)&tc3162_emac_dev) 
    return;

  /* Current descriptors index */

  /* Disable emac interrupt */
//jj   intMaskSet(INTR_EMAC,DISABLE); 
  /* read maccr */
  status = *(volatile unsigned int*)(TC3162_EMAC_ISR_REG);
  rmb(); //jj; 0719 from tc3162

#if 0 //jj
  /* check error status */
  if(status & AHB_ERR_BIT)
  {
//	LIGHTLED(~(LED1));
	printk("AHB Error\n\r");
	//skip?
  }
  if(status & PHYSTS_CHG_BIT)
  {
//	LIGHTLED(~(LED1));
	printk("PHYSTS CHG Error\n\r");
  }
  if(status & RPKT_LOST_BIT)
  {
//	LIGHTLED(~(LED2));
	printk("RPKT LOST\n\r");
  }
  if(status & RPKT_SAVE_BIT)
  {
//	LIGHTLED(~(LED3));
//	printk("RPKT SAVE\n\r");
  }
  if(status & XPKT_LOST_BIT)
  {
//	LIGHTLED(~(LED4));
	printk("XPKT_LOST\n\r");
  }
  if(status & XPKT_FINISH_BIT)
  {
//	LIGHTLED(~(LED5));
//	printk("XPKT FINISH\n\r");
  }
  if(status & NOTXBUF_BIT)
  {
//	LIGHTLED(~(LED6));
//	printk("NOTXBUF\n\r");
  }
  if(status & XPKT_SAVE_BIT)
  {
//	LIGHTLED(~(LED7));
//	printk("XPKT SAVE\n\r");
  }
  if(status & NORXBUF_BIT)
  {     
        volatile unsigned long send_pause;
//	LIGHTLED(~(LED8));
	send_pause = *(volatile unsigned long*)(TC3162_EMAC_FCR_REG); // enable flow control
	*(volatile unsigned long*)(TC3162_EMAC_FCR_REG) = send_pause|0x00000002;  // send pause frame
	printk("NORXBUF\n\r");
	//tc3162_emac_rx(dev);
  }
#endif //
  if(status & RPKT_FINISH_BIT)
  {
//	LIGHTLED(~(LED9));
/* packet received */
//	printk("RPKT FINISH\n\r");
#ifdef ISR_TASKLET 
        tasklet_p.data = (unsigned long) dev;
        tasklet_schedule(&tasklet_p);

	return;
#else
	tc3162_emac_rx(dev);	
#endif
  }
#if 1 //jj
  else {
                if (status & NORXBUF_BIT)
                {
                        volatile unsigned long send_pause;
                        send_pause = *(volatile unsigned long*)(TC3162_EMAC_FCR_REG); // enable flow control
                        *(volatile unsigned long*)(TC3162_EMAC_FCR_REG) = send_pause|0x00000002;  // send pause frame
                        printk("NORXBUF\n\r");
                }
#if 0 //jj
                else
                {
                        if(status & AHB_ERR_BIT)
                                printk("AHB Error\n\r");
                        else if(status & PHYSTS_CHG_BIT)
                                printk("PHYSTS CHG Error\n\r");
                        else if(status & RPKT_LOST_BIT)
                                printk("RPKT LOST\n\r");
                        else if(status & XPKT_LOST_BIT)
                                        printk("XPKT_LOST\n\r");
                }// end else
#endif //jj
  }
#endif
// Sam SKIP:  
  /* check interrupt status */
//jj   intMaskSet(INTR_EMAC, ENABLE);
  //Force emac to poll rx
  *(volatile unsigned int*)(TC3162_EMAC_RXPD_REG) = 0xffffffff;
  return;

}

void tc3162_emac_poll(void)
{
        tc3162_emac_isr(INTR_EMAC, &tc3162_emac_dev, 0);
}
EXPORT(tc3162_emac_poll);
                                                                                                                                  

/*
 * soft_reset: Reset ICPLUS175C
 */
static void soft_reset(void) 
{
   tc3162_emac_mii_write(30, 0,0x175C);
   mdelay(5);
   tc3162_emac_mii_read(30, 0);
}

// KenWang added 2005/12/27
#define FILL_TAG_VLAN_MASK(vid,a) TAG_VLAN_MASK[vid/2]=TAG_VLAN_MASK[vid/2]|(vid%2==1?a<<8:a)

/* Handling ioctl call */
static int tc3162_emac_ioctl(struct net_device *dev, struct ifreq *ifr, int cmd)

{
    
    
    //unsigned int port_number,vid;
    unsigned int reg=0; 
    unsigned int p0=0,p1=0,p2=0,p3=0,readback=0;
    unsigned int maskvalue=0,v1=0,v2=0,v3=0,v4=0;
    struct IP175C_vlan_ioctl_args *ioc = (struct IP175C_vlan_ioctl_args *) ifr->ifr_data;
    unsigned int b1=0, b2=0, b3=0, b4=0;
    /* KenWang added 2005/12/27 -- begin -- */
    unsigned int VID[5];              
    // this is map to TAG_VLAN_MASK registers of ICPLUS175C (30,1) - (30,8)
    unsigned short TAG_VLAN_MASK[8]; 
    char index = 0;
    /* KenWang added 2005/12/27 -- end -- */
    //printk("Enter 3162 ioctl\n");
  
    
    p0=0x01 & ioc->portmask;
    p1=0x02 & ioc->portmask;
    p2=0x04 & ioc->portmask;
    p3=0x08 & ioc->portmask;
    
    /* we can add sub-command in ifr_data if we need to in the future */
    switch (ioc->request_cmd)
    {
    case Set_VLAN_Port:
         printk("Add VLAN Port\n");
         readback=tc3162_emac_mii_read(29, 19);
               
         if(readback==0x9f9f)
         {
            tc3162_emac_mii_write(29, 19,0x8182);
            if(p0 != 0)
              {
                readback=ioc->portmask << 8;
                readback=readback | tc3162_emac_mii_read(29, 19);
                tc3162_emac_mii_write(29, 19,readback);
              } 
            if(p1 != 0)
              {
                readback=ioc->portmask;
                readback=readback | tc3162_emac_mii_read(29, 19);
                tc3162_emac_mii_write(29, 19,readback);
                } 
         }
          else
          { 
            if(p0 != 0)
              {
                readback=ioc->portmask << 8;
                readback=readback | tc3162_emac_mii_read(29, 19);
                tc3162_emac_mii_write(29, 19,readback);
              } 
            if(p1 != 0)
              {
                readback=ioc->portmask;
                readback=readback | tc3162_emac_mii_read(29, 19);
                tc3162_emac_mii_write(29, 19,readback);
              }               
           }

          readback=tc3162_emac_mii_read(29, 20);
         if(readback==0x9f9f)
         {
            tc3162_emac_mii_write(29, 20,0x8488);
            if(p2 != 0)
              {
                readback=ioc->portmask << 8;
                readback=readback | tc3162_emac_mii_read(29, 20);
                tc3162_emac_mii_write(29, 20,readback);
              } 
            if(p3 != 0)
              {
                readback=ioc->portmask;
                readback=readback | tc3162_emac_mii_read(29, 20);
                tc3162_emac_mii_write(29, 20,readback);
              } 
         }
          else
          { 
            if(p2 != 0)
              {
                readback=ioc->portmask << 8;
                readback=readback | tc3162_emac_mii_read(29, 20);
                tc3162_emac_mii_write(29, 20,readback);
              } 
            if(p3 != 0)
              {
                readback=ioc->portmask;
                readback=readback | tc3162_emac_mii_read(29, 20);
                tc3162_emac_mii_write(29, 20,readback);
              }               
           }

                printk("After setup:\n");
                readback=tc3162_emac_mii_read(29, 19);
                printk("readback 29.19==%x\n",readback);
                readback=tc3162_emac_mii_read(29, 20);
                printk("readback 29.20==%x\n",readback);
         break;
       
    case Enable_IGMP_Snoop:
        printk("Enable IGMP Snooping ether switch configuration\n");
        tc3162_emac_mii_write(29, 19,0x9f9f);
        tc3162_emac_mii_write(29, 20,0x9f9f);
        tc3162_emac_mii_write(29, 23,0x07c2 );
	tc3162_emac_mii_write(29, 24,0x0002);
	tc3162_emac_mii_write(29, 25,0x0003);
	tc3162_emac_mii_write(29, 26,0x0004 );
        tc3162_emac_mii_write(29, 27,0x0005 );
	tc3162_emac_mii_write(29, 28,0x0001 );
	tc3162_emac_mii_write(29, 30,0x0003 );
	tc3162_emac_mii_write(30, 2,0x2221 );
	tc3162_emac_mii_write(30, 1,0x3000 );
	tc3162_emac_mii_write(30, 3,0x2824 );
	tc3162_emac_mii_write(30, 9,0x0f89 );
	soft_reset();
        break;
    case VODX:// 
	    //printk("..Enable triplay..\n");
	printk("..VODX..\n");
	tc3162_emac_mii_write(29, 19,0x9f9f);
        tc3162_emac_mii_write(29, 20,0x9f9f);
	/* KenWang modified 2005/12/27 for add user specified VID -- begin -- */
	memset(TAG_VLAN_MASK, 0x2020, sizeof(TAG_VLAN_MASK));

	VID[0] = ioc->VID[0] ? ioc->VID[0] : 2;
	VID[1] = ioc->VID[1] ? ioc->VID[1] : 3;
	VID[2] = ioc->VID[2] ? ioc->VID[2] : 4;
	VID[3] = ioc->VID[3] ? ioc->VID[3] : 5;
	if(ioc->tagmask != 0) {
		//printk("tagmask: %04x, VID: %d %d %d %d\n", ioc->tagmask, VID[0], VID[1], VID[2], VID[3]);
		maskvalue = 0x0402 | ((ioc->tagmask&0xf)<<11) | (~(ioc->tagmask&0xf)&0xf<<6);
		//printk("maskvalue: %04x\n", maskvalue);
		tc3162_emac_mii_write(29, 23, maskvalue);
	}
	else {
		//printk("VID: %d %d %d %d\n", VID[0], VID[1], VID[2], VID[3]);
		tc3162_emac_mii_write(29, 23,0x07c2 );
	}
	/* KenWang modified 2005/12/27 for add user specified VID -- end -- */

	// b1b2b3b4
	b1 = 0xff & ioc->portmask;
	b2 = 0xff & (ioc->portmask >> 8);
	b3 = 0xff & (ioc->portmask >> 16);
	b4 = 0xff & (ioc->portmask >> 24);
	printk("b1 %02X %02X %02X %02X\n",b1,b2,b3,b4);

	if(b1){
		p0=b1&0x1;
		p1=b1&0x2;
		p2=b1&0x4;
		p3=b1&0x8;
		if(p0) {
				printk("b1 p0\n");
				/* KenWang modified 2005/12/27 for add user specified VID */
				//tc3162_emac_mii_write(29, 24,0x0002);  // 0
				tc3162_emac_mii_write(29, 24, VID[0]);
				index = VID[0] & 0xf;
				FILL_TAG_VLAN_MASK(index,1);
		}
		if(p1) {
				printk("b1 p1\n");
				/* KenWang modified 2005/12/27 for add user specified VID */
				//tc3162_emac_mii_write(29, 25,0x0002);  // 1
				tc3162_emac_mii_write(29, 25, VID[0]);
				index = VID[0] & 0xf;
				FILL_TAG_VLAN_MASK(index,2);
		}
		if(p2) {
				printk("b1 p2\n");
				/* KenWang modified 2005/12/27 for add user specified VID */
				//tc3162_emac_mii_write(29, 26,0x0002 ); // 2
				tc3162_emac_mii_write(29, 26, VID[0]);
				index = VID[0] & 0xf;
				FILL_TAG_VLAN_MASK(index,4);
		}
		if(p3) {
				printk("b1 p3\n");
				/* KenWang modified 2005/12/27 for add user specified VID */
				//tc3162_emac_mii_write(29, 27,0x0002 ); // 3
				tc3162_emac_mii_write(29, 27, VID[0]);
				index = VID[0] & 0xf;
				FILL_TAG_VLAN_MASK(index,8);
		}
	}
	if(b2){
		p0=b2&0x1;
		p1=b2&0x2;
		p2=b2&0x4;
		p3=b2&0x8;
		if(p0) {
				printk("b2 p0\n");
				/* KenWang modified 2005/12/27 for add user specified VID */
				//tc3162_emac_mii_write(29, 24,0x0003);  // 0
				tc3162_emac_mii_write(29, 24, VID[1]);
				index = VID[1] & 0xf;
				FILL_TAG_VLAN_MASK(index,1);
		}
		if(p1) {
				printk("b2 p1\n");
				/* KenWang modified 2005/12/27 for add user specified VID */
				//tc3162_emac_mii_write(29, 25,0x0003);  // 1
				tc3162_emac_mii_write(29, 25, VID[1]);
				index = VID[1] & 0xf;
				FILL_TAG_VLAN_MASK(index,2);
		}
		if(p2) {
				printk("b2 p2\n");
				/* KenWang modified 2005/12/27 for add user specified VID */
				//tc3162_emac_mii_write(29, 26,0x0003 ); // 2
				tc3162_emac_mii_write(29, 26, VID[1]);
				index = VID[1] & 0xf;
				FILL_TAG_VLAN_MASK(index,4);
		}
		if(p3) {
				printk("b2 p3\n");
				/* KenWang modified 2005/12/27 for add user specified VID */
				//tc3162_emac_mii_write(29, 27,0x0003 ); // 3
				tc3162_emac_mii_write(29, 27, VID[1]);
				index = VID[1] & 0xf;
				FILL_TAG_VLAN_MASK(index,8);
		}
	}
	if(b3){
		p0=b3&0x1;
		p1=b3&0x2;
		p2=b3&0x4;
		p3=b3&0x8;
		if(p0) {
				printk("b3 p0\n");
				/* KenWang modified 2005/12/27 for add user specified VID */
				//tc3162_emac_mii_write(29, 24,0x0004);  // 0
				tc3162_emac_mii_write(29, 24, VID[2]);
				index = VID[2] & 0xf;
				FILL_TAG_VLAN_MASK(index,1);
		}
		if(p1) {
				printk("b3 p1\n");
				/* KenWang modified 2005/12/27 for add user specified VID */
				//tc3162_emac_mii_write(29, 25,0x0004);  // 1
				tc3162_emac_mii_write(29, 25, VID[2]);
				index = VID[2] & 0xf;
				FILL_TAG_VLAN_MASK(index,2);
		}
		if(p2) {
				printk("b3 p2\n");
				/* KenWang modified 2005/12/27 for add user specified VID */
				//tc3162_emac_mii_write(29, 26,0x0004 ); // 2
				tc3162_emac_mii_write(29, 26, VID[2]);
				index = VID[2] & 0xf;
				FILL_TAG_VLAN_MASK(index,4);
		}
		if(p3) {
				printk("b3 p3\n");
				/* KenWang modified 2005/12/27 for add user specified VID */
				//tc3162_emac_mii_write(29, 27,0x0004 ); // 3
				tc3162_emac_mii_write(29, 27, VID[2]);
				index = VID[2] & 0xf;
				FILL_TAG_VLAN_MASK(index,8);
		}
	}
	if(b4){
		p0=b4&0x1;
		p1=b4&0x2;
		p2=b4&0x4;
		p3=b4&0x8;
		if(p0) {
				printk("b4 p0\n");
				/* KenWang modified 2005/12/27 for add user specified VID */
				//tc3162_emac_mii_write(29, 24,0x0005);  // 0
				tc3162_emac_mii_write(29, 24, VID[3]);
				index = VID[3] & 0xf;
				FILL_TAG_VLAN_MASK(index,1);
		}
		if(p1) {
				printk("b4 p1\n");
				/* KenWang modified 2005/12/27 for add user specified VID */
				//tc3162_emac_mii_write(29, 25,0x0005);  // 1
				tc3162_emac_mii_write(29, 25, VID[3]);
				index = VID[3] & 0xf;
				FILL_TAG_VLAN_MASK(index,2);
		}
		if(p2) {
				printk("b4 p2\n");
				/* KenWang modified 2005/12/27 for add user specified VID */
				//tc3162_emac_mii_write(29, 26,0x0005 ); // 2
				tc3162_emac_mii_write(29, 26, VID[3]);
				index = VID[3] & 0xf;
				FILL_TAG_VLAN_MASK(index,4);
		}
		if(p3) {
				printk("b4 p3\n");
				/* KenWang modified 2005/12/27 for add user specified VID */
				//tc3162_emac_mii_write(29, 27,0x0005 ); // 3
				tc3162_emac_mii_write(29, 27, VID[3]);
				index = VID[3] & 0xf;
				FILL_TAG_VLAN_MASK(index,8);
		}
	}
	tc3162_emac_mii_write(29, 28,0x0001 ); // 4; wan no use
	tc3162_emac_mii_write(29, 30,0x0002 ); // 5; mii
	tc3162_emac_mii_write(30, 1,0x3000 );

	/* KenWang added 2005/12/27 add vlan tag -- begin -- */
	for(index = 0; index < 8; index++) {
		//printk("TAG_VLAN_MASK%d: %x\n", index+1, TAG_VLAN_MASK[index]);
		tc3162_emac_mii_write(30, index+1, TAG_VLAN_MASK[index]);
	}
	/* KenWang added 2005/12/27 add vlan tag -- end -- */
#if 0
	v1=b1&0xf; v2=b2&0xf; v3=b3&0xf; v4=b4&0xf;

	maskvalue=0x2020|(v2<<8)|v1;
	printk("mask1 %04X\n", maskvalue);
	tc3162_emac_mii_write(30, 2, maskvalue);

	maskvalue=0x2020|(v4<<8)|v3;
	printk("mask2 %04X\n", maskvalue);
	tc3162_emac_mii_write(30, 3, maskvalue);
#endif
	//tc3162_emac_mii_write(30, 9,0x0f89 );
	tc3162_emac_mii_write(30, 9,0x1089 );//Ken Wang modified 12/12/05 for VOD case that port 1,2 transfer data , port 3 can ping.
	soft_reset();

	break; /* VODX */
    case Disable_VLAN:
        printk("Disable VLAN \n");
        tc3162_emac_mii_write(29, 19,0x9f9f);
        tc3162_emac_mii_write(29, 20,0x9f9f);
        tc3162_emac_mii_write(29, 23,0x0000);
	tc3162_emac_mii_write(30, 9,0x0000);
	tc3162_emac_mii_write(29, 30,0x0000);
	tc3162_emac_mii_write(30, 1,0x3f3f); 
        tc3162_emac_mii_write(30, 2,0x3f3f); 
    //    soft_reset();
        break;
    case Get_Lan_Mii:
        printk("Read LAN MII \n");
        reg= tc3162_emac_mii_read(ioc->phyaddr, ioc->regid);
        if (copy_to_user(&(ioc->regvalue), &reg, sizeof(unsigned int)))
        printk("Error copy to user\n");
	break;
    case Set_Lan_Mii:
        printk("Set LAN MII \n");
        tc3162_emac_mii_write(ioc->phyaddr, ioc->regid,ioc->regvalue);
        break;
    case Switch_Reset:
        printk("Switch soft-Reset\n");
        soft_reset();
        break;
    default:
	   return -ENOIOCTLCMD;
        break;   
    }
 
  return 0;
}

/* Kernel level ethernet initialization */
static int tc3162_emac_start(struct net_device *dev)
{
  if (dev == NULL)
    goto failed;

  /* Ethernet device initialization */
  ether_setup(dev);

  if (tc3162_set_mac(dev))
    goto failed;

  /* Hook up with handlers */
  dev->get_stats = tc3162_emac_stats;
  dev->hard_start_xmit = tc3162_emac_tx;
  dev->open = tc3162_emac_open;
  dev->stop = tc3162_emac_close;
  dev->set_multicast_list = tc3162_emac_set_multicast_list;
  dev->do_ioctl = tc3162_emac_ioctl;
  dev->set_mac_address = tc3162_emac_set_macaddr;

  dev->tx_queue_len = NUM_TDESC; 
  dev->flags &= ~IFF_MULTICAST;
  dev->flags |= IFF_DEBUG;

//added by jj
  tc3162_emac_descinit(1);
//jj

  return(0);

failed:
  return(-EIO);
}

static int tc3162_emac_reset_desc(struct net_device *dev, int *reset)
{
  TC3162_EMAC_PRIV *private;

  if (dev == NULL)
    return(-EIO);
  else
    private = (TC3162_EMAC_PRIV *)dev->priv;

  if (*reset)
    memset(dev->priv, 0, sizeof(TC3162_EMAC_PRIV));
  *reset = 0;

  /* Clear all tx/rx buffer memory */
#if  SKB_MEM_COPY //jj
  memset((void *)KSEG1ADDR(&rxbuf[0]), 0, NUM_RDESC * R_BUF_SIZE);
#endif
  // if we use the sk_buff to be transmit buffer, we dont need txbuf 
  //memset((void *)KSEG1ADDR(&txbuf[0]), 0, NUM_TDESC * T_BUF_SIZE);

  /* Initialize the indices */
  private->last_rxidx = 0; 
  private->next_txidx = 0;

  /* Intialize the descriptors */
  tc3162_emac_descinit(0);
  return(0);
}


static int tc3162_emac_reginit()
{
   int value32;

   value32=0;
   /* Set IMR Mask */
   value32 = RPKT_FINISH_BIT | 
//jj	     NORXBUF_BIT | 
//	     NOTXBUF_BIT | 
//	     XPKT_FINISH_BIT |
//	     XPKT_LOST_BIT |
//	     RPKT_LOST_BIT |
//	     XPKT_SAVE_BIT |
//	     RPKT_SAVE_BIT |
	     AHB_ERR_BIT |
	     PHYSTS_CHG_BIT;
   *(volatile unsigned int *)(TC3162_EMAC_IMR_REG) = value32;

   /* Set Hash Table */
   *(volatile unsigned int*)(TC3162_EMAC_MAHT0_REG) = 0;
   *(volatile unsigned int*)(TC3162_EMAC_MAHT1_REG) = 0;

   /* Set Interrupt Timer */
   *(volatile unsigned int*)(TC3162_EMAC_ITC_REG) =0;

   /* Set Auto Polling Timer */

   /* Set DMA Burst Arbitration */
   *(volatile unsigned int*)(TC3162_EMAC_DBLAC_REG) =0;

   /* Set DMA Descriptor Base */
   *(volatile unsigned int*)(TC3162_EMAC_TXR_BADR_REG) = KSeg1_to_phy(txdsc);
   *(volatile unsigned int*)(TC3162_EMAC_RXR_BADR_REG) = KSeg1_to_phy(rxdsc);

   /* Set MACCR register */
   /* MACCR_XDMA_EN | MACCR_RDMA_EN | MACCR_XMT_EN | MACCR_RCV_EN | MACCR_HT_MUL_EN | MACCR_RX_RUNT |
      MACCR_RX_FTL  | MACCR_RCV_ALL | MACCR_CRC_APD| MACCR_FULLDUP| MACCR_MULPKT    | MACCR_RX_BRDPKT */
   *(volatile unsigned int*)(TC3162_EMAC_MACCR_REG) = 0x3df23;

   /* Set reg init */
   //*(volatile unsigned long*)(TC3162_EMAC_FCR_REG) = 0x07a40000;
   *(volatile unsigned long*)(TC3162_EMAC_FCR_REG) = 0x07a4f103; // enable flow control
   return 0;

}


static long gpio_write(struct file *file, char *buf, size_t count, loff_t *fpos)
{
    int  len;
    char *p = buffer_ioctl;
#define BBUF_LEN 64
    memset(p, 0, BBUF_LEN);
    p += *fpos;
    len = 0;
    while ( count > 0 )  {
    	if ( *fpos < BBUF_LEN )  {
    	    int c = *buf++;
            p[len] = c>='a' && c<='z' ? c-'a'+'A' : c;
        }
    	(*fpos)++;
	    len++;
    	count--;
    }
printk("enter gpio write: cmd=%s\n", buffer_ioctl);

   	if( !strcmp(buffer_ioctl,"RESET") )
   	{
   		//do soft_reset
		printk("soft reset switch!\n");
		soft_reset();
		
   	}
    buffer_ioctl[0] = 0;
    return len;
}

#if 0
static int gpio_flush(struct file *file)
{
    return 0;
}

static int gpio_open(struct inode *inode, struct file *file)
{
	memset(buffer_ioctl,0,64);
    return 0;
}
#endif

static struct file_operations gpio_fops = {
//    open:	gpio_open,
 //   flush:	gpio_flush,
    write:	gpio_write,
};


/* Driver installation: this is where the thing starts */
static int __init tc3162_emac_init(void)
{
  struct net_device *dev = &tc3162_emac_dev;
  int result;
  unsigned long read_back=0;      	
  TC3162_EMAC_PRIV *private = NULL;
  int err = 0, i;

  printk("Ethernet driver for TC3162 version 1.1");
  
	//Robert add 0110
	//register dev for control switch reset
	
    result = register_chrdev(GPIO_MAJOR, "gpio", &gpio_fops);
    if ( result < 0 )   {
    	printk(KERN_ERR "gpio: can't register char device\n" );
		goto failed;
    }

  
  // Sam add 0806

  intMaskSet(INTR_EMAC, DISABLE); 
  reset_emac_reg_param();
  // do EMAC software reset 
  *(volatile unsigned int *)(TC3162_EMAC_MACCR_REG) = MACCR_SW_RST;
  // End 0806
  dev->priv = NULL;

  if (dev_count != 0) {
    err = -EIO;
    goto failed;
  }

  SET_MODULE_OWNER(&tc3162_emac_dev);

  /* Set up default mac address and others */
  /* Set up PHY link profile, 100M/duplex mode */ 
  def_mac_hi = DEF_MAC_HI;
  def_mac_lo = DEF_MAC_LO;
  max_link_loop = DEF_LINK_LOOP;
  ld_count = 1;
  reset_flag = 1;

  /* Get a device name: normally it'll be eth0 */
  if ((err = dev_alloc_name(&tc3162_emac_dev, "eth%d")) < 0)
    goto failed;

  /* Register ISR */
  if ((err = request_irq(INTR_EMAC, tc3162_emac_isr, 0, "ethernet", 
		  &tc3162_emac_dev)) != 0) {
    err = -EIO;
    goto failed;
  } else {
    printk(" on %s (IRQ: %d)", dev->name, INTR_EMAC);
  }

#ifdef ISR_TASKLET
  tasklet_init(&tasklet_p, tc3162_emac_rx, 0);
#endif

  if (tc3162_get_macaddr(&(dev->dev_addr[0]))) {
    err = -EIO;
    goto failed;
  }

//mac info should be loaded from flash directly
#if 0
  printk(", (mac %02x", dev->dev_addr[0]);
  for (i = 1; i < 6; i++)
    printk(":%02x", dev->dev_addr[i]);
  printk(")\n");
#endif


  /* Allocate memory for private data */
  private = (TC3162_EMAC_PRIV *)kmalloc(sizeof(TC3162_EMAC_PRIV), GFP_KERNEL);
  dev->priv = private;

  if (dev->priv == NULL) {
    err = -ENOMEM;
    goto failed;
  } 
  

  /* Point to driver initialization routine and register the device with kernel */
  dev->init = tc3162_emac_start;
  if (register_netdev(dev) != 0) {
    err = -EIO;
    goto failed;
  }
// Sam add
	register_reboot_notifier(&tc3162_emac_notifier); 
// End Sam

  printk("bil_%s: driver installation completed.\n", dev->name);
  dev_count++;
  return(0);

failed:
  if (dev->priv != NULL) {
    kfree(dev->priv);
    dev->priv = NULL;
  }
  printk("%s: driver installation failed.\n", dev->name);
  return(err);
}

static void __exit tc3162_emac_exit(void)
{

  if (dev_count != 0) {
    /* Turn off IRQ and stop receive/transmit */

    /* Unregister the device and ISR */
    free_irq(INTR_EMAC, &tc3162_emac_dev);
    unregister_netdev(&tc3162_emac_dev);

	/* Unregister the device */
   	unregister_chrdev(GPIO_MAJOR, "gpio");

    /* Set desc base address registers to 0 */

    /* Free up memory */
    if (tc3162_emac_dev.priv != NULL) {
      kfree(tc3162_emac_dev.priv);
      tc3162_emac_dev.priv = NULL;
    }
  }
// Sam 0806
 intMaskSet(INTR_EMAC, DISABLE); 
}

#if 0 //jj
/* Sam add byte_swap function */
unsigned int byte_swap(volatile unsigned int x)
{
        volatile unsigned int tmp=0;
        tmp|=(x&0x000000FF)<<24;
        tmp|=(x&0x0000FF00)<<8;
        tmp|=(x&0x00FF0000)>>8;
        tmp|=(x&0xFF000000)>>24;
        return tmp;
}
#endif


void intMaskSet( int int_no, int enable )
{
  if(enable == ENABLE)
     *(volatile unsigned int*)(TC3162_IMR_REG) |= (0x01 << int_no);
  else
     *(volatile unsigned int*)(TC3162_IMR_REG) &= ~((0x01) << int_no);
}

void reset_emac_reg_param(void)
{
   *(volatile unsigned int*)(TC3162_EMAC_IMR_REG)=0x0;
   *(volatile unsigned int*)(TC3162_EMAC_MAC_MADR_REG)=0x0;
   *(volatile unsigned int*)(TC3162_EMAC_MAC_LADR_REG)=0x0;
   *(volatile unsigned int*)(TC3162_EMAC_MAHT0_REG)=0x0;
   *(volatile unsigned int*)(TC3162_EMAC_MAHT1_REG)=0x0;
   *(volatile unsigned int*)(TC3162_EMAC_TXR_BADR_REG)=0x0;
   *(volatile unsigned int*)(TC3162_EMAC_RXR_BADR_REG)=0x0;
   *(volatile unsigned int*)(TC3162_EMAC_ITC_REG)=0x0;
   *(volatile unsigned int*)(TC3162_EMAC_APTC_REG)=0x0;
   *(volatile unsigned int*)(TC3162_EMAC_DBLAC_REG)=0x0;
   *(volatile unsigned int*)(TC3162_EMAC_MACCR_REG)=0x0;
   *(volatile unsigned int*)(TC3162_EMAC_PHYCR_REG)=0x0;
   *(volatile unsigned int*)(TC3162_EMAC_FCR_REG)=0x0;
   *(volatile unsigned int*)(TC3162_EMAC_BPR_REG)=0x0;
   *(volatile unsigned int*)(TC3162_EMAC_TM_REG)=0x0;
}
/* End of Sam add */

static int tc3162_emac_notify_sys(struct notifier_block *this, unsigned long code, void *unused)
{
        if(code==SYS_DOWN || code==SYS_HALT)
        {
	       printk("EMAC GET SYS_DOWN or SYS_HALT code %x \n",code);
	       reset_emac_reg_param();	
               intMaskSet(INTR_EMAC,DISABLE);
        }
        return NOTIFY_DONE;
}

/* Register startup/shutdown routines */
module_init(tc3162_emac_init);
module_exit(tc3162_emac_exit);

