#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/proc_fs.h>
#include <linux/sched.h>
#include <asm/uaccess.h>
#include <asm/tc3162/gpio.h>
//#include <asm/arch/platform.h>
//#include <asm/leds.h>

#define MODULE_VERSION "1.0"
#define MODULE_NAME "GPIOUSER"

#define DATA_LEN 8
#ifdef ACT
#define PRODUCT_ID_ADDR 0xbfc40044
#else
#define PRODUCT_ID_ADDR 0xbfc20044
#endif

static int resetGpioValue;
static int myguardValue;
static int productidValue;
extern int used_flash_type; // KenWang added 2006/12/05 for new FLASH
static struct LEDGpio {
	int gpio;
	int value;
};
static struct LEDGpio sysledGpioValue;
//static struct LEDGpio wlanledGpioValue;
static struct LEDGpio pppledGpioValue;
static struct LEDGpio adslledGpioValue;

static struct proc_dir_entry *proc_dir;
static struct proc_dir_entry *proc_reset; 
static struct proc_dir_entry *proc_sysled; 
//static struct proc_dir_entry *proc_wlanled; 
static struct proc_dir_entry *proc_pppled; 
static struct proc_dir_entry *proc_adslled; 
static struct proc_dir_entry *proc_myguard; 
static struct proc_dir_entry *proc_productid; 
static struct proc_dir_entry *proc_flashtype; // KenWang added 2006/12/05 for new FLASH
static struct proc_dir_entry *proc_bootv; 

static int resetGpio;
static int sysledGpio;
//static int wlanledGpio;
static int pppledGpio;
static int adslledGpio;

static int get_gpio_input(int gpio_pin)
{
	int j;
	j = *(volatile unsigned short*)TC3162_GPDATA_REG;
//	printk("j = 0x%08x, (1<<(gpio_pin/2)) = 0x%08x\n",j,(1<<(gpio_pin/2)));
	if ((j & (1<<(gpio_pin/2))))
	{
//		printk("detect gpio %d high, 0x%08x\n",gpio_pin/2,j);
		return 1;
	}
	else
		return 0;
}

static void set_gpio_output(int gpio_pin,int value)
{
	int j;
	j = *(volatile unsigned short*)TC3162_GPDATA_REG;
//	printk("value = %d\n",value);
	if (value)
	{
//		printk("j = 0x%08x, gpio_pin = %d, 1<<gpio_pin = 0x%08x\n",j,gpio_pin/2,(1<<(gpio_pin/2)));
		j |= (1<<(gpio_pin/2));
//		printk("j = 0x%08x, gpio_pin = %d, 1<<gpio_pin = 0x%08x\n",j,gpio_pin/2,(1<<(gpio_pin/2)));
		
	}
	else
	{
//		printk("j = 0x%08x, gpio_pin = %d, 1<<gpio_pin = 0x%08x\n",j,gpio_pin/2,(1<<(gpio_pin/2)));
		j &= ~(1<<(gpio_pin/2));
//		printk("j = 0x%08x, gpio_pin = %d, 1<<gpio_pin = 0x%08x\n",j,gpio_pin/2,(1<<(gpio_pin/2)));
	}
	*(volatile unsigned short*)TC3162_GPDATA_REG = j;
//	printk("TC3162_GPDATA_REG = 0x%08x\n",*(volatile unsigned short*)TC3162_GPDATA_REG);
}

static void set_gpio_mode(int gpio_pin, int gpio_mode)
{
	int j;
	j = *(volatile unsigned int*)(TC3162_GPCTL_REG);
	//printk("TC3162_GPCTL_REG addr = 0x%08x\n",(unsigned int)(TC3162_GPCTL_REG));
	//printk("1. TC3162_GPCTL_REG = 0x%08x\n",*(volatile unsigned int*)TC3162_GPCTL_REG);
	if(gpio_mode == GPIO_MODE_INPUT)
	{
		//printk("GPIO MODE INPUT:\n");
		j &= ~(1 << gpio_pin);
	}
	else
		j |= (1 << gpio_pin);
	*(volatile unsigned int*)TC3162_GPCTL_REG = j;
	//printk("2. TC3162_GPCTL_REG = 0x%08x\n",*(volatile unsigned int*)TC3162_GPCTL_REG);
}

static int productid_read_data(char *page, char **start,
                            off_t off, int count, 
                            int *eof, void *data)
{
	int len;
	
        MOD_INC_USE_COUNT;

#ifdef ACT
	if(*(volatile unsigned int*)(0xbfc40044) == 0xffffffff)
#else
	if(*(volatile unsigned int*)(0xbfc20044) == 0xffffffff)
#endif
		len = sprintf(page, "0\n");
	else
        len = sprintf(page, "%d\n", *(volatile unsigned int*)(PRODUCT_ID_ADDR));

        MOD_DEC_USE_COUNT;

        return len;
}

/* KenWang added 2006/12/05 for new FLASH -- begin -- */
static int flashtype_read_data(char *page, char **start,
                            off_t off, int count, 
                            int *eof, void *data)
{
	int len;
	
        MOD_INC_USE_COUNT;

        len = sprintf(page, "%d\n", used_flash_type);

        MOD_DEC_USE_COUNT;

        return len;
}
/* KenWang added 2006/12/05 for new FLASH -- end -- */

static int myguard_read_data(char *page, char **start,
                            off_t off, int count, 
                            int *eof, void *data)
{
	int len;
	
        MOD_INC_USE_COUNT;
        
#ifdef ACT
        len = sprintf(page, "%d\n", *(char*)(0xbfc40032));
#else
        len = sprintf(page, "%d\n", *(char*)(0xbfc20032));
#endif

        MOD_DEC_USE_COUNT;

        return len;
}

static int bootv_read_data(char *page, char **start,
                            off_t off, int count, 
                            int *eof, void *data)
{
	int len;
	
        MOD_INC_USE_COUNT;
        
        len = sprintf(page, "%s\n", (char*)(0xbfc1fff8)
                      );

        MOD_DEC_USE_COUNT;

        return len;
}


static int reset_read_data(char *page, char **start,
                            off_t off, int count, 
                            int *eof, void *data)
{
	int len,i,j;
	int *gpioValue = (int*)data;
	
	(*gpioValue)=get_gpio_input(resetGpio);
        MOD_INC_USE_COUNT;
        
        len = sprintf(page, "%d\n", (*gpioValue)
                      );

        MOD_DEC_USE_COUNT;

        return len;
}


static int proc_write_data(struct file *file,
                             const char *buffer,
                             unsigned long count, 
                             void *data)
{
        int len,i,j;
	struct LEDGpio *gpioValue = (struct LEDGpio *)data;
	char tmpbuf[DATA_LEN];

        MOD_INC_USE_COUNT;

        if(count > DATA_LEN)
                len = DATA_LEN;
        else
                len = count;

        if(copy_from_user(tmpbuf, buffer, len)) {
                MOD_DEC_USE_COUNT;
                return -EFAULT;
        }

        tmpbuf[len] = '\0';
	sscanf(tmpbuf,"%d",&(gpioValue->value));

        MOD_DEC_USE_COUNT;

	//printk("gpioValue = %d\n",gpioValue->value);
	switch(gpioValue->value)
	{
		case 1://led on
		set_gpio_output(gpioValue->gpio,0);
		break;
		case 2://flash
		for(i=0;i<5;i++) {
		set_gpio_output(gpioValue->gpio,1);
		for(j=0;j<1000000;j++);
		set_gpio_output(gpioValue->gpio,0);
		for(j=0;j<1000000;j++);
		}
		break;
		case 3://reverse gpio direction to INPUT
		set_gpio_mode(sysledGpio, GPIO_MODE_INPUT);
		for(i=0;i<5;i++) {
		set_gpio_output(gpioValue->gpio,1);
		for(j=0;j<1000000;j++);
		set_gpio_output(gpioValue->gpio,0);
		for(j=0;j<1000000;j++);
		}
		set_gpio_mode(sysledGpio, GPIO_MODE_OUTPUT);

		break;
		default://led off
		set_gpio_output(gpioValue->gpio,1);
	}
	
        return len;
}


static int __init init_gpioUser(void)
{
    int rv = 0;
	resetGpio = GPIO_RESET;
	//printk(KERN_INFO "RESET pin on GPIO %d\n",resetGpio/2);
	set_gpio_mode(resetGpio, GPIO_MODE_INPUT);

	sysledGpio = GPIO_SYS;
	//printk(KERN_INFO "SYSLED on GPIO %d\n",sysledGpio/2);
	set_gpio_mode(sysledGpio, GPIO_MODE_OUTPUT);

#if 0
	wlanledGpio = GPIO_WIRELESS_LED
	printk(KERN_INFO "WLANLED on GPIO %d\n",wlanledGpio/2);
	set_gpio_mode(wlanledGpio, GPIO_MODE_OUTPUT);
#endif

	pppledGpio = GPIO_PPP;
	//printk(KERN_INFO "PPPLED on GPIO %d\n",pppledGpio/2);
	set_gpio_mode(pppledGpio, GPIO_MODE_OUTPUT);
	
#if 1
	adslledGpio = GPIO_ADSL_LNK;
	//printk(KERN_INFO "ADSLLED on GPIO %d\n",adslledGpio/2);
	set_gpio_mode(adslledGpio, GPIO_MODE_OUTPUT);
#endif

        /* create directory */
        proc_dir = proc_mkdir(MODULE_NAME, NULL);
        if(proc_dir == NULL) {
                rv = -ENOMEM;
                goto out;
        }
        
        proc_dir->owner = THIS_MODULE;
        
        proc_reset = create_proc_entry("reset", 0444, proc_dir);
        if(proc_reset == NULL) {
                rv = -ENOMEM;
                goto noprocreset;
        }
 
        proc_sysled = create_proc_entry("sysled", 0222, proc_dir);
        if(proc_sysled == NULL) {
                rv = -ENOMEM;
                goto noprocsysled;
        }
#if 0
        proc_wlanled = create_proc_entry("wlanled", 0222, proc_dir);
        if(proc_wlanled == NULL) {
                rv = -ENOMEM;
                goto noprocwlanled;
        }
#endif
        proc_pppled = create_proc_entry("pppled", 0222, proc_dir);
        if(proc_pppled == NULL) {
                rv = -ENOMEM;
                goto noprocpppled;
        }
#if 1
        proc_adslled = create_proc_entry("adslled", 0222, proc_dir);
        if(proc_adslled == NULL) {
                rv = -ENOMEM;
                goto noprocadslled;
        }
#endif
		
        proc_bootv = create_proc_entry("bootv", 0222, proc_dir);
        if(proc_bootv == NULL) {
                rv = -ENOMEM;
                goto noprocbootv;
		}
				
        proc_productid = create_proc_entry("productid", 0222, proc_dir);
        if(proc_productid == NULL) {
                rv = -ENOMEM;
                goto noprocproductid;
        }

	/* KenWang added 2006/12/05 for new FLASH -- begin -- */
        proc_flashtype = create_proc_entry("flashtype", 0222, proc_dir);
        if(proc_flashtype == NULL) {
                rv = -ENOMEM;
                goto noprocflashtype;
        }
	/* KenWang added 2006/12/05 for new FLASH -- end -- */

        proc_myguard = create_proc_entry("myguard", 0222, proc_dir);
        if(proc_myguard == NULL) {
                rv = -ENOMEM;
                goto noprocmyguard;
        }
		
		
	resetGpioValue = 0;
	myguardValue=0;
	productidValue=0;
	sysledGpioValue.gpio = sysledGpio;
	sysledGpioValue.value = 0;
#if 0	
	wlanledGpioValue.gpio = wlanledGpio;
	wlanledGpioValue.value = 0;
#endif
	pppledGpioValue.gpio = pppledGpio;
	pppledGpioValue.value = 0;
#if 1
	adslledGpioValue.gpio = adslledGpio;
	adslledGpioValue.value = 0;
#endif
	proc_reset->data = &resetGpioValue;
	proc_reset->read_proc = reset_read_data;
	proc_reset->owner = THIS_MODULE;
                
	proc_sysled->data = &sysledGpioValue;
	proc_sysled->write_proc = proc_write_data;
	proc_sysled->owner = THIS_MODULE;
      
#if 0
	proc_wlanled->data = &wlanledGpioValue;
	proc_wlanled->write_proc = proc_write_data;
	proc_wlanled->owner = THIS_MODULE;
#endif
	
	proc_pppled->data = &pppledGpioValue;
	proc_pppled->write_proc = proc_write_data;
	proc_pppled->owner = THIS_MODULE;

	proc_bootv->data =0; 
	proc_bootv->read_proc = bootv_read_data;
	proc_bootv->owner = THIS_MODULE;
	
	proc_myguard->data = &myguardValue;
	proc_myguard->read_proc = myguard_read_data;
	proc_myguard->owner = THIS_MODULE;
	
	proc_productid->data = &productidValue;
	proc_productid->read_proc = productid_read_data;
	proc_productid->owner = THIS_MODULE;
	
	/* KenWang added 2006/12/05 for new FLASH -- begin -- */
	proc_flashtype->data = &used_flash_type;
	proc_flashtype->read_proc = flashtype_read_data;
	proc_flashtype->owner = THIS_MODULE;
	/* KenWang added 2006/12/05 for new FLASH -- end -- */
	
#if 1
	proc_adslled->data = &adslledGpioValue;
	proc_adslled->write_proc = proc_write_data;
	proc_adslled->owner = THIS_MODULE;
#endif
	
	/* everything OK */
        //printk(KERN_INFO "%s %s initialised\n", MODULE_NAME, MODULE_VERSION);
		
//we use gpio9 for lan reset here
#if 0
	set_gpio_mode(GPIO_LAN_RST, GPIO_MODE_INPUT);
  	set_gpio_output(GPIO_LAN_RST,1);
#endif
        return 0;
#if 1
noprocadslled:
        remove_proc_entry("adslled", proc_dir);
#endif
noprocpppled:
        remove_proc_entry("pppled", proc_dir);
noprocmyguard:
        remove_proc_entry("myguard", proc_dir);
noprocproductid:
        remove_proc_entry("productid", proc_dir);
/* KenWang added 2006/12/05 for new FLASH -- begin -- */
noprocflashtype:
        remove_proc_entry("flashtype", proc_dir);
/* KenWang added 2006/12/05 for new FLASH -- end -- */
noprocbootv:
        remove_proc_entry("bootv", proc_dir);
#if 0
noprocwlanled:
        remove_proc_entry("wlanled", proc_dir);
#endif
noprocsysled:
        remove_proc_entry("sysled", proc_dir);
noprocreset:
        remove_proc_entry("reset", proc_dir);
out:
        return rv;
}


static void __exit cleanup_gpioUser(void)
{
        remove_proc_entry("reset", proc_dir);
        remove_proc_entry("sysled", proc_dir);
        //remove_proc_entry("wlanled", proc_dir);
        remove_proc_entry("pppled", proc_dir);
        remove_proc_entry("myguard", proc_dir);
        remove_proc_entry("productid", proc_dir);
        remove_proc_entry("bootv", proc_dir);
        remove_proc_entry("adslled", proc_dir);
	
        remove_proc_entry(MODULE_NAME, NULL);

        //printk(KERN_INFO "%s %s removed\n",  MODULE_NAME, MODULE_VERSION);
}


module_init(init_gpioUser);
module_exit(cleanup_gpioUser);

MODULE_AUTHOR("Robert Wu, Billion Co.");
MODULE_DESCRIPTION("Provide User Led & Resetpin Control");

EXPORT_NO_SYMBOLS;
