/*
 * arch/mips/dec/int-handler.S
 *
 * Copyright (C) 1995, 1996, 1997 Paul M. Antoine and Harald Koerfgen
 * Copyright (C) 2000  Maciej W. Rozycki
 *
 * Written by Ralf Baechle and Andreas Busse, modified for DECStation
 * support by Paul Antoine and Harald Koerfgen.
 *
 * completly rewritten:
 * Copyright (C) 1998 Harald Koerfgen
 *
 * Copyright (C) 2002 Viosoft Corp.
 * Support for TrendChip TC3160
 *
 * Modified by Robert Wu, Billion Co.
 * Support for TrendChip TC3162
 *
 * 20041001 use direct interrupt dispatch instead of check pending register
 * Huan Shiou Tseng
 */
#include <asm/asm.h>
#include <asm/regdef.h>
#include <asm/mipsregs.h>
#include <asm/stackframe.h>
#include <asm/addrspace.h>

#include <asm/tc3162/interrupts.h>


		.text
		.set	noreorder
/*
 * tc3162_handle_int: Interrupt handler for Lexra
 */
		.align	5
		NESTED(tc3162_handle_int, PT_SIZE, ra)
		.set	noat
		SAVE_ALL
		CLI				# TEST: interrupts should be off
		.set	at
		.set	noreorder
/* to see the pending register seems to waste time. 
	it only happen at the interval between processing one interrupt
	and oterhs happen SV Tseng 20040924
*/
#if 1
		la	t0, TC3162_IntVec_REG
		lw	a0, 0(t0)
		nop
#else
		/*
		 * Get pending Interrupts
		 */
		lw	t0, TC3162_IPR_REG
		lw	t2, TC3162_IMR_REG
		la	t1, cpu_mask_tbl
	        and 	t0, t2     # isolate allowed ones

		/*
		 * Find irq with highest priority
		 */
1:		lw	t2, (t1)
      		nop
		move	t3, t0
      		nop
		and	t3, t2
		beq	t3, zero, 1b
		addu	t1, PTRSIZE		# delay slot

		/*
		 * Do the low-level stuff
		 */
		lw	a0, %lo(cpu_irq_nr-cpu_mask_tbl-PTRSIZE)(t1)
      		nop
#endif

		/*
		 * Handle it
		 */
      		jal	do_IRQ
		move	a1,sp
       
		j	ret_from_irq
		nop
		
		END(tc3162_handle_int)
      
/*
 * Generic unimplemented interrupt routines - ivec_tbl is initialised to
 * point all interrupts here.  The table is then filled in by machine-specific
 * initialisation in lexra_arch_setup().
 */
		EXPORT(tc3162_intr_unimplemented)
tc3162_intr_unimplemented:        # !!! need set real interrupt number !!!
		mfc0	a1,CP0_CAUSE		# cheats way of printing an arg!
		nop				# to be sure...
		PANIC("Unimplemented cpu interrupt! CP0_CAUSE: 0x%x");


/*
 * FIXME: This interrupt vector table is experimental.  It is initialised with 
 *	  *_intr_unimplemented and filled in with the addresses of
 *	  machine-specific interrupt routines in lexra_arch_setup().
 *
 *	  The mask_tbls contain the interrupt masks which are used. It is 
 *	  initialised with all possible interrupt status bits set, so that 
 * 	  unused Interrupts are catched.
 */
		.data
		EXPORT(cpu_mask_tbl)
cpu_mask_tbl:
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
#ifdef CONFIG_TRENDCHIP_TC3162
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
#endif
#ifndef CONFIG_TRENDCHIP_WDL100
		.word	0x00000000
		.word	0x00000000
#endif
		.word	0x0000ffff		# End of list

		EXPORT(cpu_irq_nr)
cpu_irq_nr:
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000 
		.word	0x00000000
		.word	0x00000000
#ifdef CONFIG_TRENDCHIP_TC3162
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
#endif
#ifndef CONFIG_TRENDCHIP_WDL100
		.word	0x00000000
		.word	0x00000000
#endif
		.word	0x00ffffff		# End of list

