/*
 * Setup the interrupt stuff.
 */
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/mc146818rtc.h>
#include <linux/param.h>
#include <linux/console.h>
#include <linux/init.h>
#include <asm/mipsregs.h>
#include <asm/bootinfo.h>
#include <asm/virtconvert.h>
#include <asm/irq.h>
#include <asm/reboot.h>
#include <asm/tc3160/gpio.h>
#include <asm/tc3160/mi.h>
#include <asm/tc3160/arb.h>
#include <asm/tc3162/interrupts.h>
#include <asm/tc3162/tc3162.h>

extern asmlinkage void tc3162_handle_int(void);

void tc3160_irq_init(void);

extern void (*board_time_init)(void);
extern void (*board_timer_setup)(struct irqaction *);
extern void tc3162_time_init(void);
extern void tc3162_time_setup(struct irqaction *);

volatile unsigned long *int_mask_reg = (volatile unsigned long *)TC3162_IMR_REG;

/*
 * Information regarding the IRQ Controller
 */

extern void lexra_machine_restart(char *command);
extern void lexra_machine_halt(void);
extern void lexra_machine_power_off(void);

#if defined(CONFIG_TRENDCHIP_TC3162L2) && defined(TC3162L2_IMEM) && defined(TC3162L2_DMEM)
extern int __imem, __dmem;
#endif

static void __init tc3162_component_setup(void)
{
	unsigned int controlReg;
	unsigned long flags;
#if defined(CONFIG_TRENDCHIP_TC3162L2) && defined(TC3162L2_IMEM) && defined(TC3162L2_DMEM)
	/* setup bus timeout value */
	VPint(CR_AHB_AACS) = 0x0808;

	/* setup imem start address */
	VPint(CR_IMEM) = PHYSADDR(&__imem);
	/* setup dmem start address */
	VPint(CR_DMEM) = PHYSADDR(&__dmem);

	/* clear internal imem */
	save_and_cli(flags);
	controlReg = read_32bit_cp0_register(CP0_CCTL);
	write_32bit_cp0_register(CP0_CCTL, (controlReg & ~CCTL_IMEMOFF));
	write_32bit_cp0_register(CP0_CCTL, (controlReg | CCTL_IMEMOFF));
	write_32bit_cp0_register(CP0_CCTL, controlReg);
	restore_flags(flags);

	/* refill internal imem */
	save_and_cli(flags);
	controlReg = read_32bit_cp0_register(CP0_CCTL);
	write_32bit_cp0_register(CP0_CCTL, (controlReg & ~CCTL_IMEMFILL4));
	write_32bit_cp0_register(CP0_CCTL, (controlReg | CCTL_IMEMFILL4));
	write_32bit_cp0_register(CP0_CCTL, controlReg);
	restore_flags(flags);
	
	printk("Enable IMEM addr=%x\n", PHYSADDR(&__imem));

#if 1
	/* clear internal dmem */
	save_and_cli(flags);
	controlReg = read_32bit_cp0_register(CP0_CCTL);
	write_32bit_cp0_register(CP0_CCTL, (controlReg & ~CCTL_DMEMOFF));
	write_32bit_cp0_register(CP0_CCTL, (controlReg | CCTL_DMEMOFF));
	write_32bit_cp0_register(CP0_CCTL, controlReg);
	restore_flags(flags);

	/* internal dmem on */
	save_and_cli(flags);
	controlReg = read_32bit_cp0_register(CP0_CCTL);
	write_32bit_cp0_register(CP0_CCTL, (controlReg & ~CCTL_DMEMON));
	write_32bit_cp0_register(CP0_CCTL, (controlReg | CCTL_DMEMON));
	write_32bit_cp0_register(CP0_CCTL, controlReg);
	restore_flags(flags);

	printk("Enable DMEM addr=%x\n", PHYSADDR(&__dmem));
#endif
#endif
}

void __init lexra_arch_setup(void)
{
#ifdef CONFIG_TRENDCHIP_TC3162
    tc3162_component_setup();

    board_time_init = tc3162_time_init;
    board_timer_setup = tc3162_time_setup;
#endif

    _machine_restart = lexra_machine_restart;
    _machine_halt = lexra_machine_halt;
    _machine_power_off = lexra_machine_power_off;
}

