/*  
 * Implementations used to TC3162 MTD driver.
 *
 * Written by Billion Co.
 * Support for TrendChip TC3162
 */
#include <linux/config.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/delay.h>

#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/mtd/partitions.h>

#include <asm/io.h>
#include <asm/tc3162/gpio.h>
#include <asm/tc3162/tc3162.h>
#ifdef 	DEBUG_RW
#define	DBG(x...)	printk(x)
#else
#define	DBG(x...)	
#endif

#define WINDOW_ADDR 0xBFC00000
#define WINDOW_SIZE 0x100000
#define CONFIG_TC3162		1
#ifdef ACT
#define KERNEL_START_ADDRESS 0xbfc40000
#else
#define KERNEL_START_ADDRESS 0xbfc20000
#endif
#include "flash.h"
unsigned int bi_kernel_length=0;

unsigned int flshSectorAddr[] = {
(0xbfc00000+SEC0),
(0xbfc00000+SEC1),
(0xbfc00000+SEC2),
(0xbfc00000+SEC3),
(0xbfc00000+SEC4),
(0xbfc00000+SEC5),
(0xbfc00000+SEC6),
(0xbfc00000+SEC7),
(0xbfc00000+SEC8),
(0xbfc00000+SEC9),
(0xbfc00000+SECa),
(0xbfc00000+SECb),
(0xbfc00000+SECc),
(0xbfc00000+SECd),
(0xbfc00000+SECe),
(0xbfc00000+SECf),
(0xbfc00000+SEC10),
(0xbfc00000+SEC11),
(0xbfc00000+SEC12),
(0xbfc00000+SEC13),
(0xbfc00000+SEC14),
(0xbfc00000+SEC15),
(0xbfc00000+SEC16),
(0xbfc00000+SEC17),
(0xbfc00000+SEC18),
(0xbfc00000+SEC19),
(0xbfc00000+SEC1a),
(0xbfc00000+SEC1b),
(0xbfc00000+SEC1c),
(0xbfc00000+SEC1d),
(0xbfc00000+SEC1e),
(0xbfc00000+SEC1f),
(0xbfc00000+SEC20),
(0xbfc00000+SEC21),
(0xbfc00000+SEC22),
(0xbfc00000+SEC23),
(0xbfc00000+SEC24),
(0xbfc00000+SEC25),
(0xbfc00000+SEC26),
(0xbfc00000+SEC27),
(0xbfc00000+SEC28),
(0xbfc00000+SEC29),
(0xbfc00000+SEC2a),
(0xbfc00000+SEC2b),
(0xbfc00000+SEC2c),
(0xbfc00000+SEC2d),
(0xbfc00000+SEC2e),
(0xbfc00000+SEC2f),
(0xbfc00000+SEC30),
(0xbfc00000+SEC31),
(0xbfc00000+SEC32),
(0xbfc00000+SEC33),
(0xbfc00000+SEC34),
(0xbfc00000+SEC35),
(0xbfc00000+SEC36),
(0xbfc00000+SEC37),
(0xbfc00000+SEC38),
(0xbfc00000+SEC39),
(0xbfc00000+SEC3a),
(0xbfc00000+SEC3b),
(0xbfc00000+SEC3c),
(0xbfc00000+SEC3d),
(0xbfc00000+SEC3e),
(0xbfc00000+SEC3f),
(0xbfc00000+SEC40),
(0xbfc00000+SEC41),
(0xbfc00000+SEC42),
(0xbfc00000+SEC43),
(0xbfc00000+SEC44),
(0xbfc00000+SEC45),
(0xbfc00000+SEC46)
};

#define TC3162L2_FLASH_RW	0
#define RETRY_CNT			10

__u8 physmap_read8(struct map_info *map, unsigned long ofs)
{
	__u8 ret;
#ifdef TC3162L2_FLASH_RW
	int cnt = RETRY_CNT;
	do {
		ret = __raw_readb(map->map_priv_1 + ofs);
		if (!(VPint(CR_AHB_AACS) & (1<<25))) 
			break;
		mdelay(1);
		cnt--;
	} while (cnt);
	if (cnt == 0) {
		printk("CR_AHB_AACS=%x\n", VPint(CR_AHB_AACS));
		printk("CR_AHB_ABEM=%x\n", VPint(CR_AHB_ABEM));
		printk("CR_AHB_ABEA=%x\n", VPint(CR_AHB_ABEA));
	}
#else
	ret = __raw_readb(map->map_priv_1 + ofs);
#endif
	DBG("read8 from %x, %x\n", (unsigned)(map->map_priv_1 + ofs), ret);
	return ret;
}

__u16 physmap_read16(struct map_info *map, unsigned long ofs)
{
	__u16 ret;
#ifdef TC3162L2_FLASH_RW
	int cnt = RETRY_CNT;
	do {
		ret = __raw_readw(map->map_priv_1 + ofs);
		if (!(VPint(CR_AHB_AACS) & (1<<25))) 
			break;
		mdelay(1);
		cnt--;
	} while (cnt);
	if (cnt == 0) {
		printk("CR_AHB_AACS=%x\n", VPint(CR_AHB_AACS));
		printk("CR_AHB_ABEM=%x\n", VPint(CR_AHB_ABEM));
		printk("CR_AHB_ABEA=%x\n", VPint(CR_AHB_ABEA));
	}
#else
	ret = __raw_readw(map->map_priv_1 + ofs);
#endif
	DBG("read16 from %x, %x\n", (unsigned)(map->map_priv_1 + ofs), ret);
	return ret;
}

__u32 physmap_read32(struct map_info *map, unsigned long ofs)
{
	__u32 ret;
#ifdef TC3162L2_FLASH_RW
	int cnt = RETRY_CNT;
	do {
		ret = __raw_readl(map->map_priv_1 + ofs);
		if (!(VPint(CR_AHB_AACS) & (1<<25))) 
			break;
		mdelay(1);
		cnt--;
	} while (cnt);
	if (cnt == 0) {
		printk("CR_AHB_AACS=%x\n", VPint(CR_AHB_AACS));
		printk("CR_AHB_ABEM=%x\n", VPint(CR_AHB_ABEM));
		printk("CR_AHB_ABEA=%x\n", VPint(CR_AHB_ABEA));
	}
#else
	ret = __raw_readl(map->map_priv_1 + ofs);
#endif
	DBG("read32 from %x, %x\n", (unsigned)(map->map_priv_1 + ofs), ret);
	return ret;
}

void physmap_copy_from(struct map_info *map, void *to, unsigned long from, ssize_t len)
{
	int i, cnt = RETRY_CNT;

	DBG("physmap_copy from %x to %x len=%d\n", (unsigned)from, (unsigned)to, len);
#ifdef TC3162L2_FLASH_RW
	if ((((uint32) from | (uint32) to) & 3) || (len < 4) || (len & 3)) {
		do {
			memcpy_fromio(to, map->map_priv_1 + from, len);
			if (!(VPint(CR_AHB_AACS) & (1<<25))) 
				break;
			mdelay(1);
			cnt--;
		} while (cnt);
		if (cnt == 0) {
			printk("CR_AHB_AACS=%x\n", VPint(CR_AHB_AACS));
			printk("CR_AHB_ABEM=%x\n", VPint(CR_AHB_ABEM));
			printk("CR_AHB_ABEA=%x\n", VPint(CR_AHB_ABEA));
		}
	} else {
		uint32 *dest = (uint32 *) to;
		uint32 *src = (uint32 *) (map->map_priv_1 + from);
		for (i = 0; i < len; i += 4, dest++, src++) {
			cnt = RETRY_CNT;
			do {
				*dest = *src;
				if (!(VPint(CR_AHB_AACS) & (1<<25))) 
					break;
				mdelay(1);
				cnt--;
			} while (cnt);
			if (cnt == 0) {
				printk("CR_AHB_AACS=%x\n", VPint(CR_AHB_AACS));
				printk("CR_AHB_ABEM=%x\n", VPint(CR_AHB_ABEM));
				printk("CR_AHB_ABEA=%x\n", VPint(CR_AHB_ABEA));
			}
		}
	}
#else
	memcpy_fromio(to, map->map_priv_1 + from, len);
#endif
}

void physmap_write8(struct map_info *map, __u8 d, unsigned long adr)
{
	//DBG("write8 at %x, %x\n", (unsigned)(map->map_priv_1 + adr), d);
#ifdef TC3162L2_FLASH_RW
	int cnt = RETRY_CNT;
	do {
		__raw_writeb(d, map->map_priv_1 + adr);
		if (!(VPint(CR_AHB_AACS) & (1<<25))) 
			break;
		mdelay(1);
		cnt--;
	} while (cnt);
	if (cnt == 0) {
		printk("CR_AHB_AACS=%x\n", VPint(CR_AHB_AACS));
		printk("CR_AHB_ABEM=%x\n", VPint(CR_AHB_ABEM));
		printk("CR_AHB_ABEA=%x\n", VPint(CR_AHB_ABEA));
	}
#else
	__raw_writeb(d, map->map_priv_1 + adr);
#endif
	mb();
}

void physmap_write16(struct map_info *map, __u16 d, unsigned long adr)
{
	DBG("write16 at %x, %x\n", (unsigned)(map->map_priv_1 + adr), d);
#ifdef TC3162L2_FLASH_RW
	int cnt = RETRY_CNT;
	do {
		__raw_writew(d, map->map_priv_1 + adr);
		if (!(VPint(CR_AHB_AACS) & (1<<25))) 
			break;
		mdelay(1);
		cnt--;
	} while (cnt);
	if (cnt == 0) {
		printk("CR_AHB_AACS=%x\n", VPint(CR_AHB_AACS));
		printk("CR_AHB_ABEM=%x\n", VPint(CR_AHB_ABEM));
		printk("CR_AHB_ABEA=%x\n", VPint(CR_AHB_ABEA));
	}
#else
	__raw_writew(d, map->map_priv_1 + adr);
#endif
	mb();
}

void physmap_write32(struct map_info *map, __u32 d, unsigned long adr)
{
	DBG("write32 at %x, %x\n", (unsigned)(map->map_priv_1 + adr), d);
#ifdef TC3162L2_FLASH_RW
	int cnt = RETRY_CNT;
	do {
		__raw_writel(d, map->map_priv_1 + adr);
		if (!(VPint(CR_AHB_AACS) & (1<<25))) 
			break;
		mdelay(1);
		cnt--;
	} while (cnt);
	if (cnt == 0) {
		printk("CR_AHB_AACS=%x\n", VPint(CR_AHB_AACS));
		printk("CR_AHB_ABEM=%x\n", VPint(CR_AHB_ABEM));
		printk("CR_AHB_ABEA=%x\n", VPint(CR_AHB_ABEA));
	}
#else
	__raw_writel(d, map->map_priv_1 + adr);
#endif
	mb();
}

void physmap_copy_to(struct map_info *map, unsigned long to, const void *from, ssize_t len)
{
	DBG("physmap_copy_to %x from %x\n", (unsigned)to, (unsigned)from);
#ifdef TC3162L2_FLASH_RW
	int cnt = RETRY_CNT;
	do {
		memcpy_toio(map->map_priv_1 + to, from, len);
		if (!(VPint(CR_AHB_AACS) & (1<<25))) 
			break;
		mdelay(1);
		cnt--;
	} while (cnt);
	if (cnt == 0) {
		printk("CR_AHB_AACS=%x\n", VPint(CR_AHB_AACS));
		printk("CR_AHB_ABEM=%x\n", VPint(CR_AHB_ABEM));
		printk("CR_AHB_ABEA=%x\n", VPint(CR_AHB_ABEA));
	}
#else
	memcpy_toio(map->map_priv_1 + to, from, len);
#endif
}



static struct map_info tc3162_map = {
	name:	"TC3162 nor-flash",
	read8: physmap_read8,
	read16: physmap_read16,
	read32: physmap_read32,
	copy_from: physmap_copy_from,
	write8: physmap_write8,
	write16: physmap_write16,
	write32: physmap_write32,
	copy_to: physmap_copy_to,
};


#ifdef CONFIG_TC3162

static unsigned long flash_size = 0x00200000;
static unsigned char flash_buswidth = 2;
static struct mtd_partition tc3162_partitions[] = {
#if 0
	{									 /* First partition */ 
		  name : "bootloader",			 /* Bootloader section */ 
		  size	  : 0x00010000, 		 /* Size  */ 
		  offset  : 0,					 /* Offset from start of flash- location 0x0*/  
		  mask_flags : MTD_WRITEABLE	 /* This partition is not writable */ 
	},	
	{									 /* Second partition */ 
		  name : "romfile",				 /* config filesystem section */ 
		  size	  :  0x00010000,		 /* Size */ 
		  offset : MTDPART_OFS_APPEND,	 /* Append after bootloader section */ 
//		  mask_flags : MTD_WRITEABLE	 /* This partition is not writable */ 
	},
#endif
        {
                //name: "Boot1",
                name: "boot1",
                size: 0x0002000, // 8k bootcode
                offset: 0,
        },
        {
                //name: "Mac",
                name: "mac",
                size: 0x0002000, // 8k Mac/vendor info.
                offset: MTDPART_OFS_APPEND,
        },
        {
                //name: "config",
                name: "config",
                size: 0x000C000, // 48KB config
                offset: MTDPART_OFS_APPEND,
        },
        {
                //name: "Boot2",
                name: "boot2",
                size: 0x0010000, // 64k bootcode
                offset: MTDPART_OFS_APPEND,
        },
#ifdef ACT
        {
                //name: "DSP",
                name: "DSP",
                size: 0x0020000, // 64k bootcode
                offset: MTDPART_OFS_APPEND,
        },
#endif
	{									 /* Third partition */ 
		  name : "kernel",				 /* Kernel section */ 
		  size	  :  0x00120000,		 /* Size */ 
		  offset : MTDPART_OFS_APPEND,	 /* Append after bootloader section */ 
//		  mask_flags : MTD_WRITEABLE	 /* This partition is not writable */ 
	},	
	{									 /* Fourth partition */ 
		  name : "rootfs", 			 	 /* Root filesystem section */ 
		  size	  :  MTDPART_SIZ_FULL,	 /* Occupy rest of flash */ 
		  offset :	MTDPART_OFS_APPEND	 /* Append after kernel section */ 
	}
};

#else
#error "this driver only work for TrendChip TC3162"
#endif

#define NB_OF(x)  (sizeof(x)/sizeof(x[0]))

static struct mtd_partition *parsed_parts;
static struct mtd_info *mymtd;

static int calculating_boundary(int sum)
{
int i=0;
int j=0;
#ifndef MX29LV160BT
j = (KERNEL_START_ADDRESS > 0xbfc10000)? (8 + (KERNEL_START_ADDRESS - 0xbfc10000)/0x10000) : (KERNEL_START_ADDRESS/0x2000);
#endif
#ifdef MX29LV160BT
for(i=7;i<35;i++)
#else
for(i=j;i<71;i++)
#endif
{
if((flshSectorAddr[i] - KERNEL_START_ADDRESS) > sum)
  break;
}
return (flshSectorAddr[i] - KERNEL_START_ADDRESS);

}

/****************************************************************************/
/*
 * Find the MTD device with the given name
 */

static struct mtd_info *get_mtd_named(char *name)
{
	int i;
	struct mtd_info *mtd;

	for (i = 0; i < MAX_MTD_DEVICES; i++) {
		mtd = get_mtd_device(NULL, i);
		if (mtd) {
			if (strcmp(mtd->name, name) == 0)
				return(mtd);
			put_mtd_device(mtd);
		}
	}
	return(NULL);
}

int __init tc3162_mtd_init(void)
{
	struct mtd_partition *parts;
	int nb_parts = 0, i=0;
	unsigned int gpiodata;
	struct mtd_info *mtd;
	volatile unsigned short int *flash_base=(unsigned short int*)0xbfc00000;
	int my_boundary_size=0;
	
	// set gpio5 to output pin
	gpiodata = *((unsigned int *)TC3162_GPCTL_REG);
	*((unsigned int *)TC3162_GPCTL_REG) = gpiodata|0x400;
	// set gpio5 = 1
	gpiodata = *((unsigned short int *)TC3162_GPDATA_REG);
	*((unsigned short int *)TC3162_GPDATA_REG) = gpiodata|0x20;

	/* Default flash buswidth */
	tc3162_map.buswidth = flash_buswidth;

	/* dynamically adjust mtd boarder */

	
	my_boundary_size = calculating_boundary(bi_kernel_length);
#ifdef ACT
	tc3162_partitions[5].size = my_boundary_size;
#else
	tc3162_partitions[4].size = my_boundary_size;
#endif

	/*
	 * Static partition definition selection
	 */
	parts = tc3162_partitions;
	nb_parts = NB_OF(tc3162_partitions);
	tc3162_map.size = flash_size;

	/*
	 * Now let's probe for the actual flash.  Do it here since
	 * specific machine settings might have been set above.
	 */
	printk(KERN_NOTICE "tc3162 flash: probing %d-bit flash bus\n", 	tc3162_map.buswidth*8);
	tc3162_map.map_priv_1 = (unsigned long) WINDOW_ADDR ;

	mymtd = do_map_probe("amd_flash",  &tc3162_map);

	if (!mymtd)
	{
		printk("tc3162 error no probe\n");
		return -ENXIO;
	}
	mymtd->module = THIS_MODULE;

	add_mtd_partitions(mymtd, parts, nb_parts);

#ifndef CONFIG_COMPRESS_RAMDISK
	mtd = get_mtd_named("rootfs");
	if (mtd) {
		ROOT_DEV = MKDEV(MTD_BLOCK_MAJOR, mtd->index);
		put_mtd_device(mtd);
	}
#endif

	return 0;
}

static void __exit tc3162_mtd_cleanup(void)
{
	if (mymtd) {
		del_mtd_partitions(mymtd);
		map_destroy(mymtd);
		if (parsed_parts)
			kfree(parsed_parts);
	}
}

module_init(tc3162_mtd_init);
module_exit(tc3162_mtd_cleanup);

MODULE_AUTHOR("David Liu");
MODULE_DESCRIPTION("tc3162 CFI map driver");
MODULE_LICENSE("GPL");
