#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/types.h>
#include <linux/pci.h>
#include <linux/delay.h>

#include <asm/pci_channel.h>
#include <asm/tc3162/interrupts.h>
#include <asm/tc3162/tc3162_pci.h>

// Sam add for TC3162P2
#include <asm/tc3162/tc3162.h>
// End Sam add

#include <asm/tc3162/gpio.h>

#ifdef DEBUG
#define PRINTK printk
#else
#define PRINTK(arg...) NULL
#endif

int tc_processor=0;  // TC3162P = 1, TC3162L = 0

static struct resource pci_io_resource = {
	"pci IO space", 
	TC3162_PCI_DEVICE_IO_START,
	TC3162_PCI_DEVICE_IO_END,
	IORESOURCE_IO  // define in include/linux/ioport.h
};

static struct resource pci_mem_resource = {
	"pci memory space", 
	TC3162_PCI_DEVICE_MEM_START,
	TC3162_PCI_DEVICE_MEM_END,
	IORESOURCE_MEM  //define in include/linux/ioport.h
};

extern struct pci_ops tc3162_pci_ops;

struct pci_channel mips_pci_channels[] = {
		{&tc3162_pci_ops, &pci_io_resource, &pci_mem_resource, (int)NULL, (int)NULL },
		{ NULL,           NULL,             NULL,              (int)NULL, (int)NULL },
};

/*----------------------------------------------------------*/
int access_pci_config(struct pci_dev *dev, int where, u32 *data, int write)
{
	int slot = (dev->devfn >> 3) & 0x1F;
	int func =  dev->devfn & 0x7;

	if ( slot > 2 || func > 7 ) {
		*data = 0xFFFFFFFF;
		return -1;
	}

	if (slot != 0) // host-0x80000000, slot1-0x80000800, slot2-0x80001000
    where |= TC3162_PCI_HOST_CONFIG_MEM_ADDR |(1L << (10+slot)) | (func << 8);
	else
    where |= TC3162_PCI_HOST_CONFIG_MEM_ADDR | (func << 8);

// Sam add debug
//	printk("Sam add pci where is %08x\n",where);
// End Sam 
   *(volatile unsigned long *)(TC3162_PCI_CONFIG_ADDR) = where & ~3;

	if ( write )
	 *(volatile unsigned long *)(TC3162_PCI_CONFIG_DATA) = *data;
	else
	{
            *data = *(volatile unsigned long *)(TC3162_PCI_CONFIG_DATA);
	}
	return PCIBIOS_SUCCESSFUL;
}

#define read_pci_config(dev, where, data)  access_pci_config(dev, where, data, 0)
#define write_pci_config(dev, where, data)  access_pci_config(dev, where, data, 1)

static int read_config_byte(struct pci_dev *dev, int where, u8 *val)
{
	u32 data;
    int ret = read_pci_config(dev, where, &data);
	int shift = (where&3) << 3;
    *val = (data >> shift) & 0x000000FF;
//printk("read_config_byte value=%08x\n\n",*val);
	return ret; 
}

static int read_config_word(struct pci_dev *dev, int where, u16 *val)
{
	u32 data;
    int ret = read_pci_config(dev, where, &data);
	int shift = (where&3) << 3;
    *val = (data >> shift) & 0x0000FFFF;
//printk("read_config_word value=%08x\n\n",*val);
	return ret; 
}

static int read_config_dword(struct pci_dev *dev, int where, u32 *val)
{
   int ret;
 //   return read_pci_config(dev, where, val);
       ret = read_pci_config(dev, where, val);
//printk("read_config_dword value=%08x\n\n",*val);
return ret;
}


static int write_config_byte(struct pci_dev *dev, int where, u8 val)
{
	u32 data;
	int shift = (where&3) << 3;
	if ( read_pci_config(dev, where, &data) )
	    return -1;
	data &= ~(0xFF << shift);
    data |= val << shift;
	return write_pci_config(dev, where, &data);
}

static int write_config_word(struct pci_dev *dev, int where, u16 val)
{
	u32 data;
	int shift = (where&3) << 3;
	if ( read_pci_config(dev, where, &data) )
	    return -1;
	data &= ~(0xFFFF << shift);
	data |= val << shift;
	return write_pci_config(dev, where, &data);
}

static int write_config_dword(struct pci_dev *dev, int where, u32 val)
{
    return write_pci_config(dev, where, &val);
}


struct pci_ops tc3162_pci_ops = {
    read_config_byte,
    read_config_word,
    read_config_dword,
    write_config_byte,
    write_config_word,
    write_config_dword
};


//--------------------------------------------------------------------
// needed by arch/mips/kernel/pci.c:pcibios_fixups[] needed by quirks.c:pci_do_fixups()

static unsigned long io_start = TC3162_PCI_DEVICE_IO_START;
static unsigned long mem_start= TC3162_PCI_DEVICE_MEM_START;


void __init pcibios_fixup_resources(struct pci_dev *dev)   
{
	unsigned int slot, func, i, reg, phy_addr, size;
    struct resource *res;

	if ( dev->hdr_type != PCI_HEADER_TYPE_NORMAL )  return;

    slot = PCI_SLOT(dev->devfn);

    func = PCI_FUNC(dev->devfn);

    res  = &dev->resource[0];

    if (slot == 1)
	mem_start = TC3162_PCI_DEVICE1_MEM_START; 
    else if (slot == 2)
	mem_start = TC3162_PCI_DEVICE2_MEM_START;
    else 
	mem_start = TC3162_PCI_DEVICE_MEM_START;

    for ( i = 0 ; i < 6 ; i++, res++ )  {
        if ( pci_resource_len(dev, i) == 0 )
            continue;

        size = res->end - res->start;

   	reg = PCI_BASE_ADDRESS_0 + (i << 2);

        pci_read_config_dword(dev, reg, &phy_addr);

        switch ( phy_addr & PCI_BASE_ADDRESS_SPACE )  {

          case PCI_BASE_ADDRESS_SPACE_IO:
            if ( (io_start & size) != 0 )  // start address doesn't aligned the mutiple of size
                io_start = (io_start & ~size) + (size+1);
            phy_addr = io_start;
            io_start += size < 0x1000 ? 0x1000 : size+1;
            break;
          default:

          case PCI_BASE_ADDRESS_SPACE_MEMORY:
//printk("Sam tc3162_pci.c 180 ADDRESS_SPACE_MEMORY %08x\n",mem_start);
	    if (slot == 0)
		break;
	    else
	    {
            if ( (mem_start & size) != 0 )  // start address doesn't aligned the mutiple of size
                mem_start = (mem_start & ~size) + (size+1);
                phy_addr = mem_start;
                mem_start += size < 0x1000 ? 0x1000 : size+1;
            break;
	    }

        }

//        printk( "**PCI: slot %d, func %d, base %d remap to %08x (size=%x) (reg=%08x)\n", slot, func, i, phy_addr, size, reg );

        pci_write_config_dword(dev, reg, phy_addr);

        res->start = phy_addr;

        res->end   = phy_addr + size;
    }
}


// needed by arch/mips/kernel/pci.c: pcibios_init()
void __init pcibios_fixup(void)
{
}

// needed by arch/mips/kernel/pci.c: pcibios_init()
void __init pcibios_fixup_irqs(void)
{
	unsigned int slot, func;
	unsigned char pin;
	struct pci_dev *dev;

	pci_for_each_dev(dev) {
		if (dev->bus->number != 0)
		    return;
		read_config_byte(dev, PCI_INTERRUPT_PIN, &pin);
		slot = PCI_SLOT(dev->devfn);
		func = PCI_FUNC(dev->devfn);
                if( slot == 1) 
		{      
			dev->irq = INTR_PCI_INTA;
		}
		else if (slot == 2 )
		{
                	dev->irq = INTR_PCI_INTB;
                }
		//else 
                //   	printk("Unknown PCI slot number is %d\n",slot);

		write_config_byte(dev, PCI_INTERRUPT_LINE, dev->irq);
		//PRINTK(KERN_INFO "PCI: slot %d func %d irq %d ", slot, func, dev->irq);
		//PRINTK( "start=%X end=%X\n", dev->resource[0].start, dev->resource[0].end);
	}
}

// needed by drivers/pci/pci.c: pci_scan_bridge()
unsigned int pcibios_assign_all_busses(void)
{
	return 1;
}


//---------------------------------------------------------
int tc3162_pci_init(void)
{
   unsigned long pci_reg_data;
   unsigned int gpiomode;
   unsigned short gpiodata;

#ifndef TC3162L2_7300ONLY
   printk("============================================================\n");
   printk("TC3162 PCI interface initialize\n");


#ifdef ACT  

// Sam add for test 7202VGP
    pci_reg_data = *(volatile unsigned long*)(0xbfc4000c);
    
    if (pci_reg_data == 0x64653132)
    {
    	//20070307, henry add to reset pci bus
    *(volatile unsigned int *)(0xbfb00088) &= ~(1<<31);
    mdelay(100);    	
	*(volatile unsigned int *)(0xbfb00088) = (1<<31);
	mdelay(100);
	gpiomode = *(volatile unsigned int*)CR_AHB_PCICR;

     // Step 0. set gpio 12 as output mode
     gpiomode = *(volatile unsigned int*)(TC3162_GPCTL_REG);
     gpiomode |= (1 << GPIO_12);
     *(volatile unsigned int*)TC3162_GPCTL_REG = gpiomode;
                                                                                                                                                                                   
     // Step 1. enable open drain
     gpiodata = *(volatile unsigned short*)(TC3162_GPOPENDRAIN_REG);
     gpiodata |= (1 <<(GPIO_12/2));
     *(volatile unsigned short *)TC3162_GPOPENDRAIN_REG = gpiodata;

     //Step2. 
     gpiodata = *(volatile unsigned short*)TC3162_GPDATA_REG;
     gpiodata &= ~(1<<(GPIO_12/2));
     *(volatile unsigned short*)TC3162_GPDATA_REG = gpiodata;
     //mdelay(1);
     udelay(120);
     gpiodata = *(volatile unsigned short*)TC3162_GPDATA_REG;
     gpiodata |= (1<<(GPIO_12/2));
     *(volatile unsigned short*)TC3162_GPDATA_REG = gpiodata;

	//  Sam add test
	mdelay(33);
	// End Sam

    }
    else
#endif
// End Sam add
    {
	// Sam add for TC3162P2
#if 1
    	//20070307, henry add to reset pci bus
    *(volatile unsigned int *)(0xbfb00088) &= ~(1<<31);
    mdelay(100);
	*(volatile unsigned int *)(0xbfb00088) = (1<<31);
    mdelay(100);
	gpiomode = *(volatile unsigned int*)CR_AHB_PCICR;
	// End Sam add
#endif

//sam report
	mdelay(1);
	
     // Step 0. set gpio 9 as output mode
#if 1 
     gpiomode = *(volatile unsigned int*)(TC3162_GPCTL_REG);
     gpiomode |= (1 << GPIO_9);
     *(volatile unsigned int*)TC3162_GPCTL_REG = gpiomode;
                                                                                                                                                                                   
     // Step 1. enable open drain
     gpiodata = *(volatile unsigned short*)(TC3162_GPOPENDRAIN_REG);
     gpiodata |= (1 <<(GPIO_9/2));
     *(volatile unsigned short *)TC3162_GPOPENDRAIN_REG = gpiodata;
                                                                                                                                                                                   
     //Step2. light wlan led on

     gpiodata = *(volatile unsigned short*)TC3162_GPDATA_REG;
     gpiodata &= ~(1<<(GPIO_9/2));
     *(volatile unsigned short*)TC3162_GPDATA_REG = gpiodata;
     //udelay(140);
     //sam report
     mdelay(1);

     gpiodata = *(volatile unsigned short*)TC3162_GPDATA_REG;
     gpiodata |= (1<<(GPIO_9/2));
     *(volatile unsigned short*)TC3162_GPDATA_REG = gpiodata;

     //mdelay(33);
     //sam report
     mdelay(40);
#endif
    }
//#endif



#if 1
   // Sam add for read PCI register to determine TC3162P or TC3162L
   *(volatile unsigned long *)(TC3162_PCI_CONFIG_ADDR) = (TC3162_PCI_HOST_CONFIG_MEM_ADDR | PCI_VENDOR_ID);
   pci_reg_data = *(volatile unsigned long *)(TC3162_PCI_CONFIG_DATA);
   if (pci_reg_data == 0x31629005) // yes,we get correct TC3162P PCI VENDOR_ID and DEVICE_ID, this is TC3162P
   {
	tc_processor = 1; //tc_processor = TC3162P
      
   // just assign tc3162 pci host memory base address0 any value
   *(volatile unsigned long *)(TC3162_PCI_CONFIG_ADDR) = (TC3162_PCI_HOST_CONFIG_MEM_ADDR | PCI_BASE_ADDRESS_0);
   *(volatile unsigned long *)(TC3162_PCI_CONFIG_DATA) = 0x0;


#if 0 // Sam mark the original setting
   // write cache line size (0x08 -- 8 DWORD of pre-fetched data ) and
   // latency timer (0x20) , these value are suggested by trendchip
   *(volatile unsigned long *)(TC3162_PCI_CONFIG_ADDR) = (TC3162_PCI_HOST_CONFIG_MEM_ADDR | PCI_CACHE_LINE_SIZE);
   *(volatile unsigned long *)(TC3162_PCI_CONFIG_DATA) = 0x2008;
#else  // Sam test the Ralink 2561
   // write cache line size (0x08 -- 4 DWORD of pre-fetched data ) and
   // latency timer (0x20) , these value are suggested by trendchip 
   *(volatile unsigned long *)(TC3162_PCI_CONFIG_ADDR) = (TC3162_PCI_HOST_CONFIG_MEM_ADDR | PCI_CACHE_LINE_SIZE);
   //*(volatile unsigned long *)(TC3162_PCI_CONFIG_DATA) = 0x2008;
   *(volatile unsigned long *)(TC3162_PCI_CONFIG_DATA) = 0x2008;
#endif

   /* write pci_command (0x0016) and pci_status (0xf900), these values are suggested by trendchip

      pci_command = PCI_COMMAND_MEMORY | PCI_COMMAND_MASTER | PCI_COMMAND_INVALIDATE
      
      write 1 to these pci_status register clear these status bit 
      pci_status  = PCI_STATUS_DETECTED_PARITY | PCI_STATUS_SIG_SYSTEM_ERROR | PCI_STATUS_REC_MASTER_ABORT |
		    PCI_STATUS_REC_TARGET_ABORT |  PCI_STATUS_SIG_TARGET_ABORT |  PCI_STATUS_PARITY
   */
   *(volatile unsigned long *)(TC3162_PCI_CONFIG_ADDR) = (TC3162_PCI_HOST_CONFIG_MEM_ADDR | PCI_COMMAND);
   *(volatile unsigned long *)(TC3162_PCI_CONFIG_DATA) = 0xf9000016;

    
   // Enable slot1 bus master and memory space access
   // TC3162_PCI_SLOT1_CONFIG_MEM_ADDR
	#if 1 
   *(volatile unsigned long *)(TC3162_PCI_CONFIG_ADDR) = (TC3162_PCI_SLOT1_CONFIG_MEM_ADDR | PCI_COMMAND);
   *(volatile unsigned long *)(TC3162_PCI_CONFIG_DATA) = 0x06;
	#endif

   // write slot1 cache line size (0x00) and latency timer (0x40) for RaLink wireless driveri or
   // write slot1 cache line size (0x00) and latency timer (0x28) for Inprocomm wireless driver
#if 1
   *(volatile unsigned long *)(TC3162_PCI_CONFIG_ADDR) = (TC3162_PCI_SLOT1_CONFIG_MEM_ADDR | PCI_CACHE_LINE_SIZE);

   // Ralink  
   *(volatile unsigned long *)(TC3162_PCI_CONFIG_DATA) = 0x00004000;
   // Inprocomm
   //*(volatile unsigned long *)(TC3162_PCI_CONFIG_DATA) = 0x00002800; //40
   //jj*(volatile unsigned long *)(TC3162_PCI_CONFIG_DATA) = 0x00003000;// 48
#endif

   return 0;
   }
   else // We get incorrect PCI VENDOR_ID and DEVICE_ID, this is TC3162L
	return -1;
#endif
#endif//not 7300 only
}


