/*
** $Id$
*/
/************************************************************************
 *
 *	Copyright (C) 2003 Trendchip Technologies, Corp.
 *	All Rights Reserved.
 *
 * Trendchip Confidential; Need to Know only.
 * Protected as an unpublished work.
 *
 * The computer program listings, specifications and documentation
 * herein are the property of Trendchip Technologies, Co. and shall
 * not be reproduced, copied, disclosed, or used in whole or in part
 * for any reason without the prior express written permission of
 * Trendchip Technologeis, Co.
 *
 *************************************************************************/
/*
** $Log$
*/
#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/spinlock.h>
#include <linux/interrupt.h>
#include <linux/signal.h>
#include <linux/sched.h>
#include <linux/module.h>
#include <linux/compatmac.h>
#include <linux/mm.h>
#include <linux/init.h>
#include <linux/timer.h>
#include <linux/delay.h>
#include <linux/proc_fs.h>
#include <asm/virtconvert.h>
#include <asm/tc3162/interrupts.h>
#include <asm/tc3162/tc3162.h>
#include <asm/tc3162/ledcetrl.h>
#include <asm/tc3162/tc3162l2mac.h>
#include <linux/tc3162ioctl.h> // KenWang added 2006/01/11

#define RX_BUF_LEN 	(2048 - 16 - 64 - (sizeof(struct skb_shared_info)))		

/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/
#define LAN_ST_100MB        0x01
#define LAN_ST_FULL_DUPLEX	0x02
#define LAN_ST_LINK_UP      0x04

/* auto detect 4 port switch ic */
#define SWIC_ADM6996M		1
#define SWIC_IP175C			2
#define SWIC_MARVEL6060     3
#define SWIC_RTL8305        4

/* ADMTEK6996M register */
#define ADM_PORT0_BASIC		0x01
#define ADM_PORT1_BASIC		0x03
#define ADM_PORT2_BASIC		0x05
#define ADM_PORT3_BASIC		0x07
#define ADM_PORT4_BASIC		0x08
#define ADM_PORT5_BASIC		0x09

#define ADM_CHIP_ID0		0xa0
#define ADM_CHIP_ID1		0xa1

/* Robert added */
#define MAC_START_ADDRESS (0xbfc02000)
/************************************************************************
*                            M A C R O S
*************************************************************************
*/

/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/

/************************************************************************
*              F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/
PRIVATE void tcAdmMiiStationWrite(uint32 admReg,	uint32 miiData);
PRIVATE uint32 tcAdmMiiStationRead(uint32 admReg);

static void tc3162_mac_isr(int irq, void *dev_id, struct pt_regs *regs);
static int tc3162_mac_tx(struct sk_buff *skb, struct net_device *dev);

/************************************************************************
*                        P U B L I C   D A T A
*************************************************************************
*/

int DbgSettings = DBG_TIME     |
		          //DBG_HW     |
                  //DBG_DBG      |
                  //DBG_STAT     |
                  //DBG_REG      |
                  //DBG_FUNCTION |
                  //DBG_BUF	|
                  //DBG_DES	|
                  //DBG_MSG	|
                  //DBG_ERROR    |
                  //DBG_WARN |
                  //DBG_BUFS |
                  //DBG_OUT |
                  0;


/************************************************************************
*                      E X T E R N A L   D A T A
*************************************************************************
*/

/************************************************************************
*                       P R I V A T E   D A T A
*************************************************************************
*/

/* Device data */
static struct net_device tc3162_mac_dev;
/* Reference count for device */
static int dev_count = 0;
/* Timer */
static struct timer_list eth_timer;

static macAdapter_t *mac_p = NULL;
static macAdapter_t DummyAdapter;
//__DMEM static macAdapter_t *mac_p = NULL;
//__DMEM static macAdapter_t DummyAdapter;
static macPhyLinkProfile_t enetPhyLinkProfile;
static macMemPool_t enetMemPool __attribute__ ((__aligned__(16)));
static macStat_t enetStat;

static phyDeviceList_t phyDeviceList[] = {
	{ 0x0013, "LXT972" },
	{ 0x0022, "AC101L" },
	{ 0x0243, "IP101" },
	{ 0x8201, "RTL8201" },
};

static uint8 macInitialized = 0;	/* wdl */

static uint8 def_mac_addr[] = {0x00, 0x00, 0x00, 0xaa, 0xbb, 0xcc};

uint8 swicVendor = 0;

/************************************************************************
*        F U N C T I O N   D E F I N I T I O N S
*************************************************************************
*/
#define read_reg_word(reg) 			VPint(reg)
#define write_reg_word(reg, wdata) 	VPint(reg)=wdata

#define pause(x)		mdelay(x)

PRIVATE uint32
align16Byte
(
	uint32 addr
)
{
	return (addr+15)&0xFFFFFFF0;
}

PUBLIC void
miiStationWrite(
	macAdapter_t *mac_p,
	uint32 phyReg,
	uint32 miiData
)
{
	uint32 cnt=1000;
 	macPhyCtrlReg_t      macPHYCtrl;
	macPhyWDataReg_t     macPHYWData;

  	DEBUGMSG(DBG_FUNCTION,("\r\n---->miiStationWrite"));
	// check argument
	if(mac_p==NULL){
		DEBUGMSG(DBG_ERROR,("\r\nERROR:miiStationWrite() mac_p is NULL pointer"));
	}

	if(phyReg>24){
		DEBUGMSG(DBG_ERROR,("\r\nERROR:miiStationWrite() phyReg is out of range(16)"));
	}


	macPHYWData.bits.miiwdata = miiData & 0x0000ffff;
	write_reg_word (CR_MAC_PHYWDATA, macPHYWData.value);

	macPHYCtrl.value = 0;
	macPHYCtrl.bits.phyad = mac_p->enetPhyAddr & 0x1f;
	macPHYCtrl.bits.regad = phyReg & 0x1f;
	macPHYCtrl.bits.miird = 0;
	macPHYCtrl.bits.miiwr = 1;
	write_reg_word (CR_MAC_PHYCR, macPHYCtrl.value);

	do
	{
	   pause(1);
 	   macPHYCtrl.value=read_reg_word (CR_MAC_PHYCR);
	   cnt--;
	}
	while (macPHYCtrl.bits.miiwr==1&&cnt>0);

	if(cnt==0){
		DEBUGMSG(DBG_MSG,("\r\nError:miiStationWrite Timeout!"));
	}
    DEBUGMSG(DBG_FUNCTION,("\r\n<----miiStationWrite"));
}

PUBLIC uint32
miiStationRead(
	macAdapter_t *mac_p,
	uint32 phyReg
)
{
	macPhyCtrlReg_t   macPHYCtrl;
	uint32 cnt=1000;

  	DEBUGMSG(DBG_FUNCTION,("\r\n---->miiStationRead"));
	// check argument
	if(mac_p==NULL){
		DEBUGMSG(DBG_ERROR,("\r\nERROR:miiStationWrite() mac_p is NULL pointer"));
	}

	if(phyReg>24){
		DEBUGMSG(DBG_ERROR,("\r\nERROR:miiStationWrite() phyReg is out of range(16)"));
	}

	macPHYCtrl.value = 0;
	macPHYCtrl.bits.phyad = mac_p->enetPhyAddr & 0x1f;
	macPHYCtrl.bits.regad = phyReg & 0x1f;
	macPHYCtrl.bits.miird = 1;
	macPHYCtrl.bits.miiwr = 0;

	write_reg_word (CR_MAC_PHYCR, macPHYCtrl.value);

	do
	{
    	pause(1);
	    macPHYCtrl.value=read_reg_word (CR_MAC_PHYCR);
	    cnt--;
	} while (macPHYCtrl.bits.miird==1&&cnt>0);

	if(cnt==0){
		DEBUGMSG(DBG_ERROR,("\r\nError:miiStationRead Timeout!"));
	}

  	DEBUGMSG(DBG_FUNCTION,("\r\n<----miiStationRead"));
    return macPHYCtrl.bits.miirdata & 0x0000ffff;
}

PRIVATE void
tcMiiStationWrite
(
	uint32 enetPhyAddr,
	uint32 phyReg,
	uint32 miiData
)
{
	uint32 cnt=1000;
 	macPhyCtrlReg_t      macPHYCtrl;
	macPhyWDataReg_t     macPHYWData;

  	DEBUGMSG(DBG_FUNCTION,("\r\n---->miiStationWrite"));
	// check argument
	if(mac_p==NULL){
		DEBUGMSG(DBG_ERROR,("\r\nERROR:miiStationWrite() mac_p is NULL pointer"));
	}

	if(phyReg>24){
		DEBUGMSG(DBG_ERROR,("\r\nERROR:miiStationWrite() phyReg is out of range(16)"));
	}

	macPHYWData.bits.miiwdata = miiData & 0x0000ffff;
	write_reg_word (CR_MAC_PHYWDATA, macPHYWData.value);

	macPHYCtrl.value = 0;
	macPHYCtrl.bits.phyad = enetPhyAddr & 0x1f;
	macPHYCtrl.bits.regad = phyReg & 0x1f;
	macPHYCtrl.bits.miird = 0;
	macPHYCtrl.bits.miiwr = 1;
	write_reg_word (CR_MAC_PHYCR, macPHYCtrl.value);

	do
	{
	   pause(1);
 	   macPHYCtrl.value=read_reg_word (CR_MAC_PHYCR);
	   cnt--;
	}
	while (macPHYCtrl.bits.miiwr==1&&cnt>0);

	if(cnt==0){
		DEBUGMSG(DBG_MSG,("\r\nError:miiStationWrite Timeout!"));
	}
  	DEBUGMSG(DBG_FUNCTION,("\r\n<----miiStationWrite"));
}

PRIVATE uint32
tcMiiStationRead(
	uint32 enetPhyAddr,
	uint32 phyReg
)
{
	macPhyCtrlReg_t   macPHYCtrl;
	uint32 cnt=1000;

  	DEBUGMSG(DBG_FUNCTION,("\r\n---->miiStationRead"));
	// check argument
	if(mac_p==NULL){
		DEBUGMSG(DBG_ERROR,("\r\nERROR:miiStationWrite() mac_p is NULL pointer"));
	}

	if(phyReg>24){
		DEBUGMSG(DBG_ERROR,("\r\nERROR:miiStationWrite() phyReg is out of range(16)"));
	}

	 macPHYCtrl.value = 0;
	 macPHYCtrl.bits.phyad = enetPhyAddr & 0x1f;
	 macPHYCtrl.bits.regad = phyReg & 0x1f;
	 macPHYCtrl.bits.miird = 1;
	 macPHYCtrl.bits.miiwr = 0;

	 write_reg_word (CR_MAC_PHYCR, macPHYCtrl.value);

	 do
	 {
          pause(1);
	      macPHYCtrl.value=read_reg_word (CR_MAC_PHYCR);
	      cnt--;

	 } while (macPHYCtrl.bits.miird==1&&cnt>0);

	if(cnt==0){
		DEBUGMSG(DBG_ERROR,("\r\nError:miiStationRead Timeout!"));
	}

  	DEBUGMSG(DBG_FUNCTION,("\r\n<----miiStationRead"));
        return macPHYCtrl.bits.miirdata & 0x0000ffff;
}

PRIVATE void 
tcAdmMiiStationWrite(
	uint32 admReg,
	uint32 miiData
)
{
	uint32 phyaddr;
	uint32 reg;

	phyaddr = admReg >> 5;
	reg = admReg & 0x1f;

	tcMiiStationWrite(phyaddr, reg, miiData);
}

PRIVATE uint32 
tcAdmMiiStationRead(
	uint32 admReg
)
{
	uint32 phyaddr;
	uint32 reg;
	
	phyaddr = admReg >> 5;
	reg = admReg & 0x1f;

	return tcMiiStationRead(phyaddr, reg);
}

PRIVATE void macIMRMask(void)
{
	macIntrMaskReg_t macIntrMask;
	macIntrMask.value = 0;
	macIntrMask.bits.rpkt_finish = 1;
#if 1	
    macIntrMask.bits.rpkt_ok_by_thrhold = 0;
    macIntrMask.bits.rpkt_ok_by_timeout = 0;
    macIntrMask.bits.xpkt_ok_by_thrhold = 0;
    macIntrMask.bits.xpkt_ok_by_timeout = 0;
  	macIntrMask.bits.norxbuf	 = 1;
  	macIntrMask.bits.notxbuf	 = 1;
  	macIntrMask.bits.xpkt_finish = 1;
  	macIntrMask.bits.xpkt_lost	 = 1;
  	macIntrMask.bits.rpkt_lost	 = 1;
  	macIntrMask.bits.xpkt_save   = 1;
  	macIntrMask.bits.rpkt_save   = 1;
  	macIntrMask.bits.ahb_err	 = 1;
  	macIntrMask.bits.physts_chg  = 1;
#endif	
	write_reg_word(CR_MAC_IMR, macIntrMask.value);
}

PRIVATE void macResetSwMAC(void)
{
	macCtrlReg_t	macCtrl;

	DEBUGMSG(DBG_FUNCTION,("\r\n---->macResetSwMAC"));
	// Disable DMA Enable
	macCtrl.value = 0;
	write_reg_word (CR_MAC_MACCR, macCtrl.value);
	pause(2);

	// Reset Mac
//	macCtrl.value = 0;
	macCtrl.bits.sw_rst = 1;
	write_reg_word (CR_MAC_MACCR, macCtrl.value);
	pause(2);
	DEBUGMSG(DBG_FUNCTION,("\r\n<----macResetSwMAC"));
}

PRIVATE void macSetIntTimer(void)
{
	mac_intr_timer_ctrl_reg_t macIntrTimerCtrl;

	DEBUGMSG(DBG_FUNCTION,("\r\n---->macSetIntTimer"));
    macIntrTimerCtrl.value = 0;
	write_reg_word (CR_MAC_ITC, macIntrTimerCtrl.value);
  DEBUGMSG(DBG_FUNCTION,("\r\n<----macSetIntTimer"));
}

// Assign Tx Rx Descriptor Control Registers
PRIVATE void macSetDMADescrCtrlReg(macAdapter_t *mac_p)
{
	DEBUGMSG(DBG_FUNCTION,("\r\n---->macSetDMADescrCtrlReg"));
  	write_reg_word(CR_MAC_TXR_BADR, K1_TO_PHY( mac_p->txDescrRingBaseVAddr));
  	write_reg_word(CR_MAC_RXR_BADR, K1_TO_PHY( mac_p->rxDescrRingBaseVAddr));
	
  	VPint(CR_MAC_TXR_SIZE) = mac_p->txRingSize;
  	VPint(CR_MAC_RXR_SIZE) = mac_p->rxRingSize;
  	VPint(CR_MAC_TXDESP_SIZE) = mac_p->txDescrSize;
  	VPint(CR_MAC_RXDESP_SIZE) = mac_p->rxDescrSize;
	DEBUGMSG(DBG_FUNCTION,("\r\n<----macSetDMADescrCtrlReg"));
}

PRIVATE void macSetMACCR
(
  macAdapter_t *map_p
)
{
  macCtrlReg_t macCtrl;

  DEBUGMSG(DBG_FUNCTION, ("\r\n---->macSetMACCR"));
  // clear reg
//macCtrl.value = 0x3df23;
  macCtrl.value = 0x0;
  // set flag
  macCtrl.bits.rx_broadpkt = 1;
  macCtrl.bits.rx_multipkt = 1;

  macCtrl.bits.fulltx  = 1;
  macCtrl.bits.crc_apd = 1;
  macCtrl.bits.mdc_sel = 0;
  macCtrl.bits.rcv_all = 1;

  macCtrl.bits.rx_ftl      = 1;
  macCtrl.bits.rx_runt     = 1;
  macCtrl.bits.ht_multi_en = 1;
  macCtrl.bits.rcv_en      = 0;

  macCtrl.bits.l2_mode        = 1;
  macCtrl.bits.enrx_in_halftx = 0;
  macCtrl.bits.xmt_en       	= 0;
  macCtrl.bits.crc_dis       	= 0;

  macCtrl.bits.loop_en = 0;
  macCtrl.bits.sw_rst  = 0;
  macCtrl.bits.rdma_en = 0;
  macCtrl.bits.xdma_en = 0;

  macCtrl.bits.txfifo_thr = 0x0e;
  macCtrl.bits.rx_untag   = 0;
  macCtrl.bits.drop_local = 0;

  write_reg_word (CR_MAC_MACCR, macCtrl.value);

  DEBUGMSG(DBG_FUNCTION,("\r\n<----macSetMACCR"));
}

PRIVATE void macSetMacReg
(
  macAdapter_t *mac_p
)
{
	write_reg_word(CR_MAC_MADR,mac_p->macAddr[0]<<8  | mac_p->macAddr[1]<<0);
	write_reg_word(CR_MAC_LADR,mac_p->macAddr[2]<<24 | mac_p->macAddr[3]<<16 | \
                               mac_p->macAddr[4]<<8  | mac_p->macAddr[5]<<0);
}

PRIVATE macTxDescr_t*
//__IMEM PRIVATE macTxDescr_t*
macTxRingProc
(
	macAdapter_t *mac_p
)
{
  	uint32 i;
	volatile macTxDescr_t *pTxDescp;

	spin_lock_irq(mac_p->lock);
	pTxDescp = ((macTxDescr_t*)mac_p->txDescrRingBaseVAddr) + mac_p->txUnReleasedDescp;

  	while ( mac_p->txUnReleasedBufCnt != 0 )
	{
	  	if ( (pTxDescp->tdes0.word)&(1<<31) )
		{
			spin_unlock_irq(mac_p->lock);
			return 0; // owned by MAC engine, something wrong here!
		}
		if ( mac_p->statisticOn )
		{
			if ( (pTxDescp->tdes0.word)&(1<<1) )
		  	{
		    	mac_p->macStat_p->inSilicon.txExCollisionCnt++;
		    	mac_p->macStat_p->inSilicon.txCollisionCnt++;
		    	mac_p->macStat_p->MIB_II.outErrors++;
		  	}

			if ( (pTxDescp->tdes0.word)&(1<<3|1<<2) )
			{
				mac_p->macStat_p->inSilicon.txUnderRunCnt++;
			}

	    	if ( (pTxDescp->tdes0.word)&(1<<0) )
		  	{
		     	mac_p->macStat_p->inSilicon.txLateCollisionCnt++;
		     	mac_p->macStat_p->inSilicon.txCollisionCnt++;
		     	mac_p->macStat_p->MIB_II.outErrors++;
		  	}
		} // mac_p->statisticOn

		if (pTxDescp->tdes2.buf_p == 0x0)
			printk("buf_p is null\n");
		dev_kfree_skb(pTxDescp->skb);

		pTxDescp->tdes2.buf_p = NULL;
		pTxDescp->skb = NULL;

		if ( mac_p->txUnReleasedDescp == ( mac_p->txRingSize - 1) )
		{
			mac_p->txUnReleasedDescp = 0;
		}
		else
		{
			mac_p->txUnReleasedDescp++;
		}
		mac_p->txUnReleasedBufCnt--;

		pTxDescp = ((macTxDescr_t*)mac_p->txDescrRingBaseVAddr) + mac_p->txUnReleasedDescp;

    	mac_p->macStat_p->inSilicon.txDeQueueNum++;

	} // while
	spin_unlock_irq(mac_p->lock);

	return (macTxDescr_t*)pTxDescp;
}

PUBLIC  uint32
//__IMEM PUBLIC  uint32
macTxSend
(
	struct sk_buff *skb
)
{
	volatile macTxDescr_t *currDescrp = NULL;
	uint32 length = ((ETH_ZLEN < skb->len) ? skb->len : ETH_ZLEN);
	uint8 *bufAddrp = skb->data;
  	uint32 i;

	if( skb == NULL )
	{
		DEBUGMSG(DBG_MSG,("\n\rTx a empty mbuf"));
		return 1;
	}
  
	if( skb->data == NULL )
	{
		printk("\n\rTx a empty mbuf");
		return 1;
	}

	if (mac_p->txUnReleasedBufCnt >= TX_BUF_RELEASE_THRESHOLD)
  	{
    	macTxRingProc(mac_p);
  	}

	if ( mac_p->txUnReleasedBufCnt == mac_p->txRingSize )
	{
		mac_p->macStat_p->MIB_II.outDiscards++;
		dev_kfree_skb(skb);
		return 1;
	}

	/* ----- Get the transmit descriptor ----- */
	currDescrp = ((macTxDescr_t*)mac_p->txDescrRingBaseVAddr) + mac_p->txCurrentDescp;

	if( (currDescrp->tdes0.word)&(1<<31) )
	{
		mac_p->macStat_p->MIB_II.outDiscards++;
		dev_kfree_skb(skb);
		return 1;
	}

	/* ----- Count the MIB-II ----- */
	mac_p->macStat_p->MIB_II.outOctets += length;

	if ( *bufAddrp & 0x01 )
	{
		mac_p->macStat_p->MIB_II.outMulticastPkts++;
	}
	else
	{
		mac_p->macStat_p->MIB_II.outUnicastPkts++;
	}
	/* Marked by Robert temp. */
	/*
  	ledTurnOn(LED_ETHER_ACT_STATUS);
	if (mac_p->macPhyLinkProfile_p->enetMode & LAN_ST_100MB)
		ledTurnOn(LED_ETHER_100M_ACT_STATUS);
	else
		ledTurnOn(LED_ETHER_10M_ACT_STATUS);
	*/
	spin_lock_irq(mac_p->lock);
	// tx buffer size
	currDescrp->tdes0.word = 0;
  	/* clear all except edotr, fts, lts */
  	currDescrp->tdes1.word &= (1<<31|1<<28|1<<27);
  	currDescrp->tdes1.word |= (length&0x000007ff);

  	currDescrp->tdes2.buf_p = K1_TO_PHY(skb->data);
	currDescrp->skb = skb;

	currDescrp->tdes0.word |= (1<<31);

	if( mac_p->txCurrentDescp == ( mac_p->txRingSize - 1 ))
	{
		mac_p->txCurrentDescp = 0;
	}
	else
	{
		mac_p->txCurrentDescp ++;
	}

	mac_p->txUnReleasedBufCnt++;

  	mac_p->macStat_p->inSilicon.txEnQueueNum++;
	spin_unlock_irq(mac_p->lock);

  	MAC_TX_POLL_DEMAND_(1);
	return 0;
}

PUBLIC void
macDefaultParaSet
(
	macAdapter_t *mac_p
)
{
	mac_p->rxDescrSize = MAC_RXDESCP_SIZE;
	mac_p->txDescrSize = MAC_TXDESCP_SIZE;
	mac_p->rxRingSize  = MAC_RXDESCP_NO;
	mac_p->txRingSize  = MAC_TXDESCP_NO;

  	mac_p->macStat_p->inSilicon.rxEnQueueNum = 0;
	mac_p->macStat_p->inSilicon.rxDeQueueNum = 0;
	mac_p->macStat_p->inSilicon.txEnQueueNum = 0;
	mac_p->macStat_p->inSilicon.txDeQueueNum = 0;

	return;
}

PUBLIC int
macDrvRegInit
(
	macAdapter_t *mac_p
)
{
	DEBUGMSG(DBG_FUNCTION,("\r\n---->macDrvRegInit"));

  	// ----- setup interrupt mask ---
  	//  ---- Setup HASH table------------------------------------------
  	//  -----Setup Interrupt Timer-------------------------------------
  	//  -----Setup AUTO polling timer----------------------------------
  	//  ---- Setup DMA burst and arbitration---------------------------
  	//  -----Setup DMA Descriptor Base Address Assign------------------
  	//  -----Setup MACCR-----------------------------------------------

	macResetSwMAC();

  	macIMRMask();

  	macSetIntTimer();

//	macSetAutoPollTmr();

//  macSetupDMABurstArbitration();

  	macSetDMADescrCtrlReg(mac_p);

  	macSetMACCR(mac_p);

  // --- setup MAC address ---
  	macSetMacReg(mac_p);

  	DEBUGMSG(DBG_FUNCTION,("\r\n<----macDrvRegInit"));
  	return 0;
}

PRIVATE void
macDrvDescripReset
(
	macAdapter_t *mac_p
)
{
	int i;

	macRxDescr_t * pRxDescp;
	macTxDescr_t * pTxDescp;
	struct sk_buff *skb;

  	DEBUGMSG(DBG_FUNCTION,("\r\n---->macDrvDescripReset"));

	pTxDescp = (macTxDescr_t*) mac_p->txDescrRingBaseVAddr;
	pRxDescp = (macRxDescr_t*) mac_p->rxDescrRingBaseVAddr;

	// free all un-released tx mbuf
	for (i = 0 ; i < mac_p->txRingSize ; i++)
	{
		skb = pTxDescp->skb;
		if( skb != NULL )
		{
			dev_kfree_skb(skb);
		}
		pTxDescp->tdes0.word  = DESCP_OWNED_BY_CPU_WORD;
		pTxDescp->tdes2.buf_p = NULL;
		pTxDescp->skb = NULL;
		pTxDescp++;
	}

	for (i = 0; i < mac_p->rxRingSize; i++)
	{
		pRxDescp->rdes0.word = DESCP_OWNED_BY_DMA_WORD;
		pRxDescp++;
	}

	mac_p->txCurrentDescp     = 0;
	mac_p->txUnReleasedDescp  = 0;
	mac_p->txUnReleasedBufCnt = 0;
	mac_p->rxCurrentDescp     = 0;

  	DEBUGMSG(DBG_FUNCTION,("\r\n<----macDrvDescripReset"));
}

PRIVATE uint8
macDrvDescripInit
(
	macAdapter_t *mac_p
)
{
	macRxDescr_t *pRxDescp;
  	macTxDescr_t *pTxDescp;
  	uint32 physicalAddr;
  	uint32 i;
  	struct sk_buff *skb;

  	DEBUGMSG(DBG_FUNCTION,("\r\n---->macDrvDescripInit"));
	mac_p->txDescrRingBaseVAddr = K0_TO_K1(align16Byte((uint32)&mac_p->macMemPool_p->descrPool[0]));

	mac_p->rxDescrRingBaseVAddr	= mac_p->txDescrRingBaseVAddr + (mac_p->txDescrSize * mac_p->txRingSize);

	// add function here to free all mbuf after Mac is soft-reset */
	if( macInitialized )
	{
		macDrvDescripReset( mac_p );
	}

	/* init. tx descriptor, don't allocate memory */
	pTxDescp = (macTxDescr_t*) mac_p->txDescrRingBaseVAddr;
	mac_p->txCurrDescp = pTxDescp;
	
	for ( i = 0 ; i < mac_p->txRingSize ; i++, pTxDescp++ )
	{
    	// Init descriptor
    	pTxDescp->tdes0.word = 0;
    	pTxDescp->tdes1.word = 0;
    	pTxDescp->tdes2.word = 0;

    	// Assign flag
    	pTxDescp->tdes0.bits.ownerDma = 0;  /* owned by CPU */
    	pTxDescp->tdes1.bits.sw_txtag_en = 0; /* SW Tag */
    	pTxDescp->tdes1.bits.ipcs_en     = 0; /* IP Chksum */
    	pTxDescp->tdes1.bits.vtag_en     = 0; /* VLAN Tag */
    	pTxDescp->skb = NULL;
	}

	/* init. Rx descriptor, allocate memory for each descriptor */
	pRxDescp = (macRxDescr_t*) mac_p->rxDescrRingBaseVAddr;

	for( i = 0 ; i< mac_p->rxRingSize ; i++, pRxDescp++)
	{
		// Init Descriptor
		pRxDescp->rdes0.word = 0;
		pRxDescp->rdes1.word = 0;
		pRxDescp->rdes2.word = 0;

		// Assign flag
		pRxDescp->rdes0.bits.ownerDma = 1;  /* owned by DMA */
		pRxDescp->rdes1.bits.rxBufSize = 1776;
		
		skb = dev_alloc_skb(RX_BUF_LEN);
		if( skb == NULL )
		{
			printk("tc3162_mac_descinit init fail.\n");
			return 1;
		}
		skb_reserve(skb, 2);
		//skb_reserve(skb, 8);

		pRxDescp->rdes2.buf_p = K1_TO_PHY(skb->data);
		pRxDescp->skb = skb;
	}

	mac_p->txCurrentDescp     = 0;
	mac_p->txUnReleasedDescp  = 0;
	mac_p->txUnReleasedBufCnt = 0;
	mac_p->rxCurrentDescp     = 0;

  	DEBUGMSG(DBG_FUNCTION,("\r\n<----macDrvDescripInit"));
	return 0;
}

PUBLIC void
macDrvStart
(
	void
)
{
    macCtrlReg_t macCtrl;

	macCtrl.value = VPint(CR_MAC_MACCR);
	/* enable DMA tx& rx */
	macCtrl.bits.rdma_en = 1;
	macCtrl.bits.xdma_en = 1;
	/* enable mac tx& rx */
	macCtrl.bits.rcv_en = 1;
	macCtrl.bits.xmt_en = 1;
	VPint(CR_MAC_MACCR) = macCtrl.value;
}

void
macPhyReset
(
	void
)
{

	uint32 phyAddr,mii_data,mii_addr, istate;
	macAdapter_t* mac_p;
	/* add phyLoopUp here */

	mac_p = &DummyAdapter;
	miiStationWrite( mac_p, PHY_CONTROL_REG, PHY_RESET );
	miiStationWrite( mac_p, PHY_CONTROL_REG, MIIDR_AUTO_NEGOTIATE );
}

#define IS_ETH_SKB_CB(__skb)       	(*(unsigned long *)(__skb)->cb == 0xdeadbeef)
#define MARK_ETH_SKB_CB(__skb)     	do { *(unsigned long *)(__skb)->cb = 0xdeadbeef; } while (0)
#define CLEAR_ETH_SKB_CB(__skb)    	do { *(unsigned long *)(__skb)->cb = 0x0; } while (0)

#define ETH_SKB_OFFSET				6


PUBLIC void
//__IMEM PUBLIC void
macRxRingProc
(
	struct net_device *dev
)
{
	volatile macRxDescr_t *rxDescrp;
	uint32 frameSize;
	struct sk_buff *newMbp, *oldMbp;
	uint32 procFrameCnt = 0;
	uint32 rdes0_error_check, rdes1_error_check;

	uint32 i = 0;
	uint8* pdata;

	rxDescrp = ((macRxDescr_t*)mac_p->rxDescrRingBaseVAddr) + mac_p->rxCurrentDescp;

	rdes0_error_check = (1<<27|1<<22|1<<21|1<<20|1<<19|1<<18);
  	rdes1_error_check = 0;
  	while ((rxDescrp->rdes0.word & (1<<31)) == 0) // ownerDMA for TCMAC
  	{
		// According to UNH document, the alignment error should be recorded and
	  	// the packet may be discarded if CRC error happens.
		if ( rxDescrp->rdes1.word&(1<<27) )
	  	{
	    	mac_p->macStat_p->inSilicon.rxAlignErr++;
	    	mac_p->macStat_p->inSilicon.rxDribblingErr++;
	  	}

	    if ( (rxDescrp->rdes0.word & rdes0_error_check) == 0 &&
        	(rxDescrp->rdes1.word & rdes1_error_check) == 0 )
	    {	
	    	// For removing Rx VLAN tag, we need to check if the remain length
	    	// is not smaller than minimun packet length.
	    	// rxDescrp->rdes0.bits.vlan_hit
	    	if (rxDescrp->rdes0.word & (1<<30))
	    	{
	      		frameSize = rxDescrp->rdes0.bits.rfl;
	      		frameSize = (frameSize > 60) ? frameSize : 60;
	      	}
	 		else
 	 			frameSize = rxDescrp->rdes0.bits.rfl;
 	 		if (rxDescrp->rdes1.word & (1<<28))
	        	mac_p->macStat_p->inSilicon.rx802p3FrameLengthErr++;


	      	/* lino: make VLAN friendly */
	      	if ( (frameSize < 60) || (frameSize > 1518) )
	      	{
	      		DEBUGMSG(DBG_ERROR,("\r\n frame size is out of range"));
	        	mac_p->macStat_p->inSilicon.rxEtherFrameLengthErr++;

	        	// Discard this packet & Repost this mbuf
	        	(volatile struct sk_buff *)newMbp = (volatile struct sk_buff *)(rxDescrp->skb);
	      		DEBUGMSG(DBG_ERROR,("\r\n the packet will be discarded"));
	        	goto DISCARD;
	      	}

	      	// ----- Count the MIB-II -----
		    if ( mac_p->statisticOn )
			{
	  	    	mac_p->macStat_p->MIB_II.inOctets += frameSize;
	
			    if ( (rxDescrp->rdes0.word)&(1<<16) )
			    	mac_p->macStat_p->MIB_II.inMulticastPkts++;
			    else
		        	mac_p->macStat_p->MIB_II.inUnicastPkts++;
		    }

	      	(volatile struct sk_buff *)oldMbp = (volatile struct sk_buff *)(rxDescrp->skb);
			
			newMbp = dev_alloc_skb(RX_BUF_LEN);
	        if ( !newMbp )  /* faild to allocate more mbuf -> drop this pkt */
	        {
	        	newMbp = oldMbp;
	          	mac_p->macStat_p->MIB_II.inDiscards++;
	          	goto RECVOK;
	      	}
			skb_reserve(newMbp, 2);
			//skb_reserve(newMbp, 8);

			//MARK_ETH_SKB_CB(oldMbp);
			skb_put(oldMbp, frameSize);

#ifdef CONFIG_TC3162_ADSL
			if ((oldMbp->data[12] == 0xaa) && (oldMbp->data[13] == 0xaa)) {
				MacRtsRecvPkt(oldMbp);
				dev_kfree_skb_irq(oldMbp);
			} else {
#else
			{
#endif
				oldMbp->dev = dev;
				oldMbp->ip_summed = CHECKSUM_NONE;
				oldMbp->protocol = eth_type_trans(oldMbp, dev);
				dev->last_rx = jiffies;
				netif_rx(oldMbp);
			}
DISCARD:

RECVOK:

	      	rxDescrp->rdes2.buf_p = K1_TO_PHY(newMbp->data);
			rxDescrp->skb = newMbp;
		}
		else  /* Update Error Counter and Drop it */
		{
	      	if ( mac_p->statisticOn )
    	  	{
		  		if ( rxDescrp->rdes1.word&(1<<28) )
				{
					mac_p->macStat_p->inSilicon.rx802p3FrameLengthErr++;
					mac_p->macStat_p->MIB_II.inErrors++;
				}	
				if ( rxDescrp->rdes0.word&(1<<27) )
				{
					mac_p->macStat_p->inSilicon.rxPktIPChkSumErr++;
					mac_p->macStat_p->MIB_II.inErrors++;
				}
	        	if ( rxDescrp->rdes0.word&(1<<22) )
		        {
		        	mac_p->macStat_p->inSilicon.rxCollisionErr++;
		          	mac_p->macStat_p->MIB_II.inErrors++;
		        }
		        if ( rxDescrp->rdes0.word&(1<<21) )
				{
					mac_p->macStat_p->inSilicon.rxRuntErr++;
					mac_p->macStat_p->MIB_II.inErrors++;
				}
		        if ( rxDescrp->rdes0.word&(1<<20) )
				{
					mac_p->macStat_p->inSilicon.rxLongErr++;
					mac_p->macStat_p->MIB_II.inErrors++;
				}
		        if ( rxDescrp->rdes0.word&(1<<19) )
				{
					mac_p->macStat_p->inSilicon.rxCrcErr++;
					mac_p->macStat_p->MIB_II.inErrors++;
				}
		        if ( rxDescrp->rdes0.word&(1<<18) )
				{
					mac_p->macStat_p->inSilicon.rxMiiErr++;
					mac_p->macStat_p->MIB_II.inErrors++;
				}
			} /* if mac_p->statisticOn */
		}
	
	    rxDescrp->rdes0.word |= (1<<31);
	
 		/* next descriptor*/
		if ( mac_p->rxCurrentDescp == ( mac_p->rxRingSize - 1 ))
			mac_p->rxCurrentDescp = 0;
		else
			mac_p->rxCurrentDescp++;
		
		rxDescrp = ((macRxDescr_t*)mac_p->rxDescrRingBaseVAddr) + mac_p->rxCurrentDescp;
	} /* while loop  */

    VPint(CR_MAC_RXR_SWIDX) = mac_p->rxCurrentDescp;
}

PUBLIC void
macControllerStop(macAdapter_t *mac_p)
{
	macCtrlReg_t macCtrl;

	macCtrl.value = VPint(CR_MAC_MACCR);
	/* disable DMA tx& rx */
	macCtrl.bits.rdma_en = 0;
	macCtrl.bits.xdma_en = 0;
	/* disable mac tx& rx */
	macCtrl.bits.rcv_en = 0;
	macCtrl.bits.xmt_en = 0;
	VPint(CR_MAC_MACCR) = macCtrl.value;
}

PUBLIC macAdapter_t *
macGetAdapterByChanID(void)
{
	return (macAdapter_t *)(&DummyAdapter);
}

PUBLIC void
macGetMacAddr
(
  macAdapter_t *mac_p
)
{
	uint32 i;

	for ( i = 0; i < 6; i++ )
	{
		mac_p->macAddr[i] = def_mac_addr[i];
	}
}

PRIVATE int32
macPhyLookUp(
	macAdapter_t *mac_p,
	uint32 companyId
)
{
	uint32 i;
	uint32 phyTypeSupport;

	phyTypeSupport = sizeof(phyDeviceList) / sizeof(phyDeviceList_t);
	for ( i = 0; i < phyTypeSupport; i++ ) {
		if ( companyId == phyDeviceList[i].companyId ) {
			mac_p->enetPhyId = i;
			return 1;
		}
	}
	return 0;
}

PRIVATE int32
macSearchPhyAddr(
	macAdapter_t *mac_p
)
{
	uint32 miiReg = 0;
	uint32 phyAddr;

	mac_p->enetPhyAddr = 0;
	for ( mac_p->enetPhyAddr = 0; mac_p->enetPhyAddr < 32; mac_p->enetPhyAddr++ ) {
		miiReg = miiStationRead(mac_p, PHY_ID_REG1);
		if (miiReg == 0)
			miiReg = miiStationRead(mac_p, PHY_ID_REG2);
		if ( macPhyLookUp(mac_p, miiReg) ) {
			return 0;
		}
	}

	mac_p->enetPhyAddr = 0x00000000; 

	return -1;
}

void
macPhyDisableLoopback(
	macAdapter_t *mac_p
)
{
	uint32 data;

	data = miiStationRead(mac_p,PHY_CONFIG_REG);
	data |= PHY_TP_LOOPBACK;
	miiStationWrite(mac_p, PHY_CONFIG_REG, data);
}

PUBLIC int
macSetUpPhy
(
	macAdapter_t *mac_p
)
{
	uint32 i;
	uint32 miiReg;

	macSearchPhyAddr(mac_p);

	/* ----- Hardware reset Ehernet phy chip, this address is defined by h/w engineer ----- */
	/* Marked by Robert temp. */
#if 0
	ledTurnOn(LED_LAN_RESET);
	pause(100);
	/* ----- Wait for hardware reset completed ----- */
	ledTurnOff(LED_LAN_RESET);
	pause(100);
#endif
	macPhyReset();
	pause(100);

	/* Detect 4-port switch or single port switch */
	/* detect if ADM6996M */
	if (((tcAdmMiiStationRead(ADM_CHIP_ID0) & 0xfff0) == 0x1020) &&
		(tcAdmMiiStationRead(ADM_CHIP_ID1) == 0x7)) {
		swicVendor = SWIC_ADM6996M;		    
		/* enable crossover auto detect */
		tcAdmMiiStationWrite(ADM_PORT0_BASIC, tcAdmMiiStationRead(ADM_PORT0_BASIC)|0x8000);
		tcAdmMiiStationWrite(ADM_PORT1_BASIC, tcAdmMiiStationRead(ADM_PORT1_BASIC)|0x8000);
		tcAdmMiiStationWrite(ADM_PORT2_BASIC, tcAdmMiiStationRead(ADM_PORT2_BASIC)|0x8000);
		tcAdmMiiStationWrite(ADM_PORT3_BASIC, tcAdmMiiStationRead(ADM_PORT3_BASIC)|0x8000);
	}
	/* detect if IP175C */
	if ((tcMiiStationRead(4, 2) == 0x243) && (tcMiiStationRead(4, 3) == 0xd80)) {
		swicVendor = SWIC_IP175C;
	}
	
	/* detect if RTL8305 */	
	if ((tcMiiStationRead(4, 2) == 0x1c) && (tcMiiStationRead(4, 3) == 0xc852)) {
		swicVendor = SWIC_RTL8305;
	}

	/* detect if MARVEL 88E6060 */	
	if (((tcMiiStationRead(4, 2) == 0x141) || (tcMiiStationRead(20, 2) == 0x141)) && 
	    ((tcMiiStationRead(4, 3) == 0xc87) || (tcMiiStationRead(20, 3) == 0xc87))) 
    	{
		swicVendor = SWIC_MARVEL6060;
	}

	if( swicVendor )
	{
	}
	
	/* ----- Set up Ethernet link speed, using auto negotiation ----- */
	mac_p->macPhyLinkProfile_p->linkSpeed = MIIDR_AUTO_NEGOTIATE;
	mac_p->macPhyLinkProfile_p->duplexMode = NULL;
	
	if( !swicVendor )
	{
	if ( mac_p->macPhyLinkProfile_p->linkSpeed == MIIDR_AUTO_NEGOTIATE )
	{
		miiStationWrite(mac_p, PHY_CONTROL_REG, MIIDR_AUTO_NEGOTIATE );
		pause(50); /* Timothy debug 1221, without this, negotiation is wrong... */
	}
	/* disable loopback mode */	
	macPhyDisableLoopback( mac_p );
	}
  return 0;
}

PUBLIC uint8
macInit
(
	void
)
{
	uint32 i;

	if(macInitialized)
		return 0;
	macControllerStop(mac_p);

	/* ----- Get Mac Adapter from dummy data or NDIS control block ----- */
	mac_p = macGetAdapterByChanID();
	memset(mac_p, 0x0, sizeof(macAdapter_t));

	/* ----- Assign reserved data pointer ----- */
	mac_p->macPhyLinkProfile_p = &enetPhyLinkProfile;
	mac_p->macMemPool_p = &enetMemPool;
	mac_p->macStat_p = &enetStat;

	/* ----- Set up the paramters ----- */
	macDefaultParaSet(mac_p);

	/* ----- Get the Mac address ----- */
	macGetMacAddr(mac_p);

	/* ----- Initialize Tx/Rx descriptors ----- */
	if( macInitialized == 0)
	{
		if ( macDrvDescripInit(mac_p) != 0 )
		{
			return -1;
		}
	}
	else
	{
		macDrvDescripReset( mac_p );
	}

	/* ----- Initialize the SEEQ 80225 phy chip(enet phy) ----- */
	if ( macSetUpPhy(mac_p) )
	{
		return -1;
	}

	/* ----- Initialize Registers ----- */
	macDrvRegInit(mac_p);

	mac_p->statisticOn = MAC_STATISTIC_ON;

	macInitialized = 1;
	macDrvStart();

	return 0;
}

/************************************************************************
*     A D S L   R E L A T E D    F U N C T I O N S
*************************************************************************
*/

struct sk_buff_head adsl_pkt_queue;
struct tq_struct tq_adsl;

void tc3162_emac_poll(void)
{
	tc3162_mac_isr(MAC_INT, &tc3162_mac_dev, 0);
}

/* ADSL RTS dump function */
void macSend(uint32 chanId, struct sk_buff *skb)
{
	unsigned long flags;

	local_irq_save(flags);
	__skb_queue_tail(&adsl_pkt_queue,skb);
	local_irq_restore(flags);
	schedule_task(&tq_adsl);
}

static void macSendReal(void *data)
{
	for (;;) {
		struct sk_buff *skb;

		local_irq_disable();
		skb = __skb_dequeue(&adsl_pkt_queue);
		if (skb == NULL) {
			local_irq_enable();
			break;
		}
		tc3162_mac_tx(skb, &tc3162_mac_dev);
		local_irq_enable();
	}
}

uint32 GetIpAddr(void)
{
	return 0;
}

uint8 *GetMacAddr(void)
{
	static uint8 macAddr[7];
  
  	memcpy(macAddr, def_mac_addr, 6);
	macAddr[6] = 0x0;
	return macAddr;
}

/************************************************************************
*     E T H E R N E T    D E V I C E   P R O C  D E F I N I T I O N S
*************************************************************************
*/

static int getETHState(char *buf)
{
	uint16 index = 0;

	index += sprintf(buf+index, "inOctets              = 0x%08x, ", mac_p->macStat_p->MIB_II.inOctets);
	index += sprintf(buf+index, "inUnicastPkts         = 0x%08x\n", mac_p->macStat_p->MIB_II.inUnicastPkts);

	index += sprintf(buf+index, "inMulticastPkts       = 0x%08x, ", mac_p->macStat_p->MIB_II.inMulticastPkts);
	index += sprintf(buf+index, "inDiscards            = 0x%08x\n", mac_p->macStat_p->MIB_II.inDiscards);

	index += sprintf(buf+index, "inErrors              = 0x%08x, ", mac_p->macStat_p->MIB_II.inErrors);
	index += sprintf(buf+index, "outOctets             = 0x%08x\n", mac_p->macStat_p->MIB_II.outOctets);

	index += sprintf(buf+index, "outUnicastPkts        = 0x%08x, ", mac_p->macStat_p->MIB_II.outUnicastPkts);
	index += sprintf(buf+index, "outMulticastPkts      = 0x%08x\n", mac_p->macStat_p->MIB_II.outMulticastPkts);

	index += sprintf(buf+index, "outDiscards           = 0x%08x, ", mac_p->macStat_p->MIB_II.outDiscards);
	index += sprintf(buf+index, "outErrors             = 0x%08x\n", mac_p->macStat_p->MIB_II.outErrors);

	index += sprintf(buf+index, "\n[ inSilicon Statistics Display ]\n");
	index += sprintf(buf+index, "txJabberTimeCnt       = 0x%08x  ", mac_p->macStat_p->inSilicon.txJabberTimeCnt);
	index += sprintf(buf+index, "txLossOfCarrierCnt    = 0x%08x\n", mac_p->macStat_p->inSilicon.txLossOfCarrierCnt);
	index += sprintf(buf+index, "txNoCarrierCnt        = 0x%08x  ", mac_p->macStat_p->inSilicon.txNoCarrierCnt);
	index += sprintf(buf+index, "txLateCollisionCnt    = 0x%08x\n", mac_p->macStat_p->inSilicon.txLateCollisionCnt);
	index += sprintf(buf+index, "txExCollisionCnt      = 0x%08x  ", mac_p->macStat_p->inSilicon.txExCollisionCnt);
	index += sprintf(buf+index, "txHeartbeatFailCnt    = 0x%08x\n", mac_p->macStat_p->inSilicon.txHeartbeatFailCnt);
	index += sprintf(buf+index, "txCollisionCnt        = 0x%08x  ", mac_p->macStat_p->inSilicon.txCollisionCnt);
	index += sprintf(buf+index, "txExDeferralCnt       = 0x%08x\n", mac_p->macStat_p->inSilicon.txExDeferralCnt);
	index += sprintf(buf+index, "txUnderRunCnt         = 0x%08x  ", mac_p->macStat_p->inSilicon.txUnderRunCnt);

	index += sprintf(buf+index, "rxAlignErr            = 0x%08x\n", mac_p->macStat_p->inSilicon.rxAlignErr);
	index += sprintf(buf+index, "rxDribblingErr        = 0x%08x  ", mac_p->macStat_p->inSilicon.rxDribblingErr);
	index += sprintf(buf+index, "rxSymbolErr           = 0x%08x\n", mac_p->macStat_p->inSilicon.rxSymbolErr);
	index += sprintf(buf+index, "rxMiiErr              = 0x%08x  ", mac_p->macStat_p->inSilicon.rxMiiErr);
	index += sprintf(buf+index, "rxCollisionErr        = 0x%08x\n", mac_p->macStat_p->inSilicon.rxCollisionErr);
	index += sprintf(buf+index, "rxCrcErr              = 0x%08x  ", mac_p->macStat_p->inSilicon.rxCrcErr);
	index += sprintf(buf+index, "rxEtherFrameLengthErr = 0x%08x\n", mac_p->macStat_p->inSilicon.rxEtherFrameLengthErr);
	index += sprintf(buf+index, "rx802p3FrameLengthErr = 0x%08x  ", mac_p->macStat_p->inSilicon.rx802p3FrameLengthErr);
	index += sprintf(buf+index, "rxPktIPChkSumErr      = 0x%08x\n", mac_p->macStat_p->inSilicon.rxPktIPChkSumErr);
	index += sprintf(buf+index, "rxRuntErr             = 0x%08x  ", mac_p->macStat_p->inSilicon.rxRuntErr);
	index += sprintf(buf+index, "rxLongErr             = 0x%08x\n", mac_p->macStat_p->inSilicon.rxLongErr);

	index += sprintf(buf+index, "\n[ Extra information ]\n");
	index += sprintf(buf+index, "Rx Descriptor idx     = 0x%08x  ", mac_p->rxCurrentDescp);
	index += sprintf(buf+index, "Tx Descriptor idx     = 0x%08x\n", mac_p->txCurrentDescp);
 	index += sprintf(buf+index, "Rx Enqueued cnt       = 0x%08x  ", mac_p->macStat_p->inSilicon.rxEnQueueNum);
  	index += sprintf(buf+index, "Tx Enqueued cnt       = 0x%08x\n", mac_p->macStat_p->inSilicon.txEnQueueNum);		
  	index += sprintf(buf+index, "Rx Dequeued cnt       = 0x%08x  ", mac_p->macStat_p->inSilicon.rxDeQueueNum);
  	index += sprintf(buf+index, "Tx Dequeued cnt       = 0x%08x\n", mac_p->macStat_p->inSilicon.txDeQueueNum);
  	index += sprintf(buf+index, "Tx Buf UnReleased cnt = 0x%08x  ", mac_p->txUnReleasedBufCnt);
  	index += sprintf(buf+index, "Tx Buf UnReleased idx = 0x%08x\n", mac_p->txUnReleasedDescp);

	return index;
}

static int getETHLinkSt(char *buf)
{
	uint16 index = 0;

	if (mac_p->macPhyLinkProfile_p->enetMode == 0) {
		index += sprintf(buf+index, "Down\n");
		return index;
	}

	if (mac_p->macPhyLinkProfile_p->enetMode & LAN_ST_100MB)
		index += sprintf(buf+index, "100M/");
	else 
		index += sprintf(buf+index, "10M/");
	if (mac_p->macPhyLinkProfile_p->enetMode & LAN_ST_FULL_DUPLEX)
		index += sprintf(buf+index, "Full Duplex\n");
	else 
		index += sprintf(buf+index, "Half Duplex\n");
	return index;
}

static int getETHRegDump(char *buf)
{
	uint16 index = 0;
	uint32 i;
	uint32 reg;

	index += sprintf(buf+index, "CR_MAC_ISR          (BFB50000) = 0x%08x\n", VPint(CR_MAC_ISR)); 	// --- Interrupt Status Register ---
	index += sprintf(buf+index, "CR_MAC_IMR          (BFB50004) = 0x%08x\n", VPint(CR_MAC_IMR)); 	// --- Interrupt Mask Register ---
	index += sprintf(buf+index, "CR_MAC_MADR         (BFB50008) = 0x%08x\n", VPint(CR_MAC_MADR)); 	// --- MAC Address Register [47:32] ---
	index += sprintf(buf+index, "CR_MAC_LADR         (BFB5000c) = 0x%08x\n", VPint(CR_MAC_LADR)); 	// --- MAC Address Register [31:0] ---
	index += sprintf(buf+index, "CR_MAC_TXPD         (BFB50018) = 0x%08x\n", VPint(CR_MAC_TXPD)); 	// --- Transmit Poll Demand Register ---
	index += sprintf(buf+index, "CR_MAC_RXPD         (BFB5001c) = 0x%08x\n", VPint(CR_MAC_RXPD)); 	// --- Receive Poll Demand Register ---
	index += sprintf(buf+index, "CR_MAC_TXR_BADR     (BFB50020) = 0x%08x\n", VPint(CR_MAC_TXR_BADR)); // --- Transmit Ring Base Address Register ---
	index += sprintf(buf+index, "CR_MAC_RXR_BADR     (BFB50024) = 0x%08x\n", VPint(CR_MAC_RXR_BADR)); // --- Receive Ring Base Address Register ---
	index += sprintf(buf+index, "CR_MAC_ITC          (BFB50028) = 0x%08x\n", VPint(CR_MAC_ITC)); 	// --- Interrupt Timer Control Register ---
	index += sprintf(buf+index, "CR_MAC_TXR_SIZE     (BFB5002c) = 0x%08x\n", VPint(CR_MAC_TXR_SIZE));  	// --- Automatic Polling Timer Control Register ---
	index += sprintf(buf+index, "CR_MAC_RXR_SIZE     (BFB50030) = 0x%08x\n", VPint(CR_MAC_RXR_SIZE)); 	// --- DMA Burst Length and Arbitration Control Register ---
	index += sprintf(buf+index, "CR_MAC_RXR_SWIDX    (BFB50034) = 0x%08x\n", VPint(CR_MAC_RXR_SWIDX)); 	// --- DMA Burst Length and Arbitration Control Register ---
	index += sprintf(buf+index, "CR_MAC_TXDESP_SIZE  (BFB50038) = 0x%08x\n", VPint(CR_MAC_TXDESP_SIZE)); // --- Transmit Address Register ---
	index += sprintf(buf+index, "CR_MAC_RXDESP_SIZE  (BFB5003c) = 0x%08x\n", VPint(CR_MAC_RXDESP_SIZE)); // --- Receive Address Register ---
	index += sprintf(buf+index, "CR_MAC_PRIORITY_CFG (BFB50050) = 0x%08x\n", VPint(CR_MAC_PRIORITY_CFG)); // --- Receive Address Register ---
	index += sprintf(buf+index, "CR_MAC_VLAN_CFG     (BFB50054) = 0x%08x\n", VPint(CR_MAC_VLAN_CFG)); // --- Receive Address Register ---
	index += sprintf(buf+index, "CR_MAC_TOS0_CFG     (BFB50058) = 0x%08x\n", VPint(CR_MAC_TOS0_CFG)); // --- Receive Address Register ---
	index += sprintf(buf+index, "CR_MAC_TOS1_CFG     (BFB5005c) = 0x%08x\n", VPint(CR_MAC_TOS1_CFG)); // --- Receive Address Register ---
	index += sprintf(buf+index, "CR_MAC_TOS2_CFG     (BFB50060) = 0x%08x\n", VPint(CR_MAC_TOS2_CFG)); // --- Receive Address Register ---
	index += sprintf(buf+index, "CR_MAC_TOS3_CFG     (BFB50064) = 0x%08x\n", VPint(CR_MAC_TOS3_CFG)); // --- Receive Address Register ---
	index += sprintf(buf+index, "CR_MAC_TCP_CFG      (BFB50068) = 0x%08x\n", VPint(CR_MAC_TCP_CFG)); // --- Receive Address Register ---
	index += sprintf(buf+index, "CR_MAC_SWTAG_CFG    (BFB5006c) = 0x%08x\n", VPint(CR_MAC_SWTAG_CFG)); // --- Receive Address Register ---
	index += sprintf(buf+index, "CR_MAC_PMBL_CYC_NUM (BFB50070) = 0x%08x\n", VPint(CR_MAC_PMBL_CYC_NUM)); // --- Receive Address Register ---
	index += sprintf(buf+index, "CR_MAC_FCTL_CYC_NUM (BFB50074) = 0x%08x\n", VPint(CR_MAC_FCTL_CYC_NUM)); // --- Receive Address Register ---
	index += sprintf(buf+index, "CR_MAC_JAM_CYC_NUM  (BFB50078) = 0x%08x\n", VPint(CR_MAC_JAM_CYC_NUM)); // --- Receive Address Register ---
	index += sprintf(buf+index, "CR_MAC_DEFER_VAL    (BFB5007c) = 0x%08x\n", VPint(CR_MAC_DEFER_VAL)); // --- Receive Address Register ---
	index += sprintf(buf+index, "CR_MAC_RANDOM_POLY  (BFB50080) = 0x%08x\n", VPint(CR_MAC_RANDOM_POLY)); // --- Receive Address Register ---
	index += sprintf(buf+index, "CR_MAC_MACCR        (BFB50088) = 0x%08x\n", VPint(CR_MAC_MACCR)); 	// --- MAC Control Register ---
	index += sprintf(buf+index, "CR_MAC_MACSR        (BFB5008c) = 0x%08x\n", VPint(CR_MAC_MACSR)); 	// --- MAC Status Register ---
	index += sprintf(buf+index, "CR_MAC_PHYCR        (BFB50090) = 0x%08x\n", VPint(CR_MAC_PHYCR)); 	// --- PHY Control Register ---
	index += sprintf(buf+index, "CR_MAC_PHYWDATA     (BFB50094) = 0x%08x\n", VPint(CR_MAC_PHYWDATA)); // --- PHY Write Data Register ---
	index += sprintf(buf+index, "CR_MAC_FCR          (BFB50098) = 0x%08x\n", VPint(CR_MAC_FCR)); 	// --- Flow Control Register ---
	index += sprintf(buf+index, "CR_MAC_BPR          (BFB5009c) = 0x%08x\n", VPint(CR_MAC_BPR)); 	// --- Back Pressure Register ---
	index += sprintf(buf+index, "CR_MAC_TM           (BFB500cc) = 0x%08x\n", VPint(CR_MAC_TM)); 		// --- Test Mode Register ---
	index += sprintf(buf+index, "CR_MAC_XMPG_CNT     (BFB500dc) = 0x%08x\n", VPint(CR_MAC_XMPG_CNT)); // --- XM and PG Counter Register ---
	index += sprintf(buf+index, "CR_MAC_RUNT_TLCC_CNT(BFB500e0) = 0x%08x\n", VPint(CR_MAC_RUNT_TLCC_CNT)); 	// --- Runt and Late Collision Packet Counter Register ---
	index += sprintf(buf+index, "CR_MAC_RCRC_RLONG_CNT(BFB500e4) = 0x%08x\n", VPint(CR_MAC_RCRC_RLONG_CNT)); 	// --- CRC and Long Packet Counter Register ---
	index += sprintf(buf+index, "CR_MAC_RLOSS_RCOL_CNT(BFB500e8) = 0x%08x\n", VPint(CR_MAC_RLOSS_RCOL_CNT)); 	// --- Receive Packet Loss and Receive Collision Counter Register ---
	index += sprintf(buf+index, "CR_MAC_BROADCAST_CNT(BFB500ec) = 0x%08x\n", VPint(CR_MAC_BROADCAST_CNT)); 	// --- Receive Broadcast Counter Register ---
	index += sprintf(buf+index, "CR_MAC_MULTICAST_CNT(BFB500f0) = 0x%08x\n", VPint(CR_MAC_MULTICAST_CNT)); 	// --- Receive Multicast Counter Register ---
	index += sprintf(buf+index, "CR_MAC_RX_CNT       (BFB500f4) = 0x%08x\n", VPint(CR_MAC_RX_CNT)); 	// --- Receive Good Packet Counter Register ---
	index += sprintf(buf+index, "CR_MAC_TX_CNT       (BFB500f8) = 0x%08x\n", VPint(CR_MAC_TX_CNT)); 	// --- Transmit Good Packet Counter Register ---

	index += sprintf(buf+index, "[PHY REG] PHYADDR=%d\n", mac_p->enetPhyId);
	for (i = 0; i <= 8; i++) {
   		reg = miiStationRead(mac_p, i);
		index += sprintf(buf+index, "PHY reg%d=0x%08x\n", i, reg);
	}

   	reg = miiStationRead(mac_p, 10);
	index += sprintf(buf+index, "PHY reg%d=0x%08x\n", 10, reg);
 
	for (i = 16; i <= 20; i++) {
   		reg = miiStationRead(mac_p, i);
		index += sprintf(buf+index, "PHY reg%d=0x%08x\n", i, reg);
	}

   	reg = miiStationRead(mac_p, 30);
	index += sprintf(buf+index, "PHY reg%d=0x%08x\n", 30, reg);

	return index;
}

extern struct proc_dir_entry *proc_tc3162;

static int eth_stats_read_proc(char *buf, char **start, off_t off, int count,
                 int *eof, void *data)
{
	int len = getETHState(buf);
	if (len <= off+count) 
		*eof = 1;
	*start = buf + off;
	len -= off;
	if (len>count) 
		len = count;
	if (len<0) 
		len = 0;
	return len;
}

static int eth_stats_write_proc(struct file *file, const char *buffer,
	unsigned long count, void *data)
{
	char val_string[32];
	unsigned long val;


	if (count > sizeof(val_string) - 1)
		return -EINVAL;

	if (copy_from_user(val_string, buffer, count))
		return -EFAULT;

	val_string[count] = '\0';

	memset(&mac_p->macStat_p->MIB_II, 0, sizeof(macMIB_II_t));
	memset(&mac_p->macStat_p->inSilicon, 0, sizeof(inSiliconStat_t));

	return count;
}


int eth_link_st_proc(char *buf, char **start, off_t off, int count,
                 int *eof, void *data)
{
	int len = getETHLinkSt(buf);
	if (len <= off+count) 
		*eof = 1;
	*start = buf + off;
	len -= off;
	if (len>count) 
		len = count;
	if (len<0) 
		len = 0;
	return len;
}

int eth_reg_dump_proc(char *buf, char **start, off_t off, int count,
                 int *eof, void *data)
{
	int len = getETHRegDump(buf);
	if (len <= off+count) 
		*eof = 1;
	*start = buf + off;
	len -= off;
	if (len>count) 
		len = count;
	if (len<0) 
		len = 0;
	return len;
}

/* Monitor the status of link, re-do link initialization if necessary. */
static void tc3162_mac_link_status(unsigned long dummy)
{
	struct net_device *dev = (struct net_device *)dummy;
	uint32 status = 0;
    macCtrlReg_t macCtrl;

	if (mac_p == NULL)
		return;

	if( swicVendor ) {
		mac_p->macPhyLinkProfile_p->enetMode =  LAN_ST_100MB | LAN_ST_FULL_DUPLEX | LAN_ST_LINK_UP ;
	} else {
		mac_p->macPhyLinkProfile_p->enetMode = 0;
		status = miiStationRead(mac_p, PHY_CONTROL_REG);

		/* ----- Check reset bit & power down bit ----- */
		if ( status & (PHY_RESET | PHY_POWER_DOWN) ) 
		{
			return;
		}

		status = miiStationRead(mac_p, PHY_STATUS_REG);

		if ( status & PHY_AN_COMPLETE ) {
			if ( mac_p->macPhyLinkProfile_p->ANCompFlag != 1 ) {
				mac_p->macPhyLinkProfile_p->ANCompFlag = 1;
				miiStationWrite(mac_p, PHY_CONTROL_REG, PHY_AN_ENABLE);
			}
		}
		if ( status & PHY_LINK_STATUS ) {
			mac_p->macPhyLinkProfile_p->enetMode |= LAN_ST_LINK_UP;
			/* ----- Read remote capability register for link speed ----- */
			status = miiStationRead(mac_p, PHY_REMOTE_CAP_REG);
			if ( status & PHY_100_BASE_TX_FD ) {
				mac_p->macPhyLinkProfile_p->enetMode |= (LAN_ST_FULL_DUPLEX | LAN_ST_100MB);
			} else if ( status & PHY_100_BASE_TX_HD ) {
				mac_p->macPhyLinkProfile_p->enetMode |= (LAN_ST_100MB);
			} else if ( status & PHY_10_BASE_T_FD ) {
				mac_p->macPhyLinkProfile_p->enetMode |= (LAN_ST_FULL_DUPLEX);
			} else if ( (status & PHY_10_BASE_T_HD) == 0 ) {
				mac_p->macPhyLinkProfile_p->enetMode = 0;
			}
		} /* if status & PHY_LINK_STATUS */
	}

	/* lino: according to enet phy link mode to adjust mac full duplex mode or not */
	macCtrl.value = VPint(CR_MAC_MACCR);
    macCtrl.bits.fulltx = (mac_p->macPhyLinkProfile_p->enetMode & LAN_ST_FULL_DUPLEX) ? 1 : 0;
    VPint(CR_MAC_MACCR) = macCtrl.value;

	/* Schedule for the next time */
  	mod_timer(&eth_timer, jiffies + (HZ / 2)); /* Check roughly every 0.5 seconds */	
}

/* Starting up the ethernet device */
static int tc3162_mac_open(struct net_device *dev)
{
  	if (dev == NULL)
    	return(-EIO);

  	MOD_INC_USE_COUNT;
  	printk("%s: starting interface.\n", dev->name);

  	macInit();

  	/* Schedule timer for monitoring link status */
  	init_timer(&eth_timer);
  	eth_timer.function = tc3162_mac_link_status;
  	eth_timer.data = (unsigned long)dev;
  	mod_timer(&eth_timer, jiffies + (HZ * 2));

  	netif_start_queue(dev);

  	return(0);
}

/* Stopping the ethernet device */
static int tc3162_mac_close(struct net_device *dev)
{
	printk("TC3162 MAC close\n");
  	if (dev == NULL)
    	return(-EIO);

  	/* Kill timer */
  	del_timer_sync(&eth_timer);

  	/* Stop the transmit queue */
  	netif_stop_queue(dev);

  	MOD_DEC_USE_COUNT;

  	return(0);
}

/* Setup multicast list */
static void tc3162_mac_set_multicast_list(struct net_device *dev)
{
	return; /* Do nothing */
}

/* Setting customized mac address */
static int tc3162_mac_set_macaddr(struct net_device *dev, void *p)
{
	struct sockaddr *addr=p;
	/* Check if given address is valid ethernet MAC address */
  	if (!is_valid_ether_addr(addr))
    	return(-EIO);

	/* Save the customize mac address */
  	//memcpy(&(dev->dev_addr[0]), addr+2, 6);
  	//memcpy(def_mac_addr, addr+2, 6);
  	memcpy(dev->dev_addr,addr->sa_data, dev->addr_len);
  	memcpy(def_mac_addr, addr->sa_data, dev->addr_len);

	
	macGetMacAddr(mac_p);
  	macSetMacReg(mac_p);
}

/* Transmit a packet */
static int tc3162_mac_tx(struct sk_buff *skb, struct net_device *dev)
//__IMEM static int tc3162_mac_tx(struct sk_buff *skb, struct net_device *dev)
{
	uint32 flags;

	save_flags(flags);
	cli();
	macTxSend(skb);
	restore_flags(flags);

	return 0;
}

/* Get the stats information */
static struct net_device_stats *tc3162_mac_stats(struct net_device *dev)
{
	struct net_device_stats *stats;

	if ((dev == NULL) || (mac_p == NULL) || (macInitialized == 0))
		return NULL;

	stats = &mac_p->stats;
	stats->rx_packets = mac_p->macStat_p->MIB_II.inUnicastPkts;
	stats->tx_packets = mac_p->macStat_p->MIB_II.inUnicastPkts;
	stats->rx_bytes = mac_p->macStat_p->MIB_II.inOctets;
	stats->tx_bytes = mac_p->macStat_p->MIB_II.outOctets;
	stats->rx_dropped = mac_p->macStat_p->MIB_II.inDiscards;
	stats->tx_dropped = mac_p->macStat_p->MIB_II.outDiscards;
	stats->multicast = mac_p->macStat_p->MIB_II.inMulticastPkts;
	stats->rx_errors = mac_p->macStat_p->MIB_II.inErrors;
	stats->tx_errors = mac_p->macStat_p->MIB_II.outErrors;
	stats->collisions = mac_p->macStat_p->inSilicon.txExCollisionCnt + 
			mac_p->macStat_p->inSilicon.txCollisionCnt +
			mac_p->macStat_p->inSilicon.rxCollisionErr;

	return stats;
}

/*
 * soft_reset: Reset ICPLUS175C
 */
static void soft_reset(void) 
{
   tcMiiStationWrite(30, 0,0x175C);
   mdelay(5);
   tcMiiStationRead(30, 0);
}

// KenWang added 2005/12/27
#define FILL_TAG_VLAN_MASK(vid,a) TAG_VLAN_MASK[vid/2]=TAG_VLAN_MASK[vid/2]|(vid%2==1?a<<8:a)

/* Handling ioctl call */
static int tc3162_mac_ioctl(struct net_device *dev, struct ifreq *ifr, int cmd)
{
    unsigned int reg=0; 
    unsigned int p0=0,p1=0,p2=0,p3=0,readback=0;
    unsigned int maskvalue=0,v1=0,v2=0,v3=0,v4=0;
    struct IP175C_vlan_ioctl_args *ioc = (struct IP175C_vlan_ioctl_args *) ifr->ifr_data;
    unsigned int b1=0, b2=0, b3=0, b4=0;
    /* KenWang added 2005/12/27 -- begin -- */
    unsigned int VID[8];              
    // this is map to TAG_VLAN_MASK registers of ICPLUS175C (30,1) - (30,8)
    unsigned short TAG_VLAN_MASK[8]; 
    int i;
    char index = 0;
    /* KenWang added 2005/12/27 -- end -- */
    //printk("Enter 3162 ioctl\n");
    
    p0=0x01 & ioc->portmask;
    p1=0x02 & ioc->portmask;
    p2=0x04 & ioc->portmask;
    p3=0x08 & ioc->portmask;
    
    /* we can add sub-command in ifr_data if we need to in the future */
    switch (ioc->request_cmd)
    {
    case Set_VLAN_Port:
         printk("Add VLAN Port\n");
         readback=tcMiiStationRead(29, 19);
               
         if(readback==0x9f9f)
         {
            tcMiiStationWrite(29, 19,0x8182);
            if(p0 != 0)
              {
                readback=ioc->portmask << 8;
                readback=readback | tcMiiStationRead(29, 19);
                tcMiiStationWrite(29, 19,readback);
              } 
            if(p1 != 0)
              {
                readback=ioc->portmask;
                readback=readback | tcMiiStationRead(29, 19);
                tcMiiStationWrite(29, 19,readback);
                } 
         }
          else
          { 
            if(p0 != 0)
              {
                readback=ioc->portmask << 8;
                readback=readback | tcMiiStationRead(29, 19);
                tcMiiStationWrite(29, 19,readback);
              } 
            if(p1 != 0)
              {
                readback=ioc->portmask;
                readback=readback | tcMiiStationRead(29, 19);
                tcMiiStationWrite(29, 19,readback);
              }               
           }

          readback=tcMiiStationRead(29, 20);
         if(readback==0x9f9f)
         {
            tcMiiStationWrite(29, 20,0x8488);
            if(p2 != 0)
              {
                readback=ioc->portmask << 8;
                readback=readback | tcMiiStationRead(29, 20);
                tcMiiStationWrite(29, 20,readback);
              } 
            if(p3 != 0)
              {
                readback=ioc->portmask;
                readback=readback | tcMiiStationRead(29, 20);
                tcMiiStationWrite(29, 20,readback);
              } 
         }
          else
          { 
            if(p2 != 0)
              {
                readback=ioc->portmask << 8;
                readback=readback | tcMiiStationRead(29, 20);
                tcMiiStationWrite(29, 20,readback);
              } 
            if(p3 != 0)
              {
                readback=ioc->portmask;
                readback=readback | tcMiiStationRead(29, 20);
                tcMiiStationWrite(29, 20,readback);
              }               
           }

                printk("After setup:\n");
                readback=tcMiiStationRead(29, 19);
                printk("readback 29.19==%x\n",readback);
                readback=tcMiiStationRead(29, 20);
                printk("readback 29.20==%x\n",readback);
         break;
       
    case Enable_IGMP_Snoop:
        printk("Enable IGMP Snooping ether switch configuration\n");
        tcMiiStationWrite(29, 19,0x9f9f);
        tcMiiStationWrite(29, 20,0x9f9f);
        tcMiiStationWrite(29, 23,0x07c2 );
	tcMiiStationWrite(29, 24,0x0002);
	tcMiiStationWrite(29, 25,0x0003);
	tcMiiStationWrite(29, 26,0x0004 );
        tcMiiStationWrite(29, 27,0x0005 );
	tcMiiStationWrite(29, 28,0x0001 );
	tcMiiStationWrite(29, 30,0x0003 );
	tcMiiStationWrite(30, 2,0x2221 );
	tcMiiStationWrite(30, 1,0x3000 );
	tcMiiStationWrite(30, 3,0x2824 );
	tcMiiStationWrite(30, 9,0x0f89 );
	soft_reset();
        break;
   case VODX: 
	//printk("..VODX..\n");
	tcMiiStationWrite(29, 19,0x9f9f);
        tcMiiStationWrite(29, 20,0x9f9f);
	/* KenWang modified 2005/12/27 for add user specified VID -- begin -- */
	memset(TAG_VLAN_MASK, 0x2020, sizeof(TAG_VLAN_MASK));

	VID[0] = ioc->VID[0] ? ioc->VID[0] : 2;
	VID[1] = ioc->VID[1] ? ioc->VID[1] : 3;
	VID[2] = ioc->VID[2] ? ioc->VID[2] : 4;
	VID[3] = ioc->VID[3] ? ioc->VID[3] : 5;
	VID[4] = ioc->VID[4] ? ioc->VID[4] : 0;
	VID[5] = ioc->VID[5] ? ioc->VID[5] : 0;
	VID[6] = ioc->VID[6] ? ioc->VID[6] : 0;
	VID[7] = ioc->VID[7] ? ioc->VID[7] : 0;
	if(ioc->tagmask != 0) {
		//printk("tagmask: %04x, VID: %d %d %d %d\n", ioc->tagmask, VID[0], VID[1], VID[2], VID[3]);
		//maskvalue = 0x0402 | ((ioc->tagmask&0xf)<<11) | (~(ioc->tagmask&0xf)&0xf<<6);
		maskvalue = ioc->tagmask&0xffff;
		//printk("maskvalue: %04x\n", maskvalue);
		tcMiiStationWrite(29, 23, maskvalue);
	}
	else {
		//printk("VID: %d %d %d %d\n", VID[0], VID[1], VID[2], VID[3]);
		tcMiiStationWrite(29, 23,0x07c2 );
	}
	/* KenWang modified 2005/12/27 for add user specified VID -- end -- */

	// b1b2b3b4
	b1 = 0xff & ioc->portmask;
	b2 = 0xff & (ioc->portmask >> 8);
	b3 = 0xff & (ioc->portmask >> 16);
	b4 = 0xff & (ioc->portmask >> 24);
	//printk("b1 %02X %02X %02X %02X\n",b1,b2,b3,b4);

	if(b1){
		for(i=0; i<8; i++) {
			p0 = b1&(1<<i);
			if(p0) {
				printk("VLAN%d -> p%d\n", VID[i<4?0:1], i%4);
				tcMiiStationWrite(29, 24+(i%4), VID[i<4?0:1]);
				index = VID[i<4?0:1] & 0xf;
				FILL_TAG_VLAN_MASK((int)index,1<<(i%4));
			}
		}
	}
	if(b2){
		for(i=0; i<8; i++) {
			p0 = b2&(1<<i);
			if(p0) {
				printk("VLAN%d -> p%d\n", VID[i<4?2:3], i%4);
				tcMiiStationWrite(29, 24+(i%4), VID[i<4?2:3]);
				index = VID[i<4?2:3] & 0xf;
				FILL_TAG_VLAN_MASK((int)index,1<<(i%4));
			}
		}
	}
	if(b3){
		for(i=0; i<8; i++) {
			p0 = b3&(1<<i);
			if(p0) {
				printk("VLAN%d -> p%d\n", VID[i<4?4:5], i%4);
				tcMiiStationWrite(29, 24+(i%4), VID[i<4?4:5]);
				index = VID[i<4?4:5] & 0xf;
				FILL_TAG_VLAN_MASK((int)index,1<<(i%4));
			}
		}
	}
	if(b4){
		for(i=0; i<8; i++) {
			p0 = b3&(1<<i);
			if(p0) {
				printk("VLAN%d -> p%d\n", VID[i<4?6:7], i%4);
				tcMiiStationWrite(29, 24+(i%4), VID[i<4?6:7]);
				index = VID[i<4?6:7] & 0xf;
				FILL_TAG_VLAN_MASK((int)index,1<<(i%4));
			}
		}
	}
	tcMiiStationWrite(29, 28,0x0001 ); // 4; wan no use
	tcMiiStationWrite(29, 30,0x0002 ); // 5; mii
	tcMiiStationWrite(30, 1,0x3000 );

	/* KenWang added 2005/12/27 add vlan tag -- begin -- */
	for(index = 0; index < 8; index++) {
		//printk("TAG_VLAN_MASK%d: %x\n", index+1, TAG_VLAN_MASK[(int)index]);
		tcMiiStationWrite(30, index+1, TAG_VLAN_MASK[(int)index]);
	}
	/* KenWang added 2005/12/27 add vlan tag -- end -- */
#if 0
	v1=b1&0xf; v2=b2&0xf; v3=b3&0xf; v4=b4&0xf;

	maskvalue=0x2020|(v2<<8)|v1;
	printk("mask1 %04X\n", maskvalue);
	tcMiiStationWrite(30, 2, maskvalue);

	maskvalue=0x2020|(v4<<8)|v3;
	printk("mask2 %04X\n", maskvalue);
	tcMiiStationWrite(30, 3, maskvalue);
#endif
	//tcMiiStationWrite(30, 9,0x0f89 );
	tcMiiStationWrite(30, 9,0x1089 );//Ken Wang modified 12/12/05 for VOD case that port 1,2 transfer data , port 3 can ping.
	soft_reset();

	break; /* VODX */
    case Disable_VLAN:
        printk("Disable VLAN \n");
        tcMiiStationWrite(29, 19,0x9f9f);
        tcMiiStationWrite(29, 20,0x9f9f);
        tcMiiStationWrite(29, 23,0x0000);
	tcMiiStationWrite(30, 9,0x0000);
	tcMiiStationWrite(29, 30,0x0000);
	tcMiiStationWrite(30, 1,0x3f3f); 
        tcMiiStationWrite(30, 2,0x3f3f); 
    //    soft_reset();
        break;
    case Get_Lan_Mii:
        printk("Read LAN MII \n");
        reg= tcMiiStationRead(ioc->phyaddr, ioc->regid);
        if (copy_to_user(&(ioc->regvalue), &reg, sizeof(unsigned int)))
        printk("Error copy to user\n");
	break;
    case Set_Lan_Mii:
        printk("Set LAN MII \n");
        tcMiiStationWrite(ioc->phyaddr, ioc->regid,ioc->regvalue);
        break;
    case Switch_Reset:
        printk("Switch soft-Reset\n");
        soft_reset();
        break;
    default:
	   return -ENOIOCTLCMD;
        break;   
    }
 
  return 0;
}

uint8 int_eth;

static void tc3162_mac_isr(int irq, void *dev_id, struct pt_regs *regs)
//__IMEM static void tc3162_mac_isr(int irq, void *dev_id, struct pt_regs *regs)
{
	macIntrStatusReg_t isr;

	uint32 i;
	macRxDescr_t* rxDescp;
	struct net_device *dev = (struct net_device *) dev_id;

	if (dev_id != (void *)&tc3162_mac_dev) 
		return;

	int_eth = (VPint(CR_INTC_IMR) & (1 << MAC_INT));
	if (int_eth)
		VPint(CR_INTC_IMR) &= ~(1<<MAC_INT);

	// -------Read MACCR--------------------------------
  	isr.value = VPint(CR_MAC_ISR);

#if 1
	//if (DbgSettings & DBG_HW)
	if (1)
	{
		if (isr.bits.physts_chg ) printk("\r\n--->PHYSTS_CHG  ");
		if (isr.bits.ahb_err    ) printk("\r\n--->AHB_ERR     ");
		if (isr.bits.rpkt_lost  ) printk("\r\n--->RPKT_LOST   ");
    	//if (isr.bits.rpkt_save  ) printk("\r\n--->RPKT_SAVE   ");
		if (isr.bits.xpkt_lost  ) printk("\r\n--->XPKT_LOST   ");
	    //if (isr.bits.xpkt_finish) printk("\r\n--->XPKT_FINISH ");
	    //if (isr.bits.notxbuf    ) printk("\r\n--->NOTXBUF     ");
	    //if (isr.bits.xpkt_save  ) printk("\r\n--->XPKT_SAVE   ");
		if (isr.bits.norxbuf    ) printk("\r\n--->NORXBUF     ");
	    //if (isr.bits.rpkt_finish) printk("\r\n--->RPKT_FINISH ");
	}
#endif

	// ----------Packet Received----------------------

	if (isr.bits.rpkt_finish)
	{
		/* Marked by Robert, temp. */
		/*
    	ledTurnOn(LED_ETHER_ACT_STATUS);
		if (mac_p->macPhyLinkProfile_p->enetMode & LAN_ST_100MB)
			ledTurnOn(LED_ETHER_100M_ACT_STATUS);
		else
			ledTurnOn(LED_ETHER_10M_ACT_STATUS);
			*/
    	macRxRingProc(dev);
	}

SKIP:
	int_eth = (VPint(CR_INTC_IMR) & (1 << MAC_INT));
	if (int_eth)
		VPint(CR_INTC_IMR) |= (1<<MAC_INT);

    MAC_RX_POLL_DEMAND_(1);

	return;
}

static int tc3162_mac_start(struct net_device *dev)
{
	if (dev == NULL)
    	goto failed;

  	/* Ethernet device initialization */
  	ether_setup(dev);

  	/* Hook up with handlers */
  	dev->get_stats = tc3162_mac_stats;
  	dev->hard_start_xmit = tc3162_mac_tx;
  	dev->open = tc3162_mac_open;
  	dev->stop = tc3162_mac_close;
  	dev->set_multicast_list = tc3162_mac_set_multicast_list;
  	dev->do_ioctl = tc3162_mac_ioctl;
  	dev->set_mac_address = tc3162_mac_set_macaddr;

  	dev->tx_queue_len = MAC_TXDESCP_NO; 
  	dev->flags &= ~IFF_MULTICAST;
  	dev->flags |= IFF_DEBUG;

  	return(0);

failed:
  	return(-EIO);
}


static void inline
ahb_err_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	printk("CR_AHB_ABEM=%x\n", VPint(CR_AHB_ABEM));
	printk("CR_AHB_ABEA=%x\n", VPint(CR_AHB_ABEA));
	printk("CR_AHB_AACS=%x\n", VPint(CR_AHB_AACS));
}


static int __init tc3162_mac_init(void)
{
  	struct net_device *dev = &tc3162_mac_dev;
  	int err = 0, i;
	struct proc_dir_entry *eth_proc;

  	printk("Ethernet driver for TC3162L2 version 1.0");
  	dev->priv = NULL;

  	if (dev_count != 0) {
    	err = -EIO;
    	goto failed;
  	}

  	SET_MODULE_OWNER(&tc3162_mac_dev);
	/* Copy our mac address here */

  if (((*(char *) (MAC_START_ADDRESS + 32) == 0x00) ||
       (*(char *) (MAC_START_ADDRESS + 32) == 0x02)) &&
      ((*(char *) (MAC_START_ADDRESS + 32) != 0x00) ||
       (*(char *) (MAC_START_ADDRESS + 33) != 0x00) ||
       (*(char *) (MAC_START_ADDRESS + 34) != 0x00) ||
       (*(char *) (MAC_START_ADDRESS + 35) != 0x00) ||
       (*(char *) (MAC_START_ADDRESS + 36) != 0x00) ||
       (*(char *) (MAC_START_ADDRESS + 37) != 0x00)))
    {
      def_mac_addr[0] = *(char *) (MAC_START_ADDRESS + 32);
      def_mac_addr[1] = *(char *) (MAC_START_ADDRESS + 33);
      def_mac_addr[2] = *(char *) (MAC_START_ADDRESS + 34);
      def_mac_addr[3] = *(char *) (MAC_START_ADDRESS + 35);
      def_mac_addr[4] = *(char *) (MAC_START_ADDRESS + 36);
      def_mac_addr[5] = *(char *) (MAC_START_ADDRESS + 37);
    }
  printk ("mac: %02x:%02x:%02x:%02x:%02x:%02x\n",
	  def_mac_addr[0],
	  def_mac_addr[1],
	  def_mac_addr[2], def_mac_addr[3], def_mac_addr[4], def_mac_addr[5]);

	
  	/* Get a device name: normally it'll be eth0 */
  	if ((err = dev_alloc_name(&tc3162_mac_dev, "eth%d")) < 0)
    	goto failed;

  	/* Register ISR */
  	if ((err = request_irq(MAC_INT, tc3162_mac_isr, 0, "ethernet", 
									&tc3162_mac_dev)) != 0) {
    	err = -EIO;
    	goto failed;
  	} else {
    	printk(" on %s (IRQ: %d)", dev->name, MAC_INT);
  	}

	if (request_irq(ARBITER_ERR_INT, ahb_err_interrupt, 0, "AHB_ERR", 
									&tc3162_mac_dev) != 0) {
		printk("can not allocate IRQ.\n");
	}
	VPint(CR_INTC_IMR) &= ~(1<<ARBITER_ERR_INT);

  	memcpy(&(dev->dev_addr[0]), def_mac_addr, 6);

  	printk(", (mac %02x", dev->dev_addr[0]);
  	for (i = 1; i < 6; i++)
    	printk(":%02x", dev->dev_addr[i]);
  	printk(")\n");

  	/* Point to driver initialization routine and register the device with kernel */
  	dev->init = tc3162_mac_start;
  	if (register_netdev(dev) != 0) {
    	err = -EIO;
    	goto failed;
  	}

  	printk("%s: driver installation completed.\n", dev->name);

  	/* ethernet related stats */
	eth_proc = create_proc_entry("eth_stats", 0, proc_tc3162);
	eth_proc->read_proc = eth_stats_read_proc;
	eth_proc->write_proc = eth_stats_write_proc;

  	create_proc_read_entry("eth_link_st", 0, proc_tc3162, eth_link_st_proc, NULL);
  	create_proc_read_entry("eth_reg", 0, proc_tc3162, eth_reg_dump_proc, NULL);

	skb_queue_head_init(&adsl_pkt_queue);
	INIT_TQUEUE(&tq_adsl, macSendReal, NULL);

  	dev_count++;
  	return(0);

failed:
  	if (dev->priv != NULL) {
    	kfree(dev->priv);
    	dev->priv = NULL;
  	}
  	printk("%s: driver installation failed.\n", dev->name);
  	return(err);
}

static void __exit tc3162_mac_exit(void)
{
	if (dev_count != 0) {
    	/* Turn off IRQ and stop receive/transmit */

    	/* Unregister the device and ISR */
    	free_irq(MAC_INT, &tc3162_mac_dev);
    	unregister_netdev(&tc3162_mac_dev);

    	/* Free up memory */
    	if (tc3162_mac_dev.priv != NULL) {
      		kfree(tc3162_mac_dev.priv);
      		tc3162_mac_dev.priv = NULL;
    	}

    	remove_proc_entry("eth_stats", proc_tc3162);
    	remove_proc_entry("eth_link_st", proc_tc3162);
    	remove_proc_entry("eth_reg", proc_tc3162);
  	} 
}

/* Register startup/shutdown routines */
module_init(tc3162_mac_init);
module_exit(tc3162_mac_exit);

