/*
** $Id$
*/
/************************************************************************
 *
 *	Copyright (C) 2003 Trendchip Technologies, Corp.
 *	All Rights Reserved.
 *
 * Trendchip Confidential; Need to Know only.
 * Protected as an unpublished work.
 *
 * The computer program listings, specifications and documentation
 * herein are the property of Trendchip Technologies, Co. and shall
 * not be reproduced, copied, disclosed, or used in whole or in part
 * for any reason without the prior express written permission of
 * Trendchip Technologeis, Co.
 *
 *************************************************************************/
/*
** $Log$
*/
#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/atmdev.h>
#include <linux/spinlock.h>
#include <linux/interrupt.h>
#include <linux/signal.h>
#include <linux/sched.h>
#include <linux/module.h>
#include <linux/compatmac.h>
#include <linux/mm.h>
#include <linux/init.h>
#include <linux/delay.h>
#include <linux/timer.h>
#include <asm/addrspace.h>
#include <asm/virtconvert.h>
#include <asm/tc3162/tc3162.h>	
#include <asm/tc3162/tsarm.h>	
#include <asm/tc3162/TCIfSetQuery.h>	
//#include <asm/tc3162/ledcetrl.h>	

#define NARG		40

#define FALSE		0
#define TRUE		1

typedef	struct {
	char *name;		
	int	(*func)(int argc,char *argv[],void *p);
	int	flags;	
	int	argcmin;
	char *argc_errmsg;	
} cmds_t;
extern void proc_tc3162_init(void);
extern int RtsCiCmd(int argc, char *argv[], void *p);
extern int dmt_ci_cmd(int argc, char *argv[], void *p);
extern int dmt2_ci_cmd(int argc, char *argv[], void *p);
extern int hw_ci_cmd(int argc, char *argv[], void *p);
extern int SwCiCmd(int argc, char *argv[], void *p);
extern int ghs_ci_cmd(int argc, char *argv[], void *p);
extern int TCIfCiCmd(int argc, char *argv[], void *p);

static int doWan(int argc, char *argv[], void *p);

static int doAdsl(int argc, char *argv[], void *p);
static int doAdslReset(int argc, char *argv[], void *p); 
static int doAdslOpenCmd(int argc, char *argv[], void *p); 
static int doAdslOpen(int argc, char *argv[], void *p); 
static int doAdslClose(int argc, char *argv[], void *p); 
static int doAdslGlite(int argc, char *argv[], void *p);
static int doAdslT1413(int argc, char *argv[], void *p);
static int doAdslGdmt(int argc, char *argv[], void *p);
static int doAdslMultimode(int argc, char *argv[], void *p);
static int doAdslAdsl2(int argc, char *argv[], void *p);//Jason_931110
static int doAdslAdsl2plus(int argc, char *argv[], void *p);//Jason_931110
static int doAdslFwVersion(int argc, char *argv[], void *p);
static int doAdslStatus(int argc, char *argv[], void *p);
static int doAdslChanndata(int argc, char *argv[], void *p);
static int doAdslPerfdata(int argc, char *argv[], void *p);
static int doAdslLinedata(int argc, char *argv[], void *p);
static int doAdslLinedataFar(int argc, char *argv[], void *p);
static int doAdslLinedataNear(int argc, char *argv[], void *p);
static int doAdslOpmode(int argc, char *argv[], void *p);

int cmd_reg_add(char *cmd_name, cmds_t *cmds_p);
int subcmd(const cmds_t tab[], int argc, char *argv[], void *p);

/* Command lookup and branch tables */
const cmds_t Cmds[] = {
	{"wan",			doWan,			0x12, 	0,	NULL},
	NULL,		
};

static const cmds_t wanCmds[] = {
	{"adsl",		doAdsl, 		0x10,	0,	NULL},
	{"rts",			RtsCiCmd, 		0x10,	0,	NULL},
	{"dmt", 		dmt_ci_cmd,		0x10,	0,	NULL},
	{"dmt2", 		dmt2_ci_cmd,	0x10,	0,	NULL},
	{"hwdmt", 		hw_ci_cmd,		0x10,	0,	NULL},
	{"swdmt", 		SwCiCmd,		0x10,	0,	NULL},
	{"ghs",			ghs_ci_cmd,		0x10,	0,	NULL},
	{"tcif",		TCIfCiCmd,		0x10,	0,	NULL},	
	NULL,
}; 

static const cmds_t adslCmds[] = {
	{"reset",		doAdslReset,	0x02,  	0,  NULL},
	{"open",		doAdslOpen,		0x02,  	0,  NULL},
	{"opencmd",		doAdslOpenCmd,	0x12,  	0,  NULL},
	{"fwversion",	doAdslFwVersion,0x02,  	0,  NULL},
	{"status",		doAdslStatus,	0x02,  	0,  NULL},
	{"channdata", 	doAdslChanndata,0x02,  	0,  NULL},
	{"perfdata", 	doAdslPerfdata,	0x02,  	0,  NULL},
	{"linedata", 	doAdslLinedata,	0x12,  	0,  NULL},
	{"opmode", 		doAdslOpmode,	0x02,  	0,  NULL},
	{"close",		doAdslClose,	0x02,  	0,  NULL},
	NULL,
};

static const cmds_t adslOpenCmds[] =
{
	{"glite",		doAdslGlite,	0x02,   0,  NULL},
	{"t1.413",		doAdslT1413,	0x02,	0,  NULL},
   	{"gdmt",		doAdslGdmt,		0x02,   0,  NULL},
   	{"multimode",	doAdslMultimode,0x02,  	0,  NULL},
   	{"adsl2",		doAdslAdsl2,	0x02,   0,  NULL},//Jason_931110
   	{"adsl2plus",	doAdslAdsl2plus,0x02, 	0,  NULL},//Jason_931110
    NULL,
};

static const cmds_t adslLinedataCmds[] =
{
	{"far",			doAdslLinedataFar,	0x02,   0,  NULL},
	{"near",		doAdslLinedataNear,	0x02,   0,  NULL},
	NULL,
};

int cmd_reg_add(char *cmd_name, cmds_t *cmds_p)
{
	return 0;
}

/* Call a subcommand based on the first token in an already-parsed line */
int subcmd	(
	const cmds_t tab[], int argc, char *argv[], void *p
)
{
	register cmds_t *cmdp;
	char **pargv;
	int found = 0;
	int i;

	/* Strip off first token and pass rest of line to subcommand */
	if (argc < 2) {
		if (argc < 1)
			printk("SUBCMD - Don't know what to do?\n");
		else 
			goto print_out_cmds;
			//printk("\"%s\" - takes at least one argument\n",argv[0]);
		return -1;
	}
	argc--;
	argv++;
	for(cmdp = tab;cmdp->name != NULL;cmdp++){
		if(strncmp(argv[0],cmdp->name,strlen(argv[0])) == 0){
			found = 1;
			break;
		}
	}
	if(!found){
		char buf[66];
		
print_out_cmds:
		printk("valid subcommands:\n");
		memset(buf,' ',sizeof(buf));
		buf[64] = '\n';
		buf[65] = '\0';
		for(i=0,cmdp = tab;cmdp->name != NULL;cmdp++){
			strncpy(&buf[i*16],cmdp->name,strlen(cmdp->name));
			if(i == 3){
				printk(buf);
				memset(buf,' ',sizeof(buf));
				buf[64] = '\n';
				buf[65] = '\0';
			}
			i = (i+1)%4;
		}
		if(i != 0)
			printk(buf);
		return -1;
	}
	if(argc <= cmdp->argcmin){
		if(cmdp->argc_errmsg != NULL)
			printk("Usage: %s\n",cmdp->argc_errmsg);
		return -1;
	}
	return (*cmdp->func)(argc,argv,p);
}

static char *
stringparse(line)
char *line;
{
	register char *cp = line;
	unsigned long num;

	while ( *line != '\0' && *line != '\"' ) {
		if ( *line == '\\' ) {
			line++;
			switch ( *line++ ) {
			case 'n':
				*cp++ = '\n';
				break;
			case 't':
				*cp++ = '\t';
				break;
			case 'v':
				*cp++ = '\v';
				break;
			case 'b':
				*cp++ = '\b';
				break;
			case 'r':
				*cp++ = '\r';
				break;
			case 'f':
				*cp++ = '\f';
				break;
			case 'a':
				*cp++ = '\a';
				break;
			case '\\':
				*cp++ = '\\';
				break;
			case '\?':
				*cp++ = '\?';
				break;
			case '\'':
				*cp++ = '\'';
				break;
			case '\"':
				*cp++ = '\"';
				break;
			case 'x':
				num = simple_strtoul( --line, &line, 16 );
				*cp++ = (char) num;
				break;
			case '0':
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
				num = simple_strtoul( --line, &line, 8 );
				*cp++ = (char) num;
				break;
			case '\0':
				return NULL;
			default:
				*cp++ = *(line - 1);
				break;
			};
		} else {
			*cp++ = *line++;
		}
	}

	if ( *line == '\"' )
		line++; 	/* skip final quote */
	*cp = '\0';		/* terminate string */
	return line;
}

/* replace terminating end of line marker(s) with null */
void 
rip(s)
register char *s;
{
	register char *cp;

	if((cp = strchr(s,'\n')) != NULL)
		*cp = '\0';
	if((cp = strchr(s,'\r')) != NULL)
		*cp = '\0';
}

int 
cmdparse(
	cmds_t cmds[], char *line, void *p
)
{
	cmds_t *cmdp;
	char *argv[NARG],*cp;
	char **pargv;
	int argc,i;

	/* Remove cr/lf */
	rip(line);

	for(argc = 0;argc < NARG;argc++)
		argv[argc] = NULL;

	for(argc = 0;argc < NARG;){
		register int qflag = FALSE;

		/* Skip leading white space */
		while(*line == ' ' || *line == '\t')
			line++;
		if(*line == '\0')
			break;
		/* return if comment character first non-white */
		if ( argc == 0  &&  *line == '#' )
			return 0;
		/* Check for quoted token */
		if(*line == '"'){
			line++;	/* Suppress quote */
			qflag = TRUE;
		}
		argv[argc++] = line;	/* Beginning of token */

		if(qflag){
			/* Find terminating delimiter */
			if((line = stringparse(line)) == NULL){
				return -1;
			}
		} else {
			/* Find space or tab. If not present,
			 * then we've already found the last
			 * token.
			 */
			if((cp = strchr(line,' ')) == NULL
			 && (cp = strchr(line,'\t')) == NULL){
				break;
			}
			*cp++ = '\0';
			line = cp;
		}
	}
	if (argc < 1) {		/* empty command line */
		argc = 1;
		argv[0] = "";
	}
	/* Look up command in table; prefix matches are OK */
	for(cmdp = cmds;cmdp->name != NULL;cmdp++){
		if(strncmp(argv[0],cmdp->name,strlen(argv[0])) == 0) {
			break;
		}
	}
	if(cmdp->name == NULL) {
		if(cmdp->argc_errmsg != NULL)
			printk("%s\n",cmdp->argc_errmsg);
		return -1;
	}
	if(argc < cmdp->argcmin) {
		/* Insufficient arguments */
		printk("Usage: %s\n",cmdp->argc_errmsg);
		return -1;
	}
	if(cmdp->func == NULL)
		return 0;
	return (*cmdp->func)(argc,argv,p);
}

int doWan(int argc, char *argv[], void *p)
{
	return subcmd(wanCmds, argc, argv, p);
}

/************************************************************************
*		     A D S L   D A T A   S T R U C T U R E
*************************************************************************
*/

static struct timer_list adsl_timer;

typedef struct {
	uint16          actualBitrateNearInt;
	uint16          actualBitrateNearFast;
	uint16          actualBitrateFarInt;
	uint16          actualBitrateFarFast;
} T_adslChannelOperData;

typedef struct {
    uint16          FecNotInterleaved;
    uint16          FecInterleaved;
    uint16          CrcNotInterleaved;
    uint16          CrcInterleaved;
    uint16          HecNotInterleaved;
    uint16          HecInterleaved;
} T_adsl_def_counters;

typedef struct {
    T_adsl_def_counters     near_end;
    T_adsl_def_counters     far_end;
} T_adsl_def_counter_set;

typedef struct {
        uint16          relCapacityOccupationDnstr;
        signed short     noiseMarginDnstr;//Jason_931028
        signed short     outputPowerUpstr;//Jason_931028
        uint16           attenuationDnstr;//Jason_931028
} T_NTadslNearEndLineOperData;

typedef struct {
        uint16          relCapacityOccupationUpstr;
        signed short     noiseMarginUpstr;//Jason_931028
        signed short     outputPowerDnstr;//Jason_931028
        uint16           attenuationUpstr;//Jason_931028
        uint8           carrierLoad[256]; //Jason_931130
} T_NTadslFarEndLineOperData;
//Brad disable 20050502
/*
int 
getXdslSpeed(void)
{
	uint8 modemst;
	T_adslChannelOperData adslChannelOperData;

	TCIfQuery(ADSL_QUERY_STATUS, &modemst, NULL);		   //Jason_920527
	if (modemst != ADSL_MODEM_STATE_UP)
		return 0;

	TCIfQuery(ADSL_QUERY_CH_OP_DATA, &adslChannelOperData, NULL); 		 //Jason_920527
	return (adslChannelOperData.actualBitrateFarFast+adslChannelOperData.actualBitrateFarInt);
}    
*/
//Brad disable 20050502
/*
void adslTimer(unsigned long data)
{
	static uint8 doAdslInit = 0;
	uint8 modemst;

	//TCIfQuery(ADSL_QUERY_STATUS, &modemst, NULL);	
	  
	switch (modemst) {
		case ADSL_MODEM_STATE_DOWN:
			doAdslInit = 0;
			ledTurnOn(LED_DSL_IDLE_STATUS);
			if (ledGetMode(LED_DSL_IDLE_STATUS) == LED_MODE_NOT_USED)
				ledTurnOff(LED_DSL_STATUS);
			ledTurnOff(LED_DSL_ACT_STATUS);
			ledTurnOff(LED_DSL_NOACT_STATUS);
			ledTurnOff(LED_PPP_STATUS);
			ledTurnOff(LED_PPP_ACT_STATUS);
			ledTurnOff(LED_PPP_NOACT_STATUS);
			break;
		case ADSL_MODEM_STATE_WAIT_INIT:
			doAdslInit = 0;
			ledTurnOn(LED_DSL_IDLE_STATUS);
			if (ledGetMode(LED_DSL_IDLE_STATUS) == LED_MODE_NOT_USED)
				ledTurnOff(LED_DSL_STATUS);
			ledTurnOff(LED_DSL_ACT_STATUS);
			ledTurnOff(LED_DSL_NOACT_STATUS);
			ledTurnOff(LED_PPP_STATUS);
			ledTurnOff(LED_PPP_ACT_STATUS);
			ledTurnOff(LED_PPP_NOACT_STATUS);
			break;
		case ADSL_MODEM_STATE_INIT:
			doAdslInit = 0;
			ledTurnOn(LED_DSL_TRAINING_STATUS);
			break;
		case ADSL_MODEM_STATE_UP:
			if (!doAdslInit) {
				doAdslInit = 1;
				ledTurnOn(LED_DSL_SHOWTIME_STATUS);
				ledTurnOn(LED_DSL_NOACT_STATUS);
			}
			break;
	}
	mod_timer(&adsl_timer, jiffies + (HZ / 2)); 

}
*/

/************************************************************************
*		     A D S L   C I   C O M M A N D
*************************************************************************
*/
static void adslReset(void)
{
	uint32 mask = 0xFFFFFFFE;               
	uint16 flag = 1;

	TCIfSet(ADSL_SET_DMT_CLOSE, NULL, NULL);
	TCIfSet(ADSL_SET_R_SOFT_RESET, &flag, NULL); 
   	TCIfSet(ADSL_SET_INT_MASK0, &mask, NULL );       
}

int doAdsl(int argc, char *argv[], void *p)
{
	return subcmd(adslCmds, argc, argv, p);
}

int doAdslReset(int argc, char *argv[], void *p)
{
	adslReset();
	return 0;
}

int doAdslOpenCmd(int argc, char *argv[], void *p)
{
	return subcmd(adslOpenCmds, argc, argv, p);
}

int doAdslOpen(int argc, char *argv[], void *p)
{
	uint8 modemst;
	uint32 mask = 0xFFFFFFFE;               
	uint16 flag = 1;

	TCIfQuery(ADSL_QUERY_STATUS, &modemst, NULL );		   
	if (modemst == ADSL_MODEM_STATE_UP) {
		printk("ADSL line is up.\n");
		return 0;
	}

	TCIfSet(ADSL_SET_R_SOFT_RESET, &flag, NULL); 
   	TCIfSet(ADSL_SET_INT_MASK0, &mask, NULL );       
	return 0;
}

int doAdslClose(int argc, char *argv[], void *p)
{
	TCIfSet(ADSL_SET_DMT_CLOSE, NULL, NULL);
	return 0;
}

int doAdslGlite(int argc, char *argv[], void *p)
{
	TCIfSet(ADSL_SET_MODE_LITE, NULL, NULL);
	adslReset();
	return 0;
}

int doAdslT1413(int argc, char *argv[], void *p)
{
	TCIfSet(ADSL_SET_MODE_ANSI, NULL, NULL);
	adslReset();
	return 0;
}

int doAdslGdmt(int argc, char *argv[], void *p)
{
	TCIfSet(ADSL_SET_MODE_A43, NULL, NULL);
	adslReset();
	return 0;
}

int doAdslMultimode(int argc, char *argv[], void *p)
{
	TCIfSet(ADSL_SET_MODE_MULTIMODE, NULL, NULL);
	adslReset();
	return 0;
}

int doAdslAdsl2(int argc, char *argv[], void *p)
{
	TCIfSet(ADSL_SET_MODE_ADSL2, NULL, NULL);
	adslReset();
	return 0;
}

int doAdslAdsl2plus(int argc, char *argv[], void *p)
{
	TCIfSet(ADSL_SET_MODE_ADSL2PLUS, NULL, NULL);
	adslReset();
	return 0;
}

int
getADSLFwVer(char *modemCode)
{
	uint16	index=0;

	uint8 fw_ver_r, fw_ver_p, fw_rev, tdm_ver, fdm_ver, annex_afe, b_ver;
	uint8   major;
	uint8   minor;

	TCIfQuery(ADSL_QUERY_HW_VER, &major, &minor);           //Jason_920529
   	TCIfQuery(ADSL_QUERY_FW_VER, &fw_ver_r, &fw_ver_p);     //Jason_920527
   	TCIfQuery(ADSL_QUERY_FW_REV, &fw_rev, &annex_afe);      //Jason_920527
   	TCIfQuery(ADSL_QUERY_TDM_FDM_VER, &tdm_ver, &fdm_ver);  //Jason_920527
   	TCIfQuery(ADSL_QUERY_BUILD_VERSION, &b_ver, NULL ); //Jason_931108
        
	index = sprintf( modemCode," FwVer:");
	index += sprintf( modemCode+index, "%d.", fw_ver_r);
	index += sprintf( modemCode+index, "%d.", fw_ver_p);
	index += sprintf( modemCode+index, "%d.", fw_rev);
	index += sprintf( modemCode+index, "%d_", b_ver);
	index += sprintf( modemCode+index, "%c_", (annex_afe & 0x02)?'A':'B');
	index += sprintf( modemCode+index, "%s", (annex_afe & 0x01)?"TC":"SI");
	index += sprintf( modemCode+index, " HwVer:T%d.F%d_%d.%d\n", tdm_ver, fdm_ver, major, minor);
	return index;
}

int doAdslFwVersion(int argc, char *argv[], void *p)
{
	char fwver[80];

	getADSLFwVer(fwver);
	printk("%s", fwver);
	return 0;
}

int doAdslStatus(int argc, char *argv[], void *p)
{
	uint8 modemst;

	TCIfQuery(ADSL_QUERY_STATUS, &modemst, NULL );		   
	if (modemst == ADSL_MODEM_STATE_DOWN) 
		printk("ADSL link status: down\n");
	else if (modemst == ADSL_MODEM_STATE_WAIT_INIT) 
		printk("ADSL link status: wait for init\n");
	else if (modemst == ADSL_MODEM_STATE_INIT) 
		printk("ADSL link status: initializing\n");
	else if (modemst == ADSL_MODEM_STATE_UP) 
		printk("ADSL link status: up\n");
	else 
		printk("ADSL link status: unknown\n");
	return 0;
}

int doAdslChanndata(int argc, char *argv[], void *p)
{
	uint8 modemst;
	T_adslChannelOperData adslChannelOperData;

	TCIfQuery(ADSL_QUERY_STATUS, &modemst, NULL );		   
	if (modemst != ADSL_MODEM_STATE_UP) 
		return 0;

	TCIfQuery(ADSL_QUERY_CH_OP_DATA, &adslChannelOperData, NULL); 		 

	printk("near-end interleaved channel bit rate: %u kbps\n",
				adslChannelOperData.actualBitrateNearInt);
	printk("near-end fast channel bit rate: %u kbps\n",
				adslChannelOperData.actualBitrateNearFast);
	printk("far-end interleaved channel bit rate: %u kbps\n",
				adslChannelOperData.actualBitrateFarInt);
	printk("far-end fast channel bit rate: %u kbps\n",
				adslChannelOperData.actualBitrateFarFast);
	return 0;
}

int doAdslPerfdata(int argc, char *argv[], void *p)
{
	uint8 modemst;
    T_adsl_def_counter_set adsl_def_counter_set;

	TCIfQuery(ADSL_QUERY_STATUS, &modemst, NULL );		   
	if (modemst != ADSL_MODEM_STATE_UP) 
		return 0;

    TCIfQuery(ADSL_QUERY_DEFECT_CNT, &adsl_def_counter_set, NULL);

	printk("near-end FEC error fast: %u\n",
						adsl_def_counter_set.near_end.FecNotInterleaved);
	printk("near-end FEC error interleaved: %u\n",
						adsl_def_counter_set.near_end.FecInterleaved);
	printk("near-end CRC error fast: %u\n",
						adsl_def_counter_set.near_end.CrcNotInterleaved);
	printk("near-end CRC error interleaved: %u\n",
						adsl_def_counter_set.near_end.CrcInterleaved);
	printk("near-end HEC error fast: %u\n",
						adsl_def_counter_set.near_end.HecNotInterleaved);
	printk("near-end HEC error interleaved: %u\n",
						adsl_def_counter_set.near_end.HecInterleaved);
	printk("far-end FEC error fast: %u\n",
						adsl_def_counter_set.far_end.FecNotInterleaved);
	printk("far-end FEC error interleaved: %u\n",
						adsl_def_counter_set.far_end.FecInterleaved);
	printk("far-end CRC error fast: %u\n",
						adsl_def_counter_set.far_end.CrcNotInterleaved);
	printk("far-end CRC error interleaved: %u\n",
						adsl_def_counter_set.far_end.CrcInterleaved);
	printk("far-end HEC error fast: %u\n",
						adsl_def_counter_set.far_end.HecNotInterleaved);
	printk("far-end HEC error interleaved: %u\n",
						adsl_def_counter_set.far_end.HecInterleaved);
	return 0;
}

int doAdslLinedata(int argc, char *argv[], void *p)
{
	return subcmd(adslLinedataCmds, argc, argv, p);
}

int doAdslLinedataFar(int argc, char *argv[], void *p)
{
	uint8 modemst;
	T_NTadslFarEndLineOperData adslFarEndLineOperData;
	uint8 link_mode;
	int i, j, line_num;

	TCIfQuery(ADSL_QUERY_STATUS, &modemst, NULL );		   
	if (modemst != ADSL_MODEM_STATE_UP) 
		return 0;

	TCIfQuery(ADSL_QUERY_FAR_OP_DATA, &adslFarEndLineOperData, NULL);    

	printk("relative capacity occupation: %u%% \n",
			adslFarEndLineOperData.relCapacityOccupationUpstr);
	printk("noise margin upstream: %d.%d db\n", 
			(adslFarEndLineOperData.noiseMarginUpstr/10),(adslFarEndLineOperData.noiseMarginUpstr%10)); //Jason_931028
	printk("output power downstream: %d.%d dbm\n", 
			(adslFarEndLineOperData.outputPowerDnstr/10),(adslFarEndLineOperData.outputPowerDnstr%10)); //Jason_931028
	printk("attenuation upstream: %d.%d db\n", 
			(adslFarEndLineOperData.attenuationUpstr/10),(adslFarEndLineOperData.attenuationUpstr%10));//Jason_931028

	printk("\nBit table:\n");
	TCIfQuery(ADSL_QUERY_MODE, &link_mode, NULL);//Jason_931130
	if( link_mode == ME_CMD_ADSL2PLUS )    		 //Jason_931130
		line_num = 16;
	else
		line_num = 8;
	printk("carrier load: number of bits per tone\n");
	for(i=0; i<line_num; i++) {	//Jason_931130
		printk("tone %3d-%3d: ",i*32, i*32+31);
		for (j=0; j<16; j++) {
			printk("%1x", adslFarEndLineOperData.carrierLoad[i*16+j] & 0xF);
			printk("%1x ", (adslFarEndLineOperData.carrierLoad[i*16+j] & 0xF0) >> 4);
		}
		printk("\n");
	}

	return 0;
}

int doAdslLinedataNear(int argc, char *argv[], void *p)
{
	uint8 modemst;
	T_NTadslNearEndLineOperData adslNearEndLineOperData;

	TCIfQuery(ADSL_QUERY_STATUS, &modemst, NULL );		   
	if (modemst != ADSL_MODEM_STATE_UP) 
		return 0;

	TCIfQuery(ADSL_QUERY_NEAR_OP_DATA, &adslNearEndLineOperData, NULL);    

	printk("relative capacity occupation: %u%% \n",
			adslNearEndLineOperData.relCapacityOccupationDnstr);
	printk("noise margin downstream: %d.%d db\n", 
			adslNearEndLineOperData.noiseMarginDnstr/10, adslNearEndLineOperData.noiseMarginDnstr%10);
	printk("output power upstream: %d.%01d dbm\n", 
			adslNearEndLineOperData.outputPowerUpstr/10, adslNearEndLineOperData.outputPowerUpstr%10);
	printk("attenuation downstream: %d.%01d db\n", 
			adslNearEndLineOperData.attenuationDnstr/10, adslNearEndLineOperData.attenuationDnstr%10);

	return 0;
}

int doAdslOpmode(int argc, char *argv[], void *p)
{
	uint8 modemst;
	uint8 mode;

	TCIfQuery(ADSL_QUERY_STATUS, &modemst, NULL );		   
	if (modemst != ADSL_MODEM_STATE_UP) 
		return 0;

	TCIfQuery(ADSL_QUERY_MODE, &mode, NULL);
	printk("Opmode: ");
	switch(mode) {
		case ME_CMD_ADSL_ANSI:
	    	printk("ANSI T1.413\n");
			break;
		case ME_CMD_ADSL_OPEN_GDMT:
	   		printk("ITU G.992.1(G.DMT)\n");
			break;
		case ME_CMD_ADSL_OPEN_GLITE:
       		printk("ITU G.992.2(G.Lite)\n");
			break;
		case ME_CMD_ADSL2:
       		printk("ITU G.992.3(ADSL2)\n");
			break;
		case ME_CMD_ADSL2PLUS:
       		printk("ITU G.992.5(ADSL2PLUS)\n");
		break;
	}
	return 0;
}
/************************************************************************
*		     A D S L   E V E N T   C A L L   F U N C T I O N  
*************************************************************************
*/

#define DSL_EVENT_UP			0x1
#define DSL_EVENT_DOWN			0x2
#define DSL_EVENT_TRAIN			0x3

uint32 dslEventNotify ( uint32 event, uint32 param1, uint32 param2)
{
	switch (event) {
		case DSL_EVENT_DOWN:
			VPint(CR_INTC_IMR) |= (1<<MAC_INT);
			break;
		case DSL_EVENT_UP:
			VPint(CR_INTC_IMR) |= (1<<MAC_INT);
			break;
		case DSL_EVENT_TRAIN:
			VPint(CR_INTC_IMR) &= ~(1<<MAC_INT);
			break;
	}

	return 0;
}

/************************************************************************
*		     A D S L   P R O C  D E F I N I T I O N S
*************************************************************************
*/

extern struct proc_dir_entry *proc_tc3162;

int
getADSLState(char *stateADSL, uint8 long_fmt)
{
	uint16	index=0;

	uint8 modemst;
	T_adslChannelOperData adslChannelOperData;
    T_adsl_def_counter_set adsl_def_counter_set;
	T_NTadslNearEndLineOperData adslNearEndLineOperData;
	T_NTadslFarEndLineOperData adslFarEndLineOperData;
	uint8 link_mode;
	uint8 mode;
	int i, j, line_num;

	TCIfQuery(ADSL_QUERY_STATUS, &modemst, NULL );		   
	if (modemst == ADSL_MODEM_STATE_DOWN) 
		index += sprintf( stateADSL+index, "ADSL link status: down\n");
	else if (modemst == ADSL_MODEM_STATE_WAIT_INIT) 
		index += sprintf( stateADSL+index, "ADSL link status: wait for init\n");
	else if (modemst == ADSL_MODEM_STATE_INIT) 
		index += sprintf( stateADSL+index, "ADSL link status: initializing\n");
	else if (modemst == ADSL_MODEM_STATE_UP) 
		index += sprintf( stateADSL+index, "ADSL link status: up\n");
	else 
		index += sprintf( stateADSL+index, "ADSL link status: unknown\n");

	if (modemst != ADSL_MODEM_STATE_UP) {
		return index;
	}

	TCIfQuery(ADSL_QUERY_CH_OP_DATA, &adslChannelOperData, NULL); 		 
	TCIfQuery(ADSL_QUERY_MODE, &mode, NULL);
    TCIfQuery(ADSL_QUERY_DEFECT_CNT, &adsl_def_counter_set, NULL);
	TCIfQuery(ADSL_QUERY_NEAR_OP_DATA, &adslNearEndLineOperData, NULL);    
	TCIfQuery(ADSL_QUERY_FAR_OP_DATA, &adslFarEndLineOperData, NULL);    

	index += sprintf(stateADSL+index, "\nOpmode: ");
	switch(mode) {
		case ME_CMD_ADSL_ANSI:
	    	index += sprintf(stateADSL+index, "ANSI T1.413\n");
			break;
		case ME_CMD_ADSL_OPEN_GDMT:
	   		index += sprintf(stateADSL+index, "ITU G.992.1(G.DMT)\n");
			break;
		case ME_CMD_ADSL_OPEN_GLITE:
       		index += sprintf(stateADSL+index, "ITU G.992.2(G.Lite)\n");
			break;
		case ME_CMD_ADSL2:
       		index += sprintf(stateADSL+index, "ITU G.992.3(ADSL2)\n");
			break;
		case ME_CMD_ADSL2PLUS:
       		index += sprintf(stateADSL+index, "ITU G.992.5(ADSL2PLUS)\n");
		break;
	}

	index += sprintf( stateADSL+index, "\n");
	index += sprintf( stateADSL+index, "near-end interleaved channel bit rate: %u kbps\n",
									adslChannelOperData.actualBitrateNearInt);
	index += sprintf( stateADSL+index, "near-end fast channel bit rate: %u kbps\n",
									adslChannelOperData.actualBitrateNearFast);
	index += sprintf( stateADSL+index, "far-end interleaved channel bit rate: %u kbps\n",
									adslChannelOperData.actualBitrateFarInt);
	index += sprintf( stateADSL+index, "far-end fast channel bit rate: %u kbps\n",
									adslChannelOperData.actualBitrateFarFast);

	index += sprintf( stateADSL+index, "\n");
	index += sprintf( stateADSL+index, "near-end FEC error fast: %u\n",
						adsl_def_counter_set.near_end.FecNotInterleaved);
	index += sprintf( stateADSL+index, "near-end FEC error interleaved: %u\n",
						adsl_def_counter_set.near_end.FecInterleaved);
	index += sprintf( stateADSL+index, "near-end CRC error fast: %u\n",
						adsl_def_counter_set.near_end.CrcNotInterleaved);
	index += sprintf( stateADSL+index, "near-end CRC error interleaved: %u\n",
						adsl_def_counter_set.near_end.CrcInterleaved);
	index += sprintf( stateADSL+index, "near-end HEC error fast: %u\n",
						adsl_def_counter_set.near_end.HecNotInterleaved);
	index += sprintf( stateADSL+index, "near-end HEC error interleaved: %u\n",
						adsl_def_counter_set.near_end.HecInterleaved);
	index += sprintf( stateADSL+index, "far-end FEC error fast: %u\n",
						adsl_def_counter_set.far_end.FecNotInterleaved);
	index += sprintf( stateADSL+index, "far-end FEC error interleaved: %u\n",
						adsl_def_counter_set.far_end.FecInterleaved);
	index += sprintf( stateADSL+index, "far-end CRC error fast: %u\n",
						adsl_def_counter_set.far_end.CrcNotInterleaved);
	index += sprintf( stateADSL+index, "far-end CRC error interleaved: %u\n",
						adsl_def_counter_set.far_end.CrcInterleaved);
	index += sprintf( stateADSL+index, "far-end HEC error fast: %u\n",
						adsl_def_counter_set.far_end.HecNotInterleaved);
	index += sprintf( stateADSL+index, "far-end HEC error interleaved: %u\n",
						adsl_def_counter_set.far_end.HecInterleaved);
	
	if (!long_fmt)
		return index;

	index += sprintf( stateADSL+index, "\nDownstream:\n");
	index += sprintf( stateADSL+index, "relative capacity occupation: %u%% \n",
			adslNearEndLineOperData.relCapacityOccupationDnstr);
	index += sprintf( stateADSL+index, "noise margin downstream: %d.%d db\n", 
			adslNearEndLineOperData.noiseMarginDnstr/10, adslNearEndLineOperData.noiseMarginDnstr%10);
	index += sprintf( stateADSL+index, "output power upstream: %d.%01d dbm\n", 
			adslNearEndLineOperData.outputPowerUpstr/10, adslNearEndLineOperData.outputPowerUpstr%10);
	index += sprintf( stateADSL+index, "attenuation downstream: %d.%01d db\n", 
			adslNearEndLineOperData.attenuationDnstr/10, adslNearEndLineOperData.attenuationDnstr%10);

	index += sprintf( stateADSL+index, "\nUpstream:\n");
	index += sprintf( stateADSL+index , "relative capacity occupation: %u%% \n",
			adslFarEndLineOperData.relCapacityOccupationUpstr);
	index += sprintf( stateADSL+index, "noise margin upstream: %d.%d db\n", 
			(adslFarEndLineOperData.noiseMarginUpstr/10),(adslFarEndLineOperData.noiseMarginUpstr%10)); //Jason_931028
	index += sprintf( stateADSL+index, "output power downstream: %d.%d dbm\n", 
			(adslFarEndLineOperData.outputPowerDnstr/10),(adslFarEndLineOperData.outputPowerDnstr%10)); //Jason_931028
	index += sprintf( stateADSL+index, "attenuation upstream: %d.%d db\n", 
			(adslFarEndLineOperData.attenuationUpstr/10),(adslFarEndLineOperData.attenuationUpstr%10));//Jason_931028

	index += sprintf( stateADSL+index, "\nBit table:\n");
	TCIfQuery(ADSL_QUERY_MODE, &link_mode, NULL);//Jason_931130
	if( link_mode == ME_CMD_ADSL2PLUS )    		 //Jason_931130
		line_num = 16;
	else
		line_num = 8;
	index += sprintf( stateADSL+index, "carrier load: number of bits per tone\n");
	for(i=0; i<line_num; i++) {	//Jason_931130
		index += sprintf( stateADSL+index, "tone %3d-%3d: ",i*32, i*32+31);
		for (j=0; j<16; j++) {
		index += sprintf( stateADSL+index, "%1x", adslFarEndLineOperData.carrierLoad[i*16+j] & 0xF);
		index += sprintf( stateADSL+index, "%1x ", (adslFarEndLineOperData.carrierLoad[i*16+j] & 0xF0) >> 4);
		}
		index += sprintf( stateADSL+index, "\n");
	}

	return index;
}

int adsl_proc_stats(char *buf, char **start, off_t off, int count,
                 int *eof, void *data)
{
	int len = getADSLState(buf, 1);
	if (len <= off+count) 
		*eof = 1;
	*start = buf + off;
	len -= off;
	if (len>count) 
		len = count;
	if (len<0) 
		len = 0;
	return len;
}

int adsl_proc_perf(char *buf, char **start, off_t off, int count,
                 int *eof, void *data)
{
	int len = getADSLState(buf, 0);
	if (len <= off+count) 
		*eof = 1;
	*start = buf + off;
	len -= off;
	if (len>count) 
		len = count;
	if (len<0) 
		len = 0;
	return len;
}

int adsl_proc_fwver(char *buf, char **start, off_t off, int count,
                 int *eof, void *data)
{
	int len = getADSLFwVer(buf);
	if (len <= off+count) 
		*eof = 1;
	*start = buf + off;
	len -= off;
	if (len>count) 
		len = count;
	if (len<0) 
		len = 0;
	return len;
}

static int adsl_cmd_read_proc(char *page, char **start, off_t off,
	int count, int *eof, void *data)
{
	int len;

	len = sprintf(page, "0\n");

	len -= off;
	*start = page + off;

	if (len > count)
		len = count;
	else
		*eof = 1;

	if (len < 0)
		len = 0;

	return len;
}

static int adsl_cmd_write_proc(struct file *file, const char *buffer,
	unsigned long count, void *data)
{
	char val_string[80];
	unsigned long val;

	if (count > sizeof(val_string) - 1)
		return -EINVAL;

	if (copy_from_user(val_string, buffer, count))
		return -EFAULT;

	val_string[count] = '\0';
        //printk("cmd_string=%s\n",val_string);
	cmdparse(Cmds, val_string, NULL);

	return count;
}

static void __init tc3162_adsl_init(void)
{
	struct proc_dir_entry *adsl_cmd_proc;
//Brad disable 20050502
	/* start adsl timer */
	//init_timer(&adsl_timer);
	//adsl_timer.expires = jiffies + (HZ * 2);
	//adsl_timer.function = adslTimer;
	//adsl_timer.data = 0x0;
	//add_timer(&adsl_timer);

        //printk("Create proc directory for ADSL debug\n");
        proc_tc3162_init();
	/* adsl related stats */
	create_proc_read_entry("adsl_stats", 0, proc_tc3162, adsl_proc_stats, NULL);
	/* adsl performance related stats */
	create_proc_read_entry("adsl_perf", 0, proc_tc3162, adsl_proc_perf, NULL);
	/* adsl firmware version */
	create_proc_read_entry("adsl_fwver", 0, proc_tc3162, adsl_proc_fwver, NULL);

	/* adsl commands */
	adsl_cmd_proc = create_proc_entry("adsl_cmd", 0, proc_tc3162);
	adsl_cmd_proc->read_proc = adsl_cmd_read_proc;
	adsl_cmd_proc->write_proc = adsl_cmd_write_proc;
}

static void __exit tc3162_adsl_exit(void)
{
	remove_proc_entry("adsl_stats", proc_tc3162);
	remove_proc_entry("adsl_cmd", proc_tc3162);
}

module_init (tc3162_adsl_init);
module_exit (tc3162_adsl_exit);

