/*
 *  linux/arch/mips/dec/time.c
 *
 *  Copyright (C) 1991, 1992, 1995  Linus Torvalds
 *  Copyright (C) 2000  Maciej W. Rozycki
 *
 * Implementations used for system timer controller configuration.
 *
 * Written by Robert Wu, Billion Co.
 * Support for TrendChip TC3162
 *
 */


#include <linux/errno.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/param.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/interrupt.h>

#include <asm/cpu.h>
#include <asm/bootinfo.h>
#include <asm/mipsregs.h>
#include <asm/io.h>
#include <asm/irq.h>

#include <linux/timex.h>
#include <asm/tc3162/tc3162.h>
#include <asm/tc3162/interrupts.h>
#include <asm/tc3162/timer.h>

extern rwlock_t xtime_lock;
extern struct task_struct *current;

/* define System Clock as 133 MHZ */
#define SYS_HCLK 133
#define ENABLE 1


#if 0 /* Show bar progress */
static char bars[4] = { '-', '\\', '|', '/' };
static int barcnt = 0;
#endif

static inline int timer_intr_valid(void) 
{
  unsigned long stat;

  /* Read cleared the status */
  stat = *(volatile unsigned long *)(TC3162_TIMERCTLR_REG);

	stat &=~((TIMERCTLR_Timer0_Int)|
		 (TIMERCTLR_Timer1_Int)|
		 (TIMERCTLR_Timer2_Int)|
		 (TIMERCTLR_Timer3_Int)|
		 (TIMERCTLR_Timer4_Int)|
		 (TIMERCTLR_Timer5_Int)
		);
	/* Clear the bit */
	stat |= TIMERCTLR_Timer1_Int; 
	*(volatile unsigned long *)(TC3162_TIMERCTLR_REG) = stat; 

  return(1);
}

/*
 * Call the "do_timer()" routine every clocktick
 */
static void inline
timer_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
  if (timer_intr_valid()) {
    if (!user_mode(regs)) {
      if (prof_buffer && current->pid) {
        extern int _stext;
        unsigned long pc = regs->cp0_epc;

        pc -= (unsigned long) &_stext;
        pc >>= prof_shift;

	/*
	 * Dont ignore out-of-bounds pc values silently,
	 * put them into the last histogram slot, so if
	 * present, they will show up as a sharp peak.
	 */
	if (pc > prof_len - 1)
	  pc = prof_len - 1;
        atomic_inc((atomic_t *) & prof_buffer[pc]);
      }
    }

#if 0 /* Show bar progress */
    printk("%c\b", bars[barcnt]);
    barcnt = (barcnt + 1) % 4;
#endif

    do_timer(regs);
  }
}

struct irqaction timer_irq = { timer_interrupt, SA_INTERRUPT, 0,
			       "timer", NULL, NULL};

void __init tc3162_time_init(void)
{
  unsigned int year, mon, day, hour, min, sec;
  unsigned long flags;
   
  sec = min = hour = 0;
  day = 1;
  mon = 1;
  year = 2003;

  /* Save the time */
  write_lock_irqsave(&xtime_lock, flags);
  xtime.tv_sec = mktime(year, mon, day, hour, min, sec);
  xtime.tv_usec = 0;
  write_unlock_irqrestore(&xtime_lock, flags);
}

static void setIntPriority(char source, char level)
{
  int ipRegNum,ipLevNum;
  int word;
  
  ipRegNum = level/4;
  ipLevNum = level%4;
  word =*(volatile unsigned int*)(TC3162_IPSR0_REG+ipRegNum*4);
  word |= source << ((3-ipLevNum)*8);
  *(volatile unsigned int*)(TC3162_IPSR0_REG+ipRegNum*4) =word;

}

static void setIntMask(
	int int_no,
	int enable
)
{
  if(enable == ENABLE)
     *(volatile unsigned int*)(TC3162_IMR_REG) |= (0x01 << int_no);
  else
     *(volatile unsigned int*)(TC3162_IMR_REG) &= ~((0x01) << int_no);

}

static void setTimer(
	int timer_no,
	int timerTime,
	int enable,
	int mode,
	int halt
)
{
	int word,word1;

  word = (timerTime * SYS_HCLK) * 1000/2;
  *(volatile unsigned int*)(TC3162_TIMER0LDR_REG+timer_no*0x08) = word;

  word = *(volatile unsigned int*)(TC3162_TIMERCTLR_REG);
  word1 = (enable << timer_no) |(mode << (timer_no + 8))|(halt << (timer_no + 26));
  word |= word1;
  *(volatile unsigned int*)(TC3162_TIMERCTLR_REG) = word;


}

static void set_irq_priority(void)
{
unsigned int IntPriority[32]={
  // UART_INT, RTC_ALARM_INT, RTC_TICK_INT, RESERVED0, TIMER0_INT
  IPL5, IPL19, IPL22, IPL21, IPL7,
  // TIMER1_INT, TIMER2_INT, TIMER3_INT, TIMER4_INT, TIMER5_INT
  IPL1, IPL6, IPL8, IPL9, IPL10,
  // GPIO_INT, RESERVED1, RESERVED2, RESERVED3, APB_DMA0_INT
  IPL11, IPL12, IPL13, IPL14, IPL18,
  // APB_DMA1_INT, RESERVED4, RESERVED5, DYINGGASP_INT, DMT_INT
  IPL15, IPL16, IPL17, IPL0, IPL2,
  // ARBITER_ERR_INT, MAC_INT, SAR_INT, USB_INT, RESERVED8
  IPL31, IPL4, IPL3, IPL23, IPL24,
  // RESERVED9, XSLV0_INT, XSLV1_INT, XSLV2_INT, XAPB0_INT
  IPL25, IPL26, IPL27, IPL28, IPL29,
  // XAPB1_INT, SWR_INT
  IPL30, IPL20
 };
 int i;
                                                                                                                                    
 tc_outl(CR_INTC_IPR0, 0);
 tc_outl(CR_INTC_IPR1, 0);
 tc_outl(CR_INTC_IPR2, 0);
 tc_outl(CR_INTC_IPR3, 0);
 tc_outl(CR_INTC_IPR4, 0);
 tc_outl(CR_INTC_IPR5, 0);
 tc_outl(CR_INTC_IPR6, 0);
 tc_outl(CR_INTC_IPR7, 0);
                                                                                                                                    
 for (i = 0; i < 32; i++)
  /*intPrioritySet*/setIntPriority(i, IntPriority[i]);
}


static void tc3162_init_timers(void)
{
  /* all interrupt belong to IRQ */
  *(volatile unsigned int*)(TC3162_ITR_REG) = 0;

  /* disable all IMR */
  *(volatile unsigned int*)(TC3162_IMR_REG) = 0;

#if 1 //jj; according to TC suggestion
  set_irq_priority();
  setTimer(1,TIMERTICKS_10MS,ENABLE,TIMER_TOGGLEMODE,TIMER_HALTDISABLE);
  setIntMask(INTR_TIMER1,ENABLE);
#else
  /* Set Priority */
  setIntPriority(INTR_UART,20);
  setIntPriority(INTR_TIMER1,0);
  setIntPriority(INTR_EMAC,2);
  /* Initialize timer */  
  setTimer(1,TIMERTICKS_10MS,ENABLE,TIMER_TOGGLEMODE,TIMER_HALTDISABLE);
  setIntMask(INTR_TIMER1,ENABLE);
  // Sam add debug
 // setIntMask(INTR_EMAC,0);
  setIntPriority(INTR_TIMER0,10);
  setIntPriority(INTR_TIMER2,11);
  setIntPriority(INTR_TIMER3,12);
  setIntPriority(INTR_TIMER4,14);
  setIntPriority(INTR_Watchdog,15);
  setIntPriority(INTR_GPIO,16);
  setIntPriority(INTR_APB_DMA_C0,17);
  setIntPriority(INTR_APB_DMA_C1,18);
  setIntPriority(INTR_AFE_Gasp,19);
  setIntPriority(INTR_DMT,1);
  setIntPriority(INTR_AHB_Bus_Er,21);
  setIntPriority(INTR_ATM_SAR,22);
  setIntPriority(INTR_USB,23);
  setIntPriority(INTR_PCI_INTA,24);
  setIntPriority(INTR_PCI_INTB,25);
  setIntPriority(INTR_PCI_Bus_Er,26);
  setIntPriority(INTR_SOFT_Int,27);
  // End Sam add
#if 0 //jj
  // Sam add for wireless
  setIntMask(INTR_PCI_INTA,ENABLE);
  setIntMask(INTR_PCI_INTB,ENABLE);
  // End Sam
#endif //jj
#endif //set interrupt priority
}


void __init tc3162_time_setup(struct irqaction *irq)
{
  irq->handler = timer_interrupt;
  irq->flags = SA_INTERRUPT;
  irq->name = "timer";
  irq->mask = 0x20;

  /* Initialize timers and use Timer 1 as system timer */
  tc3162_init_timers();
  setup_irq(INTR_TIMER1, irq);
}

