/*
 */

#include <linux/module.h>

#include <linux/kernel.h>
#include <linux/ioport.h>
#include <linux/sched.h>
#include <linux/logibusmouse.h>
#include <linux/signal.h>
#include <linux/errno.h>
#include <linux/miscdevice.h>
#include <linux/random.h>
#include <linux/poll.h>
#include <linux/init.h>

#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/system.h>
#include <asm/irq.h>

#include <asm/sunplus/regmap.h>

#define SCANHZ 1

extern volatile RegisterFile *dvd_regs; //define in arch/mipsnommu/sunplus/uart.c
#define BASE     0xa8000000

#include "busmouse.h"

static int msedev;
static int mouse_count=0;

typedef	struct	
{
  unsigned char	status;
  unsigned char	x;
  unsigned char	y;
} MOUSE_EVENT;

/*		
dvd_regs->iop_data[7] : PS2 output port for mouse
	
bit15 	: 1 	-> request to send
bit7~0	: cmd/data
	
PS2 mouse receiver buffer (QUEUE):
queue start address = BASE+dvd_regs->iopya*1024+0x510
queue write point value (unuseful)= BASE+dvd_regs->iopya*1024+0x500
queue len= 256 byte
*/

volatile unsigned char *pMouseResponseBuf,*pMouseRecvPoint;
unsigned char MS_initialized=0;	
unsigned char RtsPoint=0;
MOUSE_EVENT m;

//reset,stream mode,sampling rate 0x50,enable	
//const unsigned char bMouseInitCmd[]={0xFF,0xEA,0xF3,0x50,0xF4,0};	
void initial_dvd_mouse()
{
   unsigned char MouseState=0;
   unsigned char SendCmdLoc=0;	
   unsigned char NextSend=0;	
   unsigned char i;
   const unsigned char MouseInitCmd[]={0xFF,0xF3,0x0A,0xF4,0};	

   pMouseResponseBuf=(unsigned char *)(BASE+(dvd_regs->iopya<<10)+0x510);
   pMouseRecvPoint=(unsigned char *)(BASE+(dvd_regs->iopya<<10)+0x500);
   while (MouseState != 2) {
      switch (MouseState) {
         case 0: // send initial command
	   if (MouseInitCmd[SendCmdLoc]!=0) {
	      dvd_regs->iop_data[7]=0x8000|MouseInitCmd[SendCmdLoc];
	      printk("send ms cmd=%x\n",MouseInitCmd[SendCmdLoc]);
	      for (i=0;i<0xff;i++) // delay
		   ;
	      MouseState=1;
	   }
	   else {
	      MouseState=2;
	      MS_initialized=1;
           }
           break;
         case 1:			
	   if (RtsPoint==*pMouseRecvPoint)
	      break;	
	   //check response
	   NextSend=0;
	   if (pMouseResponseBuf[0]==0xFA) {
	      //if send command is reset must be check response 0xff 0xaa 
	      if (MouseInitCmd[SendCmdLoc]==0xFF) {
	         if (pMouseResponseBuf[1]==0xAA) { 
		    SendCmdLoc++;			
		    NextSend=1;
	         }	
	      }
	      else {
	         SendCmdLoc++;						
	         NextSend=1;
	      }
	   }
	   else
	      //re-send
	      if (pMouseResponseBuf[0]==0xFE)
	         NextSend=1;
	   if (NextSend) {
	      *pMouseRecvPoint=0;		
	      MouseState=0;	
	      pMouseResponseBuf[0]=0;				
	   }	
	   break;
      }
   }
}

/* This func couldn't use while loop
 * Becasuse while loop will cause process timing inconsistant
 * And that will crash the kernel 
 * So I set lower sample rate  to prevent data loss
 */ 
void pollingMouse(void)
{
   int	i;
	
   unsigned char bRtsTmp=RtsPoint;			
   unsigned char *m1=(unsigned char *)&m;
   do{
      for (i=0;i<3;i++) {
	if (bRtsTmp==*pMouseRecvPoint)
	   break;					
	*m1=pMouseResponseBuf[bRtsTmp];	
	m1++;
	bRtsTmp++;
	//printk("%x,\n",pMouseResponseBuf[bRtsPoint++]);	
      }
      if (i==3) {
         if ( (m.status&0x08) == 0x08 ) {
	    RtsPoint=bRtsTmp;
//	    printk("[s=%x,x=%x,y=%x]\n",m.status,m.x,m.y);
	    busmouse_add_movementbuttons(msedev, m.x, -m.y, m.status&0x7);
         }
         else {
//	    printk("<error>r=%d w=%d\n",RtsPoint,*pMouseRecvPoint);
	    RtsPoint = *pMouseRecvPoint; //maybe cause some problem
         }
      }
      else
	 break;
   } while(0);// end of while 1
}

struct timer_list mouse_timer;
  
static void dvd_mouse_timeup(unsigned long data)
{
   pollingMouse();
   mod_timer(&mouse_timer, jiffies + SCANHZ);
}
                                        
static void dvd_mouse_starttimer(void)
{
   init_timer(&mouse_timer);
   mouse_timer.function = dvd_mouse_timeup;
   mouse_timer.data = 0;
   mouse_timer.expires = jiffies + SCANHZ;
   add_timer(&mouse_timer);
}

static int release_mouse(struct inode * inode, struct file * file)
{
	if (--mouse_count)
	   return 0;
	//del_timer(&mouse_timer);
	return 0;
	
}

static int open_mouse(struct inode * inode, struct file * file)
{
	if (mouse_count++)
	   return 0;
//	dvd_mouse_starttimer();
	return 0;
}

static struct busmouse dvd_busmouse = {
	0, "dvd_busmouse", THIS_MODULE, open_mouse, release_mouse, 0
}; //MINOR = 0

static int __init dvd_bus_mouse_init(void)
{
	printk("Initial Sunplus DVD Mouse Driver\n");
//	initial_dvd_mouse(); //initial_dvd_mouse must be called before keyboard initial
	msedev = register_busmouse(&dvd_busmouse);
	if (msedev < 0)
		printk(KERN_WARNING "Unable to register busmouse driver.\n");
	else
		printk(KERN_INFO "Sunplus DVD BusMouse detected and installed.\n");
	dvd_mouse_starttimer();
	return msedev < 0 ? msedev : 0;
}

static void __exit dvd_bus_mouse_exit(void)
{
	del_timer(&mouse_timer);
	unregister_busmouse(msedev);
}

module_init(dvd_bus_mouse_init)
module_exit(dvd_bus_mouse_exit)

MODULE_LICENSE("GPL");
EXPORT_NO_SYMBOLS;
