/*
 */
#include <linux/config.h>
#include <linux/spinlock.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/tty.h>
#include <linux/mm.h>
#include <linux/signal.h>
#include <linux/init.h>
#include <linux/kbd_ll.h>
#include <linux/delay.h>
#include <linux/random.h>
#include <linux/poll.h>
#include <linux/miscdevice.h>
#include <linux/slab.h>
#include <linux/kbd_kern.h>
#include <linux/vt_kern.h>
#include <linux/smp_lock.h>
#include <linux/kd.h>
#include <linux/pm.h>

#include <asm/keyboard.h>
#include <asm/bitops.h>
#include <asm/uaccess.h>
#include <asm/irq.h>
#include <asm/system.h>

#include <asm/io.h>

#include <linux/pc_keyb.h>
#include <asm/keyboard.h>
#include <asm/sunplus/regmap.h>

#define SCANHZ 1

extern volatile RegisterFile *dvd_regs; //define in arch/mipsnommu/sunplus/uart.c
#define BASE     0xa8000000

typedef	struct	
{
  unsigned char	status;
  unsigned char	x;
  unsigned char	y;
} MOUSE_EVENT;

static inline void handle_mouse(unsigned char scancode);

/*		
dvd_regs->iop_data[7] : PS2 output port for mouse
	
bit15 	: 1 	-> request to send
bit7~0	: cmd/data
	
PS2 mouse receiver buffer (QUEUE):
queue start address = BASE+dvd_regs->iopya*1024+0x510
queue write point value (unuseful)= BASE+dvd_regs->iopya*1024+0x500
queue len= 256 byte
*/

unsigned char *pMouseResponseBuf,*pMouseRecvPoint;
unsigned char MS_initialized=0;	
unsigned char RtsPoint=0;
MOUSE_EVENT m;

//reset,stream mode,sampling rate 0x50,enable	
//const unsigned char bMouseInitCmd[]={0xFF,0xEA,0xF3,0x50,0xF4,0};	
void initial_dvd_mouse()
{
   unsigned char MouseState=0;
   unsigned char SendCmdLoc=0;	
   unsigned char NextSend=0;	
   unsigned char i;
   const unsigned char MouseInitCmd[]={0xFF,0xF3,0x0A,0xF4,0};	

   pMouseResponseBuf=(unsigned char *)(BASE+(dvd_regs->iopya<<10)+0x510);
   pMouseRecvPoint=(unsigned char *)(BASE+(dvd_regs->iopya<<10)+0x500);
   while (MouseState != 2) {
      switch (MouseState) {
         case 0: // send initial command
	   if (MouseInitCmd[SendCmdLoc]!=0) {
	      dvd_regs->iop_data[7]=0x8000|MouseInitCmd[SendCmdLoc];
	      printk("send ms cmd=%x\n",MouseInitCmd[SendCmdLoc]);
	      for (i=0;i<0xff;i++) // delay
		   ;
	      MouseState=1;
	   }
	   else {
	      MouseState=2;
	      MS_initialized=1;
           }
           break;
         case 1:			
	   if (RtsPoint==*pMouseRecvPoint){
	      break;
	   }	
	   //check response
	   NextSend=0;
	   if (pMouseResponseBuf[0]==0xFA) {
	      //if send command is reset must be check response 0xff 0xaa 
	      if (MouseInitCmd[SendCmdLoc]==0xFF) {
	         if (pMouseResponseBuf[1]==0xAA) { 
		    SendCmdLoc++;			
		    NextSend=1;
	         }	
	      }
	      else {
	         SendCmdLoc++;						
	         NextSend=1;
	      }
	   }
	   else
	      //re-send
	      if (pMouseResponseBuf[0]==0xFE)
	         NextSend=1;
	   if (NextSend) {
	      *pMouseRecvPoint=0;		
	      MouseState=0;	
	      pMouseResponseBuf[0]=0;				
	   }	
	   break;
      }
   }
}


void pollingMouse(void)
{
   int	i;
	
   unsigned char bRtsTmp=RtsPoint;			
   unsigned char *m1=(unsigned char *)&m;

   do{
      for (i=0;i<3;i++) {
	if (bRtsTmp==*pMouseRecvPoint)
	   break;					
	*m1=pMouseResponseBuf[bRtsTmp];	
	m1++;
	bRtsTmp++;
	//printk("%x,\n",pMouseResponseBuf[bRtsPoint++]);	
      }
      if (i==3) {
//	 printk("[s=%x,x=%x,y=%x]\n",m.status,m.x,m.y);
         if ( (m.status&0x08) == 0x08 ) {
	    RtsPoint=bRtsTmp;
	    handle_mouse(m.status);
	    handle_mouse(m.x);
	    handle_mouse(m.y);
         }
         else {
//	    printk("<error>r=%d w=%d\n",RtsPoint,*pMouseRecvPoint);
	    RtsPoint = *pMouseRecvPoint;
	    //while(1);
         }
      }
      else
	 break;
   } while(0);// end of while 1
}

struct timer_list mouse_timer;
  
static void dvd_mouse_timeup(unsigned long data)
{
   pollingMouse();
   mod_timer(&mouse_timer, jiffies + SCANHZ);   
}
                                        
static void dvd_mouse_starttimer(void)
{
   init_timer(&mouse_timer);
   mouse_timer.function = dvd_mouse_timeup;
   mouse_timer.data = 0;
   mouse_timer.expires = jiffies + SCANHZ;
   add_timer(&mouse_timer);
}

/*
 *	PS/2 Auxiliary Device
 */

static spinlock_t controller_lock = SPIN_LOCK_UNLOCKED; 
static struct aux_queue *queue;	/* Mouse data buffer. */
static int aux_count;


static inline void handle_mouse(unsigned char scancode)
{
	static unsigned char prev_code;
	int head;
	prev_code = scancode;
	add_mouse_randomness(scancode);
	if (aux_count) {
		head = queue->head;

		queue->buf[head] = scancode;
		head = (head + 1) & (AUX_BUF_SIZE-1);
		if (head != queue->tail) {
			queue->head = head;
			kill_fasync(&queue->fasync, SIGIO, POLL_IN);
			wake_up_interruptible(&queue->proc_list);
		}
	}
}

static unsigned char get_from_queue(void)
{
	unsigned char result;
	unsigned long flags;

	spin_lock_irqsave(&controller_lock, flags);
	result = queue->buf[queue->tail];
	queue->tail = (queue->tail + 1) & (AUX_BUF_SIZE-1);
	spin_unlock_irqrestore(&controller_lock, flags);
	return result;
}


static inline int queue_empty(void)
{
	return queue->head == queue->tail;
}

static int fasync_aux(int fd, struct file *filp, int on)
{
	int retval;

	retval = fasync_helper(fd, filp, on, &queue->fasync);
	if (retval < 0)
		return retval;
	return 0;
}


/*
 * Random magic cookie for the aux device
 */
#define AUX_DEV ((void *)queue)

static int release_aux(struct inode * inode, struct file * file)
{
	lock_kernel();
	fasync_aux(-1, file, 0);
	if (--aux_count) {
		unlock_kernel();
		return 0;
	}
//	del_timer(&mouse_timer);

	unlock_kernel();
	return 0;
}

/*
 * Install interrupt handler.
 * Enable auxiliary device.
 */

static int open_aux(struct inode * inode, struct file * file)
{
	if (aux_count++) {
		return 0;
	}
	queue->head = queue->tail = 0;		/* Flush input queue */
//	dvd_mouse_starttimer();
	return 0;
}

/*
 * Put bytes from input queue to buffer.
 */

static ssize_t read_aux(struct file * file, char * buffer,
			size_t count, loff_t *ppos)
{
	DECLARE_WAITQUEUE(wait, current);
	ssize_t i = count;
	unsigned char c;

	if (queue_empty()) {
		if (file->f_flags & O_NONBLOCK)
			return -EAGAIN;
		add_wait_queue(&queue->proc_list, &wait);
repeat:
		set_current_state(TASK_INTERRUPTIBLE);
		if (queue_empty() && !signal_pending(current)) {
			schedule();
			goto repeat;
		}
		current->state = TASK_RUNNING;
		remove_wait_queue(&queue->proc_list, &wait);
	}
	while (i > 0 && !queue_empty()) {
		c = get_from_queue();
		put_user(c, buffer++);
		i--;
	}
	if (count-i) {
		file->f_dentry->d_inode->i_atime = CURRENT_TIME;
		return count-i;
	}
	if (signal_pending(current))
		return -ERESTARTSYS;
	return 0;
}

/*
 * Write to the aux device.
 */
#if 0
static ssize_t write_aux(struct file * file, const char * buffer,
			 size_t count, loff_t *ppos)
{
	ssize_t retval = 0;

	if (count) {
		ssize_t written = 0;

		if (count > 32)
			count = 32; /* Limit to 32 bytes. */
		do {
			char c;
			get_user(c, buffer++);
			aux_write_dev(c);
			written++;
		} while (--count);
		retval = -EIO;
		if (written) {
			retval = written;
			file->f_dentry->d_inode->i_mtime = CURRENT_TIME;
		}
	}

	return retval;
}
#endif

/* No kernel lock held - fine */
static unsigned int aux_poll(struct file *file, poll_table * wait)
{
	poll_wait(file, &queue->proc_list, wait);
	if (!queue_empty())
		return POLLIN | POLLRDNORM;
	return 0;
}

struct file_operations psaux_fops = {
	read:		read_aux,
//	write:		write_aux,
	poll:		aux_poll,
	open:		open_aux,
	release:	release_aux,
	fasync:		fasync_aux,
};

/*
 * Initialize driver.
 */
static struct miscdevice psaux_mouse = {
	PSMOUSE_MINOR, "psaux", &psaux_fops
};

int __init psaux_init(void)
{
	int retval;
	printk("Sunplus DVD PS/2 Mouse Driver Initial\n");

	if ((retval = misc_register(&psaux_mouse)))
		return retval;

	queue = (struct aux_queue *) kmalloc(sizeof(*queue), GFP_KERNEL);
	if (queue == NULL) {
		printk(KERN_ERR "psaux_init(): out of memory\n");
		misc_deregister(&psaux_mouse);
		return -ENOMEM;
	}
	memset(queue, 0, sizeof(*queue));
	queue->head = queue->tail = 0;
	init_waitqueue_head(&queue->proc_list);

//	initial_dvd_mouse();
	dvd_mouse_starttimer();
	return 0;
}


