/*
 *  linux/drivers/video/yuv420.c -- level frame buffer operations for YUV32 packed pixels of Sunplus *					fb driver
 *
 *	Created 05 Nov 2001 by Jeff Wang
 *
 *  This file is subject to the terms and conditions of the GNU General Public
 *  License.  See the file COPYING in the main directory of this archive for
 *  more details.
 */

#include <linux/module.h>
#include <linux/tty.h>
#include <linux/console.h>
#include <linux/string.h>
#include <linux/fb.h>
#include <asm/sunplus/sp720fb.h>
#include <video/fbcon.h>


    /*
     *  YUV packed pixels
     */

#define sunplus_writechr(fontbits)				\
{								\
    int i, j;							\
    for(i = 0, j = font_width; i < font_width; i++, j--) { 	\
	if ((fontbits >> i) & 0x1)				\
            sunplus_setpixel(xx+j, yy, 0xf68080);		\
        else							\
            sunplus_setpixel(xx+j, yy, 0x8080);			\
    }								\
}								\


void fbcon_sp720_setup(struct display *p)
{
    p->next_line = p->line_length ? p->line_length : p->var.xres_virtual<<2;
    p->next_plane = 0;
}


/* Fixed me */
void fbcon_sp720_bmove(struct display *p, int sy, int sx, int dy, int dx,
		       int height, int width)
{
    int bytes = p->next_line, linesize = bytes * fontheight(p), rows;
    u8 *src, *dst;

//    printk("...fbcon_sp720_bmove...\n");

    if (sx == 0 && dx == 0 && width * fontwidth(p) * 4 == bytes) {
//	mymemmove(p->screen_base + dy * linesize,	//lobo replace
	fb_memmove(p->screen_base + dy *linesize,
		  p->screen_base + sy * linesize,
		  height * linesize);
	return;
    }
    if (fontwidthlog(p)) {
	sx <<= fontwidthlog(p)+2;
	dx <<= fontwidthlog(p)+2;
	width <<= fontwidthlog(p)+2;
    } else {
	sx *= fontwidth(p)*4;
	dx *= fontwidth(p)*4;
	width *= fontwidth(p)*4;
    }
    if (dy < sy || (dy == sy && dx < sx)) {
	src = p->screen_base + sy * linesize + sx;
	dst = p->screen_base + dy * linesize + dx;
	for (rows = height * fontheight(p); rows--;) {
	//    mymemmove(dst, src, width);
	    fb_memmove(dst, src, width); //lobo replace
	    src += bytes;
	    dst += bytes;
	}
    } else {
	src = p->screen_base + (sy+height) * linesize + sx - bytes;
	dst = p->screen_base + (dy+height) * linesize + dx - bytes;
	for (rows = height * fontheight(p); rows--;) {
	//    mymemmove(dst, src, width);
	    fb_memmove(dst, src, width); //lobo replace
	    src -= bytes;
	    dst -= bytes;
	}
    }
}


static inline 
void rectfill(u8 *dest, int width, int height, u32 data,
			    int linesize)
{
    /* replaced by sunplus_fillrect() */
}


void fbcon_sp720_clear(struct vc_data *conp, struct display *p, int sy, int sx,
		       int height, int width)
{
    sx *= fontwidth(p);
    sy *= fontheight(p);
    width *= fontwidth(p);
    height *= fontheight(p);

    sunplus_fillrect(sx, sy, width, height, 0x8080);
}


void fbcon_sp720_putc(struct vc_data *conp, struct display *p, int c, int yy,
		      int xx)
{
    u8 *cdat, bits;
    u16 u16bits;
    int rows, 
	pixel_start,
    	font_width = fontwidth(p), 
	font_height = fontheight(p);
    //u32 eorx, fgx, bgx;

    yy *= font_height;
    xx *= font_width;
    pixel_start = xx;

//    printk("...fbcon_sp720_putc()...\n");

    //fgx = ((u32 *)p->dispsw_data)[attr_fgcol(p, c)];
    //bgx = ((u32 *)p->dispsw_data)[attr_bgcol(p, c)];
    //eorx = fgx ^ bgx;

    switch (font_width) {
    case 8:
            cdat = p->fontdata + c * font_height;

            for(rows = font_height, xx = pixel_start; rows--; yy++) {
                bits = *cdat++;
                sunplus_writechr(bits);
            }

            pixel_start += font_width;
            yy -= font_height;

        break;
    case 12:
            cdat = p->fontdata + (c * font_height << 1);

            for(rows = font_height, xx = pixel_start; rows--; yy++) {
                u16bits = *cdat<<8 | (*(cdat+1) & 0xf0);
                u16bits >>= 4;
                cdat += 2;

                sunplus_writechr(u16bits);
            }

            pixel_start += font_width;
            yy -= font_height;

        break;
    default:
	printk(" Only support font type 8x8, 8x16, 12x16 \n");
	break;	
    }
}


void fbcon_sp720_putcs(struct vc_data *conp, struct display *p,
		       const unsigned short *s, int count, int yy, int xx)
{
    u8 *cdat, bits;
    u16 c, u16bits;
    //u32 eorx, fgx, bgx;
    int rows, 
	pixel_start,
	font_width = fontwidth(p), 
	font_height = fontheight(p);

    yy *= font_height;
    xx *= font_width;
    pixel_start = xx;

    //fgx = ((u32 *)p->dispsw_data)[attr_fgcol(p, scr_readw(s))];
    //bgx = ((u32 *)p->dispsw_data)[attr_bgcol(p, scr_readw(s))];
    //eorx = fgx ^ bgx;

    switch (font_width) {
    case 8:
    	while (count--) {
	    c = scr_readw(s++) & p->charmask;
            cdat = p->fontdata + c * font_height;

	    for(rows = font_height, xx = pixel_start; rows--; yy++) {
	        bits = *cdat++;

		sunplus_writechr(bits);
	    }
	    pixel_start += font_width; 
	    yy -= font_height; 
    	}
	break;
    case 12:
        while (count--) {
            c = scr_readw(s++) & p->charmask;
            cdat = p->fontdata + (c * font_height << 1);

            for(rows = font_height, xx = pixel_start; rows--; yy++) {
		u16bits = *cdat<<8 | (*(cdat+1) & 0xf0);
 		u16bits >>= 4;
		cdat += 2;

		sunplus_writechr(u16bits);
            }
            pixel_start += font_width;
            yy -= font_height;
        }
        break;
    default:
	printk(" Only support font type 8x8, 8x16, 12x16 \n");
	break;
    }
}


void fbcon_sp720_revc(struct display *p, int xx, int yy)
{
    u8 *dest;
    int bytes=p->next_line;

    yy *= fontheight(p);
    xx *= fontwidth(p);
    dest = p->screen_base + yy * fontheight(p) * bytes + xx * fontwidth(p);
    dest[0] ^= 0x1;

    sunplus_fillrect(xx, yy,  fontwidth(p), fontheight(p),                                                               ((dest[0] & 0x1)*0xf60000 | 0x8080));
}


void fbcon_sp720_clear_margins(struct vc_data *conp, struct display *p,
			       int bottom_only)
{
    u32 bgx = 0xf68080;

    unsigned int right_start = conp->vc_cols*fontwidth(p);
    unsigned int bottom_start = conp->vc_rows*fontheight(p);
    unsigned int right_width, bottom_width;

    //bgx = ((u32 *)p->dispsw_data)[attr_bgcol_ec(p, conp)];

    if (!bottom_only && (right_width = p->var.xres-right_start))
        sunplus_fillrect(right_start, 0, right_width, p->var.yres_virtual, bgx);

    if ((bottom_width = p->var.yres-bottom_start))
        sunplus_fillrect(0, bottom_start, p->var.xres, bottom_width, bgx);
}


    /*
     *  `switch' for the low level operations
     */

struct display_switch fbcon_sp720 = {
    fbcon_sp720_setup, fbcon_sp720_bmove, fbcon_sp720_clear, fbcon_sp720_putc,
    fbcon_sp720_putcs, fbcon_sp720_revc, NULL, NULL, fbcon_sp720_clear_margins,
    FONTWIDTH(4)|FONTWIDTH(8)|FONTWIDTH(12)|FONTWIDTH(16)
};


#ifdef MODULE
int init_module(void)
{
    return 0;
}

void cleanup_module(void)
{}
#endif /* MODULE */


    /*
     *  Visible symbols for modules
     */

EXPORT_SYMBOL(fbcon_sp720);
EXPORT_SYMBOL(fbcon_sp720_setup);
EXPORT_SYMBOL(fbcon_sp720_bmove);
EXPORT_SYMBOL(fbcon_sp720_clear);
EXPORT_SYMBOL(fbcon_sp720_putc);
EXPORT_SYMBOL(fbcon_sp720_putcs);
EXPORT_SYMBOL(fbcon_sp720_revc);
EXPORT_SYMBOL(fbcon_sp720_clear_margins);
