#include <asm/sunplus/regmap.h>
#include "gpio.h"
#define USE_VFD_KEY
#define GPIO_SEL_S		( GPIOSEL_10_UARTS        \
				| GPIOSEL_6_GPIO        \
				| GPIOSEL_7_GPIO        \
				| GPIOSEL_8_HVSYNC_IN   \
				| GPIOSEL_9_UART_NORMAL \
				| GPIOSEL_A_GPIO20	\
				| GPIOSEL_B_GPIO21	\
				| GPIOSEL_C_GPIO59      \
				| GPIOSEL_ED_TVOUT_ALYS \
				| GPIO_SEL_S_DISC		)

#define GPIO_SEL_AUX_S		( GPIOAUX_0_GPIO0       \
				| GPIOAUX_1_GPIO1	\
				| GPIOAUX_2_MEMWEB	\
				| GPIOAUX_3_MEMOEB      \
				| GPIOAUX_64_MEMCS3_CSX \
				| GPIOAUX_7_GPIO16	\
				| GPIOAUX_8_GPIO17	\
				| GPIOAUX_9_GPIO18	\
				| GPIOAUX_A_GPIO19	\
				)
				
extern volatile RegisterFile *dvd_regs;				

/*
** FUNCTION
** reset_gpio
** restore GPIO initial state
*/
void	reset_gpio(void)
{
	/*	
	**dvd_regs->gpio_sel	= GPIO_SEL_S;
	**dvd_regs->gpio_sel_aux	= GPIO_SEL_AUX_S;
		//reset TV/AV system
	**dvd_regs->gpio_out[0]	|= 1<<12; 
	**dvd_regs->gpio_oe[0]	|= 1<<12; 
	**dvd_regs->gpio_out[0]	&= ~(1<<12); 
		// enable GPIO[16][17] for I2C
	**dvd_regs->gpio_out[1]	= 0;
	**dvd_regs->gpio_oe[1]	= 0x03;
	*/
	// VFD BUS GPIO Setup
	dvd_regs->gpio_out[3]	|= 0x0700;  //preset pins in Hi level
	dvd_regs->gpio_oe[3]	|= 0x0600;  //preset SCK STB as output
	dvd_regs->gpio_oe[3]	&= ~0x0100; //Preset SDA as input
  
}


/*
############################################################################
#
#        I2C BLOCK !!!
#
#
#
############################################################################
*/

#define	I2C_SDA	59		// GPIOE[0]
#define	I2C_SCL	60		// GPIOE[1]


#define	I2C_SCL_SET(d)	GPIO_O_SET(I2C_SCL,d)
#define	I2C_SDA_SET(d)	GPIO_O_SET(I2C_SDA,d)
#define	I2C_SDA_GET()	GPIO_I_GET(I2C_SDA)

#define	I2C_SCL_IN()	GPIO_E_SET(I2C_SCL,0)
#define	I2C_SDA_IN()	GPIO_E_SET(I2C_SDA,0)
#define	I2C_SCL_OUT()	GPIO_E_SET(I2C_SCL,1)
#define	I2C_SDA_OUT()	GPIO_E_SET(I2C_SDA,1)

void	init_i2c(void)
{
  I2C_SCL_SET(1);
  I2C_SDA_SET(1);
  I2C_SCL_OUT();
  I2C_SDA_OUT();
  //monitor();
}

void	delay_gpio(int i)
{
  do {
    int j =10;
    do {
      asm volatile ("nop");
    } while (--j>=0);
  } while (--i>=0);
}




#define	delay_i2c(i)	delay_gpio(i)
//
// i2c_start_sig()
// SCL: --\
// SDA: -\_
//
void i2c_start_sig()
{	I2C_SCL_SET(1);
	I2C_SDA_SET(1);
	I2C_SCL_OUT();
	I2C_SDA_OUT();	
	delay_i2c(1);
	I2C_SDA_SET(0);delay_i2c(1);
}
//
// i2c_stop_sig()
// SCL: _/-
// SDA: __/
//
void i2c_stop_sig()
{	I2C_SDA_SET(0); I2C_SDA_OUT(); I2C_SCL_SET(0); delay_i2c(1);
	I2C_SCL_SET(1); delay_i2c(1);	
	I2C_SDA_SET(1);
}
//
// i2c_byte_w(data)
// SCL: _/-\_/-\_/-\_/-\_/-\_/-\_/-\_/-\_/-\_
// SDA: _<D7><D6><D5><D4><D3><D2><D1><D0>ACK__
//
int i2c_byte_w(int data)
{	int i;
	I2C_SCL_SET(0); delay_i2c(1);
	I2C_SDA_OUT();
	for(i=8;i;i--)
	{	if(data&0x80){I2C_SDA_SET(1);}
		else {I2C_SDA_SET(0);}
		delay_i2c(1);						// scl -- low
		I2C_SCL_SET(1); delay_i2c(1);		// scl -> high
		data<<=1;
		I2C_SCL_SET(0); delay_i2c(1);		// scl -> low
	}
	// Get ACK
	I2C_SDA_IN();	 delay_i2c(1);			// scl -- low
	I2C_SCL_SET(1); delay_i2c(1);			// scl -> high
	i= I2C_SDA_GET();	// 0 for no error
	I2C_SCL_SET(0); delay_i2c(1);			// scl -> low
	return ((i)?1:0);
}
//
// i2c_byte_r()
// SCL: _/-\_/-\_/-\_/-\_/-\_/-\_/-\_/-\_/-\_
// SDA: _<D7><D6><D5><D4><D3><D2><D1><D0>____
//
int i2c_byte_r()
{	int i,data;
	I2C_SCL_SET(0);
	I2C_SDA_IN(); delay_i2c(1);			// scl -> low
	for(i=8;i;i--)
	{	data<<=1;	
		I2C_SCL_SET(1); delay_i2c(1);		// scl -> high
		data |= ((I2C_SDA_GET()) ? 1 : 0);
		I2C_SCL_SET(0); delay_i2c(1);		// scl -> low
	}
	// Set ACK !!
	I2C_SDA_SET(0);						// scl -- low
	I2C_SDA_OUT();  delay_i2c(1);			// scl -- low
	I2C_SCL_SET(1); delay_i2c(1);			// scl -> high
	I2C_SCL_SET(0); delay_i2c(1);			// scl -> low
	return data;
}

int	read_i2c(int addr, int aa)
{	int data;
	// Write ADDR
	i2c_start_sig();
	i2c_byte_w((addr<<1)|0); //W add
	i2c_byte_w(aa);
	i2c_stop_sig();  	
	// Read DATA
	i2c_start_sig();
	i2c_byte_w((addr<<1)|1); //R data
	data=i2c_byte_r();
	i2c_stop_sig();  	
	return data;
}

void	write_i2c(int addr, int aa, int data)
{	// Write Addr & DATA
	i2c_start_sig();
	i2c_byte_w(addr<<1|0); //W add data
	i2c_byte_w(aa);
	i2c_byte_w(data);
	i2c_stop_sig();  	
}

/*
############################################################################
#
#        VFD BLOCK !!!
#
#
#
############################################################################
*/
#define VFD_SDA 56
#define VFD_STB 57
#define VFD_SCL 58

#define VFD_SDA_SET(d) GPIO_O_SET(VFD_SDA,d)
#define VFD_SCL_SET(d) GPIO_O_SET(VFD_SCL,d)
#define VFD_STB_SET(d) GPIO_O_SET(VFD_STB,d)
#define	VFD_SDA_GET()  GPIO_I_GET(VFD_SDA)
#define VFD_SDA_IN()   GPIO_E_SET(VFD_SDA,0)
#define VFD_SDA_OUT()   GPIO_E_SET(VFD_SDA,1)

void vfd_byte_w(int data)
{	int i;
	VFD_SCL_SET(0);
	VFD_SDA_OUT(); 
	delay_gpio(1);
	for(i=8;i;i--)
	{	if(data&0x01){VFD_SDA_SET(1);}
		else {VFD_SDA_SET(0);}
		VFD_SCL_SET(1); 
		//data<<=1;
		data>>=1;
		VFD_SCL_SET(0); 	// scl -> low
	}
	VFD_SDA_IN();
}

int vfd_byte_r()
{	int i,data=0;
	VFD_SCL_SET(0);
	VFD_SDA_IN(); delay_gpio(50);			// scl -> low
	for(i=8;i;i--)
	{	//data<<=1;	
		data>>=1;
		VFD_SCL_SET(1); 		// scl -> high
		//added jack
		//data |= (VFD_SDA_GET() ? 1 : 0);
		data |= ((VFD_SDA_GET()) ? 0x80 : 0);
		VFD_SCL_SET(0); 	// scl -> low
	}
	return data;
}

#ifdef USE_VFD_KEY
int vfd_scan(int data)
{	VFD_STB_SET(0);
	vfd_byte_w(data);
	//delay_gpio(1);
	VFD_STB_SET(1);
	data=vfd_byte_r();
	return data;
}
#endif
#ifdef USE_VFD_DBG
// for VFDDBG
int vfd_transfer(int data)
{	VFD_STB_SET(0);
	delay_gpio(1);
	vfd_byte_w(data);
	delay_gpio(3);
	data=vfd_byte_r();
	VFD_STB_SET(1);
	delay_gpio(1);
	return data;
}

void vfd_puts(char *s)
{	while((*s))
	{	vfd_transfer(*s);
		s++;
	}
}
#endif

