/*
** sp721.c
** DESCRIPTION
** support of 721
**
**write_i2c: risc controlled 721 function
**WriteI2C: iop controlled i2c function
*/

#include "func.h"

#define SP721_WRITE_SBUS(ra,d)	write_i2c(0x20,ra,d)

//#define SP721_WRITE_SBUS(ra,d)	WriteI2c(0x40,ra,d,1)

/*
** tv_encoder chip-address SP721_CA
*/

// reg0
#define	SP721_HVSYNC_MASTER	0x04
#define	SP721_HVSYNC_656	0x02

#define	SP721_STANDARD_NTSC	0x00
#define	SP721_STANDARD_PAL	0x80
#define	SP721_STANDARD_PALM	0x40
#define	SP721_STANDARD_PALN	0xc0

// reg2
#define	SP721_INTERLACED	(0<<8)
#define	SP721_NONINTERLACED	(1<<8)

#define	SP721_SUBCARRIER_DEF	(0<<4)
#define SP721_SUBCARRIER_USER	(1<<4)

// reg 6 - reg 9:
// sub_c0 ~ sub_c3
#define	SP721_FSC_NTSC	(0x21f07c1f)		// 3.5795452 MHz / 27MHz
#define	SP721_FSC_PAL	(0x21e6efe3)		// 4.43361875MHz / 27MHz
#define	SP721_FSC_PALM	(0x2a098acb)		// 3.5820558 MHz / 27MHz
#define	SP721_FSC_PALN	(0x21f69447)		// 3.57661149MHz / 27MHz

#define	SP721_FSC0(x)	((x    ) & 0x0ff)
#define	SP721_FSC1(x)	((x>> 8) & 0x0ff)
#define	SP721_FSC2(x)	((x>>16) & 0x0ff)
#define	SP721_FSC3(x)	((x>>24) & 0x0ff)

/*
** PAL60
** display: 720x480x30 (60field)
** chroma: 4.43
*/
void	tv_setup_pal60(void)
{
	SP721_WRITE_SBUS(0x00,
	SP721_STANDARD_NTSC|SP721_HVSYNC_MASTER);	// NTSC
	SP721_WRITE_SBUS(0x01, 2);
	SP721_WRITE_SBUS(0x02, SP721_INTERLACED | SP721_SUBCARRIER_USER);
	SP721_WRITE_SBUS(0x06, SP721_FSC0(SP721_FSC_PAL));
	SP721_WRITE_SBUS(0x07, SP721_FSC1(SP721_FSC_PAL));
	SP721_WRITE_SBUS(0x08, SP721_FSC2(SP721_FSC_PAL));
	SP721_WRITE_SBUS(0x09, SP721_FSC3(SP721_FSC_PAL));
}

void	tv_setup_ntsc(void)
{
	SP721_WRITE_SBUS(0x00,
	SP721_STANDARD_NTSC|SP721_HVSYNC_MASTER);	// NTSC
	SP721_WRITE_SBUS(0x01, 2);
	SP721_WRITE_SBUS(0x02, SP721_INTERLACED | SP721_SUBCARRIER_DEF);
}

void	tv_setup_pal(void)
{
	SP721_WRITE_SBUS(0x00,
	SP721_STANDARD_PAL|SP721_HVSYNC_MASTER);	// PAL
	SP721_WRITE_SBUS(0x01, 2);
	SP721_WRITE_SBUS(0x02, SP721_INTERLACED | SP721_SUBCARRIER_DEF);
}


void	tv_setup()
{

	//SP721_WRITE_SBUS(0x02, 0x02);	// composited 'n component
	//SP721_WRITE_SBUS(0x05, 0x00);		// turn on all DAC
	SP721_WRITE_SBUS(0x01, 2);
	SP721_WRITE_SBUS(0x05, 0x0e);		// turn on  DAC0
}

/*
** sp721.c
*/
