/*
 * include/linux/dvd_uart_reg.h
 *
 * These are the UART port assignments for Sunplus DVD platform 
 *
 * written by ruey-an yeh
 *
 */

/* the real base address will be defined on asm/serial.h */
#define SERIAL_BASE  0 

#define UART_DATA	(SERIAL_BASE+0*4)
#define UART_LSR	(SERIAL_BASE+1*4)
#define UART_MSR	(SERIAL_BASE+2*4)
#define UART_LCR	(SERIAL_BASE+3*4)
#define UART_MCR	(SERIAL_BASE+4*4)
#define UART_DIV_L	(SERIAL_BASE+5*4)
#define UART_DIV_H	(SERIAL_BASE+6*4)
#define UART_ISC	(SERIAL_BASE+7*4)

/*
 * These are the definitions for the Line Status Register
 */
#define UART_LSR_SOFTRST	0x80
#define UART_LSR_TXE		0x40
#define UART_LSR_BI		0x20
#define UART_LSR_FE		0x10
#define UART_LSR_OE		0x08
#define UART_LSR_PE		0x04
#define UART_LSR_RXRDY		0x02
#define UART_LSR_TXRDY		0x01

/*
 * These are the definitions for the Modem Status Register
 */
#define UART_MSR_TERI	0x80
#define UART_MSR_DDCD	0x40
#define UART_MSR_DCTS	0x20
#define UART_MSR_DDSR	0x10
#define UART_MSR_RI	0x08
#define UART_MSR_DCD	0x04
#define UART_MSR_CTS	0x02
#define UART_MSR_DSR	0x01
#define UART_MSR_ANY_DELTA 0xF0	/* Any of the delta bits! */

/*
 * These are the definitions for the Line Control Register
 * 
 * Note: if the word length is 5 bits (UART_LCR_WLEN5), then setting 
 * UART_LCR_STOP will select 1.5 stop bits, not 2 stop bits.
 */
#define UART_LCR_SBC	0x20	/* Set break control */
#define UART_LCR_EPAR	0x10	/* Even parity select */
#define UART_LCR_PARITY	0x08	/* Parity Enable */
#define UART_LCR_STOP	0x04	/* Stop bits: 0=1 stop bit, 1= 2 stop bits */
#define UART_LCR_WLEN5  0x00	/* Wordlength: 5 bits */
#define UART_LCR_WLEN6  0x01	/* Wordlength: 6 bits */
#define UART_LCR_WLEN7  0x02	/* Wordlength: 7 bits */
#define UART_LCR_WLEN8  0x03	/* Wordlength: 8 bits */

/*
 * These are the definitions for the Modem Control Register
 */
#define UART_MCR_AUTO_CALIBRATE_CLOCK	0x80
#define UART_MCR_AUTOCTS	0x40
#define UART_MCR_AUTORTS	0x20
#define UART_MCR_LOOP	0x10	/* Enable loopback test mode */
#define UART_MCR_OUT2	0x08	/* Out2 complement */
#define UART_MCR_OUT1	0x04	/* Out1 complement */
#define UART_MCR_RTS	0x02	/* RTS complement */
#define UART_MCR_DTR	0x01	/* DTR complement */

/*
 * These are the definitions for the Interrupt status/control Register
 */
#define UART_IIR_NO_INT	0x0F
#define UART_IIR_THRI	0x01
#define UART_IIR_RDI	0x02
#define UART_IIR_RLSI	0x04
#define UART_IIR_MSI	0x08

#define UART_IER_THRI	0x10
#define UART_IER_RDI	0x20
#define UART_IER_RLSI	0x40
#define UART_IER_MSI	0x80

