/*
** define register-file structure.
*/

/* =====================================================================
 *   Notes  From Lobo Wu , ITRI
 *   define all regs in a struct may look simple,
 *   but It will cause some problem ...
 *   ex:  hard to coding , and bad preformance ...... etc
 *
 *   This file is just from ucos, I just use it to save my porting time
 * =====================================================================
 */


#ifndef _SUNPLUS_REGMAP_H_
#define _SUNPLUS_REGMAP_H_

typedef	struct 
{
  /*
  ** Group 0: Chip Information 
  ** 0-3
  */
  unsigned int	stamp;			/* 0:				*/
  unsigned int	emulation;		/* 1:				*/
  unsigned int	reset;			/* 2:				*/
  unsigned int	task_priority_set;	/* 3:				*/

  /*
  ** Group 1: timers
  ** 11-regs (4-15)
  */
  unsigned int	stc_15_0;		/* 4				*/
  unsigned int	stc_31_16;		/* 5				*/
  unsigned int	stc_32;			/* 6	stc 33b counter @90kHz	*/
  unsigned int	stc_divisor;		/* 7				*/
  unsigned int	rtc_15_0;		/* 8				*/
  unsigned int	rtc_31_16;		/* 9				*/
  unsigned int	rtc_divisor;		/* 10				*/
  unsigned int	g1_unused[1];		/* 11				*/
  unsigned int	timer0_ctrl;		/* 12				*/
  unsigned int	timer0_cnt;		/* 13				*/
  unsigned int	timer1_ctrl;		/* 14				*/
  unsigned int	timer1_cnt;		/* 15				*/

  /*
  ** Group 2 : Interrupt
  **
  ** 16-18
  */
  unsigned int	intr_mask;		/* 16:				*/
  unsigned int	intr_flag;		/* 17:				*/
  unsigned int	intr_clear;		/* 18:				*/

  /*
  ** Group 3 : Decoding control
  **
  ** 19
  */
  unsigned int	pic_start;		/* 19:				*/

  /*
  ** Group 4 : Video bitstream parameters 
  **
  ** 12 regs
  ** 20-31:
  */
  unsigned int	seq_ext;		/* 20				*/
  unsigned int	h_size;			/* 21				*/
  unsigned int	v_size;			/* 22				*/
  unsigned int	qmx_par;		/* 23				*/
  unsigned int	pic_coding_type;	/* 24				*/
  unsigned int	pic_f_code;		/* 25 f_code			*/
  unsigned int	pic_coding_ext0;	/* 26				*/
  unsigned int	pic_coding_ext1;	/* 27 				*/
  unsigned int	vld_ctrl;		/* 28				*/
  unsigned int	vld_status;		/* 29				*/
  unsigned int	bs_yinit;		/* 30				*/
  unsigned int	bs_xinit;		/* 31				*/

  /*
  ** Group 9 : SDRAM control 
  ** 20 regs 
  ** 32-45
  */
  unsigned int	evbya;			/* 32:				*/
  unsigned int	evbya2;			/* 33:				*/
  unsigned int	eabya;			/* 34:				*/
  unsigned int	ref0_luma;		/* 35:				*/
  unsigned int	ref0_chroma;		/* 36:				*/
  unsigned int	ref1_luma;		/* 37				*/
  unsigned int	ref1_chroma;		/* 38				*/
  unsigned int	bidir_luma;		/* 39 				*/
  unsigned int	bidir_chroma;		/* 40 				*/
  unsigned int	dsp16ya;		/* 41				*/
  unsigned int	dsp24ya;		/* 42				*/
  unsigned int	osdya;			/* 43				*/
  unsigned int	cdya;			/* 44				*/
  unsigned int	supya;			/* 45				*/
  unsigned int	pcmya;			/* 46				*/
  unsigned int	audya;			/* 47				*/
  unsigned int	iopya;			/* 48				*/
  unsigned int	g5_unused[5];		/* 49 - 63			*/

  //
  // Group 31: CDDSP
  //
  unsigned int        cddsp_config;           // 54: config register
  unsigned int        cddsp_control;          // 55
  unsigned int        cddsp_mm_bcd;           // 56
  unsigned int        cddsp_ss_bcd;           // 57
  unsigned int        cddsp_ff_bcd;           // 58
  unsigned int        cddsp_status;           // 59
  unsigned int        cddsp_mmss;             // 60
  unsigned int        cddsp_ffmm;             // 61
  unsigned int	g31_unused[2];
   

  /*
  ** Group 7: Display Control
  */
  unsigned int	dis_pic_id;		/* 64:				*/
  unsigned int	dis_tv_std;		/* 65:				*/
  unsigned int	dis_tv_out;		/* 66:				*/
  unsigned int	dis_mixer;		/* 67:				*/
  unsigned int	display_status;		/* 68:				*/
  unsigned int	g7_unused[1];		/* 69:				*/

  /*
  ** Group 8 : RISC/SDRAM direct memory access 
  */
  unsigned int	dma_addrmode;		/* 70:				*/
  unsigned int	dma_yaddr;		/* 71:				*/
  unsigned int	dma_xaddr;		/* 72:				*/
  unsigned int	dma_addrlen;		/* 73: SRAM addr/len		*/
  unsigned int	dma_mode;		/* 74:				*/
  unsigned int	dma_done;		/* 75:	 dma done		*/
  unsigned int	g8_unused[2];		/* 76-77			*/

  /*
  ** Group 10 : OSD control 
  ** 3-regs
  ** 69-71
  */
  unsigned int	osd_en;			/* 78: 1:enable OSD		*/
  unsigned int	tlink_addr;		/* 79:				*/
  unsigned int	blink_addr;		/* 80:				*/
  unsigned int	g10_unused[1];		/* 81:				*/

#ifdef	UNIX
  unsigned int	unused[76];		/* 82 - 165			*/
  unsigned int	dsa_status;		/* 166				*/
  unsigned int	dsa_tx_reg;		/* 167				*/
  unsigned int	dsa_rx_reg;		/* 168				*/

#else
  unsigned int	unused[2];		/* 82-83			*/

  unsigned int	dvddsp_vy;		/* 84				*/
  unsigned int	dvddsp_vx;		/* 85				*/
  unsigned int	dvddsp_function;	/* 86				*/
  unsigned int	dvddsp_config;		/* 87				*/
  unsigned int	dvddsp_blocksize;	/* 88				*/
  unsigned int	dvddsp_blocklength;	/* 89				*/
  unsigned int	dvddsp_ry;		/* 90				*/
  unsigned int	dvddsp_rx;		/* 91				*/

  unsigned int	css_tk2;		/* 92				*/
  unsigned int	css_tk1;		/* 93				*/
  unsigned int	css_tk0;		/* 94				*/
  unsigned int	css_tbyte;		/* 95				*/

  /*
  ** UART0/1
  */
  unsigned int	uart0_data;             /*  96 : RBR_THR		*/
  unsigned int	uart0_lsr;              /*  97 : LSR			*/
  unsigned int        uart0_msr;              /*  98 : MSR			*/
  unsigned int        uart0_lcr;              /*  99 : LCR			*/
  unsigned int        uart0_mcr;              /* 100 : MCR			*/
  unsigned int        uart0_div_l;            /* 101 : DIV_L_AD		*/
  unsigned int        uart0_div_h;            /* 102 : DIV_H_AD		*/
  unsigned int        uart0_isc;              /* 103 : ISC_AD 		*/

  unsigned int	uart1_data;             /* 104 : RBR_THR		*/
  unsigned int	uart1_lsr;              /* 105 : LSR			*/
  unsigned int        uart1_msr;              /* 106 : MSR			*/
  unsigned int        uart1_lcr;              /* 107 : LCR			*/
  unsigned int        uart1_mcr;              /* 108 : MCR			*/
  unsigned int        uart1_div_l;            /* 109 : DIV_L_AD		*/
  unsigned int        uart1_div_h;            /* 110 : DIV_H_AD		*/
  unsigned int        uart1_isc;              /* 111 : ISC_AD 		*/

  /*
  ** DSP3 communication port
  */
  unsigned int	dsp24_port[16];		/* 112 - 127			*/

  /*
  ** I/O processor communication
  */
  unsigned int	iop_data[8];		/* 128 - 135			*/
  unsigned int	g20_unused[4];		/* 136 - 139			*/

  /*
  ** GROUP 40a: GPIO
  */
  unsigned int	gpio_sel;		// 140	control GPIO route
  unsigned int	gpio_sel_aux;		// 141	control GPIO route (aux)
  unsigned int	g40a_unused[2];		// 142 - 143

  /*
  ** Group 6: VPP Control
  */
  unsigned int	dis_x_start;		/* 144:				*/
  unsigned int	dis_y_start;		/* 145:				*/
  unsigned int	dis_x_size;		/* 146:				*/
  unsigned int	dis_y_size;		/* 147:				*/
  unsigned int	v_filter_mode;		/* 148: enable h/v filter	*/
  unsigned int	h_exp_mode;		/* 149:				*/ 
  unsigned int	h_offset;		/* 150:				*/
  unsigned int	v_offset;		/* 151:				*/

  unsigned int	luma_offset;		/* 152:				*/
  unsigned int	chroma_offset;		/* 153:				*/

  unsigned int	field_exchange;		/* 154:				*/
  unsigned int	chroma_field_rep;	/* 155:				*/

  unsigned int	g6_unused[4];		/* 156-159			*/

  /*
  ** Group 22: SUP
  */
  unsigned int	sup_mode;		/* 160: sup_mode		*/
  unsigned int	g22_unused[1];
  unsigned int	sup_cmd_addr;		/* 162: sup_cmd_addr		*/
  unsigned int	sup_pxcd_addr;		/* 163: sup_pxcd_addr		*/
  unsigned int	sup_hl0_tl_x;		/* 164: sup_hl0_tl_x		*/
  unsigned int	sup_hl0_tl_y;		/* 165: sup_hl0_tl_y		*/
  unsigned int	sup_hl0_br_x;		/* 166: 			*/
  unsigned int	sup_hl0_br_y;		/* 167: 			*/
  unsigned int	sup_hl0_col;		/* 168: 			*/
  unsigned int	sup_hl0_con;		/* 169: 			*/
  unsigned int	sup_pr_col;		/* 170: 			*/
  unsigned int	sup_pr_con;		/* 171: 			*/
#endif

  /*
  ** Group 14: DSP24
  */
  unsigned int	dsp24_control;		// 172
  unsigned int	dsp24_status;		// 173
  unsigned int	iop_control;		// 174
  unsigned int	iop_status;		// 175

  unsigned int	g14_unused[4];		// 176 - 179

  /* 
  ** Group 17: EPP
  */
  unsigned int	epp_status;		/* 180				*/
  unsigned int	epp_data;		/* 181				*/
  unsigned int	rom1_base;		/* 182 				*/
  unsigned int	rom2_base;		/* 183				*/

  /*
  ** Group 16 : AUDIO CLOCK SELECTION
  */
  unsigned int	audio_clkgen;		// 184
  unsigned int	g16_unused[1];

  /*
  ** Group 17 : Memory Configuration
  */
  unsigned int	video_compress;		// 186
  unsigned int	agdc_config;		// 187
  unsigned int	sdram_config;		// 188
  unsigned int	rom_config;		// 189
  unsigned int	watchdog;		// 190

  /*
  ** Group selector, fixed at 191
  */
  unsigned int	reg_page_num;		// 191: 0: (this-layout)

  /*
  ** GROUP 21: monitor
  */
  unsigned int	mon_mmu[2];		// 192-193
  unsigned int	mon_sys[2];		// 194-195
  unsigned int	mon_vld[4];		// 196-199
  unsigned int	mon_ogt[4];		// 200-203
  unsigned int	agdc_mon_lbound;	// 204
  unsigned int	agdc_mon_ubound;	// 205
  unsigned int	agdc_mon_ctrl;		// 206
  unsigned int	g21_unused[1];		// 207 - free

  /*
  ** GROUP 40b: GPIO
  */
  unsigned int	gpio_master[4];		// 208-211 1:RISC 0:IOP
  unsigned int	gpio_oe[4];		// 212-215 1:OUTPUT 0:INPUT
  unsigned int	gpio_out[4];		// 216-219 
  unsigned int	gpio_in[4];		// 220-223
  
  /*
  ** GROUP 50: DSP3 debug
  */
  unsigned int	dsp3dbg_ctrl;		// 224
  unsigned int	dsp3dbg_break;		// 225
  unsigned int	dsp3dbg_cmd;		// 226
  unsigned int	dsp3dbg_pc;		// 227
  unsigned int	dsp3dbg_rd_l;		// 228
  unsigned int	dsp3dbg_rd_h;		// 229
  unsigned int	dsp3dbg_wr_l;		// 230
  unsigned int	dsp3dbg_wr_h;		// 231
  
} RegisterFile;



#define SYSCLK          ((int)(27.0 * 4 * 1000000))
#define STC_DIVISOR     (((int)(SYSCLK/90000.0+0.5))-1)

/*
** 	TIMER
*/
#define	RF_TIMER_SRC_SYSCLK		(0<<14)
#define	RF_TIMER_SRC_STC		(1<<14)
#define	RF_TIMER_SRC_RTC		(2<<14)
#define	RF_TIMER_SRC_TIMER		(3<<14)
#define	RF_TIMER_RUN_ON			(1<<13)
#define	RF_TIMER_RUN_OFF		(0<<13)
#define	RF_TIMER_GO_ON			(1<<11)
#define	RF_TIMER_GO_OFF			(0<<11)
#define	RF_TIMER_MASK			(0x3ff)

#define	TIMER_CONFIG_STC		( RF_TIMER_SRC_STC	\
					| RF_TIMER_RUN_ON	\
					| RF_TIMER_GO_ON)

#define	TIMER_CONFIG_10ms		( TIMER_CONFIG_STC | (900-1))
#define	TIMER_CONFIG_100ms		( TIMER_CONFIG_STC | (9000-1))
#define	TIMER_CONFIG_4ms		( TIMER_CONFIG_STC | (360-1))
#define	TIMER_CONFIG_1ms		( TIMER_CONFIG_STC | (90-1))

#define	TIMER_CONFIG_90k(n)		( TIMER_CONFIG_STC | (n-1))

#define RGST_OFFSET	0xafff0000

/*
**      Video
*/
#define RF_CODING_EXT0_PROGRESSIVE_FRAME        (1<<0)
#define RF_CODING_EXT0_CHROMA_420_TYPE          (1<<1)
#define RF_CODING_EXT0_REPEAT_FIRST_FIELD       (1<<2)
#define RF_CODING_EXT0_ALTERNATE_SCAN           (1<<3)
#define RF_CODING_EXT0_INTRA_VLC_FORMAT         (1<<4)
#define RF_CODING_EXT0_Q_SCALE_TYPE             (1<<5)
#define RF_CODING_EXT0_CONCEAL_MOTION_VECTORS   (1<<6)
#define RF_CODING_EXT0_FRAME_PRED_FRAME_DCT     (1<<7)
#define RF_CODING_EXT0_TOP_FIELD_FIRST          (1<<8)
#define RF_CODING_EXT0_PICTURE_STRUCTURE        (0x03<<9)
#define RF_CODING_EXT0_INTRA_DC_PRECISION       (0x03<<11)
#define RF_CODING_EXT0_LAST_PICTURE             (1<<14)
#define RF_CODING_EXT0_SECOND_FIELD             (1<<15)

#define ext0_pic_struct(x)      (((x)>>9)&0x03)

/*
**      Display Output Tweaking
*/
#define RF_VOUT_SWAP_CBCR               (1<<1)
#define RF_VOUT_SWAP_LC                 (1<<2)

/*
**      Display Status
*/
#define RF_Display_OSDRegion            0x00ff
#define RF_Display_FieldNo              0x8000
#define RF_Display_VSyncB               0x4000
#define RF_Display_HSyncB               0x2000
#define RF_Display_FieldEnd             0x1000

#define IsVSync()            ((DISPLAY_STATUS & RF_Display_VSyncB)==0)
#define IsHSync()            ((DISPLAY_STATUS & RF_Display_HSyncB)==0)
#define IsTopField()         ((DISPLAY_STATUS & RF_Display_FieldNo)==0)
#define IsBottomField()      ((DISPLAY_STATUS & RF_Display_FieldNo))
#define IsFieldEnd()         ((DISPLAY_STATUS & RF_Display_FieldEnd))

#define RF_Video_VPicEnd                0x0001
#define RF_Video_VTblErr                0x0002
#define RF_Video_VRunErr                0x0004
#define RF_Video_VSliceErr              0x0008
#define RF_Video_VErr                   0x8000

#endif
