
/*  
 * Definitions used to for ATM/SAR module.
 *
 * Copyright (C) 2002 Viosoft Corp.
 * Support for TrendChip TC3160
 */

#ifndef __TC3160_ATM_H__ 
#define __TC3160_ATM_H__ 

#include <asm/addrspace.h>

/* Registers in hardware circuit (general) */
#define TC3160_ATM_HWC_BASE	(KSEG1 + 0x12001000)
#define TC3160_HWC_GCFG1_REG	(TC3160_ATM_HWC_BASE + 0x0)
#define TC3160_HWC_GCFG2_REG	(TC3160_ATM_HWC_BASE + 0x4)
#define TC3160_HWC_GCFG3_REG	(TC3160_ATM_HWC_BASE + 0x8)
#define TC3160_HWC_ISR_REG	(TC3160_ATM_HWC_BASE + 0xc)
#define TC3160_HWC_VC0_REG	(TC3160_ATM_HWC_BASE + 0x10)
#define TC3160_HWC_VC1_REG	(TC3160_ATM_HWC_BASE + 0x14)
#define TC3160_HWC_VC2_REG	(TC3160_ATM_HWC_BASE + 0x18)
#define TC3160_HWC_VC3_REG	(TC3160_ATM_HWC_BASE + 0x1c)
#define TC3160_HWC_VC4_REG	(TC3160_ATM_HWC_BASE + 0x20)
#define TC3160_HWC_VC5_REG	(TC3160_ATM_HWC_BASE + 0x24)
#define TC3160_HWC_VC6_REG	(TC3160_ATM_HWC_BASE + 0x28)
#define TC3160_HWC_VC7_REG	(TC3160_ATM_HWC_BASE + 0x2c)
#define TC3160_HWC_VC8_REG	(TC3160_ATM_HWC_BASE + 0x30)
#define TC3160_HWC_VC9_REG	(TC3160_ATM_HWC_BASE + 0x34)
#define TC3160_HWC_VC10_REG	(TC3160_ATM_HWC_BASE + 0x38)
#define TC3160_HWC_VC11_REG	(TC3160_ATM_HWC_BASE + 0x3c)
#define TC3160_HWC_VC12_REG	(TC3160_ATM_HWC_BASE + 0x40)
#define TC3160_HWC_VC13_REG	(TC3160_ATM_HWC_BASE + 0x44)
#define TC3160_HWC_VC14_REG	(TC3160_ATM_HWC_BASE + 0x48)
#define TC3160_HWC_VC15_REG	(TC3160_ATM_HWC_BASE + 0x4c)
#define TC3160_HWC_VC16_REG	(TC3160_ATM_HWC_BASE + 0x50)
#define TC3160_HWC_VC17_REG	(TC3160_ATM_HWC_BASE + 0x54)
#define TC3160_HWC_VC18_REG	(TC3160_ATM_HWC_BASE + 0x58)
#define TC3160_HWC_VC19_REG	(TC3160_ATM_HWC_BASE + 0x5c)

/* Registers in hardware circuit (transmit) */
#define TC3160_ATM_HWT_BASE	(KSEG1 + 0x12001100)
#define TC3160_HWT_CC_REG	(TC3160_ATM_HWT_BASE + 0x0)
#define TC3160_HWT_VC0_REG	(TC3160_ATM_HWT_BASE + 0x4)
#define TC3160_HWT_VC1_REG	(TC3160_ATM_HWT_BASE + 0x8)
#define TC3160_HWT_VC2_REG	(TC3160_ATM_HWT_BASE + 0xc)
#define TC3160_HWT_VC3_REG	(TC3160_ATM_HWT_BASE + 0x10)
#define TC3160_HWT_VC4_REG	(TC3160_ATM_HWT_BASE + 0x14)
#define TC3160_HWT_VC5_REG	(TC3160_ATM_HWT_BASE + 0x18)
#define TC3160_HWT_VC6_REG	(TC3160_ATM_HWT_BASE + 0x1c)
#define TC3160_HWT_VC7_REG	(TC3160_ATM_HWT_BASE + 0x20)
#define TC3160_HWT_VC8_REG	(TC3160_ATM_HWT_BASE + 0x24)
#define TC3160_HWT_VC9_REG	(TC3160_ATM_HWT_BASE + 0x28)
#define TC3160_HWT_VC10_REG	(TC3160_ATM_HWT_BASE + 0x2c)
#define TC3160_HWT_VC11_REG	(TC3160_ATM_HWT_BASE + 0x30)
#define TC3160_HWT_VC12_REG	(TC3160_ATM_HWT_BASE + 0x34)
#define TC3160_HWT_VC13_REG	(TC3160_ATM_HWT_BASE + 0x38)
#define TC3160_HWT_VC14_REG	(TC3160_ATM_HWT_BASE + 0x3c)
#define TC3160_HWT_VC15_REG	(TC3160_ATM_HWT_BASE + 0x40)
#define TC3160_HWT_VC16_REG	(TC3160_ATM_HWT_BASE + 0x44)
#define TC3160_HWT_VC17_REG	(TC3160_ATM_HWT_BASE + 0x48)
#define TC3160_HWT_VC18_REG	(TC3160_ATM_HWT_BASE + 0x4c)
#define TC3160_HWT_VC19_REG	(TC3160_ATM_HWT_BASE + 0x50)

/* Registers in SRAM0 */
#define TC3160_ATM_SRAM0_BASE		(KSEG1 + 0x12001600)

/* Registers in SRAM0 -- traffic control registers */
#define TC3160_SRAM0_CCTC_1_REG		(TC3160_ATM_SRAM0_BASE + 0x0)
#define TC3160_SRAM0_CCTC_2_REG		(TC3160_ATM_SRAM0_BASE + 0x8)
#define TC3160_SRAM0_VC0TC_1_REG	(TC3160_ATM_SRAM0_BASE + 0x10)
#define TC3160_SRAM0_VC0TC_2_REG	(TC3160_ATM_SRAM0_BASE + 0x18)
#define TC3160_SRAM0_VC1TC_1_REG	(TC3160_ATM_SRAM0_BASE + 0x20)
#define TC3160_SRAM0_VC1TC_2_REG	(TC3160_ATM_SRAM0_BASE + 0x28)
#define TC3160_SRAM0_VC2TC_1_REG	(TC3160_ATM_SRAM0_BASE + 0x30)
#define TC3160_SRAM0_VC2TC_2_REG	(TC3160_ATM_SRAM0_BASE + 0x38)
#define TC3160_SRAM0_VC3TC_1_REG	(TC3160_ATM_SRAM0_BASE + 0x40)
#define TC3160_SRAM0_VC3TC_2_REG	(TC3160_ATM_SRAM0_BASE + 0x48)
#define TC3160_SRAM0_VC4TC_1_REG	(TC3160_ATM_SRAM0_BASE + 0x50)
#define TC3160_SRAM0_VC4TC_2_REG	(TC3160_ATM_SRAM0_BASE + 0x58)
#define TC3160_SRAM0_VC5TC_1_REG	(TC3160_ATM_SRAM0_BASE + 0x60)
#define TC3160_SRAM0_VC5TC_2_REG	(TC3160_ATM_SRAM0_BASE + 0x68)
#define TC3160_SRAM0_VC6TC_1_REG	(TC3160_ATM_SRAM0_BASE + 0x70)
#define TC3160_SRAM0_VC6TC_2_REG	(TC3160_ATM_SRAM0_BASE + 0x78)
#define TC3160_SRAM0_VC7TC_1_REG	(TC3160_ATM_SRAM0_BASE + 0x80)
#define TC3160_SRAM0_VC7TC_2_REG	(TC3160_ATM_SRAM0_BASE + 0x88)
#define TC3160_SRAM0_VC8TC_1_REG	(TC3160_ATM_SRAM0_BASE + 0x90)
#define TC3160_SRAM0_VC8TC_2_REG	(TC3160_ATM_SRAM0_BASE + 0x98)
#define TC3160_SRAM0_VC9TC_1_REG	(TC3160_ATM_SRAM0_BASE + 0xa0)
#define TC3160_SRAM0_VC9TC_2_REG	(TC3160_ATM_SRAM0_BASE + 0xa8)
#define TC3160_SRAM0_VC10TC_1_REG	(TC3160_ATM_SRAM0_BASE + 0xb0)
#define TC3160_SRAM0_VC10TC_2_REG	(TC3160_ATM_SRAM0_BASE + 0xb8)
#define TC3160_SRAM0_VC11TC_1_REG	(TC3160_ATM_SRAM0_BASE + 0xc0)
#define TC3160_SRAM0_VC11TC_2_REG	(TC3160_ATM_SRAM0_BASE + 0xc8)
#define TC3160_SRAM0_VC12TC_1_REG	(TC3160_ATM_SRAM0_BASE + 0xd0)
#define TC3160_SRAM0_VC12TC_2_REG	(TC3160_ATM_SRAM0_BASE + 0xd8)
#define TC3160_SRAM0_VC13TC_1_REG	(TC3160_ATM_SRAM0_BASE + 0xe0)
#define TC3160_SRAM0_VC13TC_2_REG	(TC3160_ATM_SRAM0_BASE + 0xe8)
#define TC3160_SRAM0_VC14TC_1_REG	(TC3160_ATM_SRAM0_BASE + 0xf0)
#define TC3160_SRAM0_VC14TC_2_REG	(TC3160_ATM_SRAM0_BASE + 0xf8)
#define TC3160_SRAM0_VC15TC_1_REG	(TC3160_ATM_SRAM0_BASE + 0x100)
#define TC3160_SRAM0_VC15TC_2_REG	(TC3160_ATM_SRAM0_BASE + 0x108)
#define TC3160_SRAM0_VC16TC_1_REG	(TC3160_ATM_SRAM0_BASE + 0x110)
#define TC3160_SRAM0_VC16TC_2_REG	(TC3160_ATM_SRAM0_BASE + 0x118)
#define TC3160_SRAM0_VC17TC_1_REG	(TC3160_ATM_SRAM0_BASE + 0x120)
#define TC3160_SRAM0_VC17TC_2_REG	(TC3160_ATM_SRAM0_BASE + 0x128)
#define TC3160_SRAM0_VC18TC_1_REG	(TC3160_ATM_SRAM0_BASE + 0x130)
#define TC3160_SRAM0_VC18TC_2_REG	(TC3160_ATM_SRAM0_BASE + 0x138)
#define TC3160_SRAM0_VC19TC_1_REG	(TC3160_ATM_SRAM0_BASE + 0x140)
#define TC3160_SRAM0_VC19TC_2_REG	(TC3160_ATM_SRAM0_BASE + 0x148)

/* Registers in SRAM0 -- user data FIFO current transmit descriptor pointers */
#define TC3160_SRAM0_CC_FIFO_PTR	(TC3160_ATM_SRAM0_BASE + 0x150)
#define TC3160_SRAM0_VC0_FIFO_PTR	(TC3160_ATM_SRAM0_BASE + 0x154)
#define TC3160_SRAM0_VC1_FIFO_PTR	(TC3160_ATM_SRAM0_BASE + 0x158)
#define TC3160_SRAM0_VC2_FIFO_PTR	(TC3160_ATM_SRAM0_BASE + 0x15c)
#define TC3160_SRAM0_VC3_FIFO_PTR	(TC3160_ATM_SRAM0_BASE + 0x160)
#define TC3160_SRAM0_VC4_FIFO_PTR	(TC3160_ATM_SRAM0_BASE + 0x164)
#define TC3160_SRAM0_VC5_FIFO_PTR	(TC3160_ATM_SRAM0_BASE + 0x168)
#define TC3160_SRAM0_VC6_FIFO_PTR	(TC3160_ATM_SRAM0_BASE + 0x16c)
#define TC3160_SRAM0_VC7_FIFO_PTR	(TC3160_ATM_SRAM0_BASE + 0x170)
#define TC3160_SRAM0_VC8_FIFO_PTR	(TC3160_ATM_SRAM0_BASE + 0x174)
#define TC3160_SRAM0_VC9_FIFO_PTR	(TC3160_ATM_SRAM0_BASE + 0x178)
#define TC3160_SRAM0_VC10_FIFO_PTR	(TC3160_ATM_SRAM0_BASE + 0x17c)
#define TC3160_SRAM0_VC11_FIFO_PTR	(TC3160_ATM_SRAM0_BASE + 0x180)
#define TC3160_SRAM0_VC12_FIFO_PTR	(TC3160_ATM_SRAM0_BASE + 0x184)
#define TC3160_SRAM0_VC13_FIFO_PTR	(TC3160_ATM_SRAM0_BASE + 0x188)
#define TC3160_SRAM0_VC14_FIFO_PTR	(TC3160_ATM_SRAM0_BASE + 0x18c)
#define TC3160_SRAM0_VC15_FIFO_PTR	(TC3160_ATM_SRAM0_BASE + 0x190)
#define TC3160_SRAM0_VC16_FIFO_PTR	(TC3160_ATM_SRAM0_BASE + 0x194)
#define TC3160_SRAM0_VC17_FIFO_PTR	(TC3160_ATM_SRAM0_BASE + 0x198)
#define TC3160_SRAM0_VC18_FIFO_PTR	(TC3160_ATM_SRAM0_BASE + 0x19c)
#define TC3160_SRAM0_VC19_FIFO_PTR	(TC3160_ATM_SRAM0_BASE + 0x1a0)

/* Registers in SRAM0 -- menagement cell buffer pointers */
#define TC3160_SRAM0_VC0_CB_PTR		(TC3160_ATM_SRAM0_BASE + 0x1b0)
#define TC3160_SRAM0_VC1_CB_PTR		(TC3160_ATM_SRAM0_BASE + 0x1b4)
#define TC3160_SRAM0_VC2_CB_PTR		(TC3160_ATM_SRAM0_BASE + 0x1b8)
#define TC3160_SRAM0_VC3_CB_PTR		(TC3160_ATM_SRAM0_BASE + 0x1bc)
#define TC3160_SRAM0_VC4_CB_PTR		(TC3160_ATM_SRAM0_BASE + 0x1c0)
#define TC3160_SRAM0_VC5_CB_PTR		(TC3160_ATM_SRAM0_BASE + 0x1c4)
#define TC3160_SRAM0_VC6_CB_PTR		(TC3160_ATM_SRAM0_BASE + 0x1c8)
#define TC3160_SRAM0_VC7_CB_PTR		(TC3160_ATM_SRAM0_BASE + 0x1cc)
#define TC3160_SRAM0_VC8_CB_PTR		(TC3160_ATM_SRAM0_BASE + 0x1d0)
#define TC3160_SRAM0_VC9_CB_PTR		(TC3160_ATM_SRAM0_BASE + 0x1d4)
#define TC3160_SRAM0_VC10_CB_PTR	(TC3160_ATM_SRAM0_BASE + 0x1d8)
#define TC3160_SRAM0_VC11_CB_PTR	(TC3160_ATM_SRAM0_BASE + 0x1dc)
#define TC3160_SRAM0_VC12_CB_PTR	(TC3160_ATM_SRAM0_BASE + 0x1e0)
#define TC3160_SRAM0_VC13_CB_PTR	(TC3160_ATM_SRAM0_BASE + 0x1e4)
#define TC3160_SRAM0_VC14_CB_PTR	(TC3160_ATM_SRAM0_BASE + 0x1e8)
#define TC3160_SRAM0_VC15_CB_PTR	(TC3160_ATM_SRAM0_BASE + 0x1ec)
#define TC3160_SRAM0_VC16_CB_PTR	(TC3160_ATM_SRAM0_BASE + 0x1f0)
#define TC3160_SRAM0_VC17_CB_PTR	(TC3160_ATM_SRAM0_BASE + 0x1f4)
#define TC3160_SRAM0_VC18_CB_PTR	(TC3160_ATM_SRAM0_BASE + 0x1f8)
#define TC3160_SRAM0_VC19_CB_PTR	(TC3160_ATM_SRAM0_BASE + 0x1fc)

/* Registers in SRAM0 -- current transmit descriptor images */
#define TC3160_SRAM0_VC0_CTD_IMG	(TC3160_ATM_SRAM0_BASE + 0x200)
#define TC3160_SRAM0_VC1_CTD_IMG	(TC3160_ATM_SRAM0_BASE + 0x210)
#define TC3160_SRAM0_VC2_CTD_IMG	(TC3160_ATM_SRAM0_BASE + 0x220)
#define TC3160_SRAM0_VC3_CTD_IMG	(TC3160_ATM_SRAM0_BASE + 0x230)
#define TC3160_SRAM0_VC4_CTD_IMG	(TC3160_ATM_SRAM0_BASE + 0x240)
#define TC3160_SRAM0_VC5_CTD_IMG	(TC3160_ATM_SRAM0_BASE + 0x250)
#define TC3160_SRAM0_VC6_CTD_IMG	(TC3160_ATM_SRAM0_BASE + 0x260)
#define TC3160_SRAM0_VC7_CTD_IMG	(TC3160_ATM_SRAM0_BASE + 0x270)
#define TC3160_SRAM0_VC8_CTD_IMG	(TC3160_ATM_SRAM0_BASE + 0x280)
#define TC3160_SRAM0_VC9_CTD_IMG	(TC3160_ATM_SRAM0_BASE + 0x290)
#define TC3160_SRAM0_VC10_CTD_IMG	(TC3160_ATM_SRAM0_BASE + 0x2a0)
#define TC3160_SRAM0_VC11_CTD_IMG	(TC3160_ATM_SRAM0_BASE + 0x2b0)
#define TC3160_SRAM0_VC12_CTD_IMG	(TC3160_ATM_SRAM0_BASE + 0x2c0)
#define TC3160_SRAM0_VC13_CTD_IMG	(TC3160_ATM_SRAM0_BASE + 0x2d0)
#define TC3160_SRAM0_VC14_CTD_IMG	(TC3160_ATM_SRAM0_BASE + 0x2e0)
#define TC3160_SRAM0_VC15_CTD_IMG	(TC3160_ATM_SRAM0_BASE + 0x2f0)
#define TC3160_SRAM0_VC16_CTD_IMG	(TC3160_ATM_SRAM0_BASE + 0x300)
#define TC3160_SRAM0_VC17_CTD_IMG	(TC3160_ATM_SRAM0_BASE + 0x310)
#define TC3160_SRAM0_VC18_CTD_IMG	(TC3160_ATM_SRAM0_BASE + 0x320)
#define TC3160_SRAM0_VC19_CTD_IMG	(TC3160_ATM_SRAM0_BASE + 0x330)

/* Registers in SRAM0 -- user data transmit traffic statistics registers */
#define TC3160_SRAM0_VC0_TTS_REG	(TC3160_ATM_SRAM0_BASE + 0x340)
#define TC3160_SRAM0_VC1_TTS_REG	(TC3160_ATM_SRAM0_BASE + 0x344)
#define TC3160_SRAM0_VC2_TTS_REG	(TC3160_ATM_SRAM0_BASE + 0x348)
#define TC3160_SRAM0_VC3_TTS_REG	(TC3160_ATM_SRAM0_BASE + 0x34c)
#define TC3160_SRAM0_VC4_TTS_REG	(TC3160_ATM_SRAM0_BASE + 0x350)
#define TC3160_SRAM0_VC5_TTS_REG	(TC3160_ATM_SRAM0_BASE + 0x354)
#define TC3160_SRAM0_VC6_TTS_REG	(TC3160_ATM_SRAM0_BASE + 0x358)
#define TC3160_SRAM0_VC7_TTS_REG	(TC3160_ATM_SRAM0_BASE + 0x35c)
#define TC3160_SRAM0_VC8_TTS_REG	(TC3160_ATM_SRAM0_BASE + 0x360)
#define TC3160_SRAM0_VC9_TTS_REG	(TC3160_ATM_SRAM0_BASE + 0x364)
#define TC3160_SRAM0_VC10_TTS_REG	(TC3160_ATM_SRAM0_BASE + 0x368)
#define TC3160_SRAM0_VC11_TTS_REG	(TC3160_ATM_SRAM0_BASE + 0x36c)
#define TC3160_SRAM0_VC12_TTS_REG	(TC3160_ATM_SRAM0_BASE + 0x370)
#define TC3160_SRAM0_VC13_TTS_REG	(TC3160_ATM_SRAM0_BASE + 0x374)
#define TC3160_SRAM0_VC14_TTS_REG	(TC3160_ATM_SRAM0_BASE + 0x378)
#define TC3160_SRAM0_VC15_TTS_REG	(TC3160_ATM_SRAM0_BASE + 0x37c)
#define TC3160_SRAM0_VC16_TTS_REG	(TC3160_ATM_SRAM0_BASE + 0x380)
#define TC3160_SRAM0_VC17_TTS_REG	(TC3160_ATM_SRAM0_BASE + 0x384)
#define TC3160_SRAM0_VC18_TTS_REG	(TC3160_ATM_SRAM0_BASE + 0x388)
#define TC3160_SRAM0_VC19_TTS_REG	(TC3160_ATM_SRAM0_BASE + 0x38c)

/* Registers in SRAM0 -- AAL5 payload trailer registers */
#define TC3160_SRAM0_VC0_AALPT_REG	(TC3160_ATM_SRAM0_BASE + 0x390)
#define TC3160_SRAM0_VC1_AALPT_REG	(TC3160_ATM_SRAM0_BASE + 0x394)
#define TC3160_SRAM0_VC2_AALPT_REG	(TC3160_ATM_SRAM0_BASE + 0x398)
#define TC3160_SRAM0_VC3_AALPT_REG	(TC3160_ATM_SRAM0_BASE + 0x39c)
#define TC3160_SRAM0_VC4_AALPT_REG	(TC3160_ATM_SRAM0_BASE + 0x3a0)
#define TC3160_SRAM0_VC5_AALPT_REG	(TC3160_ATM_SRAM0_BASE + 0x3a4)
#define TC3160_SRAM0_VC6_AALPT_REG	(TC3160_ATM_SRAM0_BASE + 0x3a8)
#define TC3160_SRAM0_VC7_AALPT_REG	(TC3160_ATM_SRAM0_BASE + 0x3ac)
#define TC3160_SRAM0_VC8_AALPT_REG	(TC3160_ATM_SRAM0_BASE + 0x3b0)
#define TC3160_SRAM0_VC9_AALPT_REG	(TC3160_ATM_SRAM0_BASE + 0x3b4)
#define TC3160_SRAM0_VC10_AALPT_REG	(TC3160_ATM_SRAM0_BASE + 0x3b8)
#define TC3160_SRAM0_VC11_AALPT_REG	(TC3160_ATM_SRAM0_BASE + 0x3bc)
#define TC3160_SRAM0_VC12_AALPT_REG	(TC3160_ATM_SRAM0_BASE + 0x3c0)
#define TC3160_SRAM0_VC13_AALPT_REG	(TC3160_ATM_SRAM0_BASE + 0x3c4)
#define TC3160_SRAM0_VC14_AALPT_REG	(TC3160_ATM_SRAM0_BASE + 0x3c8)
#define TC3160_SRAM0_VC15_AALPT_REG	(TC3160_ATM_SRAM0_BASE + 0x3cc)
#define TC3160_SRAM0_VC16_AALPT_REG	(TC3160_ATM_SRAM0_BASE + 0x3d0)
#define TC3160_SRAM0_VC17_AALPT_REG	(TC3160_ATM_SRAM0_BASE + 0x3d4)
#define TC3160_SRAM0_VC18_AALPT_REG	(TC3160_ATM_SRAM0_BASE + 0x3d8)
#define TC3160_SRAM0_VC19_AALPT_REG	(TC3160_ATM_SRAM0_BASE + 0x3dc)

/* Registers in SRAM0 -- CRC accumulators for transmit */
#define TC3160_SRAM0_VC0_CRC_ACMR	(TC3160_ATM_SRAM0_BASE + 0x3e0)
#define TC3160_SRAM0_VC1_CRC_ACMR	(TC3160_ATM_SRAM0_BASE + 0x3e4)
#define TC3160_SRAM0_VC2_CRC_ACMR	(TC3160_ATM_SRAM0_BASE + 0x3e8)
#define TC3160_SRAM0_VC3_CRC_ACMR	(TC3160_ATM_SRAM0_BASE + 0x3ec)
#define TC3160_SRAM0_VC4_CRC_ACMR	(TC3160_ATM_SRAM0_BASE + 0x3f0)
#define TC3160_SRAM0_VC5_CRC_ACMR	(TC3160_ATM_SRAM0_BASE + 0x3f4)
#define TC3160_SRAM0_VC6_CRC_ACMR	(TC3160_ATM_SRAM0_BASE + 0x3f8)
#define TC3160_SRAM0_VC7_CRC_ACMR	(TC3160_ATM_SRAM0_BASE + 0x3fc)
#define TC3160_SRAM0_VC8_CRC_ACMR	(TC3160_ATM_SRAM0_BASE + 0x400)
#define TC3160_SRAM0_VC9_CRC_ACMR	(TC3160_ATM_SRAM0_BASE + 0x404)
#define TC3160_SRAM0_VC10_CRC_ACMR	(TC3160_ATM_SRAM0_BASE + 0x408)
#define TC3160_SRAM0_VC11_CRC_ACMR	(TC3160_ATM_SRAM0_BASE + 0x40c)
#define TC3160_SRAM0_VC12_CRC_ACMR	(TC3160_ATM_SRAM0_BASE + 0x410)
#define TC3160_SRAM0_VC13_CRC_ACMR	(TC3160_ATM_SRAM0_BASE + 0x414)
#define TC3160_SRAM0_VC14_CRC_ACMR	(TC3160_ATM_SRAM0_BASE + 0x418)
#define TC3160_SRAM0_VC15_CRC_ACMR	(TC3160_ATM_SRAM0_BASE + 0x41c)
#define TC3160_SRAM0_VC16_CRC_ACMR	(TC3160_ATM_SRAM0_BASE + 0x420)
#define TC3160_SRAM0_VC17_CRC_ACMR	(TC3160_ATM_SRAM0_BASE + 0x424)
#define TC3160_SRAM0_VC18_CRC_ACMR	(TC3160_ATM_SRAM0_BASE + 0x428)
#define TC3160_SRAM0_VC19_CRC_ACMR	(TC3160_ATM_SRAM0_BASE + 0x42c)

/* Registers in SRAM0 -- AAL5 transfer counts */
#define TC3160_SRAM0_VC0_AAL_TC		(TC3160_ATM_SRAM0_BASE + 0x430)
#define TC3160_SRAM0_VC1_AAL_TC		(TC3160_ATM_SRAM0_BASE + 0x434)
#define TC3160_SRAM0_VC2_AAL_TC		(TC3160_ATM_SRAM0_BASE + 0x438)
#define TC3160_SRAM0_VC3_AAL_TC		(TC3160_ATM_SRAM0_BASE + 0x43c)
#define TC3160_SRAM0_VC4_AAL_TC		(TC3160_ATM_SRAM0_BASE + 0x440)
#define TC3160_SRAM0_VC5_AAL_TC		(TC3160_ATM_SRAM0_BASE + 0x444)
#define TC3160_SRAM0_VC6_AAL_TC		(TC3160_ATM_SRAM0_BASE + 0x448)
#define TC3160_SRAM0_VC7_AAL_TC		(TC3160_ATM_SRAM0_BASE + 0x44c)
#define TC3160_SRAM0_VC8_AAL_TC		(TC3160_ATM_SRAM0_BASE + 0x450)
#define TC3160_SRAM0_VC9_AAL_TC		(TC3160_ATM_SRAM0_BASE + 0x454)
#define TC3160_SRAM0_VC10_AAL_TC	(TC3160_ATM_SRAM0_BASE + 0x458)
#define TC3160_SRAM0_VC11_AAL_TC	(TC3160_ATM_SRAM0_BASE + 0x45c)
#define TC3160_SRAM0_VC12_AAL_TC	(TC3160_ATM_SRAM0_BASE + 0x460)
#define TC3160_SRAM0_VC13_AAL_TC	(TC3160_ATM_SRAM0_BASE + 0x464)
#define TC3160_SRAM0_VC14_AAL_TC	(TC3160_ATM_SRAM0_BASE + 0x468)
#define TC3160_SRAM0_VC15_AAL_TC	(TC3160_ATM_SRAM0_BASE + 0x46c)
#define TC3160_SRAM0_VC16_AAL_TC	(TC3160_ATM_SRAM0_BASE + 0x470)
#define TC3160_SRAM0_VC17_AAL_TC	(TC3160_ATM_SRAM0_BASE + 0x474)
#define TC3160_SRAM0_VC18_AAL_TC	(TC3160_ATM_SRAM0_BASE + 0x478)
#define TC3160_SRAM0_VC19_AAL_TC	(TC3160_ATM_SRAM0_BASE + 0x47c)

/* Registers in hardware circuit (receive) */
#define TC3160_ATM_HWR_BASE	(KSEG1 + 0x12001200)
#define TC3160_HWR_CC_REG	(TC3160_ATM_HWR_BASE + 0x0)
#define TC3160_HWR_VC0_REG	(TC3160_ATM_HWR_BASE + 0x4)
#define TC3160_HWR_VC1_REG	(TC3160_ATM_HWR_BASE + 0x8)
#define TC3160_HWR_VC2_REG	(TC3160_ATM_HWR_BASE + 0xc)
#define TC3160_HWR_VC3_REG	(TC3160_ATM_HWR_BASE + 0x10)
#define TC3160_HWR_VC4_REG	(TC3160_ATM_HWR_BASE + 0x14)
#define TC3160_HWR_VC5_REG	(TC3160_ATM_HWR_BASE + 0x18)
#define TC3160_HWR_VC6_REG	(TC3160_ATM_HWR_BASE + 0x1c)
#define TC3160_HWR_VC7_REG	(TC3160_ATM_HWR_BASE + 0x20)
#define TC3160_HWR_VC8_REG	(TC3160_ATM_HWR_BASE + 0x24)
#define TC3160_HWR_VC9_REG	(TC3160_ATM_HWR_BASE + 0x28)
#define TC3160_HWR_VC10_REG	(TC3160_ATM_HWR_BASE + 0x2c)
#define TC3160_HWR_VC11_REG	(TC3160_ATM_HWR_BASE + 0x30)
#define TC3160_HWR_VC12_REG	(TC3160_ATM_HWR_BASE + 0x34)
#define TC3160_HWR_VC13_REG	(TC3160_ATM_HWR_BASE + 0x38)
#define TC3160_HWR_VC14_REG	(TC3160_ATM_HWR_BASE + 0x3c)
#define TC3160_HWR_VC15_REG	(TC3160_ATM_HWR_BASE + 0x40)
#define TC3160_HWR_VC16_REG	(TC3160_ATM_HWR_BASE + 0x44)
#define TC3160_HWR_VC17_REG	(TC3160_ATM_HWR_BASE + 0x48)
#define TC3160_HWR_VC18_REG	(TC3160_ATM_HWR_BASE + 0x4c)
#define TC3160_HWR_VC19_REG	(TC3160_ATM_HWR_BASE + 0x50)
#define TC3160_HWR_EC_CNTR	(TC3160_ATM_HWR_BASE + 0x54)
#define TC3160_HWR_IRQ_QLB_REG	(TC3160_ATM_HWR_BASE + 0x58)
#define TC3160_HWR_IRQ_QHB_REG	(TC3160_ATM_HWR_BASE + 0x5c)
#define TC3160_HWR_IRQ_QHA_REG	(TC3160_ATM_HWR_BASE + 0x60)
#define TC3160_HWR_IRQ_QTA_REG	(TC3160_ATM_HWR_BASE + 0x64)
#define TC3160_HWR_IRQ_QS_REG	(TC3160_ATM_HWR_BASE + 0x68)
#define TC3160_HWR_IRQ_QC_REG	(TC3160_ATM_HWR_BASE + 0x6c)

/* Registers in SRAM1 */
#define TC3160_ATM_SRAM1_BASE		(KSEG1 + 0x12001c00)

/* Registers in SRAM1 -- user data FIFO current receive descriptor pointers */
#define TC3160_SRAM1_CC_FIFO_PTR	(TC3160_ATM_SRAM1_BASE + 0x0)
#define TC3160_SRAM1_VC0_FIFO_PTR	(TC3160_ATM_SRAM1_BASE + 0x4)
#define TC3160_SRAM1_VC1_FIFO_PTR	(TC3160_ATM_SRAM1_BASE + 0x8)
#define TC3160_SRAM1_VC2_FIFO_PTR	(TC3160_ATM_SRAM1_BASE + 0xc)
#define TC3160_SRAM1_VC3_FIFO_PTR	(TC3160_ATM_SRAM1_BASE + 0x10)
#define TC3160_SRAM1_VC4_FIFO_PTR	(TC3160_ATM_SRAM1_BASE + 0x14)
#define TC3160_SRAM1_VC5_FIFO_PTR	(TC3160_ATM_SRAM1_BASE + 0x18)
#define TC3160_SRAM1_VC6_FIFO_PTR	(TC3160_ATM_SRAM1_BASE + 0x1c)
#define TC3160_SRAM1_VC7_FIFO_PTR	(TC3160_ATM_SRAM1_BASE + 0x20)
#define TC3160_SRAM1_VC8_FIFO_PTR	(TC3160_ATM_SRAM1_BASE + 0x24)
#define TC3160_SRAM1_VC9_FIFO_PTR	(TC3160_ATM_SRAM1_BASE + 0x28)
#define TC3160_SRAM1_VC10_FIFO_PTR	(TC3160_ATM_SRAM1_BASE + 0x2c)
#define TC3160_SRAM1_VC11_FIFO_PTR	(TC3160_ATM_SRAM1_BASE + 0x30)
#define TC3160_SRAM1_VC12_FIFO_PTR	(TC3160_ATM_SRAM1_BASE + 0x34)
#define TC3160_SRAM1_VC13_FIFO_PTR	(TC3160_ATM_SRAM1_BASE + 0x38)
#define TC3160_SRAM1_VC14_FIFO_PTR	(TC3160_ATM_SRAM1_BASE + 0x3c)
#define TC3160_SRAM1_VC15_FIFO_PTR	(TC3160_ATM_SRAM1_BASE + 0x40)
#define TC3160_SRAM1_VC16_FIFO_PTR	(TC3160_ATM_SRAM1_BASE + 0x44)
#define TC3160_SRAM1_VC17_FIFO_PTR	(TC3160_ATM_SRAM1_BASE + 0x48)
#define TC3160_SRAM1_VC18_FIFO_PTR	(TC3160_ATM_SRAM1_BASE + 0x4c)
#define TC3160_SRAM1_VC19_FIFO_PTR	(TC3160_ATM_SRAM1_BASE + 0x50)

/* Registers in SRAM1 -- current receive descriptor images */
#define TC3160_SRAM1_VC0_CRD_IMG	(TC3160_ATM_SRAM1_BASE + 0x70)
#define TC3160_SRAM1_VC1_CRD_IMG	(TC3160_ATM_SRAM1_BASE + 0x80)
#define TC3160_SRAM1_VC2_CRD_IMG	(TC3160_ATM_SRAM1_BASE + 0x90)
#define TC3160_SRAM1_VC3_CRD_IMG	(TC3160_ATM_SRAM1_BASE + 0xa0)
#define TC3160_SRAM1_VC4_CRD_IMG	(TC3160_ATM_SRAM1_BASE + 0xb0)
#define TC3160_SRAM1_VC5_CRD_IMG	(TC3160_ATM_SRAM1_BASE + 0xc0)
#define TC3160_SRAM1_VC6_CRD_IMG	(TC3160_ATM_SRAM1_BASE + 0xd0)
#define TC3160_SRAM1_VC7_CRD_IMG	(TC3160_ATM_SRAM1_BASE + 0xe0)
#define TC3160_SRAM1_VC8_CRD_IMG	(TC3160_ATM_SRAM1_BASE + 0xf0)
#define TC3160_SRAM1_VC9_CRD_IMG	(TC3160_ATM_SRAM1_BASE + 0x100)
#define TC3160_SRAM1_VC10_CRD_IMG	(TC3160_ATM_SRAM1_BASE + 0x110)
#define TC3160_SRAM1_VC11_CRD_IMG	(TC3160_ATM_SRAM1_BASE + 0x120)
#define TC3160_SRAM1_VC12_CRD_IMG	(TC3160_ATM_SRAM1_BASE + 0x130)
#define TC3160_SRAM1_VC13_CRD_IMG	(TC3160_ATM_SRAM1_BASE + 0x140)
#define TC3160_SRAM1_VC14_CRD_IMG	(TC3160_ATM_SRAM1_BASE + 0x150)
#define TC3160_SRAM1_VC15_CRD_IMG	(TC3160_ATM_SRAM1_BASE + 0x160)
#define TC3160_SRAM1_VC16_CRD_IMG	(TC3160_ATM_SRAM1_BASE + 0x170)
#define TC3160_SRAM1_VC17_CRD_IMG	(TC3160_ATM_SRAM1_BASE + 0x180)
#define TC3160_SRAM1_VC18_CRD_IMG	(TC3160_ATM_SRAM1_BASE + 0x190)
#define TC3160_SRAM1_VC19_CRD_IMG	(TC3160_ATM_SRAM1_BASE + 0x1a0)

/* Registers in SRAM1 -- user data receive traffic statistics registers */
#define TC3160_SRAM1_VC0_RTS_REG	(TC3160_ATM_SRAM1_BASE + 0x1b0)
#define TC3160_SRAM1_VC1_RTS_REG	(TC3160_ATM_SRAM1_BASE + 0x1b4)
#define TC3160_SRAM1_VC2_RTS_REG	(TC3160_ATM_SRAM1_BASE + 0x1b8)
#define TC3160_SRAM1_VC3_RTS_REG	(TC3160_ATM_SRAM1_BASE + 0x1bc)
#define TC3160_SRAM1_VC4_RTS_REG	(TC3160_ATM_SRAM1_BASE + 0x1c0)
#define TC3160_SRAM1_VC5_RTS_REG	(TC3160_ATM_SRAM1_BASE + 0x1c4)
#define TC3160_SRAM1_VC6_RTS_REG	(TC3160_ATM_SRAM1_BASE + 0x1c8)
#define TC3160_SRAM1_VC7_RTS_REG	(TC3160_ATM_SRAM1_BASE + 0x1cc)
#define TC3160_SRAM1_VC8_RTS_REG	(TC3160_ATM_SRAM1_BASE + 0x1d0)
#define TC3160_SRAM1_VC9_RTS_REG	(TC3160_ATM_SRAM1_BASE + 0x1d4)
#define TC3160_SRAM1_VC10_RTS_REG	(TC3160_ATM_SRAM1_BASE + 0x1d8)
#define TC3160_SRAM1_VC11_RTS_REG	(TC3160_ATM_SRAM1_BASE + 0x1dc)
#define TC3160_SRAM1_VC12_RTS_REG	(TC3160_ATM_SRAM1_BASE + 0x1e0)
#define TC3160_SRAM1_VC13_RTS_REG	(TC3160_ATM_SRAM1_BASE + 0x1e4)
#define TC3160_SRAM1_VC14_RTS_REG	(TC3160_ATM_SRAM1_BASE + 0x1e8)
#define TC3160_SRAM1_VC15_RTS_REG	(TC3160_ATM_SRAM1_BASE + 0x1ec)
#define TC3160_SRAM1_VC16_RTS_REG	(TC3160_ATM_SRAM1_BASE + 0x1f0)
#define TC3160_SRAM1_VC17_RTS_REG	(TC3160_ATM_SRAM1_BASE + 0x1f4)
#define TC3160_SRAM1_VC18_RTS_REG	(TC3160_ATM_SRAM1_BASE + 0x1f8)
#define TC3160_SRAM1_VC19_RTS_REG	(TC3160_ATM_SRAM1_BASE + 0x1fc)

/* Registers in SRAM1 -- CRC accumulators for receive */
#define TC3160_SRAM1_VC0_CRC_ACMR	(TC3160_ATM_SRAM1_BASE + 0x200)
#define TC3160_SRAM1_VC1_CRC_ACMR	(TC3160_ATM_SRAM1_BASE + 0x204)
#define TC3160_SRAM1_VC2_CRC_ACMR	(TC3160_ATM_SRAM1_BASE + 0x208)
#define TC3160_SRAM1_VC3_CRC_ACMR	(TC3160_ATM_SRAM1_BASE + 0x20c)
#define TC3160_SRAM1_VC4_CRC_ACMR	(TC3160_ATM_SRAM1_BASE + 0x210)
#define TC3160_SRAM1_VC5_CRC_ACMR	(TC3160_ATM_SRAM1_BASE + 0x214)
#define TC3160_SRAM1_VC6_CRC_ACMR	(TC3160_ATM_SRAM1_BASE + 0x218)
#define TC3160_SRAM1_VC7_CRC_ACMR	(TC3160_ATM_SRAM1_BASE + 0x21c)
#define TC3160_SRAM1_VC8_CRC_ACMR	(TC3160_ATM_SRAM1_BASE + 0x220)
#define TC3160_SRAM1_VC9_CRC_ACMR	(TC3160_ATM_SRAM1_BASE + 0x224)
#define TC3160_SRAM1_VC10_CRC_ACMR	(TC3160_ATM_SRAM1_BASE + 0x228)
#define TC3160_SRAM1_VC11_CRC_ACMR	(TC3160_ATM_SRAM1_BASE + 0x22c)
#define TC3160_SRAM1_VC12_CRC_ACMR	(TC3160_ATM_SRAM1_BASE + 0x230)
#define TC3160_SRAM1_VC13_CRC_ACMR	(TC3160_ATM_SRAM1_BASE + 0x234)
#define TC3160_SRAM1_VC14_CRC_ACMR	(TC3160_ATM_SRAM1_BASE + 0x238)
#define TC3160_SRAM1_VC15_CRC_ACMR	(TC3160_ATM_SRAM1_BASE + 0x23c)
#define TC3160_SRAM1_VC16_CRC_ACMR	(TC3160_ATM_SRAM1_BASE + 0x240)
#define TC3160_SRAM1_VC17_CRC_ACMR	(TC3160_ATM_SRAM1_BASE + 0x244)
#define TC3160_SRAM1_VC18_CRC_ACMR	(TC3160_ATM_SRAM1_BASE + 0x248)
#define TC3160_SRAM1_VC19_CRC_ACMR	(TC3160_ATM_SRAM1_BASE + 0x24c)

#ifndef __ASSEMBLY__
#endif /* !__ASSEMBLY__ */

#endif /* __TC3160_ATM_H__ */

