
/*  
 * Definitions used to for ethernet module.
 *
 * Copyright (C) 2002 Viosoft Corp.
 * Support for TrendChip TC3160
 */

#ifndef __TC3160_ETH_H__ 
#define __TC3160_ETH_H__ 

#include <asm/addrspace.h>

#define TC3160_ETHDMA_BASE	(KSEG1 + 0x12000180)
#define TC3160_BMR_REG		(TC3160_ETHDMA_BASE + 0x0)
#define TC3160_TPDR_REG		(TC3160_ETHDMA_BASE + 0x4)
#define TC3160_RPDR_REG		(TC3160_ETHDMA_BASE + 0x8)
#define TC3160_RLBAR_REG	(TC3160_ETHDMA_BASE + 0xc)
#define TC3160_TLBAR_REG	(TC3160_ETHDMA_BASE + 0x10)
#define TC3160_SR_REG		(TC3160_ETHDMA_BASE + 0x14)
#define TC3160_CR_REG		(TC3160_ETHDMA_BASE + 0x18)
#define TC3160_IER_REG		(TC3160_ETHDMA_BASE + 0x1c)
#define TC3160_MFCR_REG		(TC3160_ETHDMA_BASE + 0x20)
#define TC3160_IFR_REG		(TC3160_ETHDMA_BASE + 0x24)
#define TC3160_TDVPAR_REG	(TC3160_ETHDMA_BASE + 0x4c)
#define TC3160_CHTBAR_REG	(TC3160_ETHDMA_BASE + 0x50)
#define TC3160_CHRBAR_REG	(TC3160_ETHDMA_BASE + 0x54)

#define TC3160_ETHMAC_BASE	(KSEG1 + 0x12000100)
#define TC3160_MACCR_REG	(TC3160_ETHMAC_BASE + 0x0)
#define TC3160_MACAHR_REG	(TC3160_ETHMAC_BASE + 0x4)
#define TC3160_MACALR_REG	(TC3160_ETHMAC_BASE + 0x8)
#define TC3160_MCHTHR_REG	(TC3160_ETHMAC_BASE + 0xc)
#define TC3160_MCHTLR_REG	(TC3160_ETHMAC_BASE + 0x10)
#define TC3160_MIIAR_REG	(TC3160_ETHMAC_BASE + 0x14)
#define TC3160_MIIDR_REG	(TC3160_ETHMAC_BASE + 0x18)
#define TC3160_FCR_REG		(TC3160_ETHMAC_BASE + 0x1c)
#define TC3160_V1TR_REG		(TC3160_ETHMAC_BASE + 0x20)
#define TC3160_V2TR_REG		(TC3160_ETHMAC_BASE + 0x24)

/* MII interface */
#define MiiIfSpeed10M     	0x00000000
#define MiiIfSpeed100M		0x00000004
#define MiiIfExtPhyMii		0x00000000
#define MiiIfExtPhyRmii		0x00000001
#define MiiIfIntPhy    		0x00000002

#ifndef __ASSEMBLY__ 

/**********************************************************
 * MAC110 Network interface registers
 **********************************************************/

enum MacControlReg      /* MAC Control register layout */
{                                         /* Bit description                        R/W   Reset value */
  MacFilterOff            = 0x80000000,     /* Receive all incoming packets         RW                */
  MacFilterOn             = 0,              /* Receive filtered packets only                  0       */

  MacBigEndian            = 0x40000000,     /* Big endian mode                      RW                */
  MacLittleEndian         = 0,              /* Little endian                                  0       */

  MacHeartBeatOff         = 0x10000000,     /* Heart beat signal quality disable    RW                */
  MacHeartBeatOn          = 0,              /* Heart beat signal quality enable               0       */

  MacSelectSrl            = 0x08000000,     /* Select SRL port                      RW                */
  MacSelectMii            = 0,              /* Select MII port                                0       */

  MacDisableRxOwn         = 0x00800000,     /* Disable receive own packets          RW                */
  MacEnableRxOwn          = 0,              /* Enable receive own packets                     0       */

  MacLoopbackExt          = 0x00400000,     /* External loopback                    RW                */
  MacLoopbackInt          = 0x00200000,     /* Internal loopback                                      */
  MacLoopbackOff          = 0,              /* Normal mode                                    00      */

  MacFullDuplex           = 0x00100000,     /* Full duplex mode                     RW                */
  MacHalfDuplex           = 0,              /* Half duplex mode                               0       */

  MacMulticastFilterOff   = 0x00080000,     /* Pass all multicast packets           RW                */
  MacMulticastFilterOn    = 0,              /* Pass filtered multicast packets                0       */

  MacPromiscuousModeOn    = 0x00040000,     /* Receive all valid packets            RW        1       */
  MacPromiscuousModeOff   = 0,              /* Receive filtered packets only                          */

  MacFilterInverce        = 0x00020000,     /* Inverse filtering                    RW                */
  MacFilterNormal         = 0,              /* Normal filtering                               0       */

  MacBadFramesEnable      = 0x00010000,     /* Pass bad frames                      RW                */
  MacBadFramesDisable     = 0,              /* Do not pass bad frames                         0       */

  MacPerfectFilterOff     = 0x00008000,     /* Hash filtering only                  RW                */
  MacPerfectFilterOn      = 0,              /* Both perfect and hash filtering                0       */

  MacHashFilterOn         = 0x00002000,     /* perfom hash filtering                RW                */
  MacHashFilterOff        = 0,              /* perfect filtering only                         0       */

  MacLateCollisionOn      = 0x00001000,     /* Enable late collision control        RW                */
  MacLateCollisionOff     = 0,              /* Disable late collision control                 0       */

  MacBroadcastDisable     = 0x00000800,     /* Disable reception of broadcast frames RW               */
  MacBroadcastEnable      = 0,              /* Enable broadcast frames                        0       */

  MacRetryDisable         = 0x00000400,     /* Disable retransmission               RW                */
  MacRetryEnable          = 0,              /* Enable retransmission                          0       */

  MacPadStripEnable       = 0x00000100,     /* Pad stripping enable                 RW                */
  MacPadStripDisable      = 0,              /* Pad stripping disable                          0       */

  MacBackoff10            = 0,              /* Backoff Limit (not documented)       RW        00      */
  MacBackoff8             = 0x00000040,     /* Backoff Limit (not documented)       RW        01      */
  MacBackoff4             = 0x00000080,     /* Backoff Limit (not documented)       RW        10      */
  MacBackoff1             = 0x000000c0,     /* Backoff Limit (not documented)       RW        11      */

  MacDeferralCheckEnable  = 0x00000020,     /* Deferral check enable                RW                */
  MacDeferralCheckDisable = 0,              /* Deferral check disable                         0       */

  MacTxEnable             = 0x00000008,     /* Transmitter enable                   RW                */
  MacTxDisable            = 0,              /* Transmitter disable                            0       */

  MacRxEnable             = 0x00000004,     /* Receiver enable                      RW                */
  MacRxDisable            = 0,              /* Receiver disable                               0       */
};

/* Descriptor related ... */
struct tc3160_desc {
  volatile unsigned long desc0;
  volatile unsigned long desc1;
  volatile unsigned long desc2;
  volatile unsigned long desc3;
};

static inline int tc3160_rdesc_owned_by_host(struct tc3160_desc *desc)
{
  return((desc->desc0 & 0x80000000) ? 0 : 1);
}

static inline int tc3160_rdesc_last_desc(struct tc3160_desc *desc)
{
  return(desc->desc0 & 0x00000100);
}

static inline int tc3160_rdesc_first_desc(struct tc3160_desc *desc)
{
  return(desc->desc0 & 0x00000200);
}

static inline unsigned long tc3160_rdesc_frame_len(struct tc3160_desc *desc)
{
  return((desc->desc0 & 0x3fff0000) >> 16);
}

static inline int tc3160_tdesc_owned_by_host(struct tc3160_desc *desc)
{
  return((desc->desc0 & 0x80000000) ? 0 : 1);
}

#endif /* !__ASSEMBLY__ */
#endif /* __TC3160_ETH_H__ */

