/*  
 * Miscellaneous definitions used to initialise the interrupt vector table
 * with the machine-specific interrupt routines.
 *
 * Copyright (C) 2002 Viosoft Corp.
 * Support for TrendChip TC3160
 */

#ifndef __TC3160_INTERRUPTS_H__ 
#define __TC3160_INTERRUPTS_H__ 

/*
 * TC3160 Interrupts
 */
#include <asm/addrspace.h>

/*
 * Register mapping 
 */
#define TC3160_INTR_BASE	(KSEG1 + 0x12000300)
#define	TC3160_IMR_REG		(TC3160_INTR_BASE + 0x0)
#define TC3160_IHPR_REG		(TC3160_INTR_BASE + 0x4)
#define TC3160_ILPR_REG		(TC3160_INTR_BASE + 0x8)
#define TC3160_IRR_REG		(TC3160_INTR_BASE + 0xc)

/*
 * Interrupt priorities (bit number, can be found in IMR)
 * We also use this as IRQ number as well.
 */
#define INTR_MIN	0	/* Minimum IRQ */
#define	INTR_MTMR1	0	/* Timer 1 */
#define INTR_MATM	1	/* ATM */
#define INTR_MTMR2	2	/* Timer 2 */
#define INTR_METH	3	/* Ethernet */
#define INTR_MTMR3	4	/* Timer 3 */
#define INTR_MDMAC	5	/* MDMAC */
#ifdef CONFIG_TRENDCHIP_WDL100
#define INTR_MUART	6	/* UART */
#define INTR_MGPINT1	7	/* GP_INT1 */
#define INTR_MGPINT2	7	/* GP_INT2 */
#define INTR_MGPINT3	10	/* GP_INT3 */
#define INTR_MGPINT4	10	/* GP_INT4 */
#define INTR_MARB	11	/* Arbiter */
#define INTR_MAX	12	/* Maximum IRQ */
#else /* WDL200 ?? */
#define INTR_MGPINT1	6	/* GP_INT1 */
#define INTR_MUART	7	/* UART */
#define INTR_MGPINT2	8	/* GP_INT2 */
#define INTR_MGPINT3	9	/* GP_INT3 */
#define INTR_MGPINT4	10	/* GP_INT4 */
#define INTR_MARB	11	/* Arbiter */
#define INTR_MPCMCIA	12	/* PCMCIA */
#define INTR_MUSB	13	/* USB */
#define INTR_MAX	13	/* Maximum IRQ */
#endif

#define NR_INTS		(INTR_MAX + 1)

#ifndef __ASSEMBLY__

/*
 * Interrupt table structure to hide differences between different
 * systems such.
 */
extern long cpu_mask_tbl[NR_INTS];
extern long cpu_irq_nr[NR_INTS];

/*
 * Common interrupt routine prototypes
 */
extern void tc3160_intr_unimplemented(void);

#endif /* !__ASSEMBLY__ */
#endif /* __TC3160_INTERRUPTS_H__  */

