
/*  
 * Definitions used to for UART module.
 *
 * Copyright (C) 2002 Viosoft Corp.
 * Support for TrendChip TC3160
 */
 
#ifndef __TC3160_SIO_H__ 
#define __TC3160_SIO_H__ 

#include <asm/addrspace.h>

#define TC3160_UART_BASE	(KSEG1 + 0x12000200)
#define TC3160_UARTREG1_REG	(TC3160_UART_BASE + 0x0)
#define TC3160_UARTREG2_REG	(TC3160_UART_BASE + 0x4)
#define TC3160_INTENB_REG	(TC3160_UART_BASE + 0x8)
#define TC3160_BAUDREG_REG	(TC3160_UART_BASE + 0xc)
#define TC3160_TXREG_REG	(TC3160_UART_BASE + 0x10)
#define TC3160_RXREG_REG	(TC3160_UART_BASE + 0x14)
#define TC3160_TXBF_REG		(TC3160_UART_BASE + 0x18)
#define TC3160_RXBF_REG		(TC3160_UART_BASE + 0x1c)
#define TC3160_INTSTA_REG	(TC3160_UART_BASE + 0x20)
#define TC3160_MSREG_REG	(TC3160_UART_BASE + 0x24)
#define TC3160_MCREG_REG	(TC3160_UART_BASE + 0x28)

/*
 * UARTREG1
 */
#define SIO_UARTEN	0x80000000
#define SIO_TXEN	0x40000000
#define SIO_RXEN	0x20000000
#define SIO_UARTON	0x10000000
#define SIO_TXON	0x08000000
#define SIO_RXON	0x04000000
#define SIO_LOOKBK	0x02000000

/*
 * UARTREG2
 */
#define SIO_PTYEN	0x80000000 
#define SIO_ODDEVEN	0x40000000 /* 0/1: odd/even */
#define SIO_STOP	0x20000000 /* 0/1: 1/2 bit */
#define SIO_CHARLEN	0x10000000 /* 0/1: 8/7 bit */
#define SIO_TXPOL	0x08000000 /* 0/1: normal/invert */
#define SIO_RXPOL	0x04000000 /* 0/1: normal/invert */

/*
 * INTENB
 */
#define SIO_TXEMPTY	0x80000000
#define SIO_RXFULL	0x40000000
#define SIO_RXOVR	0x10000000 //kchen
#define SIO_RXBRK	0x08000000 //kchen
#define SIO_BRKEND	0x04000000 //kchen
#define SIO_MSC		0x02000000 //kechn

/*
 * kchen: RXREG
 */
#define SIO_RXREG_RXFULL	0x80000000
#define SIO_RXREG_RXBUFLEN	0x001f0000
#define SIO_RXREG_FE		0x00008000
#define SIO_RXREG_PE		0x00004000
#define SIO_RXREG_BRKIND	0x00002000
#define SIO_RXREG_RXOVR		0x00001000
#define SIO_RXREG_RXDATA	0x000000ff

/*
 *
 */
#define SIO_ENB_TR	0xe0000000
#define SIO_ON_MASK	0x10000000
#define SIO_TXON_MASK	0x08000000
#define SIO_RXON_MASK	0x04000000
#define SIO_LB_TEST	0x02000000
#define SIO_ALL_ON	(SIO_ON_MASK | SIO_TXON_MASK | SIO_RXON_MASK)

#define SIO_MSC_RTSCNT	0x00000002
#define SIO_MSC_CTSCNT	0x00000001
#define SIO_HWFLOW	(SIO_MSC_RTSCNT | SIO_MSC_CTSCNT)

#define SIO_TR_RST	0x00008000

/*
 * Baudrate 
 */
#define SIO_BAUD_921600	0
#define SIO_BAUD_230400	2
#define SIO_BAUD_115200 (SIO_BAUD_230400 * 2)
#define SIO_BAUD_57600	(SIO_BAUD_115200 * 2)
#define SIO_BAUD_38400	(SIO_BAUD_115200 * 3)
#define SIO_BAUD_28800	(SIO_BAUD_57600 * 2)
#define SIO_BAUD_19200	(SIO_BAUD_38400 * 2)
#define SIO_BAUD_14400 	(SIO_BAUD_28800 * 2)
#define SIO_BAUD_9600	(SIO_BAUD_19200 * 2)
#define SIO_BAUD_7200	(SIO_BAUD_14400 * 2)
#define SIO_BAUD_4800	(SIO_BAUD_9600 * 2)
#define SIO_BAUD_3600	(SIO_BAUD_7200 * 2)
#define SIO_BAUD_2400	(SIO_BAUD_4800 * 2)

#ifndef __ASSEMBLY__

/*
 * SIO function prototipes
 */
int tc3160_sio_get_status (void);
int tc3160_sio_get_control (void);
int tc3160_sio_get_mode (void);
int tc3160_sio_get_baundrate (void);
int tc3160_sio_get_byte (void);

void tc3160_sio_set_control (int value);
void tc3160_sio_set_mode (int value);
void tc3160_sio_set_baundrate (int value);
void tc3160_sio_set_byte (int value);

#endif /* !__ASSEMBLY__ */
#endif /* __TC3160_SIO_H__  */

