
/*  
 * Definitions used to for hardware timers.
 *
 * Copyright (C) 2002 Viosoft Corp.
 * Support for TrendChip TC3160
 */
 
#ifndef __TC3160_TIMER_H__ 
#define __TC3160_TIMER_H__ 

/* 
 * Timer 1/2 are general purpose timer, while Timer 3 can
 * be used as watchdog timer. This implementation makes use
 * of Timer 1 as system timer and Timer 3 as watchdog timer.
 */

/*
 * Timer registers
 */
#define TC3160_TIMER_BASE	(KSEG1 + 0x12000400)
#define TC3160_TL1_REG		(TC3160_TIMER_BASE + 0x0)
#define TC3160_TL2_REG		(TC3160_TIMER_BASE + 0x4)
#define TC3160_TL3_REG		(TC3160_TIMER_BASE + 0x8)
#define TC3160_TH1_REG		(TC3160_TIMER_BASE + 0xc)
#define TC3160_TH2_REG		(TC3160_TIMER_BASE + 0x10)
#define TC3160_TH3_REG		(TC3160_TIMER_BASE + 0x14)
#define TC3160_TP_REG		(TC3160_TIMER_BASE + 0x18)
#define TC3160_TCS_REG		(TC3160_TIMER_BASE + 0x1c)
#define TC3160_TM1_REG		(TC3160_TIMER_BASE + 0x20)
#define TC3160_TM2_REG		(TC3160_TIMER_BASE + 0x24)
#define TC3160_TM3_REG		(TC3160_TIMER_BASE + 0x28)
#define TC3160_WM_REG		(TC3160_TIMER_BASE + 0x2c)
#define TC3160_WS_REG		(TC3160_TIMER_BASE + 0x30)
#define TC3160_TIS1_REG		(TC3160_TIMER_BASE + 0x34)
#define TC3160_TIS2_REG		(TC3160_TIMER_BASE + 0x38)
#define TC3160_TIS3_REG		(TC3160_TIMER_BASE + 0x3c)
#define TC3160_TIM_REG		(TC3160_TIMER_BASE + 0x40)

#ifndef __ASSEMBLY__

inline void tc3160_enable_timer(int timer_num)
{
  unsigned long reg = 0;
  switch(timer_num) {
    case 1: reg = TC3160_TM1_REG;
            break;
    case 2: reg = TC3160_TM2_REG;
   	    break;
    case 3: reg = TC3160_TM3_REG;
	    break;
    default: return;
  	     break;
  }
  *(volatile unsigned long *)reg |= 0x1;
}

inline void tc3160_disable_timer(int timer_num)
{
  unsigned long reg = 0;
  switch(timer_num) {
    case 1: reg = TC3160_TM1_REG;
	    break;
    case 2: reg = TC3160_TM2_REG;
	    break;
    case 3: reg = TC3160_TM3_REG;
	    break;
    default: return;
	     break;
  }
  *(volatile unsigned long *)reg &= 0xfffffffe;
}

inline void tc3160_enable_timer_interrupt(int timer_num)
{
  if ((timer_num > 0) && (timer_num < 4)) {
    *(volatile unsigned long *)TC3160_TIM_REG |= (0x1 << (timer_num - 1));
    tc3160_enable_timer(timer_num);
  }
}

inline void tc3160_disable_timer_interrupt(int timer_num)
{
  tc3160_disable_timer(timer_num);
  if ((timer_num > 0) && (timer_num < 4))
    *(volatile unsigned long *)TC3160_TIM_REG &= ~(0x1 << (timer_num - 1));
}

inline void tc3160_init_timers(void)
{
  /* Stop all timers */
  *(volatile unsigned long *)TC3160_TM1_REG &= 0xfffffffe;
  *(volatile unsigned long *)TC3160_TM2_REG &= 0xfffffffe;
  *(volatile unsigned long *)TC3160_TM3_REG &= 0xfffffffe;

  /* Select default clock source: 10MHz */
  *(volatile unsigned long *)TC3160_TCS_REG &= 0xffffffc0;

  /* Set prescaler factor to 0 for all timers: 200ns */
  *(volatile unsigned long *)TC3160_TP_REG &=  0xfff0f0f0;

  /* Set up timer 1 as interval timer */
  *(volatile unsigned long *)TC3160_TM1_REG |= 0x6;
  *(volatile unsigned long *)TC3160_TH1_REG = (10000000 / HZ); /* 10MHz */
//  *(volatile unsigned long *)TC3160_TH1_REG = (50000 / HZ); /* 10MHz */
  *(volatile unsigned long *)TC3160_TL1_REG = 0x0;

  /* Set up timer 2 as interval timer, needed?? */
  *(volatile unsigned long *)TC3160_TM2_REG |= 0x6;
  *(volatile unsigned long *)TC3160_TH2_REG = (10000000 / HZ);
  *(volatile unsigned long *)TC3160_TL2_REG = 0x0;

  /* Set up timer 3 as watchdog */
  *(volatile unsigned long *)TC3160_TM3_REG |= 0xe;
  *(volatile unsigned long *)TC3160_WM_REG &= 0xfffffffe;
  *(volatile unsigned long *)TC3160_TH3_REG = 1000; /* ~2 sec */
  *(volatile unsigned long *)TC3160_TL3_REG = 0;
}

#endif /* !__ASSEMBLY__ */
#endif /* __TC3160_TIMER_H__ */ 

