
/*  
 * Definitions used to for ATM/SAR module.
 *
 * Copyright (C) 2002 Viosoft Corp.
 * Support for TrendChip TC3162
 */

#ifndef __TC3162_ATM_H__ 
#define __TC3162_ATM_H__ 

#include <asm/addrspace.h>
// Sam add
#include <linux/skbuff.h>

/* General Configuration Registers */
#define TC3162_SAR_REG_BASE	(KSEG1 + 0x1FB60000)

#define TC3162_SAR_RAI_REG	(TC3162_SAR_REG_BASE + 0x0)
#define TC3162_SAR_GFR_REG	(TC3162_SAR_REG_BASE + 0x4)
#define TC3162_SAR_TSTB_REG	(TC3162_SAR_REG_BASE + 0x8)
#define TC3162_SAR_RMPL_REG	(TC3162_SAR_REG_BASE + 0xc)
#define TC3162_SAR_TXDBCS_REG	(TC3162_SAR_REG_BASE + 0x10)
#define TC3162_SAR_TXMBCS_REG	(TC3162_SAR_REG_BASE + 0x14)
#define TC3162_SAR_RXDBCS_REG	(TC3162_SAR_REG_BASE + 0x18)
#define TC3162_SAR_RXMBCS_REG	(TC3162_SAR_REG_BASE + 0x1c)
#define TC3162_SAR_LIRQ_REG	(TC3162_SAR_REG_BASE + 0x20)
#define TC3162_SAR_IRQBA_REG	(TC3162_SAR_REG_BASE + 0x24)
#define TC3162_SAR_IRQLEN_REG	(TC3162_SAR_REG_BASE + 0x28)
#define TC3162_SAR_IRQH_REG	(TC3162_SAR_REG_BASE + 0x2c)
#define TC3162_SAR_IRQC_REG	(TC3162_SAR_REG_BASE + 0x30)


/* VC IRQ Mask Registers */
#define TC3162_SAR_IRQM0_REG	(TC3162_SAR_REG_BASE + 0x40)
#define TC3162_SAR_IRQM1_REG	(TC3162_SAR_REG_BASE + 0x44)
#define TC3162_SAR_IRQM2_REG	(TC3162_SAR_REG_BASE + 0x48)
#define TC3162_SAR_IRQM3_REG	(TC3162_SAR_REG_BASE + 0x4c)
#define TC3162_SAR_IRQM4_REG	(TC3162_SAR_REG_BASE + 0x50)
#define TC3162_SAR_IRQM5_REG	(TC3162_SAR_REG_BASE + 0x54)
#define TC3162_SAR_IRQM6_REG	(TC3162_SAR_REG_BASE + 0x58)
#define TC3162_SAR_IRQM7_REG	(TC3162_SAR_REG_BASE + 0x5c)
#define TC3162_SAR_IRQM8_REG	(TC3162_SAR_REG_BASE + 0x60)
#define TC3162_SAR_IRQM9_REG	(TC3162_SAR_REG_BASE + 0x64)
#define TC3162_SAR_IRQM10_REG	(TC3162_SAR_REG_BASE + 0x68)
#define TC3162_SAR_IRQM11_REG	(TC3162_SAR_REG_BASE + 0x6c)
#define TC3162_SAR_IRQM12_REG	(TC3162_SAR_REG_BASE + 0x70)
#define TC3162_SAR_IRQM13_REG	(TC3162_SAR_REG_BASE + 0x74)
#define TC3162_SAR_IRQM14_REG	(TC3162_SAR_REG_BASE + 0x78)
#define TC3162_SAR_IRQM15_REG	(TC3162_SAR_REG_BASE + 0x7c)

#define TC3162_SAR_IRQMCC_REG	(TC3162_SAR_REG_BASE + 0x80)


/* VC Configuration Registers */
#define TC3162_SAR_VCR_BASE	(KSEG1 + 0x1FB60100)

#define TC3162_SAR_VCCR0_REG	(TC3162_SAR_VCR_BASE + 0x00)
#define TC3162_SAR_VCCR1_REG	(TC3162_SAR_VCR_BASE + 0x04)
#define TC3162_SAR_VCCR2_REG	(TC3162_SAR_VCR_BASE + 0x08)
#define TC3162_SAR_VCCR3_REG	(TC3162_SAR_VCR_BASE + 0x0c)
#define TC3162_SAR_VCCR4_REG	(TC3162_SAR_VCR_BASE + 0x10)
#define TC3162_SAR_VCCR5_REG	(TC3162_SAR_VCR_BASE + 0x14)
#define TC3162_SAR_VCCR6_REG	(TC3162_SAR_VCR_BASE + 0x18)
#define TC3162_SAR_VCCR7_REG	(TC3162_SAR_VCR_BASE + 0x1c)
#define TC3162_SAR_VCCR8_REG	(TC3162_SAR_VCR_BASE + 0x20)
#define TC3162_SAR_VCCR9_REG	(TC3162_SAR_VCR_BASE + 0x24)
#define TC3162_SAR_VCCR10_REG	(TC3162_SAR_VCR_BASE + 0x28)
#define TC3162_SAR_VCCR11_REG	(TC3162_SAR_VCR_BASE + 0x2c)
#define TC3162_SAR_VCCR12_REG	(TC3162_SAR_VCR_BASE + 0x30)
#define TC3162_SAR_VCCR13_REG	(TC3162_SAR_VCR_BASE + 0x34)
#define TC3162_SAR_VCCR14_REG	(TC3162_SAR_VCR_BASE + 0x38)
#define TC3162_SAR_VCCR15_REG	(TC3162_SAR_VCR_BASE + 0x3c)
#define TC3162_SAR_CCCR_REG	(TC3162_SAR_VCR_BASE + 0x40)


/* Transmit Buffer Descriptor Registers */
#define TC3162_SAR_TBD_REG_BASE		(KSEG1 + 0x1FB60200)

#define TC3162_SAR_TXDCBDA0_REG		(TC3162_SAR_TBD_REG_BASE + 0x0)
#define TC3162_SAR_TXDCBDA1_REG		(TC3162_SAR_TBD_REG_BASE + 0x4)
#define TC3162_SAR_TXDCBDA2_REG		(TC3162_SAR_TBD_REG_BASE + 0x8)
#define TC3162_SAR_TXDCBDA3_REG		(TC3162_SAR_TBD_REG_BASE + 0xc)
#define TC3162_SAR_TXDCBDA4_REG		(TC3162_SAR_TBD_REG_BASE + 0x10)
#define TC3162_SAR_TXDCBDA5_REG		(TC3162_SAR_TBD_REG_BASE + 0x14)
#define TC3162_SAR_TXDCBDA6_REG		(TC3162_SAR_TBD_REG_BASE + 0x18)
#define TC3162_SAR_TXDCBDA7_REG		(TC3162_SAR_TBD_REG_BASE + 0x1c)
#define TC3162_SAR_TXDCBDA8_REG		(TC3162_SAR_TBD_REG_BASE + 0x20)
#define TC3162_SAR_TXDCBDA9_REG		(TC3162_SAR_TBD_REG_BASE + 0x24)
#define TC3162_SAR_TXDCBDA10_REG	(TC3162_SAR_TBD_REG_BASE + 0x28)
#define TC3162_SAR_TXDCBDA11_REG	(TC3162_SAR_TBD_REG_BASE + 0x2c)
#define TC3162_SAR_TXDCBDA12_REG	(TC3162_SAR_TBD_REG_BASE + 0x30)
#define TC3162_SAR_TXDCBDA13_REG	(TC3162_SAR_TBD_REG_BASE + 0x34)
#define TC3162_SAR_TXDCBDA14_REG	(TC3162_SAR_TBD_REG_BASE + 0x38)
#define TC3162_SAR_TXDCBDA15_REG	(TC3162_SAR_TBD_REG_BASE + 0x3c)

#define TC3162_SAR_TXMCBDA0_REG		(TC3162_SAR_TBD_REG_BASE + 0x40)
#define TC3162_SAR_TXMCBDA1_REG		(TC3162_SAR_TBD_REG_BASE + 0x44)
#define TC3162_SAR_TXMCBDA2_REG		(TC3162_SAR_TBD_REG_BASE + 0x48)
#define TC3162_SAR_TXMCBDA3_REG		(TC3162_SAR_TBD_REG_BASE + 0x4c)
#define TC3162_SAR_TXMCBDA4_REG		(TC3162_SAR_TBD_REG_BASE + 0x50)
#define TC3162_SAR_TXMCBDA5_REG		(TC3162_SAR_TBD_REG_BASE + 0x54)
#define TC3162_SAR_TXMCBDA6_REG		(TC3162_SAR_TBD_REG_BASE + 0x58)
#define TC3162_SAR_TXMCBDA7_REG		(TC3162_SAR_TBD_REG_BASE + 0x5c)
#define TC3162_SAR_TXMCBDA8_REG		(TC3162_SAR_TBD_REG_BASE + 0x60)
#define TC3162_SAR_TXMCBDA9_REG		(TC3162_SAR_TBD_REG_BASE + 0x64)
#define TC3162_SAR_TXMCBDA10_REG	(TC3162_SAR_TBD_REG_BASE + 0x68)
#define TC3162_SAR_TXMCBDA11_REG	(TC3162_SAR_TBD_REG_BASE + 0x6c)
#define TC3162_SAR_TXMCBDA12_REG	(TC3162_SAR_TBD_REG_BASE + 0x70)
#define TC3162_SAR_TXMCBDA13_REG	(TC3162_SAR_TBD_REG_BASE + 0x74)
#define TC3162_SAR_TXMCBDA14_REG	(TC3162_SAR_TBD_REG_BASE + 0x78)
#define TC3162_SAR_TXMCBDA15_REG	(TC3162_SAR_TBD_REG_BASE + 0x7c)

#define TC3162_SAR_TXCCCBDA_REG		(TC3162_SAR_TBD_REG_BASE + 0x80)


/* Receive Buffer Descriptor Registers */
#define TC3162_SAR_RBD_REG_BASE		(KSEG1 + 0x1FB60300)

#define TC3162_SAR_RXDCBDA0_REG		(TC3162_SAR_RBD_REG_BASE + 0x0)
#define TC3162_SAR_RXDCBDA1_REG		(TC3162_SAR_RBD_REG_BASE + 0x4)
#define TC3162_SAR_RXDCBDA2_REG		(TC3162_SAR_RBD_REG_BASE + 0x8)
#define TC3162_SAR_RXDCBDA3_REG		(TC3162_SAR_RBD_REG_BASE + 0xc)
#define TC3162_SAR_RXDCBDA4_REG		(TC3162_SAR_RBD_REG_BASE + 0x10)
#define TC3162_SAR_RXDCBDA5_REG		(TC3162_SAR_RBD_REG_BASE + 0x14)
#define TC3162_SAR_RXDCBDA6_REG		(TC3162_SAR_RBD_REG_BASE + 0x18)
#define TC3162_SAR_RXDCBDA7_REG		(TC3162_SAR_RBD_REG_BASE + 0x1c)
#define TC3162_SAR_RXDCBDA8_REG		(TC3162_SAR_RBD_REG_BASE + 0x20)
#define TC3162_SAR_RXDCBDA9_REG		(TC3162_SAR_RBD_REG_BASE + 0x24)
#define TC3162_SAR_RXDCBDA10_REG	(TC3162_SAR_RBD_REG_BASE + 0x28)
#define TC3162_SAR_RXDCBDA11_REG	(TC3162_SAR_RBD_REG_BASE + 0x2c)
#define TC3162_SAR_RXDCBDA12_REG	(TC3162_SAR_RBD_REG_BASE + 0x30)
#define TC3162_SAR_RXDCBDA13_REG	(TC3162_SAR_RBD_REG_BASE + 0x34)
#define TC3162_SAR_RXDCBDA14_REG	(TC3162_SAR_RBD_REG_BASE + 0x38)
#define TC3162_SAR_RXDCBDA15_REG	(TC3162_SAR_RBD_REG_BASE + 0x3c)

#define TC3162_SAR_RXMCBDA0_REG		(TC3162_SAR_RBD_REG_BASE + 0x40)
#define TC3162_SAR_RXMCBDA1_REG		(TC3162_SAR_RBD_REG_BASE + 0x44)
#define TC3162_SAR_RXMCBDA2_REG		(TC3162_SAR_RBD_REG_BASE + 0x48)
#define TC3162_SAR_RXMCBDA3_REG		(TC3162_SAR_RBD_REG_BASE + 0x4c)
#define TC3162_SAR_RXMCBDA4_REG		(TC3162_SAR_RBD_REG_BASE + 0x50)
#define TC3162_SAR_RXMCBDA5_REG		(TC3162_SAR_RBD_REG_BASE + 0x54)
#define TC3162_SAR_RXMCBDA6_REG		(TC3162_SAR_RBD_REG_BASE + 0x58)
#define TC3162_SAR_RXMCBDA7_REG		(TC3162_SAR_RBD_REG_BASE + 0x5c)
#define TC3162_SAR_RXMCBDA8_REG		(TC3162_SAR_RBD_REG_BASE + 0x60)
#define TC3162_SAR_RXMCBDA9_REG		(TC3162_SAR_RBD_REG_BASE + 0x64)
#define TC3162_SAR_RXMCBDA10_REG	(TC3162_SAR_RBD_REG_BASE + 0x68)
#define TC3162_SAR_RXMCBDA11_REG	(TC3162_SAR_RBD_REG_BASE + 0x6c)
#define TC3162_SAR_RXMCBDA12_REG	(TC3162_SAR_RBD_REG_BASE + 0x70)
#define TC3162_SAR_RXMCBDA13_REG	(TC3162_SAR_RBD_REG_BASE + 0x74)
#define TC3162_SAR_RXMCBDA14_REG	(TC3162_SAR_RBD_REG_BASE + 0x78)
#define TC3162_SAR_RXMCBDA15_REG	(TC3162_SAR_RBD_REG_BASE + 0x7c)

#define TC3162_SAR_RXCCCBDA_REG		(TC3162_SAR_RBD_REG_BASE + 0x80)


/* Traffic Schedule Registers */
#define TC3162_SAR_TSR_REG_BASE		(KSEG1 + 0x1FB60400)

#define TC3162_SAR_PCR0_REG		(TC3162_SAR_TSR_REG_BASE + 0x0) 
#define TC3162_SAR_PCR1_REG		(TC3162_SAR_TSR_REG_BASE + 0x4) 
#define TC3162_SAR_PCR2_REG		(TC3162_SAR_TSR_REG_BASE + 0x8) 
#define TC3162_SAR_PCR3_REG		(TC3162_SAR_TSR_REG_BASE + 0xc) 
#define TC3162_SAR_PCR4_REG		(TC3162_SAR_TSR_REG_BASE + 0x10) 
#define TC3162_SAR_PCR5_REG		(TC3162_SAR_TSR_REG_BASE + 0x14) 
#define TC3162_SAR_PCR6_REG		(TC3162_SAR_TSR_REG_BASE + 0x18) 
#define TC3162_SAR_PCR7_REG		(TC3162_SAR_TSR_REG_BASE + 0x1c) 
#define TC3162_SAR_PCR8_REG		(TC3162_SAR_TSR_REG_BASE + 0x20) 
#define TC3162_SAR_PCR9_REG		(TC3162_SAR_TSR_REG_BASE + 0x24) 
#define TC3162_SAR_PCR10_REG		(TC3162_SAR_TSR_REG_BASE + 0x28) 
#define TC3162_SAR_PCR11_REG		(TC3162_SAR_TSR_REG_BASE + 0x2c) 
#define TC3162_SAR_PCR12_REG		(TC3162_SAR_TSR_REG_BASE + 0x30) 
#define TC3162_SAR_PCR13_REG		(TC3162_SAR_TSR_REG_BASE + 0x34) 
#define TC3162_SAR_PCR14_REG		(TC3162_SAR_TSR_REG_BASE + 0x38) 
#define TC3162_SAR_PCR15_REG		(TC3162_SAR_TSR_REG_BASE + 0x3c) 

#define TC3162_SAR_SCR0_REG		(TC3162_SAR_TSR_REG_BASE + 0x40) 
#define TC3162_SAR_SCR1_REG		(TC3162_SAR_TSR_REG_BASE + 0x44) 
#define TC3162_SAR_SCR2_REG		(TC3162_SAR_TSR_REG_BASE + 0x48) 
#define TC3162_SAR_SCR3_REG		(TC3162_SAR_TSR_REG_BASE + 0x4c) 
#define TC3162_SAR_SCR4_REG		(TC3162_SAR_TSR_REG_BASE + 0x50) 
#define TC3162_SAR_SCR5_REG		(TC3162_SAR_TSR_REG_BASE + 0x54) 
#define TC3162_SAR_SCR6_REG		(TC3162_SAR_TSR_REG_BASE + 0x58) 
#define TC3162_SAR_SCR7_REG		(TC3162_SAR_TSR_REG_BASE + 0x5c) 
#define TC3162_SAR_SCR8_REG		(TC3162_SAR_TSR_REG_BASE + 0x60) 
#define TC3162_SAR_SCR9_REG		(TC3162_SAR_TSR_REG_BASE + 0x64) 
#define TC3162_SAR_SCR10_REG		(TC3162_SAR_TSR_REG_BASE + 0x68) 
#define TC3162_SAR_SCR11_REG		(TC3162_SAR_TSR_REG_BASE + 0x6c) 
#define TC3162_SAR_SCR12_REG		(TC3162_SAR_TSR_REG_BASE + 0x70) 
#define TC3162_SAR_SCR13_REG		(TC3162_SAR_TSR_REG_BASE + 0x74) 
#define TC3162_SAR_SCR14_REG		(TC3162_SAR_TSR_REG_BASE + 0x78) 
#define TC3162_SAR_SCR15_REG		(TC3162_SAR_TSR_REG_BASE + 0x7c) 

#define TC3162_SAR_MBSTP0_REG		(TC3162_SAR_TSR_REG_BASE + 0x80) 


/* Transmit Statistics Counter Registers */
#define TC3162_SAR_TCNT_REG_BASE	(KSEG1 + 0x1FB60600)

#define TC3162_SAR_TDCNT0_REG		(TC3162_SAR_TCNT_REG_BASE + 0x0) 

#define TC3162_SAR_TXCNTCC_REG		(TC3162_SAR_TCNT_REG_BASE + 0x40) 

/* Receive Statistics Counter Registers */
#define TC3162_SAR_RCNT_REG_BASE	(KSEG1 + 0x1FB60700)

#define TC3162_SAR_RDCNT0_REG		(TC3162_SAR_RCNT_REG_BASE + 0x0) 

#define TC3162_SAR_RXCNTCC_REG		(TC3162_SAR_RCNT_REG_BASE + 0x40) 

#define TC3162_SAR_MISCNT_REG		(TC3162_SAR_RCNT_REG_BASE + 0x44) 



/* */


#define TSARM_TXDCBDA(x)		(*(volatile unsigned long*)(TC3162_SAR_TXDCBDA0_REG + (4*x)))
#define TSARM_TXMCBDA(x)		(*(volatile unsigned long*)(TC3162_SAR_TXMCBDA0_REG + (4*x)))

#define TSARM_RXDCBDA(x)		(*(volatile unsigned long*)(TC3162_SAR_RXDCBDA0_REG + (4*x)))
#define TSARM_RXMCBDA(x)		(*(volatile unsigned long*)(TC3162_SAR_RXMCBDA0_REG + (4*x)))

#define TSARM_MBSTP(x)			(*(volatile unsigned long*)(TC3162_SAR_MBSTP0_REG + (4*x)))
#define TSARM_VCCR(x)			(*(volatile unsigned long*)(TC3162_SAR_VCCR0_REG + (4*x)))
#define TSARM_PCR(x)			(*(volatile unsigned long*)(TC3162_SAR_PCR0_REG + (4*x)))
#define TSARM_SCR(x)			(*(volatile unsigned long*)(TC3162_SAR_SCR0_REG + (4*x)))

#define TSARM_IRQM(x)			(*(volatile unsigned long*)(TC3162_SAR_IRQM0_REG + (4*x)))

/*********************************
 * Constant of ATM SAR registers *
 *********************************/
/* RAI */
#define RAI_RESET_ENB(x)                                (x) << 0
/* GFR */
#define GFR_TXENB                                       1 << 0
#define GFR_RXENB                                       1 << 1
#define GFR_GIRQEN                                      1 << 2
#define GFR_BIGENDIAN                                   1 << 3
#define GFR_UL_TX_FIX                                   1 << 4
#define GFR_UL_RX_FIX                                   1 << 5
#define GFR_REMOTE_LPK                                  1 << 6
#define GFR_LOCAL_LPK                                   3 << 6
#define GFR_UL1                                         1 << 8
#define GFR_RX_INACT_VC_M                               1 << 30
#define GFR_IRQ_Q_FULL_M                                1 << 31

/* VC(n) configuration register: ATMSAR_VCCFGR */
#define VCCFGR_VALID                                    1 << 0
#define VCCFGR_RXRAW                                    1 << 1

#define VCCFGR_PORT(x)                                  (x) << 2
#define VCCFGR_ATM_PHY0                                 0x0
#define VCCFGR_ATM_PHY1                                 0x1

#define VCCFGR_VCI(x)                                   (x)<<4
#define VCCFGR_VPI(x)                                   (x)<<20
#define LIRQ_TX_U_DONE                                  (1 << 0)
#define LIRQ_TX_M_DONE                                  (1 << 1)
#define LIRQ_RX_U_DONE                                  (1 << 2)
#define LIRQ_RX_M_DONE                                  (1 << 3)

#define LIRQ_TX_BUF_DONE                                (1 << 4)

#define LIRQ_TX_U_BD_UF                                 (1 << 8)
#define LIRQ_TX_M_BD_UF                                 (1 << 9)
#define LIRQ_TX_SW_DIS                                  (1 << 10)

#define LIRQ_RX_U_BD_UR                                 (1 << 11)
#define LIRQ_RX_M_BD_UR                                 (1 << 12)

#define LIRQ_RX_MAXLENE                                 (1 << 13)
#define LIRQ_RX_U_BD_OV                                   (1 << 14)
#define LIRQ_RX_M_BD_OV                                   (1 << 15)
#define LIRQ_RX_CRC10E                                  (1 << 16)
#define LIRQ_RX_CRC32E                                  (1 << 17)
#define LIRQ_RX_LENE                                    (1 << 18)

#define LIRQ_GET_VC_NO(x)                               (uint8) (((x) & 0x01f00000) >> 20)
#define LIRQ_RX_INACT_VC                                (1 << 30)
#define LIRQ_IRQ_Q_FULL                                 (1 << 31)


/********************************************
 * Data type and constants of ATM Tx Module *
 ********************************************/
#define TSARM_TX_DESCR_VALID                    	1 << 0
#define TSARM_TX_DESCR_SOS                              1 << 1
#define TSARM_TX_DESCR_EOS                              1 << 2
                                                                                                                                               
//#define ATM_TX_CC_DESCR_NUMMAX                  	4
//#define ATM_TX_VC_DESCR_NUMMAX                  	16
                                                                                                                                               
#define ATM_QOS_CBR                                     0
#define ATM_QOS_UBR                                     1
#define ATM_QOS_rtVBR                                   2
#define ATM_QOS_nrtVBR                                  3
#define ATM_QOS_SCR_DEC                                 2



/********************************************
 * Data type and constants of ATM Rx Module *
 ********************************************/
#define TSARM_RX_DESCR_VALID                    1 << 0

#define uint32		unsigned long
#define uint16		unsigned short
#define uint8		unsigned char

/* P R I V A T E   D A T A */
#define         ATM_VC_MAX                      8
#define 	ATM_DUMMY_VC                    21
#define         ATM_RX_CC_DESCR_NUMMAX          4
#define         ATM_RX_VC_DESCR_NUMMAX          20
#define         ATM_TX_CC_DESCR_NUMMAX          4
#define         ATM_TX_VC_DESCR_NUMMAX          16
#define         IRQ_MAX_ENTRY                   256
#define         OAM_FAULT_MANAGEMENT            0x1

/* ----- OAM function type ----- */
#define OAM_AIS                                 0x0
#define OAM_RDI                                 0x1
#define OAM_CC                                  0x4
#define OAM_LOOPBACK                            0x8



typedef struct atmRxCcDescr_s {
        uint32 rdes0;
        uint32 rdes1;
        uint32 rdes2;
        uint32 rdes3;
} atmRxCcDescr_t;

typedef struct atmRxDescr_s {
        uint32 rdes0;
        uint32 rdes1;
        uint32 rdes2;
        uint32 rdes3;
        // S1 mbuf_t *mbp;
	struct sk_buff *mbp;
        uint32 rsv[3];
} atmRxDescr_t;

typedef struct atmRxCcDescrPool_s {
        atmRxCcDescr_t rxCcDescrBuf[ATM_RX_CC_DESCR_NUMMAX];
} atmRxCcDescrPool_t;

typedef struct atmRxDescrPool_s {
        atmRxDescr_t rxDescrBuf[ATM_RX_VC_DESCR_NUMMAX];
} atmRxDescrPool_t;

/* S2 typedef struct atmRxOamDescrPool_s {
        atmCellPayload_t rxOamDescrBuf[ATM_RX_CC_DESCR_NUMMAX];
} atmRxOamDescrPool_t;
*/

typedef struct atmTxCcDescr_s {
        uint32 tdes0;
        uint32 tdes1;
        uint32 tdes2;
        uint32 tdes3;
} atmTxCcDescr_t;

typedef struct atmTxDescr_s {
        uint32 tdes0;
        uint32 tdes1;
        uint32 tdes2;
        uint32 tdes3;
        // S1 mbuf_t *mbp;
	struct sk_buff *mbp;
        uint32 rsv[3];
} atmTxDescr_t;

typedef struct atmTxCcDescrPool_s {
        atmTxCcDescr_t txCcDescrBuf[ATM_TX_CC_DESCR_NUMMAX];
} atmTxCcDescrPool_t;

typedef struct atmTxDescrPool_s {
        atmTxDescr_t txDescrBuf[ATM_TX_VC_DESCR_NUMMAX];
} atmTxDescrPool_t;


typedef struct atmRxCcCell_s {
        uint32 word[13];
        uint32 reserved[11];
} atmRxCcCell_t;

typedef struct atmOamCell_s{
        uint32 gfc: 4;
        uint32 vpi: 8;
        uint32 vci: 16;
        uint32 pti: 3;
        uint32 clp: 1;
        uint32 oamCellType: 4;
        uint32 oamFuncType: 4;
        uint8 payload[45];
} atmOamCell_t;

typedef struct atmOamCellPayload_s{
        uint32 oamCellType: 4;
        uint32 oamFuncType: 4;
        uint8 payload[45];
} atmOamCellPayload_t;


/*******************************
 * Data type of ATM vc control *
 *******************************/
typedef struct qosProfile_s {
        uint16 type;    /* CBR, VBR, UBR */
        uint16 pcr;
        uint16 scr;
        uint16 mbs;
} qosProfile_t;

typedef struct atmConfig_s {
        uint8 vcNumber;                                         /* Number of opened VC (0-20) */
        uint8 openFlag[ATM_VC_MAX];                     /* Flag to indicate if this vc is opened */
        uint8 vpi[ATM_VC_MAX];                          /* VPI value of each VC */
        uint16 vci[ATM_VC_MAX];                         /* VCI value of each VC */
} atmConfig_t;






#ifndef __ASSEMBLY__
#endif /* !__ASSEMBLY__ */

#endif /* __TC3162_ATM_H__ */

