/*  
 * Definitions used to for Ethernet MAC Controller Operation register.
 *
 * Written by Robert Wu, Billion Co.
 * Support for TrendChip TC3162
 */


#ifndef __TC3162_EMAC_H__ 
#define __TC3162_EMAC_H__ 

#include <asm/addrspace.h>
#include <linux/skbuff.h>
#include <linux/tqueue.h>
/* emac operation registers */
#define TC3162_EMAC_BASE		(KSEG1 + 0x1FB50000)
#define TC3162_EMAC_ISR_REG		(TC3162_EMAC_BASE + 0x0)
#define TC3162_EMAC_IMR_REG		(TC3162_EMAC_BASE + 0x4)
#define TC3162_EMAC_MAC_MADR_REG	(TC3162_EMAC_BASE + 0x8)
#define TC3162_EMAC_MAC_LADR_REG	(TC3162_EMAC_BASE + 0xC)
#define TC3162_EMAC_MAHT0_REG		(TC3162_EMAC_BASE + 0x10)
#define TC3162_EMAC_MAHT1_REG		(TC3162_EMAC_BASE + 0x14)
#define TC3162_EMAC_TXPD_REG		(TC3162_EMAC_BASE + 0x18)
#define TC3162_EMAC_RXPD_REG		(TC3162_EMAC_BASE + 0x1C)
#define TC3162_EMAC_TXR_BADR_REG	(TC3162_EMAC_BASE + 0x20)
#define TC3162_EMAC_RXR_BADR_REG	(TC3162_EMAC_BASE + 0x24)
#define TC3162_EMAC_ITC_REG		(TC3162_EMAC_BASE + 0x28)
#define TC3162_EMAC_APTC_REG		(TC3162_EMAC_BASE + 0x2C)
#define TC3162_EMAC_DBLAC_REG		(TC3162_EMAC_BASE + 0x30)
#define TC3162_EMAC_MACCR_REG		(TC3162_EMAC_BASE + 0x88)
#define TC3162_EMAC_MACSR_REG		(TC3162_EMAC_BASE + 0x8C)
#define TC3162_EMAC_PHYCR_REG		(TC3162_EMAC_BASE + 0x90)
#define TC3162_EMAC_PHYWDATA_REG	(TC3162_EMAC_BASE + 0x94)
#define TC3162_EMAC_FCR_REG		(TC3162_EMAC_BASE + 0x98)
#define TC3162_EMAC_BPR_REG		(TC3162_EMAC_BASE + 0x9C)
#define TC3162_EMAC_TM_REG		(TC3162_EMAC_BASE + 0xCC)
#define TC3162_EMAC_XM_PC_CNTR_REG	(TC3162_EMAC_BASE + 0xDC)
#define TC3162_EMAC_RUNT_TLCC_CNTR_REG	(TC3162_EMAC_BASE + 0xE0)
#define TC3162_EMAC_CRCER_FTL_CNTR_REG	(TC3162_EMAC_BASE + 0xE4)
#define TC3162_EMAC_RLC_RCC_CNTR_REG	(TC3162_EMAC_BASE + 0xE8)
#define TC3162_EMAC_BROC_CNTR_REG	(TC3162_EMAC_BASE + 0xEC)
#define TC3162_EMAC_MULCA_CNTR_REG	(TC3162_EMAC_BASE + 0xF0)
#define TC3162_EMAC_RP_CNTR_REG		(TC3162_EMAC_BASE + 0xF4)
#define TC3162_EMAC_XP_CNTR_REG		(TC3162_EMAC_BASE + 0xF8)

/* descriptor setting */
#define TX_DESCP_NUM	64		/* default descriptor number */
#define RX_DESCP_NUM	64	
#define EMAC_DESCP_SIZE	16	/* default descriptor size */
#ifdef CONFIG_TRENDCHIP_TC3162L2
#define TX_SKB_RELEASE_THRESHOLD 4 
#else
#define TX_BUF_RELEASE_THRESHOLD 32

#endif
/* memory mapping setting */
#define virt_to_phy(addr)	( ((unsigned int)addr) & 0x1fffffff)
#define KSeg1_to_phy(addr)	( ((unsigned int)addr) & 0x1fffffff)
#define KSeg0_to_KSeg1(addr)	( ((unsigned int)addr) | 0xa0000000)

/* MAC Control register (MACCR)*/
#define MACCR_RX_BRDPKT		0x20000
#define MACCR_MULPKT		0x10000
#define MACCR_FULLDUP		0x8000
#define MACCR_CRC_APD		0x4000
#define MACCR_MDC_SEL		0x2000
#define MACCR_RCV_ALL		0x1000
#define MACCR_RX_FTL		0x800
#define MACCR_RX_RUNT		0x400
#define MACCR_HT_MUL_EN		0x200
#define MACCR_RCV_EN		0x100
#define MACCR_ENRX_IN_HALFTX	0x40
#define MACCR_XMT_EN		0x20
#define MACCR_CRC_DIS		0x10
#define MACCR_LOOP_EN		0x8	
#define MACCR_SW_RST		0x4
#define MACCR_RDMA_EN		0x2
#define MACCR_XDMA_EN		0x1

/* MAC Status register (MACSR) */
#define MACSR_COL_EXC		0x800
#define MACSR_LATE_COL		0x400
#define MACSR_XPKT_LOST		0x200
#define MACSR_XPKT_OK		0x100
#define MACSR_RUNT		0x80
#define MACSR_FTL		0x40       	
#define MACSR_CRC_ERR	    	0x20
#define MACSR_RPKT_LOST	    	0x10
#define MACSR_RPKT_SAVE	    	0x8 
#define MACSR_COL	    	0x4
#define MACSR_BROADCAST	    	0x2
#define MACSR_MULTICAST	    	0x1

/* mbuffer data size */
// Sam #define MBUF_SIZE	256


/* Sam add */

#define MAC_DESCP_SIZE		16	/* 4 DWords */
#define TX_DESCP_NO             64      /* 5 descriptors, = Max tx buffer Cnts ,default=64 */
#define RX_DESCP_NO             64      /* 5 descriptors ,default=64 */

/* ---- Descriptor BIT definition ---- */
#define TXDMA_OWN_BIT		0x80000000
#define TXPKT_EXSCOL_BIT	0x00000002
#define TXPKT_LATECOL_BIT	0x00000001
#define EDOTR_BIT		0x80000000
#define TXIC_BIT		0x40000000
#define TX2FIC_BIT		0x20000000
#define FTS_BIT			0x10000000
#define LTS_BIT			0x08000000

#define RXDMA_OWN_BIT		0x80000000
#define FRS_BIT			0x20000000
#define LRS_BIT			0x10000000
#define COLLISION_BIT		0x00400000
#define RUNT_BIT		0x00200000
#define FTL_BIT			0x00100000
#define CRC_ERR_BIT		0x00080000
#define RX_ERR_BIT		0x00040000
#define BROADCAST_BIT		0x00020000
#define MULTICAST_BIT		0x00010000
#define EDORR_BIT		0x80000000

#define RXbuf_size		1996
/* ---- End Descriptor ----*/

/* ---- Interrupt Mask Register ---*/
#define PHYSTS_CHG_BIT		0x200
#define AHB_ERR_BIT		0x100
#define RPKT_LOST_BIT		0x80
#define RPKT_SAVE_BIT		0x40
#define XPKT_LOST_BIT		0x20
#define XPKT_FINISH_BIT		0x10
#define NOTXBUF_BIT		0x8
#define XPKT_SAVE_BIT		0x4
#define NORXBUF_BIT		0x2
#define RPKT_FINISH_BIT		0x1
/* ---- End IMR ---- */


#define MIIWR_BIT		0x08000000
#define MIIRD_BIT		0x04000000


/* ----- PHY-Chip Definition ----- */
// Sam #define PHY_WRITE               0x0400
// Sam #define PHY_BUSY                0x0800
// Please reference the RTL8201CP single chip/single port 10/100M ethernet phyceiver DATASHEET 
#define PHY_CONTROL_REG         0               /* MII Control Register */

#define PHY_RESET               0x8000
#define PHY_LOOPBACK            0x4000
#define PHY_SPEED_SELECT_100    0x2000
#define PHY_AN_ENABLE           0x1000
#define PHY_POWER_DOWN          0x0800
#define PHY_ISOLATE             0x400
#define PHY_RESTART_AN          0x200
#define PHY_DUPLEX_MODE		0x100

#define PHY_STATUS_REG          1               /* MII Status Register */
#define PHY_AN_COMPLETE         0x0020
#define PHY_LINK_STATUS         0x0004

#define PHY_ID_REG1             2               /* Identifier register 1 */
#define PHY_ID_REG2             3               /* Identifier register 2 */

#define PHY_ANAR		4		/* Auto-Negotation Advertisement Register */
#define PHY_REMOTE_CAP_REG      5               /* Auto-Negotiation Link Partner Ability Reg */

#define PHY_TX_FC		0x800
#define PHY_RX_FC		0x400
#define PHY_100_BASE_T4         0x200
#define PHY_100_BASE_TX_FD      0x100
#define PHY_100_BASE_TX_HD      0x80
#define PHY_10_BASE_T_FD        0x40
#define PHY_10_BASE_T_HD        0x20
#define PHY_SELECTOR		0x01

#define PHY_CONFIG_REG          16
#define PHY_TP_LOOPBACK         0x0100

#define SINGLE_PHY_ADDR		1
#define PHY_REG_ADDR_SHIFT	21
#define PHY_ADDR_SHIFT		16

/* ----- End Phy-Chip Definition ---- */

#define DEF_LINK_LOOP		1000
#define DEF_MAX_LINK_LOOP	0x1000000

/* End of Sam add */



/* MAC tx descriptor */

typedef union {
	struct {
		unsigned int ownerDma: 1;
		unsigned int reserved:  29;
		unsigned int txPktExsCol: 1;
		unsigned int txPktLateCol: 1;
	} bits;
	unsigned int word;
} tdes0_t;


typedef union {
	struct {
		unsigned int edotr: 1;
		unsigned int txic: 1;
		unsigned int tx2fic: 1;
		unsigned int fts: 1;
		unsigned int lts: 1;
		unsigned int reserved: 16;
		unsigned int txBufSize: 11;
	} bits;
	unsigned int word;
} tdes1_t;

typedef union {
	unsigned int buf_p;
	unsigned int word;
} tdes2_t;

// Sam marked typedef struct macTxDescr_s {
#if 0 
typedef struct macTxDescr_s{
	tdes0_t tdes0;
	tdes1_t tdes1;
	tdes2_t tdes2;
#ifdef CONFIG_TRENDCHIP_TC3162L2
	struct sk_buff* skb;
#else
// Sam	char* mbuf;
	int len;
#endif
}macTxDescr_t;
#endif
typedef struct macTxDescr_s{
	volatile unsigned long tdes0;
	volatile unsigned long tdes1;
	volatile unsigned long tdes2;
#ifdef CONFIG_TRENDCHIP_TC3162L2
	struct sk_buff* skb;
#else
	int len;
#endif
}macTxDescr_t;

/* MAC rx descriptor */

typedef union {
	struct {
		unsigned int ownerDma: 1;
		unsigned int reserved: 1;
		unsigned int frs: 1;
		unsigned int lrs: 1;
		unsigned int reserved_1: 5;
		unsigned int txOddNb: 1;
		unsigned int runt: 1;
		unsigned int ftl: 1;
		unsigned int crc_err: 1;
		unsigned int rx_err: 1;
		unsigned int broadcast: 1;
		unsigned int multicast: 1;
		unsigned int reserved_2: 5;
		unsigned int rfl: 11;	
	} bits;
	unsigned int word;
} rdes0_t;

typedef union {
	struct {
		unsigned int edorr: 1;
		unsigned int reserved: 20;
		unsigned int rxBufSize:11;
	} bits;
	unsigned int word;
} rdes1_t;

typedef union {
	unsigned int buf_p;
	unsigned int word;
} rdes2_t;

/* Sam marked */
#if 0 
typedef struct macRxDescr_s {
	rdes0_t rdes0;
	rdes1_t rdes1;
	rdes2_t rdes2;
#ifdef CONFIG_TRENDCHIP_TC3162L2
	struct sk_buff* skb;
#else
// Sam	char *mbuf;
	int len;
}#endif
}macRxDescr_t;
#endif

typedef struct macRxDescr_s {
	volatile unsigned long rdes0;
	volatile unsigned long rdes1;
	volatile unsigned long rdes2;
#ifdef CONFIG_TRENDCHIP_TC3162L2
	struct sk_buff* skb;		
#else
// Sam	char *mbuf;
	int len;
#endif
}macRxDescr_t;



//  CR_MAC_PHYCR ---
 
typedef union {
	struct {
	     unsigned int reserved: 4; // reserved
	     unsigned int  miiwr    : 1;   // MIIWR
	     unsigned int  miird    : 1;   // MIIRD
	     unsigned int  regad    : 5;   // REGAD
	     unsigned int  phyad    : 5;   // PHYAD
	     unsigned int  miirdata : 16;  // MIIRDATA
    }bits;
    unsigned int  value;
}macPhyCtrlReg_t;
 

//
// --- CR_MAC_PHYWDATA ---
//

typedef  union{
    struct
    {
    	   unsigned int reserved: 16;  // reserved
	   unsigned int  miiwdata : 16;  // MIIWDATA
    }bits;
    unsigned int  value;
}macPhyWDataReg_t;



// --- CR_MAC_MACCR ---
typedef  union{
    struct
    {
      unsigned int  reserved         :	14;
      unsigned int  rx_broadpkt   : 1;   // RX_BROADPKT
      unsigned int  rx_multipkt    : 1;   // RX_MULTIPKT
      unsigned int  fulltx               : 1;   // FULLTX
      unsigned int  crc_apd          : 1;   // CRC_APD
      unsigned int  mdc_sel          : 1;   // MDC_SEL
      unsigned int  rcv_all            : 1;   // RCV_ALL
      unsigned int  rx_ftl              : 1;   // RX_FTL
      unsigned int  rx_runt           : 1;   // RX_RUNT
      unsigned int  ht_multi_en    : 1;   // HT_MULTI_EN
      unsigned int  rcv_en            : 1;   // RCV_EN
      unsigned int  rsv                  : 1;
      unsigned int  enrx_in_halftx : 1;   // ENRX_IN_HALFTX
      unsigned int  xmt_en           : 1;   // XMT_EN
      unsigned int  crc_dis           : 1;   // CRC_DIS
      unsigned int  loop_en          : 1;   // LOOP_EN
      unsigned int  sw_rst            : 1;   // SW_RST
      unsigned int  rdma_en        : 1;   // RDMA_EN
      unsigned int  xdma_en        : 1;   // XDMA_EN*/
    }bits;
    unsigned int  value;
}macCtrlReg_t;


//
// --- CR_MAC_IMR ---
//
typedef  union
  {
     struct
    { 
    	      unsigned int  reserved:22;    // reserved
	      unsigned int  physts_chg  : 1;  // PHYSTS_CHG
	      unsigned int  ahb_err     : 1;  // AHB_ERR
	      unsigned int  rpkt_lost   : 1;  // RPKT_LOST
	      unsigned int  rpkt_save   : 1;  // RPKT_SAVE
	      unsigned int  xpkt_lost   : 1;  // XPKT_LOST
	      unsigned int  xpkt_finish : 1;  // XPKT_FINISH
	      unsigned int  notxbuf     : 1;  // NOTXBUF
	      unsigned int  xpkt_save   : 1;  // XPKT_SAVE
	      unsigned int  norxbuf     : 1;  // NORXBUF
	      unsigned int  rpkt_finish : 1;  // RPKT_FINISH
    }bits;
    unsigned int  value;
} macIntrMaskReg_t;

//
// --- CR_MAC_ISR ---
//
typedef  union
  {
    struct
    {
    	      unsigned int  reserved:   22;  // reserved
	      unsigned int  physts_chg  : 1;  // PHYSTS_CHG
	      unsigned int  ahb_err     : 1;  // AHB_ERR
	      unsigned int  rpkt_lost   : 1;  // RPKT_LOST
	      unsigned int  rpkt_save   : 1;  // RPKT_SAVE
	      unsigned int  xpkt_lost   : 1;  // XPKT_LOST
	      unsigned int  xpkt_finish : 1;  // XPKT_FINISH
	      unsigned int  notxbuf     : 1;  // NOTXBUF
	      unsigned int  xpkt_save   : 1;  // XPKT_SAVE
	      unsigned int  norxbuf     : 1;  // NORXBUF
	      unsigned int  rpkt_finish : 1;  // RPKT_FINISH
    }bits;
    unsigned int  value;
}macIntrStatusReg_t;


//
// --- PHY Advertisement register---------
//
typedef  union
  {
    unsigned int value;
    struct
    {
      unsigned int  rsv2            	:16;
      unsigned int  next_page             : 1;
      unsigned int  acknowledge           : 1;
      unsigned int  rsv1                  : 3;
      unsigned int  flow_control          : 1;
      unsigned int  rsv0                  : 1;
      unsigned int  full_duplex_100baseT  : 1;
      unsigned int  half_duplex_100baseT  : 1;
      unsigned int  full_duplex_10baseT   : 1;
      unsigned int  half_duplex_10baseT   : 1;
      unsigned int  selector_field        : 5;
    }bits;
  }macPhyAdvertismentReg_t;


// --- PHY Control Register ---
typedef  union
  {
    unsigned int value;
    struct
    {
      unsigned int  rsv1 : 16;
      unsigned int  phy_reset       : 1;
      unsigned int  loopback        : 1;
      unsigned int  speed_select    : 1;
      unsigned int enable_ANEN : 1;
      unsigned int  power_down      : 1;
      unsigned int  phy_isolate     : 1;
      unsigned int  reset_ANEN      : 1;
      unsigned int  duplex_mode     : 1;
      unsigned int  collision_test  : 1;
      unsigned int  rsv             : 7;
    }bits;
}macPhyCR_t;


typedef  union
 {
    unsigned int  value;
    struct
    {
      unsigned int  rsv1: 17;
      unsigned int  extend_capability     : 1;
      unsigned int  jabber_detect         : 1;
      unsigned int  link_status           : 1;
      unsigned int  ANEN_ability          : 1;
      unsigned int  remote_fault          : 1;
      unsigned int  ANEN_complete         : 1;
      unsigned int  MF_preamble_suppress  : 1;
      unsigned int  rsv                   : 4;
      unsigned int  half_duplex_10baseT   : 1;
      unsigned int  full_duplex_10baseT   : 1;
      unsigned int  half_duplex_100baseT  : 1;
      unsigned int  full_duplex_100baseT  : 1;
    }bits;
}phyStatusReg_t;


// --- CR_MAC_ITC ---
typedef  union
  {
    unsigned int  value;
    struct
    {
    	
      unsigned int  reserved       : 16;  // Reserved
      unsigned int  txint_time_sel : 1;   // TXINT_TIME_SEL   
      unsigned int  txint_thr      : 3;   // TXINT_THR
      unsigned int  txint_cnt      : 4;   // TXINT_CNT
      unsigned int  rxint_time_sel : 1;   // RXINT_TIME_SEL
      unsigned int  rxint_thr      : 3;   // RXINT_THR
      unsigned int  rxint_cnt      : 4;   // RXINT_CNT

    }bits;
  }mac_intr_timer_ctrl_reg_t;

// --- CR_MAC_APTC ---
typedef  union
  {
    unsigned int  value;
    struct
    {
      unsigned int  reserved        : 9;  // Reserved
      unsigned int  txpoll_time_sel : 1;  // TXPOLL_TIME_SEL  
      unsigned int  txpoll_cnt      : 4;  // TXPOLL_CNT
      unsigned int  rsv             : 3;
      unsigned int  rxpoll_time_sel : 1;  // RXPOLL_TIME_SEL
      unsigned int  rxpoll_cnt      : 4;  // RXPOLL_CNT
    }bits;
  }mac_autopoll_timer_ctrl_reg_t;

// --- CR_MAC_DBLAC ---
typedef union
  {
    unsigned int  value;
    struct
    {
      unsigned int    reserved   :24;   // Reserved
      unsigned int  rx_thr_en  : 1;   // RX_THR_EN
      unsigned int  rxfifo_thr : 3;   // RXFIFO_THR
      unsigned int  incr16_en  : 1;   // INCR16_EN
      unsigned int  incr8_en   : 1;   // INCR8_EN
      unsigned int  incr4_en   : 1;   // INCR4_EN
    }bits;
 }mac_dma_arb_ctrl_reg_t;
 
 
// --- Force MAC to poll Tx ---
#define   MAC_TX_POLL_DEMAND \
          *(volatile unsigned int *)(CR_MAC_TXPD) = (0xffffffff);

// --- Force MAC to poll Rx ---
#define   MAC_RX_POLL_DEMAND \
          *(volatile unsigned int *)(CR_MAC_RXPD) = (0xffffffff);

// --- setup MAC address ---
          // --- Setup MAC address ---
#define   MAC_SET_MAC_ADDRESS_(x,y) \
          *(volatile unsigned int *)(CR_MAC_LADR) = (x&0x0000ffff); \
          *(volatile unsigned int *)(CR_MAC_MADR) = (y&0xffffffff);
          
/*********************
 * Mac Control Block *
 *********************/


typedef struct macDrvCtrl_s {
	macTxDescr_t *txFreeDescr_p;	/* Tx free descriptor */

	macTxDescr_t *txBusyDescr_p;	/* Tx busy descriptor */
	unsigned short txQueuedFrameCnt;		/* cnts of frames haven't freed yet */
	unsigned int macDrvResetCnt;			/* reset counts */
} macDrvCtrl_t;


/* ----- Frame Block ----- */
typedef struct macFrame_s {
	struct macFrame_s *next;
	unsigned char *bufAddrp;
} macFrame_t;


/* ----- MIB-II ----- */
typedef struct macMIB_II_s {
#ifndef CONFIG_TRENDCHIP_TC3162L2
	unsigned int ifadminstatus;		/* MIB adminstatus */
//    uint8 *ifDescr;				/* MIB Interface Name */
#endif
	unsigned int inOctets;			/* Receive Octets */
	unsigned int inUnicastPkts;		/* Receive Unicast Packets */
	unsigned int inMulticastPkts;		/* Receive Multicast Packets */
	unsigned int inDiscards;			/* Receive Discard Packets */
	unsigned int inErrors;			/* Receive Error Packets */
	unsigned int inUnknownProtocols;	/* Receive Unknown Prototol Packets */
	unsigned int outOctets;			/* Transmit Octets */
	unsigned int outUnicastPkts;		/* Transmit Unicast Packets */
	unsigned int outMulticastPkts;	/* Transmit Multicast Packets */
	unsigned int outDiscards;			/* Transmit Discard Packets */
	unsigned int outErrors;			/* Transmit Error Packets */
} macMIB_II_t;

/* ----- Adapter Statistics ----- */
typedef struct inSiliconStat_s {
	unsigned int txJabberTimeCnt;
	unsigned int txLossOfCarrierCnt;
	unsigned int txNoCarrierCnt;
	unsigned int txLateCollisionCnt;
	unsigned int txExCollisionCnt;
	unsigned int txHeartbeatFailCnt;
	unsigned int txCollisionCnt;
	unsigned int txExDeferralCnt;
	unsigned int txUnderRunCnt;		

	unsigned int rxAlignErr;
	unsigned int rxSymbolErr;
	unsigned int rxMiiErr;
	unsigned int rxCrcErr;
	unsigned int rxLengthErr;
	unsigned int rxDribblingErr;
	unsigned int rxRuntErr;
	unsigned int rxCollisionErr;
} inSiliconStat_t;


/* ----- Ethernet Link Profile ----- */
typedef struct macPhyLinkProfile_s {
	unsigned int linkSpeed;							/* 10Mbps or 100Mbps */
	unsigned int duplexMode;							/* Half/Full Duplex Mode */
	unsigned int enetMode;
	unsigned int ANCompFlag;							/* auto_negotiation complete Flag */
	unsigned int PollCount;							/* auto_negotiation polling check count */
} macPhyLinkProfile_t;

#define DESCRIPTOR_POOL_SIZE	( MAC_DESCP_SIZE * (RX_DESCP_NO + TX_DESCP_NO) + 16 ) /* with 16 byte alignment */

/* ----- Ethernet private memory pool ----- */
typedef struct macMemPool_s {
	unsigned char descrPool[DESCRIPTOR_POOL_SIZE];	/* Descr pool area */
} macMemPool_t;
 
/* ----- Ethernet memory allocation map ----- */
typedef struct macAllocMap_s {
//	allocMap_t descrRing;			/* Descr Ring Allocation Map		*/
//	allocMap_t rxBufMap[MAX_FBL_ENTRIES];
//	allocMap_t maxTxBufMap[MAX_TFD_ENTRIES];
	unsigned int	txDescpBaseVA;	/* tx descriptor base virtual address */
	unsigned int	rxDescpBaseVA;	/* rx descriptor base virtual address */
} macAllocMap_t;

/* ----- Statistics for TC35815 ----- */
typedef struct macStat_s {
	macMIB_II_t MIB_II;	/* MIB-II */
	inSiliconStat_t inSilicon;	/* TC35815 statistics  */
	
} macStat_t;


#if 0 /* Sam marked */
/* ----- Adapter Card Table ------ */
typedef struct macAdapter_s {
	unsigned int ChanID;							/* Channel ID */
	//uint32 slotNumber;						/* PCI Slot No.(Indx No.) */
	unsigned char macAddr[6];						/* MAC-Address */

	macPhyLinkProfile_t *macPhyLinkProfile_p;
	//macAllocMap_t *macAllocMap_p;
	macMemPool_t *macMemPool_p;
	macStat_t *macStat_p;
	unsigned char statisticOn;						/* Flag to record statistics or not */

	//uint32 ioBaseAddr;						/* PCI I/O Base Address */

	unsigned int descrSize;					/* The size of a descr entry into the descr ring */
	unsigned int rxRingSize;						/* Receive Descr Ring Size */
	unsigned int resetNum;						/* Number of Reset the LAN Cont.	*/
	unsigned int enetPhyAddr;
	unsigned char enetPhyId;

	/* zd2001 */
	macTxDescr_t* txCurrDescp;				/* Currently working tx descriptor */
	unsigned int txDescrRingBaseVAddr;			/* Transmit Descr Ring Virtual Address with 4byte alignment	*/
	unsigned int rxDescrRingBaseVAddr;			/* Receive Descr Ring Virtual Address with 4byte alignment*/
	unsigned int txCurrentDescp;					/* index to current tx descriptor */
	unsigned int txUnReleasedDescp;				/* index to the unreleased descp of Tx */
	unsigned int txUnReleasedBufCnt;				/* Unreleased buffer cnt hold by Tx	*/
	unsigned int rxCurrentDescp;					/* index to current rx descriptor */

	int (*DrvRxUpCall)(int ChanID, mbuf_t *mbuf);
	

	int (*PktFilter)(uint8 id, uint8 *sharemem, int length);	/* NULL: perform routing, Non-NULL:bridging table check */
	int (*DrvErrorLog)(int errType, uint16 *edp);
	void (*DrvTxCompleted)(int ChanID);

	unsigned char DevType;
	unsigned char ifaceType;
	unsigned char TxSignal;
//	chann_t *chann_p; //marked by Robert

} macAdapter_t;
#endif

typedef struct phyDeviceList_s {
	unsigned short companyId;
	char vendorName[30];
} phyDeviceList_t;




#ifdef CONFIG_TRENDCHIP_TC3162L2
/* huan shiou modify to add index indicator and spinlock */
/* private data for the ethernet device */
typedef struct {
	struct net_device_stats stats; /* stats */
	volatile unsigned long last_txbytes;
	volatile unsigned long last_rxidx;      /* last rx idx to descriptor */
	volatile unsigned long current_txidx;	/* current tx index */
	volatile unsigned long current_rxidx;	/* current rx index */
	volatile unsigned long unreleased_txcnt;	/* unrelease descricptor */
	volatile unsigned long released_txidx;
// Sam marked   macAdapter_t mac;	/* emac adapter */
// HuanShiou add to fit the isa-skelton 20040917
	spinlock_t lock;
	int sk_count;
} TC3162_EMAC_PRIV;
#else

/* private data for the ethernet device */
typedef struct {
  struct net_device_stats stats; /* stats */
  unsigned long last_rxidx;      /* last rx idx to descriptor */
  unsigned long next_txidx;      /* next tx idx to descriptor */
  unsigned long last_txbytes;    /* last tx transferred bytes */
// Sam marked   macAdapter_t mac;	/* emac adapter */
} TC3162_EMAC_PRIV;

#endif

// Sam #endif /* !__ASSEMBLY__ */
#endif /* __TC3162_EMAC_H__ */

