
/*  
 * Definitions used to for hardware timers.
 *
 * Written by Robert Wu, Billion Co.
 * Support for TrendChip TC3162
 */
 
#ifndef __TC3162_TIMER_H__ 
#define __TC3162_TIMER_H__ 

/* There are six 32-bit programmed timers */
/* Each timer can be programmed as interval mode or toggle mode. */
/* Timer5 is programmed as software watchdog timer. */

/*
 * Timer registers
 */
#define TC3162_TIMER_BASE	(KSEG1 + 0x1FBF0100)
#define TC3162_TIMERCTLR_REG	(TC3162_TIMER_BASE + 0x0)
#define TC3162_TIMER0LDR_REG	(TC3162_TIMER_BASE + 0x4)
#define TC3162_TIMER0VLR_REG	(TC3162_TIMER_BASE + 0x8)
#define TC3162_TIMER1LDR_REG	(TC3162_TIMER_BASE + 0xC)
#define TC3162_TIMER1VLR_REG	(TC3162_TIMER_BASE + 0x10)
#define TC3162_TIMER2LDR_REG	(TC3162_TIMER_BASE + 0x14)
#define TC3162_TIMER2VLR_REG	(TC3162_TIMER_BASE + 0x18)
#define TC3162_TIMER3LDR_REG	(TC3162_TIMER_BASE + 0x1C)
#define TC3162_TIMER3VLR_REG	(TC3162_TIMER_BASE + 0x20)
#define TC3162_TIMER4LDR_REG	(TC3162_TIMER_BASE + 0x24)
#define TC3162_TIMER4VLR_REG	(TC3162_TIMER_BASE + 0x28)
#define TC3162_TIMER5LDR_REG	(TC3162_TIMER_BASE + 0x2C)
#define TC3162_TIMER5VLR_REG	(TC3162_TIMER_BASE + 0x30)

/* Timer Control register setting */

#define TIMERCTLR_Timer3_ExtHalt_Enable		0x20000000
#define TIMERCTLR_Timer2_ExtHalt_Enable 	0x10000000
#define TIMERCTLR_Timer1_ExtHalt_Enable 	0x8000000
#define TIMERCTLR_Timer0_ExtHalt_Enable 	0x4000000
#define TIMERCTLR_Watchdog_Timer_Enable 	0x2000000
#define TIMERCTLR_TICK_Enable			0x1000000
#define TIMERCTLR_Timer5_Int 			0x200000
#define TIMERCTLR_Timer4_Int			0x100000
#define TIMERCTLR_Timer3_Int			0x80000	
#define TIMERCTLR_Timer2_Int			0x40000
#define TIMERCTLR_Timer1_Int			0x20000
#define TIMERCTLR_Timer0_Int			0x10000
#define TIMERCTLR_Timer5_Watchdog_Select	0x2000
#define TIMERCTLR_Timer4_Select			0x1000
#define TIMERCTLR_Timer3_Select			0x800
#define TIMERCTLR_Timer2_Select			0x400
#define TIMERCTLR_Timer1_Select			0x200	
#define TIMERCTLR_Timer0_Select			0x100
#define TIMERCTLR_Timer5_Enalbe			0x20
#define TIMERCTLR_Timer4_Enalbe			0x10
#define TIMERCTLR_Timer3_Enalbe			0x8
#define TIMERCTLR_Timer2_Enalbe			0x4
#define TIMERCTLR_Timer1_Enalbe			0x2
#define TIMERCTLR_Timer0_Enalbe			0x1

#define SYS_TIMER	0
#define SYS_TIMER1	1
#define SYS_TIMER2	2
#define SYS_TIMER3	3
#define SYS_TIMER4	4
#define WD_TIMER	5

#define     TIMER_ENABLE         1
#define     TIMER_DISABLE        0
#define     TIMER_TOGGLEMODE     1
#define     TIMER_INTERVALMODE   0
#define     TIMER_TICKENABLE     1
#define     TIMER_TICKDISABLE    0
#define     TIMER_WDENABLE       1
#define     TIMER_WDDISABLE      0
#define     TIMER_HALTENABLE     1
#define     TIMER_HALTDISABLE    0
      
#define     TIMERTICKS_1MS            1  
#define     TIMERTICKS_10MS          10  // set timer ticks as 10 ms
#define     TIMERTICKS_100MS        100
#define     TIMERTICKS_1S          1000 
#define     TIMERTICKS_10S        10000

#ifndef __ASSEMBLY__

static inline void tc3162_enable_timer(int timer_num)
{
  unsigned long reg = TC3162_TIMERCTLR_REG;
  switch(timer_num) {
    case SYS_TIMER: *(volatile unsigned long *)reg |= TIMERCTLR_Timer0_Enalbe;
	    break;
    case SYS_TIMER1: *(volatile unsigned long *)reg |= TIMERCTLR_Timer1_Enalbe;
	    break;
    case SYS_TIMER2: *(volatile unsigned long *)reg |= TIMERCTLR_Timer2_Enalbe;
	    break;
    case SYS_TIMER3: *(volatile unsigned long *)reg |= TIMERCTLR_Timer3_Enalbe;
	    break;
    case SYS_TIMER4: *(volatile unsigned long *)reg |= TIMERCTLR_Timer4_Enalbe;
	    break;
    case WD_TIMER: *(volatile unsigned long *)reg |= TIMERCTLR_Timer5_Enalbe;
	    break;
    default: return;
  	     break;
  }
}

static inline void tc3162_disable_timer(int timer_num)
{
  unsigned long reg = TC3162_TIMERCTLR_REG;
  switch(timer_num) {
    case SYS_TIMER: *(volatile unsigned long *)reg &= ~(TIMERCTLR_Timer0_Enalbe);
	    break;
    case SYS_TIMER1: *(volatile unsigned long *)reg &= ~(TIMERCTLR_Timer1_Enalbe);
	    break;
    case SYS_TIMER2: *(volatile unsigned long *)reg &= ~(TIMERCTLR_Timer2_Enalbe);
	    break;
    case SYS_TIMER3: *(volatile unsigned long *)reg &= ~(TIMERCTLR_Timer3_Enalbe);
	    break;
    case SYS_TIMER4: *(volatile unsigned long *)reg &= ~(TIMERCTLR_Timer4_Enalbe);
	    break;
    case WD_TIMER: *(volatile unsigned long *)reg &= ~(TIMERCTLR_Timer5_Enalbe);
	    break;
    default: return;
  	     break;
  }
}

#ifndef CONFIG_TRENDCHIP_TC3162
static inline void tc3160_enable_timer_interrupt(int timer_num)
{
  if ((timer_num > 0) && (timer_num < 4)) {
    *(volatile unsigned long *)TC3160_TIM_REG |= (0x1 << (timer_num - 1));
    tc3160_enable_timer(timer_num);
  }
}

static inline void tc3160_disable_timer_interrupt(int timer_num)
{
  tc3160_disable_timer(timer_num);
  if ((timer_num > 0) && (timer_num < 4))
    *(volatile unsigned long *)TC3160_TIM_REG &= ~(0x1 << (timer_num - 1));
}

static inline void tc3162_init_timers(void)
{

  /* Stop all timers */
  *(volatile unsigned long *)TC3160_TM1_REG &= 0xfffffffe;
  *(volatile unsigned long *)TC3160_TM2_REG &= 0xfffffffe;
  *(volatile unsigned long *)TC3160_TM3_REG &= 0xfffffffe;

  /* Select default clock source: 10MHz */
  *(volatile unsigned long *)TC3160_TCS_REG &= 0xffffffc0;

  /* Set prescaler factor to 0 for all timers: 200ns */
  *(volatile unsigned long *)TC3160_TP_REG &=  0xfff0f0f0;

  /* Set up timer 1 as interval timer */
  *(volatile unsigned long *)TC3160_TM1_REG |= 0x6;
  *(volatile unsigned long *)TC3160_TH1_REG = (10000000 / HZ); /* 10MHz */
//  *(volatile unsigned long *)TC3160_TH1_REG = (50000 / HZ); /* 10MHz */
  *(volatile unsigned long *)TC3160_TL1_REG = 0x0;

  /* Set up timer 2 as interval timer, needed?? */
  *(volatile unsigned long *)TC3160_TM2_REG |= 0x6;
  *(volatile unsigned long *)TC3160_TH2_REG = (10000000 / HZ);
  *(volatile unsigned long *)TC3160_TL2_REG = 0x0;

  /* Set up timer 3 as watchdog */
  *(volatile unsigned long *)TC3160_TM3_REG |= 0xe;
  *(volatile unsigned long *)TC3160_WM_REG &= 0xfffffffe;
  *(volatile unsigned long *)TC3160_TH3_REG = 1000; /* ~2 sec */
  *(volatile unsigned long *)TC3160_TL3_REG = 0;
}
#endif


#endif /* !__ASSEMBLY__ */
#endif /* __TC3162_TIMER_H__ */ 

