/*
 * SIP extension for IP connection tracking. 
 *
 * Copyright (C) 2004, CyberTAN Corporation
 * All Rights Reserved.
 *
 * THIS SOFTWARE IS OFFERED "AS IS", AND CYBERTAN GRANTS NO WARRANTIES OF ANY
 * KIND, EXPRESS OR IMPLIED, BY STATUTE, COMMUNICATION OR OTHERWISE. CYBERTAN
 * SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A SPECIFIC PURPOSE OR NONINFRINGEMENT CONCERNING THIS SOFTWARE.
 *
 */

#include <linux/module.h>
#include <linux/netfilter.h>
#include <linux/ip.h>
#include <linux/ctype.h>
#include <net/checksum.h>
#include <net/tcp.h>

#include <linux/netfilter_ipv4/lockhelp.h>
#include <linux/netfilter_ipv4/ip_conntrack_helper.h>
#include <linux/netfilter_ipv4/ip_conntrack_sip.h>
#include <linux/netfilter_ipv4/ip_conntrack_core.h>
#include <linux/proc_fs.h>

DECLARE_LOCK (ip_sip_lock);
/* work around for sip alg rtp port */
#define DATA_LEN 128
static char wk_buffer[64];
static int PID;
char r_SIP_flag;
char r_SIP_address[2][DATA_LEN];
//int r_SIP_address_len[2];
//EXPORT_SYMBOL (r_SIP_address);
//export ?
struct module *ip_conntrack_sip = THIS_MODULE;

#define MAX_PORTS 8
static int ports[MAX_PORTS];
static int ports_c;
#ifdef MODULE_PARM
MODULE_PARM (ports, "1-" __MODULE_STRING (MAX_PORTS) "i");
#endif

#if 0
#define DEBUGP printk
#else
#define DEBUGP(format, args...)
#endif

#define RTCP_SUPPORT
static int
set_expected_rtp (struct ip_conntrack *ct, u_int32_t dstip,
		  u_int16_t dstport, unsigned int conntype)
{
  struct ip_conntrack_expect expect, *exp = &expect;
  struct ip_ct_sip_expect *exp_sip_info = &exp->help.exp_sip_info;
  int ret = 0;

  memset (&expect, 0, sizeof (expect));
  LOCK_BH (&ip_sip_lock);

  DEBUGP ("conntrack_sip: %s: [%s]: DST=%u.%u.%u.%u:%u\n", __FUNCTION__,
	  conntype == CONN_RTP ? "RTP" : "RTCP", NIPQUAD (dstip), dstport);

  /* exp_sip_info */
  exp_sip_info->port = dstport;
  exp_sip_info->type = conntype;
  exp_sip_info->nated = 0;

  /* new expectation */
  exp->tuple = ((struct ip_conntrack_tuple)
		{
		{
		0,
		{
		0}},
		{
		dstip,
		{
		htons (dstport)}, IPPROTO_UDP}});
  exp->mask = ((struct ip_conntrack_tuple)
	       {
	       {
	       0,
	       {
	       0}},
	       {
	       0xFFFFFFFF,
	       {
	       0xFFFF}, 0xFFFF}});
  exp->expectfn = NULL;

  if ((ret = ip_conntrack_expect_related (ct, exp)) != 0)
    {
      DEBUGP ("Can't add new expectation. \n");
    }

  UNLOCK_BH (&ip_sip_lock);

  /* work around code for rtp port */
  memset (wk_buffer, 1, 64);
  sprintf (wk_buffer, "%u.%u.%u.%u:%u\n", NIPQUAD (dstip), dstport);
  /* signal workaround ap */
  DEBUGP ("%x %u.%u.%u.%u:%u\n", dstip, NIPQUAD (dstip), dstport);
#if 1
  if ( PID > 0)
  {
	  DEBUGP("PID=%d\n",PID);
  sys_kill(PID, SIGUSR2);
  }
#endif
  return ret;
}

/*
static int unset_expected_rtp(struct ip_conntrack *ct, u_int32_t dstip, u_int16_t dstport)
{
	struct ip_conntrack_expect *exp;
	const struct ip_conntrack_tuple tuple = { { 0, { 0 } },
			{ dstip, { htons(dstport) }, IPPROTO_UDP }};

	LOCK_BH(&ip_sip_lock);

	exp = ip_conntrack_expect_find_get(&tuple);
	if (exp) {
		DEBUGP("Find the expectation %p, then delete it.\n", exp);
		ip_conntrack_unexpect_related(exp);
	}

	UNLOCK_BH(&ip_sip_lock);

	return 0;
}*/

int
find_sip_uri (const char *data, size_t dlen,
		   unsigned int *numoff, unsigned int *numlen, unsigned int *numoff1, unsigned int *numlen1, int flag)
{
  char *st, *p = (char *) data;
  const char *limit = data + dlen;
//  unsigned char p1, p2, p3, p4;
#if 1
  while (p < limit)
    {
      /* find the To: 'sip:' line */

      if (strncmp (p, "From:", 5) )
	{
	  p++;
	  continue;
	}
      p += 5;
      break;
    }
  if(p>=limit)
  {
	  return 0;
  }
#endif
  while (p < limit)
    {
      /* find the first 'sip:' line */
      if (strncmp (p, "<sip:", 5) )
	{
	  p++;
	  continue;
	}
      p += 5;

      /* SIP URI */
      st = p;

      /* SIP URI */
      while (isalpha (*p) || isdigit (*p) || (*p == '.') || (*p == '-') || (*p == '@') || (*p == '_') || (*p == '+'))
	{
	  p++;
	  if (p >= limit)
	  {
	     return 0;
	  }
	}

      *numoff = st - data;
      *numlen = p - st;
      
	if(!flag)	
      	   return 1;
	else
		break;

    }
  if( p >= limit)
  {
  printk("alg_from\n");
	  return 0;
  }
  
  while (p < limit)
    {
      /* find the To: 'sip:' line */

      if (strncmp (p, "To:", 3) )
	{
	  p++;
	  continue;
	}
      p += 3;
      break;
    }
  if( p >= limit)
  {
  printk("alg_to\n");
	  return 0;
  }


  while (p < limit)
    {
      if (strncmp (p, "<sip:", 5) )
	{
	  p++;
	  continue;
	}
      p += 5;

      /* SIP URI */
      st = p;

      /* SIP URI */
      while (isalpha (*p) || isdigit (*p) || (*p == '.') || (*p == '-') || (*p == '@') || (*p == '_') || (*p == '+') )
	{
	  p++;
	  if (p >= limit)
	  {
	    return 0;
	  }
	}

      *numoff1 = st - data;
      *numlen1 = p - st;
      
	if(flag)
	{
      	   return 2;
	}
	else
		break;

    }

  *numoff=0;
  *numoff1=0;
  *numlen=0;
  *numlen1=0;
  printk("alg_none\n");
  return 0;
}

/* return:
 *       0 : Not found
 *       1 : Found domain name
 *       2 : Found dotted quads
 */
int
find_sdp_rtp_addr (const char *data, size_t dlen,
		   unsigned int *numoff, unsigned int *numlen,
		   u_int32_t * addr)
{
  char *st, *p = (char *) data;
  const char *limit = data + dlen;
  unsigned char p1, p2, p3, p4;

  while (p < limit)
    {
      /* find 'c=' line */
      if (strncmp (p, "\nc=", 3) && strncmp (p, "\rc=", 3))
	{
	  p++;
	  continue;
	}
      p += 3;
      break;
    }
	  if (p >= limit)
	  {
	     return 0;
	  }
  while(p < limit)
  {
      if (strncmp (p, "IN IP4 ", 7))
      {
	      p++;
	continue;
      }
      p += 7;

      /* IP address */
      st = p;

      /* FQDNs or dotted quads */
      while (isalpha (*p) || isdigit (*p) || (*p == '.') || (*p == '-'))
	{
	  p++;
	  if (p >= limit)
	    return 0;
	}

      *numoff = st - data;
      *numlen = p - st;

      /* Convert the IP address */
      p1 = simple_strtoul (st, &st, 10);
      if (*st != '.')
	return 1;
      p2 = simple_strtoul (st + 1, &st, 10);
      if (*st != '.')
	return 1;
      p3 = simple_strtoul (st + 1, &st, 10);
      if (*st != '.')
	return 1;
      p4 = simple_strtoul (st + 1, &st, 10);

      *addr = (p1 << 24) | (p2 << 16) | (p3 << 8) | p4;

      return 2;
    }

  return 0;
}

u_int16_t
find_sdp_audio_port (const char *data, size_t dlen,
		     unsigned int *numoff, unsigned int *numlen)
{
  char *st, *p = (char *) data;
  const char *limit = data + dlen;
  u_int16_t port = 0;

  while (p < limit)
    {
      /* find 'm=' */
      if (strncmp (p, "\nm=", 3) && strncmp (p, "\rm=", 3))
	{
	  p++;
	  continue;
	}
      p += 3;
      break;
    }
	  if (p >= limit)
	  {
	     return 0;
	  }

  while (p < limit)
    {
      /* audio stream */
      if (strncmp (p, "audio ", 6))
      {
	      p++;
	continue;
      }
      p += 6;

      st = p;
      port = simple_strtoul (p, &p, 10);

      *numoff = st - data;
      *numlen = p - st;

      return port;
    }

  return 0;
}

static int
help (const struct iphdr *iph, size_t len,
      struct ip_conntrack *ct, enum ip_conntrack_info ctinfo)
{
  int dir = CTINFO2DIR (ctinfo);
  unsigned int matchlen, matchoff;
  unsigned int matchlen1, matchoff1;

  u_int32_t ipaddr = 0;
  u_int16_t port = 0;

  int found = 0;
  struct udphdr *udph = (void *) iph + iph->ihl * 4;
  const char *data = (const char *) udph + 8;
  unsigned int udplen = len - iph->ihl * 4;
  unsigned int datalen = udplen - 8;
  struct ip_ct_sip_master *ct_sip_info = &ct->help.ct_sip_info;

  DEBUGP ("\nconntrack_sip: help(): DIR=%d, conntrackinfo=%u\n", dir, ctinfo);
  DEBUGP ("conntrack_sip: %u.%u.%u.%u:%u -> %u.%u.%u.%u:%u\n",
	  NIPQUAD (ct->tuplehash[dir].tuple.src.ip),
	  ntohs (ct->tuplehash[dir].tuple.src.u.udp.port),
	  NIPQUAD (ct->tuplehash[dir].tuple.dst.ip),
	  ntohs (ct->tuplehash[dir].tuple.dst.u.udp.port));


  
  /* Reset for a new incoming packet */
  ct_sip_info->mangled = 0;

  /* keep the connection alive */
  ip_ct_refresh (ct, (SIP_EXPIRES * HZ));

  /* Don't need to set the expectation for upstream direction */
  if (dir == IP_CT_DIR_REPLY)
    return NF_ACCEPT;

  /* bypass act */
#if 0
  if( (ct->tuplehash[dir].tuple.src.ip & 0xffff0000) != (0xc0a80000) ) //192.168.0.0
  {
	  return NF_ACCEPT;
  }
#else
  if(r_SIP_flag)
  {
  	found = find_sip_uri (data, datalen, &matchoff, &matchlen, &matchoff1, &matchlen1, 1);
	DEBUGP("f=%d\n",found);
	if (found)
	{
		char buf[64];
		char buf1[64];
		snprintf(buf,matchlen+1,"%s",(data + matchoff));
		DEBUGP("%s\n",buf);
		snprintf(buf1,matchlen+1-4,"%s",(data + matchoff+4));
//		printk("%s\n",buf1);
		DEBUGP("%s\n",(r_SIP_address[0]+1));
		DEBUGP("%s\n",(r_SIP_address[1]+1));
		if((! strcmp(buf,r_SIP_address[0]) ) || (! strcmp(buf,r_SIP_address[1]) ))
		{
		return NF_ACCEPT;
		}
		else if ((! strcmp(buf1,(r_SIP_address[0]+1)) ) || (! strcmp(buf1,(r_SIP_address[1]+1)) ))
		{
			DEBUGP("IAD CONN phone1\n");
			return NF_ACCEPT;

		}
		memset(buf,0,64);
		memset(buf1,0,64);
//		printk("--%d %d\n",matchoff1,matchlen1);
		snprintf(buf,matchlen1+1,"%s",(data + matchoff1));
		printk("%s\n",buf);
		snprintf(buf1,matchlen1+1-4,"%s",(data + matchoff1+4));
		printk("==%s\n",buf1);
		if((! strcmp(buf,r_SIP_address[0]) ) || (! strcmp(buf,r_SIP_address[1]) ))
		{
		return NF_ACCEPT;
		}
		else if ((! strcmp(buf1,(r_SIP_address[0]+1)) ) || (! strcmp(buf1,(r_SIP_address[1]+1)) ))
		{
			DEBUGP("IAD CONN phone2\n");
			return NF_ACCEPT;

		}
	}

  }
#endif
  
  /* Need to set the expected connection for further incoming RTP stream */
  if (strncmp (data, "INVITE", 6) != 0
      && strncmp (data, "SIP/2.0 200", 11) != 0)
    {
      DEBUGP ("conntrack_sip: Not interesting packet.\n");
      return NF_ACCEPT;
    }

  /* Find RTP address */
  found = find_sdp_rtp_addr (data, datalen, &matchoff, &matchlen, &ipaddr);
  if (!found)
  { 
	  DEBUGP("not found!\n");
    return NF_ACCEPT;
  }


  DEBUGP ("conntrack_sip: 'IN IP4' is %s.\n",
	  (found == 1) ? "FQDNs" : "dotted quads");

  /* If it's a null address, then the call is on hold */
  if (found == 2 && ipaddr == 0)
    {
      DEBUGP ("conntrack_sip: Null address is found.\n");
      return NF_ACCEPT;
    }


  /* Find audio port, and we don't like the well-known ports,
   * which is less than 1024 */
  port = find_sdp_audio_port (data, datalen, &matchoff, &matchlen);
  if (port < 1024)
    return NF_ACCEPT;

//  printk ("conntrack_sip: audio port=%d.\n", port);

  ipaddr = ct->tuplehash[dir].tuple.src.ip;
  ct_sip_info->rtpport = port;
    

  /* RFC1889 - RTP uses an even port number and the corresponding RTCP
   * stream uses the next higher (odd) port number. */
  if (set_expected_rtp (ct, ipaddr, port, CONN_RTP) == 0)
    {
#ifdef RTCP_SUPPORT
	    //marked temp
      set_expected_rtp (ct, ipaddr, port + 1, CONN_RTCP);
#endif
    }
  return NF_ACCEPT;
}

static struct ip_conntrack_helper sip[MAX_PORTS];


/* Not __exit: called from init() */
static void
fini (void)
{
  int i;
  for (i = 0; (i < MAX_PORTS) && ports[i]; i++)
    {
      DEBUGP ("ip_ct_sip: unregistering helper for port %d\n", ports[i]);
      ip_conntrack_helper_unregister (&sip[i]);
    }

cleanup_proc:
  proc_net_remove ("ip_sip_conntrack");
  proc_net_remove ("ip_sip_apid");
  proc_net_remove ("ip_sip_line1");
  proc_net_remove ("ip_sip_line2");
  proc_net_remove ("ip_sip_ct_clean");
}
static inline int sip_line(struct file *file, const char *buffer,
	  unsigned long count, void *data, int i)
{
  int len;
  char tmpbuf[DATA_LEN];
  memset(tmpbuf,0,DATA_LEN);
  MOD_INC_USE_COUNT;
  if (count > DATA_LEN)
    len = DATA_LEN;
  else
    len = count;
  memset(r_SIP_address[i],0,DATA_LEN);

  if (copy_from_user (tmpbuf, buffer, len))
    {
      MOD_DEC_USE_COUNT;
      return -EFAULT;
    }

  tmpbuf[len] = '\0';
  //sscanf(tmpbuf, "%64[^:]:",r_SIP_address[i]);
  strcpy(r_SIP_address[i],tmpbuf);
  MOD_DEC_USE_COUNT;
  printk("%s\n",r_SIP_address[i]);
	r_SIP_flag=1;
  return len;
}
	
static int
sip_line1 (struct file *file, const char *buffer,
	  unsigned long count, void *data)
{
	return sip_line(file,buffer,count,data,0);
}
static int
sip_line2 (struct file *file, const char *buffer,
	  unsigned long count, void *data)
{
	return sip_line(file,buffer,count,data,1);
}
static inline int
sip_cmp(const struct ip_conntrack *i, void *_ina)
{
	int ret = 0;

	READ_LOCK(&ip_sip_lock);
	/* If it's masquerading out this interface with a different address,
	 * or we don't know the new address of this interface. */
	if (i->tuplehash[IP_CT_DIR_REPLY].tuple.dst.u.udp.port == 5060)
		ret = 1;
	else if (i->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.u.udp.port == 5060)
		ret = 1;

	READ_UNLOCK(&ip_sip_lock);

	return ret;
}

static int
sip_ct_clean (struct file *file, const char *buffer,
	  unsigned long count, void *data)
{
  int len;
  char tmpbuf[8];
  int flag=0;
  MOD_INC_USE_COUNT;

  if (count > DATA_LEN)
    len = DATA_LEN;
  else
    len = count;

  if (copy_from_user (tmpbuf, buffer, len))
    {
      MOD_DEC_USE_COUNT;
      return -EFAULT;
    }

  tmpbuf[len] = '\0';
  sscanf (tmpbuf, "%d\n", &flag);

  //printk ("%d\n", PID);
  MOD_DEC_USE_COUNT;
  printk("flag=%d\n",flag);
  if(flag)
  {
ip_ct_selective_cleanup(sip_cmp,NULL);
//	ip_conntrack_cleanup();
  }
  return len;
}
static int
sip_apid (struct file *file, const char *buffer,
	  unsigned long count, void *data)
{
  int len;
  char tmpbuf[8];
  MOD_INC_USE_COUNT;

  if (count > DATA_LEN)
    len = DATA_LEN;
  else
    len = count;

  if (copy_from_user (tmpbuf, buffer, len))
    {
      MOD_DEC_USE_COUNT;
      return -EFAULT;
    }

  tmpbuf[len] = '\0';
  sscanf (tmpbuf, "%d\n", &PID);

  //printk ("%d\n", PID);
  MOD_DEC_USE_COUNT;
  //r_SIP_address_len = strlen(tmpbuf+8)-1;
//printk("%d\n",r_SIP_address_len );
//  memcpy(r_SIP_address,tmpbuf+8,r_SIP_address_len);
//r_SIP_flag=1;
  return len;

}

static int
list_sip_conntracks (char *buffer, char **start, off_t offset, int length)
{
  unsigned int len = 0;
  len = sprintf (buffer, "%d\n%s\n", PID, wk_buffer);
  return len;
}

static int __init
init (void)
{
  int i, ret;
  struct proc_dir_entry *proc;

	r_SIP_flag=0;
	memset(r_SIP_address[0],0,DATA_LEN);
	memset(r_SIP_address[1],0,DATA_LEN);
  
  if (ports[0] == 0)
    ports[0] = SIP_PORT;

  for (i = 0; (i < MAX_PORTS) && ports[i]; i++)
    {
      memset (&sip[i], 0, sizeof (struct ip_conntrack_helper));
      sip[i].tuple.dst.u.udp.port = htons (ports[i]);
      sip[i].tuple.dst.protonum = IPPROTO_UDP;
      sip[i].mask.dst.u.udp.port = 0xF0FF;
      sip[i].mask.dst.protonum = 0xFFFF;
      sip[i].help = help;
      sip[i].timeout = RTP_TIMEOUT;
      DEBUGP ("ip_ct_sip: registering helper for port %d\n", ports[i]);
      ret = ip_conntrack_helper_register (&sip[i]);

      if (ret)
	{
	  fini ();
	  return ret;
	}
      ports_c++;
    }
/* work around for sip alg, you should find out more than 16 bytes offset problem */
#if 1
  proc = proc_net_create ("ip_sip_conntrack", 0, list_sip_conntracks);
  if (!proc)
    goto cleanup_proc;
  //proc->owner = THIS_MODULE;
  proc = proc_net_create ("ip_sip_ct_clean", 0222, 0);
  proc->write_proc = sip_ct_clean;
  proc = proc_net_create ("ip_sip_apid", 0222, 0);
  proc->write_proc = sip_apid;
  proc = proc_net_create ("ip_sip_line1", 0222, 0);
  proc->write_proc = sip_line1;
  proc = proc_net_create ("ip_sip_line2", 0222, 0);
  proc->write_proc = sip_line2;
  if (!proc)
    goto cleanup_proc;

#endif
  return 0;
cleanup_proc:
  proc_net_remove ("ip_sip_conntrack");
  proc_net_remove ("ip_sip_apid");
  proc_net_remove ("ip_sip_line1");
  proc_net_remove ("ip_sip_line2");
  proc_net_remove ("ip_sip_ct_clean");
}


EXPORT_SYMBOL (ip_sip_lock);
EXPORT_SYMBOL (ip_conntrack_sip);
EXPORT_SYMBOL (r_SIP_flag);
EXPORT_SYMBOL (r_SIP_address);
EXPORT_SYMBOL (find_sdp_rtp_addr);
EXPORT_SYMBOL (find_sip_uri);
EXPORT_SYMBOL (find_sdp_audio_port);

module_init (init);
module_exit (fini);
