/* Kernel module to match a string into a packet.
 *
 * Copyright (C) 2000 Emmanuel Roger  <winfield@freegates.be>
 * 
 * ChangeLog
 *	19.02.2002: Gianni Tedesco <gianni@ecsc.co.uk>
 *		Fixed SMP re-entrancy problem using per-cpu data areas
 *		for the skip/shift tables.
 *	02.05.2001: Gianni Tedesco <gianni@ecsc.co.uk>
 *		Fixed kernel panic, due to overrunning boyer moore string
 *		tables. Also slightly tweaked heuristic for deciding what
 * 		search algo to use.
 * 	27.01.2001: Gianni Tedesco <gianni@ecsc.co.uk>
 * 		Implemented Boyer Moore Sublinear search algorithm
 * 		alongside the existing linear search based on memcmp().
 * 		Also a quick check to decide which method to use on a per
 * 		packet basis.
 */

/* Kernel module to match a http header string into a packet.
 *
 * Copyright (C) 2003, CyberTAN Corporation
 * All Rights Reserved.
 *
 * Description:
 *   This is kernel module for web content inspection. It was derived from 
 *   'string' match module, declared as above.
 *
 *   The module follows the Netfilter framework, called extended packet 
 *   matching modules. 
 */
#define LOGERROR(format, args...) printk(format, ## args) 
//do { if (net_ratelimit()) printk(format, ## args); } while (0)
//#define SPARQ_DEBUG
#ifdef SPARQ_DEBUG
#define SPARQ_LOG(format, args...) printk(format, ## args) 

#define LOGDEBUG(format, args...) printk(format, ## args)
#define LOGINFO(format, args...) printk(format, ## args)
#else
#define SPARQ_LOG(format, args...)
#define LOGDEBUG(format, args...)
#define LOGINFO(format, args...)
#endif

#include <linux/module.h>
#include <linux/skbuff.h>
#include <linux/shm.h>
#include <linux/msg.h>
#include <net/sock.h>
#include <linux/ip.h>
#include <linux/udp.h>
#include <linux/icmp.h>
#include <net/icmp.h>
#include <net/ip.h>
#include <net/tcp.h>
#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter_ipv4/ipt_webstr.h>
#ifdef _TMSS_KERNEL_
#include "../../apps/tmss/include/tmauIniParser.h"
#include "../../apps/tmss/include/tmClientInfo.h"
#include "../../apps/tmss/include/tmGetHardwareInfo.h"
#include "../../apps/tmss/include/tmAVEnforcer.h"
#include "../../apps/tmss/include/tmURLCache.h"
#include "../../apps/tmss/include/tmURLFilter.h"
#include "../../apps/tmss/include/tmSemaphore.h"
#include "../../apps/tmss/http_proxy/include/thpd.h"
#include "../../apps/tmss/include/tmURLFilterConf.h"
#include "../../apps/tmss/include/tmCGIErrMsg.h"
#endif

#define split(word, wordlist, next, delim) \
    for (next = wordlist, \
	strncpy(word, next, sizeof(word)), \
	word[(next=strstr(next, delim)) ? strstr(word, delim) - word : sizeof(word) - 1] = '\0', \
	next = next ? next + sizeof(delim) - 1 : NULL ; \
	strlen(word); \
	next = next ? : "", \
	strncpy(word, next, sizeof(word)), \
	word[(next=strstr(next, delim)) ? strstr(word, delim) - word : sizeof(word) - 1] = '\0', \
	next = next ? next + sizeof(delim) - 1 : NULL)

#ifdef _TMSS_KERNEL_
#define _QUEUE_URL_REQ_
#define BUFSIZE 	512
#else
#define BUFSIZE 	1024
#endif

/* Flags for get_http_info() */
#define HTTP_HOST	0x01
#define HTTP_URL	0x02
/* Flags for mangle_http_header() */
#define HTTP_COOKIE	0x04

#ifdef _TMSS_KERNEL_
#define HTTP_TMSS	0x08
#endif


typedef struct httpinfo {
    char host[BUFSIZE + 1];
    int hostlen;
    char url[BUFSIZE + 1];
    int urllen;
#ifdef _TMSS_KERNEL_
    int lastUrlReqInBlockedCache;// use to keep the cache checking results for next iptables uages.
    unsigned long rsp_blkc;// keep last query results.
#endif
} httpinfo_t;

static httpinfo_t htinfo;// use global var. to keep the parsed info for next ipatbles rules.  It will also be used in ipt_REJECT.c
#ifdef _TMSS_KERNEL_

static unsigned int dashboard_ip=0;
static int tmci_InitClientTableOK=0;

//extern int tmave_HookActionKernel(const struct sk_buff *skb, const char *url,int *newClient);
extern int tmave_HookActionKernel(const struct sk_buff *skb,const char *url, const char *host,int *newClient);
extern int urlFilterBasicChecking(unsigned char * mac,unsigned char *req_url);
extern int loadExceptionInLocal(void *user,unsigned int len);

extern url_filter_config_t UrlFilterConfig;
int sendMessageUp(uint32_t srcip,httpinfo_t *htinfo,int CommandType,const struct sk_buff *skb);

#define TMSS_BLOCK_PAGE_MESSAGE                                                 \
"HTTP/1.1 200 OKAY\r\nConnection: close\r\nContent-Type: text/html\r\n\r\n"\
"<HTML><HEAD><TITLE>URL Filter Violation</TITLE>"\
"<META http-equiv=content-type content=\"text/html; charset=ISO-8859-1\">"\
"<BODY><FONT size=6>"\
"<P>Parental Controls Violation<FONT size=2></FONT></P>"\
"</FONT><FONT size=2><P>&nbsp;</P>"\
"<P>Your Trend Micro Security Services Parental Controls service does not allow you to access this site.</P>"\
"<P>Contact your network administrator if you still want to view this site, or login to the router Web console (<a href=\"http://%s\">http://%s</a>) to change the configurations.</P>"\
"<P>The URL you requested was: http://%s%s</P>"\
"<P>This site may contain content from the following blocked category:</P>"\
"<P>%s</P>"\
"<P>Your attempt to access this site has been logged.</P></FONT>"\
"</BODY></HTML>\r\n"

/*
   To change HV Defined Categories, change the followings:
   1. tmURLFilter.h: All HV defined Categories
   2. tmURLFilter.c: *gszAllHVCat
   3. tmURLFilter.c: gTmUrlFilterDefinedCatMap[]
   4. tmURLFilter.c: UrlFilterCategoryList[]
*/

/* 
   Put all HV defined Categories in gszAllHVCat
   HV_DEFINED_CAT_PORN               01,03,06
   HV_DEFINED_CAT_ALCOHOL_TOBACCO    08
   HV_DEFINED_CAT_ILLEGAL            09
   HV_DEFINED_CAT_GAMBLING           0B
   HV_DEFINED_CAT_VIOLENCE           0E
   HV_DEFINED_CAT_ABORTION           10
   HV_DEFINED_CAT_ILLEGAL_DRUGS      19
   HV_DEFINED_CAT_OTHERS
*/
unsigned char* gszAllHVCat = "01030608090B0E1019"; /* In HEX */

const char *UrlFilterCategoryList[HV_DEFINED_CAT_NUM] =
{
    URLFILTER_CATEGORY_PORNOGRAPHY,
    URLFILTER_CATEGORY_ALCOHOL_TABACCO,
    URLFILTER_CATEGORY_ILLEGAL_QUEST,
    URLFILTER_CATEGORY_GAMBLING,
    URLFILTER_CATEGORY_VIOLENCE,
    URLFILTER_CATEGORY_ABORTION,
    URLFILTER_CATEGORY_ILLEGAL_DRUGS,
    "Unused"
};

#define TM_URL_FILTER_MAX_CAT_NUMBER 100
/* Map the TrendMicro defined Categories with HV defined Categories */
const int gTmUrlFilterDefinedCatMap[TM_URL_FILTER_MAX_CAT_NUMBER] = 
{
    HV_DEFINED_CAT_OTHERS,         //"Overides"                     00, HEX=00
    HV_DEFINED_CAT_PORN,           //"Adult/Mature Content"         01, HEX=01
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       02, HEX=02
    HV_DEFINED_CAT_PORN,           //"Pornography"                  03, HEX=03
    HV_DEFINED_CAT_OTHERS,         //"Sex Education"                04, HEX=04
    HV_DEFINED_CAT_OTHERS,         //"Intimate Apparel/Swimsuit"    05, HEX=05
    HV_DEFINED_CAT_PORN,           //"Nudity"                       06, HEX=06
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       07, HEX=07
    HV_DEFINED_CAT_ALCOHOL_TOBACCO,//"Alcohol/Tabacco"              08, HEX=08
    HV_DEFINED_CAT_ILLEGAL,        //"Illegal/Questionable"         09, HEX=09
    HV_DEFINED_CAT_OTHERS,         //"Drugs"                        10, HEX=0A
    HV_DEFINED_CAT_GAMBLING,       //"Gambling"                     11, HEX=0B
    HV_DEFINED_CAT_OTHERS,         //"Hate"                         12, HEX=0C
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       13, HEX=0D
    HV_DEFINED_CAT_VIOLENCE,       //"Violence/Hate/Racism"         14, HEX=0E
    HV_DEFINED_CAT_OTHERS,         //"Weapons"                      15, HEX=0F
    HV_DEFINED_CAT_ABORTION,       //"Abortion"                     16, HEX=10
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       17, HEX=11
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       18, HEX=12
    HV_DEFINED_CAT_OTHERS,         //"Activist/Advocacy Group"      19, HEX=13
    HV_DEFINED_CAT_OTHERS,         //"Arts/Entertainment"           20, HEX=14
    HV_DEFINED_CAT_OTHERS,         //"Business/Economy"             21, HEX=15
    HV_DEFINED_CAT_OTHERS,         //"Cult/Occult"                  22, HEX=16
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       23, HEX=17
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       24, HEX=18
    HV_DEFINED_CAT_ILLEGAL_DRUGS,  //"Illegal Drugs"                25, HEX=19
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       26, HEX=1A
    HV_DEFINED_CAT_OTHERS,         //"Education"                    27, HEX=1B
    HV_DEFINED_CAT_OTHERS,         //"Educational Institutions"     28, HEX=1C
    HV_DEFINED_CAT_OTHERS,         //"Cultural Institutions"        29, HEX=1D
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       30, HEX=1E
    HV_DEFINED_CAT_OTHERS,         //"Financial Services"           31, HEX=1F
    HV_DEFINED_CAT_OTHERS,         //"Brokerage/Trading"            32, HEX=20
    HV_DEFINED_CAT_OTHERS,         //"Games"                        33, HEX=21
    HV_DEFINED_CAT_OTHERS,         //"Government/Legal"             34, HEX=22
    HV_DEFINED_CAT_OTHERS,         //"Millitary"                    35, HEX=23
    HV_DEFINED_CAT_OTHERS,         //"Political/Activist Group"     36, HEX=24
    HV_DEFINED_CAT_OTHERS,         //"Health"                       37, HEX=25
    HV_DEFINED_CAT_OTHERS,         //"Computer/Internet"            38, HEX=26
    HV_DEFINED_CAT_OTHERS,         //"Hacking/Proxy Avoidance"      39, HEX=27
    HV_DEFINED_CAT_OTHERS,         //"Search Engines/ Portals"      40, HEX=28
    HV_DEFINED_CAT_OTHERS,         //"Web Hosting"                  41, HEX=29
    HV_DEFINED_CAT_OTHERS,         //"Web Communications"           42, HEX=2A
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       43, HEX=2B
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       44, HEX=2C
    HV_DEFINED_CAT_OTHERS,         //"Job Search/Careers"           45, HEX=2D
    HV_DEFINED_CAT_OTHERS,         //"News/Media"                   46, HEX=2E
    HV_DEFINED_CAT_OTHERS,         //"Personals/Dating"             47, HEX=2F
    HV_DEFINED_CAT_OTHERS,         //"Recreation and Hobbies"       48, HEX=30  
    HV_DEFINED_CAT_OTHERS,         //"Reference"                    49, HEX=31
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       50, HEX=32
    HV_DEFINED_CAT_OTHERS,         //"Chat/Instant Message"         51, HEX=33
    HV_DEFINED_CAT_OTHERS,         //"Email"                        52, HEX=34
    HV_DEFINED_CAT_OTHERS,         //"Newsgroups"                   53, HEX=35
    HV_DEFINED_CAT_OTHERS,         //"Religion"                     54, HEX=36
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       55, HEX=37
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       56, HEX=38
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       57, HEX=39
    HV_DEFINED_CAT_OTHERS,         //"Shopping"                     58, HEX=3A
    HV_DEFINED_CAT_OTHERS,         //"Auctions"                     59, HEX=3B
    HV_DEFINED_CAT_OTHERS,         //"Real Estate"                  60, HEX=3C
    HV_DEFINED_CAT_OTHERS,         //"Society/Lifestyle"            61, HEX=3D
    HV_DEFINED_CAT_OTHERS,         //"Gay/Lesbian"                  62, HEX=3E
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       63, HEX=3F
    HV_DEFINED_CAT_OTHERS,         //"Restaurants/Dining/Food"      64, HEX=40
    HV_DEFINED_CAT_OTHERS,         //"Sports/Recreation/Hobbies"    65, HEX=41
    HV_DEFINED_CAT_OTHERS,         //"Travel"                       66, HEX=42
    HV_DEFINED_CAT_OTHERS,         //"Vehicles"                     67, HEX=43
    HV_DEFINED_CAT_OTHERS,         //"Humor/Jokes"                  68, HEX=44
    HV_DEFINED_CAT_OTHERS,         //"Streaming Media/MP3"          69, HEX=45
    HV_DEFINED_CAT_OTHERS,         //"Educational Cheating"         70, HEX=46
    HV_DEFINED_CAT_OTHERS,         //"Software Downloads"           71, HEX=47
    HV_DEFINED_CAT_OTHERS,         //"Pay to Surf"                  72, HEX=48
    HV_DEFINED_CAT_OTHERS,         //"Glamour"                      73, HEX=49
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       74, HEX=4A
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       75, HEX=4B
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       76, HEX=4C
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       77, HEX=4D
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       78, HEX=4E
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       79, HEX=4F
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       80, HEX=50
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       81, HEX=51
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       82, HEX=52
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       83, HEX=53
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       84, HEX=54
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       85, HEX=55
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       86, HEX=56
    HV_DEFINED_CAT_OTHERS,         //"For Kids"                     87, HEX=57
    HV_DEFINED_CAT_OTHERS,         //"Web Advertisement"            88, HEX=58
    HV_DEFINED_CAT_OTHERS,         //"Web Hosting"                  89, HEX=59
    HV_DEFINED_CAT_OTHERS,         //"Unrated"                      90, HEX=5A
    HV_DEFINED_CAT_OTHERS,         //"Miscellaneous"                91, HEX=5B
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       92, HEX=5C
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       93, HEX=5D
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       94, HEX=5E
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       95, HEX=5F
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       96, HEX=60
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       97, HEX=61
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       98, HEX=62
    HV_DEFINED_CAT_OTHERS,         //"Unused"                       99, HEX=63
};

int printTMSSBlockPageInAString(char *buf)
{
	char   szBoxIp[TMSS_MAX_BUF_BOX_IP];
	//extern const int gTmUrlFilterDefinedCatMap[];
	//extern const char *UrlFilterCategoryList[HV_DEFINED_CAT_NUM];
			
	tmGetBoxIp(szBoxIp);
	if (htinfo.rsp_blkc == 0)
	{
		/* URL No Rating Result, but blocked by config */
		#define BLOCK_PAGE_NO_RATING "No Rating"
		sprintf(buf,TMSS_BLOCK_PAGE_MESSAGE, szBoxIp, szBoxIp,htinfo.host,htinfo.url,BLOCK_PAGE_NO_RATING);
	}
	else
	{
		sprintf(buf,TMSS_BLOCK_PAGE_MESSAGE, szBoxIp, szBoxIp,htinfo.host,htinfo.url,UrlFilterCategoryList[gTmUrlFilterDefinedCatMap[htinfo.rsp_blkc]]);
	}
	return 1;
}
EXPORT_SYMBOL(printTMSSBlockPageInAString);


#endif

/* Return 1 for match, 0 for accept, -1 for partial. */
static int find_pattern2(const char *data, size_t dlen,
	const char *pattern, size_t plen,
	char term,
	unsigned int *numoff,
	unsigned int *numlen)
{
    size_t i, j, k;
    int state = 0;
    *numoff = *numlen = 0;

    SPARQ_LOG("%s: pattern = '%s', dlen = %u\n",__FUNCTION__, pattern, dlen);
    if (dlen == 0)
	return 0;

    if (dlen <= plen) {	/* Short packet: try for partial? */
	if (strnicmp(data, pattern, dlen) == 0)
	    return -1;
	else 
	    return 0;
    }
    for (i = 0; i <= (dlen - plen); i++) {
	/* DFA : \r\n\r\n :: 1234 */
	if (*(data + i) == '\r') {
	    if (!(state % 2)) state++;	/* forwarding move */
	    else state = 0;		/* reset */
	}
	else if (*(data + i) == '\n') {
	    if (state % 2) state++;
	    else state = 0;
	}
	else state = 0;

	if (state >= 4)
	    break;

	/* pattern compare */
	if (memcmp(data + i, pattern, plen ) != 0)
	    continue;

	/* Here, it means patten match!! */
	*numoff=i + plen;
	for (j = *numoff, k = 0; data[j] != term; j++, k++)
	    if (j > dlen) return -1 ;	/* no terminal char */

	*numlen = k;
	return 1;
    }
    return 0;
}

static int mangle_http_header(const struct sk_buff *skb, int flags)
{
    struct iphdr *iph = (skb)->nh.iph;
    struct tcphdr *tcph = (void *)iph + iph->ihl*4;
    unsigned char *data = (void *)tcph + tcph->doff*4;
    unsigned int datalen = (skb)->len - (iph->ihl*4) - (tcph->doff*4);

    int found, offset, len;
    int ret = 0;


    SPARQ_LOG("%s: seq=%u\n", __FUNCTION__, ntohl(tcph->seq));

    /* Basic checking, is it HTTP packet? */
    if (datalen < 10)
	return ret;	/* Not enough length, ignore it */
    if (memcmp(data, "GET ", sizeof("GET ") - 1) != 0 &&
        memcmp(data, "POST ", sizeof("POST ") - 1) != 0)
	return ret;	/* Pass it */

    /* COOKIE modification */
    if (flags & HTTP_COOKIE) {
	found = find_pattern2(data, datalen, "Cookie: ", 
		sizeof("Cookie: ")-1, '\r', &offset, &len);
	if (found) {
	    char c;
	    offset -= (sizeof("Cookie: ") - 1);
	    /* Swap the 2rd and 4th bit */
	    c = *(data + offset + 2) ;
	    *(data + offset + 2) = *(data + offset + 4) ;
	    *(data + offset + 4) = c ;
	    ret++;
	}
    }

    return ret;
}

static int get_http_info(const struct sk_buff *skb, int flags, httpinfo_t *info,int urlWithHost)
{
    struct iphdr *iph = (skb)->nh.iph;
    struct tcphdr *tcph = (void *)iph + iph->ihl*4;
    unsigned char *data = (void *)tcph + tcph->doff*4;
    unsigned int datalen = (skb)->len - (iph->ihl*4) - (tcph->doff*4);

    int found, offset;
    int hostlen, pathlen;
    int ret = 0;


   // SPARQ_LOG("%s: seq=%u\n", __FUNCTION__, ntohl(tcph->seq));

    /* Basic checking, is it HTTP packet? */
    if (datalen < 10)
	return ret;	/* Not enough length, ignore it */
    if (memcmp(data, "GET ", sizeof("GET ") - 1) != 0 &&
        memcmp(data, "POST ", sizeof("POST ") - 1) != 0)
	return ret;	/* Pass it */

#ifdef _TMSS_KERNEL_
    if (!(flags & (HTTP_HOST | HTTP_URL|HTTP_TMSS)))
#else
    if (!(flags & (HTTP_HOST | HTTP_URL)))
#endif
	return ret;

    /* find the 'Host: ' value */
    found = find_pattern2(data, datalen, "Host: ", 
	    sizeof("Host: ") - 1, '\r', &offset, &hostlen);
    SPARQ_LOG("Host found=%d\n", found);

    if (!found || !hostlen)
	return ret;

    ret++;	/* Host found, increase the return value */
    hostlen = (hostlen < BUFSIZE) ? hostlen : BUFSIZE;
    strncpy(info->host, data + offset, hostlen);
    *(info->host + hostlen) = 0;		/* null-terminated */
    info->hostlen = hostlen;
    SPARQ_LOG("HOST=%s, hostlen=%d\n", info->host, info->hostlen);

#ifdef _TMSS_KERNEL_
    if (!(flags & (HTTP_URL|HTTP_TMSS)))
#else
    if (!(flags & HTTP_URL))
#endif
	return ret;

    /* find the 'GET ' or 'POST ' value */
    found = find_pattern2(data, datalen, "GET ",
	    sizeof("GET ") - 1, '\r', &offset, &pathlen);
    if (!found)
	found = find_pattern2(data, datalen, "POST ",
		sizeof("POST ") - 1, '\r', &offset, &pathlen);
  //  SPARQ_LOG("GET/POST found=%d\n", found);
    if (!found)
	    return ret;
  //  SPARQ_LOG("%s: seq=%u\n", __FUNCTION__, ntohl(tcph->seq));
  if(urlWithHost)
    {
	    if ((pathlen -= (sizeof(" HTTP/x.x") - 1)) <= 0)/* ignor this field */
		    return ret;

	    ret++;	/* GET/POST found, increase the return value */
	    pathlen = ((pathlen + hostlen) < BUFSIZE) ? pathlen : BUFSIZE - hostlen;
	    strncpy(info->url, info->host, hostlen);
	    strncpy(info->url + hostlen, data + offset, pathlen);
	    *(info->url + hostlen + pathlen) = 0;	/* null-terminated */
	    info->urllen = hostlen + pathlen;
    }
    else
    {
	    ret++;	/* GET/POST found, increase the return value */
	    if ((pathlen -= (sizeof(" HTTP/x.x") - 1)) > 0)
	    {
		    pathlen = (pathlen  < BUFSIZE) ? pathlen : BUFSIZE ;
		    strncpy(info->url, data + offset, pathlen);
	            *(info->url + pathlen) = 0;	/* null-terminated */
		    info->urllen =  pathlen;
 	    }
	    else
	    {
		    strcpy(info->url,"/");//default
		    info->urllen=1;
	    }
	    	    
    }
    SPARQ_LOG("URL=%s, urllen=%d\n", info->url, info->urllen);

    return ret;
}

/* Linear string search based on memcmp() */
static char *search_linear (char *needle, char *haystack, int needle_len, int haystack_len) 
{
	char *k = haystack + (haystack_len-needle_len);
	char *t = haystack;
	
	SPARQ_LOG("%s: haystack=%s, needle=%s\n", __FUNCTION__, t, needle);
	for(; t <= k; t++) {
		//SPARQ_LOG("%s: haystack=%s, needle=%s\n", __FUNCTION__, t, needle);
		if (strnicmp(t, needle, needle_len) == 0) return t;
		//if ( memcmp(t, needle, needle_len) == 0 ) return t;
	}

	return NULL;
}
#ifdef _TMSS_KERNEL_
/*******************************************************************************
* Porting Required Section Start
*******************************************************************************/
/*
 *  check_dashboard_url(): check if this url is dashboard's url or not
 *
 *  @param      req_url [IN] requested urlremoveFirstItemFromRatingReponseQ
 *
 *  @return     0 : not dashboard's url
 *              1 : this is dashboard's url
 */
#define DASHBOARD_PARAM_LEN 38
static int check_dashboard_url(char *req_url)
{								
    const char *full_dashboard_url;
    const char *full_dashboard_url_e; 
    const char *dashboard_url = NULL;
 //   LOGDEBUG("Calling check_dashboard_url\n");
    full_dashboard_url = tmave_GetAVServerURL(); /* http://211.76.136.195/dashboard/dashboard.aspx */
    full_dashboard_url_e = full_dashboard_url + strlen(full_dashboard_url) - 1;
     if(strcmp(full_dashboard_url,"/")==0)
    {
        LOGDEBUG("dashboard_url is not ready \n");
        return 0;
    }
 //   LOGDEBUG("Calling check_dashboard_url 33333\n");

   dashboard_url = full_dashboard_url;
    while (dashboard_url < full_dashboard_url_e && *dashboard_url != '/')
        dashboard_url++;
    dashboard_url++;
    dashboard_url++;
    while (dashboard_url < full_dashboard_url_e && *dashboard_url != '/')
        dashboard_url++;
  //   LOGDEBUG("Calling check_dashboard_url 4444444\n");
   if (strstr(req_url, dashboard_url) != NULL && strlen(req_url) >=  (strlen(dashboard_url) + DASHBOARD_PARAM_LEN))
    {
        LOGDEBUG("replace dashboard param, Get dashboard request\n");//, url is %s\n", req_url);
        LOGDEBUG("url is %s\n", req_url);
        LOGDEBUG("full_dashboard_url is %s\n", full_dashboard_url);
        LOGDEBUG("dashboard_url is %s\n", dashboard_url);
        return 1;
    }
    
   //  LOGDEBUG("Leaving check_dashboard_url\n");
   return 0;
}


int check_dashboard_urlKernel(const struct sk_buff *skb,httpinfo_t *htinfo)
{
/*******************************************************************************
* Porting Required Section Start
*******************************************************************************/                    
	uint32_t dstip=skb->nh.iph->daddr;
	int is_dashboard_url = check_dashboard_url(htinfo->url);
	unsigned char szBoxId[TMSS_MAX_BUF_BOX_ID];
	unsigned char szVendorId[TMSS_MAX_BUF_VENDER_ID];
	if (is_dashboard_url)
	{
                        ClientInfo          info;
                        
                      // if (IP_MAC(&srcip, mac) == 0)
                        {
                            if (tmci_GetClient(skb->mac.ethernet->h_source, &info) >= 0)
                            {
                                CTREDIRECTFLAG(&info) = 1;
                               tmci_SetTime(&(info.idinfo), &(info.timeinfo));
                            }
                            else
                            {
                                // No client record *
				unsigned char       *mac=skb->mac.ethernet->h_source;
				
                               LOGERROR("http_client_handler(), Client MAC(%02X:%02X:%02X:%02X:%02X:%02X) is not found in Client Table.\n", 
				mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
                            }                            
                        }
                     /*   else
                        {
                            LOGERROR("http_client_handler(), ARP table lookup fail for client %u.%u.%u.%u", NIPQUAD(srcip));
                        } */
		tmGetBoxID(szBoxId);
		tmGetVendorID(szVendorId);
	}
	//NEED TO CHECK AND REMOVE ....................................
	// Wierd, our system have bugs... Some redundant bytes is following the lfcr, we have to remove it.
	if (is_dashboard_url)
	{ // If it is dashboard, we shall not check for av enforcer and url fitler.
		// We will replace it by looking for "?_"
		char BoxIDVid[DASHBOARD_PARAM_LEN+1],*bp;
		struct tcphdr *tcph= (struct tcphdr *)((u_int32_t*)skb->nh.iph + skb->nh.iph->ihl);
		unsigned int tcplen;
		unsigned char *data = (void *)tcph + tcph->doff*4;
		/* set request URL */

		//
		if((bp=strstr(data,"/dashboard.aspx?"))==NULL)
		{
			LOGERROR( "can not get \"/dashboard.aspx?\" to replace in the url\n");//==>:%s",buff);
			// We have to pass through.....
			//We keep the IP address for this AV server, so if any traffic go to it, we will simply pass through
			return 1;
		}
		bp+=(strlen("/dashboard.aspx?"));					
		// replace 37(minus the ?) bytes after /dashboard.aspx?
 		sprintf(BoxIDVid,"BOXID=%s&VID=%s",szBoxId,szVendorId); 
		memcpy(bp,BoxIDVid,strlen(BoxIDVid));
		// And we need to recalculate the checksum.
		tcplen = skb->len - skb->nh.iph->ihl*4;

		/* Adjust TCP checksum */
		tcph->check = 0;
		tcph->check = tcp_v4_check(tcph, tcplen,
				   skb->nh.iph->saddr,
				   skb->nh.iph->daddr,
				   csum_partial((char *)tcph,
						tcplen, 0));
		/* Adjust IP checksum */
		skb->nh.iph->check = 0;
		skb->nh.iph->check = ip_fast_csum((unsigned char *)skb->nh.iph, 
					   skb->nh.iph->ihl);
		
		dashboard_ip = dstip;
		return 1;
	}
	return 0;
}

int urlInCache(httpinfo_t *htinfo)
{  // So, the MAX_URL_BUF shall be the host and one byte for "/" and one byte for end of string at least
	// So, the host can only up to 78 byte if MAX_URL_BUF in cache is 80
     char                    szReqUrlWithoutArg[MAX_URL_BUF]; // concat the host name: www.yahoo.com/index.htm
     int nRetValRemoveOptionalArgInURL,action;
      /* Remove optional arguments, the characters after '?' and concat with hostname */
#ifdef URL_RATING
  xcxcx RemoveOptionalArgInURL(htinfo->host, htinfo->hostlen, htinfo->url, htinfo->urllen, szReqUrlWithoutArg, MAX_URL_BUF);
#else
	if(htinfo->hostlen>(MAX_URL_BUF-2))
	{
		LOGERROR("Host length(=%d) to large(>%d)\n",htinfo->hostlen,(MAX_URL_BUF-2));
		return HTTP_MSG_ACTION_RELAY;
	}
 		
    nRetValRemoveOptionalArgInURL=RemoveOptionalArgInURL(htinfo->host, htinfo->hostlen, "/",strlen("/"), szReqUrlWithoutArg, MAX_URL_BUF);
    if (nRetValRemoveOptionalArgInURL == URL_TOO_LONG)
    {
         LOGERROR("concat URL too long(length=%d): %s\n",htinfo->hostlen,htinfo->host);
	 // We do not eval it. Let it pass
	 return HTTP_MSG_ACTION_RELAY;
    }
#endif
    LOGDEBUG("Call tmQueryUrlCache(%d,%s,%d)\n", UrlFilterConfig.nUrlCacheSize, szReqUrlWithoutArg, UrlFilterConfig.nUrlFilterCacheExpireTime);
    //return tmQueryUrlCache(UrlFilterConfig.nUrlCacheSize, szReqUrlWithoutArg, &(htinfo->rsp_blkc),UrlFilterConfig.nUrlFilterCacheExpireTime);	
    action=tmQueryUrlCache(szReqUrlWithoutArg, &(htinfo->rsp_blkc));
    if(action==HTTP_MSG_ACTION_BLOCK)
    {
#ifdef _CLEAER_URL_FILTER_CACHE_WHEN_URL_FILTER_CONFIG_CHANGED_
	sendMessageUp(htinfo->rsp_blkc,NULL,KERNEL_TO_THPD_CMD_INCREASE_BLOCK_CAT,NULL);		
#else
	if (UrlFilterConfig.nCategory[gTmUrlFilterDefinedCatMap[htinfo->rsp_blkc]] == 1)
	{
                //LOGINFO("Rating Result: URL: %s%s In Category, TrendMicro Definedand: %lu, HV Defined: %d, match config setting, return HTTP_MSG_ACTION_BLOCK",p_hdr_host,p_req_url, p_rating_response->rsp_blkc, gTmUrlFilterDefinedCatMap[p_rating_response->rsp_blkc]);
		sendMessageUp(htinfo->rsp_blkc,NULL,KERNEL_TO_THPD_CMD_INCREASE_BLOCK_CAT,NULL);		
	}
	else
	{
		//LOGINFO("Rating Result: URL: %s%s In Category, TrendMicro Definedand: %lu, HV Defined: %d do not match config setting, return HTTP_MSG_ACTION_RELAY", p_hdr_host,p_req_url,p_rating_response->rsp_blkc, gTmUrlFilterDefinedCatMap[p_rating_response->rsp_blkc]);
 		sendMessageUp(htinfo->rsp_blkc,NULL,KERNEL_TO_THPD_CMD_INCREASE_BLOCK_CAT,NULL);// also, we have to add the access counts.		
               return HTTP_MSG_ACTION_RELAY;
	} 
#endif
    }
    return action;	
}




static int UrlRatingReqSent=0;
#ifdef _QUEUE_URL_REQ_
static struct sk_buff *tmpskb=NULL;
#endif

#define   PERM S_IRUSR|S_IWUSR 
int sendMessageUp(uint32_t srcip,httpinfo_t *htinfo,int CommandType,const struct sk_buff *skb)
{
     int msgid = 0;
    URL_REQUEST_MESSAGE_ITEM QueueItem,* pMessageItem=&QueueItem;
      int retVal=0;
  
  /* set up the size to be total of queue size + queue head + queue tail index */
      LOGDEBUG("sendMessageUp for ip=%x for command type=%d(length=%d)\n",srcip,CommandType,sizeof(URL_REQUEST_MESSAGE_ITEM)-sizeof(long));
     if ((msgid = sys_msgget((key_t)URL_REQUEST_SHARE_KEY, PERM)) == -1)
     {
	    LOGERROR("sendMessageUp:msg queue for URL_REQUEST_SHARE_KEY is not initialized\n");
	    return -1;
     }
    pMessageItem->mtype=1;	
    pMessageItem->commandType=CommandType;	
    pMessageItem->clientIP=srcip;
    if(CommandType==KERNEL_TO_THPD_CMD_URL_REQ)
    {
	    //to be short the host len is less then MAX_URL_BUF-2 
	    if(htinfo->hostlen>(MAX_URL_BUF-2))
	    {
		    LOGERROR("WARNING length(=%d) for host to large(>%d)\n",htinfo->hostlen,MAX_URL_BUF-2);
		    return -1;
	    }
	    strncpy(pMessageItem->host,htinfo->host,htinfo->hostlen);
	    pMessageItem->host[htinfo->hostlen]='\0';
	    //pMessageItem->hostlen=htinfo->hostlen;
#ifdef URL_RATING
cxzcxzcxzc	    strncpy(pMessageItem->url,htinfo->url,htinfo->urllen);
	zxcxzc    pMessageItem->url[htinfo->urllen]='\0';
#endif
	    //pMessageItem->urllen=htinfo->urllen;
    }
    retVal=sys_msgsnd(msgid,(struct msgbuf *)pMessageItem,sizeof(URL_REQUEST_MESSAGE_ITEM)-sizeof(long),IPC_NOWAIT);  
    //a.mtype=1;
    //strcpy(a.t,"hello");
    //retVal=sys_msgsnd(msgid,&a,10,0);  
    if(retVal)
	{
		LOGERROR("sys_msgsnd failed=%x...............................................................\n",retVal);
		return -1;
    }
	else if(CommandType==KERNEL_TO_THPD_CMD_URL_REQ)
    { // We then, keep it into the temp buffer if it is rated OK to pass.
	
	 UrlRatingReqSent++;
#ifdef _QUEUE_URL_REQ_
	 tmpskb = skb_copy(skb, GFP_ATOMIC);
#endif
    }
    return 1;
}
int tmnf_UpdateClientInfoKernel(__u32 nClientIP)
{
    LOGINFO("(tmnf_UpdateClientInfoKernel) Sending Discover Information Request to PCC client\n"); 
		  
    return sendMessageUp(nClientIP,NULL,KERNEL_TO_THPD_CMD_UPDATE_CLIENT_INFO,NULL);		
}

/* interface to notify PCC Communication module to notify PCC client to update
   pattern and engine. return 0 for OK, -1 for error  */
int tmnf_UpdateClientPatternKernel(__u32 nClientIP)
{		
    //LOGERROR("tmnf_UpdateClientPatternKernel Sending Update Pattern/Engine Notification to PCC client\n");
		
    return sendMessageUp(nClientIP,NULL, KERNEL_TO_THPD_CMD_UPDATE_CLIENT_PTRN,NULL);		  		
}

#if 0
/* Stolen from ip_finish_output2 */
static void ip_direct_send(struct sk_buff *skb)
{
	struct dst_entry *dst = skb->dst;
	struct hh_cache *hh = dst->hh;

	if (hh) {
		int hh_alen;

		read_lock_bh(&hh->hh_lock);
		hh_alen = HH_DATA_ALIGN(hh->hh_len);
  		memcpy(skb->data - hh_alen, hh->hh_data, hh_alen);
		read_unlock_bh(&hh->hh_lock);
	        skb_push(skb, hh->hh_len);
		hh->hh_output(skb);
	} else if (dst->neighbour)
		dst->neighbour->output(skb);
	else {
		printk(KERN_DEBUG "khm in MIRROR\n");
		kfree_skb(skb);
	}
}
#endif
	
#ifdef SPARQ_DEBUG
static char *ipt_webstr_typestr[]={"IPT_WEBSTR_URL",
	"IPT_WEBSTR_HOST",
	"IPT_WEBSTR_CONTENT",
	"IPT_WEBSTR_TMSS"};
#endif
static int newClient=0;// This is used to keep the status about the client is a new one. So that, we do'nt 
                         // have to check if it is new client for each stage.
#endif
static int
match(const struct sk_buff *skb,
      const struct net_device *in,
      const struct net_device *out,
      const void *matchinfo,
      int offset,
      const void *hdr,
      u_int16_t datalen,
      int *hotdrop)
{
	const struct ipt_webstr_info *info = matchinfo;
	struct iphdr *ip = skb->nh.iph;
	proc_ipt_search search=search_linear;

	char token[] = "<&nbsp;>";
	char *wordlist = (char *)&info->string;
//	httpinfo_t htinfo;
	int flags = 0;
	int found = 0;
	long int opt = 0;
	int action;


	if (!ip || info->len < 1)
	    return 0;

	/* Determine the flags value for get_http_info(), and mangle packet 
	 * if needed. */
	switch(info->type)
	{
	    case IPT_WEBSTR_URL:	/* fall through */
		flags |= HTTP_URL;

	    case IPT_WEBSTR_HOST:
		flags |= HTTP_HOST;
		break;

	    case IPT_WEBSTR_CONTENT:
		opt = simple_strtol(wordlist, (char **)NULL, 10);
		SPARQ_LOG("%s: string=%s, opt=%#lx\n", __FUNCTION__, wordlist, opt);

		if (opt & (BLK_JAVA | BLK_ACTIVE | BLK_PROXY))
		    flags |= HTTP_URL;
		if (opt & BLK_PROXY)
		    flags |= HTTP_HOST;
		if (opt & BLK_COOKIE)
		    mangle_http_header(skb, HTTP_COOKIE);
		break;
		
#ifdef _TMSS_KERNEL_
	    case IPT_WEBSTR_TMSS:
		opt = simple_strtol(wordlist, (char **)NULL, 10);
		flags |= HTTP_TMSS;
		if (opt == CHECK_IF_URL_IN_DASH_BOARD)
		{
			// We only parsing the url in first iptables rule for TMSS
			if (get_http_info(skb, flags, &htinfo,0) < 1)
			{
				found = 1;// PASS IT. IT will "return" in the iptables rules
				return (found ^ info->invert);
			}
		}
		
		//SPARQ_LOG("\n************************************************\n"
		//"%s: type=%s\n", __FUNCTION__,ipt_webstr_typestr[info->type]);
		SPARQ_LOG("%s: string=%s, opt=%#lx\n", __FUNCTION__, wordlist, opt);

		if (opt == CHECK_IF_URL_IN_DASH_BOARD)
		{// We only parsing the url in first iptables rule for TMSS
			if(!tmci_InitClientTableOK)
			{// shared memory is not OK, we let it passes.
					//printk("SHARED MEM for Client Info is not ready.....\n");
					found=1;// let it pass anyway
					goto match_ret;
			}
			found=check_dashboard_urlKernel(skb,&htinfo);			
			goto match_ret;
		}
		else if (opt == CHECK_IF_CLIENT_NEED_AVE_REDIRECT)
		{
		    newClient=0;
			//found=tmave_HookActionKernel(skb,htinfo.url,&newClient);//(skb->nh.iph->saddr,skb->nh.iph->daddr,htinfo.url);
			found=tmave_HookActionKernel(skb,htinfo.url,htinfo.host,&newClient);//(skb->nh.iph->saddr,skb->nh.iph->daddr,htinfo.url);
			goto match_ret;
		}
		else if (opt == CHECK_IF_URL_NO_NEED_TO_BE_RATED)
		{
			int retVal;
			if(dashboard_ip==skb->nh.iph->daddr)
			{
				found=1;
				goto match_ret;
			}
			
			retVal=urlFilterBasicChecking(skb->mac.ethernet->h_source,htinfo.url);
			//printk("after Calling urlFilterBasicChecking, the ret=%d\n",retVal);
			//checked if url license disbled, url filtering disabled, not in schedule, the mine type is not rateable.
			if(retVal==HTTP_MSG_ACTION_PASS)
			{
				if(newClient==1)
				{
				// THis is a just new client, It have not yet created the clientInfo in the table.
				// So, it can not send the redirect pages to the client until this url request is resent.
				// So, we need to drop it here.
				goto match_ret;
				}
				else
				{				
				found=1;
				goto match_ret;
				}
			}
			LOGDEBUG("check if in pass/blocked cache\n");
			// then check if pass/blocked in cache
			htinfo.lastUrlReqInBlockedCache=0;
			action=urlInCache(&htinfo);
			LOGDEBUG("finished checkin pass/blocked cache action=%d\n",action);
			
			if(action==HTTP_MSG_ACTION_RELAY)
			{
				found=1;
				goto match_ret;
			}
			else if(action==HTTP_MSG_ACTION_BLOCK)
			{
				htinfo.lastUrlReqInBlockedCache=1;
			}	
			//else
			// not determined.   As the survelenent server.
			goto match_ret;
		}
		else if (opt == CHECK_IF_URL_IN_BLOCKED_CACHE)
		{
			if(htinfo.lastUrlReqInBlockedCache==1)
			{
				found=1;
				goto match_ret;
			}
			//// NO NEED TO CHECK if it is new client since the new client creation will be 
			//sent up before this by tmnf_UpdateClientInfoKernel.
			//if(!newClient)
				//send message to userland task to query the survellance server.

				if(UrlRatingReqSent==0)// Send only no queued request. Otherwise ,drop.
					sendMessageUp(skb->nh.iph->saddr,&htinfo,KERNEL_TO_THPD_CMD_URL_REQ,skb);
				else
				{
					//printk("CAN NOT sendMessageUp( UrlRatingReqSent=%d)\n",UrlRatingReqSent);
				}
				//Shall we keep this url request page into our page cache so that we can resend it later
				
			goto match_ret;
		}
		
		break;
#endif
	    default:
		LOGERROR("%s: Sorry! Cannot find this match option.\n", __FILE__);
		return 0;
	}

	/* Get the http header info */
	if (get_http_info(skb, flags, &htinfo,1) < 1)
	    return 0;

	/* Check if the http header content contains the forbidden keyword */
	if (info->type == IPT_WEBSTR_HOST || info->type == IPT_WEBSTR_URL) {
	    int nlen = 0, hlen = 0;
	    char needle[BUFSIZE], *haystack = NULL;
	    char *next;

	    if (info->type == IPT_WEBSTR_HOST) {
		haystack = htinfo.host;
		hlen = htinfo.hostlen;
	    }
	    else {
		haystack = htinfo.url;
		hlen = htinfo.urllen;
	    }
	    split(needle, wordlist, next, token) {
		nlen = strlen(needle);
		SPARQ_LOG("keyword=%s, nlen=%d, hlen=%d\n", needle, nlen, hlen);
		if (!nlen || !hlen || nlen > hlen) continue;
		if (search(needle, haystack, nlen, hlen) != NULL) {
		    found = 1;
		    break;
		}
	    }
	}
	else {		/* IPT_WEBSTR_CONTENT */
	    int vicelen;

	    if (opt & BLK_JAVA) {
		vicelen = sizeof(".js") - 1;
		if (strnicmp(htinfo.url + htinfo.urllen - vicelen, ".js", vicelen) == 0) {
		    SPARQ_LOG("%s: MATCH....java\n", __FUNCTION__);
		    found = 1;
		    goto match_ret;
		}
		vicelen = sizeof(".class") - 1;
		if (strnicmp(htinfo.url + htinfo.urllen - vicelen, ".class", vicelen) == 0) {
		    SPARQ_LOG("%s: MATCH....java\n", __FUNCTION__);
		    found = 1;
		    goto match_ret;
		}
	    }
	    if (opt & BLK_ACTIVE){
		vicelen = sizeof(".ocx") - 1;
		if (strnicmp(htinfo.url + htinfo.urllen - vicelen, ".ocx", vicelen) == 0) {
		    SPARQ_LOG("%s: MATCH....activex\n", __FUNCTION__);
		    found = 1;
		    goto match_ret;
		}
		vicelen = sizeof(".cab") - 1;
		if (strnicmp(htinfo.url + htinfo.urllen - vicelen, ".cab", vicelen) == 0) {
		    SPARQ_LOG("%s: MATCH....activex\n", __FUNCTION__);
		    found = 1;
		    goto match_ret;
		}
	    }
	    if (opt & BLK_PROXY){
		if (strnicmp(htinfo.url + htinfo.hostlen, "http://", sizeof("http://") - 1) == 0) {
		    SPARQ_LOG("%s: MATCH....proxy\n", __FUNCTION__);
		    found = 1;
		    goto match_ret;
		}
	    }
	}

match_ret:
	SPARQ_LOG("%s: Verdict =======> %s \n",__FUNCTION__
		, found ? "MATCH" : "MISMATCH");

	return (found ^ info->invert);
}

static int
checkentry(const char *tablename,
           const struct ipt_ip *ip,
           void *matchinfo,
           unsigned int matchsize,
           unsigned int hook_mask)
{

       if (matchsize != IPT_ALIGN(sizeof(struct ipt_webstr_info)))
               return 0;

       return 1;
}

#ifdef _TMSS_KERNEL_
extern void setTMSS_AVE_config(AVEConf *pAVEConf);
extern url_filter_config_t UrlFilterConfig;

static int do_tmss_set_ctl(struct sock *sk, int cmd, void *user, unsigned int len)
{
	int ret=0;
	AVEConf aAVEConf;
	add_url_cache the_add_url_cache;

	if (!capable(CAP_NET_ADMIN))
		return -EPERM;

	switch (cmd) {
	/*	case TMSS_SO_URL_RATING_COMPLETE:  // each completion of URL Rating shall notice the kernel
		  if(UrlRatingReqSent<=0)
		  {
			  printk("do_tmss_set_ctl:TMSS_SO_URL_RATING_COMPLETE:ERROR:UrlRatingReqSent is 0\n");
		  }
		  else
			UrlRatingReqSent--;  
		LOGERROR("do_tmss_set_ctl:TMSS_SO_URL_RATING_COMPLETE:UrlRatingReqSent is %d\n",UrlRatingReqSent);
		break; */

	case TMSS_SO_SET_AVE_CONFIG:
		if (copy_from_user(&aAVEConf, user, sizeof(AVEConf)) != 0)
		{
		LOGERROR("do_tmss_set_ctl:data error TMSS_SO_SET_AVE_CONFIG\n"); 
		return -EFAULT;
		}
		if (len != sizeof(AVEConf))
		{
		LOGERROR("do_tmss_set_ctl:data len(%x)!=sizeof(AVEConf)(%x)\n",len,sizeof(AVEConf)); 
		return -EINVAL;
		}
		setTMSS_AVE_config(&aAVEConf);
		break;

	case TMSS_SO_ADD_URL_FILTER_CACHE_NODE:
		//printk("do_tmss_set_ctl:TMSS_SO_ADD_URL_FILTER_CACHE_NODE:UrlRatingReqSent=%d\n",
		//UrlRatingReqSent);
		if(UrlRatingReqSent<=0)
		{
			LOGERROR("do_tmss_set_ctl:TMSS_SO_URL_RATING_COMPLETE:ERROR:UrlRatingReqSent is 0\n");
		}
		else
			UrlRatingReqSent--;  
		if (copy_from_user(&the_add_url_cache, user, sizeof(add_url_cache)) != 0)
		{
		LOGERROR("do_tmss_set_ctl:data error for TMSS_SO_SET_URL_FILTER_CONFIG\n"); 
#ifdef _QUEUE_URL_REQ_
		if(tmpskb)
		{
			kfree_skb(tmpskb);
			tmpskb=NULL;
		}
#endif
		return -EFAULT;
		}
		if (len != sizeof(add_url_cache))
		{
		LOGERROR("do_tmss_set_ctl:data len(%x)!=sizeof(add_url_cache)(%x)\n",len,sizeof(add_url_cache)); 
#ifdef _QUEUE_URL_REQ_
		if(tmpskb)
		{
			kfree_skb(tmpskb);
			tmpskb=NULL;
		}
#endif
		return -EINVAL;
		}
		
		LOGDEBUG("do_tmss_set_ctl:TMSS_SO_ADD_URL_FILTER_CACHE_NODE:nRateRetVal=%d\n",
		the_add_url_cache.nRateRetVal);
		tmUrlCacheAddNode(&the_add_url_cache);
#ifdef _QUEUE_URL_REQ_
		if(the_add_url_cache.nRateRetVal==HTTP_MSG_ACTION_RELAY)
		{// We send the buffer skb.
			//ip_forward(tmpskb);
			//ip_direct_send(tmpskb);
#ifdef CONFIG_NETFILTER_DEBUG			
			//turn of the nf_debug to avoid showing message like
			//nf_hook: hook 2 already set.
			//skb: pf=2 (unowned) dev=br0 len=274
			//PROTO=6 192.168.1.60:3026 204.176.88.18:80 L=274 S=0x00 I=150 F=0x4000 T=126
		    tmpskb->nf_debug = 0;
#endif
			ip_rcv(tmpskb, tmpskb->dev, NULL);

		}
		else
		{
		if(tmpskb)
		{
			kfree_skb(tmpskb);
			tmpskb=NULL;
		}
		}
#endif
		//memcpy(pUrlFilterConfig,pUrlFilterConf,sizeof(url_filter_config_t));
		//setTMSS_AVE_config(&aAVEConf);
		break;

	case TMSS_SO_SET_URL_FILTER_CONFIG:
	
		// also clean the UrlRatingReqSent
		//UrlRatingReqSent=0;
		if (copy_from_user(&UrlFilterConfig, user, sizeof(url_filter_config_t)) != 0)
		{
	memset(&UrlFilterConfig,0,sizeof(url_filter_config_t));
		LOGERROR("do_tmss_set_ctl:data error for TMSS_SO_SET_URL_FILTER_CONFIG\n"); 
		return -EFAULT;
		}
		if (len != sizeof(url_filter_config_t))
		{
	memset(&UrlFilterConfig,0,sizeof(url_filter_config_t));
		LOGERROR("do_tmss_set_ctl:data len(%x)!=sizeof(url_filter_config_t)(%x)\n",len,sizeof(url_filter_config_t)); 
		return -EINVAL;
		}
		break;
     // Aki removed since we using fixed string comparison for those exceptionals. 03/18/2005.
	//case TMSS_SO_TM_EXCEPTION:	
	//	return loadExceptionInLocal(user,len);

	case TMSS_SO_INIT_CLIENT_INFO:// NOTE !!! NOTE !!! shm_at can not be run in runtime. Must be in passive Mode.
		LOGINFO("do_tmss_set_ctl:TMSS_SO_INIT_CLIENT_INFO is called \n"); 
		tmci_InitClientTableOK=tmci_InitClientTable();
		if(!tmci_InitClientTableOK)
		{// shared memory is not OK, we let it passes.
			LOGERROR("TMSS_SO_INIT_CLIENT_INFO:ERROR!!! SHARED MEM for Client Info is not ready.....\n");
		}
		break;


	default:
		LOGERROR("do_atmss_set_ctl:  unknown request %i\n", cmd);
		ret = -EINVAL;
	}

	return ret;
}

static int do_tmss_get_ctl(struct sock *sk, int cmd, void *user, int *len)
{
	int ret=0;

/*	if (!capable(CAP_NET_ADMIN))
		return -EPERM;

	switch (cmd) {
	case TMSS_SO_GET_INFO: {
		printk("TMSS_SO_GET_INFO is called\n");

	}
	break;

	case TMSS_SO_GET_ENTRIES: {
		printk("TMSS_SO_GET_ENTRIES is called\n");
	}

	default:
		printk("do_tmsss_get_ctl: unknown request %i\n", cmd);
		ret = -EINVAL;
	} */

	return ret;
}
static struct nf_sockopt_ops tmss_sockopts
= { { NULL, NULL }, PF_INET, TMSS_BASE_CTL, TMSS_SO_SET_MAX+1, do_tmss_set_ctl,
    TMSS_BASE_CTL, TMSS_SO_GET_MAX+1, do_tmss_get_ctl, 0, NULL  };
extern int tmInitUrlFilterCache(void);
#endif

static struct ipt_match webstr_match
= { { NULL, NULL }, "webstr", &match, &checkentry, NULL, THIS_MODULE };
static int __init init(void)
{
#ifdef _TMSS_KERNEL_
	int ret=0;
	memset(&UrlFilterConfig,0,sizeof(url_filter_config_t));
	tmInitUrlFilterCache();
	ret = nf_register_sockopt(&tmss_sockopts);
	if (ret < 0) {
		LOGERROR("webstr:Unable to register sockopts.\n");
		return ret;
	}
#endif
	return ipt_register_match(&webstr_match);
}

static void __exit fini(void)
{
	ipt_unregister_match(&webstr_match);
}

module_init(init);
module_exit(fini);
