/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file tmAVEnforcer.c
 *  @brief TMSS AVEnforcer Module
 */

/*
 *  Change Log
 *  2004-02-18 Eric Li    
 *  Initialization version
 */


#include <linux/module.h>
#include <linux/skbuff.h>
#include <linux/shm.h>
#include <net/sock.h>

#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter_ipv4/ipt_webstr.h>
#include "../../apps/tmss/include/tmauIniParser.h"
#include "../../apps/tmss/include/tmClientInfo.h"
#include "../../apps/tmss/include/tmAVEnforcer.h"
#include "../../apps/tmss/include/tmURLCache.h"
#include "../../apps/tmss/include/tmSemaphore.h"
#include "../../apps/tmss/include/tmrqRequestQueue.h"
#include "../../apps/tmss/http_proxy/include/thpd.h"



//static const char ModuleName[] = "tmAVEnforcer";
//static unsigned int AVEConfigVersion=9999;

static unsigned int gEnable = 0;   // shall be zero to make sure the share memory is ready in the userland task.
static unsigned int gTimeReport = 86400;     /* interval to show report */
static unsigned int gTimeOutdate = 1800;   /* Timeout for any record */
static unsigned int gTimeUpdate = 300;     /* Timeout for triggering update */
static char gAvSvrURL[MAX_URL_LENGTH] = "/";//"http://www.hackercheck.com/";
//static char gAvSvrURL[MAX_URL_LENGTH] = "http://211.76.136.195/dashboard/dashboard.aspx";
#define LOGERROR(format, args...) do { if (net_ratelimit()) printk(format, ## args); } while (0)
#if 0
#define LOGDEBUG(format, args...) do { if (net_ratelimit()) printk(format, ## args); } while (0)
#define LOGINFO(format, args...) do { if (net_ratelimit()) printk(format, ## args); } while (0)
#else
#define LOGDEBUG(format, args...)
#define LOGINFO(format, args...)
#endif

extern int tmnf_UpdateClientPatternKernel(__u32 nClientIP);
extern int tmnf_UpdateClientInfoKernel(__u32 nClientIP);
//extern int tmexp_IsDstException(uint32_t ip);
extern int tmexp_IsDstException(const char *host);
unsigned int lan_if=LAN_IF_ETH0;

/**
  * @brief return the Health-Check URL
  * @return URL of Health-Check
  */
const char *tmave_GetAVServerURL(void)
{
    return gAvSvrURL;
}

EXPORT_SYMBOL(tmave_GetAVServerURL);

time_t time(register time_t *tloc)
{
	struct timeval tv;
	register struct timeval *p = &tv;

	do_gettimeofday(p);		/* This should never fail... */

	if (tloc) {
		*tloc = p->tv_sec;
	}

	return p->tv_sec;
}


/**
  * @brief decide whether to trigger Discovery to detect the client
  * @param ip IP address of the client
  * @return 1 for triggered, 0 means not triggered
  */

static int tmave_HookTriggerKernel(const struct sk_buff *skb,int *newClient,ClientInfo         *info)
{
  //  ClientInfo         info;
    unsigned char      *mac;//[MAC_SIZE];
    unsigned long      now;
    __u32 ip=skb->nh.iph->saddr;

    if (!gEnable)
        return 0; 

  /*  memset(mac, 0, sizeof(mac));
    if (IP_MAC(&ip, mac) != 0)
    {
        LOGERROR("tmave_HookTrigger(), ARP table lookup fail for client %u.%u.%u.%u\n", NIPQUAD(ip));
        goto END;
    } */
    // get it from
    if((skb->mac.raw >= skb->head)
	    && ((skb->mac.raw + ETH_HLEN) <= skb->data))
    {
    }
    else
    {
        LOGERROR("tmave_HookTriggerKernel(), No valid mac addrees for client %u.%u.%u.%u\n", NIPQUAD(ip));
        goto END;
    }
    
 
    *newClient=0;
    
    // WE HAVE TO CHECK IF the clientInfo Shared Memory is OK or NOT ......
    if ((tmci_GetClient(skb->mac.ethernet->h_source, info) < 0) ||
	   (CSTATUS(info)== STATUS_READ_FROM_CONFIG) )  // Overwrite the one read from config.
    {
       mac=skb->mac.ethernet->h_source;

	  //  LOGERROR("tmave_HookTriggerKernel(), New machine, %u.%u.%u.%u(%02X:%02X:%02X:%02X:%02X:%02X), try to have HTTP request\n", 
	  //  NIPQUAD(ip),mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
        // Case 1: not client record 
        //SetPendingAV(ip, mac); // We let the tmdpd to set the pending AV in the ClientInfo.
	//update=KERNEL_TO_THPD_CMD_UPDATE_CLIENT_INFO_NEW;
	*newClient=1;
        goto TODO;
    }
//
    now = time(NULL);
    //check if the id.ip is not the same, we need to change it to the new ones.
/*    if(ip!=info->idinfo.ip)
    {
	          info->idinfo.ip=ip;
		  tmci_SetIP(ip,info->idinfo.mac);
    } */
 
    //Case 2: client record is out of date 
    if (now - CTAV(info) > gTimeOutdate)
    {
    //printk("now=%lx, CTAV(info)=%lx, gTimeOutdate=%x\n",now,CTAV(info),gTimeOutdate);
       // LOGERROR("tmave_HookTriggerKernel(), Time to refresh %u.%u.%u.%u again\n", NIPQUAD(ip));
	//We need to modify time here.
	CTAV(info)=now;
        tmci_SetTime(&(info->idinfo), &(info->timeinfo));
        goto TODO;
    } 
    

    // Case 3: client pattern or engine is old 
    if (CSTATUS(info) == STATUS_AVOLD && now - CTUPDATE(info) > gTimeUpdate)
    {
     //printk("now=%lx,CSTATUS(info)=%d, CTUPDATE(info)=%lx, gTimeUpdate=%x\n",now,CSTATUS(info),CTUPDATE(info),gTimeUpdate);
      // LOGERROR("tmave_HookTriggerKernel(), Notify %u.%u.%u.%u to update pattern\n", NIPQUAD(ip));
        CTUPDATE(info) = now;
        // WE let user land task to modify the access time in the sharing memory
	// That if there are others coming very short after this one??? Then, we may send lots of same time to user land
	// prorgam to update it.
	tmci_SetTime(&(info->idinfo), &(info->timeinfo));
        tmnf_UpdateClientPatternKernel(ip);
     }
       
    goto END;

TODO:
    LOGINFO("tmave_HookTriggerKernel(), Trigger tmnf_UpdateClientInfo(%u.%u.%u.%u)\n", NIPQUAD(ip));

    if (tmnf_UpdateClientInfoKernel(ip) < 0)
        LOGERROR("tmave_HookTriggerKernel(), Fail to tmnf_UpdateClientInfo()\n");

    return 1;

END:
    ; // do nothing 
    return 0;
}


/**
  * @brief decide the fate of the client
  * @param srcip IP address of the client
  * @param dstip IP address of the target webserver
  * @param url the requested URL or URI
  * @return verdict of the client
  */
//int tmave_HookAction(const struct sk_buff *skb, const char *url,const char *host,ClientInfo         *info)
int tmave_HookAction(const char *url,const char *host,ClientInfo         *info)
{
   // ClientInfo         info;
    //unsigned char      *mac;//[MAC_SIZE];
    unsigned long      now, ltime = 0;
    //uint32_t srcip=skb->nh.iph->saddr;
   //// uint32_t dstip=skb->nh.iph->daddr;
   //LOGDEBUG("Entering tmave_HookAction\n");


    if (!gEnable)
	  return AV_PASS;

  /*  memset(mac, 0, sizeof(mac));
    if (IP_MAC(&srcip, mac) != 0)
    {
        LOGERROR("tmave_HookAction(), ARP table lookup fail for client %u.%u.%u.%u\n", NIPQUAD(srcip));
        goto END;
    } */

    now = time(NULL);

  /*NO NEED TO CHECK AGAIN since have been checked in  tmave_HookTrigger
  if (tmci_GetClient(skb->mac.ethernet->h_source, &info) < 0)
    {
       // Case 1: not client record 
       mac=skb->mac.ethernet->h_source;
        LOGINFO("????tmave_HookAction(), Client %u.%u.%u.%u (%02X:%02X:%02X:%02X:%02X:%02X) is not found in Client Table.\n", NIPQUAD(srcip), mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
       // SetPendingAV(srcip, mac); //it shall be done already in tmave_HookTrigger
        goto END;
    } */

    if ((now - CTCREATE(info) <= DO_NOT_REDIR_TIME) || CTCREATE(info) == 0)
    {
        LOGDEBUG("tmave_HookAction(), Client %u.%u.%u.%u startup in %d seconds, do not redirect. now=%lu create_time=%lu\n", NIPQUAD(srcip), DO_NOT_REDIR_TIME, now, CTCREATE(info));
        goto END;
    }
    else
    {
        LOGDEBUG("tmave_HookAction(), Client %u.%u.%u.%u startup over %d seconds, able to do redirect. now=%lu create_time=%lu\n", NIPQUAD(srcip), DO_NOT_REDIR_TIME, now, CTCREATE(info));
    }  

    if (CSTATUS(info) == STATUS_NOAVUNIX)
    {
        LOGINFO("tmave_HookAction(), Client %u.%u.%u.%u is Non-Windows.\n", NIPQUAD(srcip));
         goto END;
    }

    //if (CISEXPREPORT(info) || tmexp_IsDstException(dstip))
    if (CISEXPREPORT(info) || tmexp_IsDstException(host))
    {
         LOGINFO("tmave_HookAction(), Client %u.%u.%u.%u is exception.\n", NIPQUAD(srcip));
         goto END;
    }
 /*   else if (!IsHtmlURL(url))
    {
         LOGDEBUG("tmave_HookAction(), URL %s is not HTML.\n", url);
         goto END;
    } */

    // Case 2 
    if (CTREDIRECTFLAG(info) || CTREDIRECTRETRY(info) >= REDIR_RETRY_MAX )  //initial false
    {
        ltime = gTimeReport;
    }
    else 
    {  
        ltime = REDIRECT_INTERVAL_SHORT;
    }
    
    if (now - CTREDIRECT(info) > ltime)
    {
        LOGDEBUG("tmave_HookAction(), Time for a redirect (%s). now=%lu, last redirect=%lu ltime=%lu retry=%d\n", url, now, CTREDIRECT(info), ltime, CTREDIRECTRETRY(info));
        goto TODO;
    }
    else
    {
        LOGDEBUG("tmave_HookAction(), NOT Time for a redirect (%s). now=%lu, last redirect=%lu ltime=%lu retry=%d\n", url, now, CTREDIRECT(info), ltime, CTREDIRECTRETRY(info));
        goto END;
    }

TODO:
    LOGINFO("tmave_HookAction(), Redirect %u.%u.%u.%u for [%s]\n", NIPQUAD(srcip), url);
    CTREDIRECTRETRY(info)++;
    if (ltime == gTimeReport)
    {
        CTREDIRECTFLAG(info) = 0;
        CTREDIRECTRETRY(info) = 0;
    }
    CTREDIRECT(info) = now;
    tmci_SetTime(&(info->idinfo), &(info->timeinfo)); //NO NEED TO Check if it is in ClientInfo tables since we have tested it before.
   LOGDEBUG("tmave_HookAction(), Leaving tmave_HookAction redirect time=%lu flag=%d retry=%d\n",CTREDIRECT(info),CTREDIRECTFLAG(info),CTREDIRECTRETRY(info)); 
    return AV_REDIRECT;

END:
   // LOGDEBUG("tmave_HookAction(), Leaving tmave_HookAction\n");
    return AV_PASS;
}

int tmave_HookActionKernel(const struct sk_buff *skb, const char *url, const char *host,int *newClient)
{
	ClientInfo         info;
	tmave_HookTriggerKernel(skb,newClient,&info);//(skb->nh.iph->saddr);, it is done in Applicatin task, httpd.
    
    	if(*newClient==1)// A new client will not need to perform AVE redirect
		return 0;

	//if(tmave_HookAction(skb,url,host,&info)==AV_REDIRECT)
	if(tmave_HookAction(url,host,&info)==AV_REDIRECT)
		 return 1;
	 return 0;
}
void setTMSS_AVE_config(AVEConf *pAVEConf)
{
LOGINFO("setTMSS_AVE_config: YES we got the new AVEconf\n");
    gEnable=pAVEConf->gEnable;
    gTimeReport=pAVEConf->gTimeReport;
    gTimeOutdate=pAVEConf->gTimeOutdate;
    gTimeUpdate=pAVEConf->gTimeUpdate;
    strcpy(gAvSvrURL,pAVEConf->gAvSvrURL);
    lan_if=pAVEConf->lan_if;
LOGINFO("loadAVEConfFromCache lan_if=%d\n",lan_if);
    LOGINFO("loadAVEConfFromCache: gAvSvrURL=%s\n",gAvSvrURL);
 }

