/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file  tmException.c
 *  @brief TMOG AVEnforcer Module - Exception List
 */

/*
 *  Change Log
 *  2004-02-18 Eric Li Initialization version
 */
#define LOGERROR(format, args...) do { if (net_ratelimit()) printk(format, ## args); } while (0)
#if 0
#define LOGDEBUG(format, args...) do { if (net_ratelimit()) printk(format, ## args); } while (0)
#define LOGINFO(format, args...) do { if (net_ratelimit()) printk(format, ## args); } while (0)
#else
#define LOGDEBUG(format, args...)
#define LOGINFO(format, args...)
#endif


#include <linux/module.h>
#include <linux/skbuff.h>
#include <linux/shm.h>
#include <net/sock.h>

#if 0
#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter_ipv4/ipt_webstr.h>
#include "../../apps/tmss/include/tmauIniParser.h"
#include "../../apps/tmss/include/tmClientInfo.h"
#include "../../apps/tmss/include/tmAVEnforcer.h"
#include "../../apps/tmss/include/tmURLCache.h"
#include "../../apps/tmss/include/tmSemaphore.h"
#include "../../apps/tmss/include/tmrqRequestQueue.h"
#include "../../apps/tmss/include/tmException.h"

#define MAX_IP_LIST             100
#define MAX_PATH                256
#define MAX_LINE                 80

#define EXCEPTION_SIZE           1
#define EXCEPTION_DST            0


static const char ModuleName[] = "exception";

typedef struct 
{
    uint32_t enabled;
    uint32_t size;
    uint32_t addr[MAX_IP_LIST];
    char     path[MAX_PATH];
} ExceptionList;

static ExceptionList gList[EXCEPTION_SIZE] = 
{
    { 0, 0, {0},"avedst.dat"},
};


static int bisearch(uint32_t data[], int length, uint32_t target)
{
    int high = length, low = -1, probe;

    while (high - low > 1)
    {
        probe = (high + low) / 2;
        if (data[probe] < target)
            low = probe;
        else
            high = probe;
    }
    if (high == length || data[high] != target)
        return -1;
    else
        return high;
}

int loadExceptionInLocal(void *user, unsigned int len)
{
    int idx = EXCEPTION_DST;
    if (copy_from_user(&gList[idx], user, sizeof(ExceptionList)) != 0)
    {
	gList[idx].enabled=0;
	LOGERROR("loadExceptionInLocal:data error \n"); 
	return -EFAULT;
    }
    if (len != sizeof(ExceptionList))
    {
	gList[idx].enabled=0;
	LOGERROR("loadExceptionInLocal:data len(%x)!=sizeof(ExceptionList)(%x)\n",len,sizeof(ExceptionList)); 
	return -EINVAL;
    }
    return 0;
}

/**
  * @brief determine if the IP is in destination exception list
  * @param ip IP address to be determined
  * @return 0 for no, 1 for yes
  */
int tmexp_IsDstException(uint32_t ip)
{
    int idx = EXCEPTION_DST;

    if (gList[idx].enabled)
    {
        return (bisearch(gList[idx].addr, gList[idx].size, ip) >= 0);
    }

    return 0;
}

#else
#define url_avserver "beta.tmss.trendmicro.com"
#define au_server_url "beta.activeupdate.trendmicro.com"
#define url_avserver_production "tmss.trendmicro.com"
#define au_server_url_production "tmss-p.activeupdate.trendmicro.com"
#define ave_except1 "tmss-sp.trendmicro.com"
#define ave_except2 "tmss.url.trendmicro.com"
#define ave_except3 "tmssreg.trendmicro.com"
#define ave_except4 "drh.digitalriver.com"
#define ave_except5 "ccc.trendmicro.com"
#define ave_except6 "www.trendmicro.com"

#define NUM_OF_EXCEPTIONAL 10
char *exceptionHostLst[NUM_OF_EXCEPTIONAL]={url_avserver,au_server_url,url_avserver_production,au_server_url_production,ave_except1,
ave_except2,ave_except3,ave_except4,ave_except5,ave_except6};


/**
  * @bchcek if host is in exceptional list
  * @param ip IP address to be determined
  * @return 0 for no, 1 for yes
  */
int tmexp_IsDstException(const char *host)
{
    int i;
	for(i=0;i<NUM_OF_EXCEPTIONAL;i++)
	{
		    if(strnicmp(host, exceptionHostLst[i], strlen(exceptionHostLst[i])) == 0) return 1;
	}
    return 0;
}
#endif
