/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file tmGetHardwareInfo.c
 *  @brief TMSS Get Hardware Information
 */

/*
 *  Change Log
 *  2004-05-19 Henry Lin    Initialization version
 */

#include <linux/types.h>
#include <linux/string.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/config.h>
#include <linux/socket.h>
#include <linux/sockios.h>
#include <linux/errno.h>
#include <linux/in.h>
#include <linux/mm.h>
#include <linux/inet.h>
#include <linux/netdevice.h>
#include <linux/netfilter_ipv4/ipt_webstr.h>
#include "../../apps/tmss/include/tmMd5.h"
#include "../../apps/tmss/include/tmGetHardwareInfo.h"

//static const char *ModuleName = "tmGetHardwareInfo";

/**
  * @brief Gets MAC address from system.
  *
  * @see 
  * @return 
  * @remarks 
  */
  
/* assume eth1 is the LAN port, need hardware vendor to customerize this, and get the MAC addrss of WAN port */
//#define DEFAULT_IF  "vlan2" 
//#define LOCAL_IF  "vlan1" 
// Using LAN since WLAN interface can not be determined???
#define DEFAULT_IF  "eth0" 
#define LOCAL_IF  "eth0" 
#define LOGERROR(format, args...) do { if (net_ratelimit()) printk(format, ## args); } while (0)
#if 0
#define LOGDEBUG(format, args...) do { if (net_ratelimit()) printk(format, ## args); } while (0)
#define LOGINFO(format, args...) do { if (net_ratelimit()) printk(format, ## args); } while (0)
#else
#define LOGDEBUG(format, args...)
#define LOGINFO(format, args...)
#endif
extern unsigned int lan_if;



char *LAN_IF[]={"eth0","br0"};

int GetMACAddress(char *mac, size_t size) 
{
    /*
     * In embedded systems, there might be some other way to get MAC address
     * from system. Here, we abstract the implemtation for cross-platform
     * compatibility.
     */
    int retval;
   // int sock;
    struct ifreq ifr;
    
    //LOGINFO("Entering GetMACAddress");

    memset(&ifr, 0, sizeof(struct ifreq));
    //strcpy(ifr.ifr_name, DEFAULT_IF);
    strcpy(ifr.ifr_name, LAN_IF[lan_if]);
    retval = dev_ioctl(SIOCGIFHWADDR, &ifr);
    if(retval < 0) 
    {
        /* cannot invoke ioctl succefully */
        LOGERROR("cannot invoke ioctl succefully \n");
        return -1;
    }
    
    /* mac contains terminating character, so size should be 13 */
    snprintf(mac, size, "%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X",
            (unsigned char)ifr.ifr_hwaddr.sa_data[0],
            (unsigned char)ifr.ifr_hwaddr.sa_data[1],
            (unsigned char)ifr.ifr_hwaddr.sa_data[2],
            (unsigned char)ifr.ifr_hwaddr.sa_data[3],
            (unsigned char)ifr.ifr_hwaddr.sa_data[4],
            (unsigned char)ifr.ifr_hwaddr.sa_data[5]);
         
   LOGINFO("GetMACAddress:mac is %s\n", mac);
             
    return 0;
}

int tmGetBoxID(char *szBoxId)
{   
    char szMac[13] = "";
    char szTempBoxId[17] = "";
    
    //LOGINFO("Entering tmGetBoxID");
    
    /* get mac address, HW vendor needs to port this function */
    GetMACAddress(szMac, 13);
    
    /* make checksum based on mac address */
    MakeCheckSum(szMac, szTempBoxId);
    
    strncpy(szBoxId, szTempBoxId, TMSS_MAX_BUF_BOX_ID - 1);    
    
    szBoxId[TMSS_MAX_BUF_BOX_ID - 1] = '\0';
        
    return 0;
}

int tmGetVendorID(char *szVendorId)
{
    strncpy(szVendorId, TMDP_Vendor_ID, TMSS_MAX_BUF_VENDER_ID - 1);
    
    return 0;
}

extern int devinet_ioctl(unsigned int cmd, void *arg);

int tmGetBoxIp(char *szBoxIp)
{
//    strncpy(szBoxIp, TM_BOX_CONFIG_URL, TMSS_MAX_BUF_BOX_IP - 1);
    int retval;
   // int sock;
    struct ifreq ifr;
    struct sockaddr_in *sin;
    
    //LOGINFO("Entering GetMACAddress");

    memset(&ifr, 0, sizeof(struct ifreq));
    
//    strcpy(ifr.ifr_name, LOCAL_IF);
     strcpy(ifr.ifr_name, LAN_IF[lan_if]);
    retval = devinet_ioctl(SIOCGIFADDR, &ifr);
    if(retval < 0) 
    {
        /* cannot invoke ioctl succefully */
        LOGERROR("tmGetBoxIp:cannot invoke ioctl succefully \n");
        return -1;
    }
    sin = (struct sockaddr_in *)&ifr.ifr_addr;
    /* mac contains terminating character, so size should be 13 */
    sprintf(szBoxIp, "%u.%u.%u.%u",NIPQUAD(sin->sin_addr.s_addr));
 
         
   LOGINFO("GetMACAddress:mac is %s\n", szBoxIp);
 
    return 0;
}

int tmGetBoxVersion(char *szBoxVersion)
{
    strncpy(szBoxVersion, TM_BOX_FIRMWARE_VERSION, TMSS_MAX_BUF_BOX_VERSION - 1);

    return 0;
}

int tmGetBoxBuild(char *szBoxBuild)
{
    strncpy(szBoxBuild, TM_BOX_FIRMWARE_BUILD, TMSS_MAX_BUF_BOX_BUILD - 1);

    return 0;
}

