/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file tmURLCache.c
 *  @brief TMOG URL Filtering Cache Module
 */

/*
 *  Change Log
 *  2004-02-13 Henry Lin    Initialization version
 */

#ifndef TM_NO_URL_FILTERING_CACHE
#define LOGERROR(format, args...) printk(format, ## args)
#if 0
#define LOGDEBUG(format, args...) printk(format, ## args)
#define LOGINFO(format, args...) printk(format, ## args)
#else
#define LOGDEBUG(format, args...)
#define LOGINFO(format, args...)
#endif


#include <linux/module.h>
#include <linux/skbuff.h>
#include <linux/shm.h>
#include <net/sock.h>

#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter_ipv4/ipt_webstr.h>
#include "../../apps/tmss/include/tmauIniParser.h"
#include "../../apps/tmss/include/tmClientInfo.h"
#include "../../apps/tmss/include/tmURLCache.h"
#include "../../apps/tmss/include/tmSemaphore.h"
#include "../../apps/tmss/include/tmURLFilterConf.h"
#include "../../apps/tmss/include/tmURLFilterStatistic.h"
#include "../../apps/tmss/include/tmURLFilter.h"
#define TM_URL_FILTER_MAX_CAT_NUMBER 100

// NOTE NOTE NOTE, for cachesize and cache expire time, we do not use the configured one. We use the deafult one here.
#define nUrlFilterCacheSize   500
static  int nUrlFilterCacheExpireTime=3600;

extern time_t time(register time_t *tloc);

static const char ModuleName[] = "tmURLCache";
url_cache_t  the_url_cache_t;
requested_url_t p_requested_url[nUrlFilterCacheSize];


/* Get the category of the url - www.yahoo.com => y => url_cache[24] */
static int tmGetUrlCacheHashIndex(const char* pszURL)
{
    char *pIndexChar = NULL;
    
    if (pszURL == NULL)
    {
        return -1;
    }
    
    if (strlen(pszURL) > 5)
    {
        if ((pszURL[0] == 'w' || pszURL[0] == 'W') &&
            (pszURL[1] == 'w' || pszURL[1] == 'W') &&
            (pszURL[2] == 'w' || pszURL[2] == 'W') &&
            (pszURL[3] == '.'))
        {
            pIndexChar = (char*)pszURL + 4;
        }
        else
        {
            pIndexChar = (char*)pszURL;
        }
    }
    else
    {
        pIndexChar = (char*)pszURL;
    }
    

    
    if ((*pIndexChar) >= 'A' && (*pIndexChar) <= 'Z')
    {
        LOGDEBUG("URL Categorize as %c\n", (*pIndexChar));
        return ((*pIndexChar) - 'A');
    }
    else if ((*pIndexChar) >= 'a' && (*pIndexChar) <= 'z')
    {
        LOGDEBUG("URL Categorize as %c\n", (*pIndexChar));
        return ((*pIndexChar) - 'a');
    }
    else
    {
        LOGDEBUG("URL Categorize as Others\n");
        return (CACHE_HASH_TABLE_SIZE - 1);
    }
}


/* Remove One node from Lru Mru Link List */
/*
static int tmUrlCacheRemoveOneNodeFromLruMruList(url_cache_t* p_url_cache, requested_url_t *pNode)
{
    requested_url_t *pTemp = NULL;
    
    pTemp = p_url_cache->cache_lru;
    while(pTemp != NULL)
    {
        //pTemp = tmOffset2Addr(p_url_cache, pTemp); // use offset 
        if (pTemp == pNode)
        {
            LOGDEBUG("Found the free node in lru mru list\n");
            if (pTemp->mru != NULL)
            {
                pTemp->mru->lru = pTemp->lru;
               // (tmOffset2Addr(p_url_cache, pTemp->mru))->lru = pTemp->lru; // use offset 
            }
            else
            {
                // first node from cache_lru 
                p_url_cache->cache_lru = pTemp->lru;
            }
            
            if (pTemp->lru != NULL)
            {
                pTemp->lru->mru = pTemp->mru;
                //(tmOffset2Addr(p_url_cache, pTemp->lru))->mru = pTemp->mru;
            }
            else
            {
                // first node from cache_mru 
                p_url_cache->cache_mru = pTemp->mru;
            }
            pTemp->lru = NULL;
            pTemp->mru = NULL;
            break;
        }
        else
        {
            pTemp = pTemp->lru;
        }
    }
    return 0;
}*/


/* Remove One node from Hash Table Link List */
static int tmUrlCacheRemoveOneNodeFromHashTableList(url_cache_t* p_url_cache, requested_url_t *pNode)
{
    requested_url_t **ppHash1 = NULL;
    requested_url_t *pHash2 = NULL;
    
    ppHash1 = &((p_url_cache->hash_table)[tmGetUrlCacheHashIndex(pNode->req_url)]);
    pHash2 = (*ppHash1);
    while(pHash2 != NULL)
    {
        //pHash2 = tmOffset2Addr(p_url_cache, pHash2); /* use offset */
        if (pHash2 == pNode)
        {
            (*ppHash1) = pHash2->next;
            LOGDEBUG("Found the free node in hash table\n");
            break;;
        }
        else
        {
            ppHash1 = &(pHash2->next);
            pHash2 = pHash2->next;
        }
    }
    
    return 0;
}
 

/* Add One node to Lru Mru Link List at Mru side */
/*
static int tmUrlCacheAddOneNodeToMruList(url_cache_t* p_url_cache, requested_url_t *pNode)
{
    if (p_url_cache->cache_mru == NULL)
    {
        p_url_cache->cache_lru = pNode;
        p_url_cache->cache_mru = pNode;
        //p_url_cache->cache_lru = tmAddr2Offset(p_url_cache, pNode); //use offset 
        //p_url_cache->cache_mru = tmAddr2Offset(p_url_cache, pNode); //use offset 
        pNode->lru = NULL;
        pNode->mru = NULL;
    }
    else
    {
        pNode->lru = NULL;
        pNode->mru = p_url_cache->cache_mru;
        p_url_cache->cache_mru->lru = pNode;
        p_url_cache->cache_mru = pNode;
        //(tmOffset2Addr(p_url_cache,p_url_cache->cache_mru))->lru = tmAddr2Offset(p_url_cache, pNode); //use offset 
        //p_url_cache->cache_mru = tmAddr2Offset(p_url_cache, pNode); //use offset 
    }    
    return 0;
}
*/

/* Add One node to Hash Table Link List */
static int tmUrlCacheAddOneNodeToHashTableList(url_cache_t* p_url_cache, requested_url_t *pNode)
{
    int nUrlCacheHashIndex;
    requested_url_t *pTemp = NULL;
    requested_url_t **ppTemp = NULL;
    
    /* Get the category of the url - www.yahoo.com => y => url_cache[24] */
    nUrlCacheHashIndex = tmGetUrlCacheHashIndex(pNode->req_url);
    
    /* add to the category link list */
    ppTemp = &((p_url_cache->hash_table)[nUrlCacheHashIndex]);
    pTemp = (*ppTemp);
    while(pTemp != NULL)
    {
        //pTemp = tmOffset2Addr(p_url_cache, pTemp); /* use offset */
        ppTemp = &(pTemp->next);
        pTemp = pTemp->next;
    }
    (*ppTemp) = pNode;
    //(*ppTemp) = tmAddr2Offset(p_url_cache, pNode); /* use offset */
    pNode->next = NULL;
    
    return 0;
}


/* Add One node to Free Link List */
static void tmUrlCacheAddFreeNode(url_cache_t* p_url_cache, requested_url_t *pNode)
{
    pNode->next = p_url_cache->free_node;
    p_url_cache->free_node = pNode;
    //p_url_cache->free_node = tmAddr2Offset(p_url_cache, pNode); /* use offset */
}


/* Remove One node from Free Link List */
static requested_url_t* tmUrlCacheRemoveFreeNode(url_cache_t* p_url_cache)
{
    requested_url_t *pFree = NULL;
    
    pFree = p_url_cache->free_node;
    //pFree = tmOffset2Addr(p_url_cache ,p_url_cache->free_node); /* use offset */
    p_url_cache->free_node = pFree->next;
    
    return pFree;
}


/* Remove One node from URL Cache */
static int tmUrlCacheRemoveOneSelectedNode(url_cache_t* p_url_cache, requested_url_t *pNode)
{
    if (pNode == NULL)
    {
	    LOGERROR("tmUrlCacheRemoveOneSelectedNode:pNode == NULL\n");
        return -1;
    }
  
    /* remove this node from cache_lru and cache_mru link list */
    //tmUrlCacheRemoveOneNodeFromLruMruList(p_url_cache, pNode);

    /* remove one node from hash table */
    tmUrlCacheRemoveOneNodeFromHashTableList(p_url_cache, pNode);

    /* add this node to free link list */
    tmUrlCacheAddFreeNode(p_url_cache, pNode);

    return 0;
}


/* Get a new node from free link list, if no free node, get one from lru link list */
static requested_url_t* tmUrlCacheGetNewNode(url_cache_t* p_url_cache)
{
    requested_url_t *pFree = NULL;
    
    if (p_url_cache->free_node == NULL) /* no free node available, get one from lru link list */
    {
        ////tmUrlCacheRemoveOneSelectedNode(p_url_cache, p_url_cache->cache_lru);
		//Aki, if no free node, we reset it 
		tmInitUrlFilterCache();
	};

    /* remove one free node */
    pFree = tmUrlCacheRemoveFreeNode(p_url_cache);

    return pFree;
}


/**
  * @brief concatenate the host name with url and remove the optional arguments
  * @param pszHost input - host name
  * @param nHostLen input - host name length
  * @param pszUrl input - url
  * @param nUrlLen input - url length
  * @param szReqUrlWithoutArg output - url
  * @param nUrlBufLen input - output url buffer length
  * @return URL_OK concat successful
  * @return URL_TOO_LONG url too long
  */
int RemoveOptionalArgInURL(
    const char *pszHost,
    int nHostLen,
    const char *pszUrl,
    int nUrlLen,
    char *szReqUrlWithoutArg,
    int nUrlBufLen)
{
    int nBufTempIndex = 0;
    int i;

    LOGDEBUG("Entering RemoveOptionalArgInURL()\n");

    if (nHostLen > (nUrlBufLen - 1))
    {
        LOGDEBUG("Host name too long, host name = %s, host length = %d\n", pszHost, nHostLen);
        szReqUrlWithoutArg[0] = '\0';
        return URL_TOO_LONG;
    }    

     for (i = 0; i < nHostLen; i++)
    {
        szReqUrlWithoutArg[nBufTempIndex++] = pszHost[i];
    }

    for (i = 0; i < nUrlLen; i++)
    {
        if (pszUrl[i] == '?')
        {
            break;
        }
        szReqUrlWithoutArg[nBufTempIndex++] = pszUrl[i];
        if (nBufTempIndex >= nUrlBufLen)
        {
            LOGDEBUG("URL too long, url = %s , url length = %d\n", pszUrl, nUrlLen);
            szReqUrlWithoutArg[0] = '\0';
            return URL_TOO_LONG;
        }
    }
    
    szReqUrlWithoutArg[nBufTempIndex] = '\0';
    
    return URL_OK;
}

/**
  * @brief Initialize the cache memory space and initialize link lists
  * @param nUrlFilterCacheSize input - URL Filtering Cache Size (from config file)
  * @return 0 ok
  * @return -1 fail
  */
  
int tmInitUrlFilterCache(void)//int nUrlFilterCacheSize)
{
    //int nUrlFilterCacheShmid;
    url_cache_t *p_url_cache = &the_url_cache_t;
   // requested_url_t *p_requested_url = NULL;
    int i;
    //struct tm_sem_st UrlFilterCacheLock;
    int nRetVal = 0;
    
     LOGDEBUG("calling tmInitUrlFilterCache\n");
    
    for (i = 0; i < CACHE_HASH_TABLE_SIZE; i++)
    {
        p_url_cache->hash_table[i] = NULL;
    }
    
    p_url_cache->cache_lru = NULL;
    p_url_cache->cache_mru = NULL;
    
    //p_url_cache->free_node = (requested_url_t*)((unsigned long)p_url_cache + sizeof(url_cache_t));
    p_url_cache->free_node = &p_requested_url[0];

 
    for (i = 0; i < nUrlFilterCacheSize - 1; i++)
    {
        p_requested_url[i].lru = NULL;
        p_requested_url[i].mru = NULL;
        p_requested_url[i].next = p_requested_url + i + 1;
     }
    p_requested_url[nUrlFilterCacheSize - 1].lru = NULL;
    p_requested_url[nUrlFilterCacheSize - 1].mru = NULL;
    p_requested_url[nUrlFilterCacheSize - 1].next = NULL;
    
    return nRetVal;
}


/**
  * @brief add one URL in the cache
  * @param nUrlFilterCacheSize input - URL Filtering Cache Size (from config file)
  * @param szURL input - URL to be cached
  * @param nUrlFilterCacheExpireTime input - URL Expire time in seconds (from config file)
  * @return 0 ok
  * @return -1 fail
  */
 int tmUrlCacheAddNode(add_url_cache *auc)
{
    //int nUrlFilterCacheShmid;
     url_cache_t *p_url_cache = &the_url_cache_t;
   requested_url_t *p_new_node = NULL;
    time_t now;
   // int ret;
   // struct tm_sem_st UrlFilterCacheLock;
    int nRetVal = 0;
    
    LOGDEBUG("Entering tmUrlCacheAddNode(%d,%s)\n", nUrlFilterCacheSize, auc->szReqUrlWithoutArg);
    
    /* allocate memory for new node and fill with information of url */
    p_new_node = tmUrlCacheGetNewNode(p_url_cache);
    p_new_node->req_url[MAX_URL_BUF - 1] = '\0';
    strncpy(p_new_node->req_url, auc->szReqUrlWithoutArg, MAX_URL_BUF - 1);
    
    time(&now);
    
    p_new_node->expire_time = now + nUrlFilterCacheExpireTime;
    p_new_node->action = auc->nRateRetVal;
    p_new_node->rsp_blkc=auc->rsp_blkc;
    
    LOGDEBUG("Add cache node data: URL: %s ,action=%d, current time %lu, expire setting %d, expire %lu\n", 
	p_new_node->req_url,p_new_node->action,now, nUrlFilterCacheExpireTime,p_new_node->expire_time);
    
    /* add to the cache_lru cache_mru link list at MRU side */
    //tmUrlCacheAddOneNodeToMruList(p_url_cache, p_new_node);

    /* add to the category link list */
    tmUrlCacheAddOneNodeToHashTableList(p_url_cache, p_new_node);
    
    return nRetVal;
}


/**
  * @brief search one URL in the cache
  * @param nUrlFilterCacheSize input - URL Filtering Cache Size (from config file)
  * @param nUrlFilterCacheExpireTime input - URL Expire time in seconds (from config file)
  * @return URL_IN_CACHE in cache
  * @return URL_NOT_INCACHE not in cache
  * @return URL_IN_CACHE_BUT_EXPIRED expired
  */
int tmQueryUrlCache(/*int nUrlFilterCacheSize,*/ const char* szURL,unsigned long *rsp_blkc)//, int nUrlFilterCacheExpireTime)
{
    int nUrlCacheHashIndex;
    requested_url_t *pTemp = NULL;
    requested_url_t **ppTemp = NULL;
    time_t now;
     url_cache_t *p_url_cache = &the_url_cache_t;
    int nRetVal = URL_NOT_INCACHE;
    int action = HTTP_MSG_ACTION_NOT_DETERMINED;
    
    *rsp_blkc=0;// default to NO RATE
    LOGDEBUG("Entering tmQueryUrlCache(%d,%s)\n", nUrlFilterCacheSize, szURL);
    
    /* Get the category of the url - www.yahoo.com => y => url_cache[24] */
    nUrlCacheHashIndex = tmGetUrlCacheHashIndex(szURL);
    LOGDEBUG("\ntmQueryUrlCache:Category URL %s as index %d\n", szURL, nUrlCacheHashIndex);
    /* search the link list start from the catogory found above */
    ppTemp = &((p_url_cache->hash_table)[nUrlCacheHashIndex]);
    pTemp = (*ppTemp);
    while(pTemp != NULL)
    {
        //pTemp = tmOffset2Addr(p_url_cache, pTemp); /* use offset */
	LOGDEBUG("compare %s(%d) with %s\n",pTemp->req_url,pTemp->action,szURL);
        if (strcmp(pTemp->req_url,szURL) == 0)
        {
            /* check expire */
	    // Note, we do not modify the cache.   
	    // So, if it is expired, then we will send a new url req up to userland program to query the surveilance server.
	    // Then, who have the responbility to remove the expired one from Cache.
	    //.  Each responses from the surveilance server in the userland program shall remove the expired ones.
	    //  We will not move the referenced one form LRU to MRU. So, we provide a FIFO with timestamping.
            time(&now);
            if (now > pTemp->expire_time)
            {
                /* remove this node */
 		LOGDEBUG("Found URL %s in Cache but expired\n",szURL);
                tmUrlCacheRemoveOneSelectedNode(p_url_cache, pTemp);
                nRetVal = URL_IN_CACHE_BUT_EXPIRED;
 		LOGDEBUG("Found URL %s in Cache but expired\n",szURL);
               goto end;
            }
            else
            {
                /* move this node to first node from cache_mru link list */
               // tmUrlCacheRemoveOneNodeFromLruMruList(p_url_cache, pTemp);
               // tmUrlCacheAddOneNodeToMruList(p_url_cache, pTemp);
                
                /* Update expire time */
                pTemp->expire_time = now + nUrlFilterCacheExpireTime;
                nRetVal = URL_IN_CACHE;
		action=pTemp->action;
		*rsp_blkc=pTemp->rsp_blkc;
		LOGDEBUG("Found URL %s in Cache and action=%d and rsp_blkc=%d\n",szURL,action,(int)*rsp_blkc);

                goto end;
            }
        }
        ppTemp = &(pTemp->next);
        pTemp = pTemp->next;
    }

    
end:  
    
    if (nRetVal ==  URL_IN_CACHE)
    {
        LOGDEBUG("Found URL %s in Cache\n",szURL);
    }
    else if (nRetVal ==  URL_IN_CACHE_BUT_EXPIRED)
    {
        LOGDEBUG("Found URL %s in Cache but expired\n", szURL);
    }
    else
    {
        LOGDEBUG("Didn't find ULR %s in Cache\n", szURL);
    }
    return action;
 }



#endif //#ifndef TM_NO_URL_FILTERING_CACHE
