/******************************************************************************
 *
 *  (C) Copyright 2004 Trend Micro, Inc.
 *
 *  All Rights Reserved.
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Trend Micro, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Trend Micro, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 *****************************************************************************/

/**
 *  @file tmURLFilterConf.c
 *  @brief TMOG URL Filtering Configuration Loading Module
 */

/*
 *  Change Log
 *  2004-03-03 Henry Lin    Initialization version
 */
//#define LOGERROR       printk
#define LOGERROR(format, args...) do { if (net_ratelimit()) printk(format, ## args); } while (0)
#if 0
//#define LOGDEBUG       printk
//#define LOGINFO       printk
#define LOGDEBUG(format, args...) do { if (net_ratelimit()) printk(format, ## args); } while (0)
#define LOGINFO(format, args...) do { if (net_ratelimit()) printk(format, ## args); } while (0)
#else
#define LOGDEBUG(format, args...)
#define LOGINFO(format, args...)
#endif

#include <linux/module.h>
#include <linux/skbuff.h>
#include <linux/shm.h>
#include <net/sock.h>
#include <linux/time.h>

#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter_ipv4/ipt_webstr.h>
#include "../../apps/tmss/include/tmauIniParser.h"
#include "../../apps/tmss/include/tmClientInfo.h"
#include "../../apps/tmss/include/tmURLCache.h"
#include "../../apps/tmss/include/tmSemaphore.h"
#include "../../apps/tmss/include/tmURLFilterConf.h"


static const char ModuleName[] = "tmURLFilterConf";
//static unsigned int urlFilterConfigVersion=9999;
url_filter_config_t UrlFilterConfig;
static unsigned char *g_ignore_mime[] = {
                                            ".gif", ".jpg", ".jpe", ".tif", ".png", ".bmp", ".crl", ".css", ""
                                        };
extern int IsInSchedule(url_filter_config_t *pUrlFilterConfig);
extern time_t time(register time_t *tloc);

struct tm
{
	int tm_sec;                   /* Seconds.     [0-60] (1 leap second) */
	int tm_min;                   /* Minutes.     [0-59] */
	int tm_hour;                  /* Hours.       [0-23] */
	int tm_mday;                  /* Day.         [1-31] */
	int tm_mon;                   /* Month.       [0-11] */
	int tm_year;                  /* Year - 1900.  */
	int tm_wday;                  /* Day of week. [0-6] */
	int tm_yday;                  /* Days in year.[0-365] */
	int tm_isdst;                 /* DST.         [-1/0/1]*/

	long int tm_gmtoff;           /* we don't care, we count from GMT */
	const char *tm_zone;          /* we don't care, we count from GMT */
};
void localtime(const time_t *timepr, struct tm *r);


int
tm_url_filter_tmufsaci_check_mimetype(unsigned char *ap_url, int a_url_len)
{
    unsigned char *tp_suffix; /* pointer to suffix string */
    int t_suffix_len; /* suffix string length */
    int t_ii; /* loop control */
    t_suffix_len = 4; /* set suffix length */


    /* if URL too short we rate */
    if(a_url_len <= t_suffix_len)
        /* suffix too short = return OK to rate */
        return 0;

    /* find the end of the directory in the URL */
    for(t_ii = 0, tp_suffix = ap_url; t_ii < a_url_len; t_ii++, tp_suffix++)
    {
        /* if we find a '?' then quit */
        if((*tp_suffix == (unsigned char)NULL) || (*tp_suffix == '?'))
            break;
    }

    /* back up to what should be the start of the suffix */
    tp_suffix -= t_suffix_len;
    /* if no dot found, return OK to rate it */
    if(*tp_suffix != '.')
        /* no dot = return OK to rate */
        return 0;

    /* run through the list of mime types to ignore */
    for(t_ii = 0; *(g_ignore_mime+t_ii) != (unsigned char)NULL; t_ii++)
    {
        /* check that we have found the match we want */
        if(strncmp(tp_suffix, *(g_ignore_mime+t_ii), t_suffix_len) == 0)
            /* mime type is present so we don't rate it */
            return 1;
    }

    /* item is not present so we do rate it */
    return 0;
}

int IsInScheduleKernel(url_filter_config_t *pUrlFilterConfig)
{
    time_t now;
   struct tm currenttime;                          /* time human readable */
    struct tm *now_local=&currenttime;
  unsigned short now_min;
  unsigned short now_hour;
    unsigned short now_day;
    
    time(&now);
    localtime(&now, now_local);
   
    now_min = now_local->tm_min;
    now_hour = now_local->tm_hour;
    now_day = now_local->tm_wday;
    
    /* Looks like 05/31/2004 16:32:26 Day:1 */
    LOGDEBUG("Schedule Blocking: now is %02d/%02d/%04d %2d:%02d:%02d Day:%d\n", now_local->tm_mon+1, now_local->tm_mday, now_local->tm_year + 1900, now_local->tm_hour, now_local->tm_min, now_local->tm_sec, now_local->tm_wday);
    if(pUrlFilterConfig->alwaysBlock)
	    return 1;
    
    if (pUrlFilterConfig->nScheduleDay[now_day] == 0)
    {
        LOGDEBUG("Schedule Blocking: day %d not enabled\n", now_day);
        return 0;
    }
    
    if ((pUrlFilterConfig->nScheduleFromHour > now_hour)||(pUrlFilterConfig->nScheduleToHour < now_hour)) 
    {
	    return 0;
    }
    if(
	((pUrlFilterConfig->nScheduleFromHour == now_hour) &&(pUrlFilterConfig->nScheduleFromMin > now_min)) ||
	((pUrlFilterConfig->nScheduleToHour == now_hour) &&(pUrlFilterConfig->nScheduleToMin < now_min)))
	{
		return 0;
	}
        return 1;
}


int urlFilterBasicChecking(unsigned char      *mac,unsigned char *req_url)
{
    unsigned char           *p_req_url;
    int                     nUrlLen;
    
   /* if(CopyUrlConfToLocalKernel(&UrlFilterConfig)!=0)
    {// UrlConf, not yet ready, simply pass it.
        return HTTP_MSG_ACTION_PASS;
    } */
    if (!UrlFilterConfig.nUrlLicenseEnable)
    {
        LOGDEBUG("URL Filter Function disabled by License Control\n");
        return HTTP_MSG_ACTION_PASS;
    }

    if (!UrlFilterConfig.nUrlFilterEnable)
    {
        LOGDEBUG("URL Filter Function disabled by user config\n");
        return HTTP_MSG_ACTION_PASS;
    }
    
    if (UrlFilterConfig.nUrlFilterExceptionEnable)
    {
       LOGDEBUG("URL Filter Exception List enabled\n");
        if (tmci_IsExpFilter(mac))
        {
            LOGDEBUG("URL Filter Function disabled by source IP Exception list , mac %s",  mac);
            return HTTP_MSG_ACTION_PASS;
        }
    }
    else
    {
        LOGDEBUG("URL Filter Exception List disabled\n");
    }
    
    if (!IsInScheduleKernel(&UrlFilterConfig))
    {
        LOGDEBUG("Schedule Blodking: URL Filter Function disabled by Schedule\n");
        return HTTP_MSG_ACTION_PASS;
    }
    else
    {
        LOGDEBUG("Schedule Blodking: URL Filter Function not disabled by Schedule\n");
    }

	
    p_req_url = req_url;
    if (strncmp(req_url,"http://",7) == 0)
    {
        p_req_url += 7;
        while(*p_req_url != '\0')
        {
            if (*p_req_url == '/')
            {
                break;
            }
            p_req_url++;
        }
    }
    if (*p_req_url == '\0')
    {
        LOGERROR("Input URL Error: %s\n",req_url);
    }

   nUrlLen = strlen(p_req_url);
 /* If the URL is a file ".gif", ".jpg", ".jpe", ".tif", ".png", ".bmp", ".crl", ".css" 
       Allow this URL without rating */
    if(tm_url_filter_tmufsaci_check_mimetype(p_req_url, nUrlLen)) /* access approved - no nead to rate */
    {
        LOGDEBUG("mimetype, return allow without rating\n");
        return (HTTP_MSG_ACTION_PASS);
    }
    return HTTP_MSG_ACTION_NOT_DETERMINED;

}



