/*
 ***************************************************************************
 * Ralink Tech Inc.
 * 4F, No. 2 Technology 5th Rd.
 * Science-based Industrial Park
 * Hsin-chu, Taiwan, R.O.C.
 *
 * (c) Copyright 2002-2005, Ralink Technology, Inc.
 *
 * All rights reserved. Ralink's source code is an unpublished work and the
 * use of a copyright notice does not imply otherwise. This source code
 * contains confidential trade secret material of Ralink Tech. Any attempt
 * or participation in deciphering, decoding, reverse engineering or in any
 * way altering the source code is stricitly prohibited, unless the prior
 * written consent of Ralink Technology, Inc. is obtained.
 ***************************************************************************

    Module Name:
    rtmp_info.c

    Abstract:
    IOCTL related subroutines

    Revision History:
    Who         When          What
    --------    ----------    ----------------------------------------------
    Rory Chen   01-03-2003    created
	Rory Chen   02-14-2005    modify to support RT61
*/

#include	"rt_config.h"

struct iw_priv_args privtab[] = {
{ RTPRIV_IOCTL_SET, 
  IW_PRIV_TYPE_CHAR | 1024, 0,
  "set"},  
{ RTPRIV_IOCTL_GSITESURVEY,
  IW_PRIV_TYPE_CHAR | 1024, IW_PRIV_TYPE_CHAR | 1024 ,
  "get_site_survey"}, 
{ RTPRIV_IOCTL_GET_MAC_TABLE,
  IW_PRIV_TYPE_CHAR | 1024, IW_PRIV_TYPE_CHAR | 1024 ,
  "get_mac_table"}, 
#ifdef DBG
{ RTPRIV_IOCTL_BBP,
  IW_PRIV_TYPE_CHAR | 1024, IW_PRIV_TYPE_CHAR | 1024,
  "bbp"},
{ RTPRIV_IOCTL_MAC,
  IW_PRIV_TYPE_CHAR | 1024, IW_PRIV_TYPE_CHAR | 1024,
  "mac"},
{ RTPRIV_IOCTL_E2P,
  IW_PRIV_TYPE_CHAR | 1024, IW_PRIV_TYPE_CHAR | 1024,
  "e2p"},
#endif
{ RTPRIV_IOCTL_STATISTICS,
  IW_PRIV_TYPE_CHAR | 1024, IW_PRIV_TYPE_CHAR | 1024,
  "stat"}
};

static struct {
	CHAR *name;
	INT (*set_proc)(PRTMP_ADAPTER pAdapter, PUCHAR arg);
} *PRTMP_PRIVATE_SET_PROC, RTMP_PRIVATE_SUPPORT_PROC[] = {
	{"DriverVersion",				Set_DriverVersion_Proc},
	{"CountryRegion",				Set_CountryRegion_Proc},
	{"CountryRegionABand",			Set_CountryRegionABand_Proc},
	{"CountryCode",					Set_CountryCode_Proc},
	{"SSID",						Set_SSID_Proc},
	{"WirelessMode",				Set_WirelessMode_Proc},
	{"TxRate",						Set_TxRate_Proc},
	{"BasicRate",					Set_BasicRate_Proc},
	{"Channel",						Set_Channel_Proc},
	{"BeaconPeriod",				Set_BeaconPeriod_Proc},
	{"DtimPeriod",					Set_DtimPeriod_Proc},
	{"TxPower",						Set_TxPower_Proc},
	{"BGProtection",				Set_BGProtection_Proc},
	{"DisableOLBC", 				Set_OLBCDetection_Proc},
	{"TxAntenna",					Set_TxAntenna_Proc},
	{"RxAntenna",					Set_RxAntenna_Proc},
	{"TxPreamble",					Set_TxPreamble_Proc},
	{"RTSThreshold",				Set_RTSThreshold_Proc},
	{"FragThreshold",				Set_FragThreshold_Proc},
	{"TxBurst",						Set_TxBurst_Proc},
	{"PktAggregate",				Set_PktAggregate_Proc},
	{"TurboRate",					Set_TurboRate_Proc},
	{"NoForwardingBTNBSSID",		Set_NoForwardingBTNSSID_Proc},
	{"WmmCapable",					Set_WmmCapable_Proc},
	{"NoForwarding",				Set_NoForwarding_Proc},
	{"HideSSID",					Set_HideSSID_Proc},
	{"ShortSlot",					Set_ShortSlot_Proc},
	{"IEEE8021X",					Set_IEEE8021X_Proc},
	{"IEEE80211H",					Set_IEEE80211H_Proc},
	{"CSPeriod",					Set_CSPeriod_Proc},
	{"PreAuth",						Set_PreAuth_Proc},
	{"VLANID",						Set_VLANID_Proc},
	{"VLANPriority",				Set_VLANPriority_Proc},
	{"AuthMode",					Set_AuthMode_Proc},
	{"EncrypType",					Set_EncrypType_Proc},
	{"DefaultKeyID",				Set_DefaultKeyID_Proc},
	{"Key1",						Set_Key1_Proc},
	{"Key2",						Set_Key2_Proc},
	{"Key3",						Set_Key3_Proc},
	{"Key4",						Set_Key4_Proc},
	{"HSCounter",					Set_HSCounter_Proc},
	{"AccessPolicy",				Set_AccessPolicy_Proc},
	{"AccessControlList",			Set_AccessControlList_Proc},
	{"WPAPSK",						Set_WPAPSK_Proc},
	{"ResetCounter",				Set_ResetStatCounter_Proc},
	{"RekeyMethod",					Set_RekeyMethod_Proc},
	{"RekeyInterval",				Set_RekeyInterval_Proc},
	{"PMKCachePeriod",				Set_PMKCachePeriod_Proc},
	{"SiteSurvey",					Set_SiteSurvey_Proc},
// Sam add
	{"AddWdsList",					Set_AddWdsList_Proc},
	{"DelWdsList",					Set_DelWdsList_Proc},
	{"WdsEncrypType",				Set_WdsEncrypType_Proc},
	{"WdsKey",					Set_WdsKey_Proc},
	{"WdsEnable",					Set_WdsEnable_Proc},
	{"APEnable",					Set_APEnable_Proc},
// End Sam add
#ifdef DBG	
	{"Debug",						Set_Debug_Proc},
#endif
#ifdef RALINK_ATE
	{"ATE",							Set_ATE_Proc},
	{"ATEDA",						Set_ATE_DA_Proc},
	{"ATESA",						Set_ATE_SA_Proc},
	{"ATEBSSID",					Set_ATE_BSSID_Proc},
	{"ATECHANNEL",					Set_ATE_CHANNEL_Proc},
	{"ATETXPOW",					Set_ATE_TX_POWER_Proc},
	{"ATETXFREQOFFSET",				Set_ATE_TX_FREQOFFSET_Proc},
	{"ATETXLEN",					Set_ATE_TX_LENGTH_Proc},
	{"ATETXCNT",					Set_ATE_TX_COUNT_Proc},
	{"ATETXRATE",					Set_ATE_TX_RATE_Proc},
#endif	// RALINK_ATE
	{NULL,}
};

INT RTMPSetInformation(
	IN	PRTMP_ADAPTER pAdapter,
	IN	OUT	struct ifreq	*rq,
	IN	INT					cmd)
{
	INT	Status = NDIS_STATUS_SUCCESS;
	
	return Status;
}

INT RTMPQueryInformation(
	IN	PRTMP_ADAPTER       pAdapter,
	IN	OUT	struct ifreq    *rq,
	IN	INT                 cmd)
{
    INT	Status = NDIS_STATUS_SUCCESS;

	return Status;
}

INT RT61_ioctl(
	IN	struct net_device	*net_dev, 
	IN	OUT	struct ifreq	*rq, 
	IN	INT					cmd)
{
    VIRTUAL_ADAPTER                     *pVirtualAd;
	RTMP_ADAPTER                        *pAd = net_dev->priv;
    struct iwreq                        *wrq = (struct iwreq *) rq;
    INT                                 Status = NDIS_STATUS_SUCCESS;
    USHORT                              subcmd, index;

    // determine this ioctl command is comming from which interface.
    if (net_dev->priv_flags == INT_MAIN)
    {
    	pAd= net_dev->priv;
        pAd->IoctlIF = MAIN_MBSSID;
        DBGPRINT(RT_DEBUG_INFO, "RT61_ioctl I/F(ra%d)(flags=%d): cmd = 0x%08x\n", pAd->IoctlIF, net_dev->priv_flags, cmd);
    }
    else if (net_dev->priv_flags == INT_MBSSID)
    {
    	pVirtualAd = net_dev->priv;
        pAd = pVirtualAd->RtmpDev->priv;

    	if (!RTMPEqualMemory(net_dev->name, pAd->net_dev->name, 3))  // for multi-physical card, no MBSSID
    	{
	        for (index = 1; index < pAd->PortCfg.BssidNum; index++)
	    	{
	    	    if (pAd->PortCfg.MBSSID[index].MSSIDDev == net_dev)
	    	    {
	    	        pAd->IoctlIF = index;
	    	        
	    	        DBGPRINT(RT_DEBUG_INFO, "RT61_ioctl I/F(ra%d)(flags=%d): cmd = 0x%08x\n", index, net_dev->priv_flags, cmd);
	    	        break;
	    	    }
	    	}
	        // Interface not found!
	        if(index == pAd->PortCfg.BssidNum)
	        {
	        	DBGPRINT(RT_DEBUG_ERROR, "RT61_ioctl can not find I/F\n");
	            return -ENETDOWN;
	        }
	    }
	    else    // ioctl command from I/F(ra0)
	    {
	    	pAd= net_dev->priv;
    	    pAd->IoctlIF = MAIN_MBSSID;
	        DBGPRINT(RT_DEBUG_ERROR, "RT61_ioctl can not find I/F and use default: cmd = 0x%08x\n", cmd);
	    }
    }
    else
    {
    	DBGPRINT(RT_DEBUG_WARN, "IOCTL is not supported in WDS interface\n");
    	return -EOPNOTSUPP;
    }

	switch(cmd)
	{
        case SIOCGIFHWADDR:
			DBGPRINT(RT_DEBUG_TRACE, "IOCTLIOCTLIOCTL::SIOCGIFHWADDR\n");
			strcpy(wrq->u.name, pAd->PortCfg.MBSSID[pAd->IoctlIF].Bssid);
			break;
		case SIOCGIWNAME:
			DBGPRINT(RT_DEBUG_TRACE, "IOCTL::SIOCGIWNAME\n");
			strcpy(wrq->u.name, "RT61 SoftAP");
			break;
		case SIOCSIWESSID:  //Set ESSID
			Status = -EOPNOTSUPP;
			break;
		case SIOCGIWESSID:  //Get ESSID
			{
				struct iw_point *erq = &wrq->u.essid;

				erq->flags=1;
				erq->length = pAd->PortCfg.MBSSID[pAd->IoctlIF].SsidLen;

				if(erq->pointer)
				{
					if(copy_to_user(erq->pointer, pAd->PortCfg.MBSSID[pAd->IoctlIF].Ssid, erq->length))
					{
						Status = -EFAULT;
						break;
					}
				}
				DBGPRINT(RT_DEBUG_TRACE, "IOCTL::SIOCGIWESSID (Len=%d, ssid=%s...)\n", erq->length, erq->pointer);
			}
			break;
		case SIOCGIWNWID: // get network id 
		case SIOCSIWNWID: // set network id (the cell)
			Status = -EOPNOTSUPP;
			break;
		case SIOCGIWFREQ: // get channel/frequency (Hz)
			wrq->u.freq.m = pAd->PortCfg.Channel;
			wrq->u.freq.e = 0;
			wrq->u.freq.i = 0;
			break; 
		case SIOCSIWFREQ: //set channel/frequency (Hz)
		case SIOCGIWNICKN:
		case SIOCSIWNICKN: //set node name/nickname
		case SIOCGIWRATE:  //get default bit rate (bps)
			wrq->u.bitrate.value = RateIdTo500Kbps[pAd->PortCfg.MBSSID[pAd->IoctlIF].TxRate] * 500000;
			wrq->u.bitrate.disabled = 0;
			break;
		case SIOCSIWRATE:  //set default bit rate (bps)
		case SIOCGIWRTS:  // get RTS/CTS threshold (bytes)
		case SIOCSIWRTS:  //set RTS/CTS threshold (bytes)
		case SIOCGIWFRAG:  //get fragmentation thr (bytes)
		case SIOCSIWFRAG:  //set fragmentation thr (bytes)
		case SIOCGIWENCODE:  //get encoding token & mode
		case SIOCSIWENCODE:  //set encoding token & mode
			Status = -EOPNOTSUPP;
			break;
		case SIOCGIWAP:  //get access point MAC addresses
			wrq->u.ap_addr.sa_family = ARPHRD_ETHER;
			memcpy(wrq->u.ap_addr.sa_data, &pAd->PortCfg.MBSSID[pAd->IoctlIF].Bssid, ETH_ALEN);
			DBGPRINT(RT_DEBUG_TRACE, "IOCTL::SIOCGIWAP(=%02x:%02x:%02x:%02x:%02x:%02x)\n",
				pAd->PortCfg.MBSSID[pAd->IoctlIF].Bssid[0],pAd->PortCfg.MBSSID[pAd->IoctlIF].Bssid[1],pAd->PortCfg.MBSSID[pAd->IoctlIF].Bssid[2],
				pAd->PortCfg.MBSSID[pAd->IoctlIF].Bssid[3],pAd->PortCfg.MBSSID[pAd->IoctlIF].Bssid[4],pAd->PortCfg.MBSSID[pAd->IoctlIF].Bssid[5]);
			break;
		case SIOCGIWMODE:  //get operation mode
			wrq->u.mode = IW_MODE_INFRA;   //SoftAP always on INFRA mode.
			break;
		case SIOCSIWAP:  //set access point MAC addresses
		case SIOCSIWMODE:  //set operation mode
		case SIOCGIWSENS:   //get sensitivity (dBm)
		case SIOCSIWSENS:	//set sensitivity (dBm)
		case SIOCGIWPOWER:  //get Power Management settings
		case SIOCSIWPOWER:  //set Power Management settings
		case SIOCGIWTXPOW:  //get transmit power (dBm)
		case SIOCSIWTXPOW:  //set transmit power (dBm)
		case SIOCGIWRANGE:	//Get range of parameters
		case SIOCGIWRETRY:	//get retry limits and lifetime
		case SIOCSIWRETRY:	//set retry limits and lifetime
			Status = -EOPNOTSUPP;
			break;
		case RT_PRIV_IOCTL:
			subcmd = wrq->u.data.flags;
			if( subcmd & OID_GET_SET_TOGGLE)
				Status = RTMPSetInformation(pAd, rq, subcmd);
			else
				Status = RTMPQueryInformation(pAd, rq, subcmd);
			break;
		case SIOCGIWPRIV:
			if (wrq->u.data.pointer) {
				if ( verify_area(VERIFY_WRITE, wrq->u.data.pointer, sizeof(privtab)) )
					break;
				wrq->u.data.length = sizeof(privtab) / sizeof(privtab[0]);
				if (copy_to_user(wrq->u.data.pointer, privtab, sizeof(privtab)))
					Status = -EFAULT;
			}
			break;
		case RTPRIV_IOCTL_SET:
			{				
				CHAR *this_char;
				CHAR *value;

				if( verify_area(VERIFY_READ, wrq->u.data.pointer, wrq->u.data.length) )
					break;

				while ((this_char = strsep(&wrq->u.data.pointer, ",")) != NULL) 
				{
					if (!*this_char)
						 continue;

					if ((value = strchr(this_char, '=')) != NULL)
						*value++ = 0;

					if (!value)
						continue;  							

					for (PRTMP_PRIVATE_SET_PROC = RTMP_PRIVATE_SUPPORT_PROC; PRTMP_PRIVATE_SET_PROC->name; PRTMP_PRIVATE_SET_PROC++)
					{
						if (!strcmp(this_char, PRTMP_PRIVATE_SET_PROC->name)) 
						{						
							if(!PRTMP_PRIVATE_SET_PROC->set_proc(pAd, value))
							{   //FALSE:Set private failed then return Invalid argument 								
								Status = -EINVAL;							
							}
							break;  //Exit for loop.
						}
					}

					if(PRTMP_PRIVATE_SET_PROC->name == NULL)
					{  //Not found argument
						Status = -EINVAL;
						DBGPRINT(RT_DEBUG_TRACE, "IOCTL::(iwpriv) Command not Support [%s=%s]\n", this_char, value);
						break;
					}	
				}
			}
			break;
		    
		case RTPRIV_IOCTL_GET_MAC_TABLE:
			//printk("Sam_A RTPRIV_IOCTL_GET_MAC_TABLE\n");
			RTMPIoctlGetMacTable(pAd,wrq);
		    break;
		    
		case RTPRIV_IOCTL_GSITESURVEY:
			RTMPIoctlGetSiteSurvey(pAd,wrq);
		    break;
		
		case RTPRIV_IOCTL_STATISTICS:
		    RTMPIoctlStatistics(pAd, wrq);
		    break;

		case RTPRIV_IOCTL_RADIUS_DATA:
		    RTMPIoctlRadiusData(pAd, wrq);
		    break;

		case RTPRIV_IOCTL_ADD_WPA_KEY:
		    RTMPIoctlAddWPAKey(pAd, wrq);
		    break;

		case RTPRIV_IOCTL_ADD_PMKID_CACHE:
		    RTMPIoctlAddPMKIDCache(pAd, wrq);
		    break;

#ifdef DBG
		case RTPRIV_IOCTL_BBP:
			RTMPIoctlBBP(pAd, wrq);
			break;
			
		case RTPRIV_IOCTL_MAC:
			RTMPIoctlMAC(pAd, wrq);
			break;

		case RTPRIV_IOCTL_E2P:
			RTMPIoctlE2PROM(pAd, wrq);
			break;
#endif

		default:
			DBGPRINT(RT_DEBUG_ERROR, "IOCTL::unknown IOCTL's cmd = 0x%08x\n", cmd);
			Status = -EOPNOTSUPP;
			break;
	}

	return Status;
}

/*
	========================================================================
	
	Routine Description:
		Change NIC PHY mode. Re-association may be necessary.

	Arguments:
		pAdapter						Pointer to our adapter
        phmode
		
	========================================================================
*/
VOID	RTMPSetPhyMode(
	IN	PRTMP_ADAPTER	pAd,
	IN  ULONG phymode,
	IN  UCHAR apidx)
{
	DBGPRINT(RT_DEBUG_TRACE,"IF(ra%d) RTMPSetPhyMode(=%d)\n", apidx, phymode);
    
	NdisZeroMemory(pAd->PortCfg.SupportedRates, MAX_LEN_OF_SUPPORTED_RATES);
	NdisZeroMemory(pAd->PortCfg.MBSSID[apidx].DesiredRates, MAX_LEN_OF_SUPPORTED_RATES);
	
    switch (phymode) {
    	case PHY_11A:
            pAd->PortCfg.SupportedRates[0]  = 0x8C;    // 6 mbps, in units of 0.5 Mbps, basic rate
            pAd->PortCfg.SupportedRates[1]  = 0x12;    // 9 mbps, in units of 0.5 Mbps
            pAd->PortCfg.SupportedRates[2]  = 0x98;    // 12 mbps, in units of 0.5 Mbps, basic rate
            pAd->PortCfg.SupportedRates[3]  = 0x24;    // 18 mbps, in units of 0.5 Mbps
            pAd->PortCfg.SupportedRates[4]  = 0xb0;    // 24 mbps, in units of 0.5 Mbps, basic rate
            pAd->PortCfg.SupportedRates[5]  = 0x48;    // 36 mbps, in units of 0.5 Mbps
            pAd->PortCfg.SupportedRates[6]  = 0x60;    // 48 mbps, in units of 0.5 Mbps
            pAd->PortCfg.SupportedRates[7]  = 0x6c;    // 54 mbps, in units of 0.5 Mbps
            pAd->PortCfg.SupportedRatesLen  = 8;
            pAd->PortCfg.MBSSID[apidx].DesiredRates[0]  = 12;    // 6 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[1]  = 18;    // 9 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[2]  = 24;    // 12 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[3]  = 36;    // 18 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[4]  = 48;    // 24 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[5]  = 72;    // 36 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[6]  = 96;    // 48 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[7]  = 108;   // 54 mbps, in units of 0.5 Mbps
            break;
        case PHY_11B:
            pAd->PortCfg.SupportedRates[0]  = 0x82;    // 1 mbps, in units of 0.5 Mbps, basic rate
            pAd->PortCfg.SupportedRates[1]  = 0x84;    // 2 mbps, in units of 0.5 Mbps, basic rate
            pAd->PortCfg.SupportedRates[2]  = 0x0B;    // 5.5 mbps, in units of 0.5 Mbps
            pAd->PortCfg.SupportedRates[3]  = 0x16;    // 11 mbps, in units of 0.5 Mbps
            pAd->PortCfg.SupportedRatesLen  = 4;
            pAd->PortCfg.MBSSID[apidx].DesiredRates[0]  = 2;     // 1 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[1]  = 4;     // 2 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[2]  = 11;    // 5.5 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[3]  = 22;    // 11 mbps, in units of 0.5 Mbps
            break;
        case PHY_11BG_MIXED:
            // 2004-1-12 to keep maximum compatibility with 802.11b-only clients, we should not turn on
            // BASIC RATE bit of any OFDM rates in outgoing BEACON and ProbrResponse frames
            pAd->PortCfg.SupportedRates[0]  = 0x82;    // 1 mbps, in units of 0.5 Mbps, basic rate
            pAd->PortCfg.SupportedRates[1]  = 0x84;    // 2 mbps, in units of 0.5 Mbps, basic rate
            pAd->PortCfg.SupportedRates[2]  = 0x8B;    // 5.5 mbps, in units of 0.5 Mbps, basic rate
            pAd->PortCfg.SupportedRates[3]  = 0x96;    // 11 mbps, in units of 0.5 Mbps, basic rate
            pAd->PortCfg.SupportedRates[4]  = 0x0C;    // 6 mbps, in units of 0.5 Mbps
            pAd->PortCfg.SupportedRates[5]  = 0x12;    // 9 mbps, in units of 0.5 Mbps
            pAd->PortCfg.SupportedRates[6]  = 0x18;    // 12 mbps, in units of 0.5 Mbps
            pAd->PortCfg.SupportedRates[7]  = 0x24;    // 18 mbps, in units of 0.5 Mbps
            pAd->PortCfg.SupportedRates[8]  = 0x30;    // 24 mbps, in units of 0.5 Mbps
            pAd->PortCfg.SupportedRates[9]  = 0x48;    // 36 mbps, in units of 0.5 Mbps
            pAd->PortCfg.SupportedRates[10] = 0x60;    // 48 mbps, in units of 0.5 Mbps
            pAd->PortCfg.SupportedRates[11] = 0x6C;    // 54 mbps, in units of 0.5 Mbps
            pAd->PortCfg.SupportedRatesLen  = 12;
            pAd->PortCfg.MBSSID[apidx].DesiredRates[0]  = 2;     // 1 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[1]  = 4;     // 2 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[2]  = 11;    // 5.5 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[3]  = 22;    // 11 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[4]  = 12;    // 6 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[5]  = 18;    // 9 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[6]  = 24;    // 12 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[7]  = 36;    // 18 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[8]  = 48;    // 24 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[9]  = 72;    // 36 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[10] = 96;    // 48 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[11] = 108;   // 54 mbps, in units of 0.5 Mbps
            break;
        case PHY_11G:
            // 2004-1-12 to keep maximum compatibility with 802.11b-only clients, we should not turn on
            // BASIC RATE bit of any OFDM rates in outgoing BEACON and ProbrResponse frames
            pAd->PortCfg.SupportedRates[0]  = 0x82;    // 1 mbps, in units of 0.5 Mbps, basic rate
            pAd->PortCfg.SupportedRates[1]  = 0x84;    // 2 mbps, in units of 0.5 Mbps, basic rate
            pAd->PortCfg.SupportedRates[2]  = 0x8B;    // 5.5 mbps, in units of 0.5 Mbps, basic rate
            pAd->PortCfg.SupportedRates[3]  = 0x96;    // 11 mbps, in units of 0.5 Mbps, basic rate
            pAd->PortCfg.SupportedRates[4]  = 0x8C;    // 6 mbps, in units of 0.5 Mbps, basic rate
            pAd->PortCfg.SupportedRates[5]  = 0x12;    // 9 mbps, in units of 0.5 Mbps
            pAd->PortCfg.SupportedRates[6]  = 0x98;    // 12 mbps, in units of 0.5 Mbps, basic rate
            pAd->PortCfg.SupportedRates[7]  = 0x24;    // 18 mbps, in units of 0.5 Mbps
            pAd->PortCfg.SupportedRates[8]  = 0xB0;    // 24 mbps, in units of 0.5 Mbps, basic rate
            pAd->PortCfg.SupportedRates[9]  = 0x48;    // 36 mbps, in units of 0.5 Mbps
            pAd->PortCfg.SupportedRates[10] = 0x60;    // 48 mbps, in units of 0.5 Mbps
            pAd->PortCfg.SupportedRates[11] = 0x6C;    // 54 mbps, in units of 0.5 Mbps
            pAd->PortCfg.SupportedRatesLen  = 12;
            pAd->PortCfg.MBSSID[apidx].DesiredRates[0]  = 2;     // 1 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[1]  = 4;     // 2 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[2]  = 11;    // 5.5 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[3]  = 22;    // 11 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[4]  = 12;    // 6 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[5]  = 18;    // 9 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[6]  = 24;    // 12 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[7]  = 36;    // 18 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[8]  = 48;    // 24 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[9]  = 72;    // 36 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[10] = 96;    // 48 mbps, in units of 0.5 Mbps
            pAd->PortCfg.MBSSID[apidx].DesiredRates[11] = 108;   // 54 mbps, in units of 0.5 Mbps
            break;
        default:
            break;
    }
}

INT Set_DriverVersion_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
    DBGPRINT(RT_DEBUG_TRACE, "Driver version-%s\n", DRIVER_VERSION);
    
    return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set Country Region.
        This command will not work, if the field of CountryRegion in eeprom is programmed.
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT Set_CountryRegion_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	ULONG region;
		
	region = simple_strtol(arg, 0, 10);

	// Country can be set only when EEPROM not programmed
	if (pAd->PortCfg.CountryRegion & 0x80)
	{
		DBGPRINT(RT_DEBUG_ERROR, "Set_CountryRegion_Proc::parameter of CountryRegion in eeprom is programmed \n");
		return FALSE;
	}
	
	if(region >= 0)
	{
		pAd->PortCfg.CountryRegion = (UCHAR) region;
	}
	else
	{
		DBGPRINT(RT_DEBUG_ERROR, "Set_CountryRegion_Proc::parameters out of range\n");
		return FALSE;
	}

	// if set country region, driver needs to be reset
	BuildChannelList(pAd);

	DBGPRINT(RT_DEBUG_TRACE, "Set_CountryRegion_Proc::(CountryRegion=%d)\n", pAd->PortCfg.CountryRegion);

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set Country Region for A band.
        This command will not work, if the field of CountryRegion in eeprom is programmed.
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT Set_CountryRegionABand_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	ULONG region;
		
	region = simple_strtol(arg, 0, 10);

	// Country can be set only when EEPROM not programmed
	if (pAd->PortCfg.CountryRegionForABand & 0x80)
	{
		DBGPRINT(RT_DEBUG_ERROR, "Set_CountryRegionABand_Proc::parameter of CountryRegion in eeprom is programmed \n");
		return FALSE;
	}
	
	if(region >= 0)
	{
		pAd->PortCfg.CountryRegionForABand = (UCHAR) region;
	}
	else
	{
		DBGPRINT(RT_DEBUG_ERROR, "Set_CountryRegionABand_Proc::parameters out of range\n");
		return FALSE;
	}

	// if set country region, driver needs to be reset
	BuildChannelList(pAd);

	DBGPRINT(RT_DEBUG_TRACE, "Set_CountryRegionABand_Proc::(CountryRegion=%d)\n", pAd->PortCfg.CountryRegionForABand);

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set Country Code.
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT Set_CountryCode_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{

	if(strlen(arg) == 2)
	{
		NdisMoveMemory(pAd->PortCfg.CountryCode, arg, 2);
		pAd->PortCfg.CountryCode[2] = ' ';
		pAd->PortCfg.bCountryFlag = TRUE;
	}
	else
	{
		NdisZeroMemory(pAd->PortCfg.CountryCode, 3);
		pAd->PortCfg.bCountryFlag = FALSE;
	}	
		
	DBGPRINT(RT_DEBUG_TRACE, "Set_CountryCode_Proc::(bCountryFlag=%d, CountryCode=%s)\n", pAd->PortCfg.bCountryFlag, pAd->PortCfg.CountryCode);

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set SSID
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_SSID_Proc(
	IN	PRTMP_ADAPTER	pAdapter, 
	IN	PUCHAR			arg)
{
	INT   success = FALSE;

	if(strlen(arg) <= MAX_LEN_OF_SSID)
	{
		NdisZeroMemory(pAdapter->PortCfg.MBSSID[pAdapter->IoctlIF].Ssid, MAX_LEN_OF_SSID);
		NdisMoveMemory(pAdapter->PortCfg.MBSSID[pAdapter->IoctlIF].Ssid, arg, strlen(arg));
		pAdapter->PortCfg.MBSSID[pAdapter->IoctlIF].SsidLen = (UCHAR)strlen(arg);
		success = TRUE;

		// If in detection mode, need to stop detect first.
		if (pAdapter->PortCfg.RadarDetect.RDMode == RD_SILENCE_MODE)
		{
			RadarDetectionStop(pAdapter);
		}

    		ApStop(pAdapter);
    		ApStartUp(pAdapter);

		DBGPRINT(RT_DEBUG_TRACE, "I/F(ra%d) Set_SSID_Proc::(Len=%d,Ssid=%s)\n", pAdapter->IoctlIF,
			pAdapter->PortCfg.MBSSID[pAdapter->IoctlIF].SsidLen, pAdapter->PortCfg.MBSSID[pAdapter->IoctlIF].Ssid);
	}
	else
		success = FALSE;

	return success;
}

/* 
    ==========================================================================
    Description:
        Set Wireless Mode
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_WirelessMode_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	ULONG	WirelessMode;
	INT		i, success = TRUE;

	WirelessMode = simple_strtol(arg, 0, 10);

	if(WirelessMode == PHY_11A)
		pAd->PortCfg.PhyMode = PHY_11A;
	else if(WirelessMode == PHY_11B)
		pAd->PortCfg.PhyMode = PHY_11B;
	else if(WirelessMode == PHY_11BG_MIXED)
		pAd->PortCfg.PhyMode = PHY_11BG_MIXED;
	else if(WirelessMode == PHY_11G)
		pAd->PortCfg.PhyMode = PHY_11G;
	else
		success = FALSE;

	BuildChannelList(pAd);

	for(i=0; i<pAd->PortCfg.BssidNum;i++)
	{
		RTMPSetPhyMode(pAd, pAd->PortCfg.PhyMode, i);
		RTMPBuildDesireRate(pAd, i,	pAd->PortCfg.MBSSID[i].DesiredRatesIndex);
	}

	if(success)
	{
		ApStop(pAd);
		ApStartUp(pAd);
		DBGPRINT(RT_DEBUG_TRACE, "Set_WirelessMode_Proc::(=%d)\n", pAd->PortCfg.PhyMode);
	}
	
	return success;
}

/* 
    ==========================================================================
    Description:
        Set TxRate
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT Set_TxRate_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	NdisZeroMemory(pAd->PortCfg.MBSSID[pAd->IoctlIF].DesiredRates, MAX_LEN_OF_SUPPORTED_RATES);

	pAd->PortCfg.MBSSID[pAd->IoctlIF].DesiredRatesIndex = simple_strtol(arg, 0, 10);
	RTMPBuildDesireRate(pAd, pAd->IoctlIF, pAd->PortCfg.MBSSID[pAd->IoctlIF].DesiredRatesIndex);
	
	MlmeUpdateTxRates(pAd);

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set BasicRate
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT Set_BasicRate_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
    pAd->PortCfg.BasicRateBitMap = (ULONG) simple_strtol(arg, 0, 10);

    if (pAd->PortCfg.BasicRateBitMap > 4095) // (2 ^ MAX_LEN_OF_SUPPORTED_RATES) -1
        return FALSE;

    MlmeUpdateTxRates(pAd);

    DBGPRINT(RT_DEBUG_TRACE, "Set_BasicRate_Proc::(BasicRateBitMap=0x%08x)\n", pAd->PortCfg.BasicRateBitMap);
    
    return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set Channel
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_Channel_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	UCHAR channel;	

	channel = (UCHAR) simple_strtol(arg, 0, 10);

	pAd->PortCfg.Channel = channel;
	if (pAd->PortCfg.RadarDetect.IEEE80211H == FALSE)
	{
		ApStop(pAd);
		ApStartUp(pAd);
	}
	else
	{
		if (pAd->PortCfg.RadarDetect.RDMode == RD_SILENCE_MODE)
		{
			RadarDetectionStop(pAd);
			ApStop(pAd);
			ApStartUp(pAd);
		}
		else if (pAd->PortCfg.RadarDetect.RDMode == RD_NORMAL_MODE)
		{
			pAd->PortCfg.RadarDetect.RDMode = RD_SWITCHING_MODE;
			pAd->PortCfg.RadarDetect.CSCount = 0;
		}
		else if (pAd->PortCfg.RadarDetect.RDMode == RD_SWITCHING_MODE)
		{
			//do nothing
		}
	}
	
	DBGPRINT(RT_DEBUG_TRACE, "Set_Channel_Proc::(Channel=%d)\n", pAd->PortCfg.Channel);

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set Beacon Period
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_BeaconPeriod_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	USHORT BeaconPeriod;
	INT   success = FALSE;

	BeaconPeriod = (USHORT) simple_strtol(arg, 0, 10);
	if((BeaconPeriod >= 20) && (BeaconPeriod < 1024))
	{
		pAd->PortCfg.BeaconPeriod = BeaconPeriod;
		success = TRUE;		
	}
	else
		success = FALSE;

	DBGPRINT(RT_DEBUG_TRACE, "Set_BeaconPeriod_Proc::(BeaconPeriod=%d)\n", pAd->PortCfg.BeaconPeriod);

	return success;
}

/* 
    ==========================================================================
    Description:
        Set Dtim Period
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_DtimPeriod_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	USHORT DtimPeriod;
	INT   success = FALSE;

	DtimPeriod = (USHORT) simple_strtol(arg, 0, 10);
	if((DtimPeriod >= 1) && (DtimPeriod <= 255))
	{
		pAd->Mlme.DtimPeriod = DtimPeriod;
		success = TRUE;
	}
	else
		success = FALSE;

	DBGPRINT(RT_DEBUG_TRACE, "Set_DtimPeriod_Proc::(DtimPeriod=%d)\n", pAd->Mlme.DtimPeriod);

	return success;
}

/* 
    ==========================================================================
    Description:
        Set Tx power
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_TxPower_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	ULONG TxPower;
	INT   success = FALSE;

	TxPower = (ULONG) simple_strtol(arg, 0, 10);
	if (TxPower <= 100)
    {
		pAd->PortCfg.TxPowerPercentage = TxPower;
		success = TRUE;
	}
	else
		success = FALSE;

	DBGPRINT(RT_DEBUG_TRACE, "Set_TxPower_Proc::(TxPowerPercentage=%d)\n", pAd->PortCfg.TxPowerPercentage);

	return success;
}

#ifdef DBG
/* 
    ==========================================================================
    Description:
        For Debug information
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_Debug_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	DBGPRINT(RT_DEBUG_TRACE, "==> Set_Debug_Proc *******************\n");

    if(simple_strtol(arg, 0, 10) <= RT_DEBUG_LOUD)
        RTDebugLevel = simple_strtol(arg, 0, 10);

	DBGPRINT(RT_DEBUG_TRACE, "<== Set_Debug_Proc(RTDebugLevel = %d)\n", RTDebugLevel);

	return TRUE;
}
#endif

/* 
    ==========================================================================
    Description:
        Disable/enable OLBC detection manually
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_OLBCDetection_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	switch (simple_strtol(arg, 0, 10))
	{
		case 0: //enable OLBC detect
			pAd->PortCfg.DisableOLBCDetect = 0;
			break;
		case 1: //disable OLBC detect
			pAd->PortCfg.DisableOLBCDetect = 1;
			break;
		default:  //Invalid argument 
			return FALSE;
	}

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set 11B/11G Protection
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_BGProtection_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	switch (simple_strtol(arg, 0, 10))
	{
		case 0: //AUTO
			pAd->PortCfg.UseBGProtection = 0;
			break;
		case 1: //Always On
			pAd->PortCfg.UseBGProtection = 1;
			break;
		case 2: //Always OFF
			pAd->PortCfg.UseBGProtection = 2;
			break;		
		default:  //Invalid argument 
			return FALSE;
	}
	ApUpdateCapabilityAndErpIe(pAd);
	DBGPRINT(RT_DEBUG_TRACE, "Set_BGProtection_Proc::(BGProtection=%d)\n", pAd->PortCfg.UseBGProtection);	

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set TxAntenna
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_TxAntenna_Proc(
	IN	PRTMP_ADAPTER	pAdapter, 
	IN	PUCHAR			arg)
{
	DBGPRINT(RT_DEBUG_TRACE, "Set_TxAntenna_Proc\n");	

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set RxAntenna
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_RxAntenna_Proc(
	IN	PRTMP_ADAPTER	pAdapter, 
	IN	PUCHAR			arg)
{
	DBGPRINT(RT_DEBUG_TRACE, "Set_RxAntenna_Proc\n");

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set TxPreamble
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_TxPreamble_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	switch (simple_strtol(arg, 0, 10))
	{
		case Rt802_11PreambleLong:
			pAd->PortCfg.TxPreamble = Rt802_11PreambleLong;
			break;
		case Rt802_11PreambleShort:
			pAd->PortCfg.TxPreamble = Rt802_11PreambleShort;
			break;
		default: //Invalid argument 
			return FALSE;
	}

	DBGPRINT(RT_DEBUG_TRACE, "Set_TxPreamble_Proc::(TxPreamble=%d)\n", pAd->PortCfg.TxPreamble);

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set RTS Threshold
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_RTSThreshold_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	ULONG	RtsThresh;

	RtsThresh = simple_strtol(arg, 0, 10);

	if((RtsThresh >= 1) && (RtsThresh <= MAX_RTS_THRESHOLD))
		pAd->PortCfg.RtsThreshold  = (USHORT)RtsThresh;
	else
		return FALSE; //Invalid argument 

	DBGPRINT(RT_DEBUG_TRACE, "Set_RTSThreshold_Proc::(RTSThreshold=%d)\n", pAd->PortCfg.RtsThreshold);

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set Fragment Threshold
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_FragThreshold_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	ULONG	FragThresh;

	FragThresh = simple_strtol(arg, 0, 10);

	if ((FragThresh >= MIN_FRAG_THRESHOLD) || (FragThresh <= MAX_FRAG_THRESHOLD))
	{
		pAd->PortCfg.FragmentThreshold = FragThresh;
		pAd->PortCfg.AllowFragSize = (pAd->PortCfg.FragmentThreshold) - LENGTH_802_11 - LENGTH_CRC;
		pAd->PortCfg.bFragmentZeroDisable = TRUE;
	}
	else
		return FALSE; //Invalid argument 

	DBGPRINT(RT_DEBUG_TRACE, "Set_FragThreshold_Proc::(FragThreshold=%d)\n", pAd->PortCfg.FragmentThreshold);

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set TxBurst
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_TxBurst_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	ULONG TxBurst;
	//ULONG temptime;
	TxBurst = simple_strtol(arg, 0, 10);

	if (TxBurst == 1)
	{
		pAd->PortCfg.bEnableTxBurst = TRUE;
		// Sam add debug
		//RTMP_IO_READ32(pAd,MAC_CSR8,&temptime);

		//printk("read MAC_CSR8 %08X\n",temptime);
		//temptime &= 0xfffffffc;
		//RTMP_IO_WRITE32(pAd,MAC_CSR8,temptime); 
		//printk("write MAC_CSR8 %08X\n",temptime);

		//RTMP_IO_READ32(pAd,MAC_CSR9,&temptime);

		//printk("read MAC_CSR9 %08X\n",temptime);
		// End Sam add
	}
	else if (TxBurst == 0)
		pAd->PortCfg.bEnableTxBurst = FALSE;
	else
		return FALSE;  //Invalid argument 
	
	DBGPRINT(RT_DEBUG_TRACE, "Set_TxBurst_Proc::(TxBurst=%d)\n", pAd->PortCfg.bEnableTxBurst);

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set TxBurst
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_PktAggregate_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	ULONG aggre;

	aggre = simple_strtol(arg, 0, 10);

	if (aggre == 1 && (pAd->pPci_Dev->device != NIC2561_PCI_DEVICE_ID))
		pAd->PortCfg.bAggregationCapable = TRUE;
	else if (aggre == 0)
		pAd->PortCfg.bAggregationCapable = FALSE;
	else
		return FALSE;  //Invalid argument 
	
	DBGPRINT(RT_DEBUG_TRACE, "Set_PktAggregate_Proc::(AGGRE=%d)\n", pAd->PortCfg.bAggregationCapable);

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set TurboRate Enable or Disable
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_TurboRate_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	ULONG TurboRate;

	TurboRate = simple_strtol(arg, 0, 10);

	if (TurboRate == 1)
		pAd->PortCfg.EnableTurboRate = TRUE;
	else if (TurboRate == 0)
		pAd->PortCfg.EnableTurboRate = FALSE;
	else
		return FALSE;  //Invalid argument 
	
	DBGPRINT(RT_DEBUG_TRACE, "Set_TurboRate_Proc::(TurboRate=%d)\n", pAd->PortCfg.EnableTurboRate);

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set WmmCapable Enable or Disable
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_WmmCapable_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	BOOLEAN	bWmmCapable;

	bWmmCapable = simple_strtol(arg, 0, 10);

	if (bWmmCapable == 1)
		pAd->PortCfg.MBSSID[pAd->IoctlIF].bWmmCapable = TRUE;
	else if (bWmmCapable == 0)
		pAd->PortCfg.MBSSID[pAd->IoctlIF].bWmmCapable = FALSE;
	else
		return FALSE;  //Invalid argument 
	
	DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_WmmCapable_Proc::(bWmmCapable=%d)\n", 
		pAd->IoctlIF, pAd->PortCfg.MBSSID[pAd->IoctlIF].bWmmCapable);

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set No Forwarding Enable or Disable
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_NoForwarding_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	ULONG NoForwarding;

	NoForwarding = simple_strtol(arg, 0, 10);

	if (NoForwarding == 1)
		pAd->PortCfg.MBSSID[pAd->IoctlIF].IsolateInterStaTraffic = TRUE;
	else if (NoForwarding == 0)
		pAd->PortCfg.MBSSID[pAd->IoctlIF].IsolateInterStaTraffic = FALSE;
	else
		return FALSE;  //Invalid argument 
	
	DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_NoForwarding_Proc::(NoForwarding=%d)\n", 
		pAd->IoctlIF, pAd->PortCfg.MBSSID[pAd->IoctlIF].IsolateInterStaTraffic);

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set No Forwarding between each SSID
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_NoForwardingBTNSSID_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	ULONG NoForwarding;

	NoForwarding = simple_strtol(arg, 0, 10);

	if (NoForwarding == 1)
		pAd->PortCfg.IsolateInterStaTrafficBTNBSSID = TRUE;
	else if (NoForwarding == 0)
		pAd->PortCfg.IsolateInterStaTrafficBTNBSSID = FALSE;
	else
		return FALSE;  //Invalid argument 
	
	DBGPRINT(RT_DEBUG_TRACE, "Set_NoForwardingBTNSSID_Proc::(NoForwarding=%d)\n", pAd->PortCfg.IsolateInterStaTrafficBTNBSSID);

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set Hide SSID Enable or Disable
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_HideSSID_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	BOOLEAN bHideSsid;

	bHideSsid = simple_strtol(arg, 0, 10);

	if (bHideSsid == 1)
		bHideSsid = TRUE;
	else if (bHideSsid == 0)
		bHideSsid = FALSE;
	else
		return FALSE;  //Invalid argument 
	
	if (pAd->PortCfg.MBSSID[pAd->IoctlIF].bHideSsid != bHideSsid)
	{
		pAd->PortCfg.MBSSID[pAd->IoctlIF].bHideSsid = bHideSsid;
	}

	DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_HideSSID_Proc::(HideSSID=%d)\n", pAd->IoctlIF, pAd->PortCfg.MBSSID[pAd->IoctlIF].bHideSsid);

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set Short Slot Time Enable or Disable
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_ShortSlot_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	ULONG ShortSlot;

	ShortSlot = simple_strtol(arg, 0, 10);

	if (ShortSlot == 1)
		pAd->PortCfg.UseShortSlotTime = TRUE;
	else if (ShortSlot == 0)
		pAd->PortCfg.UseShortSlotTime = FALSE;
	else
		return FALSE;  //Invalid argument 
	
	DBGPRINT(RT_DEBUG_TRACE, "Set_ShortSlot_Proc::(ShortSlot=%d)\n", pAd->PortCfg.UseShortSlotTime);

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set IEEE8021X.
        This parameter is 1 when 802.1x-wep turn on, otherwise 0
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_IEEE8021X_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
    ULONG ieee8021x;

	ieee8021x = simple_strtol(arg, 0, 10);

	if (ieee8021x == 1)
		pAd->PortCfg.MBSSID[pAd->IoctlIF].IEEE8021X = TRUE;
	else if (ieee8021x == 0)
		pAd->PortCfg.MBSSID[pAd->IoctlIF].IEEE8021X = FALSE;
	else
		return FALSE;  //Invalid argument 
	
	DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_IEEE8021X_Proc::(IEEE8021X=%d)\n", pAd->IoctlIF, pAd->PortCfg.MBSSID[pAd->IoctlIF].IEEE8021X);

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set IEEE80211H.
        This parameter is 1 when needs radar detection, otherwise 0
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_IEEE80211H_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
    ULONG ieee80211h;

	ieee80211h = simple_strtol(arg, 0, 10);

	if (ieee80211h == 1)
		pAd->PortCfg.RadarDetect.IEEE80211H = TRUE;
	else if (ieee80211h == 0)
		pAd->PortCfg.RadarDetect.IEEE80211H = FALSE;
	else
		return FALSE;  //Invalid argument 
	
	DBGPRINT(RT_DEBUG_TRACE, "Set_IEEE80211H_Proc::(IEEE80211H=%d)\n", pAd->PortCfg.RadarDetect.IEEE80211H);

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set channel switch Period
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_CSPeriod_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	pAd->PortCfg.RadarDetect.CSPeriod = (USHORT) simple_strtol(arg, 0, 10);

	DBGPRINT(RT_DEBUG_TRACE, "Set_CSPeriod_Proc::(CSPeriod=%d)\n", pAd->PortCfg.RadarDetect.CSPeriod);

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set pre-authentication enable or disable when WPA/WPA2 turn on
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_PreAuth_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
    ULONG PreAuth;

	PreAuth = simple_strtol(arg, 0, 10);

	if (PreAuth == 1)
		pAd->PortCfg.MBSSID[pAd->IoctlIF].PreAuth = TRUE;
	else if (PreAuth == 0)
		pAd->PortCfg.MBSSID[pAd->IoctlIF].PreAuth = FALSE;
	else
		return FALSE;  //Invalid argument 
	
	DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_PreAuth_Proc::(PreAuth=%d)\n", pAd->IoctlIF, pAd->PortCfg.MBSSID[pAd->IoctlIF].PreAuth);

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set VLAN's ID field
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_VLANID_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	pAd->PortCfg.MBSSID[pAd->IoctlIF].VLAN_VID = simple_strtol(arg, 0, 10);
	
	DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_VLANID_Proc::(VLAN_VID=%d)\n", pAd->IoctlIF, pAd->PortCfg.MBSSID[pAd->IoctlIF].VLAN_VID);

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set VLAN's priority field
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_VLANPriority_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	pAd->PortCfg.MBSSID[pAd->IoctlIF].VLAN_Priority = simple_strtol(arg, 0, 10);

	DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_VLANPriority_Proc::(VLAN_Priority=%d)\n", pAd->IoctlIF, pAd->PortCfg.MBSSID[pAd->IoctlIF].VLAN_Priority);

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set Authentication mode
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_AuthMode_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	ULONG       sec_csr4 = 0;

	if ((strcmp(arg, "WEPAUTO") == 0) || (strcmp(arg, "wepauto") == 0))
		pAd->PortCfg.MBSSID[pAd->IoctlIF].AuthMode = Ndis802_11AuthModeAutoSwitch;
	else if ((strcmp(arg, "OPEN") == 0) || (strcmp(arg, "open") == 0))
		pAd->PortCfg.MBSSID[pAd->IoctlIF].AuthMode = Ndis802_11AuthModeOpen;
	else if ((strcmp(arg, "SHARED") == 0) || (strcmp(arg, "shared") == 0))
		pAd->PortCfg.MBSSID[pAd->IoctlIF].AuthMode = Ndis802_11AuthModeShared;
	else if ((strcmp(arg, "WPAPSK") == 0) || (strcmp(arg, "wpapsk") == 0))
		pAd->PortCfg.MBSSID[pAd->IoctlIF].AuthMode = Ndis802_11AuthModeWPAPSK;
	else if ((strcmp(arg, "WPA") == 0) || (strcmp(arg, "wpa") == 0))
		pAd->PortCfg.MBSSID[pAd->IoctlIF].AuthMode = Ndis802_11AuthModeWPA;
	else if ((strcmp(arg, "WPA2PSK") == 0) || (strcmp(arg, "wpa2psk") == 0))
		pAd->PortCfg.MBSSID[pAd->IoctlIF].AuthMode = Ndis802_11AuthModeWPA2PSK;
	else if ((strcmp(arg, "WPA2") == 0) || (strcmp(arg, "wpa2") == 0))
		pAd->PortCfg.MBSSID[pAd->IoctlIF].AuthMode = Ndis802_11AuthModeWPA2;
	else if ((strcmp(arg, "WPA1WPA2") == 0) || (strcmp(arg, "wpa1wpa2") == 0))
		pAd->PortCfg.MBSSID[pAd->IoctlIF].AuthMode = Ndis802_11AuthModeWPA1WPA2;
	else if ((strcmp(arg, "WPAPSKWPA2PSK") == 0) || (strcmp(arg, "wpapskwpa2psk") == 0))
		pAd->PortCfg.MBSSID[pAd->IoctlIF].AuthMode = Ndis802_11AuthModeWPAPSKWPA2PSK;
	else
		return FALSE;  

	pAd->PortCfg.MBSSID[pAd->IoctlIF].PortSecured = WPA_802_1X_PORT_NOT_SECURED;
    RTMPMakeRSNIE(pAd, pAd->PortCfg.MBSSID[pAd->IoctlIF].AuthMode, pAd->PortCfg.MBSSID[pAd->IoctlIF].WepStatus, pAd->IoctlIF);

	// set key table enable bit
	RTMP_IO_READ32(pAd, SEC_CSR4, &sec_csr4);
    if (pAd->PortCfg.MBSSID[pAd->IoctlIF].AuthMode >= Ndis802_11AuthModeWPA)
    {
    	sec_csr4 |= BIT32[pAd->IoctlIF];
        DBGPRINT(RT_DEBUG_TRACE, "AP AuthMode=%d, Pairwise Key Table in-used\n", pAd->PortCfg.MBSSID[pAd->IoctlIF].AuthMode);
    }
    else
    {
    	sec_csr4 &= ~BIT32[pAd->IoctlIF];
        DBGPRINT(RT_DEBUG_TRACE, "AP AuthMode=%d, disable Pairwise Key Table\n", pAd->PortCfg.MBSSID[pAd->IoctlIF].AuthMode);
    }
    RTMP_IO_WRITE32(pAd, SEC_CSR4, sec_csr4);

	DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_AuthMode_Proc::(AuthMode=%d, sec_csr4=0x%x)\n", pAd->IoctlIF, pAd->PortCfg.MBSSID[pAd->IoctlIF].AuthMode, sec_csr4);

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set Encryption Type
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_EncrypType_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	if ((strcmp(arg, "NONE") == 0) || (strcmp(arg, "none") == 0))
		pAd->PortCfg.MBSSID[pAd->IoctlIF].WepStatus = Ndis802_11WEPDisabled;
	else if ((strcmp(arg, "WEP") == 0) || (strcmp(arg, "wep") == 0))
		pAd->PortCfg.MBSSID[pAd->IoctlIF].WepStatus = Ndis802_11WEPEnabled;
	else if ((strcmp(arg, "TKIP") == 0) || (strcmp(arg, "tkip") == 0))
		pAd->PortCfg.MBSSID[pAd->IoctlIF].WepStatus = Ndis802_11Encryption2Enabled;
	else if ((strcmp(arg, "AES") == 0) || (strcmp(arg, "aes") == 0))
		pAd->PortCfg.MBSSID[pAd->IoctlIF].WepStatus = Ndis802_11Encryption3Enabled;
	else if ((strcmp(arg, "TKIPAES") == 0) || (strcmp(arg, "tkipaes") == 0))
		pAd->PortCfg.MBSSID[pAd->IoctlIF].WepStatus = Ndis802_11Encryption4Enabled;
	else
		return FALSE;

    RTMPMakeRSNIE(pAd, pAd->PortCfg.MBSSID[pAd->IoctlIF].AuthMode, pAd->PortCfg.MBSSID[pAd->IoctlIF].WepStatus, pAd->IoctlIF);
	DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_EncrypType_Proc::(EncrypType=%d)\n", pAd->IoctlIF, pAd->PortCfg.MBSSID[pAd->IoctlIF].WepStatus);

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set Default Key ID
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_DefaultKeyID_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	ULONG KeyIdx;

	KeyIdx = simple_strtol(arg, 0, 10);
	if((KeyIdx >= 1 ) && (KeyIdx <= 4))
		pAd->PortCfg.MBSSID[pAd->IoctlIF].DefaultKeyId = (UCHAR) (KeyIdx - 1 );
	else
		return FALSE;  //Invalid argument 
	
	DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_DefaultKeyID_Proc::(DefaultKeyID(0~3)=%d)\n", pAd->IoctlIF, pAd->PortCfg.MBSSID[pAd->IoctlIF].DefaultKeyId);

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set WEP KEY1
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_Key1_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	INT		KeyLen;
	INT		i;
	UCHAR	CipherAlg = CIPHER_WEP64;

	KeyLen = strlen(arg);

	switch (KeyLen)
	{
		case 5: //wep 40 Ascii type
			pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[0].KeyLen = KeyLen;
			memcpy(pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[0].Key, arg, KeyLen);
			CipherAlg = CIPHER_WEP64;
			DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_Key1_Proc::(Key1=%s and type=%s)\n", pAd->IoctlIF, arg, "Ascii");		
			break;
		case 10: //wep 40 Hex type
			for(i=0; i < KeyLen; i++)
			{
				if( !isxdigit(*(arg+i)) )
					return FALSE;  //Not Hex value;
			}
			pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[0].KeyLen = KeyLen/2 ;
			AtoH(arg, pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[0].Key, KeyLen/2);
			CipherAlg = CIPHER_WEP64;
			DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_Key1_Proc::(Key1=%s and type=%s)\n", pAd->IoctlIF, arg, "Hex");		
			break;
		case 13: //wep 104 Ascii type
			pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[0].KeyLen = KeyLen;
			memcpy(pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[0].Key, arg, KeyLen);
			CipherAlg = CIPHER_WEP128;
			DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_Key1_Proc::(Key1=%s and type=%s)\n", pAd->IoctlIF, arg, "Ascii");		
			break;
		case 26: //wep 104 Hex type
			for(i=0; i < KeyLen; i++)
			{
				if( !isxdigit(*(arg+i)) )
					return FALSE;  //Not Hex value;
			}
			pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[0].KeyLen = KeyLen/2 ;
			AtoH(arg, pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[0].Key, KeyLen/2);
			CipherAlg = CIPHER_WEP128;
			DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_Key1_Proc::(Key1=%s and type=%s)\n", pAd->IoctlIF, arg, "Hex");		
			break;
		default: //Invalid argument 
			pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[0].KeyLen = 0;
			DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_Key1_Proc::Invalid argument (=%s)\n", pAd->IoctlIF, arg);		
			return FALSE;
	}

	pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[0].CipherAlg = CipherAlg;

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set WEP KEY2
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_Key2_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	INT		KeyLen;
	INT		i;
	UCHAR	CipherAlg = CIPHER_WEP64;

	KeyLen = strlen(arg);

	switch (KeyLen)
	{
		case 5: //wep 40 Ascii type
			pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[1].KeyLen = KeyLen;
			memcpy(pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[1].Key, arg, KeyLen);
			CipherAlg = CIPHER_WEP64;
			DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_Key2_Proc::(Key2=%s and type=%s)\n", pAd->IoctlIF, arg, "Ascii");		
			break;
		case 10: //wep 40 Hex type
			for(i=0; i < KeyLen; i++)
			{
				if( !isxdigit(*(arg+i)) )
					return FALSE;  //Not Hex value;
			}
			pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[1].KeyLen = KeyLen/2 ;
			AtoH(arg, pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[1].Key, KeyLen/2);
			CipherAlg = CIPHER_WEP64;
			DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_Key2_Proc::(Key2=%s and type=%s)\n", pAd->IoctlIF, arg, "Hex");		
			break;
		case 13: //wep 104 Ascii type
			pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[1].KeyLen = KeyLen;
			memcpy(pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[1].Key, arg, KeyLen);
			CipherAlg = CIPHER_WEP128;
			DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_Key2_Proc::(Key2=%s and type=%s)\n", pAd->IoctlIF, arg, "Ascii");		
			break;
		case 26: //wep 104 Hex type
			for(i=0; i < KeyLen; i++)
			{
				if( !isxdigit(*(arg+i)) )
					return FALSE;  //Not Hex value;
			}
			pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[1].KeyLen = KeyLen/2 ;
			AtoH(arg, pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[1].Key, KeyLen/2);
			CipherAlg = CIPHER_WEP128;
			DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_Key2_Proc::(Key2=%s and type=%s)\n", pAd->IoctlIF, arg, "Hex");		
			break;
		default: //Invalid argument 
			pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[1].KeyLen = 0;
			DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_Key2_Proc::Invalid argument (=%s)\n", pAd->IoctlIF, arg);		
			return FALSE;
	}

	pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[1].CipherAlg = CipherAlg;

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set WEP KEY3
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_Key3_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	INT		KeyLen;
	INT		i;
	UCHAR	CipherAlg = CIPHER_WEP64;

	KeyLen = strlen(arg);

	switch (KeyLen)
	{
		case 5: //wep 40 Ascii type
			pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[2].KeyLen = KeyLen;
			memcpy(pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[2].Key, arg, KeyLen);
			CipherAlg = CIPHER_WEP64;
			DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_Key3_Proc::(Key3=%s and type=%s)\n", pAd->IoctlIF, arg, "Ascii");		
			break;
		case 10: //wep 40 Hex type
			for(i=0; i < KeyLen; i++)
			{
				if( !isxdigit(*(arg+i)) )
					return FALSE;  //Not Hex value;
			}
			pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[2].KeyLen = KeyLen/2 ;
			AtoH(arg, pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[2].Key, KeyLen/2);
			CipherAlg = CIPHER_WEP64;
			DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_Key3_Proc::(Key3=%s and type=%s)\n", pAd->IoctlIF, arg, "Hex");		
			break;
		case 13: //wep 104 Ascii type
			pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[2].KeyLen = KeyLen;
			memcpy(pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[2].Key, arg, KeyLen);
			CipherAlg = CIPHER_WEP128;
			DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_Key3_Proc::(Key3=%s and type=%s)\n", pAd->IoctlIF, arg, "Ascii");		
			break;
		case 26: //wep 104 Hex type
			for(i=0; i < KeyLen; i++)
			{
				if( !isxdigit(*(arg+i)) )
					return FALSE;  //Not Hex value;
			}
			pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[2].KeyLen = KeyLen/2 ;
			AtoH(arg, pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[2].Key, KeyLen/2);
			CipherAlg = CIPHER_WEP128;
			DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_Key3_Proc::(Key3=%s and type=%s)\n", pAd->IoctlIF, arg, "Hex");		
			break;
		default: //Invalid argument 
			pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[2].KeyLen = 0;
			DBGPRINT(RT_DEBUG_ERROR, "IF(ra%d) Set_Key3_Proc::Invalid argument (=%s)\n", pAd->IoctlIF, arg);		
			return FALSE;
	}

	pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[2].CipherAlg = CipherAlg;

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set WEP KEY4
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_Key4_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	INT		KeyLen;
	INT		i;
	UCHAR	CipherAlg = CIPHER_WEP64;

	KeyLen = strlen(arg);

	switch (KeyLen)
	{
		case 5: //wep 40 Ascii type
			pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[3].KeyLen = KeyLen;
			memcpy(pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[3].Key, arg, KeyLen);
			CipherAlg = CIPHER_WEP64;
			DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_Key4_Proc::(Key4=%s and type=%s)\n", pAd->IoctlIF, arg, "Ascii");		
			break;
		case 10: //wep 40 Hex type
			for(i=0; i < KeyLen; i++)
			{
				if( !isxdigit(*(arg+i)) )
					return FALSE;  //Not Hex value;
			}
			pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[3].KeyLen = KeyLen/2 ;
			AtoH(arg, pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[3].Key, KeyLen/2);
			CipherAlg = CIPHER_WEP64;
			DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_Key4_Proc::(Key4=%s and type=%s)\n", pAd->IoctlIF, arg, "Hex");		
			break;
		case 13: //wep 104 Ascii type
			pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[3].KeyLen = KeyLen;
			memcpy(pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[3].Key, arg, KeyLen);
			CipherAlg = CIPHER_WEP128;
			DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_Key4_Proc::(Key4=%s and type=%s)\n", pAd->IoctlIF, arg, "Ascii");		
			break;
		case 26: //wep 104 Hex type
			for(i=0; i < KeyLen; i++)
			{
				if( !isxdigit(*(arg+i)) )
					return FALSE;  //Not Hex value;
			}
			pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[3].KeyLen = KeyLen/2 ;
			AtoH(arg, pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[3].Key, KeyLen/2);
			CipherAlg = CIPHER_WEP128;
			DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_Key4_Proc::(Key4=%s and type=%s)\n", pAd->IoctlIF, arg, "Hex");		
			break;
		default: //Invalid argument 
			pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[3].KeyLen = 0;
			DBGPRINT(RT_DEBUG_ERROR, "IF(ra%d) Set_Key4_Proc::Invalid argument (=%s)\n", pAd->IoctlIF, arg);		
			return FALSE;
	}

	pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[3].CipherAlg = CipherAlg;

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set HotSpot counter policy
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_HSCounter_Proc(
	IN	PRTMP_ADAPTER	pAdapter, 
	IN	PUCHAR			arg)
{
	switch (simple_strtol(arg, 0, 10))
	{
		case 0: //Disable
			pAdapter->PortCfg.bEnableHSCounter = FALSE;
			break;
		case 1: //Enable
			pAdapter->PortCfg.bEnableHSCounter = TRUE;
			break;
		default: //Invalid argument 
			DBGPRINT(RT_DEBUG_ERROR, "Set_HSCounter_Proc::Invalid argument (=%s)\n", arg);		
			return FALSE;
	}
	
	DBGPRINT(RT_DEBUG_TRACE, "Set_HSCounter_Proc::(bEnableHSCounter=%d)\n", pAdapter->PortCfg.bEnableHSCounter);

	return TRUE;	
}

/* 
    ==========================================================================
    Description:
        Set Access ctrol policy
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_AccessPolicy_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	switch (simple_strtol(arg, 0, 10))
	{
		case 0: //Disable
			pAd->PortCfg.MBSSID[pAd->IoctlIF].AccessControlList.Policy = 0;
			break;
		case 1: //Allow All
			pAd->PortCfg.MBSSID[pAd->IoctlIF].AccessControlList.Policy = 1;
			break;
		case 2: //Reject All
			pAd->PortCfg.MBSSID[pAd->IoctlIF].AccessControlList.Policy = 2;
			break;
		default: //Invalid argument 
			DBGPRINT(RT_DEBUG_ERROR, "Set_AccessPolicy_Proc::Invalid argument (=%s)\n", arg);		
			return FALSE;
	}

	// check if ACL change affects any existent associtions
	ApUpdateAccessControlList(pAd, pAd->IoctlIF);
	
	DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) Set_AccessPolicy_Proc::(AccessPolicy=%d)\n", pAd->IoctlIF, pAd->PortCfg.MBSSID[pAd->IoctlIF].AccessControlList.Policy);

	return TRUE;	
}

/* 
    ==========================================================================
    Description:
        Set Access ctrol mac table list
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_AccessControlList_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	UCHAR					macAddr[MAC_ADDR_LEN];
	RT_802_11_ACL			acl;
	CHAR					*this_char;
	CHAR					*value;
	INT						i, j;
	BOOLEAN					isDuplicat=FALSE;

	memset(&acl, 0x00, sizeof(RT_802_11_ACL));

	while ((this_char = strsep((char **)&arg, ";")) != NULL) 
	{
		if(strlen(this_char) != 17)  //Mac address acceptable format 01:02:03:04:05:06 length 17
			return FALSE;

         for (i=0, value = strtok(this_char,":"); value; value = strtok(NULL,":")) 
		 {
			if((strlen(value) != 2) || (!isxdigit(*value)) || (!isxdigit(*(value+1))) ) 
				return FALSE;  //Invalid

			AtoH(value, &macAddr[i++], 2);
		 }

		if(i != 6)
			return FALSE;  //Invalid

		//Check is duplicate
		isDuplicat = FALSE;
		for(j=0; j<acl.Num; j++)
			if(memcmp(acl.Entry[j].Addr, &macAddr, 6) == 0)
				isDuplicat = TRUE;

		if(!isDuplicat)
			memcpy(acl.Entry[acl.Num++].Addr, &macAddr, 6);

		if(acl.Num >= MAX_LEN_OF_MAC_TABLE)
			break;
	}

	acl.Policy = pAd->PortCfg.MBSSID[pAd->IoctlIF].AccessControlList.Policy;
	memcpy(&pAd->PortCfg.MBSSID[pAd->IoctlIF].AccessControlList, &acl, sizeof(RT_802_11_ACL));
	if (pAd->PortCfg.MBSSID[pAd->IoctlIF].AccessControlList.Num > MAX_LEN_OF_MAC_TABLE)
		pAd->PortCfg.MBSSID[pAd->IoctlIF].AccessControlList.Num = MAX_LEN_OF_MAC_TABLE;

	// check if ACL change affects any existent associtions
	ApUpdateAccessControlList(pAd, pAd->IoctlIF);

	DBGPRINT(RT_DEBUG_TRACE, "Set::Set_AccessControlList_Proc(Policy=%d, Entry#=%d\n",
        pAd->PortCfg.MBSSID[pAd->IoctlIF].AccessControlList.Policy, pAd->PortCfg.MBSSID[pAd->IoctlIF].AccessControlList.Num);

#ifdef DBG
	DBGPRINT(RT_DEBUG_TRACE, "=============== Entry ===============\n");
	for(i=0; i<pAd->PortCfg.MBSSID[pAd->IoctlIF].AccessControlList.Num; i++)
	{
		printk("Entry #%02d: ", i+1);
		for(j=0; j<6; j++)
		   printk("%02X ", pAd->PortCfg.MBSSID[pAd->IoctlIF].AccessControlList.Entry[i].Addr[j]);
		printk("\n");
	}
#endif
	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set WPA PSK key

    Arguments:
        pAdapter            Pointer to our adapter
        arg                 WPA pre-shared key string

    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_WPAPSK_Proc(
	IN	PRTMP_ADAPTER	pAdapter, 
	IN	PUCHAR			arg)
{
	UCHAR       keyMaterial[40];

	DBGPRINT(RT_DEBUG_TRACE, "Set_WPAPSK_Proc::(WPAPSK=%s)\n", arg);

	if ((strlen(arg) < 8) || (strlen(arg) > 64))
	{
		DBGPRINT(RT_DEBUG_TRACE, "Set failed!!(WPAPSK=%s), WPAPSK key-string required 8 ~ 64 characters \n", arg);
		return FALSE;
	}

	if (strlen(arg) == 64)
	{
	    AtoH(arg, pAdapter->PortCfg.MBSSID[pAdapter->IoctlIF].PMK, 32);
	}
	else
	{
	    PasswordHash((CHAR *)arg, pAdapter->PortCfg.MBSSID[pAdapter->IoctlIF].Ssid, pAdapter->PortCfg.MBSSID[pAdapter->IoctlIF].SsidLen, keyMaterial);
	    NdisMoveMemory(pAdapter->PortCfg.MBSSID[pAdapter->IoctlIF].PMK, keyMaterial, 32);
	}

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Reset statistics counter

    Arguments:
        pAdapter            Pointer to our adapter
        arg                 

    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_ResetStatCounter_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
    UCHAR           i;
    MAC_TABLE_ENTRY *pEntry;
    
    DBGPRINT(RT_DEBUG_TRACE, "==>Set_ResetStatCounter_Proc\n");

    // add the most up-to-date h/w raw counters into software counters
    NICUpdateRawCounters(pAd);
    
    NdisZeroMemory(&pAd->WlanCounters, sizeof(COUNTER_802_11));
    NdisZeroMemory(&pAd->Counters8023, sizeof(COUNTER_802_3));
    NdisZeroMemory(&pAd->RalinkCounters, sizeof(COUNTER_RALINK));

    // Reset HotSpot counter
    for (i = 0; i < MAX_LEN_OF_MAC_TABLE; i++)
    {
        pEntry = &pAd->MacTab.Content[i];

        if ((pEntry->Valid == FALSE) || (pEntry->Sst != SST_ASSOC))
            continue;

        pEntry->HSCounter.LastDataPacketTime = 0;
        pEntry->HSCounter.TotalRxByteCount= 0;
        pEntry->HSCounter.TotalTxByteCount= 0;
    }
    
    return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set Group Rekey interval type 

    Arguments:
        pAdapter            Pointer to our adapter
        arg                 

    Return:
          
    ==========================================================================
*/
INT	Set_RekeyMethod_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
    if ((strcmp(arg, "TIME") == 0) || (strcmp(arg, "time") == 0))
        pAd->PortCfg.WPAREKEY.ReKeyMethod = TIME_REKEY;
    else if ((strcmp(arg, "PKT") == 0) || (strcmp(arg, "pkt") == 0))
        pAd->PortCfg.WPAREKEY.ReKeyMethod = PKT_REKEY;
    else if ((strcmp(arg, "NONE") == 0) || (strcmp(arg, "none") == 0))
    {
        pAd->PortCfg.WPAREKEY.ReKeyMethod = DISABLE_REKEY;
        pAd->PortCfg.WPAREKEY.ReKeyInterval = 0;
    }
    else
        return FALSE;
    
    if ((pAd->PortCfg.REKEYTimerRunning == FALSE) && (pAd->PortCfg.WPAREKEY.ReKeyInterval != 0)
        && (pAd->PortCfg.WPAREKEY.ReKeyMethod != DISABLE_REKEY))
    {
        add_timer(&pAd->PortCfg.REKEYTimer);
        pAd->PortCfg.REKEYTimer.expires = jiffies + GUPDATE_EXEC_INTV;
        pAd->PortCfg.REKEYTimerRunning = TRUE;
        pAd->PortCfg.REKEYCOUNTER = 0;
    }
    DBGPRINT(RT_DEBUG_TRACE, "Set_RekeyMethod_Proc::(RekeyMethod=%d)\n", pAd->PortCfg.WPAREKEY.ReKeyMethod);
    
    return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set Group Rekey interval

    Arguments:
        pAdapter            Pointer to our adapter
        arg                 

    Return:
          
    ==========================================================================
*/
INT	Set_RekeyInterval_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
    ULONG   period;
    
    period = simple_strtol(arg, 0, 10);
    if ((pAd->PortCfg.REKEYTimerRunning == TRUE) && (period == 0))
    {
        del_timer_sync(&pAd->PortCfg.REKEYTimer);
        pAd->PortCfg.WPAREKEY.ReKeyInterval = 0;
        pAd->PortCfg.REKEYTimerRunning = FALSE;
    }
    else if (period <= MAX_REKEY_INTER)
    {
        pAd->PortCfg.WPAREKEY.ReKeyInterval = period;
        if (pAd->PortCfg.REKEYTimerRunning == FALSE)
        {
            add_timer(&pAd->PortCfg.REKEYTimer);
            pAd->PortCfg.REKEYTimer.expires = jiffies + GUPDATE_EXEC_INTV;
            pAd->PortCfg.REKEYTimerRunning = TRUE;
            pAd->PortCfg.REKEYCOUNTER = 0;
        }
    }
    else
        return FALSE;

    DBGPRINT(RT_DEBUG_TRACE, "Set_RekeyInterval_Proc::(RekeyInterval=%d)\n", period);
    return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set time period for WPA-PMK

    Arguments:
        pAdapter            Pointer to our adapter
        arg                 

    Return:
          
    ==========================================================================
*/
INT	Set_PMKCachePeriod_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
    pAd->PortCfg.PMKCachePeriod = simple_strtol(arg, 0, 10) * 60 * HZ;
    
    DBGPRINT(RT_DEBUG_TRACE, "Set_PMKCachePeriod_Proc::(PMKCachePeriod=%d)\n", pAd->PortCfg.PMKCachePeriod);
    return TRUE;
}

/* 
    ==========================================================================
    Description:
        Issue a site survey command to driver
	Arguments:
	    pAdapter                    Pointer to our adapter
	    wrq                         Pointer to the ioctl argument

    Return Value:
        None

    Note:
        Usage: 
               1.) iwpriv ra0 set site_survey
    ==========================================================================
*/
INT Set_SiteSurvey_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
    ApSiteSurvey(pAd);
    
    DBGPRINT(RT_DEBUG_TRACE, "Set_SiteSurvey_Proc\n");

    return TRUE;
}

/* 
    ==========================================================================
    Description:
        UI should not call this function, it only used by 802.1x daemon
	Arguments:
	    pAd		Pointer to our adapter
	    wrq		Pointer to the ioctl argument
    ==========================================================================
*/
VOID RTMPIoctlAddPMKIDCache(
	IN	PRTMP_ADAPTER	pAd, 
	IN	struct iwreq	*wrq)
{
	NDIS_802_11_KEY *pKey = (PNDIS_802_11_KEY) wrq->u.data.pointer;
    
    if (pAd->PortCfg.MBSSID[pAd->IoctlIF].AuthMode >= Ndis802_11AuthModeWPA2)
	{
		if(pKey->KeyLength == 32)
		{
			UCHAR	digest[80], PMK_key[20], macaddr[MAC_ADDR_LEN];
			
			// Calculate PMKID
			NdisMoveMemory(&PMK_key[0], "PMK Name", 8);
			NdisMoveMemory(&PMK_key[8], pAd->PortCfg.MBSSID[pAd->IoctlIF].Bssid, MAC_ADDR_LEN);
			NdisMoveMemory(&PMK_key[14], pKey->addr, MAC_ADDR_LEN);
			HMAC_SHA1(PMK_key, 20, pKey->KeyMaterial, PMK_LEN, digest);

			NdisMoveMemory(macaddr, pKey->addr, MAC_ADDR_LEN);
			RTMPAddPMKIDCache(pAd, pAd->IoctlIF, macaddr, digest, pKey->KeyMaterial);
			
			DBGPRINT(RT_DEBUG_TRACE, "WPA2(pre-auth):(%02x:%02x:%02x:%02x:%02x:%02x)Calc PMKID=%02x:%02x:%02x:%02x:%02x:%02x\n", 
				pKey->addr[0],pKey->addr[1],pKey->addr[2],pKey->addr[3],pKey->addr[4],pKey->addr[5],digest[0],digest[1],digest[2],digest[3],digest[4],digest[5]);
			DBGPRINT(RT_DEBUG_TRACE, "PMK =%02x:%02x:%02x:%02x-%02x:%02x:%02x:%02x\n",pKey->KeyMaterial[0],pKey->KeyMaterial[1],
				pKey->KeyMaterial[2],pKey->KeyMaterial[3],pKey->KeyMaterial[4],pKey->KeyMaterial[5],pKey->KeyMaterial[6],pKey->KeyMaterial[7]);
		}
		else
            DBGPRINT(RT_DEBUG_ERROR, "Set::RT_OID_802_11_WPA2_ADD_PMKID_CACHE ERROR or is wep key \n");
	}
    
    DBGPRINT(RT_DEBUG_TRACE, "<== RTMPIoctlAddPMKIDCache\n");
}

/* 
    ==========================================================================
    Description:
        UI should not call this function, it only used by 802.1x daemon
	Arguments:
	    pAd		Pointer to our adapter
	    wrq		Pointer to the ioctl argument
    ==========================================================================
*/
VOID RTMPIoctlRadiusData(
	IN	PRTMP_ADAPTER	pAd, 
	IN	struct iwreq	*wrq)
{
	DBGPRINT(RT_DEBUG_INFO, "RTMPIoctlRadiusData, IF(ra%d)\n", pAd->IoctlIF);
	
	if ((pAd->PortCfg.MBSSID[pAd->IoctlIF].AuthMode == Ndis802_11AuthModeWPA) 
    	|| (pAd->PortCfg.MBSSID[pAd->IoctlIF].AuthMode == Ndis802_11AuthModeWPA2)
    	|| (pAd->PortCfg.MBSSID[pAd->IoctlIF].AuthMode == Ndis802_11AuthModeWPA1WPA2) 
    	|| (pAd->PortCfg.MBSSID[pAd->IoctlIF].IEEE8021X == TRUE))
    	WpaSend(pAd, wrq->u.data.pointer, wrq->u.data.length);
}

/* 
    ==========================================================================
    Description:
        UI should not call this function, it only used by 802.1x daemon
	Arguments:
	    pAd		Pointer to our adapter
	    wrq		Pointer to the ioctl argument
    ==========================================================================
*/
VOID RTMPIoctlAddWPAKey(
	IN	PRTMP_ADAPTER	pAd, 
	IN	struct iwreq	*wrq)
{
	NDIS_802_11_KEY *pKey;
	ULONG			KeyIdx;
	INT				i;
		
    DBGPRINT(RT_DEBUG_TRACE, "RTMPIoctlAddWPAKey-IF(ra%d)\n", pAd->IoctlIF);

	pKey = (PNDIS_802_11_KEY) wrq->u.data.pointer;

	if (pAd->PortCfg.MBSSID[pAd->IoctlIF].AuthMode >= Ndis802_11AuthModeWPA)
	{
		if(pKey->KeyLength == 32)
		{
			NdisMoveMemory(pAd->PortCfg.MBSSID[pAd->IoctlIF].PMK, pKey->KeyMaterial, 32);
            DBGPRINT(RT_DEBUG_TRACE, "After ADD_KEY : PMK=%02x:%02x:%02x:%02x-%02x:%02x:%02x:%02x\n",
            	pAd->PortCfg.MBSSID[pAd->IoctlIF].PMK[0],pAd->PortCfg.MBSSID[pAd->IoctlIF].PMK[1],pAd->PortCfg.MBSSID[pAd->IoctlIF].PMK[2],pAd->PortCfg.MBSSID[pAd->IoctlIF].PMK[3],
            	pAd->PortCfg.MBSSID[pAd->IoctlIF].PMK[4],pAd->PortCfg.MBSSID[pAd->IoctlIF].PMK[5],pAd->PortCfg.MBSSID[pAd->IoctlIF].PMK[6],pAd->PortCfg.MBSSID[pAd->IoctlIF].PMK[7]);
		}
		else
            DBGPRINT(RT_DEBUG_TRACE, "RTMPIoctlAddWPAKey ERROR \n");
	}
	else	// Old WEP stuff
	{
		// remove all key in MAC ASIC
		for (i=0; i<4; i++)
		{
			AsicRemoveSharedKeyEntry(pAd, pAd->IoctlIF, i);
		}
		
		KeyIdx = pKey->KeyIndex & 0x0fffffff;

		// it is a shared key
		if (KeyIdx <= 4)
		{
			pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[KeyIdx].KeyLen = (UCHAR) pKey->KeyLength;
			NdisMoveMemory(pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[KeyIdx].Key, &pKey->KeyMaterial, pKey->KeyLength);
			if (pKey->KeyIndex & 0x80000000)
			{
				// Default key for tx (shared key)
				pAd->PortCfg.MBSSID[pAd->IoctlIF].DefaultKeyId = (UCHAR) KeyIdx;								
			}
			pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[KeyIdx].CipherAlg = CIPHER_WEP128;
			AsicAddSharedKeyEntry(
				pAd,
				pAd->IoctlIF,
				KeyIdx,
				pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[KeyIdx].CipherAlg,
				pAd->PortCfg.MBSSID[pAd->IoctlIF].SharedKey[KeyIdx].Key,
				NULL,
				NULL);
		}
	}
}


#ifdef DBG
/* 
    ==========================================================================
    Description:
        Read / Write BBP
Arguments:
    pAdapter                    Pointer to our adapter
    wrq                         Pointer to the ioctl argument

    Return Value:
        None

    Note:
        Usage: 
               1.) iwpriv ra0 bbp               ==> read all BBP
               2.) iwpriv ra0 bbp 1             ==> read BBP where RegID=1
               3.) iwpriv ra0 bbp 1=10		    ==> write BBP R1=0x10
    ==========================================================================
*/
VOID RTMPIoctlBBP(
	IN	PRTMP_ADAPTER	pAdapter,
	IN	struct iwreq	*wrq)
{
	CHAR				*this_char;
	CHAR				*value;
	UCHAR				regBBP;
	CHAR				msg[2048];
	CHAR				arg[255];
	CHAR				*ptr;
	LONG				bbpId;
	LONG				bbpValue;
	BOOLEAN				bIsPrintAllBBP = FALSE;

	DBGPRINT(RT_DEBUG_INFO, "==>RTMPIoctlBBP\n");

	memset(msg, 0x00, 2048);
	if (wrq->u.data.length > 1) //No parameters.
	{
		memcpy(arg, wrq->u.data.pointer, (wrq->u.data.length > 255) ? 255 : wrq->u.data.length);
		ptr = arg;
		sprintf(msg, "\n");
		//Parsing Read or Write
		while ((this_char = strsep(&ptr, ",")) != NULL)
		{
			DBGPRINT(RT_DEBUG_INFO, "this_char=%s\n", this_char);
			if (!*this_char)
				continue;

			if ((value = strchr(this_char, '=')) != NULL)
				*value++ = 0;

			if (!value || !*value)
			{ //Read
				DBGPRINT(RT_DEBUG_INFO, "this_char=%s, value=%s\n", this_char, value);
				if (sscanf(this_char, "%d", &(bbpId)) == 1)
				{
					if (bbpId <= 108)
					{
						RTMP_BBP_IO_READ8_BY_REG_ID(pAdapter, bbpId, &regBBP);
						sprintf(msg+strlen(msg), "R%02d[0x%02X]:%02X  ", bbpId, bbpId*2, regBBP);
						DBGPRINT(RT_DEBUG_INFO, "msg=%s\n", msg);
					}
					else
					{//Invalid parametes, so default printk all bbp
						bIsPrintAllBBP = TRUE;
						break;
					}
				}
				else
				{ //Invalid parametes, so default printk all bbp
					bIsPrintAllBBP = TRUE;
					break;
				}
			}
			else
			{ //Write
				DBGPRINT(RT_DEBUG_INFO, "this_char=%s, value=%s\n", this_char, value);
				if ((sscanf(this_char, "%d", &(bbpId)) == 1) && (sscanf(value, "%x", &(bbpValue)) == 1))
				{
					DBGPRINT(RT_DEBUG_INFO, "bbpID=%02d, value=0x%x\n", bbpId, bbpValue);
					if (bbpId <= 108)
					{
						RTMP_BBP_IO_WRITE8_BY_REG_ID(pAdapter, (UCHAR)bbpId,(UCHAR) bbpValue);
						//Read it back for showing
						RTMP_BBP_IO_READ8_BY_REG_ID(pAdapter, bbpId, &regBBP);
						sprintf(msg+strlen(msg), "R%02d[0x%02X]:%02X\n", bbpId, bbpId*2, regBBP);
						DBGPRINT(RT_DEBUG_INFO, "msg=%s\n", msg);
					}
					else
					{//Invalid parametes, so default printk all bbp
						bIsPrintAllBBP = TRUE;
						break;
					}
				}
				else
				{ //Invalid parametes, so default printk all bbp
					bIsPrintAllBBP = TRUE;
					break;
				}
			}
		}
	}
	else
		bIsPrintAllBBP = TRUE;

	if (bIsPrintAllBBP)
	{
		memset(msg, 0x00, 2048);
		sprintf(msg, "\n");
		for (bbpId = 0; bbpId <= 108; bbpId++)
		{
			RTMP_BBP_IO_READ8_BY_REG_ID(pAdapter, bbpId, &regBBP);
			sprintf(msg+strlen(msg), "R%02d[0x%02X]:%02X    ", bbpId, bbpId*2, regBBP);
			if (bbpId%5 == 4)
				sprintf(msg+strlen(msg), "\n");
		}
		// Copy the information into the user buffer
		DBGPRINT(RT_DEBUG_TRACE, "strlen(msg)=%d\n", strlen(msg));
		wrq->u.data.length = strlen(msg);
		copy_to_user(wrq->u.data.pointer, msg, wrq->u.data.length);
	}
	else
	{
		DBGPRINT(RT_DEBUG_INFO, "copy to user [msg=%s]\n", msg);
		// Copy the information into the user buffer
		DBGPRINT(RT_DEBUG_INFO, "strlen(msg) =%d\n", strlen(msg));
		wrq->u.data.length = strlen(msg);
		copy_to_user(wrq->u.data.pointer, msg, wrq->u.data.length);
	}
	DBGPRINT(RT_DEBUG_TRACE, "<==RTMPIoctlBBP\n\n");
}

/* 
    ==========================================================================
    Description:
        Read / Write MAC
Arguments:
    pAdapter                    Pointer to our adapter
    wrq                         Pointer to the ioctl argument

    Return Value:
        None

    Note:
        Usage: 
               1.) iwpriv ra0 mac 0        ==> read MAC where Addr=0x0
               2.) iwpriv ra0 mac 0=12     ==> write MAC where Addr=0x0, value=12
    ==========================================================================
*/
VOID RTMPIoctlMAC(
	IN	PRTMP_ADAPTER	pAdapter, 
	IN	struct iwreq	*wrq)
{
	CHAR				*this_char;
	CHAR				*value;
	INT					j = 0, k = 0;
	CHAR				msg[1024];
	CHAR				arg[255];
	CHAR				*ptr;
	ULONG				macAddr = 0;
	UCHAR				temp[16], temp2[16];
	ULONG				macValue;

	DBGPRINT(RT_DEBUG_INFO, "==>RTMPIoctlMAC\n");
	memset(msg, 0x00, 1024);
	if (wrq->u.data.length > 1) //No parameters.
	{
		memcpy(arg, wrq->u.data.pointer, (wrq->u.data.length > 255) ? 255 : wrq->u.data.length);
		ptr = arg;
		sprintf(msg, "\n");
		//Parsing Read or Write
		while ((this_char = strsep(&ptr, ",")) != NULL)
		{
			DBGPRINT(RT_DEBUG_INFO, "this_char=%s\n", this_char);
			if (!*this_char)
				continue;

			if ((value = strchr(this_char, '=')) != NULL)
				*value++ = 0;

			if (!value || !*value)
			{ //Read
				DBGPRINT(RT_DEBUG_INFO, "Read: this_char=%s, strlen=%d\n", this_char, strlen(this_char));

				// Sanity check
				if(strlen(this_char) > 4)
					break;

				j = strlen(this_char);
				while(j-- > 0)
				{
					if(this_char[j] > 'f' || this_char[j] < '0')
						return;
				}

				// Mac Addr
				k = j = strlen(this_char);
				while(j-- > 0)
				{
					this_char[4-k+j] = this_char[j];
				}
				
				while(k < 4)
					this_char[3-k++]='0';
				this_char[4]='\0';

				if(strlen(this_char) == 4)
				{
					AtoH(this_char, temp, 4);
					macAddr = *temp*256 + temp[1];					
					if (macAddr < 0xFFFF)
					{
						RTMP_IO_READ32(pAdapter, macAddr, &macValue);
						DBGPRINT(RT_DEBUG_TRACE, "MacAddr=%x, MacValue=%x\n", macAddr, macValue);
						sprintf(msg+strlen(msg), "[0x%08X]:%08X  ", macAddr , macValue);
						DBGPRINT(RT_DEBUG_INFO, "msg=%s\n", msg);
					}
					else
					{//Invalid parametes, so default printk all bbp
						break;
					}
				}
			}
			else
			{ //Write
				DBGPRINT(RT_DEBUG_INFO, "Write: this_char=%s, strlen(value)=%d, value=%s\n", this_char, strlen(value), value);
				memcpy(&temp2, value, strlen(value));
				temp2[strlen(value)] = '\0';

				// Sanity check
				if((strlen(this_char) > 4) || strlen(temp2) > 8)
					break;

				j = strlen(this_char);
				while(j-- > 0)
				{
					if(this_char[j] > 'f' || this_char[j] < '0')
						return;
				}

				j = strlen(temp2);
				while(j-- > 0)
				{
					if(temp2[j] > 'f' || temp2[j] < '0')
						return;
				}

				//MAC Addr
				k = j = strlen(this_char);
				while(j-- > 0)
				{
					this_char[4-k+j] = this_char[j];
				}

				while(k < 4)
					this_char[3-k++]='0';
				this_char[4]='\0';

				//MAC value
				k = j = strlen(temp2);
				while(j-- > 0)
				{
					temp2[8-k+j] = temp2[j];
				}
				
				while(k < 8)
					temp2[7-k++]='0';
				temp2[8]='\0';

				{
					AtoH(this_char, temp, 4);
					macAddr = *temp*256 + temp[1];

					AtoH(temp2, temp, 8);
					macValue = *temp*256*256*256 + temp[1]*256*256 + temp[2]*256 + temp[3];

					// debug mode
					if (macAddr == (HW_DEBUG_SETTING_BASE + 4))
					{
						// 0x2bf4: byte0 non-zero: enable R17 tuning, 0: disable R17 tuning
                        if (macValue & 0x000000ff) 
                        {
                            pAdapter->BbpTuning.bEnable = TRUE;
                            DBGPRINT(RT_DEBUG_TRACE,"turn on R17 tuning\n");
                        }
                        else
                        {
                            UCHAR R17;
                            pAdapter->BbpTuning.bEnable = FALSE;
                            if (pAdapter->PortCfg.Channel > 14)
                                R17 = pAdapter->BbpTuning.R17LowerBoundA;
                            else
                                R17 = pAdapter->BbpTuning.R17LowerBoundG;
                            RTMP_BBP_IO_WRITE8_BY_REG_ID(pAdapter, 17, R17);
                            DBGPRINT(RT_DEBUG_TRACE,"turn off R17 tuning, restore to 0x%02x\n", R17);
                        }
						return;
					}

					DBGPRINT(RT_DEBUG_TRACE, "MacAddr=%02x, MacValue=0x%x\n", macAddr, macValue);
					
					RTMP_IO_WRITE32(pAdapter, macAddr, macValue);
					sprintf(msg+strlen(msg), "[0x%08X]:%08X  ", macAddr, macValue);
					DBGPRINT(RT_DEBUG_INFO, "msg=%s\n", msg);
				}
			}
		}
	}

	if(strlen(msg) == 1)
		sprintf(msg+strlen(msg), "===>Error command format!");
	DBGPRINT(RT_DEBUG_INFO, "copy to user [msg=%s]\n", msg);
	// Copy the information into the user buffer
	DBGPRINT(RT_DEBUG_INFO, "strlen(msg) =%d\n", strlen(msg));
	wrq->u.data.length = strlen(msg);
	copy_to_user(wrq->u.data.pointer, msg, wrq->u.data.length);
	
	DBGPRINT(RT_DEBUG_TRACE, "<==RTMPIoctlMAC\n\n");
}

/* 
    ==========================================================================
    Description:
        Read / Write E2PROM
Arguments:
    pAdapter                    Pointer to our adapter
    wrq                         Pointer to the ioctl argument

    Return Value:
        None

    Note:
        Usage: 
               1.) iwpriv ra0 e2p 0     	==> read E2PROM where Addr=0x0
               2.) iwpriv ra0 e2p 0=1234    ==> write E2PROM where Addr=0x0, value=1234
    ==========================================================================
*/
VOID RTMPIoctlE2PROM(
	IN	PRTMP_ADAPTER	pAdapter, 
	IN	struct iwreq	*wrq)
{
	CHAR				*this_char;
	CHAR				*value;
	INT					j = 0, k = 0;
	CHAR				msg[1024];
	CHAR				arg[255];
	CHAR				*ptr;
	USHORT				eepAddr = 0;
	UCHAR				temp[16], temp2[16];
	USHORT				eepValue;

	DBGPRINT(RT_DEBUG_INFO, "==>RTMPIoctlE2PROM\n");
	memset(msg, 0x00, 1024);
	if (wrq->u.data.length > 1) //No parameters.
	{
		memcpy(arg, wrq->u.data.pointer, (wrq->u.data.length > 255) ? 255 : wrq->u.data.length);
		ptr = arg;
		sprintf(msg, "\n");
		//Parsing Read or Write
		while ((this_char = strsep(&ptr, ",")) != NULL)
		{
			DBGPRINT(RT_DEBUG_INFO, "this_char=%s\n", this_char);
			if (!*this_char)
				continue;

			if ((value = strchr(this_char, '=')) != NULL)
				*value++ = 0;

			if (!value || !*value)
			{ //Read
				DBGPRINT(RT_DEBUG_INFO, "Read: this_char=%s, strlen=%d\n", this_char, strlen(this_char));

				// Sanity check
				if(strlen(this_char) > 4)
					break;

				j = strlen(this_char);
				while(j-- > 0)
				{
					if(this_char[j] > 'f' || this_char[j] < '0')
						return;
				}

				// E2PROM addr
				k = j = strlen(this_char);
				while(j-- > 0)
				{
					this_char[4-k+j] = this_char[j];
				}
				
				while(k < 4)
					this_char[3-k++]='0';
				this_char[4]='\0';

				if(strlen(this_char) == 4)
				{
					AtoH(this_char, temp, 4);
					eepAddr = *temp*256 + temp[1];					
					if (eepAddr < 0xFFFF)
					{
						eepValue = RTMP_EEPROM_READ16(pAdapter, eepAddr);
						DBGPRINT(RT_DEBUG_INFO, "eepAddr=%x, eepValue=0x%x\n", eepAddr, eepValue);
						sprintf(msg+strlen(msg), "[0x%04X]:0x%X  ", eepAddr , eepValue);
						DBGPRINT(RT_DEBUG_INFO, "msg=%s\n", msg);
					}
					else
					{//Invalid parametes, so default printk all bbp
						break;
					}
				}
			}
			else
			{ //Write
				DBGPRINT(RT_DEBUG_INFO, "Write: this_char=%s, strlen(value)=%d, value=%s\n", this_char, strlen(value), value);
				memcpy(&temp2, value, strlen(value));
				temp2[strlen(value)] = '\0';

				// Sanity check
				if((strlen(this_char) > 4) || strlen(temp2) > 8)
					break;

				j = strlen(this_char);
				while(j-- > 0)
				{
					if(this_char[j] > 'f' || this_char[j] < '0')
						return;
				}
				j = strlen(temp2);
				while(j-- > 0)
				{
					if(temp2[j] > 'f' || temp2[j] < '0')
						return;
				}

				//MAC Addr
				k = j = strlen(this_char);
				while(j-- > 0)
				{
					this_char[4-k+j] = this_char[j];
				}

				while(k < 4)
					this_char[3-k++]='0';
				this_char[4]='\0';

				//MAC value
				k = j = strlen(temp2);
				while(j-- > 0)
				{
					temp2[4-k+j] = temp2[j];
				}
				
				while(k < 4)
					temp2[3-k++]='0';
				temp2[4]='\0';

				AtoH(this_char, temp, 4);
				eepAddr = *temp*256 + temp[1];

				AtoH(temp2, temp, 4);
				eepValue = *temp*256 + temp[1];

				DBGPRINT(RT_DEBUG_INFO, "eepAddr=%02x, eepValue=0x%x\n", eepAddr, eepValue);
				
				RTMP_EEPROM_WRITE16(pAdapter, eepAddr, eepValue);
				sprintf(msg+strlen(msg), "[0x%02X]:%02X  ", eepAddr, eepValue);
				DBGPRINT(RT_DEBUG_INFO, "msg=%s\n", msg);
			}
		}
	}

	if(strlen(msg) == 1)
		sprintf(msg+strlen(msg), "===>Error command format!");

	// Copy the information into the user buffer
	DBGPRINT(RT_DEBUG_INFO, "copy to user [msg=%s]\n", msg);
	wrq->u.data.length = strlen(msg);
	copy_to_user(wrq->u.data.pointer, msg, wrq->u.data.length);
	
	DBGPRINT(RT_DEBUG_TRACE, "<==RTMPIoctlE2PROM\n");
}
#endif //#ifdef DBG

/* 
    ==========================================================================
    Description:
        Read statistics counter
Arguments:
    pAdapter                    Pointer to our adapter
    wrq                         Pointer to the ioctl argument

    Return Value:
        None

    Note:
        Usage: 
               1.) iwpriv ra0 stat 0     	==> Read statistics counter
    ==========================================================================
*/
VOID RTMPIoctlStatistics(
	IN	PRTMP_ADAPTER	pAd, 
	IN	struct iwreq	*wrq)
{
	char                msg[1600];

    memset(msg, 0x00, 1600);
    sprintf(msg, "\n");

    sprintf(msg+strlen(msg), "Tx success                      = %d\n", (ULONG)pAd->WlanCounters.TransmittedFragmentCount);
    sprintf(msg+strlen(msg), "Tx success without retry        = %d\n", (ULONG)pAd->WlanCounters.TransmittedFragmentCount - (ULONG)pAd->WlanCounters.RetryCount);
    sprintf(msg+strlen(msg), "Tx success after retry          = %d\n", (ULONG)pAd->WlanCounters.RetryCount);
    sprintf(msg+strlen(msg), "Tx fail to Rcv ACK after retry  = %d\n", (ULONG)pAd->WlanCounters.FailedCount);
    sprintf(msg+strlen(msg), "RTS Success Rcv CTS             = %d\n", (ULONG)pAd->WlanCounters.RTSSuccessCount);
    sprintf(msg+strlen(msg), "RTS Fail Rcv CTS                = %d\n", (ULONG)pAd->WlanCounters.RTSFailureCount);

    sprintf(msg+strlen(msg), "Rx success                      = %d\n", (ULONG)pAd->WlanCounters.ReceivedFragmentCount);
    sprintf(msg+strlen(msg), "Rx with CRC                     = %d\n", (ULONG)pAd->WlanCounters.FCSErrorCount);
    sprintf(msg+strlen(msg), "Rx drop due to out of resource  = %d\n", (ULONG)pAd->Counters8023.RxNoBuffer);
    sprintf(msg+strlen(msg), "Rx duplicate frame              = %d\n", (ULONG)pAd->WlanCounters.FrameDuplicateCount);

    sprintf(msg+strlen(msg), "False CCA (one second)            = %d\n", (ULONG)pAd->RalinkCounters.OneSecFalseCCACnt);
    sprintf(msg+strlen(msg), "RSSI-A                          = %d\n", (LONG)(pAd->PortCfg.LastRssi - pAd->BbpRssiToDbmDelta));
    sprintf(msg+strlen(msg), "RSSI-B (if available)           = %d\n\n", (LONG)(pAd->PortCfg.LastRssi2 - pAd->BbpRssiToDbmDelta));
    
    // Copy the information into the user buffer
    DBGPRINT(RT_DEBUG_INFO, "copy to user [msg=%s]\n", msg);
    wrq->u.data.length = strlen(msg);
    copy_to_user(wrq->u.data.pointer, msg, wrq->u.data.length);
	
    DBGPRINT(RT_DEBUG_TRACE, "<==RTMPIoctlStatistics\n");
}

/* 
    ==========================================================================
    Description:
        Parse encryption type
Arguments:
    pAdapter                    Pointer to our adapter
    wrq                         Pointer to the ioctl argument

    Return Value:
        None

    Note:
    ==========================================================================
*/
CHAR *GetEncryptType(CHAR enc)
{
    if(enc == Ndis802_11WEPDisabled)
        return "NONE";
    if(enc == Ndis802_11WEPEnabled)
    	return "WEP";
    if(enc == Ndis802_11Encryption2Enabled)
    	return "TKIP";
    if(enc == Ndis802_11Encryption3Enabled)
    	return "AES";
    else
    	return "UNKNOW";
}		
    	
/* 
    ==========================================================================
    Description:
        Get site survey results
	Arguments:
	    pAdapter                    Pointer to our adapter
	    wrq                         Pointer to the ioctl argument

    Return Value:
        None

    Note:
        Usage:
        		1.) UI needs to wait 4 seconds after issue a site survey command
        		2.) iwpriv ra0 get_site_survey
        		3.) UI needs to prepare at least 4096bytes to get the results
    ==========================================================================
*/
#define	LINE_LEN	(8+8+35+20+8)	// Channel+RSSI+SSID+Bssid+WepStatus
VOID RTMPIoctlGetSiteSurvey(
	IN	PRTMP_ADAPTER	pAdapter, 
	IN	struct iwreq	*wrq)
{
	CHAR		*msg;
    INT 		i=0;     
    
	msg = (CHAR *) kmalloc(sizeof(CHAR)*(MAX_LEN_OF_BSS_TABLE*LINE_LEN), MEM_ALLOC_FLAG);

    memset(msg, 0 ,MAX_LEN_OF_BSS_TABLE*LINE_LEN );
    sprintf(msg,"%s","\n");
	sprintf(msg+strlen(msg),"%-8s%-8s%-35s%-20s%-8s\n", "Channel", "RSSI", "SSID", "BSSID", "Security");
    for(i=0;i<MAX_LEN_OF_BSS_TABLE;i++)
    {
		if( pAdapter->ScanTab.BssEntry[i].Channel==0)
	    break;
	if((strlen(msg)+LINE_LEN ) >= (MAX_LEN_OF_BSS_TABLE*LINE_LEN) )
	    break;	
			
		sprintf(msg+strlen(msg),"%-8d", pAdapter->ScanTab.BssEntry[i].Channel);
		sprintf(msg+strlen(msg),"%-8d", pAdapter->ScanTab.BssEntry[i].Rssi - pAdapter->BbpRssiToDbmDelta);
		sprintf(msg+strlen(msg),"%-35s", pAdapter->ScanTab.BssEntry[i].Ssid);
		sprintf(msg+strlen(msg),"%02x:%02x:%02x:%02x:%02x:%02x  ", 
			pAdapter->ScanTab.BssEntry[i].Bssid[0], 
			pAdapter->ScanTab.BssEntry[i].Bssid[1],
			pAdapter->ScanTab.BssEntry[i].Bssid[2], 
			pAdapter->ScanTab.BssEntry[i].Bssid[3], 
			pAdapter->ScanTab.BssEntry[i].Bssid[4], 
			pAdapter->ScanTab.BssEntry[i].Bssid[5]);
		sprintf(msg+strlen(msg),"%-8s\n",GetEncryptType(pAdapter->ScanTab.BssEntry[i].WepStatus));
    }

    wrq->u.data.length = strlen(msg);
    copy_to_user(wrq->u.data.pointer, msg, wrq->u.data.length);
	kfree(msg);
}

VOID RTMPIoctlGetMacTable(
	IN PRTMP_ADAPTER pAd, 
	IN struct iwreq *wrq)
{
	INT						i;
	RT_802_11_MAC_TABLE		MacTab;

	MacTab.Num = 0;
	for (i=0; i<MAX_LEN_OF_MAC_TABLE; i++)
	{
		if (pAd->MacTab.Content[i].Valid && (pAd->MacTab.Content[i].Sst == SST_ASSOC))
		{
			COPY_MAC_ADDR(MacTab.Entry[MacTab.Num].Addr, &pAd->MacTab.Content[i].Addr);
			MacTab.Entry[MacTab.Num].Aid = (UCHAR)pAd->MacTab.Content[i].Aid;
			MacTab.Entry[MacTab.Num].Psm = pAd->MacTab.Content[i].PsMode;
			MacTab.Entry[MacTab.Num].HSCounter.LastDataPacketTime = pAd->MacTab.Content[i].HSCounter.LastDataPacketTime;
			MacTab.Entry[MacTab.Num].HSCounter.TotalRxByteCount = pAd->MacTab.Content[i].HSCounter.TotalRxByteCount;
			MacTab.Entry[MacTab.Num].HSCounter.TotalTxByteCount = pAd->MacTab.Content[i].HSCounter.TotalTxByteCount;
			MacTab.Num += 1;
		}
	}    
	wrq->u.data.length = sizeof(RT_802_11_MAC_TABLE);
	copy_to_user(wrq->u.data.pointer, &MacTab, wrq->u.data.length);
}

#ifdef RALINK_ATE
UCHAR TemplateFrame[24] = {0x08,0x00,0x00,0x00,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x00,0xAA,0xBB,0x12,0x34,0x56,0x00,0x11,0x22,0xAA,0xBB,0xCC,0x00,0x00};	// 802.11 MAC Header, Type:Data, Length:24bytes 

/*
    ==========================================================================
    Description:
        Set ATE operation mode to
        0. APSTOP  = Stop AP Mode
        1. APSTART = Start AP Mode
        2. TXCONT  = Continuous Transmit
        3. TXCARR  = Transmit Carrier
        4. TXFRAME = Transmit Frames
        5. RXFRAME = Receive Frames
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_ATE_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	UCHAR			BbpData;
	ULONG			MacData;
	PTXD_STRUC		pTxD;
	PUCHAR			pDest;
	UINT			i, j, atemode;

#ifdef	BIG_ENDIAN
    PTXD_STRUC      pDestTxD;
    TXD_STRUC       TxD;
#endif

	ATEAsicSwitchChannel(pAd, pAd->ate.Channel);
	AsicLockChannel(pAd, pAd->ate.Channel);
	RTMPusecDelay(5000);

	RTMP_BBP_IO_READ8_BY_REG_ID(pAd, 63, &BbpData);
	RTMP_IO_READ32(pAd, PHY_CSR1, &MacData);

	BbpData = 0;
	MacData &= 0xFFFFFFFE;

	if (!strcmp(arg, "APSTOP")) 
	{						
		DBGPRINT(RT_DEBUG_TRACE, "ATE: APSTOP\n");
		
		atemode = pAd->ate.Mode;
		pAd->ate.Mode = ATE_APSTOP;
		pAd->ate.TxDoneCount = pAd->ate.TxCount;
		
		// we should free some resource which allocate when ATE_TXFRAME
		if (atemode == ATE_TXFRAME)
		{
			RTMP_IO_WRITE32(pAd, TX_CNTL_CSR, 0x000f0000);      // abort all TX rings
			RTMPusecDelay(5000);

			for (i=0; i<TX_RING_SIZE; i++)
			{
				PRTMP_DMABUF    pDmaBuf = &pAd->TxRing[QID_AC_BE].Cell[i].DmaBuf;
				pTxD = (PTXD_STRUC)pAd->TxRing[QID_AC_BE].Cell[i].AllocVa;
#ifndef BIG_ENDIAN
			    pTxD = (PTXD_STRUC)pAd->TxRing[QID_AC_BE].Cell[i].AllocVa;
#else
        		pDestTxD = (PTXD_STRUC)pAd->TxRing[QID_AC_BE].Cell[i].AllocVa;
        		TxD = *pDestTxD;
        		pTxD = &TxD;
        		RTMPDescriptorEndianChange((PUCHAR)pTxD, TYPE_TXD);
#endif
				pTxD->Owner = DESC_OWN_NIC;
#ifdef BIG_ENDIAN
				RTMPDescriptorEndianChange((PUCHAR)pTxD, TYPE_TXD);
				WriteBackToDescriptor((PUCHAR)pDestTxD, (PUCHAR)pTxD, FALSE, TYPE_TXD);
#endif
				if (pDmaBuf->pSkb)
				{
					// Sam M pci_unmap_single(pAd->pPci_Dev, pTxD->BufPhyAddr1, pTxD->BufLen1, PCI_DMA_TODEVICE);
					RELEASE_NDIS_PACKET(pAd, pDmaBuf->pSkb);
					pDmaBuf->pSkb = NULL;
				}
			}
		}
		
    	RTMP_BBP_IO_WRITE8_BY_REG_ID(pAd, 63, BbpData);
    	RTMP_IO_WRITE32(pAd, PHY_CSR1, MacData);

		netif_stop_queue(pAd->net_dev);
		ApStop(pAd);
		RTMP_IO_WRITE32(pAd, TXRX_CSR0, 0x0257b032);        // Disable Rx
	}
	else if (!strcmp(arg, "APSTART")) 
	{						
		DBGPRINT(RT_DEBUG_TRACE, "ATE: APSTART\n");
		
		RTMP_BBP_IO_WRITE8_BY_REG_ID(pAd, 63, BbpData);
		RTMP_IO_WRITE32(pAd, PHY_CSR1, MacData);
		
		pAd->ate.Mode = ATE_APSTART;

		RTMP_IO_WRITE32(pAd, TXRX_CSR0, 0x0256b032);       // enable RX of MAC block
		netif_start_queue(pAd->net_dev);

		ApStartUp(pAd);
	}
	else if (!strcmp(arg, "TXCONT")) 		// Continuous Tx
	{						
		DBGPRINT(RT_DEBUG_TRACE, "ATE: TXCONT\n");
		
		pAd->ate.Mode = ATE_TXCONT;

		BbpData |= 0x80;
		MacData |= 0x00000001;

		RTMP_BBP_IO_WRITE8_BY_REG_ID(pAd, 63, BbpData);
		RTMP_IO_WRITE32(pAd, PHY_CSR1, MacData);

		RTMP_IO_WRITE32(pAd, TXRX_CSR0, 0x0257b032);        // Disable Rx
	}
	else if (!strcmp(arg, "TXCARR"))			// Tx Carrier
	{
		DBGPRINT(RT_DEBUG_TRACE, "ATE: TXCARR\n");
		pAd->ate.Mode = ATE_TXCARR;

		BbpData |= 0x40;
		MacData |= 0x00000001;

		RTMP_BBP_IO_WRITE8_BY_REG_ID(pAd, 63, BbpData);
		RTMP_IO_WRITE32(pAd, PHY_CSR1, MacData);

		RTMP_IO_WRITE32(pAd, TXRX_CSR0, 0x0257b032);        // Disable Rx
	}
	else if (!strcmp(arg, "TXFRAME"))			// Tx Frames
	{						
		DBGPRINT(RT_DEBUG_TRACE, "ATE: TXFRAME(Count=%d)\n", pAd->ate.TxCount);
		pAd->ate.Mode = ATE_TXFRAME;

		RTMP_BBP_IO_WRITE8_BY_REG_ID(pAd, 63, BbpData);
		RTMP_IO_WRITE32(pAd, PHY_CSR1, MacData);

		pAd->ate.TxDoneCount = 0;
		
		for (i = 0; (i < TX_RING_SIZE) && (i < pAd->ate.TxCount); i++)
		{
			struct sk_buff	*pSkb;
			PRTMP_TX_RING   pTxRing = &pAd->TxRing[QID_AC_BE];
			PRTMP_DMABUF    pDmaBuf = &pAd->TxRing[QID_AC_BE].Cell[pTxRing->CurTxIndex].DmaBuf;
			PUCHAR			pDestBufVA = (PUCHAR) pTxRing->Cell[pTxRing->CurTxIndex].DmaBuf.AllocVa;

			if (!pDmaBuf->pSkb)
			{
				pSkb = __dev_alloc_skb(LOCAL_TXBUF_SIZE, MEM_ALLOC_FLAG);
				RTMP_SET_PACKET_SOURCE(pSkb, PKTSRC_DRIVER);

				pDmaBuf->pSkb = pSkb;
			}
			else
				pSkb = pDmaBuf->pSkb;
		
#ifndef BIG_ENDIAN
			pTxD = (PTXD_STRUC)pAd->TxRing[QID_AC_BE].Cell[pTxRing->CurTxIndex].AllocVa;
#else
        	pDestTxD = (PTXD_STRUC)pAd->TxRing[QID_AC_BE].Cell[pTxRing->CurTxIndex].AllocVa;
        	TxD = *pDestTxD;
        	pTxD = &TxD;
        	RTMPDescriptorEndianChange((PUCHAR)pTxD, TYPE_TXD);
#endif
			pDest = (PUCHAR) pSkb->data;
			pSkb->len = pAd->ate.TxLength - LENGTH_802_11;
		
			// Prepare frame payload
			memcpy(pDestBufVA, &TemplateFrame, LENGTH_802_11);
			memcpy(&pDestBufVA[4], pAd->ate.Addr1, ETH_LENGTH_OF_ADDRESS);
			memcpy(&pDestBufVA[10], pAd->ate.Addr2, ETH_LENGTH_OF_ADDRESS);
			memcpy(&pDestBufVA[16], pAd->ate.Addr3, ETH_LENGTH_OF_ADDRESS);
			
			pTxD->BufLen0		= LENGTH_802_11;
			pTxD->BufLen1		= pAd->ate.TxLength - LENGTH_802_11;
			pTxD->BufPhyAddr1	= pci_map_single(pAd->pPci_Dev, pSkb->data, pSkb->len, PCI_DMA_TODEVICE);
			pTxD->BufCount		= 2;
			
			for(j = 0; j < (pAd->ate.TxLength - LENGTH_802_11); j++)
				pDest[j] = 0xA5;

#ifdef BIG_ENDIAN
			RTMPDescriptorEndianChange((PUCHAR)pTxD, TYPE_TXD);
		    *pDestTxD = TxD;
		    pTxD = pDestTxD;
#endif			
			RTMPWriteTxDescriptor(pAd, pTxD, CIPHER_NONE, 0, 0, FALSE, FALSE, FALSE, SHORT_RETRY, 
				IFS_BACKOFF, pAd->ate.TxRate, pAd->ate.TxLength, QID_AC_BE, 0, pSkb, NULL, FALSE);

			INC_RING_INDEX(pTxRing->CurTxIndex, TX_RING_SIZE);
		}
		pAd->ate.TxDoneCount += i;

		RTMP_IO_WRITE32(pAd, TX_CNTL_CSR, BIT8[QID_AC_BE]);
		
		RTMP_IO_WRITE32(pAd, TXRX_CSR0, 0x0257b032);        // Disable Rx
	}
	else if (!strcmp(arg, "RXFRAME")) 			// Rx Frames
	{						
		DBGPRINT(RT_DEBUG_TRACE, "ATE: RXFRAME\n");

    	RTMP_BBP_IO_WRITE8_BY_REG_ID(pAd, 63, BbpData);
    	RTMP_IO_WRITE32(pAd, PHY_CSR1, MacData);
    	
		pAd->ate.Mode = ATE_RXFRAME;
		
		RTMP_IO_WRITE32(pAd, TX_CNTL_CSR, 0x000f0000);      // abort all TX rings
		RTMP_IO_WRITE32(pAd, TXRX_CSR0, 0x0256b032);       // enable RX of MAC block
	}
	else
	{	
		DBGPRINT(RT_DEBUG_TRACE, "ATE: Invalid arg!\n");
		return FALSE;
	}
	RTMPusecDelay(5000);

	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set ATE ADDR1=DA for TxFrame
        
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_ATE_DA_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	CHAR				*value;
	INT					i;
	
	if(strlen(arg) != 17)  //Mac address acceptable format 01:02:03:04:05:06 length 17
		return FALSE;

    for (i=0, value = strtok(arg,":"); value; value = strtok(NULL,":")) 
	{
		if((strlen(value) != 2) || (!isxdigit(*value)) || (!isxdigit(*(value+1))) ) 
			return FALSE;  //Invalid

		AtoH(value, &pAd->ate.Addr1[i++], 2);
	}

	if(i != 6)
		return FALSE;  //Invalid
		
	DBGPRINT(RT_DEBUG_TRACE, "Set_ATE_DA_Proc (DA = %2X:%2X:%2X:%2X:%2X:%2X)\n", pAd->ate.Addr1[0],
		pAd->ate.Addr1[1], pAd->ate.Addr1[2], pAd->ate.Addr1[3], pAd->ate.Addr1[4], pAd->ate.Addr1[5]);
	
	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set ATE ADDR2=SA for TxFrame
        
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_ATE_SA_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	CHAR				*value;
	INT					i;
	
	if(strlen(arg) != 17)  //Mac address acceptable format 01:02:03:04:05:06 length 17
		return FALSE;

    for (i=0, value = strtok(arg,":"); value; value = strtok(NULL,":")) 
	{
		if((strlen(value) != 2) || (!isxdigit(*value)) || (!isxdigit(*(value+1))) ) 
			return FALSE;  //Invalid

		AtoH(value, &pAd->ate.Addr2[i++], 2);
	}

	if(i != 6)
		return FALSE;  //Invalid

	DBGPRINT(RT_DEBUG_TRACE, "Set_ATE_SA_Proc (DA = %2X:%2X:%2X:%2X:%2X:%2X)\n", pAd->ate.Addr2[0],
		pAd->ate.Addr2[1], pAd->ate.Addr2[2], pAd->ate.Addr2[3], pAd->ate.Addr2[4], pAd->ate.Addr2[5]);
	
	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set ATE ADDR3=BSSID for TxFrame

    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_ATE_BSSID_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	CHAR				*value;
	INT					i;
	
	if(strlen(arg) != 17)  //Mac address acceptable format 01:02:03:04:05:06 length 17
		return FALSE;

    for (i=0, value = strtok(arg,":"); value; value = strtok(NULL,":")) 
	{
		if((strlen(value) != 2) || (!isxdigit(*value)) || (!isxdigit(*(value+1))) ) 
			return FALSE;  //Invalid

		AtoH(value, &pAd->ate.Addr3[i++], 2);
	}

	if(i != 6)
		return FALSE;  //Invalid

	DBGPRINT(RT_DEBUG_TRACE, "Set_ATE_BSSID_Proc (DA = %2X:%2X:%2X:%2X:%2X:%2X)\n",	pAd->ate.Addr3[0],
		pAd->ate.Addr3[1], pAd->ate.Addr3[2], pAd->ate.Addr3[3], pAd->ate.Addr3[4], pAd->ate.Addr3[5]);
	
	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set ATE Tx Length

    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_ATE_CHANNEL_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	pAd->ate.Channel = simple_strtol(arg, 0, 10);

	if((pAd->ate.Channel < 1) || (pAd->ate.Channel > 200))// to allow A band channel
	{
		pAd->ate.Channel = 1;
		DBGPRINT(RT_DEBUG_ERROR, "Set_ATE_CHANNEL_Proc::Out of range\n");
		return FALSE;
	}

	DBGPRINT(RT_DEBUG_TRACE, "Set_ATE_CHANNEL_Proc (ATE Channel = %d)\n", pAd->ate.Channel);
	
	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set ATE Tx Power
        
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_ATE_TX_POWER_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	ULONG R3;
	
	pAd->ate.TxPower = simple_strtol(arg, 0, 10);

	if(pAd->ate.TxPower >= 32)
	{
		DBGPRINT(RT_DEBUG_ERROR, "Set_ATE_TX_POWER_Proc::Out of range\n");
		return FALSE;
	}

	R3 = pAd->ate.TxPower << 9;		// shift TX power control to correct RF register bit position
	R3 |= (pAd->LatchRfRegs.R3 & 0xffffc1ff);
	pAd->LatchRfRegs.R3 = R3;

	RTMP_RF_IO_WRITE32(pAd, pAd->LatchRfRegs.R1);
	RTMP_RF_IO_WRITE32(pAd, pAd->LatchRfRegs.R2);
	RTMP_RF_IO_WRITE32(pAd, pAd->LatchRfRegs.R3);
	RTMP_RF_IO_WRITE32(pAd, pAd->LatchRfRegs.R4);

	DBGPRINT(RT_DEBUG_TRACE, "Set_ATE_TX_POWER_Proc (TxPower = %d)\n", pAd->ate.TxPower);
	
	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set ATE RF frequence offset
        
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_ATE_TX_FREQOFFSET_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	ULONG R4;
	
	pAd->ate.RFFreqOffset = simple_strtol(arg, 0, 10);

	if(pAd->ate.RFFreqOffset >= 64)
	{
		DBGPRINT(RT_DEBUG_ERROR, "Set_ATE_TX_FREQOFFSET_Proc::Out of range\n");
		return FALSE;
	}

	R4 = pAd->ate.RFFreqOffset << 12;		// shift TX power control to correct RF register bit position
	R4 |= (pAd->LatchRfRegs.R4 & (~0x0003f000));
	pAd->LatchRfRegs.R4 = R4;
	
	RTMP_RF_IO_WRITE32(pAd, pAd->LatchRfRegs.R1);
	RTMP_RF_IO_WRITE32(pAd, pAd->LatchRfRegs.R2);
	RTMP_RF_IO_WRITE32(pAd, pAd->LatchRfRegs.R4);
	RTMP_RF_IO_WRITE32(pAd, pAd->LatchRfRegs.R4);

	DBGPRINT(RT_DEBUG_TRACE, "Set_ATE_TX_FREQOFFSET_Proc (RFFreqOffset = %d)\n", pAd->ate.RFFreqOffset);
	
	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set ATE Tx frame length
        
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_ATE_TX_LENGTH_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	pAd->ate.TxLength = simple_strtol(arg, 0, 10);

	if((pAd->ate.TxLength < 24) || (pAd->ate.TxLength > 1500))
	{
		pAd->ate.TxLength = 1500;
		DBGPRINT(RT_DEBUG_ERROR, "Set_ATE_TX_LENGTH_Proc::Out of range\n");
		return FALSE;
	}

	DBGPRINT(RT_DEBUG_TRACE, "Set_ATE_TX_LENGTH_Proc (TxLength = %d)\n", pAd->ate.TxLength);
	
	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set ATE Tx frame count
        
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_ATE_TX_COUNT_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	pAd->ate.TxCount = simple_strtol(arg, 0, 10);

	DBGPRINT(RT_DEBUG_TRACE, "Set_ATE_TX_COUNT_Proc (TxCount = %d)\n", pAd->ate.TxCount);
	
	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set ATE Tx frame rate
        
        Return:
        	TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_ATE_TX_RATE_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
	pAd->ate.TxRate = simple_strtol(arg, 0, 10);

	if(pAd->ate.TxRate > RATE_54)
	{
		pAd->ate.TxRate = RATE_11;
		DBGPRINT(RT_DEBUG_ERROR, "Set_ATE_TX_RATE_Proc::Out of range\n");
		return FALSE;
	}

	DBGPRINT(RT_DEBUG_TRACE, "Set_ATE_TX_RATE_Proc (TxRate = %d)\n", pAd->ate.TxRate);
	
	return TRUE;
}

/*
    ==========================================================================
    Description:
    ==========================================================================
 */
VOID ATEAsicSwitchChannel(
    IN PRTMP_ADAPTER pAd, 
    IN UCHAR Channel) 
{
	ULONG R3, R4;
    UCHAR index, BbpReg, Bbp94 = 0;
    RTMP_RF_REGS *RFRegTable;

    // Select antenna
    AsicAntennaSelect(pAd, Channel);
        
    R3 = pAd->ate.TxPower << 9; // shift TX power control to correct RF R3 bit position
    
	if (pAd->RFProgSeq == 0)
		RFRegTable = RF5225RegTable;
	else
		RFRegTable = RF5225RegTable_1;
    
    switch (pAd->RfIcType)
    {
        case RFIC_5225:
        case RFIC_5325:
        case RFIC_2527:
		case RFIC_2529:
            for (index = 0; index < NUM_OF_5225_CHNL; index++)
            {
                if (Channel == RFRegTable[index].Channel)
                {
                    R3 = R3 | (RFRegTable[index].R3 & 0xffffc1ff); // set TX power
                    R4 = (RFRegTable[index].R4 & (~0x0003f000)) | (pAd->RfFreqOffset << 12);

                    // Update variables
                    pAd->LatchRfRegs.Channel = Channel;
                    pAd->LatchRfRegs.R1 = RFRegTable[index].R1;
                    pAd->LatchRfRegs.R2 = RFRegTable[index].R2;
                    pAd->LatchRfRegs.R3 = R3;
                    pAd->LatchRfRegs.R4 = R4;
                    
                    // Set RF value
                    RTMP_RF_IO_WRITE32(pAd, pAd->LatchRfRegs.R1);
                    RTMP_RF_IO_WRITE32(pAd, pAd->LatchRfRegs.R2);
                    RTMP_RF_IO_WRITE32(pAd, pAd->LatchRfRegs.R3);
                    RTMP_RF_IO_WRITE32(pAd, pAd->LatchRfRegs.R4);
                    break;
                }
            }
            RTMP_BBP_IO_READ8_BY_REG_ID(pAd, BBP_R3, &BbpReg);
            if ((pAd->RfIcType == RFIC_5225) || (pAd->RfIcType == RFIC_2527))
                BbpReg &= 0xFE;    // b0=0 for none smart mode
            else
                BbpReg |= 0x01;    // b0=1 for smart mode
            RTMP_BBP_IO_WRITE8_BY_REG_ID(pAd, BBP_R3, BbpReg);

            if (Bbp94 > 0)
                RTMP_BBP_IO_WRITE8_BY_REG_ID(pAd, BBP_R94, Bbp94);
            break;

        default:
            break;
    }

	//
	// On 11A, We should delay and wait RF/BBP to be stable
	// and the appropriate time should be 1000 micro seconds 
	//
    if (Channel > 14)
    	RTMPusecDelay(1000);
    
    DBGPRINT(RT_DEBUG_TRACE, "ATEAsicSwitchChannel(RF=%d, Pwr=%d) to #%d, R1=0x%08x, R2=0x%08x, R3=0x%08x, R4=0x%08x\n",
        pAd->RfIcType, 
        (R3 & 0x00003e00) >> 9,
        Channel, 
        pAd->LatchRfRegs.R1, 
        pAd->LatchRfRegs.R2, 
        pAd->LatchRfRegs.R3, 
        pAd->LatchRfRegs.R4);
}

#endif	// RALINK_ATE

// Sam add for Billion 

/* 
    ==========================================================================
    Description:
        Set Add WDS List
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_AddWdsList_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)

{
	UCHAR				macAddr[MAC_ADDR_LEN];
	CHAR				*this_char;
	CHAR				*value;
	INT				i, j,free_entry_id;
	BOOLEAN				find_it=FALSE,free_entry=FALSE;

//printk("Command AddWdsList\n");


	while ((this_char = strsep((char **)&arg, ";")) != NULL) 
	{
		if(strlen(this_char) != 17)  //Mac address acceptable format 01:02:03:04:05:06 length 17
			return FALSE;

		for (i=0, value = strtok(this_char,":"); value; value = strtok(NULL,":")) 
		{
			if((strlen(value) != 2) || (!isxdigit(*value)) || (!isxdigit(*(value+1))) ) 
				return FALSE;  //Invalid
			AtoH(value, &macAddr[i++], 2);
		}

		if(i != 6)
			return FALSE;  //Invalid

		#if 0
		//printk("Sam_A AddWdsList MAC ADDRESS : 0x");
		for (i=0 ; i < ETH_LENGTH_OF_ADDRESS; i++)
		printk("%02X ",macAddr[i]);
		printk("\n");
		#endif

		//Check is duplicate
		find_it  = FALSE;
		free_entry = FALSE;

		for(j = 0; j < MAX_WDS_ENTRY; j++)
		{
			if(memcmp(&pAd->WdsTab.WdsEntry[j].WdsAddr[0], &macAddr, ETH_LENGTH_OF_ADDRESS) == 0) // find it, enable it
			{
				find_it = TRUE;
				pAd->WdsTab.WdsEntry[j].Valid = TRUE;
                        	pAd->WdsTab.WdsEntry[j].RestrictEntry = TRUE;
				// do nothing
				break;  // find it and leave it anyway 
			}
			if(pAd->WdsTab.WdsEntry[j].Valid == FALSE && free_entry == FALSE)
			{
				free_entry_id = j;
				free_entry = TRUE;
			}
		}
		
		if( find_it == FALSE )  // not find item, try record it and enable it
		{
			if (free_entry == TRUE) // found free entry to record it
			{
                        	memcpy(&pAd->WdsTab.WdsEntry[free_entry_id].WdsAddr[0], &macAddr, ETH_LENGTH_OF_ADDRESS);
				pAd->WdsTab.WdsEntry[free_entry_id].Valid = TRUE;
                        	pAd->WdsTab.WdsEntry[free_entry_id].RestrictEntry = TRUE;
			}
			else
				return FALSE; // no more entry to record it
		}

	}
	// Sam add
	if(pAd->WdsTab.Mode == WDS_DISABLE_MODE)
		pAd->WdsTab.Mode = WDS_RESTRICT_MODE;
	// End Sam 
	return TRUE;
}



/* 
    ==========================================================================
    Description:
        Set Del WDS List
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_DelWdsList_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)

{
	UCHAR				macAddr[MAC_ADDR_LEN];
	UCHAR				NullMAC[MAC_ADDR_LEN]={0x00,0x00,0x00,0x00,0x00,0x00};
	CHAR				*this_char;
	CHAR				*value;
	INT				i, j,free_entry_id;
	BOOLEAN				find_it=FALSE;

//printk("Command DelWdsList\n");


	while ((this_char = strsep((char **)&arg, ";")) != NULL) 
	{
		if(strlen(this_char) != 17)  //Mac address acceptable format 01:02:03:04:05:06 length 17
			return FALSE;

		for (i=0, value = strtok(this_char,":"); value; value = strtok(NULL,":")) 
		{
			if((strlen(value) != 2) || (!isxdigit(*value)) || (!isxdigit(*(value+1))) ) 
				return FALSE;  //Invalid
			AtoH(value, &macAddr[i++], 2);
		}

		if(i != 6)
			return FALSE;  //Invalid

		#if 0
		//printk("Sam_A DelWdsList MAC ADDRESS : 0x");
		for (i=0 ; i < ETH_LENGTH_OF_ADDRESS; i++)
		printk("%02X ",macAddr[i]);
		printk("\n");
		#endif

		//Check is duplicate
		find_it  = FALSE;

		for(j = 0; j < MAX_WDS_ENTRY; j++)
		{
			if(memcmp(&pAd->WdsTab.WdsEntry[j].WdsAddr[0], &macAddr, ETH_LENGTH_OF_ADDRESS) == 0) // find it, disable it
			{
				find_it = TRUE;
				pAd->WdsTab.WdsEntry[j].Valid = FALSE;
                        	pAd->WdsTab.WdsEntry[j].RestrictEntry = FALSE;
	                       	memcpy(&pAd->WdsTab.WdsEntry[j].WdsAddr[0], &NullMAC, ETH_LENGTH_OF_ADDRESS);
				// do nothing
				break;  // find it and leave it anyway 
			}
		}
		
		//if( find_it == FALSE )  // not find item, try record it and enable it
			//printk("Sam_A not find WdsList to delete it \n");
	}
	return TRUE;
}


/* 
    ==========================================================================
    Description:
        Set WDS Encryp Type
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_WdsEncrypType_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)

{

//printk("Command WdsEncrypType\n");

	if ((strcmp(arg, "NONE") == 0) || (strcmp(arg, "none") == 0))
		pAd->WdsTab.WepStatus = Ndis802_11WEPDisabled;
	else if ((strcmp(arg, "WEP") == 0) || (strcmp(arg, "wep") == 0))
		pAd->WdsTab.WepStatus = Ndis802_11WEPEnabled;
	else if ((strcmp(arg, "TKIP") == 0) || (strcmp(arg, "tkip") == 0))
		pAd->WdsTab.WepStatus = Ndis802_11Encryption2Enabled;
	else if ((strcmp(arg, "AES") == 0) || (strcmp(arg, "aes") == 0))
		pAd->WdsTab.WepStatus = Ndis802_11Encryption3Enabled;
	else
		return FALSE;
	#if 0 
        printk("Sam_A Set_WdsEncrypType=%d\n", pAd->WdsTab.WepStatus);
	#endif
	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set WDS Key
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_WdsKey_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR		arg)
{
	
	UCHAR       keyMaterial[40];
int i;
	
//printk("Command WdsKey\n");

	NdisZeroMemory(&pAd->WdsTab.Wpa_key, sizeof(CIPHER_KEY));

	if (((pAd->WdsTab.WepStatus == Ndis802_11Encryption2Enabled) || (pAd->WdsTab.WepStatus == Ndis802_11Encryption3Enabled))
                                       && (strlen(arg) >= 8) && (strlen(arg) <= 64))
	{
		PasswordHash((char *)arg, RALINK_PASSPHRASE, sizeof(RALINK_PASSPHRASE), keyMaterial);
                                                                                                                                                                                  
		if (pAd->WdsTab.WepStatus == Ndis802_11Encryption3Enabled)
			pAd->WdsTab.Wpa_key.CipherAlg = CIPHER_AES;
                else
			pAd->WdsTab.Wpa_key.CipherAlg = CIPHER_TKIP;

			NdisMoveMemory(&pAd->WdsTab.Wpa_key.Key, keyMaterial, 16);
			pAd->WdsTab.Wpa_key.KeyLen = 16;
			NdisMoveMemory(&pAd->WdsTab.Wpa_key.RxMic, keyMaterial+16, 8);
			NdisMoveMemory(&pAd->WdsTab.Wpa_key.TxMic, keyMaterial+16, 8);
                                                                                                                                                                                    
#if 0 
			//printk("Sam_A set_WdsKey_Proc=>\n");
			for (i=0; i<16; i++)
                        printk("%02x:", pAd->WdsTab.Wpa_key.Key[i]);
                        printk("\nRxMic Key=>\n");
                        for (i=0; i<8; i++)
                        printk("%02x:", pAd->WdsTab.Wpa_key.RxMic[i]);
                        printk("\nTxMic Key=>\n");
                        for (i=0; i<8; i++)
                        printk("%02x:", pAd->WdsTab.Wpa_key.TxMic[i]);
                        printk("\n");
#endif
	}
	else 
		return FALSE;
	return TRUE;
}


/* 
    ==========================================================================
    Description:
        Set WDS Enable
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_WdsEnable_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR		arg)
{
int i;
//printk("Command WdsEnable\n");

	switch (simple_strtol(arg, 0, 10))
	{
		case 1: // Restricted mode
			pAd->WdsTab.Mode = WDS_RESTRICT_MODE;
			break;
		case 2: // Bridge mode, DisAllow association(stop Beacon generation and Probe Req.)
			pAd->WdsTab.Mode = WDS_BRIDGE_MODE;
			break;
		case 3: // Repeater mode
			pAd->WdsTab.Mode = WDS_REPEATER_MODE;
			break;
		case 4: // Lazy mode, Auto learn wds entry by same SSID, channel, security policy
			for(i = 0; i < MAX_WDS_ENTRY; i++)
			{
				pAd->WdsTab.WdsEntry[i].Valid = FALSE;
			}
			pAd->WdsTab.Mode = WDS_LAZY_MODE;
			break;
		// Sam Add Show information
		case 5:
			printk("WDS Mode=%d\n",pAd->WdsTab.Mode);
			for(i = 0; i < MAX_WDS_ENTRY; i++)
			{
				printk("WdsEntry.[%d].Valid=%d\n",i,pAd->WdsTab.WdsEntry[i].Valid);
				printk("WdsEntry.[%d].WdsAddr=%02x:%02x:%02x:%02x:%02x:%02x\n",i,pAd->WdsTab.WdsEntry[i].WdsAddr[0],pAd->WdsTab.WdsEntry[i].WdsAddr[1],pAd->WdsTab.WdsEntry[i].WdsAddr[2],pAd->WdsTab.WdsEntry[i].WdsAddr[3],pAd->WdsTab.WdsEntry[i].WdsAddr[4],pAd->WdsTab.WdsEntry[i].WdsAddr[5]);
				printk("WdsEntry.[%d]->dev=%08x\n",i,pAd->WdsTab.WdsEntry[i].dev);
			}
			break;
		// End Sam add
		case 0: // Disable mode
		default:
			//NdisZeroMemory(&pAd->WdsTab, sizeof(RT_802_11_WDS_TABLE));
			for(i = 0; i < MAX_WDS_ENTRY; i++)
			{
				pAd->WdsTab.WdsEntry[i].Valid = FALSE;
			}
			pAd->WdsTab.Mode = WDS_DISABLE_MODE;
			break;
	}
	return TRUE;
}

/* 
    ==========================================================================
    Description:
        Set AP Enable
    Return:
        TRUE if all parameters are OK, FALSE otherwise
    ==========================================================================
*/
INT	Set_APEnable_Proc(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR		arg)
{
#if 0  // Sam: Its not implement finish ...
	unsigned long command;
	command = simple_strtol(arg,0,10);
	


	if (command == 0)  // disable AP function
	{
		UCHAR			BbpData;
		ULONG			MacData;
		PTXD_STRUC		pTxD;
		PUCHAR			pDest;
		UINT			i, j, atemode;

#ifdef	BIG_ENDIAN
		PTXD_STRUC      pDestTxD;
		TXD_STRUC       TxD;
#endif


		RTMP_BBP_IO_READ8_BY_REG_ID(pAd, 63, &BbpData);
		RTMP_IO_READ32(pAd, PHY_CSR1, &MacData);
	
		BbpData = 0;
		MacData &= 0xFFFFFFFE;


		//atemode = pAd->ate.Mode;
		//pAd->ate.Mode = ATE_APSTOP;
		//pAd->ate.TxDoneCount = pAd->ate.TxCount;
		
		// we should free some resource which allocate when ATE_TXFRAME
		//if (atemode == ATE_TXFRAME)
		//{
			RTMP_IO_WRITE32(pAd, TX_CNTL_CSR, 0x000f0000);      // abort all TX rings
			//RTMPusecDelay(5000);

			for (i=0; i<TX_RING_SIZE; i++)
			{
				PRTMP_DMABUF    pDmaBuf = &pAd->TxRing[QID_AC_BE].Cell[i].DmaBuf;
				pTxD = (PTXD_STRUC)pAd->TxRing[QID_AC_BE].Cell[i].AllocVa;
#ifndef BIG_ENDIAN
			    pTxD = (PTXD_STRUC)pAd->TxRing[QID_AC_BE].Cell[i].AllocVa;
#else
        		pDestTxD = (PTXD_STRUC)pAd->TxRing[QID_AC_BE].Cell[i].AllocVa;
        		TxD = *pDestTxD;
        		pTxD = &TxD;
        		RTMPDescriptorEndianChange((PUCHAR)pTxD, TYPE_TXD);
#endif
				pTxD->Owner = DESC_OWN_NIC;
#ifdef BIG_ENDIAN
				RTMPDescriptorEndianChange((PUCHAR)pTxD, TYPE_TXD);
				WriteBackToDescriptor((PUCHAR)pDestTxD, (PUCHAR)pTxD, FALSE, TYPE_TXD);
#endif
				if (pDmaBuf->pSkb)
				{
					// Sam_M pci_unmap_single(pAd->pPci_Dev, pTxD->BufPhyAddr1, pTxD->BufLen1, PCI_DMA_TODEVICE);
					RELEASE_NDIS_PACKET(pAd, pDmaBuf->pSkb);
					pDmaBuf->pSkb = NULL;
				}
			}
		//}
		
	    	//RTMP_BBP_IO_WRITE8_BY_REG_ID(pAd, 63, 0);
	    	//RTMP_IO_WRITE32(pAd, PHY_CSR1, MacData);

		netif_stop_queue(pAd->net_dev);
		ApStop(pAd);
		//RTMP_IO_WRITE32(pAd, TXRX_CSR0, 0x0257b032);        // Disable Rx

		return TRUE;
	}
	else if(command == 1) // Enable AP dunction
	{
		netif_start_queue(pAd->net_dev);

		ApStartUp(pAd);

		return TRUE;
	}
	else
#endif
	return FALSE;
}
