/*
 ***************************************************************************
 * Ralink Tech Inc.
 * 4F, No. 2 Technology	5th	Rd.
 * Science-based Industrial	Park
 * Hsin-chu, Taiwan, R.O.C.
 *
 * (c) Copyright 2002-2005, Ralink Technology, Inc.
 *
 * All rights reserved.	Ralink's source	code is	an unpublished work	and	the
 * use of a	copyright notice does not imply	otherwise. This	source code
 * contains	confidential trade secret material of Ralink Tech. Any attempt
 * or participation	in deciphering,	decoding, reverse engineering or in	any
 * way altering	the	source code	is stricitly prohibited, unless	the	prior
 * written consent of Ralink Technology, Inc. is obtained.
 ***************************************************************************

	Module Name:
	soft_ap.c

	Abstract:

	Revision History:
	Who			When			What
	--------	----------		----------------------------------------------
	John		2004-08-08		Major modification from RT2560
*/
#include "rt_config.h"

char const *pEventText[EVENT_MAX_EVENT_TYPE] = {
    "restart access point",
    "successfully associated",
    "has disassociated",
    "has been aged-out and disassociated" ,    
    "active countermeasures",
    "has disassociated with invalid PSK password"};
    
/*
    ==========================================================================
    Description:
        Initialize AP specific data
    ==========================================================================
 */
NDIS_STATUS ApInitialize(
	IN	PRTMP_ADAPTER	pAd)
{
    UCHAR       apidx;

    DBGPRINT(RT_DEBUG_TRACE, "---> ApInitialize\n");

    // Initialize MAC table and allocate spin lock
    NdisZeroMemory(&pAd->MacTab, sizeof(MAC_TABLE));
    NdisAllocateSpinLock(&pAd->MacTabLock);

    for (apidx = 0; apidx < pAd->PortCfg.BssidNum; apidx++)
    {
        // Init TKIP Group-Key-related variables
        GenRandom(pAd,pAd->PortCfg.MBSSID[apidx].GMK, apidx);
        GenRandom(pAd,pAd->PortCfg.MBSSID[apidx].GNonce, apidx);

        pAd->PortCfg.MBSSID[apidx].PortSecured = WPA_802_1X_PORT_NOT_SECURED;
    }

    // Init Group key update timer, and countermeasures timer
    init_timer(&pAd->PortCfg.REKEYTimer);
    pAd->PortCfg.REKEYTimer.data = (unsigned long)pAd;
    pAd->PortCfg.REKEYTimer.function = &GREKEYPeriodicExec;

    init_timer(&pAd->PortCfg.CounterMeasureTimer);
    pAd->PortCfg.CounterMeasureTimer.data = (unsigned long)pAd;
    pAd->PortCfg.CounterMeasureTimer.function = &CMTimerExec;

    init_timer(&pAd->QuickResponeForRateUpTimer);
    pAd->QuickResponeForRateUpTimer.data = (unsigned long)pAd;
    pAd->QuickResponeForRateUpTimer.function = &QuickResponeForRateUp;

    DBGPRINT(RT_DEBUG_TRACE, "<--- ApInitialize\n");
    
	return NDIS_STATUS_SUCCESS;
}

/*
    ==========================================================================
    Description:
        Shutdown AP and free AP specific resources
    ==========================================================================
 */
VOID ApShutdown(
    IN PRTMP_ADAPTER pAd)
{
    DBGPRINT(RT_DEBUG_TRACE, "---> ApShutdown\n");
    ApStop(pAd);
    DBGPRINT(RT_DEBUG_TRACE, "<--- ApShutdown\n");
}

VOID ApStartUp(
    IN PRTMP_ADAPTER pAd)
{
	UCHAR       apidx, i;
	ULONG       sec_csr4 = 0;
	BOOLEAN		bWmmCapable = FALSE;

    DBGPRINT(RT_DEBUG_TRACE, "---> ApStartUp\n");

    for (apidx = 0; apidx < pAd->PortCfg.BssidNum; apidx++)
    {
        if ((pAd->PortCfg.MBSSID[apidx].SsidLen <= 0) || (pAd->PortCfg.MBSSID[apidx].SsidLen > MAX_LEN_OF_SSID))
            return;
        
        // when B-only mode, beacon will not report using short slot time
        if (pAd->PortCfg.PhyMode == PHY_11B)
        {
        	pAd->PortCfg.UseShortSlotTime = FALSE;
        }
        
        pAd->PortCfg.MBSSID[apidx].CapabilityInfo = CAP_GENERATE((pAd->PortCfg.MBSSID[apidx].WepStatus != Ndis802_11EncryptionDisabled), (pAd->PortCfg.TxPreamble == Rt802_11PreambleShort), pAd->PortCfg.UseShortSlotTime, pAd->PortCfg.bCountryFlag);

		// if one bssid turn on the WMM, the parameter will set to ASIC once.
        if (pAd->PortCfg.MBSSID[apidx].bWmmCapable)
        	bWmmCapable = TRUE;

        DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d) CapabilityInfo=%x, WepStatus=%d\n", apidx,
            pAd->PortCfg.MBSSID[apidx].CapabilityInfo,pAd->PortCfg.MBSSID[apidx].WepStatus);

        // if AuthMode >= WPA,then pairwise key table must be used; 
	    // if legacy WEP inused, then only shared key table is used
	    if (pAd->PortCfg.MBSSID[apidx].AuthMode >= Ndis802_11AuthModeWPA)
	    {
	    	sec_csr4 |= BIT32[apidx];
	        DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d)-AP AuthMode=%d, Pairwise Key Table in-used\n", apidx, pAd->PortCfg.MBSSID[apidx].AuthMode);
	    }
	    else
	    {
	        DBGPRINT(RT_DEBUG_TRACE, "IF(ra%d)-AP AuthMode=%d, disable Pairwise Key Table\n", apidx, pAd->PortCfg.MBSSID[apidx].AuthMode);
	    }
	    RTMP_IO_WRITE32(pAd, SEC_CSR4, sec_csr4);

	    // remove group key table
    	for (i=0; i<SHARE_KEY_NUM; i++)
    	{
			AsicRemoveSharedKeyEntry(pAd, apidx, i);
    	}

    	// remove pairwise key table
    	for (i=0; i<PAIRWISE_KEY_NO; i++)
    	{
			AsicRemovePairwiseKeyEntry(pAd, i);
    	}

		// set shared key to ASIC
		// Only SHARED-WEP and OPEN-WEP(802.1X=0) need to set key, OPEN-WEP(802.1x=1) should not set key
    	if ((pAd->PortCfg.MBSSID[apidx].AuthMode == Ndis802_11AuthModeShared) ||
    		(((pAd->PortCfg.MBSSID[apidx].AuthMode == Ndis802_11AuthModeAutoSwitch) || (pAd->PortCfg.MBSSID[apidx].AuthMode == Ndis802_11AuthModeOpen)) && (pAd->PortCfg.MBSSID[apidx].WepStatus == Ndis802_11Encryption1Enabled) && (pAd->PortCfg.MBSSID[apidx].IEEE8021X == FALSE)))
    	{
    		for (i=0; i<SHARE_KEY_NUM; i++)
	    	{
	    		if (pAd->PortCfg.MBSSID[apidx].SharedKey[i].KeyLen != 0)
	    		{
		    		AsicAddSharedKeyEntry(pAd, apidx, i, pAd->PortCfg.MBSSID[apidx].SharedKey[i].CipherAlg, pAd->PortCfg.MBSSID[apidx].SharedKey[i].Key, NULL, NULL);
	    		}
	    	}
    	}

        // Add wds key to pairwise key table when TKIP/AES
    	if (((pAd->WdsTab.WepStatus == Ndis802_11Encryption2Enabled) || (pAd->WdsTab.WepStatus == Ndis802_11Encryption3Enabled)) && (pAd->WdsTab.Wpa_key.KeyLen > 0))
    	{
			AsicAddPairwiseKeyEntry(pAd, pAd->WdsTab.WdsEntry[0].WdsAddr, 60, pAd->WdsTab.Wpa_key.CipherAlg, 
				pAd->WdsTab.Wpa_key.Key, pAd->WdsTab.Wpa_key.TxMic, pAd->WdsTab.Wpa_key.RxMic);
			AsicAddPairwiseKeyEntry(pAd, pAd->WdsTab.WdsEntry[1].WdsAddr, 61, pAd->WdsTab.Wpa_key.CipherAlg,	
				pAd->WdsTab.Wpa_key.Key, pAd->WdsTab.Wpa_key.TxMic, pAd->WdsTab.Wpa_key.RxMic);
			AsicAddPairwiseKeyEntry(pAd, pAd->WdsTab.WdsEntry[2].WdsAddr, 62, pAd->WdsTab.Wpa_key.CipherAlg,	
				pAd->WdsTab.Wpa_key.Key, pAd->WdsTab.Wpa_key.TxMic, pAd->WdsTab.Wpa_key.RxMic);
			AsicAddPairwiseKeyEntry(pAd, pAd->WdsTab.WdsEntry[3].WdsAddr, 63, pAd->WdsTab.Wpa_key.CipherAlg,	
				pAd->WdsTab.Wpa_key.Key, pAd->WdsTab.Wpa_key.TxMic, pAd->WdsTab.Wpa_key.RxMic);
    	}

		// Send singal to daemon to indicate driver had restarted
	    if ((pAd->PortCfg.MBSSID[apidx].AuthMode == Ndis802_11AuthModeWPA) || (pAd->PortCfg.MBSSID[apidx].AuthMode == Ndis802_11AuthModeWPA2)
        	|| (pAd->PortCfg.MBSSID[apidx].AuthMode == Ndis802_11AuthModeWPA1WPA2))
        {
        	SendSingalToDaemon(pAd);
    	}
    }

    if (bWmmCapable)
    {
        // EDCA parameters used for AP's own transmission
        if (pAd->PortCfg.APEdcaParm.bValid == FALSE)
        {
            pAd->PortCfg.APEdcaParm.bValid = TRUE;
            pAd->PortCfg.APEdcaParm.Aifsn[0] = 3;
            pAd->PortCfg.APEdcaParm.Aifsn[1] = 7;
            pAd->PortCfg.APEdcaParm.Aifsn[2] = 1;
            pAd->PortCfg.APEdcaParm.Aifsn[3] = 1;
                              
            pAd->PortCfg.APEdcaParm.Cwmin[0] = 4;
            pAd->PortCfg.APEdcaParm.Cwmin[1] = 4;
            pAd->PortCfg.APEdcaParm.Cwmin[2] = 3;
            pAd->PortCfg.APEdcaParm.Cwmin[3] = 2;
                                    
            pAd->PortCfg.APEdcaParm.Cwmax[0] = 10;
            pAd->PortCfg.APEdcaParm.Cwmax[1] = 6;
            pAd->PortCfg.APEdcaParm.Cwmax[2] = 4;
            pAd->PortCfg.APEdcaParm.Cwmax[3] = 3;
        
            pAd->PortCfg.APEdcaParm.Txop[0]  = 0;
            pAd->PortCfg.APEdcaParm.Txop[1]  = 0;
            pAd->PortCfg.APEdcaParm.Txop[2]  = 94;
            pAd->PortCfg.APEdcaParm.Txop[3]  = 47;
        }
        AsicSetEdcaParm(pAd, &pAd->PortCfg.APEdcaParm);

        // EDCA parameters to be annouced in outgoing BEACON, used by WMM STA
        if (pAd->PortCfg.BssEdcaParm.bValid == FALSE)
        {
            pAd->PortCfg.BssEdcaParm.bValid = TRUE;
            pAd->PortCfg.BssEdcaParm.Aifsn[0] = 3;
            pAd->PortCfg.BssEdcaParm.Aifsn[1] = 7;
            pAd->PortCfg.BssEdcaParm.Aifsn[2] = 2;
            pAd->PortCfg.BssEdcaParm.Aifsn[3] = 2;
                              
            pAd->PortCfg.BssEdcaParm.Cwmin[0] = 4;
            pAd->PortCfg.BssEdcaParm.Cwmin[1] = 4;
            pAd->PortCfg.BssEdcaParm.Cwmin[2] = 3;
            pAd->PortCfg.BssEdcaParm.Cwmin[3] = 2;
                                    
            pAd->PortCfg.BssEdcaParm.Cwmax[0] = 10;
            pAd->PortCfg.BssEdcaParm.Cwmax[1] = 10;
            pAd->PortCfg.BssEdcaParm.Cwmax[2] = 4;
            pAd->PortCfg.BssEdcaParm.Cwmax[3] = 3;
        
            pAd->PortCfg.BssEdcaParm.Txop[0]  = 0;
            pAd->PortCfg.BssEdcaParm.Txop[1]  = 0;
            pAd->PortCfg.BssEdcaParm.Txop[2]  = 94;
            pAd->PortCfg.BssEdcaParm.Txop[3]  = 47;
        }
        DBGPRINT(RT_DEBUG_TRACE,"STA        : AIFSN CWmin CWmax TXOP(us)  ACM\n");
        DBGPRINT(RT_DEBUG_TRACE,"    AC_BE     %d     %d   %d    %4d     %d\n",
            pAd->PortCfg.BssEdcaParm.Aifsn[0], pAd->PortCfg.BssEdcaParm.Cwmin[0], pAd->PortCfg.BssEdcaParm.Cwmax[0],
            pAd->PortCfg.BssEdcaParm.Txop[0]<<5, pAd->PortCfg.BssEdcaParm.bACM[0]);
        DBGPRINT(RT_DEBUG_TRACE,"    AC_BK     %d     %d   %d    %4d     %d\n",
            pAd->PortCfg.BssEdcaParm.Aifsn[1], pAd->PortCfg.BssEdcaParm.Cwmin[1], pAd->PortCfg.BssEdcaParm.Cwmax[1],
            pAd->PortCfg.BssEdcaParm.Txop[1]<<5, pAd->PortCfg.BssEdcaParm.bACM[1]);
        DBGPRINT(RT_DEBUG_TRACE,"    AC_VI     %d     %d     %d    %4d     %d\n",
            pAd->PortCfg.BssEdcaParm.Aifsn[2], pAd->PortCfg.BssEdcaParm.Cwmin[2], pAd->PortCfg.BssEdcaParm.Cwmax[2],
            pAd->PortCfg.BssEdcaParm.Txop[2]<<5, pAd->PortCfg.BssEdcaParm.bACM[2]);
        DBGPRINT(RT_DEBUG_TRACE,"    AC_VO     %d     %d     %d     %4d     %d\n",
            pAd->PortCfg.BssEdcaParm.Aifsn[3], pAd->PortCfg.BssEdcaParm.Cwmin[3], pAd->PortCfg.BssEdcaParm.Cwmax[3],
            pAd->PortCfg.BssEdcaParm.Txop[3]<<5, pAd->PortCfg.BssEdcaParm.bACM[3]);
    }

	AsicSetBssid(pAd, pAd->CurrentAddress); 
    AsicSwitchChannel(pAd, pAd->PortCfg.Channel);
    AsicLockChannel(pAd, pAd->PortCfg.Channel);
    MlmeUpdateTxRates(pAd);
	AsicSetSlotTime(pAd, (BOOLEAN)pAd->PortCfg.UseShortSlotTime);
    
	MakeAllBssBeacon(pAd);
	UpdateAllBeaconFrame(pAd);
	AsicSendCommandToMcu(pAd, 0x60, 0xff, 0x00, 0x00);   // send stop-RD command to MCU
	RTMPusecDelay(500000);
	if ((pAd->PortCfg.PhyMode == PHY_11A) && (pAd->PortCfg.RadarDetect.IEEE80211H == TRUE) && RadarChannelCheck(pAd, pAd->PortCfg.Channel))
	{
		pAd->PortCfg.RadarDetect.RDMode = RD_SILENCE_MODE;
		pAd->PortCfg.RadarDetect.RDCount = 0;
		pAd->PortCfg.RadarDetect.InServiceMonitorCount = 0;
		RadarDetectionStart(pAd);
	}
	else
	{
        AsicEnableBssSync(pAd);
	}
	
    // Set LED
	RTMPSetLED(pAd, LED_RADIO_ON);
	RTMPSetLED(pAd, LED_LINK_UP);

    // Group rekey related
    if ((pAd->PortCfg.WPAREKEY.ReKeyInterval != 0) && ((pAd->PortCfg.WPAREKEY.ReKeyMethod == TIME_REKEY) || (pAd->PortCfg.WPAREKEY.ReKeyMethod == PKT_REKEY))) 
    {
        // Regularly check the timer
        if (pAd->PortCfg.REKEYTimerRunning == FALSE)
        {
            pAd->PortCfg.REKEYTimer.expires = jiffies + GUPDATE_EXEC_INTV;
            add_timer(&pAd->PortCfg.REKEYTimer);
            pAd->PortCfg.REKEYTimerRunning = TRUE;
            pAd->PortCfg.REKEYCOUNTER = 0;
        }
    }
    else
    	pAd->PortCfg.REKEYTimerRunning = FALSE;

	pAd->Mlme.PeriodicRound = 0;

    ApLogEvent(pAd, pAd->CurrentAddress, EVENT_RESET_ACCESS_POINT, MAIN_MBSSID);

    DBGPRINT(RT_DEBUG_TRACE, "<--- ApStartUp (sec_csr4=0x%x)\n", sec_csr4);
}

VOID SendSingalToDaemon(
	IN PRTMP_ADAPTER pAd)
{
    PUCHAR					src;
    struct file				*srcf;
    INT 					retval, orgfsuid, orgfsgid;
    mm_segment_t			orgfs;
    CHAR					buffer[MAX_INI_BUFFER_SIZE];
    CHAR					tmpbuf[600];
    ULONG					ApdPid;

	src = PROFILE_PATH;

    // Save uid and gid used for filesystem access.
    // Set user and group to 0 (root)	
    orgfsuid = current->fsuid;
    orgfsgid = current->fsgid;
    current->fsuid=current->fsgid = 0;
    orgfs = get_fs();
    set_fs(KERNEL_DS);

    if (src && *src) 
    {
        srcf = filp_open(src, O_RDONLY, 0);
	    if (IS_ERR(srcf)) 
	    {
		    DBGPRINT(RT_DEBUG_ERROR, "--> Error %ld opening %s\n", -PTR_ERR(srcf),src);    
	    }
	    else 
	    {
		    // The object must have a read method
		    if (srcf->f_op && srcf->f_op->read) 
		    {
			    memset(buffer, 0x00, MAX_INI_BUFFER_SIZE);
			    retval=srcf->f_op->read(srcf, buffer, MAX_INI_BUFFER_SIZE, &srcf->f_pos);
			    if (retval < 0)
			    {
				    DBGPRINT(RT_DEBUG_ERROR, "--> Read %s error %d\n", src, -retval);
			    }
			    else
			    {
				    // set file parameter to portcfg
				    // pid
				    if(RTMPGetKeyParameter("pid", tmpbuf, 255, buffer))
				    {
					    ApdPid= (ULONG) simple_strtol(tmpbuf, 0, 10);
					    DBGPRINT(RT_DEBUG_TRACE, "ApdPid=%d\n", ApdPid);
                        kill_proc(ApdPid, SIGUSR1, 0);
				    }
			    }
            }
            else
            {
                DBGPRINT(RT_DEBUG_ERROR, "--> %s does not have a write method\n", src);
            }
	
		    retval=filp_close(srcf,NULL);
	
		    if (retval)
		    {
			    DBGPRINT(RT_DEBUG_ERROR, "--> Error %d closing %s\n", -retval, src);
		    }
        }
	}
    set_fs(orgfs);
    current->fsuid = orgfsuid;
    current->fsgid = orgfsgid;
}

VOID ApStop(
    IN PRTMP_ADAPTER pAd) 
{
	DBGPRINT(RT_DEBUG_TRACE, "!!! ApStop !!!\n");

    AsicDisableSync(pAd);
	RTMPSetLED(pAd, LED_RADIO_OFF);
    RTMPSetLED(pAd, LED_LINK_DOWN);

    if(pAd->PortCfg.REKEYTimerRunning==TRUE)
    {
        del_timer_sync(&pAd->PortCfg.REKEYTimer);
        pAd->PortCfg.REKEYTimerRunning=FALSE;
    }

    if(pAd->PortCfg.CMTimerRunning==TRUE)
    {
        del_timer_sync(&pAd->PortCfg.CounterMeasureTimer);
    	pAd->PortCfg.CMTimerRunning = FALSE;
    }

    MacTableReset(pAd);
}

/*
    ==========================================================================
    Description:
        This routine is used to clean up a specified power-saving queue. It's
        used whenever a wireless client is deleted.
    ==========================================================================
 */
VOID CleanupPsQueue(
    IN  PRTMP_ADAPTER   pAd,
    IN  PQUEUE_HEADER   pQueue)
{
    struct sk_buff	*pSkb;
    
    while (pQueue->Head)
    {
        pSkb = (struct sk_buff *)RemoveHeadQueue(pQueue);
        if(pSkb)		RELEASE_NDIS_PACKET(pAd, pSkb);
    }
}

/*
    ==========================================================================
    Description:
        This routine reset the entire MAC table. All packets pending in
        the power-saving queues are freed here.
    ==========================================================================
 */
VOID MacTableReset(
    IN  PRTMP_ADAPTER  pAd)
{
    INT				i;
    ULONG			IrqFlags;
    PCHAR        	pOutBuffer = NULL;
    ULONG       	FrameLen = 0;
    HEADER_802_11	DisassocHdr;
    USHORT      	Reason;

#ifndef TC_NO_SPIN_LOCK  // Sam add
    spin_lock_irqsave(&pAd->MacTabLock, IrqFlags);
#endif
    CleanupPsQueue(pAd, &pAd->MacTab.McastPsQueue);
    
    for (i=0; i<MAX_LEN_OF_MAC_TABLE; i++)
    {
        if (pAd->MacTab.Content[i].Valid)
       {
            if (pAd->MacTab.Content[i].RetryTimerRunning == TRUE)
            {
                DBGPRINT(RT_DEBUG_TRACE, "pAd->MacTab.Content[%d]  Cancel Retry Timer !\n",i);
                del_timer_sync(&pAd->MacTab.Content[i].RetryTimer);
                pAd->MacTab.Content[i].RetryTimerRunning = FALSE;             
            }
            CleanupPsQueue(pAd, &pAd->MacTab.Content[i].PsQueue);

            // Before reset MacTable, send disassociation packet to client.
            if (pAd->MacTab.Content[i].Sst == SST_ASSOC)
            {
                //  send out a DISASSOC request frame
                pOutBuffer = kmalloc(MAX_LEN_OF_MLME_BUFFER, MEM_ALLOC_FLAG);
                if(pOutBuffer == NULL)
                	return;

                Reason = REASON_DISASSOC_INACTIVE;
                DBGPRINT(RT_DEBUG_ERROR, "ASSOC - Send DISASSOC  Reason = %d frame  TO %x %x %x %x %x %x \n",Reason,pAd->MacTab.Content[i].Addr[0],
                    pAd->MacTab.Content[i].Addr[1],pAd->MacTab.Content[i].Addr[2],pAd->MacTab.Content[i].Addr[3],pAd->MacTab.Content[i].Addr[4],pAd->MacTab.Content[i].Addr[5]);
                MgtMacHeaderInit(pAd, &DisassocHdr, SUBTYPE_DISASSOC, 0, pAd->MacTab.Content[i].Addr, pAd->PortCfg.MBSSID[pAd->MacTab.Content[i].ApIdx].Bssid);
                MakeOutgoingFrame(pOutBuffer, &FrameLen, sizeof(HEADER_802_11), (CHAR *)&DisassocHdr, 2, (char*)&Reason, END_OF_ARGS);
                MiniportMMRequest(pAd, pOutBuffer, FrameLen);
                kfree(pOutBuffer);

                RTMPusecDelay(2000);
            }
        }
    }
    
    NdisZeroMemory(&pAd->MacTab, sizeof(MAC_TABLE));
    InitializeQueueHeader(&pAd->MacTab.McastPsQueue);

#ifndef TC_NO_SPIN_LOCK  // Sam add
    spin_unlock_irqrestore(&pAd->MacTabLock, IrqFlags);
#endif
}

/*
    ==========================================================================
    Description:
        Add and new entry into MAC table
    ==========================================================================
 */
MAC_TABLE_ENTRY *MacTableInsertEntry(
    IN	PRTMP_ADAPTER	pAd, 
    IN  PUCHAR        pAddr,
    IN  UCHAR           apidx) 
{
    UCHAR HashIdx;
    int i;
    MAC_TABLE_ENTRY *pEntry = NULL, *pCurrEntry;
    ULONG	IrqFlags;

    // if FULL, return
    if (pAd->MacTab.Size >= MAX_LEN_OF_MAC_TABLE) 
        return NULL;

    // allocate one MAC entry

#ifndef TC_NO_SPIN_LOCK  // Sam add
    spin_lock_irqsave(&pAd->MacTabLock, IrqFlags);
#endif
    for (i = 1; i< MAX_LEN_OF_MAC_TABLE; i++)		// skip entry#0 so that "entry index == AID" for fast lookup
    {
        // pick up the first available vacancy
        if (pAd->MacTab.Content[i].Valid == FALSE)
        {
            pEntry = &pAd->MacTab.Content[i];
            NdisZeroMemory(pEntry, sizeof(MAC_TABLE_ENTRY));
            pEntry->Valid = TRUE;

            // Move init entry retry timer to where it first created
            init_timer(&pEntry->RetryTimer);
            
            pEntry->RetryTimerRunning = FALSE;
            pEntry->CMTimerRunning = FALSE;
            pEntry->RSNIE_Len = 0;
            NdisMoveMemory(pEntry->R_Counter, pAd->PortCfg.R_Counter, sizeof(pEntry->R_Counter));
            pEntry->ReTryCounter = PEER_MSG1_RETRY_TIMER_CTR;
            pEntry->AuthMode = pAd->PortCfg.MBSSID[apidx].AuthMode;
            pEntry->WepStatus = pAd->PortCfg.MBSSID[apidx].WepStatus;
            pEntry->pAd = (unsigned long)pAd;
            pEntry->PMKID_CacheIdx = ENTRY_NOT_FOUND;

            if (pEntry->AuthMode < Ndis802_11AuthModeWPA)
                pEntry->WpaState = AS_NOTUSE;
            else
                pEntry->WpaState = AS_INITIALIZE;
            pEntry->GTKState = REKRY_NEGOTIATING;
            pEntry->PrivacyFilter = Ndis802_11PrivFilterAcceptAll;        
            pEntry->PairwiseKey.KeyLen = 0;
            pEntry->PortSecured = WPA_802_1X_PORT_NOT_SECURED;
            AsicRemovePairwiseKeyEntry(pAd, (UCHAR)i);
            COPY_MAC_ADDR(pEntry->Addr, pAddr);
            pEntry->Sst = SST_NOT_AUTH;
            pEntry->AuthState = AS_NOT_AUTH;
            pEntry->Aid = 0;
            pEntry->CapabilityInfo = 0;
            pEntry->PsMode = PWR_ACTIVE;
            pEntry->MaxSupportedRate = RATE_11;
            pEntry->CurrTxRate = RATE_11;
            pEntry->PsQIdleCount = 0;
            pEntry->NoDataIdleCount = 0;
            InitializeQueueHeader(&pEntry->PsQueue);

            pEntry->ApIdx = apidx;
            
            pAd->MacTab.Size ++;

            DBGPRINT(RT_DEBUG_TRACE, "MacTableInsertEntry -IF(ra%d) allocate entry #%d, Total= %d\n",apidx, i, pAd->MacTab.Size);
            break;
        }
    }

    // add this MAC entry into HASH table
    if (pEntry)
    {
        HashIdx = MAC_ADDR_HASH_INDEX(pAddr);
        if (pAd->MacTab.Hash[HashIdx] == NULL)
        {
            pAd->MacTab.Hash[HashIdx] = pEntry;
        }
        else
        {
            pCurrEntry = pAd->MacTab.Hash[HashIdx];
            while (pCurrEntry->pNext != NULL)
                pCurrEntry = pCurrEntry->pNext;
            pCurrEntry->pNext = pEntry;
        }
    }
    
#ifndef TC_NO_SPIN_LOCK  // Sam add
    spin_unlock_irqrestore(&pAd->MacTabLock, IrqFlags);
#endif
    return pEntry;
}

/*
    ==========================================================================
    Description:
        Delete a specified client from MAC table
    ==========================================================================
 */
BOOLEAN MacTableDeleteEntry(
    IN PRTMP_ADAPTER pAd, 
    IN PUCHAR pAddr) 
{
    USHORT HashIdx;
    MAC_TABLE_ENTRY *pEntry, *pPrevEntry;
    ULONG	IrqFlags;


#ifndef TC_NO_SPIN_LOCK  // Sam add
    spin_lock_irqsave(&pAd->MacTabLock, IrqFlags);
#endif
    HashIdx = MAC_ADDR_HASH_INDEX(pAddr);
    pEntry = pAd->MacTab.Hash[HashIdx];
    if (pEntry)
    {
        if (MAC_ADDR_EQUAL(pEntry->Addr, pAddr))
        {
        	if(pEntry->RetryTimerRunning==TRUE)
	        {
	            DBGPRINT(RT_DEBUG_TRACE, "MacTableDeleteEntry : Cancel Retry Timer !\n");
	            del_timer_sync(&pEntry->RetryTimer);
	            pEntry->RetryTimerRunning=FALSE;                
	        }

            pAd->MacTab.Hash[HashIdx] = pEntry->pNext;
            CleanupPsQueue(pAd, &pEntry->PsQueue); // return all skb buffer in PSQ
            AsicRemovePairwiseKeyEntry(pAd, (UCHAR)((ULONG)pEntry - (ULONG)&pAd->MacTab.Content[0])/sizeof(MAC_TABLE_ENTRY));
            NdisZeroMemory(pEntry, sizeof(MAC_TABLE_ENTRY));
            pAd->MacTab.Size --;
            DBGPRINT(RT_DEBUG_TRACE, "MacTableDeleteEntry 1 - Total= %d\n", pAd->MacTab.Size);
        }
        else
        {
            while (pEntry->pNext)
            {
                pPrevEntry = pEntry;
                pEntry = pEntry->pNext;
                if (MAC_ADDR_EQUAL(pEntry->Addr, pAddr))
                {
                	if(pEntry->RetryTimerRunning==TRUE)
			        {
			            DBGPRINT(RT_DEBUG_TRACE, "MacTableDeleteEntry : Cancel Retry Timer !\n");
			            del_timer_sync(&pEntry->RetryTimer);
			            pEntry->RetryTimerRunning=FALSE;                
			        }
                    pPrevEntry->pNext = pEntry->pNext;
                    CleanupPsQueue(pAd, &pEntry->PsQueue); // return all skb buffer in PSQ
                    AsicRemovePairwiseKeyEntry(pAd, (UCHAR)((ULONG)pEntry - (ULONG)&pAd->MacTab.Content[0])/sizeof(MAC_TABLE_ENTRY));
                    NdisZeroMemory(pEntry, sizeof(MAC_TABLE_ENTRY));
                    pAd->MacTab.Size --;
                    DBGPRINT(RT_DEBUG_TRACE, "MacTableDeleteEntry 2 - Total= %d\n", pAd->MacTab.Size);
                    break;
                }
            }
        }
    }

#ifndef TC_NO_SPIN_LOCK  // Sam add
    spin_unlock_irqrestore(&pAd->MacTabLock, IrqFlags);
#endif
    ApUpdateCapabilityAndErpIe(pAd);
    
    return TRUE;
}

/*
    ==========================================================================
    Description:
        Look up the MAC address in the MAC table. Return NULL if not found.
    Return:
        pEntry - pointer to the MAC entry; NULL is not found
    ==========================================================================
*/
MAC_TABLE_ENTRY *MacTableLookup(
    IN PRTMP_ADAPTER pAd, 
    IN PUCHAR pAddr) 
{
    ULONG HashIdx;
    MAC_TABLE_ENTRY *pEntry = NULL;

    HashIdx = MAC_ADDR_HASH_INDEX(pAddr);
    pEntry = pAd->MacTab.Hash[HashIdx];

    while (pEntry && pEntry->Valid)
    {
        if (MAC_ADDR_EQUAL(pEntry->Addr, pAddr)) 
        {
            break;
        }
        else
            pEntry = pEntry->pNext;
    }

    return pEntry;
}

/*
    ==========================================================================
    Description:
        This routine is called by MlmePeriodicExec() every second to check if
        1. any associated client in PSM. If yes, then TX MCAST/BCAST should be
           out in DTIM only
        2. any client being idle for too long and should be aged-out from MAC table
        3. garbage collect PSQ
    ==========================================================================
*/
VOID MacTableMaintenance(
    IN PRTMP_ADAPTER pAd)
{
    INT     i;
    ULONG	IrqFlags;

    pAd->MacTab.fAnyStationInPsm = FALSE;
    for (i = 0; i < MAX_LEN_OF_MAC_TABLE; i++) 
    {
        MAC_TABLE_ENTRY *pEntry = &pAd->MacTab.Content[i];

        if (pEntry->Valid == FALSE)
            continue;
        
        pEntry->NoDataIdleCount ++;

        DBGPRINT(RT_DEBUG_INFO, "(ra%d)%02x:%02x:%02x:%02x:%02x:%02x,TxByte=%d,RxByte=%d,LastDataTime=%d\n", pEntry->ApIdx,
            pEntry->Addr[0],pEntry->Addr[1],pEntry->Addr[2],pEntry->Addr[3],
            pEntry->Addr[4],pEntry->Addr[5],pEntry->HSCounter.TotalTxByteCount,
            pEntry->HSCounter.TotalRxByteCount,pEntry->HSCounter.LastDataPacketTime);

        // 0. STA failed to complete association should be removed to save MAC table space.
        if ((pEntry->Sst != SST_ASSOC) && (pEntry->NoDataIdleCount >= MAC_TABLE_ASSOC_TIMEOUT))
        {
            DBGPRINT(RT_DEBUG_TRACE, "%02x:%02x:%02x:%02x:%02x:%02x fail to complete ASSOC in %d sec\n",
                    pEntry->Addr[0],pEntry->Addr[1],pEntry->Addr[2],pEntry->Addr[3],
                    pEntry->Addr[4],pEntry->Addr[5],MAC_TABLE_ASSOC_TIMEOUT);
            MacTableDeleteEntry(pAd, pEntry->Addr);
            continue;
        }
        
        // 1. check if there's any associated STA in power-save mode. this affects outgoing
        //    MCAST/BCAST frames should be stored in PSQ till DtimCount=0
        if (pEntry->PsMode == PWR_SAVE)
            pAd->MacTab.fAnyStationInPsm = TRUE;

        // 2. delete those MAC entry that has been idle for a long time
        if (pEntry->NoDataIdleCount >= MAC_TABLE_AGEOUT_TIME)
        {
            DBGPRINT(RT_DEBUG_TRACE, "ageout %02x:%02x:%02x:%02x:%02x:%02x after %d-sec silence\n",
                    pEntry->Addr[0],pEntry->Addr[1],pEntry->Addr[2],pEntry->Addr[3],
                    pEntry->Addr[4],pEntry->Addr[5],MAC_TABLE_AGEOUT_TIME);
            ApLogEvent(pAd, pEntry->Addr, EVENT_AGED_OUT, pEntry->ApIdx);
            MacTableDeleteEntry(pAd, pEntry->Addr);
            continue;
        }
        
        // 3. garbage collect the PsQueue if the STA has being idle for a while
        if (pEntry->PsQueue.Head)
        {
            pEntry->PsQIdleCount ++;  
            if (pEntry->PsQIdleCount > 2) 
            {

#ifndef TC_NO_SPIN_LOCK  // Sam add
                spin_lock_irqsave(&pAd->MacTabLock, IrqFlags);
#endif
                CleanupPsQueue(pAd, &pEntry->PsQueue);

#ifndef TC_NO_SPIN_LOCK  // Sam add
                spin_unlock_irqrestore(&pAd->MacTabLock, IrqFlags);
#endif
                pEntry->PsQIdleCount = 0;
            }
        }
        else
            pEntry->PsQIdleCount = 0;
    }
    
    // 4. garbage collect pAd->MacTab.McastPsQueue if backlogged MCAST/BCAST frames
    //    stale in queue. Since MCAST/BCAST frames always been sent out whenever 
    //    DtimCount==0, the only case to let them stale is surprise removal of the NIC,
    //    so that ASIC-based Tbcn interrupt stops and DtimCount dead.
    if (pAd->MacTab.McastPsQueue.Head)
    {
        pAd->MacTab.PsQIdleCount ++;
        if (pAd->MacTab.PsQIdleCount > 1)
        {

#ifndef TC_NO_SPIN_LOCK  // Sam add
            spin_lock_irqsave(&pAd->MacTabLock, IrqFlags);
#endif
            CleanupPsQueue(pAd, &pAd->MacTab.McastPsQueue);

#ifndef TC_NO_SPIN_LOCK  // Sam add
            spin_unlock_irqrestore(&pAd->MacTabLock, IrqFlags);
#endif
            pAd->MacTab.PsQIdleCount = 0;
        }
    }
    else
        pAd->MacTab.PsQIdleCount = 0;
}

/*
    ==========================================================================
    Description:
        Look up a STA MAC table. Return its Sst to decide if an incoming
        frame from this STA or an outgoing frame to this STA is permitted.
    Return:
    ==========================================================================
*/
MAC_TABLE_ENTRY *SsPsInquiry(
    IN	PRTMP_ADAPTER	pAd, 
    IN PUCHAR pAddr, 
    OUT SST   *Sst, 
    OUT USHORT *Aid,
    OUT UCHAR *PsMode,
    OUT UCHAR *Rate) 
{
    MAC_TABLE_ENTRY *pEntry = NULL;

    if (MAC_ADDR_IS_GROUP(pAddr)) // mcast & broadcast address
    {
        *Sst        = SST_ASSOC;
        *Aid        = 0;
        *PsMode     = PWR_ACTIVE;
        //*Rate       = pAd->PortCfg.MlmeRate; //pAd->PortCfg.MaxBasicRate;
        if (pAd->PortCfg.PhyMode == PHY_11A) 
     		*Rate   = RATE_6;	
		else 
			*Rate   = RATE_11;	
    } 
    else // unicast address
    {
        pEntry = MacTableLookup(pAd, pAddr);
        if (pEntry) 
        {
            *Sst        = pEntry->Sst;
            *Aid        = pEntry->Aid;
            *PsMode     = pEntry->PsMode;
             if((pAd->PortCfg.MBSSID[pEntry->ApIdx].AuthMode >= Ndis802_11AuthModeWPA) && (pEntry->GTKState != REKEY_ESTABLISHED)) 
             {
             	if (pAd->PortCfg.PhyMode == PHY_11A) 
             		*Rate   = RATE_6;	
				else 
					*Rate   = RATE_11;	
             }                
             else
                *Rate   = pEntry->CurrTxRate;
        } 
        else 
        {
            *Sst        = SST_NOT_AUTH;
            *Aid        = 0;
            *PsMode     = PWR_ACTIVE;
            //*Rate       = pAd->PortCfg.MlmeRate; //pAd->PortCfg.MaxBasicRate;
            if (pAd->PortCfg.PhyMode == PHY_11A) 
         		*Rate   = RATE_6;
			else 
				*Rate   = RATE_11;
        }
    }
    return pEntry;
}

/*
    ==========================================================================
    Description:
        Update the station current power save mode. Calling this routine also
        prove the specified client is still alive. Otherwise AP will age-out
        this client once IdleCount exceeds a threshold.
    ==========================================================================
 */
BOOLEAN PsIndicate(
    IN PRTMP_ADAPTER pAd, 
    IN PUCHAR pAddr, 
    IN UCHAR Rssi,
    IN UCHAR Psm) 
{
    MAC_TABLE_ENTRY *pEntry;

    pEntry = MacTableLookup(pAd, pAddr);
    if (pEntry) 
    {
        if ((pEntry->PsMode == PWR_SAVE) && (Psm == PWR_ACTIVE))
        {
            DBGPRINT(RT_DEBUG_TRACE, "PsIndicate - %02x:%02x:%02x:%02x:%02x:%02x wakes up, act like rx PS-POLL\n", pAddr[0],pAddr[1],pAddr[2],pAddr[3],pAddr[4],pAddr[5]);
            // sleep station awakes, move all pending frames from PSQ to TXQ if any
            RTMPHandleRxPsPoll(pAd, pAddr, pEntry);
        }
        else if ((pEntry->PsMode != PWR_SAVE) && (Psm == PWR_SAVE))
        {
            DBGPRINT(RT_DEBUG_TRACE, "PsIndicate - %02x:%02x:%02x:%02x:%02x:%02x sleeps\n", pAddr[0],pAddr[1],pAddr[2],pAddr[3],pAddr[4],pAddr[5]);
        }
        
        pEntry->NoDataIdleCount = 0;
        pEntry->PsMode = Psm;
        pEntry->LastRssi = Rssi;
    } 
    else 
    {
        // not in table, try to learn it ???? why bother?
    }
    return TRUE;
}

#if BIG_ENDIAN == FALSE && RTMP_EMBEDDED == FALSE
extern unsigned long get_cmos_time(void);
#endif
/*
    ==========================================================================
    Description:
        This routine is called to log a specific event into the event table.
        The table is a QUERY-n-CLEAR array that stop at full.
    ==========================================================================
 */
VOID ApLogEvent(
    IN PRTMP_ADAPTER pAd,
    IN PUCHAR   pAddr,
    IN USHORT   Event,
    IN UCHAR    apidx)
{
    if (pAd->EventTab.Num < MAX_NUM_OF_EVENT)
    {
        RT_802_11_EVENT_LOG *pLog = &pAd->EventTab.Log[pAd->EventTab.Num];
#if BIG_ENDIAN == FALSE && RTMP_EMBEDDED == FALSE
        pLog->SystemTime = get_cmos_time();
#else
        pLog->SystemTime = jiffies;
#endif
        COPY_MAC_ADDR(&pLog->Addr, pAddr);
        pLog->Event = Event;
        pLog->ApIdx = apidx;
        DBGPRINT(RT_DEBUG_TRACE,"LOG#%d %02x:%02x:%02x:%02x:%02x:%02x %s\n",
            pAd->EventTab.Num, pAddr[0], pAddr[1], pAddr[2], 
            pAddr[3], pAddr[4], pAddr[5], pEventText[Event]);
        pAd->EventTab.Num += 1;
    }
}

/*
    ==========================================================================
    Description:
        Update ERP IE and CapabilityInfo based on STA association status.
        The result will be auto updated into the next outgoing BEACON in next
        TBTT interrupt service routine
    ==========================================================================
 */
VOID ApUpdateCapabilityAndErpIe(
    IN PRTMP_ADAPTER pAd)
{
    UCHAR  i, ErpIeContent = 0, apidx;
    BOOLEAN ShortSlotCapable = (BOOLEAN)pAd->PortCfg.UseShortSlotTime;

    if ((pAd->PortCfg.PhyMode == PHY_11A) || (pAd->PortCfg.PhyMode == PHY_11B))
        return;

    for (i=0; i<MAX_LEN_OF_MAC_TABLE; i++)
    {
        PMAC_TABLE_ENTRY pEntry = &pAd->MacTab.Content[i];
        if ((pEntry->Valid != TRUE) || (pEntry->Sst != SST_ASSOC))
            continue;
        
        // at least one 11b client associated, turn on ERP.NonERPPresent bit
        // almost all 11b client won't support "Short Slot" time, turn off for maximum compatibility
        if (pEntry->MaxSupportedRate < RATE_FIRST_OFDM_RATE)
        {
            ShortSlotCapable = FALSE;
            ErpIeContent |= 0x01;
        }

        // at least one client can't support short slot
        if ((pEntry->CapabilityInfo & 0x0400) == 0)
            ShortSlotCapable = FALSE;
    }

    // decide ErpIR.UseProtection bit, depending on pAd->PortCfg.UseBGProtection
    //    AUTO (0): UseProtection = 1 if any 11b STA associated
    //    ON (1): always USE protection
    //    OFF (2): always NOT USE protection
    if (pAd->PortCfg.UseBGProtection == 0)
    {
        ErpIeContent = (ErpIeContent)? 0x03 : 0x00;
        if ((pAd->Mlme.LastOLBCDetectTime + (10 * HZ)) > pAd->Mlme.Now32) // legacy BSS exist within 10 sec
        {
            DBGPRINT(RT_DEBUG_INFO, "ApUpdateCapabilityAndErpIe - Legacy 802.11b BSS overlaped\n");
            ErpIeContent |= 0x02;                                     // set Use_Protection bit
        }
    }
    else if (pAd->PortCfg.UseBGProtection == 1)   
        ErpIeContent |= 0x02;
    else
        ;

    if((pAd->PortCfg.TxPreamble == Rt802_11PreambleLong) || (ApCheckLongPreambleSTA(pAd) == TRUE))
    {
    	if (pAd->PortCfg.TxPreambleInUsed != Rt802_11PreambleLong)
        	MlmeSetTxPreamble(pAd, Rt802_11PreambleLong);
        pAd->Mlme.ErpIeContent = (ErpIeContent | 0x04);
    }
    else
    {
    	if (pAd->PortCfg.TxPreambleInUsed != Rt802_11PreambleShort)
        	MlmeSetTxPreamble(pAd, Rt802_11PreambleShort);
        pAd->Mlme.ErpIeContent = ErpIeContent;
    }

    //
    // deicide CapabilityInfo.ShortSlotTime bit
    //
    for (apidx = 0; apidx < pAd->PortCfg.BssidNum; apidx++)
    {
        if (ShortSlotCapable)
            pAd->PortCfg.MBSSID[apidx].CapabilityInfo |= 0x0400;
        else
            pAd->PortCfg.MBSSID[apidx].CapabilityInfo &= 0xfbff;

        DBGPRINT(RT_DEBUG_INFO, "ApUpdateCapabilityAndErpIe IF(ra%d)- Capability= 0x%04x, ERP is 0x%02x\n", 
            apidx, pAd->PortCfg.MBSSID[apidx].CapabilityInfo, pAd->Mlme.ErpIeContent);
    }

    AsicSetSlotTime(pAd, ShortSlotCapable);
}

/*
    ==========================================================================
    Description:
        Check to see the exist of long preamble STA in associated list
    ==========================================================================
 */
BOOLEAN ApCheckLongPreambleSTA(
    IN PRTMP_ADAPTER pAd)
{
    UCHAR   i;
    
    for (i=0; i<MAX_LEN_OF_MAC_TABLE; i++)
    {
        PMAC_TABLE_ENTRY pEntry = &pAd->MacTab.Content[i];
        if ((pEntry->Valid != TRUE) || (pEntry->Sst != SST_ASSOC))
            continue;
    
        if (!CAP_IS_SHORT_PREAMBLE_ON(pEntry->CapabilityInfo))
        {
            DBGPRINT(RT_DEBUG_TRACE, "Long preamble capable STA exist\n");
            return TRUE;
        }
    }

    return FALSE;
}

BOOLEAN ApCheckAccessControlList(
    IN PRTMP_ADAPTER pAd,
    IN PUCHAR      pAddr,
    IN UCHAR         Apidx) 
{
	BOOLEAN Result = TRUE;

    if (pAd->PortCfg.MBSSID[Apidx].AccessControlList.Policy == 0)       // ACL is disabled
        Result = TRUE;
    else
    {
        ULONG i;
        if (pAd->PortCfg.MBSSID[Apidx].AccessControlList.Policy == 1)   // ACL is a positive list
            Result = FALSE;
        else                                              // ACL is a negative list
            Result = TRUE;
        for (i=0; i<pAd->PortCfg.MBSSID[Apidx].AccessControlList.Num; i++)
        {
            if (MAC_ADDR_EQUAL(pAddr, pAd->PortCfg.MBSSID[Apidx].AccessControlList.Entry[i].Addr))
            {
                Result = !Result;
                break;
            }
        }
    }

    if (Result == FALSE)
    {
        DBGPRINT(RT_DEBUG_TRACE, "%02x:%02x:%02x:%02x:%02x:%02x failed ACL checking\n",
        pAddr[0],pAddr[1],pAddr[2],pAddr[3],pAddr[4],pAddr[5]);
    }

    return Result;
}

VOID ApUpdateAccessControlList(
    IN PRTMP_ADAPTER pAd,
    IN UCHAR         Apidx)
{
    USHORT      AclIdx, MacIdx;
    BOOLEAN     Matched;

    // ACL is disabled. do nothing about the MAC table
    if (pAd->PortCfg.MBSSID[Apidx].AccessControlList.Policy == 0)
        return;

    for (MacIdx=0; MacIdx < MAX_LEN_OF_MAC_TABLE; MacIdx++)
    {
        if (! pAd->MacTab.Content[MacIdx].Valid) 
            continue;

        Matched = FALSE;
        for (AclIdx = 0; AclIdx < pAd->PortCfg.MBSSID[Apidx].AccessControlList.Num; AclIdx++)
        {
            if (MAC_ADDR_EQUAL(&pAd->MacTab.Content[MacIdx].Addr, pAd->PortCfg.MBSSID[Apidx].AccessControlList.Entry[AclIdx].Addr))
            {
                Matched = TRUE;
                break;
            }
        }
            
        if ((Matched == FALSE) && (pAd->PortCfg.MBSSID[Apidx].AccessControlList.Policy == 1))
        {
            DBGPRINT(RT_DEBUG_TRACE, "STA not on positive ACL. remove it...\n");
            MacTableDeleteEntry(pAd, pAd->MacTab.Content[MacIdx].Addr);
        }
        else if ((Matched == TRUE) && (pAd->PortCfg.MBSSID[Apidx].AccessControlList.Policy == 2))
        {
            DBGPRINT(RT_DEBUG_TRACE, "STA on negative ACL. remove it...\n");
            MacTableDeleteEntry(pAd, pAd->MacTab.Content[MacIdx].Addr);
        }
    }
}

UCHAR APAutoSelectChannel(
    IN PRTMP_ADAPTER	pAd)
{
	UCHAR ch, i;
    UCHAR dirtyness[MAX_NUM_OF_CHANNELS+1], dirty;
    UCHAR max_rssi[MAX_NUM_OF_CHANNELS+1];
    ULONG FalseCca, FcsError;
    BOOLEAN bFindIt = FALSE;

    // passive scan. collect statistics
    NdisZeroMemory(dirtyness, MAX_NUM_OF_CHANNELS+1);

    if (pAd->PortCfg.PhyMode == PHY_11A)
    {
    	if (pAd->PortCfg.RadarDetect.IEEE80211H)
    	{
    		while(TRUE)
    		{
    			struct timeval tv;
    			
    			do_gettimeofday(&tv);
    			
    			ch = pAd->ChannelList[tv.tv_usec%pAd->ChannelListNum].Channel;

    			if (ch == 0)
    				ch = FirstChannel(pAd);
    			if (!RadarChannelCheck(pAd, ch))
    				continue;

    			for (i=0; i<pAd->ChannelListNum; i++)
    			{
    				if (pAd->ChannelList[i].Channel == ch)
    				{
    					if (pAd->ChannelList[i].RemainingTimeForUse == 0)
    						bFindIt = TRUE;
    					
   						break;
    				}
    			}
    			
    			if (bFindIt == TRUE)
    				break;
    		};
    	}
    	else
    	{
    		struct timeval tv;
    			
    		do_gettimeofday(&tv);
    		
    		ch = pAd->ChannelList[tv.tv_usec%pAd->ChannelListNum].Channel;
    		if (ch == 0)
   				ch = FirstChannel(pAd);
    	}
    	DBGPRINT(RT_DEBUG_TRACE,"1.APAutoSelectChannel pick up ch#%d\n",ch);
    	return ch;
    }
    else
    {
	    for (ch=14; ch>=1; ch--)
	    {
	        AsicSwitchChannel(pAd, ch);
	        pAd->Counters8023.GoodReceives = 0;
	        pAd->Counters8023.RxErrors = 0;
	        pAd->PortCfg.AutoChannel_MaxRssi = 0;
	        pAd->PortCfg.AutoChannel_Channel = ch;
	        max_rssi[ch]=0;
	        RTMPusecDelay(500000); // 0.5 sec at each channel
	        max_rssi[ch] = pAd->PortCfg.AutoChannel_MaxRssi;

	        RTMP_IO_READ32(pAd, STA_CSR1, &FalseCca);
	        FalseCca = FalseCca >> 16;
	        RTMP_IO_READ32(pAd, STA_CSR0, &FcsError);
	        FcsError &= 0x0000ffff;
	        
	        if (max_rssi[ch])
	        {
	            dirtyness[ch] += 10;
	            if (ch > 1) dirtyness[ch-1] += 1;
	            if (ch > 2) dirtyness[ch-2] += 1;
	            if (ch > 3) dirtyness[ch-3] += 1;
	            if (ch > 4) dirtyness[ch-4] += 1;
	            if (ch < 14) dirtyness[ch+1] += 1;
	            if (ch < 13) dirtyness[ch+2] += 1;
	            if (ch < 12) dirtyness[ch+3] += 1;
	            if (ch < 11) dirtyness[ch+4] += 1;
	        }
	        DBGPRINT(RT_DEBUG_TRACE,"Sleep at ch#%d to collect RX=%d, RSSI=%d, CRC error=%d, False CCA=%d\n", 
	            ch, pAd->Counters8023.GoodReceives, max_rssi[ch] - pAd->BbpRssiToDbmDelta, FcsError, FalseCca);
	    }
    
	    pAd->PortCfg.AutoChannel_Channel = 0;

	    DBGPRINT(RT_DEBUG_TRACE,"Dirtyness = %d.%d.%d.%d - %d.%d.%d.%d - %d.%d.%d.%d - %d.%d\n", 
	        dirtyness[1], dirtyness[2], dirtyness[3], dirtyness[4], dirtyness[5], dirtyness[6], dirtyness[7],
	        dirtyness[8], dirtyness[9], dirtyness[10], dirtyness[11], dirtyness[12], dirtyness[13], dirtyness[14]);

	    // RULE 1. pick up a good channel that no one used
	    for (ch = FirstChannel(pAd); ch !=0; ch = NextChannel(pAd, ch))
	    {
	        if (dirtyness[ch] == 0) break;
	    }
	    if (ch != 0)
	    {
	        DBGPRINT(RT_DEBUG_TRACE,"2.APAutoSelectChannel pick up ch#%d\n",ch);
	        return ch;
	    }

	    // RULE 2. if not available, then co-use a channel that's no interference (dirtyness=10)
	    // RULE 3. if not available, then co-use a channel that has minimum interference (dirtyness=11,12)
	    for (dirty = 10; dirty <= 12; dirty++)
	    {
	        UCHAR candidate[MAX_NUM_OF_CHANNELS+1], candidate_num;
	        UCHAR min_rssi = 255, final_channel = 0;
	        
	        candidate_num = 0;
	        NdisZeroMemory(candidate, MAX_NUM_OF_CHANNELS+1);
	        for (ch = FirstChannel(pAd); ch != 0; ch = NextChannel(pAd, ch))
	        {
	            if (dirtyness[ch] == dirty) 
	            { 
	                candidate[ch]=1; 
	                candidate_num++; 
	            }
	        }
	        // if there's more than 1 candidate, pick up the channel with minimum RSSI
	        if (candidate_num)
	        {
	            for (ch = 1; ch <= 14; ch ++)
	            {
	                if (candidate[ch] && (max_rssi[ch] < min_rssi))
	                {
	                    final_channel = ch;
	                    min_rssi = max_rssi[ch];
	                }
	            }
	            if (final_channel != 0)
	            {
	                DBGPRINT(RT_DEBUG_TRACE,"3.APAutoSelectChannel pick up ch#%d\n",final_channel);
	                return final_channel;
	            }
	        }
	    }

	    // RULE 4. still not available, pick up the first channel
	    ch = FirstChannel(pAd);
	    DBGPRINT(RT_DEBUG_TRACE,"4.APAutoSelectChannel pick up ch#%d\n",ch);
	    
	    return ch;
    }
}

/* 
    ==========================================================================
    Description:
        Send out a NULL frame to a specified STA at a higher TX rate. The 
        purpose is to ensure the designated client is okay to received at this
        rate.
    ==========================================================================
 */
VOID ApEnqueueNullFrame(
    IN PRTMP_ADAPTER pAd,
    IN PUCHAR        pAddr,
    IN UCHAR         TxRate,
    IN UCHAR         PID,
    IN UCHAR         Apidx) 
{
    PHEADER_802_11 pNullFr;

    // since TxRate may change, we have to change Duration each time
    pNullFr = kmalloc(MAX_LEN_OF_MLME_BUFFER, MEM_ALLOC_FLAG);
    if (pNullFr)
    {
    	// Send to BSS
        if ((PID & 0x3f) < WDS_PAIRWISE_KEY_OFFSET) // send to client
        {
            MgtMacHeaderInit(pAd, pNullFr, SUBTYPE_NULL_FUNC, 0, pAddr, pAd->PortCfg.MBSSID[Apidx].Bssid);
            pNullFr->FC.Type = BTYPE_DATA;
            pNullFr->FC.FrDs = 1;
            pNullFr->Duration = RTMPCalcDuration(pAd, TxRate, 14);
            ApSendFrame(pAd, pNullFr, sizeof(HEADER_802_11), TxRate, PID);
        }
        else
        {
        	// Send to WDS
        	UCHAR ToWhichWds = (PID & 0x3f) - WDS_PAIRWISE_KEY_OFFSET;
        	
            NdisZeroMemory(pNullFr, LENGTH_802_11_WITH_ADDR4);
            pNullFr->FC.Type = BTYPE_DATA;
            pNullFr->FC.SubType = SUBTYPE_NULL_FUNC;
            pNullFr->FC.FrDs = 1;
            pNullFr->FC.ToDs = 1;
            COPY_MAC_ADDR(pNullFr->Addr1, pAd->WdsTab.WdsEntry[ToWhichWds].WdsAddr);
            COPY_MAC_ADDR(pNullFr->Addr2, pAd->PortCfg.MBSSID[MAIN_MBSSID].Bssid);
            COPY_MAC_ADDR(pNullFr->Addr3, pAd->WdsTab.WdsEntry[ToWhichWds].WdsAddr);
            COPY_MAC_ADDR(pNullFr->Addr3 + MAC_ADDR_LEN, pAd->PortCfg.MBSSID[MAIN_MBSSID].Bssid);
            pNullFr->Duration = RTMPCalcDuration(pAd, TxRate, 14);
            ApSendFrame(pAd, pNullFr, LENGTH_802_11_WITH_ADDR4, TxRate, PID);
        }

        kfree(pNullFr);
    }
}

// Sam add
#ifndef NOT_CHNAGE_ENDIAN_3
VOID    ApSendFrame(
    IN  PRTMP_ADAPTER   pAd,
    IN  PVOID           pBuffer,
    IN  ULONG           Length,
    IN  UCHAR           TxRate,
    IN  UCHAR           PID)
{
    PUCHAR          pDest;
    PTXD_STRUC      pTxD;
    PRTMP_TX_RING   pTxRing = &pAd->TxRing[QID_AC_BE];
    ULONG			IrqFlags;
#ifdef BIG_ENDIAN
    PTXD_STRUC      pDestTxD;
    TXD_STRUC       TxD;
#endif

    if (RTMP_TEST_FLAG(pAd, fRTMP_ADAPTER_RESET_IN_PROGRESS))
    {
        return;
    }

    // Make sure Tx ring resource won't be used by other threads
    spin_lock_irqsave(&pAd->TxRingLock, IrqFlags);
    
    // Get the Tx Ring descriptor & Dma Buffer address
    pDest = (PUCHAR) pTxRing->Cell[pTxRing->CurTxIndex].DmaBuf.AllocVa;
#ifndef BIG_ENDIAN
    pTxD  = (PTXD_STRUC) pTxRing->Cell[pTxRing->CurTxIndex].AllocVa;
#else
    pDestTxD  = (PTXD_STRUC) pTxRing->Cell[pTxRing->CurTxIndex].AllocVa;
    TxD = *pDestTxD;
    pTxD = &TxD;
    RTMPDescriptorEndianChange((PUCHAR)pTxD, TYPE_TXD);
#endif

    if ((pTxD->Owner == DESC_OWN_NIC) || (pTxD->bWaitingDmaDoneInt == 1))
    {
        // Descriptor owned by NIC. No descriptor avaliable
        // This should not happen since caller guaranteed.
        DBGPRINT_ERR("APSendNullFrame: TX RING full\n");
        pAd->RalinkCounters.TxRingErrCount++;
        spin_unlock_irqrestore(&pAd->TxRingLock, IrqFlags);
        return;
    }
        
    DBGPRINT(RT_DEBUG_TRACE, "send NULL Frame @%d Mbps...\n", RateIdToMbps[TxRate]);
#ifdef BIG_ENDIAN
   RTMPFrameEndianChange(pAd, (PUCHAR)pBuffer, DIR_WRITE, FALSE);
#endif
    NdisMoveMemory(pDest, pBuffer, Length);
//  pTxRing->Cell[pTxRing->CurTxIndex].FrameType = BTYPE_DATA;

#ifdef BIG_ENDIAN
    RTMPDescriptorEndianChange((PUCHAR)pTxD, TYPE_TXD);
    *pDestTxD = TxD;
    pTxD = pDestTxD;
#endif

    RTMPWriteTxDescriptor(pAd, 
                          pTxD, 
                          CIPHER_NONE, 0, 0, 
                          TRUE, 
                          FALSE, 
                          FALSE, 
                          SHORT_RETRY, 
                          IFS_BACKOFF, 
                          TxRate, 
                          Length, 
                          QID_AC_BE,
                          PID,
                          NULL,
                          NULL,
                          FALSE);

    INC_RING_INDEX(pTxRing->CurTxIndex, TX_RING_SIZE);
    pAd->RalinkCounters.KickTxCount++;
    spin_unlock_irqrestore(&pAd->TxRingLock, IrqFlags);

    RTMP_IO_WRITE32(pAd, TX_CNTL_CSR, BIT8[QID_AC_BE]); // use AC_BE
            
}

#else  // define NOT_CHANGE_ENDIAN_3

VOID    ApSendFrame(
    IN  PRTMP_ADAPTER   pAd,
    IN  PVOID           pBuffer,
    IN  ULONG           Length,
    IN  UCHAR           TxRate,
    IN  UCHAR           PID)
{
    PUCHAR          pDest;
    PTXD_STRUC      pTxD;
    PRTMP_TX_RING   pTxRing = &pAd->TxRing[QID_AC_BE];
    ULONG			IrqFlags;
#ifdef BIG_ENDIAN
    PTXD_BIG_STRUC	pTxBD;
#endif

    if (RTMP_TEST_FLAG(pAd, fRTMP_ADAPTER_RESET_IN_PROGRESS))
    {
        return;
    }

    // Make sure Tx ring resource won't be used by other threads

#ifndef TC_NO_SPIN_LOCK  // Sam add
    spin_lock_irqsave(&pAd->TxRingLock, IrqFlags);
#endif    
    // Get the Tx Ring descriptor & Dma Buffer address
    pDest = (PUCHAR) pTxRing->Cell[pTxRing->CurTxIndex].DmaBuf.AllocVa;
#ifndef BIG_ENDIAN
    pTxD  = (PTXD_STRUC) pTxRing->Cell[pTxRing->CurTxIndex].AllocVa;
#else
    pTxBD  = (PTXD_STRUC) pTxRing->Cell[pTxRing->CurTxIndex].AllocVa;
#endif

    // Sam if ((pTxD->Owner == DESC_OWN_NIC) || (pTxD->bWaitingDmaDoneInt == 1))
    if ((pTxBD->ulword0 & 0x01000000) || (pTxBD->ulword5 & 0x00000001))
    {
        // Descriptor owned by NIC. No descriptor avaliable
        // This should not happen since caller guaranteed.
        DBGPRINT_ERR("APSendNullFrame: TX RING full\n");
        pAd->RalinkCounters.TxRingErrCount++;
#ifndef TC_NO_SPIN_LOCK  // Sam add
        spin_unlock_irqrestore(&pAd->TxRingLock, IrqFlags);
#endif
        return;
    }
        
    DBGPRINT(RT_DEBUG_TRACE, "send NULL Frame @%d Mbps...\n", RateIdToMbps[TxRate]);
#ifdef BIG_ENDIAN
   RTMPFrameEndianChange(pAd, (PUCHAR)pBuffer, DIR_WRITE, FALSE);
#endif
    NdisMoveMemory(pDest, pBuffer, Length);
//  pTxRing->Cell[pTxRing->CurTxIndex].FrameType = BTYPE_DATA;

#ifdef BIG_ENDIAN
    // Sam RTMPDescriptorEndianChange((PUCHAR)pTxD, TYPE_TXD);
    // Sam *pDestTxD = TxD;
    // Sam  pTxD = pDestTxD;
    pTxD = (PTXD_STRUC)pTxBD;
#endif

    RTMPWriteTxDescriptor(pAd, 
                          pTxD, 
                          CIPHER_NONE, 0, 0, 
                          TRUE, 
                          FALSE, 
                          FALSE, 
                          SHORT_RETRY, 
                          IFS_BACKOFF, 
                          TxRate, 
                          Length, 
                          QID_AC_BE,
                          PID,
                          NULL,
                          NULL,
                          FALSE);

    INC_RING_INDEX(pTxRing->CurTxIndex, TX_RING_SIZE);
    pAd->RalinkCounters.KickTxCount++;
#ifndef TC_NO_SPIN_LOCK
    spin_unlock_irqrestore(&pAd->TxRingLock, IrqFlags);
#endif

    RTMP_IO_WRITE32(pAd, TX_CNTL_CSR, BIT8[QID_AC_BE]); // use AC_BE
            
}


#endif  // end NOT_CHANGE_ENDIAN_3, End Sam add

