
#ifdef TC_RT61AP_PROC
#include "linux/proc_fs.h"
#include "rt_config.h"
#include "linux/netdevice.h"

struct proc_dir_entry *rt61ap_dir = NULL;

#if 0
static int proc_version_read(char *buf, char **start,
                        off_t off, int count, int *eof, void *data)
{
        char                  *p = buf;
        p+=sprintf(p, "version = %s\n\n", DRIVER_VERSION);
	return (p - buf);
}
#endif     
                                                                                                                     
int proc_info_read(char *buf, char **start,
                        off_t off, int count, int *eof, void *data)
{
	struct net_device *net_dev;
	char *p = buf;
	net_dev = (struct net_device *)data;
#if 1  // Sam add for show different version information between RT2561T and RT2661T(MIMO)
	PRTMP_ADAPTER pAd = net_dev->priv;
	if ((pAd->pPci_Dev->device == NIC2561_PCI_DEVICE_ID) || (pAd->pPci_Dev->device == NIC2561S_PCI_DEVICE_ID))
		p+=sprintf(p,"Version = RT2561T %s\nMacAddress = %02X:%02X:%02X:%02X:%02X:%02X\n",DRIVER_VERSION,net_dev->dev_addr[0],net_dev->dev_addr[1],net_dev->dev_addr[2],net_dev->dev_addr[3],net_dev->dev_addr[4],net_dev->dev_addr[5]);
	else if (pAd->pPci_Dev->device == NIC2661_PCI_DEVICE_ID)
		p+=sprintf(p,"Version = RT2661T %s\nMacAddress = %02X:%02X:%02X:%02X:%02X:%02X\n",DRIVER_VERSION,net_dev->dev_addr[0],net_dev->dev_addr[1],net_dev->dev_addr[2],net_dev->dev_addr[3],net_dev->dev_addr[4],net_dev->dev_addr[5]);

#else 
	p+=sprintf(p,"Version = %s\nMacAddress = %02X:%02X:%02X:%02X:%02X:%02X\n",DRIVER_VERSION,net_dev->dev_addr[0],net_dev->dev_addr[1],net_dev->dev_addr[2],net_dev->dev_addr[3],net_dev->dev_addr[4],net_dev->dev_addr[5]);

#endif
	return (p - buf);

}
int wlan_ignore_interrupt=0;
static int proc_wlan_ignore_interrupt_read(char *buf, char **start,
                        off_t off, int count, int *eof, void *data)
{
  unsigned int len = 0;
 len = sprintf (buf, "%d\n", wlan_ignore_interrupt);
 return len;
}


static int
proc_wlan_ignore_interrupt_write (struct file *file, const char *buffer,
	  unsigned long count, void *data)
{
  int len;
  char tmpbuf[64];
  MOD_INC_USE_COUNT;

  if (count > 64)
    len = 64;
  else
    len = count;

  if (copy_from_user (tmpbuf, buffer, len))
    {
      MOD_DEC_USE_COUNT;
      return -EFAULT;
    }

  tmpbuf[len] = '\0';
  printk("the configure wlan_ignore_interrupt=%s\n",tmpbuf);
  sscanf (tmpbuf, "%d", &wlan_ignore_interrupt);
  printk("after  wlan_ignore_interrupt=%d\n",wlan_ignore_interrupt);

  
  MOD_DEC_USE_COUNT;
  return len;

}

void init_rt61ap_proc(void *param)
{
	struct proc_dir_entry *entry;
	struct net_device* net_dev;
	net_dev = (struct net_device *)param;	
	
	rt61ap_dir = proc_mkdir("rt61ap",NULL);
	if (rt61ap_dir == NULL)
		return;
	entry = create_proc_read_entry("info",0,rt61ap_dir, proc_info_read, net_dev);
 	entry = create_proc_read_entry("ignore_int",0,rt61ap_dir, proc_wlan_ignore_interrupt_read, net_dev); 
        if (entry) entry->write_proc = proc_wlan_ignore_interrupt_write;
        //entry = create_proc_read_entry("version", 0, rt61ap_dir, proc_version_read, NULL);
        
    //Henry add for getting the Mac address for web dispaly in probe
    get_mac_address_from_eeprom(net_dev);        
}


void remove_rt61ap_proc(void)
{

	if( rt61ap_dir == NULL )
		return;
	
	remove_proc_entry("info",rt61ap_dir);
	remove_proc_entry("ignore_int",rt61ap_dir);
	rt61ap_dir = NULL;
}

void    NICReadEEPROMParameters_MAC(struct net_device *net_dev, PRTMP_ADAPTER pAd)
{
	ULONG			data;

	// Init EEPROM Address Number, before access EEPROM; if 93c46, EEPROMAddressNum=6, else if 93c66, EEPROMAddressNum=8
	RTMP_IO_READ32(pAd, E2PROM_CSR, &data);

	if(data & 0x20)
		pAd->EEPROMAddressNum = 6;		// 93C46
	else
		pAd->EEPROMAddressNum = 8;     // 93C66

    // RT2660 MAC no longer auto load MAC address from E2PROM. Driver has to intialize
    // MAC address registers according to E2PROM setting
    {
        USHORT  Addr01,Addr23,Addr45 ;

        Addr01=RTMP_EEPROM_READ16(pAd, 0x04);
        Addr23=RTMP_EEPROM_READ16(pAd, 0x06);
        Addr45=RTMP_EEPROM_READ16(pAd, 0x08);

        net_dev->dev_addr[0] = (UCHAR)(Addr01 & 0xff);
        net_dev->dev_addr[1] = (UCHAR)(Addr01 >> 8);
        net_dev->dev_addr[2] = (UCHAR)(Addr23 & 0xff);
        net_dev->dev_addr[3] = (UCHAR)(Addr23 >> 8);
        net_dev->dev_addr[4] = (UCHAR)(Addr45 & 0xff);
        net_dev->dev_addr[5] = (UCHAR)(Addr45 >> 8);
    }
}

void get_mac_address_from_eeprom(struct net_device *net_dev)
{
	PRTMP_ADAPTER   pAd = net_dev->priv;
  	// Wait for hardware stable
	{
		ULONG MacCsr0, Index = 0;		
		do
		{
			RTMP_IO_READ32(pAd, MAC_CSR0, &MacCsr0);
			if (MacCsr0 != 0)
				break;			
			RTMPusecDelay(1000);
		} while (Index++ < 1000);
	}
	// We should read EEPROM for mac
	NICReadEEPROMParameters_MAC(net_dev, pAd);	

}
#endif
