/*
** $Id: ccmp.c 1.21 2004/06/02 17:50:54 chyeh Exp $
*/

/*************************************************************************
* Copyright (c) 2002 Integrated Programmable Communications, Inc.
*
* All rights reserved. Copying, compilation, modification, distribution
* or any other use whatsoever of this material is strictly prohibited
* except in accordance with a Software License Agreement with
* Integrated Programmable Communications, Inc.
**************************************************************************/
/*************************************************************************
GENERAL DESCRIPTION
********************
This CCMP (Counter-Mode/CBC-MAC Protocol) is implemented according to
IEEE Std 802.11i/D3.0, Nov 2002
*/

/************************************************************************
*                E X T E R N A L   R E F E R E N C E S
*************************************************************************
*/
#include "precomp.h"

/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/
#define IPN2220_IS_QOS_DATA(MacHeader_p)     ((MacHeader_p)->frameCtrl & BIT(3) &&  \
                                               (MacHeader_p)->frameCtrl & BIT(7))

#define IPN2220_IS_A4_EXIST(MacHeader_p)     (sdl_toDs(MacHeader_p) && \
                                               sdl_frDs(MacHeader_p))
#define UCHAR_BIT_0         0x01
#define UCHAR_BIT_1         0x02
/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/

/************************************************************************
*                        P U B L I C   D A T A
*************************************************************************
*/

/************************************************************************
*                       P R I V A T E   D A T A
*************************************************************************
*/

/************************************************************************
*                             M A C R O S
*************************************************************************
*/

/************************************************************************
*              F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/

/************************************************************************
*                          F U N C T I O N S
*************************************************************************
*/

/*_________________________________________________________________________
** ccmpHwMpduEncapsulate
**
** description: This function encrypt MPDU frame body in AES_CCM mode
**              using H/W security engine mode operation. After encryption,
**              the original MPDU size is expended by 16-octet: 4 for the
**              IV/Key ID field,4 for the extended IV field, and 8 for the
**              MIC.
** parameters:  adapter_p,   Pointer to the adapter object data area.
**              macHeader_p, Pointer to MPDU frame MAC header
**              frameBody_p, Pointer to MPDU frame body
**              frameBodyLen,Length of frame body, i.e. Dlen, in byte(s),
**                           exclude IV, ExtIV and MIC
**              key_p,       Pointer to encrypt key
**              keyId,       key index
**              resultFrameBodyLen_p, Pointer to where to put the result
**                                    length after encapsultation
** return:      CCMP_ENCAPSULATE_SUCCESS, if encapsulate successfully
** note:
**_________________________________________________________________________*/
UCHAR
ccmpHwMpduEncapsulate(IN PMP_ADAPTER adapter_p,
                      IN OUT PTFCB_STRUC  tfcb_p,
                      IN UCHAR keyId,
                      IN PUCHAR key_p)
{
    UCHAR   iv_KeyId;
    UINT    i;

    DEBUGFUNC("ccmpHwMpduEncapsulate");
    DBGLOG(RSN, TRACE, ("\n"));

    /* Increase TSC by 1 for each MPDU */
    INC_48_BITS(adapter_p->TSC);

    /* Generate IV with Packet Number(PN0 & PN1)
     * Never use casting to what Byte array copy is supposed to do.
     * It seems tfcb_p->u_sciiv.sciIv  can be assigned a value  instead of OR.
     */
    ipn_memcpy(&tfcb_p->u_sciiv.sciIv, &adapter_p->TSC[0], 2);
    ipn_memcpy(&tfcb_p->sciExtendedIv, &adapter_p->TSC[2], 4);

    if (keyId < 4) { /* default key */
        /* add key ID and padding zeros to IV, in byte 3 */
        iv_KeyId= (UCHAR)keyId << 6;    /* Key ID(bit 6 & 7) */
    }
    else { /* mapping key */
        /* set key ID to 0 */
        iv_KeyId = 0; /* mapping key. so, set key ID to 0 */
    }
    tfcb_p->u_sciiv.s.keyId = iv_KeyId;
    tfcb_p->u_sciiv.s.keyId |= BIT(5);     /* Set ExtIV(bit 5) flag on */
    tfcb_p->iv = tfcb_p->u_sciiv.sciIv;
    tfcb_p->extendedIv = tfcb_p->sciExtendedIv;
    DBGPRINT(RSN, LOUD, (" IV = 0x%08X\n", tfcb_p->u_sciiv.sciIv));
    DBGPRINT(RSN, LOUD, (" ExtIV = 0x%08X\n", tfcb_p->sciExtendedIv));

    tfcb_p->dmaFifoHeaderLen += 2;    /* size in unit of DWORD(4 bytes) */
    DBGPRINT(RSN, LOUD, (" Fifo Length = %d\n", tfcb_p->dmaFifoHeaderLen));

    tfcb_p->secOpMode = CIPHER_SUITE_CCMP;
    DBGPRINT(RSN, LOUD, (" Cipher Suite = 0x%02X\n", tfcb_p->secOpMode));

    /* fill up the Encryption Key(16 bytes) */
    DBGPRINT(RSN, LOUD, (" Tfcb_p->encryptionKey ==>\n"));
    for (i = 0; i < 16; i++) {
        tfcb_p->encryptionKey[i] = *(key_p + i);
        DBGPRINT_RAW(RSN, LOUD, ("[%02X]", tfcb_p->encryptionKey[i]));
    }
    DBGPRINT(RSN, LOUD, ("\n"));

    /* MPDU Length + IV(4) + ExtIV(4) + MIC(8) */
    tfcb_p->mpduLen += 16;
    DBGPRINT(RSN, LOUD, (" MPDU Length = %d\n", tfcb_p->mpduLen));

    return CCMP_ENCAPSULATE_SUCCESS;
}       /* End of ccmpHwMpduEncapsulate */


