/*
** $Id: cipher.c 1.12 2004/09/14 11:34:42 whsu Exp $
*/

/*!     \file crypt.c
        \brief The security function
*/

/******************************************************************************
Copyright (c) 2000-2002 Integrated Programmable Communications, Inc.

All rights reserved. Copying, compilation, modification, distribution
or any other use whatsoever of this material is strictly prohibited
except in accordance with a Software License Agreement with
Integrated Programmable Communications, Inc.
******************************************************************************/

/************************************************************************
*                                E X T E R N A L   R E F E R E N C E S
*************************************************************************
*/

/* System Includes */
#include "precomp.h"

/************************************************************************
*                                                  C O N S T A N T S
*************************************************************************
*/

/************************************************************************
*                    D A T A    T Y P E S
*************************************************************************
*/

/************************************************************************
*                    P U B L I C   D A T A
*************************************************************************
*/


/************************************************************************
*                    P R I V A T E  D A T A
*************************************************************************
*/

/************************************************************************
*                    M A C R O S
*************************************************************************
*/

/************************************************************************
*                    F U N C T I O N       D E C L A R A T I O N S
*************************************************************************
*/
/************************************************************************
*                    F U N C T I O N S
*************************************************************************
*/

/************************************************************************
*                    F U N C T I O N       D E C L A R A T I O N S
*************************************************************************
*/

/************************************************************************
*                          F U N C T I O N S
*************************************************************************
*/

/*_________________________________________________________________________
* Function   : TxMpduEncrypt
* Input      : priv_p - Pointer to the adapter object data area.
*              iv_p, pointer to frame IV field
*              macHeader_p, pointer to frame MAC header
* Output:      ciphersuite_p, pointer to put result of selected ciphersuite
*              key_pp, pointer to secret key address
*              status, pointer to result, the return value is key ID
* Return     : FALSE, if the frame is not encryptable. Then the caller
*                     function should discard the related frame and return
*                     the RFB.
*              TRUE, if the frame is encryptable
* Description:
**__________________________________________________________________________*/
UCHAR
TxMpduEncrypt(IN i2220mgmt_t *priv_p,
                 IN hw_tfcb_t *hw_tfcb_p,
                 IN FragSdu_t *fsdu)
{
        UINT8                   cipherSuite, keyId;
        UINT8                   encryptResult;
        UINT8                   *key_p;

        cipherSuite = fsdu->cipherSuite;
        keyId = fsdu->encryptionKeyId;
        key_p = &fsdu->encryptKey[0];

        switch (cipherSuite) {
        case CIPHER_SUITE_WEP40: /* WEP 40-bit */
                encryptResult = wepHwMpduEncapsulate(priv_p, hw_tfcb_p,
                        (UCHAR) keyId, key_p, 5, CIPHER_SUITE_WEP40);
                break;
        case CIPHER_SUITE_WEP104: /* WEP 104-bit */
                encryptResult = wepHwMpduEncapsulate(priv_p, hw_tfcb_p,
                        (UCHAR) keyId, key_p, 13, CIPHER_SUITE_WEP104);
              break;
        case CIPHER_SUITE_WEP128: /* WEP 128-bit */
                encryptResult = wepHwMpduEncapsulate(priv_p, hw_tfcb_p,
                        (UCHAR) keyId, key_p, 16, CIPHER_SUITE_WEP128);
                break;
#if SUPPORT_TKIP
        case CIPHER_SUITE_TKIP: /* TKIP */
                encryptResult = tkipHwMpduEncapsulate(priv_p, hw_tfcb_p,
                        (UCHAR) keyId, key_p);
                break;
#endif
        case CIPHER_SUITE_CCMP: /* AES_CCM */
                encryptResult = ccmpHwMpduEncapsulate(priv_p, hw_tfcb_p,
                        (UCHAR) keyId, key_p);
                break;
        default:
                KDBG_LOG_TX(KDBG_DEBUG, "Error, MPDU encryption using pairwise key with wrong cipher\n");
                KDBG_LOG_TX(KDBG_DEBUG, "Cipher suite %d\n", cipherSuite);
                encryptResult = ENCAPSULATE_ERROR;
        }

        return encryptResult;
}

/*_________________________________________________________________________
* Function   : TxMsduEncrypt
* Input      : priv_p - Pointer to the adapter object data area.
*              macHeader_p, pointer to frame MAC header
* Output:      resultFramebodyLen_p, pointer to put result of selected ciphersuite
*              key_pp, pointer to secret key address
*              status, pointer to result, the return value is key ID
* Return     :
* Description: this function decides whether the Tx MSDU is needed to be
*              encrypted or not
**__________________________________________________________________________*/
UINT8 TxMsduEncrypt(IN i2220mgmt_t *priv_p,
                 IN Frame_t *pdu,
                 IN sta_info_t *sta_p,
                 OUT UINT8 *keyId_p,
                 OUT UINT8 **key_pp)
{
        UINT8   keyId;
        UINT8   cipherSuite = (UINT8) CipherMode_NONE;

        /* Check if this packet is always not ciphered */
        if (!(pdu->mac_ctl & MACCTL_CHECK_CIPHER) || pdu->LLClength == 0) {
                return CipherMode_NONE;
        }

        /* Check if pairwise key should be used */
        if (sta_p && sta_p->KeyMapOn) {
                cipherSuite = sta_p->cipherSuite;
                *keyId_p = (UINT8) sta_p->keyId;
                *key_pp = &sta_p->encryptKey[0];
        }
        else if (priv_p->BCcipherSuite != CipherMode_NONE &&
                 !CheckFor1xFrame(pdu->LLCdata, pdu->LLClength) &&
                 !(pdu->mac_ctl & MACCTL_WDS_FRAME)) {
                /* Group key or default key setting. The 802.1X and WDS
                 * packet is encrypted only when pairwise key exists.
                 */
                cipherSuite = priv_p->BCcipherSuite;
                keyId = (UINT8) priv_p->DefaultKeyID;
                *keyId_p = keyId;
                *key_pp = &mib_import(
                        dot11WEPDefaultKeysTable[keyId].dot11WEPDefaultKeyValue[0]);
        } /* end of default key case */

        return cipherSuite;
}

/*_________________________________________________________________________
* Function   : RxMsduDecrypt
* Input      : priv_p - Pointer to the adapter object data area.
*              macHeader_p, pointer to frame MAC header
*              frameBody_p, pointer to frame body
*              frameBodyLen, length of frame body
*              keyId, key ID which is used to decrypt MSDU
* Output:      ciphersuite_p, pointer to put result of selected ciphersuite
* Return     : DECRYPT_SUCCESS, if the frame is decryptable
*              otherwise, if the frame is not decryptable. Then the caller
*              function should discard the related frame and return the RFB.
* Description: this function decides whether the received WEP frame needs to
*              be decrypted or not.
**__________________________________________________________________________*/
int RxMsduDecrypt(i2220mgmt_t *priv_p,
                IN sta_info_t *sta_p,
                IN Frame_t *pdu,
                OUT PUINT keyId_p,
                OUT PUCHAR *key_pp)
{
        int     result = FALSE; /* Not check TKIP MIC */

        /* Check if packet len is too short. If OK, subtract rxMIC */
        if (pdu->LLClength <= 8) {
                pdu->LLClength = 0;
                return result;
        }
        pdu->LLClength -= 8;

        /* Don't check rx MIC during guard time */
        if (check_timeout(priv_p->rekey_eol) ) {
                priv_p->rekey_eol = kal_jiffies; /* Avoid timer wrapping */

                /* Check if group key or pairwise key */
                if (sdl_isGroup(pdu->addr1)) {
                        *keyId_p = priv_p->DefaultKeyID;
                        *key_pp = &mib_import(
                                   dot11WEPDefaultKeysTable[priv_p->DefaultKeyID].
                                   dot11WEPDefaultKeyValue[0]);
                        result = TRUE;
                }
        #if RX_MIC_PARTIAL_CHECK
                else if (sta_p && sta_p->micTick < 10 && sta_p->KeyMapOn) {
                        sta_p->micTick ++;
                        *keyId_p = sta_p->keyId;
                        *key_pp = &sta_p->encryptKey[0];
                        result = TRUE;
                }
        #else
                else if (sta_p && sta_p->KeyMapOn) {
                        *keyId_p = sta_p->keyId;
                        *key_pp = &sta_p->encryptKey[0];
                        result = TRUE;
                }
        #endif
        }
        return result;
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
*
* \return
*/
/*----------------------------------------------------------------*/
BOOLEAN RxMpduDecrypt(i2220mgmt_t *priv_p, rfb_t *rfb_p, UINT32 cipherSuite)
{
#if DBG
        KDBG_LOG_INIT(KDBG_DEBUG, "Logging HW Key table.\n");
        IPN2220_MCR_READ(priv_p, MCR_SCPR0, &temp32);
        KDBG_LOG_RX(KDBG_DEBUG, "SCPR0: 0x%x\n", temp32);
        IPN2220_MCR_READ(priv_p, MCR_SKACR, &temp32);
        KDBG_LOG_RX(KDBG_DEBUG, "SKACR: 0x%x\n", temp32);
        for (i=0; i<8; i++) {
              KDBG_LOG_RX(KDBG_DEBUG, "Key %d: \n", i);
              IPN2220_MCR_READ(priv_p, MCR_SKACR, &temp32);
              temp32 = SKACR_ACCESS_START | (temp32 & SKACR_TABLE_SIZE) | i;
              IPN2220_MCR_WRITE(priv_p, MCR_SKACR, temp32);
              IPN2220_MCR_READ(priv_p, MCR_SKADR0, &temp32);
              KDBG_LOG_RX(KDBG_DEBUG, "SKADR0: 0x%x: \n", temp32);
              IPN2220_MCR_READ(priv_p, MCR_SKADR1, &temp32);
              KDBG_LOG_RX(KDBG_DEBUG, "SKADR1: 0x%x: \n", temp32);
              IPN2220_MCR_READ(priv_p, MCR_SKADR2, &temp32);
              KDBG_LOG_RX(KDBG_DEBUG, "SKADR2: 0x%x: \n", temp32);
              IPN2220_MCR_READ(priv_p, MCR_SKADR3, &temp32);
              KDBG_LOG_RX(KDBG_DEBUG, "SKADR3: 0x%x: \n", temp32);
              IPN2220_MCR_READ(priv_p, MCR_SKADR4, &temp32);
              KDBG_LOG_RX(KDBG_DEBUG, "SKADR4: 0x%x: \n", temp32);
              IPN2220_MCR_READ(priv_p, MCR_SKADR5, &temp32);
              KDBG_LOG_RX(KDBG_DEBUG, "SKADR5: 0x%x: \n", temp32);
          }
#endif

        /* H/W decryption */
        rfb_p->sw.secMode = cipherSuite;
        switch (cipherSuite) {
                case CIPHER_SUITE_WEP40 :
                case CIPHER_SUITE_WEP104 :
                case CIPHER_SUITE_WEP128 :
                        KDBG_LOG_RX(KDBG_DEBUG, " Decapsulate WEP packet by H/W. \n");
                        rfb_p->sw.keyId = (UCHAR)((rfb_p->hw.iv & BITS(30,31)) >> 30);
                        rfb_p->hw.rxByteCount -= 8; /* IV and ICV */
                     break;
        #if SUPPORT_TKIP
                case CIPHER_SUITE_TKIP :
                        KDBG_LOG_RX(KDBG_DEBUG, " Decapsulate TKIP packet by H/W. \n");
                        rfb_p->sw.keyId = (UCHAR)((rfb_p->hw.iv & BITS(30,31)) >> 30);
                        rfb_p->hw.rxByteCount -= 12; /* IV, EIV and ICV */
                     break;
        #endif
                case CIPHER_SUITE_CCMP :
                        KDBG_LOG_INIT(KDBG_DEBUG, " Decapsulate CCMP packet by H/W. \n");
                        rfb_p->sw.keyId = (UCHAR)((rfb_p->hw.iv & BITS(30,31)) >> 30);
                        rfb_p->hw.rxByteCount -= 16; /* IV, EIV and MIC */
                     break;
                default:
                        KDBG_LOG_RX(KDBG_WARN, " ***** Packet decapsulated error. *****. \n");
                        //decryptResult = DECAPSULATE_ERROR;
                     return FALSE;
        }
        return TRUE;
}


