/*
** $Id: tkip_sw.c 1.5 2004/07/15 09:49:06 cmchang Exp $
*/

/*! \file tkip_sw.c
    \brief This implementation is developed based on IEEE 802.11i draft 4.0
*/

/*************************************************************************
* Copyright (c) 2002 Inprocomm, Inc.
*
* All rights reserved. Copying, compilation, modification, distribution
* or any other use whatsoever of this material is strictly prohibited
* except in accordance with a Software License Agreement with
* Inprocomm, Inc.
**************************************************************************/

/************************************************************************
*                E X T E R N A L   R E F E R E N C E S
*************************************************************************
*/
#include "precomp.h"

/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/

/************************************************************************
*                             M A C R O S
*************************************************************************
*/
#define LO_8BITS(x)     ((x) & 0x00ff)  // obtain low 8-bit from 16-bit value, OK
#define HI_8BITS(x)     ((x) >> 8)      // obtain high 8-bit from 16-bit value, OK

#define ROTR32(x,y)     (((x) >> (y)) | ((x) << (32 - (y)))) // rotate-right 32-bit variable x y-bit, OK
#define ROTL32(x,y)     (((x) << (y)) | ((x) >> (32 - (y)))) // rotate-left 32-bit variable x y-bit, OK
#define ROTR16(x,y)     (((x) >> (y)) | ((x) << (16 - (y)))) // rotate-right 16-bit variable x y-bit, OK
#define ROTL16(x,y)     (((x) << (y)) | ((x) >> (16 - (y)))) // rotate-left 16-bit variable x y-bit, OK

#define XSWAP32(x)      ((((x) & 0xFF00FF00) >> 8) | (((x) & 0x00FF00FF) << 8))

/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/

/************************************************************************
*                        P U B L I C   D A T A
*************************************************************************
*/

/************************************************************************
*                       P R I V A T E   D A T A
*************************************************************************
*/

/************************************************************************
*              F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/

/************************************************************************
*                          F U N C T I O N S
*************************************************************************
*/

/*_________________________________________________________________________
** tkipMicB
**
** description: TKIP Michael block function
** parameters:  l_p, pointer to left value
**              r_p, pointer to right value
** return:
** note:
**__________________________________________________________________________*/
VOID
tkipMicB(IN OUT PUINT_32 l_p,
         IN OUT PUINT_32 r_p)
{
    *r_p = *r_p ^ ROTL32(*l_p, 17); // r <- r ^ (l<<<17)
    *l_p = (*l_p + *r_p);           // l <- (l+r) mod 2^32
    *r_p = *r_p ^ XSWAP32(*l_p);    // r <- r ^ XSWAP(l)
    *l_p = (*l_p + *r_p);           // l <- (l+r) mod 2^32
    *r_p = *r_p ^ ROTL32(*l_p, 3);  // r <- r ^ (l<<<3)
    *l_p = (*l_p + *r_p);           // l <- (l+r) mod 2^32
    *r_p = *r_p ^ ROTR32(*l_p, 2);  // r <- r ^ (l>>>2)
    *l_p = (*l_p + *r_p);           // l <- (l+r) mod 2^32
}   /* tkipMicB */

/*_________________________________________________________________________
** tkipSwGenMic
**
** description: TKIP Michael generation function
** parameters:  mickey_p, pointer to MIC key
**              data_p, pointer to message
**              dataLen, message length, in byte(s)
**              sa_p, pointer to source address SA
**              da_p, pointer to destination address DA
**              priority, priority of IEEE 802.11 traffic class
**              mic_p, pointer to 64-bit MIC
** return:
** note:
**__________________________________________________________________________*/
VOID
tkipSwGenMic(IN PUINT_8 mickey_p,
             IN PUINT_8 data_p,
             IN UINT_32 dataLen,
             IN PUINT_8 sa_p,
             IN PUINT_8 da_p,
             IN UINT_8 priority,
             OUT PUINT_8 mic_p)
{
    UINT i;
    UINT_32 l, r;
    UINT_32 msg[3];

    WLAN_GET_FIELD_32(mickey_p, &l);
    WLAN_GET_FIELD_32(mickey_p + 4, &r);

    /* Michael message processing for DA and SA. */
    WLAN_GET_FIELD_32(da_p, &msg[0]);
    msg[1] = ((UINT_32) da_p[4]) | ((UINT_32) da_p[5] << 8) |
        ((UINT_32) sa_p[0] << 16) | ((UINT_32) sa_p[1] << 24);
    WLAN_GET_FIELD_32(sa_p + 2, &msg[2]);
    for (i = 0; i < 3; i++) {
        l = l ^ msg[i];
        tkipMicB(&l, &r);
    }

    /* Michael message processing for priority. */
#if SUPPORT_WME
    msg[0] = (UINT_32) priority;
#else
    msg[0] = (UINT_32) priority << 24;
#endif
    l = l ^ msg[0];
    tkipMicB(&l, &r);

    /* Michael message processing for MSDU data playload except the last octets
       which cannot be partitioned into a 32-bit word. */
    for (i = 0; i < (UINT) dataLen / 4; i++) {
        WLAN_GET_FIELD_32(data_p + i * 4, &msg[0]);
        l = l ^ msg[0];
        tkipMicB(&l, &r);
    }

    /* Michael message processing for the last uncomplete octets, if present,
       and the padding. */
    switch (dataLen & 3) {
        case 1:
             msg[0] = ((UINT_32) data_p[dataLen - 1]) | 0x00005A00;
             break;

        case 2:
             msg[0] = ((UINT_32) data_p[dataLen - 2]) |
                 ((UINT_32) data_p[dataLen - 1] << 8) | 0x005A0000;
             break;

        case 3:
             msg[0] = ((UINT_32) data_p[dataLen - 3]) |
                 ((UINT_32) data_p[dataLen - 2] << 8) |
                 ((UINT_32) data_p[dataLen - 1] << 16) | 0x5A000000;
             break;

        default:
             msg[0] = 0x0000005A;
    }
    msg[1] = 0;
    for (i = 0; i < 2; i++) {
        l = l ^ msg[i];
        tkipMicB(&l, &r);
    }

    /* return ( l, r ), i.e. MIC */
    WLAN_SET_FIELD_32(mic_p, l);
    WLAN_SET_FIELD_32(mic_p + 4, r);
}   /* tkipSwGenMic */


