/*
** $Id: wds_staservice.c 1.9 2004/04/06 12:23:54 cmchang Exp $
*/
/*! \file wds_staservice.c
    \brief Handling WDS Association Task.

    This file concerns about WDS(Wireless Distribution System) Association Task. We expect AP will connect
    each other just like a station connecting with a AP. The only difference is we need append IE element to
    tell the other AP that we are AP.
*/

/******************************************************************************
Copyright (c) 2000-2002 Integrated Programmable Communications, Inc.

All rights reserved. Copying, compilation, modification, distribution
or any other use whatsoever of this material is strictly prohibited
except in accordance with a Software License Agreement with
Integrated Programmable Communications, Inc.
******************************************************************************/

#include "precomp.h"

/*Site survey table*/
struct site_info
{
        UCHAR   bssid[8];
        UCHAR   ssid[MAX_SSID_LEN+4];
        UCHAR   channel;
        UCHAR   sig_strength;
        /*Support rate??*/
};

struct ie
{
        unsigned char ie_id;
        unsigned char len;
        unsigned char data[0];
};

sta_info_t* sta_by_ip(UCHAR* ip_addr);
extern UINT8* __set_SupRate_InfoElem(UINT8 *cp);

struct site_info g_site_survey[MAX_SITESURVEY_AP];
int g_site_survey_index = 0;
/*----------------------------------------------------------------*/
/*!
* \brief  SDL service as a machine.
*
* This service handles Associate, Reassociate and Disassociate requests
* at non-AP stations. This service also generates responses for class 3
* errors and incoming (re)association requests.
*
* \param   sig_code  the FSM event code
* \return  SDL scheduling status code, not used in this AP implementation.
*/
/*----------------------------------------------------------------*/
int wds_deauth(i2220mgmt_t *priv_p, deauth_frame_t* pdu)
{
        int aid,status,i;
        sta_info_t* lan_sta_p;

        pdu             = va_arg(ap,deauth_frame_t*);
        lan_sta_p    = va_arg(ap,sta_info_t*);

        printk("LAN "MACSTR" is deauthenticated\n",MAC2STR(pdu->addr1));
        if((lan_sta_p = task_sta_lookup(pdu->addr1)))
                lan_sta_p ->flags = 0;

        return SDL_SUCCESSFUL;
}
/*----------------------------------------------------------------*/
/*!
* \brief  SDL process as a function.
*
* this service handles (De)Authenticate requests.
* This service also handles incoming the generation of responses for
* class 2 errors. this state machine handles Mlme requests sequentially,
* which is the simplest case. It is permissible to have several authentications
* in progress at once, provided the destination stations are all different.
* To support concurrent sequences this state machine gets collapsed
* into one state, with sequence state held in a variable. The local variables
* are replicated for each sequence, selected by responder address.
*
* \param   sig_code  the FSM event code
* \return  SDL scheduling status code, not used in this AP implementation.
*/
/*----------------------------------------------------------------*/
int wds_auth_rsp(i2220mgmt_t *priv_p, auth_frame_t* pdu)
{
        assoc_req_frame_t  *as_req=NULL;
        int arSeq, arAlg,status;
        MacAddr_t        my_mac;
        UINT8          *cp;
        eSsId_t        *eSsId_p;
        int                     i;
        sta_info_t*     lan_sta_p,*remote_ap;


        arSeq= ieee2host16(pdu->authTransSeqNo);
        arAlg = ieee2host16(pdu->authAlgNum);
        status = ieee2host16(pdu->statusCode);

        my_mac = pdu->addr1;
        remote_ap = task_sta_lookup(priv_p->wds->remote_addr);

                if((lan_sta_p = task_sta_lookup(my_mac)) && !(lan_sta_p ->flags & STA_INFO_ASSOC) && remote_ap)
                        ;
                else
                        return;

                if(status != SYN_success){
                        printf("Auth fail %d %d %d  from "MACSTR" to "MACSTR"\n",arSeq,arAlg,status,MAC2STR(pdu->addr2),MAC2STR(pdu->addr1));
                        if(lan_sta_p->flags & STA_INFO_AUTH_OS)
                                remote_ap->wds_link->auth_type = SYN_shared_key;
                        else
                                remote_ap->wds_link->auth_type = SYN_open_system;

                        start_auth_ap(remote_ap,lan_sta_p->asoc.adAddr);
                }else{
                        if(memcmp(my_mac,mib_import(dot11MACAddress), ETH_ALEN)==0)
                                priv_p->wds->assoc_state = SYN_auth_open;

                        switch(arAlg){
                                case SYN_open_system:
                                        if(arSeq == 2){
                                                lan_sta_p ->flags = STA_INFO_AUTH_OS;
                                                start_assoc_ap(remote_ap,my_mac);
                                        }else
                                                lan_sta_p ->flags =SYN_not_auth;
                                        break;
                                case SYN_shared_key:
                                        if(arSeq == 2){
                                                lan_sta_p ->flags = STA_INFO_AUTH_KEY;
                                                wep_auth_ap(remote_ap,pdu);
                                                }
                                        else if(arSeq == 4){
                                                lan_sta_p ->flags = STA_INFO_AUTH_KEY;
                                                start_assoc_ap(remote_ap,my_mac);
                                                }
                                        else{
                                                lan_sta_p ->flags =SYN_not_auth;
                                                }
                                        break;
                                }
                }
        return SDL_SUCCESSFUL;
}

/*----------------------------------------------------------------*/
/*!
* \brief  SDL service as a machine.
*
* This service handles Associate, Reassociate and Disassociate requests
* at non-AP stations. This service also generates responses for class 3
* errors and incoming (re)association requests.
*
* \param   sig_code  the FSM event code
* \return  SDL scheduling status code, not used in this AP implementation.
*/
/*----------------------------------------------------------------*/
int wds_assoc(i2220mgmt_t *priv_p, assoc_rsp_frame_t* pdu)
{
        assoc_rsp_frame_t* pdu;
        int aid,status,i;
        sta_info_t* lan_sta_p;
        UINT32        tmpUlong;

        status = ieee2host16(pdu->statusCode);
        aid = ieee2host16(pdu->assocId);

        if(status == 0){
                if(memcmp(pdu->addr1, sdl_import(mBssId),ETH_ALEN)==0){
                        priv_p->wds->assoc_state = SYN_asoc;
                        priv_p->wds->aid = aid;
                }

                if(lan_sta_p = task_sta_lookup(pdu->addr1)){
                        lan_sta_p ->flags = STA_INFO_ASSOC;
                }
        }
        return SDL_SUCCESSFUL;
}

/*----------------------------------------------------------------*/
/*!
* \brief  SDL service as a machine.
*
* This service handles Associate, Reassociate and Disassociate requests
* at non-AP stations. This service also generates responses for class 3
* errors and incoming (re)association requests.
*
* \param   sig_code  the FSM event code
* \return  SDL scheduling status code, not used in this AP implementation.
*/
/*----------------------------------------------------------------*/
int wds_disassoc(i2220mgmt_t *priv_p, assoc_rsp_frame_t* pdu)
{
        assoc_rsp_frame_t* pdu;
        int aid,status,i;
        sta_info_t* lan_sta_p;
        UINT32        tmpUlong;

                 if((lan_sta_p = task_sta_lookup(pdu->addr1)))
                        lan_sta_p ->flags = 0;/*back to beginning*/

        return SDL_SUCCESSFUL;
}
/*----------------------------------------------------------------*/
/*!
* \brief  Utility function to send associate packet to remote AP.
*
*   \param priv_p  The remote ap you wish to delete.
*   \param wds             Related wdslink_info.
*   \return  Return successful or not.
*/
/*----------------------------------------------------------------*/
void wds_connect_remote_ap(i2220mgmt_t* priv_p, wdslink_info_t* wds)
{
        int i;
        sta_info_t*     remote_ap_p;

        /*We should do some thing to distinguish signal strength*/
        for(i=0;i<MAX_SITESURVEY_AP;i++){
                if(memcmp(g_site_survey[i].bssid,wds->remote_addr)==0){
                        remote_ap_p = sdlu_query_sta(priv_p, wds->remote_addr);
                        remote_ap_p->wds_link = wds;
                        if(g_site_survey[i].channel != priv_p->curChannel){
                                priv_p->curChannel = g_site_survey[i].channel;
                                MlmeStart_request(priv_p);
                        }
                        start_auth_ap(priv_p, remote_ap_p, priv_p->mib.dot11MACAddress);
                        return;
                }
        }

        /*We could not find the matching mac address.*/
        for(i=0;i<MAX_SITESURVEY_AP;i++){
                if(strcmp(g_site_survey[i].ssid,wds->bdSsId)==0){
                        memcpy(wds->remote_addr,g_site_survey[i].bssid,MAC_ADDR_LEN);
                        remote_ap_p = sdlu_query_sta(priv_p, wds->remote_addr);
                        if(remote_ap_p->wds_link)
                                continue;
                        remote_ap_p->wds_link = wds;
                        start_auth_ap(priv_p, remote_ap_p, priv_p->mib.dot11MACAddress);
                }
        };
        priv_p->need_site_survey = true;
};

/*----------------------------------------------------------------*/
/*!
* \brief  Utility function to send associate packet to remote AP.
*
*   \param remote_addr  The remote ap you wish to delete.
*   \param name             Related wdslink_info.
*   \return  Return successful or not.
*/
/*----------------------------------------------------------------*/
void start_auth_ap(i2220mgmt_t *priv_p,sta_info_t* remote_ap_p, UCHAR* assoc_mac)
{
        auth_frame_t  *arSdu= NULL;
        UCHAR        my_mac[6];
        wdslink_info_t* wds   = NULL;
        MacAddr_t        remote_addr = NULL;
        UINT32        tmpUlong;

        if (!remote_ap_p || !remote_ap_p->wds_link)
                return;

        wds   = remote_ap_p->wds_link;
        remote_addr = wds->remote_addr;
        memcpy(my_mac,assoc_mac,ETH_ALEN);

        sdl_mkGenFrame(arSdu,auth_frame_t,SYN_auth,remote_addr,my_mac,remote_addr);
        arSdu->authAlgNum = host2ieee16(wds->auth_type);
        arSdu->authTransSeqNo = host2ieee16(1);
        arSdu->statusCode = host2ieee16(0);
        arSdu ->LLClength = sMacHdrLng + 6;

        sdu_send_request(priv_p, (Frame_t *)arSdu, remote_ap_p, SYN_contention);
#if 0
        /*How do we set-up timer?*/
        sdl_set_timer( &sta_p->auth_timer
                , TU2JIFFIES(mib_import(dot11AuthenticationResponseTimeOut))
                , SIG_Tauth
                , sta_p, sdlf_AuthReqService_STA );
#endif
};
/*----------------------------------------------------------------*/
/*!
* \brief  infra_auth_timeout Handling station mode authenticate time out.*
*   \param remote_addr  The remote ap you wish to delete.
*   \param name             Related wdslink_info.
*   \return  Return successful or not.
*/
/*----------------------------------------------------------------*/
void infra_auth_timeout(i2120sta_mgmt_t* priv_p)
{
        IOLog("infra_auth_timeout\n");
        if((priv_p->wds->assoc_state == SYN_auth_open) ||(priv_p->wds->assoc_state == SYN_asoc))
                return;

        start_auth_ap(priv_p->wds->remote_ap, priv_p->mib.dot11MACAddress);
};
/*----------------------------------------------------------------*/
/*!
* \brief  Utility function to send associate packet to remote AP.
*
*   \param remote_addr  The remote ap you wish to delete.
*   \param name             Related wdslink_info.
*   \return  Return successful or not.
*/
/*----------------------------------------------------------------*/
void start_assoc_ap(i2220mgmt_t *priv_p,sta_info_t* remote_ap_p, UCHAR* assoc_mac)
{
        MacAddr_t my_mac;
        wdslink_info_t* wds;
        assoc_req_frame_t  *as_req=NULL;
        UINT8          *cp;
        eSsId_t        *eSsId_p;

        my_mac = assoc_mac;

        if(!remote_ap_p || !remote_ap_p->wds_link)
                return;

        wds=remote_ap_p->wds_link;
        sdl_mkGenFrame(as_req,assoc_req_frame_t,SYN_asoc_req,wds->remote_addr,my_mac,wds->remote_addr);
        as_req->listenInterval = host2ieee16(mib_import(dot11BeaconPeriod)); /* Beacon interval */
        as_req->capInfo = host2ieee16(sdl_import(mCap));   /* Capability information */
        /* Information elements */

        cp = &as_req->infoElem[0];
        memset(cp,0,sizeof(as_req->infoElem));

        sdl_mkElem( cp, eSsId, strlen(wds->bdSsId));
        memcpy( eSsId_p->ssid, wds->bdSsId, strlen(wds->bdSsId) );
        cp = __set_SupRate_InfoElem(cp);

        // set the frame length accordingly
        as_req->LLClength = sMacHdrLng + 4 + (cp - &as_req->infoElem[0]);
        sdu_send_request(priv_p, (Frame_t *)as_req, remote_ap_p, SYN_contention);
#if 0
        sdl_set_timer( &sta_p->auth_timer
                , TU2JIFFIES(mib_import(dot11AuthenticationResponseTimeOut))
                , SIG_Tauth
                , sta_p, sdlf_AsocService_STA );
#endif
};
/*----------------------------------------------------------------*/
/*!
* \brief  infra_auth_timeout Handling station mode authenticate time out.*
*   \param remote_addr  The remote ap you wish to delete.
*   \param name             Related wdslink_info.
*   \return  Return successful or not.
*/
/*----------------------------------------------------------------*/
void infra_assoc_timeout(i2220mgmt_t* priv_p)
{
        IOLog("infra_auth_timeout\n");
        if(priv_p->wds->assoc_state == SYN_asoc)
                return;

        if(priv_p->wds->assoc_state == SYN_auth_open)
                start_assoc_ap(priv_p->wds->remote_ap, priv_p->mib.dot11MACAddress);
        else
                start_auth_ap(priv_p->wds->remote_ap, priv_p->mib.dot11MACAddress);
};
/*----------------------------------------------------------------*/
/*!
* \brief  Utility function to send wep authentication packet 2 back to remote AP.
*
*   \param sta_p  The remote ap.
*   \param pdu    Received encrped.
*   \return  Return successful or not.
*/
/*----------------------------------------------------------------*/
void wep_auth_ap(i2220mgmt_t *priv_p,sta_info_t* remote_ap_p,  auth_frame_t* pdu)
{
        auth_frame_t  *arSdu= NULL;
        MacAddr_t        my_mac = mib_import(dot11MACAddress);
        MacAddr_t        remote_addr = NULL;
        wdslink_info_t* wds   = NULL;

        if (!remote_ap_p || !remote_ap_p->wds_link)
                return;

        wds   = remote_ap_p->wds_link;
        remote_addr = wds->remote_addr;

        sdl_mkGenFrame(arSdu,auth_frame_t,SYN_auth,remote_addr,my_mac,remote_addr);
        arSdu->authAlgNum = host2ieee16(SYN_shared_key);
        arSdu->authTransSeqNo = host2ieee16(3);
        arSdu->statusCode = host2ieee16(0);
        memcpy( &(arSdu->ElementID), &(pdu->ElementID), 2 +256);
        arSdu->LLClength = sMacHdrLng + 264;
        sdl_setWepBit(arSdu,1);
        sdu_send_request(priv_p, (Frame_t *)arSdu, remote_ap_p, SYN_contention);

        /*
        sdl_set_timer( &sta_p->auth_timer
                , TU2JIFFIES(mib_import(dot11AuthenticationResponseTimeOut))
                , SIG_Tauth
                , sta_p, sdlf_AuthReqService_STA );*/
}

/*
    IP -based AP Client Related Functions
*/
/*
*  Manuplaing frame when sending to remote_ap
*/
void wds_apclient_to_ap(Frame_t* rsdu, MacAddr_t addr)
{
        UINT8           *llc_p;
        int             type;
        sta_info_t      *sta_p;

        llc_p =  kal_skb_data(rsdu->skb);
        if (memcmp(llc_p,rfc1042_header,sizeof(rfc1042_header)) == 0) {
                /* If the LLC is RFC1042 protocol, we need to check the Type field in
                   the LLC against the Selective Translation Table.
                   Currently, there are two types in the Selective Translation Table
                   according to the WiFi's requirements: AppleTalk ARP (0x80F3) and
                   DIX II IPX (0x8137) */
                llc_p += sizeof(rfc1042_header);
                type = kal_ntohs( *(UINT16*)llc_p);
                if (type == ETH_P_ARP) {
                        /* If the type is included in the Selective Translation Table, the
                           802.11 MAC is replaced by the ISO/IEC 8802-3 MAC below the LLC.
                        */
                        llc_p += 8;
                        type = kal_ntohs( *(UINT16*)llc_p);
                        if(type == 0x0001 | type == 0x0002)
                                {
                                llc_p += 2;

                                //if this mac_address is one of our stations
                                if(sta_p=task_sta_lookup(llc_p))
                                        {
                                        llc_p[0] = addr[0];
                                        llc_p[1] = addr[1];
                                        llc_p[2] = addr[2];
                                        llc_p[3] = addr[3];
                                        llc_p[4] = addr[4];
                                        llc_p[5] = addr[5];
                                        sta_p->ip_addr[0]= llc_p[6];
                                        sta_p->ip_addr[1]= llc_p[7];
                                        sta_p->ip_addr[2]= llc_p[8];
                                        sta_p->ip_addr[3]= llc_p[9];
                                        }
                                llc_p += 10;
                                if(task_sta_lookup(llc_p))
                                        {
                                        llc_p[0] = addr[0];
                                        llc_p[1] = addr[1];
                                        llc_p[2] = addr[2];
                                        llc_p[3] = addr[3];
                                        llc_p[4] = addr[4];
                                        llc_p[5] = addr[5];
                                        sta_p->ip_addr[0]= llc_p[6];
                                        sta_p->ip_addr[1]= llc_p[7];
                                        sta_p->ip_addr[2]= llc_p[8];
                                        sta_p->ip_addr[3]= llc_p[9];
                                        }
                                }
                }
        } else {
                /* If the LLC is neither Bridge-Tunnel Encapsulation Protocol nor
                   RFC1042 protocol, the 802.11 MAC is replaced by the ISO/IEC 8802-3
                   MAC below the LLC. */
                KDBG_LOG_RX(KDBG_WARN,"wds_apclient_to_ap: Other LLC\n");
        }   /* end of check the LLC header */
}

/*
*  Manuplaing frame when sending to stations
*/
void wds_apclient_to_sta(Frame_t* rsdu, MacAddr_t addr)
{
        UINT8           *llc_p;
        int type;
        sta_info_t*      sta_p;

        llc_p =  rsdu->skb->data + sizeof(Frame_t);
        if (memcmp(llc_p,rfc1042_header,sizeof(rfc1042_header)) == 0) {
                /* If the LLC is RFC1042 protocol, we need to check the Type field in
                   the LLC against the Selective Translation Table.
                   Currently, there are two types in the Selective Translation Table
                   according to the WiFi's requirements: AppleTalk ARP (0x80F3) and
                   DIX II IPX (0x8137) */
                llc_p += sizeof(rfc1042_header);
                type = kal_ntohs( *(UINT16*)llc_p);
		printk(KERN_ALERT "type:%d\n",type);

                if (type == ETH_P_IP) {
                        /* If the type is included in the Selective Translation Table, the
                           802.11 MAC is replaced by the ISO/IEC 8802-3 MAC below the LLC.
                        */
                        printk(KERN_ALERT "wds_apclient_to_ap:IP Frame\n");
                        llc_p += 2;
                        printk(KERN_ALERT "version:0x%08x\n",*llc_p);
                        llc_p += 12;
                        printk(KERN_ALERT "Source IP: %d.%d.%d.%d\n",llc_p[0],llc_p[1],llc_p[2],llc_p[3]);
                        llc_p += 4;
                        printk(KERN_ALERT "Dest IP: %d.%d.%d.%d\n",llc_p[0],llc_p[1],llc_p[2],llc_p[3]);
                        if(sta_p=sta_by_ip(llc_p))
                                {
                                printk(KERN_ALERT "Dest MAC"MACSTR"\n",MAC2STR(sta_p->asoc.adAddr));
                                rsdu->addr1[0] = sta_p->asoc.adAddr[0];
                                rsdu->addr1[1] = sta_p->asoc.adAddr[1];
                                rsdu->addr1[2] = sta_p->asoc.adAddr[2];
                                rsdu->addr1[3] = sta_p->asoc.adAddr[3];
                                rsdu->addr1[4] = sta_p->asoc.adAddr[4];
                                rsdu->addr1[5] = sta_p->asoc.adAddr[5];
                                }
                        }
                else if(type == ETH_P_ARP)
                        {
                        printk(KERN_ALERT "wds_apclient_to_ap:ARP Frame\n");
                        rsdu->addr1[0] = 0xff;
                        rsdu->addr1[1] = 0xff;
                        rsdu->addr1[2] = 0xff;
                        rsdu->addr1[3] = 0xff;
                        rsdu->addr1[4] = 0xff;
                        rsdu->addr1[5] = 0xff;
                        }
                }
        else {
                /* If the LLC is neither Bridge-Tunnel Encapsulation Protocol nor
                   RFC1042 protocol, the 802.11 MAC is replaced by the ISO/IEC 8802-3
                   MAC below the LLC. */
                KDBG_LOG_RX(KDBG_WARN,"wds_apclient_to_ap: Other LLC\n");
        }   /* end of check the LLC header */
}

/*
*  Return sta_info_t by its IP address
*/
sta_info_t* sta_by_ip(UCHAR* ip_addr)
{
        sta_info_t* sta_p;
        int i;

        for (i=0; i<x_MAX_AID; i++) {
                sta_p = sdl_this->ap.sta_aid[i];
                if (sta_p) {
                        if(memcmp(sta_p->ip_addr,ip_addr,4)==0)
                                return sta_p;
                        }
                }
        return NULL;
}


/*
    Site Survey Related Functions
*/
int find_sitesurvey_record(UCHAR* mac,struct site_info* wls,int num){

        int i;

        for(i = 0 ; i< num; i++ ){
                if(memcmp(mac,wls[i].bssid,ETH_ALEN) == 0)
                        return i;
        }
        return -1;
};
int find_available_sitesurvey_record(struct site_info* wls,int start){
        int i;

        for(i = start ; i< MAX_SITESURVEY_AP; i++ ){
                if((wls[i].bssid[0] == 0) &&
                        (wls[i].bssid[1] == 0) &&
                        (wls[i].bssid[2] == 0) &&
                        (wls[i].bssid[3] == 0) &&
                        (wls[i].bssid[4] == 0) &&
                        (wls[i].bssid[5] == 0))
                        return i;
        }
        return -1;
};
void DUMP_SITE_SURVEY(struct site_info* g_site_survey){
        int i;

        for(i = 0 ; i< MAX_SITESURVEY_AP; i++ ){
                printf("No:%d "MACSTR" SSID: %s CNL: %d\n",
                        i,MAC2STR(g_site_survey[i].bssid),g_site_survey[i].ssid,g_site_survey[i].channel);
        }
};
/*
*        Update Site Survey information
*/
/*----------------------------------------------------------------*/
/*!
* \brief  Utility function to broadcast probe packet .
*
*   \param assoc_mac  Local mac_address.
*   \return  Return successful or not.
*/
/*----------------------------------------------------------------*/
void start_probe_request_ap(i2220mgmt_t* priv_p, bool fixed_ssid)
{
        probe_req_frame_t* pq_sdu= NULL;
        unsigned char*         cp;
        eSsId_t        *eSsId_p;
        Octet_t*       ssid;
        int                i,str_len;
        char*           empty_str = "";
        struct ie*    ie_p;

        if(!priv_p)
                return;

        sdl_mkGenFrame(pq_sdu,probe_req_frame_t,SYN_probe_req,
                bcstAddr,priv_p->mib.dot11MACAddress,bcstAddr);

        /* Information elements */
        cp = &pq_sdu->infoElem[0];
        memset(cp,0,sizeof(pq_sdu->infoElem));

        if(fixed_ssid){
                ssid = priv_p->mib.dot11DesiredSSID.ssid;
                str_len = strlen(ssid);
                sdl_mkElem( cp, eSsId, str_len);
                memcpy( eSsId_p->ssid,ssid, str_len);
        }
        else{
                ie_p = (struct ie *)cp;
                ie_p->ie_id = SYN_eSsId;
                ie_p->len    = 0;
                cp += 2;
        }

        cp = __set_SupRate_InfoElem(cp);

        pq_sdu->LLClength = 24  + (unsigned long)(cp - &pq_sdu->infoElem[0]);
        sdu_send_request(priv_p, (Frame_t *)pq_sdu, NULL, SYN_contention);
};
void wds_handle_beacon_probersp(i2220mgmt_t *priv_p, probe_rsp_frame_t* pdu)
{
        unsigned char* cp;
        struct ie*        ie_p;
        bool        ssid_comp = false;
        int          channel_comp = 0;
        sta_info_t *     sta_p;
        char        temp_ssid[64];
        int             i = 0;
        int           show_ssid = false;
        UINT32    bssid_mac;
        UINT32    value32;

        if(priv_p->need_site_survey== true){
                i = find_sitesurvey_record(pdu->addr2,g_site_survey,g_site_survey_index);

                if(i == -1){
                        i = find_available_sitesurvey_record(g_site_survey,g_site_survey_index);
                        if(i == -1)
                                i = 0;//Too many access point, I don't know what to do.
                        memcpy(g_site_survey[i].bssid,pdu->addr2,ETH_ALEN);
                        show_ssid = true;
                        g_site_survey_index++;
                }else
                        return;//This means we have seen this beacon before!

                /*
                        If use has determined the remote AP's MAC address, then
                        we just compare mac address.
                */
                if(sdl_isGroup(priv_p->wds->remote_addr) ||
                        ((priv_p->wds->remote_addr[0]==0x00) &&
                        (priv_p->wds->remote_addr[1]==0x00) &&
                        (priv_p->wds->remote_addr[2]==0x00) &&
                        (priv_p->wds->remote_addr[3]==0x00) &&
                        (priv_p->wds->remote_addr[4]==0x00) &&
                        (priv_p->wds->remote_addr[5]==0x00) ))
                        ;
                 else if(memcmp(priv_p->wds->remote_addr,pdu->addr2,ETH_ALEN)==0){
                        ssid_comp = true;
                 }

                cp = ((unsigned char*)(tcast_wlan_header(pdu)))+ 24 + 12;

                while(cp <= ((unsigned char*)(tcast_wlan_header(pdu))+ pdu->LLClength)){
                        ie_p = (struct ie*)cp;

                        switch(ie_p ->ie_id){
                                case SYN_eSsId:
                                        memcpy(temp_ssid,ie_p->data,ie_p->len);
                                        temp_ssid[ie_p->len]='\0';

                                        if(strlen(mib_import(dot11DesiredSSID.ssid)) == ie_p->len)
                                                if((memcmp(mib_import(dot11DesiredSSID.ssid),ie_p->data,ie_p->len) == 0) )
                                                        ssid_comp = true;

                                        memcpy(g_site_survey[i].ssid,ie_p->data,ie_p->len);
                                        g_site_survey[i].ssid[ie_p->len] = '\0';

                                break;
                         case SYN_eSupRates:
                                break;
                         case SYN_eDsParms:
                                channel_comp = ie_p->data[0];
                                g_site_survey[i].channel = ie_p->data[0];
                                break;
                         default:
                                break;
                        }
                        cp = cp + ie_p->len + 2;
                }

                /*
                        If beacon's SSID and its channel is the same as we wish, then we send auth with it.
                */
                if(ssid_comp && (channel_comp == priv_p->mib.dot11CurrentChannel)){
                        sta_info_t *     remote_ap_p;
                        sta_info_t*      this_ap;

                        priv_p->need_site_survey = false;

                        remote_ap_p = sdlu_query_sta(pdu->addr2);
                        remote_ap_p->wds_link   = priv_p->wds;
                        memcpy(sta_p->wds_link->remote_addr,pdu->addr2, ETH_ALEN);
                        strcpy(priv_p->wds->bdSsId, temp_ssid);
                        remote_ap_p->wds_link->auth_type=SYN_open_system;
                        remote_ap_p->wds_link->role = IW_MODE_INFRA;//we are station to remote AP.
                        remote_ap_p->role = IW_MODE_MASTER;//it's an AP

                        /*This should be left in associate, but I have not refined it.*/
                        remote_ap_p->tx_max_rate = RATE_11M;
                        remote_ap_p->curTxRate   = RATE_11M;

                        start_auth_ap(remote_ap_p,priv_p->mib.dot11MACAddress);
                   }
            } else{
                        wdslink_info_t* wds;
                        wds = wds_by_mac(pdu->addr2);
                        /*Keep watch on Beacon count to determine if this Access Point is alive*/
                        if(wds)
                                wds->ap_beacon_count++;
                }
};
void init_sitesurvey_record(struct site_info* ss){
        memset(ss, 0, sizeof(struct site_info) *MAX_SITESURVEY_AP);
};
void sv_timer_func(unsigned long data)
{
        static int init_channel;
        static int init =true;
        i2220mgmt_t* priv_p = (i2220mgmt_t*)data;

        if(priv_p->iw_mode != NIC_STATION)
                return;

        if(priv_p->need_site_survey == true){
                /*
                        init == true, means this is a whole new round of site survey
                */
                if(init){
                        init_channel = priv_p->mib.dot11CurrentChannel;
                        init = false;
                        g_site_survey_index = 0;
                }
                /*
                        init == false, means we are in the middle of site survey.
                        if init_channel equals current channel, that means we go
                        back to the beginning channel. Site survey is complete.
                */
                else if(init_channel == priv_p->mib.dot11CurrentChannel){
                        init = true;
                        priv_p->need_site_survey == false;
                        return;
                }

                priv_p->mib.dot11CurrentChannel ++;
                if(priv_p->mib.dot11CurrentChannel > 14)
                        priv_p->mib.dot11CurrentChannel -= 14;

                MlmeStart_request(priv_p);
                start_probe_request_ap(priv_p,false);
        }else{
                /*
                        How do we monitor a dead remote Access Point?
                */
                #if 0
                if(priv_p->wds->last_ap_beacon_count == priv_p->wds->ap_beacon_count){
                        priv_p->wds->assoc_state == SYN_not_auth;
                        priv_p->need_site_survey == true = false;
                        memset(priv_p->wds->remote_addr,0,ETH_ALEN);
                        init_sitesurvey_record(g_site_survey);
                }else
                        priv_p->wds->last_ap_beacon_count = priv_p->wds->ap_beacon_count;
                #endif
        }
};

void authenticate_timer_func(unsigned long data)
{
        sta_info_t* sta_p =NULL;
        int                       i;
        UCHAR      addr[ETH_ALEN];
        sta_info_t **sta_pp, *lan_sta_p;

        if(g_ap_found){
                sta_p = task_sta_lookup(sdl_this->wds->remote_addr);
                addr[0]=0;addr[1]=0;addr[2]=0;addr[3]=0;addr[4]=0;addr[5]=0;

                if(sta_p){
                        sta_pp = &sdl_this->ap.sta_aid[0];

                        for (i=1; i<=x_MAX_AID; i++,sta_pp++) {
                                lan_sta_p = *sta_pp;
                                if (lan_sta_p) {
                                        if(lan_sta_p->flags & STA_INFO_ASSOC)
                                                continue;
                                        if(lan_sta_p->flags & (STA_INFO_AUTH_KEY | STA_INFO_AUTH_OS))
                                                {
//                                                printf("Auth Timer: asoc"MACSTR"\n",MAC2STR(lan_sta_p->asoc.adAddr));
//                                                start_assoc_ap(sta_p,lan_sta_p->asoc.adAddr);
                                                }
                                        else {
                                                printf("Auth Timer: auth"MACSTR"\n",MAC2STR(lan_sta_p->asoc.adAddr));
                                                start_auth_ap(sta_p,lan_sta_p->asoc.adAddr);
                                                }
                                }
                        }
                }
        }
};

