/*
** $Id: ap_dbg.h 1.9 2004/01/12 20:37:17 chyeh Exp $
*/

/******************************************************************************
Copyright (c) 2002-2003 Integrated Programmable Communications, Inc.

All rights reserved. Copying, compilation, modification, distribution
or any other use whatsoever of this material is strictly prohibited
except in accordance with a Software License Agreement with
Integrated Programmable Communications, Inc.
******************************************************************************/

#ifndef _I2220DBG_H
#define _I2220DBG_H

/************************************************************************
*                     C O M P I L E R   F L A G S
*************************************************************************
*/

/************************************************************************
*                E X T E R N A L   R E F E R E N C E S
*************************************************************************
*/

/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/

/************************************************************************
*           Special flag for dumping RX/TX packets
*************************************************************************
*/

#define DBFLG_RX_SHOW_REG           BIT(31)
#define DBFLG_RX_RFB                BIT(30)
#define DBFLG_RX_HEADER             BIT(29)
#define DBFLG_RX_ALL_FRAME          BIT(28)

#define DBFLG_TX_SHOW_REG           BIT(31)
#define DBFLG_TX_TFCB               BIT(30)
#define DBFLG_TX_TBB                BIT(29)
#define DBFLG_TX_ALL_FRAME          BIT(28)
#define DBFLG_TX_HEADER             BIT(27)

#define DBFLG_RX    (DBFLG_RX_SHOW_REG | DBFLG_RX_RFB | \
                     DBFLG_RX_HEADER | DBFLG_RX_ALL_FRAME)

/*------------------------------------------------------------
 *      InProComm 2220 DEBUG enable bit definitions
 *------------------------------------------------------------
 * x_kdbg_gate: nibble mapped debugging control variable 1
 *      controls whether printk of debugging message will be invoked
 *      expanding to g_kdbg_gate_0 ~ g_kdbg_gate_7 for each category.
 * x_kdbg_prtk: nibble mapped debugging control variable 2
 *      controls the printk log level <n>
 *      expanding to g_kdbg_prtk_0 ~ g_kdbg_prtk_7 for each category.
 *------------------------------------------------------------
 *      0x0000000n  ==>  driver initialization / tracking category
 *                       KDBG_ENTER/EXIT KDBG_FSM_ENTER KDBG_LOG_INIT
 *      0x000000n0  ==>  TX data path category
 *                       KDBG_LOG_TX
 *      0x00000n00  ==>  RX data path category
 *                       KDBG_LOG_RX
 *      0x0000n000  ==>  SDL hypothetical architecture or management category
 *                       KDBG_LOG_SDL
 *      0x000n0000  ==>  System Resource Category
 *                       such as kmalloc, sk_buff, list ...
 *                       and interrupt
 *                       KDBG_LOG_SYS KDBG_LOG_INT KDBG_LOG_UTIL
 *      0x00n00000  ==>  MLME (ioctl) category
 *                       and SDL env_XXX supporting category
 *                       and debugging facility for Windows Driver
 *                       KDBG_LOG_MLME KDBG_LOG_ENV KDBG_LOG_WIN
 *      0x0n000000  ==>  temporary category
 *                       We never staticly use this category, but only as a temporary
 *                       provision to watch some kind of messages when implementing
 *                       new code or tracking some special BUGs.
 *                       KDBG_LOG_TEMP
 *      oxn0000000  ==>  debugging thread category
 *                       This category can also serve for debugging thread.
 *                       %%NOTE%% They should be erased or #if 0 blocked when job done.
 *
 *------------------------------------------------------------
 * n, the smaller, the more severe
 *  <0> KDBG_CRIT     - Used for emergency messages, usually those that precede a crash.
 *                    - A situation requiring immediate action.
 *                    - Critical conditions, often related to serious hardware or
 *                      software failures.
 *                    - For example, memory shortage
 *  <1> KDBG_ERR      - Used to report error conditions; device drivers will often
 *                      use KDBG_ERR to report hardware difficulties.
 *  <2> KDBG_WARN     - Warnings about problematic situations that do not, in
 *                      themselves, create serious problems with the system.
 *  <3> KDBG_NOTICE   - Situations that are normal, but still worthy of note.
 *                      A number of security related conditions are reported at
 *                      this level.
 *  <4> KDBG_INFO     - Informational messages. Many drivers print information about
 *                      the hardware they find at startup time at this level.
 *  <5> KDBG_DEBUG    - Used for debugging messages.
 *------------------------------------------------------------
 * For example:
 *      KDBG_LOG_TX(5,"the value of var is %d", var)
 * It will get 'printk' to be executed only when
 *      g_kdbg_gate_1 > 5
 * However, the 'printk' debug level, is controlled by
 *      (x_kdbg_prtk & 0x000000f0)
 * say, if it is 3, then the message will become "<3>the value ..."
 * Therefore, 'console_loglevel', only when it is 4 or above, will this
 * message be printed to CONSOLE.
 *------------------------------------------------------------
 * Kernel printk controlling variables:
 *      console_loglevel, default_message_loglevel,
 *      minimum_console_loglevel, default_console_loglevel
 *------------------------------------------------------------*/

#define KDBG_CATEGORY_MASK(cat)     ( 0x0f << (4*cat) )   /* cat: 0 ~ 7 */
#define KDBG_CATEGORY_VAL(cat,v)    ( ((v) & KDBG_CATEGORY_MASK(cat)) >> (4*cat) )

/* generic KDBG level (severities) */
#define KDBG_CRIT           0       /* severity levels of debugging message */
#define KDBG_ERR            1
#define KDBG_WARN           2
#define KDBG_NOTICE         3
#define KDBG_INFO           4
#define KDBG_DEBUG          5

/* special purpose KDBG level */
#define KDBG_KALMEM         4
#define KDBG_PERF           5       /* Performance measurement */
#define KDBG_SDL_TRACK      6
#define KDBG_TRACK          7
#define KDBG_FRAME          8
#define KDBG_RAW_DUMP       8
#define KDBG_TXHW           8
#define KDBG_RXHW           8
#define KDBG_FREQ           6       /* show up very frequently */

/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/

/************************************************************************
*                        P U B L I C   D A T A
*************************************************************************
*/
#if WLAN_INCLUDE_DEBUG
extern int g_kdbg_gate_0, g_kdbg_prtk_0;
extern int g_kdbg_gate_1, g_kdbg_prtk_1;
extern int g_kdbg_gate_2, g_kdbg_prtk_2;
extern int g_kdbg_gate_3, g_kdbg_prtk_3;
extern int g_kdbg_gate_4, g_kdbg_prtk_4;
extern int g_kdbg_gate_5, g_kdbg_prtk_5;
extern int g_kdbg_gate_6, g_kdbg_prtk_6;
extern int g_kdbg_gate_7, g_kdbg_prtk_7;
extern int g_kdbg_thread;
extern int g_kdbg_count;
extern int g_kdbg_jiffies;
#endif

#ifdef __STACK_FRAME_CHECK__
extern void *g_esp;
#endif

/************************************************************************
*                             M A C R O S
*************************************************************************
*/
#define KDBG_generic_logic(cat,level) \
        int __gate = g_kdbg_gate_##cat; \
        int __prtk = g_kdbg_prtk_##cat; \
        if (g_kdbg_thread) { __gate = g_kdbg_gate_7; __prtk = g_kdbg_prtk_7; } \
        if (level < __gate)

#ifdef __KERNEL__
    // This one is for kernel space
    #define KDBG_LOG_GENERIC(cat,level,fmt,args...) do { \
        KDBG_generic_logic(cat,level) { \
            if (level < __prtk)  __prtk = level; \
            g_kdbg_count++; \
            printk( "<%d>%d-%d.%s: ", __prtk, g_kdbg_count, \
                    (int)(kal_jiffies - g_kdbg_jiffies), __FUNCTION__ );  \
            printk( fmt, ##args); \
            g_kdbg_jiffies = kal_jiffies; \
        } \
    } while (0)

    #define KDBG_LOG_WINDRV(cat,level,S) do { \
        KDBG_generic_logic(5,level) { \
            printk( "<%d>%s: ", __prtk, __FUNCTION__ );  \
            printk S; \
        } \
    } while (0)

#endif



// k = gate / prtk
#define INIT_KDBG_VARIABLES(k,m,n) do { \
        g_kdbg_##k##_##n = KDBG_CATEGORY_VAL(n,x_kdbg_##k); \
        if (g_kdbg_##k##_##n > m)       g_kdbg_##k##_##n = m; \
} while (0)

#if WLAN_INCLUDE_DEBUG
#define INIT_KDBG_VARS(k,m)    do { \
        INIT_KDBG_VARIABLES(k,m,0); \
        INIT_KDBG_VARIABLES(k,m,1); \
        INIT_KDBG_VARIABLES(k,m,2); \
        INIT_KDBG_VARIABLES(k,m,3); \
        INIT_KDBG_VARIABLES(k,m,4); \
        INIT_KDBG_VARIABLES(k,m,5); \
        INIT_KDBG_VARIABLES(k,m,6); \
        INIT_KDBG_VARIABLES(k,m,7); \
} while (0)
#define KGBG_SAVE_THREAD(t)       do { t = g_kdbg_thread; g_kdbg_thread = false; } while(0)
#define KGBG_RESTORE_THREAD(t)    do { g_kdbg_thread = t; } while(0)
#define KGBG_START_THREAD()       g_kdbg_thread = true
#define KGBG_STOP_THREAD()        g_kdbg_thread = false
#else
#define INIT_KDBG_VARS(k,m)
#define KGBG_SAVE_THREAD(t)
#define KGBG_RESTORE_THREAD(t)
#define KGBG_START_THREAD()
#define KGBG_STOP_THREAD()
#endif

#ifdef __I386__
#define KDBG_LOAD_ESP()  ( { \
        void *esp; \
        __asm__("andl %%esp,%0; ":"=r" (esp) : "0" (-1)); \
        esp; \
} )
#else
#define KDBG_LOAD_ESP()  0
#undef __STACK_FRAME_CHECK__   /* only working on I386 */
#endif

#ifdef __STACK_FRAME_CHECK__
#define KDBG_CHECK_STACK()  \
        if (g_esp && ((g_esp - KDBG_LOAD_ESP()) >= 4096)) { \
                KDBG_LOG_SYS(KDBG_CRIT,"System STACK overflow 0x%08lx (0x%08lx)\n", \
                        (unsigned long)KDBG_LOAD_ESP(), (unsigned long)g_esp); \
                BUG(); \
        }

#define KDBG_STACK_DIRTY_CHECK(msg) do { \
        ULONG *pBottom = (ULONG*)((char*)current + 8192); \
        ULONG *pStk = (ULONG*)KDBG_LOAD_ESP(); \
        KDBG_LOG_SYS(KDBG_TRACK, #msg "Check STACK [0x%08lx,0x%08lx]\n", \
                        (unsigned long)pStk, (unsigned long)pBottom); \
        while (pStk < pBottom) { \
                if (pStk[0] == 0x5a5a5a5a) { \
                       if (pStk[1] == 0x5a5a5a5a) { \
                               KDBG_LOG_SYS(KDBG_CRIT,"System STACK tainted at 0x%08lx\n", \
                                        (unsigned long)pStk); \
                               BUG(); \
                       } \
                } \
                pStk++; \
        } \
} while (0)


#else
#define KDBG_CHECK_STACK()
#define KDBG_STACK_DIRTY_CHECK(msg)
#endif

/************************************************************************
   Definitions for all of the Debug macros.  If we're in a debug (DBG) mode,
   these macros will print information to the debug terminal.  If the
   driver is compiled in a free (non-debug) environment the macros become
   NOPs.
*************************************************************************
*/
#if WLAN_INCLUDE_DEBUG

/*------------------------------------------------------------
 *      Debugging MACROs of commonly usages
 *------------------------------------------------------------*/
// Category FUNC_TRACKING
#define KDBG_ENTER          KDBG_LOG_GENERIC( 0, KDBG_TRACK \
                                , "Enter  @%08lx\n", (unsigned long)KDBG_LOAD_ESP() ); \
                            KDBG_CHECK_STACK()

// Category FUNC_TRACKING
#define KDBG_EXIT           KDBG_LOG_GENERIC( 0, KDBG_TRACK \
                                , "Exit  @%08lx\n", (unsigned long)KDBG_LOAD_ESP() )

// Category INIT
#define KDBG_LOG_INIT(level,fmt,args...) \
                        KDBG_LOG_GENERIC( 0, level, fmt, ##args )

// Category TX_DATA
#define KDBG_LOG_TX(level,fmt,args...) \
                        KDBG_LOG_GENERIC( 1, level, fmt, ##args )

// Category RX_DATA
#define KDBG_LOG_RX(level,fmt,args...) \
                        KDBG_LOG_GENERIC( 2, level, fmt, ##args )

// Category SDL_FSM
#define KDBG_LOG_SDL(level,fmt,args...) \
                        KDBG_LOG_GENERIC( 3, level, fmt, ##args )

// Category SYSTEM
#define KDBG_LOG_SYS(level,fmt,args...) \
                        KDBG_LOG_GENERIC( 4, level, fmt, ##args )

// Category INTERRUPT
#define KDBG_LOG_INT(level,fmt,args...) \
                        KDBG_LOG_GENERIC( 4, level, fmt, ##args )

// Category PLUG-IN
#define KDBG_LOG_PLUG(level,fmt,args...) \
                        KDBG_LOG_GENERIC( 4, level, fmt, ##args )

// Category UTILITIES
#define KDBG_LOG_UTIL(level,fmt,args...) \
                        KDBG_LOG_GENERIC( 4, level, fmt, ##args )

// Category MLME
#define KDBG_LOG_MLME(level,fmt,args...) \
                        KDBG_LOG_GENERIC( 5, level, fmt, ##args )

// Category ENV
#define KDBG_LOG_ENV(level,fmt,args...) \
                        KDBG_LOG_GENERIC( 5, level, fmt, ##args )

// Category TEMPORARY
#define KDBG_LOG_TEMP(level,fmt,args...) \
                        KDBG_LOG_GENERIC( 6, level, fmt, ##args )

/*------------------------------------------------------------
 *      Debugging MACROs for Window Driver compatibility
 *------------------------------------------------------------*/
#define WDBG_LV_TERSE           KDBG_INFO
#define WDBG_LV_DBGSTR          KDBG_WARN
#define WDBG_LEVEL1             4
#define WDBG_LEVEL2             5
#define WDBG_LEVEL3             6
#define WDBG_LEVEL4             7
#define WDBG_PACKET             KDBG_RAW_DUMP

#define DEBUGFUNC(__F)  /* static const char __FUNC__[] = __F; */
#define DEBUGTERSE(A,S)         KDBG_LOG_WINDRV(4, WDBG_LV_TERSE, S)
#define DEBUGCHAR(A,C)
#define DEBUGSTR(S)             KDBG_LOG_WINDRV(4, WDBG_LV_DBGSTR, S)
#define DEBUGSTR_IF(EXP,S)      do { if (EXP) KDBG_LOG_WINDRV(4, WDBG_LV_DBGSTR, S); } while(0)
#define INITSTR(S)              KDBG_LOG_WINDRV(0, WDBG_LEVEL1, S)
#define INITLOG1(A,S)           KDBG_LOG_WINDRV(0, WDBG_LEVEL1, S)
#define INITLOG2(A,S)           KDBG_LOG_WINDRV(0, WDBG_LEVEL2, S)
#define INITLOG3(A,S)           KDBG_LOG_WINDRV(0, WDBG_LEVEL3, S)
#define INTLOG(A,S)             KDBG_LOG_WINDRV(4, WDBG_LEVEL3, S)
#define MMILOG(A,S)             KDBG_LOG_WINDRV(4, WDBG_LEVEL4, S)
#define HWLOG1(A,S)             KDBG_LOG_WINDRV(4, WDBG_LEVEL1, S)
#define HWLOG2(A,S)             KDBG_LOG_WINDRV(4, WDBG_LEVEL2, S)
#define RSNLOG1(A,S)            KDBG_LOG_WINDRV(6, WDBG_LEVEL1, S)
#define RSNLOG2(A,S)            KDBG_LOG_WINDRV(6, WDBG_LEVEL2, S)
#define TXLOG1(A,S)             KDBG_LOG_WINDRV(1, WDBG_LEVEL1, S)
#define TXLOG2(A,S)             KDBG_LOG_WINDRV(1, WDBG_LEVEL2, S)
#define TXLOG3(A,S)             KDBG_LOG_WINDRV(1, WDBG_LEVEL3, S)
#define TXLOGPACKET(A,S)        KDBG_LOG_WINDRV(1, KDBG_PACKET, S)
#define RXLOG1(A,S)             KDBG_LOG_WINDRV(2, WDBG_LEVEL1, S)
#define RXLOG2(A,S)             KDBG_LOG_WINDRV(2, WDBG_LEVEL2, S)
#define RXLOG3(A,S)             KDBG_LOG_WINDRV(2, WDBG_LEVEL3, S)
#define RXLOGPACKET(A,S)        KDBG_LOG_WINDRV(2, KDBG_PACKET, S)

#undef ASSERT
#define ASSERT(exp) \
        if (!(exp)) { \
            KDBG_LOG_GENERIC( 0, KDBG_CRIT, "Assertion Failed: %s:%d %s\n" \
                            , __FILE__, __LINE__, #exp ); \
            BUG(); \
        }

#define KDBG_HEX_TDUMP( s, p, n) /* cat: 1-TX */ do { \
    if( KDBG_RAW_DUMP < g_kdbg_gate_1) HEX_DUMP_BODY(s,p,n); \
} while (0)

#define KDBG_HEX_RDUMP( s, p, n) /* cat: 2-RX */ do { \
    if( KDBG_RAW_DUMP < g_kdbg_gate_2) HEX_DUMP_BODY(s,p,n); \
} while (0)


#else /* !WLAN_INCLUDE_DEBUG */
/*------------------------------------------------------------*/
#define KDBG_FSM_ENTER_F
#define KDBG_FSM_ENTER_M
#define KDBG_ENTER
#define KDBG_EXIT
#if KDBG_INLINE
__KAL_INLINE__ void KDBG_LOG_INIT(int a,...){}
__KAL_INLINE__ void KDBG_LOG_TX(int a,...){}
__KAL_INLINE__ void KDBG_LOG_RX(int a,...){}
__KAL_INLINE__ void KDBG_LOG_SDL(int a,...){}
__KAL_INLINE__ void KDBG_LOG_SYS(int a,...){}
__KAL_INLINE__ void KDBG_LOG_INT(int a,...){}
__KAL_INLINE__ void KDBG_LOG_MLME(int a,...){}
__KAL_INLINE__ void KDBG_LOG_ENV(int a,...){}
__KAL_INLINE__ void KDBG_LOG_PLUG(int a,...){}
__KAL_INLINE__ void KDBG_LOG_UTIL(int a,...){}
__KAL_INLINE__ void KDBG_LOG_TEMP(int a,...){}
#else
#define KDBG_LOG_INIT(level,fmt,args...)
#define KDBG_LOG_TX(level,fmt,args...)
#define KDBG_LOG_RX(level,fmt,args...)
#define KDBG_LOG_SDL(level,fmt,args...)
#define KDBG_LOG_SYS(level,fmt,args...)
#define KDBG_LOG_INT(level,fmt,args...)
#define KDBG_LOG_MLME(level,fmt,args...)
#define KDBG_LOG_ENV(level,fmt,args...)
#define KDBG_LOG_PLUG(level,fmt,args...)
#define KDBG_LOG_UTIL(level,fmt,args...)
#define KDBG_LOG_TEMP(level,fmt,args...)
#endif
#define DBG_HEX_DUMP( Typ, s, p, n)
#define DEBUGFUNC(__F)
#define DEBUGTERSE(A,S)
#define DEBUGCHAR(A,C)
#define DEBUGSTR(S)
#define DEBUGSTR_IF(EXP,S)
#define INITSTR(S)
#define INITLOG1(A,S)
#define INITLOG2(A,S)
#define INITLOG3(A,S)
#define INTLOG(A,S)
#define MMILOG(A,S)
#define HWLOG1(A,S)
#define HWLOG2(A,S)
#define RSNLOG1(A,S)
#define RSNLOG2(A,S)
#define TXLOG1(A,S)
#define TXLOG2(A,S)
#define TXLOG3(A,S)
#define TXLOGPACKET(A,S)
#define RXLOG1(A,S)
#define RXLOG2(A,S)
#define RXLOG3(A,S)
#define RXLOGPACKET(A,S)
#define KDBG_HEX_TDUMP( s, p, n)
#define KDBG_HEX_RDUMP( s, p, n)

#ifndef ASSERT
#define ASSERT(exp)
#endif

#endif /* WLAN_INCLUDE_DEBUG */
/*------------------------------------------------------------*/

#define DbgPrint printk

#define HEX_DUMP_BODY(s,p,n) do { \
        int __i__; \
        printk("%s:: " #s ": length %d", __FUNCTION__, (int)n); \
        for( __i__=0; __i__ < (n); __i__++) { \
            if ((__i__%8) == 0) DbgPrint("\n%s:: (%08lx) ", __FUNCTION__, (unsigned long)((UINT8*)(p)+__i__));\
            printk(" %02x", ((UINT8*)(p))[__i__]); \
        } \
        printk("\n=======================\n"); \
} while(0)

/* The macro to dump the value of a varaible with its name shown. */
#define DUMPVAR(variable, format)       #variable " = " format "\n", variable

// #define DBG 1           /* enabling the Windows DEBUG mode */

/************************************************************************
*                 F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/

/************************************************************************
*           I N L I N E   F U N C T I O N   D E F I N I T I O N S
*************************************************************************
*/

#endif  /* _I2220DBG_H */

