/*
** $Id: i2220hw.h 1.37 2004/05/10 22:24:29 chyeh Exp $
*/

/******************************************************************************
Copyright (c) 2002-2003 Integrated Programmable Communications, Inc.

All rights reserved. Copying, compilation, modification, distribution
or any other use whatsoever of this material is strictly prohibited
except in accordance with a Software License Agreement with
Integrated Programmable Communications, Inc.
******************************************************************************/

#ifndef _I2220_HARDWARE_H
#define _I2220_HARDWARE_H

/************************************************************************
*                     C O M P I L E R   F L A G S
*************************************************************************
*/

/************************************************************************
*                E X T E R N A L   R E F E R E N C E S
*************************************************************************
*/

/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/
/* The number of transmit queues */
#define NUM_TDMA_CHANNELS               NUM_TX_QUEUES

#define MAX_MULTICAST_ADDRESSES         32
#define MAX_HW_MULTICAST_REGS           6

/* Host Maintained State Info */
#define I2220HW_STATE_PREINIT           0
#define I2220HW_STATE_INIT              1
#define I2220HW_STATE_RUNNING           2
#define I2220HW_STATE_SCANNING          3

#define TX_TFCB_SUCCESS_MORE_BUFF       0
#define TX_TFCB_SUCCESS_NO_BUFF         1
#define TX_TFCB_FAILURE                 2

/* Return code of env_PHY_SAP_RX() */
#define RX_DRV_HANDLED_PACKET           0
#define RX_INDICATE_PACKET_UPWARD       1

#define CHECK_ADJ_CHNL_INTERVAL         1000

/* Define how many Beacon not sent out to reset MAC */
#define BCN_TX_LOSS_LIMIT               4

#if SUPPORT_TKIP
#define TKIP_HW_MIC_TBB_NUM_PER_TFCB    3
#endif

/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/

__KAL_PACK_BEGIN__
typedef struct _hw_tbb_t {
    UCHAR       tbbFlags;
    UCHAR       tbdChecksum;
    USHORT      txDataBufferLen;
    ULONG       nextTbbPointer;
    ULONG       txDataBufferPointer;

    UCHAR       revBuf[TBB_REVBUFFER_SIZE];
} __KAL_ATTRIB_PACKED__ hw_tbb_t;
__KAL_PACK_END__

typedef struct _tbb_t {
    hw_tbb_t            hw;
    kal_dma_addr_t      dmaAddr;
    struct {
                struct list_head        link;
    } sw;
    BOOLEAN             do_map_unmap;
} tbb_t, *ptbb_t;

__KAL_PACK_BEGIN__
typedef struct _hw_tfcb_t {
    UCHAR       tfdCtrlFlag;
    UCHAR       tfdChecksum;
    UCHAR       reserved1[2];
    ULONG       tbbPointer;
    ULONG       secModeDestAddr;
    ULONG       nextTfcbPointer;

    UCHAR       sciCtrlFlag;
    UCHAR       secOpMode;
    UCHAR       dmaFifoHeaderLen;
    UCHAR       reserved2;
    union{
        ULONG       sciIv;
        struct {
            UCHAR   initialVector[3];
            UCHAR   keyId;
        } __KAL_ATTRIB_PACKED__ s;
    } __KAL_ATTRIB_PACKED__ u_sciiv;
    ULONG       sciExtendedIv;
    UCHAR       encryptionKey[16];

    UCHAR       tciCtrlFlag;
    UCHAR       rtsRetryLimit;
    UCHAR       rtsCtsRate;
    UCHAR       tidMacHdrCtrlFlag;
    USHORT      rtsCtsDuration;
    USHORT      cfPollTxopLimit;
    USHORT      packetTxTime1;
    USHORT      packetTxTime2;
    USHORT      minFrameExchangeTime1;
    USHORT      minFrameExchangeTime2;
    USHORT      multirateSwCtrl;
    USHORT      txRates;
    ULONG       tQ0IdxBitMap;
    UCHAR       txPower;
    UCHAR       reserved3[3];
    USHORT      mpduLen;
    USHORT      totalTxDataLen;

    UCHAR       macHeaderLen;
    UCHAR       reserved4;
    USHORT      durationId2;
    USHORT      frameCtrl;
    USHORT      durationId1;
    UCHAR       macHeader[28];
    ULONG       iv;
    ULONG       extendedIv;

    ULONG       txTimestamp[2];
    USHORT      txedPower;
    USHORT      reserved5;
    USHORT      txStatus;
    UCHAR       rtsRetryCount;
    UCHAR       mpduRetryCount;
} __KAL_ATTRIB_PACKED__ hw_tfcb_t;
__KAL_PACK_END__

typedef hw_tfcb_t *PTFCB_STRUC;

typedef struct _tfcb_t  {
    hw_tfcb_t           hw;
    kal_dma_addr_t      dmaAddr;
    struct _tfcb_t      *nextTfcb;
    UINT8               tkipMic[48];
    struct {
        struct list_head    link;

        struct list_head    tbb_list;

        FragSdu_t           *fsdu;
        int                 frag_nr;
    } sw;
} tfcb_t, *ptfcb_t;

__KAL_PACK_BEGIN__
typedef struct _hw_rfb_t {
    UCHAR       rfbFlags;
    UCHAR       reserved1[2];
    UCHAR       rfdChecksum;
    ULONG       rxBufferPointer;
    ULONG       nextRfbPointer;

    ULONG       fcs;
    ULONG       rxTimestamp[2];
    ULONG       rxStatus;
    ULONG       rxByteCount;
    UCHAR       bbCr24;
    UCHAR       bbCr25;
    UCHAR       bbCr26;
    UCHAR       bbCr27;
    UCHAR       bbCr28;
    UCHAR       bbCr29;
    UCHAR       bbCr30;
    UCHAR       bbCr31;
    UCHAR       bbCr32;
    UCHAR       bbCr33;
    UCHAR       bbCr34;
    UCHAR       bbCr35;
    UCHAR       bbCrA;
    UCHAR       bbCrB;
    UCHAR       bbCrC;
    UCHAR       bbCrD;

    ULONG       secStatus;
    ULONG       iv;
    ULONG       extendedIv;
} __KAL_ATTRIB_PACKED__ hw_rfb_t;
__KAL_PACK_END__

typedef struct _rfb_t {
    hw_rfb_t            hw;
    kal_dma_addr_t      dmaAddr;
    struct {
        struct list_head    link;
        kal_skb_t           *rfb_skb;
        UINT32              secMode;
        UINT32              keyId;
    } sw;
} rfb_t, *prfb_t;

typedef struct _tdma_parameters {
        TX_QUEUE_NUM    tdmaChannel;
        tfcb_t          *head_p;
        tfcb_t          *tail_p;
        int             count;
        chain_list_t    waitQ;
} tdma_parameters_t;


/************************************************************************
*                        P U B L I C   D A T A
*************************************************************************
*/

/************************************************************************
*                             M A C R O S
*************************************************************************
*/

#define tfcb_for_each(head,tfcb_p) \
        for ( tfcb_p = head; \
              (tfcb_p->hw.tfdCtrlFlag & TFCB_FLAG_EOL) == 0; \
              tfcb_p = tfcb_p->nextTfcb \
            )

#define tx_check_TDMA(chain_p,nTFCB,nTBB) ( \
        chain_num_elem(chain_p) >= (nTFCB) && \
        chain_num_elem(&priv_p->tbb_chain) >= (nTBB) )

// Additional piggybacked information on sk_buff
// 1. The TimeStamp information, for performance measurement
#define skb_set_Flag(skb,f)             ( *(UINT32*)(&(skb)->cb[0]) |= f )
#define skb_get_Flag(skb)               ( *(UINT32*)(&(skb)->cb[0]) )
#define skb_clear_Flag(skb)             ( *(UINT32*)(&(skb)->cb[0]) = 0 )
#define skb_set_TimeStamp(skb,t)        ( *(UINT32*)(&(skb)->cb[4]) = t )
#define skb_get_TimeStamp(skb)          ( *(UINT32*)(&(skb)->cb[4]) )

// For skb_set_Flag/get macros
#define SKB_TSTAMP_IN_USE       BIT(0)

#ifdef IPN_NEW_KAL
/* Endian conversion of MAC Hardware intergers to/from HOST */
#if __CPU_ENDIAN__ != __LITTLE_ENDIAN__
#define chip2host_16(n)         __swap16(n)
#define chip2host_32(n)         __swap32(n)
#define host2chip_16(n)         __swap16(n)
#define host2chip_32(n)         __swap32(n)
#else
#define chip2host_16(n)         (n)
#define chip2host_32(n)         (n)
#define host2chip_16(n)         (n)
#define host2chip_32(n)         (n)
#endif
#endif
/************************************************************************
*                 F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/

/************************************************************************
*           I N L I N E   F U N C T I O N   D E F I N I T I O N S
*************************************************************************
*/

#endif /* _I2220_HARDWARE_H */

