/*
** $Id: i2220mgmt.h 1.105 2004/10/04 17:24:25 cmchang Exp $
*/

/******************************************************************************
Copyright (c) 2002-2003 Integrated Programmable Communications, Inc.

All rights reserved. Copying, compilation, modification, distribution
or any other use whatsoever of this material is strictly prohibited
except in accordance with a Software License Agreement with
Integrated Programmable Communications, Inc.
******************************************************************************/

#ifndef _IPN2220MGMT_H
#define _IPN2220MGMT_H

/************************************************************************
*                     C O M P I L E R   F L A G S
*************************************************************************
*/

/************************************************************************
*                E X T E R N A L   R E F E R E N C E S
*************************************************************************
*/

/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/

/*
 * flags for env_MSDU_from_LLC()
 */
#define MaUdat_ORDER            BIT(0)  /* (1/0 strictlyOrdered / reorderable) */
#define MaUdat_CONTENTION       BIT(1)  /* (1/0 contentionFree / contention) */

/* Max number of RF channels */
#if MAX_NUM_2G_RF_CHANNELS < MAX_NUM_5G_RF_CHANNELS
#define MAX_NUM_RF_CHANNELS     MAX_NUM_5G_RF_CHANNELS
#else
#define MAX_NUM_RF_CHANNELS     MAX_NUM_2G_RF_CHANNELS
#endif

#define WLAN_STATE_STOPPED      0   /* Network is not active. */
#define WLAN_STATE_STARTED      1   /* Network has been started. */

#define WLAN_AUTH_MAX           60  /* Max. # of authenticated stations. */
#define WLAN_ACCESS_MAX         60  /* Max. # of stations in an access list. */
#define WLAN_ACCESS_NONE        0   /* No stations may be authenticated. */
#define WLAN_ACCESS_ALL         1   /* All stations may be authenticated. */
#define WLAN_ACCESS_ALLOW       2   /* Authenticate only "allowed" stations. */
#define WLAN_ACCESS_DENY        3   /* Do not authenticate "denied" stations. */

#define WLAN_COMMENT_MAX        80  /* Max. length of user comment string. */

#define WDS_MAX_NUM             8   /* Max. number of WDS networks (remote APs) */

#define BSS_MAX_NUM_PER_CHNL    30  /* Max. number of BSS per channel for scanning */

#define CTS_PROTECT_AUTO        0   /* ERP CTS protection mode: auto */
#define CTS_PROTECT_MUST        1   /* ERP CTS protection mode: always */
#define CTS_PROTECT_NONE        2   /* ERP CTS protection mode: none */

/* Definition of stop_accept_station in priv_p */
#define AP_MODE_ENABLED         0   /* Normal mode, enabled */
#define AP_MODE_DISABLED        1   /* AP function is disabled */
#define AP_MODE_BLOCKED         2   /* AP function is blocked temporarily */

/* Definition of preambleMode in priv_p */
#define PREAMBLE_MODE_AUTO      0
#define PREAMBLE_MODE_LONG      1
#define PREAMBLE_MODE_SHORT     2


/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/

typedef chain_list_t SduQueue_t;

typedef enum _IPN2220_PHY_PREAMBLE_TYPE {
    phyLongPreamble,
    phyShortPreamble
} IPN2220_PHY_PREAMBLE_TYPE, *PIPN2220_PHY_PREAMBLE_TYPE;

typedef enum _IPN2220_PHY_SLOT_TYPE {
    phyLongSlot,
    phyShortSlot
} IPN2220_PHY_SLOT_TYPE, *PIPN2220_PHY_SLOT_TYPE;

/* S/W security engine */
typedef struct _IPN2220_SW_SECURITY_ENGINE {
    UINT8 swEncryptionEnabled;
    UINT8 swDecryptionEnabled;
} IPN2220_SW_SECURITY_ENGINE, *PIPN2220_SW_SECURITY_ENGINE;

typedef enum _IPN2220_SECURITY_ENGINE {
    SECURITY_ENGINE_HW, /* H/W WEP Decryption engine */
    SECURITY_ENGINE_SW  /* S/W WEP Decryption engine */
} IPN2220_SECURITY_ENGINE;

typedef enum _IPN2220_SECURITY_OPERATION {
    ENCRYPTION, /* WEP ENCRYPTION */
    DECRYPTION  /* WEP DECRYPTION */
} IPN2220_SECURITY_OPERATION;

typedef struct i2220hw_stats_s {
    UINT                txTfcbTxOkCounter;
    UINT                txTfcbTxErrCounter;

    //ULONGLONG           rxMdRdyCounter;
    UINT                rxRfbRxOkCounter;
    UINT                rxRfbRxFcsErrCounter;
    UINT                rxRfbRxWepKeyDisabledErrCounter;
    UINT                rxRfbRxWepIcvErrCounter;

    UINT                freeRfbCount;

    UINT                usedSKBCount;
    UINT                usedMEMCount;

    ULONGLONG           rxFcsErrorCount;
    ULONGLONG           rxFifoFullCount;
    ULONGLONG           rxPacketCount;
    ULONGLONG           channelIdleCount;
    ULONGLONG           ccaActiveCount;
    ULONGLONG           ccaSampledActiveCount;
    ULONGLONG           rtsRetryCount;
    ULONGLONG           frameRetryCount;
    ULONGLONG           txFrameCount;
    ULONGLONG           txBeaconCount;

    UINT                RcvFifoFulls;
} i2220hw_stats_t;

/* WEP counters */
typedef struct _IPN2220_WEP_COUNTERS {
        ULONG length;   /* Length of the IPN2220_WEP_COUNTERS structure in bytes */
        ULONG wepIcvErrorCount;
        ULONG wepExcludedCount;
        ULONG wepUndecryptableCount;
} wep_counters_t;

typedef struct _IPN2220_WEP_ENGINE {
        bool swWepEngineEncryptEnable;
        bool swWepEngineDecryptEnable;
} wep_engine_t;

/* structure of BSSDescription */
typedef struct _BSS_DESCRIPTION {
    UCHAR bssid[6];
} BSS_DESCRIPTION, *PBSS_DESCRIPTION;

#if 0
// ivan modify, 0714
typedef struct _RX_STATUS_STRUC {
        UINT    intRxOk;   // number of packets that Rx ok from interrupt
        UINT    intCrcErr; // number of packets that CRC error from interrupt
        UINT    intShort;  // number of packets that is short preamble from interrupt
        UINT    intLong;   // number of packets that is long preamble from interrupt
        UINT    pauRxPktCount;       // number of packets that Rx ok from PAU
        UINT    pauCrcErrCount;      // number of packets that CRC error from PAU
        UINT    pauRxFifoFullCount;  // number of packets that is short preamble from PAU
} rx_status_struc_t;
#endif

/* added by CMC venus 8/21 */
typedef struct chnl_info_s {
        UINT32          bss_nonERP;
        UINT32          bss_ERP;
        UINT32          CCA_activeTime;         /* traffic indication */
        UINT32          rxChannelPower;         /* BB_CR116 */
        sta_info_t      *bss[BSS_MAX_NUM_PER_CHNL];
} chnl_info_t;

/* the management control structure for IPN2220 STA
 * the corresponding object pointer will often named as
 *   <adapter_p> : when refered by code directly transfered from WINDOWS
 *   <priv_p> : when refered by code specific to this LINUX driver
 */
struct i2220mgmt_
{
        /**** Hareware Control Related ****/

        /* Prepared TFCB, TBB and RFB list chains */
        chain_list_t            tfcb_chain;
        chain_list_t            sec_tfcb_chain;
        chain_list_t            tbb_chain;
        chain_list_t            rfb_chain;

        tdma_parameters_t       tdma[NUM_TDMA_CHANNELS];

        /* Interrupt enable and status used for interrupt handlers. */
        UINT32                  interruptEnable;
        UINT32                  interruptStatus;
        UINT32                  qInterruptEnable;

        UINT16                  seqnum;
        UINT8                   dtimCount;
        UINT8                   hw_state;       /* INIT, RUNNING, STOP */

        /* Resource configuration */
        UINT32                  membase;

        /* Linux net_device and support */
        kal_net_device_t        *netdev;        /* ptr to linux netdevice */
        //struct net_device_stats *linux_stats;
        kal_pci_dev_t           *pcidev;        /* pointer to PCI device */

        /* Store the RF chipset information (control mode) */
        UINT8                   rfCtrlMode;

        UINT8                   primaryTxAntenna; /* primary Tx antenna number */
        UINT8                   primaryRxAntenna; /* primary Rx antenna number */
        UINT8                   currentTxAntenna; /* current Tx antenna number */

        /* EEPRom Data */
        UINT8                   permanentMacAddress[MAC_ADDR_LEN];
        ULONG                   swKeyTableSize;
        IPN_SW_KEY_TABLE_ENTRY  swKeyTable[IPN_KEY_TABLE_SIZE];

        /* Switch of S/W security engine */
        IPN2220_SW_SECURITY_ENGINE  swSecurityEngine;

#if WLAN_INCLUDE_PROC
        struct proc_dir_entry   *proc_root_dir;
        struct proc_dir_entry   *proc_hw_dir;
#if WLAN_INCLUDE_DEBUG
        struct proc_dir_entry   *proc_STA_dir;
#endif
#endif

        /* IEEE 802.11 MIB */
        IEEE_802_11_MIB         mib;
        /*for linux device records*/
        struct ipn_net_device_stats     ap_statistics;

        /* The PHY preamble type */
        IPN2220_PHY_PREAMBLE_TYPE       phyPreambleType;
        IPN2220_PHY_SLOT_TYPE           phySlotType;

        UINT32                  supportRates;   /* supported rates (bitwise) */
        UINT32                  basicRates;     /* basic rates (bitwise) */
        txRate_info_t           *txRates_p;     /* global rates info */
        txRate_info_t           *lowestBRate_p; /* lowest basic rate */
        txRate_info_t           *protectRate_p; /* RTS/CTS protection tx rate */

        /* User defined fields 2003/9/5 */
        txRate_info_t           *fixedTxRate_p; /* NULL is auto, else fixed data rate */

        /* The hardware statistics structure to be queried. */
        i2220hw_stats_t         hwStatistics;

        /*-----------------2002/10/1 10:33------------------
         * this wlan box will be an AP
         * --------------------------------------------------*/
        ap_info_t               ap;

        SduQueue_t              bcQ;
        UINT8                   sta_sleeping;   /* number of STAs in sleeping */
        UINT8                   sta_nonERP;     /* number of NonERP STAs */
        UINT8                   sta_LngPreamble;/* number of only long preamble STAs */
        UINT8                   sta_LngSlotTime;/* number of only long slot time STAs */
        UINT8                   bss_nonERP;     /* number of neighbor nonERP BSS */

        UINT8                   curChannel;     /* real current channel including scanning */
        UINT8                   channelBand;    /* 2.4G band or 5G band */

        UINT8                   protectMode;    /* 0: auto, 1: always, 2: none */
        UINT8                   preambleMode;   /* 0:auto, 1:long, 2:short */
        UINT8                   x_hide_ssid;    /* hide SSID: 0 disable, others enable */
        UINT8                   x_CfbBurst;     /* burst mode: 0 disable, others enable */
        int                     cfbCount;       /* avoid blocking medium when too many packets */

        mgmt_timer_t            scan_timer;     /* scan interval of each channel */
        chnl_info_t             *chnl_info_p;   /* scan results for each channel */
        UINT32                  rekey_eol;      /* for a gaurd time to avoid rx */
                                                /* micFailure when rekey */

#if SUPPORT_ED_ADJUST
        /* adaptive for adjacent channel interference */
        int                     state_noise;    /* 0: low noise, 1: high noise */
        UINT8                   initAGC_OPMODE;
        UINT16                  cur_EDThreshold;
        UINT32                  hysteresisCount;
        UINT32                  rxRfbRxOkCounter;
        int                     tbttCount;

        /* The following variables for new ED mechanism */
        INT16                   ED_adjust_dir;
        INT16                   ED_target_offset;
        int                     previous_fa_count_ed;
        UINT8                   ED_ThresholdForceDown_flag;
        UINT8                   ED_floatLevel;
#endif
        INT32                   currentRssi;

        int                     do_checkHang;
        BOOLEAN                 chk_txTimout;   /* TRUE: tx_timeout timer is enabled */
        mgmt_timer_t            tx_timer;       /* tx timeout to reset MAC */
        UINT32                  timeTick;       /* in unit of SDL_TIMER_INTERVAL */
        int                     macResetCount;  /* for debug, how many times to reset MAC/BB */
        int                     macResetBcnCount;
        int                     macResetUFIFOCount;
        int                     macResetAbnormalCount;
        int                     macResetRxEmpty;

        UINT8                   rftest_burst;   /* rftest send burst packets */
        UINT8                   stop_accept_station; /* TRUE: disable AP function */
        UINT8                   param_keyid_index;   /* just for configuration */
#if SUPPORT_MSSID
        UINT8                   param_ssid_index;    /* just for configuration to indicate which ssid*/
#endif
        UINT8                   rxQ_empty;      /* true: rxQ is empty */
        UINT8                   relay_mode;     /* refer to param_relayMode_t*/
        /* The current WEP statistics for the 802.11 interface to be queried */
//        wep_counters_t          current80211WepStatistics;
//        UINT                    rxRfbRxWepKeyDisabledErrCounter;
//        UINT                    rxRfbRxWepIcvErrCounter;
//        UINT                    rxRfbRxTkipKeyDisabledErrCounter;
//        UINT                    rxRfbRxTkipIcvErrCounter;
//        UINT                    rxRfbRxCcmpKeyDisabledErrCounter;
//        UINT                    rxRfbRxCcmpIcvErrCounter;

        UINT8                   authMode;   /* Current auth mode */

        UINT8                   BCcipherSuite;
        UINT8                   DefaultKeyID;
        UINT32                  seed;

        UINT8                   directPskValue;
        UINT8                   rsnBlockingTraffic;     /* If the sta have 2 MIC error within 60 sec */
        UINT8                   TSC[6];         /* 48-bit counter, for 802.11i Draft 2.3 */
        UINT8                   addIE_len;      /* additional IE length set from upper layer program */
        UINT8                   addIE[80];      /* additional IE set from upper layer program */
                                                /* 32 for WPA max IE len, 32 for RSN max IE len , and reserved 16 for vender specific IE */

#if SUPPORT_TKIP & SUPPORT_SW_TKIP_MIC
        UINT16                  HWMICthreshold;
#endif

#if SUPPORT_IPNRSN | SUPPORT_WPA2
        kal_net_device_t        *dev_1x;        /* ptr to linux 1x netdevice */
#endif

#if SUPPORT_IPNRSN
        /* TKIP sequence counter, TSC */
        mgmt_timer_t            sm_timer;
        UINT32                  MIC_block_timer;/* The AP can't accept any associate in 60 sec, timer*/
        BOOLEAN                 WPA_PSK;        /* The AP use PSK or 1X, WPA_PSK = 0 for 1x, 1 for PSK */
        UCHAR                   Counter[8];     /* The replay Counter for WPA */
        UINT                    GNoStations;    /* The counter record how many sta connect with wpa */
        UCHAR                   GNonce[32];
        UCHAR                   GMK[32];
        UCHAR                   GTK[64];
        INT                     GKRekeyTimer;
        INT                     GroupKeyUpdateTimer;
        BOOLEAN                 GroupKeyUpdate;
#endif
        nic_op_mode_t           iw_mode;        /* operating mode (IW_MODE_*) */

#if WLAN_INCLUDE_WDS
        /* WDS Links information */
        wdslink_info_t          *wds;
        int                     wds_connections;

        /* Used to set maxum tx failure allowance for remote AP. */
        int                     wds_tx_failure_limit;
        int                     wds_disable_limit;
#endif
        beacon_frame_t          beacon;         /* Beacon content */

#if SUPPORT_LOOPBACK
        struct {
                unsigned char   enable;
                unsigned char   times;
                unsigned short  length;
                unsigned char   value;
        } loop;
#endif

        /*-------------------------------------------------------*/
        /* Added by CMC for IPN5320, 2004/7/19 */
        UINT32                  currentPhyType;

        UINT8                   txPowerGainForCCK;
        UINT8                   txPowerGainForOFDM;
        UINT8                   currentTxPower_dBm;
        UINT                    numOperatingChannels24;
        UINT8                   operatingChannels24[MAX_NUM_2G_RF_CHANNELS];
#if SUPPORT_802_11A
        UINT                    numOperatingChannels5;
        UINT8                   operatingChannels5[MAX_NUM_5G_RF_CHANNELS];
#endif
        UINT8                   scan_index;

        UINT8                   regCountryString[3];
        UINT16                  nicConfiguredCountry;
        UINT16                  currentCountry;
        DOMAIN_INFO_ENTRY       *domainInfo_p;


        /* RF-specific configuration table */
        NIC_RF_SPECIFIC_CONFIG  *rfSpecificConfig_p;

        /* The NIC per-channel configuration tables */
        NIC_PER_CHANNEL_CONFIG  nicPerChannelConfig24[MAX_NUM_2G_RF_CHANNELS];
#if SUPPORT_802_11A
        NIC_PER_CHANNEL_CONFIG  nicPerChannelConfig5[MAX_NUM_5G_RF_CHANNELS];
#endif

};

/************************************************************************
*                             M A C R O S
*************************************************************************
*/

 /*******************************************************************
 *        Generator for Queue sorts
 *******************************************************************/
/* The Queue generator is derived from the String0 generator */
/* to create Queues of any sort.  Queues operators are: */
/*    Qfirst(queue,item)  adds item as the first queue element */
/*    Qlast(queue,item)   adds item as the last queue element */
/* and the String0 operators Length, //, First, Last, Head, Tail */
/* Because operators can only return a single value, removing an */
/* element from a queue is a 2-step process: */
/*    dequeue first:  item:=First(queue);  queue:=Tail(queue); */
/*    dequeue last:   item:=Last(queue);   queue:=Head(queue); */

#define linuxQ_peek_item(q,list_dir,item) peek_chain_member(q,FragSdu_t,list_dir,item)

#define linuxQ_dequeue_item(q,list_dir,item) get_chain_member(q,FragSdu_t,list_dir,item)

#define linuxQ_add_item(q,item,list_action) add_chain_member(q,FragSdu_t,item,list_action)

#define sdlQ_Length(q)            /* Queue -> Integer; number of items on queue */ \
        (q)->num_elem

#define sdlQ_First(q,item)        /* Queue-> Item; first item on queue */ \
        linuxQ_peek_item(q,next,item)

#define sdlQ_Last(q,item)         /* Queue -> Item; last item on queue */ \
        linuxQ_peek_item(q,prev,item)

#define sdlQ_emptyQ(q)       init_chain_list(q)

#define sdlQ_Qfirst(q,item)       /* Queue,Item -> Queue; add item as first on queue */ \
        linuxQ_add_item(q,item,list_add)

#define sdlQ_Qlast(q,item)   do { /* Queue,Item -> Queue; add item as last on queue */ \
        KDBG_LOG_SDL(KDBG_DEBUG,"item[0x%08lx] queue to %s[%d]\n", \
                (unsigned long)item, #q, sdlQ_Length(q)); \
        linuxQ_add_item(q,item,list_add_tail); \
} while (0)

#define sdlQ_dequeue_first(q,item)     /* item:=First(queue);  queue:=Tail(queue); */ do { \
        linuxQ_dequeue_item(q,next,item); \
        KDBG_LOG_SDL(KDBG_DEBUG,"item[0x%08lx] removed from queue %s[%d]\n", \
                (unsigned long)item, #q, sdlQ_Length(q)); \
} while (0)

#define sdlQ_dequeue_last(q,item)      /* item:=Last(queue);   queue:=Head(queue); */ do { \
        linuxQ_dequue_item(q,prev,item); \
        KDBG_LOG_SDL(KDBG_DEBUG,"item[0x%08lx] removed from queue %s[%d]\n", \
                (unsigned long)item, #q, sdlQ_Length(q)); \
} while (0)

#define sdlQ_Qfirst_limit(q,item,limit)   do { \
        FragSdu_t       *age_fsdu; \
        if (sdlQ_Length(q) >= limit) { \
                /* throw off the last one */ \
                sdlQ_dequeue_last(q, age_fsdu); \
                sdlu_release_fsdu(age_fsdu); \
        } \
        sdlQ_Qfirst(q,item); \
} while (0)

/* Hold old packets to avoid order problem 4/15/2004.
 * If the first packet is throw off, the following packets
 * will be meaningless??
 */
#if 0
#define sdlQ_Qlast_limit(q,item,limit)   do { \
        FragSdu_t       *age_fsdu; \
        if (sdlQ_Length(q) >= limit) { \
                /* throw off the first one */ \
                sdlQ_dequeue_first(q, age_fsdu); \
                sdlu_release_fsdu(age_fsdu); \
        } \
        sdlQ_Qlast(q,item); \
} while (0)
#else
#define sdlQ_Qlast_limit(q,item,limit)   do { \
        if (sdlQ_Length(q) >= limit) { \
                /* throw off the new one */ \
                sdlu_release_fsdu(item); \
                break; \
        } \
        sdlQ_Qlast(q,item); \
} while (0)
#endif

#define sdlQ_dequeue(q,item)    del_chain_member(q,FragSdu_t,item)

#define sdlQ_rm_in_loop(q,lst)  rm_chain_member_in_loop(q,FragSdu_t,lst)

#define sdlQ_empty(q)          (chain_num_elem(q) == 0)

#define sdlQ_Tail(q)              /* Queue -> Queue; all but first item on queue */
#define sdlQ_Head(q)              /* Queue -> Queue; all but last item on queue */
#define sdlQ_cat(q1,q2)           /* Queue, Queue -> Queue; "//" concatenation */
#define sdlQ_Extract(q,i)         /* Queue,Integer -> Item; copy item from queue */
#define sdlQ_Modify(q,i,item)     /* Queue,Integer,Item -> Queue; modify item in queue */
#define sdlQ_SubQ(q,i,j)          /* Queue,Integer,Integer -> Queue; SubQ(q,i,j) queue of length j starting from queue(i) */

#define waitQ_toTDMAChannel(q,item,tdmaChannel) \
        while (sdlQ_Length(q) != 0) { \
                sdlQ_First(q, item); \
                if (check_timeout(item->eol)) { \
                        sdlQ_dequeue(q, item); \
                        sdlu_release_fsdu(item); \
                        continue; \
                } \
                if (!tx_check_TDMA(&priv_p->tfcb_chain, item->fTot, item->fTot)) { \
                        break; \
                } \
                sdlQ_dequeue(q, item); \
                sdlu_tx_FragSdu(priv_p, item, tdmaChannel, FALSE); \
        }

#if SUPPORT_TKIP
#define waitQ_toTDMA_SecChannel(q,item,tdmaChannel) \
        while (sdlQ_Length(q) != 0) { \
                sdlQ_First(q, item); \
                if (check_timeout(item->eol)) { \
                        sdlQ_dequeue(q, item); \
                        sdlu_release_fsdu(item); \
                        continue; \
                } \
                if (!tx_check_TDMA(&priv_p->sec_tfcb_chain, 1, \
                                        TKIP_HW_MIC_TBB_NUM_PER_TFCB)) { \
                        break; \
                } \
                sdlQ_dequeue(q, item); \
                tkipHwGenMic(priv_p, item, tdmaChannel, FALSE); \
        }
#endif

/************************************************************************
*              F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/

/************************************************************************
*           I N L I N E   F U N C T I O N   D E F I N I T I O N S
*************************************************************************
*/

#endif /* _IPN2220MGMT_H */

