/*
** $Id: wlan_lib.h 1.43 2004/10/04 17:35:36 cmchang Exp $
*/

/******************************************************************************
Copyright (c) 2002-2003 Inprocomm, Inc.

All rights reserved. Copying, compilation, modification, distribution
or any other use whatsoever of this material is strictly prohibited
except in accordance with a Software License Agreement with Inprocomm, Inc.
******************************************************************************/

#ifndef __WLAN_LIB_H
#define __WLAN_LIB_H

/************************************************************************
*                     C O M P I L E R   F L A G S
*************************************************************************
*/

/************************************************************************
*                E X T E R N A L   R E F E R E N C E S
*************************************************************************
*/

/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/
#define PARAM_MAX_SSID_LEN              32
#define PARAM_MAX_PHRASE_LEN            64
#define PARAM_MAX_KEY_LEN               32
#define PARAM_MAC_ADDR_LEN              6

/* Define regulation domain for function param_set_regDomain() */
#define MIB_REG_DOMAIN_FCC              0x10    /* FCC (US) */
#define MIB_REG_DOMAIN_IC               0x20    /* IC or DOC (Canada) */
#define MIB_REG_DOMAIN_ETSI             0x30    /* ETSI (Europe) */
#define MIB_REG_DOMAIN_SPAIN            0x31    /* Spain */
#define MIB_REG_DOMAIN_FRANCE           0x32    /* France */
#define MIB_REG_DOMAIN_JAPAN_1          0x40    /* MPHPT (Japan) Channel */
#define MIB_REG_DOMAIN_JAPAN_2          0x41    /* MPHPT (Japan) Channel */
#define MIB_REG_DOMAIN_JAPAN            0x40    /* MPHPT (Japan) */
#define MIB_REG_DOMAIN_OTHER            0x00    /* other */

/* Define RF testing mode for function param_set_rftestMode() */
#define RFTEST_MODE_STOP                0
#define RFTEST_MODE_RX                  1
#define RFTEST_MODE_TONE                2
#define RFTEST_MODE_BURST               3

typedef enum {
        Legacy_Associated,
        TOTAL_AUTH_STAGE
} param_auth_stage_t;

/* Define authMode for function param_set_authmode() */
typedef enum {
        AuthMode_OPEN     = 0,
        AuthMode_SHARED   = 1,
        AuthMode_SWITCH   = 2,
        AuthMode_OPEN_1X  = 3,
        AuthMode_WPA      = 4,
        AuthMode_WPAPSK   = 5
} param_authMode_t;

/* Define cipherMode for functions
 *  param_set_cipherMode(), param_set_wds_cipherMode().
 *  Please note only  CipherMode_NONE ,  CipherMode_WEP40,  CipherMode_WEP104 = 5,
 *  and CipherMode_WEP128 can be param_set_cipherMode's parameter. Because WPA has different
 *  mechanism to do key negotiation, please do not use param_set_cipherMode() to set TKIP or CCMP.
 */
typedef enum {
        CipherMode_NONE   = 0,  /* Disable */
        CipherMode_WEP40  = 1,
        CipherMode_TKIP   = 2,
        CipherMode_WRAP   = 3,  /* Not supported */
        CipherMode_CCMP   = 4,
        CipherMode_WEP104 = 5,
        CipherMode_WEP128 = 6
} param_cipherMode_t;

/* Define bit-map of basic rate set for function param_set_basicRateSet()
 * Default: BasicRate_1M | BasicRate_2M | BasicRate_5_5M | BasicRate_11M
 */
typedef enum {
        BasicRate_1M    = 0x0001,
        BasicRate_2M    = 0x0002,
        BasicRate_5_5M  = 0x0004,
        BasicRate_11M   = 0x0008,
        BasicRate_6M    = 0x0010,
        BasicRate_9M    = 0x0020,
        BasicRate_12M   = 0x0040,
        BasicRate_18M   = 0x0080,
        BasicRate_24M   = 0x0100,
        BasicRate_36M   = 0x0200,
        BasicRate_48M   = 0x0400,
        BasicRate_54M   = 0x0800
} param_basicRate_t;


typedef enum {
        RelayMode_SWITCH             = 0,
        RelayMode_SWITCH_NO_FORWARD  = 1,
        RelayMode_DATA_PIPE          = 2
} param_relayMode_t;

/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/
typedef struct param_ssid_s {
        char            ssid[PARAM_MAX_SSID_LEN];
        int             len;
} param_ssid_t;

typedef struct param_cipher_key_s {
        int             len;
        unsigned char   key[PARAM_MAX_KEY_LEN];
} param_cipher_key_t;


typedef unsigned char   param_mac_addr_t[PARAM_MAC_ADDR_LEN];
typedef unsigned char   param_passphrase_t[PARAM_MAX_PHRASE_LEN + 1];


typedef union param_req {
/* Basic AP */
        param_ssid_t            ssid;           /* refer to param_ssid_t */
        int                     bcnPeriod;      /* 20 ~ 1000 */
        int                     dtimPeriod;     /* 1 ~ 10 */
        int                     setChannel;     /* 1 ~ 14 based on regulation domain */
        int                     curChannel;
        int                     rtsThreshold;   /* 0 ~ 2347 */
        int                     fragThreshold;  /* 0 ~ 2346 */

        int                     basicRateSet;   /* refer to param_basicRate_t */
        int                     rateMode;       /* 0: b+g, 1: g only, 2: b only */

/* Extra function */
        int                     bcastSSID;      /* 0: disable, 1: enable */
        int                     ctsProtect;     /* 0:auto, 1: always, 2: none */
        int                     dataRate;       /* 1Mbps will be 1000000 */
        int                     burstMode;      /* 0: disable, 1: enable */
        int                     noRelay;        /* 0: disable, 1: enable */
        param_mac_addr_t        killmac;

/* Security */
        param_authMode_t        authmode;       /* refer to param_authMode_t */
        param_cipherMode_t      cipherMode;     /* refer to param_cipherMode_t */
        int                     DefaultKeyId;   /* 0, 1, 2, 3 */
        int                     wepKeyId;       /* 0, 1, 2, 3 */
        param_cipher_key_t      wepKey;
        param_passphrase_t      passphrase;     /* string */

/* WDS */
        int                     noStation;      /* 0: disable, 1: enable */
        int                     wds_rateMode;   /* 0: 802.11b, 1: 802.11g */
        param_mac_addr_t        wds_addmac;
        param_mac_addr_t        wds_delmac;
        param_cipher_key_t      wds_cipherKey;
        param_cipherMode_t      wds_cipherMode; /* refer to param_cipherMode_t */

/* Regulation and EMI */
        int                     regDomain;      /* refer to MIB_REG_DOMAIN_FCC */
        int                     rftestMode;     /* 0:stop, 1:rx, 2:tone, 3:burst */
        int                     rftestPower;    /* 0 ~ 255 */

} param_req_t;


typedef struct cmd_key_s {
        unsigned char   keyId;          /* 0 ~ 3 */
        unsigned char   cipherMode;     /* param_cipherMode_t */
        unsigned char   keyLen;         /* max 32 */
        unsigned char   key[32];
} cmd_key_t;

/*
 * Structure used by wlan to store the log.
 */
struct net_stat
{
        unsigned long   txunicastframes;
        unsigned long   txmulticastframes;
        unsigned long   txfragments;
        unsigned long   txsingleretryframes;
        unsigned long   txmultipleretryframes;
        unsigned long   txretrylimitexceeded;
        unsigned long   txdiscards;
        unsigned long   txbytes;
        unsigned long   rxmulticastframes;
        unsigned long   rxbytes;
        unsigned long   rxfragments	;
        unsigned long   rxfcserrors;
        unsigned long   rxdiscardsnobuffer;
        unsigned long   rxdiscardswepundecr;
};

typedef struct sta_status_ {
        short           aid;         /* STA's unique AID (1 .. 2007) or */
                                     /* 0 if not yet assigned */
        unsigned char   adAddr[6];   /* Address of associated station */
        unsigned char   auth_state;  /* Station status */
        unsigned char   ps_state;    /* Station power save status */
        unsigned long   assoc_time;  /* Associated time in unit of ticks */
} sta_status_t;

/************************************************************************
*                        P U B L I C   D A T A
*************************************************************************
*/

/* A string shows the version of IPN2220.o */
extern char *ipn2220_drv_ver;

/************************************************************************
*                             M A C R O S
*************************************************************************
*/

/************************************************************************
*              F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/

/* IPN2220 HW initialization */
extern int i2220ap_pci_probe(void *dev);

/* Restore IPN2220 and free memory */
extern void i2220ap_remove_pci(void *dev);

/* Create private data structure */
extern void *i2220ap_priv_create(void *pdev,
                        void *netdev, unsigned long membase);

/* Free private data structure */
extern void i2220ap_priv_destroy(void *dev);

/* Change MAC address */
extern int i2220ap_set_mac_address(void *dev, unsigned char *new_mac);

/* Packet transmission */
extern int i2220ap_hard_start_xmit(void *netdev, void* skb);

/* IPN2220 interrupt routine */
extern int i2220hw_interrupt(void *dev_priv);

/* Defer interrupt routine */
extern void do_isr_tasklet(void *dev_priv);

/* Periodic time out checking routine */
extern void do_timer_check(void);

/*========================================================
 * Utilitis for WDS management
 *=======================================================*/

/* To determine if this dev is already the main device
 * presenting the WLAN adaptor
 */
extern int is_main_device(void *dev);

/* To determine if this MAC address is already register as the
 * remote Access Point. Return true if yes, false if no.
 */
extern int is_wds_by_mac(void *dev, unsigned char *addr);

/*========================================================
 * Functions of get/set parameter (IOCTL config)
 *=======================================================*/

/*-------------------------------*/
/* Functions to Setting Basic AP */
/*-------------------------------*/
extern int param_set_ssid(void *dev, param_ssid_t *data);

/* Beacon interval is between 20 and 1000 kus */
extern int param_set_bcnPeriod(void *dev, int data);

/* dtim period is between 1 and 5 */
extern int param_set_dtimPeriod(void *dev, int data);

/* New channel is between 1 and 14, depending on regulation domain */
extern int param_set_setChannel(void *dev, int new_channel);

/* rts_threshold is between 1 and 2347 */
extern int param_set_rts(void *dev, int rts_threshold);

/* frag_threshold is between 256 and 2346 */
extern int param_set_frag(void *dev, int frag_threshold);

/* basic_rates is a bit-or of param_basicRate_t.
 * For example, if you want to set 1M and 2M as basic rate,
 * then basic_rates= BasicRate_1M | BasicRate_2M.
 */
extern int param_set_basicRateSet(void *dev, int basic_rates);

/* MIXED mode: rate_mode = 0. Pure G mode: rate_mode = 1.
 * Pure B mode: rate_mode = 2.
 */
extern int param_set_rateMode(void *dev, int rate_mode);

/*-----------------*/
/* Extra functions */
/*-----------------*/

/*
**  whichSsid is allowed to be between 0 ~ 7
 */
extern int param_set_ssid_id(void* dev, int whichSsid);

/*
**  set SSID information
 */
extern int param_set_m_ssid(void* dev, param_ssid_t *data);
extern int param_set_m_vlanid(void* dev, int vlanid);
extern int param_set_m_vlantag_enable(void* dev, int enable);
extern int param_set_m_enable(void *dev, int enable);

/* Allow = 1 means allow broadcasting with SSID and accept
 *           probe request with empty SSID
 * Allow = 0 means broadcasting with empty SSID and no response to
 *           probe request with empty SSID
 */
extern int param_set_bcastSSID(void *dev, int allow);

/* cts_enble = 2 means never use
 * cts_enble = 1 means always use
 * cts_enble = 0 means auto. Only use cts when B AP or station is present.
 */
extern int param_set_ctsProtect(void *dev, int cts_enble);

/* mode = 2 means short preamble only
 * mode = 1 means long preamble only
 * mode = 0 means auto
 */
extern int param_set_preambleMode(void *dev, int mode);

/* Autorate: datarate = 0
 * Fixed rate: for example, if you want set fixed 54M bit rate.
 *             use fix_data_rate = 54000000
 */
extern int param_set_dataRate(void *dev, int fix_data_rate);

/* When use_burst = 1, AP utilize 802.11e CFB to transmit packets.
 * When not, normal AP (CSMA/CA for each packet).
 */
extern int param_set_burstMode(void *dev, int use_burst);

/*
** RelayMode_SWITCH: Driver takes care WLAN to WLAN, and only indicate unknow
**                   destination and brocasting to OS.
** RelayMode_SWITCH_NO_FORWARD: Driver drops WLAN to WLAN, and only indicate
**                   unknow destination and brocasting to OS.
** RelayMode_DATA_PIPE = 2:Driver indicates all data frames to OS. OS should
**                   do WLAN to WLAN forwarding by sending indicated frame
**                   to driver again.
*/
extern int param_set_noRelay(void *dev, param_relayMode_t relay_mode);

/* Use this function to kill a station record. Beware, this function
 * is obselete. Use cmd_deauth_sta instead.
 */
extern int param_set_killmac(void *dev, unsigned char *station_mac);

/* When stop_accepting_sta = 1, AP stop sending beacons, probe response,
 * auth response, associate response. But WDS function is still working.
 */
extern int param_set_noStation(void *dev, int stop_accepting_sta);

/* rxAntenna= 0: auto, 1: primary antenna, 2: secondary antenna */
extern int param_set_rxAntenna(void *dev, int rxAntenna);

/*-------------------------------*/
/* 802.11 Security excluding WPA */
/*-------------------------------*/

/* Set 802.11 Authenticate mode,ie. open, shared-key, switch.
 * AuthMode_OPEN_1X, AuthMode_WPA and AuthMode_WPAPSK are supported only
 * when IPNRSN is enabled.
 */
extern int param_set_authmode(void *dev, param_authMode_t data);

/* Set 802.11 Cipher Algorithm. Please use CipherMode_NONE, CipherMode_WEP40,
 * CipherMode_WEP104 , CipherMode_WEP128. CipherMode_TKIP and CipherMode_CCMP
 * are supported in this funtion only when IPNRSN is enabled.
 */
extern int param_set_cipherMode(void *dev, param_cipherMode_t data);

/* Set 802.11 WepDefaultkey between 0 to 3 */
extern int param_set_wepDefaultKeyId(void *dev, int data);

/* Following two functions are a pair. You need set wep key ID first,
 * then set wep key
 */
extern int param_set_wepKeyId(void *dev, int data);
extern int param_set_wepKey(void *dev, param_cipher_key_t *data);

/*-----------------------*/
/* WDS related functions */
/*-----------------------*/

/* Rate_mode = 1, then remote AP is assume to be 11G-capable.
 * Rate_mode = 0, remote AP is assumed to be 11B only.
 * Please note dev must the one representing remote Access Point, which means
 * you can set one remote AP as 11B AP, while set another one as 11G AP.
 */
extern int param_set_wds_rateMode(void *dev, int rate_mode);

/* Please note dev must be the one representing remote Access Point.*/
extern int param_set_wds_cipherKey(void *dev, param_cipher_key_t *data);
extern int param_set_wds_cipherMode(void *dev, param_cipherMode_t data);

/* main_dev is the one representing WLAN Adaptor.
 * remote_addr = remote AP's MAC Address
 * Return value is the net device pointer representing remote AP.
 */
extern void*param_del_wdslink2(void *main_dev, unsigned char *remote_addr);

/* main_dev is the one representing WLAN Adaptor.
 * remote_addr = remote AP's MAC Address
 * new_dev is the device represents remote AP
 * mode, auth, and remote bssid is no use for this release, all should be 0.
 */
extern int param_add_wdslink2(void *main_dev,
        unsigned char *remote_addr, void *new_dev, int mode,
        int auth, char *remote_bssid);

/* num_of_failed_packets means after how many packets tx fails,
 * this remote AP can be treated as down.
 */
extern int param_set_wds_tx_failure_limit(void *main_dev,
                        int num_of_failed_packets);

/* How long(in seconds) will the treated-as-down AP have a chance to
 * transmit a packet again.
 */
extern int param_set_wds_disable_limit(void *main_dev,
                        int seconds_of_disabled_duration);

/*--------------------*/
/* Regulation and EMI */
/*--------------------*/

/* Please refer to the MIB_REG_DOMAIN_XXXX */
extern int param_set_regDomain(void *dev, int domain);

/* Please refer to the RFTEST_MODE_XXX */
extern int param_set_rftestMode(void *dev, int test_mode);

/* Because tx_power has connection with PHY, you must consulte this value
 * with Inprocomm, Inc.
 */
extern int param_set_rftestPower(void *dev, int tx_power);

/*--------------------------------*/
/* Functions to Getting Parameter */
/*--------------------------------*/

/* Return pointer to a string, which presents the driver version */
extern char *param_get_version(void *dev);

/* Get current channel */
extern int param_get_curChannel(void *dev, int *cur_channel_p);

/* You must provide buffer for return result in out_stat */
extern int param_get_Statistic(void *dev, struct net_stat *out_stat);

/* You must provide buffer for param_get_Statistic to return result in staS.
 * size_of_buffer indicates size of buffer( in unit of byte).
 */
extern int param_get_StaList(void *dev, int size_of_buffer,
        sta_status_t *staS);

/* WLAN chip provides time stamp information in unit of micro second. */
extern int param_get_curTimeStamp(void *dev, unsigned long *timeStamp);

/* When payload length is bigger than level's value, driver uses HW MIC
 * generation. Otherwise, it uses SW to generate. Default value is 0.
 */
extern int param_set_hwMICthreshold(void *dev, unsigned short level);

/*------------------------------------*/
/* Commands from upper layer  for WPA */
/*------------------------------------*/

/* Accept = 1 means to put macAddr in white list.
 * Accept = 0 means to put macAddr in black list.
 */
extern int cmd_do_acl(void *dev, unsigned char *macAddr, int accept);

/* Deauthenticate station. bcast mac address means deauth all associated STAs */
extern int cmd_deauth_sta(void *dev, unsigned char *macAddr);

/* Deauthenticate station. bcast mac address means deauth all associated STAs.
 * Reason code should follow IEEE 802.11 Spec.
 */
extern int cmd_deauth_sta_with_reason(void *dev, unsigned char *macAddr,
                                        int reason_code);

/* block =1 means close 1X port. Block =0 means open 1X port. */
extern int cmd_macPort_ctl(void *dev, unsigned char *macAddr, int block);

/* Set Access Point's WPA IE (Beacon and probe response) */
extern int cmd_set_RsnIE(void *dev, unsigned char *ie, int size);

/* Set/clear group key (group macAddr) or pairwise key */
extern int cmd_set_macKey(void *dev, unsigned char *macAddr, cmd_key_t *key);

/*------------------------*/
/* WLAN information pages */
/*------------------------*/

/*Read Adaper's Memory Control Register.  Output str should have 4K buffer.
** Return: number of character in output string */
extern int wlan_read_mcr(void* dev, char *output_str);

/*Read Adaper's Baseband Control Register Output str should have 4K buffer.
** Return: number of character in output string */
extern int wlan_read_bcr(void* dev, char *output_str);

/*Read Adaper's configuration information Output str should have 4K buffer.
** Return: number of character in output string */
extern int wlan_read_config(void* dev, char *output_str);

/*Read Adaper's status information Output str should have 4K buffer.
** Return: number of character in output string */
extern int wlan_read_info(void* dev, char *output_str);

/*Read Adaper's Statistics Output str should have 4K buffer.
** Return: number of character in output string */
extern int wlan_read_stat(void* dev, char *output_str);

/*Read Adaper's EEPROM's content Output str should have 4K buffer.
** Return: number of character in output string */
extern int wlan_read_eeprom(void* dev, char *output_str);

/*-----------------------------*/
/* Support 802.11a and 802.11d */
/*-----------------------------*/

/* RF band is 2.4GHz (0) or 5GHz (1) before setting channel */
extern int param_set_setRFBand(void *dev, int new_RFBand);

/* Set the country code (US, JP and so on) for 802.11d */
extern int param_set_country(void *dev, char *countryStr);

/************************************************************************
*           I N L I N E   F U N C T I O N   D E F I N I T I O N S
*************************************************************************
*/

#endif  /* __WLAN_LIB_H */

