/*
** $Id: i2220ap.c 2.1 2004/10/04 17:37:18 cmchang Exp $
*/

/*! \file i2220ap.c
    \brief IPN2220 AP Main entry file

    Driver entry / exit point, linking with PCI and NET device structure.
*/

/******************************************************************************
Copyright (c) 2002-2003 Integrated Programmable Communications, Inc.

All rights reserved. Copying, compilation, modification, distribution
or any other use whatsoever of this material is strictly prohibited
except in accordance with a Software License Agreement with
Integrated Programmable Communications, Inc.
******************************************************************************/

/************************************************************************
*                     C O M P I L E R   F L A G S
*************************************************************************
*/

/************************************************************************
*                E X T E R N A L   R E F E R E N C E S
*************************************************************************
*/
#include "precomp.h"
#include "version.h"

/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/

/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/


/************************************************************************
*                        P U B L I C   D A T A
*************************************************************************
*/

/*----------------------------------------------------------------*/
/* --Module Parameters */

/*! x_kdbg_gate: nibble mapped debugging control variable 1
 *      controls whether printk of debugging message will be invoked
 */
unsigned int    x_kdbg_gate=0x99677557;
/*! x_kdbg_prtk: nibble mapped debugging control variable 2
 *      controls the printk log level <n>
 */
unsigned int    x_kdbg_prtk=0xffffffff;

/* DMA Control parameters */
int             x_NumTFCB=(4096)/((sizeof(tfcb_t)+3)&~3);
                                     /*!< number of DMA Control Structures for TFCB/TBB/RFB */
int             x_NumTBB=(4096+4096)/((sizeof(tbb_t)+3)&~3);
                                     /*!< number of DMA Control Structures for TFCB/TBB/RFB */
int             x_NumRFB=24;         /*!< number of DMA Control Structures for TFCB/TBB/RFB */

int             x_MAX_AID=sMaxAId;   /*!< Maximum number of AIDs to use for STAs; must be 2007 or lower */
int             x_channel=11;        /*!< temporary parameter, the Channel to use */
int             x_preTBTT=2000;      /*!< u-seconds before TBTT */
int             x_hide_ssid=0;       /*!< Hidden SSID feature */
int             x_CfbBurst=0;        /*!< Tx CFB Burst */

#if WLAN_INCLUDE_DEBUG
int g_kdbg_gate_0, g_kdbg_prtk_0;
int g_kdbg_gate_1, g_kdbg_prtk_1;
int g_kdbg_gate_2, g_kdbg_prtk_2;
int g_kdbg_gate_3, g_kdbg_prtk_3;
int g_kdbg_gate_4, g_kdbg_prtk_4;
int g_kdbg_gate_5, g_kdbg_prtk_5;
int g_kdbg_gate_6, g_kdbg_prtk_6;
int g_kdbg_gate_7, g_kdbg_prtk_7;
int g_kdbg_count = 0;                   /*!< KDBG counter, to see if kernel buffer wrap around */
int g_kdbg_thread = 0;                  /*!< KDBG debugging thread enabling/disabling flag */
int g_kdbg_jiffies = 0;                 /*!< KDBG debugging relative time */
#endif

#ifdef __STACK_FRAME_CHECK__
void *g_esp=NULL;                       /* x86 CPU's stack pointer */
#endif

char    *ipn2220_drv_ver = "IPN2220AP Ver:" IPN2220_VERSION;

/************************************************************************
*                       P R I V A T E   D A T A
*************************************************************************
*/
#if SUPPORT_PERF_MEASURE
UINT32      tx_counter;
#endif

/************************************************************************
*                             M A C R O S
*************************************************************************
*/

/************************************************************************
*              F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/

/************************************************************************
*                          F U N C T I O N S
*************************************************************************
*/

/*----------------------------------------------------------------*/
/*!
* \brief  A method of struct net_device, initiates the transmission
*         of a packet. The full packet (protocol headers and all) is
*         contained in a socket buffer (buff ) structure.
*
* \param   skb      Linux sk_buff containing the frame.
* \param   netdev   Linux netdevice.
* \return  zero on success, non-zero on failure.
*
*       Linux netdevice method for transmitting a frame.
*       If the lower layers report that buffers are full. netdev->tbusy
*       will be set to prevent higher layers from sending more traffic.
*       Note: If this function returns non-zero, higher layers retain
*             ownership of the skb.
*       The packets will be dispatched depending on
*               1) if packets generated from local-host : env_MAC_SAP_req
*               2) if packets routed/briged from ethernet : env_DSM_fromdsm
*               3) for wlan environment dignostic, raw packets can be
*                  sent without driver's intervenence: env_MAC_SAP_raw
* \sa  env_MAC_SAP_req, env_DSM_fromdsm, env_MAC_SAP_raw
*/
/*----------------------------------------------------------------*/
int i2220ap_hard_start_xmit(void *netdev, void * os_skb)
{
        i2220mgmt_t     *priv_p = kal_dev_priv((kal_net_device_t*)netdev);
        ethhdr_t        eth;
        Frame_t         *tsdu;
        sta_info_t      *sta_p = NULL;
        kal_skb_t*      skb = (kal_skb_t*)os_skb;
#if SUPPORT_MSSID
        int                  if_vlan_pkt;
        unsigned int    pri,vlan_id;
#endif
#if WLAN_INCLUDE_WDS
        wdslink_info_t* remote_ap;
#endif
#if SUPPORT_PERF_MEASURE
UINT32      t1;
#endif
        KDBG_ENTER;
//printk("Sam enter 157\n");
        if (!priv_p || priv_p->hw_state != I2220HW_STATE_RUNNING) {
                KDBG_LOG_TX(KDBG_WARN, "AP not in running state or disabled\n");
//printk("Sam 159 AP not in running state or disabled\n");
                kal_free_skb(skb);
                KDBG_EXIT;
                return 0;
        }

#if SUPPORT_PERF_MEASURE
        tx_counter++;
        if (tx_counter % 3000 == 0) {
            skb_set_Flag(skb,SKB_TSTAMP_IN_USE);
            NIC_MCR_READ(priv_p, MCR_LTTR0, &t1);
            skb_set_TimeStamp(skb,t1);
        }
#endif
        ipn_memcpy(&eth, kal_skb_data(skb), sizeof(ethhdr_t));
#if SUPPORT_MSSID
        if_vlan_pkt = remove_vlang_tag(skb,&pri,&vlan_id);
#endif

        if ( wlanhdr_from_ether(skb) != 0 ) {
                kal_free_skb(skb);
//printk("Sam 181\n");
                return 0;
        }

#if WLAN_INCLUDE_WDS
        /*
        *  If netdev is not the main device, then its's WDS device.
        */
        if (netdev != priv_p->netdev){
//printk("Sam HaHa.. \n");
                /* This netdev is WDS */
                remote_ap   = wds_by_dev(netdev);
                if(remote_ap){
                        sta_p = remote_ap->remote_ap;
                }
                if (!remote_ap || !sta_p) {
                        kal_free_skb(skb);
//printk("Sam 198\n");
                        return 0;
                }
                /* If this remote AP is disabled due to TX failure, and it's
                 * disable limit is not expired yet. We free this skb.
                 */
                if(remote_ap->disabled_status &&
                   !check_timeout(remote_ap->last_disable_time +
                                  priv_p->wds_disable_limit)) {
//printk("Sam 206\n");
                        kal_free_skb(skb);
                        return 0;
                }

                sdl_mkLlcFrame(tsdu, remote_ap->remote_addr,eth.h_dest,
                                eth.h_source, skb);
                if (tsdu == NULL ) {
//printk("Sam 214\n");
                        kal_free_skb(skb);
                        return 0;
                }

        #if RX_HEAD_SIZE_WORKAROUND
                /* When using CCMP with WDS frame, we are forced to use QoS data type*/
                if(sta_p && sta_p->cipherSuite == CIPHER_SUITE_CCMP) {
                        tsdu->frameCtrl = host2ieee16(SYN_qos_data);
                        tsdu->mac_ctl |= MACCTL_QOS_FRAME;
                        tsdu->qosCtrl = 0x0;
                }
        #else
                /* If this bug is fixed, but our remote AP may have still have
                 * this bug. We are forced to use QoS data type as well.
                 */
                if (sta_p && sta_p->cipherSuite == CIPHER_SUITE_CCMP &&
                    remote_ap->mode == 0) {
                        tsdu->frameCtrl = host2ieee16(SYN_qos_data);
                        tsdu->mac_ctl |= MACCTL_QOS_FRAME;
                        tsdu->qosCtrl = 0x0;
                }
        #endif
                sdl_setFrDs( tsdu, 1 );
                sdl_setToDs( tsdu, 1 );

                /* Send out this packet, always return 0 */
                sdu_send_request(priv_p, tsdu, sta_p, SYN_contention);
//printk("Sam 242\n");
                return 0;
        }
#endif

        /* Check if associated STA is roaming out to other AP
         * If we can find other mechanism to know STA raoming, remove
         * this section, which wastes much time to search hash table.
         */
        sta_p = task_sta_lookup(priv_p, eth.h_source);
        if (sta_p && sta_p->aid > 0 &&
                check_timeout(sta_p->actTime + BRIDGE_PROCESSING_DELAY) &&
                (priv_p->relay_mode == RelayMode_SWITCH ||
                priv_p->relay_mode == RelayMode_SWITCH_NO_FORWARD)) {
printk("roam out\n");
                KDBG_LOG_MLME(KDBG_NOTICE, MACSTR " roam out.\n",
                        MAC2STR(eth.h_source));
                free_sta(priv_p, sta_p);
        }
        sta_p = task_sta_lookup(priv_p, eth.h_dest);
//printk("Sam 262 sta_p =%08x\n",sta_p);
#if SUPPORT_MSSID
        /* sta_p == NULL, either group or no station. Group frame always
        ** can be received by all stations.
        ** VLAN has no effect. no station is going to be dropped later.
        */
        if (sta_p && if_vlan_pkt) {
                if (priv_p->ap.ssid_list[sta_p->whichSsid].vlanid != vlan_id) {
                        kal_free_skb(skb);
                        return 0;
                }
        }
#endif

        if ((sdl_isGroup(eth.h_dest) || (sta_p && sta_p->aid > 0)) &&
            !priv_p->stop_accept_station) {
                /* Params: Frame_t,addr1, addr3, addr4, skb */
                sdl_mkLlcFrame(tsdu, eth.h_dest, eth.h_source, 0, skb);
                if (tsdu == NULL) {
                        kal_free_skb(skb);
                        return 0;
                }

//printk("Sam 285\n");
                sdl_setFrDs( tsdu, 1 );

                /* Send out this packet, always return 0 */
                sdu_send_request(priv_p, tsdu, sta_p, SYN_contention);
        }
        else {
                KDBG_LOG_TX(KDBG_DEBUG,"BAD da " MACSTR "\n",
                              MAC2STR(eth.h_dest));
//printk("Sam 293\n");
                kal_free_skb(skb);
        }

        KDBG_EXIT;
        return 0;
}

/*----------------------------------------------------------------*/
/*!
* \brief  A method of struct net_device, indicates the interface MAC
*         address has been changed.
*
* Handles the ioctl for changing the MACAddress of a netdevice
*
* references: linux/netdevice.h and drivers/net/net_init.c
*
* \param  dev     netdevice struct
* \param  addr    the new MACAddress (a struct)
* \return  zero on success, a negative errno on failure.  Possible values:
*               -EBUSY  device is bussy (cmd not possible)
*               -and errors returned by: p80211req_dorequest(..)
*/
/*----------------------------------------------------------------*/
int i2220ap_set_mac_address(void *dev, unsigned char *new_mac)
{
        i2220mgmt_t     *priv_p = kal_dev_priv((kal_net_device_t*)dev);
        int             result = 0;

        KDBG_ENTER;

        if (sdl_macEqu(new_mac,nullAddr) || sdl_isGroup(new_mac)) {
                KDBG_LOG_INIT(KDBG_NOTICE, "New MAC " MACSTR " is invalid\n",
                        MAC2STR(new_mac));
                result = -22;
        }
        else {
                ipn_memcpy(mib_import(dot11MACAddress), new_mac, MAC_ADDR_LEN);
                hw_setOwnMacAddress(priv_p, (UINT8*)new_mac);
                pauSetCurrentBssid(priv_p, (PUINT_8)new_mac);
        }

        KDBG_EXIT;
        return result;
}

/*----------------------------------------------------------------*/
/*!
* \brief  PCI probe call back function, called when a PCI device of matching
*         manufacture/device ID is found.
*
* This function initializes the device it is passed. Actually it is this place,
* the per instance of IPN2220 NIC card get started execution.
*
* \param   pdev            pointer to PCI device structure containing info about
*                          PCI configuration.
* \param   id              pointer to the device ID entry that matched this device.
* \return  zero when success, negative as failure
*/
/*----------------------------------------------------------------*/
void *i2220ap_priv_create(void *pdev, void *netdev, UINT32 membase)
{
        i2220mgmt_t     *priv_p;

        KDBG_ENTER;

        ASSERT((sizeof(Frame_t)&3) == 0);   /* Make sure 4 alignment of IP layer */
        ASSERT((sizeof(FragSdu_t)&3) == 0); /* Make sure 4 alignment */

        priv_p = (i2220mgmt_t*)kal_malloc(sizeof(i2220mgmt_t));
        if (priv_p == NULL) {
                printk(/*KDBG_LOG_INIT(KDBG_WARN,*/"Memory allocation failure.\n");
                return NULL;
        }
        ipn_memset(priv_p, 0, sizeof(i2220mgmt_t));
        priv_p->membase = membase;
        priv_p->netdev = netdev;
        priv_p->pcidev = pdev;
//printk("Sam add 363 priv_p->membase=%08x,priv_p->netdev=%08x,priv_p->pcidev=%08x cpuendian=%d\n",priv_p->membase,priv_p->netdev,priv_p->pcidev,__CPU_ENDIAN__);
/**********Merlin 20041118**************/        
       // Sam  priv_p->HWMICthreshold = 1600;
/***********************************/
#if WLAN_USE_MEMPOOL
        if (init_mgt_mem_pool(priv_p) != 0) {
                kal_free(priv_p);
                return NULL;
        }
#endif

        /* Initialize MIB database */
        mib_initialize(priv_p);

        /* Initialize all structures */
        sdlu_init_ap(priv_p);

        KDBG_EXIT;
        return priv_p;
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param   dev            pointer to kal_net_device_t
* \return  none
*/
/*----------------------------------------------------------------*/
void i2220ap_priv_destroy(void *dev)
{
        i2220mgmt_t     *priv_p = kal_dev_priv((kal_net_device_t*)dev);

        if (priv_p == NULL) return;

        sdlu_destroy_ap(priv_p);

#if WLAN_USE_MEMPOOL
        destroy_mgt_mem_pool(priv_p);
#endif
        kal_free(priv_p);
}

/*----------------------------------------------------------------*/
/*!
* \brief  PCI probe call back function, called when a PCI device of matching
*         manufacture/device ID is found.
*
* This function initializes the device it is passed. Actually it is this place,
* the per instance of IPN2220 NIC card get started execution.
*
* \param   pdev            pointer to PCI device structure containing info about
*                          PCI configuration.
* \param   id              pointer to the device ID entry that matched this device.
* \return  zero when success, negative as failure
*/
/*----------------------------------------------------------------*/
int i2220ap_pci_probe(void *dev)
{
        i2220mgmt_t     *priv_p = kal_dev_priv((kal_net_device_t*)dev);
        int             result;

        if (priv_p == NULL) return -1;

        if ((result = i2220hw_create(priv_p)) == 0) {
                /* Start state machine and enable interrupt */
                MlmeStart_request(priv_p);
        }
//printk("Sam ipn2220 430 result=%d\n",result);
        return result;
}

/*----------------------------------------------------------------*/
/*!
* \brief  PCI remove call back function, called when the PCI device should
*         be stopped.
*
* The remove method is used to tell the device driver that it should shut down
* the device and stop dealing with it, releasing any associated storage. The
* function is called either when the device is removed from the system or when
* the driver calls pci_unregister_driver in order to be unloaded from the system.
* Unlike probe, this method is specific to one PCI device, not to the whole set
* handled by this driver; the specific device is passed as an argument.
*
* \param   pdev            pointer to PCI device structure
* \return  none
*/
/*----------------------------------------------------------------*/
void i2220ap_remove_pci(void *dev)
{
        i2220mgmt_t     *priv_p = kal_dev_priv((kal_net_device_t*)dev);

        KDBG_ENTER;

        if (priv_p == NULL) return;

        /* Stop network and hardware */
        i2220hw_drvr_stop(priv_p);

        /* Cleanup the hardware-related structs (TFCB,RFB) */
        i2220hw_destroy(priv_p);

        KDBG_EXIT;
}


