/*
** $Id: i2220hw.c 2.21 2004/10/04 17:39:52 cmchang Exp $
*/

/*! \file i2220hw.c
    \brief IPN2220 MAC hardware control interface

    IPN2220 MAC chipset hardware related functions, such as TFCB/RFB DMA
    operation functions, Interrupt handling routines, hardware startup /
    stop operations.
*/

/******************************************************************************
Copyright (c) 2002-2003 Integrated Programmable Communications, Inc.

All rights reserved. Copying, compilation, modification, distribution
or any other use whatsoever of this material is strictly prohibited
except in accordance with a Software License Agreement with
Integrated Programmable Communications, Inc.
******************************************************************************/

/************************************************************************
*                     C O M P I L E R   F L A G S
*************************************************************************
*/

/************************************************************************
*                E X T E R N A L   R E F E R E N C E S
*************************************************************************
*/
#include "precomp.h"
/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/
#define RFB_SECSTATUS_MASK      BITS(16,31)

/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/

/************************************************************************
*                        P U B L I C   D A T A
*************************************************************************
*/
#if SUPPORT_PERF_MEASURE
UINT32  rx_counter;
#endif
/************************************************************************
*                       P R I V A T E   D A T A
*************************************************************************
*/

/* This list should be moved to priv_p for multiple NICs
 * Don't change rate order in this list because we cal duration
 * of response frame in function hw_set_rate_table().
 */
static txRate_info_t txDataRateList[] = {
    { RATE_1M,  0, 0, 0, 0, 0 },
    { RATE_2M,  0, 0, 0, 0, 0 },
    { RATE_5_5M,0, 0, 0, 0, 0 },
    { RATE_11M, 0, 0, 0, 0, 0 },
    { RATE_6M,  0, 0, 0, 0, 1 },
    { RATE_9M,  0, 0, 0, 0, 1 },
    { RATE_12M, 0, 0, 0, 0, 1 },
    { RATE_18M, 0, 0, 0, 0, 1 },
    { RATE_24M, 0, 0, 0, 0, 1 },
    { RATE_36M, 0, 0, 0, 0, 1 },
    { RATE_48M, 0, 0, 0, 0, 1 },
    { RATE_54M, 0, 0, 0, 0, 1 },
    {        0, 0, 0, 0, 0, 0 }  /* Terminator */
};

/************************************************************************
*                             M A C R O S
*************************************************************************
*/
#if !WP_3210_WORKAROUND /* Waveplus 3210 define 8-bytes align */
#define create_chain_list(chain,type,pci_p,num) do { \
        int i, size=(sizeof(type)+3)&~3; \
        kal_dma_addr_t dmaAddr; \
        type *elem = (type *)kal_pci_alloc_sz(pci_p,size*(num),&dmaAddr); \
        ASSERT(elem); \
        init_chain_list(chain); \
        (chain)->buf = (void*)elem; \
        (chain)->dmaAddr = dmaAddr; \
        (chain)->buf_sz = size * (num); \
        (chain)->num_elem = num;  \
        for (i=0; i<(num); i++) { \
                ipn_memset( elem, 0, size ); \
                elem->dmaAddr=(kal_dma_addr_t)((UINT8*)((chain)->dmaAddr)+i*size); \
                list_add_tail( &((type*)elem)->sw.link, &(chain)->head ); \
                elem=(type*)((UINT8*)elem+size); \
        } \
} while (0)
#else
#define create_chain_list(chain,type,pci_p,num) do { \
        int i, offset, size=(sizeof(type)+15)&~15; \
        kal_dma_addr_t dmaAddr; \
        type *elem = (type *)kal_pci_alloc_sz(pci_p,size*(num)+16,&dmaAddr); \
        ASSERT(elem); \
        init_chain_list(chain); \
        (chain)->buf = (void*)elem; \
        (chain)->dmaAddr = dmaAddr; \
        (chain)->buf_sz = size * (num) + 16; \
        (chain)->num_elem = num;  \
        offset = 16 - (((UINT32)elem) & 15); \
        elem=(type*)((UINT8*)elem + offset); \
        for (i=0; i<(num); i++) { \
                ipn_memset( elem, 0, size ); \
                elem->dmaAddr=(kal_dma_addr_t)((UINT8*)((chain)->dmaAddr)+i*size+offset); \
                list_add_tail( &((type*)elem)->sw.link, &(chain)->head ); \
                elem=(type*)((UINT8*)elem+size); \
        } \
} while (0)
#endif

#define destroy_chain_list(chain,type,pci_p) do { \
        struct list_head  *lst; \
        list_for_each(lst,&(chain)->head) { \
                type *elem = list_entry(lst, type, sw.link); \
                rm_chain_member_in_loop(chain, type, lst); \
                if ((chain)->buf) continue; \
                kal_pci_free(pci_p,type,elem,elem->dmaAddr); \
        } \
        if ((chain)->buf) { \
                kal_pci_free_sz(pci_p,(chain)->buf_sz,(chain)->buf,(chain)->dmaAddr); \
        } \
} while (0)

#define cleanup_TFCB_chained_list(chain_p,lst_p,tfcb_p) do { \
        tfcb_for_each(lst_p,tfcb_p) \
        { \
                KDBG_LOG_TX(KDBG_DEBUG, \
                        "Clean up " #lst_p " TFCB 0x%08lx (tx status 0x%08lx)\n", \
                        (unsigned long)tfcb_p, (unsigned long)tfcb_p->hw.txStatus); \
                /* Put back the TFCB to the free chain list */ \
                add_chain_tail(chain_p,tfcb_t,tfcb_p); \
                /* Cleanup associated TBB and/or data buffers */ \
                isr_transmit_cleanup_TBB(priv_p, tfcb_p, TRUE); \
                /* tell the MAC machine to release the associated objects with this TFCB */ \
                /* But we don't confirm PM_Filter_AP process, otherwise, recursive !! */ \
                /* CMC: venus env_IF_MAC_HARDWARE(SIG_PhyTxEnd_confirm,priv_p,tfcb_p,false); */ \
                sdlu_release_tfcb(tfcb_p); \
        } \
} while (0)


#if WLAN_INCLUDE_DEBUG
#define queue_TQ_interrupt_process(ch) do { \
        if (qStatus & QSR1_TQ##ch##_EVENT) { \
            /* Process TDMA frame transmit done interrupts */ \
            if (qStatus & QSR1_TQ##ch##_TX_DONE) { \
                isr_process_tx_done(priv_p, TXQ_TQ##ch); \
            } \
            /* Process TDMA transmit buffer empty interrupts */ \
            if (qStatus & QSR1_TQ##ch##_QUEUE_EMPTY) { \
                isr_process_tx_buffer_empty(priv_p, TXQ_TQ##ch); \
            } \
            /* Process TDMA transmit timeout interrupts */ \
            if (qStatus & QSR1_TQ##ch##_TIMEOUT) { \
                KDBG_LOG_TX(KDBG_INFO,"TQ " #ch " transmit timeout\n"); \
            } \
        } \
} while (0)

#define queue_AC_interrupt_process(ch) do { \
        if (qStatus & QSR1_AC##ch##_EVENT) { \
            /* Process TDMA frame transmit done interrupts */ \
            if (qStatus & QSR1_AC##ch##_TX_DONE) { \
                isr_process_tx_done(priv_p, TXQ_AC##ch); \
            } \
            /* Process TDMA transmit buffer empty interrupts */ \
            if (qStatus & QSR1_AC##ch##_QUEUE_EMPTY) { \
                isr_process_tx_buffer_empty(priv_p, TXQ_AC##ch); \
            } \
        } \
} while (0)

#define queue_TS_interrupt_process(ch) do { \
        if (qStatus & QSR1_TS##ch##_EVENT) { \
            /* Process TDMA frame transmit done interrupts */ \
            if (qStatus & QSR1_TS##ch##_TX_DONE) { \
                isr_process_tx_done(priv_p, TXQ_TS##ch); \
            } \
            /* Process TDMA transmit buffer empty interrupts */ \
            if (qStatus & QSR1_TS##ch##_QUEUE_EMPTY) { \
                isr_process_tx_buffer_empty(priv_p, TXQ_TS##ch); \
            } \
        } \
} while (0)

#else

#define queue_TQ_interrupt_process(ch) do { \
        /* Process TDMA frame transmit done interrupts */ \
        if (qStatus & QSR1_TQ##ch##_TX_DONE) { \
            isr_process_tx_done(priv_p, TXQ_TQ##ch); \
        } \
} while (0)

#define queue_AC_interrupt_process(ch) do { \
        /* Process TDMA frame transmit done interrupts */ \
        if (qStatus & QSR1_AC##ch##_TX_DONE) { \
            isr_process_tx_done(priv_p, TXQ_AC##ch); \
        } \
} while (0)

#define queue_TS_interrupt_process(ch) do { \
        /* Process TDMA frame transmit done interrupts */ \
        if (qStatus & QSR1_TS##ch##_TX_DONE) { \
            isr_process_tx_done(priv_p, TXQ_TS##ch); \
        } \
} while (0)

#endif /* WLAN_INCLUDE_DEBUG */

/************************************************************************
*              F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/

/*--------------------------------------------------------*/
/* Local function declarations */
int  rx_init_and_chain_packet(i2220mgmt_t *priv_p, rfb_t *rfb_p, bool arrange_skb);
void isr_transmit_update_stats(i2220mgmt_t *priv_p, tfcb_t *tfcb_p, sta_info_t* sta_p);
void isr_transmit_cleanup_TBB(i2220mgmt_t *priv_p, tfcb_t *tfcb_p, int remove);
int  isr_process_tx_done(i2220mgmt_t *priv_p, UINT tdmaChannel);

void tx_setup_transmit_queues(i2220mgmt_t *priv_p);
void tx_cleanup_transmit_queues(i2220mgmt_t *priv_p);
void rx_setup_receive_queues(i2220mgmt_t *priv_p);
void rx_cleanup_receive_queues(i2220mgmt_t *priv_p);
kal_dma_addr_t rx_start_recv_unit(i2220mgmt_t *priv_p);

/************************************************************************
*                          F U N C T I O N S
*************************************************************************
*/

#if WLAN_INCLUDE_DEBUG

/*----------------------------------------------------------------*/
/*!
* \brief Debugging log supporting function, for TX packets
*
* Depends on the \a flag value, different of \a ptr pointer context will
* be displayed.
*
* \param  priv_p  pointer to i2220mgmt_t, the device's private structure
* \param  ptr     pointer to the object to be displayed
* \param  flag    control how to show the contents of \a ptr
* \return none
*/
/*----------------------------------------------------------------*/
void dbg_tx_tdma(i2220mgmt_t *priv_p, int tdmaChannel, int chkNum)
{
        tdma_parameters_t       *tdma_p;
        tfcb_t                  *tfcb_p;
        int                     nTFCB;
        int                     tmp;
        int                     count1, count2;
        int                     start, index;

        tmp = g_kdbg_gate_1;
        g_kdbg_gate_1 = 0xF;

        nTFCB = count1 = count2 = start = index = 0;
        tdma_p = &priv_p->tdma[tdmaChannel];
        if(tdma_p->head_p == NULL) {
                KDBG_LOG_TX(KDBG_WARN,"TDMA%d not activated.\n", tdmaChannel);
                g_kdbg_gate_1 = tmp;
                return;
        }
        tfcb_for_each(tdma_p->head_p, tfcb_p) {
                index++;
                if (chip2host_16(tfcb_p->hw.txStatus) &
                    TFCB_STATUS_MASK) {
                        count1++;
                        if (start == 0) start = index;
                }
                else {
                        count2++;
                }
                if (++nTFCB > chkNum) continue;
                dbg_tx_packet(priv_p, tfcb_p, DBFLG_TX_TFCB);
        }
        //dbg_tx_packet(priv_p, tfcb_p, DBFLG_TX_TFCB); /* EOL TFCB */
        KDBG_LOG_TX(KDBG_WARN," %d TFCBs in TDMA%d (y=%d,n=%d,s=%d)\n",
                nTFCB, tdmaChannel, count1, count2, start);

        g_kdbg_gate_1 = tmp;
}

/*----------------------------------------------------------------*/
/*!
* \brief Debugging log supporting function, for TX packets
*
* Depends on the \a flag value, different of \a ptr pointer context will
* be displayed.
*
* \param  priv_p  pointer to i2220mgmt_t, the device's private structure
* \param  ptr     pointer to the object to be displayed
* \param  flag    control how to show the contents of \a ptr
* \return none
*/
/*----------------------------------------------------------------*/
void dbg_tx_packet(i2220mgmt_t *priv_p, void *ptr, UINT32 flag)
{
        UINT8* tmp_p;
        UINT32 tmpUlong;
        UINT32 len = flag & 0xfff;

        tmp_p = (UINT8*)ptr;

        if (flag & DBFLG_TX_SHOW_REG) {
                NIC_MCR_READ(priv_p, MCR_ISAR, &tmpUlong);
                KDBG_LOG_TX(KDBG_TXHW, "ISAR = 0x%08lx\n", (unsigned long)tmpUlong);

                NIC_MCR_READ(priv_p, MCR_QSR0, &tmpUlong);
                KDBG_LOG_TX(KDBG_TXHW, "QSR0 = 0x%08lx\n", (unsigned long)tmpUlong);

                NIC_MCR_READ(priv_p, MCR_QSR1, &tmpUlong);
                KDBG_LOG_TX(KDBG_TXHW, "QSR1 = 0x%08lx\n", (unsigned long)tmpUlong);

                NIC_MCR_READ(priv_p, MCR_AC0SAR, &tmpUlong);
                KDBG_LOG_TX(KDBG_TXHW, "AC0SAR = 0x%08lx\n", (unsigned long)tmpUlong);

                NIC_MCR_READ(priv_p, MCR_AC0CPR, &tmpUlong);
                KDBG_LOG_TX(KDBG_TXHW, "AC0CPR = 0x%08lx\n", (unsigned long)tmpUlong);
        }

        if (flag & DBFLG_TX_HEADER) {
                if (len > 56UL)  len = 56UL;    // 32 wlan-header + 24 body (for mgmt frame)
                KDBG_HEX_TDUMP("Transmitted frame",tmp_p,len);
        }
        else
        if (flag & DBFLG_TX_ALL_FRAME) {
                /* Debug messages. Dump the content of the transmitted frame. */
                KDBG_HEX_TDUMP("Transmitted frame",tmp_p,len);
        }

        if (flag & DBFLG_TX_TFCB) {
                KDBG_HEX_TDUMP("TFCB",tmp_p,sizeof(tfcb_t));
        }

        if (flag & DBFLG_TX_TBB) {
                KDBG_HEX_TDUMP("TBB",tmp_p,sizeof(tbb_t));
        }
}

/*----------------------------------------------------------------*/
/*!
* \brief Debugging log supporting function, for RX packets
*
* Depends on the \a flag value, different of \a rfb_p pointer context will
* be displayed.
*
* \param  priv_p  pointer to i2220mgmt_t, the device's private structure
* \param  rfb_p   pointer to RFB to be displayed
* \param  flag    control how to show the contents of \a ptr
* \return none
*/
/*----------------------------------------------------------------*/
void dbg_rx_rdma(i2220mgmt_t *priv_p, int chkNum)
{
        struct list_head        *lst;
        rfb_t                   *rfb_p;
        kal_skb_t               *skb;
        int                     tmp, nRFB;
        int                     count1, count2;
        UINT32                  rxQ_curAddr;

        tmp = g_kdbg_gate_2;
        g_kdbg_gate_2 = 0xF;

        /* read current RX_Q pointer register */
        NIC_MCR_READ(priv_p, MCR_RQCPR, &rxQ_curAddr);
        KDBG_LOG_RX(KDBG_WARN,"rxQ_CurAddr=0x%08lx\n", rxQ_curAddr);

        nRFB = count1 = count2 = 0;
        list_for_each(lst,&priv_p->rfb_chain.head) {
                rfb_p = list_entry(lst, rfb_t, sw.link);

                if (chip2host_32(rfb_p->hw.secStatus) & RFB_SECSTATUS_MASK) {
                        count2++; /* Empty rx'd RFB */
                }
                else {
                        count1++; /* Completed rx'd RFB */
                }

                skb = rfb_p->sw.rfb_skb;
                kal_pci_unmap_single(priv_p->pcidev,
                                chip2host_32(rfb_p->hw.rxBufferPointer),
                                sizeof(rxbuf_frame_t) - wlanheader_offset,
                                PCI_DMA_FROMDEVICE);

                ++nRFB;
                if (rfb_p->dmaAddr == rxQ_curAddr) {
                        KDBG_LOG_RX(KDBG_WARN,"rxQ_CurAddr -> %d RFB\n",nRFB);
                }
                else if (nRFB > chkNum) continue;
                dbg_rx_packet(priv_p,rfb_p,DBFLG_RX_RFB);
        }
        KDBG_LOG_RX(KDBG_WARN, "%d RFBs in RxQ (y=%d,n=%d)\n",
                nRFB, count1, count2);

        g_kdbg_gate_2 = tmp;
}

/*----------------------------------------------------------------*/
/*!
* \brief Debugging log supporting function, for RX packets
*
* Depends on the \a flag value, different of \a rfb_p pointer context will
* be displayed.
*
* \param  priv_p  pointer to i2220mgmt_t, the device's private structure
* \param  rfb_p   pointer to RFB to be displayed
* \param  flag    control how to show the contents of \a ptr
* \return none
*/
/*----------------------------------------------------------------*/
void dbg_rx_packet(i2220mgmt_t *priv_p, rfb_t *rfb_p, UINT32 flag)
{
        UINT32 tmpUlong;

        UINT32 len = min(rfb_p->hw.rxByteCount,72UL);   // 32 wlan-header + 40 body
        kal_skb_t *skb = rfb_p->sw.rfb_skb;

        if (flag & DBFLG_RX_SHOW_REG) {
                NIC_MCR_READ(priv_p, MCR_ISAR, &tmpUlong);
                KDBG_LOG_TX(KDBG_TXHW, "ISAR = 0x%08lx\n", (unsigned long)tmpUlong);

                NIC_MCR_READ(priv_p, MCR_RFECR, &tmpUlong);
                mib_import(dot11FCSErrorCount) += tmpUlong;
                KDBG_LOG_RX(KDBG_RXHW, "RFECR = 0x%08lx\n", (unsigned long)tmpUlong);

                NIC_MCR_READ(priv_p, MCR_RFFCR, &tmpUlong);
                priv_p->hwStatistics.RcvFifoFulls += tmpUlong;
                KDBG_LOG_RX(KDBG_RXHW, "RFFCR = 0x%08lx\n", (unsigned long)tmpUlong);
        }

        if (flag & DBFLG_RX_RFB) {
                KDBG_LOG_RX(KDBG_RXHW, "Received packet length %d\n",
                        (int)rfb_p->hw.rxByteCount);
                KDBG_LOG_RX(KDBG_RXHW, DUMPVAR(rfb_p->hw.rxStatus, "0x%08lx"));

                KDBG_HEX_RDUMP("RFB",rfb_p,sizeof(rfb_t));
        }

        if (flag & DBFLG_RX_HEADER) {
                KDBG_HEX_RDUMP("Received frame",tcast_wlan_header(kal_skb_data(skb)),len);
        }
        else
        if (flag & DBFLG_RX_ALL_FRAME) {
                KDBG_HEX_RDUMP("Received frame",tcast_wlan_header(kal_skb_data(skb)),rfb_p->hw.rxByteCount);
        }
}

/*----------------------------------------------------------------*/
/*!
* \brief Debugging log supporting function, for \a Frame_t object
*
* The 802.11 wlan frame components are displayed, besides, the MAC Control
* descripter, as implementation properties of Frame_t structure, are also
* displayed.
*
* \param  sdu     pointer to the Frame_t to be displayed
* \return none
*/
/*----------------------------------------------------------------*/
void dbg_frame_header2(const Frame_t *sdu, int dbg_level)
{
#if 1   //CMC venus
        return;
#else

        KDBG_LOG_UTIL(dbg_level, "   LLC_data=0x%08x LLC_len=%d skb=0x%08x\n",
               (unsigned)sdu->LLCdata, sdu->LLClength, (unsigned)sdu->skb);

        KDBG_LOG_UTIL(dbg_level, "   mac_ctl=0x%0x rx.TStamp=%08lx rssi=%d QLink=%d\n",
               (unsigned)sdu->mac_ctl, (unsigned long)sdu->rx.time_stamp,
               sdu->rx.rssi, sdu->rx.QLink);

        KDBG_LOG_UTIL(dbg_level, "   FC=0x%04x (type=%04x) dur=0x%04x seq=0x%04x%s%s\n",
               ieee2host16(sdu->frameCtrl), (int)sdl_ftype(sdu),
               ieee2host16(sdu->durationID), ieee2host16(sdu->seqCtrl),
               sdl_toDs(sdu) ? " [ToDS]" : "",
               sdl_frDs(sdu) ? " [FromDS]" : "");

        KDBG_LOG_UTIL(dbg_level, "   A1=" MACSTR " A2=" MACSTR " A3=" MACSTR "\n",
               MAC2STR(sdu->addr1), MAC2STR(sdu->addr2), MAC2STR(sdu->addr3));

#endif  //CMC venus
}
void dbg_frame_header(const Frame_t *sdu)
{
        dbg_frame_header2(sdu,KDBG_FRAME);
}
#endif


/*----------------------------------------------------------------*/
/*!
* \brief All chip-related paramters are set here. The priv_p struct
*        must be ready now.
*        Note: Starting address of RFD and TFCB are not setup here.
*
* \param
* \return 0:success, others:fail
*/
/*----------------------------------------------------------------*/
int hw_initializeAdapter(i2220mgmt_t *priv_p)
{
        KDBG_ENTER;

        //phyRFInitChannelList(priv_p);

        pauSoftwareReset(priv_p);       /* now PAU, FIFO, DMA are reset and disable */
        pauMCRInit(priv_p);             /* init MCR registers */
        phyBBInit(priv_p);              /* init baseband registers */
        phyRFInitAndWait(priv_p);       /* init RF in Normal mode except Channel and TX Power setting */

        /* Set the max rx packet size */
        ASSERT(MAX_RX_PKT_LEN>=32);
        NIC_MCR_WRITE(priv_p, MCR_MRBS, MAX_RX_PKT_LEN);
        KDBG_LOG_INIT(KDBG_INFO, "Max rx packet size = %d\n",MAX_RX_PKT_LEN);

        /* Set slot time (20u or 9u) depending on priv_p->phySlotType */
        if (priv_p->phySlotType == phyShortSlot) {
                pauChangeSlotTime(priv_p, SLOT_TIME_ERP_SHORT,
                        priv_p->x_CfbBurst ? FALSE : TRUE);
        }
        else {
                pauChangeSlotTime(priv_p, SLOT_TIME_ERP_LONG,
                        priv_p->x_CfbBurst ? FALSE : TRUE);
        }

#if SUPPORT_ED_ADJUST
        /* Get initial ED threshold and operation mode for adjacent channel */
        BCIWrite8(priv_p, 45, 0, FALSE, 0);
        BCIRead8(priv_p, 46, &priv_p->initAGC_OPMODE, FALSE, 0);
        KDBG_LOG_INIT(KDBG_INFO, "Init_AGCMode=0x%02x\n", priv_p->initAGC_OPMODE);
#endif

        /* Set the Rx antenna mode according to the MIB. */
        antSetRxAntennaMode(priv_p);

        /* Configure LED --> No trigger and TX: ON, RX: blink */
        pauConfigureLED(priv_p, NIC_LED_OUTPUT_RX, NIC_LED_NO_TRIGGER_ON);

        /* Set the response rates with basic rate(1,2,5.5 and 11) */
        pauSetCtrlRspTxRate(priv_p, RATE_SET_MASK_HR_DSSS);

        /* Set MAC address to both NIC card and NET4 net_device structure */
        hw_setOwnMacAddress(priv_p, priv_p->mib.dot11MACAddress);
        pauSetCurrentBssid(priv_p, priv_p->mib.dot11MACAddress);

        /* Set security keys and related (will moved to MLME_request)
         * the key talbes include soft(TX) and hw(RX)
         */
        /* a function call here */

        /* Set the filter of MAC fields for AP mode (0x00017801) */
        NIC_MCR_WRITE(priv_p, MCR_DRCR,
            DRCR_FCS_ERROR_FRAME_DISCARD | DRCR_RX_BROADCAST_ENABLE |
            DRCR_ADDR_2_BSSID_VALIDATION | DRCR_ADDR_3_BSSID_VALIDATION |
            DRCR_ADDR_3_OWN_SA_FILTERING | DRCR_ADDR_1_VALIDATION);

        /* Enable AP mode */
        pauSetMacOperationMode(priv_p, MAC_OPERATION_MODE_ESS_AP);

        /* Acknowledge all pending interrupts. */
        NIC_MCR_WRITE(priv_p, MCR_ISAR, 0xFFFFFFFF);
        NIC_MCR_WRITE(priv_p, MCR_QSR0, 0xFFFFFFFF);
        NIC_MCR_WRITE(priv_p, MCR_QSR1, 0xFFFFFFFF);

        /* Initialize the interrupt enable bits. */
#if WLAN_INCLUDE_DEBUG
        priv_p->interruptEnable = IER_RX_DATA_Q_DONE_INT | IER_RX_DATA_Q_EMPTY_INT | IER_PRE_TBTT_INT
                                 | IER_ABNORMAL_INT | IER_TX_DATA_Q_DONE_INT | IER_TX_DATA_Q_EMPTY_INT
                                 | IER_RFIFO_OVERFLOW_INT | IER_TARGET_HANG_INT
            #if SUPPORT_TKIP
                                 | IER_SEC_Q_DONE_INT | IER_SEC_Q_EMPTY_INT
            #endif
                                 ;
#else
        priv_p->interruptEnable = IER_RX_DATA_Q_DONE_INT | IER_RX_DATA_Q_EMPTY_INT | IER_PRE_TBTT_INT
                                 | IER_ABNORMAL_INT | IER_TX_DATA_Q_DONE_INT
            #if SUPPORT_TKIP
                                 | IER_SEC_Q_DONE_INT | IER_SEC_Q_EMPTY_INT
            #endif
                                 ;
#endif
        priv_p->qInterruptEnable = 0; /* Unmask Tx Queue in function dmaStartTxQueue() */

        /* Enable interrupts on our PCI board by setting the interrupt
         * enable bits to the IER.
         */
        NIC_MCR_WRITE(priv_p, MCR_IER, priv_p->interruptEnable);
        NIC_MCR_WRITE(priv_p, MCR_TXQIER, priv_p->qInterruptEnable);

        KDBG_EXIT;
        return 0;
}   /* hw_initializeAdapter */

/*----------------------------------------------------------------*/
/*!
* \brief  Set the MAC address for this NIC card.
*
* This routine will write a given individual address to the
* Own MAC Address Registers (OMARs) of the IPN2220
* chip.
*
* \param  priv_p        pointer to i2220mgmt_t, the device's private structure
* \param  ownMacAddr_p  Pointer to the own MAC address to be written to the OMRs
* \return none
* \post   the \a priv_p->netdev->dev_addr will also be set with the \a ownMacAddr_p
*/
/*----------------------------------------------------------------*/
void hw_setOwnMacAddress(i2220mgmt_t *priv_p, UINT8 *ownMacAddr_p)
{
        pauSetOwnMacAddress(priv_p, ownMacAddr_p);

        /* set the MAC address to NET4 net-device structure */
        kal_set_dev_macAddr(priv_p->netdev, ownMacAddr_p);
}   /* hw_setOwnMacAddress */

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
void hw_set_rate_table(i2220mgmt_t *priv_p)
{
        txRate_info_t   *txRate_p;
        UINT8           txRate;
        UINT32          tmpUlong, rateCode;
        UINT16          ctsAckDuration;
        int             i;

        /* This case may occur only when remove module, i.e.,
         * wlan_net_destroy() is invoked after i2220ap_remove_pci().
         * We should put txRate_info_t[] into priv_p in the future.
         */
        if (!priv_p->txRates_p) return;

        /* Fill the tx rate code by the preamble type */
        for (txRate_p = priv_p->txRates_p; txRate_p->txRate; txRate_p++) {
                rateGetTxRateCode(txRate_p->txRate,
                        priv_p->phyPreambleType, &rateCode);
                txRate_p->txRateCode = rateCode;
        }

        /* Update tx private table, which is consistent with HW table */
        txRate_p = priv_p->txRates_p;
        NIC_MCR_READ(priv_p, MCR_CRFCR0, &tmpUlong);
        for (i = 0; i < MAX_NUM_SUPPORTED_DATA_RATES; i++, txRate_p++) {
                if (!txRate_p->txRate) break;

                if (i == 6) {
                        NIC_MCR_READ(priv_p, MCR_CRFCR1, &tmpUlong);
                }
                rateCode = tmpUlong & RATE_CODE_RATE_MASK;
                if (rateGetRateByRateCode(rateCode, &txRate)) {
                        rateComputeCtsAckDuration(txRate,
                                priv_p->phyPreambleType,
                                &ctsAckDuration);
                }
                else {
                        KDBG_LOG_INIT(KDBG_WARN, "Wrong rsp rate\n");
                }
                txRate_p->rspRate = txRate;
                txRate_p->rspRateCode = rateCode;
                txRate_p->rspDuration = ctsAckDuration;
                tmpUlong >>= 5;
        }
}

/*----------------------------------------------------------------*/
/*!
* \brief All chip-related paramters are set here. The priv_p struct
*        must be ready now.
*        Note: Starting address of RFD and TFCB are not setup here.
*
* \param
* \return 0:success, others:fail
*/
/*----------------------------------------------------------------*/
void hw_set_rate_info(i2220mgmt_t *priv_p)
{
        UINT8           txRate;
        UINT32          cwin;
#if WLAN_INCLUDE_DEBUG
        txRate_info_t   *txRate_p;
        UINT32          tmpUlong, rateCode;
#endif
        KDBG_ENTER;

        priv_p->supportRates = 0;
        priv_p->basicRates = 0;

        /* Convert txrate to rate_set_bits */
        if (__convert_rates(mib_import(dot11OperationalRateSet.rateSet),
                        &priv_p->supportRates, &priv_p->basicRates,
                        mib_import(dot11OperationalRateSet.rateSetSize))
            != SYN_successful) {
                KDBG_LOG_INIT(KDBG_WARN, "Unknown supported rate in "
                        "MIB operation rate set\n");
        }
        if (priv_p->currentPhyType == PHY_TYPE_OFDM) {
                priv_p->supportRates &= RATE_SET_MASK_OFDM;
                priv_p->basicRates &= RATE_SET_MASK_OFDM;
        }
        KDBG_LOG_INIT(KDBG_DEBUG, "SuptRateSet=0x%08lx, BasicRateSet=0x%08lx\n",
                priv_p->supportRates, priv_p->basicRates);

        /* Set CWmin, CWmax, AIFSN and TXOPLimit of all queues and Beacon
         * depending on supported rates
         */
        cwin = priv_p->supportRates & RATE_SET_MASK_ERP_OFDM ? 15 : 31;
        /* Set the CWmin of Beacon */
        NIC_MCR_WRITE(priv_p, MCR_BCWR, cwin);
        /* Set the CWmin, CWmax, and AIFSN values for AC0 queue */
        pauSetACParameters(priv_p, TXQ_AC0, cwin, 1023, 2, 0x0ffff);
        /* Set the CWmin, CWmax, and AIFSN values for TQ0/TQ1. */
        pauSetACParameters(priv_p, TXQ_TQ0, cwin, 1023, 2, 0x0ffff);
        KDBG_LOG_INIT(KDBG_INFO, "Apply CFB burst to 2097ms\n");

        /* Configure respone rate table in NIC */
        pauSetCtrlRspTxRate(priv_p, priv_p->basicRates);

        /* Force slot time to long when only CCK supported rates */
        if (!(priv_p->supportRates & RATE_SET_MASK_ERP_OFDM)) {
                priv_p->phySlotType = phyLongSlot;
                pauChangeSlotTime(priv_p, SLOT_TIME_ERP_LONG,
                                priv_p->x_CfbBurst ? FALSE : TRUE);
        }

#if WLAN_INCLUDE_DEBUG
        NIC_MCR_READ(priv_p, MCR_CRFCR0, &tmpUlong);
        KDBG_LOG_INIT(KDBG_DEBUG, "CRFCR0=0x%08lx\n", tmpUlong);
        NIC_MCR_READ(priv_p, MCR_CRFCR1, &tmpUlong);
        KDBG_LOG_INIT(KDBG_DEBUG, "CRFCR1=0x%08lx\n", tmpUlong);
#endif
        /* Set tx rate table by preamble type */
        hw_set_rate_table(priv_p);

        /* Choose lowest basic rate for bcast and non-associated STA */
        if (!rateGetLowestRate(priv_p->basicRates, RATE_1M, &txRate, NULL)) {
                txRate = priv_p->currentPhyType == PHY_TYPE_OFDM ?
                         RATE_6M : RATE_1M;
        }
        priv_p->lowestBRate_p = __get_txRate_info(priv_p, txRate);

        /* Choose the highest nonERP basic rate as protection tx rate */
        if (!rateGetHighestRate(priv_p->basicRates & RATE_SET_MASK_HR_DSSS,
             RATE_11M, &txRate, NULL)) {
                /* All rates are ERP, use 1M as protection tx rate */
                txRate = RATE_1M;
        }
        priv_p->protectRate_p = __get_txRate_info(priv_p, txRate);

        /* Set the nonERP protect mode and tx rate by user's mib */
        //priv_p->protectMode = CTS_PROTECT_NONE;
        //priv_p->fixedTxRate_p = __get_txRate_info(priv_p, RATE_54M);


#if WLAN_INCLUDE_DEBUG
        for (txRate_p = priv_p->txRates_p; txRate_p->txRate; txRate_p++) {
                KDBG_LOG_INIT(KDBG_DEBUG,"[txRate=%dMbps]: "
                        "txRateCode=0x%04lx rspRate=%dMbps rspDur=%d\n",
                        txRate_p->txRate>>1, txRate_p->txRateCode,
                        txRate_p->rspRate>>1, txRate_p->rspDuration);
        }
        KDBG_LOG_INIT(KDBG_DEBUG,"lowestBRate=%d Mbps\n",
                priv_p->lowestBRate_p->txRate>>1);

        KDBG_LOG_INIT(KDBG_DEBUG,"protectRate=%d Mbps\n",
                priv_p->protectRate_p->txRate>>1);
#endif

        KDBG_EXIT;
}

/*----------------------------------------------------------------*/
/*!
* \brief All chip-related paramters are set here. The priv_p struct
*        must be ready now.
*        Note: Starting address of RFD and TFCB are not setup here.
*
* \param
* \return 0:success, others:fail
*/
/*----------------------------------------------------------------*/
void hw_set_channel(i2220mgmt_t *adapter_p, UINT8 txPowerLimit)
{
    BOOLEAN     status;
#if SUPPORT_802_11A
    UINT32      newPhyType;
    UINT32      tmp32;
    UINT8       newChannelBand = adapter_p->channelBand;
#endif
    UINT8       newChannelNum  = adapter_p->curChannel;

    KDBG_ENTER;

#if SUPPORT_802_11A
    status = TRUE;

    if (newChannelBand == CHANNEL_BAND_5) {
        newPhyType = PHY_TYPE_OFDM;
    } else {
        newPhyType = PHY_TYPE_ERP;
    }

    /* Change the current PHY type and program the Baseband for different PHY
       types, if necessary. */
    if (adapter_p->currentPhyType != newPhyType) {
        status = phyChangePhyType(adapter_p, newPhyType);

        if (status) {
            INITLOG(("Change PHY type: %s ---> %s\n",
                (adapter_p->currentPhyType == PHY_TYPE_OFDM) ?
                "5 GHz" : "2.4 GHz",
                (newPhyType == PHY_TYPE_OFDM) ? "5 GHz" : "2.4 GHz"));

            adapter_p->currentPhyType = newPhyType;

            NIC_MCR_READ(adapter_p, MCR_PCICR, &tmp32);

            /* Change to the slot time setting and minimum value of the
               contention window. */
            if (newPhyType == PHY_TYPE_OFDM) {
                /* Switch to the 5 GHz band. (RFBand = 1) */
                tmp32 |= PCICR_RF_BAND;
            } else {
                /* Switch to the 2.4 GHz band. (RFBand = 0) */
                tmp32 &= ~PCICR_RF_BAND;
            }

            /* Use short slot time as initial setting */
            adapter_p->phySlotType = phyShortSlot;
            pauChangeSlotTime(adapter_p,SLOT_TIME_SHORT,!adapter_p->x_CfbBurst);

            NIC_MCR_WRITE(adapter_p, MCR_PCICR, tmp32);
        }
    }

    if (status) {
        /* Switch the channel. */
        status = phyRFSwitchChannel(adapter_p, newChannelNum, newChannelBand,
            txPowerLimit);
    }
#else
    /* Switch the channel. */
    status = phyRFSwitchChannel(adapter_p, newChannelNum, txPowerLimit);
#endif

}

/*----------------------------------------------------------------*/
/*!
* \brief All chip-related paramters are set here. The priv_p struct
*        must be ready now.
*        Note: Starting address of RFD and TFCB are not setup here.
*
* \param
* \return 0:success, others:fail
*/
/*----------------------------------------------------------------*/
void hw_set_beacon(i2220mgmt_t *priv_p, BOOLEAN genesis)
{
        beacon_frame_t  *yTdu = &priv_p->beacon;
        UINT8           *cp;

        KDBG_ENTER;

        if (genesis) {
                /* Unchanged (static) Beacon content */
                yTdu->frameCtrl = host2ieee16(SYN_beacon);
                ipn_memcpy( yTdu->addr1, bcstAddr, MAC_ADDR_LEN );
                ipn_memcpy( yTdu->addr2, mib_import(dot11MACAddress), MAC_ADDR_LEN );
                ipn_memcpy( yTdu->addr3, mib_import(dot11MACAddress), MAC_ADDR_LEN );
                yTdu->LLCdata = (UINT8*)yTdu + wlanbody_offset;
                yTdu->LLClength = sizeof(beacon_frame_t) - wlanbody_offset;

                /* Enable TBTT, PreTBTT and DTIM timers */
                priv_p->dtimCount = 0;
                tsftimerInitializeTsfTimer(priv_p);
                NIC_MCR_WRITE(priv_p, MCR_TTAR1, (priv_p->mib.dot11MACAddress[5] << 2)
                        + priv_p->mib.dot11MACAddress[4]); /* for difference of each AP */
                tsftimerEnableTbttTimer(priv_p,
                        mib_import(dot11BeaconPeriod),
                        mib_import(dot11BeaconPeriod));

                tsftimerEnablePreTbttTimer(priv_p,
                        mib_import(dot11BeaconPeriod)*1024 - x_preTBTT,
                        mib_import(dot11BeaconPeriod));

                /* Note: dot11DTIMPeriod*dot11BeaconPeriod < 65535 */
                if (mib_import(dot11BeaconPeriod)*mib_import(dot11DTIMPeriod) > 65535) {
                        mib_import(dot11DTIMPeriod) = 65535 / mib_import(dot11BeaconPeriod);
                }
                tsftimerEnableDtimTimer(priv_p,
                        mib_import(dot11BeaconPeriod)*mib_import(dot11DTIMPeriod),
                        mib_import(dot11BeaconPeriod)*mib_import(dot11DTIMPeriod));

        } /* end of genesis */

        /* Stop Beacon by acquiring beacon control */
        if (priv_p->stop_accept_station) {
                UINT32          regValue;

                /* Check if we need to change the own bit of the Beacon
                 * Generation Control Register 1 (BGCR1) to the software.
                 */
                NIC_MCR_READ(priv_p, MCR_BGCR1, &regValue);

                if (regValue & BGCR1_BC_OWN) {
                        /* Force the software to own the BGRs. */
                        regValue &= ~BGCR1_BC_OWN;
                        NIC_MCR_WRITE(priv_p, MCR_BGCR1, regValue);
                }
                priv_p->do_checkHang = 0; /* Disable checking of Bcn Hang */
                return;
        }

        cp = __set_beacon_probrsp_frame(priv_p, yTdu, priv_p->x_hide_ssid, 0);
        sdlhw_send_beacon(priv_p, yTdu, cp - &yTdu->infoElem[0]);

        KDBG_EXIT;
        return;
}

/*----------------------------------------------------------------*/
/*!
* \brief  Read the MAC address from EEPROM.
*
* This routine will read the MAC addresss and other
* adapter configuration parameters from the EEPROM on the
* NIC.
*
* \pre  when \a priv_p->mib.dot11MACAddress isn't null when this subroutine is
*       invoked, it won't be overridden by the MAC address stored in EEPROM.
* \param  priv_p        pointer to i2220mgmt_t, the device's private structure
* \return true if success
*/
/*----------------------------------------------------------------*/
int hw_read_adapter_info(i2220mgmt_t *priv_p)
{
        int             i;
        BOOLEAN         eepromReady = TRUE;
        UINT16          eepromWordValue;
        UINT16          eepromPhyModeSetting = 0;
        UINT32          tmp32;
        KDBG_ENTER;

        /* Read the chip ID and version and RF chipset information
         * (control mode) from the PCICR.
         */
        
	NIC_MCR_READ(priv_p, MCR_PCICR, &tmp32);
//printk("Sam hw_read_adapter_info 970 NIC_MCR_READ tmp32=%08x membase=%08x\n",tmp32,priv_p->membase);
        /* Check the chip ID and version. */
        if ((tmp32 & (PCICR_CHIP_ID_VERSION | PCICR_REV_ID)) != IPN_CHIP_ID_REV
            && (tmp32 & (PCICR_CHIP_ID_VERSION | PCICR_REV_ID)) != IPN2220_A0) {
                KDBG_LOG_INIT(KDBG_INFO, "Unsupported chip ID & version:"
                        " PCICR = 0x%08lx\n", tmp32);
//printk("Sam hw_read_adapter_info 976\n");
                KDBG_EXIT;
                return NDIS_STATUS_HARD_ERRORS;
        }

        /* Check if the EEPROM is present and checksum correct. */
        NIC_MCR_READ(priv_p, MCR_ECSR, &tmp32);
        if ((tmp32 & ECSR_EE_TYPE) == ECSR_EE_TYPE_NOT_PRESENT ||
            (tmp32 & ECSR_EE_CSERR)) {
                KDBG_LOG_INIT(KDBG_INFO, "EEPROM not present/checksum error"
                        " -- Use default settings\n");
                eepromReady = FALSE;

//printk("Sam hw_read_adapter_info 989\n");
        }

        if (eepromReady) {
                /* Save the current RF mode. */
                NIC_MCR_READ(priv_p, MCR_PCICR, &tmp32);
                priv_p->rfCtrlMode = tmp32 & PCICR_RF_CTRL_MODE;

                /* Read the NIC PHY mode from the EEPROM. */
                eepromPhyModeSetting = ReadEEProm16(priv_p,
                                                EEPROM_LED_CFG_PHY_MODE);

                /* Set up the RF-specific configuration table
                 * according to the current RF mode.
                 */
        #if SUPPORT_802_11A
                nicSetupRFSpecificConfig(priv_p, eepromPhyModeSetting);
        #else
                nicSetupRFSpecificConfig(priv_p);
        #endif
                if (priv_p->rfSpecificConfig_p == NULL) {
                        KDBG_LOG_INIT(KDBG_INFO, "Unsupported RF mode: %d\n",
                                priv_p->rfCtrlMode);

//printk("Sam hw_read_adapter_info 1013\n");
                        KDBG_EXIT;
                        return NDIS_STATUS_HARD_ERRORS;
                }

                KDBG_LOG_INIT(KDBG_INFO,"RF control mode: %s (%d)\n",
                        priv_p->rfSpecificConfig_p->rfName_p,
                        priv_p->rfCtrlMode);
        }

        /* Read the permanent MAC address from the EEPROM. */
        if (eepromReady) {
                for (i = 0; i < 3; i++) {
                        eepromWordValue = ReadEEProm16(priv_p,
                                EEPROM_MAC_ADDR_BYTE_0_1 + (UCHAR) i);
                        KDBG_LOG_INIT(KDBG_INFO, "EEPROM word offset 0x%02x: 0x%04x\n",
                                EEPROM_MAC_ADDR_BYTE_0_1 + i, eepromWordValue);
                        priv_p->permanentMacAddress[i * 2] =
                                (UCHAR) (eepromWordValue & 0x00FF);
                        priv_p->permanentMacAddress[i * 2 + 1] =
                                (UCHAR) (eepromWordValue >> 8);
                }
        }

        /* If the EEPROM is not programmed, we use an all-zero-bits MAC address to
           replace the permanent address. */
        if (priv_p->permanentMacAddress[0] == 0xff &&
            priv_p->permanentMacAddress[1] == 0xff &&
            priv_p->permanentMacAddress[2] == 0xff &&
            priv_p->permanentMacAddress[3] == 0xff &&
            priv_p->permanentMacAddress[4] == 0xff &&
            priv_p->permanentMacAddress[5] == 0xff) {
                KDBG_LOG_INIT(KDBG_INFO, "Permanent address not programmed\n");

                ipn_memset(priv_p->permanentMacAddress, 0, 6);
        }
        /* Save permanent MAC address to MIB */
        COPY_MAC_ADDR(priv_p->mib.dot11MACAddress, priv_p->permanentMacAddress);

        /* CMC: temporary, just for testing 6/18 */
        //ipn_memset(priv_p->mib.dot11MACAddress, 0x6c, MAC_ADDR_LEN);

        KDBG_LOG_INIT(KDBG_INFO, "MAC Address is %02x-%02x-%02x-%02x-%02x-%02x\n",
        priv_p->mib.dot11MACAddress[0], priv_p->mib.dot11MACAddress[1],
        priv_p->mib.dot11MACAddress[2], priv_p->mib.dot11MACAddress[3],
        priv_p->mib.dot11MACAddress[4], priv_p->mib.dot11MACAddress[5]);

        /* Validate our current node address (make sure it's not a group address) */
        if (priv_p->mib.dot11MACAddress[0] & 1) {
                KDBG_LOG_INIT(KDBG_INFO, "Node address invalid -- "
                        "It's a group address\n");

//printk("Sam hw_read_adapter_info 1065\n");
                KDBG_EXIT;
                return NDIS_STATUS_INVALID_ADDRESS;
        }

        /* Read the current regulatory domain from the EEPROM. */
        if (eepromReady) {
                eepromWordValue = ReadEEProm16(priv_p, EEPROM_REG_DOMAIN);
                KDBG_LOG_INIT(KDBG_INFO, "EEPROM word offset 0x%02x: 0x%04x\n",
                        EEPROM_REG_DOMAIN, eepromWordValue);
        } else {
                /* The default value of the current regulatory domain is 0 when the
                   EEPROM is not ready. */
                eepromWordValue = 0;
        }

        /* Set up the regulatory domain related information according to the
         * value read from the EEPROM.
         */
        domainInitCurrentRegDomain(priv_p, eepromWordValue);

        /* Read the NIC antennas settings from the EEPROM. */
        if (eepromReady) {
                eepromWordValue = ReadEEProm16(priv_p,
                        EEPROM_INTERFACE_CFG_CHECKSUM);
                KDBG_LOG_INIT(KDBG_INFO, "EEPROM word offset 0x%02x: 0x%04x\n",
                        EEPROM_INTERFACE_CFG_CHECKSUM, eepromWordValue);
        } else {
                eepromWordValue = 0;
                KDBG_LOG_INIT(KDBG_INFO, "EEPROM not ready -- use default "
                        "values for antenna list\n");
        }
        /* Initialize the list of NIC antennas. */
#if SUPPORT_802_11A
        antInitAntennaList(priv_p, eepromWordValue, eepromPhyModeSetting);
#else
        antInitAntennaList(priv_p, eepromWordValue);
#endif
        /* Initialize channel power */
        if (eepromReady) {
                nicReadPerChannelConfig(priv_p);
        }

        KDBG_EXIT;
        return NDIS_STATUS_SUCCESS;
}   /* hw_read_adapter_info */


/*----------------------------------------------------------------*/
/*!
* \brief  An deferred Interrupt Service Routine for INT_PRE_TBTT interrupt.
*
* This routine is called by \a do_isr_tasklet to process pre-TBTT interrupts,
* the SDL env interface will signify the timing occurrence of TBTT and beacon
* frame will be generated.
*
* \param  priv_p        pointer to i2220mgmt_t, the device's private structure
* \return none
* \remarks the \a env_IF_MAC_HARDWARE with \a SIG_Tbcn signal, is a call back
*       function to the state machine handling the Beacon generation.
*/
/*----------------------------------------------------------------*/
void
isr_process_preTBTT(i2220mgmt_t *priv_p)
{
        UINT32          rc_count;
        KDBG_ENTER;

#if SUPPORT_LOOPBACK
        if (priv_p->loop.enable) {
               return;
        }
#endif
        /* Check for Hang with CFB is enabled. do_checkHang is
         * to avoid wrong action when the fisrt preTBTT.
         */
        NIC_MCR_READ(priv_p, MCR_ATBCR, &rc_count);
        if (priv_p->do_checkHang) {
                priv_p->do_checkHang = rc_count ? 1 : priv_p->do_checkHang+1;
                if (priv_p->do_checkHang > BCN_TX_LOSS_LIMIT) {
                        priv_p->macResetBcnCount++; /* debug */
                        i2220hw_reset_mac(priv_p);
                        hw_set_beacon(priv_p, TRUE);
                        priv_p->do_checkHang = 0;
                        return;
                }
        }
        else if (rc_count > 0) {
                /* Startup checking mechanism. In some platform, we fail to
                 * read RC counter of chip, i.e., counter=0
                 */
                priv_p->do_checkHang = 1;
        }

        /* Calculate new dtimCount */
        if (priv_p->dtimCount == 0){
                ASSERT(mib_import(dot11DTIMPeriod)>0);
                priv_p->dtimCount = mib_import(dot11DTIMPeriod) - 1;
        }
        else {
                priv_p->dtimCount--;
        }
        /* Update beacon content */
        hw_set_beacon(priv_p, FALSE);

        /* Tx out all bufferd bcast frames if dtimCount=0 */
        if (priv_p->dtimCount == 0) {
                FragSdu_t       *fsdu;

                /* dequeue all buffered bcast frames */
                while (sdlQ_Length(&priv_p->bcQ) != 0) {
                        sdlQ_dequeue_first(&priv_p->bcQ, fsdu);
                        /* bcast frame is sent on DTIM, not too long,
                         * so we don't check time out here.
                         */
                        if (sdlQ_Length(&priv_p->bcQ) != 0){
                                sdl_setMoreData(fsdu->sw.sdu, 1);
                        }
                        sdlu_tx_FragSdu(priv_p, fsdu, TXQ_TQ1, TRUE);
                }
        }

#if SUPPORT_ED_ADJUST
        /* Adjust noise floor to anti adjacent channel interference
         * Now the calculation interval 1000TU is about 1sec.
         */
        if (++priv_p->tbttCount >=
            CHECK_ADJ_CHNL_INTERVAL / mib_import(dot11BeaconPeriod)) {
                priv_p->tbttCount = 0;

                /* Idle and CCA counts can't be zero at the same time */
                NIC_MCR_READ(priv_p, MCR_CACR, &rc_count);
                if (rc_count == 0) {
                        NIC_MCR_READ(priv_p, MCR_CICR, &rc_count);
                }

                if (rc_count > 0) {
                        edUpdateNoiseFloor(priv_p, CHECK_ADJ_CHNL_INTERVAL,
                                priv_p->hwStatistics.rxRfbRxOkCounter -
                                priv_p->rxRfbRxOkCounter, ED_NORMAL);

                        priv_p->rxRfbRxOkCounter =
                                        priv_p->hwStatistics.rxRfbRxOkCounter;
                }
        }
#endif
        KDBG_EXIT;
        return;
}   /* isr_process_preTBTT */

#if WLAN_INCLUDE_DEBUG
/*----------------------------------------------------------------*/
/*!
* \brief  An deferred Interrupt Service Routine for TDMA interrupt of
*          TDSR_TDMA##ch##_BUFFER_EMPTY
*
* This routine is called by \a do_isr_tasklet to
* process TDMA transmit buffer empty interrupts for IPN2220
* adapters. Currently, this routine will restart the TDMA
* if the TDMA active chain is not empty in order to work
* around Bug 152.
*
* \param  priv_p       pointer to i2220mgmt_t, the device's private structure
* \param  tdmaChannel  The number of the TDMA channel on which transmit
*                      interrupts occurred
* \return none
*/
/*----------------------------------------------------------------*/
void
isr_process_tx_buffer_empty(i2220mgmt_t *priv_p, UINT tdmaChannel)
{
        return;
}   /* isr_process_tx_buffer_empty */
#endif

/*----------------------------------------------------------------*/
/*!
* \brief  An deferred Interrupt Service Routine for TDMA interrupt of
*          TDSR_TDMA##ch##_FRAME_TX_DONE
*
* This routine is called by \a do_isr_tasklet to
* process transmit interrupts for IPN2220 adapters.
* Basically, this routine will remove any completed packets
* from the active transmit chain, append them to the
* completed list.  isr_transmit_cleanup will be called to free
* up the TFCBs that the packets on the completed list
* consumed.
*
* \param  priv_p       pointer to i2220mgmt_t, the device's private structure
* \param  tdmaChannel  The number of the TDMA channel on which transmit
*                      interrupts occurred
* \return -1    Failure \n
*         1     more TFCBs to be processed, unfinished \n
*         0     all TFCBs are done
* \remarks the \a env_IF_MAC_HARDWARE with \a SIG_PhyTxEnd_confirm signal,
*       is a call back function to clean up the \a tfcb_p TFCB control object
*       and associated structures.
*/
/*----------------------------------------------------------------*/
int isr_process_tx_done(i2220mgmt_t *priv_p, UINT tdmaChannel)
{
        tdma_parameters_t       *tdma_p;
        tfcb_t          *tfcb_p;
        FragSdu_t       *fsdu;
        sta_info_t      *sta_p;
        int             i, nTFCB, txQs_empty=TRUE;

        KDBG_ENTER;

        ASSERT(tdmaChannel<NUM_TDMA_CHANNELS-2);
        tdma_p = &priv_p->tdma[tdmaChannel];

        if (tdma_p->head_p == NULL) {
                KDBG_LOG_TX(KDBG_ERR,"TDMA%d head_p is NULL!!\n", tdmaChannel);
                KDBG_EXIT;
                return -1;
        }

        /* Look at the TFCB at the head of the queue.  If it has been completed
           then pop it off and place it at the tail of the completed list.
           Repeat this process until all the completed TFCBs have been moved to
           the completed list */
        tfcb_for_each(tdma_p->head_p, tfcb_p)
        {
                /* check to see if the TFCB has been DMA'd */
                if (!(chip2host_16(tfcb_p->hw.txStatus) & TFCB_STATUS_MASK)) {

                #if WLAN_INCLUDE_DEBUG | WP_3210_WORKAROUND
                        tfcb_t          *next_tfcb_p;
                        next_tfcb_p = tfcb_p->nextTfcb;
                        if (!(next_tfcb_p->hw.tfdCtrlFlag & TFCB_FLAG_EOL) &&
                            (chip2host_16(next_tfcb_p->hw.txStatus) & TFCB_STATUS_MASK)) {
                                KDBG_LOG_TX(KDBG_ERR,"\nTDMA %d TxStatus is skipped\n\n",
                                        tdmaChannel);
                                dbg_tx_tdma(priv_p, tdmaChannel, 2);
                                goto L_tx_skipping;
                        }
                #endif
                        break;
                }

#if WLAN_INCLUDE_DEBUG | WP_3210_WORKAROUND
L_tx_skipping:
#endif

        #if __CPU_ENDIAN__ != __LITTLE_ENDIAN__
                tfcb_p->hw.txStatus = chip2host_16(tfcb_p->hw.txStatus);
        #endif

                ASSERT(tfcb_p->sw.fsdu);
                fsdu = tfcb_p->sw.fsdu;
                nTFCB = 1;

                /* Update some information in sta_p, like tx rate,
                 * activity time and statistics
                 */
                sta_p = fsdu->sw.sta_p;
                if (sta_p && sta_p->magicID == STA_MAGIC_ID) {
                        /* Make decision of tx rate swith */
                        auto_txrate_switch(priv_p, sta_p, tfcb_p);

                        if (tfcb_p->hw.txStatus & TFCB_STATUS_OK) {
                                sta_p->actTime = kal_jiffies; /* update activity time */
                        }
                        else if (check_timeout(sta_p->actTime + STA_MAX_INACTIVITY)
                                 && sta_p->aid >= 0) {
                                /* Inactivity time out (aging) */
                                free_sta(priv_p, sta_p);
                        }
                } /* end of sta_p really exists, some risks */

        #if WLAN_INCLUDE_DEBUG
                if (tfcb_p->hw.txStatus & TFCB_STATUS_OK) {
                        KDBG_LOG_TX(KDBG_DEBUG,"Tx OK(frag=%d/%d, count=%d), txStatus=0x%02x\n",
                                tfcb_p->sw.frag_nr, fsdu->fTot, tdma_p->count, tfcb_p->hw.txStatus);
                }
                else {
                        KDBG_LOG_TX(KDBG_DEBUG,"Tx failed(frag=%d/%d, count=%d), txStatus=0x%02x\n",
                                tfcb_p->sw.frag_nr, fsdu->fTot, tdma_p->count, tfcb_p->hw.txStatus);
                }
        #endif
                isr_transmit_update_stats(priv_p, tfcb_p,sta_p);

                if (tfcb_p->hw.txStatus & TFCB_STATUS_FRAG_SKIP_ERR){
                        nTFCB = fsdu->fTot - tfcb_p->sw.frag_nr;
                        KDBG_LOG_TX(KDBG_DEBUG,"Frag_skip nTFCB=%d(%d-%d),txTFCBs=%d\n",
                                nTFCB, fsdu->fTot, tfcb_p->sw.frag_nr, tdma_p->count);
                }

                for (i = 0; i < nTFCB; i++) {
                        tdma_p->count--;

                        /* Cleanup after the transmits that have already been sent -- free their
                           TFCBs and coalesce buffers. */
                        isr_transmit_cleanup_TBB(priv_p, tfcb_p, TRUE);

                        sdlu_release_tfcb(tfcb_p);

                        /* Put back the TFCB to the free chain list */
                        add_chain_tail(&priv_p->tfcb_chain, tfcb_t, tfcb_p);

                        if (i < nTFCB - 1) {
                                /* last one is reserved for tfcb_for_each loop */
                                tfcb_p = tfcb_p->nextTfcb;
                        }
                } /* end of i<nTFCB */
        }

        tdma_p->head_p = tfcb_p;  /* important */

        /* Check wait queue of all TDMA channels */
        for (i = 0; i < NUM_TDMA_CHANNELS - 2; i++) {
                tdma_p = &priv_p->tdma[i];
                waitQ_toTDMAChannel(&tdma_p->waitQ, fsdu, i);

                /* Check if all txQs are empty */
                if (tdma_p->head_p &&
                    !(tdma_p->head_p->hw.tfdCtrlFlag & TFCB_FLAG_EOL)) {
                        txQs_empty = FALSE;
                }
        }

#if !WLAN_INCLUDE_DEBUG
        /* Restart tx_timeout value if txQs are not empty */
        if ( !txQs_empty ) {
                mgmt_renew_timer(&priv_p->tx_timer, TX_TIMEOUT_INTERVAL);
        }
        else {
                mgmt_stop_timer(&priv_p->tx_timer);
                priv_p->chk_txTimout = FALSE;
        }
#endif

#if WLAN_SUPPORT_RFTEST
        if (priv_p->rftest_burst && txQs_empty) {
                i2220_rftest_burst(priv_p);
        }
#endif
        KDBG_EXIT;
        return 0;
}

#if SUPPORT_TKIP
/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
int isr_process_TxCalcMic_done(i2220mgmt_t *priv_p)
{
        tdma_parameters_t       *tdma_p;
        tfcb_t                  *tfcb_p;
        FragSdu_t               *fsdu;
        int                     i;
#if SINGLE_SEC_QUEUE
        Frame_t                 *pdu;
        sta_info_t              *sta_p;
        int                     forward_stat;
#endif

        KDBG_ENTER;
        tdma_p = &priv_p->tdma[SECQ_TX];

        if (tdma_p->head_p == NULL) {
                KDBG_LOG_TX(KDBG_ERR,"TDMA%d head_p is NULL!!\n", SECQ_TX);
                KDBG_EXIT;
                return -1;
        }

        /* Look at the TFCB at the head of the queue.  If it has been completed
         * then pop it off and place it at the tail of the completed list.
         * Repeat this process until all the completed TFCBs have been moved to
         * the completed list
         */
        tfcb_for_each(tdma_p->head_p, tfcb_p)
        {
                /* Check to see if the TFCB has been DMA'd */
                ASSERT(tfcb_p->sw.fsdu);
                fsdu = tfcb_p->sw.fsdu;

                /* Don't care MCR_SEORCAR and OK in whatever endian */
                if (tfcb_p->tkipMic[39] == 0 ||
                    (tfcb_p != tdma_p->head_p && *(UINT32*)&tfcb_p->tkipMic[44] == 0)) {
                        break;
                }
                if (*(UINT32*)&tfcb_p->tkipMic[44] == 0) {
                        kal_udelay(20);
                }

                /* Un-map data buffer */
                isr_transmit_cleanup_TBB(priv_p, tfcb_p, FALSE);

                /* Put back the TFCB to the free chain list */
                add_chain_tail(&priv_p->sec_tfcb_chain, tfcb_t, tfcb_p);
                tdma_p->count--;

                KDBG_LOG_TX(KDBG_DEBUG,"Tx Mic calc loopback OK(frag=%d/%d, count=%d)\n",
                        tfcb_p->sw.frag_nr, fsdu->fTot, tdma_p->count);

        #if SINGLE_SEC_QUEUE
                /* Check this packet is in RX or TX path */
                if (tfcb_p->tkipMic[39] == 0xa1) {

                        ipn_memcpy(fsdu->tkipMic, &tfcb_p->tkipMic[40], 8);

                        env_PHY_SAP_TX(priv_p, fsdu); /* TX path */
                }
                else {                                /* RX path */
                        pdu = fsdu->sw.sdu;
                        sta_p = fsdu->sw.sta_p;

                        /* Check rx pakcet MIC error, then do fordwarding */
                        if (!ipn_memcmp(&tfcb_p->tkipMic[40],
                                    pdu->LLCdata + pdu->LLClength, 8)) {

                                /* whsu say: the fsdu should be freed before
                                 * the new fsdu use for pdu...
                                 */
                                kal_mempool_free(fsdu);

                                if (pdu->mac_ctl & MACCTL_WDS_FRAME) {
                                        forward_stat = wds_rx_do_forward(priv_p, pdu, sta_p);
                                }
                                else {
                                        forward_stat = rx_do_forward(priv_p, pdu, sta_p);
                                }
                                if (forward_stat == RX_DRV_HANDLED_PACKET) {
                                        /* We are here when 802.1x packet is
                                         * ciphered with TKIP
                                         */
                                        KDBG_LOG_RX(KDBG_WARN, "Drv handle this"
                                                " pkt after cal rxMIC.\n");
                                        sdlu_release_sdu(pdu);
                                }
                        }
                        else {
                                /* Check the TKIP MIC and MIC':
                                 * if the MIC check is fail, then we have to invoke
                                 * appropriate countermeasure here
                                 */
                                KDBG_LOG_RX(KDBG_WARN, "Rx Mic verify error!!\n");
                                KDBG_LOG_RX(KDBG_WARN, "calc tkip mic %02x %02x"
                                        " %02x %02x %02x %02x %02x %02x \n",
                                        tfcb_p->tkipMic[40], tfcb_p->tkipMic[41],
                                        tfcb_p->tkipMic[42], tfcb_p->tkipMic[43],
                                        tfcb_p->tkipMic[44], tfcb_p->tkipMic[45],
                                        tfcb_p->tkipMic[46], tfcb_p->tkipMic[47]);
                                KDBG_LOG_RX(KDBG_WARN, "orig tkip mic %02x %02x"
                                        " %02x %02x %02x %02x %02x %02x \n",
                                        pdu->LLCdata[pdu->LLClength],
                                        pdu->LLCdata[pdu->LLClength+1],
                                        pdu->LLCdata[pdu->LLClength+2],
                                        pdu->LLCdata[pdu->LLClength+3],
                                        pdu->LLCdata[pdu->LLClength+4],
                                        pdu->LLCdata[pdu->LLClength+5],
                                        pdu->LLCdata[pdu->LLClength+6],
                                        pdu->LLCdata[pdu->LLClength+7]);

                                kal_tkip_micFailure(priv_p->netdev, pdu->addr2);
                                sdlu_release_fsdu(fsdu);
                        }
                } /* end of processing rx packet */
        #else
                ipn_memcpy(fsdu->tkipMic, &tfcb_p->tkipMic[40], 8);

                env_PHY_SAP_TX(priv_p, fsdu);

        #endif /* end of SINGLE_SEC_QUEUE */
        }

        tdma_p->head_p = tfcb_p;  /* Important */

        /* Check wait queue of security queue channels */
        for (i = SECQ_RX; i < NUM_TDMA_CHANNELS; i++) {
                tdma_p = &priv_p->tdma[i];
                waitQ_toTDMA_SecChannel(&tdma_p->waitQ, fsdu, i);
        }

        KDBG_EXIT;
        return 0;
}

#if !SINGLE_SEC_QUEUE
/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
int isr_process_RxCalcMic_done(i2220mgmt_t *priv_p)
{
        tdma_parameters_t       *tdma_p;
        tfcb_t                  *tfcb_p;
        FragSdu_t               *fsdu;
        Frame_t                 *pdu;
        sta_info_t              *sta_p;
        int                     i;
        int                     forward_stat;

        KDBG_ENTER;
        tdma_p = &priv_p->tdma[SECQ_RX];

        if (tdma_p->head_p == NULL) {
                KDBG_LOG_TX(KDBG_ERR,"TDMA%d head_p is NULL!!\n", SECQ_RX);
                KDBG_EXIT;
                return -1;
        }

        /* Look at the TFCB at the head of the queue.  If it has been completed
           then pop it off and place it at the tail of the completed list.
           Repeat this process until all the completed TFCBs have been moved to
           the completed list */
        tfcb_for_each(tdma_p->head_p, tfcb_p)
        {
                /* check to see if the TFCB has been DMA'd */
                ASSERT(tfcb_p->sw.fsdu);
                fsdu = tfcb_p->sw.fsdu;

                /* Don't care MCR_SEORCAR and OK in whatever endian */
                if (tfcb_p->tkipMic[39] == 0 ||
                    (tfcb_p != tdma_p->head_p && *(UINT32*)&tfcb_p->tkipMic[44] == 0)) {
                        break;
                }
                if (*(UINT32*)&tfcb_p->tkipMic[44] == 0) {
                        kal_udelay(20);
                }

                /* Un-map data buffer */
                isr_transmit_cleanup_TBB(priv_p, tfcb_p, FALSE);

                /* Put back the TFCB to the free chain list */
                add_chain_tail(&priv_p->sec_tfcb_chain, tfcb_t, tfcb_p);
                tdma_p->count--;

        #if SINGLE_SEC_QUEUE
                if (tfcb_p->tkipMic[39] == 0xa0) {
        #endif
                pdu = fsdu->sw.sdu;
                sta_p = fsdu->sw.sta_p;

                /* Compare the tkip MIC */
                if (!ipn_memcmp(&tfcb_p->tkipMic[40], pdu->LLCdata + pdu->LLClength, 8)) {
                        /* whsu say: the fsdu should be kill before the
                         * new fsdu use for pdu...
                         */
                        kal_mempool_free(fsdu);

                        if (pdu->mac_ctl & MACCTL_WDS_FRAME) {
                                forward_stat = wds_rx_do_forward(priv_p, pdu, sta_p);
                        }
                        else {
                                forward_stat = rx_do_forward(priv_p, pdu, sta_p);
                        }
                        if (forward_stat == RX_DRV_HANDLED_PACKET) {
                                /* We are here when 802.1x packet is ciphered with TKIP */
                                KDBG_LOG_RX(KDBG_WARN, "Drv handle this pkt after cal rxMIC.\n");
                                sdlu_release_sdu(pdu);
                        }
                }
                else {
                        /* Check the TKIP MIC and MIC':
                           if the MIC check is fail, then we have to invoke
                           appropriate countermeasure here */
                        KDBG_LOG_RX(KDBG_WARN, "Rx Mic verify error!!\n");
                        KDBG_LOG_RX(KDBG_WARN, "calc tkip mic %02x %02x %02x "
                            "%02x %02x %02x %02x %02x\n",
                            tfcb_p->tkipMic[40], tfcb_p->tkipMic[41],
                            tfcb_p->tkipMic[42], tfcb_p->tkipMic[43],
                            tfcb_p->tkipMic[44], tfcb_p->tkipMic[45],
                            tfcb_p->tkipMic[46], tfcb_p->tkipMic[47]);
                        KDBG_LOG_RX(KDBG_WARN, "orig tkip mic %02x %02x %02x "
                            "%02x %02x %02x %02x %02x\n",
                            pdu->LLCdata[pdu->LLClength], pdu->LLCdata[pdu->LLClength+1],
                            pdu->LLCdata[pdu->LLClength+2], pdu->LLCdata[pdu->LLClength+3],
                            pdu->LLCdata[pdu->LLClength+4], pdu->LLCdata[pdu->LLClength+5],
                            pdu->LLCdata[pdu->LLClength+6], pdu->LLCdata[pdu->LLClength+7]);

                        kal_tkip_micFailure(priv_p->netdev, pdu->addr2);
                        sdlu_release_fsdu(fsdu);
                }
        #if SINGLE_SEC_QUEUE
                }
                else {
                        /* TX path */
                        ipn_memcpy(fsdu->tkipMic, &tfcb_p->tkipMic[40], 8);

                        env_PHY_SAP_TX(priv_p, fsdu);
                }
        #endif
        }

        tdma_p->head_p = tfcb_p;  /* Important */

        /* Check wait queue of all security queue channels */
        for (i = SECQ_RX; i < NUM_TDMA_CHANNELS; i++) {
                tdma_p = &priv_p->tdma[i];
                waitQ_toTDMA_SecChannel(&tdma_p->waitQ, fsdu, i);
        }

        KDBG_EXIT;
        return 0;
}
#endif /* !SINGLE_SEC_QUEUE */
#endif /* SUPPORT_TKIP */

/*----------------------------------------------------------------*/
/*!
* \brief  A ISR supporting subroutine for \a isr_process_tx_done.
*
* This routine will update the transmit statistic counters.
*
* \param  priv_p       pointer to i2220mgmt_t, the device's private structure
* \param  tfcb_p       pointer to the TFCB just being transmitted
* \return none
*/
/*----------------------------------------------------------------*/
void isr_transmit_update_stats(i2220mgmt_t *priv_p, tfcb_t *tfcb_p,sta_info_t* sta_p)
{
        UINT16          txStatus;
        BOOLEAN         rtsCtsUsage = FALSE;
        FragSdu_t       *fsdu;
        DOT11_COUNTERS_ENTRY       *tx_done_sta_stat;
        DOT11_COUNTERS_ENTRY       fake_sta_stat;

        KDBG_ENTER;
        fsdu = tfcb_p->sw.fsdu;
        if(fsdu && fsdu->sw.sta_p){
                tx_done_sta_stat = &fsdu->sw.sta_p->sta_statistics;
        }else{
                //To save us from if(sta_p)
                tx_done_sta_stat = &fake_sta_stat;
        }

        /* Update the 802.11 MIB counter dot11ACKFailureCount according to the
         * MPDU Retry Count field.
         */
        priv_p->mib.dot11ACKFailureCount += (ULONG) tfcb_p->hw.mpduRetryCount;
        tx_done_sta_stat->dot11ACKFailureCount += tfcb_p->hw.mpduRetryCount;

        /* Update the 802.11 MIB counters dot11RTSSuccessCount and
         * dot11RTSFailureCount according to the RTS Retry Count and MPDU Retry
         * Count fields.
         */
        if (tfcb_p->hw.tciCtrlFlag & TFCB_CTRL_RTS) {
                rtsCtsUsage = TRUE;

                priv_p->mib.dot11RTSSuccessCount += (ULONG) tfcb_p->hw.mpduRetryCount;
                priv_p->mib.dot11RTSFailureCount += (ULONG) tfcb_p->hw.rtsRetryCount;

                tx_done_sta_stat->dot11RTSSuccessCount += tfcb_p->hw.mpduRetryCount;
                tx_done_sta_stat->dot11RTSFailureCount += tfcb_p->hw.rtsRetryCount;
        }

        /* Get transmit status from the TFCB */
        txStatus = tfcb_p->hw.txStatus;

        /* For debug use */
        //KDBG_LOG_TX(KDBG_DEBUG, "Tx%s %d %ldus\n", (txStatus & TFCB_STATUS_OK) ? "OK" : "BAD"
        //        , tfcb_p->hw.mpduLength, currentSystemTime - tfcb_p->sw.transmitStartTime);

        if (txStatus & TFCB_STATUS_OK) {

                /* For debug */
                priv_p->hwStatistics.txTfcbTxOkCounter++;

                /* Since we successfuly transmitted an MPDU of type Data or
                 * Management, we increment the 802.11 MIB counter
                 * dot11TransmittedFragmentCount.
                 */
                priv_p->mib.dot11TransmittedFragmentCount++;
                if ( tfcb_p->hw.mpduRetryCount > 0 ){
                        priv_p->mib.dot11RetryCount++;
                        tx_done_sta_stat->dot11RetryCount++;
                        if ( tfcb_p->hw.mpduRetryCount > 1 ){
                                priv_p->mib.dot11MultipleRetryCount++;
                                tx_done_sta_stat->dot11MultipleRetryCount++;
                        }
                }

                /* If this MPDU was transmitted successfully and the RTS/CTS
                 * procedure was used, we increment the 802.11 MIB counter
                 * dot11RTSSuccessCount by 1 for the RTS success of the last
                 * transmission.
                 */
                if (rtsCtsUsage) {
                        priv_p->mib.dot11RTSSuccessCount++;
                        tx_done_sta_stat->dot11RTSSuccessCount++;
                }


                /*When successful TX to a remote AP, we will update tx fail count.*/
                #if WLAN_INCLUDE_WDS
                if(sta_p && sta_p->wds_link){
                        wdslink_info_t* wds = sta_p->wds_link;
                        wds->tx_fail_count = 0;

                        if(wds->disabled_status){
                                kal_indicate_wds_tx_up(wds->dev);
                                wds->disabled_status = 0;
                        }
                }
                #endif
        }
        else {
                /* For debug */
                priv_p->hwStatistics.txTfcbTxErrCounter++;

                /* If the transmission of this MPDU was failed due to the MPDU
                 * retry error, we increment the 802.11 MIB counter
                 * dot11ACKFailureCount by 1 for the ACK failure of the last
                 * retransmission.
                 */
                if (txStatus & TFCB_STATUS_MPDU_RETRY_ERR) {
                        priv_p->mib.dot11ACKFailureCount++;
                        tx_done_sta_stat->dot11ACKFailureCount++;
                        #if WLAN_INCLUDE_WDS
                        if(sta_p && sta_p->wds_link){
                                wdslink_info_t* wds = sta_p->wds_link;
                                wds->tx_fail_count ++;
                                if(wds->tx_fail_count >= priv_p->wds_tx_failure_limit){
                                        kal_indicate_wds_tx_down(wds->dev);
                                        wds->tx_fail_count = 0;
                                        wds->last_disable_time = kal_jiffies;
                                        wds->disabled_status = 1;
                                }
                        }
                        #endif

                }

                /* If the RTS/CTS procedure was used before transmitting this MPDU
                 * and the RTS retry error occured, we increment the 802.11 MIB
                 * counter dot11RTSFailureCount by 1 for the RTS failure of the
                 * last transmission.
                 */
                if (rtsCtsUsage && (txStatus & TFCB_STATUS_RTS_RETRY_ERR)) {
                        priv_p->mib.dot11RTSFailureCount++;
                        tx_done_sta_stat->dot11RTSFailureCount++;
                }
        }

        /* check if this is the last fragment of the packet */
        fsdu = tfcb_p->sw.fsdu;
        if ((fsdu && tfcb_p->sw.frag_nr == fsdu->fTot-1) ||
             txStatus & TFCB_STATUS_FRAG_SKIP_ERR) {
                if ( txStatus & TFCB_STATUS_OK ) {
                        /* Increment the 802.11 MIB counter dot11TransmittedFrameCount
                         * since this MSDU was transmitted successfully.
                         */
                        priv_p->mib.dot11TransmittedFrameCount++;
                        priv_p->ap_statistics.tx_bytes += fsdu->sw.sdu->LLClength;
                        tx_done_sta_stat->dot11TransmittedFrameCount++;
                        if (fsdu->grpa) {
                                /* Increment the 802.11 MIB counter
                                 * dot11MulticastTransmittedFrameCount when the multicast bit
                                 * is set in the destination MAC address of this successfully
                                 * transmitted MSDU.
                                 */
                                priv_p->mib.dot11MulticastTransmittedFrameCount++;
                                tx_done_sta_stat->dot11MulticastTransmittedFrameCount++;
                        }
                }
                else {
                        /* If this MSDU was not transmitted successfully due to the
                         * number of transmit attempts exceeding the retry limit, we
                         * increment the 802.11 MIB counter dot11FailedCount.
                         */
                        priv_p->mib.dot11FailedCount++;
                        tx_done_sta_stat->dot11FailedCount++;
                }
        }

        KDBG_EXIT;
}

/*----------------------------------------------------------------*/
/*!
* \brief  A ISR supporting subroutine for \a isr_process_tx_done.
*
* This routine will clean up after a transmitted frame.
* It will free up TFCBs, TBBs, and related payload data buffer if
* it was malloc'ed before.
*
* \param  priv_p       pointer to i2220mgmt_t, the device's private structure
* \param  tfcb_p       pointer to the TFCB to be cleaned up
* \return none
*/
/*----------------------------------------------------------------*/
void isr_transmit_cleanup_TBB(i2220mgmt_t *priv_p, tfcb_t *tfcb_p, int remove)
{
        struct list_head        *lst;

        KDBG_ENTER;

        /* release all the TBB resource */
        list_for_each(lst,&tfcb_p->sw.tbb_list) {
                tbb_t   *tbb_p = list_entry( lst, tbb_t, sw.link );

                KDBG_LOG_TX(KDBG_DEBUG, "Releasing TBB 0x%08lx of TFCB 0x%08lx\n",
                       (unsigned long)tbb_p, (unsigned long)tfcb_p);

                if (tbb_p->do_map_unmap) {
                        kal_pci_unmap_single(priv_p->pcidev,
                                chip2host_32(tbb_p->hw.txDataBufferPointer),
                                chip2host_16(tbb_p->hw.txDataBufferLen),
                                PCI_DMA_TODEVICE);
                }

                if (remove) {
                        /* Remove the TBB from the active queue.
                        * !! be careful, the end-loop post assignment of lst=lst->next
                        *    can cause endless loop situration of this loop */
                        lst = lst->prev;
                        list_del( lst->next );

                        /* Put back the TFCB to the free chain list */
                        add_chain_tail(&priv_p->tbb_chain,tbb_t,tbb_p);
                }
        }

        KDBG_EXIT;
}

/*----------------------------------------------------------------*/
/*!
* \brief  A ISR supporting subroutine for \a isr_process_rx_done.
*
* Re-initializes the given SwRfb super-structure and
* chains it onto the current receive chain. The \a skb will be allocated when
* it has already been passed to upper layer and a new \a skb is required to
* be created to be associated with this \a rfb_p.
*
* \param  priv_p       pointer to i2220mgmt_t, the device's private structure
* \param  rfb_p        pointer to RFB
* \param  arrange_skb  whether to allocate a new sk_buff for this rfb_p
* \return none
*/
/*----------------------------------------------------------------*/
int rx_init_and_chain_packet(i2220mgmt_t *priv_p, rfb_t *rfb_p, bool arrange_skb)
{
        kal_skb_t               *skb;
        Frame_t                 *pdu;
        rfb_t                   *lastRfb_p;
        kal_dma_addr_t          dmaAddr;
        UINT8                   checksum;

        KDBG_ENTER;

        ASSERT(rfb_p);

        /* Now we can release the receive resources back to the NIC.
         * We are done with this RFB so re-initialize it.
         */
        rfb_p->hw.rxStatus = 0;
        rfb_p->hw.secStatus = host2chip_32(RFB_SECSTATUS_MASK);
        rfb_p->hw.rfbFlags = RFB_FLAG_EOL;

        if (arrange_skb) {
                /* Allocate the associated socket buffer, with space for the Receive Buffer
                 * of maximum possible data frame size. If no skb is available, we will
                 * recover all RFB buffer when rx empty interrupt occurs (one packet is lost).
                 */
		//jjunsigned long save_flags;
		//jjlocal_irq_save(save_flags);
                skb = kal_alloc_skb(sizeof(rxbuf_frame_t)); /* RX_SKB_BUF_SZ */
                rfb_p->sw.rfb_skb = skb;
//jj		local_irq_restore(save_flags);
                if (skb == NULL) {
                        printk(/*KDBG_LOG_RX(KDBG_WARN,*/ " Memory squeeze, dropping packet\n");
                        //priv_p->linux_stats->rx_dropped++;
                        goto L_chain;
                }
                /* The starting address should be 4-wise boundary */
                ASSERT((((UINT32)kal_skb_data(skb))&3)==0);
        }
        else {
                ASSERT(rfb_p->sw.rfb_skb);
                skb = rfb_p->sw.rfb_skb;

                /* Empty skb. Since skb_put() is called only in wlanhdr_to_ether(),
                 * we do not need to restore skb's ponter and length now.
                 */

                /* skb->tail = kal_skb_data(skb);
                 * kal_skb_len(skb) = 0;
                 */
        }

        /* Initialize Frame_t pdu in skb->data */
        pdu = (Frame_t *)kal_skb_data(skb);
        pdu->skb = skb;
        pdu->mac_ctl = MACCTL_SDU_FRAME_IN_SKB;

        /* Synchronize the data between CPU's D-CACHE and system Memory */
        dmaAddr = kal_pci_map_single(priv_p->pcidev, tcast_wlan_header(pdu),
                        sizeof(rxbuf_frame_t) - wlanheader_offset,
                        PCI_DMA_FROMDEVICE);

        /* Store the buffer physical address in the HW RFB Structure */
        rfb_p->hw.rxBufferPointer = host2chip_32(dmaAddr);

    #if WLAN_INCLUDE_DEBUG
        skb_clear_Flag(skb);
    #endif

        /* Append the RFB to the end of the NIC's RFB chain.  Normally, this
         * will always be done.  The only exception would be if the driver
         * was configured to use only 1 RFB (a degenerate case anyway).
         */
        if ( !list_empty(&priv_p->rfb_chain.head) ){
                peek_chain_tail(&priv_p->rfb_chain, rfb_t, lastRfb_p);
                ASSERT(lastRfb_p);

                /* Link it onto the end of the chain dynamically */
                lastRfb_p->hw.nextRfbPointer =
                                host2chip_32(rfb_p->dmaAddr);
                /* add checksum calculation */
                lastRfb_p->hw.rfdChecksum = 0;
                nicCalculateChecksumByte((PVOID)lastRfb_p, RFB_RFD_SIZE,
                                &checksum);
                lastRfb_p->hw.rfdChecksum = ~checksum;

                /* clear EOL (rfbFlags -= RFB_FLAG_EOL) */
                lastRfb_p->hw.rfdChecksum += RFB_FLAG_EOL;
                lastRfb_p->hw.rfbFlags &= ~RFB_FLAG_EOL;
        }

L_chain:
        /* The this RFB is done, so put it back on the tail of our list */
        add_chain_tail(&priv_p->rfb_chain, rfb_t, rfb_p);

        priv_p->hwStatistics.freeRfbCount++;

        KDBG_EXIT;
        return 0;
}   /* rx_init_and_chain_packet */


/*----------------------------------------------------------------*/
/*!
* \brief  A ISR supporting subroutine for \a isr_process_rx_done.
*
* Initializes the \a Frame_t associated structure for the specified
* \a rfb_p. Properties such as \a rx.time_stamp and \a rx.rssi, etc, are
* gotten directly or indirectly from \a rfb_p information.
* Multi-cast is also filtered here.
*
* \param  priv_p       pointer to i2220mgmt_t, the device's private structure
* \param  rfb_p        pointer to RFB
* \return TRUE if the packet is directed to self, FALSE otherwise.
*/
/*----------------------------------------------------------------*/
int rx_initial_packet(i2220mgmt_t *priv_p, rfb_t *rfb_p)
{
        kal_skb_t       *skb = rfb_p->sw.rfb_skb;
        Frame_t         *pdu = (Frame_t *)kal_skb_data(skb);
        UINT32          secMode;

        /* Update statistics counts */
        priv_p->hwStatistics.rxRfbRxOkCounter++; /* For debug use */
        mib_import(dot11ReceivedFragmentCount)++;
        if (sdl_isGroup(pdu->addr1)) {
                mib_import(dot11MulticastReceivedFrameCount)++;
        }

        /* Check decryption mode to decide rx packet length */
        secMode = rfb_p->hw.secStatus & RFB_SECSTATUS_DATA_PRIVACY_MASK;
        if (secMode != 0) { /* Decrypted data */
                if ((rfb_p->hw.secStatus & RFB_SECSTATUS_RXDEC_ERR_MASK) &&
                    sdl_ftype(pdu) != SYN_auth) {

                        /* If key is not found, we deauth it to restart */
                        if ((rfb_p->hw.secStatus & RFB_SECSTATUS_RXKEY_NOT_FOUND_ERR)
                            && !sdl_isWDS(pdu)) {
                                sdlf_send_deauth(priv_p, pdu->addr2, SYN_class3_err);
                        }
                        priv_p->mib.dot11WEPICVErrorCount++;
                        KDBG_LOG_RX(KDBG_DEBUG,"Discard rx error decrypted frame from "
                                MACSTR " secStatus=0x%08lx\n", MAC2STR(pdu->addr2),
                                rfb_p->hw.secStatus);
                        return 1;
                }
                if (!RxMpduDecrypt( priv_p, rfb_p, secMode )) {
                        KDBG_LOG_RX(KDBG_WARN, "Receive undecapsulate data\n");
                        return 1;
                }
        }

#if SUPPORT_PERF_MEASURE
        rx_counter ++;
        if (rx_counter % 3000 == 0)
        {
                UINT32  t1;
                if (secMode == CIPHER_SUITE_TKIP) {
                        NIC_MCR_READ(priv_p, MCR_LTTR0, &t1);
                        skb_set_Flag(skb,SKB_TSTAMP_IN_USE);
                        skb_set_TimeStamp(skb,t1);
                }
        }
#endif

        priv_p->ap_statistics.rx_bytes += rfb_p->hw.rxByteCount;
        /* Setup up the _pdu_ information from _skb_ and _rfb_ */
        if (sdl_frDs(pdu) && sdl_toDs(pdu)) {
                /* the Endian-ship was already converted */
                if (rfb_p->hw.rxByteCount <= 30 + sCrcLng) {
                        pdu->LLClength = 0;
                }
                else {
                        pdu->LLClength = rfb_p->hw.rxByteCount
                                                - 30 - sCrcLng;
                }
        #if RX_HEAD_SIZE_WORKAROUND
                /* Word around for header size is not 4-multiple
                 * Here we don't care management and control frames.
                 */
                if (sdl_dataQos(pdu)) {
                        pdu->LLClength = (pdu->LLClength > 2) ?
                                        pdu->LLClength - 2 : 0;
                }
                else {
                        /* Higher two bytes in last DWORD are our need */
                        ipn_memcpy(&pdu->addr4[4], &pdu->qosCtrl, 2);
                }
        #endif
                pdu->LLCdata = (UINT8 *)pdu + wlanbody_offset;
                pdu->mac_ctl |= MACCTL_WDS_FRAME;
        }
        else {
                /* The Endian-ship was already converted */
                if (rfb_p->hw.rxByteCount <= 24 + sCrcLng) {
                        pdu->LLClength = 0;
                }
                else {
                        pdu->LLClength = rfb_p->hw.rxByteCount
                                                - 24 - sCrcLng;
                }
        #if RX_HEAD_SIZE_WORKAROUND
                /* Word around for header size is not 4-multiple
                 * Here we don't care management and control frames.
                 */
                if (sdl_dataQos(pdu)) {
                        /* Higher two bytes in last DWORD are our need */
                        ipn_memcpy(&pdu->addr4[0], &pdu->addr4[2], 2);
                        pdu->LLClength = (pdu->LLClength > 2) ?
                                        pdu->LLClength - 2 : 0;
                }
        #endif
                pdu->LLCdata = (UINT8 *)pdu + wlanbody_offset;
        }

        return 0;
}

/*----------------------------------------------------------------*/
/*!
* \brief  An deferred Interrupt Service Routine for TDMA interrupt of
*         RDSR_RDMA_FRAME_RX_DONE or RDSR_RDMA_BUFFER_EMPTY
*
* This routine will indicate any received packets to the
* NDIS wrapper.  This routine is architected so that we
* will indicate all of the completed receive frames that
* we have in our buffer, before we exit the routine.  If
* we receive any errored frames, those frames are dropped
* by the driver, and not indicated to the ndis wrapper.
*
* \param  priv_p       pointer to i2220mgmt_t, the device's private structure
* \return none
*/
/*----------------------------------------------------------------*/
KAL_IRAM void isr_process_rx_done(i2220mgmt_t *priv_p)
{
        int             i;
        rfb_t           *rfb_p;
        kal_skb_t       *skb;

        KDBG_ENTER;
//printk("Sam i2220hw.c 2143 isr_process_rx_done\n");
        /* x_NumRFB just to limit loop count because the processed RFB
         * is re-appended to chain's tail as new EOL RFB. We can set
         * loop limit to any constant.
         */
        for (i = 0; i < x_NumRFB; i++) {
                /* Get the next unprocessed RFB */
                peek_chain_head(&priv_p->rfb_chain, rfb_t, rfb_p);
                ASSERT(rfb_p);

                /* If the RFB does not have its complete bit set, then bail out. */
                /* In IPN2220, the last filled field is secStatus, not rxStatus */
                if (chip2host_32(rfb_p->hw.secStatus) & RFB_SECSTATUS_MASK)
                        break;

        #if __CPU_ENDIAN__ != __LITTLE_ENDIAN__
                /* converting Endian of frequent use fields */
                rfb_p->hw.rxByteCount = chip2host_32(rfb_p->hw.rxByteCount);
                rfb_p->hw.rxStatus    = chip2host_32(rfb_p->hw.rxStatus);
                rfb_p->hw.secStatus   = chip2host_32(rfb_p->hw.secStatus);
                rfb_p->hw.iv          = chip2host_32(rfb_p->hw.iv);
                rfb_p->hw.extendedIv  = chip2host_32(rfb_p->hw.extendedIv);
        #endif

                skb = rfb_p->sw.rfb_skb;
                kal_pci_unmap_single(priv_p->pcidev,
                                chip2host_32(rfb_p->hw.rxBufferPointer),
                                sizeof(rxbuf_frame_t) - wlanheader_offset,
                                PCI_DMA_FROMDEVICE);

                /* This packet is completed and thus can be removed from the queue */
                del_chain_member(&priv_p->rfb_chain, rfb_t, rfb_p);

                //priv_p->hwStatistics.freeRfbCount--;
                //ASSERT(priv_p->hwStatistics.freeRfbCount <= x_NumRFB);

        #if SUPPORT_LOOPBACK
                if(priv_p->loop.enable && priv_p->loop.times){
                        int i =0;
                        unsigned char* cp = kal_skb_data(rfb_p->sw.rfb_skb) + wlanbody_offset;

                        for(i=0;i<priv_p->loop.length;i++){
                                if(cp[i] != priv_p->loop.value){
                                        LOOP_PRINT("Error on %d! 0x%02x 0x%02x len:%d\n",i,priv_p->loop.value,cp[i],priv_p->loop.length);
                                }
                        }

                        rx_init_and_chain_packet(priv_p, rfb_p, FALSE);

                        priv_p->loop.value ++;
                        priv_p->loop.times -- ;

                        if(priv_p->loop.times > 0)
                                sdlu_send_looppacket(priv_p, priv_p->loop.length, priv_p->loop.value);

                        continue;
                }
        #endif

                /* If the length of this frame is invalid, discard it. */
                if (rfb_p->hw.rxByteCount > MAX_RX_PKT_LEN ||
                    rfb_p->hw.rxByteCount == 0) {
                #if WLAN_INCLUDE_DEBUG
                        int             tmp;

                        tmp = g_kdbg_gate_2;
                        g_kdbg_gate_2 = 0xF;

                        KDBG_LOG_RX(KDBG_WARN, "Too Lengthy or Zero!!! (status=0x%08lx, len=%ld)\n",
                                rfb_p->hw.rxStatus, rfb_p->hw.rxByteCount);
                        dbg_rx_packet(priv_p,rfb_p,DBFLG_RX_RFB | DBFLG_RX_HEADER);

                        g_kdbg_gate_2 = tmp;
                #endif
                        rx_init_and_chain_packet(priv_p, rfb_p, FALSE);
                        continue;
                }

                /* Check the Status of this RFB.  Don't indicate if the frame
                   wasn't good. */
                if (!(rfb_p->hw.rxStatus & RFB_STATUS_OK)) {
                        KDBG_LOG_RX(KDBG_WARN, "Receive error!!! (status=0x%08lx)\n",
                                rfb_p->hw.rxStatus);
                        rx_init_and_chain_packet(priv_p, rfb_p, FALSE);
                        continue;
                }

                /* update the info of Frame_t in skb */
                if (rx_initial_packet(priv_p, rfb_p) != 0) {
                        rx_init_and_chain_packet(priv_p, rfb_p, FALSE);
                        continue;
                }

                dbg_rx_packet(priv_p,rfb_p,DBFLG_RX_RFB | DBFLG_RX_HEADER);
                dbg_frame_header((Frame_t *)kal_skb_data(skb));


        #if WLAN_INCLUDE_DEBUG
                //skb_set_Flag(skb,SKB_TSTAMP_IN_USE);
                //skb_set_TimeStamp(skb,t1);
        #endif

                /* now indicate to upper layer of the just received packet */
                switch (env_PHY_SAP_RX(priv_p, rfb_p)) {
                  case RX_INDICATE_PACKET_UPWARD :
                        /* Packet has already indicated upward to upper layer
                         * require to allocate a new SKB for this TBB
                         */
                        KDBG_LOG_RX(KDBG_DEBUG, "skb upward\n");
                        rx_init_and_chain_packet(priv_p, rfb_p, TRUE);
                        break;

                  case RX_DRV_HANDLED_PACKET :
                        /* Packet is handled soly by driver itself, perhaps
                         * being dropped, ignored, or internally processed.
                         * The associated SKB needn't be re-allocated again.
                         */
                        KDBG_LOG_RX(KDBG_DEBUG, "skb reused\n");
                        rx_init_and_chain_packet(priv_p, rfb_p, FALSE);
                        break;
                }
        }

#if WLAN_INCLUDE_DEBUG
        if (i > 8) {
                KDBG_LOG_RX(KDBG_DEBUG, "%d RFBs used.\n", i);
        }
#endif
        KDBG_EXIT;
} /* isr_process_rx_done */

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param  priv_p       pointer to i2220mgmt_t, the device's private structure
* \return none
*/
/*----------------------------------------------------------------*/
void isr_process_rx_empty(i2220mgmt_t *priv_p)
{
        int     i;
        rfb_t   *rfb_p;

        KDBG_ENTER;

        priv_p->hwStatistics.freeRfbCount = 0;
        for (i = 0; i < x_NumRFB; i++) {
                get_chain_head(&priv_p->rfb_chain, rfb_t, rfb_p);
                if (rfb_p->sw.rfb_skb) {
                        kal_pci_unmap_single(priv_p->pcidev,
                                chip2host_32(rfb_p->hw.rxBufferPointer),
                                sizeof(rxbuf_frame_t) - wlanheader_offset,
                                PCI_DMA_FROMDEVICE);

                        rx_init_and_chain_packet(priv_p, rfb_p, FALSE);
                }
                else {
                        rx_init_and_chain_packet(priv_p, rfb_p, TRUE);
                }
        }

        /* Check received buffer is available before restart rxQ */
        peek_chain_head(&priv_p->rfb_chain, rfb_t, rfb_p);
        if (!(rfb_p->hw.rfbFlags & RFB_FLAG_EOL)) {
                priv_p->macResetRxEmpty++; /* debug */
                i2220hw_reset_mac(priv_p);
                priv_p->rxQ_empty = FALSE;
        }
        else {
                priv_p->rxQ_empty = TRUE;
        }

        KDBG_EXIT;
} /* isr_process_rx_empty */

/*----------------------------------------------------------------*/
/*!
* \brief  The core of deferred Interrupt Service Routine for every kind of
*         interrupts interested to this device. It is also called bottom half
*         in Linux.
*
* Depends on ISR (Interrupt Status Register) and TDMA/RDMA status registers,
* different routines are further invoked to handle the required interrupted
* service.
*
* \param  obj  an cookie by Linux ISR dispatching routine, to get our
*              pointer to i2220mgmt_t, the device's private structure
* \return none
*/
/*----------------------------------------------------------------*/
void do_isr_tasklet(void *dev_priv)
{
        i2220mgmt_t     *priv_p = (i2220mgmt_t*)dev_priv;
        UINT32          interruptStatus;
        UINT32          qStatus;
        int             int_work=0;
#if WLAN_INCLUDE_DEBUG
        UINT32          temp32, tmp32;
#endif

        /* Get ISR from i2220hw_interrupt function */
        interruptStatus = priv_p->interruptStatus;
        priv_p->interruptStatus = 0;

        do {
                /* Acknowledge all pending interrupts */
                NIC_MCR_WRITE(priv_p, MCR_ISAR, interruptStatus);

        #if WLAN_INCLUDE_DEBUG
                /* Abnormal handle */
                if (interruptStatus & ISAR_ABNORMAL_INT) {
                        NIC_MCR_READ(priv_p, MCR_QSR0, &temp32);
                        KDBG_LOG_INT(KDBG_WARN,"Abnormal INT happens: QSR0 = 0x%08lx\n", temp32);

                        temp32 &= (QSR0_RFD_CHECK_SUM_ERR | QSR0_TFD_CHECK_SUM_ERR | QSR0_TBD_CHECK_SUM_ERR);
                        NIC_MCR_WRITE(priv_p, MCR_QSR0, temp32);

                        if (temp32 & QSR0_RFD_CHECK_SUM_ERR) {
                                NIC_MCR_READ(priv_p, MCR_RQCPR, &tmp32);
                                KDBG_LOG_INT(KDBG_WARN, "RQ CPR: 0x%08lx\n", tmp32);

                                NIC_MCR_READ(priv_p, MCR_RQSAR, &tmp32);
                                KDBG_LOG_INT(KDBG_WARN, "RQ SAR: 0x%08lx\n", tmp32);

                                NIC_MCR_READ(priv_p, MCR_RFECR, &tmp32);
                                mib_import(dot11FCSErrorCount) += tmp32;
                                KDBG_LOG_INT(KDBG_WARN, "RX CRC error count: %ld\n", tmp32);

                                NIC_MCR_READ(priv_p, MCR_RFFCR, &tmp32);
                                priv_p->hwStatistics.RcvFifoFulls += tmp32;
                                KDBG_LOG_INT(KDBG_WARN, "RX FIFO Full count: %ld\n", tmp32);

                                /* Here log all RFD */
                        }

                        if (temp32 & (QSR0_TFD_CHECK_SUM_ERR | QSR0_TBD_CHECK_SUM_ERR)) {
                                NIC_MCR_READ(priv_p, MCR_AC0CPR, &tmp32);
                                KDBG_LOG_INT(KDBG_WARN, "AC0 CPR: 0x%08lx\n", tmp32);
                                NIC_MCR_READ(priv_p, MCR_AC0SAR, &tmp32);
                                KDBG_LOG_INT(KDBG_WARN, "AC0 SAR: 0x%08lx\n", tmp32);
                        }

                        if (temp32 == 0) {
                                KDBG_LOG_INT(KDBG_WARN, "Abnormal false alarm!\n");
                        }
                }

                /* Rx FIFO overflow indication */
                if (interruptStatus & ISAR_RFIFO_OVERFLOW_INT) {
                        NIC_MCR_READ(priv_p, MCR_RFFCR, &temp32);
                        priv_p->hwStatistics.RcvFifoFulls += temp32;
                        KDBG_LOG_RX(KDBG_WARN, "RxFIFO overflow, RFFCR=%ld\n", temp32);
                        NIC_MCR_READ(priv_p, MCR_QSR0, &temp32);
                        KDBG_LOG_RX(KDBG_WARN, "MCR_QSR0=0x%08lx\n", temp32);
                        NIC_MCR_READ(priv_p, MCR_QSR1, &temp32);
                        KDBG_LOG_RX(KDBG_WARN, "MCR_QSR1=0x%08lx\n", temp32);

                        NIC_MCR_READ(priv_p, MCR_PRBCR, &temp32);
                        tmp32 = temp32; /* original debug port */
                        NIC_MCR_WRITE(priv_p, MCR_PRBCR, 0xA01);
                        NIC_MCR_READ(priv_p, MCR_PRBSR, &temp32);
                        NIC_MCR_WRITE(priv_p, MCR_PRBCR, tmp32); /* resotre debug port */
                        KDBG_LOG_RX(KDBG_WARN, "Debug Port(0xA01)=0x%08lx\n", temp32);
                }

                if (interruptStatus & ISAR_TARGET_HANG_INT) {
                        KDBG_LOG_INT(KDBG_CRIT,"\nTarget Hang!!!\n\n");
                }
        #else
                if (interruptStatus & ISAR_ABNORMAL_INT) {
			printk("abnormal\n");
                        NIC_MCR_READ(priv_p, MCR_QSR0, &qStatus);
                        qStatus &= (QSR0_RFD_CHECK_SUM_ERR
                                        | QSR0_TFD_CHECK_SUM_ERR
                                        | QSR0_TBD_CHECK_SUM_ERR);
                        NIC_MCR_WRITE(priv_p, MCR_QSR0, qStatus);

                        priv_p->macResetAbnormalCount++; /* debug */
                        i2220hw_reset_mac(priv_p); /* reset */

                        /* Don't process tx/rx events after MAC reset */
                        interruptStatus &= ISAR_PRE_TBTT_INT;
                }

        #endif /* WLAN_INCLUDE_DEBUG */

                /* Go handle pre-TBTT event */
                if (interruptStatus & ISAR_PRE_TBTT_INT) {
                        isr_process_preTBTT(priv_p);
                }

                /* Go handle receive events */
                if (interruptStatus & ISAR_RXQ_EVENT) {
                #if WLAN_INCLUDE_DEBUG
                        if (interruptStatus & ISAR_RX_DATA_Q_EMPTY_INT) {
                                KDBG_LOG_RX(KDBG_WARN, "RX Q Empty. RFB: free=%d\n",
                                        priv_p->hwStatistics.freeRfbCount);
                                dbg_rx_rdma(priv_p, 2);
                        }
                #endif
                        /* Process RDMA frame receive done and buffer empty interrupts */
                        if (interruptStatus & ISAR_RX_DATA_Q_DONE_INT) {
                                isr_process_rx_done(priv_p);
                        }

                        if (interruptStatus & ISAR_RX_DATA_Q_EMPTY_INT) {
                                isr_process_rx_empty(priv_p);
                        }
                }

                /* Go handle transmit events */
                if (interruptStatus & ISAR_TXQ_EVENT) {
                        /* Read Q status */
                        NIC_MCR_READ(priv_p, MCR_QSR1, &qStatus);
                        NIC_MCR_WRITE(priv_p, MCR_QSR1, qStatus);

                #if WLAN_INCLUDE_DEBUG
                        queue_TQ_interrupt_process(0);
                        queue_TQ_interrupt_process(1);

                        queue_AC_interrupt_process(0);
                        queue_AC_interrupt_process(1);
                        queue_AC_interrupt_process(2);
                        queue_AC_interrupt_process(3);

                        queue_TS_interrupt_process(0);
                        queue_TS_interrupt_process(1);
                        queue_TS_interrupt_process(2);
                        queue_TS_interrupt_process(3);
                #else
                        queue_TQ_interrupt_process(1);
                        queue_AC_interrupt_process(0);
                #endif

                } /* end of Go handle transmit events */

        #if SUPPORT_TKIP
                /* Go handle Security Queue events */
                if (interruptStatus & ISAR_SECQ_EVENT) {
                        /* Process secQ done interrupt */
                        KDBG_LOG_TX(KDBG_DEBUG, "ISAR_SEC_Q_INT\n");
                        NIC_MCR_READ(priv_p, MCR_QSR0, &qStatus);
                        NIC_MCR_WRITE(priv_p, MCR_QSR0, qStatus & BITS(0,3));

                #if !SINGLE_SEC_QUEUE
                        if (qStatus & QSR0_SEC_ENGINE_OP_RX_EVENT) {
                                isr_process_RxCalcMic_done(priv_p);
                        }
                #endif
                        if (qStatus & QSR0_SEC_ENGINE_OP_TX_EVENT) {
                                isr_process_TxCalcMic_done(priv_p);
                        }
                }/* End of Go handle Security Queue events */
        #endif

                /* Read Interrupt Status Register and strip off
                 * the disabled interrupts.
                 */
                NIC_MCR_READ(priv_p, MCR_ISAR, &interruptStatus);
                interruptStatus &= priv_p->interruptEnable;

        } while (interruptStatus && int_work++ < 10);

        /* Check new skb when rx queue empty (no skb for rxQ) */
        if (priv_p->rxQ_empty &&
             (priv_p->interruptEnable & ISAR_RX_DATA_Q_EMPTY_INT)) {
                isr_process_rx_empty(priv_p);
        }

        nicEnableInterrupt(priv_p);
}

/*----------------------------------------------------------------*/
/*!
* \brief  Prepare a TFCB for the transmission of a wlan frame.
*
* This routine will setup all of the necessary structures
* (TFCBs, TBBs, etc.) to send a frame.  If the device has
* the necessary resources, the frame will be sent out on
* the active chain.  If the device doesn't have the
* resources to send the frame, then the routine will return
* differently.
*
* \param  priv_p       pointer to i2220mgmt_t, the device's private structure
* \param  tdmaChannel  The number of the TDMA channel on which we will send out
*                      the frame
* \param  data         NULL terminated list of buffer to be sent out
* \param  len          0 terminated list of lengths, NEGATIVE means to be released
*                      by lower layer
* \param  tfcb_pp      the acquired TFCB, returned back to the caller, to setup
*                      the association linkage with the frame structure.
* \return TX_TFCB_SUCCESS_MORE_BUFF  successful, and more buffer after this operation \n
*         TX_TFCB_SUCCESS_NO_BUFF  successful, but no more buffer \n
*         TX_TFCB_FAILURE  failure, the current operation is aborted
*/
/*----------------------------------------------------------------*/
int tx_prepare_TBB(i2220mgmt_t *priv_p,
                UINT8 *data[], int len[], tfcb_t *tfcb_p)
{
        struct list_head        *lst;
        tbb_t                   *tbb_p;
        UINT32                  *tbb_link;
        kal_dma_addr_t          dmaAddr;
        UINT8                   checksum;
        int                     tbb_len = 0;
        //int                     i;

        KDBG_ENTER;
#if 0
        /* Count the TBB buffer availability and compare with the request buffer counts */
        for (i=0; len[i]; i++);
        if (chain_num_elem(&priv_p->tbb_chain) < i) {
                KDBG_LOG_TX(KDBG_WARN,"Insufficient TBB buffers, request %d, remaining %d\n"
                             , i, chain_num_elem(&priv_p->tbb_chain));
                return TX_TFCB_FAILURE;
        }
#endif
        /* Start to prepare the TBB array of the TFCB for transmission
         * of this fragment.
         */
        tbb_link = (UINT32*)&tfcb_p->hw.tbbPointer;
        while (*len) {

                ASSERT(*data!=NULL);

                /* Attempt to acquire a TBB for the packet sub blocks */
                get_chain_head(&priv_p->tbb_chain, tbb_t, tbb_p);
                ASSERT(tbb_p);
                KDBG_LOG_TX(KDBG_DEBUG,"Setting TBB at virtual addr 0x%08lx\n"
                                , (unsigned long)tbb_p);

                /* Add the TBB to the active chain list */
                list_add_tail(&tbb_p->sw.link, &tfcb_p->sw.tbb_list);

                if (*len < 0) {
                        *len = -(*len);
                        dmaAddr = (kal_dma_addr_t)(*data);
                        tbb_p->do_map_unmap = FALSE;
                }
                else {
                        /* Synchronize data between D-CACHE and system Memory */
                        dmaAddr = kal_pci_map_single(priv_p->pcidev, *data, *len,
                                                        PCI_DMA_TODEVICE);
                        tbb_p->do_map_unmap = TRUE;
                }
                tbb_p->hw.txDataBufferPointer = host2chip_32(dmaAddr);
                tbb_p->hw.txDataBufferLen = host2chip_16(*len);
                tbb_len += *len;

                /* for the very 1st TBB, <tbb_link> will be linked from TFCB
                 * for all subsequent TBBs, it is the address of
                 * previous TBB's nextTbbPointer
                 */
                *tbb_link = host2chip_32(tbb_p->dmaAddr);
                KDBG_LOG_TX(KDBG_DEBUG,"Linking TBB addr *(0x%08lx) = 0x%08lx\n"
                                , (unsigned long)tbb_link, (unsigned long)tbb_p);

                dbg_tx_packet(priv_p,*data,DBFLG_TX_HEADER | *len);

                /* proceed to next packet fragment */
                data++; len++;
                if (*len) {
                        /* still require more TBBs */
                        tbb_p->hw.tbbFlags = 0;
                }
                else {
                        /* this is the last, set the END-LIST mark */
                        tbb_p->hw.tbbFlags = TBB_FLAG_EOL;
                }

                tbb_link = (UINT32*)&tbb_p->hw.nextTbbPointer;
        } /* end of while(*data) loop */

        /* 2nd round loop of all TBBs for the calculation of the checksum */
        list_for_each(lst,&tfcb_p->sw.tbb_list) {
                tbb_p = list_entry( lst, tbb_t, sw.link );

                /* calculate the checksum of the last TBB */
                tbb_p->hw.tbdChecksum = 0;
                nicCalculateChecksumByte((PVOID) tbb_p,
                        sizeof(hw_tbb_t) - TBB_REVBUFFER_SIZE, &checksum);
                tbb_p->hw.tbdChecksum = ~checksum;

                dbg_tx_packet(priv_p,tbb_p,DBFLG_TX_TBB);
        }

        KDBG_LOG_TX(KDBG_DEBUG,"send TBB(body) %d bytes\n", tbb_len);

        KDBG_EXIT;
        return 0;
}

/*----------------------------------------------------------------*/
/*!
* \brief  Further setup of the TFCB protocol control information.
*
* This routine will setup all of the necessary transmit
* control information in the TFCB associated with a
* transmitted frame.
*
* \param  priv_p       pointer to i2220mgmt_t, the device's private structure
* \param  tfcb_p       pointer to the current TFCB to be transmitted
* \param  prev_tfcb_p  pointer to the previous TFCB, for the calculation of
*                      durationID for successive fragments.
* \return none
*/
/*----------------------------------------------------------------*/
KAL_IRAM void tx_setup_TFCB_ctrl_info(i2220mgmt_t *priv_p, tfcb_t *tfcb_p,
                                tfcb_t *prev_tfcb_p)
{
        hw_tfcb_t       *hwTfcb_p = &tfcb_p->hw;
        FragSdu_t       *fsdu = tfcb_p->sw.fsdu;
        Frame_t         *sdu = fsdu->sw.sdu;
        FragMpdu_t      *tpdu = fsdu_pdu(fsdu, tfcb_p->sw.frag_nr);
        sta_info_t      *sta_p = fsdu->sw.sta_p;
        txRate_info_t   *txRate_p;
        UINT16          ctsAckDuration, ppduDuration;
        UINT32          txRateCode;
        UINT8           txRate;
        bool            useProtection = FALSE;

        /* Not fill every field from scratch to reduce process time */
        *(UINT32*)&hwTfcb_p->sciCtrlFlag = 0;
        *(UINT32*)&hwTfcb_p->tciCtrlFlag = 0;

        /*-----------------------------------------------------
         * MPDU header (0x48--0x6c) and security (0x10--0x28)
         * ==> including length fields
         * Note: for big endian, some short and long fields
         *       are swapped at the end
         *----------------------------------------------------*/
        hwTfcb_p->totalTxDataLen = tpdu->LLClength;
        hwTfcb_p->macHeaderLen = sMacHdrLng;
        if (sdu->mac_ctl & MACCTL_WDS_FRAME) {
                hwTfcb_p->macHeaderLen += MAC_ADDR_LEN;
                hwTfcb_p->sciCtrlFlag |= BIT(0); /* Addr4 */
        }
        if (sdu->mac_ctl & MACCTL_QOS_FRAME) {
                hwTfcb_p->macHeaderLen += 2;
                hwTfcb_p->sciCtrlFlag |= BIT(1); /* QoS */
        }
        hwTfcb_p->dmaFifoHeaderLen = 17; /* TCI dword */
        hwTfcb_p->mpduLen = hwTfcb_p->macHeaderLen +
                            hwTfcb_p->totalTxDataLen + sCrcLng;

        ipn_memcpy(&hwTfcb_p->frameCtrl, &sdu->frameCtrl, hwTfcb_p->macHeaderLen);
        *(UINT16*)&hwTfcb_p->macHeader[18] = host2chip_16
                (priv_p->seqnum | (tfcb_p->sw.frag_nr & 0xF));

        if (sdl_wepBit(sdu)) {
                TxMpduEncrypt(priv_p, hwTfcb_p, fsdu);
        }

        /*------------------------------------------------
         * setup tx rate and duration fields
         * (decide tx rate, ack rate and protection)
         *-----------------------------------------------*/
        if (priv_p->fixedTxRate_p) {
                /* fixed tx rate for management and data frames */
                txRate_p = priv_p->fixedTxRate_p;
        }
        else if (sta_p && sta_p->curTxRate_p) {
                /* unicast frame, with record */
                txRate_p = sta_p->curTxRate_p;
        }
        else {
                /* braodcast frame or unicast frame without record */
                txRate_p = priv_p->lowestBRate_p;
        }
        /* judge if this fsdu needs nonERP CTS protection */
        if (txRate_p->isERP && priv_p->protectMode != CTS_PROTECT_NONE) {
                if (priv_p->sta_nonERP || priv_p->bss_nonERP ||
                    priv_p->protectMode == CTS_PROTECT_MUST) {
                        if (fsdu->fTot > 1) {
                                /* use NonERP tx rate for fragment */
                                txRate_p = priv_p->protectRate_p;
                        }
                        else {
                                /* include broadcast frame */
                                useProtection = TRUE;
                        }
                }
        }
        txRate = txRate_p->txRate;
        txRateCode = txRate_p->txRateCode;
        ctsAckDuration = txRate_p->rspDuration;

        /* set tx power based on modulation mode */
        hwTfcb_p->txPower = txRate_p->isERP ?
                priv_p->txPowerGainForOFDM : priv_p->txPowerGainForCCK;

        if (fsdu->grpa) {
                /* braodcast frame */
                hwTfcb_p->tciCtrlFlag |= TFCB_CTRL_NO_ACK;
                hwTfcb_p->durationId1 = 0;
                hwTfcb_p->durationId2 = 0;
        }
        else {
                /* unicast frame */
                hwTfcb_p->durationId1 = SIFS_TIME_ERP + ctsAckDuration;
                hwTfcb_p->durationId2 = SIFS_TIME_ERP +
                                priv_p->lowestBRate_p->rspDuration;
        }

#if 0
        /* now only support one step tx rate */
        hwTfcb_p->multirateSwCtrl = host2chip_16(TFCB_MULTIRATE_1_STEP_SWITCH
                | (TFCB_MULTIRATE_MPDU_RETRY_LIMIT_1 &
                    (mib_import(dot11ShortRetryLimit) - 1)));
        hwTfcb_p->txRates = host2chip_16(txRateCode);
        rateComputePpduDuration(hwTfcb_p->mpduLen, txRate,
                        priv_p->phyPreambleType, &ppduDuration);
        hwTfcb_p->packetTxTime1 = ppduDuration;
        hwTfcb_p->minFrameExchangeTime1 = hwTfcb_p->packetTxTime1 +
                                          hwTfcb_p->durationId1;
#else
        /* Note: wrong duration may occur when RTS,CTS is applied */
        /* now only support two step tx rate */
        hwTfcb_p->multirateSwCtrl = host2chip_16(TFCB_MULTIRATE_2_STEP_SWITCH |
                BIT(7) | (TFCB_MULTIRATE_MPDU_RETRY_LIMIT_1 &
                         (mib_import(dot11ShortRetryLimit) - 2)));
        hwTfcb_p->txRates = host2chip_16
                ((priv_p->lowestBRate_p->txRateCode << 8) | txRateCode);

        rateComputePpduDuration(hwTfcb_p->mpduLen, txRate,
                        priv_p->phyPreambleType, &ppduDuration);
        hwTfcb_p->packetTxTime1 = ppduDuration;
        hwTfcb_p->minFrameExchangeTime1 = hwTfcb_p->packetTxTime1 +
                                          hwTfcb_p->durationId1;

        rateComputePpduDuration(hwTfcb_p->mpduLen, priv_p->lowestBRate_p->txRate,
                        priv_p->phyPreambleType, &ppduDuration);
        hwTfcb_p->packetTxTime2 = ppduDuration;
        hwTfcb_p->minFrameExchangeTime2 = hwTfcb_p->packetTxTime2 +
                                          hwTfcb_p->durationId2;
#endif

        /* RTS/CTS, each duration based on tx rate */
        if (useProtection) {
                /* NonERP protection tx rate */
                txRate = priv_p->protectRate_p->txRate;
                txRateCode = priv_p->protectRate_p->txRateCode;
                ctsAckDuration = priv_p->protectRate_p->rspDuration;
        }
        if (!fsdu->grpa && hwTfcb_p->mpduLen >= mib_import(dot11RTSThreshold)) {
                hwTfcb_p->rtsRetryLimit = mib_import(dot11LongRetryLimit) - 1;
                hwTfcb_p->rtsCtsDuration = 2*SIFS_TIME_ERP +
                        hwTfcb_p->minFrameExchangeTime1 +  ctsAckDuration;
                hwTfcb_p->rtsCtsRate = txRateCode;
                hwTfcb_p->tciCtrlFlag |= TFCB_CTRL_RTS;
        }
        else if (useProtection) {
                hwTfcb_p->rtsRetryLimit = 0;
                hwTfcb_p->rtsCtsDuration = SIFS_TIME_ERP +
                        hwTfcb_p->minFrameExchangeTime1;
                hwTfcb_p->rtsCtsRate = txRateCode;
                hwTfcb_p->tciCtrlFlag |= TFCB_CTRL_CTS;
        }

        /* Check if inserting time stamp */
        if (sdu->mac_ctl & MACCTL_NEED_TIMESTAMP) {
                hwTfcb_p->tciCtrlFlag |= TFCB_CTRL_TSF;
        }

        /* Choose tx antenna */
        if (priv_p->currentTxAntenna == 1) {
                hwTfcb_p->tidMacHdrCtrlFlag |= TFCB_TX_ANT;
        }

        /* Update previous TFCB for fragment */
        if (prev_tfcb_p) {
                hw_tfcb_t       *preHwTfcb_p = &prev_tfcb_p->hw;

                /* more frag */
                preHwTfcb_p->tfdCtrlFlag |= TFCB_FLAG_MORE;
                preHwTfcb_p->tciCtrlFlag |= TFCB_CTRL_MORE;
                preHwTfcb_p->frameCtrl |= host2ieee16(BIT(10));

                /* add extra duration of this new frame to prev_tfcb_p */
                preHwTfcb_p->durationId1 = host2chip_16(
                        SIFS_TIME_ERP + hwTfcb_p->minFrameExchangeTime1 +
                        chip2host_16(preHwTfcb_p->durationId1));
                preHwTfcb_p->rtsCtsDuration = host2chip_16(
                        SIFS_TIME_ERP + hwTfcb_p->minFrameExchangeTime1 +
                        chip2host_16(preHwTfcb_p->rtsCtsDuration));

                /* It is not the first fragment */
                hwTfcb_p->tidMacHdrCtrlFlag |= TFCB_FRAG;
        }

#if __CPU_ENDIAN__ != __LITTLE_ENDIAN__
        hwTfcb_p->totalTxDataLen = host2chip_16(hwTfcb_p->totalTxDataLen);
        hwTfcb_p->mpduLen = host2chip_16(hwTfcb_p->mpduLen);
        hwTfcb_p->durationId1 = host2chip_16(hwTfcb_p->durationId1);
        hwTfcb_p->durationId2 = host2chip_16(hwTfcb_p->durationId2);
        hwTfcb_p->packetTxTime1 = host2chip_16(hwTfcb_p->packetTxTime1);
        hwTfcb_p->packetTxTime2 = host2chip_16(hwTfcb_p->packetTxTime2);
        hwTfcb_p->rtsCtsDuration = host2chip_16(hwTfcb_p->rtsCtsDuration);
#endif

#if SUPPORT_LOOPBACK
        if(priv_p->loop.times){
                unsigned char check_sum;
                hwTfcb_p->reserved4 = 0;
                hwTfcb_p->durationId2 = hwTfcb_p->totalTxDataLen;
                nicCalculateChecksumByte(&hwTfcb_p->macHeaderLen,
                        4, &check_sum);
                hwTfcb_p->reserved4   = ~check_sum;
        }
#endif
        return;
}   /* tx_setup_TFCB_ctrl_info */

/*----------------------------------------------------------------*/
/*!
* \brief  Initialize the chains of TFCB and TBB lists.
*
* Setup the TFCB and TBB for each TDMA channel at INIT time.
*
* \param  priv_p       pointer to i2220mgmt_t, the device's private structure
* \return none
*/
/*----------------------------------------------------------------*/
void tx_setup_transmit_queues(i2220mgmt_t *priv_p)
{
        int               i;
        struct list_head  *lst;
        tfcb_t            *tfcb_p;
#if SUPPORT_TKIP
        UINT32            *tbb_link;
        tbb_t             *tbb_p;
        UINT8             checksum, j;
#endif

        KDBG_ENTER;

        /* Initialize TX Frame Buffer control objects. */
        ASSERT(x_NumTFCB>NUM_TDMA_CHANNELS-2);
        ASSERT(x_NumTBB>1);
        KDBG_LOG_INIT(KDBG_INFO, "%d TFCBs, size=%d\n",
                                x_NumTFCB,(int)((sizeof(tfcb_t)+3)&~3));
        KDBG_LOG_INIT(KDBG_INFO, "%d TBBs, size=%d\n",
                                x_NumTBB, (int)((sizeof(tbb_t)+3)&~3));

        create_chain_list( &priv_p->tfcb_chain, tfcb_t, priv_p->pcidev, x_NumTFCB);
        create_chain_list( &priv_p->tbb_chain,  tbb_t,  priv_p->pcidev, x_NumTBB );

        list_for_each(lst,&priv_p->tfcb_chain.head) {
                tfcb_p = list_entry( lst, tfcb_t, sw.link );
                INIT_LIST_HEAD(&tfcb_p->sw.tbb_list);
        }

#if SUPPORT_TKIP
        ASSERT(x_NumTBB>x_NumTFCB*4);
        create_chain_list( &priv_p->sec_tfcb_chain, tfcb_t, priv_p->pcidev, x_NumTFCB);

        list_for_each(lst,&priv_p->sec_tfcb_chain.head) {
                tfcb_p = list_entry( lst, tfcb_t, sw.link );
                INIT_LIST_HEAD(&tfcb_p->sw.tbb_list);

                /* Prepare fixed fields of HW TFCB */
                tfcb_p->hw.secOpMode = TFCB_SEC_MODE_TKIP_MIC;
                tfcb_p->hw.dmaFifoHeaderLen = 0x13;
                /* Returned bytes count: IV, EIV, MIC key */
                tfcb_p->hw.durationId2 = host2chip_16(16);
                tfcb_p->hw.macHeaderLen = 24;
                /* 0xff - tfcb_p->hw.durationId2 - tfcb_p->hw.macHeaderLen */
                tfcb_p->hw.reserved4 = 0xd7; /* checksum */

                tbb_link = (UINT32*)&tfcb_p->hw.tbbPointer;
                for (i = 0; i < TKIP_HW_MIC_TBB_NUM_PER_TFCB; i++) {

                        /* Attempt to acquire a TBB for the packet sub blocks */
                        get_chain_head(&priv_p->tbb_chain, tbb_t, tbb_p);
                        ASSERT(tbb_p);
                        KDBG_LOG_TX(KDBG_DEBUG,"TBB at virtual addr 0x%08lx\n",
                                (unsigned long)tbb_p);

                        /* Add the TBB to the active chain list of TFCB */
                        list_add_tail(&tbb_p->sw.link, &tfcb_p->sw.tbb_list);

                        /* for the very 1st TBB, <tbb_link> will be linked from TFCB
                        * for all subsequent TBBs, it is the address of
                        * previous TBB's nextTbbPointer
                        */
                        *tbb_link = host2chip_32(tbb_p->dmaAddr);
                        KDBG_LOG_TX(KDBG_DEBUG,"Linking TBB addr *(0x%08lx) = 0x%08lx\n"
                                , (unsigned long)tbb_link, (unsigned long)tbb_p);

                        /* Proceed to next packet fragment */
                        if (i != TKIP_HW_MIC_TBB_NUM_PER_TFCB - 1) {
                                /* Still require more TBBs */
                                tbb_p->hw.tbbFlags = 0;
                        }
                        else {
                                /* This is the last, set the END-LIST mark */
                                tbb_p->hw.tbbFlags = TBB_FLAG_EOL;

                                /* The 3rd TBB (padding of 0x5a and 0x00) */
                                tbb_p->hw.revBuf[0] = 0x5a;
                                for (j = 1; j < 8; j++) {
                                        tbb_p->hw.revBuf[j] = 0x00;
                                }
                        }

                        tbb_link = (UINT32*)&tbb_p->hw.nextTbbPointer;
                } /* end of for TBB loop */

                /* The first TBB has fixed values -> fixed checksum */
                tbb_p = list_entry( tfcb_p->sw.tbb_list.next, tbb_t, sw.link );
                tbb_p->hw.txDataBufferLen = host2chip_16(16);
                tbb_p->hw.txDataBufferPointer =
                        host2chip_32(tbb_p->dmaAddr + offsetof(hw_tbb_t, revBuf));
                tbb_p->do_map_unmap = FALSE;

                tbb_p->hw.tbdChecksum = 0;
                nicCalculateChecksumByte((PVOID) tbb_p, TBB_TBD_SIZE, &checksum);
                tbb_p->hw.tbdChecksum  = ~checksum;

        } /* end of list_for_each secTFCB loop */
#endif

        for (i = 0; i < NUM_TDMA_CHANNELS; i++) {
                tdma_parameters_t *tdma_p = &priv_p->tdma[i];

                ipn_memset(tdma_p, 0, sizeof(tdma_parameters_t));
                tdma_p->tdmaChannel = i;
                sdlQ_emptyQ(&tdma_p->waitQ);
        }

        KDBG_EXIT;
}

/*----------------------------------------------------------------*/
/*!
* \brief  Cleanup the chains of TFCB and TBB lists.
*
* Cleanup the TFCB and TBB for each TDMA channel at SHUTDOWN time.
*
* \param  priv_p       pointer to i2220mgmt_t, the device's private structure
* \return none
*/
/*----------------------------------------------------------------*/
void tx_cleanup_transmit_queues(i2220mgmt_t *priv_p)
{
        int               tdmaChannel;
        chain_list_t      *chain_p;
        tdma_parameters_t *tdma_p;
        tfcb_t            *tfcb_p;
        FragSdu_t         *fsdu;

        KDBG_ENTER;

        for (tdmaChannel = 0; tdmaChannel < NUM_TDMA_CHANNELS; tdmaChannel++) {
                tdma_p = &priv_p->tdma[tdmaChannel];

                /* Stop the TDMA and SecQ operation */
                if (tdmaChannel <= (int)TXQ_TS3) {
                        dmaStopTxQueue(priv_p, (TX_QUEUE_NUM)tdmaChannel, FALSE);
                        chain_p = &priv_p->tfcb_chain;
                }
                else {
                        dmaStopSecQueue(priv_p, (TX_QUEUE_NUM)tdmaChannel);
                        chain_p = &priv_p->sec_tfcb_chain;
                }

                /* Clean up all the packets we have put into the primary transmit queue */
                if (tdma_p->head_p) {
                        cleanup_TFCB_chained_list(chain_p, tdma_p->head_p, tfcb_p);

                        /* we should be at the end of TFCB list now */
                        if (tfcb_p->hw.tfdCtrlFlag & TFCB_FLAG_EOL) {
                                /* all TFCB sent out, Put back the end-list TFCB to the free chain list */
                                add_chain_tail(chain_p,tfcb_t,tfcb_p);
                        }
                        else {
                                KDBG_LOG_TX(KDBG_WARN, "TFCB chain list incorrect\n");
                        }
                }

                /* Clean up all buffered fsdu */
                while (sdlQ_Length(&tdma_p->waitQ) != 0) {
                        sdlQ_dequeue_first(&tdma_p->waitQ, fsdu);
                        sdlu_release_fsdu(fsdu);
                }
        }

#if SUPPORT_TKIP
        destroy_chain_list(&priv_p->sec_tfcb_chain, tfcb_t, priv_p->pcidev);
#endif
        destroy_chain_list(&priv_p->tfcb_chain, tfcb_t, priv_p->pcidev);
        destroy_chain_list(&priv_p->tbb_chain, tbb_t, priv_p->pcidev);

        KDBG_EXIT;
}


/*----------------------------------------------------------------*/
/*!
* \brief  Initialize the chains of RFB lists.
*
* Setup the Receive Frame Buffer (RFB) at INIT time. \a rx_init_and_chain_packet
* is invoked to chain all the allocated RFB together.
*
* \param  priv_p       pointer to i2220mgmt_t, the device's private structure
* \return none
*/
/*----------------------------------------------------------------*/
void rx_setup_receive_queues(i2220mgmt_t *priv_p)
{
        rfb_t   *rfb_p;
        int     i;
        KDBG_ENTER;

        /*----------------------------------------------
         * Initialize the logical and physical RFB lists
         *---------------------------------------------*/
        ASSERT(x_NumRFB>2);
        KDBG_LOG_INIT(KDBG_INFO, "%d RFBs, size=%d\n", x_NumRFB,(int)((sizeof(rfb_t)+3)&~3));

        /* Initialize the lists */
        create_chain_list(&priv_p->rfb_chain, rfb_t, priv_p->pcidev, x_NumRFB);

        /* Setup each RFB */
        for (i = 0; i < x_NumRFB; i++) {
                /* Get the next unprocessed RFB */
                get_chain_head(&priv_p->rfb_chain, rfb_t, rfb_p);
                ASSERT(rfb_p);

                /* Allocate skb and associate with rfb_p */
                rx_init_and_chain_packet(priv_p, rfb_p, TRUE);

                KDBG_LOG_INIT(KDBG_DEBUG,"RFB %08lx(%08lx) SKB %08lx  buffer %08lx\n",
                       (ULONG)rfb_p, (ULONG)rfb_p->dmaAddr, (ULONG)rfb_p->sw.rfb_skb,
                       (ULONG)kal_skb_data(rfb_p->sw.rfb_skb));

                // dbg_rx_packet(priv_p,rfb_p,DBFLG_RX_RFB);
        }

        KDBG_EXIT;
}   /* rx_setup_receive_queues */

/*----------------------------------------------------------------*/
/*!
* \brief  Cleanup the chains of RFB lists.
*
* Cleanup of the Receive Frame Buffer (RFB) at SHUTDOWN time.
*
* \param  priv_p       pointer to i2220mgmt_t, the device's private structure
* \return none
*/
/*----------------------------------------------------------------*/
void rx_cleanup_receive_queues(i2220mgmt_t *priv_p)
{
        struct list_head        *lst;
        rfb_t                   *rfb_p;
        int                     i=0;

        KDBG_ENTER;

        dmaStopRxQueue(priv_p); /* important */

        /* Free skb associated to each RFB */
        list_for_each(lst,&priv_p->rfb_chain.head) {
                rfb_p = list_entry( lst, rfb_t, sw.link );

                if (++i > x_NumRFB) {
                        KDBG_LOG_INIT(KDBG_WARN,"More SKB than expected, RFB %08lx SKB %08lx\n",
                                (unsigned long)rfb_p, (unsigned long)rfb_p->sw.rfb_skb);
                }

                priv_p->hwStatistics.freeRfbCount--;
                if ( rfb_p->sw.rfb_skb ){
                        kal_free_skb(rfb_p->sw.rfb_skb);
                }
        }

        destroy_chain_list(&priv_p->rfb_chain, rfb_t, priv_p->pcidev);

        KDBG_EXIT;
}

/*----------------------------------------------------------------*/
/*!
* \brief  Start the Receiving Unit - RDMA for operation
*
* This routine checks the status of the 2220's receive
* unit(RU), and starts the RU if it was not already
* active.  However, before restarting the RU, the driver
* cleans up any recent pending receives (this is very important).
*
* \param  priv_p       pointer to i2220mgmt_t, the device's private structure
* \return Pointer to the rdmaStartAddr to be used for the parameter of StartRDMA()
*/
/*----------------------------------------------------------------*/
kal_dma_addr_t rx_start_recv_unit(i2220mgmt_t *priv_p)
{
        rfb_t           *rfb_p;
        kal_dma_addr_t  startAddr;

        KDBG_ENTER;

        dmaStopRxQueue(priv_p); /* stop Q if it is operating */

        /* Attempt to acquire a new TFCB for end-list marker */
        peek_chain_head(&priv_p->rfb_chain, rfb_t, rfb_p);
        ASSERT(rfb_p);

        startAddr = rfb_p->dmaAddr;
//printk("Sam i2220hw.c 3151 startAddr=%08x\n",startAddr);
        dmaStartRxQueue(priv_p, (UINT32)startAddr);

        KDBG_EXIT;

        return startAddr;
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* This routine checks the status of the 2220's receive
*
* \param  priv_p       pointer to i2220mgmt_t, the device's private structure
* \return none
*/
/*----------------------------------------------------------------*/
#include <asm/system.h>
void i2220hw_reset_mac(i2220mgmt_t *priv_p)
{
        tdma_parameters_t       *tdma_p;
        tfcb_t                  *tfcb_p;
        int                     i, txQs_empty = TRUE;
        UINT32                  tmp32;
	unsigned long save_flags;

//local_irq_disable(); //jj
//jjlocal_irq_save(save_flags);
        /* Stop UDMA queues to make sure no PCI access */
        dmaStopRxQueue(priv_p);
        dmaStopSecQueue(priv_p, SECQ_RX);
        dmaStopSecQueue(priv_p, SECQ_TX);
        kal_udelay(10);

        /* Stop all Tx queues */
        for (i = 0; i < NUM_TDMA_CHANNELS - 2; i++) {
                dmaStopTxQueue(priv_p, (TX_QUEUE_NUM)i, FALSE);
        }

        /* Check if ARB is in idle state */
        NIC_MCR_WRITE(priv_p, MCR_PRBCR, 0x301);
        for (i = 0; i < 5000; i++) {
                NIC_MCR_READ(priv_p, MCR_PRBSR, &tmp32);
                if ( !(tmp32 & BITS(0,3)) ) {
                        /* bit[3:0] == 0000 (idle state) */
                        break;
                }
                kal_udelay(1);
        }

        /* Reset MAC/BB hardware */
        NIC_MCR_READ(priv_p, MCR_SCR, &tmp32);
        NIC_MCR_WRITE(priv_p, MCR_SCR, tmp32 | SCR_MAC_RST);
        kal_udelay(1);
        NIC_MCR_WRITE(priv_p, MCR_SCR, tmp32 | SCR_BB_MRST);
        kal_udelay(1);

        /* Restore tx queues. Clearing status is necessary */
        mgmt_stop_timer(&priv_p->tx_timer);
        priv_p->chk_txTimout = FALSE;
        for (i = 0; i < NUM_TDMA_CHANNELS - 2; i++) {
                tdma_p = &priv_p->tdma[i];
                if (tdma_p->head_p) {
                        tfcb_for_each(tdma_p->head_p, tfcb_p) {
                                tfcb_p->hw.txStatus = 0; /* clear status */
                        }
                        /* We must do startTDMA even EOL TFCB is the first one */
                        dmaStartTxQueue(priv_p, (TX_QUEUE_NUM)i,
                                        tdma_p->head_p->dmaAddr);
                        if (!(tdma_p->head_p->hw.tfdCtrlFlag & TFCB_FLAG_EOL)) {
                                txQs_empty = FALSE;
                        }
                }
        }
#if !WLAN_INCLUDE_DEBUG
        /* Restart tx_timeout mechanism if tx queues are not empty */
        if ( !txQs_empty ) {
                set_txTimeout(priv_p);
                priv_p->chk_txTimout = TRUE;
        }
#endif

#if SUPPORT_TKIP
        /* Restore tx/rx sec queues. Clearing status is necessary */
        for (i = SECQ_RX; i < NUM_TDMA_CHANNELS; i++) {
                tdma_p = &priv_p->tdma[i];
                if (tdma_p->head_p) {
                        tfcb_for_each(tdma_p->head_p, tfcb_p) {
                                /* clear the tkip mic field */
                                ipn_memset(tfcb_p->tkipMic, 0, 48);
                        }
                        /* We must do startTDMA even EOL TFCB is the first one */
                        dmaStartSecQueue(priv_p, (TX_QUEUE_NUM)i,
                                         tdma_p->head_p->dmaAddr);
                }
        }
#endif
        /* Restore rx queue.
         * Process all received frames ans clear their status.
         */
        isr_process_rx_done(priv_p);
        rx_start_recv_unit(priv_p);
        priv_p->macResetCount++;
//local_irq_enable(); //jj
//jjlocal_irq_restore(save_flags);

        KDBG_LOG_SYS(KDBG_WARN," ** MAC is reset! (count=%d) **\n",
                priv_p->macResetCount);
}

/*----------------------------------------------------------------*/
/*!
* \brief  Initializes the i2220hw_ctl_t data structure for use.
*
* Store the i/o address information for later access to the device.
* Calls the \a tx_setup_transmit_queues and \a rx_setup_receive_queues to
* initialize the TFCB / TBB / RFB DMA control structures.
* The configuration data in EEPROM are read in, such as MAC address, and
* IPN2220 hardware will then be initialized.
*
* \param  membase   memory base address for IPN2220 register access
* \param  priv_p    pointer to i2220mgmt_t, the device's private structure
* \return TRUE      if the IPN2220 hardware is successfully initialized \n
*         FALSE     otherwise
*/
/*----------------------------------------------------------------*/
int i2220hw_create( i2220mgmt_t *priv_p)
{
        int     status = 0;     /* success */
        KDBG_ENTER;

        /* Wakeup NIC */
        nicEnterStandby(priv_p);

        /* Read additional information from the NIC, such as MAC address. */
        if ( hw_read_adapter_info(priv_p) ){
                KDBG_LOG_INIT(KDBG_WARN, "Fail to read card info!");
                nicEnterSleep(priv_p);
                status = -2;
                goto fail;
        }

        /* Do basic hardware initialization */
        if ( hw_initializeAdapter(priv_p) ) {
                nicEnterSleep(priv_p);
                status = -3;
                goto fail;
        }

        tx_setup_transmit_queues(priv_p);
        rx_setup_receive_queues(priv_p);
        rx_start_recv_unit(priv_p);     /* Setup RDMA address */

        /* pointer to new struct or existed struct if no memory */
        if (priv_p->txRates_p == NULL) {
                priv_p->txRates_p = (txRate_info_t *)
                                kal_malloc(sizeof(txDataRateList));

                if (priv_p->txRates_p) {
                        ipn_memcpy(priv_p->txRates_p, &txDataRateList[0],
                                sizeof(txDataRateList));
                }
                else {
			printk("i2220hw_create: malloc fail\n");
                        priv_p->txRates_p = &txDataRateList[0];
                }
        }

        priv_p->hw_state = I2220HW_STATE_INIT;

        /* Set security table size and clean up SW and HW security table.
         * This function should be called after hw_initializeAdapter().
         */
        privacyInitialize(priv_p);

        /* Set default encrypt mode is no security, open system auth */
        wlanoidSetAuthenticationMode(priv_p, AuthMode_OPEN);
        wlanoidSetEncryptionStatus(priv_p, CipherMode_NONE);

    fail:
        KDBG_EXIT;
        return status;
}

/*----------------------------------------------------------------*/
/*!
* \brief  Destroy the i2220hw_ctl_t data structure.
*
* Partner to i2220hw_create().  This function cleans up the hw
* structure so that it can be freed by the caller using a simple
* kfree.  Currently, this function is just a placeholder.  If, at some
* point in the future, an hw in the 'shutdown' state requires a 'deep'
* kfree, this is where it should be done.  Note that if this function
* is called on a _running_ hw structure, the drvr_stop() function is
* called.
*
* \param  priv_p    pointer to i2220mgmt_t, the device's private structure
* \return none
*/
/*----------------------------------------------------------------*/
void i2220hw_destroy( i2220mgmt_t *priv_p)
{
        KDBG_ENTER;

        priv_p->hw_state = I2220HW_STATE_PREINIT;

        if (priv_p->txRates_p &&
            priv_p->txRates_p != &txDataRateList[0]) {
                kal_free(priv_p->txRates_p);
                priv_p->txRates_p = NULL;
        }

        tx_cleanup_transmit_queues(priv_p);
        rx_cleanup_receive_queues(priv_p);

        KDBG_EXIT;
        return;
}


/*----------------------------------------------------------------*/
/*!
* \brief  The registered driver interrupt handler, also called top half in Linux.
*
* Simply acknowledge the interrupt and scheduled the bottom-half tasklet to
* continue with further processing of the interrupt.
* For some platform this ISR might handle all interrupt context within this function
* without spawning a new TASK as a deferred procedure call. Therefore our interrupt
* handler have a compiler option to decide whether to invoke the bottom half directly
* or defer the execution to tasklet.
*
* \param  irq             irq number, don't care here
* \param  regs            registers, don't care here
* \param  dev_id          pointer to i2220mgmt_t, the device's private structure
* \return none
* \sa     do_isr_tasklet
*/
/*----------------------------------------------------------------*/
int i2220hw_interrupt(void *dev_priv)
{
        ULONG                   isr;
        ULONG                   scr;
        i2220mgmt_t             *priv_p = (i2220mgmt_t*)dev_priv;

        KDBG_ENTER;

        /* Read the SCR to check if the global interrupt mask bit is enabled. */
        NIC_MCR_READ(priv_p, MCR_IER, &scr);

        /* Read Interrupt Status Register */
        NIC_MCR_READ(priv_p, MCR_ISAR, &isr);
        /* Strip off the disabled interrupts */
        isr &= priv_p->interruptEnable;

        /* We want to process the interrupt if the output from our interrupt line
         * is high, and our interrupt is not masked.  If our interrupt line
         * is already masked, then we must be currently processing interrupts.
         */
        if ((scr & IER_GLOBAL_ENABLE) && isr) {
                nicDisableInterrupt(priv_p);

                /* Save interrupt status */
                priv_p->interruptStatus |= isr;
                return TRUE;
        }

        KDBG_EXIT;
        return FALSE;
}


/*----------------------------------------------------------------*/
/*!
* \brief  Startup the IPN2220 MAC, PAU and RF hardware for operation.
*
* The IPN2220 MAC, PAU and RF hardware are enabled for operation via this
* routine. Interrupt enabling bits are opened and receiving operation is
* start up.
*
* \pre \a priv_p->hwCtl->interruptEnable  contains the desired interrupts
* \param  priv_p    pointer to i2220mgmt_t, the device's private structure
* \return none
*/
/*----------------------------------------------------------------*/
int i2220hw_drvr_start(i2220mgmt_t *priv_p)
{
        KDBG_ENTER;

        /* Disable board interrupts */
        nicDisableInterrupt(priv_p);

        if ( priv_p->hw_state == I2220HW_STATE_RUNNING ||
             priv_p->hw_state == I2220HW_STATE_SCANNING ){
                pauDisable(priv_p);
        }
        else if ( priv_p->hw_state == I2220HW_STATE_PREINIT ){
                KDBG_LOG_INIT(KDBG_ERR, "Cannot startup because of wrong HW state\n");
                return 1;
        }
        kal_udelay(20); /* Just for safe */

        /* The flowing functions set related parameters to chip
         * All parameters should be ready before here.
         */
        hw_set_channel(priv_p, 0xff);
        hw_set_rate_info(priv_p);
        hw_set_beacon(priv_p, TRUE);

        priv_p->hw_state = I2220HW_STATE_RUNNING;

        /* Acknowledge all pending interrupts. */
        NIC_MCR_WRITE(priv_p, MCR_ISAR, 0xFFFFFFFF);
        NIC_MCR_WRITE(priv_p, MCR_QSR0, 0xFFFFFFFF);
        NIC_MCR_WRITE(priv_p, MCR_QSR1, 0xFFFFFFFF);

        pauEnable(priv_p);      /* Enable PAU (start to work) */

        NIC_MCR_WRITE(priv_p, 0x80, 0x0b10); /* MAC debug port */

        /* Enable board interrupts */
        nicEnableInterrupt(priv_p);

        KDBG_EXIT;
        return 0;
}


/*----------------------------------------------------------------*/
/*!
* \brief  Stop the IPN2220 MAC, PAU and RF hardware from further operation.
*
* We will disable the hardware by this function, after which, the device
* should be virtually detached from the wireless media.
*
* \param  priv_p    pointer to i2220mgmt_t, the device's private structure
* \return none
*/
/*----------------------------------------------------------------*/
int i2220hw_drvr_stop(i2220mgmt_t *priv_p)
{
        KDBG_ENTER;

        priv_p->hw_state = I2220HW_STATE_INIT;

        /* Disable the device's interrupt line. */
        nicDisableInterrupt(priv_p);

        phyRFEnterSleep(priv_p);
        pauDisable(priv_p);
        NIC_MCR_WRITE(priv_p, MCR_BGCR1, 0); /* Disable Beacon */

        mgmt_stop_timer(&priv_p->scan_timer);

        /* Packet cannot be sent out now */
        mgmt_stop_timer(&priv_p->tx_timer);
        priv_p->chk_txTimout = FALSE;

        KDBG_EXIT;
        return 0;
}

