/*
** $Id: rftest.c 1.9 2004/10/04 17:41:24 cmchang Exp $
*/

/*! \file rftest.c
    \brief IPN2220 RF test routines
*/

/******************************************************************************
Copyright (c) 2002-2003 Integrated Programmable Communications, Inc.

All rights reserved. Copying, compilation, modification, distribution
or any other use whatsoever of this material is strictly prohibited
except in accordance with a Software License Agreement with
Integrated Programmable Communications, Inc.
******************************************************************************/

/************************************************************************
*                     C O M P I L E R   F L A G S
*************************************************************************
*/

/************************************************************************
*                E X T E R N A L   R E F E R E N C E S
*************************************************************************
*/
#include "precomp.h"

/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/

/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/


/************************************************************************
*                        P U B L I C   D A T A
*************************************************************************
*/


/************************************************************************
*                       P R I V A T E   D A T A
*************************************************************************
*/

/************************************************************************
*                             M A C R O S
*************************************************************************
*/

/************************************************************************
*              F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/

/************************************************************************
*                          F U N C T I O N S
*************************************************************************
*/

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return none
*/
/*----------------------------------------------------------------*/
void i2220_rftest_power(i2220mgmt_t *priv_p, UINT8 power)
{
        int     i;

        /* TxPower for burst mode and tone mode, even after switching channel */
        for (i = 0; i < MAX_NUM_2G_RF_CHANNELS; i++) {
                priv_p->nicPerChannelConfig24[i].txPowerGainCCK.gainMax = power;
                priv_p->nicPerChannelConfig24[i].txPowerGainOFDM.gainMax = power;
        }
#if SUPPORT_802_11A
        for (i = 0; i < MAX_NUM_5G_RF_CHANNELS; i++) {
                priv_p->nicPerChannelConfig5[i].txPowerGainCCK.gainMax = power;
                priv_p->nicPerChannelConfig5[i].txPowerGainOFDM.gainMax = power;
        }
#endif
        priv_p->txPowerGainForCCK = power;
        priv_p->txPowerGainForOFDM = power;

        BCIWrite8(priv_p, BBCR_CR14, power, TRUE, 0xff);
        BCIWrite8(priv_p, BBCR_CR15, power, TRUE, 0xff);
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return none
*/
/*----------------------------------------------------------------*/
void i2220_rftest_tone(i2220mgmt_t *priv_p)
{
        UINT32  tmp32, pctx;
        int     pa2, pa5;

        /* Check PA polarity */
        NIC_MCR_READ(priv_p, MCR_PCICR, &tmp32);
        pa2 = (tmp32 & PCICR_PA2_POLARITY) ? 1 : 0;
        pa5 = (tmp32 & PCICR_PA5_POLARITY) ? 1 : 0;

        /* Enable TX_RX, PA_PE, TR_SW = T */
        pctx = PPMCR_GPIO_TR_SW | PPMCR_GPIO_RF_TX | PPMCR_GPIO_RF_SHDN;
        if (pa2 ) {
                pctx |= PPMCR_GPIO_PA2_EN;
        }
        if (pa5 ) {
                pctx |= PPMCR_GPIO_PA5_EN;
        }
        NIC_MCR_WRITE(priv_p, MCR_PPMCR, pctx);

        /* Configure BBP to forced DA mode,
         * After test, should reset these BB CR to 0
         */

        /* AFE AD/DA on */
        BCIWrite8(priv_p, 82, 0xA0, TRUE, 0xff);
        BCIWrite8(priv_p, 83, 0xAA, TRUE, 0xff);

        /* set CLK80M/40M clock */
        BCIWrite8(priv_p, 88, 0xC6, TRUE, 0xff);
        BCIWrite8(priv_p, 89, 0x04, TRUE, 0xff);
        BCIWrite8(priv_p, 90, 0x80, TRUE, 0xff);

        /* force AFE AD/DA data */
        BCIWrite8(priv_p, 84, 0x80, TRUE, 0xff);
        BCIWrite8(priv_p, 85, 0xFF, TRUE, 0xff);
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return none
*/
/*----------------------------------------------------------------*/
void i2220_rftest_burst(i2220mgmt_t *priv_p)
{
        kal_skb_t       *skb;
        Frame_t         *tpdu;
        int             i;

        priv_p->protectMode = CTS_PROTECT_NONE; /* No CTS protect */

        for (i = 0; i < 10; i++) {
                skb = kal_alloc_skb(sizeof(Frame_t) + 1024);
                if (skb == NULL) { printk("i2220_rftest_burst alloc skb fail\n"); break;}

                /* The starting address should be 4-wise boundary */
                ASSERT((((UINT32)kal_skb_data(skb))&3)==0);

                tpdu = (Frame_t *)kal_skb_data(skb);
                tpdu->skb = skb;
                tpdu->mac_ctl = MACCTL_SDU_FRAME_IN_SKB;
                tpdu->LLCdata = (UINT8*)tpdu + sizeof(Frame_t);
                tpdu->LLClength = 1024;

                ipn_memset(tpdu->addr1, 0xff, MAC_ADDR_LEN); /* bcast */
                ipn_memcpy(tpdu->addr2, mib_import(dot11MACAddress), MAC_ADDR_LEN);
                tpdu->frameCtrl = host2ieee16(SYN_data);

                sdu_send_request(priv_p, tpdu, NULL, SYN_contention);
        }
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return none
*/
/*----------------------------------------------------------------*/
void i2220_rftest_mode(i2220mgmt_t *priv_p, UINT8 mode)
{
        i2220hw_drvr_stop(priv_p);
        priv_p->chk_txTimout = TRUE; /* Avoid time out to reset */

        priv_p->rftest_burst = FALSE;
        KDBG_LOG_INIT(KDBG_NOTICE, "rftest_mode=%d\n", mode);

        /* Clean up history of tone mode */
        BCIWrite8(priv_p, 82, 0x00, TRUE, 0xff);
        BCIWrite8(priv_p, 85, 0x00, TRUE, 0xff);
        BCIWrite8(priv_p, 84, 0x00, TRUE, 0xff);
        BCIWrite8(priv_p, 90, 0x00, TRUE, 0xff);
        BCIWrite8(priv_p, 88, 0x00, TRUE, 0xff);

        switch (mode) {
        case RFTEST_MODE_STOP:
                break;
        case RFTEST_MODE_RX:
                phyRFEnterActive(priv_p);
                break;
        case RFTEST_MODE_TONE:
                i2220_rftest_tone(priv_p);
                break;
        case RFTEST_MODE_BURST:
                phyRFEnterActive(priv_p);
                pauEnable(priv_p);

                /* Only enable tx done interrupt */
                priv_p->interruptEnable = IER_TX_DATA_Q_DONE_INT;
                NIC_MCR_WRITE(priv_p, MCR_IER, priv_p->interruptEnable);
                nicEnableInterrupt(priv_p);

                priv_p->rftest_burst = TRUE;
                i2220_rftest_burst(priv_p);
                break;
        } /* end of switch(mode) */
}


