# ---------------------------------------------------------------------------
# Copyright (c) 2002-2003 Inprocomm, Inc.
#
# All rights reserved. Copying, compilation, modification, distribution
# or any other use whatsoever of this material is strictly prohibited
# except in accordance with a Software License Agreement with Inprocomm, Inc.
# ---------------------------------------------------------------------------

ARCH=$(shell echo $$ARCH)
ifeq ($(strip $(ARCH)), )
ARCH=i386
endif

KCFLAGS         = -D__KERNEL__ -Wstrict-prototypes -Wno-trigraphs -O2 -fomit-frame-pointer \
                        -fno-strict-aliasing -fno-common
KERNEL_MODFLAGS = -DMODULE

# Renesa (formal Hitachi) SH-4 Little Endian
ifeq ($(ARCH), sh4)
D_ARCH=__ARCH_HITACHI_RENESA__
CPU_ENDIAN=0
LINUX_SRC=/opt/hardhat/previewkit/lsp/hitachi-ms7751rse01-sh_sh4_le-previewkit/linux-2.4.18_mvl30
CROSS_COMPILE = /opt/hardhat/previewkit/sh/sh4_le/bin/sh_sh4_le-
KERNEL_CFLAGS=$(KCFLAGS) -Wall -fno-common -ml -m4 -pipe -mno-implicit-fp
endif

# Sunplus MIPS-R3000 CPU, Little Endian
# Using mipsel-linux-gcc v3.0.1
ifeq ($(ARCH), sunplus)
D_ARCH=__ARCH_SUNPLUS_R3000__
CPU_ENDIAN=0
LINUX_SRC=/tmp/sunplus2/linux-2.4.21
CROSS_COMPILE = /tmp/sunplus2/usr/local/sunplus/mipsel/bin/mipsel-linux-
KERNEL_CFLAGS=$(KCFLAGS) -Wall -G 0 -mno-abicalls -fno-pic -pipe -mcpu=r3000 -mips1 -mlong-calls
endif

# NEC VR4131 MIPS-R4100 CPU, Little Endian
# Using mipsel-linux-gcc v3.0.1
ifeq ($(ARCH), vr4131)
D_ARCH=__ARCH_NEC_VR4131__
CPU_ENDIAN=0
LINUX_SRC=/usr/src/linux-vr4131
CROSS_COMPILE = /usr/local/bin/mipsel-linux-
KERNEL_CFLAGS=$(KCFLAGS) -Wall -G 0 -mno-abicalls -fno-pic -mcpu=vr4100 -mips2 \
                -pipe -mlong-calls
endif

# WavePlus MIPS-R3000 CPU, Little Endian
# Using mipsel-linux-gcc v3.0.1
ifeq ($(ARCH), wp)
D_ARCH=__ARCH_WAVEPLUS_WP3210__
CPU_ENDIAN=0
#LINUX_SRC=/usr/src/linux-wp
#CROSS_COMPILE = /usr/local/bin/mipsel-linux-
LINUX_SRC=/Waveplus/linux-2.4.17-uc0
CROSS_COMPILE = /Waveplus/uClinux-tool/bin/mipsel-linux-
KERNEL_CFLAGS=$(KCFLAGS) -Wall -G 0 -mno-abicalls -fno-pic -mcpu=r3000 -mips1 -pipe
endif

# Toshiba TX4925 MIPS-R4600 CPU, Little Endian
# Using binutils v2.13, mipsel-linux-gcc v3.2, glibc 2.2.5
ifeq ($(ARCH), tx49)
D_ARCH=__ARCH_TOSHIBA_TX4925__
CPU_ENDIAN=0
LINUX_SRC=/usr/src/linux-tx49
CROSS_COMPILE = /usr/mipsel/bin/mipsel-linux-
KERNEL_CFLAGS=$(KCFLAGS) -I$(LINUX_SRC)/include/asm/gcc -Wall -G 0 -mno-abicalls \
                -fno-pic -mcpu=r4600 -mips2 -Wa,--trap -pipe -mlong-calls
endif

# ADMtek adm5120 MIPS-R4600 CPU, Little Endian
# Using mipsel-linux-gcc v2.96
ifeq ($(ARCH), adm5120)
D_ARCH=__ARCH_ADMTEK_ADM5120__
CPU_ENDIAN=0
LINUX_SRC=/usr/src/linux-adm5120
#LINUX_SRC=/opt/adm5120/ev_board/adm5120-pkg-1.02.00/linux-2.4.18
CROSS_COMPILE = /export/tools/bin/mipsel-linux-
KERNEL_CFLAGS=$(KCFLAGS) -I$(LINUX_SRC)/include/asm/gcc -Wall -G 0 -mno-abicalls \
                -fno-pic -march=r4600 -mips2 -Wa,--trap -pipe -mlong-calls
endif

# Motolola M68K CPU, Big Endian
ifeq ($(ARCH), m68k)
D_ARCH=__ARCH_MOTO_M68K__
CPU_ENDIAN=1
LINUX_SRC=/usr/src/linux-m68k
CROSS_COMPILE = /usr/local/bin/m68k-elf-
KERNEL_CFLAGS=$(KCFLAGS) -Wall -fno-builtin -nostdinc -pipe -DNO_MM -DNO_FPU -m5200 \
                -Wa,-S -Wa,-m5200 -D__ELF__ -DMAGIC_ROM_PTR -DUTS_SYSNAME="uClinux" \
                -D__linux__   -nostdinc -iwithprefix include
endif

# Samsung S3C2510 ARM940T CPU, Little Endian
# Using binutils v2.11-7, arm-elf-gcc v2.95.3-4, uC-libc-arm v0.9.8.1-3, uC-libm-arm v0.9.8.1-3
ifeq ($(ARCH), s3c2510)
D_ARCH=__ARCH_SAMSUNG_S3C2510__
CPU_ENDIAN=0
LINUX_SRC=/usr/src/linux-s3c2510
CROSS_COMPILE = /opt/uClinux/bin/arm-elf-
KERNEL_CFLAGS=$(KCFLAGS) -Wall -pipe -fno-builtin -D__linux__ -DNO_MM -mapcs-32 \
                -mshort-load-bytes -msoft-float
endif

# Realtek rtl8651 MIPS-R3000 CPU, Big Endian
ifeq ($(ARCH), rtl8651)
D_ARCH=__ARCH_REALTEK_RTL8651__
CPU_ENDIAN=1
KERNEL_MODFLAGS =
LINUX_SRC=/opt/abocom/fsw410/linux-2.4.x
CROSS_COMPILE = /opt/323/mips/bin/mips-uclibc-
KERNEL_CFLAGS=$(KCFLAGS) -Wall -G 0 -mno-abicalls -fno-pic -mcpu=r3000 -mips1 -pipe
endif

# Realtek rtl8651 MIPS-R3000 CPU, Big Endian
ifeq ($(ARCH), rtl8651_333)
D_ARCH=__ARCH_REALTEK_RTL8651__
CPU_ENDIAN=1
KERNEL_MODFLAGS =
LINUX_SRC=/mnt/linux_opt/abocom/fsw410/linux-2.4.x
CROSS_COMPILE = /opt/toolchain_mips/bin/mips-linux-uclibc-
KERNEL_CFLAGS=$(KCFLAGS) -Wall -G 0 -mno-abicalls -fno-pic -mips1 -pipe
endif


# Kendin ks8695px ARM 9 CPU, Little Endian
ifeq ($(ARCH), kendin)
D_ARCH=__ARCH_KENDIN_KS8695PX__
CPU_ENDIAN=0
LINUX_SRC=/usr/src/linux-kendin
CROSS_COMPILE = /opt/kendin/tools/arm-linux/usr/local/bin/arm-linux-
KERNEL_CFLAGS=$(KCFLAGS) -Wall -G 0 -mno-abicalls -fno-pic -mcpu=r3000 -mips1 -pipe
endif

# Intel x86 CPU, Little Endian
ifeq ($(ARCH), i386)
D_ARCH=__ARCH_INTEL_I386__
CPU_ENDIAN=0
LINUX_SRC=/usr/src/linux-i386
CROSS_COMPILE=
KERNEL_CFLAGS=$(KCFLAGS) -Wall -Wno-unused -pipe -mpreferred-stack-boundary=2 -march=i686 \
                -malign-functions=4
endif

# SD Lexra, Inc. 4189 , Big Endian
ifeq ($(ARCH), lex4189)
D_ARCH=__ARCH_SD_LEX4189__
CPU_ENDIAN=1
LINUX_SRC=/mnt/hda6/sd-linux/linux-2.4.17_mvl21
#LINUX_SRC=/mnt/hda6/sd-linux0330/Linux-sd-00-01-09/kernel/linux-2.4.17_mvl21
CROSS_COMPILE = /opt/hardhat/devkit/mips/lexra_fp_be/bin/mips_lexra_fp_be-
KERNEL_CFLAGS=$(KCFLAGS) -Wall -I$(LINUX_SRC)/include/asm \
                -mips1 -mno-abicalls -fno-pic -pipe -mlong-calls
endif

# Intel IXP Snap Gear, Big Endian
ifeq ($(ARCH), ixp)
D_ARCH=__ARCH_INTEL_IXP__
CPU_ENDIAN=1
LINUX_SRC=/mnt/hda6/intel/snapgear/linux-2.4.x
CROSS_COMPILE = /mnt/hda6/intel/usr/local/bin/arm-linux-
KERNEL_CFLAGS=$(KCFLAGS) -Wall -pipe  -nostdinc -iwithprefix include -mbig-endian
LDFLAGS=-EB
endif

# TrendChip TC3162P Lexra-4180 (MIPS-R3000 like) CPU, Big Endian
ifeq ($(ARCH), tc3162p)
D_ARCH=__ARCH_TRENDCHIP_TC3162P__
CPU_ENDIAN=1
KERNEL_MODFLAGS =
LINUX_SRC=/home/TC/linux
#CROSS_COMPILE = /usr/local/mips-kgcc/bin/mips-linux-
CROSS_COMPILE = /usr/local/mips-uclinux/bin/mips-uclibc-
KERNEL_CFLAGS=$(KCFLAGS) -G 0 -mno-abicalls -fno-pic -mips1 -pipe -Wall -mcpu=r3000 -mno-unaligned-insn \
                -DNO_UNALIGNED_INSN -DNO_FPU -msoft-float -gstabs+ -pipe -DNO_MM -I$(LINUX_SRC)/include/asm -I$(LINUX_SRC)/include -I$(LINUX_SRC)/include/asm/gcc
LDFLAGS=
endif


# ---------------------------------------------------
#MAKE     = make

#AS       = $(CROSS_COMPILE)as
#LD       = $(CROSS_COMPILE)ld
#CC       = $(CROSS_COMPILE)gcc
#AR       = $(CROSS_COMPILE)ar
#NM       = $(CROSS_COMPILE)nm
#STRIP    = $(CROSS_COMPILE)strip
#OBJCOPY  = $(CROSS_COMPILE)objcopy
#OBJDUMP  = $(CROSS_COMPILE)objdump
#RANLIB   = $(CROSS_COMPILE)ranlib
#CPP      = $(CC) -E

# ---------------------------------------------------
# Build options
# ---------------------------------------------------
# Enable debug mode
WLAN_DEBUG=n

# Support proc file system
WLAN_PROC=y

# Support WDS networks
WLAN_WDS=y

# Support RF testing for EMI certification
WLAN_Rftest=y

# Support Memory Pool instead of system malloc 
WLAN_MEMPool=y

# Support WPA and ACL functions for Linux hostapd ; jj origional is y
#Sam modify WPA-PSK error 20050415
WLAN_support_IPNRSN=y

# Support the extern Linux-based Hostapd for 802-11i, inlude the RSN and WPA
WLAN_support_80211i=n

# Support the TKIP security 
WLAN_support_TKIP=y

# Support software calculation of partial TKIP MIC
# Sam WLAN_support_TKIP_SW_MIC=y
WLAN_support_TKIP_SW_MIC=n

# Support IPN5320 for dual band operation
WLAN_support_IPN5320=n

# ---------------------------------------------------
ifeq ($(WLAN_DEBUG), y)
WLAN_INCLUDE_DEBUG=-DWLAN_INCLUDE_DEBUG=1
endif

ifeq ($(WLAN_PROC), y)
WLAN_INCLUDE_PROC=-DWLAN_INCLUDE_PROC=1
endif

ifeq ($(WLAN_WDS), y)
WLAN_INCLUDE_WDS=-DWLAN_INCLUDE_WDS=1
endif

ifeq ($(WLAN_Rftest), y)
WLAN_SUPPORT_RFTEST=-DWLAN_SUPPORT_RFTEST=1
endif

ifeq ($(WLAN_MEMPool), y)
WLAN_USE_MEMPOOL=-DWLAN_USE_MEMPOOL=1
endif

ifeq ($(WLAN_support_80211i), y)
WLAN_SUPPORT_80211i=-DSUPPORT_WPA2=1
WLAN_support_IPNRSN=n
endif

ifeq ($(WLAN_support_IPNRSN), y)
WLAN_SUPPORT_IPNRSN=-DSUPPORT_IPNRSN=1
WLAN_SUPPORT_80211i=-DSUPPORT_WPA2=0
endif

ifeq ($(WLAN_support_TKIP), y)
WLAN_SUPPORT_TKIP=-DSUPPORT_TKIP=1
endif

ifeq ($(WLAN_support_TKIP_SW_MIC), y)
WLAN_SUPPORT_TKIP_SW_MIC=-DSUPPORT_SW_TKIP_MIC=1
endif

ifeq ($(WLAN_support_IPN5320), y)
WLAN_SUPPORT_IPN5320=-DIPN5320=1
endif

# ---------------------------------------------------
# Preprocessor Options
# ---------------------------------------------------
#CFLAGS = $(KERNEL_CFLAGS) $(KERNEL_MODFLAGS) \
        -D$(D_ARCH) -D__CPU_ENDIAN__=$(CPU_ENDIAN)
# Sam re-write the FLAGS
OPTFLAGS = $(KERNEL_MODFLAGS) \
        -D$(D_ARCH) -D__CPU_ENDIAN__=$(CPU_ENDIAN)

CPPFLAGS = $(OPTFLAGS) -I$(OS_DIR) -I$(LINUX_SRC)/include -I../include/ipn2220 \
        $(WLAN_INCLUDE_PROC) $(WLAN_SUPPORT_RFTEST) \
        $(WLAN_INCLUDE_WDS) $(WLAN_USE_MEMPOOL) $(WLAN_SUPPORT_IPNRSN) \
        $(WLAN_SUPPORT_TKIP) $(WLAN_SUPPORT_TKIP_SW_MIC) $(WLAN_SUPPORT_80211i) \
        $(WLAN_SUPPORT_IPN5320)
LDFLAGS += -r
