/*
** $Id: netdev_1x.c 1.15 2004/09/23 19:31:03 cmchang Exp $
*/

/*! \file netdev_1x.c
    \brief IPN2120 Plug-in module for 802.1x interface, Linux NET4 net_device interface.

    Providing a specific NET device interface for 802.1x packets, as the 1x
    state machine is run under User Mode.
*/

/******************************************************************************
Copyright (c) 2002-2003 Inprocomm, Inc.

All rights reserved. Copying, compilation, modification, distribution
or any other use whatsoever of this material is strictly prohibited
except in accordance with a Software License Agreement with Inprocomm, Inc.
******************************************************************************/

/************************************************************************
*                E X T E R N A L   R E F E R E N C E S
*************************************************************************
*/
/* System Includes */
#include "wlan_os.h"
#include "precomp.h"
#include "netdev_1x.h"

/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/

/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/

/************************************************************************
*                             M A C R O S
*************************************************************************
*/

/************************************************************************
*              F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/
extern int wlan_1x_request(struct sk_buff *skb, struct net_device *dev);
#if SUPPORT_IPNRSN
extern void set_sm_timer(void *priv_p);
#endif
/************************************************************************
*                        P U B L I C   D A T A
*************************************************************************
*/

/************************************************************************
*                       P R I V A T E   D A T A
*************************************************************************
*/

/************************************************************************
*                          F U N C T I O N S
*************************************************************************
*/

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
static int i2220_init(struct net_device *dev)
{
        return 0;
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
static int i2220_open(struct net_device *dev)
{
        MOD_INC_USE_COUNT;
        netif_start_queue(dev);

        return 0; /* success */
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
static int i2220_close(struct net_device *dev)
{
        MOD_DEC_USE_COUNT;
        netif_stop_queue(dev);

        return 0; /* success */
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
static void __setup_1x_netdev(struct net_device *dev)
{
        ether_setup(dev);

        dev->init             = &i2220_init;
        dev->open             = &i2220_open;
        dev->stop             = &i2220_close;
        dev->hard_start_xmit  = &wlan_1x_request;

        dev->type = ARPHRD_IEEE80211;

        netif_stop_queue(dev);
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
int init_dev1x(struct net_device *apdev)
{
        i2220mgmt_t             *priv_p = kal_dev_priv(apdev);
        struct net_device       *dev;

        if (priv_p == NULL) return -1;

        dev = kal_malloc(sizeof(struct net_device));
        if (dev == NULL) { printk("init_dev1x: malloc fail\n"); return -ENOMEM;}

        /* Setup the device for 802.1x */
        memset(dev, 0, sizeof(struct net_device));
        dev->priv = priv_p;
        priv_p->dev_1x = dev; /* Cross reference */

        sprintf(dev->name, "%sap", apdev->name);

        __setup_1x_netdev(dev);

        /* Use register_netdev() instead of register_netdevice(). So we don't
         * apply rtnl_lock() and rtnl_unlock() mechanism.
         */
        if (register_netdev(dev)) {
                kal_free(dev);
                priv_p->dev_1x = NULL;
                return -1;
        }
        #if SUPPORT_IPNRSN
        set_sm_timer(kal_dev_priv(apdev));
        #endif
        return 0;
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
void rm_dev1x(struct net_device *apdev)
{
        i2220mgmt_t     *priv_p = kal_dev_priv(apdev);

        if (priv_p && priv_p->dev_1x) {
                unregister_netdev(priv_p->dev_1x);
                kal_free(priv_p->dev_1x);
                priv_p->dev_1x = NULL;
        }
}

