/*
** $Id: dma.c 1.5 2004/10/04 18:09:14 cmchang Exp $
*/

/*************************************************************************
* Copyright (c) 2002-2003 Inprocomm, Inc.
*
* All rights reserved. Copying, compilation, modification, distribution
* or any other use whatsoever of this material is strictly prohibited
* except in accordance with a Software License Agreement with
* Inprocommm, Inc.
**************************************************************************/
/*************************************************************************
GENERAL DESCRIPTION
********************
This file contains the Tx/Sec/Rx queue routines of Windows driver for
INPROCOMM 802.11 Wireless LAN Adapters.
*/

/************************************************************************
*                E X T E R N A L   R E F E R E N C E S
*************************************************************************
*/
#include "precomp.h"

/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/

/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/

/************************************************************************
*                        P U B L I C   D A T A
*************************************************************************
*/

/************************************************************************
*                       P R I V A T E   D A T A
*************************************************************************
*/
DEBUGFILE("dma.c");

/************************************************************************
*                             M A C R O S
*************************************************************************
*/

/************************************************************************
*              F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/

/************************************************************************
*                          F U N C T I O N S
*************************************************************************
*/

/*_________________________________________________________________________
**  dmaStartTxQueue
**
**  descriptions: This function is called by a transmit routine to start
**                the specific Tx queue operation to transmit frames.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**      txQNum - The TX Queue number to be started
**      startingAddress - Starting address to begin the DMA transfer
**  return:
**      TRUE - Tx queue operation is started
**      FALSE - The start of the Tx queue operation is failed
**  note:
**__________________________________________________________________________*/
BOOLEAN
dmaStartTxQueue(IN PMP_ADAPTER adapter_p,
                IN TX_QUEUE_NUM txQNum,
                IN UINT_32 startingAddress)
{
    UINT_32 txQStartMask;
    UINT_32 tdsarOffset;
    UINT_32 txQOperatingMask;
    UINT_32 temp32;

    DEBUGFUNC("dmaStartTxQueue");

    DBGLOG(TX, TRACE, ("\n"));

    ASSERT(txQNum <= TXQ_TS3);
    ASSERT(startingAddress);

    txQStartMask = 0x00000001 << (2 * txQNum);
    DBGLOG(TX, LOUD, ("QCR command: %x\n", txQStartMask));

    tdsarOffset = MCR_TQ0SAR + 4 * txQNum;
    DBGLOG(TX, LOUD, ("SAR offset: %x\n", tdsarOffset));

    txQOperatingMask = QSR1_TQ0_OPERATING << txQNum;

#if DBG
    NIC_MCR_READ(adapter_p, MCR_QSR1, &temp32);
    if (temp32 & txQOperatingMask) {
        ERRORLOG(("Q (%d) is not idle when starting it\n", txQNum));
        NIC_MCR_READ(adapter_p, MCR_MPTCR, &temp32);
        ERRORLOG(("MPTCR: %x\n", temp32));
        NIC_MCR_READ(adapter_p, MCR_ACWLR, &temp32);
        ERRORLOG(("ACWlR: %x\n", temp32));
        ASSERT(0);
        return FALSE;
    }
#else
    NIC_MCR_READ(adapter_p, MCR_QSR1, &temp32);
    if (temp32 & txQOperatingMask) {
        printk("Q (%d) is not idle when starting it\n", txQNum);
        ASSERT(0);
	return FALSE;
    }
#endif

    switch (txQNum) {
        case TXQ_TQ0:
             temp32 = TXQIER_TQ0_EVENT_EN;
             break;
        case TXQ_TQ1:
             temp32 = TXQIER_TQ1_EVENT_EN;
             break;
        default:
             temp32 = TXQIER_AC0_EVENT_EN << (2 * (txQNum - 2));
             break;
    }
    NIC_MCR_WRITE(adapter_p, MCR_QSR1, temp32);
    NIC_UNMASK_TXQ_INTERRUPT(adapter_p, temp32);

    NIC_MCR_WRITE(adapter_p, tdsarOffset, startingAddress);

    NIC_MCR_WRITE(adapter_p, MCR_QCR, txQStartMask);

    NIC_MCR_READ(adapter_p, MCR_QSR1, &temp32);
    if ((temp32 & txQOperatingMask) == 0) {
        ERRORLOG(("Q (%d) is not operating after starting it\n", txQNum));
        return FALSE;
    }

    return TRUE;
}   /* dmaStartTxQueue */

/*_________________________________________________________________________
**  dmaStopTxQueue
**
**  descriptions: This function is called by a transmit routine to stop
**                the specific Tx queue operation immediately.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**      txQNum - The TX Queue number to be stopped
**  return:
**      TRUE - Tx queue operation is stopped
**      FALSE - The current stop command of the Tx queue is failed
**  note:
**__________________________________________________________________________*/
BOOLEAN
dmaStopTxQueue(IN PMP_ADAPTER adapter_p,
               IN TX_QUEUE_NUM txQNum,
               IN BOOLEAN chkStopStatus)
{
    UINT_32 txQStopCommand;
    UINT_32 txQOperatingMask;
    BOOLEAN txQStopped;
    UINT_32 temp32;

    DEBUGFUNC("dmaStopTxQueue");

    DBGLOG(TX, TRACE, ("\n"));

    ASSERT(txQNum <= TXQ_TS3);

    txQStopCommand = 0x00000002 << (2 * txQNum);
    txQOperatingMask = QSR1_TQ0_OPERATING << txQNum;
    DBGLOG(TX, LOUD, ("QCR command: 0x%08x, QSR operating mask: 0x%08x\n",
        txQStopCommand, txQOperatingMask));

    NIC_MCR_READ(adapter_p, MCR_QSR1, &temp32);
    DBGLOG(TX, TRACE, ("Q (%d) Status before TX Q Stop: %x.\n", txQNum,
        temp32 & txQOperatingMask));


    NIC_MCR_WRITE(adapter_p, MCR_QCR, txQStopCommand);

#if WORKAROUND_TX_STOP_STATUS
    if (chkStopStatus) {
        txQStopped = dmaCheckTxQueueStopStatus(adapter_p);
    } else {
        txQStopped = TRUE;
    }
#endif

    switch (txQNum) {
        case TXQ_TQ0:
             temp32 = TXQIER_TQ0_EVENT_EN;
             break;
        case TXQ_TQ1:
             temp32 = TXQIER_TQ1_EVENT_EN;
             break;
        default:
             temp32 = TXQIER_AC0_EVENT_EN << (2 * (txQNum - 2));
    }

    NIC_MASK_TXQ_INTERRUPT(adapter_p, temp32);
    NIC_MCR_WRITE(adapter_p, MCR_QSR1, temp32);

#if WORKAROUND_TX_STOP_STATUS
    if (!txQStopped) {
        ERRORLOG(("Q (%d) is still operating after stoping it.\n", txQNum));
        //ASSERT(0);
        return FALSE;
    }
#endif

    return TRUE;
}	/* dmaStopTxQueue */

/*_________________________________________________________________________
**  dmaResumeTxQueue
**
**  descriptions: This function is called by a transmit routine to resume
**                the specific Tx queue operation immediately.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**      txQNum - The TX Queue number to be resumed
**  return:
**      TRUE - Tx queue operation is resumed successfully
**      FALSE - The resume command of the Tx queue is failed
**  note:
**__________________________________________________________________________*/
BOOLEAN
dmaResumeTxQueue(IN PMP_ADAPTER adapter_p,
                 IN TX_QUEUE_NUM txQNum)
{
    UINT_32 temp32;
    UINT_32 txQResumeMask;

    DEBUGFUNC("dmaResumeTxQueue");

    DBGLOG(TX, TRACE, ("\n"));

    ASSERT(txQNum <= TXQ_TS3);

    txQResumeMask = 0x00000001 << txQNum;
    DBGLOG(TX, LOUD, ("QRCR command: %x\n", txQResumeMask));

    NIC_MCR_WRITE(adapter_p, MCR_QRCR, txQResumeMask);

    NIC_MCR_READ(adapter_p, MCR_QSR1, &temp32);

    if ((temp32 & (QSR1_TQ0_OPERATING << txQNum)) == 0) {
        //NIC_LAUNCH_DBG_SIG(adapter_p, 0x60);
        ERRORLOG(("Q (%d) is not operating after resume it\n", txQNum));
        //NIC_MCR_WRITE(adapter_p, MCR_TDSR, registerValue & tdmaStatusMask);

        return FALSE;
    }

    return TRUE;
}	/* dmaResumeTxQueue */

#if WORKAROUND_TX_STOP_STATUS
/*_________________________________________________________________________
**  dmaCheckTxQueueStopStatus
**
**  descriptions: This function is called after stopping a TX queue
**                to check the status of arbiter in MAC.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**  return:
**      TRUE - Arbiter has stopped the queue
**      FALSE - Arbiter has not stopped the queue
**  note:
**__________________________________________________________________________*/
BOOLEAN
dmaCheckTxQueueStopStatus(IN PMP_ADAPTER adapter_p)
{
    UINT_32 temp32;
    BOOLEAN status;

    DEBUGFUNC("dmaCheckTxQueueStopStatus");

    DBGLOG(TX, INFO, ("\n"));

    NIC_MCR_WRITE(adapter_p, MCR_PRBCR, 0x30b);

    NIC_MCR_READ_AND_WAIT(adapter_p, MCR_PRBSR, &temp32,
        ((temp32 & TXQ_STOP_STATUS_MASK) == 0), TXQ_STOP_STATUS_WAIT_TIME,
        TXQ_STOP_STATUS_WAIT_COUNT, status);

#if DBG
    NIC_MCR_WRITE(adapter_p, MCR_PRBCR, adapter_p->MACDebugPort);
#endif

    if (!status) {
        ERRORLOG(("Arbiter is not stopped in %d us\n",
            TXQ_STOP_STATUS_WAIT_TIME * TXQ_STOP_STATUS_WAIT_COUNT));
    }

    return  status;
}	/* dmaCheckTxQueueStopStatus */
#endif

/*_________________________________________________________________________
**  dmaStartRxQueue
**
**  descriptions: This routine is called to start the Rx queue.
**  parameters:
**      adapter_p - Pointer to the adapter structure
**      startingAddress - Starting address to begin the Rx queue operation
**  return:
**      TRUE - The Rx queue is started successfully
**      FALSE - The start of the Rx queue is failed
**  note:
**__________________________________________________________________________*/
BOOLEAN
dmaStartRxQueue(IN PMP_ADAPTER adapter_p,
                IN UINT_32 startingAddress)
{
    UINT_32 qCmd;

    DEBUGFUNC("dmaStartRxQueue");

    DBGLOG(RX, TRACE, ("\n"));

    ASSERT(startingAddress);

    if (!startingAddress) {
        ERRORLOG(("Null Rx Queue starting address\n"));
        return FALSE;
    }

    NIC_MCR_READ(adapter_p, MCR_QSR0, &qCmd);
    if (qCmd & QSR0_RX_DATA_Q_OPERATING) {
        DBGLOG(RX, INFO, ("RX Queue in operation\n"));
        return TRUE;
    }

    NIC_MCR_WRITE(adapter_p, MCR_ISAR, ISAR_RXQ_EVENT);

    NIC_UNMASK_INTERRUPT(adapter_p, ISAR_RXQ_EVENT);

    NIC_MCR_WRITE(adapter_p, MCR_RQSAR, startingAddress);

    NIC_MCR_WRITE(adapter_p, MCR_QCR, QCR_RX_DATA_Q_START);

    return TRUE;
}   /* dmaStartRxQueue */

/*_________________________________________________________________________
**  dmaStopRxQueue
**
**  descriptions: This routine is called to stop the Rx queue immediately.
**  parameters:
**      adapter_p - Pointer to the adapter structure
**  return:
**      TRUE - The Rx queue is stopped successfully
**      FALSE - Fail to stop the Rx queue
**  note:
**__________________________________________________________________________*/
BOOLEAN
dmaStopRxQueue(IN PMP_ADAPTER adapter_p)
{
    UINT_32 qStatus;

    DEBUGFUNC("dmaStopRxQueue");

    DBGLOG(RX, TRACE, ("\n"));

    NIC_MCR_WRITE(adapter_p, MCR_QCR, QCR_RX_DATA_Q_STOP);

    NdisStallExecution(5);

    NIC_MCR_WRITE(adapter_p, MCR_ISAR, ISAR_RXQ_EVENT);

    NIC_MASK_INTERRUPT(adapter_p, ISAR_RXQ_EVENT);

    NIC_MCR_READ(adapter_p, MCR_QSR0, &qStatus);

    if (qStatus & QSR0_RX_DATA_Q_OPERATING) {
        ERRORLOG(("Failed, RX Queue won't stop -- QSR0 = 0x%08x\n", qStatus));
        return FALSE;
    }

    return TRUE;
}   /* dmaStopRxQueue */

/*_________________________________________________________________________
**  dmaStartSecQueue
**
**  descriptions: This function is called to start the specific Security
**                queue operation.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**      secQNum - The Security Queue number to be started
**      startingAddress - Starting address to begin the Security queue
**                        operation
**  return:
**      TRUE - Security queue operation is started
**      FALSE - The start of the Security queue operation is failed
**  note:
**__________________________________________________________________________*/
BOOLEAN
dmaStartSecQueue(IN PMP_ADAPTER adapter_p,
                 IN TX_QUEUE_NUM secQNum,
                 IN UINT_32 startingAddress)
{
    UINT_32 secQStartMask;
    UINT_32 sarOffset;
    UINT_32 secQEventMask;

    DEBUGFUNC("dmaStartTxQueue");

    DBGLOG(TX, TRACE, ("\n"));

    ASSERT(secQNum == SECQ_TX || secQNum == SECQ_RX);
    ASSERT(startingAddress);

    switch (secQNum) {
        case SECQ_TX:
             secQStartMask = QCR_SEC_ENGINE_OP_TX_START;
             sarOffset = MCR_SEOTSAR;
             secQEventMask = QSR0_SEC_ENGINE_OP_TX_DONE |
                 QSR0_SEC_ENGINE_OP_TX_Q_EMPTY;
             break;

        case SECQ_RX:
             secQStartMask = QCR_SEC_ENGINE_OP_RX_START;
             sarOffset = MCR_SEORSAR;
             secQEventMask = QSR0_SEC_ENGINE_OP_RX_DONE |
                 QSR0_SEC_ENGINE_OP_RX_Q_EMPTY;
             break;

        default:
             return FALSE;
    }

    DBGLOG(TX, LOUD, ("QCR command: %x\n", secQStartMask));

    NIC_MCR_WRITE(adapter_p, MCR_QSR0, secQEventMask);
    NIC_UNMASK_INTERRUPT(adapter_p, IER_SEC_Q_EVENT_EN);

    NIC_MCR_WRITE(adapter_p, sarOffset, startingAddress);

    NIC_MCR_WRITE(adapter_p, MCR_QCR, secQStartMask);

    return TRUE;
}   /* dmaStartSecQueue */

/*_________________________________________________________________________
**  dmaStopSecQueue
**
**  descriptions: This function is called to stop the specific security
**                queue operation immediately.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**      secQNum - The Security Queue number to be stopped
**  return:
**      TRUE - Tx queue operation is stopped
**      FALSE - The current stop command of the Tx queue is failed
**  note:
**__________________________________________________________________________*/
BOOLEAN
dmaStopSecQueue(IN PMP_ADAPTER adapter_p,
                IN TX_QUEUE_NUM secQNum)
{
    UINT_32 secQStopCommand;
    UINT_32 secQEventMask;
    BOOLEAN status;
    UINT_32 temp32;

    DEBUGFUNC("dmaStopSecQueue");

    DBGLOG(TX, TRACE, ("\n"));

    ASSERT(secQNum == SECQ_RX || secQNum == SECQ_TX);

    switch (secQNum) {
        case SECQ_RX:
             secQStopCommand = QCR_SEC_ENGINE_OP_RX_STOP;
             secQEventMask = QSR0_SEC_ENGINE_OP_RX_DONE |
                 QSR0_SEC_ENGINE_OP_RX_Q_EMPTY;
             break;

        case SECQ_TX:
             secQStopCommand = QCR_SEC_ENGINE_OP_TX_STOP;
             secQEventMask = QSR0_SEC_ENGINE_OP_TX_DONE |
                 QSR0_SEC_ENGINE_OP_TX_Q_EMPTY;
             break;

        default:
             return FALSE;
    }
    DBGLOG(TX, LOUD, ("QCR command: 0x%08x\n", secQStopCommand));

    NIC_MCR_WRITE(adapter_p, MCR_QCR, secQStopCommand);

    NIC_MASK_INTERRUPT(adapter_p, IER_SEC_Q_EVENT_EN);

#if WORKAROUND_STOP_SEC_ARBITER_STATUS
    NIC_MCR_WRITE(adapter_p, MCR_PRBCR, 0x301);

    if (secQNum == SECQ_RX) {
        NIC_MCR_READ_AND_WAIT(adapter_p, MCR_PRBSR, &temp32,
            (((temp32 & ARBITER_RX_SECQ) != ARBITER_RX_SECQ) &&
             ((temp32 & ARBITER_RX_SECQ) != ARBITER_SEC_WR)), ARBITER_STATUS_WAIT_TIME,
            ARBITER_SECQ_STATUS_WAIT_COUNT, status);
    } else {
        NIC_MCR_READ_AND_WAIT(adapter_p, MCR_PRBSR, &temp32,
            (((temp32 & ARBITER_TX_SECQ) != ARBITER_TX_SECQ) &&
             ((temp32 & ARBITER_RX_SECQ) != ARBITER_SEC_WR)), TXQ_STOP_STATUS_WAIT_TIME,
            TXQ_STOP_STATUS_WAIT_COUNT, status);
    }

#if DBG
    NIC_MCR_WRITE(adapter_p, MCR_PRBCR, adapter_p->MACDebugPort);
#endif

    if (!status) {
        ERRORLOG(("Arbiter is not stopped in %d us for SEC Q(%d)\n",
            TXQ_STOP_STATUS_WAIT_TIME * TXQ_STOP_STATUS_WAIT_COUNT, secQNum));
    }
#endif /* WORKAROUND_STOP_SEC_ARBITER_STATUS */

    NIC_MCR_WRITE(adapter_p, MCR_QSR0, secQEventMask);

#if DBG
    NIC_MCR_READ(adapter_p, MCR_ISAR, &temp32);
    DBGLOG(TX, TRACE, ("ISAR after stop Q:%x\n", temp32));
    NIC_MCR_READ(adapter_p, MCR_QSR0, &temp32);
    DBGLOG(TX, TRACE, ("QSR0 after stop Q:%x\n", temp32));
#endif

    return TRUE;
}	/* dmaStopSecQueue */

/*_________________________________________________________________________
**  dmaResumeSecQueue
**
**  descriptions: This function is called to resume the specific Security
**                queue operation immediately.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**      secQNum - The Security Queue number to be resumed
**  return:
**      TRUE - The Security queue operation is resumed successfully
**      FALSE - The resume command of the Security queue is failed
**  note:
**__________________________________________________________________________*/
BOOLEAN
dmaResumeSecQueue(IN PMP_ADAPTER adapter_p,
                  IN TX_QUEUE_NUM secQNum)
{
    UINT_32 secQResumeMask;

    DEBUGFUNC("dmaResumeSecQueue");

    DBGLOG(TX, TRACE, ("\n"));

    ASSERT(secQNum == SECQ_RX || secQNum == SECQ_TX);

    switch (secQNum) {
        case SECQ_RX:
             secQResumeMask = QRCR_SEC_ENGINE_OP_RX_RESUME;
             break;

        case SECQ_TX:
             secQResumeMask = QRCR_SEC_ENGINE_OP_TX_RESUME;
             break;

        default:
             return FALSE;
    }
    DBGLOG(TX, LOUD, ("QRCR command: %x\n", secQResumeMask));

    NIC_MCR_WRITE(adapter_p, MCR_QRCR, secQResumeMask);

    return TRUE;
}	/* dmaResumeSecQueue */

