/*
** $Id: eeprom.h 1.20 2004/10/04 18:10:35 cmchang Exp $
*/

/*************************************************************************
* Copyright (c) 2002-2003 Inprocomm, Inc.
*
* All rights reserved. Copying, compilation, modification, distribution
* or any other use whatsoever of this material is strictly prohibited
* except in accordance with a Software License Agreement with
* Inprocomm, Inc.
**************************************************************************/
/*************************************************************************
GENERAL DESCRIPTION
********************
*/

#ifndef _EEPROM_H
#define _EEPROM_H

/************************************************************************
*                     C O M P I L E R   F L A G S
*************************************************************************
*/

/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/
#define EEPROM_SIGNATURE                0x00
#define EEPROM_PCI_DEVICE_ID            0x01
#define EEPROM_PCI_VENDOR_ID            0x02
#define EEPROM_PCI_CLASS_CODE_BYTE_1_2  0x03
#define EEPROM_PCI_CLASS_CODE_BYTE_0    0x04
#define EEPROM_PCI_SUBSYSTEM_ID         0x05
#define EEPROM_PCI_SUBSYSTEM_VENDOR_ID  0x06
#define EEPROM_PCI_MIN_GNT_MAX_LAT      0x07
#define EEPROM_CARDBUS_CIS_PTR          0x08
#define EEPROM_PCI_POWER_MGT_CTRL       0x09
#define EEPROM_INTERFACE_CFG_CHECKSUM   0x0A
#define EEPROM_LED_CFG_PHY_MODE         0x0B

#define EEPROM_MAC_ADDR_BYTE_0_1        0x10
#define EEPROM_MAC_ADDR_BYTE_2_3        0x11
#define EEPROM_MAC_ADDR_BYTE_4_5        0x12
#define EEPROM_REG_DOMAIN               0x13

#define EEPROM_TXDAC_STATE              0x15
#define EEPROM_TX_PWR_MARK_START        0x16
#define EEPROM_TX_PWR_MARK_END          0x24
#define EEPROM_TX_PWR_2_4G_CCK_0        0x16
#define EEPROM_TX_PWR_2_4G_CCK_5        0x16
#define EEPROM_TX_PWR_2_4G_CCK_10       0x17
#define EEPROM_TX_PWR_2_4G_OFDM_0       0x17
#define EEPROM_TX_PWR_2_4G_OFDM_5       0x18
#define EEPROM_TX_PWR_2_4G_OFDM_10      0x18
#define EEPROM_TX_PWR_4_9G_0            0x19
#define EEPROM_TX_PWR_4_9G_5            0x19
#define EEPROM_TX_PWR_4_9G_10           0x1A
#define EEPROM_TX_PWR_5G_GROUP_1_0      0x1A
#define EEPROM_TX_PWR_5G_GROUP_1_5      0x1B
#define EEPROM_TX_PWR_5G_GROUP_1_10     0x1B
#define EEPROM_TX_PWR_5G_GROUP_2_0      0x1C
#define EEPROM_TX_PWR_5G_GROUP_2_5      0x1C
#define EEPROM_TX_PWR_5G_GROUP_2_10     0x1D
#define EEPROM_TX_PWR_5G_GROUP_3_0      0x1D
#define EEPROM_TX_PWR_5G_GROUP_3_5      0x1E
#define EEPROM_TX_PWR_5G_GROUP_3_10     0x1E
#define EEPROM_TX_PWR_5G_GROUP_4_0      0x1F
#define EEPROM_TX_PWR_5G_GROUP_4_5      0x1F
#define EEPROM_TX_PWR_5G_GROUP_4_10     0x20
#define EEPROM_TX_PWR_5G_GROUP_5_0      0x20
#define EEPROM_TX_PWR_5G_GROUP_5_5      0x21
#define EEPROM_TX_PWR_5G_GROUP_5_10     0x21
#define EEPROM_TX_PWR_5G_GROUP_6_0      0x22
#define EEPROM_TX_PWR_5G_GROUP_6_5      0x22
#define EEPROM_TX_PWR_5G_GROUP_6_10     0x23
#define EEPROM_TX_PWR_MARK_VALID_2_4G   0x23
#define EEPROM_TX_PWR_MARK_VALID_5G     0x24
#define EEPROM_TX_PWR_MARK_CHECKSUM     0x24

#define EEPROM_RF_CAL_TX_START              0x25
#define EEPROM_RF_CAL_TX_END                0x47
#define EEPROM_RF_CAL_TX_MAP                0x48
#define EEPROM_RF_CAL_TX_TABLE_CHECK_SUM    0x49

#define EEPROM_RF_CAL_RX_START              0x4A
#define EEPROM_RF_CAL_RX_END                0x99
#define EEPROM_RF_CAL_RX_MAP                0x9A
#define EEPROM_RF_CAL_RX_TABLE_CHECK_SUM    0x9B

#define EEPROM_2G_CCK_TXPWR_GAIN_START      0xBC
#define EEPROM_2G_CCK_TXPWR_GAIN_END        0xC2
#define EEPROM_2G_OFDM_TXPWR_GAIN_START     0xC3
#define EEPROM_2G_OFDM_TXPWR_GAIN_END       0xC9
#define EEPROM_5G_OFDM_TXPWR_GAIN_START     0xCA
#define EEPROM_5G_OFDM_TXPWR_GAIN_END       0xDA

#define MAX_READ_EEPROM_LENGTH          0x31

#define EEPROM_SIGNATURE_IPN2220        0x2220
#define EEPROM_SIGNATURE_IPN5320        0x5320

#define EEPROM_PCI_MIN_GNT_MASK         BITS(0,7)
#define EEPROM_PCI_MAX_LAT_MASK         BITS(8,15)

#define EEPROM_CARDBUS_CIS_END_PTR      BITS(0,7)
#define EEPROM_CARDBUS_CIS_START_PTR    BITS(8,15)

#define EEPROM_INTERFACE_CFG_MASK       BITS(0,7)
#define EEPROM_CHECKSUM_MASK            BITS(8,15)

#define EEPROM_GPIO_ENABLE_MASK         BIT(0)
#define EEPROM_GPIO_ENABLED             0x0001
#define EEPROM_GPIO_DISABLED            0x0000

#define EEPROM_RX_ANT_PRESENT_MASK      BITS(1,2)
#define EEPROM_RX_ANT_0_PRESENT         BIT(1)
#define EEPROM_RX_ANT_1_PRESENT         BIT(2)

#define EEPROM_TX_ANT_PRESENT_MASK      BITS(3,4)
#define EEPROM_TX_ANT_0_PRESENT         BIT(3)
#define EEPROM_TX_ANT_1_PRESENT         BIT(4)

#define EEPROM_RF_MODE_MASK             BITS(5,7)
#define EEPROM_RF_MODE_MAXIM            0x0000
#define EEPROM_RF_MODE_RFMD             0x0020
#define EEPROM_RF_MODE_RADIA            0x0040

#define EEPROM_LED_MODE_MASK            BITS(0,2)
#define EEPROM_LED_MODE_DEFAULT         0x0000
#define EEPROM_LED_MODE_CUSTOM_1        0x0001
#define EEPROM_LED_MODE_CUSTOM_2        0x0002

#define EEPROM_PHY_MODE_MASK            BITS(8,9)
#define EEPROM_PHY_MODE_G               0x0000
#define EEPROM_PHY_MODE_A_G             0x0100
#define EEPROM_PHY_MODE_A               0x0200

#define EEPROM_TX_PWR_MARK_2_4G_CCK_VALID_MASK      BIT(8)
#define EEPROM_TX_PWR_MARK_2_4G_OFDM_VALID_MASK     BIT(9)

#define EEPROM_TX_PWR_MARK_4_9G_VALID_MASK          BIT(0)
#define EEPROM_TX_PWR_MARK_5G_1_VALID_MASK          BIT(1)
#define EEPROM_TX_PWR_MARK_5G_2_VALID_MASK          BIT(2)
#define EEPROM_TX_PWR_MARK_5G_3_VALID_MASK          BIT(3)
#define EEPROM_TX_PWR_MARK_5G_4_VALID_MASK          BIT(4)
#define EEPROM_TX_PWR_MARK_5G_5_VALID_MASK          BIT(5)
#define EEPROM_TX_PWR_MARK_5G_6_VALID_MASK          BIT(6)

#define EEPROM_RF_CAL_2_4G_GROUP_1_VALID_MASK       BIT(0)
#define EEPROM_RF_CAL_2_4G_GROUP_2_VALID_MASK       BIT(1)
#define EEPROM_RF_CAL_2_4G_GROUP_3_VALID_MASK       BIT(2)
#define EEPROM_RF_CAL_4_9G_GROUP_VALID_MASK         BIT(3)
#define EEPROM_RF_CAL_5G_GROUP_1_VALID_MASK         BIT(4)
#define EEPROM_RF_CAL_5G_GROUP_2_VALID_MASK         BIT(5)
#define EEPROM_RF_CAL_5G_GROUP_3_VALID_MASK         BIT(6)
#define EEPROM_RF_CAL_5G_GROUP_4_VALID_MASK         BIT(7)
#define EEPROM_RF_CAL_5G_GROUP_5_VALID_MASK         BIT(8)
#define EEPROM_RF_CAL_5G_GROUP_6_VALID_MASK         BIT(9)


/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/

/************************************************************************
*                             M A C R O S
*************************************************************************
*/

/************************************************************************
*                    D A T A   D E C L A R A T I O N S
*************************************************************************
*/

/************************************************************************
*                 F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/

#endif /* _EEPROM_H */

