/*
 * The program is refered from iptables.c, iptables-standalone.c and iptabled-restore.c
 * The main purpose is to prevent from calling the iptc_init() and iptc_commit() while every one iptables command is issued.
 * It can reduce the timing a lot.
 *
 * NOTE:
 * It is not allowed to have more than two different cur-processing. It will cause the 2nd iptc_commit() fail.
 *
*/
                                                                                                                                               

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <iptables.h>
#include <getopt.h>
#include <sys/errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <syslog.h>
#include <stdarg.h>
#include <signal.h>
#include <stdarg.h>
#include <ctype.h>
#include <sys/types.h>

#define DPRINTF(args...)

static char *newargv[64]={0};
static int newargc;
static char buffer[256];

/* function adding one argument to newargv, updating newargc
 * returns true if argument added, false otherwise */
static int add_argv(char *what) {
        if (what && ((newargc + 1) < sizeof(newargv)/sizeof(char *))) {
                newargv[newargc] = strdup(what);
                newargc++;
                return 1;
        } else
                return 0;
}
                                                                                                                                               
static void free_argv(void) {
        int i;
                                                                                                                                               
        for (i = 0; i < newargc; i++)
                free(newargv[i]);
}

static char filter_table[] = "filter";
static char nat_table[] = "nat";
static char mangle_table[] = "mangle";

void iptableScriptInstall(char *iptableScriptFile)
{
        char *cur_table;
        iptc_handle_t handle=NULL;
        char /*buffer[256],*/ *buf = NULL;
        int ret;
        FILE *in=NULL;
        char *param_start=NULL, *curchar=NULL;
        int quote_open, table_id;

        memset(buffer,0,sizeof(buffer));
        in = fopen(iptableScriptFile, "r");
        if (!in) {
             fprintf(stderr, "Can't open %s: %s", iptableScriptFile, strerror(errno));
             return;
        }

        DPRINTF("######## start  %s  #########\n",iptableScriptFile);
        while (fgets(buffer, sizeof(buffer), in)) {
                if (buffer[0] == '\n')
                        continue;
                /* skip the first space character first */
                for (buf = buffer; (*buf) && (*buf == ' ' || *buf == '\t' || *buf == '\n'); buf++);
                if (!(*buf) || *buf == '\n')
                        continue; /* empty line */
                                                                                                                                               
                if (*buf == '#')  /* comment; skip */
                        continue;
                                                                                                                                               
                if (strncmp(buf, "iptables ", 9))
                        continue; /* does not start with iptables */
                                                                                                                                              
                // reset the counter
                newargc = quote_open = table_id = 0;
                param_start = buf;
		cur_table = filter_table;
               for (curchar = buf; *curchar; curchar++) {
                        if (*curchar == '"') {
                                /* quote_open cannot be true if there
                                 * was no previous character.  Thus,
                                 * curchar-1 has to be within bounds */
                                if (quote_open &&
                                    *(curchar-1) != '\\') {
                                        quote_open = 0;
                                        *curchar = ' ';
                                } else {
                                        quote_open = 1;
                                        param_start++;
                                }
                        }
                        if (*curchar == ' '
                            || *curchar == '\t'
                            || * curchar == '\n') {
                                char param_buffer[256];
                                int param_len = curchar-param_start;
                                  char valueBuf[60];
                                                                                                                                               
                                if (quote_open)
                                        continue;
                                                                                                                                               
                                if (!param_len) {
                                        /* two spaces? */
                                        param_start++;
                                        continue;
                                }
                                                                                                                                               
                                /* end of one parameter */
                                strncpy(param_buffer, param_start,
                                        param_len);
                                *(param_buffer+param_len) = '\0';
                                param_start += param_len + 1;

                                /* check if table name specified */
                                if (!strncmp(param_buffer, "-t", 3)
                                   || !strncmp(param_buffer, "--table", 8)) {
                                        /* table specified */
                                        table_id = 1;
                                        add_argv(param_buffer);
                                        continue;
                                }
                                                                                                                                               
                                if (table_id==1) {
                                        if (!strcmp(param_buffer, "filter")) {
                                                cur_table = filter_table;
                                        } else if (!strcmp(param_buffer, "nat")) {
                                                cur_table = nat_table;
                                        } else if (!strcmp(param_buffer, "mangle")) {
                                                cur_table = mangle_table;
                                        }
                                        /* reset the id */
                                        table_id = 0;
                                }

                                add_argv(param_buffer);
                        } /* endof of para */
                } /* end of for */

		handle = iptc_init(cur_table);
		iptc_commit(&handle);
                ret = do_command(newargc, newargv, &cur_table, &handle);
                if (!ret){
                        DPRINTF("%s : failed=> %s\n", buffer, iptc_strerror(errno));
                }
		iptc_commit(&handle);
                                                                                                                                               
                free_argv();
        }
        fclose(in);
}

int
iptables_script_main(int argc, char *argv[])
{
        program_name = "iptables_script";
        program_version = IPTABLES_VERSION;
	if (argc != 2) {
                fprintf(stderr,"iptables_script <script files>\n");
                _exit (-1);
        }

#ifdef NO_SHARED_LIBS
        init_extensions();
#endif

	iptableScriptInstall(argv[1]);
	exit(0);
}

extern int iptables_main(int argc, char *argv[]);

int
main(int argc, char *argv[])
{
	char *app_name, *p;

	app_name = ((p = strrchr (argv[0], '/')) ? ++p : argv[0]);
	if (!strcmp(app_name, "iptables_script"))
		iptables_script_main(argc, argv);
	else if (!strcmp(app_name, "iptables"))
		iptables_main(argc, argv);
	else
	     fprintf(stderr, "wrong appication name : %s ..\n", app_name);

	exit(0);
}
