#include <asm/tc3162/tc3162dmtctrl.h>

static int dmt_open( struct inode *inode, struct file *filp );
static int dmt_ioctl( struct inode *inode, struct file *flip,unsigned int command, unsigned long arg );
static void DoGetDmtInfo( unsigned long arg );
static void DoGetDmtFwVersion( unsigned long arg );
static void DoGetDmtOpmode( unsigned long arg );
static void DoGetConnectionInfo( unsigned long arg );
static void DoGetLineDataFADSL( unsigned long arg );
static void DoGetLineDataN( unsigned long arg );

static struct file_operations dmt_fops =
{
    ioctl:      dmt_ioctl,
    open:       dmt_open,
};
static spinlock_t mylock=SPIN_LOCK_UNLOCKED;

static int dmt_open( struct inode *inode, struct file *filp )
{
     return( 0 );
} /* dmt_open */

static int dmt_ioctl( struct inode *inode, struct file *flip,unsigned int command, unsigned long arg )
{
    if (adsl_dev_ops == NULL)
	return -EINVAL;
    switch (command)
    {
	case DMTIOCTL_GET_DMT_INFO:
	    DoGetDmtInfo(arg);
	    break;	
	case DMTIOCTL_GET_DMT_FW_VERSION:
	    DoGetDmtFwVersion(arg);
	    break;
	case DMTIOCTL_GET_LINE_OPINFO:
	    DoGetDmtOpmode(arg);
	    break;
	case DMTIOCTL_GET_CONNECTION_INFO:
	    DoGetConnectionInfo(arg);
	    break;
	case DMTIOCTL_GET_LINE_DATA_F_INFO_ADSL:
	    DoGetLineDataFADSL(arg);
	    break;
	case DMTIOCTL_GET_LINE_DATA_N_INFO:
	    DoGetLineDataN(arg);
	    break;
	default:
	     return -EINVAL;
    }
    return 0;
}

static void DoGetDmtInfo( unsigned long arg )
{
   DMTCTRL_DMT_INFO KArg;

    KArg.dmt_status=0;
    spin_lock(&mylock);
    adsl_dev_ops->query(ADSL_QUERY_STATUS, &(KArg.dmt_status), 0);
    copy_to_user( (void *) arg, &(KArg.dmt_status), sizeof(DMTCTRL_DMT_INFO) );
    spin_unlock(&mylock);
} /* DoGetLineInfo */

static void DoGetDmtFwVersion( unsigned long arg )
{

    DMTCTRL_DMT_VERSION KArg;

    adsl_dev_ops->query(ADSL_QUERY_HW_VER, &(KArg.major), &(KArg.minor));
    adsl_dev_ops->query(ADSL_QUERY_FW_VER, &(KArg.fw_ver_r), &(KArg.fw_ver_p));
    adsl_dev_ops->query(ADSL_QUERY_FW_REV, &(KArg.fw_rev), &(KArg.annex_afe));
    adsl_dev_ops->query(ADSL_QUERY_TDM_FDM_VER, &(KArg.tdm_ver), &(KArg.fdm_ver));
    adsl_dev_ops->query(ADSL_QUERY_BUILD_VERSION, &(KArg.b_ver), 0);
    copy_to_user( (void *) arg, &(KArg), sizeof(DMTCTRL_DMT_VERSION) ); 

} /* DoGetDmtFwVersion */

static void DoGetDmtOpmode( unsigned long arg )
{

    unsigned char KArg=0;
    unsigned char dmt_status2=0;

    spin_lock(&mylock);
    adsl_dev_ops->query(ADSL_QUERY_STATUS, &(dmt_status2), 0);
    spin_unlock(&mylock);

    if(dmt_status2 != ADSL_UP)
       copy_to_user( (void *) arg, &(KArg), sizeof(unsigned char) );
    else
      {
       adsl_dev_ops->query(ADSL_QUERY_MODE, &(KArg), 0);
       copy_to_user( (void *) arg, &(KArg), sizeof(unsigned char) );
      }

} /* DoSetDmtOpmode */
static void DoGetConnectionInfo( unsigned long arg )
{
    DMTCTRL_CONNECTION_INFO KArg;
    unsigned char dmt_status1=0;

    spin_lock(&mylock);
    adsl_dev_ops->query(ADSL_QUERY_STATUS, &(dmt_status1), 0);
    spin_unlock(&mylock);
    memset(&KArg,0,sizeof(DMTCTRL_CONNECTION_INFO));
    if(dmt_status1 != ADSL_UP)
       copy_to_user( (void *) arg, &(KArg), sizeof(DMTCTRL_CONNECTION_INFO) );
    else
      {
        adsl_dev_ops->query(ADSL_QUERY_CH_OP_DATA, &(KArg), 0);
        copy_to_user( (void *) arg, &(KArg), sizeof(DMTCTRL_CONNECTION_INFO) );
      }
} /* DoGetConnectionInfo */

static void DoGetLineDataFADSL( unsigned long arg )
{

    unsigned char dmt_status2=0;
    void *data;

    spin_lock(&mylock);
    adsl_dev_ops->query(ADSL_QUERY_STATUS, &(dmt_status2), 0);
    spin_unlock(&mylock);
    (DMTCTRL_LINE_DATA_F_INFO_ADSL *)data=kmalloc(sizeof(DMTCTRL_LINE_DATA_F_INFO_ADSL), GFP_ATOMIC);
    if(data)
    {
      memset((DMTCTRL_LINE_DATA_F_INFO_ADSL *)data,0,sizeof(DMTCTRL_LINE_DATA_F_INFO_ADSL));

      if(dmt_status2 != ADSL_UP)
        copy_to_user( (void *) arg, (DMTCTRL_LINE_DATA_F_INFO_ADSL *)data, sizeof(DMTCTRL_LINE_DATA_F_INFO_ADSL) );
      else
      {
          adsl_dev_ops->query(ADSL_QUERY_FAR_OP_DATA, (DMTCTRL_LINE_DATA_F_INFO_ADSL *)data, 0);
          copy_to_user( (void *) arg, (DMTCTRL_LINE_DATA_F_INFO_ADSL *)data, sizeof(DMTCTRL_LINE_DATA_F_INFO_ADSL) );
      }
    }
     if(data)
        kfree((DMTCTRL_LINE_DATA_F_INFO_ADSL *)data);
} /* DoGetLineDataF */

static void DoGetLineDataN( unsigned long arg )
{
    DMTCTRL_LINE_DATA_N_INFO KArg;
    unsigned char dmt_status2=0;

    spin_lock(&mylock);
    adsl_dev_ops->query(ADSL_QUERY_STATUS, &(dmt_status2), 0);
    spin_unlock(&mylock);
    memset(&KArg,0,sizeof(DMTCTRL_LINE_DATA_N_INFO));
    if(dmt_status2 != ADSL_UP)
       copy_to_user( (void *) arg, &(KArg), sizeof(DMTCTRL_LINE_DATA_N_INFO) );
    else
      {
       adsl_dev_ops->query(ADSL_QUERY_NEAR_OP_DATA, &(KArg), 0);
       copy_to_user( (void *) arg, &(KArg), sizeof(DMTCTRL_LINE_DATA_N_INFO) );
      }

} /* DoGetLineDataN */
