/*
** $Id: wep_func.c 2.13 2004/10/04 17:21:11 cmchang Exp $
*/

/******************************************************************************
Copyright (c) 2000-2002 Integrated Programmable Communications, Inc.

All rights reserved. Copying, compilation, modification, distribution
or any other use whatsoever of this material is strictly prohibited
except in accordance with a Software License Agreement with
Integrated Programmable Communications, Inc.
******************************************************************************
This header file consists of the definitions for AP / STA wlan-boxes.
******************************************************************************/

/************************************************************************
*                E X T E R N A L   R E F E R E N C E S
*************************************************************************
*/

/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/

#include "precomp.h"

/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/
unsigned char ETH_P_PAE[2]       = { 0x88, 0x8e }; /* Port Access Entity (IEEE 802.1X) */
unsigned char ETH_P_PAE_PA[2]    = { 0x88, 0x7c }; /* Port Access Entity for Pre Auth (IEEE 802.1X) */
unsigned char ETH_P_PAE_START[2] = { 0x01, 0x01 }; /* EAPoL Start Frame (IEEE 802.1X) */
unsigned char ETH_P_PAE_KEY[2]   = { 0x01, 0x03 }; /* EAPoL Key Frame (IEEE 802.1X) */
unsigned char IE_WPA_OUI[4]      = { 0x00, 0x50, 0xf2, 0x01 }; /* WPA OUI */

/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/

/************************************************************************
*                        P U B L I C   D A T A
*************************************************************************
*/

/************************************************************************
*                       P R I V A T E   D A T A
*************************************************************************
*/

/************************************************************************
*                             M A C R O S
*************************************************************************
*/

/************************************************************************
*              F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/

/************************************************************************
*                          F U N C T I O N S
*************************************************************************
*/

UINT8 *__mkWpaIE(i2220mgmt_t *priv_p, UINT8 *cp)
{
    if (priv_p->addIE_len != 0) {
            ipn_memcpy(cp, priv_p->addIE, priv_p->addIE_len);
            cp += priv_p->addIE_len;
    }

    return cp;
}

/*_________________________________________________________________________
* Function   : CheckFor1xFrame
* Input      : frameBody_p - Pointer to the frame body.
* Return     : TRUE, if the frame body consist of IEEE 802.1X frame
*              FALSE, otherwise
* Description: this function decides whether frame body consists of IEEE
*              802.1X frame or not
**__________________________________________________________________________*/
BOOLEAN
CheckFor1xFrame(UINT8 *snapHeader_p, UINT16 bodyLen)
{
        if (bodyLen >= 10 &&
            (!ipn_memcmp(&snapHeader_p[6], ETH_P_PAE, sizeof(ETH_P_PAE)) ||
             !ipn_memcmp(&snapHeader_p[6], ETH_P_PAE_PA, sizeof(ETH_P_PAE_PA)) )
            && !ipn_memcmp(snapHeader_p , rfc1042_header, sizeof(rfc1042_header))) {
                return TRUE;
        }
        else {
                return FALSE;
        }
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
*
* \return
*/
/*----------------------------------------------------------------*/
ULONG getRandomNumber(IN PMP_ADAPTER priv_p)
{
        ULONG           randomNum;

        /* Not really random number, just for IV here */
        randomNum = kal_jiffies + (kal_jiffies << 8) + priv_p->seed;
        priv_p->seed = randomNum ^ (randomNum << 7);

        return randomNum;
}

/*_________________________________________________________________________
**  wlanoidSetAuthenticationMode
**
**  descriptions: This routine is called to set the IEEE 802.11
**                authentication mode to the driver.
**  parameters:
**      priv_p - Pointer to the Adapter structure
**      newAuthMode_p - Pointer to the new authentication mode
**  return:
**      NDIS_STATUS_SUCCESS
**      NDIS_STATUS_INVALID_DATA
**  note:
**__________________________________________________________________________*/
int wlanoidSetAuthenticationMode(i2220mgmt_t *priv_p, param_authMode_t newAuthMode)
{
        KDBG_ENTER;

        switch (newAuthMode) {
        case AuthMode_OPEN:
                mib_import(dot11AuthenticationAlgorithmsTable[0]).
                        dot11AuthenticationAlgorithmsEnable = TRUE;
                mib_import(dot11AuthenticationAlgorithmsTable[1]).
                        dot11AuthenticationAlgorithmsEnable = FALSE;
                break;
        case AuthMode_SWITCH:
                mib_import(dot11AuthenticationAlgorithmsTable[0]).
                        dot11AuthenticationAlgorithmsEnable = TRUE;
                mib_import(dot11AuthenticationAlgorithmsTable[1]).
                        dot11AuthenticationAlgorithmsEnable = TRUE;
                break;
        case AuthMode_SHARED:
                mib_import(dot11AuthenticationAlgorithmsTable[0]).
                        dot11AuthenticationAlgorithmsEnable = FALSE;
                mib_import(dot11AuthenticationAlgorithmsTable[1]).
                        dot11AuthenticationAlgorithmsEnable = TRUE;
                break;

        default:
                return -1;
        }

        /* Save the new authentication mode. */
        priv_p->authMode = newAuthMode;

        KDBG_EXIT;
        return 0;
}   /* wlanoidSetAuthenticationMode */

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
int wlanoidSetEncryptionStatus(IN i2220mgmt_t *priv_p,
                               IN param_cipherMode_t cipherSuite)
{
        priv_p->BCcipherSuite = cipherSuite;
        return 0; /* Success */
}


/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
*
* \return
*/
/*----------------------------------------------------------------*/
int wlanoidSetAddKey(i2220mgmt_t *priv_p, IPN_KEY_STRUC *newKey_p)
{
        IPN_KEY_CFG_ENTRY       ipn2220Key;
        IPN_KEY_STRUC           grpKey;
        ULONG                   index, grpKeyIndex;
        int                     i;

        /* Find if key mapping exists in key table */
        if (privacySeekForEntry(priv_p, newKey_p, &index)) {
                goto L_set_key;
        }

        /* Check new key is a grp key or pairwise key. If address is not
         * checked, we consider it as a group key.
         */
        if (!newKey_p->chkAddr1 && !newKey_p->chkAddr2) {
                /* Group key: just find top index */
                if (!privacySeekForEmptyEntry(priv_p, TRUE, &index)) {
                        return -1; /* Key table is full */
                }
        }
        else {
                /* Pairwise key: swap first grp key and free entry if
                 * free entry is located after first grp key.
                 */
                grpKeyIndex = priv_p->swKeyTableSize;
                /* Look for the first group key */
                grpKey.chkKeyId = TRUE;
                grpKey.chkAddr1 = FALSE;
                grpKey.chkAddr2 = FALSE;
                for (i = 0; i < 4; i++) {
                        grpKey.keyId = (UINT8) i;
                        if (privacySeekForEntry(priv_p, &grpKey, &index)) {
                                grpKeyIndex = index;
                                break;
                        }
                }

                if (!privacySeekForEmptyEntry(priv_p, FALSE, &index)) {
                        return -1; /* Key table is full */
                }

                /* All pairwise keys should be located before group keys */
                if (index > grpKeyIndex) {
                        KDBG_LOG_SDL(KDBG_DEBUG,"swap grp key from %ld to %ld\n",
                                grpKeyIndex, index);

                        priv_p->swKeyTable[index].entryUsed = TRUE;
                        ipn_memcpy(&priv_p->swKeyTable[index].key,
                               &priv_p->swKeyTable[grpKeyIndex].key,
                               sizeof(IPN_KEY_STRUC));

                        /* Add key to H/W key table */
                        ipn2220Key.keyTable = 0; /* Hardware table */
                        ipn2220Key.entryIndex = index;
                        ipn_memcpy(&ipn2220Key.key, &priv_p->swKeyTable[index].key,
                               sizeof(IPN_KEY_STRUC));
                        pauAccessHwKey(priv_p, &ipn2220Key, TRUE);

                        index = grpKeyIndex; /* New key use grpKey position */
                }
        } /* end of pairwise key */

L_set_key:
        /* Update Software key table */
        KDBG_LOG_SDL(KDBG_DEBUG,"Set keyIndex = %ld\n", index);
        priv_p->swKeyTable[index].entryUsed = TRUE;
        ipn_memcpy(&priv_p->swKeyTable[index].key, newKey_p, sizeof(IPN_KEY_STRUC));

        /* Add key to H/W key table */
        ipn2220Key.keyTable = 0;      /* Hardware table */
        ipn2220Key.entryIndex = index;
        ipn_memcpy(&ipn2220Key.key, newKey_p, sizeof(IPN_KEY_STRUC));
        pauAccessHwKey(priv_p, &ipn2220Key, TRUE);

        return 0; /* success */
}   /* wlanoidSetAddKey */

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
*
* \return
*/
/*----------------------------------------------------------------*/
int wlanoidSetClearKey(i2220mgmt_t *priv_p, IPN_KEY_STRUC *key_p)
{
        IPN_KEY_CFG_ENTRY       ipn2220Key;
        ULONG                   index;

        /* Find entry in key table */
        if (privacySeekForEntry(priv_p, key_p, &index)) {
                KDBG_LOG_SDL(KDBG_DEBUG,"Clear keyIndex = %ld\n", index);
                priv_p->swKeyTable[index].entryUsed = FALSE;

                /* Add key to H/W key table */
                ipn2220Key.keyTable = 0;      /* Hardware table */
                ipn2220Key.entryIndex = index;

                ipn2220Key.key.keyId = 0; /* For checking of pauAccessHwKey() */
                ipn2220Key.key.chkKeyId = FALSE;
                ipn2220Key.key.chkAddr1 = FALSE;
                ipn2220Key.key.chkAddr2 = FALSE;

                pauAccessHwKey(priv_p, &ipn2220Key, TRUE);
                return 0; /* Success */
        }

        return -1; /* Not found */
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
*
* \return
*/
/*----------------------------------------------------------------*/
int SetPTK(i2220mgmt_t *priv_p, UINT8 *macAddr, cmd_key_t *key_p)
{
        sta_info_t              *sta_p;
        IPN_KEY_STRUC           k;

        sta_p = task_sta_lookup(priv_p, macAddr);
        if (sta_p == NULL) return FALSE;

        KDBG_LOG_INIT(KDBG_DEBUG,"SetPTK AID [%d], keyLen=%d\n",
                                        sta_p->aid, key_p->keyLen);
        k.chkKeyId = TRUE;
        k.chkAddr2 = TRUE;
        k.chkAddr1 = FALSE;
        k.keyId = key_p->keyId;
        k.cipher = key_p->cipherMode;
        ipn_memcpy(k.addr, macAddr, 6);
        ipn_memcpy(k.keyMaterial, key_p->key, min(16,(int)key_p->keyLen));

        /* Set H/W key table */
        if (wlanoidSetAddKey(priv_p, &k) ) {
                KDBG_LOG_INIT(KDBG_DEBUG,"Key table is full\n");
                return -1; /* Fail */
        }

        /* Update station record */
        ipn_memcpy(sta_p->encryptKey, key_p->key, key_p->keyLen);
        sta_p->cipherSuite = key_p->cipherMode;
        sta_p->keyId = key_p->keyId;
        sta_p->KeyMapOn = TRUE;

        /* Set guard time for tkipMIC when rekeying occurs */
        priv_p->rekey_eol = kal_jiffies + 2*HZ;

        return 0; /* Success */
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
*
* \return
*/
/*----------------------------------------------------------------*/
int SetGTK(i2220mgmt_t *priv_p, cmd_key_t *key_p)
{
        DOT11_WEP_DEFAULT_KEYS_ENTRY    *defKey_p;
        IPN_KEY_STRUC                   k;

        KDBG_LOG_INIT(KDBG_DEBUG,"SetGTK, keyId=%d\n", key_p->keyId);

        k.chkKeyId = TRUE;
        k.chkAddr2 = FALSE;
        k.chkAddr1 = FALSE;
        k.keyId = key_p->keyId;
        k.cipher = key_p->cipherMode;
        ipn_memcpy(k.addr, bcstAddr, 6);
        ipn_memcpy(k.keyMaterial, key_p->key, min(16,(int)key_p->keyLen));

        /* Set H/W key table */
        if (wlanoidSetAddKey(priv_p, &k) != 0) {
                KDBG_LOG_INIT(KDBG_DEBUG, "Key table is full\n");
                return -1; /* Fail */
        }

        /* Store key to default cipher setting, too */
        defKey_p = &mib_import(dot11WEPDefaultKeysTable[key_p->keyId]);
        ipn_memcpy(defKey_p->dot11WEPDefaultKeyValue, key_p->key, key_p->keyLen);

        if (priv_p->BCcipherSuite != key_p->cipherMode) {
                wlanoidSetEncryptionStatus(priv_p, key_p->cipherMode);
        }
        priv_p->DefaultKeyID = key_p->keyId;

        /* Set guard time for tkipMIC when rekeying occurs */
        priv_p->rekey_eol = kal_jiffies + 2*HZ;

        return 0; /* Success */
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
*
* \return
*/
/*----------------------------------------------------------------*/
int ClearPTK(i2220mgmt_t *priv_p, UINT8 *macAddr, UINT8 keyId)
{
        sta_info_t              *sta_p;
        IPN_KEY_STRUC           k;

        sta_p = task_sta_lookup(priv_p, macAddr);
        if (sta_p == NULL) return FALSE;

        k.chkKeyId = TRUE;
        k.chkAddr2 = TRUE;
        k.chkAddr1 = FALSE;
        k.keyId = keyId;
        ipn_memcpy(k.addr, macAddr, 6);

        /* Set H/W key table */
        wlanoidSetClearKey(priv_p, &k);

        /* Update station record */
        sta_p->cipherSuite = CipherMode_NONE;
        sta_p->KeyMapOn = FALSE;

        /* Set guard time for tkipMIC when rekeying occurs */
        priv_p->rekey_eol = kal_jiffies + 2*HZ;

        return 0; /* Success */
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
*
* \return
*/
/*----------------------------------------------------------------*/
int ClearGTK(i2220mgmt_t *priv_p, UINT8 keyId)
{
        IPN_KEY_STRUC   k;

        k.chkKeyId = TRUE;
        k.chkAddr2 = FALSE;
        k.chkAddr1 = FALSE;
        k.keyId = keyId;

        /* Set H/W key table */
        wlanoidSetClearKey(priv_p, &k);

        if (priv_p->DefaultKeyID == keyId) {
                wlanoidSetEncryptionStatus(priv_p, CipherMode_NONE);
        }

        /* Set guard time for tkipMIC when rekeying occurs */
        priv_p->rekey_eol = kal_jiffies + 2*HZ;

        return 0; /* Success */
}


