/*
** $Id: ieee8021x.c 1.24 2004/09/23 19:30:44 cmchang Exp $
*/

/*! \file ieee8021x.c
    \brief IPN2220 AP/STA common supporting subroutines.
*/

/******************************************************************************
Copyright (c) 2002-2003 Inprocomm, Inc.

All rights reserved. Copying, compilation, modification, distribution
or any other use whatsoever of this material is strictly prohibited
except in accordance with a Software License Agreement with Inprocomm, Inc.
******************************************************************************/

/************************************************************************
*                E X T E R N A L   R E F E R E N C E S
*************************************************************************
*/
#include "wlan_os.h"
#include "precomp.h"

/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/

/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/


/************************************************************************
*                        P U B L I C   D A T A
*************************************************************************
*/

/************************************************************************
*                       P R I V A T E   D A T A
*************************************************************************
*/

/************************************************************************
*                             M A C R O S
*************************************************************************
*/

/************************************************************************
*              F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/

/************************************************************************
*                          F U N C T I O N S
*************************************************************************
*/

/*----------------------------------------------------------------*/
/*!
* \brief    A call-back function for IP 1X station to send wireless frame(1X frame or Deauthenticate)
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
int wlan_1x_request(kal_skb_t *skb, kal_net_device_t *dev)
{
        i2220mgmt_t     *priv_p = kal_dev_priv(dev);
        Frame_t         *sdu;
        sta_info_t      *sta_p;

        KDBG_ENTER;

        sdu = (Frame_t*) kal_mempool_alloc(sizeof(Frame_t));
        if (priv_p == NULL || kal_skb_len(skb) < 24 || sdu == NULL) {
                kal_free_skb(skb);
                if (sdu) kal_mempool_free(sdu);
                return 0; /* return 0 although error */
        }

        memcpy(&sdu->frameCtrl, kal_skb_data(skb), min((int)kal_skb_len(skb),30) );
        sdu->skb = skb;
        sdu->mac_ctl = MACCTL_CHECK_CIPHER | MACCTL_CHECK_SLEEP;

        /* Protocol version = 0 */
        sdu->frameCtrl = host2ieee16(ieee2host16(sdu->frameCtrl) & ~BITS(0,1));

        if (sdl_isWDS(sdu) && kal_skb_len(skb) >= 30) {
                /* Addr4 */
                sdu->LLCdata   = (UINT8*)kal_skb_data(skb) + 30; /* points to frame-body */
                sdu->LLClength = kal_skb_len(skb) - 30;
                sdu->mac_ctl  |= MACCTL_WDS_FRAME;
        }
        else {
                sdu->LLCdata   = (UINT8*)kal_skb_data(skb) + 24;
                sdu->LLClength = kal_skb_len(skb)  - 24;
        }

        sta_p = task_sta_lookup(priv_p, sdu->addr1);

        if (sdl_basetype(sdu) == SYN_ft_data && mib_import(dot11Pure1XEnabled)) {
                sdu_send_request(priv_p, sdu, sta_p, SYN_contention);
                goto L_success;
        }
        else if (sdl_ftype(sdu) == SYN_deauth) {
                cmd_deauth_sta(dev, sdu->addr1);
        }
#if SUPPORT_WPA2
        else if (sta_p && sdl_ftype(sdu) == SYN_asoc && mib_import(dot11Pure1XEnabled)) {
                if (*(UINT16*)&sdu->LLCdata[2] != SYN_sts_successful) {
                    cmd_deauth_sta_with_reason(dev, sdu->addr1, SYN_invalid_ie);
                }
        }
#endif

#if SUPPORT_HOSTAPD_ACL
        /* Parsing the auth rsp frame (seqNum=2) from hostapd */
        else if (sta_p && sdl_ftype(sdu) == SYN_auth &&
                 sdu->LLCdata[2] == 2 && sdu->LLCdata[3] == 0) {
                /* Check the response status */
                if (*(UINT16*)&sdu->LLCdata[4] != SYN_sts_successful) {
                        cmd_do_acl(dev, sdu->addr1, FALSE); /* reject */
                }
                else {
                        cmd_do_acl(dev, sdu->addr1, TRUE); /* accept */
                }
        }
#endif
        sdlu_release_sdu(sdu); /* free packet */

L_success:

        KDBG_EXIT;
        return 0; /* success */
}

/*----------------------------------------------------------------*/
/*!
* \param  dev    wlan0 net_device
* \param  staAddr   set the key to
* \param  res       currently, always 1
* \param  RadiusKey the broadcast key
* \param  len       the key length, it's a variable and depend on cipher mode
*/
/*----------------------------------------------------------------*/
void param_set_1x_key(kal_net_device_t *dev, unsigned char* staAddr , BOOLEAN res, UINT8 *RadiusKey, UINT8 len)
{
        i2220mgmt_t *priv_p = kal_dev_priv(dev);
        sta_info_t *sta_p = NULL;
        struct cmd_key_s *key = NULL;
        cmd_key_t       k;

        sta_p = task_sta_lookup(priv_p,staAddr);

        if (!mib_import(dot11Pure1XEnabled))
                return;

    #if SUPPORT_IPNRSN
        if (priv_p->authMode < AuthMode_WPA) {
                if ((len == 5)||((len == 13))){
                        key = &k;
                        memcpy(key->key,RadiusKey,len);
                        key->keyId = 0;
                        key->keyLen = len;
                        key->cipherMode = (len==5) ? 1 : 5;
                        cmd_set_macKey(dev, staAddr, key);
                }
        }

        if (!sta_p)
                return;

        if (priv_p->authMode >= AuthMode_WPA) {
        #if SUPPORT_MSSID
                memcpy(priv_p->ap.ssid_list[sta_p->whichSsid].dot11RSNConfigPSKValue, RadiusKey, len);
        #else
                memcpy(mib_import(dot11RSNConfigTable).dot11RSNConfigPSKValue, RadiusKey, len);
        #endif
                priv_p->GNoStations++;
                //KDBG_LOG_SDL(KDBG_DEBUG, "Get the PSK key from plug_in, authenable %d!!\n",enable);
                sta_p->author_sm->authSuccess = res;
                Author_sm_step(priv_p, sta_p->author_sm);
        } else {
                KDBG_LOG_SDL(KDBG_DEBUG, "rcv auth frame from "MACSTR"\n",MAC2STR(staAddr));
        }
    #else
         //if (staAddr[0]!=0xff || !sta_p)
         //       return;
         key = &k;
         memcpy(key->key, RadiusKey, len);
         key->keyId = res;
         key->keyLen = len;
         if (len==5) key->cipherMode = CipherMode_WEP40;
         else if (len==13) key->cipherMode = CipherMode_WEP104;
         else if (len==16) key->cipherMode = CipherMode_CCMP;
         else if (len==32) key->cipherMode = CipherMode_TKIP;
         else if (len==0) key->cipherMode = CipherMode_NONE;
         cmd_set_macKey(dev, staAddr, key);
         cmd_macPort_ctl(dev, staAddr, FALSE);
    #endif

        return;
}

/*----------------------------------------------------------------*/
/*!
* \brief  The function is use for the 802.1x upper layer Daemon to indicate 1X port open or close,
*
* \param  dev              wlan0 net_device
* \param  staAddr        set the key to
* \param  port_enable  1 if open, 0 if close
*/
/*----------------------------------------------------------------*/
int  param_set_1xport(kal_net_device_t *dev, unsigned char* staAddr, int port_enable)
{
        i2220mgmt_t *priv_p = kal_dev_priv(dev);
        sta_info_t *sta_p;

        sta_p = task_sta_lookup(priv_p,staAddr);

        if (!sta_p)
                return -1;

        if (port_enable){
                if (!mib_import(dot11RSNEnabled))
                        cmd_macPort_ctl(dev, staAddr, FALSE);
        } else {
                cmd_deauth_sta_with_reason(dev,staAddr, SYN_1x_auth_fail);
        }

        return 0;
}
