/*
** $Id: md5.c 1.2 2004/05/03 18:00:08 chyeh Exp $
*/

/*************************************************************************
* Copyright (c) 2002-2003 Inprocomm, Inc.
*
* All rights reserved. Copying, compilation, modification, distribution
* or any other use whatsoever of this material is strictly prohibited
* except in accordance with a Software License Agreement with
* Inprocomm, Inc.
**************************************************************************/
/*************************************************************************
GENERAL DESCRIPTION
********************
This file implement MD5 Message-Digest Algorithm and
Keyed-Hashing for Message Authentication: HMAC-MD5
The MD5 is implemented according to RFC 1321 (MD5) and
RFC2104 (Keyed-Hashing for Message Authentication, HMAC)
*/

/************************************************************************
*                E X T E R N A L   R E F E R E N C E S
*************************************************************************
*/
#include "precomp.h"

/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/

/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/
//typedef struct {
//    ULONG A, B, C, D;
//    ULONG X[16];
//    ULONG AA, BB, CC, DD;
//    ULONG bitCount[2];     /* processed bit count */
//    ULONG unProcByteCount; /* number of un-processed byte count in X[] */
//} MD5_CTX, *PMD5_CTX;

/************************************************************************
*                        P U B L I C   D A T A
*************************************************************************
*/

/************************************************************************
*                       P R I V A T E   D A T A
*************************************************************************
*/
static ULONG T[65] = {0,
    0xd76aa478, 0xe8c7b756, 0x242070db, 0xc1bdceee,
    0xf57c0faf, 0x4787c62a, 0xa8304613, 0xfd469501,
    0x698098d8, 0x8b44f7af, 0xffff5bb1, 0x895cd7be,
    0x6b901122, 0xfd987193, 0xa679438e, 0x49b40821,
    0xf61e2562, 0xc040b340, 0x265e5a51, 0xe9b6c7aa,
    0xd62f105d, 0x02441453, 0xd8a1e681, 0xe7d3fbc8,
    0x21e1cde6, 0xc33707d6, 0xf4d50d87, 0x455a14ed,
    0xa9e3e905, 0xfcefa3f8, 0x676f02d9, 0x8d2a4c8a,
    0xfffa3942, 0x8771f681, 0x6d9d6122, 0xfde5380c,
    0xa4beea44, 0x4bdecfa9, 0xf6bb4b60, 0xbebfbc70,
    0x289b7ec6, 0xeaa127fa, 0xd4ef3085, 0x04881d05,
    0xd9d4d039, 0xe6db99e5, 0x1fa27cf8, 0xc4ac5665,
    0xf4292244, 0x432aff97, 0xab9423a7, 0xfc93a039,
    0x655b59c3, 0x8f0ccc92, 0xffeff47d, 0x85845dd1,
    0x6fa87e4f, 0xfe2ce6e0, 0xa3014314, 0x4e0811a1,
    0xf7537e82, 0xbd3af235, 0x2ad7d2bb, 0xeb86d391
};

/************************************************************************
*                             M A C R O S
*************************************************************************
*/
/* left rotation */
#define LROT(X,s) (((X) << (s)) | ((X) >> (32-(s))))
/* auxiliary function for MD5 */
#define F(X,Y,Z)  (((X)&(Y))|((~X)&(Z)))
#define G(X,Y,Z)  (((X)&(Z))|((Y)&(~Z)))
#define H(X,Y,Z)  ((X)^(Y)^(Z))
#define I(X,Y,Z)  ((Y)^((X)|(~Z)))
/* round function for MD5 */
#define R1(a,b,c,d,k,s,i) { \
    (a) += F((b),(c),(d)) + (k) + T[(i)]; \
    (a) = LROT((a),(s)); \
    (a) += (b); \
}
#define R2(a,b,c,d,k,s,i) { \
    (a) += G((b),(c),(d)) + (k) + T[(i)]; \
    (a) = LROT((a),(s)); \
    (a) += (b); \
}
#define R3(a,b,c,d,k,s,i) { \
    (a) += H((b),(c),(d)) + (k) + T[(i)]; \
    (a) = LROT((a),(s)); \
    (a) += (b); \
}
#define R4(a,b,c,d,k,s,i) { \
    (a) += I((b),(c),(d)) + (k) + T[(i)]; \
    (a) = LROT((a),(s)); \
    (a) += (b); \
}

/************************************************************************
*              F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/

/************************************************************************
*                          F U N C T I O N S
*************************************************************************
*/
/***************************************************************************
* Function   : md5Encode
* Input      : input_p, pointer to 32-bit value
*              outputLen, length of output string, in bytes
* Output     : output_p, pointer to starting address of first 8-bit value
* Return     :
* Description: This function encode single 32-bit value into 4 8-bit values
* Note       :
****************************************************************************/
VOID
md5Encode(IN PULONG input_p,
          OUT PUCHAR output_p,
          IN ULONG outputLen)
{
    ULONG i, j;
    for (i = 0, j = 0; j < outputLen; i++, j += 4){
        output_p[j    ] = (UCHAR)( input_p[i]        & 0x000000ff);
        output_p[j + 1] = (UCHAR)((input_p[i] >>  8) & 0x000000ff);
        output_p[j + 2] = (UCHAR)((input_p[i] >> 16) & 0x000000ff);
        output_p[j + 3] = (UCHAR)(( input_p[i] >> 24)  & 0x000000ff);
    }
}

/***************************************************************************
* Function   : md5Decode
* Input      : input_p, pointer to starting address of first 8-bit value
*              inputLen, length of output string, in bytes
* Output     : output_p, pointer to 32-bit value
* Return     :
* Description: This function decode 4 8-bit values into single 32-bit value
* Note       :
****************************************************************************/
VOID
md5Decode(IN PUCHAR input_p,
          IN ULONG inputLen,
          OUT PULONG output_p)
{
    ULONG i, j;
    for (i = 0, j = 0; j < inputLen; i++, j += 4){
        output_p[i] =  input_p[j  ]        |
                      (input_p[j+1] <<  8) |
                      (input_p[j+2] << 16) |
                      (input_p[j+3] << 24);
    }
}

/***************************************************************************
* Function   : md5Init
* Input      : ctx_p, pointer to MD5 context
* Output     :
* Return     :
* Description: This function initialize MD5 context value
* Note       :
****************************************************************************/
VOID
md5Init(IN PMD5_CTX ctx_p)
{
    ULONG i;

    /* Step 3. Initialize MD Buffer */
    ctx_p->A = 0x67452301;
    ctx_p->B = 0xEFCDAB89;
    ctx_p->C = 0x98BADCFE;
    ctx_p->D = 0x10325476;

    ctx_p->bitCount[0] = 0;
    ctx_p->bitCount[1] = 0;
    ctx_p->unProcByteCount = 0;

    for (i=0; i<16; i++){
        ctx_p->X[i] = 0;
    }
}

/***************************************************************************
* Function   : md5Transform
* Input      : ctx_p, pointer to MD5 context
* Output     :
* Return     :
* Description: This function perform MD5 algorithm with specified MD5 context
* Note       :
****************************************************************************/
VOID
md5Transform(IN PMD5_CTX ctx_p)
{
    ctx_p->AA = ctx_p->A;
    ctx_p->BB = ctx_p->B;
    ctx_p->CC = ctx_p->C;
    ctx_p->DD = ctx_p->D;

    /* Round 1 */
    R1(ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->X[ 0],  7,  1);
    R1(ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->X[ 1], 12,  2);
    R1(ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->X[ 2], 17,  3);
    R1(ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->X[ 3], 22,  4);
    R1(ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->X[ 4],  7,  5);
    R1(ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->X[ 5], 12,  6);
    R1(ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->X[ 6], 17,  7);
    R1(ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->X[ 7], 22,  8);
    R1(ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->X[ 8],  7,  9);
    R1(ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->X[ 9], 12, 10);
    R1(ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->X[10], 17, 11);
    R1(ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->X[11], 22, 12);
    R1(ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->X[12],  7, 13);
    R1(ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->X[13], 12, 14);
    R1(ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->X[14], 17, 15);
    R1(ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->X[15], 22, 16);
    /* Round 2 */
    R2(ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->X[ 1],  5, 17);
    R2(ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->X[ 6],  9, 18);
    R2(ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->X[11], 14, 19);
    R2(ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->X[ 0], 20, 20);
    R2(ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->X[ 5],  5, 21);
    R2(ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->X[10],  9, 22);
    R2(ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->X[15], 14, 23);
    R2(ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->X[ 4], 20, 24);
    R2(ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->X[ 9],  5, 25);
    R2(ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->X[14],  9, 26);
    R2(ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->X[ 3], 14, 27);
    R2(ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->X[ 8], 20, 28);
    R2(ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->X[13],  5, 29);
    R2(ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->X[ 2],  9, 30);
    R2(ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->X[ 7], 14, 31);
    R2(ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->X[12], 20, 32);
    /* Round 3 */
    R3(ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->X[ 5],  4, 33);
    R3(ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->X[ 8], 11, 34);
    R3(ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->X[11], 16, 35);
    R3(ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->X[14], 23, 36);
    R3(ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->X[ 1],  4, 37);
    R3(ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->X[ 4], 11, 38);
    R3(ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->X[ 7], 16, 39);
    R3(ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->X[10], 23, 40);
    R3(ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->X[13],  4, 41);
    R3(ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->X[ 0], 11, 42);
    R3(ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->X[ 3], 16, 43);
    R3(ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->X[ 6], 23, 44);
    R3(ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->X[ 9],  4, 45);
    R3(ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->X[12], 11, 46);
    R3(ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->X[15], 16, 47);
    R3(ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->X[ 2], 23, 48);
    /* Round 4 */
    R4(ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->X[ 0],  6, 49);
    R4(ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->X[ 7], 10, 50);
    R4(ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->X[14], 15, 51);
    R4(ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->X[ 5], 21, 52);
    R4(ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->X[12],  6, 53);
    R4(ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->X[ 3], 10, 54);
    R4(ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->X[10], 15, 55);
    R4(ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->X[ 1], 21, 56);
    R4(ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->X[ 8],  6, 57);
    R4(ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->X[15], 10, 58);
    R4(ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->X[ 6], 15, 59);
    R4(ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->X[13], 21, 60);
    R4(ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->X[ 4],  6, 61);
    R4(ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->C, ctx_p->X[11], 10, 62);
    R4(ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->B, ctx_p->X[ 2], 15, 63);
    R4(ctx_p->B, ctx_p->C, ctx_p->D, ctx_p->A, ctx_p->X[ 9], 21, 64);

    ctx_p->A += ctx_p->AA;
    ctx_p->B += ctx_p->BB;
    ctx_p->C += ctx_p->CC;
    ctx_p->D += ctx_p->DD;
}

/***************************************************************************
* Function   : md5Update
* Input      : ctx_p, pointer to MD5 context
*              input_p, pointer to input data
*              inputLen, length of input data
* Output     :
* Return     :
* Description: This function perform MD5 algorithm with specified MD5 context
*              and first 1 ~ N-1 data block
* Note       : The last data block N is put in X of context
****************************************************************************/
VOID
md5Update(IN PMD5_CTX ctx_p,
          IN PUCHAR input_p,
          IN ULONG inputLen)
{
    ULONG i;
    ULONG remainLen;
    ULONG curInputIdx;

    remainLen = inputLen;
    curInputIdx = 0;

    if (ctx_p->unProcByteCount + inputLen >= 64) {
        while (ctx_p->unProcByteCount + remainLen >= 64) {
            for (i = ctx_p->unProcByteCount; i < 64; i++) {
                #if __CPU_ENDIAN__ != __LITTLE_ENDIAN__
    	        if (i%4 == 0)
	       	        *((PUCHAR)&ctx_p->X[0]+i+3) = input_p[curInputIdx];
                if (i%4 == 1)
	                *((PUCHAR)&ctx_p->X[0]+i+1) = input_p[curInputIdx];
                if (i%4 == 2)
	                *((PUCHAR)&ctx_p->X[0]+i-1) = input_p[curInputIdx];
                if (i%4 == 3)
	                *((PUCHAR)&ctx_p->X[0]+i-3) = input_p[curInputIdx];
	#else
        		*((PUCHAR)&ctx_p->X[0]+i) = input_p[curInputIdx];
	#endif
                curInputIdx++;
            }
            remainLen -= (64 - ctx_p->unProcByteCount);
            md5Transform(ctx_p);
            ctx_p->unProcByteCount = 0;
        }
    }

    if (0xffffffff - ctx_p->bitCount[0] < inputLen * 8) {
        ctx_p->bitCount[1]++;
    }
    ctx_p->bitCount[0] += inputLen * 8;

    /* clear X[] */
    for (i = 0; i < 16; i++) {
        ctx_p->X[i] = 0;
    }

    if (remainLen == 0) {
        ctx_p->unProcByteCount = 0;
    }
    else {
        ctx_p->unProcByteCount = remainLen;
        /* copy remain input data to ctx_p->X[] */
        #if __CPU_ENDIAN__ != __LITTLE_ENDIAN__
        for (i = 0; i < remainLen; i++) {
                if (i%4 == 0)
                        *((PUCHAR)&ctx_p->X[0]+i+3) = input_p[curInputIdx+i];
                if (i%4 == 1)
                        *((PUCHAR)&ctx_p->X[0]+i+1) = input_p[curInputIdx+i];
                if (i%4 == 2)
                        *((PUCHAR)&ctx_p->X[0]+i-1) = input_p[curInputIdx+i];
                if (i%4 == 3)
                        *((PUCHAR)&ctx_p->X[0]+i-3) = input_p[curInputIdx+i];
                }
        #else
        for (i = 0; i < remainLen; i++) {
                *((PUCHAR)&ctx_p->X[0] + i) = input_p[curInputIdx + i];
        }
        #endif
    }
}

/***************************************************************************
* Function   : md5Final
* Input      : ctx_p, pointer to MD5 context
* Output     : digest_p, pointer to the address to put the digest
* Return     :
* Description: This function perform MD5 algorithm with specified MD5 context
*              and last data block N. Finally, this function put the digest
*              into specified digest_p.
* Note       :
****************************************************************************/
VOID
md5Final(IN PMD5_CTX ctx_p,
         OUT PUCHAR digest_p)
{
    ULONG i;
    /* Process last block:
       1. padding 1
       2. padding 0's
       3. padding input data length, in bits
       4. MD5 final transform
     */

    /* 1. padding 1 */
    #if __CPU_ENDIAN__ != __LITTLE_ENDIAN__
    if (ctx_p->unProcByteCount%4 == 0)
       	    *((PUCHAR)&ctx_p->X[0]+ctx_p->unProcByteCount+3) = 0x80;
    if (ctx_p->unProcByteCount%4 == 1)
            *((PUCHAR)&ctx_p->X[0]+ctx_p->unProcByteCount+1) = 0x80;
    if (ctx_p->unProcByteCount%4 == 2)
            *((PUCHAR)&ctx_p->X[0]+ctx_p->unProcByteCount-1) = 0x80;
    if (ctx_p->unProcByteCount%4 == 3)
            *((PUCHAR)&ctx_p->X[0]+ctx_p->unProcByteCount-3) = 0x80;
#else
    *((PUCHAR)&ctx_p->X[0] + ctx_p->unProcByteCount) = 0x80;
#endif

    /* 2. padding 0's. Note: we have pdded 0's in ctx.X[] in md1Update() */
    /* check if remainder space is enough for length field: 8-byte */
    if (64 - ctx_p->unProcByteCount <= 8) {
        md5Transform(ctx_p);

        /* need one more transform, so clear X[] */
        for (i = 0; i < 16; i++){
            ctx_p->X[i] = 0;
        }
    }

    /* 3. padding input data length, in bits */
    ctx_p->X[14] = ctx_p->bitCount[0];
    ctx_p->X[15] = ctx_p->bitCount[1];

    /* 4. MD5 final transform */
    md5Transform(ctx_p);
    md5Encode(&ctx_p->A, digest_p,      4);
    md5Encode(&ctx_p->B, digest_p +  4, 4);
    md5Encode(&ctx_p->C, digest_p +  8, 4);
    md5Encode(&ctx_p->D, digest_p + 12, 4);
}

/***************************************************************************
* Function   : hmacMd5
* Input      : data_p, pointer to input data
*              dataLen, length of input data, in bytes
*              key_p, pointer to key
*              keyLen, length of key, in bytes
* Output     : digest_p, pointer to the address to put the digest
* Return     :
* Description: This function perform Keyed-Hashing for Message
*              Authentication (HMAC) using MD5 algorithm. Finally, this
*              function put the digest into specified digest_p.
* Note       :
****************************************************************************/
VOID
hmacMd5(IN PUCHAR data_p,
        IN ULONG dataLen,
        IN PUCHAR key_p,
        IN ULONG keyLen,
        OUT PUCHAR digest_p)
{
    MD5_CTX context;
    UCHAR k_ipad[65];
    UCHAR k_opad[65];
    ULONG i;

    if (keyLen > 64) {
        MD5_CTX tctx;
        md5Init(&tctx);
        md5Update(&tctx, key_p, keyLen);
        md5Final(&tctx, key_p);

        keyLen = 16;
    }

    for (i = 0; i < 65; i++) {
        k_ipad[i] = 0;
        k_opad[i] = 0;
    }

    for (i = 0; i < keyLen; i++) {
        k_ipad[i] = key_p[i];
        k_opad[i] = key_p[i];
    }

    for (i = 0; i < 64; i++) {
        k_ipad[i] ^= 0x36;
        k_opad[i] ^= 0x5c;
    }

    md5Init(&context);
    md5Update(&context, k_ipad, 64);
    md5Update(&context, data_p, dataLen);
    md5Final(&context, digest_p);

    md5Init(&context);
    md5Update(&context, k_opad, 64);
    md5Update(&context, digest_p, 16);
    md5Final(&context, digest_p);
}
