/*
** $Id: rc4.c 1.1 2004/02/28 22:26:35 whsu Exp $
*/

/******************************************************************************
Copyright (c) 2002-2003 Inprocomm, Inc.

All rights reserved. Copying, compilation, modification, distribution
or any other use whatsoever of this material is strictly prohibited
except in accordance with a Software License Agreement with
Inprocomm, Inc.
******************************************************************************/
/*************************************************************************
GENERAL DESCRIPTION
********************
The RC4 is implemented according to ANSI/IEEE Std 802.11, 1999 edition,
Clause 8.
*/

/************************************************************************
*                E X T E R N A L   R E F E R E N C E S
*************************************************************************
*/
#include "precomp.h"

/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/

/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/

/************************************************************************
*                        P U B L I C   D A T A
*************************************************************************
*/

/************************************************************************
*                       P R I V A T E   D A T A
*************************************************************************
*/

/************************************************************************
*                             M A C R O S
*************************************************************************
*/
#define RC4_BYTE_SWAP(x, y) \
{ \
    UCHAR temp8; \
    temp8 = (x); \
    (x) = (y); \
    (y) = temp8; \
}

/************************************************************************
*              F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/

/************************************************************************
*                          F U N C T I O N S
*************************************************************************
*/
/*_________________________________________________________________________
** rc4SwGenKeySeq
** description: this function generate RC4 key sequence based on RSA RC4
** parameters : seed_p, pointer to RC4 PRNG seed
**              seedLen, length of seed, the unit is byte
**              discardLen, length of generated RC4 key sequence to be
**                          start from first octet
**              keySeqLen, length of result key sequence, the unit is byte
**              keySeq_p, pointer to result key sequence
** Return     : FALSE, fail to generate RC4 key sequence.
**
**              TRUE, success to generate RC4 key sequence
** note       : Seed length should range between 1 and 256 byte(s)
**_______________________________________________________________________*/
BOOLEAN
rc4SwGenKeySeq(IN  PUCHAR seed_p,
               IN  ULONG  seedLen,
               IN  ULONG  discardLen,
               IN  ULONG  keySeqLen,
               OUT PUCHAR keySeq_p)
{
    INT i, j, k;
    UCHAR SBOX[256];   /* state array */
    UCHAR KEYBOX[256]; /* temp secret key sequence */

    /*  the maximum seed length is 256 bytes */
    if (seedLen > 256) {
        return FALSE;
    }

    /* initialize SBOX and KEYBOX */
    for (i = 0; i < 256; i++) {
        SBOX[i] = (UCHAR)i;
        KEYBOX[i] = seed_p[i % seedLen];
    }
    for (i = 0, j = 0; i < 256; i++) {
        j = ( j + SBOX[i] + KEYBOX[i] ) % 256;
        RC4_BYTE_SWAP(SBOX[i], SBOX[j]);
    }

    /* Generate key sequence */
    i = 0; j = 0; /* set initial value to zero */
    /* Discard first discardLen key stream */
    for (k = 0; k < (INT)discardLen; k++) {
        i = (i + 1) % 256;
        j = (j + (INT)SBOX[i]) % 256;
        RC4_BYTE_SWAP(SBOX[i], SBOX[j]);
    }
    /* Generate key stream for cipher encryption/decryption */
    for (k = 0; k < (INT)keySeqLen; k++) {
        i = (i + 1) % 256;
        j = (j + (INT)SBOX[i]) % 256;
        RC4_BYTE_SWAP(SBOX[i], SBOX[j]);
        keySeq_p[k] = SBOX[(SBOX[i] + SBOX[j]) % 256];
    }

    return TRUE;
}
