/*
** $Id: ed.c 1.7 2004/08/23 10:37:07 patlu Exp $
*/

/*************************************************************************
* Copyright (c) 2004 Inprocomm, Inc.
*
* All rights reserved. Copying, compilation, modification, distribution
* or any other use whatsoever of this material is strictly prohibited
* except in accordance with a Software License Agreement with
* Inprocomm, Inc.
**************************************************************************/
/*************************************************************************
GENERAL DESCRIPTION
********************
*/

/************************************************************************
*                E X T E R N A L   R E F E R E N C E S
*************************************************************************
*/
#include "precomp.h"


#if SUPPORT_ED_ADJUST

/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/

/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/

/************************************************************************
*                        P U B L I C   D A T A
*************************************************************************
*/

/************************************************************************
*                       P R I V A T E   D A T A
*************************************************************************
*/
DEBUGFILE("ed.c");


/************************************************************************
*                             M A C R O S
*************************************************************************
*/

/************************************************************************
*              F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/

/************************************************************************
*                          F U N C T I O N S
*************************************************************************
*/

#if SUPPORT_ED_ADJUST_MULTI_LEVEL_FLOAT_STATE
/*_________________________________________________________________________
**  edSetNoiseFloorFloatMargin
**
**  descriptions: This function will set the ED/CCK/OFDM margins in float
**                ED.
**
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**      channelNum - The current channel number
**      channelBand - The current channel band
**  return:       (none)
**  note:
**__________________________________________________________________________*/
VOID
edSetNoiseFloorFloatMargin(IN PMP_ADAPTER adapter_p)
{
    UINT_8 marginAGCED;
    UINT_8 marginCCK;
    UINT_8 marginOFDM;

    DEBUGFUNC("edSetNoiseFloorFloatMargin");

    DBGLOG(HW, INFO,("\n"));

    if (adapter_p->ED_floatLevel >= ED_FLOAT_ED_LEVELS) {
        DBGLOG(HW, INFO, ("Invalid floatLevel! Correct to 0\n"));
        adapter_p->ED_floatLevel = 0;
    }

    if (adapter_p->rfSpecificConfig_p == NULL) {
        ERRORLOG(("Unsupported RF Type\n"));
        return;
    }


    ASSERT(adapter_p->rfSpecificConfig_p->edGetMarginsHandler);

    if (!adapter_p->rfSpecificConfig_p->edGetMarginsHandler(adapter_p,
        (UINT) adapter_p->ED_floatLevel, &marginAGCED, &marginCCK,
        &marginOFDM)) {
        WARNLOG(("Failed to get floated ED margins\n"));
        return;
    }

    DBGLOG(HW, INFO, ("ED_floatLevel=%d, margin = 0x%02x 0x%02x 0x%02x\n",
        adapter_p->ED_floatLevel, marginAGCED, marginCCK, marginOFDM));

    do {
        if (!phyBBWriteAGCTable(adapter_p, 75, marginAGCED)) {
            ERRORLOG(("Float ED: AGC ED margin update failed\n"));
            break;
        }

        if (!phyBBWriteAGCTable(adapter_p, 120, marginCCK)) {
            ERRORLOG(("Float ED: CCK margin update failed\n"));
            break;
        }

        if (!phyBBWriteAGCTable(adapter_p, 121, marginOFDM)) {
            ERRORLOG(("Float ED: OFDM margin update failed\n"));
            break;
        }
    } while (FALSE);
}   /* edSetNoiseFloorFloatMargin */
#endif

#if SUPPORT_ED_ADJUST_BOUND_BY_RSSI
/*_________________________________________________________________________
**  edCalculateEDfromRssi
**
**  descriptions:   This function will calculate the ed, corresponding to
**                  (currentRssi - margin) .
**
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**      margin_dB
**  return:
**      ed
**      0 - when unable to calculate
**  note:
**__________________________________________________________________________*/
UINT_16
edCalculateEDfromRssi(IN PMP_ADAPTER adapter_p,
                      INT_32 margin_dB)
{
    const UINT_32 dB_convertTable[6][2] = {
        // 1.0000    1.1220    1.2589    1.4125    1.5849    1.7783
        {1,     1   },
        {112,   100 },
        {126,   100 },
        {141,   100 },
        {158,   100 },
        {178,   100 },
    };
    INT_32 rssiDiff;
    UINT_32 q, r;
    UINT_32 ed;

    DEBUGFUNC("edCalculateEDfromRssi");

    DBGLOG(HW, INFO,("\n"));

    if (adapter_p->currentRssi <= ED_RSSI_VALID_LOWER_LIMIT) {
        return 0;
    }

    if (adapter_p->rfSpecificConfig_p == NULL) {
        ERRORLOG(("Unsupported RF Type\n"));
        return 0;
    }

#if SUPPORT_802_11A
    if (adapter_p->currentPhyType == PHY_TYPE_OFDM) {
        if (adapter_p->rfSpecificConfig_p->rssiAtNoiseFloorBaseline5 == 0) {
            DBGLOG(HW, INFO, ("Function not supported for %s RF (5 GHz)\n",
                adapter_p->rfSpecificConfig_p->rfName_p));
            return 0;
        }

        rssiDiff = adapter_p->currentRssi - margin_dB -
            adapter_p->rfSpecificConfig_p->rssiAtNoiseFloorBaseline5;
    } else
#endif
    {
        if (adapter_p->rfSpecificConfig_p->rssiAtNoiseFloorBaseline24 == 0) {
            DBGLOG(HW, INFO, ("Function not supported for %s RF (2.4 GHz)\n",
                adapter_p->rfSpecificConfig_p->rfName_p));
            return 0;
        }

        rssiDiff = adapter_p->currentRssi - margin_dB -
            adapter_p->rfSpecificConfig_p->rssiAtNoiseFloorBaseline24;
    }


    if (rssiDiff < 0) {
        rssiDiff = -rssiDiff;
        q = rssiDiff / 6;
        r = rssiDiff % 6;

        ed = ED_NOISE_FLOOR_BASELINE * dB_convertTable[r][1] /
            dB_convertTable[r][0] / (1 << q);

        DBGLOG(HW, INFO,
            ("currentRssi = %d, margin =%d, rssiDiff = %d, ed = %d\n",
            adapter_p->currentRssi, margin_dB, -rssiDiff, ed));
    } else {
        q = rssiDiff / 6;
        r = rssiDiff % 6;

        ed = ED_NOISE_FLOOR_BASELINE * dB_convertTable[r][0] /
            dB_convertTable[r][1] * (1 << q);

        DBGLOG(HW, INFO,
            ("currentRssi = %d, margin =%d, rssiDiff = %d, ed = %d\n",
            adapter_p->currentRssi, margin_dB, rssiDiff, ed));
    }

    if ( ed > ED_THRESHOLD_MAX ) {
        return ED_THRESHOLD_MAX;
    } else {
        return (UINT_16) ed;
    }
}   /* edCalculateEDfromRssi */
#endif

/*_________________________________________________________________________
**  edSetNoiseFloorFixed
**
**  descriptions:   This function is used to set BB to use Fix Noise Floor
**                  mechanism.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**  return:       (none)
**  note:
**__________________________________________________________________________*/
VOID
edSetNoiseFloorFixed(IN PMP_ADAPTER adapter_p)
{
    DEBUGFUNC("edSetNoiseFloorFixed");

    DBGLOG(HW, INFO,("\n"));

    phyBBWriteAGCTable(adapter_p, 0,
        (UINT_8) (adapter_p->initAGC_OPMODE | (UINT_8) BITS(2, 4)));

    adapter_p->hysteresisCount = 0;
    adapter_p->state_noise = NOISE_STATE_HIGH_START;
}   /* edSetNoiseFloorFixed */

/*_________________________________________________________________________
**  edSetNoiseFloorFloat
**
**  descriptions: This function will set BB to use fixed noise floor.
**
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**  return:       (none)
**  note:
**__________________________________________________________________________*/
VOID
edSetNoiseFloorFloat(IN PMP_ADAPTER adapter_p)
{
#if DBG
    UINT_8 bciData;
#endif

    DEBUGFUNC("edSetNoiseFloorFloat");
    DBGLOG(HW, INFO,("\n"));

    phyBBWriteAGCTable(adapter_p, 0,
        (UINT_8) (adapter_p->initAGC_OPMODE & ~((UINT_8) BITS(2, 4))));

#if DBG
    BCIRead8(adapter_p, BBCR_CR45, &bciData, TRUE, 0xff);
    DBGLOG(HW, INFO, ("BBCR 45: 0x%x\n", bciData));

    BCIRead8(adapter_p, BBCR_CR46, &bciData, TRUE, 0xff);
    DBGLOG(HW, INFO, ("BBCR 46: 0x%x\n", bciData));
#endif

    adapter_p->state_noise = NOISE_STATE_LOW;

#if SUPPORT_ED_ADJUST_MULTI_LEVEL_FLOAT_STATE
    adapter_p->ED_floatLevel = 0;
    edSetNoiseFloorFloatMargin(adapter_p);
#endif
}   /* edSetNoiseFloorFloat */

/*_________________________________________________________________________
**  edStepEDThreshold
**
**  descriptions: This function is used to update ED Threshold by step.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**      step - indicate if step up or down
**  return:       new ED threshold
**  note:
**__________________________________________________________________________*/
UINT_16
edStepEDThreshold(IN PMP_ADAPTER adapter_p,
                  IN INT step)
{
    UINT_16 ed;

    DEBUGFUNC("edStepEDThreshold");

    DBGLOG(HW, INFO,("\n"));

    ed = adapter_p->cur_EDThreshold;

    ASSERT((ed >= ED_THRESHOLD_MIN) && (ed <= ED_THRESHOLD_MAX));

    if (step > 0) {
        ed += ed / 10;
        if (ed > ED_THRESHOLD_MAX) {
            ed = ED_THRESHOLD_MAX;
        }
    } else {
        ed -= ed / 11;
        if (ed < ED_THRESHOLD_MIN) {
            ed = ED_THRESHOLD_MIN;
        }
    }

    do {
        if (!phyBBWriteAGCTable(adapter_p, 0x50, (UINT_8) ed)) {
            ERRORLOG(("ED Threshold update failed (1)\n"));
            break;
        }

        if (!phyBBWriteAGCTable(adapter_p, 0x4f, (UINT_8) (ed >> 8))) {
            ERRORLOG(("ED Threshold update failed (2)\n"));
            break;
        }
    } while (FALSE);

    adapter_p->cur_EDThreshold = ed;
    DBGLOG(HW, INFO, ("New EDThreshold=%d\n", ed));

    return ed;
}   /* edStepEDThreshold */

/*_________________________________________________________________________
**  edUpdateNoiseFloor
**
**  descriptions: This function is called to checking the ED state and
**                update it.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**      updateInterval - time unit is Kus (TU)
**      macSuccessCount - successful received packet count till calling the
**                        fucntion.
**      edMode
**  return:       (none)
**  note:
**__________________________________________________________________________*/
VOID
edUpdateNoiseFloor(IN PMP_ADAPTER adapter_p,
                   IN UINT_32 updateInterval,
                   IN UINT_32 macSuccessCount,
                   IN ED_ADJUST_MODE edMode)
{
    UINT_32 rxMdRdyCount;
    UINT_32 osdPassCount, sq1PassCount, edPassCount, sfdPassCount;
    INT fa_count_detect, fa_count_ed, fa_count_detect_ofdm, fa_count_detect_cck;

    UINT_8 forceDown_flag;
    UINT_16 edRssi, edFastRssi;
    UINT_8 edFast_flag;
    UINT_8 rssiValid, edRssiValid;
    UINT_16 tmp16;
    UINT_32 tmp32;

    DEBUGFUNC("edUpdateNoiseFloor");

    DBGLOG(HW, INFO,("\n"));

    BCIWrite8(adapter_p, BBCR_CR97, 0, TRUE, BITS(6,7));

    NIC_MCR_READ(adapter_p, MCR_RPCR, &rxMdRdyCount);
    adapter_p->hwStatistics.rxPacketCount += (UINT_64) rxMdRdyCount;

    if (!bciRead16(adapter_p, BBCR_CR100, BBCR_CR101, &tmp16, TRUE, 0xffff)) {
        ERRORLOG(("OSD Pass counter R failed\n"));
        return;
    }

    osdPassCount = (UINT_32) tmp16;

    DBGLOG(HW, INFO, ("BB OSD Pass Counter = %d\n", osdPassCount));

    if (!bciRead16(adapter_p, BBCR_CR104, BBCR_CR105, &tmp16, TRUE, 0xffff)) {
        ERRORLOG(("SQ1 Pass counter R failed\n"));
        return;
    }

    sq1PassCount = (UINT_32) tmp16;

    DBGLOG(HW, INFO, ("BB SQ1 Pass Counter = %d\n", sq1PassCount));

    if (!bciRead16(adapter_p, BBCR_CR110, BBCR_CR111, &tmp16, TRUE, 0xffff)) {
        ERRORLOG(("SFD Pass counter R failed\n"));
        return;
    }

    sfdPassCount = (UINT_32) tmp16;

    DBGLOG(HW, INFO, ("BB SFD Pass Counter = %d\n", sfdPassCount));

    if (!bciRead16(adapter_p, BBCR_CR98, BBCR_CR99, &tmp16, TRUE, 0xffff)) {
        ERRORLOG(("ED Pass counter R failed\n"));
        return;
    }

    edPassCount = (UINT_32) tmp16;

    DBGLOG(HW, INFO, ("BB ED Pass Counter = %d\n", edPassCount));

#if DBG
    if (!bciRead16(adapter_p, BBCR_CR108, BBCR_CR109, &tmp16, TRUE, 0xffff)) {
        ERRORLOG(("CRC16 Pass counter R failed\n"));
        return;
    }

    DBGLOG(HW, INFO, ("BB CRC16 Pass Counter = %d\n", tmp16));
#endif

    if (!BCIWrite8(adapter_p, BBCR_CR97, BIT(6), TRUE, BITS(6,7))) {
        ERRORLOG(("Counter reset failed.\n"));
    }

    NIC_MCR_READ(adapter_p, MCR_RPCR, &tmp32);
    adapter_p->hwStatistics.rxPacketCount += (UINT_64) tmp32;

    if (!BCIWrite8(adapter_p, BBCR_CR97, BIT(7), TRUE, BITS(6,7))) {
        ERRORLOG(("Counter enable failed.\n"));
    }


    fa_count_detect = osdPassCount + sq1PassCount - rxMdRdyCount;

    if (fa_count_detect >= 0) {
        fa_count_detect = fa_count_detect * 100 / updateInterval;
    } else {
        DBGLOG(HW, INFO,
            ("Detection False Alarm is minus. OSD Pass: %d, SQ1 Pass: %d, MDRDY: %d\n",
            osdPassCount, sq1PassCount, rxMdRdyCount));
        fa_count_detect = 0;
    }
    DBGLOG(HW, INFO, ("Detection False Alarm: %d\n", fa_count_detect));

    fa_count_detect_ofdm = osdPassCount + sfdPassCount - rxMdRdyCount;

    if (fa_count_detect_ofdm >= 0) {
        fa_count_detect_ofdm = fa_count_detect_ofdm * 100 / updateInterval;
    } else {
        DBGLOG(HW, INFO,
            ("OFDM Detection False Alarm is minus. OSD Pass: %d, SFD Pass: %d, MDRDY: %d\n",
            osdPassCount, sfdPassCount, rxMdRdyCount));
        fa_count_detect_ofdm = 0;
    }
    DBGLOG(HW, INFO, ("OFDM Detection False Alarm: %d\n",
        fa_count_detect_ofdm));

    fa_count_detect_cck = sq1PassCount - sfdPassCount;

    if (fa_count_detect_cck >= 0) {
        fa_count_detect_cck = fa_count_detect_cck * 100 / updateInterval;
    } else {
        DBGLOG(HW, INFO,
            ("CCK Detection False Alarm is minus. SQ1 Pass: %d, SFD Pass: %d\n",
            sq1PassCount, sfdPassCount));
        fa_count_detect_cck = 0;
    }
    DBGLOG(HW, INFO, ("CCK Detection False Alarm: %d\n", fa_count_detect_cck));

    fa_count_ed = edPassCount - rxMdRdyCount;

    if (fa_count_ed >= 0) {
        fa_count_ed = fa_count_ed * 100 / updateInterval;
    } else {
        DBGLOG(HW, INFO, ("ED False Alarm is minus. ED Pass: %d, MDRDY: %d\n",
            edPassCount, rxMdRdyCount));
        fa_count_ed = 0;
    }
    DBGLOG(HW, INFO, ("ED False Alarm: %d\n", fa_count_ed));

    macSuccessCount = macSuccessCount * 100 / updateInterval;
    DBGLOG(HW, INFO, ("MAC Success Count: %d\n", macSuccessCount));

    DBGLOG(HW, INFO, ("Current rssi: %d\n", adapter_p->currentRssi));


    switch (edMode) {
        case ED_NORMAL:
            rssiValid = edRssiValid = 0;
            edRssi = edFastRssi = 0;
            edFast_flag = 0;

#if SUPPORT_ED_ADJUST_BOUND_BY_RSSI
            if (adapter_p->currentRssi > ED_RSSI_VALID_LOWER_LIMIT) {
                rssiValid = 1;
                edRssi = edCalculateEDfromRssi(adapter_p, ED_RSSI_MARGIN_DB);

                if (edRssi) {
                    edRssiValid = 1;

#if SUPPORT_ED_ADJUST_FAST_FIND
                    edFastRssi = edCalculateEDfromRssi(adapter_p,
                        ED_FAST_FIND_RSSI_MARGIN_DB);
                    if (edFastRssi > ED_FAST_FIND_ED_THRESHOLD_MAX) {
                        edFastRssi = ED_FAST_FIND_ED_THRESHOLD_MAX;
                    }

                    edFast_flag =
                    ( fa_count_detect_ofdm > ED_FAST_FIND_THRESHOLD_RATIO*OFDM_DETECTION_FA_THRESHOLD
                    || fa_count_detect_cck > ED_FAST_FIND_THRESHOLD_RATIO*CCK_DETECTION_FA_THRESHOLD
                    || fa_count_ed > ED_FAST_FIND_THRESHOLD_RATIO*ED_FA_THRESHOLD ) ? 1 : 0;

#endif /* SUPPORT_ED_ADJUST_FAST_FIND */
                }
            }
#endif /* SUPPORT_ED_ADJUST_BOUND_BY_RSSI */


            DBGLOG(HW, INFO, ("rssiValid: %d, edRssiValid: %d, edFast_flag: %d\n",
                rssiValid, edRssiValid, edFast_flag));
            DBGLOG(HW, INFO, ("edRssi: %d, edFastRssi: %d\n",
                edRssi, edFastRssi));

            forceDown_flag = adapter_p->ED_ThresholdForceDown_flag;
            adapter_p->ED_ThresholdForceDown_flag = 0;

            switch(adapter_p->state_noise) {
                case NOISE_STATE_LOW:

#if SUPPORT_ED_ADJUST_MULTI_LEVEL_FLOAT_STATE
                    DBGLOG(HW, INFO, ("Float noise floor, current level = %d\n",
                            adapter_p->ED_floatLevel));
#endif

                    if (/*fa_count_detect > OSD_SQ_FA_THRESHOLD ||*/
                        fa_count_detect_ofdm > OFDM_DETECTION_FA_THRESHOLD ||
                        fa_count_detect_cck > CCK_DETECTION_FA_THRESHOLD ||
                        fa_count_ed > ED_FA_THRESHOLD) {


#if SUPPORT_ED_ADJUST_MULTI_LEVEL_FLOAT_STATE
                        if (adapter_p->ED_floatLevel < ED_FLOAT_ED_LEVELS - 1) {
                            (adapter_p->ED_floatLevel)++;
                            edSetNoiseFloorFloatMargin(adapter_p);
                            DBGLOG(HW, INFO, ("Float noise floor, level up to %d\n", adapter_p->ED_floatLevel));
                        }
                        else {

#if SUPPORT_ED_ADJUST_BOUND_BY_RSSI
                            if (rssiValid &&
                                adapter_p->currentRssi <
                                ED_RSSI_THRESHOLD_FLOAT_TO_FIX) {
                                DBGLOG(HW, INFO, ("Stay in floating noise floor, bounded by rssi\n"));
                                break;
                            }
#endif
                            adapter_p->ED_floatLevel = 0;
                            edSetNoiseFloorFloatMargin(adapter_p);


#if SUPPORT_ED_ADJUST_BOUND_BY_RSSI && SUPPORT_ED_ADJUST_FAST_FIND
                            if ( edFastRssi > ED_THRESHOLD_START ) {
                                adapter_p->cur_EDThreshold = edFastRssi;
                                edStepEDThreshold(adapter_p, 1);
                                edSetNoiseFloorFixed(adapter_p);

                                DBGLOG(HW, INFO, ("Change to Fix noise floor, fast find\n"));

                                break;
                            }
#endif
                            adapter_p->cur_EDThreshold = ED_THRESHOLD_START;
                            edStepEDThreshold(adapter_p, 1);
                            edSetNoiseFloorFixed(adapter_p);


                            DBGLOG(HW, INFO, ("Change to Fix noise floor\n"));
                        }
                        break;
#endif /* SUPPORT_ED_ADJUST_MULTI_LEVEL_FLOAT_STATE */

                        {
#if SUPPORT_ED_ADJUST_BOUND_BY_RSSI
                            if (rssiValid &&
                                adapter_p->currentRssi <
                                ED_RSSI_THRESHOLD_FLOAT_TO_FIX) {
                                DBGLOG(HW, INFO, ("Stay in floating noise floor, bounded by rssi\n"));
                                break;
                            }
#endif

#if SUPPORT_ED_ADJUST_BOUND_BY_RSSI && SUPPORT_ED_ADJUST_FAST_FIND
                            if ( edFastRssi > ED_THRESHOLD_START ) {
                                adapter_p->cur_EDThreshold = edFastRssi;
                                edStepEDThreshold(adapter_p, 1);
                                edSetNoiseFloorFixed(adapter_p);

                                DBGLOG(HW, INFO, ("Change to Fix noise floor, fast find\n"));

                                break;
                            }
#endif
                            adapter_p->cur_EDThreshold = ED_THRESHOLD_START;
                            edStepEDThreshold(adapter_p, 1);
                            edSetNoiseFloorFixed(adapter_p);


                            DBGLOG(HW, INFO, ("Change to Fix noise floor\n"));
                        }
                    }
                    break;

                case NOISE_STATE_HIGH_START:
#if SUPPORT_ED_ADJUST_BOUND_BY_RSSI
                    if ( edRssiValid && adapter_p->cur_EDThreshold > edRssi ) {

                        edStepEDThreshold(adapter_p, -1);
                        adapter_p->ED_adjust_dir = -1;
                        adapter_p->ED_target_offset = ED_TARGET_NOT_FOUND;
                        adapter_p->state_noise = NOISE_STATE_HIGH_SEARCH;

                        adapter_p->ED_ThresholdForceDown_flag = 1;
                        DBGLOG(HW, INFO, ("Force Down, currentRssi = %d\n", adapter_p->currentRssi));


                    }
                    else
#endif
                    if ( fa_count_detect_ofdm > OFDM_DETECTION_FA_THRESHOLD ||
                        fa_count_detect_cck > CCK_DETECTION_FA_THRESHOLD ||
                        fa_count_ed > ED_FA_THRESHOLD ) {

                        edStepEDThreshold(adapter_p, 1);
                        adapter_p->ED_adjust_dir = 1;
                        adapter_p->ED_target_offset = ED_TARGET_NOT_FOUND;
                        adapter_p->state_noise = NOISE_STATE_HIGH_SEARCH;

                    }
                    else {
                        edStepEDThreshold(adapter_p, -1);
                        adapter_p->ED_adjust_dir = -1;
                        adapter_p->ED_target_offset = ED_TARGET_NOT_FOUND;
                        adapter_p->state_noise = NOISE_STATE_HIGH_SEARCH;
                    }
                    DBGLOG(HW, INFO, ("Noise state HI START -> SEARCH\n"));
                    DBGLOG(HW, INFO, ("Dir: %d, offset: %d\n",
                                adapter_p->ED_adjust_dir, adapter_p->ED_target_offset));

                    break;
                case NOISE_STATE_HIGH_SEARCH:
#if SUPPORT_ED_ADJUST_BOUND_BY_RSSI
                    if ( edRssiValid && adapter_p->cur_EDThreshold > edRssi ) {

                        edStepEDThreshold(adapter_p, -1);
                        adapter_p->ED_adjust_dir = -1;
                        adapter_p->ED_ThresholdForceDown_flag = 1;
                        DBGLOG(HW, INFO, ("Force Down, currentRssi = %d\n", adapter_p->currentRssi));
                        DBGLOG(HW, INFO, ("Dir: %d, offset: %d\n",
                                    adapter_p->ED_adjust_dir, adapter_p->ED_target_offset));

                        break;
                    }

#if SUPPORT_ED_ADJUST_FAST_FIND
                    if ( edRssiValid && edFast_flag &&
                        edFastRssi > adapter_p->cur_EDThreshold ) {

                        adapter_p->cur_EDThreshold = edFastRssi;
                        edStepEDThreshold(adapter_p, 1);

                        adapter_p->ED_adjust_dir = 1;
                        adapter_p->ED_target_offset = ED_TARGET_NOT_FOUND;
                        adapter_p->state_noise = NOISE_STATE_HIGH_START;

                        DBGLOG(HW, INFO, ("Fast find, in fixed noise floor state."));
                        break;
                    }
#endif

#endif

                    if ( fa_count_detect_ofdm > OFDM_DETECTION_FA_THRESHOLD ||
                        fa_count_detect_cck > CCK_DETECTION_FA_THRESHOLD ||
                        fa_count_ed > ED_FA_THRESHOLD ) {

                        if ( adapter_p->ED_adjust_dir == 1 || forceDown_flag == 1 ) {
                            edStepEDThreshold(adapter_p, 1);
                            adapter_p->ED_adjust_dir = 1;

                        }
                        else {

                            edStepEDThreshold(adapter_p, 1);
                            adapter_p->ED_adjust_dir = 1;
                            adapter_p->ED_target_offset = 0;
                            adapter_p->state_noise = NOISE_STATE_HIGH_TUNE;
                            DBGLOG(HW, INFO, ("Noise state HI SEARCH -> TUNE\n"));


                        }
                    }
                    else {
                        if ( adapter_p->ED_adjust_dir == 1 ) {
                            edStepEDThreshold(adapter_p, 1);
                            adapter_p->ED_adjust_dir = 1;
                            adapter_p->ED_target_offset = 1;
                            adapter_p->state_noise = NOISE_STATE_HIGH_TUNE;
                            DBGLOG(HW, INFO, ("Noise state HI SEARCH -> TUNE\n"));

                        }
                        else {
                            edStepEDThreshold(adapter_p, -1);
                            adapter_p->ED_adjust_dir = -1;

                        }
                    }
                    DBGLOG(HW, INFO, ("Dir: %d, offset: %d\n",
                                adapter_p->ED_adjust_dir, adapter_p->ED_target_offset));

                    break;
                case NOISE_STATE_HIGH_TUNE:
#if SUPPORT_ED_ADJUST_BOUND_BY_RSSI
                    if ( edRssiValid && adapter_p->cur_EDThreshold > edRssi ) {
                        adapter_p->ED_ThresholdForceDown_flag = 1;
                        DBGLOG(HW, INFO, ("Force Down, currentRssi = %d\n", adapter_p->currentRssi));

                        if ( adapter_p->ED_target_offset <= 0 ) {
                            edStepEDThreshold(adapter_p, -1);
                            adapter_p->ED_adjust_dir = -1;
                            adapter_p->ED_target_offset = ED_TARGET_NOT_FOUND;
                            adapter_p->state_noise = NOISE_STATE_HIGH_SEARCH;
                            DBGLOG(HW, INFO, ("Noise state HI TUNE -> SEARCH\n"));
                        }
                        else {
                            edStepEDThreshold(adapter_p, -1);
                            adapter_p->ED_adjust_dir = -1;
                            adapter_p->ED_target_offset--;
                        }

                        DBGLOG(HW, INFO, ("Dir: %d, offset: %d\n",
                                    adapter_p->ED_adjust_dir, adapter_p->ED_target_offset));
                        break;

                    }

#if SUPPORT_ED_ADJUST_FAST_FIND
                    if ( edRssiValid && edFast_flag &&
                        edFastRssi > adapter_p->cur_EDThreshold ) {

                        adapter_p->cur_EDThreshold = edFastRssi;
                        edStepEDThreshold(adapter_p, 1);

                        adapter_p->ED_adjust_dir = 1;
                        adapter_p->ED_target_offset = ED_TARGET_NOT_FOUND;
                        adapter_p->state_noise = NOISE_STATE_HIGH_START;

                        DBGLOG(HW, INFO, ("Fast find, in fixed noise floor state."));
                        break;
                    }
#endif

#endif

                    if ( fa_count_detect_ofdm > OFDM_DETECTION_FA_THRESHOLD ||
                        fa_count_detect_cck > CCK_DETECTION_FA_THRESHOLD ||
                        fa_count_ed > ED_FA_THRESHOLD) {

                        edStepEDThreshold(adapter_p, 1);
                        adapter_p->ED_adjust_dir = 1;
                        adapter_p->ED_target_offset = ED_TARGET_NOT_FOUND;
                        adapter_p->state_noise = NOISE_STATE_HIGH_SEARCH;
                        DBGLOG(HW, INFO, ("Noise state HI TUNE -> SEARCH\n"));
                    }
                    else {
                        if ( ( adapter_p->ED_adjust_dir == 1 &&
                            (fa_count_ed+ED_FA_COMPARE_THRESHOLD) >= adapter_p->previous_fa_count_ed ) ||
                            ( adapter_p->ED_adjust_dir == -1 &&
                            fa_count_ed <= (adapter_p->previous_fa_count_ed+ED_FA_COMPARE_THRESHOLD) ) ) {

                            if ( adapter_p->ED_target_offset <= 0 ) {
                                edStepEDThreshold(adapter_p, -1);
                                adapter_p->ED_adjust_dir = -1;
                                adapter_p->ED_target_offset = ED_TARGET_NOT_FOUND;
                                adapter_p->state_noise = NOISE_STATE_HIGH_SEARCH;
                                DBGLOG(HW, INFO, ("Noise state HI TUNE -> SEARCH\n"));
                            }
                            else {
                                edStepEDThreshold(adapter_p, -1);
                                adapter_p->ED_adjust_dir = -1;
                                adapter_p->ED_target_offset--;
                            }

                        }
                        else {

                            if ( adapter_p->ED_target_offset >= ED_TARGET_OFFSET_MAX ) {
                                edStepEDThreshold(adapter_p, -1);
                                adapter_p->ED_adjust_dir = -1;
                                adapter_p->ED_target_offset--;
                            }
                            else {
                                edStepEDThreshold(adapter_p, 1);
                                adapter_p->ED_adjust_dir = 1;
                                adapter_p->ED_target_offset++;
                            }
                        }



                    }
                    DBGLOG(HW, INFO, ("Dir: %d, offset: %d\n",
                                adapter_p->ED_adjust_dir, adapter_p->ED_target_offset));

                    break;
            } /* end of switch (adapter_p->state_noise) */

            if ( adapter_p->state_noise != NOISE_STATE_LOW &&
                adapter_p->cur_EDThreshold <= ED_THRESHOLD_MIN) {
                edSetNoiseFloorFloat(adapter_p);
                DBGLOG(HW, INFO, ("Change to Float noise floor\n"));
            }
#if SUPPORT_ED_ADJUST_BOUND_BY_RSSI
            else if (adapter_p->state_noise != NOISE_STATE_LOW && rssiValid
                     && adapter_p->currentRssi <
                     ED_RSSI_THRESHOLD_FIX_TO_FLOAT) {
                edSetNoiseFloorFloat(adapter_p);
                DBGLOG(HW, INFO, ("Change to Float noise floor, due to currentRssi = %d\n",
                    adapter_p->currentRssi));
            }
#endif
            adapter_p->previous_fa_count_ed = fa_count_ed;

            break;
        case ED_FLOAT:
            edSetNoiseFloorFloat(adapter_p);
            DBGLOG(HW, INFO, ("Force to float ED.\n"));
            break;
        case ED_NOT_ADJUST:
            DBGLOG(HW, INFO, ("Not adjust ED.\n"));
            break;
        default:
            ERRORLOG(("Not identified ED mode.\n"));
            break;
    } /* end of switch (edMode) */
}   /* edUpdateNoiseFloor */

#if SUPPORT_DYNAMIC_THRESHOLD_FOR_ED
/*_________________________________________________________________________
**  edCalculateAverageNoiseFloor
**
**  descriptions:
**  parameters:  This function is called to calculate an average value of
**               noise floor by the old average value and the current noise
**               floor.
**      adapter_p - Pointer to the Adapter structure
**      cr114 - The content of the Baseband CR114 register
**      cr115 - The content of the Baseband CR115 register
**  return: The new average value of noise floor.
**  note:
**__________________________________________________________________________*/
VOID
edCalculateAverageNoiseFloor(IN PMP_ADAPTER adapter_p,
                             IN UINT_8 cr114,
                             IN UINT_8 cr115)
{
    UINT_32 noiseFloor;

    DEBUGFUNC("edCalculateAverageNoiseFloor");

    noiseFloor = (((UINT_32) cr114) << 8) | ((UINT_32) cr115);

    if (adapter_p->averageNoiseFloor) {
        noiseFloor = ((UINT_32) adapter_p->averageNoiseFloor * 99 +
            noiseFloor) / 100;
    }

    adapter_p->averageNoiseFloor = (UINT_16) noiseFloor;
}   /* edCalculateAverageNoiseFloor */
#endif

#endif  /* end of #if SUPPORT_ED_ADJUST */

