/*
** $Id: inlinef.h 1.3 2004/08/11 16:20:23 patlu Exp $
*/

/*************************************************************************
* Copyright (c) 2004 Inprocomm, Inc.
*
* All rights reserved. Copying, compilation, modification, distribution
* or any other use whatsoever of this material is strictly prohibited
* except in accordance with a Software License Agreement with
* Inprocomm, Inc.
**************************************************************************/
/*************************************************************************
GENERAL DESCRIPTION
********************
*/

#ifndef _INLINEF_H
#define _INLINEF_H

/************************************************************************
*                     C O M P I L E R   F L A G S
*************************************************************************
*/

/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/

/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/

/************************************************************************
*                             M A C R O S
*************************************************************************
*/

/************************************************************************
*                    D A T A   D E C L A R A T I O N S
*************************************************************************
*/

/************************************************************************
*                 F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/

/*_________________________________________________________________________
**  nicDisableInterrupt
**
**  descriptions: This routine disables interrupts at the hardware.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**  return:       (none)
**  note:
**__________________________________________________________________________*/
_INLINE_FUNC
VOID
nicDisableInterrupt(IN PMP_ADAPTER adapter_p)
{
    UINT_32 tmp;

    DEBUGFUNC("nicDisableInterrupt");

    DBGLOG(INTR, TRACE, ("\n"));

    NIC_MCR_READ(adapter_p, MCR_IER, &tmp);
    DBGLOG(INTR, INFO, ("IER = 0x%08x\n", tmp));

    tmp &= ~IER_GLOBAL_ENABLE;

    NIC_MCR_WRITE(adapter_p, MCR_IER, tmp);
    DBGLOG(INTR, INFO, ("Interrupt disabled\n"));

#if DBG
    NIC_MCR_READ(adapter_p, MCR_IER, &tmp);
    DBGLOG(INTR, INFO, ("IER = 0x%08x\n", tmp));
#endif
}   /* nicDisableInterrupt */

/*_________________________________________________________________________
**  nicEnableInterrupt
**
**  descriptions: This routine enables interrupts at the hardware.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**  return:       (none)
**  note:
**__________________________________________________________________________*/
_INLINE_FUNC
VOID
nicEnableInterrupt(IN PMP_ADAPTER adapter_p)
{
    UINT_32 tmp;

    DEBUGFUNC("nicEnableInterrupt");

    DBGLOG(INTR, TRACE, ("\n"));

    NIC_MCR_READ(adapter_p, MCR_IER, &tmp);
    DBGLOG(INTR, INFO, ("IER = 0x%08x\n", tmp));

    tmp |= IER_GLOBAL_ENABLE;

    NIC_MCR_WRITE(adapter_p, MCR_IER, tmp);

    DBGLOG(INTR, INFO, ("Interrupt enabled\n"));

#if DBG
    NIC_MCR_READ(adapter_p, MCR_IER, &tmp);
    DBGLOG(INTR, INFO, ("IER = 0x%08x\n", tmp));
#endif
}   /* nicEnableInterrupt */

/*_________________________________________________________________________
**  nicCalculateChecksumByte
**
**  descriptions: This routine calculates the 8-bit checksum of a memory
**                block.
**  parameters:
**      startingAddr_p - The starting address of the memory block to
**                       calculate checksum
**      length - The length of the given memory block
**  result:
**      checksum_p - Pointer to the 8-bit calculated checksum
**  return:       (none)
**  note:
**__________________________________________________________________________*/
_INLINE_FUNC
VOID
nicCalculateChecksumByte(IN PVOID startingAddr_p,
                         IN UINT length,
                         OUT PUINT_8 checksum_p)
{
    UINT i;
    PUINT_8 cp;

    *checksum_p = 0;
    cp = (PUINT_8) startingAddr_p;
    for (i = 0; i < length; i++, cp++) {
        *checksum_p += *cp;
    }
}   /* nicCalculateChecksumByte */

/*_________________________________________________________________________
**  nicEnterStandby
**
**  descriptions: This routine initialize NIC into standby mode when wake up
**                from sleep or idle mode.
**                That means all clocks and regulators are enabled.
**                Baseband and MAC state machine is reset.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**  return:       TRUE:  MAC successfully entering standby
**                FALSE: MAC enter standby failed
**  note:
**__________________________________________________________________________*/
_INLINE_FUNC
BOOLEAN
nicEnterStandby(IN PMP_ADAPTER adapter_p)
{
    UINT i;
    UINT_8 bbcrData;
    BOOLEAN status = TRUE;
    UINT_32 temp32;

    DEBUGFUNC("nicEnterStandby");

    DBGLOG(HW, TRACE, ("\n"));

    NIC_MCR_READ(adapter_p, MCR_SCR, &temp32);
    DBGLOG(HW, INFO, ("SCR when setting NIC to standby: 0x%x\n", temp32));

    if (!(temp32 & SCR_GPIO_PD_N)) {
        NIC_MCR_WRITE(adapter_p, MCR_SCR,
            (temp32 & ~(SCR_CLK_ALL_EN | SCR_PAU_ENABLE)) | SCR_GPIO_PD_N);
        mpStallExecutionMsec(15);
        if (temp32 & SCR_PLL_ENABLE) {
            DBGLOG(HW, ERROR,
                ("PLL is enabled but OSC is disabled. SCR: 0x5x\n", temp32));
            mpStallExecutionMsec(20);
        }
    }

    NIC_MCR_READ(adapter_p, MCR_SCR, &temp32);
    temp32 |= SCR_GPIO_PD_N | SCR_PLL_ENABLE;
    NIC_MCR_WRITE(adapter_p, MCR_SCR, temp32);
    mpStallExecutionMsec(1);
    temp32 |= SCR_CLK_ALL_EN;
    NIC_MCR_WRITE(adapter_p, MCR_SCR, temp32);
    NdisStallExecution(10);

    if (BCIRead8(adapter_p, BBCR_CR82, &bbcrData, TRUE, 0xff)) {
        BCIWrite8(adapter_p, BBCR_CR82, (UINT_8) (bbcrData & 0xfe), TRUE, 0xff);
    } else {
        status = FALSE;
        for (i = 0; i < 5; i++) {
            if (BCIWrite8(adapter_p, BBCR_CR82, 0x00, TRUE, 0xff)) {
                status = TRUE;
                break;
            }
        }
    }

#if WORKAROUND_TXDAC_POWER_CONSUMPTION
    if (adapter_p->rfSpecificConfig_p != NULL) {
        if (adapter_p->rfSpecificConfig_p->txDACon) {
            BCIWrite8(adapter_p, BBCR_CR88, 0x80, TRUE, 0xff);
        } else {
            BCIWrite8(adapter_p, BBCR_CR88, 0x00, TRUE, 0xff);
        }
    }
#endif

    NIC_MCR_WRITE(adapter_p, MCR_SCR, temp32 | SCR_MAC_RST);
    mpStallExecutionMsec(5);
    NIC_MCR_WRITE(adapter_p, MCR_SCR, temp32 | SCR_BB_MRST);
    mpStallExecutionMsec(5);
    return status;
}   /* nicEnterStandby */

/*_________________________________________________________________________
**  nicEnterIdle
**
**  descriptions: This routine initialize NIC into Idle mode.
**                That means all clocks are disenabled except PLL.
**                Bandgape, BB, PAU and RF are in idle or sleep mode
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**  return:       (none)
**  note:
**__________________________________________________________________________*/
_INLINE_FUNC
VOID
nicEnterIdle(IN PMP_ADAPTER adapter_p)
{
    UINT_8 bbcrData;
    UINT_32 temp32;

    DEBUGFUNC("nicEnterIdle");

    DBGLOG(HW, TRACE, ("\n"));

    phyRFEnterSleep(adapter_p);

    NIC_MCR_READ(adapter_p, MCR_SCR, &temp32);
    //temp32 &= ~(PPMCR_PAU_ENABLE | PPMCR_POWER_SAVE_SWITCH);
    temp32 &= ~SCR_PAU_ENABLE;
    NIC_MCR_WRITE(adapter_p, MCR_SCR, temp32);

    NdisStallExecution(2);

    if (BCIRead8(adapter_p, BBCR_CR82, &bbcrData, TRUE, 0xff)) {
        BCIWrite8(adapter_p, BBCR_CR82,(UINT_8) (bbcrData | 0x01), TRUE, 0xff);
    }

#if WORKAROUND_TXDAC_POWER_CONSUMPTION
    BCIWrite8(adapter_p, BBCR_CR88, 0x80, TRUE, 0xff);
#endif

    NIC_MCR_READ(adapter_p, MCR_SCR, &temp32);
#if WORKAROUND_CLK20M_GATING_BUG790
    temp32 = (temp32 & (~(SCR_CLK_ALL_EN | SCR_PAU_ENABLE))) |
        SCR_PLL_ENABLE | SCR_MAC_CLK33M_ENABLE | SCR_MAC_CLK20M_ENABLE;
#else
    temp32 = (temp32 & (~(SCR_CLK_ALL_EN | SCR_PAU_ENABLE))) |
        SCR_PLL_ENABLE | SCR_MAC_CLK33M_ENABLE;
#endif
    NIC_MCR_WRITE(adapter_p, MCR_SCR, temp32);
}   /* nicEnterIdle */

/*_________________________________________________________________________
**  nicEnterSleep
**
**  descriptions: This routine initialize NIC into sleep mode.
**                That means all clocks and regulators are disabled.
**                Bandgape, BB, PAU and are in idle or sleep mode.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**  return:       (none)
**  note:
**__________________________________________________________________________*/
_INLINE_FUNC
VOID
nicEnterSleep(IN PMP_ADAPTER adapter_p)
{
    UINT_32 temp32;

    DEBUGFUNC("nicEnterSleep");

    DBGLOG(HW, TRACE, ("\n"));

    nicEnterIdle(adapter_p);

    NIC_MCR_READ(adapter_p, MCR_SCR, &temp32);
    temp32 &= ~(SCR_CLK_ALL_EN | SCR_PAU_ENABLE);
    NIC_MCR_WRITE(adapter_p, MCR_SCR, temp32);

    temp32 &= ~SCR_GPIO_PD_N;
    NIC_MCR_WRITE(adapter_p, MCR_SCR, temp32);
}   /* nicEnterSleep */

#endif /* _INLINEF_H */

